/******************************************************************************
* FILE PURPOSE: Run Stand-alone XDM compliant Encoder
******************************************************************************
* FILE NAME:   atk_encode.c
*
* DESCRIPTION: Contains routines that support stand-alone Encoder test
*
* TABS: NONE
*
*
* (C) Copyright 2011, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
/* Standard C header files                                                    */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "codecs/atkCodecAPI.h"
#include <ti/mas/iface/ifvisys/ifvisys.h>
#include "xdm.h"
#include "atkloc.h"

/******************************************************************************
* FUNCTION PURPOSE: Main Function acting as a client for audio Encode Call
******************************************************************************
* DESCRIPTION: Main Function acting as a client for audio Encode Call
*
*****************************************************************************/
tint atkEncode (void *atkInst, ifvisys_audEncInput_t *audEncInput, ifvisys_audEncOutput_t *audEncOutput)
{
    atkInst_t *inst  = (atkInst_t *)atkInst;
    /* Input Buffer Management specific variables                              */
    XDAS_UInt32 size;
    audEncoderAPI_t *encAPI;
    tint retVal;

    encAPI = getAudEncoderAPI(inst->codecName);

    switch (encAPI->encXdmVer)
    {
        case ATK_XDM_VER_0P9:
            {
            IAUDENC_Fxns               *iaudENCFxns;
            IAUDENC_Status             *status;
            IAUDENC_InArgs             *inArgs;
            IAUDENC_OutArgs            *outArgs;
            IAUDENC_DynamicParams      *dynamicParams;

            XDAS_Int8*  pInputBuf   [XDM_MAX_IO_BUFFERS];
            XDAS_Int8*  pOutputBuf  [XDM_MAX_IO_BUFFERS];
            XDAS_Int32  inBufSize   [XDM_MAX_IO_BUFFERS];
            XDAS_Int32  outBufSize  [XDM_MAX_IO_BUFFERS];

            /* Input/Output Buffer Descriptors                                         */
            XDM_BufDesc inputBufDesc;
            XDM_BufDesc outputBufDesc;

            /* Get XDM0.9 pointers from the codec */
            inArgs                = (IAUDENC_InArgs *)encAPI->encGetInArgs(inst->codecInst);
            outArgs               = (IAUDENC_OutArgs *)encAPI->encGetOutArgs(inst->codecInst);
            dynamicParams         = (IAUDENC_DynamicParams *)encAPI->encGetDynamicParams(inst->codecInst, &size);
            status                = (IAUDENC_Status *)encAPI->encGetStatus(inst->codecInst);

            /* Assigning Algorithm handle fxns field to iaudEncFxns */
            iaudENCFxns = (IAUDENC_Fxns *)inst->handle->fxns;

            iaudENCFxns->control((IAUDENC_Handle)inst->handle, XDM_SETPARAMS,dynamicParams,status);

            retVal= iaudENCFxns->control((IAUDENC_Handle)inst->handle,
                                          XDM_GETBUFINFO,
                                          (IAUDENC_DynamicParams *)dynamicParams,
                                          (IAUDENC_Status *)status);

            /*Fill up the buffers as required by algorithm                            */

            inputBufDesc.bufs         = (XDAS_Int8 **)pInputBuf;
            inputBufDesc.bufSizes     = (XDAS_Int32 *)inBufSize;
            inputBufDesc.numBufs      = status->bufInfo.minNumInBufs ;
            inputBufDesc.bufs[0]      = (XDAS_Int8 *)audEncInput->bitStream.ptr;
            inputBufDesc.bufSizes[0]  = status->bufInfo.minInBufSize[0];

            outputBufDesc.bufs        = (XDAS_Int8 **)pOutputBuf;
            outputBufDesc.bufSizes    = (XDAS_Int32 *)outBufSize;
            outputBufDesc.numBufs     = status->bufInfo.minNumOutBufs ;
            outputBufDesc.bufs[0]     = (XDAS_Int8 *)audEncOutput->bitStream.ptr;
            outputBufDesc.bufSizes[0] = status->bufInfo.minOutBufSize[0];

            /* Basic Algorithm process() call */
            retVal = iaudENCFxns->process((IAUDENC_Handle)inst->handle,
                (XDM_BufDesc *)&inputBufDesc,
                (XDM_BufDesc *)&outputBufDesc,
                (IAUDENC_InArgs *)inArgs,
                (IAUDENC_OutArgs *)outArgs);

            if ((retVal == XDM_EFAIL) || (outArgs->extendedError != XDM_EOK)) {
                inst->stats.errorFrames++;
                inst->stats.lastErrorCode = outArgs->extendedError;
            } else {
                inst->stats.goodFrames++;
            }

            /* Optional: Read status via control()                                  */
            iaudENCFxns->control((IAUDENC_Handle)inst->handle,
                XDM_GETSTATUS,
                (IAUDENC_DynamicParams *)dynamicParams,
                (IAUDENC_Status *)status);

            audEncOutput->bitStream.size = outArgs->bytesGenerated;
            }
        break;

        case ATK_XDM_VER_1P0:
            {
            IAUDENC1_Fxns               *iaudENCFxns;
            IAUDENC1_Status             *status;
            IAUDENC1_InArgs             *inArgs;
            IAUDENC1_OutArgs            *outArgs;
            IAUDENC1_DynamicParams      *dynamicParams;
            XDM1_BufDesc                inputBufDesc; /*! Input Buffer Descriptor */
            XDM1_BufDesc                outputBufDesc;/*! Output Buffer Descriptor */

            inArgs                = (IAUDENC1_InArgs *)encAPI->encGetInArgs(inst->codecInst);
            outArgs               = (IAUDENC1_OutArgs *)encAPI->encGetOutArgs(inst->codecInst);
            dynamicParams         = (IAUDENC1_DynamicParams *)encAPI->encGetDynamicParams(inst->codecInst, &size);
            status                = (IAUDENC1_Status *)encAPI->encGetStatus(inst->codecInst);

            /* Assigning Algorithm handle fxns field to iaudEncFxns */
            iaudENCFxns = (IAUDENC1_Fxns *)inst->handle->fxns;

            iaudENCFxns->control((IAUDENC1_Handle)inst->handle, XDM_SETPARAMS,dynamicParams,status);

            retVal= iaudENCFxns->control((IAUDENC1_Handle)inst->handle,
                                          XDM_GETBUFINFO,
                                          (IAUDENC1_DynamicParams *)dynamicParams,
                                          (IAUDENC1_Status *)status);

            /*Fill up the buffers as required by algorithm                            */

            inputBufDesc.numBufs           = status->bufInfo.minNumInBufs ;
            inputBufDesc.descs[0].buf      = (XDAS_Int8 *)audEncInput->bitStream.ptr;
            inputBufDesc.descs[0].bufSize  = status->bufInfo.minInBufSize[0];

            outputBufDesc.numBufs          = status->bufInfo.minNumOutBufs ;
            outputBufDesc.descs[0].buf     = (XDAS_Int8 *)audEncOutput->bitStream.ptr;
            outputBufDesc.descs[0].bufSize = status->bufInfo.minOutBufSize[0];

            /* Basic Algorithm process() call */
            retVal = iaudENCFxns->process((IAUDENC1_Handle)inst->handle,
                                          (XDM1_BufDesc *)&inputBufDesc,
                                          (XDM1_BufDesc *)&outputBufDesc,
                                          (IAUDENC1_InArgs *)inArgs,
                                          (IAUDENC1_OutArgs *)outArgs);

            if ((retVal == XDM_EFAIL) || (outArgs->extendedError != XDM_EOK)) {
                inst->stats.errorFrames++;
                inst->stats.lastErrorCode = outArgs->extendedError;
            } else {
                inst->stats.goodFrames++;
            }

            /* Optional: Read status via control()                                  */
            iaudENCFxns->control((IAUDENC1_Handle)inst->handle,
                                  XDM_GETSTATUS,
                                 (IAUDENC1_DynamicParams *)dynamicParams,
                                 (IAUDENC1_Status *)status);

            audEncOutput->bitStream.size = outArgs->bytesGenerated;
            }
        break;

        default:
            {
    
            }
        break;
    }
    return(ATK_NOERR);
}

/* nothing past this point */
