/******************************************************************************
 * FILE PURPOSE:  Routines that create a DSP ATK channel
 ******************************************************************************
 * FILE NAME:     atkinit.c 
 *
 * DESCRIPTION:   Contains routines that create a DSP ATK unit
 *
 * FUNCTION                     DESCRPTION
 * --------                     ----------
 *
 * (C) Copyright 2011, Texas Instruments Inc. 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
/* Standard C header files */
#include <stdio.h>
#include <stdlib.h>
#include "xdc/std.h"

/* System utility files */
#include "types.h"
#include "memmgm.h"     /* memory management */

/* ATK files */
#include "atk.h"
#include "atkloc.h"
#include "codecs/atkCodecAPI.h"

/* number of buffers required by ATK */
#define ATK_NBUFS                2
#define ATK_INSTANCE_BUFN        0
#define ATK_CODEC_INSTANCE_BUFN  1

/* buffers used by ATK */
memBuffer_t atkBufs[ATK_NBUFS] = {
  MEM_CLASS_EXTERNAL, 0, 0, FALSE, NULL,  /* instance structure */
  MEM_CLASS_EXTERNAL, 0, 0, FALSE, NULL,  /* codec instance structure */
};

/******************************************************************************
 * FUNCTION PURPOSE: Sizes of buffers in use by a ATK instance.
 ******************************************************************************
 * DESCRIPTION: 
 * int atkGetSizes (
 *                 int *nbufs,               - number of buffers
 *                 const memBuffer_t **bufs, - buffer descriptions
 *                 atkSizeCfg_t *cfg)        - size config struct
 *
 * Return value : ATK_NOERR - successful in getting buffer sizes
 *                ATK_ERROR - 'cfg' information in conflict with pre-set
 *                                     limits or available resources
 *****************************************************************************/
tint atkGetSizes (tint *nbufs, const memBuffer_t **bufs, atkSizeCfg_t *cfg)
{
  /* Calculate the buffer sizes and report their descriptions */
  *nbufs                                = ATK_NBUFS;             /* Report the number of buffers */
  atkBufs[ATK_INSTANCE_BUFN].size       = sizeof(atkInst_t);
  atkBufs[ATK_CODEC_INSTANCE_BUFN].size = getMaxAudInstSize();
  *bufs                                 = atkBufs;

  return (ATK_NOERR);
} /* atkGetSizes() */

/******************************************************************************
 * FUNCTION PURPOSE: Create an instance of ATK
 ******************************************************************************
 * DESCRIPTION: Create an an instance of ATK and initialize its memory buffers.
 *
 * tint atkNew(
 *   void            **atkInst,    - an address of memory location that will
 *                                  receive a pointer to instance structure
 *   tint            nbufs,        - number of buffers used
 *   memBuffer_t     *bufs,        - a vector of buffer descriptors
 *   atkNewConfig_t  *cfg)         - a pointer to configuration structure
 *
 * Return values:  ATK_NOMEMORY    - some buffers have base address NULL
 *                 ATK_NOERR       - success
 *                 ATK_ERROR       - invalid input parameters   
 *
 *****************************************************************************/
tint atkNew (void **atkInst, tint nbufs, memBuffer_t *bufs, atkNewCfg_t *cfg)
{
  int   i;
  atkInst_t *inst;
  memBuffer_t *bufp;
  tint retval = ATK_NOERR;

  /* Test instance pointer (Must be NULL)  */
  if (*atkInst != NULL || nbufs != ATK_NBUFS)
    return(ATK_ERROR);

  /* Allocate all dynamic buffers (base address != NULL ?)   */
  for (i = 0, bufp = &bufs[0]; i < nbufs; i++, bufp++) {
    if ( (bufp->size != 0) && (bufp->base == NULL) )
      retval = ATK_NOMEMORY;
  }

  if (retval != ATK_NOERR) 
     return retval;
  
  /*  Identify the instance structure  */
  bufp = &bufs[ATK_INSTANCE_BUFN];
  if (bufp->size < sizeof(atkInst_t))
    return(ATK_ERROR);
  inst = (atkInst_t *) bufp->base;
  /* Reset instance buffer to known state */
  memset (bufp->base, 0, sizeof(atkInst_t));

/*  Identify the instance structure  */
  bufp = &bufs[ATK_CODEC_INSTANCE_BUFN];
  if (bufp->size < getMaxAudInstSize())
    return(ATK_ERROR);
  inst->codecInst = (void *) bufp->base;
  /* Reset instance buffer to known state */
  memset (bufp->base, 0, bufp->size);

  /* Save configuration data  */
  inst->ID         = cfg->ID;
  inst->state      = ATK_STATE_CLOSED;  /* set ATK state to CLOSED  */
  *atkInst = (void *)inst;   /* return instance pointer  */
  return(ATK_NOERR);
} /*  atkNew  */

/* Nothing past this point */

