#ifndef _ATK_H
#define _ATK_H
/******************************************************************************
 * FILE PURPOSE: Defines API and public data definitions for ATK
 ******************************************************************************
 * FILE NAME: atk.h  
 *
 * DESCRIPTION: Defines API and public data definitions for atk module
 *
 * (C) Copyright 2011, Texas Instruments Inc. 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

/* System level header files */
#include <ti/mas/types/types.h>
#include "utlxferpkt.h"
#include "utldbg.h"
#include "memmgm.h"
#include "util.h"  /* bytes to words conversion macros */
#include "utlbitfield.h"
#include "xdc/std.h"
#include "ti/xdais/ialg.h"
#include <ti/mas/iface/ifvisys/ifvisys.h>

/* ATK error codes */
enum {
  ATK_NOERR    = 0,
  ATK_ERROR    = 1,
  ATK_NOMEMORY = 2,
  ATK_SKIP_PROCESS = 3
};

/* Exception Messages */
enum {
   ATK_EXC_OPEN                 = 1, /* Opened an already open channel */
   ATK_EXC_DELETE               = 2, /* Deleted an open instance */
   ATK_EXC_CONTROL              = 3, /* Control request received by a closed channel */
   ATK_EXC_INVD_CODEC           = 4,
   ATK_EXC_DECODE               = 5,
   ATK_EXC_ENCODE               = 6,
   ATK_EXC_ALLOCHEAP            = 7,
   ATK_EXC_ALLOCSCRATCH         = 8,
   ATK_EXC_COMMIT_FAIL          = 9
};


/******************************************************************************
 * DATA DEFINITION: ATK global parameters
 *****************************************************************************/
/******************************************************************************
 * DATA DEFINITION: Get ATK Instance
 *****************************************************************************/
/******************************************************************************
 * DATA DEFINITION:  Parameters which determine buffer sizes
 *****************************************************************************/
typedef struct {
  tint dummy;
} atkSizeCfg_t;

/******************************************************************************
 * DATA DEFINITION:  New parameters
 *****************************************************************************/
typedef struct {
  tuint ID;                  /* Channel instance ID */
  atkSizeCfg_t   sizeCfg;    /* Size configuration structure */ 
} atkNewCfg_t;

/******************************************************************************
 * DATA DEFINITION: ATK Configuration
 *                : This data structure is used to retrieve codec configuration
 *                  Inorder for the content in this structure to be valid, codec
 *                  has to receive both static & dynamic configuration
 *****************************************************************************/
typedef struct {
  char   *codecName;                    /**< String name of the RFC to be opened */
  void*  sysInst;
} atkCfg_t;

/******************************************************************************
 * DATA DEFINITION: ATK statistics 
 ******************************************************************************
 * DESCRIPTION: This structure monitors atk statistics
 *
 *****************************************************************************/
typedef struct atkStats_t {
   tulong totalNumBytes; /* total Bytes: Consumed by Dec or Produced by Enc */
   tulong bitrate;       /* estimated average bps (Q4) */
   tulong errorFrames;   /* counter of undecodable frames */
   tulong goodFrames;    /* counter of undecodable frames */
   tulong lastErrorCode; /* extended errors, decoder dependent */
   tuint  bitRateFrameCounter; /* count of N (=30?) frames and update bit rate */
   tulong bitRateByteCount;/* reset every N frames when bit rate is computed */
   tulong bitRateTime;   /* time when previous bit rate is computed */
} atkStats_t;
   
/******************************************************************************
 * DATA DEFINITION:  Realtime control data structure
 *****************************************************************************/
/* ATK Control Messages */
typedef struct {
  tulong *cfgPtr;
  tulong size;
} atkCodecCfgPtr_t;

typedef enum {
  ATK_GET_STATIC_CFG_PTR   = 0x00,
  ATK_GET_DYNAMIC_CFG_PTR,
  ATK_COMMIT_CODEC,
  ATK_GET_CFG,
  ATK_GET_ENC_DYN_CFG,
  ATK_SET_ENC_DYN_CFG,
  ATK_GET_STATS,
  ATK_RESET_CODEC,
  ATK_IS_DYN_UPDATE_ALLOWED,
  ATK_FLUSH_DEC
} atkCtrlCode_t;

typedef struct {
  atkCtrlCode_t code;
  union {
    atkCfg_t atkCfg;
    atkCodecCfgPtr_t codecCfgPtr;
  }u;
} atkCtrl_t;

/*****************************************************************************/
typedef struct {
  debugInfo_t debugInfo; /* Debug/exception function */
  tulong      *curTime;  /* wallclock */
  tword       *scratch;
  tulong      scratchSize;
  IALG_Handle (*ALG_Create)(void *sysInst, IALG_Fxns *fxns, IALG_Handle p, IALG_Params *params, tulong *resDepBitMap);
  void        (*ALG_Free)  (void *sysInst);
} atkContext_t;   

/* External references to the ATK context */
extern atkContext_t atkContext;

/* ATK Public functions */
void atkClose (void *atkInst);
tint atkControl (void *atkInst, atkCtrl_t *ctrl);
void atkDelete (void **atkInst);
tint atkGetSizes (tint *nbufs, const memBuffer_t **bufs, atkSizeCfg_t *cfg);
tint atkNew (void **atkInst, tint nbufs, memBuffer_t *bufs, atkNewCfg_t *cfg);
tint atkOpen (void *atkInst, atkCfg_t *cfg);
tint atkDecode   (void *atkInst, ifvisys_audDecInput_t *audDecInput, ifvisys_audDecOutput_t *audDecOutput);
tint atkEncode   (void *atkInst, ifvisys_audEncInput_t *audEncInput, ifvisys_audEncOutput_t *audEncOutput);
void atkGetStats (void *atkInst, bool reset, atkStats_t *stats);

/* ATK Call Table */
typedef struct {
  void (*atkClose)     (void *atkInst);
  tint (*atkControl)   (void *atkInst, atkCtrl_t *ctrl);
  void (*atkDelete)    (void **atkInst);
  tint (*atkGetSizes)  (tint *nbufs, const memBuffer_t **bufs, atkSizeCfg_t *cfg);
  tint (*atkNew)       (void **atkInst, tint nbufs, memBuffer_t *bufs, atkNewCfg_t *cfg);
  tint (*atkOpen)      (void *atkInst, atkCfg_t *cfg);
  tint (*atkDecode)    (void *atkInst, ifvisys_audDecInput_t *audDecInput, ifvisys_audDecOutput_t *audDecOutput);
  tint (*atkEncode)    (void *atkInst, ifvisys_audEncInput_t *audEncInput, ifvisys_audEncOutput_t *audEncOutput);
} atkCallTable_t;

#endif
/* Nothing past this point */
