/******************************************************************************
 * FILE PURPOSE: Bit Error Rate Test unit(BERT) main source file.
 ******************************************************************************
 * FILE NAME: bert.c   
 *
 * DESCRIPTION: The BERT is a software ECO which performs Bit Error Rate test
 *				of the telephony network and packet network. The module can be
 *				configured to send specified pattern to packet network or 
 *				telephony network, to find error bits from input data received 
 *				from telephony network or packet network.
 * GLOBAL FUNCTIONS:
 *
 * bertClose ()       : close a BERT instance
 * bertControl ()     : control function for a BERT instance
 * bertDelete ()      : delete a BERT instance
 * bertOpen ()        : open a BERT instance
 * (C) Copyright 2002, Texas Instruments Incorporated. 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
/* System level header files */
#include "types.h"

/* System utilities' API's */
#include "memmgm.h"

/* BERT high level header files */
#include "bert.h"
#include "bertloc.h"
#include "bertport.h"

#ifdef DIAG
#include "diagappif.h"
#else
#include "mgmicdsp.h"
#endif
/******************************************************************************
 * FUNCTION PURPOSE: Open and configure an instance of BERT.
 ******************************************************************************
 * DESCRIPTION: This function configures an instance of BERT.
 *  void bertOpen (
 *    void        *bertInst,   - a pointer to BERT instance
 *    bertConfig_t *cfg)       - a pointer to BERT configuration structure
 *
 *****************************************************************************/
tint bertOpen(void **bertInst, bertOpenConfig_t *cfg)
{
  bertInst_t *inst = (bertInst_t *)bertInst;
  if(inst->state != BERT_STATE_CLOSED)
	  return -1;	//assert?
  inst->moduleConfig = cfg->moduleConfig;
  if(inst->moduleConfig & MGB_BERT_TELE_TX_ENABLE)
  {
	  inst->telephony.txScratch = 0xffff;
  }
  if(inst->moduleConfig & MGB_BERT_TELE_RX_ENABLE)
  {
	  inst->telephony.rxScratch = 0xffff;
	  inst->telephony.rxRunning = FALSE;
	  inst->telephony.upperThreshold = ber2samples(cfg->telUpperThreshold, cfg->measureInterval);
	  inst->telephony.lowerThreshold = ber2samples(cfg->telLowerThreshold, cfg->measureInterval);
	  inst->telephony.rxSync = FALSE;
	  inst->telephony.rxOKFlag = FALSE;

	  inst->telephony.rxErrBits = 
      inst->telephony.rxErrBitsOld = 0;
	  inst->telephony.rxErrBitsSaturation = 0;
  }
  if(inst->moduleConfig & MGB_BERT_NET_TX_ENABLE)
  {
	  inst->packetNet.txScratch = 0xffff;
  }
  if(inst->moduleConfig & MGB_BERT_NET_RX_ENABLE)
  {
	  inst->packetNet.rxScratch = 0xffff;
	  inst->packetNet.rxRunning = FALSE;
	  inst->packetNet.rxSync = FALSE;
	  inst->packetNet.rxOKFlag = FALSE;
	  inst->packetNet.upperThreshold = ber2samples(cfg->netUpperThreshold, cfg->measureInterval);
	  inst->packetNet.lowerThreshold = ber2samples(cfg->netLowerThreshold, cfg->measureInterval);
    inst->packetNet.rxErrBitsSaturation = 0;
  }

  if((cfg->bitsPerSample & MGB_BERT_TEL_7BIT) == 0)
	  inst->bitsPerSample = 8;
  else
      inst->bitsPerSample = 7;

  inst->telephony.testPattern = cfg->telTestPattern;
  inst->packetNet.testPattern = cfg->netTestPattern;
	
  inst->telephony.channelID = cfg->telChannelID;
  inst->packetNet.channelID = cfg->netChannelID;

  inst->telephony.measureInterval = cfg->measureInterval;
  inst->packetNet.measureInterval = cfg->measureInterval;
  inst->statsReportMode = cfg->statsReportMode;
  inst->telephony.last_byte = 0;

  inst->report_timer = 
  inst->prdRptInterval = cfg->reportInterval;
  inst->sys_timer = 8000;
  inst->net_packet_click = 0;

  inst->statsReportFn = cfg->statsReportFn;
  inst->statsReportTargetInst = cfg->statsReportTargetInst;

  return BERT_NOERR;
} /* bertOpen() */

/******************************************************************************
 * FUNCTION PURPOSE: Close an instance of BERT.
 ******************************************************************************
 * DESCRIPTION: Closes an instance of BERT.
 *****************************************************************************/

void bertClose (void **bertInst)
{
  bertInst_t *inst = (bertInst_t *)bertInst;
  inst->state = BERT_STATE_CLOSED; /* set state to closed */ 

} /* bertClose */

/******************************************************************************
 * FUNCTION PURPOSE: Delete an instance of BERT.
 ******************************************************************************
 * DESCRIPTION: Deletes an instance of BERT. Clears the instance pointer.
 *              Actual "deallocation" of instance should be done elsewhere
 *              after this function is called. Hence, one has to save the
 *              instance pointer if necessary to make "deallocation" possible.
 *
 *  void bertDelete (
 *    void        **bertInst,  - an address of memory location that contains
 *                              a pointer to instance structure
 *    tint        nbufs,      - number of memory buffers
 *    memBuffer_t *bufs)      - a vector of buffer descriptors
 *
 *****************************************************************************/

void bertDelete (void **bertInst)
{

//?????TODO  bert_exc_assert (bertIsClosed(inst->state, BERT_SLOT_BERT), inst);
        
  /*---------------------------*/
  /* Mark the instance as free */
  /*---------------------------*/

   *bertInst = 0; //NULL;      /* clear the instance pointer */
} /* bertDelete */

void bertGetStats (void **bertInst, bool reset, bertStats_t *stats)
{
	bertInst_t *inst = (bertInst_t *)bertInst;
	
	stats->telSyncLostNum = inst->telephony.syncLostNum;
	stats->telBerCurrent = inst->telephony.instantBER; 
	stats->telRcvBits = inst->telephony.rxBits;
	if(inst->telephony.rxErrBitsSaturation)
	    stats->telErrBits = 0xffffffff;
	else
	    stats->telErrBits = inst->telephony.rxErrBits;

	stats->telRcvMode = inst->telephony.rxSync;
	if(inst->telephony.rxOKFlag == 0)
		stats->telRcvMode |= MGB_BERT_TEL_RX_ERROR;

	stats->telErrSeconds = inst->telephony.rxErrSecond;
	stats->netSyncLostNum = inst->packetNet.syncLostNum;
	stats->netBerCurrent = inst->packetNet.instantBER;
	stats->netRcvBits = inst->packetNet.rxBits;

	if(inst->packetNet.rxErrBitsSaturation)
	    stats->netErrBits = 0xffffffff;
	else
	    stats->netErrBits = inst->packetNet.rxErrBits;

	stats->netRcvMode = inst->packetNet.rxSync;
	if(inst->packetNet.rxOKFlag == 0)
		stats->netRcvMode |= MGB_BERT_TEL_RX_ERROR;

	stats->netErrSeconds = inst->packetNet.rxErrSecond;
	if(reset)
	{
		inst->telephony.syncLostNum =
		inst->telephony.rxBits =
		inst->telephony.rxErrBits =
		inst->telephony.rxErrBitsOld =
		inst->telephony.rxErrSecond =
		inst->packetNet.syncLostNum =
		inst->packetNet.rxBits = 
		inst->packetNet.rxErrBits = 
		inst->packetNet.rxErrBitsOld =
		inst->packetNet.rxErrSecond = 
		inst->telephony.rxErrBitsSaturation =
		inst->packetNet.rxErrBitsSaturation = 0;
	}
}

tint ber2samples(tint ber, tint measureInterval)
{
	tint	mantissa, exponent;
	tulong	scratch;

	exponent = ber & 0xff;
	exponent -= 3;					  /* mult 1k */
	mantissa = (ber >> 2) & 0x3fc0;   /* mantissa * 64 */
	scratch = measureInterval * mantissa;
	while((scratch >= 10) && (exponent >= 1))
	{
		scratch /= 10;
		exponent--;
	}
	if(scratch < 10 && exponent >= 1)
		return(1);
	else
		return((tint)scratch);
}

tuint bertGetModuleConfig(void **bertInst)
{
  bertInst_t *inst = (bertInst_t *)bertInst;

  return(inst->moduleConfig); /* set state to closed */ 

} /* bertClose */


/* nothing past this point */
