/************************************************************************
:*                                                                      *
:*     @ Copyright (C) 2009  Texas Instruments Incorporated.            *
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include "stdio.h"
#include "bert.h"
#include "bertloc.h"
#include "bertport.h"
#ifdef DIAG
#include "diagappif.h"
#else
#include "vca.h"
#include "vctypes.h"
#include "mgmicdsp.h"
#include "gmp.h"
#endif


tuint errSamples2ber(tulong errbits, tuint interval);
void bertStatusCheckSync(bertInterface_t *inst);
void bertStatusCheckAsync(bertInterface_t *inst);
/******************************************************************************
 * FUNCTION PURPOSE: generate scrambled 1s for network interface	
 ******************************************************************************
 * DESCRIPTION: generate scrambled 1s for network interface, 
 *              the output is in words.
 *
 *****************************************************************************/
void bertGenx14x15Net(bertInterface_t *inst, tint nBlockSize, tuint *outPtr)
{
	tint    i;
	tuint	tmp;
	tuint	scratch = inst->txScratch;

	for(i = 0; i < nBlockSize; i++)
	{
		tmp = scratch;
		tmp = (((tmp >> 7) ^ (tmp >> 6))) & 0xff; 
		if(i & 0x1)
		    *outPtr++ |= tmp;
		else
			*outPtr = tmp << 8;
		scratch = (scratch << 8) + tmp;
	}
	inst->txScratch = scratch;
}
/******************************************************************************
 * FUNCTION PURPOSE: generate scrambled 1s for telephony interface	
 ******************************************************************************
 * DESCRIPTION: generate scrambled 1s for telephony interface, 
 *              the output is in bytes (low byte of a 16bits word).
 *****************************************************************************/

void bertGenx14x15Tel(bertInterface_t *inst, tint nBlockSize, tuint *outPtr, tuint bitsPerSample)
{
	tint    i;
	tuint	tmp;
	tuint	bitMask = (1 << bitsPerSample) - 1;
	tuint	scratch = inst->txScratch;

	for(i = 0; i < nBlockSize; i++)
	{
		tmp = scratch;
		tmp = (((tmp >> (15 - bitsPerSample)) ^ (tmp >> (14 - bitsPerSample)))) & bitMask; 	
	    *outPtr++ = tmp << (8 - bitsPerSample);
		scratch = (scratch << bitsPerSample) + tmp;
	}
	inst->txScratch = scratch;
}

/******************************************************************************
 * FUNCTION PURPOSE: generate pattern 0~255 for network interface
 ******************************************************************************
 * DESCRIPTION: generate pattern 0~255 for network interface
 *              
 *****************************************************************************/

void bertGenPatternNet2(bertInterface_t *inst, tint nBlockSize, tuint *outPtr)
{
	tint    i;
	tuint	scratch = inst->txScratch;
	
	/* working in word base */
	nBlockSize >>= 1;

	for(i = 0; i < nBlockSize; i++)
	{
		*outPtr++ = (scratch << 8) | ((scratch + 1) & 0xff);
		scratch += 2;
		scratch &= 0xff;
	}
	inst->txScratch = scratch;
}
/******************************************************************************
 * FUNCTION PURPOSE: generate pattern 0~255 for network interface
 ******************************************************************************
 * DESCRIPTION: generate pattern 0~255 for network interface
 *              
 *****************************************************************************/

void bertGenPatternTel2(bertInterface_t *inst, tint nBlockSize, tuint *outPtr)
{
	tint    i;
	tuint	scratch = inst->txScratch;
	
	for(i = 0; i < nBlockSize; i++)
	{
		*outPtr++ = scratch++;
		scratch &= 0xff;
	}
	inst->txScratch = scratch;
}

/******************************************************************************
 * FUNCTION PURPOSE: check the error bits in input stream
 *****************************************************************************/
void bertUtilSeqCheckTel(tint nSize, tuint *inptr, tint bitsPerSample, 
						 tulong *error, tuint *scratch)
{
	tuint chkresult, i, j;
	tuint bitMask = (1 << bitsPerSample) - 1;

	if(nSize == 0) return;

	for(i = 0; i < nSize; i++)
	{
		chkresult = ((*scratch >> (15 - bitsPerSample)) 
				    ^ (*scratch >> (14 - bitsPerSample))
				    ^ (*inptr >> (8 - bitsPerSample))) & bitMask;
		*scratch = (*scratch << bitsPerSample) 
		                + (*inptr++ >> (8 - bitsPerSample));

		for(j = 0; j < bitsPerSample; j++)
		{
			if(chkresult & 0x1)
				(*error)++;
			chkresult >>= 1;
		}
	}
}

/******************************************************************************
 *
 * FUNCTION PURPOSE: check the error bits of input stream in sync mode
 *
 ******************************************************************************/
void bertUtilSeqMatchTel(tint nSize, tuint *inptr, tint bitsPerSample, 
					  tulong *error, tuint *scratch)
{
	tuint 	tmp, chkresult, i, j;
	tuint   bitMask = (1 << bitsPerSample) - 1;

	if(nSize == 0) return;

	for(i = 0; i < nSize; i++)
	{
		tmp = ((*scratch >> (14 - bitsPerSample)) 
			 ^ (*scratch >> (15 - bitsPerSample))) & bitMask;
		chkresult = tmp ^ (*inptr++ >> (8 - bitsPerSample));
		*scratch = (*scratch << bitsPerSample) + tmp;

		if(chkresult != 0)
		{
			for(j = 0; j < bitsPerSample; j++)
			{
				if(chkresult & 0x1)
					(*error)++;
				chkresult >>= 1;
			}
		}
	}
}

/******************************************************************************
 *
 * FUNCTION PURPOSE: check the error bits of input stream in sync mode
 *
 *****************************************************************************/
void bertUtilSeqMatchNet(tint nSize, tuint *inptr, tulong *error, tuint *scratch)
{
	tuint tmp, chkresult, i, j;

	for(i = 0; i < nSize; i++)
    {
		tmp = ((*scratch >> 6) 
				 ^ (*scratch >> 7) ) & 0xff; 

		chkresult = tmp ^ (*inptr >> 8);
		*scratch = (*scratch << 8) | tmp;

		tmp = ((*scratch >> 6) 
				 ^ (*scratch >> 7) ) & 0xff; 

		chkresult = (chkresult << 8) | (tmp ^ (*inptr++ & 0xff));
		*scratch = (*scratch << 8) | tmp;

		if(chkresult != 0)
		{
			for(j = 0; j < 16; j++)
			{
				if(chkresult & 0x1)
					(*error)++;
				chkresult >>= 1;
			}
		}
	}
}

/******************************************************************************
 *
 * FUNCTION PURPOSE: check the error bits of input stream in unsync mode
 *
 *****************************************************************************/
void bertUtilSeqCheckNet(tint nSize, tuint *inptr, tulong *error, tuint *scratch)
{
	tuint chkresult, i, j;


	for(i = 0; i < nSize; i++)
	{
		chkresult = (((*scratch >> 7) ^ (*scratch >> 6))) & 0xff 
		            ^ ((*inptr >> 8) & 0xff);

		*scratch <<= 8;
		*scratch |= (*inptr >> 8) & 0xff;

		chkresult <<= 8;
		chkresult |= (((*scratch >> 7) ^ (*scratch >> 6)) & 0xff) ^ (*inptr & 0xff);

		*scratch = *inptr++;

		if(chkresult != 0)
		{
			for(j = 0; j < 16; j++)
			{
				if(chkresult & 0x1)
					(*error)++;
				chkresult >>= 1;
			}
		}
	}
}
/******************************************************************************
 * FUNCTION PURPOSE: check the error bits in input stream
 ******************************************************************************
 * DESCRIPTION:  In sync mode, the function will run local scrambler, check 
 *               the input data with the local scramlber output. 
 *               In async mode, the function will run descrambler to find sync
 *
 *****************************************************************************/

void bertDecodeTel(bertInterface_t *inst, tint nBlockSize, tuint *inPtr, 
					tuint bitsPerSample)
{
	tint    i,j;
	tuint	chkresult, tmp, localScratch = inst->rxScratch;

	if(inst->rxSync)
	{
		inst->rxBits += bitsPerSample * nBlockSize;
		inst->rxShortCnt += nBlockSize;

		if(inst->testPattern == MGB_BERT_PATTERN_255)
		{
			for(i = 0; i < nBlockSize; i++)
			{
				if(localScratch++ != *inPtr++)
				{
					inst->rxErrBits += bitsPerSample;

				}
				localScratch &= 0xff;
			}
		}
		else if(inst->testPattern == MGB_BERT_PATTERN_RANDOM) 
		{
			bertUtilSeqMatchTel(nBlockSize, inPtr, 
							bitsPerSample, &inst->rxErrBits, &localScratch);
		}
		else
		{
			if(inst->id_sync_offset == (nBlockSize - 1))
			{
				chkresult = (inPtr[0] + (inPtr[nBlockSize - 1] << 8))
				            ^ inst->channelID;
				if(bitsPerSample == 7)
					chkresult &= 0xfefe;
				if(chkresult != 0)
				{
					for(j = 0; j < 16; j++)
					{
						if(chkresult & 0x1)
							inst->rxErrBits++;
						chkresult >>= 1;
					}
				}
				inPtr++;
				bertUtilSeqMatchTel(nBlockSize - 2, inPtr, bitsPerSample,
				                    &inst->rxErrBits, &localScratch);
			}
			else
			{

				bertUtilSeqMatchTel(inst->id_sync_offset, inPtr, 
								bitsPerSample, &inst->rxErrBits, &localScratch);
				inPtr += inst->id_sync_offset;

				chkresult = (*inPtr++ << 8);
				chkresult |= *inPtr++;
				chkresult ^= inst->channelID;

				if(bitsPerSample == 7)
					chkresult &= 0xfefe;

				if(chkresult != 0)
				{
					for(j = 0; j < 16; j++)
					{
						if(chkresult & 0x1)
							inst->rxErrBits++;
						chkresult >>= 1;
					}
				}

				bertUtilSeqMatchTel(nBlockSize - 2 - inst->id_sync_offset, inPtr,
									bitsPerSample, &inst->rxErrBits, &localScratch);
			}
		}

		if(inst->rxShortCnt >= 8000)
		{
			inst->rxShortCnt -= 8000;
			bertStatusCheckSync(inst);
			inst->rxErrBitsOld = inst->rxErrBits;
		}
	}
	else /* un-sync */
	{	
		inst->rxBits += bitsPerSample * nBlockSize;
		inst->rxShortCnt += nBlockSize;

		if(inst->testPattern == MGB_BERT_PATTERN_255)
		{
			for(i = 0; i < nBlockSize; i++)
			{
				if(localScratch++ != *inPtr)
					inst->rxErrBits += bitsPerSample;

				localScratch = (*inPtr++ + 1) & 0xff;
			}
		}
		else if(inst->testPattern == MGB_BERT_PATTERN_RANDOM)
		{
			bertUtilSeqCheckTel(nBlockSize, inPtr, 
							bitsPerSample, &inst->rxErrBits, &localScratch);
		}
		else
		{
			if(inst->sync_id_counter > 5)
			{ /* channel ID matched, try to sync the pattern */
				if(inst->id_sync_offset == (nBlockSize - 1))
				{	/* idL ..........idH */
					chkresult = (inPtr[0] + (inPtr[nBlockSize - 1] << 8))
					            ^ inst->channelID;
					if(bitsPerSample == 7)
						chkresult &= 0xfefe;
					if(chkresult != 0)
					{
						for(j = 0; j < 16; j++)
						{
							if(chkresult & 0x1)
								inst->rxErrBits++;
							chkresult >>= 1;
						}
					}
					inPtr++;
					bertUtilSeqCheckTel(nBlockSize - 2, inPtr, bitsPerSample,
					                    &inst->rxErrBits, &localScratch);
				}
				else
				{	/* ......idH idL........*/
					bertUtilSeqCheckTel(inst->id_sync_offset, inPtr, bitsPerSample,
					                    &inst->rxErrBits, &localScratch);
					inPtr += inst->id_sync_offset;

					chkresult = (*inPtr++ << 8);
					chkresult |= *inPtr++;
					chkresult ^= inst->channelID;

					if(bitsPerSample == 7)
						chkresult &= 0xfefe;

					if(chkresult != 0)
					{
						for(j = 0; j < 16; j++)
						{
							if(chkresult & 0x1)
								inst->rxErrBits++;
							chkresult >>= 1;
						}
					}

					bertUtilSeqCheckTel(nBlockSize - (2 + inst->id_sync_offset), inPtr,
										bitsPerSample, &inst->rxErrBits, &localScratch);
				}
			}
			else
			{ /* looks for channel ID */
				tmp = inst->channelID >> 8;
				chkresult = inst->sync_id_counter;

				if((inst->last_byte == tmp) && (*inPtr == (inst->channelID & 0xff)))
				{
					if(inst->id_sync_offset == (nBlockSize - 1))
						inst->sync_id_counter++;
					else
						inst->sync_id_counter = 1;
				}

				for(i = 0; i < nBlockSize - 1; i++)
				{
					if(*inPtr++ == tmp)
						if(*inPtr == (inst->channelID & 0xff))
							if(i == inst->id_sync_offset)
								inst->sync_id_counter++;
							else
							{
								inst->sync_id_counter = 1;
								inst->id_sync_offset = i;
							}		

				}
				inst->last_byte = *inPtr;
				if(chkresult == inst->sync_id_counter)
					inst->sync_id_counter = 0;

				inst->rxErrBits += bitsPerSample * nBlockSize;
			}
		}
	}

	/* check if the descrambler is re-synced in each second */
	if(inst->rxShortCnt >= 8000)
	{
		inst->rxShortCnt -= 8000;
		bertStatusCheckAsync(inst);
		inst->rxErrBitsOld = inst->rxErrBits;
	}

	/* scratch health check 
	 * for 2n15 pseudo random sequence, scratch may converge to 16 consecutive 1s 
	 * set scratch to 0 when detects more 16 consecutive 1s 
	 */
	if((inst->testPattern == MGB_BERT_PATTERN_RANDOM) && (localScratch == 0))
		localScratch = 0xffff;

	inst->rxScratch = localScratch;
}

/******************************************************************************
 * FUNCTION PURPOSE: check the error bits in input stream for network interface
 ******************************************************************************
 * DESCRIPTION:  In sync mode, the function will run local scrambler, check 
 *               the input data with the local scramlber output. 
 *               In async mode, the function will run descrambler to find sync
 *
 *****************************************************************************/

void bertDecodeNet(bertInterface_t *inst, tuint nBlockSize, tuint *inPtr)
{
	tuint   i,j;
	tuint	chkresult, tmp, localScratch = inst->rxScratch;

    /* convert to words */
	nBlockSize = nBlockSize / 2;

#ifdef LITTLE_ENDIAN
    /* Convert incoming byte stream to 16 bit endian corrected*/
    utlBigEndianTransform(inPtr, nBlockSize);
#endif

    if(inst->rxSync)
	{
 		inst->rxBits += 16 * nBlockSize;
		inst->rxShortCnt += 2 * nBlockSize;

		if(inst->testPattern == MGB_BERT_PATTERN_255)
		{	/* pattern (0 ~ 255) */
 			for(i = 0; i < nBlockSize; i++)
	        {
				tmp = *inPtr >> 8;
				if(tmp != localScratch++)
					inst->rxErrBits += 8;
				localScratch &= 0xff;

				tmp = *inPtr++ & 0xff;
				if(tmp != localScratch++)
					inst->rxErrBits += 8;
				localScratch &= 0xff;
			}
		}
		else if(inst->testPattern == MGB_BERT_PATTERN_RANDOM)
		{
			bertUtilSeqMatchNet(nBlockSize, inPtr, &inst->rxErrBits, &localScratch); 
		}
		else
		{  /* random sequence with channel ID */
		   /* channel ID is always the first word of a frame */ 
			chkresult = inst->channelID ^ (*inPtr++);   //TODO_BERT 16bit channel id ?
			if(chkresult != 0)
			{
			    for(j = 0; j < 16; j++)
				{
				    if(chkresult & 0x1)
					    inst->rxErrBits++;
					chkresult >>= 1;
				}
			}

			bertUtilSeqMatchNet(nBlockSize - 1, inPtr, &inst->rxErrBits,
			                    &localScratch); 
		}

/* 		if(inst->rxShortCnt >= 8000)
		{
			inst->rxShortCnt -= 8000;
			bertStatusCheckSync(inst);
			inst->rxErrBitsOld = inst->rxErrBits;
		} */
	}
	else   /* in un-sync mode */
	{
		inst->rxBits += 16 * nBlockSize;
		inst->rxShortCnt += 2 * nBlockSize;

		if(inst->testPattern == MGB_BERT_PATTERN_255)
		{	/* pattern (0 ~ 255) */
 			for(i = 0; i < nBlockSize; i++)
	        {
			    if((*inPtr >> 8) != (localScratch & 0xff))
				    inst->rxErrBits += 8;

			    if((*inPtr & 0xff) != (((*inPtr >> 8) + 1) & 0xff)) 
				    inst->rxErrBits += 8;

			    localScratch = (*inPtr++ + 1) & 0xff; 
            }
		}
		else if(inst->testPattern == MGB_BERT_PATTERN_RANDOM)
		{   /* pseudo random sequence */
			/* use de-scrambler to find sync pattern, once the output of
			   de-scrambler are 1s in most of time in one second, the sync is 
			   found and decoder will transfer to sync mode*/
			bertUtilSeqCheckNet(nBlockSize, inPtr, &inst->rxErrBits, &localScratch); 
		}
		else
		{
		    /* go to sequence match after 5 consecutive ID matches. */
		    if(inst->sync_id_counter >= 5)
			{
			    chkresult = *inPtr++ ^ inst->channelID;

				if(chkresult != 0)
				{
					for(j = 0; j < 16; j++)
					{
						if(chkresult & 0x1)
							inst->rxErrBits++;
						chkresult >>= 1;
					}
				}

				bertUtilSeqCheckNet(nBlockSize - 1, inPtr, &inst->rxErrBits, 
			                        &localScratch); 
			}
			else
			{
			    if(*inPtr ^ inst->channelID == 0)
			        inst->sync_id_counter++;
				else
				    inst->sync_id_counter = 0;
				inst->rxErrBits += 16 * nBlockSize;
			}

		}

		/* check if the descrambler is re-synced */
/*		if(inst->rxShortCnt >= 8000)
		{
			inst->rxShortCnt -= 8000;
			bertStatusCheckAsync(inst);
			inst->rxErrBitsOld = inst->rxErrBits;
		} */

	}

	/* scratch health check 
	 * for 2n15 pseudo random sequence, scratch may converge to 16 consecutive 1s 
	 * set scratch to 0xffff when detects more 16 consecutive 0s 
	 */
	if((inst->testPattern == MGB_BERT_PATTERN_RANDOM) && (localScratch == 0))
	    localScratch = 0xffff;
	inst->rxScratch = localScratch;
}

/******************************************************************************
 * FUNCTION PURPOSE: update the status in sync mode
 ******************************************************************************
 * DESCRIPTION:  In sync mode, this function will called per second to check 
 *               if there is a error ok transition or sync lost event
 *
 *****************************************************************************/
void bertStatusCheckSync(bertInterface_t *inst)
{

		/* saturation */	
	if((inst->rxErrBits < inst->rxErrBitsOld) && (inst->rxRunning))
 	    inst->rxErrBitsSaturation = 1;

	if(inst->rxErrBits > inst->rxErrBitsOld)
	{
		inst->rxErrSecond++;
		inst->rxErrLongCnt += inst->rxErrBits - inst->rxErrBitsOld;

		/* if ber is bigger than 0.2 in one second, go to NOSYNC mode */
		if(inst->rxErrBits - inst->rxErrBitsOld >= 8 * 1600)
		{
			inst->rxSync = FALSE;
			inst->rxOKFlag = FALSE;
			inst->rxStateTransition = TRUE;
			inst->sync_id_counter = 0;
			inst->id_sync_offset = 0;
			inst->syncLostNum++;     /* sync lost */
			return;
		}
	}
	if(--(inst->rxIntervalCounter) <= 0)
	{
		inst->rxIntervalCounter = inst->measureInterval;

		/* calculate the instant BER */
		inst->instantBER = errSamples2ber(inst->rxErrLongCnt, inst->measureInterval);

		if(inst->rxOKFlag)			/* current state is OK */
		{
			if(inst->rxErrLongCnt >= inst->upperThreshold)
			{	/* has transition */
				inst->rxOKFlag = FALSE;	/* switch to ERROR state */
				inst->rxStateTransition = TRUE;
/*				if(inst->statsReportMode == BERT_REPORT_TRANSITION)
					inst->reportFlag = BERT_REPORT_REASON_OK2ERR; */
				inst->rxOK2Err++;
			}
		}
		else
		{
			if(inst->rxErrLongCnt <= inst->lowerThreshold)
			{	/* has transition */
				inst->rxOKFlag = TRUE;
				inst->rxStateTransition = TRUE;
/* 				if(inst->statsReportMode == BERT_REPORT_TRANSITION)
					inst->reportFlag = BERT_REPORT_REASON_ERR2OK; */
				inst->rxErr2OK++;
			}
		}
		inst->rxErrLongCnt = 0;
	}
}

/******************************************************************************
 * FUNCTION PURPOSE: update the status in nosync mode
 ******************************************************************************
 * DESCRIPTION:  In nosync mode, this function will called per second to check 
 *               if there is a get sync event
 *
 *****************************************************************************/

void bertStatusCheckAsync(bertInterface_t *inst)
{
		/* saturation */	
	if((inst->rxErrBits < inst->rxErrBitsOld) && (inst->rxRunning))
 	    inst->rxErrBitsSaturation = 1;

    if(--(inst->rxIntervalCounter) <= 0)
    {
	    inst->rxIntervalCounter = inst->measureInterval;
	    /* calculate the instant BER */
	    inst->instantBER = errSamples2ber(inst->rxErrLongCnt, inst->measureInterval);
	}

	inst->rxShortCnt = 0;
	inst->rxErrSecond++;

	if(inst->rxErrBits - inst->rxErrBitsOld <= 8 * 100)
	{
		inst->rxSync = TRUE;
/* 		inst->rxIntervalCounter = inst->rxMeasureInterval; */
		inst->rxOKFlag = TRUE;
		inst->rxErrLongCnt = 0;
		inst->rxStateTransition = TRUE;

		if(!inst->rxRunning)
		{
			inst->rxRunning = TRUE;
			inst->rxBits = 
			inst->rxErrBits=
			inst->rxErrSecond = 
			inst->rxErr2OK = 
			inst->rxOK2Err = 
			inst->syncLostNum = 0;
			inst->rxIntervalCounter = inst->measureInterval;
		}
	}
}


tuint errSamples2ber(tulong errbits, tuint interval)
{
	tulong	scratch;
	tuint	result, mantissa, exponent = 0;

	if(errbits == 0)
		return(0);
	scratch = interval * 64000;
	while(errbits <= scratch)
	{
		errbits *= 10;
		exponent++;
	};

	mantissa = errbits / scratch;
	result = (mantissa << 8) + exponent;
	return(result);
}

/******************************************************************************
 * FUNCTION PURPOSE: receive data from vpu buffer
 ******************************************************************************
 * DESCRIPTION:  check the frame lost info and compensate the frame lost,
 *               the check the error bits in the received frame.
 *
 *****************************************************************************/
void bertReceiveIn(void **bertInst, tint frame_type, tint cwseg_size, void *cw_in,
                   tint noise_level, tint fifo_size, tint seq, void **decode_out)
{
	tint		j;
	tuint		*inPtr = (tuint *)cw_in;
	bertInst_t	*inst = (bertInst_t *)bertInst;
	tuint       tmp;
	tuint       scratch = inst->packetNet.rxScratch;


    /* legnth IDLE packet is 0 */
    if(cwseg_size == 0)
		cwseg_size = inst->sys_blk_size;

    	cwseg_size >>= 3;   /* bytes in one packet or frame*/

	// not process any non-voice packet
	if(inst->moduleConfig & MGB_BERT_NET_RX_ENABLE)
	{
		if(frame_type == VCA_PACKET_REPLAY || frame_type == VCA_PACKET_IDLE)
		{
			if(inst->packetNet.testPattern == MGB_BERT_PATTERN_255)
			{
				scratch += cwseg_size;
				scratch &= 0xff;
			}
			else if(inst->packetNet.testPattern == 	MGB_BERT_PATTERN_RANDOM)
			{
				for(j = 0; j < cwseg_size; j++)
				{	
					tmp = ((scratch >> 6) 
						 ^ (scratch >> 7) ^ 0xff) & 0xff;
					scratch = (scratch << 8) + tmp;
				}
			}
			else
			{
			    for(j = 0; j < cwseg_size - 2; j++)
				{
					tmp = ((scratch >> 6) 
						 ^ (scratch >> 7) ^ 0xff) & 0xff;
					scratch = (scratch << 8) + tmp;
				}
			}
            
			if(0xffffffff - inst->packetNet.rxErrBits <= cwseg_size * 8)
			    inst->packetNet.rxErrBitsSaturation = 1;
			else
			    inst->packetNet.rxErrBits += cwseg_size * 8;
			inst->packetNet.rxBits += cwseg_size * 8;
      		inst->packetNet.rxScratch = scratch;
			inst->packetNet.rxShortCnt += cwseg_size;
		}
		else if(frame_type == VCA_PACKET_MORE_ON || frame_type == VCA_PACKET_MORE_OFF)
		{
		    bertDecodeNet(&inst->packetNet, cwseg_size, inPtr);
		}

		if(inst->packetNet.rxShortCnt >= 8000)
		{
			inst->packetNet.rxShortCnt -= 8000;

			if(inst->packetNet.rxSync)
			    bertStatusCheckSync(&inst->packetNet);
			else
				bertStatusCheckAsync(&inst->packetNet);

			inst->packetNet.rxErrBitsOld = inst->packetNet.rxErrBits;
		}

	}

    /* packet received during a period of time */
	inst->net_packet_click++;

#ifndef DIAG

    if(frame_type != VCA_PACKET_REPLAY && frame_type != VCA_PACKET_CLOCK)
	{
 		/* free the packet buffer in GMP, no matter BERT RX is enabled or not */
 		gmpFreeGmc (BERT_GMC_ALLOCATOR | (inst->ID & 0x00ff), cw_in);  
	};
#endif

}

/******************************************************************************
 * FUNCTION PURPOSE: creat pseudo random sequence to the output buffer
 ******************************************************************************
 * DESCRIPTION:  In bert mode, this function will be called to replace the output
 *               data of TGU.
 *
 *****************************************************************************/
void bertPatternGenTel(void **bertInst, tuint frame_size, void *frame_ptr)
{
	bertInst_t *inst = (bertInst_t *)bertInst;
	tuint *out_ptr = (tuint *)frame_ptr;
	if(inst->moduleConfig & MGB_BERT_TELE_TX_ENABLE)
	{
		if(inst->telephony.testPattern == MGB_BERT_PATTERN_255)
			bertGenPatternTel2(&inst->telephony, frame_size, out_ptr);
		else if(inst->telephony.testPattern == MGB_BERT_PATTERN_RANDOM)
			bertGenx14x15Tel(&inst->telephony, frame_size, out_ptr, inst->bitsPerSample);
		else
		{
			*out_ptr++ = ((inst->telephony.channelID & 0xff00) >> 8);
			*out_ptr++ = (inst->telephony.channelID & 0xff);
			bertGenx14x15Tel(&inst->telephony, frame_size - 2, 
			                 out_ptr, inst->bitsPerSample);
		}
	}
}


/*******************************************************************************
 * FUNCTION PURPOSE: Check report state
 *******************************************************************************/
void bert_check_report (bertInst_t *inst, tuint block_size)
{
    tuint reportFlag = 0;

	if((inst->telephony.rxStateTransition)
	   && (inst->statsReportMode & MGB_BERT_STATS_TRANS_TEL))
	{
	    inst->telephony.rxStateTransition = FALSE;
		reportFlag |= (MGB_BERT_REPORT_TRANSITION | MGB_BERT_REPORT_TEL_SIDE);
	}

	if((inst->packetNet.rxStateTransition)
	   && (inst->statsReportMode & MGB_BERT_STATS_TRANS_NET))
	{
	    inst->packetNet.rxStateTransition = FALSE;
		reportFlag |= (MGB_BERT_REPORT_TRANSITION | MGB_BERT_REPORT_NET_SIDE);
	}

	inst->sys_timer -= block_size;
	if(inst->sys_timer <= 0)
	{
	    inst->sys_timer += 8000;

		/* check if network rx is still running */
		if((inst->moduleConfig & MGB_BERT_NET_RX_ENABLE)
			&& (inst->net_packet_click <= 30))
		{	/* rx of network is dead */
			if(inst->packetNet.rxSync)
				inst->packetNet.syncLostNum++;
			inst->packetNet.rxSync = FALSE;
			inst->packetNet.rxOKFlag = FALSE;
			inst->packetNet.rxErrBitsOld += 64000;
			inst->packetNet.rxErrBits += 64000;
			inst->packetNet.rxBits += 64000;
			inst->packetNet.rxErrSecond++;
		}
		inst->net_packet_click = 0;

	    /* update stats report timer, send stats if needed */
	    if((inst->statsReportMode & MGB_BERT_STATS_PERIODIC) != 0)
		{
		    if(--inst->report_timer <= 0)
			{
				inst->report_timer = inst->prdRptInterval;
                
				if((inst->moduleConfig & MGB_BERT_TELE_RX_ENABLE) 
				   ||(inst->moduleConfig & MGB_BERT_NET_RX_ENABLE))
				{
    	 			reportFlag |= MGB_BERT_REPORT_PERIODIC;
                    
				    if(inst->moduleConfig & MGB_BERT_TELE_RX_ENABLE)
					    reportFlag |= MGB_BERT_REPORT_TEL_SIDE;
	                    
				    if(inst->moduleConfig & MGB_BERT_NET_RX_ENABLE)
					    reportFlag |= MGB_BERT_REPORT_NET_SIDE;
				}
			}
		}
	}
	if(reportFlag != 0)
		(inst->statsReportFn)(inst->statsReportTargetInst, 0, 0, reportFlag, 0, FALSE);
}

/******************************************************************************
 * FUNCTION PURPOSE: Decode Tel pattern
 ******************************************************************************/
void bertPatternDetectTelFcn (void **bertInst, void *cw, tuint block_size)
{
	bertInst_t	*inst = (bertInst_t *)bertInst;
	tuint	*inPtr = (tuint *)cw;

    inst->sys_blk_size = block_size << 3;

	if(inst->moduleConfig & MGB_BERT_TELE_RX_ENABLE)
		bertDecodeTel(&inst->telephony, block_size, inPtr, inst->bitsPerSample);

    bert_check_report (inst, block_size);

} 


/******************************************************************************
 * FUNCTION PURPOSE: check telephony received data and send packet to network		
 *****************************************************************************/
void	bertSendIn(void **bertInst, void *cw, void *oPtr, tuint block_size)
{
	bertInst_t	*inst = (bertInst_t *)bertInst;
	tuint	*inPtr = (tuint *)cw;
	tuint   *output = (tuint *)oPtr;

    inst->sys_blk_size = block_size << 3;

	if(inst->moduleConfig & MGB_BERT_TELE_RX_ENABLE)
		bertDecodeTel(&inst->telephony, block_size, inPtr, inst->bitsPerSample);

	if(inst->moduleConfig & MGB_BERT_NET_TX_ENABLE)
	{
		if(inst->packetNet.testPattern == MGB_BERT_PATTERN_255)
			bertGenPatternNet2(&inst->packetNet, block_size, output);
		else if(inst->packetNet.testPattern == MGB_BERT_PATTERN_RANDOM)
			bertGenx14x15Net(&inst->packetNet, block_size, output);
		else
		{	/* random sequence with channel ID */
			*output++ = inst->packetNet.channelID;
			bertGenx14x15Net(&inst->packetNet, block_size - 2, output);
		}	 
#ifdef LITTLE_ENDIAN
              /* Convert incoming byte stream to 16 bit endian corrected*/
              utlBigEndianTransform(output, block_size>>1);
#endif
        
	}


    bert_check_report (inst, block_size);

}


/* end of file */
