#!/usr/bin/perl

#******************************************************************************
#* FILE PURPOSE: Block expansion preprocessor for Telogy memory maps
#******************************************************************************
#* FILE NAME: blockexpn.pl
#*
#* DESCRIPTION: This utility adds the following preprocessor operations
#*              used to expand Telogy memory maps while reducing human error.
#*              
#*              The following operations are supported when defined
#*              inside a C comment:
#*
#*              /* foreach $var in (low..high) expand */
#*              .
#*              .
#*              /* endfor $var */
#*
#*              This operation textually expands the block between the
#*              foreach and endfor lines.  All occurances of $(var)
#*              are replaced, in turn, with each value between "low"
#*              and "high", inclusive.  The variable name is user-defined
#*              and must match the corresponding endfor.  Foreach loops
#*              can be nested to any depth.
#* 
#*              After all expansions are made, the resulting file is
#*              searched for expressions of the form:
#*               
#*              $(1+(2+3)*4)
#* 
#*              All such expressions are replaced with the evaluated value
#*              iff there are no variables/text remaining in the expression.  
#*              If variables or text are found, the expression is left in the 
#*              output.  Numbers  may be in decimal or hex (prefixed by 0x).  
#*              The supported operators are: (, ), +, -, /, *, %, &, |, ~, ^.  
#*              Each operator and precedence follows that of C or Perl.
#*     
#*              The command line invocation syntax (which may be driven
#*              by a makefile) is:
#* 
#*              ccperl blockexpn.pl [-d] input.ext output.ext
#* 
#*              The option -d disables the deletion of output.ext when
#*              errors are encountered.  The script will return
#*              a nonzero exit value on failure, to stop make.  
#*
#* (C) Copyright 2002, Texas Instruments Incorporated.
# * 
# *  Redistribution and use in source and binary forms, with or without 
# *  modification, are permitted provided that the following conditions 
# *  are met:
# *
# *    Redistributions of source code must retain the above copyright 
# *    notice, this list of conditions and the following disclaimer.
# *
# *    Redistributions in binary form must reproduce the above copyright
# *    notice, this list of conditions and the following disclaimer in the 
# *    documentation and/or other materials provided with the   
# *    distribution.
# *
# *    Neither the name of Texas Instruments Incorporated nor the names of
# *    its contributors may be used to endorse or promote products derived
# *    from this software without specific prior written permission.
# *
# *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
# *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
# *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
# *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
# *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
# *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
# *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
# *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
# *
# */

$infile = "";
$outfile = "";
$filecnt = 0;
$keep_output = 0;
$error = 0;

foreach $arg (@ARGV) {
  if ($arg eq "-d") {
    $keep_output = 1; 
  } else {
    $infile = $arg if ($filecnt == 0); 
    $outfile = $arg if ($filecnt == 1);
    $numchans = $arg if ($filecnt == 2);
    $numslots = $arg if ($filecnt == 3);
    if ($filecnt >= 4) {
      @asn = split (/=/,$arg);
      if ($#asn != 1) {
        $error=1;
      }
      $vars{$asn[0]}=$asn[1];
    }
    $filecnt++;
  }
}

die "Syntax: $0 [-d] input output [numchans numslots [var=val] [var2=val]]\n" if ($error || ($filecnt < 2));

open INPUT, $infile or die "Can't open input $infile\n";
open OUTPUT, ">$outfile" or die "Can't open input $outfile\n";

@source = <INPUT>;
close INPUT;

$line = 0;
$idx = 0;

# blockStartLine, blockStartIdx, blockStopLine, blockStopIdx
# for open foreach ... expand blocks
@begins = ();


$STATE_OPENCOMMENT = 1;
$STATE_BUILDCOMMENTBODY = 2;
$state = $STATE_OPENCOMMENT;

while ($line <= $#source) {
  if ($state == $STATE_OPENCOMMENT) {
    # look for comments
    $commentBeginIdx = index $source[$line], "/*", $idx;
    if ($commentBeginIdx < $[) {
       $line             ++; 
       $idx              = 0;
    } else {
       $state            = $STATE_BUILDCOMMENTBODY;
       $idx              = $commentBeginIdx + 2;
       $commentBeginLine = $line;
       $commentBody      = "";
    }
  } elsif ($state == $STATE_BUILDCOMMENTBODY) {
    $commentEndIdx = index $source[$line], "*/", $idx;
    if ($commentEndIdx < $[) {
      $commentBody      .= substr $source[$line], $idx; 
      $line              ++;
      $idx               = 0;
    } else {
      $commentBody      .= substr $source[$line], $idx, $commentEndIdx - $idx;
      $idx               = $commentEndIdx + 2;
      $state             = $STATE_OPENCOMMENT;

      # We have a whole comment in $commentBody now
      if ($commentBody =~ /foreach\s+\$([a-zA-Z][a-zA-Z0-9_]*)
                           \s+in\s+\(\s*(\d+|[a-zA-Z][a-zA-Z0-9]+)\s*\.\.\s*(\d+|[a-zA-Z][a-zA-Z0-9]+)\s*\)\s+expand/x) {
        # begin block found, push its properties onto the stack
        $endlimit = $3;
        $startlimit = $2;
        $varname = $1;
        if ($endlimit eq "NUMCH") {
          $endlimit = $numchans;
        }
        if ($endlimit eq "NUMSL") {
          $endlimit = $numslots;
        }
        if ($endlimit eq "slotOrlastregch") {
          if(defined $numslots) {
            if (!defined $vars{"lastregch"}) {
              die "Variable lastregch not defined\n"; 
            }
            if ($numslots != $numchans) {
              $endlimit = $numslots;
            } else {
              $endlimit = $vars{"lastregch"};
            }
          } else {
            die "numslots not defined\n";
          }
        }
        if (defined $numslots) {
          if ($endlimit > $numslots) {
            $endlimit = $numslots;
          }
        } elsif (defined $numchans) {
          if ($endlimit > $numchans) {
            $endlimit = $numchans;
          }
        }
        if ($endlimit =~ /^[a-zA-Z]/) {
          if (!defined $vars{$endlimit}) {
            die "Variable $endlimit not defined\n"; 
          }
          $endlimit = $vars{$endlimit};
        }
        if ($startlimit =~ /^[a-zA-Z]/) {
          if (!defined $vars{$startlimit}) {
            die "Variable $startlimit not defined\n"; 
          }
          $startlimit = $vars{$startlimit};
        }

        push @begins, [$commentBeginLine, $commentBeginIdx, $line, 
                       $commentEndIdx + 2, $varname, $startlimit, $endlimit];
      } elsif ($commentBody =~ /endfor\s+\$([a-zA-Z][a-zA-Z0-9_]*)/) {
        # end block found, perform substitutions
        ($line, $idx) = substitute_fcn ($commentBeginLine, $commentBeginIdx, 
                                        $line, $commentEndIdx + 2, $1);
      }
    }
  }
}

# Check that all open foreach are closed
for ($i=0;$i<=$#begins;$i++) {
  $beginref = $begins[$i];
  $errorline = $$beginref[0]+1;
  print "Unmatched foreach \$$$beginref[4] on line $errorline\n";
}
error_exit ("Unmatched foreach errors.") if ($#begins>=0);

# Evaluate constant expressions inside $(..)
evaluate_fcn ();

# Write results
print OUTPUT @source;
close OUTPUT;

# Successful completion - 0 keeps make going
exit (0);

sub substitute_fcn
{
   my ($lastExpLine, $lastExpIdx, $lastCommentLine, $lastCommentIdx, $var) = @_;
   my ($i, $beginref, @expanded, $expline, $line, $idx, $linedata);
   my ($errorLine1, $errorLine2);

   
   # The end has no match
   $errorLine1 = $lastExpLine + 1;
   error_exit ("\"endfor \$$var\" on line $errorLine1 ".
               "has no matching foreach") if ($#begins < 0);
   
   $beginref = pop @begins;
   
   # The variable in the begin block mush match the variable in the end block
   $errorLine1 = $lastExpLine + 1;
   $errorLine2 = $$beginref[0] + 1;
   error_exit ("\"endfor \$$var\" on line $errorLine1 does not match" .
               " \"foreach $$beginref[4]\" on line $errorLine2.") if 
              ($$beginref[4] ne $var);
          
   $expline = 0;
   for ($i = $$beginref[5];$i<=$$beginref[6];$i++) {
     $expanded[$expline++] = "/* Expanding \$$var = $i */";
     
     # expand everything between the comments 
     $line = $$beginref[2];
     $idx = $$beginref[3];
     while ($line < $lastExpLine) {
       $linedata = substr $source[$line], $idx; 
       $linedata =~ s/\$\($var\)/$i/g;
       $expanded[$expline++] = $linedata;
       $line++;
       $idx = 0;
     }
     $linedata = substr $source[$line], $idx, $lastExpIdx - $idx; 
     $linedata =~ s/\$\($var\)/$i/g;
     $expanded[$expline++] = $linedata;
     $expanded[$expline++] = "/* End expansion \$$var = $i */\n";
   }
   
   # put the expanded data back into the input array
   $firstline = substr $source[$$beginref[0]], 0, $$beginref[1] - $[;
   $lastline  = substr $source[$lastCommentLine], $lastCommentIdx;
   @source = (@source[0..($$beginref[0]-1)], $firstline, @expanded, $lastline, 
              @source[($lastCommentLine + 1)..$#source]);
              
   # Point to end of expanded data
   return ($#expanded + 2 + $$beginref[0], 0);
} # substitute_fcn

sub evaluate_fcn
{
  my ($i, $data, $operators, $decd, $hexd, $startIdx, $idx, $openParens, 
      $chr, $exp, $val, $reduced);
  
  $hexd = "0x[0-9a-fA-F]+";
  $decd = "\\d+";
  $operators = "([()+-/*%&|~^]|\\s)*";
  
  for ($i=0;$i<=$#source;$i++) {
    do {
      $reduced = 0; # Nothing has been reduced/evaluated on this line
      # Locate expressions of $(..) which only contain decimal or hex numbers,
      # space, and operators.
      if ($source[$i] =~ /\$\([^\)]+\)/ ) {
        # Possible expression -- handle nested parens
        $idx = index $source[$i], "\$("; 
        if ($idx >= $[) {
  	      $openParens = 1;
          $idx = $startIdx = $idx + 2;
          while ($openParens && ($idx < length($source[$i]))) {
            $chr = substr $source[$i], $idx++, 1;
            if ($chr eq "(") {
              $openParens++; 
            } elsif ($chr eq ")") {
              $openParens--;
            }
          }
          if (! $openParens) {
            # This is a potentially valid expression	
            $exp = substr $source[$i], $startIdx, $idx - $startIdx - 1;		  
            # Check for valid expression (no variables)
            if ($exp =~ /^(${operators}($hexd|$decd){1})+(\s|\))*$/) {
              $val = eval $exp;
              $reduced = 1;
              error_exit ("Cannot eval expression ($exp) on line $i of expanded ".
                          "output.\nUse command line option -d to not delete ".
                          "output file to debug this.") if (! defined $val);
              substr ($source[$i], $startIdx-2, $idx - $startIdx + 2) = "$val";
            }
          }
        }
      }
    } while ($reduced);
  }
} # evaluate_fcn

sub error_exit
{
  if ($keep_output) {
    print OUTPUT @source;
    close OUTPUT;
  } else {
    close OUTPUT;
    unlink "$ARGV[1]"; 
  }
  $! = 1; # Nonzero return value to kill make
  die "$_[0]\n";
} # error_exit

# Nothing past this point
