#*
#*
#* Copyright (C) 2010 Texas Instruments Incorporated - http://www.ti.com/ 
#* 
#* 
#*  Redistribution and use in source and binary forms, with or without 
#*  modification, are permitted provided that the following conditions 
#*  are met:
#*
#*    Redistributions of source code must retain the above copyright 
#*    notice, this list of conditions and the following disclaimer.
#*
#*    Redistributions in binary form must reproduce the above copyright
#*    notice, this list of conditions and the following disclaimer in the 
#*    documentation and/or other materials provided with the   
#*    distribution.
#*
#*    Neither the name of Texas Instruments Incorporated nor the names of
#*    its contributors may be used to endorse or promote products derived
#*    from this software without specific prior written permission.
#*
#*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
#*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
#*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
#*  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
#*  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
#*  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
#*  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
#*  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
#*  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
#*  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
#*  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#*


#!/usr/bin/perl

use warnings;
use strict 'vars';

if ($#ARGV < 3) {
  print STDERR "Syntax: $0 [-f] infozip_unzip scratch_dir output_dir input.txt\n".
               "        -f: ignore statfile's in input.txt\n".
               "        infozip_unzip: path to infozip's unzip executable\n".
               "        scratch_dir: temp directory to extract zips\n".
               "        output_dir: reformatted output directory\n".
               "        input.txt: list of one or more zips to extract\n\n";
  print STDERR "Syntax of input file:\n".
               " zipfile.zip [-sstatfile.txt] [-rdir1 [-rdir2]] # [regex1 [regex2]]\n".
               "   zipfile.zip: zip file to consider\n".
               "   -sstatfile.txt: file [relative to output_dir] to hold fstat\n".
               "                   information to avoid redundant unzip.  Must\n".
               "                   NOT be a pure number\n".
               "   -rdir: directory (or file) to delete (unless statfile.txt matches)\n".
               "   -s and -r can appear anywhere in any order\n".
               "   #: number of initial directories to drop from each file\n".
               "   regex#: regular expression in perl s/from/to/ syntax\n".
               "           to modify each filename in zipfile.zip.\n";
  exit(1);
}

my $arg;
my $ignoreStat = 0;
if ($ARGV[0] eq "-f") {
  $ignoreStat = 1;
  shift @ARGV;
}
my ($unzip, $scratch, $outputdir, $script) = @ARGV;

# Make paths uniform (unix) format
fixpath (\$unzip);
fixpath (\$scratch);
fixpath (\$outputdir);
fixpath (\$script);

if (!(-e $unzip)) {
  print STDERR "$unzip does not exist\n";
  exit(1);
}
if ((-e $scratch) && (! -d $scratch)) {
  print STDERR "Scratch $scratch already exists as file\n";
  exit(1);
}

if ((-e $outputdir) && (! -d $outputdir)) {
  print STDERR "Output $outputdir already exists as file\n";
  exit(1);
}

if (! -f $script) {
  print STDERR "Input $script not found\n";
  exit(1);
}

if (! open INPUT, $script) {
  print STDERR "Cannot open $script\n";
  exit(1);
}

# Make missing directories
maketree ($scratch);
maketree ($outputdir);

# Read and process input
my @cmdLineTok;
my $cmdLine;
my %symTab = ();
my $line = 0;
my $continue = 0;

while (<INPUT>) {
  $line++;
  chomp;
  # Tokenize on space, handling quoted strings
  $cmdLine = $_;
  while (! ($cmdLine =~ /^\s*$/)) {
    # Dispose of leading space
    $cmdLine =~ s/^\s+//;
    # Dispose of trailing space
    $cmdLine =~ s/\s+$//;
    my $quoteStart = index ($cmdLine, '"');
    my $preQuote = "";
    if ($quoteStart == $[) {
      # At beginning of string; find end quote, ignoring \"
      my $quoteEnd = $quoteStart;
      while ($quoteEnd >= $[) {
        $quoteEnd = index ($cmdLine, '"', $quoteEnd + 1);
        if (substr($cmdLine, $quoteEnd - 1, 1) ne "\\") {
          last;
        }
      }
      if ($quoteEnd < $[) {
        print STDERR "$script($line): Runaway quote\n";
        exit(1);
      }
      push @cmdLineTok, substr ($cmdLine, $quoteStart + 1, 
                                $quoteEnd - $quoteStart - 1);
      $cmdLine = substr ($cmdLine, $quoteEnd + 1);
    } elsif ($quoteStart >= $[) {
      # Quote found, process whats before quote
      $preQuote = substr ($cmdLine, 0, $quoteStart);
      $cmdLine = substr ($cmdLine, $quoteStart);
    } else {
      # No quote; process whole line
      $preQuote = $cmdLine;
      $cmdLine = "";
    }
    # Get rid of comments.  Any quotes after # are ignored (but # within quote is OK)
    if ($preQuote =~ s/#.*$//) {
      # Rest of line is commented out
      $cmdLine = "";
    }
    # Line continuation
    if ($cmdLine eq "") {
      $continue = $preQuote =~ s/\\$//;
    }
    if (! ($preQuote =~ /^\s*$/)) {
      push @cmdLineTok, split (/\s+/, $preQuote);
    }
  }
  if ((! $continue) && ($#cmdLineTok >= $[)) {
    processCmd ($line, $script, \@cmdLineTok, \%symTab);
  }
}
close INPUT;

sub readtree {
  my ($rootdir, $filesRef, $dirsRef, $emptyDirsRef) = @_;

  print "readtree $rootdir\n";
  my @dirStack = ("");

  while(@dirStack) {
    my $dir = pop(@dirStack);
    my $opendir = $rootdir;
    if ($dir ne "") {
      $dir .= "/";
      $opendir.= "/$dir";
    }
    if (!opendir DIR, $opendir) {
      print STDERR "readtree: Can't open $opendir\n";
      exit(1);
    }
    my @files = readdir(DIR);
    closedir(DIR);
    my $file;
    my $empty = 1;
    foreach $file (@files) {
      next if ($file eq ".");
      next if ($file eq "..");
      $empty = 0;
      if (-d "$rootdir/$dir$file") {
        push @dirStack, "$dir$file";
        push @$dirsRef, "$dir$file";
      } else {
        push @$filesRef, "$dir$file";
      }
    }
    if ($empty && defined($emptyDirsRef)) {
      push @$emptyDirsRef, $dir;
    }
  }
}

# rm -rf a tree (w/ error checking) and no external deps (File::Path)
sub prunetree {
  my ($root) = @_;

  return if (! -e $root);
  print "prunetree: $root\n";

  my (@files, @dirs);

  readtree ($root, \@files, \@dirs);

  # Got whole tree, nuke the files
  my $file;
  foreach $file (@files) {
    if (! unlink "$root/$file") {
      print STDERR "Can't delete file $root/$file\n";
      exit(1);
    }
  }
  # Now nuke the directories, in reverse order
  while (@dirs) {
    my $dir = pop(@dirs);
    if (! rmdir "$root/$dir") {
      print STDERR "Can't delete directory $root/$dir\n";
      exit(1);
    }
  }
}

# Make a tree similar to mkdir -p without external deps (File::Path)
sub maketree {
  my ($path) = @_;

  print "maketree: $path\n";
  my (@split_path) = split (/\/+/, $path);
  my $i;
  my $build_path = "";
  my $begun_mkdir = 0;
  my $directories_created = 0;
  for ($i = 0; $i <= $#split_path; $i++) {
    $build_path .= "/" if ($i > 0);
    $build_path .= $split_path[$i]; 
    if (-e $build_path) {
      if (-d $build_path) {
        if ($begun_mkdir) {
          print STDERR "maketree: impossible: transition from nonexist to exist\n";
          print STDERR "Probably racing another task\n";
          print STDERR "Error found at $build_path (is a directory)\n";
          exit(1);
        }
      } else {
        if ($begun_mkdir) {
          print STDERR "maketree: impossible: transition from nonexist to exist\n";
          print STDERR "Probably racing another task\n";
          print STDERR "Error found at $build_path (is a file)\n";
          exit(1);
        } else {
          print STDERR "Found $build_path but is not a directory\n";
          exit(1);
        }
      }
    } else {
      if (! $begun_mkdir) {
        my $j;
        for ($j = $i+1; $j <= $#split_path; $j++) {
          if ($split_path[$j] eq "..") {
            print STDERR "maketree: All directories after $build_path must not be ..\n";
            exit(1);
          }
        }
      }
      print "mkdir $build_path\n";
      if (! mkdir $build_path) {
        if ($begun_mkdir) {
          print STDERR "maketree: Subsequent mkdir failed; bad umask or racing another task\n";
        } else {
          print STDERR "maketree: First mkdir failed\n";
        }
        exit(1);
      }
      $directories_created++;
      $begun_mkdir = 1;
    }
  }
}

# Make pathnames uniform
sub fixpath {
  my ($pathRef) = @_;
  # Convert \ to /
  $$pathRef =~ s/\\/\//g;
  # dispose of "/./"
  $$pathRef =~ s/\/\.\//\//g;
  # Dispose of trailing /
  $$pathRef =~ s/\/$//;
  # Dispose of leading ./ or trailing /.
  $$pathRef =~ s/^\.\///;
  $$pathRef =~ s/\/\.$//;
  # Dispose of double / except at beginning (windows network path)
  my $leadingDouble = $$pathRef =~ /^\/\/[^\/]/;
  $$pathRef =~ s/\/+/\//g;
  if ($leadingDouble) {
    # Put back double
    $$pathRef = "/".$$pathRef;
  }
}

# Process commands from input file
sub processCmd {
  my ($lineNum, $fname, $cmdRef, $symRef) = @_;
  my $errHdr = "ERROR: $fname($lineNum):";
  if (($#$cmdRef >= 1) && ($$cmdRef[1] eq "=")) {
    if ($#$cmdRef != 2) {
      print STDERR "$errHdr assignment requires exactly 3 tokens: var,=,val\n";
      exit(1);
    }
    if ($$cmdRef[0] =~ /^\$?([0-9A-Za-z_]+)$/) {
      # Assign variable
      $$symRef{$1} = $$cmdRef[2];
    } else {
      print STDERR "$errHdr assignment sym ($$cmdRef[0]) must be alphanumeric\n";
      exit(1);
    }
  } else {
    # Normal command
    my $zipFile;
    my $skipCount;
    my @regex;
    my $statFile;
    my @delDirs;

    my $thisArg;
    my $dashDone = 0;
    foreach $thisArg (@$cmdRef) {
      if (! $dashDone) {
        if ($thisArg =~ /^-/) {
          if ($thisArg eq "--") {
            $dashDone = 1;
          } else {
            if (length($thisArg) <= 2) {
              print STDERR "$errHdr $thisArg needs argument\n";
              exit(1);
            }
            if ($thisArg =~ /^-r/) {
              my $thisDir = substr ($thisArg, 2);
              replaceVars (\$thisDir, $symRef, $errHdr);
              push @delDirs, $thisDir;
            } elsif ($thisArg =~ /^-s/) {
              $statFile = substr ($thisArg, 2);
              replaceVars (\$statFile, $symRef, $errHdr);
            } else {
              print STDERR "$errHdr Unknown option ", substr($thisArg, 0, 2);
              exit(1);
            }
          }
	  next;
        }
      }
      if (! defined($zipFile)) {
        $zipFile = $thisArg;
        replaceVars (\$zipFile, $symRef, $errHdr);
      } elsif (! defined($skipCount)) {
        $skipCount = $thisArg;
        replaceVars (\$skipCount, $symRef, $errHdr);
      } else {
        push @regex, $thisArg;
      }
    }
    if ((! defined ($skipCount)) || (! ($skipCount =~ /^\d+$/))) {
      print STDERR "$errHdr Must have at least two tokens in command ".
                   "(filename and skip count)\n";
      exit(1);
    }
 
    # Unixafy path
    fixpath (\$zipFile);
    if (! -e $zipFile) {
      print STDERR "$errHdr Zip file $zipFile does not exist\n";
      exit(1);
    }

    my @newStat = stat $zipFile;
    # 4 = uid; 5 = gid; 7 = size; 9 = mtime
    my $newStatInfo = "$newStat[4] $newStat[5] $newStat[7] $newStat[9]";
    # If statFile exists and matches current zip, stop
    if ((! $ignoreStat) && (defined($statFile) && (-e "$outputdir/$statFile"))) {
      if (! open STAT, "$outputdir/$statFile") {
        print STDERR "$errHdr Cannot open $outputdir/$statFile\n";
        exit(1);
      }
      my $oldStatInfo = <STAT>;
      close STAT;
      chomp $oldStatInfo;
      if ($newStatInfo eq $oldStatInfo) {
        print "For zip $zipFile, stat info matches that in $statFile\n";
        print "If you want to skip this check, don't specify $statFile\n";
        print "Skipping\n";
        @$cmdRef = ();
        return 0;
      }
    }
                            
    # Delete scratch
    prunetree ($scratch);

    # Unzip file to scratch
    my @unzipCmd = ($unzip, "-q", "-d", $scratch, $zipFile);
    print "Run: ",join (" ", @unzipCmd), "\n";
    my $ret = system (@unzipCmd);
    if ($ret) {
      print STDERR "$errHdr Unzip failed ($ret)\n";
      exit(1);
    }

    my (@zipFiles, @zipDirs, @emptyDirs);
    readtree ($scratch, \@zipFiles, \@zipDirs, \@emptyDirs);

    # apply $skipCount
    my @destFiles = @zipFiles;
    my @destDirs = @emptyDirs;
    
    if ($skipCount) {
      my $filePrefix = removePrefix (\@destFiles, $skipCount, $errHdr);
      if (@destDirs) {
        my $dirPrefix = removePrefix (\@destDirs, $skipCount, $errHdr);
        if ($filePrefix ne $dirPrefix) {
          print STDERR "$errHdr Prefix from dir list doesn't match file list\n";
          exit(1);
        }
      }
    }
    # Apply regular expressions and check for duplicates
    my $dups = applyRe (\@destFiles, \@regex, $errHdr);
    if (@destDirs) {
      $dups += applyRe (\@destDirs, \@regex, $errHdr);
    }
    if ($dups) {
      print STDERR "$errHdr Duplicated filenames seen; abort\n";
      exit(1);
    }
    # Remove specified destination directories
    cleanDests (\@delDirs, $outputdir, $errHdr);

    my $moveFailed = 0;
    my $i;
    for ($i = 0; $i <= $#destFiles; $i++) {
      if (! -f "$scratch/$zipFiles[$i]") {
        print STDERR "$errHdr source file $scratch/$zipFiles[$i] doesn't exist\n";
        $moveFailed = 1;
        last;
      }
      if (-e "$outputdir/$destFiles[$i]") {
        print STDERR "$errHdr dest file $outputdir/$destFiles[$i] DOES exist\n";
        $moveFailed = 1;
        last;
      }
      my $dirPartIdx = rindex ($destFiles[$i], "/");
      if ($dirPartIdx >= $[) {
        my $dirPart = substr ($destFiles[$i], 0, $dirPartIdx);
        $dirPart = "$outputdir/$dirPart";
        if (! -d $dirPart) {
          if (-e $dirPart) {
            print STDERR "$errHdr directory $dirPart already exists as a file\n";
            $moveFailed = 1;
            last;
          }
          maketree ($dirPart);
        }
      }
      if (! rename_chmod ("$scratch/$zipFiles[$i]", "$outputdir/$destFiles[$i]")) {
        print STDERR "$errHdr move $scratch/$zipFiles[$i] to $outputdir/$destFiles[$i] failed\n";
        $moveFailed = 1;
        last;
      }
    }
    if ($moveFailed) {
      cleanDests (\@delDirs, $outputdir, $errHdr);
      exit(1);
    } else {
      # Create necessary empty directories
      my $empty;
      foreach $empty (@destDirs) {
        $empty = "$outputdir/$empty";
        fixpath (\$empty);
        if (! -e $empty) {
          maketree ($empty);
        }
      }
      # create/update statfile
      if (defined($statFile)) {
        if (! open STAT, ">$outputdir/$statFile") {
         print "Failed to create $outputdir/$statFile (skipping it)\n";
        } else {
          print STAT "$newStatInfo\n";
          close STAT;
        }
      }
    }
    prunetree ($scratch);
  }
  @$cmdRef = ();
}

# Remove prefix of $skipCount directories from each file/directory in $listRef
sub removePrefix {
  my ($listRef, $skipCount, $errHdr) = @_;

  # Find the prefix determined by $skipCount
  my $prefIdx = $[ - 1;
  my $i;
  if (@$listRef) {
    for ($i = 0; $i < $skipCount; $i++) {
      $prefIdx = index ($$listRef[0], "/", $prefIdx + 1);
      if ($prefIdx < $[) {
        last;
      }
    }
  } else {
    print STDERR "$errHdr No files found after unzip\n";
    exit(1);
  }

  if ($prefIdx < $[) {
    print STDERR "$errHdr Prefix of $skipCount not found on $$listRef[0]\n";
    exit(1);
  }

  $prefIdx++;
  my $prefix = substr ($$listRef[0], 0, $prefIdx);

  # Check prefix is same for all files
  for ($i = 0; $i <= $#$listRef; $i++) {
    if (substr ($$listRef[$i], 0, $prefIdx) ne $prefix) {
      print STDERR "$errHdr Prefix of $$listRef[$i] doesn't match $prefix\n";
      print STDERR "When skip count $skipCount specified, must all match\n";
      exit(1);
    }
    $$listRef[$i] = substr ($$listRef[$i], $prefIdx);
  }

  return $prefix;
}

# Apply all regex in $reRef to all files/directories in $listRef
sub applyRe {
  my ($listRef, $reRef, $errHdr) = @_;

  my %uniqHash;
  my $dups = 0;
  my $i;
  for ($i = 0; $i <= $#$listRef; $i++) {
    my $j;
    for ($j = 0; $j <= $#$reRef; $j++) {
      my $cmd = "\$\$listRef[\$i] =~ $$reRef[$j]";
      eval $cmd;
      if ($@) {
        print STDERR "$errHdr eval of $cmd failed\n";
        print STDERR "eval return: $@\n";
        exit(1);
      }
    }
    fixpath (\$$listRef[$i]);
    if (defined($uniqHash{$$listRef[$i]})) {
      print STDERR "$errHdr $$listRef[$i] is duplicated\n";
      $dups++;
    } else {
      $uniqHash{$$listRef[$i]} = 1;
    }
  }
  return $dups;
}

sub cleanDests {
  my ($listRef, $outputDir, $errHdr) = @_;

  my $deletePath;

  foreach $deletePath (@$listRef) {
    $deletePath = "$outputDir/$deletePath";
    if (-e $deletePath) {
      if (-f $deletePath) {
        if (! unlink($deletePath)) {
          print STDERR "$errHdr Could not delete file $deletePath\n";
          exit(1);
        }
      } elsif (-d $deletePath) {
        prunetree ($deletePath); 
        if (-d $deletePath) {
          if (! rmdir ($deletePath)) {
            print STDERR "$errHdr Could not delete dir $deletePath\n";
            exit(1);
          }
        }
      } else {
        print STDERR "$errHdr Do not know how to delete $deletePath\n";
        exit(1);
      }
    }
  }
}

sub replaceVars {
  my ($strRef, $symRef, $errHdr) = @_;

  while ($$strRef =~ /\$\(([0-9A-Za-z_]+)\)/) {
    my $varName = $1;
    my $varVal;
    if (defined($$symRef{$varName})) {
      $varVal = $$symRef{$varName};
    } elsif (defined($ENV{$varName})) {
      $varVal = $ENV{$varName};
    }
    if (defined($varVal)) {
      if (! ($$strRef =~ s/\$\($varName\)/$varVal/)) {
        print STDERR "$errHdr Failed to replace \$\($varName\) with $varVal\n";
        exit(1);
      }
    } else {
      print STDERR "$errHdr Failed to find value for \$\($varName\)\n";
      exit(1);
    }
  }
}

sub rename_chmod {
  my ($src, $dst) = @_;
  my $lastSlash = rindex ($src, "/");
  my $baseDir = substr ($src, 0, $lastSlash);
  if ((!defined($baseDir)) || ($baseDir eq "")) {
    $baseDir = ".";
  }
  my $restoreFilePerms = 0;
  my @stats = stat $src;
  my $srcOrigPerm = $stats[2] & 07777;
  if (! @stats) {
    print STDERR "Unable to stat $src\n";
    return 0;
  }
  if (! -w $src) {
    # Make file writable (windows requires it; unix doesn't
    if (! chmod ($srcOrigPerm | 0666, $src)) {
      print STDERR "Failed to make r/o file $src r/w\n";
      return 0;
	}
	$restoreFilePerms = 1;
  }
  if (! -w $baseDir) {
    # Make directory containing file writable (unix requires, windows may not)
	@stats = stat $baseDir;
	if (! @stats) {
	  # Warning because move will fail if the directory really had to be r/w
	  print STDERR "Warning: Cannot stat directory $baseDir\n";
	} else {
      my $dirPerm = $stats[2] & 07777;
	  if (! chmod ($dirPerm | 0666, $baseDir) ) {
  	    # Warning because move will fail if the directory really had to be r/w
	    print STDERR "Warning: Could not make directory $baseDir read/write\n:";
	  }
	}	
  }
  
  if (! rename ($src, $dst)) {
    return 0;
  }
  
  if ($restoreFilePerms) {
    if (! chmod $srcOrigPerm, $dst) {
	  print STDERR "Warning: Could not restore original permissions from $src to $dst\n";
	}
  }
  return 1;
}  
