#!/usr/local/bin/perl

# Generate DSP Build ID
#
# The only command line argument is a 32 bit number, in C representation 
# (0x prefix indicates hex, 0 prefix indicates octal, and 0b prefix indicates 
# binary, no leading 0 indicates decimal).  This 32 bit number is the
# "build ID" generally specified by CM.  
#
# Converts current date and time, in GMT, to the format:
#     MSB15    12      8       4       0LSB
#       | | | | | | | | | | | | | | | | |
# DATE  |M M M M|D D D D D|Y Y Y Y Y Y Y|
# TIME  |H H H H H|m m m m m m|S S S S S|
#
# M = Month = 1-12 (1=jan; 12=dec)
# D = Day Of Month (0-31)
# Y = Year, 0-127 (0=2000; 127=2127)
#
# H = Hour = 0-23 (0-midnight; 23=11pm)
# m = Minute = 0-59 (minutes)
# S = Second = 0-29 (2 second units)
#
# The output is a make include file, defining the variables GGBUILDID_IDMSW,
# GGBUILDID_IDLSW, GGBUILDID_DATE, GGBUILDID_TIME, GGBUILDID_IDSTR, 
# GGBUILDID_DATESTR
#
# If the build ID specified on the command line is 0x12345678, and
# it is run on 1/1/2003 (0x1083) at 15:00 GMT (0xF000), then the output would be 
#
# GGBUILDID_IDMSW=0x1234U
# GGBUILDID_IDLSW=0x5678U
# GGBUILDID_DATE=0x1083U
# GGBUILDID_TIME=0xF000U
# GGBUILDID_IDSTR=305419896
# GGBUILDID_DATESTR=(MM/DD/YYYY HH:MM:SS) 01/01/2003 15:00:00 GMT
# 
# (C) Copyright 2003, Texas Instruments Incorporated.
# * 
# *  Redistribution and use in source and binary forms, with or without 
# *  modification, are permitted provided that the following conditions 
# *  are met:
# *
# *    Redistributions of source code must retain the above copyright 
# *    notice, this list of conditions and the following disclaimer.
# *
# *    Redistributions in binary form must reproduce the above copyright
# *    notice, this list of conditions and the following disclaimer in the 
# *    documentation and/or other materials provided with the   
# *    distribution.
# *
# *    Neither the name of Texas Instruments Incorporated nor the names of
# *    its contributors may be used to endorse or promote products derived
# *    from this software without specific prior written permission.
# *
# *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
# *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
# *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
# *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
# *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
# *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
# *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
# *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
# *
# */


die "Syntax: $0 makeinc.mk buildid\n".
    "        Where makeinc.mk is the output make include file,\n".
    "        and buildid is a 32 bit number in C syntax.\n" if $#ARGV != 1;

# Open files
open OUTPUT, ">$ARGV[0]" or die "Can't open $ARGV[0] for writing.\n";

$GMTdate = time();
($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = gmtime($GMTdate);

# Convert units
$year -= 100;
$mon ++;
$sec /= 2;

# Convert buildid from hex, oct, binary, decimal using C sytnax
$buildid = $ARGV[1];
$buildid = oct($buildid) if $buildid =~ /^0/;

$buildidMSW = ($buildid >> 16) & 0xFFFF;
$buildidLSW = ($buildid      ) & 0xFFFF;
$timeword = (($sec  & 0x1f) <<  0) | 
            (($min  & 0x3f) <<  5) | 
            (($hour & 0x1f) << 11) ;
$dateword = (($year & 0x7F) <<  0) | 
            (($mday & 0x1f) <<  7) | 
            (($mon  & 0x0f) << 12) ;

print OUTPUT sprintf "GGBUILDID_IDMSW=0x%04xU\n", $buildidMSW;
print OUTPUT sprintf "GGBUILDID_IDLSW=0x%04xU\n", $buildidLSW;
print OUTPUT sprintf "GGBUILDID_DATE=0x%04xU\n", $dateword;
print OUTPUT sprintf "GGBUILDID_TIME=0x%04xU\n", $timeword;
print OUTPUT sprintf "GGBUILDID_IDSTR=$buildid\n";
print OUTPUT sprintf "GGBUILDID_DATESTR=(MM/DD/YYYY HH:MM:SS) ".
                     "%02d/%02d/%04d %02d:%02d:%02d GMT\n", 
                     $mon, $mday, $year+2000, $hour, $min, int($sec)*2;

close OUTPUT;

# Nothing past this point

