################################################################################
# Purpose: Library routines manipulating binary data
#
# Public functions
#   binUtils::scalarsToBin : convert list of scalars to binary string
#   binUtils::binToScalars : convert binary string to list of scalars
#   binUtils::hexDump      : hex dump a binary string
# 
# (C) Copyright 2010, Texas Instruments Incorporated.
# * 
# *  Redistribution and use in source and binary forms, with or without 
# *  modification, are permitted provided that the following conditions 
# *  are met:
# *
# *    Redistributions of source code must retain the above copyright 
# *    notice, this list of conditions and the following disclaimer.
# *
# *    Redistributions in binary form must reproduce the above copyright
# *    notice, this list of conditions and the following disclaimer in the 
# *    documentation and/or other materials provided with the   
# *    distribution.
# *
# *    Neither the name of Texas Instruments Incorporated nor the names of
# *    its contributors may be used to endorse or promote products derived
# *    from this software without specific prior written permission.
# *
# *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
# *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
# *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
# *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
# *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
# *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
# *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
# *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
# *
# */

package binUtils;

use strict;
use diagnostics;
use warnings;

###############################################################################
# FUNCTION: binUtils::scalarsToBin
#
# DESCR: Converts a list of 32-bit words into bytes (inverse of convert32bitsw
#        considering endian.
#
# ARGS: @scalars:    (i) List of one or more 32-bit words
#       $bigEndian:  (i) nonzero implies big endian format
#
# Return Val: Resulting bytes (perl string)
################################################################################
sub scalarsToBin
{
  my ($bigEndian, @scalars) = @_;
  my $str = "";
  my $i;
  for ($i = 0; $i <= $#scalars; $i++) {
    if ($bigEndian) {
      $str .= sprintf ("%c%c%c%c", 
                       ($scalars[$i] >> 24) & 0xff,
                       ($scalars[$i] >> 16) & 0xff,
                       ($scalars[$i] >> 8) & 0xff,
                       ($scalars[$i]) & 0xff);
    } else {
      $str .= sprintf ("%c%c%c%c", 
                       ($scalars[$i]) & 0xff,
                       ($scalars[$i] >> 8) & 0xff,
                       ($scalars[$i] >> 16) & 0xff,
                       ($scalars[$i] >> 24) & 0xff);
    }
  }
  return $str;
} # binUtils::scalarsToBin

################################################################################
# FUNCTION: binUtils::binToScalars
#
# DESCR: String contains "binary" representation of 32 bits; convert to number
#        considering endian.
#
# ARGS: $str:       (i) Input binary data
#       $bigEndian: (i) nonzero implies big endian format
#
# Return Val: Array of 32 bit results
################################################################################
sub binToScalars
{
  my ($str, $bigEndian) = @_;
  my $len = length($str);
  # Add missing zeros at end to align to 32 bits
  my $rem = ($len % 4);
  $rem = 4 - $rem if ($rem);
  $str .= chr(0) x $rem;
  $len += $rem;
  my $i;
  my @results;

  for ($i = 0; $i < $len; $i += 4) {
    my $word = substr ($str, $i, 4);
    $word = reverse($word) unless ($bigEndian);
    my @bytes = split(//,$word);
    my $val = (ord ($bytes[0]) << 24) |
              (ord ($bytes[1]) << 16) |
              (ord ($bytes[2]) <<  8) |
              (ord ($bytes[3])      );
    push @results, $val;
  }
  return @results;
} # binUtils::binToScalars

################################################################################
# FUNCTION: binUtils::hexDump
#
# DESCR: Creates 32-bit hexdump of $data into $fname for debug
#
# ARGS: $fname:     (i) App name or file name to dump to 
#                       (extension changed to .hex)
#       $data:      (i) Binary data to dump (string)
#       $bigEndian: (i) nonzero implies big endian format
#
# Return Val: None
################################################################################
sub hexDump
{
  my ($fname, $data, $bigEndian) = @_;
  local *OUTPUT;

  $fname =~ s/\\/\//g;
  $fname = File::Basename::fileparse ($fname, qr/\.[^.]*/) . ".hex";

  print "Creating $fname for debug\n";

  my $l = length($data);
  open OUTPUT, ">$fname" or die "Can't open $fname\n";
  my ($i);
  my ($byte1, $byte2, $byte3, $byte4);
  for ($i = 0; $i < $l; $i+=4) {
    $byte1 = ord( substr ($data, $i, 1));
    $byte2 = ord( substr ($data, $i+1, 1));
    $byte3 = ord( substr ($data, $i+2, 1));
    $byte4 = ord( substr ($data, $i+3, 1));
    my $val;
    if ($bigEndian) {
      $val = ($byte1 << 24) | ($byte2 << 16) | ($byte3 << 8) | $byte4;
    } else {
      $val = ($byte4 << 24) | ($byte3 << 16) | ($byte2 << 8) | $byte1;
    }
    printf OUTPUT ("0x%08x\n", $val);
  }
  close OUTPUT;
  return 0;
} # binUtils::hexDump

return 1; # Make use/require happy

# Nothing past this point
