################################################################################
# Purpose: Acts as an elf loader into a virtual machine, then outputs the
#          results into a coff file.
#
# Syntax: See syntax below in error message
#
# (C) Copyright 2011, Texas Instruments Incorporated.
# * 
# *  Redistribution and use in source and binary forms, with or without 
# *  modification, are permitted provided that the following conditions 
# *  are met:
# *
# *    Redistributions of source code must retain the above copyright 
# *    notice, this list of conditions and the following disclaimer.
# *
# *    Redistributions in binary form must reproduce the above copyright
# *    notice, this list of conditions and the following disclaimer in the 
# *    documentation and/or other materials provided with the   
# *    distribution.
# *
# *    Neither the name of Texas Instruments Incorporated nor the names of
# *    its contributors may be used to endorse or promote products derived
# *    from this software without specific prior written permission.
# *
# *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
# *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
# *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
# *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
# *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
# *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
# *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
# *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
# *
# */

use strict;
use diagnostics;
use warnings;

# Stick directory containing script into search path to find libraries
use File::Basename;
use lib File::Basename::dirname($0);
use lib "."; # Put current directory first so "require" below works as expected

use ofdParser;
use ofdUtils;
use binUtils;

my $zeroSects = 1;  # generate zero sections
my $genSyms = 1;    # generate symbol table

$ARGV[0] = "" if !defined($ARGV[0]);

my $badArg = 0;
while (($ARGV[0] =~ /^-/) && (! $badArg)) {
  $badArg = 1;
  if ($ARGV[0] eq "-no_zero") {
    $zeroSects = 0; # don't generate zero sections
    $badArg = 0;
  } elsif ($ARGV[0] eq "-no_syms") {
    $genSyms = 0; # don't generate symbol table
    $badArg = 0;
  }
  shift @ARGV if (! $badArg);
}

if (($#ARGV != 2) || $badArg) {
  print STDERR "Invalid argument: $ARGV[0]\n" if ($badArg);
  print STDERR <<SYNTAX
Syntax:
  $0 [-no_zero] [-no_syms] ofd.exe elf.out coff.out

where

  -no_zero:        do NOT comply with elf req to generate 0s for uninit sects
  -no_syms:        do NOT generate symbol table
  ofd.exe:         path to ofd6x.exe utility
  elf.out:         Input file (non relocatable elf)
  coff.out:        Output file (non relocatable stripped coff)
SYNTAX
  ;
  exit(1);
}

my ($ofdUtil, $elfFile, $coffFile) = @ARGV;

if (! -f $elfFile) {
  print STDERR "ERROR: $elfFile not found\n";
  exit(1);
}
if (! -f $ofdUtil) {
  print STDERR "ERROR: Ofd6x.exe utility $ofdUtil not found\n";
  exit(1);
}

my @filters = (
# Get section attributes
  [qw (elf section_table section elf32_shdr sh_name_string)],
  [qw (elf section_table section elf32_shdr sh_type)],
  [qw (elf section_table section elf32_shdr sh_addr), \&ofdUtils::convertNumber],
  [qw (elf section_table section elf32_shdr sh_size), \&ofdUtils::convertNumber],
  [qw (elf section_table section elf32_shdr sh_flags flag)],
  [qw (elf section_table section raw_data), \&ofdUtils::convertBase64],
  [qw (elf section_table section index), \&ofdUtils::convertNumber],
  [qw (elf program_table elf32_phdr p_paddr), \&ofdUtils::convertNumber],
  [qw (elf program_table elf32_phdr p_vaddr), \&ofdUtils::convertNumber],
  [qw (elf program_table elf32_phdr p_memsz), \&ofdUtils::convertNumber],
  [qw (elf program_table elf32_phdr p_type)],
  [qw (elf32_ehdr ei_data)],
  [qw (elf32_ehdr e_entry), \&ofdUtils::convertNumber],
  [qw (elf32_ehdr e_type)]
);

if ($genSyms) {
  push @filters, (
    [qw (elf section_table section symbol_table elf32_sym st_name_string)],
    [qw (elf section_table section symbol_table elf32_sym st_value), \&ofdUtils::convertNumber],
    [qw (elf section_table section symbol_table elf32_sym st_bind)],
    [qw (elf section_table section symbol_table elf32_sym st_type)],
    [qw (elf section_table section symbol_table elf32_sym st_shndx)]
  );
}

my %parseTree;

my $err;
$err = ofdParser::parse ($ofdUtil, $elfFile, \%parseTree, \@filters);
if ($err) {
  print STDERR "ERROR: Parse of $elfFile failed: $err\n";
  exit(1);
}

my $isElf;
($isElf, $err) = ofdUtils::isElf(\%parseTree);
if ($err || ! $isElf) {
  print STDERR "ERROR: $elfFile is NOT elf\n";
  exit(1);
}

my $bigEndian;
($bigEndian, $err) = ofdUtils::getBigEndian (\%parseTree);
if ($err) {
  print STDERR "ERROR: Could not determine endian of $elfFile: $err\n";
  exit(1);
}
print "bigEndian: $bigEndian\n";


my @memory = ();
my @nonAlloc = ();
my @specialSections = ();

$err = makeMemTab (\@memory, \@nonAlloc, \%parseTree);
if ($err) {
  print STDERR "ERROR: makeMemTab failed: $err\n";
  exit(1);
}

# get the entry point
my @query = (
  {name => "ofd",         maxMatches => 1},
  {name => "object_file", maxMatches => 1},
  {name => "elf",         maxMatches => 1},
  {name => "elf32_ehdr",  maxMatches => 1},
  {name => "e_entry",     maxMatches => 1}
);

my $entryPoint;
($err, $entryPoint) = ofdUtils::lookUp (\%parseTree, \@query);

if ($err) {
  print STDERR "Lookup of ofd.obect_file.elf.elf32_ehdr.e_entry failed ($err)";
  exit(1);
}

my @symTab = ();

if ($genSyms) {
  $err = makeSymTab (\@symTab, \%parseTree);
  if ($err) {
    print STDERR "makeSymTab failed: $err\n";
    exit(1);
  }
}

%parseTree = ();

my $coffData;
my $timeStamp = (stat($elfFile))[9];
$err = makeCoff (\$coffData, \@memory, \@nonAlloc, $timeStamp, 
                 $bigEndian, $entryPoint, $zeroSects, $genSyms ? \@symTab : undef);

if ($err) {
  print STDERR "ERROR: makeCoff failed: $err\n";
  exit(1);
}

open OUTPUT, ">$coffFile" or die "Can't open $coffFile";
binmode OUTPUT;
print OUTPUT $coffData;
close OUTPUT;

exit(0);

################################################################################
# FUNCTION: makeMemTab
#
# DESCR: Goes through parse tree to create $allocRef, which has all the
#        initialized, allocated (loaded) sections ordered by address.  These
#        can't overlap.  Also generates $nonAllocRef, which has all the 
#        initialized but unallocated (not loaded) sections which are unordered.
#        These can overlap.
#
# ARGS: $allocRef:    - memory desc for allocated sections
#       $nonAllocRef: - memory desc for unallocated sections
#       $parseRef:    - ofd parse tree
#
# Return value: error
################################################################################
sub makeMemTab {
  my ($allocRef, $nonAllocRef, $parseRef) = @_;

  my @query;

  # Barf if the elf file isn't fully relocated
  @query = (
    {name => "ofd",         maxMatches => 1},
    {name => "object_file", maxMatches => 1},
    {name => "elf",         maxMatches => 1},
    {name => "elf32_ehdr",  maxMatches => 1},
    {name => "e_type",      maxMatches => 1}
  );

  my $e_type;
  my $err;
  ($err, $e_type) = ofdUtils::lookUp ($parseRef, \@query);

  if ($err) {
    return "Lookup of ofd.obect_file.elf.elf32_ehdr.e_type failed ($err)";
  }

  if ($e_type ne "ET_EXEC") {
    return "$elfFile is not executable (e_type=$e_type)";
  }

  # walk the program table to start all memory regions at zero
  @query = (
    {name => "ofd",           maxMatches => 1},
    {name => "object_file",   maxMatches => 1},
    {name => "elf",           maxMatches => 1},
    {name => "program_table", maxMatches => 1},
    {name => "elf32_phdr",    maxMatches => 1}
  );

  my $progTab;
  ($err, $progTab) = ofdUtils::lookUp ($parseRef, \@query);

  if ($err) {
    # This is required to initialize the memory.
    # if it can't be found there is probably a parse bug (did ofd6x change?)
    print STDERR "WARNING: Cannot find ofd.object_file.elf.program_table.elf32_phdr[] ($err)\n";
  } else {
    if (ref($progTab) ne "ARRAY") {
      # turn single entry into an array for common processing below
      $progTab = [ $progTab ];
    }

    foreach my $progTabEnt (@$progTab) {
      if (ref($progTabEnt) ne "HASH") {
        ofdUtils::print(\*STDERR, $progTabEnt);
        return "Program table entry is not a hash";
      }
      if (! $$progTabEnt{"p_type"} ) {
        return "Found a program table entry without a p_type";
      }
      if (! defined($$progTabEnt{"p_paddr"}) ) {
        return "Found a program table entry of type " . $$progTabEnt{"p_type"} . 
               " without a p_paddr";
      }
      if (! defined($$progTabEnt{"p_vaddr"}) ) {
        return "Found a program table entry of type " . $$progTabEnt{"p_type"} . 
               " without a p_vaddr";
      }
      if ($$progTabEnt{"p_type"} eq "PT_LOAD") {
        $err = insertSectionDef ($allocRef, "", $$progTabEnt{"p_paddr"}, 
                                 $$progTabEnt{"p_vaddr"}, 
                                 $$progTabEnt{"p_memsz"}, "zero_fill", undef, 
				 -1);
        if ($err) {
          return "insertSectionDef failed: $err\n";
        }
        else {
#          printf "Skipping program table ent at 0x%08x of type %s\n", 
#                 $$progTabEnt{"p_paddr"}, $$progTabEnt{"p_type"};
        }
      }
    }
  }

  # Now walk all the sections and emit those with data
  @query = (
    {name => "ofd",           maxMatches => 1},
    {name => "object_file",   maxMatches => 1},
    {name => "elf",           maxMatches => 1},
    {name => "section_table", maxMatches => 1},
    {name => "section",       maxMatches => 1}
  );

  my $sectTab;
  ($err, $sectTab) = ofdUtils::lookUp ($parseRef, \@query);

  if ($err) {
    # This is required to initialize the memory.
    # if it can't be found there is probably a parse bug (did ofd6x change?)
    return "Can't find ofd.object_file.elf.section_table.elf32_shdr[] ($err)\n";
  }

  if (ref($sectTab) ne "ARRAY") {
    # turn single entry into an array for common processing below
    $sectTab = [ $sectTab ];
  }

  foreach my $sectTabEnt (@$sectTab) {
    if (ref($sectTabEnt) ne "HASH") {
      print STDERR "Program table entry is not a hash\n";
      ofdUtils::print(\*STDERR, $sectTabEnt);
      exit(1);
    }
    my $shdr = $$sectTabEnt{"elf32_shdr"};
    return "Found section without header" if (ref($shdr) ne "HASH");
    return "Found section without sh_type" if (!$$shdr{"sh_type"});
    if ($$shdr{"sh_type"} eq "SHT_PROGBITS") {
      my $name  = $$shdr{"sh_name_string"};
      my $addr  = $$shdr{"sh_addr"};
      my $size  = $$shdr{"sh_size"};
      my $flags = $$shdr{"sh_flags"};
      return "Found SH_PROGBITS section without a sh_name_string" if (! $name);
      return "Found section $name without an address" if (! defined($addr));
      if ($flags) {
        $flags = $$flags{"flag"};
      }
      # make it an array for common processing below
      $flags = "no flags" if (! $flags); # there are none
      $flags = [ $flags] if (! ref($flags));
      $flags = [ $$flags ] if (ref($flags) eq "SCALAR");
      if (ref($flags) ne "ARRAY") {
        return "Found section $name whose perl type is not an array (" .
               ref($flags) . ")=$flags";
      }
      my $insertionArray;
      my $sectionClass;
      if (grep(/^SHF_ALLOC$/, @$flags)) {
        $sectionClass = "raw_data";
        $insertionArray = $allocRef;
      } else {
        $sectionClass = "unordered raw_data";
        $insertionArray = $nonAllocRef;
      }
      $err = insertSectionDef ($insertionArray,
                               $name,
                               $$shdr{"sh_addr"}, 
                               $$shdr{"sh_addr"},
                               $$shdr{"sh_size"}, 
                               $sectionClass,
                               $$sectTabEnt{"raw_data"},
		               $$sectTabEnt{"index"});
      if ($err) {
        print STDERR "insertSectionDef failed: $err\n";
        exit(1);
      }
    } else {
      my $name = $$shdr{"sh_name_string"};
      $name = "" if (! $name);
      printf "Discarding section %s of type %s at 0x%08x len 0x%08x\n", 
             $name, $$shdr{"sh_type"},
             $$shdr{"sh_addr"}, $$shdr{"sh_size"};
    }
  }
  return undef;
}

################################################################################
# FUNCTION: insertSectionDef
#
# DESCR: Inserts specified section into the memory map
#
# ARGS: $memoryRef:      - memory descriptor array sorted by address
#       $name:           - name of new section
#       $loadAddr:       - 32 bit address of section
#       $runAddr:        - 32 bit address of section
#       $length:         - 32 bit length of section
#       $type:           - "zero_fill" or "raw_data"
#       $contents:       - if $type == "raw_data"
#       $index           - section index in elf file
#
# Return value: error
#       - cannot overlap another memory section except for "zero_fill" sections
################################################################################
sub insertSectionDef 
{
  my ($memoryRef, $name, $loadAddr, $runAddr, 
      $length, $type, $contents, $index) = @_;

  return "Invalid load address" if (! defined($loadAddr));
  return "Invalid run  address" if (! defined($runAddr));
  if ($type eq "zero_fill") {
    return "zero_fill cannot have contents" if ($contents);
    return "zero_fill requires length" if (! $length);
    $contents = "";
  } elsif (($type eq "raw_data") || ($type eq "unordered raw_data")) {
    return "raw_data requires contents" if (! $contents);
    $length = length($contents) if (! $length);
    if (length($contents) < $length) {
      $contents .= chr(0) x ($length - length($contents));
    }
    if (length($contents) > $length) {
      return sprintf "raw_data (%d) is bigger than length (%d)", 
                     length($contents), $length;
    }
  } else {
    return "invalid type\n";
  }

  my $record = {name=>$name, loadAddr=>$loadAddr, runAddr=>$runAddr, 
                length=>$length, type=>$type, contents=>$contents,
	        index=>$index };

  if ($type eq "unordered raw_data") {
    push @$memoryRef, $record;
#    printf "Add unordered $name 0x%08x 0x%08x\n", $loadAddr, $length;
    return undef;
  }

  # Find insertion point
  my $insertIdx;

  for ($insertIdx = $[; $insertIdx <= $#$memoryRef; $insertIdx++) {
    last if ($loadAddr < ${$$memoryRef[$insertIdx]}{"loadAddr"});
  }

  # this section can only overlap a subset of a zero section.
  # It cannot span another section (part inside it and part outside it).

  # Does it overlap its predecessor?
  if ($insertIdx > $[) {
    my $predRef = $$memoryRef[$insertIdx - 1];
    my $predBegin = $$predRef{"loadAddr"};
    my $predEnd = $$predRef{"loadAddr"} + $$predRef{"length"};
    if ($predEnd > $loadAddr) {
      # it is allowed for a nonzero section to be completely contained
      # inside a zero section
      if (($type eq "raw_data") &&
          ($$predRef{"type"} eq "zero_fill") &&
          ($loadAddr >= $predBegin) &&
          (($loadAddr + $length) <= $predEnd)) {
        my %leadingZeros = %$predRef;
        my %trailingZeros = %$predRef;
        my @newZeros = (); 
        my $zeroIdx = $insertIdx - 1;
        # split up the zeros into up to 3 parts (leading zeros, raw_data, trailing zeros)
        if ($loadAddr > $predBegin) {
          # Some leading zeros are needed
          $leadingZeros{"length"} = $loadAddr - $predBegin;
          push @newZeros, \%leadingZeros;
        } else {
          $insertIdx --; # No leading zeros needed
        }
        if (($loadAddr + $length) < $predEnd) {
          $trailingZeros{"loadAddr"} = $loadAddr + $length;
          $trailingZeros{"runAddr"} = $trailingZeros{"loadAddr"} +
                                      ($$predRef{"runAddr"} - $$predRef{"loadAddr"});
          $trailingZeros{"length"} = $predEnd - $loadAddr - $length;
          push @newZeros, \%trailingZeros;
        }
        # Adjust the run address
        $$record{"runAddr"} = $$record{"loadAddr"} + 
                              ($$predRef{"runAddr"} - $$predRef{"loadAddr"});
        # Put the zeros into the memory.  The new record will be inserted below
        splice (@$memoryRef, $zeroIdx, 1, @newZeros);
      } elsif (($type eq "zero_fill") && ($$predRef{"type"} eq "zero_fill")) {
	# both sections are zero fill.  Merge them.
	# delete $$predRef and insert the new combined region
	my $newStart = $loadAddr;
	my $newRun   = $runAddr;
	if ($loadAddr > $predBegin) {
          $newStart = $predBegin;
	  $newRun   = $$predRef{"runAddr"};
	}
	my $newEnd = ($predEnd > ($loadAddr + $length)) ? 
	             $predEnd : ($loadAddr + $length);
        my $newLen = $newEnd - $newStart;
        splice (@$memoryRef, $insertIdx - 1, 1); # delete
	my $err = insertSectionDef ($memoryRef, $name, $newStart, $newRun, $newLen,
	                            $type, $contents, $index);
        if ($err) {
          return sprintf "Replacement of zero section at [0x%08x-0x%08x) with ".
	                 "[0x%08x-0x%08x) failed: $err",
			 $loadAddr, $loadAddr+$length,
			 $newStart, $newEnd;
	} else {
          return undef;
	}

      } else {
        return sprintf "Section %s %s at [0x%08x-0x%08x) overlaps previous ".
                       "section %s %s at [0x%08x-0x%08x)",
                       $type, $name, $loadAddr, $loadAddr+$length,
                       $$predRef{"type"}, $$predRef{"name"}, 
		       $$predRef{"loadAddr"}, 
                       $$predRef{"loadAddr"} + $$predRef{"length"};
      }
    }
  }
  # Does it overlap its successor?
  if ($insertIdx <= $#$memoryRef) {
    my $nextRef = $$memoryRef[$insertIdx];
    if ( ($loadAddr + $length) > $$nextRef{"loadAddr"} ) {
      return sprintf "Section %s %s at [0x%08x-0x%08x) overlaps next ".
                     "section %s %s at [0x%08x-0x%08x)",
                     $type, $name, $loadAddr, $loadAddr+$length,
                     $$nextRef{"type"}, $$nextRef{"name"}, 
		     $$nextRef{"loadAddr"}, 
                     $$nextRef{"loadAddr"} + $$nextRef{"length"};
    }
  }

  printf "Add $type $name 0x%08x 0x%08x at %d\n", $loadAddr, $length, $insertIdx;
  splice (@$memoryRef, $insertIdx, 0, $record);
  return undef;
}

################################################################################
# FUNCTION: makeSymTab
#
# DESCR: Goes through parse tree to create the symbol table
#
# ARGS: $symTabRef:   - symbol table
#       $parseRef:    - ofd parse tree
#
# Return value: error
################################################################################
sub makeSymTab {
  my ($symTabRef, $parseRef) = @_;

  my @query;

  # Barf if the elf file isn't fully relocated
  @query = (
    {name => "ofd",           maxMatches => 1},
    {name => "object_file",   maxMatches => 1},
    {name => "elf",           maxMatches => 1},
    {name => "section_table", maxMatches => 1},
    {name => "section",       maxMatches => 1},
    {name => "symbol_table",  maxMatches => 1},
    {name => "elf32_sym"}
  );

  my $err;
  my $symTabEnts;
  ($err, $symTabEnts) = ofdUtils::lookUp ($parseRef, \@query);

  return "ofd.object_file.elf.section_table.symbol_table failed $err\n" if ($err);

  # Turn non-array into array for common processing below
  if (ref($symTabEnts) ne "ARRAY") {
    $symTabEnts = [ $symTabEnts ];
  }

  my $symN = 0;
  foreach my $symEnt (@$symTabEnts) {
    my $type    = $$symEnt{"st_type"}; 
    my $name    = $$symEnt{"st_name_string"};
    my $val     = $$symEnt{"st_value"};
    my $bind    = $$symEnt{"st_bind"};
    my $sectIdx = $$symEnt{"st_shndx"};
    if ($type) {
      if (($type eq "STT_OBJECT") || ($type eq "STT_FUNC")) {
        return "symbol of $type requires name" if (! $name);
        return "symbol $name doesn't have value" if (! defined($val));
        return "symbol $name doesn't have bind" if (! defined($bind));
        return "symbol $name doesn't have section idx" if (! defined($sectIdx));
        if ($sectIdx =~ /^[a-fA-F0-9xX]+$/) {
          $sectIdx = oct ($sectIdx);
        } else {
          return "Invalid st_shndx for $name: $sectIdx";
        }
        push @$symTabRef, { name=>$name, value=>$val, 
                            bind=>$bind, sectIdx=>$sectIdx };
      }
    }
  }
}

################################################################################
# FUNCTION: makeCoff
#
# DESCR: Goes through the loaded and unloaded sections, and outputs them
#        into a TI-coff format file.
#
# ARGS: $coffRef:     - Raw data for COFF file
#       $allocRef:    - memory desc for allocated sections
#       $nonAllocRef: - memory desc for unallocated sections
#       $timeStamp    - .out's unix timestamp
#       $bigEndian    - target endianness
#       $entryPoint   - program entry point
#       $emitZeroPad  - emit zero padding?
#       $symTabRef    - symbol table (or unref for none)
#
# Return value: error
################################################################################
sub makeCoff {
  my ($coffRef, $allocRef, $nonAllocRef, $timeStamp, $bigEndian, 
      $entryPoint, $emitZeroPad, $symTabRef) = @_;

  my $nSects = $#$allocRef + $#$nonAllocRef + 2;
  # Drop the zero sections if requested
  if (! $emitZeroPad) {
    foreach my $section (@$allocRef, @$nonAllocRef) {
      $nSects-- if ($$section{"type"} eq "zero_fill");
    }
  }

  my $sectionHeaders = "";
  my $sectionBodies = "";
  my $symTab = "";
  my $stringTable = "";
  my $sectionOffset = 50 + $nSects * 48;
  my $strTabOffset = 4;
  my $zeroPadN = 0;
  my @sectIdxToPos;
  my $sectNum = 1; # coff starts numbering at 1

  foreach my $section (@$allocRef, @$nonAllocRef) {
    my $flags = 0x20; # STYP_TEXT
    my $name = $$section{"name"};
    my $raw_data = $$section{"contents"};
    my $sectIdx = $$section{"index"};
    if ($$section{"type"} eq "zero_fill") {
      next if (! $emitZeroPad);
      $name = sprintf("zeroPad%04d", $zeroPadN++);
      $raw_data = chr(0) x $$section{"length"};
    } elsif (! defined($sectIdxToPos[$sectIdx])) {
      $sectIdxToPos[$sectIdx] = $sectNum;
    } elsif ($sectIdxToPos[$sectIdx] != $sectNum) {
      return "Section $name re-uses index $sectIdx!";
    }

    if ($$section{"type"} eq "unordered raw_data") {
      $flags = 0x10; # COPY
    }
    my ($nameWord1, $nameWord2);
    if (length($name) > 8) {
      # Put it in the string table
      $nameWord1 = 0;
      $nameWord2 = $strTabOffset;
      $stringTable .= $name . chr(0);
      $strTabOffset += length($name) + 1;
    } else {
      # Inline the name
      ($nameWord1, $nameWord2) = 
        binUtils::binToScalars ($name . (chr(0) x (8 - (length($name)))), 0);
    }
    $sectionBodies .= $raw_data;
    my @sectHdr = (
      # 0-3   = name, or 0 if too long
      $nameWord1,
      # 4-7   = name, or string table offset
      $nameWord2,
      # 8-11  = load addr
      $$section{"loadAddr"},
      # 12-15 = run addr
      $$section{"runAddr"},
      # 16-19 = length
      $$section{"length"},
      # 20-23 = file offset
      $sectionOffset,
      # 24-27 = relocation offset
      0,
      # 28-31 = reserved
      0,
      # 32-35 = num reloc ents
      0,
      # 36-39 = reserved
      0,
      # 40-43 = flags
      $flags,
      # 44-45 = reserved
      0 |
      # 46-47 = page
      0 << 16
    );
    $sectionOffset += $$section{"length"};
    $sectionHeaders .= binUtils::scalarsToBin (0, @sectHdr);
    $sectNum ++;
#    print "Output section $name\n";
  }

  # Add the symtab
  my $symbols = 0;
  foreach my $sym (@$symTabRef) {
    my $name    = $$sym{"name"};
    my $val     = $$sym{"value"};
    my $bind    = $$sym{"bind"};
    my $sectIdx = $$sym{"sectIdx"};
    my $sectNum = $sectIdxToPos[$sectIdx];
    if (! defined($sectNum)) {
      $sectNum = -1; # absolute symbol - its really in an unemitted section
    }

    my ($nameWord1, $nameWord2);
    if (length($name) > 8) {
      # Put it in the string table
      $nameWord1 = 0;
      $nameWord2 = $strTabOffset;
      $stringTable .= $name . chr(0);
      $strTabOffset += length($name) + 1;
    } else {
      # Inline the name
      ($nameWord1, $nameWord2) = 
        binUtils::binToScalars ($name . (chr(0) x (8 - (length($name)))), 0);
    }

    my $class;
    if ($bind eq "STB_GLOBAL") {
      $class = 2; # C_EXT
    } else {
      $class = 6; # C_LABEL
    }

    my @symEnt = (
      # 0-3   = name, or 0 if too long
      $nameWord1,
      # 4-7   = name, or string table offset
      $nameWord2,
      # 8-11  = value
      $val,
      # 12-13 = sect number
      $sectNum |
      # 14-15 = storage type
      (4 << 16),
      # 16    = class
      $class |
      # 17    = aux entries
      (0 << 8)
    );
    
    $symbols++;
    $symTab .= substr(binUtils::scalarsToBin (0, @symEnt), 0, 18);
  }

  # COFF files always come out with headers in little endian
  my @fileHdr = (
    # COFF format comes from spraao8.
    # 0-1 version ID
    0x00c2 |
    # 2-3: # of sections
    $nSects << 16,
    # 4-7 = timestamp
    $timeStamp,
    # 8-11 : symtab offset
    $sectionOffset,
    # 12-15: symtab # entries
    $symbols,
    # 16-17: optional header length
    28 |
    # 18-19: flags
    (0x0001 |  # reloc stripped
    0x0002 |  # executable (fully relocated)
    ($symbols ? 0 : 0x0008) |  # symbols stripped
    ($bigEndian ? 0x0000 : 0x0100) |  # little endian
    ($bigEndian ? 0x0200 : 0x0000)    # big endian
    ) << 16,
    # 20-21: target ID
    0x99
  );

  my @optHdr = (
    # 0-1: magic = 0x0108
    0x0108 |
    # 2-3: version
    0x1b59 << 16,
    # 4-7: text size
    0,
    # 8-11: data size
    0,
    # 12-15: bss size
    0,
    # 16-19: entrypoint
    $entryPoint,
    # 20-23: exec address
    0,
    # 24-27: initialized data address
    0
  );

  $$coffRef = 
    # scalarsToBin returns 24 bytes, so throw away last 2
    # coff does NOT align-and-pad!
    substr (binUtils::scalarsToBin(0, @fileHdr), 0, 22) .
    binUtils::scalarsToBin(0, @optHdr) .
    $sectionHeaders .
    $sectionBodies .
    $symTab . 
    binUtils::scalarsToBin (0, $strTabOffset) .
    $stringTable;
  return undef;
}

# Nothing past this point

