################################################################################
# Purpose: Library routines for handling TI ofd6x.exe output
#
# Public functions
#    ofdParser::parse: pipe ofd.exe and return parse tree
#    ofdParser::printxml: pipe ofd.exe and convert to dotted C-like format
#
# (C) Copyright 2009-2010, Texas Instruments Incorporated.
# * 
# *  Redistribution and use in source and binary forms, with or without 
# *  modification, are permitted provided that the following conditions 
# *  are met:
# *
# *    Redistributions of source code must retain the above copyright 
# *    notice, this list of conditions and the following disclaimer.
# *
# *    Redistributions in binary form must reproduce the above copyright
# *    notice, this list of conditions and the following disclaimer in the 
# *    documentation and/or other materials provided with the   
# *    distribution.
# *
# *    Neither the name of Texas Instruments Incorporated nor the names of
# *    its contributors may be used to endorse or promote products derived
# *    from this software without specific prior written permission.
# *
# *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
# *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
# *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
# *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
# *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
# *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
# *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
# *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
# *
# */

package ofdParser;

use strict;
use diagnostics;
use warnings;

use XML::Parser;

my @xmlStack;
my $val = "";
my $globParseTreeRef;

my $filtering = 0;
my %filterHash;

################################################################################
# FUNCTION: expatStart
#
# DESCR: Called by expat whenver an "begin" operator <operator> is found.
#        Operator is used to find a hash entry for the operator.  If it
#        doesn't exist it is created.  The hash entry contains an array,
#        where the content "under" this start is added to that array.
#
# ARGS: unused:     (i) expat instance pointer
#       $arg:       (i) operator
#
# Globals for all states: 
#    @xmlStack:     (i) each element is string in each nested <>.
#
# Return Val: Nothing
################################################################################
my $expatStart = sub 
{
  my ($arg) = $_[1];

  push @xmlStack, [$globParseTreeRef, $arg];
  my $arrayRef = ${$globParseTreeRef}{$arg};
  if (!defined($arrayRef)) {
    $arrayRef = ${$globParseTreeRef}{$arg} = [];
  }
  $globParseTreeRef = {};
  push @$arrayRef, $globParseTreeRef;
  $val = "";
  return undef;
}; # expatStart

################################################################################
# FUNCTION: expatEnd
#
# DESCR: Called by expat whenver an "end" operator </operator> is found.
#        This records the value found by expatChar into the hash.  Runs
#        assigned filtering functions.  Values that don't match filter
#        or are deleted by the filter are thrown away.
#
# ARGS: unused:     (i) expat instance pointer
#       arg:        (i) "operator"
#
# If the filtering discoveres a callback it is called with the arguments
# callBack ($val, $keyRef, $matchRef)
#    $valRef   : value assigned to key
#    $keyRef   : Array containing key names from root
#    $matchRef : Array containing key names that matched filter
#
# The filter can replace $valRef with a new value or delete it.
#
# Return Val: Nothing
################################################################################
my $expatEnd = sub
{
  my ($arg) = $_[1];

  if ($filtering) {
    # Search to see if this key is allowed
    my $filterHashRef = \%filterHash;
    my $i;
    for ($i = $#xmlStack; $i >= $[; $i--) {
      my $key = ${$xmlStack[$i]}[1];
      my $filterHashDeRef = $$filterHashRef{$key};
      if (defined($filterHashDeRef)) {
        my $type = ref($filterHashDeRef);
        if ($type eq "HASH") {
          $filterHashRef = $filterHashDeRef;
          next;
        }
        if ($type eq "CODE") {
          # Construct the keys for the filter function error msg
          my @keyName;
          my @matched;
          my $j;
          for ($j = $[; $j <= $#xmlStack; $j++) {
            push @keyName, ${$xmlStack[$j]}[1];
            push @matched, ${$xmlStack[$j]}[1] if ($j >= $i);
          }
          # Run the filtering function
          &$filterHashDeRef(\$val, $globParseTreeRef, \@keyName, \@matched);
        } elsif ($type || !$filterHashDeRef) {
          $val = ""; # filter out this value
        }
      } else {
        $val = ""; # filter out this value
      }
      last;
    }
  }
  my ($oldParseTreeRef, $startArg) = @{pop @xmlStack};
  die "Argument passed to expatStart ($startArg) ".
      "doesn't match the one passed to expatEnd ($arg)" if ($arg ne $startArg);
  if (! %$globParseTreeRef) {
    # Empty hash -- delete it 
    my $arrayRef = ${$oldParseTreeRef}{$arg};
    if ($val ne "") {
      # Replace the hash with the value
      $$arrayRef[$#$arrayRef] = $val;
      $val = "";
    } else {
      if ($#$arrayRef > $[) {
        # delete the element
        $#$arrayRef --;
      } else {
        # delete the whole array (hash entry)
        delete ${$oldParseTreeRef}{$arg};
      }
    }
  }

  $globParseTreeRef = $oldParseTreeRef;
  return undef;
}; # expatEnd

################################################################################
# FUNCTION: expatChar
#
# DESCR: Called by expat whenver a string is found that is outside of <>'s.  
#        The $arg is used to construct $val step by step.  Blank/whitespace values
#        are discarded.  
#
# ARGS: unused:     (i) expat instance pointer
#       $arg:       (i) string that is outside of <> in XML
#
# Return Val: Nothing
################################################################################
my $expatChar = sub
{
  my ($arg) = $_[1];

  if (! ($arg =~ /^\s*$/)) {
    $val .= $arg; 
  }
  return undef;
}; #expatChar

################################################################################
# FUNCTION: pruneTree
#
# DESCR: Traverses a resulting pruneTree and simplifies it by
#        A) collapsing arrays of one element (inlining the array[0])
#        B) Removing arrays of no elements
#        C) Removing hashes with no keys
#
# ARGS: $treeRefRef  (i/o) double reference to a tree (hash) which is
#       modified in-place.  Remember perl auto-frees the associated memory via
#       garbage collection and perhaps defragmentation.
#
# Return Val: Nothing
################################################################################
my $pruneTree;
$pruneTree = sub
{
  my ($treeRefRef) = @_;

  return undef if (! defined ($treeRefRef));
  return undef if (! ref ($treeRefRef));
  my $treeRef = $$treeRefRef;
  return undef if (! defined ($treeRef));
  return undef if (! ref ($treeRef));
  my $type = ref ($treeRef);
  if ($type eq "ARRAY") {
    # Iterate over each element in the array
    my ($i, $j);
    for ($i = $j = $[; $i <= $#$treeRef; $i++) {
      &$pruneTree (\$$treeRef[$i]);
      if (defined ($$treeRef[$i])) {
        $$treeRef[$j++] = $$treeRef[$i];
      }
    }
    if ($j == $[) {
      # The whole array is gone
      $$treeRefRef = undef;
    } elsif ($j == 1) {
      # The array is one element -- replace array with contents
      $$treeRefRef = $$treeRef[0];
    }
  } elsif ($type eq "HASH") {
    # Iterate over each key in the hash
    my @keys = keys (%$treeRef);
    my $key;
    foreach $key (@keys) {
      &$pruneTree (\$$treeRef{$key});
      if (! defined ($$treeRef{$key}) ) {
        delete $$treeRef{$key};
      }
    }
    if (! %$treeRef) {
      # The whole hash is gone
      $$treeRefRef = undef;
    }
    # Keep hashes of one element...
  }
  
  return undef;
}; # pruneTree

################################################################################
# FUNCTION: ofdUtil::parse
#
# DESCR: Construct parse tree in $parseTreeRef (which should be a reference
#        to a hash).  
#
# ARGS: $ofdUtil:    (i) path to ofd6x.exe or similar tool
#       $outFile:    (i) application file (eg th##.out)
#       $parseTreeRef: (i/o) Reference (pointer) to parse tree (hash)
#       $restrictRef (i) Reference to array of interesting parse trees
#
# Sample XML for formats below
# 
# <?xml version="1.0"?>
# <level 1>
#   <level 2>
#     <level 3>
#       <l3v1>value 1</l3v1>
#       <l3v2>value 2</l3v2>
#       <l3v1>value 3</l3v1>
#     </level 3>
#   </level 2>
#   <level 2b>
#     <l2v1>value 4</l2v1>
#   </level 2b>
# </level 1>
#
# Resulting contents of the $parseTreeRef without filtering
# Remember that {} is a hash and [] is an array
#
# This is packed together much as XML::Simple would.
#
# {
#   'level1' => 
#   {
#     'level2' => 
#     [
#       {
#         'level3' => 
#         {
#           'l3v1' => 
#           [
#            'value 1',
#            'value 3'
#           ],
#           'l3v2' => 'value 2'
#         }
#       },
#       {
#         'l2v1' => 'value 4'
#       }
#     ]
#   }
# };
#
# However for ofd this is way too big
# This parser allows for filtering out unwanted fields before
# storing to limit memory.  Without filtering a 12.1 th10 can
# take 2+GB RAM.
# 
# Detailed format of $restrictRef
#
# [ 
#   [ "path1", "path2" ],
#   [ "path3" ]
# ]
#
# Each list is the xml keys that are allowed.  The rightmost
# key is matched first to the leaf of the tree then the match
# goes backwards towards the root.  The pattern matches
# if all keys match relative to the leaf.  It does not have to
# match the root (ie, the keys are right/leaf justified).
#
# For example in the above xml/tree use the following restrict
#
# [
#   [ qw(level2 level3 l3v2)],
#   [ "l2v1" ]
# ]
#
# The resulting parse tree is:
#
# Returns: error message
#
################################################################################
sub parse 
{
  my ($ofdUtil, $outFile, $parseTreeRef, $restrictRef) = @_;
  local *OFD;

  return "The specified ofd utility ($ofdUtil) is not found\n" if (! -f $ofdUtil);
  return "The specified .out file ($outFile) is not found\n" if (! -f $outFile);

  return "\$parseTreeRef must be a reference to a hash\n" 
    if (ref ($parseTreeRef) ne "HASH");

  # Reset tree
  %$parseTreeRef = ();
  $globParseTreeRef = $parseTreeRef;

  # Convert restrictRef by reversing each rule into a hash (for fast searching
  # when each key from the ofd is added
  if (defined($restrictRef)) {
    $filtering = 1;
    %filterHash = ();
    my $rule;
    foreach $rule (@$restrictRef) {
      my $j;
      my $filterHashRef = \%filterHash;
      my $ruleVal = 1;
      my $loopStart = $#$rule;
      if (ref($$rule[$loopStart]) eq "CODE") {
        $ruleVal = $$rule[$loopStart--];
      }
      for ($j = $loopStart; $j >= $[; $j--) {
        my $filterHashDeref = $$filterHashRef{$$rule[$j]};
        if (defined ($filterHashDeref)) {
          if (ref($filterHashDeref) eq "HASH") {
            if ($j == $[) {
              # Its the end of this rule.  Since there is already a hash,
              # it can be dropped since this rule is more permissive
              $$filterHashRef{$$rule[$j]} = $ruleVal;
              print STDERR "Warning: Filter rule (",
                           join (" ", @$rule), 
                           ") is more general than a predecessor.  Predecessor overridden.\n";
            }
          } else {
            print STDERR "Warning: Filter rule (",
                         join (" ", @$rule), 
                         ") is less general than a predecessor (",
                         join (" ", @$rule[$j..$#$rule]),
                         ").  Dropping this rule.\n";
            last;
          }
        } else {
          # Nothing exists
          if ($j > $[) {
            # There is more in this rule.  Create a hash for it.
            $filterHashDeref = $$filterHashRef{$$rule[$j]} = {};
          } else {
            # Rule fully defined
            $$filterHashRef{$$rule[$j]} = $ruleVal;
          }
        }
        $filterHashRef = $filterHashDeref;
      }
    }
  } else {
    $filtering = 0;
  }

  # Parse it
  print "Processing $outFile -- this can take a couple minutes\n";

  open OFD, "\"$ofdUtil\" -x $outFile |" or return "Can't open pipe to $ofdUtil\n";
  binmode OFD;
  my $xmlHandle = new XML::Parser(Handlers=> {
                                  Start => $expatStart,
                                  End => $expatEnd,
                                  Char => $expatChar
                                 });
  $xmlHandle->parse(*OFD);
  close OFD;

  # Clean up the tree removing blank hashes and one-element arrays
  &$pruneTree (\$parseTreeRef);

  return undef;
} # ofdParser::parse

################################################################################
# FUNCTION: expatDumpStart (when producing xml debug dump)
#
# DESCR: Called by expat whenver an "begin" operator <operator> is found.
#        The value "operator" is removed added to @xmlStack.
#
# ARGS: unused:     (i) expat instance pointer
#       $arg:       (i) operator
#
# Globals for all states: 
#    @xmlStack:     (i) each element is string in each nested <>.
#
# Return Val: Nothing
################################################################################
my $expatDumpStart = sub
{
  my ($arg) = $_[1];

  push @xmlStack, $arg;
  return undef;
}; # expatDumpStart

################################################################################
# FUNCTION: expatDumpEnd (when producing xml debug dump)
#
# DESCR: Called by expat whenver an "end" operator </operator> is found.
#        The value "operator" is removed from @xmlStack.
#
# ARGS: unused:     (i) expat instance pointer
#       unused:     (i) operator
#
# Globals for all states: 
#    @xmlStack:     (i) each element is string in each nested <>.
#
# Return Val: Nothing
################################################################################
my $expatDumpEnd = sub
{
  pop @xmlStack;
  return undef;
}; # expatDumpEnd


################################################################################
# FUNCTION: expatDumpChar (when producing xml debug dump)
#
# DESCR: Called by expat whenver a string is found that is outside of <>'s.  
#        Using @xmlStack, the value is printed in dotted C like format.
#
# ARGS: unused:     (i) expat instance pointer
#       $arg:       (i) string that is outside of <> in XML
#
# Globals for all states: 
#    @xmlStack:     (i) each element is string in each nested <>.
#
# Return Val: Nothing
################################################################################
my $expatDumpChar = sub
{
  my ($arg) = $_[1];
  return if ($arg =~ /^\s*$/);
  print OFDDUMP join(".",@xmlStack), "=$arg\n";
  return undef;
}; # expatDumpChar

################################################################################
# FUNCTION: ofdUtil::printxml
#
# DESCR: Convert xml to dotted C format for easy searching with regular editor
#
# ARGS: $ofdUtil:    (i) path to ofd6x.exe or similar tool
#       $outFile:    (i) application file (eg th##.out)
#       $outputFile: (i) path to write output
#
################################################################################
sub printxml 
{
  my ($ofdUtil, $outFile, $outputFile) = @_;

  die "The specified ofd utility ($ofdUtil) is not found\n" if (! -f $ofdUtil);
  die "The specified .out file ($outFile) is not found\n" if (! -f $outFile);

  local *OFD;
  open OFDDUMP, ">$outputFile" or die "Can't open $outputFile for writing\n";
  open OFD, "$ofdUtil -x $outFile |" or die "Can't open pipe to $ofdUtil\n";
  binmode OFD;
  my $xmlHandle = new XML::Parser(Handlers=> {
                                  Start => $expatDumpStart,
                                  End => $expatDumpEnd,
                                  Char => $expatDumpChar
                                 });
  $xmlHandle->parse(*OFD);
  close OFD;
  return undef;
} # ofdUtil::printxml

return 1; # Required to make "use" happy

# Nothing past this point

