################################################################################
# Purpose: Library routines for traversing OFD parse trees
#
# Public functions
#    ofdUtils::lookUp:        search for a key with constraints
#    ofdUtils::print:         prints parse tree for debug
#    ofdUtils::getBigEndian   Return 1 when big endian
#
#    The following functions can all be used with hte filters
#    provided to ofdParser:parse()
#
#    ofdUtils::convertBool:   converts a boolean to a number
#    ofdUtils::convertNumber: converts a number in a string to a number like C
#    ofdUtils::convertBase64: Converts base64 encoding to binary
#
#    The following functions can be used by filters to delete a field (entry)
#    or record (entire structure).
#
#    ofdUtils::deleteField:   delete field (name/value pair)
#    ofdUtils::deleteRecord:  delete structure containing this name/value pair
#
# (C) Copyright 2010, Texas Instruments Incorporated.
# * 
# *  Redistribution and use in source and binary forms, with or without 
# *  modification, are permitted provided that the following conditions 
# *  are met:
# *
# *    Redistributions of source code must retain the above copyright 
# *    notice, this list of conditions and the following disclaimer.
# *
# *    Redistributions in binary form must reproduce the above copyright
# *    notice, this list of conditions and the following disclaimer in the 
# *    documentation and/or other materials provided with the   
# *    distribution.
# *
# *    Neither the name of Texas Instruments Incorporated nor the names of
# *    its contributors may be used to endorse or promote products derived
# *    from this software without specific prior written permission.
# *
# *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
# *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
# *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
# *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
# *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
# *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
# *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
# *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
# *
# */

package ofdUtils;

use strict;
use diagnostics;
use warnings;
use MIME::Base64;
use File::Basename;

################################################################################
# FUNCTION: ofdUtils::lookUp
#
# DESCR: Look up a key in a parse tree
#
# ARGS: $parseTreeRef (i) Parse tree to search
#       $queryRef     (i/o) Query array
#
# The query consists of a reference to an array of keys to search for.
# Each entry is a reference to a hash containing the following hash keys:
#
# name       => (i) string/name for this key (required)
# maxMatches => (i) Fails if this key is found more than maxMatches times 
#                   default: unlimited
# index      => (i) When there are more than one matches, which index to return.
#                   default: first
# numMatches => (i/o) If defined on entry, count the total number of matches
#                     and return here.
# lastMatchPos => (i/o) (pvt) Last position in array where match found
# lastMatchIdx => (i/o) (pvt) Index value that last matched
#
# Return Val: (errorString, results)
#
################################################################################
sub lookUp 
{
  my ($parseTreeRef, $queryRef) = @_;

  my $errorString = "";
  my $i;
  my $result;
  for ($i = $[; $i <= $#$queryRef; $i++) {
    my $queryKeyRef = $$queryRef[$i];
    if (ref($queryKeyRef) ne "HASH") {
      $errorString .= "Query key $i is not a reference to a hash\n";
      last;
    }
    my %queryCopy = %$queryKeyRef;
    my $name = delete ($queryCopy{"name"});
    if (! $name) {
      $errorString .= "Query key $i does not specify a name\n";
      last;
    }
    my $maxMatches = delete ($queryCopy{"maxMatches"});
    $maxMatches = 0 if (!defined($maxMatches));
    my $index = delete($queryCopy{"index"});
    $index = 0 if (!defined($index));
    my $cntMatches = delete($queryCopy{"numMatches"});
    $cntMatches = $maxMatches ? 1 : 0;
    my $numMatches = 0;
    my $lastMatchPos = delete($queryCopy{"lastMatchPos"});
    my $lastMatchIdx = delete($queryCopy{"lastMatchIdx"});
    if (%queryCopy) {
      $errorString .= "For query key $i($name), found unexpected keys: ".
                      join (",",keys(%queryCopy))."\n";
      last;
    }
    my $match;
    my $matchPos;
    if (ref($parseTreeRef) eq "HASH") {
      # there is one possible match
      $match = $$parseTreeRef{$name};
      if (defined($match)) {
        if ($index > 0) {
          $errorString .= "For query key $i($name), requested $index too big\n";
          last;
        }
        $matchPos = 0;
        $numMatches = 1;
      } else {
        $errorString .= "For query key $i($name) found hash with no match\n";
        last;
      }
    } elsif (ref($parseTreeRef) eq "ARRAY") {
      # Go find the requested match
      my $startI = $lastMatchPos;
      $startI = $[ - 1 if (!defined($startI));
      if ($cntMatches) {
        $startI = $[ - 1;
      }
      $startI++;
      my $j;
      my $matchNum = $lastMatchIdx;
      $matchNum = $[ - 1 if (!defined($matchNum));
      $matchNum = $[ - 1 if ($cntMatches);
      $matchNum++;
      for ($j = $startI; $j <= $#$parseTreeRef; $j++) {
        if (defined(${$$parseTreeRef[$j]}{$name})) {
          # This is a match
          if ($matchNum == $index) {
            # Selected match
            $match = ${$$parseTreeRef[$j]}{$name};
            $matchPos = $j;
          } else {
            $matchNum++;
          }
          $numMatches++;
          last if (! $cntMatches);
        }
      }
    }
    if (defined($match)) {
      if ($i == $#$queryRef) {
        $result = $match;
      } else {
        $parseTreeRef = $match;
      }
    } else {
      if ($i < $#$queryRef) {
        $errorString .= "Query key $i($name), index $index not found\n";
      }
      last;
    }
    if ($cntMatches) {
      $$queryKeyRef{"numMatches"} = $numMatches;
    }
    if ($maxMatches) {
      if ($numMatches > $maxMatches) {
        $errorString .= "Actual matches $numMatches exceeds max allows $maxMatches\n";
        last;
      }
    }
    $$queryKeyRef{"lastMatchPos"} = $matchPos;
    $$queryKeyRef{"lastMatchIdx"} = $index;
  }
  # Get rid of arrays of 1 element
  $result = $$result[0] if ((ref($result) eq "ARRAY") && ($#$result == $[));
  return ($errorString, $result);
} # ofdUtils::lookUp

################################################################################
# FUNCTION: ofdUtils::print
#
# DESCR: Prints parse tree (similar to Data::Dumper, but without crashing)
#
################################################################################
################################################################################
#        Hash keys starting with "link" are not traversed.  This prevents
#        circular referencing/infinite recursion.  Data::Dumper handles
#        this but this probably leads to its crash on large data structures.
################################################################################
################################################################################
#
# To pass a file handle, use ofdUtils::print (*STDOUT, $parseTree);
#
# ARGS: $fhandle:      (i) Handle to print to
#       $treeRef:      (i) parse tree to print
#       $indent:       (i) indent level (for recursion)
#
# Return Val: None
################################################################################
sub print 
{
  my ($fhandle, $treeRef, $indent) = @_;
  $indent = 0 if (!defined $indent);
  my $spaces = " " x ($indent * 4);
  
  if (ref($treeRef) eq "HASH") {
    my @keys = keys (%$treeRef);
    my $key;
    foreach $key (@keys) {
      print $fhandle "\n$spaces\{\"$key\"\} = ";
      my $recurseRef = $$treeRef{$key};
      if (($key =~ /^link[A-Z]/) && ref($recurseRef)) {
        # If they key starts with "link" presume its a cross
        # reference/circular link unless it is a scalar.  
        # Convert the circular link to a scalar to stop recursion.
        $recurseRef = "link to $recurseRef";
      }
      ofdUtils::print ($fhandle, $recurseRef, $indent + 1);
    }
  } elsif (ref($treeRef) eq "ARRAY") {
    my $i;
    for ($i = $[; $i <= $#$treeRef; $i++) {
      print $fhandle "\n$spaces\[$i] = ";
      ofdUtils::print ($fhandle, $$treeRef[$i], $indent + 1);
    }
  } elsif (ref($treeRef) ne "") {
    print STDERR "ofdUtils::print: unknown reference type ",ref($treeRef), "\n";
  } else {
    if ($treeRef =~ /[[:cntrl:]]/) {
      # Binary data
      my $i;
      my $len = length($treeRef);
      my $trailer = "";
      if ($len > 40) {
        $trailer = " ...";
        $len = 40;
      }
      print $fhandle "0x";
      for ($i = 0; $i < $len; $i++) {
        printf $fhandle ("%02x", ord substr ($treeRef, $i, 1));
      }
      print $fhandle $trailer;
    } else {
      my $quote = ($treeRef =~ /^\d/) ? "" : '"';
      print $fhandle $quote, $treeRef, $quote;
    }
  }
  # When done printing everything (ending recursion) end with newline
  print $fhandle "\n" if (! $indent);
  return undef;
} # ofdUtils::print

################################################################################
# FUNCTION: ofdUtils::isElf
#
# DESCR: finds ofd.object_file.ti_coff.file_header (to decide it is coff)
#        or    ofd.object_file.elf.elf32_ehdr (to decide it is elf)
#
# ARGS: $treeRef:      (i) parse tree to search
#
# Return value ($isElf, $error)
################################################################################
sub isElf {
  my ($treeRef) = @_;

  my @queryCoff = (
    {name => "ofd", maxMatches => 1},
    {name => "object_file", maxMatches => 1},
    {name => "ti_coff", maxMatches => 1},
    {name => "file_header", maxMatches => 1}
  );

  my @queryElf = (
    {name => "ofd", maxMatches => 1},
    {name => "object_file", maxMatches => 1},
    {name => "elf", maxMatches => 1},
    {name => "elf32_ehdr", maxMatches => 1}
  );

  my ($ofdErrorCoff, $coff) = ofdUtils::lookUp ($treeRef, \@queryCoff);
  my ($ofdErrorElf, $elf) = ofdUtils::lookUp ($treeRef, \@queryElf);

  if ($ofdErrorCoff && $ofdErrorElf) {
    # Try again without leading "ofd"
    shift @queryCoff;
    shift @queryElf;
    ($ofdErrorCoff, $coff) = ofdUtils::lookUp ($treeRef, \@queryCoff);
    ($ofdErrorElf, $elf) = ofdUtils::lookUp ($treeRef, \@queryElf);
    if ($ofdErrorCoff && $ofdErrorElf) {
      return (undef, "Not coff or elf: coff error=$ofdErrorCoff".
                     " and elf error=$ofdErrorElf");
    }
  }
  if ((! $ofdErrorCoff) && (! $ofdErrorElf)) {
    return (undef, "somehow matched both elf and coff!");
  }
  if ($ofdErrorCoff) {
    return (1, undef); # it is elf
  } else {
    return (0, undef); # it is coff
  }
} # ofdUtils::isElf

################################################################################
# FUNCTION: ofdUtils::getBigEndian
#
# DESCR: finds ofd.object_file.ti_coff.file_header.endian (for out/obj)
#           or object_file.ti_coff.file_header.endian (for archive)
#
# ARGS: $treeRef:      (i) parse tree to search
#
# Return value ($bigEndian, $error)
################################################################################
sub getBigEndian { 
  my ($treeRef) = @_;

  my ($isElf, $isElfError) = ofdUtils::isElf ($treeRef);
  return (undef, "isElf failed: $isElfError") if ($isElfError);

  my @query;

  if ($isElf) {
    @query = (
      {name => "ofd", maxMatches => 1},
      {name => "object_file", maxMatches => 1},
      {name => "elf", maxMatches => 1},
      {name => "elf32_ehdr", maxMatches => 1},
      {name => "ei_data", maxMatches => 1}
    );
  } else {
    @query = (
      {name => "ofd", maxMatches => 1},
      {name => "object_file", maxMatches => 1},
      {name => "ti_coff", maxMatches => 1},
      {name => "file_header", maxMatches => 1},
      {name => "endian", maxMatches => 1}
    );
  }

  my ($ofdError, $endian) = ofdUtils::lookUp ($treeRef, \@query);

  if ($ofdError) {
    shift @query; # Try again without "ofd" prefix
    my $objError;
    ($objError, $endian) = ofdUtils::lookUp ($treeRef, \@query);
    if ($objError) {
      return (undef, "lookUp error with ofd prefix: $ofdError; ".
                     "lookUp error without ofd prefix: $objError");
    }
  }

  if (($isElf && ($endian eq "ELFDATA2MSB")) ||
      ((! $isElf) && ($endian eq "big"))) {
    return (1, undef);
  } elsif (($isElf && ($endian eq "ELFDATA2LSB")) ||
      ((! $isElf) && ($endian eq "little"))) {
    return (0, undef);
  }
  return (undef, "Unknown endian $endian\n");
} # ofdUtils::getBigEndian

################################################################################
# FUNCTION: ofdUtils::convertBool
#
# DESCR: Converts true/false string in $$inRef to 0 or 1.
#
# ARGS: $inRef:      (i/o) string to convert
#       $subTreeRef: (i) subtree below this key (not used)
#       $keyRef:     (i) entire key name associated with $$inRef
#       $matchRef    (i) matched key name associated with $$inRef
#
# Return Val: None
################################################################################
sub convertBool 
{
  my ($inRef, $parseTreeRef, $keyRef, $matchRef) = @_;
  my $val = lc ($$inRef);
  if ($val eq "true") {
    $$inRef = 1;
  } elsif ($val eq "false") {
    $$inRef = 0;
  } else {
    die "Found invalid boolean: ", join(".",@$keyRef), "=$val\n";
  }
  return undef;
} # ofdUtils::convertBool

################################################################################
# FUNCTION: ofdUtils::convertNumber
#
# DESCR: Converts the stringified number in $$inRef into a number.
#        eval() is used which can execute arbitrary code, but
#        this is controlled environment so ignore any security warnings 
#        from perl.
#
# ARGS: $inRef:      (i/o) string to convert
#       $subTreeRef: (i) subtree below this key (not used)
#       $keyRef:     (i) entire key name associated with $$inRef
#       $matchRef    (i) matched key name associated with $$inRef
#
# Return Val: None
################################################################################
sub convertNumber 
{
  my ($inRef, $parseTreeRef, $keyRef, $matchRef) = @_;
  my $val = eval ($$inRef);
  if ($@) {
    die "Returned error $@ while evaluating numeric value for key ", 
        join (".", @$keyRef), "=$$inRef\n";
  }
  $$inRef = 0 + $val;
  return undef;
} # ofdUtils::convertNumber

################################################################################
# FUNCTION: ofdUtils::convertBase64
#
# DESCR: Converts a base64 (MIME) encoded string to binary
#
# ARGS: $inRef:      (i/o) string to convert
#       $subTreeRef: (i) subtree below this key (not used)
#       $keyRef:     (i) entire key name associated with $$inRef
#       $matchRef    (i) matched key name associated with $$inRef
#
# Return Val: None
################################################################################
sub convertBase64
{
  my ($inRef, $parseTreeRef, $keyRef, $matchRef) = @_;
  my ($rawDataRef) = @_;
  # Whack spaces to allow length check to work
  $$inRef =~ s/\s+//g;
  my $inLen = length ($$inRef);
  $$inRef = decode_base64($$inRef);
  my $outLen = length ($$inRef);
  my $expLen = int($inLen / 4 * 3);
  if (($outLen < ($expLen - 3)) || ($outLen > $expLen)) {
    die "decode_base64 returned to few bytes for key ",
        join (".", @$keyRef), "\n.  Got $outLen bytes from
        $inLen input bytes.  Expected $expLen output bytes.\n";
  }
  return undef;
} # ofdUtils::convertBase64

################################################################################
# FUNCTION: ofdUtils::deleteField
#
# DESCR: Deletes field ($$inRef)
#
# ARGS: $inRef:      (i/o) string to convert
#       $subTreeRef: (i) subtree below this key (not used)
#       $keyRef:     (i) entire key name associated with $$inRef
#       $matchRef    (i) matched key name associated with $$inRef
#
# Return Val: None
################################################################################
sub deleteField
{
  my ($inRef, $parseTreeRef, $keyRef, $matchRef) = @_;
  $$inRef = "";
  return undef;
} # ofdUtils::deleteField

################################################################################
# FUNCTION: ofdUtils::deleteRecord
#
# DESCR: Deletes record containing this field
#
# ARGS: $inRef:      (i/o) string to convert
#       $subTreeRef: (i) subtree below this key (not used)
#       $keyRef:     (i) entire key name associated with $$inRef
#       $matchRef    (i) matched key name associated with $$inRef
#
# Return Val: None
################################################################################
sub deleteRecord
{
  my ($inRef, $parseTreeRef, $keyRef, $matchRef) = @_;
  if (ref($parseTreeRef) eq "HASH") {
    %$parseTreeRef = ();
  } elsif (ref($parseTreeRef) eq "ARRAY") {
    @$parseTreeRef = ();
  }
  $$inRef = "";
  return undef;
} # ofdUtils::deleteRecord

return 1; # Required to make "use" happy
# Nothing past this point

