#*
#*
#* Copyright (C) 2010 Texas Instruments Incorporated - http://www.ti.com/ 
#* 
#* 
#*  Redistribution and use in source and binary forms, with or without 
#*  modification, are permitted provided that the following conditions 
#*  are met:
#*
#*    Redistributions of source code must retain the above copyright 
#*    notice, this list of conditions and the following disclaimer.
#*
#*    Redistributions in binary form must reproduce the above copyright
#*    notice, this list of conditions and the following disclaimer in the 
#*    documentation and/or other materials provided with the   
#*    distribution.
#*
#*    Neither the name of Texas Instruments Incorporated nor the names of
#*    its contributors may be used to endorse or promote products derived
#*    from this software without specific prior written permission.
#*
#*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
#*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
#*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
#*  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
#*  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
#*  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
#*  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
#*  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
#*  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
#*  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
#*  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#*


# Syntax: rtsc.pl <arguments>
#         -xdc=path             - path to xdc.exe
#         -rtscdir=path         - root to untar repository
#         -arg.##=arg           - ## represents phases to use this argument
#                                 arg = argument to XDCARGS
#         -pkgroot=path         - path to rtsc repository
#         -pkg=component        - rtsc target/package to operate on
#         -clean=YES|NO|ONLY|   - YES: clean then build component
#                                 NO/nothing: build component normally
#                                 ONLY: clean and exit
#         -tarrep=path          - use pre-built .tar for stg1 in place of -pkgroot
#                                 (this can also be defined through RTSC_TARREP
#                                  environment variable)
#         -cspec=path           - path to configspec.clearcase (version info)

use strict;
use Diagnostics;
use Digest::MD5;

# Variables for command line arguments
my ($xdc, $rtscdir, @args_stg1, @args_stg2, $pkgroot, $pkg, $clean, 
    $clean_exit, $tarrep, $cspec);

# Local variables
my ($arg, $cmd, $ret, @tars, $tar, $md5file, %md5old, %md5new, $error_exit);
my (@cleanup_deletes, @md5_cleanup_on_untar, $do_untar, $tarpath, $bypass_option);

$bypass_option = "bypass";
if (defined ($ENV{"RTSC_ENABLE_WRITE"})) {
  if ($ENV{"RTSC_ENABLE_WRITE"} eq "enable_write") {
    $bypass_option = "enable_write";
  } elsif ($ENV{"RTSC_ENABLE_WRITE"} eq "verify") {
    $bypass_option = "";
  }
}

$error_exit = 0;
$clean = 0;
$clean_exit = 0;
# Parse arguments
foreach $arg (@ARGV) {
  $arg =~ s/\\/\//g;
  if ($arg =~ /^-xdc=(.*)/) {
    $xdc = $1;
  } elsif ($arg =~ /^-rtscdir=(.*)/) {
    $rtscdir = $1;
  } elsif ($arg =~ /^-arg\.(\d+)=(.*)/) {
    if ($1 eq "1") {
      push @args_stg1, $2;
    } elsif ($1 eq "2") {
      push @args_stg2, $2;
    } elsif ($1 eq "12") {
      push @args_stg1, $2;
      push @args_stg2, $2;
    } else {
      error_exit("For argument $arg, can't identify stages");
    }
  } elsif ($arg =~ /^-pkgroot=(.*)/) {
    $pkgroot = $1;
  } elsif ($arg =~ /^-pkg=(.*)/) {
    $pkg = $1;
  } elsif ($arg =~ /^-clean=(.*)/) {
    if ($1 =~ /yes/i) {
      $clean = 1;
      $clean_exit = 0;
    }
    if ($1 =~ /only/i) {
      $clean = 1;
      $clean_exit = 1;
    }
  } elsif ($arg =~ /^-tarrep=(.*)/) {
    # Repository mode -- skip stage 1
    $tarrep = $1;
  } elsif ($arg =~ /^-cspec=(.*)/) {
    $cspec = $1;
  } else {
    error_exit("Unknown argument $arg");
  }
}

# Check that required arguments are present
if (!defined($xdc)) {
  error_exit("-xdc not defined");
}

if (!defined($rtscdir)) {
  error_exit("-rtscdir not defined");
}

if (!defined($pkgroot) && !defined($tarrep)) {
  error_exit("-pkgroot not defined");
}

if (!defined($pkg)) {
  error_exit("-pkg not defined");
}

print "$0: Working on RTSC package $pkg\n";

# Use environment var if defined but arg not used
if (!defined($tarrep) && $ENV{"RTSC_TARREP"}) {
  $tarrep = $ENV{"RTSC_TARREP"};
}

if (defined($tarrep)) {
  if (!defined($cspec)) {
    error_exit("-cspec must be defined when using repository");
  }

  # Use pre-existing tar repository
  open INPUT, $cspec or die "Can't open $cspec\n";
  # Skip to Begin_DEP
  while(<INPUT>) {
    last if /^\s*Begin_DEP\s*$/;
  }
  my ($component) = $pkg;
  my ($version, $target);
 
  $component =~ s/[\/\\]/./g;
  my $test_pkg=0;
  
  if($component=~ m/test/) {
  # Test Component, so skip the version number check
    $test_pkg=1;
  } else {
    $test_pkg=0;
    while(<INPUT>) {
      chomp;
      # Find component version
      # Delete comment
      s/\#.*//;
      # Skip blank line
      next if (/^\s*$/);
      last if (/^\s*End_DEP\s*$/);
      if (/^\s*$component\s+(\d+\.\d+\.\d+\.\d+)\s*$/) {
        $version = $1;
        $version =~ s/\./_/g;
        last;
      }
    }
    if (!defined($version)) {
      error_exit("Can't find version for $component");
    }
  }  

  # Find the right .tar for this version
  findtars (\@tars, $tarrep, "");
  my ($i, $j);
  for ($i = 0, $j = 0; $i <= $#tars; $i++) {
    if($test_pkg==0) { 
    #Only non-test src tars have version numbers
      if (($tars[$i] =~ /^${component}(.*)$version\.zip$/)  || 
          ($tars[$i] =~ /^${component}(.*)$version\_beta\.zip$/)){
         $tars[$j++] = $tars[$i];
      }
    } else {
      $component =~ s/.test$//;
      if ($tars[$i] =~ /^${component}(.*)test\.zip$/) {
         $tars[$j++] = $tars[$i];
      }
    }
      
  }
  # Truncate the mismatched files
  $#tars = $j - 1;
  if ($j != 1) {
    # Fix this by coming up with rules to select best match
    error_exit("Found $j matches; expect exactly one tar to match");
  }
  $tarpath = "$tarrep";
  print "$0: Using repository tar $tarpath/$tars[0]\n";
} else {
  # Run clean if needed
  if ($clean) {
    $ret = run_rtsc($pkgroot, $pkg, "$xdc clean");
    # Fix for ignoring the error for non existent components to take care of clean
    if (($ret) && ($ret != 512)) {
      error_exit("Clean failed with code $ret");
    }
    exit(0) if ($clean_exit);
  }

  # Delete existing tar files
  findtars (\@tars, $pkgroot, $pkg);
  foreach $tar (@tars) {
    print "$0: Delete tar file $pkgroot/$pkg/$tar\n";
    $! = 0;
    unlink "$pkgroot/$pkg/$tar";
    if ($!) {
      error_exit("Can't delete $pkgroot/$pkg/$tar ($!)");
    }
  }

  # Run first stage build
  $cmd = "$xdc ";
  $cmd .= "XDCARGS=\"" . join (" ", @args_stg1) . '"' if (@args_stg1);
  $cmd .= " release";
  print "$0: Running first stage build: $cmd\n";
  $ret = run_rtsc($pkgroot, $pkg, $cmd);
  if ($ret) {
    error_exit("Stage 1 build failed with code $ret");
  }

  # Cannot just md5 each tar file, because the file date/time within the tar
  # for some of the .xml files changes.  Therefore, untar to scratch, and
  # calculate md5 of the results.
  # 
  # Also, cannot compare directly to what is in output directory, because
  # it will be changed for a source release.

  # Find each new tar file
  findtars (\@tars, $pkgroot, $pkg);

  if ($#tars < $[) {
    error_exit("No tars found for target $pkg");
  }
  $tarpath = "$pkgroot/$pkg";
}

# find all possible .md5 files that reflect this package
my (@md5list);
findtars (\@md5list, $rtscdir, ".", $pkg);

# There should be exactly one tar found, and 0 or 1 md5
if ($#tars != $[) {
  # While there is a loop below, multiple output packages doesn't
  # match expected functionality.  Need to select which package
  # (instead of all) to extract
  error_exit ("package should produce exactly one tar/zip\n");
}

if ($#md5list >= $[) {
  # Unless the md5 file found exactly matches the current tar/zip, delete
  # (ignore) the md5 files
  my $md5for_tar = $tars[0];
  $md5for_tar =~ s/\.(?:zip|tar)/.md5/;
  my $md5match;
  foreach $md5match (@md5list) {
    if ($md5match ne $md5for_tar) {
      print "Found md5 file $md5match that doesn't match this tar $tars[0]; deleting\n";
      unlink "$rtscdir/$md5match" or error_exit("delete $rtscdir/$md5match failed\n");
    }
  }
}

# Go through each tar and see if anything changed
$do_untar = 0;
foreach $tar (@tars) {
  my ($writemd5);
  $md5file = $tar;
  $md5file =~ s/\.zip$/.md5/;
  $writemd5 = ! md5read (\%md5old, "$rtscdir/$md5file");

  # Find new md5 information
  prunetree ("$rtscdir/scratch");
  untar ("$tarpath/$tar", "$rtscdir/scratch", undef);
  md5tree (\%md5new, "$rtscdir/scratch/$pkg");
  prunetree ("$rtscdir/scratch");
  if (!$writemd5) {
    # Successfully read a file, so check it
    $writemd5 = md5compare(\%md5new, \%md5old);
  }
  if ($writemd5) {
    # Either file read failed, or compare failed, so write md5
    md5write (\%md5new, "$rtscdir/$md5file");
    # Save written filename, so any failure after this point will clean it
    push @cleanup_deletes, "$rtscdir/$md5file";
    $do_untar = 1;
    print "$0: md5s for $tar have changed, extracting...\n";
  } else {
    # Save md5 file on list to delete, if untar fails
    push @md5_cleanup_on_untar, "$rtscdir/$md5file";
    print "$0: md5s for $tar have not changed; skipping extract\n";
  }
}

# Untar all tar files, after deleting current untar directory
if ($do_untar) {
  # Add rest of md5 files to delete list if untar fails;
  push @cleanup_deletes, @md5_cleanup_on_untar;
  prunetree ("$rtscdir/$pkg");
  # Clean result of untar if any errors after this point
  push @cleanup_deletes, "$rtscdir/$pkg";

  foreach $tar (@tars) {
    untar ("$tarpath/$tar", $rtscdir, $pkg);
  }
}

# Since untar was successful, nothing to clean up on failure
@cleanup_deletes = ();

# Do second stage build if required
if (@args_stg2) {
  # Run second stage build
  $cmd = "$xdc XDCARGS=\"" . join (" ", @args_stg2) . "\""; # release";
  print "$0: Running second stage build: $cmd\n";
  $ret = run_rtsc($rtscdir, $pkg, $cmd);
  if ($ret) {
    error_exit("Stage 2 build failed with code $ret");
  }
}

# Find tar files in the package directory
sub findtars {
  my ($tarref, $root, $pkgdir, $pkgname) = @_;
  my (@files, $file, $tarbase);

  $pkgname = $pkgdir if (! defined $pkgname);

  @$tarref = ();

  if (!opendir DIR, "$root/$pkgdir") {
    error_exit("Can't open directory $root/$pkgdir");
  }
  @files = readdir(DIR);
  closedir DIR;

  $tarbase = $pkgname;
  $tarbase =~ s/\//_/g;
  $tarbase =~ s/_test$//;
  $tarbase =~ s/_beta$//;
  foreach $file (@files) {
    if ($file =~ /$tarbase[^.]*\.(?:zip|tar|md5)/) {
      push @$tarref, $file;
    }
  }
}

# rm -rf a tree (w/ error checking)
sub prunetree {
  my ($root) = @_;

  my (@dirstack, @allfiles, @alldirs, @files, $file, $dir);

  return if (! -e $root);

  push @dirstack, $root;
  push @alldirs, $root;

  # Read directory tree, and split into files and directories
  while(@dirstack) {
    $dir = pop(@dirstack);
    if (!opendir DIR, $dir) {
      error_exit("Can't open directory $dir");
    }
    @files = readdir(DIR);
    closedir(DIR);
    foreach $file (@files) {
      next if ($file eq ".");
      next if ($file eq "..");
      if (-d "$dir/$file") {
	push @dirstack, "$dir/$file";
	push @alldirs, "$dir/$file";
      } else {
	push @allfiles, "$dir/$file";
      }
    }
  }
  # Got whole tree, nuke the files
  foreach $file (@allfiles) {
    $! = 0;
    unlink $file;
    if ($!) {
      error_exit("Can't delete file $file");
    }
  }
  # Now nuke the directories, in reverse order
  while (@alldirs) {
    $dir = pop(@alldirs);
    $! = 0;
    rmdir $dir;
    if ($!) {
      error_exit("Can't delete directory $dir");
    }
  }
}

# untar a tar file
sub untar {
  my ($tar, $dest, $pkg) = @_;
  my ($ret, $cmd);

  if (! -e $dest) {
    $! = 0;
    mkdir ($dest);
    if ($!) {
      error_exit("Create output directory $dest failed");
    }
  }
  print "$0: Untar $tar to $dest\n";
  $cmd ="unzip $tar -d $dest";
  $ret = system($cmd);
  if ($ret) {
    error_exit("untar failed ($cmd)");
  }
  if (defined($pkg)) {
    touch_files_recursive ("$dest/$pkg");
  }
}

# compute md5 for all files in a tree
sub md5tree {
  my ($hashref, $root) = @_;
  my (@dirstack, $file, @files, $dir, $md5);

  $md5 = Digest::MD5->new();

  push @dirstack, ".";
  while(@dirstack) {
    $dir = pop(@dirstack); 
    if (!opendir DIR, "$root/$dir") {
      error_exit("Can't open directory $root/$dir");
    }
    @files = readdir(DIR);
    closedir(DIR);
    foreach $file (@files) {
      next if ($file eq ".");
      next if ($file eq "..");
      if (-d "$root/$dir/$file") {
        push @dirstack, "$dir/$file";
      } else {
        if (! open INPUT, "$root/$dir/$file") {
          error_exit("Can't open $root/$dir/$file to do md5");
	}
	binmode INPUT;
	$md5->addfile(*INPUT);
	close INPUT;
	$$hashref{"$dir/$file"} = $md5->hexdigest();
      }
    }
  }
}

# compare md5 of two trees and return 0 if they are same
sub md5compare {
  my ($hashref1, $hashref2) = @_;
  my (@keys1, @keys2, $differ, $key);

  @keys1 = keys(%$hashref1);
  @keys2 = keys(%$hashref2);

  $differ = 0;

  if ($#keys1 != $#keys2) {
    $differ = 1;
  } else {
    foreach $key (@keys1) {
      if (! defined($$hashref2{$key})) {
        $differ = 1;
	last;
      } elsif ($$hashref1{$key} ne $$hashref2{$key}) {
        $differ = 1;
	last;
      }
    }
  }
  return $differ;
}

# Read .md5 file.  Return 0 if not found
sub md5read {
  my ($hashref, $file) = @_;
  my ($path, $md5, $line);
  return 0 if (! -e $file);
  
  %$hashref = ();

  if (!open INPUT, $file) {
    error_exit("Can't open md5 file $file");
  }

  while(<INPUT>) {
    $line++;
    chomp;
    ($path, $md5) = split(/;/);
    if (!defined($path) || !defined($md5)) {
      error_exit("Can't split line $line of md5 file $file");
    }
    $$hashref{$path} = $md5;
  }
  return 1;
}

# Write .md5 file
sub md5write {
  my ($hashref, $md5file) = @_;
  my (@files, $file);

  if (! open OUTPUT, ">$md5file") {
    error_exit("Can't open md5 $md5file for writing\n");
  }
  @files = keys (%$hashref);
  foreach $file (@files) {
    print OUTPUT "$file;$$hashref{$file}\n";
  }
  close OUTPUT;
}

# Exit on error, avoiding recursion, and cleaning up
sub error_exit {
  my ($msg) = @_;
  my ($delete);
  print STDERR "$msg\n";
  if (! $error_exit) {
    $error_exit = 1;
    foreach $delete (@cleanup_deletes) {
      if (-d $delete) {
	prunetree($delete);
      } else {
	$! = 0;
	unlink ($delete);
	if ($!) {
	  $error_exit = 2; # double error!
	}
      }
    }
  } else {
    # detect double error
    $error_exit = 2;
    return;
  }
  if ($error_exit == 2) {
    print STDERR "SERIOUS ERROR: Encounter an error while cleaning up after error\n";
    print STDERR "Suggest manually cleaning $rtscdir completely!\n";
  } else {
    print STDERR "ERROR EXIT; cleanup succeeded\n";
  }
  exit(1);
}

sub run_rtsc {
  my ($pkgroot, $cmp, $cmd) = @_;
  my ($retval, $cygwinxdc, $swtools, $newcmpdir);
  # Create batch file to execute cygwinxdc.bat and then run xdc
  if(! open OUTPUT, ">runxdc.bat") {
    error_exit("Can't write mkrel/CPUDIR/runxdc.bat");
  }
  $cygwinxdc = $cmp."/cygwinxdc.bat";
  print OUTPUT "\@echo off\n";
  if(-e $cygwinxdc) {
    print OUTPUT "cd $cmp\n";
    print OUTPUT "call cygwinxdc $bypass_option\n";
    print OUTPUT "$cmd\n";
  } else {
    $swtools = $pkgroot."/ti/mas/swtools";
    $cygwinxdc = $swtools."/cygwinxdc.bat";
    if(0 && -e $cygwinxdc) {
      print OUTPUT "cd $swtools\n";
      print OUTPUT "call cygwinxdc $bypass_option\n";
      # Add the override commands that run after the config script
      if ($ENV{"SWTOOLS_OVERRIDES"}) {
        my @overrides = split /\|/, $ENV{"SWTOOLS_OVERRIDES"};
        my $override;
        print OUTPUT "echo ************* BEGIN ENVIRONMENT OVERRIDES **************\n";
        foreach $override (@overrides) {
          chomp $override;
          # Clean up the leading/trailing space and flip DOS slashes
          $override =~ s/\\/\//g;
          $override =~ s/^\s*"?//;
          $override =~ s/\"?\s*$//;
          # If it has an = then assume it is an env var
          if ((! ($override =~ /^set/)) && ($override =~ /\=/)) {
            $override = "set " . $override;
          }
          print OUTPUT "$override\n";
          print OUTPUT "echo $override\n";
        }
        print OUTPUT "echo ************** END ENVIRONMENT OVERRIDES ***************\n";
      }
      $newcmpdir = "../../../$cmp";
      print OUTPUT "cd $newcmpdir\n";
      print OUTPUT "$cmd\n";
    } else {
      print STDERR "Warning: No prep script for $pkgroot/$cmp\n";
      print OUTPUT "cd $pkgroot/$cmp\n";
      print OUTPUT "$cmd\n";
    }
  }
  close OUTPUT;
  print "$0: $cmp> $cmd\n";
  $retval = system("runxdc.bat");
  $! = 0;
  unlink "runxdc.bat";
  if ($!) {
    error_exit("Can't delete mkrel/CPUDIR/runxdc.bat");
  }
  return $retval;
}

# Simulate unix "touch" on all files under the specified directory
sub touch_files_recursive 
{
  my ($dir) = @_;

  print "Recursive touch of $dir\n";
  my (@dirStack) = ($dir);

  while (@dirStack) {
    my $path = pop @dirStack;

    opendir DIR, $path;
    my @files = readdir DIR;
    closedir DIR;
    my $file;
    foreach $file (@files) {
      next if ($file eq ".");
      next if ($file eq "..");
      my $filespec = "$path/$file";
      if (-d $filespec) {
        push @dirStack, $filespec;
      } else {
	my $time = time;
	my $curPerm = (stat $filespec)[2];
	chmod $curPerm | 0222, $filespec or die "$filespec: chmod r/w failed\n";
	utime $time, $time, $filespec or die "$filespec: touch failed\n"; 
	chmod $curPerm, $filespec or die "$filespec: chmod restore failed\n";
      }
    }
  }
}
