#!/usr/bin/perl

#*******************************************************************************
#* This program scans make command line for validity using the assumption
#* that all arguments placed on make command line are now environment variables.
#*
#* The command line of this program consists of 3 lists.
#* The first list, from the beginning of the command line up to the word 
#* "choices" is a set of all possible valid options and allowed values.
#* The options between "choices" and "invariants" list the options, described
#* in first list, that are allowed with restrictions.  A colon separated list 
#* restricts the values to a subset.  
#* The final list, after "invariants" is the list of values that cannot
#* be selected for the build.  The argument after = must be in the valid 
#* option list (list 1), and the variable may not be defined in the
#* environment.  
#*
#* All items in the first list must be present in the environment (possibly
#* restricted by the choices list) unless they are listed in the invariants list.
#* 
#* If the command line is valid, returns exit status of 0, otherwise nonzero.
#*
#* (C) Copyright 2001 Texas Instruments Incorporated.
# * 
# *  Redistribution and use in source and binary forms, with or without 
# *  modification, are permitted provided that the following conditions 
# *  are met:
# *
# *    Redistributions of source code must retain the above copyright 
# *    notice, this list of conditions and the following disclaimer.
# *
# *    Redistributions in binary form must reproduce the above copyright
# *    notice, this list of conditions and the following disclaimer in the 
# *    documentation and/or other materials provided with the   
# *    distribution.
# *
# *    Neither the name of Texas Instruments Incorporated nor the names of
# *    its contributors may be used to endorse or promote products derived
# *    from this software without specific prior written permission.
# *
# *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
# *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
# *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
# *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
# *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
# *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
# *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
# *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
# *
# */


# Configure perl to be helpful
#use diagnostics;
#use strict {vars};

#Split command line into 3 lists described above
@options=();  # All available options
@choices=();  # Options with restricted choices
@invars =();  # Options with no choices (can't be in environment)
@listref = (\@options, \@choices, \@invars);
$listid = 0;

$target = $ARGV[0];

for ($i=1;$i<=$#ARGV;$i++) {
  if ($ARGV[$i] eq "choices") {
     $listid = 1; 
  } elsif ($ARGV[$i] eq "invariants") {
     $listid = 2;
  } else {
     push @{$listref[$listid]}, $ARGV[$i];
  }
}

# If development defaults is used, remove the DEV=YES option from environment,
# and from variable MAKEFLAGS to prevent infinite recursion
#
$dev_defaults = 0;
if (($ENV{"DEV"} eq "YES") || ($ENV{"DEV"} eq "yes")) {
  $dev_defaults = 1;
  delete $ENV{"DEV"};
  $ENV{"MAKEFLAGS"} =~ s/DEV=$ENV{"DEV"}//;
  print "Development defaults used (first valid option is used for each missing option)\n";
}

#Count errors found
$errors = 0;
$defaults = 0;
$recursecmd = "make $target";

# Each option in @invars cannot exist in the environment; however these
# options must exist in @options.  All @invars options are struck from
# @options because they are settled.

for $opt (@invars) {
   if ($opt =~ /:/) {
      $errors++; 
      print sprintf ("%-20s [makefile error] invariant cannot have choices\n", 
                     $opt);
   } else {
      @validate = split /=/, $opt;
      if ($#validate != 1) {
        $errors++; 
        print sprintf ("%-20s [makefile error] invariant must assign value\n", 
                       $opt);
      } else {
        ($validvals, $pass) = checkopt ($validate[0], $validate[1]);
        if (!$pass) {
           $errors++;   
           print sprintf ("%-20s [makefile error] valid values: $validvals\n", 
                          "$opt" );
        } elsif ($envarg = $ENV{$validate[0]}) {
           $errors++; 
           print sprintf ("%-20s [command error] invariant cannot be set on CLI\n", 
                          "$validate[0]=$envarg" );
        }
      }
   }
}

# For each item in choice list, enforce/validate subset
for $opt (@choices) {
  @subset = split /[=:]/, $opt;
  $found = 0;
  
  # Find related option 
  for ($i=0;$i<=$#options;$i++) {
     @fullset = split /[=:]/, $options[$i];
     if ($fullset[0] eq $subset[0]) {
        $found=1;
        last;
     }
  }
  
  if (!$found) {
     $errors++;
     print sprintf ("%-20s [makefile error] choice restriction not found\n", 
                    $opt);
  } else {
     for ($j=1;$j<=$#subset;$j++) {
        $found=0;
        for ($k=1;$k<=$#fullset; $k++) {
           if ($subset[$j] eq $fullset[$k]) {
              $found = 1; 
           }
        }
        if (!$found) {
           $errors++;
           print sprintf ("%-20s [makefile error] choice restriction not subset\n", 
                          "$subset[0]=$subset[$j]");
        
           last;
        }
     }
     if ($found) {
       $options[$i] = $opt;
     } else {
       $options[$i] = "";
     }
  }
}

# For each option provided, validate against the environment
for $arg (@options)
{
   next if ($arg eq "");
   
   # Split the option specification into parameters
   # $validate[0] = variable name
   # $validate[1..n] = allowed values
   @validate = split /[=:]/, $arg;
   
   # Load requested variable from environment
   $envarg = $ENV{$validate[0]};
   
   ($validvals, $pass) = checkopt ($validate[0], $envarg);
   
   if (! $pass) {
     if ($dev_defaults && (! defined $envarg)) {
       $validvals =~ /^\{([^,]*),/;
       $recursecmd .= " $validate[0]=$1";
       $defaults++;
     } else {
       $errors++;
       print sprintf ("%-20s [command error] valid values: $validvals\n", 
                      "$validate[0]=$envarg");
     }
   } 
}

if ($errors) {
   print "\nErrors detected; Please correct them.\n";
   exit($errors);
}
if ($defaults) {
   print "Recursing with defaults (options on original command line are inherited): \n";
   print "$recursecmd\n";
   $ret = system($recursecmd);
   if ($ret) {
     print "Recursive build failed.\n";
   } else {
     print "**************************************************************\n";
     print "******** Ignore following error; recursion succeeded *********\n";
     print "**************************************************************\n";
   }
   exit(1);
}
exit(0);

sub checkopt {
  my ($opt, $val) = @_;
  my ($i, $arg, $comma, $validvals, $pass, @validate); 
  
  $pass = 0;
  # Find $opt in @options
  for ($i=0;$i<=$#options;$i++) {
     @validate = split /[=:]/, $options[$i];
     if ($validate[0] eq $opt) {
        # Remove the option from option list 
        $options[$i]="";
  
        # Find #val in #validate
        for ($i=1; $i<=$#validate; $i++) {
           if ($validate[$i] eq "#") {
             $pass = 1 if ($val =~ /^\d+$/);
           } else {
             $pass = 1 if ($validate[$i] eq $val); 
           }
        } 
  
        # Construct list of valid responses for error messages
        $validvals = "{";
        $comma = "";
        for ($i=1; $i<=$#validate;$i++) {
          if ($validate[$i] eq "") {
            $validvals.="$comma<blank>";    
          } else {
            $validvals.="$comma$validate[$i]";
          }
          $comma = ",";
        }
        $validvals .= "}";
        last;
     }
  }
  return ($validvals, $pass);
}
