/******************************************************************************
 * FILE PURPOSE: Custom ECO (Overlay Implementation) for plugging into VIGDK FW
 ******************************************************************************
 * FILE NAME:   ovly.c
 *
 * DESCRIPTION: 
 *
 * TABS: NONE
 *
 *
 * (C) Copyright 2008, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include "customEcoIface.h"
#include "customEcoloc.h"
#include "vctypes.h"

/* mips profiling */
#include "emureg.h"

/* number of buffers required by CUSTOM_ECO */
#define CUSTOM_ECO_NBUFS                1
#define CUSTOM_ECO_INSTANCE_BUFN        0

/* buffers used by CUSTOM_ECO */
memBuffer_t customEcoBufs[CUSTOM_ECO_NBUFS] = {
  MEM_CLASS_EXTERNAL, 0, 0, FALSE, NULL,  /* instance structure */
};

/*********************************************************************************
 * FUNCTION PURPOSE: Exception handle
 *********************************************************************************
  DESCRIPTION:      This function is called upon encountering a FATAL exception. This
                    calls the registered System Exception with this ECO's ID and 
                    exception code.
  Parameters :      Inputs: ID  : Module ID of the ECO
                            code: Exception code
                    Outputs: None
 *********************************************************************************/
void customEco_exception(tuint ID, tuint code)
{
   customEcoContext.debugInfo (ID, DEBUG_EXCEPTION_FATAL, code, 0, NULL);
}

/*********************************************************************************
 * FUNCTION PURPOSE: GetSizes
 *********************************************************************************
  DESCRIPTION:      This function is called by the system to enquire the resources
                    that are required before calling New. Module communicates the 
                    resource requirements to the system.
  Parameters :      Inputs: nbufs: Number of Buffers needed by the ECO
                            bufs : Description about the buffer including Memory class
                                   (For example: external, internal), Alignment, 
                                   Size,Volatile (For example: persistent, scratch)
                            cfg  : Any configuration needed by the ECO, incase the 
                                   buffer requirements change per enabled/disabled 
                                   feature.
                    Outputs: Success/Failure
 *********************************************************************************/
tint customEcoGetSizes (tint *nbufs, const memBuffer_t **bufs, customEcoSizeCfg_t *cfg) {
  /* Calculate the buffer sizes and report their descriptions */
  *nbufs = CUSTOM_ECO_NBUFS;             /* Report the number of buffers */

  customEcoBufs[CUSTOM_ECO_INSTANCE_BUFN].size        = sizeof(customEcoInst_t);
  
  *bufs = customEcoBufs;
  return (CUSTOM_ECO_NOERR);
}

/*********************************************************************************
 * FUNCTION PURPOSE: New
 *********************************************************************************
  DESCRIPTION:      This function is called by the system to new the ECO after 
                    allocating the resources requested by the ECO
  Parameters :      Inputs: customEcoInst : Reference to the ecoInst's pointer
                            nbufs         : Number of Buffers requested by the ECO
                            bufs          : Buffer descriptors w/ base address filled
                                            by the System
                            cfg           : Configuration data for the module if any
                    Outputs: Success/Failure
 *********************************************************************************/
tint customEcoNew (void **customEcoInst, tint nbufs, memBuffer_t *bufs, customEcoNewCfg_t *cfg) {
  tint i;
  customEcoInst_t *inst;
  memBuffer_t *bufp;
  tint retval;

  /* Test instance pointer (Must be NULL)  */
  if (*customEcoInst != NULL || nbufs != CUSTOM_ECO_NBUFS)
    return(CUSTOM_ECO_ERROR);

  /* Allocate all dynamic buffers (base address != NULL ?)   */
  retval = CUSTOM_ECO_NOERR;
  for (i = 0, bufp = &bufs[0]; i < nbufs; i++, bufp++) {
    if ( (bufp->size != 0) && (bufp->base == NULL) )
      retval = CUSTOM_ECO_NOMEMORY;
  }

  if (retval != CUSTOM_ECO_NOERR) 
     return retval;
      
  /*  Identify the instance structure  */
  bufp = &bufs[CUSTOM_ECO_INSTANCE_BUFN];
  if (bufp->size < sizeof(customEcoInst_t))
    return(CUSTOM_ECO_ERROR);
  inst = (customEcoInst_t *) bufp->base;

  /* Save configuration data  */
  inst->ID         = cfg->ID;
  inst->state      = CUSTOM_ECO_STATE_CLOSED;  /* set VPPU state to CLOSED  */
  *customEcoInst = (void *)inst;   /* return instance pointer  */
  return(CUSTOM_ECO_NOERR);

}

/*********************************************************************************
 * FUNCTION PURPOSE: Open
 *********************************************************************************
  DESCRIPTION:      This function is called by the system to new the ECO after 
                    the module is New-ed.
  Parameters :      Inputs: customEcoInst : ecoInst's pointer
                            cfg           : Configuration data for the module/channel
                    Outputs: Success/Failure
 *********************************************************************************/
tint customEcoOpen (void *customEcoInst, customEcoCfg_t *cfg)
{
  customEcoInst_t *inst = (customEcoInst_t *)customEcoInst;

  customEco_exc_assert((inst->state == CUSTOM_ECO_STATE_CLOSED), CUSTOM_ECO_EXC_OPEN, inst);
  customEco_exc_assert((cfg->img_width  != 0),CUSTOM_ECO_EXC_INVD_IMG_WIDTH, inst);
  customEco_exc_assert((cfg->img_height != 0),CUSTOM_ECO_EXC_INVD_IMG_HEIGHT, inst);

  /* Store configuration data */
  inst->cfg = *cfg;

  /* reset tx and rx stats */
  inst->state = CUSTOM_ECO_STATE_OPEN;
  
  return(CUSTOM_ECO_NOERR);
}

/*********************************************************************************
 * FUNCTION PURPOSE: Control
 *********************************************************************************
  DESCRIPTION:      This function is called by the system to control
  Parameters :      Inputs: customEcoInst : ecoInst's pointer
                            cfg           : Configuration data for the module/channel
                            ctrl          : Control message
                    Outputs: Success/Failure
 *********************************************************************************/
tint customEcoControl (void *customEcoInst, customEcoCfg_t *cfg, customEcoCtrl_t *ctrl) {
    customEcoInst_t *inst = (customEcoInst_t *)customEcoInst;
    if (ctrl->control == CUSTOM_ECO_SET)
    {
      inst->cfg.action_type = cfg->action_type;
    }
    return (CUSTOM_ECO_NOERR);
}

/*********************************************************************************
 * FUNCTION PURPOSE: Do Overlay
 *********************************************************************************
  DESCRIPTION:      
  Parameters :      Inputs: customEcoInst   : Pointer 
                            inputData       : pointer to the encoded frame
                            inLength        : length of valid data on the buffer
                            outputData      : YUV buffer output
                            outLength       : size of YUV data generated
                    Output: return value    : (success/failure) 
 *********************************************************************************/
static const char *customEco_ovlyMpeg4_str = "MPEG4SP";
static const char *customEco_ovlyH263_str  = "H.263";
static const char *customEco_ovlyH264_str  = "H.264BP";
static word  customEco_YUV[3] = { 82, 90, 240 };
#define YUVLogoShift 4

tint customEcoProcess  (void *customEcoInst, word *inputData)
{
    customEcoInst_t *inst = (customEcoInst_t *)customEcoInst;
    word  *yuv_planes = inputData;
    tuint action_type;
    tuint inHeight = (tuint)inst->cfg.img_height;
    tuint inWidth  = (tuint)inst->cfg.img_width;
    tuint retCode =  CUSTOM_ECO_NOERR;

#ifdef EMU_PROFILE
  tuint chnum = ((inst->ID & 0xFF) - 1);
  EMU_PROFILE_BEGIN(EMU_TASK_ID_CUSTOM_ECO,chnum)
#endif
   if ((inHeight == 0 || inWidth == 0) || (inst->state != CUSTOM_ECO_STATE_OPEN))
   {
      retCode = CUSTOM_ECO_SKIP_PROCESS;
      goto end;
   }
  action_type = inst->cfg.action_type;

  if(action_type & CUSTOM_ECO_DIAG) 
  { /* Toy algorithm: draw two diagonals, Y plane affected */
    tuint ii;
    for (ii = 0; ii < inHeight; ii ++) {
       yuv_planes[ii + ii * inWidth]            = 255;
       yuv_planes[inHeight - ii + ii * inWidth] = 255;
    }
  } 
  if(action_type & CUSTOM_ECO_LOGO) {
    customEcoContext.ovly_logo (yuv_planes, inWidth, inHeight, inWidth - 40, YUVLogoShift, 
               customEco_YUV[0], customEco_YUV[1], customEco_YUV[2]);
  }
  if(action_type & CUSTOM_ECO_FONT) {
    customEcoContext.ovly_render_simplefont ((word *)yuv_planes, inWidth, inHeight, 8, inHeight - 2 * 16, (word *)"Texas", 255, 255, 255);
    customEcoContext.ovly_render_simplefont ((word *)yuv_planes, inWidth, inHeight, 8, inHeight - 1 * 16, (word *)"Instruments", 255, 255, 255);
  }
  if(action_type & CUSTOM_ECO_CHID) { 
    word comp_str[32], *dst_str;
    
    sprintf ((char *)comp_str, "CORE:%d CHID:%d", inst->cfg.coreId, (inst->ID & 0x00FF) - 1);
    customEcoContext.ovly_render_simplefont (yuv_planes, inWidth, inHeight, 8, 16,  comp_str, 255, 255, 255);
    
    if(inst->cfg.codingType == EVCT_VIDEO_MPEG4SP_ENC) {
      dst_str =  (word *)customEco_ovlyMpeg4_str;
    } else if(inst->cfg.codingType == EVCT_VIDEO_H263P_ENC) {
      dst_str =  (word *)customEco_ovlyH263_str;
    } else dst_str =  (word *)customEco_ovlyH264_str;   
    
    sprintf ((char *)comp_str, "%s", dst_str);
    customEcoContext.ovly_render_simplefont (yuv_planes, inWidth, inHeight, 8, 32, comp_str,  255, 255, 255);
  }

end:
#ifdef EMU_PROFILE
    EMU_PROFILE_END()
#endif

    return (retCode);
}

/******************************************************************************
 * FUNCTION PURPOSE:  Close a CUSTOM ECO instance 
 ******************************************************************************
 * DESCRIPTION: Closes a CUSTOM ECO instance.  Instance must be "allocated" by
 *              customEcoNew() prior to vppuClose.  
 *
 *  void customEcoClose(
 *       void     *customEcoInst   -  A pointer to CUSTOM ECO instance
 *
 *****************************************************************************/
void customEcoClose (void *customEcoInst)
{
  customEcoInst_t *inst = (customEcoInst_t *) customEcoInst;
  inst->state = CUSTOM_ECO_STATE_CLOSED;   /* declare a channel as closed  */

} /* customEcoClose */

/******************************************************************************
 * FUNCTION PURPOSE: Delete an instance of CUSTOM ECO.
 ******************************************************************************
 * DESCRIPTION: Deletes an instance of CUSTOM ECO.  The CUSTOM ECO must be in 
 *              closed state.
 *
 * void customEcoDelete (
 *    void          **customEcoInst)  - an address of memory location that contains
 *                                a pointer to instance structure
 *
 *****************************************************************************/
void customEcoDelete (void **customEcoInst)
{
   customEcoInst_t *inst = (customEcoInst_t *) *customEcoInst;
   /* Ensure it is already closed */
   customEco_exc_assert (inst->state == CUSTOM_ECO_STATE_CLOSED, CUSTOM_ECO_EXC_DELETE, inst);

   *customEcoInst = NULL;   /* Mark the instance as free */
}

/* nothing past this point */
