/******************************************************************************
 * FILE PURPOSE: Custom ECO (Overlay Interface) for plugging into VIGDK FW
 ******************************************************************************
 * FILE NAME:   customEcoIface.h
 *
 * DESCRIPTION: 
 *
 * TABS: NONE
 *
 *
 * (C) Copyright 2008, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include "types.h"
#include "utldbg.h"
#include "memmgm.h"
#include "util.h"  /* bytes to words conversion macros */
#include "utlbitfield.h"

/* CUSTOM ECO error codes */
enum {
  CUSTOM_ECO_NOERR        = 0,
  CUSTOM_ECO_ERROR        = 1,
  CUSTOM_ECO_NOMEMORY     = 2,
  CUSTOM_ECO_SKIP_PROCESS = 3
};

/* Exception Messages */
enum {
   CUSTOM_ECO_EXC_OPEN                 = 1, /* Opened an already open channel */
   CUSTOM_ECO_EXC_DELETE               = 2, /* Deleted an open instance */
   CUSTOM_ECO_EXC_CONTROL              = 3, /* Control request received by a closed channel */
   CUSTOM_ECO_EXC_INVD_IMG_HEIGHT      = 4, /* Invalid Image Height */
   CUSTOM_ECO_EXC_INVD_IMG_WIDTH       = 5, /* Invalid Image Width  */
   CUSTOM_ECO_EXC_RESOLUTION_MISMATCH  = 6, /* Resolution mismatch between decoder & overlay */
   CUSTOM_ECO_EXC_ACTION               = 7  /* Process request received by a closed channel  */
};

/* Configuration structure */
typedef struct {
  tuint coreId;
  tuint codingType;
  tuint img_width;
  tuint img_height;
/* Bits indicating requested action */
#define CUSTOM_ECO_FONT     0x0001
#define CUSTOM_ECO_LOGO     0x0002
#define CUSTOM_ECO_DIAG     0x0004
#define CUSTOM_ECO_CHID     0x0008
#define CUSTOM_ECO_INVERT_Y 0x0010 
  tuint action_type;
} customEcoCfg_t;

/* Control code for Messages */
typedef enum {
  CUSTOM_ECO_SET            = 0x00 /* Set reconfigurable parameters */
} customEcoCtrlCode_t;

/* Control for Messages */
typedef struct {
  customEcoCtrlCode_t control;
} customEcoCtrl_t;

/* Structure to convey any special size requirements */
typedef struct {
  tint dummy;
} customEcoSizeCfg_t;

/******************************************************************************
 * DATA DEFINITION:  New parameters
 *****************************************************************************/
typedef struct {
  tuint ID;                      /* Channel instance ID */
  customEcoSizeCfg_t   sizeCfg;  /* Size configuration structure */ 
} customEcoNewCfg_t;

/******************************************************************************
 * DATA DEFINITION:  System Context for the ECO
 *                   Any virtual interface has to be defined here
 *****************************************************************************/
typedef struct {
  debugInfo_t debugInfo; /* Debug/exception function */
  void (*ovly_bmp) (word *yuv_planes, tuint Width, tuint Height, tuint pos_x, tuint pos_y, 
                    word *bmp_arr, tuint bmp_arr_w, tuint bmp_arr_h, word luma, word chroma_r, word chroma_b);
  void (*ovly_logo) (word *yuv_planes, tuint Width, tuint Height, tuint pos_x, tuint pos_y,
                     word luma, word chroma_r, word chroma_b);
  void (*ovly_render_simplefont) (word *yuv_planes, tuint w, tuint h, tuint pos_x, tuint pos_y, word *str_ovly, 
                                  word luma, word chroma_r, word chroma_b);
  void (*ovly_fixed_logo) (word *yuv_planes, tuint Width, tuint Height, tuint pos_x, tuint pos_y);                            
} customEcoContext_t;   

/* External references to the custom ECO context */
extern customEcoContext_t customEcoContext;

/******************************************************************************
 * APIs implemented by the custom ECO
 *****************************************************************************/
void customEcoClose     (void *customEcoInst);
tint customEcoControl   (void *customEcoInst, customEcoCfg_t *cfg, customEcoCtrl_t *ctrl);
void customEcoDelete    (void **customEcoInst);
tint customEcoGetSizes  (tint *nbufs, const memBuffer_t **bufs, customEcoSizeCfg_t *cfg);
tint customEcoNew       (void **customEcoInst, tint nbufs, memBuffer_t *bufs, customEcoNewCfg_t *cfg);
tint customEcoOpen      (void *customEcoInst, customEcoCfg_t *cfg);
tint customEcoProcess   (void *customEcoInst, word *inputData);

/******************************************************************************
 * custom ECO Call Table
 *****************************************************************************/
typedef struct {
  void (*customEcoClose)     (void *customEcoInst);
  tint (*customEcoControl)   (void *customEcoInst, customEcoCfg_t *cfg, customEcoCtrl_t *ctrl);
  void (*customEcoDelete)    (void **customEcoInst);
  tint (*customEcoGetSizes)  (tint *nbufs, const memBuffer_t **bufs, customEcoSizeCfg_t *cfg);
  tint (*customEcoNew)       (void **customEcoInst, tint nbufs, memBuffer_t *bufs, customEcoNewCfg_t *cfg);
  tint (*customEcoOpen)      (void *customEcoInst, customEcoCfg_t *cfg);
  tint (*customEcoProcess)   (void *customEcoInst, word *inputData);
} customEcoCallTable_t;

/* nothing past this point */
