#ifndef _DIAGCFG_H
#define _DIAGCFG_H
/*****************************************************************************
 * FILE PURPOSE: Diagnostic build time configuration structure
 *****************************************************************************
 * FILE NAME: diagcfg.h
 *
 * FILE PURPOSE: Defines a structure which reflects build time parameters
 *
 * TABS: None
 *
 * (C) Copyright 2002, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include "types.h"
#include "dsiuloc.h"
#include "dglob.h"
#include "dpiu.h"

#ifdef ITDM
#include "ipiu.h"
#include "ippu.h"
#endif

#include "dvpu.h"
#include "bert.h"
#include "dtgu.h"
#include "dpvp.h"
#include "dsiuubt.h"
#include "siundu.h"
#include <ti/mas/neu/neu.h>
#include "dsiuctbl.h"
#include "hal.h"

#include "dsiuarun.h"

/*****************************************************************************
 * Definition: System checksum block attributes
 *             /----------------------------------------------------------\
 *             | 15       -         8  |   7    |  6       -           0  |
 *             |     Reserved          | Shared |     Core Number         |
 *             \----------------------------------------------------------/
 *****************************************************************************/      
#define DCFG_SYS_CHKSUM_FLAG_SHARED          (1<<7)
#define DCFG_SYS_CHKSUM_FLAG_CORE(x)         ((x)&0x7f)
#define DCFG_SYS_CHKSUM_ATTR_IS_SHARED(x)    ((x)&(1<<7))
#define DCFG_SYS_CHKSUM_ATTR_GET_CORE(x)     ((x)&0x7f)

/******************************************************************************
 * Definition: external variables defined by the linker to allow copy from
 *             load address to run address.
 ******************************************************************************/
extern tulong boverlay_load_start;
extern tulong boverlay_load_size;
extern tulong boverlay_run_start;


/*******************************************************************************
 * Definition: The location of the configuration parameter block
 *******************************************************************************/
extern diagApp_t diagAppMem;

/* NEU is a full application module, and the call tables are handled differently.
 * The full application module does not declare the call table in the module
 * code itself. The extern will be declared here, the call table is created in
 * dsiuneu.c */
#ifdef DIAG_INCLUDE_NEU
extern neuCallTable_t neuCallTable;
#endif

#ifdef DIAG_INCLUDE_BERT
extern gg_bert_ctable_t bertCallTable;
#endif

#ifdef ITDM
extern ippuCallTable_t ippuCallTable;
#endif

/******************************************************************************
 * DEFINITION: Module call tables. Every modules API is contained in a call
 *             table defined by the module. A pointer to every modules call
 *             table is available to the system through the module call table.
 *****************************************************************************/
typedef struct {
  dpiuCallTable_t       *dpiu;        /* Call table for dpiu       */
#ifdef ITDM
  ipiuCallTable_t       *ipiu;        /* Call table for ipiu       */
  ippuCallTable_t       *ippu;        /* Call table for ippu       */
#endif
  dvpuCallTable_t       *dvpu;        /* Call table for dvpu       */
  dtguCallTable_t       *dtgu;        /* Call table for dtgu       */
  dpvpCallTable_t       *dpvp;        /* Call table for dpvp       */
  neuCallTable_t        *neu;         /* Call table for neu        */
  siuNduCallTable_t     *siuUtNdu;    /* Call table for utopia ndu */
  siuNduCallTable_t     *siuHpiNdu;   /* Call table for HPI ndu    */
  siuNduCallTable_t     *siuGmacNdu;  /* Call table for GMAC ndu   */
  gg_bert_ctable_t      *bert;        /* Call table for bert       */
  dsiuCallTable_t       *dsiu;        /* Call table for siu        */
  dsiuAutoRun_t         *dsiuArun;    /* Call table for automated run */
} diagCallTable_t;


/******************************************************************************
 * DEFINITION: Memory block addresses. The block is tested from startAddr
 *             to endAddr-1. If both addresses are equal, the block is not
 *             tested. Block start and end addresses are defined in the linker
 *             command file.
 ******************************************************************************/
extern tulong _mem_test_primary_base;  /* Relocatable memory test addresses */
extern tulong _mem_test_primary_end;

typedef struct {
  word   *startAddr;     /* 32 bit memory address */
  tulong  bufLenWords;
  tuint   flags;
} dcfgMemBlock_t;

#if (defined (c55mc))
typedef struct {
  tulong mcStartAddr;     /* 32 bit memory address */
  tulong mcEndAddr;       /* 32 bit memory address */
} dcfgMcMemBlock_t;
#endif
extern dcfgMemBlock_t dcfgMemBlock[];

/* Read the flags */
#define DCFG_MEMTEST_ATTR_IS_DATA(x)      UTL_GET_BITFIELD((x),0,1)
#define DCFG_MEMTEST_ATTR_IS_PROG(x)      UTL_GET_BITFIELD((x),1,1)
#define DCFG_MEMTEST_ATTR_IS_RXNET(x)     UTL_GET_BITFIELD((x),2,1)
#define DCFG_MEMTEST_ATTR_IS_L1P(x)       UTL_GET_BITFIELD((x),3,1)
#define DCFG_MEMTEST_ATTR_IS_L1D(x)       UTL_GET_BITFIELD((x),4,1)
#define DCFG_MEMTEST_ATTR_IS_L2(x)        UTL_GET_BITFIELD((x),5,1)
#define DCFG_MEMTEST_ATTR_IS_LOCAL(x)     UTL_GET_BITFIELD((x),6,1)
#define DCFG_MEMTEST_ATTR_IS_DSCRATCH(x)  UTL_GET_BITFIELD((x),7,1)
#define DCFG_MEMTEST_ATTR_IS_SSCRATCH(x)  UTL_GET_BITFIELD((x),8,1)
#define DCFG_MEMTEST_ATTR_IS_PSCRATCH(x)  UTL_GET_BITFIELD((x),9,1)

/* Write the flags */
#define DCFG_MEMTEST_ATTR_DATA(x,v)       UTL_SET_BITFIELD((x),(v),0,1)
#define DCFG_MEMTEST_ATTR_PROG(x,v)       UTL_SET_BITFIELD((x),(v),1,1)
#define DCFG_MEMTEST_ATTR_RXNET(x,v)      UTL_SET_BITFIELD((x),(v),2,1)
#define DCFG_MEMTEST_ATTR_L1P(x,v)        UTL_SET_BITFIELD((x),(v),3,1)
#define DCFG_MEMTEST_ATTR_L1D(x,v)        UTL_SET_BITFIELD((x),(v),4,1)
#define DCFG_MEMTEST_ATTR_L2(x,v)         UTL_SET_BITFIELD((x),(v),5,1)
#define DCFG_MEMTEST_ATTR_LOCAL(x,v)      UTL_SET_BITFIELD((x),(v),6,1)
#define DCFG_MEMTEST_ATTR_DSCRATCH(x,v)   UTL_SET_BITFIELD((x),(v),7,1)
#define DCFG_MEMTEST_ATTR_SSCRATCH(x,v)   UTL_SET_BITFIELD((x),(v),8,1)
#define DCFG_MEMTEST_ATTR_PSCRATCH(x,v)   UTL_SET_BITFIELD((x),(v),9,1)

/* Compile time values used in diagbld.h */
#define MEMTEST_ATTR_DATA     (1 << 0)
#define MEMTEST_ATTR_PROG     (1 << 1)
#define MEMTEST_ATTR_RXNET    (1 << 2)
#define MEMTEST_ATTR_L1P      (1 << 3)
#define MEMTEST_ATTR_L1D      (1 << 4)
#define MEMTEST_ATTR_L2       (1 << 5)
#define MEMTEST_ATTR_LOCAL    (1 << 6)
#define MEMTEST_ATTR_DSCRATCH (1 << 7)
#define MEMTEST_ATTR_SSCRATCH (1 << 8)
#define MEMTEST_ATTR_PSCRATCH (1 << 9)


/*****************************************************************************
 * DEFINITION: DMA buffer memory for ports. An array of these is used
 *             for a variable number of ports.
 *****************************************************************************/
typedef struct dcfgDma_s
{
  tuint *txStart;  /* DMA start buffer location tx */
  tuint *txEnd;    /* DMA end buffer location tx   */
  tuint *rxStart;  /* DMA start buffer location rx */
  tuint *rxEnd;    /* DMA end buffer location rx   */
} dcfgDma_t;


/*****************************************************************************
 * DEFINITION: A generic data memory block descriptor
 *****************************************************************************/
typedef struct dcfgMemBlk_s
{
  tuint *start;
  tuint *end;
} dcfgMemBlk_t;

/*****************************************************************************
 * DEFINTIION: Utopia build time configuration. All elements are in arrays
 *             to allow different configurations for multiple ports. An
 *             array of structures wasn't used to make declaration of
 *             the structure easier in diagcfg.c
 *****************************************************************************/
typedef struct dcfgUtConfig_s
{
  /* Array of utopia dma memory sizes */ 
  dcfgDma_t *utopiaDmaMem; 
  
  /* Catch all buffer */
  tuint **caStart;
  tuint **caEnd;
  
  /* Port configuration */
  halUtopiaMode_t *routeMode;
 
} dcfgUtConfig_t;

/*****************************************************************************
 * DEFINITION: TDM build time configuration. All elements are in arrays
 *             to allow different configurations for multiple ports. 
 *****************************************************************************/ 
typedef struct dcfgTdmConfig_s
{
  /* Array of tdm dma memory sizes */
  dcfgDma_t *tdmDmaMem;
  
  /* Array of sub-frame sizes */
  tint *subFrameSize;
  
  /* Array of single sample word sizes (bits) */
  tint *wordSize;
  
  /* Array of buffer increments */
  tint *buffIncr;
  
  /* Max sub and mega phase values */
  tint *maxPhase;
  tint *maxMegaPhase;
  
} dcfgTdmConfig_t;

/*****************************************************************************
 * DEFINITION: iTDM build time configuration
 *****************************************************************************/
typedef struct dcfgiTdmConfig_s
{
  /* Array of itdm dma memory sizes */
  dcfgDma_t *tdmDmaMem;
  
} dcfgiTdmConfig_t;
/****************************************************************************
 * DEFINITION: Build time memory configuration.
 ****************************************************************************/
typedef struct dcfgMemCfg64x_s {

  tint l1p;
  tint l1d;
  tint l2;
  
} dcfgMemCfg64x_t;
 
typedef struct dcfgMemCfg_s  {

  union  {
  
    dcfgMemCfg64x_t mem64x;
    
  } u;
  
} dcfgMemCfg_t;
 

/*****************************************************************************
 * DEFINITION: The maximum number of memory blocks per type that can be
 *             specified.
 *****************************************************************************/
#define DCFG_MAX_MEM_BLOCKS    5

typedef struct {

 /* System checksum blocks */
 tint    sysChkNblocks;             /* The number of memory blocks      */
 tulong *sysChkBlockSizesWords;     /* The size of each block           */
 tint   *sysChkBlockFlags;          /* The attributes of each block     */
 word  **sysChkBlocks;              /* The memory block start addresses */

  /* External DDR data memory on Monticello */
#if (defined (c55mc))
  dcfgMcMemBlock_t externalProgBlocks[DCFG_MAX_MEM_BLOCKS];

  tulong burstLength;

  tulong cycleTestBlock;
#endif

/*****************************************************************************
 * Location of heap memory
 *****************************************************************************/
  word *bootHeapBufStart;      /* Boot heap memory start address  */
  word *bootHeapBufEnd;        /* Boot heap memory end address    */

  word *diagHeapBufStart;      /* Diag heap memory start address  */
  word *diagHeapBufEnd;        /* Diag heap memory end address    */

/*****************************************************************************
 * Location of data memory blocks reserved for HAL buffers
 *****************************************************************************/
  word *halHeapBufStart; /* HAL heap memory start address      */
  word *halHeapBufEnd;   /* HAL heap memory end address        */

/*****************************************************************************
 * Number of channels. Specifies the number of tdm and network channels.
 * There are two times as many routers.
 *****************************************************************************/
 tint maxChannels;

/*****************************************************************************
 * Number of ITDM Streams 
 * There are two times as many routers.
 *****************************************************************************/

 tint maxItdmStreams;

 tint maxItdm1milliSecStreams;

 tint maxItdm125microSecStreams;

 tint maxChanIDsPer1msecStream;

 tint maxChanIDsPer125microsecStream;

/*****************************************************************************
 * Peripheral usage
 *****************************************************************************/
  /* Number of TDM ports used at one time */
  tint nTdmPorts;
  
  /* Number of iTDM ports used at one time */
  tint nItdmPorts;
  
  /* Tdm build time configuration */
  dcfgTdmConfig_t *tdmConfig;
  
  /* iTdm build time configuration */
  dcfgiTdmConfig_t *itdmConfig;
  
  /* Operational block - only used for C54x McBSP */
  tint opBlock;
  
  /* Type of TDM port - only used for C54x eMcBsp/McBsp discrimination */
  tint portType;

  /* BDX delay - used for wire-ord TDM ports */
  tint bdxDelay;

  /* Buffering, in samples provided by DPIU to the application */
  tint tdmBufSize;

  /* Number of Utopia interfaces used at one time */
  tint nUtopiaPorts;
  
  /* Utopia configuration */
  dcfgUtConfig_t *utConfig;
  
  /* Gmac buffering information. Note that the buffer sizes here do NOT
   * refer to the entire buffer size, but to the size handled by each
   * buffer descriptor */
  tint nGmacPorts;              /* Number of GMAC ports to configure */
  
  tint *gmacBufsizeWordsTx;     /* Size array (by port number) of each transmit unit */
  tint *gmacBufsizeWordsRx;     /* Size array (by port number) of each receive unit  */
  tint *gmacRxAssemSizeWords;   /* Size of the rx assembly buffer                    */
  
  /* Array of mac dma memory provided */
  dcfgDma_t *gmacDmaMem;
  
  /* Array of assembly buffers */
  dcfgMemBlk_t *gmacRxAssemMem;
  
  /* Number of AIC ports used */
  tint nAicPorts;
  
  /* Memory configuration */
  dcfgMemCfg_t *memCfg;
  
/*****************************************************************************
 * Parameters used to measure the ratio between the system clock and the frame
 * sync clock 
 *****************************************************************************/
  tuint  reportDivisor;  /* The reported value is the number of cpu cycles
                         * per frame sync, divided by this value          */
  tulong timerPeriod;   /* Resolution, in CPU cycles, of the system clock */
  tulong timerTimeout;  /* Number of cycles in loop waiting for valid frame*/

/*****************************************************************************
 * TDM application frame size. Must be an integral number of the subFrameRates
 *****************************************************************************/
  tint apRate0;           /* Application frame rate (samples) 0 */
  tint apRate1;           /* Application frame rate (samples) 1 */

/*****************************************************************************
 * Network receive data fifo buffer size
 *****************************************************************************/
  tuint vpuBufSize;

/******************************************************************************
 * Message NEU and AAL5 buffers
 *****************************************************************************/
  tuint aal5AssemblySize;   /* Aal5 rx packet assembly buffer size */
  tuint neuSegSize;         /* NEU tx segmentation buffer size     */
  
/******************************************************************************
 * Voice NEU and AAL5 buffers
 ******************************************************************************/
  tuint voiceAssemblySize;  /* AAL5 rx packet assembly buffer size   */
  tuint voiceSegSize;       /* NEU voice tx segmentation buffer size */
  
/******************************************************************************
 * Message NEU UDP core adjustments mask and base port for voice
 ******************************************************************************/
  tuint neuUdpSrcMask;
  tuint neuUdpVoiceBase;
  
/******************************************************************************
 * Network receive fifos
 *****************************************************************************/
  tuint netFifoSize;

/******************************************************************************
 * System message task queue size and tx/rx buffers
 ******************************************************************************/
  tint  msgTaskSize;        /* Size of message task buffer */
  tuint msgRxBufferSize;    /* Rx Assembled message buffer */
  tuint msgTxBufferSize;    /* Tx Assembled message buffer */
  tuint msgRxFifoSize;      /* Rx Message Fifo size        */
  
/****************************************************************************
 * Message sequence messages, if supported
 ****************************************************************************/
  tint nBlockMessages;      /* Number of message blocks */
  bool blkMsgCacheFlush;    /* If true cache is flushed before reading control */

/*****************************************************************************
 * Boot Config and boot ack function pointers
 *****************************************************************************/
 tint (*bootConfig)(void);
 void (*bootAck)(tint);

/*****************************************************************************
 * Simulated application boot configuration function 
 *****************************************************************************/
  dsiuReturn_t (*bootSimuAppConfig)(void);

/*****************************************************************************
 * System Core Bootup Function Call - processor type dependent, and system
 * default entry point
 *****************************************************************************/
  dsiuCoreBootReturn_t (*systemBoot)(tint, tulong);
  tulong entry;
  
/*****************************************************************************
 * System Core Reset Function Call - process type dependent.
 *****************************************************************************/
  dsiuCoreResetReturn_t (*systemReset)(tint);
  
/*****************************************************************************
 * System function to handle periodic TDM callout
 *****************************************************************************/
 void (*dsiuSubRateFcn)(void *, tuint);
 
/*****************************************************************************
 * System functions for memory test (diagnostic) and checksum (bootloader)
 *****************************************************************************/
  tint numMemTestBlocks;
  void (*dsiuTestMemory) (dmsg_t *msg);
  tint (*dsiuChksum) (dmsg_hd_sys_mem_chksum_t *, dmsg_dh_sys_chksum_ret_t *);
 
/****************************************************************************
 * System function for ramset/cache testing
 ****************************************************************************/
  tint (*dsiuTestRamset) (void);
 
/*****************************************************************************
 * CPU specific startup code
 *****************************************************************************/
  void (*dsiuProcStartup) (void);

/****************************************************************************
 * Pointers to blocks of memory used for system port status registers 
 ****************************************************************************/
  dsiuPort_t *dcfgTdmPorts_p;
  dsiuPort_t *dcfgUtopiaPorts_p;
  dsiuPort_t *dcfgMacPorts_p;
  
/******************************************************************************
 * Synchronous start memory test message id. Configured through gghwcfg.h
 * in devices which use an IPC. Other devices configured through diagbld.h
 ******************************************************************************/
  tulong memTestMsg;
  
/******************************************************************************
 * Secondary core startup delay. Used in auto-run to stall secondary
 * cores while core 0 does shared peripheral global configuration
 ******************************************************************************/
  tulong secCoreStartDelay;

  
 /*****************************************************************************
  * Module function call tables
  *****************************************************************************/
 diagCallTable_t ctable;

} diagCfg_t;

extern diagCfg_t diagCfg;
/*****************************************************************************
 * External declaration. The neu port map table is defined in diagcfg.c. The
 * name must match that used by the full application.
 *****************************************************************************/
extern neuPortMap_t neuPortMap;

/*****************************************************************************
 * Function Prototype: The linker constants are not recognized by the compiler,
 * so this function is used to load them.
 *****************************************************************************/
void diagCfgInit (void);

#endif /* _DIAGCFG_H */

