/******************************************************************************
 * FILE PURPOSE: utility FIFO implementation
 ******************************************************************************
 * FILE NAME: utlfifo.c
 *
 * DESCRIPTION: Implements a basic FIFO
 *
 * TABS: None
 *
 * Copyright (C) 2002, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include "utlfifo.h"
#include "dmiscport.h"

/******************************************************************************
 * FUNCTION PURPOSE: Return the amount of space available in the fifo
 ******************************************************************************
 * DESCRIPTION: The amount of space available in the fifo is one element less
 *              the fifo capacity. The fifo is not allowed to "fill" since
 *              no mechanism is included to then distinguish between an empty
 *              fifo and a full fifo.
 ******************************************************************************/
tuint utlFifoSpaceAvail (utlFifo_t *fifo)
{
  tuint avail;

  if (fifo->inIndex >= fifo->outIndex)
    avail = fifo->size16 - (fifo->inIndex - fifo->outIndex) - 1;
  else
    avail = fifo->outIndex - fifo->inIndex - 1;

  return (avail);

} /* utlFifoSpaceAvail */

/*****************************************************************************
 * FUNCTION PURPOSE: Return the number of elements in the fifo
 *****************************************************************************
 * DESCRIPTION: The number of elements in the fifo is computed.
 *****************************************************************************/
tuint utlFifoNelems(utlFifo_t *fifo)
{
  tuint n;

  n = fifo->size16 - utlFifoSpaceAvail(fifo) - 1;
  return (n);

} /* utlFifoNelems */

/******************************************************************************
 * FUNCTION PURPOSE: Enter data into the fifo
 ******************************************************************************
 * DESCRIPTION: Places data into the fifo if enough space is available.
 *              Returns a pointer to the address of the 1st element in the
 *              fifo. Returns NULL if not enough space was available in
 *              the fifo.
 ******************************************************************************/
word *utlFifoIn (utlFifo_t *fifo, globData_t *data)
{
  tuint avail;
  tuint size1, size2;
  word *ibase, *tdata;

  tdata = (word *)data->data;

  /* Check that there is enough space in the fifo */
  avail = utlFifoSpaceAvail (fifo);
  if (avail < data->dataSize)
    return (NULL);

  /* Store the address of the 1st address of stored block for return */
  ibase = &(fifo->base[DMISC_UINT16_TO_WORDS(fifo->inIndex)]);

  /* Compute write sizes, including wrap around */
  size1 = fifo->size16 - fifo->inIndex;
  if (size1 >= data->dataSize)  {
    size1 = data->dataSize;
    size2 = 0;
  }
  else  {
    size2 = data->dataSize - size1;
  } 

  /* Write the 1st block of data to the fifo, update the fifo pointer */
  memcpy (&(fifo->base[DMISC_UINT16_TO_WORDS(fifo->inIndex)]), tdata, DMISC_UINT16_TO_WORDS(size1));
  fifo->inIndex += size1;
  if (fifo->inIndex >= fifo->size16)
    fifo->inIndex = 0;

  /* Copy the 2nd block of data to the fifo, update the fifo pointer */
  memcpy (&(fifo->base[DMISC_UINT16_TO_WORDS(fifo->inIndex)]), &(tdata[DMISC_UINT16_TO_WORDS(size1)]), DMISC_UINT16_TO_WORDS(size2));
  fifo->inIndex += size2;
  /* This second wrap check should not be necessary, since the second block
   * cannot cause an address wrap */
  if (fifo->inIndex >= fifo->size16)
    fifo->inIndex = 0;
  
  /* Return the address where the 1st element of the fifo lives */
  return (ibase);

} /* utlFifoIn */


/*****************************************************************************
 * FUNCTION PURPOSE: Extract data from the fifo
 *****************************************************************************
 * DESCRIPTION: Extracts data from the fifo. The return pointer is the
 *              same as that in the global block. Returns NULL if 
 *              a request was made for more data then is in the fifo
 *****************************************************************************/
word *utlFifoOut (utlFifo_t *fifo, globData_t *data)
{
  tuint avail;
  tuint size1, size2;
  word *tdata;

  tdata = (word *)data->data;

  /* Check that the requested number of element is in the fifo */
  avail = utlFifoNelems(fifo);
  if (data->dataSize > avail)
    return (NULL);

  /* Compute write sizes, including wrap around */
  size1 = fifo->size16 - fifo->outIndex;
  if (size1 >= data->dataSize)  {
    size1 = data->dataSize;
    size2 = 0;
  }
  else {
    size2 = data->dataSize - size1;
  }

  /* Extract the 1st block of the fifo, update the fifo index */
  memcpy (tdata, &(fifo->base[DMISC_UINT16_TO_WORDS(fifo->outIndex)]), DMISC_UINT16_TO_WORDS(size1));
  fifo->outIndex += size1;
  if (fifo->outIndex >= fifo->size16)
    fifo->outIndex = 0;

    
  /* Extract the 2nd block of the fifo */
  memcpy (&(tdata[DMISC_UINT16_TO_WORDS(size1)]), &(fifo->base[DMISC_UINT16_TO_WORDS(fifo->outIndex)]), DMISC_UINT16_TO_WORDS(size2));
  fifo->outIndex += size2;
  /* The wrap check should not be necessary, since the second write cannot
   * case a fifo wrap. */
  if (fifo->outIndex >= fifo->size16)
    fifo->outIndex = 0;

  return (data->data);

} /* utlFifoOut */

/******************************************************************************
 * FUNCTION PURPOSE: Back data out from a fifo
 ******************************************************************************
 * DESCRIPTION: Data is removed from the TAIL of the fifo. This is required
 *              when a single block of data must be entered in multiple
 *              fifo writes. If the first write succeeds, but the second
 *              fails due to lack of space in the fifo, it is necessary to
 *              back the first write out. Essentially this backs up the
 *              in pointer in the fifo.
 *
 *              !!! No check for wrap past the out pointer is made !!!
 *****************************************************************************/
void utlFifoBackout (utlFifo_t *fifo, tuint backoutLen)
{
  if (fifo->inIndex < backoutLen)  
    fifo->inIndex = fifo->size16 - backoutLen + fifo->inIndex;
  else
    fifo->inIndex = fifo->inIndex - backoutLen;
} /* utlFifoBackout */
   

/*****************************************************************************
 * FUNCTION PURPOSE: Reset a fifo
 *****************************************************************************
 * DESCRIPTION: Resets the fifo in and out indices to 0. Used when fifo
 *              error is encountered.
 *****************************************************************************/
void utlFifoReset (utlFifo_t *fifo)
{
   fifo->inIndex = fifo->outIndex = 0;
} /* utlFifoReset */

