#ifndef _dpiu_h
#define _dpiu_h
/******************************************************************************
 * FILE PURPOSE: Defines API for diagnostic PIU (PCM input/output Unit)
 ******************************************************************************
 * FILE NAME: dpiu.h
 *
 * DESCRIPTION: Defines the API used by the system to exchange data with DPIU
 *
 * TABS: None
 *
 * Copyright (C) 2002, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include "types.h"
#include "dglob.h"

/****************************************************************************
 * DEFINITION: Return values from dpiu
 *****************************************************************************/
typedef enum {
  DPIU_SUCCESS = 0,  
  DPIU_ERR_STATE_CLOSED,
  DPIU_ERR_INVALID_OPEN,
  DPIU_ERR_INVALID_DATA_LENGTH,
  DPIU_ERR_INVALID_CTL
} dpiuReturn_t;

/*****************************************************************************
 * DPIU size configuration structure. Used by DPIU to determine its memory
 * requirements.
 *****************************************************************************/
typedef struct {
  tint tdmBufSize;   /* Size, in words of the DPIU TDM buffers */
} dpiuSizeConfig_t;

/****************************************************************************
 * DPIU New configuration structure. 
 ****************************************************************************/
typedef struct {
  tuint Id;                   /* Instance ID                                 */
  void *taskInst;             /* Task posting instance                       */
  tint (*txTaskCall)(void *); /* Function to call when tdm tx data needed    */
  tint (*rxTaskCall)(void *); /* Function to call when tdm rx data available */
  dpiuSizeConfig_t sizeCfg;
} dpiuNewConfig_t;

/*****************************************************************************
 * DPIU Open configuration structre.
 *****************************************************************************/
typedef struct {
  tuint  tdmFrameSize;   /* Input to DPIU - initial frame size used by TDM  */
  tuint  rxInitVal;      /* Initial value sent to TDM out                   */ 
  tuint *rxInitBuf;      /* Output - 1st two rx (to tdm) buffer locations   */
  tuint *txInitBuf;      /* Output - 1st two tx (from tdm) buffer locations */
} dpiuOpenConfig_t;


/*****************************************************************************
 * DPIU control
 *****************************************************************************/
typedef struct dpiuReset_s {
  linSample *txBuf;
  linSample *rxBuf;
} dpiuReset_t;

enum {
  DPIU_CTL_MODE_RESET  = 0
};

typedef struct dpiuControl_s {

  tint mode;
  
  union {
    dpiuReset_t reset;
  } u;
  
} dpiuControl_t;
   


/*****************************************************************************
 * DPIU data transfer API. dpiuTx gets data received from the TDM, dpiuRx
 * passes data to the TDM.
 *****************************************************************************/
dpiuReturn_t dpiuTx (void *dpiuInst, globData_t *tx);
dpiuReturn_t dpiuRx (void *dpiuInst, globData_t *rx);

/*****************************************************************************
 * DPIU TDM data transfer API. The application gives a pointer to this function
 * to the TDM driver of the HAL unit. This function is called when the TDM
 * driver is ready to exchange data. This function is called from an ISR
 * context.
 *****************************************************************************/
void dpiuTdmTxExchange (void *dpiuInst, linSample **toTdmdata, 
                        linSample **rInDummy, tulong ts, tuint nsamples);
void dpiuTdmRxExchange (void *dpiuInst, linSample **fromTdmdata, 
                        linSample **rInDummy, tulong ts, tuint nsamples);

/*****************************************************************************
 * DPIU Initialization and configuration
 *****************************************************************************/
dpiuReturn_t dpiuGetSizes (tint *nbufs, dmemBuffer_t **bufs, 
                           dpiuSizeConfig_t *cfg);
void *dpiuNew (dpiuNewConfig_t *cfg, dmemBuffer_t *bufs);
dpiuReturn_t dpiuOpen (void *dpiuInst, dpiuOpenConfig_t *cfg);
dpiuReturn_t dpiuControl (void *vinst, dpiuControl_t *ctl);

/*****************************************************************************
 * DPIU API call table. The dpiu API is referenced by the system through a 
 * pointer to this call table.
 *****************************************************************************/
typedef struct {
  dpiuReturn_t (*dpiuGetSizes) (tint *, dmemBuffer_t **,  dpiuSizeConfig_t *);
  void *(*dpiuNew) (dpiuNewConfig_t *, dmemBuffer_t *);
  dpiuReturn_t (*dpiuOpen) (void *, dpiuOpenConfig_t *);
  dpiuReturn_t (*dpiuControl) (void *, dpiuControl_t *);
  dpiuReturn_t (*dpiuTx) (void *, globData_t *);
  dpiuReturn_t (*dpiuRx) (void *, globData_t *);
  void (*dpiuTdmTxExchange)(void *, linSample **, linSample **, tulong, tuint);
  void (*dpiuTdmRxExchange)(void *, linSample **, linSample **, tulong, tuint);
} dpiuCallTable_t;
  
extern dpiuCallTable_t dpiuCallTable;
 


#endif




