/******************************************************************************
 * FILE PURPOSE: Perform Core Boot
 ******************************************************************************
 * FILE NAME: dsiu5561.c
 *
 * DESCRIPTION: Contains the CPU core boot code specific to the Himalaya
 *              processor.
 *
 * TABS: None
 *
 * Copyright (C) 2002, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include "dsiuloc.h"
#include "diagcfg.h"
#include "types.h"
#include "hal.h"
#include "tiboot_him.h"

/**************************************************************************************
 * Declaration: The mac chip boot loader leaves configuration information in memory.
 *              This information will be used to configure the mac address for 
 *              reception of the boot config packet.
 **************************************************************************************/
#pragma DATA_SECTION(chipBoot, ".chipBoot")
BOOT_PARAMS_T chipBoot;

/*****************************************************************************
 * FUNCTION PURPOSE: Boot Himalaya core
 *****************************************************************************
 * DESCRIPTION: Core 0 is booted by call the function at the entry symbol. 
 *              This is a one way call, with no return. The remaining cores
 *              are booted by copying the entry address into the local memory
 *              of that core at word address 0x0100.
 *
 *****************************************************************************/
void  dummy_code(void);

dsiuCoreBootReturn_t himCoreBoot (tuint core, tulong entry)
{
  void (*goodbye)(void);

  DSIU_DUMMY_ARG(core);

  goodbye = (void (*)())(entry);
  (*goodbye)(); /* And don't come back! */

  return (DSIU_CORE_BOOT_SUCCESS);

} /* HimCoreBoot */
  

void  dummy_code(void)
{
   asm(" NOP"); 
 
}
 

/**********************************************************************************
 * FUNCTION PURPOSE: Get boot information from the I2C boot loader
 **********************************************************************************
 * DESCRIPTION: Extracst the mac boot information left by the rom boot loader
 **********************************************************************************/
dsiuReturn_t himI2cBootInfo (dsiuPrevBootInfo_t *info)
{
  /* Currently only mac is supported in the i2c */
  info->physIf   = DSIU_PREV_BOOT_PHYS_IFACE_MAC;
  info->physPort = 0;   /* There is only one mac port on Himalaya */

  /* Setup the interface options */
  info->u.macInfo.iftype = DSIU_PREV_MAC_IF_MII;

  if (chipBoot.eth.options & BOOT_PARAMS_ETH_OPTIONS_GE)  {

    if (chipBoot.eth.options & BOOT_PARAMS_ETH_OPTIONS_RMII)
      info->u.macInfo.iftype = DSIU_PREV_MAC_IF_RGMII;
    else
      info->u.macInfo.iftype = DSIU_PREV_MAC_IF_GMII;

  }

  if (chipBoot.eth.options & BOOT_PARAMS_ETH_OPTIONS_HD)
    info->u.macInfo.halfDuplex = TRUE;
  else
    info->u.macInfo.halfDuplex = FALSE;

  if (chipBoot.eth.options & BOOT_PARAMS_ETH_OPTIONS_FC)
    info->u.macInfo.flowControl = TRUE;
  else
    info->u.macInfo.flowControl = FALSE;

  info->u.macInfo.macAddr[0] = chipBoot.eth.mac_addr_h;
  info->u.macInfo.macAddr[1] = chipBoot.eth.mac_addr_m;
  info->u.macInfo.macAddr[2] = chipBoot.eth.mac_addr_l;

  return (DSIU_SUCCESS);

} /* himI2cBootInfo */

