/******************************************************************************
 * FILE PURPOSE: Perform Core Boot
 ******************************************************************************
 * FILE NAME: dsiukailash.c
 *
 * DESCRIPTION: Contains the CPU core boot code specific to the Himalaya
 *              processor.
 *
 * TABS: None
 *
 * Copyright (C) 2002, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include "dsiuloc.h"
#include "diagcfg.h"
#include "types.h"
#include "hal.h"
#include "tiboot_kls.h"
/**************************************************************************************
 * Declaration: The mac chip boot loader leaves configuration information in memory.
 *              This information will be used to configure the mac address for 
 *              reception of the boot config packet.
 **************************************************************************************/
#pragma DATA_SECTION(chipBoot, ".chipBoot")
BOOT_PARAMS_T chipBoot;

/*****************************************************************************
 * FUNCTION PURPOSE: Boot Himalaya core
 *****************************************************************************
 * DESCRIPTION: Core 0 is booted by call the function at the entry symbol. 
 *              This is a one way call, with no return. The remaining cores
 *              are booted by copying the entry address into the local memory
 *              of that core at word address 0x0100.
 *
 *****************************************************************************/
void  dummy_code(void);

dsiuCoreBootReturn_t kailashCoreBoot (tuint core, tulong entry)
{
  
  halControl_t ctl;

  void (*goodbye)(void);
#ifdef __TMS320C55X__
  entry = entry >> 1;  /* Undo the divide by 2 doen in the message processing */
#endif
  if (core == dsiuContext.coreId)  {
    (*dsiuContext.hal->halIntrLevel) (HAL_INT_LEVEL_DISABLE);
  goodbye = (void (*)())(entry);
  (*goodbye)(); /* And don't come back! */
  }

  /* Direct the hal to boot a different core */
  ctl.cmd                  = HAL_COMMAND_CORE_BOOT;
  ctl.u.coreBoot.core      = core;
  ctl.u.coreBoot.entryAddr = entry;

  if ( ((*dsiuContext.hal->halControl)(&ctl)) == HAL_SUCCESS)
    return (DSIU_CORE_BOOT_SUCCESS);

  return (DSIU_CORE_BOOT_INIT_FAIL);
} /* kailashCoreBoot */
  
/***************************************************************************************
 * FUNCTION PURPOSE: Put a core into reset
 ***************************************************************************************
 * DESCRIPTION: Puts a core into reset. Invokes the hal to keep the specified core
 *              into reset.
 ***************************************************************************************/
dsiuCoreResetReturn_t kailashCoreReset (tint core)
{
  halControl_t ctl;

  /* Direct hal to reset the core */
  ctl.cmd              = HAL_COMMAND_CORE_RESET;
  ctl.u.coreReset.core = core;

  if ( ((*dsiuContext.hal->halControl)(&ctl)) == HAL_SUCCESS)
    return (DSIU_CORE_RESET_SUCCESS);

  return (DSIU_CORE_RESET_FAIL);
}
/**************************************************************************************
 * FUNCTION PURPOSE: Return the mac boot parameters 
 **************************************************************************************
 * DESCRIPTION: Returns the relevent mac boot parameters left from the rom boot
 **************************************************************************************/
dsiuReturn_t dsiu_kailash_mac_boot_info (dsiuPrevBootInfo_t *info)
{
  tuint opt;

  info->physIf = DSIU_PREV_BOOT_PHYS_IFACE_MAC;
  info->physPort = chipBoot.eth.portNum;

  opt = chipBoot.eth.options & 0x7;

  switch (opt)  {
    
    case BOOT_PARAMS_ETH_OPTIONS_MII:
         info->u.macInfo.iftype = DSIU_PREV_MAC_IF_MII;
         break;

    case BOOT_PARAMS_ETH_OPTIONS_RMII:
         info->u.macInfo.iftype = DSIU_PREV_MAC_IF_RMII;
         break;

    case BOOT_PARAMS_ETH_OPTIONS_GMII:
         info->u.macInfo.iftype = DSIU_PREV_MAC_IF_GMII;
         break;

    case BOOT_PARAMS_ETH_OPTIONS_RGMII:
         info->u.macInfo.iftype = DSIU_PREV_MAC_IF_RGMII;
         break;

    case BOOT_PARAMS_ETH_OPTIONS_SMII:
         info->u.macInfo.iftype = DSIU_PREV_MAC_IF_SMII;
         break;

    case BOOT_PARAMS_ETH_OPTIONS_S3MII:
         info->u.macInfo.iftype = DSIU_PREV_MAC_IF_S3MII;
         break;

    case BOOT_PARAMS_ETH_OPTIONS_RMII_10:
         info->u.macInfo.iftype = DSIU_PREV_MAC_IF_RMII_10;
         break;

    case BOOT_PARAMS_ETH_OPTIONS_RMII_100:
         info->u.macInfo.iftype = DSIU_PREV_MAC_IF_RMII_100;
         break;

  }

  if (chipBoot.eth.options & BOOT_PARAMS_ETH_OPTIONS_HD)
    info->u.macInfo.halfDuplex = TRUE;
  else
    info->u.macInfo.halfDuplex = FALSE;

  if (chipBoot.eth.options & BOOT_PARAMS_ETH_OPTIONS_FC)
    info->u.macInfo.flowControl = TRUE;
  else
    info->u.macInfo.flowControl = FALSE;

  info->u.macInfo.macAddr[0] = chipBoot.eth.mac_addr_h;
  info->u.macInfo.macAddr[1] = chipBoot.eth.mac_addr_m;
  info->u.macInfo.macAddr[2] = chipBoot.eth.mac_addr_l;

  return (DSIU_SUCCESS);

} /* dsiu_kailash_mac_boot_info */
    
        

/**************************************************************************************
 * FUNCTION PURPOSE: Get boot information from the ROM boot loader
 **************************************************************************************
 * DESCRIPTION: Extracts the mac boot information left by the rom boot loader.
 **************************************************************************************/
dsiuReturn_t kailashRomBootInfo (dsiuPrevBootInfo_t *info)
{

  /* This is not clean as there is difference in the way the ROM bootloader 
     structure is populated in Kailash and Tomahawk. Presently for kailash the assumption
     is the boot mode is always ethernet and the only information extracted is MAC addresses*/
  info->u.macInfo.macAddr[0] = chipBoot.eth.mac_addr_h;
  info->u.macInfo.macAddr[1] = chipBoot.eth.mac_addr_m;
  info->u.macInfo.macAddr[2] = chipBoot.eth.mac_addr_l;
#if 0
  switch (chipBoot.common.boot_mode)  {
  
    case BOOT_MODE_ETH: return (dsiu_kailash_mac_boot_info(info));

    default: return (DSIU_FAIL);

  }
#endif
  return (DSIU_SUCCESS);

} /* kailashRomBootInfo */
  


/***************************************************************************************
 * FUNCTION PURPOSE: Configure the memory interface
 ***************************************************************************************
 * DESCRIPTION: The cache configurations are setup
 ***************************************************************************************/
dsiuReturn_t kailashMemConfig (void)
{
  halMemConfig_t halMemCfg;

  halMemCfg.memCfgFunction       = HAL_MEMCFGFUNC_CACHECONFIG;
  halMemCfg.u.cacheCfg.cacheType = HAL_CACHE_L1D_FDEF;

  if (dsiuContext.hal->halMemConfig != NULL)
    (*dsiuContext.hal->halMemConfig)(&halMemCfg);


  halMemCfg.memCfgFunction       = HAL_MEMCFGFUNC_CACHECONFIG;
  halMemCfg.u.cacheCfg.cacheType = HAL_CACHE_L1P_FDEF;

  if (dsiuContext.hal->halMemConfig != NULL)
    (*dsiuContext.hal->halMemConfig)(&halMemCfg);


  halMemCfg.memCfgFunction       = HAL_MEMCFGFUNC_CACHECONFIG;
  halMemCfg.u.cacheCfg.cacheType = HAL_CACHE_L2_FDEF;

  if (dsiuContext.hal->halMemConfig != NULL)
    (*dsiuContext.hal->halMemConfig)(&halMemCfg);

  return (DSIU_SUCCESS);

} /* kailashMemConfig */


