/*******************************************************************************************
 * FILE PURPOSE: C64x memory test
 *******************************************************************************************
 * DESCRIPTION: Runs the C64x Memory tests
 *
 * FILE NAME dsiumtst64x.c
 *
 * Copyright(c) 2008, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include "types.h"
#include "dsiuloc.h"
#include "dsiuport.h"
#include "diagcfg.h"
#include "string.h"
#include "diagcfg.h"
#include "utlbitfield.h"
#include "llmt.h"


typedef struct relocFcn_s {

  void (*marching) (word *, tulong, llmtScratchBlock_t *);  /* Marching 1s/0s  */
  void (*addr)     (word *, tulong, llmtScratchBlock_t *);  /* Address/inverse */
  void (*aa55)     (word *, tulong, llmtScratchBlock_t *);  /* AA/55           */
  void (*pseudo)   (word *, tulong, llmtScratchBlock_t *);  /* Pseudo random   */
  void (*code)     (word *, tulong, llmtScratchBlock_t *);  /* code test       */

} relocFcn_t;


/******************************************************************************************
 * FUNCTION PURPOSE: Determine if a memory block is one of the three scratch blocks
 ******************************************************************************************
 * DESCRIPTION: The start address is compared to see if a memory block is a scratch block
 ******************************************************************************************/
bool dsiu_is_scratch_block (llmtScratchBlock_t *scratch, dcfgMemBlock_t *block)
{
  if (block->startAddr == scratch->dscratchBlk->startAddr)
    return (TRUE);

  if (block->startAddr == scratch->sscratchBlk->startAddr)
    return (TRUE);

  if (block->startAddr == scratch->cscratchBlk->startAddr)
    return (TRUE);

  return (FALSE);

} /* dsiu_is_scratch_block */

/*******************************************************************************************
 * FUNCTION PURPOSE: Record memory test errors
 *******************************************************************************************
 * DESCRIPTION: Records errors from a specific memory test into the diagnostic status
 *              registers. Returns non-zero if all report slots are filled.
 *******************************************************************************************/
tint dsiu_record_errors (llmtErr_t *err)
{
  tint i;
  dregMemTest_t *memrep;
  dmsg_dh_mem_test_status_t *srep;

  memrep = &(dsiuContext.dregStatus->memTest);
  srep   = &(dsiuContext.msgTxBuf->msg_body.mem_test_status);


  for (i = 0; i < err->errCount; i++)  {

    if (srep->fail_count < DSIU_MSG_MEM_TEST_STATUS_FAIL_ARRAY_SIZE)  {

      srep->failures[srep->fail_count].address_msw = err->errAddr[i] >> 16;
      srep->failures[srep->fail_count].address_lsw = err->errAddr[i] & 0xffff;

      srep->failures[srep->fail_count].expected_msw = err->errExptd[i] >> 16;
      srep->failures[srep->fail_count].expected_lsw = err->errExptd[i] & 0xffff;

      srep->failures[srep->fail_count].actual_msw = err->errActual[i] >> 16;
      srep->failures[srep->fail_count].actual_lsw = err->errActual[i] & 0xffff;

      srep->fail_count += 1;
    }

    memrep->fail[memrep->nFailed].address = err->errAddr[i];
    memrep->fail[memrep->nFailed].written = err->errExptd[i];
    memrep->fail[memrep->nFailed].read    = err->errActual[i];
    memrep->nFailed += 1;
    if (memrep->nFailed >= DREG_MAX_MEM_FAILURES)
      return (-1);
  }

  return (0);

} /* dsiu_record_errors */


/*******************************************************************************************
 * FUNCTION PURPOSE: Perform a set of direct memory tests
 *******************************************************************************************
 * DESCRIPTION: Memory tests are performed on the scratch blocks without a data save
 *              or context switch.
 *******************************************************************************************/
tint dsiu_memtest_direct (dcfgMemBlock_t *block, tuint control_code, tint tranId, tulong testBlockSize)
{
  llmtErr_t  err;
  word      *testAddr;
  tulong     testLen;
  bool       testAbort;
  tint       count;
  tulong     cumBytes;
  tulong     blockLen;
  tint       nblocks;

  /* If the testBlockSize is set to 0, the entire block is tested in one pass */
  if (testBlockSize == 0)
    testBlockSize = block->bufLenWords;


  nblocks = block->bufLenWords / testBlockSize;
  if (block->bufLenWords % testBlockSize)
    nblocks += 1;

  for (testAddr = block->startAddr, testLen = block->bufLenWords, testAbort = FALSE, count = 1, cumBytes = 0; 
         testLen > 0; 
           testAddr += testBlockSize, testLen -= testBlockSize, count += 1)  { 

    err.errCount = 0;

    blockLen = (testLen < testBlockSize ? testLen : testBlockSize);

    /* Prepare a block report reply. It may not be sent if block updates are disabled */
    dsiuContext.msgTxBuf->header.length   = DSIU_MSG_HEADER_SIZE + DSIU_MSG_MEM_TEST_REPLY_SIZE;
    dsiuContext.msgTxBuf->header.chnum    = 0;
    dsiuContext.msgTxBuf->header.trans_id = tranId;
    dsiuContext.msgTxBuf->header.msg_id   = 156;

    memset (&dsiuContext.msgTxBuf->msg_body.mem_test_status, 0, sizeof (dmsg_dh_mem_test_status_t));


    if ( (testAbort == FALSE) && (control_code & DMSG_CODE_CTRL_MARCH) )  {
      llmtMarchingContinue(testAddr, blockLen, &err);

      if (dsiu_record_errors (&err))
        testAbort = TRUE;

    }

    if ( (testAbort == FALSE) && (control_code & DMSG_CODE_CTRL_55) ) {
      llmtAA55Continue (testAddr, blockLen, &err);

      if (dsiu_record_errors (&err))
        testAbort = TRUE;

    }

    if ( (testAbort == FALSE) && (control_code & DMSG_CODE_CTRL_PSEUDO) )  {
      llmtPseudoContinue (testAddr, blockLen, &err);

      if (dsiu_record_errors (&err))
        testAbort = TRUE;

    }

    if ( (testAbort == FALSE) && (control_code & DMSG_CODE_CTRL_ADDR) )  {
      llmtAddrContinue (testAddr, blockLen, &err);

      if (dsiu_record_errors (&err))
        testAbort = TRUE;

    }

    if  ( (testAbort == FALSE) && (control_code & DMSG_CODE_CTRL_PROG) )  {
      llmtProgContinue (testAddr, blockLen, &err);

      if (dsiu_record_errors (&err))
        testAbort = TRUE;

    }

    /* Update message parameters */
    cumBytes += blockLen;

    dsiuContext.msgTxBuf->msg_body.mem_test_status.block_addr_msw = ((tulong)testAddr) >> 16;
    dsiuContext.msgTxBuf->msg_body.mem_test_status.block_addr_lsw = ((tulong)testAddr) & 0xffff;

    dsiuContext.msgTxBuf->msg_body.mem_test_status.block_size_msw = blockLen >> 16;
    dsiuContext.msgTxBuf->msg_body.mem_test_status.block_size_lsw = blockLen & 0xffff;

    dsiuContext.msgTxBuf->msg_body.mem_test_status.blocks_complete = count;
  
    dsiuContext.msgTxBuf->msg_body.mem_test_status.bytes_complete_msw = cumBytes >> 16;
    dsiuContext.msgTxBuf->msg_body.mem_test_status.bytes_complete_lsw = cumBytes & 0xffff;

    dsiuContext.msgTxBuf->msg_body.mem_test_status.blocks_remaining = nblocks - count;

    dsiuContext.msgTxBuf->msg_body.mem_test_status.bytes_remaining_msw = (block->bufLenWords - cumBytes) >> 16;
    dsiuContext.msgTxBuf->msg_body.mem_test_status.bytes_remaining_lsw = (block->bufLenWords - cumBytes) & 0xffff;


    if (control_code & DMSG_CODE_CTRL_STATUS)
      dsiu_send_msg (dsiuContext.msgTxBuf);

    /* Check for pending messages */
    dsiuPollTasks();

  }

  return (0);

} /* dsiu_memtest_direct */

/*******************************************************************************************
 * FUNCTION PURPOSE: Perform a set of context switch memory test
 *******************************************************************************************
 * DESCRIPTION: Memory tests are run after switching the stack context and tracking
 *              variables to scratch.
 *
 *******************************************************************************************/
tint dsiu_memtest_context (dcfgMemBlock_t *block, llmtScratchBlock_t *scratch, tuint control_code, relocFcn_t *relocFcn, tuint tid)
{
  halQuery_t          query;
  halUtopiaControl_t  utCtl;
  halGmacControl_t    gmacCtl;
  word               *testAddr;
  llmtb_t            *mcxt;
  tulong              testLen;
  tulong              blockLen;
  tulong              cumBytes;
  halMemConfig_t      memCfg;
  bool                testAbort;

  tint i;
  tint count;
  tint nblocks;


  /* Do not check the block if it is not local and the local only test flag is set */
  query.periph = HAL_QUERY_MEMORY;
  query.u.mem.address = block->startAddr;
  (*dsiuContext.hal->halQuery)(&query);

  if (control_code & DMSG_CODE_CTRL_LOCAL)  {
    if (query.u.mem.type != HAL_MEM_LOCAL)
      return (0);
  }

  /* Do not check the block if global test is enabled (local only disabled), but the 
   * don't test other cores local bit is set. This test is essentially local plus shared 
   * memory  test */
  if ( ((control_code & DMSG_CODE_CTRL_LOCAL) == 0 )      &&
        (control_code & DMSG_CODE_CTRL_NO_OCORE_LOCAL) )  {

    if (query.u.mem.type == HAL_MEM_OTHER_CORE)
      return (0);
  }

  /* If the block is local and has the l1p flag, disable l1p */
  if (DCFG_MEMTEST_ATTR_IS_L1P(block->flags))  {
    return (0);   /* Temporarily disable l1p memory testing */

#if 0
    if (query.u.mem.type == HAL_MEM_LOCAL)  {
      memCfg.memCfgFunction       = HAL_MEMCFGFUNC_CACHECONFIG;
      memCfg.u.cacheCfg.cacheType = HAL_CACHE_L1P;
      memCfg.u.cacheCfg.cacheMode = 0;
      (*dsiuContext.hal->halMemConfig)(&memCfg);
    }
#endif

  }


  /* Disable the network rx interface if flagged. */
  if (DCFG_MEMTEST_ATTR_IS_RXNET(block->flags))  {

    if (dsiuContext.hal->utopia)  {
      if (control_code & DMSG_CODE_CTRL_LOCAL)
        utCtl.mode = HAL_UTOPIA_RX_TMP_DISABLE_CORE;
      else
        utCtl.mode = HAL_UTOPIA_RX_TMP_DISABLE_ALL;

      for (i = 0; i < dsiuContext.utLogicAlloc; i++)  
        (*dsiuContext.hal->utopia->halUtopiaControl)(dsiuContext.utopiaPort[i].halId, &utCtl);

    }

    if (dsiuContext.hal->gmac)  {

      gmacCtl.code    = HAL_GMAC_CTL_CODE_TMP_DISABLE;
      for (i = 0; i < dsiuContext.macLogicAlloc; i++)
      {
         gmacCtl.u.halId = dsiuContext.macPort[i].halId;
        (*dsiuContext.hal->gmac->halGmacControl)(&gmacCtl);
      }

    }
  }

  /* The results are returned in the stack/data scratch block */
  mcxt = (llmtb_t *)(scratch->sscratchBlk->startAddr);
  /* Prepare a block report reply. It may not be sent if block updates are disabled */
  dsiuContext.msgTxBuf->header.length   = DSIU_MSG_HEADER_SIZE + DSIU_MSG_MEM_TEST_REPLY_SIZE;
  dsiuContext.msgTxBuf->header.chnum    = 0;
  dsiuContext.msgTxBuf->header.trans_id = tid;
  dsiuContext.msgTxBuf->header.msg_id   = 156;

  memset (&dsiuContext.msgTxBuf->msg_body.mem_test_status, 0, sizeof (dmsg_dh_mem_test_status_t));

  nblocks = block->bufLenWords / scratch->dscratchBlk->bufLenWords;
  if (block->bufLenWords % scratch->dscratchBlk->bufLenWords)
    nblocks += 1;

  for (testAddr = block->startAddr, testLen = block->bufLenWords, testAbort = FALSE, count = 1, cumBytes = 0; 
         testLen > 0; 
           testAddr += blockLen, testLen -= blockLen, count += 1)  { 

    mcxt->err.errCount = 0;
    blockLen = (testLen < scratch->dscratchBlk->bufLenWords ? testLen : scratch->dscratchBlk->bufLenWords);


    /* Update message parameters */
    cumBytes += blockLen;

    dsiuContext.msgTxBuf->msg_body.mem_test_status.block_addr_msw = ((tulong)testAddr) >> 16;
    dsiuContext.msgTxBuf->msg_body.mem_test_status.block_addr_lsw = ((tulong)testAddr) & 0xffff;

    dsiuContext.msgTxBuf->msg_body.mem_test_status.block_size_msw = blockLen >> 16;
    dsiuContext.msgTxBuf->msg_body.mem_test_status.block_size_lsw = blockLen & 0xffff;

    dsiuContext.msgTxBuf->msg_body.mem_test_status.blocks_complete = count;
  
    dsiuContext.msgTxBuf->msg_body.mem_test_status.bytes_complete_msw = cumBytes >> 16;
    dsiuContext.msgTxBuf->msg_body.mem_test_status.bytes_complete_lsw = cumBytes & 0xffff;

    dsiuContext.msgTxBuf->msg_body.mem_test_status.blocks_remaining = nblocks - count;

    dsiuContext.msgTxBuf->msg_body.mem_test_status.bytes_remaining_msw = (block->bufLenWords - cumBytes) >> 16;
    dsiuContext.msgTxBuf->msg_body.mem_test_status.bytes_remaining_lsw = (block->bufLenWords - cumBytes) & 0xffff;



    /* Marking ones/zeros test */

    if ( (testAbort == FALSE) && (control_code & DMSG_CODE_CTRL_MARCH) )  {
      (*relocFcn->marching) (testAddr, blockLen, scratch);

      if (dsiu_record_errors (&mcxt->err))
        testAbort = TRUE;
    }

    /* Alternating bit pattern test */
    if ( (testAbort == FALSE) && (control_code & DMSG_CODE_CTRL_55) )  {
      (*relocFcn->aa55) (testAddr, blockLen, scratch);

      if (dsiu_record_errors (&mcxt->err))
        testAbort = TRUE;
    }

    if ( (testAbort == FALSE) && (control_code & DMSG_CODE_CTRL_ADDR) )  {
      (*relocFcn->addr) (testAddr, blockLen, scratch);

      if (dsiu_record_errors (&mcxt->err))
        testAbort = TRUE;
    }


    if ( (testAbort == FALSE) && (control_code & DMSG_CODE_CTRL_PSEUDO) ) {
      (*relocFcn->pseudo) (testAddr, blockLen, scratch);

      if (dsiu_record_errors (&mcxt->err))
        testAbort = TRUE;
    }

    if (control_code & DMSG_CODE_CTRL_STATUS)
      dsiu_send_msg (dsiuContext.msgTxBuf);

  }

  

  /* Restore the network interface if it was disabled */
  if (DCFG_MEMTEST_ATTR_IS_RXNET(block->flags))  {

    if (dsiuContext.hal->utopia)  {

      if (control_code & DMSG_CODE_CTRL_LOCAL)
        utCtl.mode = HAL_UTOPIA_RX_RE_ENABLE_CORE;
      else
        utCtl.mode = HAL_UTOPIA_RX_RE_ENABLE_ALL;

      for (i = 0; i < dsiuContext.utLogicAlloc; i++)  
        (*dsiuContext.hal->utopia->halUtopiaControl)(dsiuContext.utopiaPort[i].halId, &utCtl);

    }

    if (dsiuContext.hal->gmac)  {

      gmacCtl.code    = HAL_GMAC_CTL_CODE_RE_ENABLE;
      for (i = 0; i < dsiuContext.macLogicAlloc; i++)
      {
        gmacCtl.u.halId = dsiuContext.macPort[i].halId;
        (*dsiuContext.hal->gmac->halGmacControl)(&gmacCtl);
      }
    }
  }

  /* Restore the L1P if required */
  if (DCFG_MEMTEST_ATTR_IS_L1P(block->flags))  {

    if (query.u.mem.type == HAL_MEM_LOCAL)  {
      memCfg.memCfgFunction       = HAL_MEMCFGFUNC_CACHECONFIG;
      memCfg.u.cacheCfg.cacheType = HAL_CACHE_L1P;
      memCfg.u.cacheCfg.cacheMode = diagCfg.memCfg->u.mem64x.l1p;
      (*dsiuContext.hal->halMemConfig)(&memCfg);
    }
  }

  return (0);

} /* dsiu_memtest_context */


/*******************************************************************************************
 * FUNCTION PURPOSE: Begin a set of memory tests
 *******************************************************************************************
 * DESCRIPTION: A new memory test cycle is started.
 *******************************************************************************************/
void dsiu_memtest_top (dmsg_t *imsg)
{
  tint   i;
  tulong size;


  llmtScratchBlock_t scratch = { NULL, NULL, NULL };

  halTdmControl_t tctl;
  halQuery_t      query;
  halMemConfig_t  memCfg;

  bool testAbort;

  dmsg_hd_diag_code_control_t *dctl;


  relocFcn_t relocFcn;

  dcfgMemBlock_t ddrMemBlock;
  tulong ddrBlock;

  dmsg_t  pmsg;
  dmsg_t *msg;

  /* Because there is only 1 receive message buffer in the system it is necessary to
   * copy the received message into a local copy. This allows for processesing of
   * messages during the dsiuPoll callouts */
  msg = &pmsg;
  memcpy (msg, imsg, sizeof(dmsg_t));
  dctl = &(msg->msg_body.diag_code_control);


  /* Make sure the compile time max number of tdm ports is sufficient */
  if (diagCfg.nTdmPorts > DSIU_MAX_TDM_PORTS)  {
    dsiuErrorIndication (msg->header.trans_id, msg->header.chnum, MGB_ERR_HAL);
    dsiuContext.dregStatus->memTest.status  = DREG_MEM_STATUS_NOT_RUN;
    return;
  }



  /* Locate the three scratch blocks. There can be multiple blocks marked as scratch
   * if these reside in local memory, so the one corresponding to the current core
   * must be used. If the scratch block is in global memory only one will be marked
   * for each type */
  query.periph = HAL_QUERY_MEMORY;

  for (i = 0; i < diagCfg.numMemTestBlocks; i++)  {

    /* Data scratch */
    if (DCFG_MEMTEST_ATTR_IS_DSCRATCH(dcfgMemBlock[i].flags))  {

      /* Found a block marked for data scratch. Check if this 
       * block is a shared address or local */
      query.u.mem.address = dcfgMemBlock[i].startAddr;
      (*dsiuContext.hal->halQuery)(&query);

      if ( (query.u.mem.type == HAL_MEM_LOCAL) ||
           (query.u.mem.type == HAL_MEM_SHARED)  )  {

        /* Found a match. Make sure there are not two marked */
        if (scratch.dscratchBlk != NULL)  {
          dsiuErrorIndication (msg->header.trans_id, msg->header.chnum, MGB_ERR_MEMTEST);
          dsiuContext.dregStatus->memTest.status  = DREG_MEM_STATUS_NOT_RUN;
          return;
        }

        scratch.dscratchBlk = &dcfgMemBlock[i];
      }
    }

    if (DCFG_MEMTEST_ATTR_IS_SSCRATCH(dcfgMemBlock[i].flags))  {

      /* Found a block marked for stack scratch. Check if this 
       * block is a shared address or local */
      query.u.mem.address = dcfgMemBlock[i].startAddr;
      (*dsiuContext.hal->halQuery)(&query);

      if ( (query.u.mem.type == HAL_MEM_LOCAL) ||
           (query.u.mem.type == HAL_MEM_SHARED)  )  {

        /* Found a match. Make sure there are not two marked */
        if (scratch.sscratchBlk != NULL)  {
          dsiuErrorIndication (msg->header.trans_id, msg->header.chnum, MGB_ERR_MEMTEST);
          dsiuContext.dregStatus->memTest.status  = DREG_MEM_STATUS_NOT_RUN;
          return;
        }

        scratch.sscratchBlk = &dcfgMemBlock[i];
  
      }
    }
      
    if (DCFG_MEMTEST_ATTR_IS_PSCRATCH(dcfgMemBlock[i].flags))  {

      /* Found a block marked for stack scratch. Check if this 
       * block is a shared address or local */
      query.u.mem.address = dcfgMemBlock[i].startAddr;
      (*dsiuContext.hal->halQuery)(&query);

      if ( (query.u.mem.type == HAL_MEM_LOCAL) ||
           (query.u.mem.type == HAL_MEM_SHARED)  )  {

        /* Found a match. Make sure there are not two marked */
        if (scratch.cscratchBlk != NULL)  {
          dsiuErrorIndication (msg->header.trans_id, msg->header.chnum, MGB_ERR_MEMTEST);
          dsiuContext.dregStatus->memTest.status  = DREG_MEM_STATUS_NOT_RUN;
          return;
        }

        scratch.cscratchBlk = &dcfgMemBlock[i];

      }
    }
      
  }

  /* Make sure one of each scratch type is found */
  if ((scratch.dscratchBlk == NULL)  ||
      (scratch.sscratchBlk == NULL)  ||
      (scratch.cscratchBlk == NULL)  )  {

    dsiuErrorIndication (msg->header.trans_id, msg->header.chnum, MGB_ERR_MEMTEST);
    dsiuContext.dregStatus->memTest.status  = DREG_MEM_STATUS_NOT_RUN;
    return;
  }

  /* Make sure there is enough room in the stack scratch block for the
   * memory test context */
  if (scratch.sscratchBlk->bufLenWords < sizeof(llmtb_t))  {
    dsiuErrorIndication (msg->header.trans_id, msg->header.chnum, MGB_ERR_MEMTEST);
    dsiuContext.dregStatus->memTest.status  = DREG_MEM_STATUS_NOT_RUN;
    return;
  }

  /* Disable the L1D cache. The L1P is disabled if necessary during individual
   * block test */
  memCfg.memCfgFunction       = HAL_MEMCFGFUNC_CACHECONFIG;
  memCfg.u.cacheCfg.cacheType = HAL_CACHE_L1D;
  memCfg.u.cacheCfg.cacheMode = 0;
  (*dsiuContext.hal->halMemConfig)(&memCfg);

  /* Disable the L2 cache. */
  memCfg.memCfgFunction       = HAL_MEMCFGFUNC_CACHECONFIG;
  memCfg.u.cacheCfg.cacheType = HAL_CACHE_L2;
  memCfg.u.cacheCfg.cacheMode = 0;
  (*dsiuContext.hal->halMemConfig)(&memCfg);

  testAbort = FALSE;


  /* Skip internal memory tests if requested */
  if (msg->msg_body.diag_code_control.ddr_only == 0)  {

    testAbort = FALSE;

    /* Test each of the scratch blocks. */
    if (dsiu_memtest_direct (scratch.dscratchBlk, dctl->control_code, msg->header.trans_id, 0))  {
      dsiuContext.dregStatus->memTest.status = DREG_MEM_STATUS_DONE;
      testAbort = TRUE;
    }


    if (testAbort == FALSE)  {
      if (dsiu_memtest_direct (scratch.sscratchBlk, dctl->control_code, msg->header.trans_id, 0))  {
        dsiuContext.dregStatus->memTest.status = DREG_MEM_STATUS_DONE;
        testAbort = TRUE;
      }
    }

    if (testAbort == FALSE)  {
      if (dsiu_memtest_direct (scratch.cscratchBlk, dctl->control_code, msg->header.trans_id, 0))  {
        dsiuContext.dregStatus->memTest.status = DREG_MEM_STATUS_DONE;
        testAbort = TRUE;
      }
    }

    if (testAbort == FALSE)  {

      /* Copy the relocatable memory test code to the code scratch block */
      size = (tulong)&_mem_test_primary_end - (tulong)&_mem_test_primary_base;
      if (size > scratch.cscratchBlk->bufLenWords)  {
        dsiuErrorIndication (msg->header.trans_id, msg->header.chnum, MGB_ERR_MEMTEST);
        dsiuContext.dregStatus->memTest.status  = DREG_MEM_STATUS_NOT_RUN;
        testAbort = TRUE;
      }

      memcpy (scratch.cscratchBlk->startAddr, (word *)&_mem_test_primary_base, size);

      /* Compute the function pointers to the relocated functions */
      relocFcn.marching = (void (*)(word *, tulong, llmtScratchBlock_t *))((tulong)llmtMarching + (tulong)scratch.cscratchBlk->startAddr - (tulong)&(_mem_test_primary_base));
      relocFcn.addr     = (void (*)(word *, tulong, llmtScratchBlock_t *))((tulong)llmtAddr     + (tulong)scratch.cscratchBlk->startAddr - (tulong)&(_mem_test_primary_base));
      relocFcn.aa55     = (void (*)(word *, tulong, llmtScratchBlock_t *))((tulong)llmtAA55     + (tulong)scratch.cscratchBlk->startAddr - (tulong)&(_mem_test_primary_base));
      relocFcn.pseudo   = (void (*)(word *, tulong, llmtScratchBlock_t *))((tulong)llmtPseudo   + (tulong)scratch.cscratchBlk->startAddr - (tulong)&(_mem_test_primary_base));
      relocFcn.code     = (void (*)(word *, tulong, llmtScratchBlock_t *))((tulong)llmtProg     + (tulong)scratch.cscratchBlk->startAddr - (tulong)&(_mem_test_primary_base));


      /* At the start of the non-scratch mem test is is necessary to disable any peripherals
       * that can effect the memory. If a local only memory test is requested
       * then only the dma for that channel is disabled. */

      for (i = 0; i < diagCfg.nTdmPorts; i++)  {

        if (DREG_PERIPH_READ_STATUS((dsiuContext.tdmPort[i]).status->idNumStatus) == DREG_PERIPH_STATUS_CFG_ENABLED)  {

          tctl.mode = HAL_TDM_HALT_TDM;

          if ( (dctl->control_code & DMSG_CODE_CTRL_LOCAL) || (dctl->control_code & DMSG_CODE_CTRL_NO_OCORE_LOCAL) )
            tctl.u.haltRestore.operation  = HAL_TDM_CTL_HALT_LOCAL;
          else
            tctl.u.haltRestore.operation  = HAL_TDM_CTL_HALT_GLOBAL;

          (*dsiuContext.hal->tdm->halTdmControl)(dsiuContext.tdmPort[i].halId, &tctl);
        }

      }


      /* Test the rest of the blocks. */

      /* Always start with blocks marked as RXNET. This will make the disable of the rx network
       * peripheral happen at the start of the test, instead of at a random point in the test.
       * There is no need to check for messages between calls since the message network path is
       * off. */

      for (i = 0; i < diagCfg.numMemTestBlocks; i++)  {
        if ((testAbort == FALSE)                                           && 
            (dsiu_is_scratch_block (&scratch, &dcfgMemBlock[i]) == FALSE)  &&
            (DCFG_MEMTEST_ATTR_IS_RXNET(dcfgMemBlock[i].flags) == 1)       )  {

          if (dsiu_memtest_context (&dcfgMemBlock[i], &scratch, dctl->control_code, &relocFcn, msg->header.trans_id))  {
            dsiuContext.dregStatus->memTest.status = DREG_MEM_STATUS_DONE;
            testAbort = TRUE;
          }
        }
      }


      /* Check blocks that are not rx net */
      for (i = 0; i < diagCfg.numMemTestBlocks; i++)  {

  
        if ((testAbort == FALSE)                                            && 
            (dsiu_is_scratch_block (&scratch, &dcfgMemBlock[i]) == FALSE)   &&
            (DCFG_MEMTEST_ATTR_IS_RXNET(dcfgMemBlock[i].flags) == 0)        ) {

          if (dsiu_memtest_context (&dcfgMemBlock[i], &scratch, dctl->control_code, &relocFcn, msg->header.trans_id))  {
            dsiuContext.dregStatus->memTest.status = DREG_MEM_STATUS_DONE;
            testAbort = TRUE;
          }

          /* Check for messages after each block */
          dsiuPollTasks();


          /* Check for an abort message */
          if (dsiuContext.dregStatus->memTest.status == DREG_MEM_STATUS_ABORT)  {
            dsiuContext.dregStatus->memTest.status = DREG_MEM_STATUS_DONE;
            testAbort = TRUE;
            break;
          }


        }
      }


      /* Restore the tdm */
      for (i = 0; i < diagCfg.nTdmPorts; i++)  {


        if (DREG_PERIPH_READ_STATUS((dsiuContext.tdmPort[i]).status->idNumStatus) == DREG_PERIPH_STATUS_CFG_ENABLED)  {

          tctl.mode = HAL_TDM_RESTORE_TDM;

          if (dctl->control_code & DMSG_CODE_CTRL_LOCAL)
            tctl.u.haltRestore.operation  = HAL_TDM_CTL_RESTORE_LOCAL;
          else
            tctl.u.haltRestore.operation  = HAL_TDM_CTL_RESTORE_GLOBAL;

          (*dsiuContext.hal->tdm->halTdmControl)(dsiuContext.tdmPort[i].halId, &tctl);
        }

      }

    } 

  } /* end skip internal test */

  /* Test external memory if requested */
  ddrMemBlock.startAddr   = (word *) ((((tulong)dctl->ddr_start_addr_msw) << 16) | dctl->ddr_start_addr_lsw);
  ddrMemBlock.bufLenWords = ((tulong)dctl->ddr_size_msw << 16) | dctl->ddr_size_lsw;
  ddrMemBlock.flags       = 0;

  ddrBlock = ((tulong)dctl->block_size_msw) << 16 | dctl->block_size_lsw;

  dsiu_memtest_direct (&ddrMemBlock, dctl->control_code, msg->header.trans_id, ddrBlock);



  /* Restore cache */
  memCfg.memCfgFunction       = HAL_MEMCFGFUNC_CACHECONFIG;
  memCfg.u.cacheCfg.cacheType = HAL_CACHE_L1D;
  memCfg.u.cacheCfg.cacheMode = diagCfg.memCfg->u.mem64x.l1d;
  (*dsiuContext.hal->halMemConfig)(&memCfg);

  memCfg.memCfgFunction       = HAL_MEMCFGFUNC_CACHECONFIG;
  memCfg.u.cacheCfg.cacheType = HAL_CACHE_L2;
  memCfg.u.cacheCfg.cacheMode = diagCfg.memCfg->u.mem64x.l2;
  (*dsiuContext.hal->halMemConfig)(&memCfg);

  /* Update the status */
  if (dsiuContext.dregStatus->memTest.status == DREG_MEM_STATUS_IN_PROGRESS)  
    dsiuContext.dregStatus->memTest.status = DREG_MEM_STATUS_DONE;




}

/*******************************************************************************************
 * FUNCTION PURPOSE: Top level memory test
 *******************************************************************************************
 * DESCRIPTION: The entry point for the memory tests.
 *              Note that this code can be re-entered, since after each memory block
 *              a message poll is made. This is typically done so status response messages
 *              can be replied to, but a new memory test message can be requested.
 *
 *******************************************************************************************/
void dsiumtst64x (dmsg_t *msg)
{

  tint i;
  dmsg_hd_diag_code_control_t *dctl = &(msg->msg_body.diag_code_control);

  /* Handle memory test stop requests. It is not valid to request a 
   * test halt and a new test, so only the stop bit can be set. */
  if (dctl->control_code & DMSG_CODE_CTRL_STOP_TEST)  {

    if (dctl->control_code & ~DMSG_CODE_CTRL_STOP_TEST)  {
      dsiuErrorIndication (msg->header.trans_id, msg->header.chnum, 
                           MGB_ERR_INVD_MSG_FUNC_NOT_SUPPORTED);
      return;
    }

    /* If there is a test in progress change the status to abort. The actual
     * abort will be handled in the block processign */
    if (dsiuContext.dregStatus->memTest.status == DREG_MEM_STATUS_IN_PROGRESS)
      dsiuContext.dregStatus->memTest.status = DREG_MEM_STATUS_ABORT;

    return;

  }


  /* If there is currently a memory test in progress return an error */
  if (dsiuContext.dregStatus->memTest.status == DREG_MEM_STATUS_IN_PROGRESS)  {
    dsiuErrorIndication (msg->header.trans_id, msg->header.chnum, 
                         MGB_ERR_INVD_MSG_FUNC_NOT_SUPPORTED);
    return;
  }

  /* A new memory test is requested. */
  dsiuContext.dregStatus->memTest.status  = DREG_MEM_STATUS_IN_PROGRESS;
  dsiuContext.dregStatus->memTest.nFailed = 0;

  for (i = 0; i < DREG_MAX_MEM_FAILURES; i++)
    memset (&(dsiuContext.dregStatus->memTest.fail[i]), 0, sizeof (dregMemLoc_t));

  /* Run the memory test */
  dsiu_memtest_top (msg);

} /* dsiumtst64x */








