; *****************************************************************************************************
; * FILE PURPOSE Perform a context switch for memory check
; *****************************************************************************************************
; * FILE NAME: llmt.s
; *
; * DESCRIPTION: A stack switch is performed
; *
; * Copyright (C) 2008, Texas Instruments Incorporated
; * 
; *  Redistribution and use in source and binary forms, with or without 
; *  modification, are permitted provided that the following conditions 
; *  are met:
; *
; *    Redistributions of source code must retain the above copyright 
; *    notice, this list of conditions and the following disclaimer.
; *
; *    Redistributions in binary form must reproduce the above copyright
; *    notice, this list of conditions and the following disclaimer in the 
; *    documentation and/or other materials provided with the   
; *    distribution.
; *
; *    Neither the name of Texas Instruments Incorporated nor the names of
; *    its contributors may be used to endorse or promote products derived
; *    from this software without specific prior written permission.
; *
; *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
; *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
; *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
; *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
; *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
; *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
; *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
; *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
; *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
; *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
; *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
; *
; */


  .cdecls C,LIST
  %{
    #include "llmt.h"
  %}

  .if __TI_EABI__
  ; aliases so rest of code is ABI agnostic
  .asg  llmt_block_copy,        _llmt_block_copy
  .asg  _mem_test_primary_base, __mem_test_primary_base 
  .asg  llmt_context_switch,    _llmt_context_switch
  .endif

; ************************************************************************************************
; * FUNCTION PURPOSE: Perform a switch to the memory test context
; ************************************************************************************************
; * DESCRIPTION: The stack pointer is changed to the memory test stack. 
; *     
; *             On entry the following register assignments exist:
; *               llmt_context_switch (void (*)(word *, tulong, llmtErr_t *), word *, tulong, llmtScratchBlock_t *)
; *               A4 - the function to call when leaving this function
; *               B4 - the address to be tested
; *               A6 - the number of words (bytes) to test
; *               B6 - the scratch structure
; *              
; *
; *    The following values are stored on the old stack:
; *               B3 - the old return value
; *               A10 - used as a scratch during switch
; *     
; *    The following values are saved on the new stack
; *               B15 (old) - the original stack value after adjustment in this function
; *               B4 - the original test block address
; *               A6 - the number of bytes to test
; *               B6 - the scratch structure    
; *
; *
; *    The following values are changed for the call to the memory test function
; *               A4 - the address to test
; *               B4 - the number of words to test
; *               A6 - the error structure stored in sscratch
; *     
; *
; *    The new stack value is taken from sscratch and 8 byte alignment is forced
; *
; ***************************************************************************************************

    .sect ".llmtReloc"
_llmt_context_switch:

    ; Ajust the current stack and save values
    mv    b15,     b21

    ; switch the stack
    ldw   *b6(llmtScratchBlocks_s.sscratchBlk), b17
    nop    5
    ldw   *b17(dcfgMemBlock_t.startAddr), b18
    nop    5
    mv     b18, b15
    addk   llmtb_s.stack, b15


    addk   LLMT_STACKSIZE - 4,     b15
    and    ~7,    b15,  b15

    ; Store the scratch structure on the new stack
    addk  -32,    b15
    stw   b21,   *b15(4)  ; the old stack value
    stw    b4,   *b15(8)  ; the address of the memory block to test
    stw    a6,   *b15(12) ; the number of bytes to test
    stw    b6,   *b15(16) ; the scratch structure
    stw    b3,   *b15(28) ; the original b3
    stw   a10,   *b15(32) ; the original a10

    ; Compute the address of the callout. The run address is calculated since
    ; the code has been relocated
    mvkl __mem_test_primary_base, a10
    mvkh __mem_test_primary_base, a10
    sub  a4, a10, a5
    ldw  *b6(llmtScratchBlocks_s.cscratchBlk), b1
    nop 5
    ldw *b1(dcfgMemBlock_t.startAddr), b1
    nop 5
    add a5, b1, b5


    ; The return address must be placed into b3. The return address
    ; is the context restore function
    mvkl _llmt_context_restore, a4
    mvkh _llmt_context_restore, a4

    sub a4, a10, a5
    add a5, b1, b3

    ; Store the relocated address of the copy block function used for context restore. 
    ; This could be done in the restore section, but the base is already computed
    mvkl _llmt_block_copy, a4
    mvkh _llmt_block_copy, a4
    sub a4, a10, a5
    add a5, b1, b20
    stw b20, *b15(20)

    ; Store the data scratch block start address. This must be saved because
    ; the address is on the stack, which can be wiped during the memory test
    ldw   *b6(llmtScratchBlocks_s.dscratchBlk), b17
    nop    5
    ldw   *b17(dcfgMemBlock_t.startAddr), b20
    nop    5
    stw    b20, *b15(24)


    ; Setup the parameters for the call to the c function to do the memory test
    mv  b4, a4
    mv  a6, b4
    mv  b18, a6
    addk llmtb_s.err, a6

    bnop b5, 5


; **************************************************************************************************************
; * FUNCTION PURPOSE: Restore the original code context
; **************************************************************************************************************
; * DESCRIPTION: The original context is restored. Prior to the restore the original data is copied
; *              from the data scratch to the test block. This must be done in this context
; *              because the test itself may have wiped the old context.
; **************************************************************************************************************
_llmt_context_restore:

  ; Copy the scratch data block to the block under test
  ldw  *b15(16), b21  ; Pointer to the scratch structure 

  ldw  *b15(8),  b4   ; The destination address is the address of the block under test 
  ldw  *b15(12), a6   ; The number of bytes to copy */
  ldw  *b15(24), a4   ; The source address for the restore was saved on context switch
  ldw  *b15(20), b5   ; The relocated llmt_copy_block_function

  nop  5

  b  b5
  addkpc crest, b3, 4

crest:
  ; Restore the original stack and context
  ldw *b15(4),  b23   ; The original stack pointer
  ldw *b15(28), b3    ; The original b3
  ldw *b15(32), a10   ; The original a10  

  nop 5
  mv b23, b15


  bnop b3, 5


