/***************************************************************************************************
 * FILE PURPOSE: Low level relocatable memory tests
 ***************************************************************************************************
 * FILE NAME: llmtreloc.c
 *
 * DESCRIPTION: Peform memory tests with context save. All of the functions in this file
 *              should be placed into the .llmtReloc section. All function except the entry
 *              functions must be declared near to make sure that the code can be
 *              relocated.
 *
 * Copyright(C) 2008, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include "types.h"
#include "llmt.h"
#include "diagreg.h"

/***************************************************************************************************
 * FUNCTION PURPOSE: A simply memory copy
 ***************************************************************************************************
 * DESCRIPTION: The standard memcpy function can't be used since the copy itself may wipe the
 *              memcpy function.
 ***************************************************************************************************/
#pragma CODE_SECTION(llmt_block_copy, ".llmtReloc")
near void llmt_block_copy (word *src, word *dst, tulong lenWords)
{
  tulong i;
  word * restrict srcr = src;
  word * restrict dstr = dst;

  for (i = 0; i < lenWords; i++)
    dstr[i] =  srcr[i];

} /* llmt_block_copy */

/****************************************************************************************************
 * FUNCTION PURPOSE: Fill memory with a fixed 16 bit pattern
 ****************************************************************************************************
 * DESCRIPTION: A simple 16 bit memory fill.
 ****************************************************************************************************/
#pragma CODE_SECTION(llmt_16bit_pattern_set, ".llmtReloc")
near void llmt_16bit_pattern_set (word *addr, tulong len16bit, tuint v)
{
  tulong i;
  tuint *addr16 = (tuint *)addr;

  for (i = 0; i < len16bit; i++)
    addr16[i] = v;

} /* llmt_16bit_pattern_set */


/****************************************************************************************************
 * FUNCTION PURPOSE: Return the value of a memory bit
 ****************************************************************************************************
 * DESCRIPTION: A single bit value in a bit stream is checked. The bit stream is done byte by byte
 *              with the lsb the first bit in a byte.
 ****************************************************************************************************/
#pragma CODE_SECTION(llmt_memoryBit, ".llmtReloc")
near tint llmt_memoryBit (word *addr, tulong abit)
{
  tulong byte;
  tulong bit;
  unsigned char v;

  /* Locate the specific byte and bit in a byte stream */
  byte = abit >> 3;
  bit  = abit & 0x7;

  v = addr[byte];
  v = (v >> bit) & 1;

  return (v);

} /* llmt_memoryBit */

/****************************************************************************************************
 * FUNCTION PURPOSE: Report an error during marching 0 test
 ****************************************************************************************************
 * DESCRIPTION: The error stats are updated. Returns non-zero if the error report has reached
 *              the report limit.
 ****************************************************************************************************/
#pragma CODE_SECTION(llmt_march_error, ".llmtReloc")
near tint llmt_march_error (word *addr, tulong abit, llmtErr_t *err, tint eval)
{
  tulong  v;
  tulong  word32;
  tulong  bit32;
  tulong *laddr = (tulong *)addr;

  /* Locate the nearest 32 bit word that is 4 byte aligned and contains the error byte */
  word32 = abit >> 5;  
  bit32  = abit & 0x1f;
  v      = laddr[word32];

  err->errAddr[err->errCount]   = (tulong)addr + (word32 << 2);  /* Convert the 32 bit word address to byte address */

  if (eval == 0)
    err->errExptd[err->errCount]  = v & ~(1 << bit32);  /* Was expecting a 0 in this bit */
  else
    err->errExptd[err->errCount]  = v | (1 << bit32);   /* Was expecting a 1 in this bit */

  err->errActual[err->errCount] = v;

  err->errCount += 1;

  if (err->errCount >= DREG_MAX_MEM_FAILURES)
    return (-1);

  return (0);

} /* llmt_march_error */
  


/***************************************************************************************************
 * FUNCTION PURPOSE: Set a bit in a bitstream
 ***************************************************************************************************
 * DESCRIPTION: A single bit value is set
 ***************************************************************************************************/
#pragma CODE_SECTION(llmt_memorySetBit, ".llmtReloc")
near void llmt_memorySetBit (word *addr, tulong abit, tint bval)
{
  tulong byte;
  tulong bit;
  unsigned char v;

  /* Locate the specific byte and bit in a byte stream */
  byte = abit >> 3;
  bit  = abit & 0x7;

  v = addr[byte];

  if (bval == 0)
    v = v & (~(1 << bit));
  else
    v = v | (1 << bit);

   addr[byte] = v;


} /* llmt_memorySetBit */



/****************************************************************************************************
 * FUNCTION PURPOSE: Perform a marching 0s/1s pattern test
 ****************************************************************************************************
 * DESCRIPTION: A basic marching 0s/1s test is performed over the entire block
 ****************************************************************************************************/
#pragma CODE_SECTION(llmt_marching, ".llmtReloc")
near void llmt_marching (word *addr, tulong lenWords, llmtErr_t *err)
{
  tulong i, j;
  tulong memSizeBits;

  
  err->errCount = 0;

  /* The memory size in bits. For very large blocks there could be 
   * an overflow */
  memSizeBits = lenWords * 8;

  llmt_16bit_pattern_set (addr, lenWords >> 1, 0);

  /* Marching 1 first */
  for (i = 0; i < memSizeBits; i++)  {
    if (llmt_memoryBit (addr, i) != 0)  {
      if (llmt_march_error (addr, i, err, 0))
        return;
    }  else
      llmt_memorySetBit (addr, i, 1);
  }

  /* Marching 0 second */
  for (i = 0; i < memSizeBits; i++)  {
    j = memSizeBits - i - 1;
    if (llmt_memoryBit (addr, j) != 1)  {
       if (llmt_march_error(addr, j, err, 1))
         return;
    }  else
      llmt_memorySetBit (addr, j, 0);
  } 

} /* llmt_marching */


/****************************************************************************************************
 * FUNCTION PURPOSE: Marching 0s/1s test
 ****************************************************************************************************
 * DESCRIPTION: Test a memory block using the marching 0s/1s. The block under test is
 *              copied to the scratch block, a context switch is made to the scratch
 *              data/stack, then the memory is tested. After the test the original memory
 *              contents are restored.
 ****************************************************************************************************/
#pragma CODE_SECTION(llmtMarching, ".llmtReloc")
void llmtMarching (word *addr, tulong lenWords, llmtScratchBlock_t *scratch)
{
  
  /* Copy the block to scratch. Note that the block sizes are not checked, this
   * must be done outside this function */
  llmt_block_copy (addr, scratch->dscratchBlk->startAddr, lenWords);

  /* Perform a context switch and branch to the actual test code. The function
   * is an assembly function. */
  llmt_context_switch (llmt_marching, addr, lenWords, scratch);

  /* The restore of the test block is done in the other stack context. This
   * is required because the test may have wiped out the stack that this function
   * was using. This is setup in the context switch */

} /* llmtMarching */


/*****************************************************************************************************
 * FUNCTION PURPOSE: Record an error in the fixed pattern test
 *****************************************************************************************************
 * DESCRIPTION: The error is recorded. Returns non-zero if the error recorder is full
 *****************************************************************************************************/
#pragma CODE_SECTION(llmt_fixedpat_err, ".llmtReloc")
near tint llmt_fixedpat_err (word *addr, tulong lenWords, llmtErr_t *err, tulong idx, word v)
{
  tulong  word32;
  tulong *laddr = (tulong *)addr;

  /* Locate the nearest 32 bit word that is 4 byte aligned and contains the error byte */
  word32 = idx >> 2;  

  err->errAddr[err->errCount]   = (tulong)addr + (word32 << 2);  /* Convert the 32 bit word address to byte address */
  err->errExptd[err->errCount]  = ((tulong)v << 24) | (v << 16) | (v << 8) | v;
  err->errActual[err->errCount] = laddr[word32];

  err->errCount += 1;

  if (err->errCount >= DREG_MAX_MEM_FAILURES)
    return (-1);

  return (0);

} /* llmt_fiedpat_err */




/*****************************************************************************************************
 * FUNCTION PURPOSE: Perform a fixed byte pattern test
 *****************************************************************************************************
 * DESCRIPTION: A basic fixed pattern test is performed
 *****************************************************************************************************/
#pragma CODE_SECTION(llmt_fixedpat_test, ".llmtReloc")
near tint llmt_fixedpat_test (word *addr, tulong lenWords, llmtErr_t *err, word v)
{
  tulong i;

  /* Write the pattern */
  for (i = 0; i < lenWords; i++)
    addr[i] = v;

  /* Verify the pattern */
  for (i = 0; i < lenWords; i++)  {
    if (addr[i] != v)  {
     if (llmt_fixedpat_err (addr, lenWords, err, i, v))
       return (-1);
    }
  }

  return (0);

} /* llmt_fixed_pat_test */


/*****************************************************************************************************
 * FUNCTION PURPOSE: Peform the aa/55 memory test
 *****************************************************************************************************
 * DESCRIPTION: The alternating bit test is performed
 *****************************************************************************************************/
#pragma CODE_SECTION(llmt_aa55, ".llmtReloc")
near void llmt_aa55(word *addr, tulong lenWords, llmtErr_t *err)
{

  if (llmt_fixedpat_test (addr, lenWords, err, 0xaa))
    return;

  llmt_fixedpat_test (addr, lenWords, err, 0x55);

} /* llmt_aa55 */


/*****************************************************************************************************
 * FUNCTION PURPOSE: Alternating bit test
 *****************************************************************************************************
 * DESCRIPTION: The attern 0xaa is written to memory, then verified. Then 0x55 is written and 
 *              verified.
 *****************************************************************************************************/
#pragma CODE_SECTION(llmtAA55, ".llmtReloc")
void llmtAA55 (word *addr, tulong lenWords, llmtScratchBlock_t *scratch)
{

  /* Copy the block to scratch. Note that the block sizes are not checked, this
   * must be done outside this function */
  llmt_block_copy (addr, scratch->dscratchBlk->startAddr, lenWords);

  /* Perform a context switch and branch to the actual test code. The function
   * is an assembly function. */
  llmt_context_switch (llmt_aa55, addr, lenWords, scratch);

  /* The restore of the test block is done in the other stack context. This
   * is required because the test may have wiped out the stack that this function
   * was using. This is setup in the context switch */
}


/*****************************************************************************************************
 * FUNCTION PURPOSE: Report bit errors for the pseudo random test
 *****************************************************************************************************
 * DESCRIPTION: Bit error report for the pseudo random test
 *****************************************************************************************************/
#pragma CODE_SECTION(llmt_pseudo_err, ".llmtReloc")
tint llmt_pseudo_err (word *addr, tulong lenWords, llmtErr_t *err, tulong i, tulong expVal)
{
  tulong *laddr = (tulong *)addr;

  err->errAddr[err->errCount]   = (tulong) (&(laddr[i]));
  err->errExptd[err->errCount]  = expVal;
  err->errActual[err->errCount] = laddr[i];

  err->errCount += 1;

  if (err->errCount >= DREG_MAX_MEM_FAILURES)
    return (-1);

  return (0);

} /* llmt_pseudo_err */


/*****************************************************************************************************
 * FUNCTION PURPOSE: Pseudo random sequence memory test
 *****************************************************************************************************
 * DESCRIPTION: The pseudo random pattern is generated for test
 *****************************************************************************************************/
#define BIT(v,b)  (((v) >> b) & 0x1)
#pragma CODE_SECTION(llmt_pseudo, ".llmtReloc")
near void llmt_pseudo (word *addr, tulong lenWords, llmtErr_t *err)
{
  tulong *laddr = (tulong *)addr;
  tulong  genVal;
  tulong  newBit;
  tulong  i;
  tuint   j;

  /* Convert the byte count to a word count */
  lenWords = lenWords >> 2;

  genVal = DSIU_RANDOM_SEED();

  for (i = 0; i < lenWords; i++)  {

    for (j = 0; j < 32; j++)  {
      newBit = BIT(genVal,30) ^ BIT(genVal,2);
      genVal = (genVal << 1) | newBit;
    }

    laddr[i] = genVal;

  }

  /* Check the pattern */
  genVal = laddr[0];
  for (i = 1; i < lenWords; i++)  {

    for (j = 0; j < 32; j++)  {
      newBit = BIT(genVal,30) ^ BIT(genVal,2);
      genVal = (genVal << 1) | newBit;
    }

    if (laddr[i] != genVal)  {
      if (llmt_pseudo_err (addr, lenWords, err, i, genVal))
        return;
    }
  }

} /* llmt_pseudo */



 
#pragma CODE_SECTION(llmtPseudo, ".llmtReloc")
void llmtPseudo (word *addr, tulong lenWords, llmtScratchBlock_t *scratch)
{
  /* Copy the block to scratch. Note that the block sizes are not checked, this
   * must be done outside this function */
  llmt_block_copy (addr, scratch->dscratchBlk->startAddr, lenWords);

  /* Perform a context switch and branch to the actual test code. The function
   * is an assembly function. */
  llmt_context_switch (llmt_pseudo, addr, lenWords, scratch);

  /* The restore of the test block is done in the other stack context. This
   * is required because the test may have wiped out the stack that this function
   * was using. This is setup in the context switch */
}

/*****************************************************************************************************
 * FUNCTION PURPOSE: Record an error in the address test
 *****************************************************************************************************
 * DESCRIPTION: The address error is recorded
 *****************************************************************************************************/
#pragma CODE_SECTION(llmt_addr_error, ".llmtReloc")
near int llmt_addr_error (word *addr, tulong lenWords, llmtErr_t *err, tulong i, bool inverse)
{
  
  tulong  vact;
  tulong  vexp;
  tuint   v1;
  tuint   v2;
  tuint  *waddr = (tuint *)addr;

  if (i & 1)  {
    vact = (((tulong)waddr[i-1]) << 16) | waddr[i]; 
    v1 = ((tulong)(waddr + i - 1)) & 0xffff;
    v2 = ((tulong)(waddr + i - 0)) & 0xffff;
    vexp = (((tulong)(v1)) << 16) | (tulong)(v2);
    
  }  else  {
    vact = (((tulong)waddr[i]) << 16) | waddr[i+1];
    v1 = ((tulong)(waddr + i + 0)) & 0xffff;
    v2 = ((tulong)(waddr + i + 1)) & 0xffff;
    vexp = (((tulong)(v1)) << 16) | (tulong)(v2);
  }

  if (inverse)
    vexp = ~vexp;
  
  err->errAddr[err->errCount]   = (tulong)addr + (i << 1);  /* Convert the word address to byte address */
  err->errExptd[err->errCount]  = vexp;
  err->errActual[err->errCount] = vact;

  err->errCount += 1;

  if (err->errCount >= DREG_MAX_MEM_FAILURES)
    return (-1);

  return (0);

} /* llmt_addr_error */




/*****************************************************************************************************
 * FUNCTION PURPOSE: Perform the address / address inverse test
 *****************************************************************************************************
 * DESCRIPTION: The 16 bit address of each word is written and verified, followed by the bit
 *              inverse of the address.
 *****************************************************************************************************/
#pragma CODE_SECTION(llmt_addr, ".llmtReloc")
near void llmt_addr (word *addr, tulong lenWords, llmtErr_t *err)
{
  tuint   *waddr = (tuint *)addr;
  tulong   wLen  = lenWords >> 1;
  tulong   i;
  tuint    v;

  /* write address */
  for (i = 0; i < wLen; i++)  {
    v = ((tulong)(waddr + i)) & 0xffff;
    waddr[i] = v;
  }

  /* Verify address */
  for (i = 0; i < wLen; i++)  {
    v = ((tulong)(waddr + i)) & 0xffff;
    if (waddr[i] != v)  {
      if (llmt_addr_error (addr, lenWords, err, i, FALSE))
        return;
    }
  }

  /* write address inverse */
  for (i = 0; i < wLen; i++)  {
    v = ((tulong)(waddr + i)) & 0xffff;
    waddr[i] = ~v;
  }

  /* Verify address inverse */
  for (i = 0; i < wLen; i++)  {
    v = ((tulong)(waddr + i)) & 0xffff;
    v = ~v;
    if (waddr[i] != v)  {
      if (llmt_addr_error (addr, lenWords, err, i, TRUE))
        return;
    }
  }

} /* llmt_addr */



 
#pragma CODE_SECTION(llmtAddr, ".llmtReloc")
void llmtAddr (word *addr, tulong lenWords, llmtScratchBlock_t *scratch)
{

  /* Copy the block to scratch. Note that the block sizes are not checked, this
   * must be done outside this function */
  llmt_block_copy (addr, scratch->dscratchBlk->startAddr, lenWords);

  /* Perform a context switch and branch to the actual test code. The function
   * is an assembly function. */
  llmt_context_switch (llmt_addr, addr, lenWords, scratch);

  /* The restore of the test block is done in the other stack context. This
   * is required because the test may have wiped out the stack that this function
   * was using. This is setup in the context switch */

}
 

#pragma CODE_SECTION(llmtProg, ".llmtReloc")
void llmtProg (word *addr, tulong lenWords, llmtScratchBlock_t *scratch)
{
  /* empty */
}

/******************************************************************************************
 * The following functions are used for direct memory test - without data save
 * and context switch. They are just redirection functions placed in the
 * .llmtReloc section so they can be called from any space.
 ******************************************************************************************/
#pragma CODE_SECTION(llmtMarchingContinue, ".llmtReloc")
void llmtMarchingContinue (word *addr, tulong len, llmtErr_t *err)
{
  llmt_marching (addr, len, err);
}

#pragma CODE_SECTION(llmtAA55Continue, ".llmtReloc")
void llmtAA55Continue (word *addr, tulong len, llmtErr_t *err)
{
  llmt_aa55 (addr, len, err);
}

#pragma CODE_SECTION(llmtPseudoContinue, ".llmtReloc")
void llmtPseudoContinue (word *addr, tulong len, llmtErr_t *err)
{
  llmt_pseudo (addr, len, err);
}

#pragma CODE_SECTION(llmtAddrContinue, ".llmtReloc")
void llmtAddrContinue (word *addr, tulong len, llmtErr_t *err)
{
  llmt_addr (addr, len, err);
}

#pragma CODE_SECTION(llmtProgContinue, ".llmtReloc")
void llmtProgContinue (word *addr, tulong len, llmtErr_t *err)
{
  /* empty */
}


 
