/******************************************************************************
 * FILE PURPOSE: main - starts the DSP functionality.
 ******************************************************************************
 * FILE NAME:  ddsp.c  
 *
 * DESCRIPTION:  This files contains the main function necessary to start the
 *               siu to the right after download state. 
 * TABS: NONE
 *
 * Initial revision Shiju Sivasankaran
 * REVISION HISTORY:
 *
 * (C) Copyright 2003, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include "types.h"
#include "micdspdp.h"
#include "dsp.h"
#include "dsiuloc.h"
#include "diagcfg.h"

/******************************************************************************
 * FUNCTION PURPOSE: Write 16 bit value into 16 bit word (macro version)
 ******************************************************************************
 * DESCRIPTION: Writes 16 bit value into 16 bit word.  No assumptions
 * 
 * void utlWrite16bits_m (
 *    word  *base,  - Base of byte array
 *    tuint byte,   - byte offset to write; assumed to be even
 *    tuint val)    - 16 bit val
 * 
 *****************************************************************************/
void utlWrite16bits_m (word *base, tuint byte, tuint val) 
{
  char *wptr = ((char *)base + byte);

  /* Shift/mask is endian-portable, but look out for stupid compilers */
  if (base == NULL)
	  return;
  wptr[0] = (char)(val>>8);
  wptr[1] = (char)(val & 0xff);
} /* utlWrite16bits_m */

/***************************************************************************
 * Get DSP image map details.
 * Calling side fills in the num_params (in 16bit words) and passes a 
 * pointer to the location where it wants the called side to copy the params.
 * Called side should copy as many params as it has upto max of num_params
 * into the area pointed to by the buffer. The return value contains the
 * actual number of params copied if < num_params or it returns the actual
 * number of params available if > num_params.
 **************************************************************************/
INT32 dspGetImageMap(UINT16  num_params, UINT16* p_params)
{
  tint i;
  tint max_num_parms = 32; /* update this with any future modifications */ 
//  extern tuint dspImageMap[];

  if (num_params <= max_num_parms) {
  /* copy the entire DSP image data.  This is hardcoded to 32 */
    for (i = 0;i < num_params;i++) {
//      utlWrite16bits_m ((word *)p_params,0,dspImageMap[i]);
      p_params++;
	}
    return (max_num_parms);
  }
  else return (NULL);
}

/****************************************************************************
 * Function:    dsp_get_interface()
 ****************************************************************************
 * Description:
 *  Provides DSP inputs to outside modules.
 * 
 * RETURNS:
 *  - Interface table pointer if available.
 *  - NULL otherwise.
 *
 ***************************************************************************/
void *dspGetInterface(DSP_IF_ID_T if_id)
{
  switch(if_id) {
     case DSP_FROM_REQ_IF:
       return(&dsp_cli_if);
     default:
       return(NULL);
  }
}

/***************************************************************************
 * FUNCTION:    dspGetSizes
 ***************************************************************************
 * Description: This returns the memory requirement of DSP.
 *
 * Parameters:
 *
 * return:      DSP_RESULT_OK: Successful operation.
 ***************************************************************************/
DSP_RESULT_T dspGetSizes(UINT32 *p_mem_req)
{
  p_mem_req = (UINT32) DSP_MEM_BLOCK_SIZE;
  return(DSP_RESULT_OK);
}

/***************************************************************************
 * FUNCTION:    dspMemInit
 ***************************************************************************
 * Description: This passes the DSP required memory information to the dsp.
 * 
 *
 * Parameters:
 *
 * return:      DSP_RESULT_OK: Successful operation.
 ***************************************************************************/
void *hpiMemBaseAddress (word *base, tulong hpi_offset);
DSP_RESULT_T dspMemInit(UINT32 *base_ptr, UINT32 **p_hpi_base)
{

  /* Initilize the DSP mem base, allocated by micro */
  word *dspMemBase = (word *) base_ptr;  
  tulong hpi_offset= 0x8000;
  /**************************************************************************
   * Initialize heap memory control
   **************************************************************************/
  diagCfg.bootHeapBufStart = (word *)dspMemBase;
  diagCfg.bootHeapBufEnd   = (word *)dspMemBase + 0x3000;

  diagCfg.diagHeapBufStart = (word *)dspMemBase + 0x3000;
  diagCfg.diagHeapBufEnd   = (word *)dspMemBase + 0x7000;
  
  diagCfg.halHeapBufStart  = (word *)dspMemBase + 0x7000;
  diagCfg.halHeapBufEnd    = (word *)dspMemBase + 0x7C00;
  
  *p_hpi_base = hpiMemBaseAddress(dspMemBase, hpi_offset);

  return(DSP_RESULT_OK);
}
/****************************************************************************
 * FUNCTION PURPOSE: main routine - exceutes after DSP Startup
 *
 * First DSP function, which initiate the SIU and further DSP processing. This
 * is moved from siuinit.c to support the Core 10.2, whose main is different 
 * from the convensional DSP implementation. 
*****************************************************************************/
DSP_RESULT_T dspInit(DSP_TO_CLIENT_IF_T *p_dsp_to_client_if,
              UINT16 *bsp_tx_ptr, UINT16 *bsp_rx_ptr)
{
  /* Setting the BSP tx and rx buffers */
  diagCfg.dmaTxBufAStart   = (word *)bsp_tx_ptr;
  diagCfg.dmaTxBufAEnd     = (word *)bsp_tx_ptr + 0x200;
  diagCfg.dmaRxBufAStart   = (word *)bsp_rx_ptr;
  diagCfg.dmaRxBufAEnd     = (word *)bsp_rx_ptr + 0x200;  
  /* start the SIU */
  siuMain();
  return(DSP_RESULT_OK);

}  /* dspInit */

/*****************************************************************************
 * DSP function,which initiate the task processing. This is is moved from 
 * siumain.c to support the Core 10.2, which is slightly different from the 
 * convensional DSP implementations.
*****************************************************************************/
DSP_RESULT_T dspTaskScheduler(void)
{
  siuTaskScheduler();
  return(DSP_RESULT_OK);
}

/***************************************************************************
 * FUNCTION:    halTdmDmaIsr
 ***************************************************************************
 * Description: This function is used as a trigger to provide and 
 *              get back TDM data from the dsp. The data is passed in
 *              buffers that are passed to dsp in the (*init) call.
 *
 * Parameters:
 *
 * return:      DSP_RESULT_OK: Successful operation.
 ***************************************************************************/
DSP_RESULT_T dspTdmDmaIsr(void)
{
  halTdmDmaIsr();
  return(DSP_RESULT_OK);

}

/*****************************************************************************
 * DSP From Client (Micro code) interface
 ****************************************************************************/
DSP_FROM_CLIENT_IF_T   dsp_cli_if = 
{
  dspGetSizes,
  dspMemInit,
  dspInit,
  dspTdmDmaIsr,
  dspTaskScheduler,
};

/* nothing after this point */


