/******************************************************************************
 * FILE PURPOSE: Defines the memory block used to control diag
 ******************************************************************************
 * FILE NAME: diagreg.c
 *
 * DESCRIPTION: Defines a structure which is used to control the operation of
 *              diag.
 *
 * TABS: None
 *
 * (C) Copyright 2002, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include "types.h"
#include "diagreg.h"
#include "dspver.h"
#include "hal.h"
#include "dsiuloc.h"
#include "diagcfg.h"
#include "dsiuutop.h"
#include "cmisc.h"
#include "utlbitfield.h"
#include "utlrepck.h"
#include "dsiuotbrun.h"

#include <string.h>

/* Flag differentiates the voice configuration from the message
 * configuration in dsiu_net_config_diagapp. 
 * voice = flag 0, message = flag 1  */

#define DSIU_CONFIG_VOICE       0            
#define DSIU_CONFIG_MSG         1         

#define DSIU_TDM_MAP_MAX_2048_TIMESLOTS
#define DSIU_TDM_TSLOT_PER_LINK                     256
#define DSIU_TDM_MAP_MAX_2048_MAX_CORES             8
#define DSIU_TDM_MAP_MAX_2048_NUM_CORE_PER_PORT     4
#define DSIU_TDM_MAP_MAX_2048_NUM_TDM_PORTS         2

#if (defined(DSIU_SIMULATE_APPLICATION) || defined(DSIU_DAUGHTER_TEST))
/*******************************************************************************
 * FUNCTION PURPOSE: Setup the initial tdm configuration.
 *******************************************************************************
 * DESCRIPTION: The default tdm configuration divides the max number of
 *              timeslots between all available ports. The timeslots
 *              are not enabled here, just the initial map is setup
 *******************************************************************************/
void dsiuDefaultTdmConfig (tuint core_id)
{
  tint i, j, p;
  tint tsPerPort;
  tint startTs;
  halQuery_t query;

#ifdef DSIU_TDM_MAP_MAX_2048_TIMESLOTS
    /* Mapping of Timeslot in the case of 
    * more than 4 cores, 2 TSIP port and 2 TSIP links each supporting
    * 512 timeslots. Mapping is as follows for an 8 core device:
    *               -------------------------------------------
    *               TSIP Port      Link         Time slot range
    *               -------------------------------------------
    *  Core 0       0               0           0 - 255
    *  Core 1       0               0           256 -511
    *  Core 2       0               1           0 - 255
    *  Core 3       0               1           256 -511
    *  Core 4       1               0           0 - 255
    *  Core 5       1               0           256 -511
    *  Core 6       1               1           0 - 255
    *  Core 7       1               1           256 -511
    */

    if(diagCfg.nTdmPorts == 0)
    {
        return;
    }

    query.periph = HAL_QUERY_NCORES;
    (*dsiuContext.hal->halQuery) (&query);
    if((query.u.core.nCores > DSIU_TDM_MAP_MAX_2048_MAX_CORES) 
       || ((query.u.core.nCores > DSIU_TDM_MAP_MAX_2048_NUM_CORE_PER_PORT) && (diagCfg.nTdmPorts < DSIU_TDM_MAP_MAX_2048_NUM_TDM_PORTS))
       || (diagCfg.maxChannels > DSIU_TDM_TSLOT_PER_LINK))
    {
      /* Die as this mapping will not work */
      /* Number of links configuration currently no check */
      while(1);
    }
    
    query.u.ts.port = 0;
    if(core_id >= DSIU_TDM_MAP_MAX_2048_NUM_CORE_PER_PORT)
    {
        query.u.ts.port = 1;
        core_id = (core_id - DSIU_TDM_MAP_MAX_2048_NUM_CORE_PER_PORT);
    }

    query.u.ts.link = 0;
    startTs = 0;
    switch(core_id)
    {
        case 0:
            /* Link and start TS = 0 */
            break;

        case 1:
            {
                 query.u.ts.link = 0;
                 startTs = DSIU_TDM_TSLOT_PER_LINK;
                 break;
            }
        case 2:
            {
                query.u.ts.link = 1;
                startTs = 0;
                break;
            }
        case 3:
            {
                 query.u.ts.link = 1;
                 startTs = DSIU_TDM_TSLOT_PER_LINK;
                 break;
            }
        default:
            {
                /* die */
                while(1);
                break;
            }
    }

    for (i = 0; i < diagCfg.maxChannels; i++)  
    {
        query.periph    = HAL_QUERY_TS_ENCODE;
        query.u.ts.ts   = startTs + i;
        (*dsiuContext.hal->halQuery) (&query);
        dsiuContext.dregTdmCfg[i].txTimeslot = query.u.ts.port_link_ts;
        dsiuContext.dregTdmCfg[i].rxTimeslot = query.u.ts.port_link_ts;
        dsiuContext.dregTdmCfg[i].compand    = DREG_TDM_COMPAND_ULAW;
        dsiuContext.dregTdmCfg[i].control    = DREG_TDM_CONTROL_ENABLE_TS;
        dsiuContext.dregTdmCfg[i].status     = DREG_TDM_STATUS_DISABLED;

        dsiuContext.dregTdmCfgShadow[i] = dsiuContext.dregTdmCfg[i];
    }

#else
   
    tsPerPort = diagCfg.maxChannels / diagCfg.nTdmPorts;
    startTs = core_id * tsPerPort;
    for (i = p = 0; i < diagCfg.nTdmPorts; i++)  {
        for (j = 0; j < tsPerPort; j++, p++)  {
          query.periph    = HAL_QUERY_TS_ENCODE;
          query.u.ts.port = i;
          query.u.ts.link = 0;
          query.u.ts.ts   = startTs + j;
          (*dsiuContext.hal->halQuery) (&query);

          dsiuContext.dregTdmCfg[p].txTimeslot = query.u.ts.port_link_ts;
          dsiuContext.dregTdmCfg[p].rxTimeslot = query.u.ts.port_link_ts;
          dsiuContext.dregTdmCfg[p].compand    = DREG_TDM_COMPAND_ULAW;
          dsiuContext.dregTdmCfg[p].control    = DREG_TDM_CONTROL_ENABLE_TS;
          dsiuContext.dregTdmCfg[p].status     = DREG_TDM_STATUS_DISABLED;

          dsiuContext.dregTdmCfgShadow[p] = dsiuContext.dregTdmCfg[p];
        }
    }
#endif
  

} /* dsiuDefaultTdmConfig */
#endif

/******************************************************************************
 * FUNCTION PURPOSE: Adds a value to a mac address
 ******************************************************************************
 * DESCRIPTION: A 48 bit unsigned sum is performed
 ******************************************************************************/
void dsiu_add_tomac (tuint *hdr, tuint sm)
{
  tuint newhdr[3];

  newhdr[2] = hdr[2] + sm;
  newhdr[1] = hdr[1];
  newhdr[0] = hdr[0];

  if (newhdr[2] < hdr[2])
    newhdr[1] += 1;

  if (newhdr[1] < hdr[1])
    newhdr[0] += 1;

  hdr[0] = newhdr[0];
  hdr[1] = newhdr[1];
  hdr[2] = newhdr[2];

} /* dsiu_add_tomac */

/******************************************************************************
 * FUNCTION PURPOSE: Configure a voice/message channel instance
 ******************************************************************************
 * DESCRIPTION: The voice/message network configuration parameters
 *              are configured based on the values received in the
 *              diagApp structure stored by the boot loader.
 *              Flag differentiates the voice configuration from the message
 *              configuration. voice = flag 0, message = flag 1
 ******************************************************************************/
diagAppHeader_t *dsiu_net_config_diagapp (tint core_id, dregNetCfg_t *netCfg, tuint flag)
{
  diagAppHeader_t *a;
  tint encap;
  tint count;

  /* Initialize the header copy */
  memset (netCfg, 0, sizeof(dregNetCfg_t));

  /* Determine the routing to use for initial configuration */
  netCfg->route = dsiuContext.diagApp.route;
  switch (dsiuContext.diagApp.route)  {

    case DIAGAPP_ROUTE_UTOPIA:
      a = &(dsiuContext.diagApp.u.utopia.msg);
      dsiuContext.siuNduContext.cellSize = dsiuContext.diagApp.u.utopia.cellSize;
      break;

#ifdef NDU_GMAC
    case DIAGAPP_ROUTE_GMAC:
      a = &(dsiuContext.diagApp.u.mac.msg);
      break;
#endif
    /* For messages through HPI, configure for utopia if present, otherwise
     * use mac */
    case DIAGAPP_ROUTE_DPRAM:
      if (diagCfg.ctable.siuUtNdu)  {
        dsiuContext.siuNduContext.cellSize = dsiuContext.diagApp.u.utopia.cellSize;
        a = &(dsiuContext.diagApp.u.utopia.msg);

        if (flag != DSIU_CONFIG_MSG)
          netCfg->route = DIAGAPP_ROUTE_UTOPIA;
        else
          netCfg->route = DIAGAPP_ROUTE_DPRAM;

      } 
#ifdef NDU_GMAC
      else if (diagCfg.ctable.siuGmacNdu)  {
        a = &(dsiuContext.diagApp.u.mac.msg);
        netCfg->route = DIAGAPP_ROUTE_GMAC;
      }
#endif
      else
        a = NULL;

      break;

  } /* end switch */


  /* Infer the encapusulation type based on the offsets. If the ip, udp, and
   * ethernet offsets are all negative, assume aal1 encapsulation */
  if (a)  {

    if ( (a->ipOffsetBytes  < 0)  &&
         (a->udpOffsetBytes < 0)  &&
         (a->ethOffsetBytes < 0) )  {

      encap = DREG_NET_ENCAP_AAL1;

    } else {
      
      encap = DREG_NET_ENCAP_RAW;

    }

  } else  {

    encap = DREG_NET_ENCAP_NO_VOICE;

  }

  /* All of the channels will be initialized with the same header information,
   * with the expeption of the lowest nibble of the mac address and possibly 
   * the UDP source/destination ports. Create a master copy of the configuration 
   * then replicate to all the channels */
  netCfg->encap = encap;
  netCfg->port  = dsiuContext.diagApp.portNum;

  if (encap != DREG_NET_ENCAP_NO_VOICE)  {

    /* Copy the transmit header */
    netCfg->tx_total_hdr_len_bytes = a->totalHeaderLenBytes;
    memcpy (netCfg->header, a->hdrImage, SIU_BYTES_TO_WORDS(netCfg->tx_total_hdr_len_bytes));


    /* Copy the flags */
    netCfg->cfg_flag1 = a->flags1;
    netCfg->cfg_flag2 = a->flags2;

    /* The alternate port is unused in initial config */
    netCfg->alt_local_port  = 0;
    netCfg->alt_remote_port = 0;
    count = 0;
    while ((count < DMSG_IP_ADDR_MAX_NUM_TUINT) && (count < DREG_IP_ADDR_MAX_NUM_TUINT) ){
      if (count == 1)
        netCfg->alt_remote_addr[count] = a->priDiffTag;
      else
        netCfg->alt_remote_addr[count] = 0;
      count++;
    }

    /* Configure the PDT format */
    /* For Message the source channel identifier is same as destination channel identifier */
    /* Derive the source channel identifier from the tx header */
    if (dsiuContext.diagApp.tag_desc)
    {
      tuint  tag_len_b;
      tuint  pdt_len_bits;
      tuint  pdt_byte_offset;
      tuint  pdt_bit_offset;   
      tuint  lkup_tag;
      tint   lkup_tag_len_b;
      tuint  lkup_tag_1, lkup_tag_2;
      tuint  lkup_tag_len_1;   
      
      pdt_len_bits = neu_EXT_PDT_FIELD_LEN(a->priDiffTag);
      pdt_byte_offset = neu_EXT_PDT_FIELD_BYTE_OFFSET(a->priDiffTag);
      pdt_bit_offset = neu_EXT_PDT_FIELD_BIT_OFFSET(a->priDiffTag);
      tag_len_b = neu_EXT_PDT_FIELD_TAG_LENGTH(a->priDiffTag);
      lkup_tag_len_b = tag_len_b - pdt_len_bits;     
  
      if (lkup_tag_len_b > 0) {
      
        /* Read the lower 16 bits*/
        lkup_tag_1 = utlRead16bits((word *)a->hdrImage,
                                     (pdt_byte_offset-1));
        /* Calculate the number of lookup tag bits possible in the lower 2 bytes */
        lkup_tag_len_1 = (16 - (pdt_bit_offset+pdt_len_bits));
        
        if (lkup_tag_len_b > lkup_tag_len_1) /* Lookup Tag is spread across 3 bytes */
        {
          lkup_tag_2 = utlRead16bits((word *)a->hdrImage,
                                       (pdt_byte_offset-3));
          lkup_tag_2 = lkup_tag_2 << lkup_tag_len_1;
          lkup_tag = lkup_tag_2 | (lkup_tag_1 >> (pdt_bit_offset+pdt_len_bits));
        }
        else /* Lookup tag lies within the lower 2 bytes */
        {
          lkup_tag = lkup_tag_1 >> (pdt_bit_offset+pdt_len_bits);    
        }
        
        lkup_tag &= neu_PDT_LEN2MASK(lkup_tag_len_b);
        netCfg->alt_remote_addr[0] = neu_PDT_TYPE_EXT;
        netCfg->alt_remote_addr[2] = lkup_tag;
      }
    }

    /* No second protocol allowed */
    netCfg->second_prototype = DIAGAPP_PROTO_NULL;
    netCfg->rx_offset_bytes  = 0;

    /* Payload type is based on encapsulation */
    if (encap == DREG_NET_ENCAP_AAL1)  {
      netCfg->rx_payload_type = DIAGAPP_PROTO_AAL1;
       /* We are assuming that the payload type is to be either AAL1 or AAL5 
       (DREG_NET_ENCAP_AAL1/AAL5) when ip, udp, eth offset bytes are zeros, can be modified later depending on the actual payload types */
        if (flag) {
            netCfg->rx_offset_bytes  = 0;
        } else {
            netCfg->rx_offset_bytes  = dsiuContext.diagApp.u.utopia.cellSize - 48;
        }
    }  else  {
      netCfg->rx_offset_bytes  = a->rxOffsetBytes;
      netCfg->rx_payload_type = a->rxPayloadType;
    }

    /* Tx Parameters */
    if (a->ipOffsetBytes >= 0)
      netCfg->tx_ip_offset_bytes = a->ipOffsetBytes;
    else
      netCfg->tx_ip_offset_bytes = -1;

    if (a->udpOffsetBytes >= 0)
      netCfg->tx_udp_offset_bytes = a->udpOffsetBytes;
    else
      netCfg->tx_udp_offset_bytes = -1;

    if (a->ethOffsetBytes >= 0)
      netCfg->tx_802_3_offset_bytes = a->ethOffsetBytes;
    else
      netCfg->tx_802_3_offset_bytes = -1;

    netCfg->tx_min_cell_bytes = a->txMinCellBytes;

    /* No UDP ID source mask is used for voice configurations */
    netCfg->tx_udp_src_mask = diagCfg.neuUdpSrcMask;
    netCfg->tx_udp_src_val  = core_id;

    /* For aal1 routing modify the route tag/atm header fields as required */
    if (encap == DREG_NET_ENCAP_AAL1)  {
    
      switch (dsiuContext.siuNduContext.cellSize)  {
        case 58:
          DSIU_UTOPIA_RT_SET_CORE_ID((dsiuRouteTagHeader_t *)netCfg->header, core_id);
          if (encap == DREG_NET_ENCAP_AAL1)
            SIU_UTOPIA_RT_SET_TYPE((dsiuRouteTagHeader_t *)netCfg->header, SIU_UTOPIA_RT_TYPE_AAL1);
          break;

        case 54:
          DSIU_UTOPIA_VC_SET_VCI_CORE ((dsiuVcDemuxHeader_t *)netCfg->header, core_id);
          break;
      }

    } 

  }

  return (a);
 

} /* dsiu_net_config_diagapp */


#if (defined(DSIU_SIMULATE_APPLICATION) || defined(DSIU_DAUGHTER_TEST))
/******************************************************************************
 * FUNCTION PURPOSE: Setup the default network configuration
 ******************************************************************************
 * DESCRIPTION: The default network configuration is derived from the 
 *              diagApp message configuration parameters.
 ******************************************************************************/
void dsiuDefaultVchConfig (tint core_id)
{
  dregNetCfg_t    *v;
  dregNetCfg_t     ch;
  diagAppHeader_t *a;
  tuint i;


  /* Extract the message configuration from the diagApp information */
  a = dsiu_net_config_diagapp (core_id, &ch, DSIU_CONFIG_VOICE);

  /* Copy the basic header to all the channels. Adjust the UDP port if
   * the udp offset is not negative */
  for (i = 0; i < diagCfg.maxChannels; i++)  {
    v = &((dsiuContext.dregVchCfgBlk[i]).vch);
    *v = ch;
    if ((v->cfg_flag1 & (neuproto_FLAGS1_PDT_RX_CHK|neuproto_FLAGS1_PDT_TX_CHK)) &&
        (neu_PDT_TYPE(v->alt_remote_addr[0])==neu_PDT_TYPE_EXT))
    {
      v->alt_remote_addr[2] = i;
    }
    else if (a->udpOffsetBytes >= 0)  {
      utlWrite16bits ((word *)v->header, a->udpOffsetBytes, diagCfg.neuUdpVoiceBase + i);
      utlWrite16bits ((word *)v->header, (a->udpOffsetBytes)+2, diagCfg.neuUdpVoiceBase + i);
    }

    /* For aal1 encapsulation over utopia, modify route tag/vc demux headers as
     * required */
    if (v->route == DIAGAPP_ROUTE_UTOPIA)  {

      switch (dsiuContext.diagApp.u.utopia.cellSize)  {
        case 58:
          SIU_UTOPIA_RT_SET_CHANNEL((dsiuRouteTagHeader_t *)v->header, i+1);
          break;

        case 54:
          DSIU_UTOPIA_VC_SET_VCI_CHANNEL((dsiuVcDemuxHeader_t *)v->header, i+6);
          break;
      }
    }

    dsiuContext.dregVchCfgShadow[i] = dsiuContext.dregVchCfgBlk[i].vch;
  }

    
} /* dsiuVchConfig */

#endif


/*******************************************************************************
 * FUNCTION PURPOSE: Configure the default message parameters
 *******************************************************************************
 * DESCRIPTION: The default message parameters are created from the
 *              diagApp information
 *******************************************************************************/
void dsiuDefaultMsgConfig (tint core_id)
{
  /* Extract the message configuration from the diagApp information */
  dsiu_net_config_diagapp (core_id, dsiuContext.dregMsgCfg, DSIU_CONFIG_MSG);

  /* Copy to the shadow register */
  dsiuContext.dregMsgCfgShadow = dsiuContext.dregMsgCfg;

} /* dsiuDefaultMsgConfig */


#if (defined(DSIU_SIMULATE_APPLICATION) || defined(DSIU_DAUGHTER_TEST))

/*******************************************************************************
 * FUNCTION PURPOSE: Setup the default router configuration
 *******************************************************************************
 * DESCRIPTION: The initial router config loops tdm in back out the same
 *              timeslot, and network data in back out the same channel.
 *
 *              Each tdm in and network in has a dedicated router. The
 *              map is router 0 for tdm input channel 0, router 1 for 
 *              network channel 0, router 2 for tdm input channel 1, etc.
 *              The output follows the same mapping. 
 *******************************************************************************/
void dsiuDefaultRoute (void)
{
  tint i;
  dregRouterCfg_t *rtr;

  /* Routers from the TDM side default to route back to the tdm */
  for (i = 0; i < diagCfg.maxChannels; i++)  {
    rtr = &(dsiuContext.dregRouterCfg[DREG_ROUTER_TDM(i)]);
    rtr->status     = DREG_ROUTE_STATUS_ENABLED;
    rtr->bertDetect = DREG_ROUTE_BERT_DETECT_OFF;
    rtr->outSrc     = DREG_ROUTE_OUT_SRC_INMUX;
    
    /*Cross connect Channels 1 and 0 IF and ONLY IF BERT out of the box run is not enabled*/
    if( (i==0) && (dsiuSimConfig.opt_diag_otb_BERT==FALSE) )
    {
      rtr->outCh      = 1;
    }
    else if( (i ==1) && (dsiuSimConfig.opt_diag_otb_BERT==FALSE) )
    {
      rtr->outCh      = 0;
    }
    else
      rtr->outCh      = i;
    
    rtr->outDir     = DREG_ROUTE_OUTDIR_TDM;

    dsiuContext.dregRouterCfgShadow[DREG_ROUTER_TDM(i)] = *rtr;
  }

  /* Routers from the network side default to route back to the net */
  for (i = 0; i < diagCfg.maxChannels; i++)  {
    rtr = &(dsiuContext.dregRouterCfg[DREG_ROUTER_NET(i)]);
    rtr->status     = DREG_ROUTE_STATUS_ENABLED;
    rtr->bertDetect = DREG_ROUTE_BERT_DETECT_OFF;
    rtr->outSrc     = DREG_ROUTE_OUT_SRC_INMUX;//DREG_ROUTE_OUT_SRC_BERT_255
    rtr->outCh      = i;
    rtr->outDir     = DREG_ROUTE_OUTDIR_NET;

    dsiuContext.dregRouterCfgShadow[DREG_ROUTER_NET(i)] = *rtr;
  }

} /* dsiuDefaultRoute */

#endif    
/*******************************************************************************
 * FUNCTION PURPOSE: Initialize the diag status and control registers
 *******************************************************************************
 * DESCRIPTION: Sets the initial state of all the externally visible
 *              registers.
 *******************************************************************************/
void dsiuDefaultRegValues(tuint core_id)
{
  tint i, c, t;
  tuint type;
  halQuery_t query;
  dregPeriph_t *periph;

  memset (&dregStatus, 0, sizeof(dregStatus_t));

  
  /* Device PG value */
  query.periph = HAL_QUERY_PG;
  (*dsiuContext.hal->halQuery) (&query);
  dregStatus.pg = query.u.pg.pg;

  /* Software ID */ 
  dregStatus.softId = ((tulong)GG_DSPVER_DSP_REL       << 24)  |
                      ((tulong)GG_DSPVER_DSP_SUBREL    << 16)  |
                              (GG_DSPVER_MAKE_REV      <<  8)  |
                              (GG_DSPVER_MAKE_SUBREV1  <<  0);

  /* mem Test */
  dregStatus.memTest.status  = DREG_MEM_STATUS_NOT_RUN;
  for (i = 0; i < DREG_MAX_MEM_FAILURES; i++)  
    memset ((void *)&(dregStatus.memTest.fail), 0, sizeof (dregMemLoc_t));

  dregStatus.cacheTest    = DREG_STATUS_CACHE_TEST_NOT_RUN;
  dregStatus.nChannels    = diagCfg.maxChannels;
  dregStatus.pLoopCounter = 0;

#if (defined(DSIU_SIMULATE_APPLICATION) || defined(DSIU_DAUGHTER_TEST))
 /* BERT Test */
  for (i = 0; i < DIAG_MAX_CHANNELS; i++)  
    memset((void *)&(dregBertStats[i]), 0, sizeof(dregBertStats_t));
#endif

  /* Cycle through and get info about the tdm and network peripherals */
  c = t = 0;

  /* Clear the peripheral status information */
  for (i = 0; i < DREG_MAX_PERIPHS; i++)  {
    periph = &(dregStatus.periph[i]);
    memset (periph, 0, sizeof(dregPeriph_t));
  }


  /* MCBSP/EMCBSP/MCBSP512/MCBSP256 */
  for (i = 0; i < diagCfg.nTdmPorts; i++)  {
    query.periph  = HAL_QUERY_MCBSP;
    query.portNum = i;
    if ((*dsiuContext.hal->halQuery) (&query) == HAL_SUCCESS)  {

      if (query.isPresent && query.isAvail)  {
        periph = &(dregStatus.periph[c++]); 
      
        switch (query.u.mcbsp.type)  {
          case HAL_MCBSP_TYPE_MCBSP:    type = DREG_SOFTID_MCBSP;    break;
          case HAL_MCBSP_TYPE_EMCBSP:   type = DREG_SOFTID_EMCBSP;   break;
          case HAL_MCBSP_TYPE_MCBSP512: type = DREG_SOFTID_MCBSP512; break;
          case HAL_MCBSP_TYPE_MCBSP256: type = DREG_SOFTID_MCBSP256; break;
          default: type = DREG_SOFTID_MCBSP_OTHER; break;
        }

        DREG_PERIPH_SET_SOFTID(periph->idNumStatus, type);   /* software Id */
        DREG_PERIPH_SET_PORT  (periph->idNumStatus, 0);       /* physical port num */
        DREG_PERIPH_SET_STATUS(periph->idNumStatus, DREG_PERIPH_STATUS_UNALLOCATED);  /* status */

        /* hwId is setup when the device is assigned and enabled */

        /* Link the peripheral to the dsiuContext */
        dsiuContext.tdmPort[t].status = periph;
        dsiuContext.tdmPort[t++].status->hwId = 0; 

        if (c > DREG_MAX_PERIPHS)
          dsiuPanic (DSIU_FAIL_NO_PERIPH_REGS);
      }
    }
  }

  /* TSIP */
  for (i = 0; i < diagCfg.nTdmPorts; i++)  {
    query.periph = HAL_QUERY_TSIP;
    query.portNum = i;
    if ((*dsiuContext.hal->halQuery) (&query) == HAL_SUCCESS)  {

      if (query.isPresent && query.isAvail)  {

        periph = &(dregStatus.periph[c++]);
        DREG_PERIPH_SET_SOFTID(periph->idNumStatus, DREG_SOFTID_TSIP);   /* software Id */
        DREG_PERIPH_SET_PORT  (periph->idNumStatus, 0);                   /* physical port num */
        DREG_PERIPH_SET_STATUS(periph->idNumStatus, DREG_PERIPH_STATUS_UNALLOCATED);  /* status */


        /* Link the peripheral to the dsiuContext */
        dsiuContext.tdmPort[t].status = periph;
        dsiuContext.tdmPort[t++].status->hwId = 0; 

        if (c > DREG_MAX_PERIPHS)
          dsiuPanic (DSIU_FAIL_NO_PERIPH_REGS);

      }

    }
  }

  /* Utopia */
  for (i = 0; i < diagCfg.nUtopiaPorts; i++)  {
    query.periph = HAL_QUERY_UTOPIA;
    query.portNum = i;
    if ((*dsiuContext.hal->halQuery) (&query) == HAL_SUCCESS)  {

      if (query.isPresent && query.isAvail)  {

        periph = &(dregStatus.periph[c++]);
        DREG_PERIPH_SET_SOFTID(periph->idNumStatus, DREG_SOFTID_UTOPIA);  /* software Id */
        DREG_PERIPH_SET_PORT  (periph->idNumStatus, 0);                    /* physical port number */
        DREG_PERIPH_SET_STATUS(periph->idNumStatus, DREG_PERIPH_STATUS_UNALLOCATED);  /* status */

        /* Link the peripheral to the dsiuContext */
        dsiuContext.utopiaPort[i].status = periph;
        dsiuContext.utopiaPort[i].status->hwId = 0; 

        if (c > DREG_MAX_PERIPHS)
          dsiuPanic (DSIU_FAIL_NO_PERIPH_REGS);
      }
    }
  }

  /* Mac */
  for (i = 0; i < diagCfg.nGmacPorts; i++)  {
    query.periph = HAL_QUERY_MAC;
    query.portNum = i;
    if ((*dsiuContext.hal->halQuery) (&query) == HAL_SUCCESS)  {

      if (query.isPresent && query.isAvail)  {

        periph = &(dregStatus.periph[c++]);
        DREG_PERIPH_SET_SOFTID(periph->idNumStatus, DREG_SOFTID_MAC);  /* software Id */
        DREG_PERIPH_SET_PORT  (periph->idNumStatus, 0);                 /* physical port number */
        DREG_PERIPH_SET_STATUS(periph->idNumStatus, DREG_PERIPH_STATUS_UNALLOCATED); /* status */

        /* Link the peripheral to the dsiuContext and store the peripheral id */
        dsiuContext.macPort[i].status       = periph;
        dsiuContext.macPort[i].status->hwId = 0; 

        if (c > DREG_MAX_PERIPHS)
          dsiuPanic (DSIU_FAIL_NO_PERIPH_REGS);

      }
    }
  }

  /* Message parameter setup */
  dsiuDefaultMsgConfig (core_id);


  /* Setup the master control register */
  dregMasterCntrl.tdmCntrl.command = DREG_CNTRL_NO_CMD;
  dregMasterCntrl.tdmCntrl.lastCommand = DREG_CNTRL_NO_CMD;
  dregMasterCntrl.tdmCntrl.lastCommandStatus = DREG_CNTLSTAT_SUCCESS;

  dregMasterCntrl.netCntrl    = dregMasterCntrl.tdmCntrl;
  dregMasterCntrl.routerCntrl = dregMasterCntrl.tdmCntrl;

  dregMasterCntrl.bertClearCntrl.channel = DREG_BERTCLEAR_CNTL_IDLE;

  if (diagCfg.ctable.dsiu->blockMsgInit)
    (*diagCfg.ctable.dsiu->blockMsgInit) ();


/* The TDM, network, and router initialization is not required for the
 * boot loader */

#if (defined(DSIU_SIMULATE_APPLICATION) || defined(DSIU_DAUGHTER_TEST))


  /* Status registers */
  memset ((void *)&dregOpStats, 0, sizeof(dregOpStats_t));

  /* TDM registers. The timeslots are allocated based on the number of tdm ports
   * and on the core ID. The macro is used to indicate port number, and the
   * port location can depend on the build. */
  dsiuDefaultTdmConfig (core_id);


  /* Network registers. The initial configuration is based on the message
   * header information provided in the diagApp structure */
  dsiuDefaultVchConfig (core_id);


  /* Initial Routing. The initial route is to loop tdm data in back
   * to tdm data out, network data in back to network data out */
  dsiuDefaultRoute ();

  /* Bert stats clear */
  for (i = 0; i < diagCfg.maxChannels; i++)
    memset (&dregBertStats[i], 0, sizeof (dregBertStats_t));

#else

  DSIU_DUMMY_ARG(core_id);

#endif

#ifdef DIAG_INCLUDE_OTBRUN
  dsiuCfgOtbBert();
#endif

} /* End of dsiuDefaultRegValues() */

/* Nothing Past this Point */

