#ifndef _DIAGREG_H
#define _DIAGREG_H
/******************************************************************************
 * FILE PURPOSE: Defines the memory block used to control diag
 ******************************************************************************
 * FILE NAME: diagreg.h
 *
 * DESCRIPTION: Defines a structure which is used to control the operation of
 *              diag.
 *
 * TABS: None
 *
 * (C) Copyright 2002, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include "types.h"
#include "dregbld.h"  /* Specifies array sizes specific to the build */
#include "utlbitfield.h"
#include "dglob.h"

/* Do not include diagbld.h  */

/******************************************************************************
 * DEFINITION: Macros to convert BYTES into WORDS for any processos
 ******************************************************************************/
#ifdef DSP_ON_MIPS
#define DREG_B2W(NUMBYTES)  (((NUMBYTES) + ((TYP_WORD_SIZE/8) - 1)) >> (TYP_WORD_SIZE/8))
#else
#define DREG_B2W(NUMBYTES)  (((NUMBYTES) + ((TYP_WORD_SIZE/8) - 1)) / (TYP_WORD_SIZE/8))
#endif

/****************************************************************************
 * DEFINITION: Peripheral status. Provides hardware and run time
 *             information about peripherals used by the system.
 ****************************************************************************/
typedef struct dregPeriph_s {


  /*  /--------------------------------------------------\
      | 15           12 | 11            8 | 7          0 |
      |   software ID   |   port Number   |   status     |
      \--------------------------------------------------/  */
  tuint idNumStatus;  
  
  tulong hwId;      /* 32 bit hardware ID, 0 if not available          */
  
  tuint  intCount;  /* Count of interrupts generated by the peripheral */
  
  tuint  rxDataCount; /* Cell/packet/frame counter from peripheral     */
  tuint  txDataCount; /* Cell/packet/frame counter to peripheral       */
  tuint  rxDrop;      /* Count of incoming data drops                  */
  tuint  txDrop;      /* Count of outgoing data drops                  */
  
} dregPeriph_t;


/* The Software ID values: */
enum {
   
   DREG_SOFTID_UNUSED         = 0,      /* Element is not a valid peripheral */
   DREG_SOFTID_MCBSP512,    /*  1 */
   DREG_SOFTID_MCBSP256,    /*  2 */
   DREG_SOFTID_MCBSP,       /*  3 */
   DREG_SOFTID_EMCBSP,      /*  4 */
   DREG_SOFTID_MCBSP_OTHER, /*  5 */
   DREG_SOFTID_TSIP,        /*  6 */
   DREG_SOFTID_MAC,         /*  7 */
   DREG_SOFTID_UTOPIA       /*  8 */
};


/* Peripheral status values */
enum {
   DREG_PERIPH_STATUS_UNALLOCATED         = 0,
   DREG_PERIPH_STATUS_UNINITIALIZED,     /* 1 */
   DREG_PERIPH_STATUS_CFGD_DISABLED,     /* 2 */
   DREG_PERIPH_STATUS_CFG_XMT_ENBALED,   /* 3 */
   DREG_PERIPH_STATUS_CFG_RCV_ENABLED,   /* 4 */
   DREG_PERIPH_STATUS_CFG_ENABLED        /* 5 */
};   

#define DREG_PERIPH_READ_STATUS(x)  ((x) & 0x00ff)
#define DREG_PERIPH_SET_STATUS(x,y)  UTL_SET_BITFIELD(x,y,0,8)
#define DREG_PERIPH_SET_PORT(x,y)    UTL_SET_BITFIELD(x,y,8,4)
#define DREG_PERIPH_READ_PORT(x)     UTL_GET_BITFIELD(x,8,4)
#define DREG_PERIPH_SET_SOFTID(x,y)  UTL_SET_BITFIELD(x,y,12,4)


/*****************************************************************************
 * DEFINITION: Memory Test results. Records the status of the memory tests
 *****************************************************************************/
typedef struct dregMemLoc_s  {

  tulong address;       /* The address of a failed memory location */ 
  tuint  written;       /* The value written to that location      */
  tuint  read;          /* The value read from that location       */

} dregMemLoc_t;

enum  {
  DREG_MEM_STATUS_NOT_RUN      = 0,
  DREG_MEM_STATUS_IN_PROGRESS,
  DREG_MEM_STATUS_DONE,
  DREG_MEM_STATUS_ABORT
};


typedef struct dregMemTest_s  {

   tuint status;    /* Memory test status      */
   
   tuint nFailed;   /* Number of locations which failed */
   
   dregMemLoc_t fail[DREG_MAX_MEM_FAILURES];
   
} dregMemTest_t;
   


/*****************************************************************************
 * DEFINITION: Boot track progress codes. Indicates the status of the boot. 
 *             Used to debug boot problems before the message interface has 
 *             been initialized.
 *****************************************************************************/
typedef enum {
  DREG_BOOT_TRACK_BOOT_REG_INIT = 0,   /* boot.s: cpu registers set up        */
  DREG_BOOT_TRACK_BOOT_BSS_ZERO,       /* 1   boot.s: bss zeroed            */
  DREG_BOOT_TRACK_BOOT_AUTO_INIT,      /* 2   boot.s: auto init completed   */
  DREG_BOOT_TRACK_BOOT_MAIN_RET,       /* 3   boot.s: error - return from main */ 
  DREG_BOOT_TRACK_IN_MAIN,             /* 4   dsiuini.c: in function main      */
  DREG_BOOT_TRACK_DEFAULT_INIT,        /* 5   dsiuini.c: default registers setup */
  DREG_BOOT_TRACK_PERIPH_RESET,        /* 6   dsiuini.c: peripheral reset        */
  DREG_BOOT_TRACK_PSEUDO_APP_INIT,     /* 7   dsiuini.c: pseudo app initialized  */
  DREG_BOOT_TRACK_HAL_INIT_BEGIN,      /* 8   dsiuini.c: in function dsiu_hal_init */
  DREG_BOOT_TRACK_HAL_NEW_SUCCESS,     /* 9   dsiuini.c: halNew successful   */
  DREG_BOOT_TRACK_HAL_CONTROL_SUCCESS, /* 10  dsiuini.c: halControl successful */
  DREG_BOOT_TRACK_AAL5_INIT,           /* 11  dsiuini.c: in dsiu_aal5_init  */
  DREG_BOOT_TRACK_AAL5_INIT_SUCCESS,   /* 12  dsiuini.c: aal5 init complete    */
  DREG_BOOT_TRACK_SIMU_APP_CONFIG,     /* 13  dsiuubt.c: in dsiuSimuAppConfig */
  DREG_BOOT_TRACK_SEND_RESTART,        /* 14  dsiuhdm.c: sending restart      */
  DREG_BOOT_TRACK_PANIC = (int)0xFF00u /* Last nibble contains panic code     */
} codeStatus_t;

enum {
  DREG_STATUS_CACHE_TEST_NOT_RUN  =  0,
  DREG_STATUS_CACHE_TEST_SUCCESS,
  DREG_STATUS_CACHE_TEST_FAIL
};


/**************************************************************************************
 * DEFINITION: The System status registers. These maintain the current status of 
 *             the system.
 **************************************************************************************/
typedef struct dregStatus_s  {

  /* Code status tracks the system before messaging is enabled */
  codeStatus_t codeStatus;             
  

  tuint pg;              /* The device PG value, the 4 lsbs are the fractional part    */
  
  tulong softId;         /* The software release number, eg 11.0.0.1, 8 bits per value */
  
  dregMemTest_t memTest; /* Memory test status/results                                 */
  
  tuint cacheTest;       /* Results of the cache test                                  */
  
  tuint nChannels;       /* The number of channels supported in the diagnostics        */
  
  tuint pLoopCounter;    /* Counts passes through the main execution loop              */
  
  tuint clkRatio;        /* Ratio of tdm clocks to frame sync pulses                   */

  dregPeriph_t periph[DREG_MAX_PERIPHS];  /* Peripheral info                           */
  
} dregStatus_t;

extern dregStatus_t dregStatus;


/***************************************************************************************
 * DEFINITION: The master control registers. Causes implementation of the configuration
 *             registers to be processed.
 ***************************************************************************************/
enum {
  DREG_CNTRL_NO_CMD     = 0,   /* No command */
  DREG_CNTRL_CONFIG,           /* Process config registers */
  DREG_CNTRL_RETRIEVE          /* Retrieve current register state */
};
 
 enum {
   DREG_CNTLSTAT_SUCCESS   = 0,  /* Last command was processes successfully */
   DREG_CNTLSTAT_FAIL            /* Invalid register configuration provided */
};
 
typedef struct dregCntrl_s
{
  tuint command;
  tuint lastCommand;
  tuint lastCommandStatus;
  
} dregCntrl_t;

#define DREG_BERTCLEAR_CNTL_CLEAR_ALL  -1
#define DREG_BERTCLEAR_CNTL_IDLE       -2

typedef struct dregBertClearCntl_s
{
  tint  channel;   /* Channel to clear. */

} dregBertClearCntrl_t;


typedef struct dregMasterCntrl_s
{
  dregCntrl_t  tdmCntrl;
  dregCntrl_t  netCntrl;
  dregCntrl_t  routerCntrl;
  dregCntrl_t  msgCntrl;
  
  dregBertClearCntrl_t  bertClearCntrl;
  
} dregMasterCntrl_t;

extern dregMasterCntrl_t dregMasterCntrl;

/* block message control is broken out of master control because it must
 * reside on its own cache line if cross core message passing is allowed.
 * Alignment is handled through the linker command file. Not all builds
 * require cross core messages */
typedef struct dregBlockMsgCntl_s
{
    tuint firstMsg;
    tuint nMsg;
  
} dregBlockMsgCntl_t;

extern dregBlockMsgCntl_t dregBlockMsgCntl;

/* DREG_BLOCK_MSG_CTL_SIZE should be defined to the size of a cache line
 * if cross core messaging is allowed. */
#ifndef DREG_BLOCK_MSG_CTL_SIZE_TUINT
#define DREG_BLOCK_MSG_CTL_SIZE_TUINT  1
#endif

typedef struct dregBlockMsgCntlMem_s
{
  union  {
    dregBlockMsgCntl_t cntrl;
    tint               dalign[DREG_BLOCK_MSG_CTL_SIZE_TUINT];
  } u;
  
} dregBlockMsgCntlMem_t;


extern dregBlockMsgCntlMem_t dregBlockMsgCntlMem;

/***************************************************************************************
 * DEFINITION: TDM configuration and status
 ***************************************************************************************/
 
enum {
  DREG_TDM_CONTROL_DISABLE_TS  = 0,
  DREG_TDM_CONTROL_ENABLE_TS
};
 
enum {
  DREG_TDM_STATUS_DISABLED  = 0,
  DREG_TDM_STATUS_ENABLED
};

enum {
  DREG_TDM_COMPAND_ULAW   = 0,
  DREG_TDM_COMPAND_ALAW,
  DREG_TDM_COMPAND_LINEAR
};

 
typedef struct dregTdmCfg_t  {

  tuint txTimeslot;     /* To TDM timeslot (may contain port and link as well) */
  tuint rxTimeslot;     /* From TDM timeslot                                   */
  tint  compand;        /* Companding - used for silence and tone only         */
  tuint control;        /* Control information                                 */
  tuint status;         /* Current status                                      */

} dregTdmCfg_t;
 
extern dregTdmCfg_t dregTdmCfg[];
extern dregTdmCfg_t dregTdmCfgShadow[];

/*******************************************************************************
 * DEFINITION: Network configuration and status.
 *             These registers are used to control network encapuslation and routing.
 *******************************************************************************/
enum {
  DREG_NET_ENCAP_NO_VOICE      = 0,
  DREG_NET_ENCAP_AAL1,        /* 1 */
  DREG_NET_ENCAP_RAW,         /* 2 */
  DREG_NET_ENCAP_AAL1_16BIT,  /* 3 */
  DREG_NET_ENCAP_RAW_16BIT    /* 4 */
};

 
 
 
#define DREG_ENCAP_MAX_HEADER_SIZE_TUINT 32
#define DREG_IP_ADDR_MAX_NUM_TUINT       8

typedef struct dregNetCfg_s  {
  
  tint encap;           

  tint route;  /* DIAGAPP_ROUTE_UTOPIA or  DIAGAPP_ROUTE_GMAC */
  tint port;   /* Physical port to route output */
  
  /* The following parameters are from the neu configuration */
  tuint cfg_flag1;
  tuint cfg_flag2;
  tuint itdmMode;
  tuint alt_local_port;         /* Alternate local Port for 3rd & 4th port */
  tuint alt_remote_port;        /* Alternate remote Port for 3rd & 4th port */
  tuint alt_remote_addr[DREG_IP_ADDR_MAX_NUM_TUINT];/* Alternate Remote Address */ 
  tuint rsvd1;                  /* Report control bitmap for packet violations */
  tuint rsvd2;                  /* Throttle time in msecs for packet violations */
  tuint second_prototype;       /* payload type */
  tuint rx_offset_bytes;        /* rx header size */
  tuint rx_payload_type;        /* payload type */
  tuint tx_min_cell_bytes;      /* the required cell size toward network */
  tint  tx_ip_offset_bytes;     /* ip offset size */
  tint  tx_udp_offset_bytes;    /* udp offset size */
  tint  tx_802_3_offset_bytes;  /* 802.3 offset. Only used for tx frame length */
  tint  tx_total_hdr_len_bytes; /* total header size */
  tint  tx_udp_src_mask;        /* UDP src mask used with tx_udp_src_val  */
  tint  tx_udp_src_val;         /* UDP src value put into UDP source port */
  tuint header[DREG_ENCAP_MAX_HEADER_SIZE_TUINT];
  
} dregNetCfg_t;


typedef struct itdmStreamCfg_s {
    itdmMode_t mode;
    tuint      mac_port_id;
    tuint      tx_flow_id_msb;
    tuint      tx_flow_id_lsb;
    tuint      tx_flow_mpls_label_msb;
    tuint      tx_flow_mpls_label_lsb;
    tuint      tx_begin_seq_num;
    tuint      tx_num_init_channels;
    tuint      tx_first_channel_ID_msb;
    tuint      tx_first_channel_ID_lsb;
    tuint      tx_first_timeslot;
    tuint      tx_fill_pattern;
    tuint      rx_flow_id_msb;
    tuint      rx_flow_id_lsb;
    tuint      rx_flow_mpls_label_msb;
    tuint      rx_flow_mpls_label_lsb;
    tuint      rx_num_init_channels;
    tuint      rx_first_channel_ID_msb;
    tuint      rx_first_channel_ID_lsb;
    tuint      rx_first_timeslot;
    tuint      rx_fill_pattern;
} itdmStreamCfg_t;

extern itdmStreamCfg_t itdmStreamCfg[];

typedef struct itdmChanUpdateMsg_s {
    itdmChanUpdate_t action;
    itdmDirection_t  dir;
    tuint            chanID_MSB;
    tuint            chanID_LSB;
    tuint            timeslot;
} itdmChanUpdateMsg_t;

extern itdmChanUpdateMsg_t itdmChanUpdateMsg;

/* The voice configuration is unionized to make each channel use 64 words */
#define DREG_VCH_CONFIG_SIZE    64
typedef union {
  dregNetCfg_t vch;
  tuint           block[DREG_VCH_CONFIG_SIZE];
} dregVchCfgBlk_t;

extern dregVchCfgBlk_t dregVchCfgBlk[];
extern dregNetCfg_t    dregVchCfgShadow[];


/*****************************************************************************
 * Message Network Configuration
 *****************************************************************************/ 

extern dregNetCfg_t dregMsgCfg;
extern dregNetCfg_t dregMsgCfgShadow;

#define DIAG_REG_NET_TYPE_MSG   0
#define DIAG_REG_NET_TYPE_VCH   1


/*****************************************************************************
 * DEFINITION: Router control
 *****************************************************************************/

enum  {
  DREG_ROUTE_STATUS_DISABLED,  /* Output from mux is lost */
  DREG_ROUTE_STATUS_ENABLED
};

enum {
  DREG_ROUTE_OUT_SRC_INMUX         = 0,  /* Output from mux input      */
  DREG_ROUTE_OUT_SRC_BERT_RANDOM,        /* Output from BERT generator, random 32768 */
  DREG_ROUTE_OUT_SRC_BERT_255,           /* Output from BERT generator, 0-255 repeat */
  DREG_ROUTE_OUT_SRC_BERT_RND_ID,        /* Output from BERT generator, with ID      */
  DREG_ROUTE_OUT_SRC_TONE,               /* Output from tone generator */
  DREG_ROUTE_OUT_SRC_PATTERN             /* Output from pattern generator */
};

enum {
  DREG_ROUTE_OUTDIR_TDM   = 0,    /* Output sent to TDM */
  DREG_ROUTE_OUTDIR_NET           /* Output sent to net */
};

enum {
  DREG_ROUTE_BERT_DETECT_OFF  = 0,  /* BERT detection disabled */
  DREG_ROUTE_BERT_DETECT_ON
};
 
 
typedef struct dregRouterCfg_s  {

  tuint status;
  
  tuint bertDetect;     /* If set BERT detection is enabled       */
  tuint outSrc;         /* Output generated from BERT/Tone/inmux  */
  tuint outCh;          /* Output channel                         */
  tuint outDir;         /* Output direction (tdm/net)             */

} dregRouterCfg_t;
  
extern dregRouterCfg_t dregRouterCfg[];
extern dregRouterCfg_t dregRouterCfgShadow[];

/* Macros to get the router index based on channel number and side (net, tdm) */
#define DREG_ROUTER_TDM(x)   (x)*2
#define DREG_ROUTER_NET(x)  ((x)*2)+1
  
/*****************************************************************************
 * BERT stats
 *****************************************************************************/
  
/* The bert statistics registers */
#define DREG_BERT_CH_STATS_SIZE  32
typedef struct dregBertStats_s
{
  tuint block[DREG_BERT_CH_STATS_SIZE];
} dregBertStats_t;

extern dregBertStats_t dregBertStats[];
  
  
/*****************************************************************************
 * Operational statistics registers
 *****************************************************************************/
typedef struct dregOpStats_s {
  
  tuint pkt_lookup_err;      /* Receive packet didn't match any lookup */
  tuint tdmCfgErr;           /* Count of unexpected tdm config errors  */
  tuint tdmTsEnableFail;     /* Count of failures to enable a timeslot */
  tuint dpvpEncapFail;       /* Count of failures to configure dpvp    */
  tuint dpvpResetFail;       /* Count of pvp shadow reset failures     */
  tuint neuCfgFail;          /* Count of neu configuration failures    */
  tuint neuPortMapFail;      /* Count of neu port map failures         */
  tuint neuResetFail;        /* Count of neu reset failures            */
  tuint chRouteFail;         /* Count of route map failures            */
  tuint chPortFail;          /* Count of invalid channel routes        */
  tuint dpiuFail;            /* Count of dpiu failures                 */
  tuint bertCtlFail;         /* Count of bert control failures         */
  tuint ipiuFail;            /* Count of ipiu failures                 */
 
} dregOpStats_t;

extern dregOpStats_t dregOpStats;

/******************************************************************************
 * Boot tracking structure. Use caution when modifying this structure. The
 * bootstrap file boot.s uses the base address of this structure to place
 * status info.
 ******************************************************************************/
typedef struct dregBootTrack_s {

  tulong location;           /* Indicates boot progress, symbols in dsiuloc.h */
 
} dregBootTrack_t;

extern dregBootTrack_t dregBootTrack;


/******************************************************************************
 * Automated message control - this is optional per build through diagbld.h
 ******************************************************************************/
typedef struct dregAutoCtl_s  {
  volatile tuint  msg;           /* The message register - host to DSP */
  tuint           dummy;
  volatile tuint  stat;          /* The stat register, DSP to host */

} dregAutoCtl_t;

extern dregAutoCtl_t dregAutoCtl;

/******************************************************************************
 * Message sequence block. A sequence of messages can be executed through the
 * automated block. Additionally pre-defined messages on a build by build
 * basis can be placed here.
 ******************************************************************************/
#define DREG_MSG_BLOCK_SIZE_TUINT   64

typedef struct dregMsgBlock_s  {

  tuint msgParam[DREG_MSG_BLOCK_SIZE_TUINT];
  
} dregMsgBlock_t;

extern dregMsgBlock_t dregMsgBlock[];
extern dregMsgBlock_t dregMsgOTBBertInit[];


/*******************************************************************************
 * Reboot block. The Code can enter a small loop where it waits for a master
 * control mechanism to write to the boot address field.
 *******************************************************************************/
#define DREG_REBOOT_ZERO_STATUS_ZERO_PENDING    0xffff
#define DREG_REBOOT_ZERO_STATUS_ZERO_DONE       0
typedef struct dregReboot_s {

  tuint  dummy;
  tuint  zeroStatus;
  volatile tulong bootAddress;

} dregReboot_t;

extern dregReboot_t dregReboot;


/**********************************************************************************
 * Memory clear addresses. In auto-run mode the local memory is cleared prior
 * to code exit. This specifies the memory to be cleared
 **********************************************************************************/
#define DREG_MEM_CLEAR_FLAG_VALID       (1 << 0)  /* Set if entry is valid                   */
#define DREG_MEM_CLEAR_FLAG_SHARED      (1 << 1)  /* Set if memory is shared between cores   */
#define DREG_MEM_CLEAR_FLAG_DMA         (1 << 2)  /* Set if the memory can be cleared by DMA */
#define DREG_MEM_CLEAR_FLAG_UNUSED      (1 << 3)  /* Set if the memory is otherwise unused   */
typedef struct dregMemClear_s  {

  tuint   flags;     /* Set for shared memory. Only core 0 can clear shared memory */
  void   *sAddr;      /* Start address                                              */
  tulong  sizeBytes;  /* Memory block size in bytes                                 */
  
} dregMemClear_t;

extern dregMemClear_t dregMemClearLoad[];
extern dregMemClear_t dregMemClearRun[];



void dsiuDefaultRegValues(tuint dsp_id);


#endif /* _DIAGREG_H */
