/************************************************************************
:*                                                                      *
:*     @ Copyright (C) 2009  Texas Instruments Incorporated.            *
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

/*********************************************************************
 * FILE PURPOSE: Functions supporting automated run
 *********************************************************************
 * FILE NAME: dsiuarun.c
 *
 * DESCRIPTION: Functions required for automated run
 *
 *********************************************************************/
#include "types.h"
#include "diagreg.h"
#include "diagcfg.h"
#include "dsiuloc.h"
#include "hal.h"
#include "string.h"

dsiuAutoRunContext_t dsiuAutoContext;

/* The predefined messages (parameters in diagbld.h) must
 * be in the following order */

enum  {

  DSIU_ARUN_PREDEF_MSG_EMIF_CONFIG,
  DSIU_ARUN_PREDEF_MSG_HW_CONFIG,
  DSIU_ARUN_PREDEF_MSG_MEMTEST_GLOBAL,
  DSIU_ARUN_PREDEF_MSG_MEMTEST_LOCAL,
  DSIU_ARUN_PREDEF_MSG_PORTCFG,
  DSIU_AURN_PREDEF_MSG_IDLE

};

/* The following values are declared in the linker command file */
extern int dsiu_arun_run;
extern int dsiu_arun_load;
extern int dsiu_arun_size;
extern int dsiu_reg_mem_clear_size;

/**********************************************************************
 * DESCRIPTION: This is a tiny stack used for the final spin
 **********************************************************************/
#define DSIU_ASTACK_SIZE_BYTES  32
#pragma DATA_SECTION(dsiuAstack, ".spin_stack")
word dsiuAstack[SIU_BYTES_TO_WORDS(DSIU_ASTACK_SIZE_BYTES)];

/**********************************************************************
 * FUNCTION PURPOSE: Clear memory and wait for new boot address
 **********************************************************************
 * DESCRIPTION: This function resides in a small relocatable block
 *              with its own small stack. There is never a return
 *              from this function.
 **********************************************************************/
#pragma CODE_SECTION(dsiuSpinDry, ".spin_code")
void dsiuSpinDry (tint core)
{
  void (*goodbye)(void);
  word   *wp;
  tint    i;
  tulong  j;


  /* Memset can't be used because it is about to be wiped! */

  for (i = 0; dregMemClearRun[i].flags & DREG_MEM_CLEAR_FLAG_VALID; i++)  {

    if ((dregMemClearRun[i].flags & DREG_MEM_CLEAR_FLAG_UNUSED) == 0)  {

      if (  ((dregMemClearRun[i].flags & DREG_MEM_CLEAR_FLAG_SHARED) == 0)     ||
            (core == 0) )                                   {

        wp = (word *)dregMemClearRun[i].sAddr;

        for (j = 0; j < SIU_BYTES_TO_WORDS(dregMemClearRun[i].sizeBytes); j++)
          wp[j] = 0;
      }
    }

    dregMemClearRun[i].flags = dregMemClearRun[i].sizeBytes = 0;
    dregMemClearRun[i].sAddr = (void *)0;
  }


  dregReboot.zeroStatus = DREG_REBOOT_ZERO_STATUS_ZERO_DONE;

  while (dregReboot.bootAddress == 0);

  goodbye = (void (*)(void))dregReboot.bootAddress;
  (*goodbye)();

} /* dsiuSpinDry */
  

/**********************************************************************
 * FUNCTION PURPOSE: Initialize auto-run registers
 **********************************************************************
 * DESCRIPTION: The auto-run control registers are initialized
 **********************************************************************/
void dsiuAutoRunInit (void)
{
  halQuery_t    hq;
  halControl_t  hCtl;

  dregAutoCtl.msg   = 0;
  dregAutoCtl.stat  = 0;
  dregAutoCtl.dummy = 0;

  dsiuAutoContext.state = DSIU_AUTO_RUN_STATE_WAIT;

  hq.periph = HAL_QUERY_NCORES;
  (dsiuContext.hal->halQuery)(&hq);
  dsiuAutoContext.core   = hq.u.core.thisCore;
  dsiuAutoContext.ncores = hq.u.core.nCores;

  dsiuAutoContext.waitStatClear = FALSE;

  dregReboot.zeroStatus  = DREG_REBOOT_ZERO_STATUS_ZERO_PENDING;
  dregReboot.bootAddress = 0;

  /* Initialize the memory test hold */
  hCtl.cmd                  = HAL_COMMAND_IPC_WRITE;
  hCtl.u.ipcWrite.setMask   = 0;
  hCtl.u.ipcWrite.clearMask = diagCfg.memTestMsg;
  (*dsiuContext.hal->halControl) (&hCtl);

} /* dsiuAutoRunInit */


/**********************************************************************
 * FUNCTION PURPOSE: Secondary core stall
 **********************************************************************
 * DESCRIPTION: Secondary cores are stalled to allow core 0 to
 *              do global configuration. This is required if all
 *              cores begin execution at once.
 **********************************************************************/
void dsiuAutoRunStall (void)
{
  halQuery_t hq;
  volatile   int i;

  hq.periph = HAL_QUERY_NCORES;
  (dsiuContext.hal->halQuery)(&hq);

  if (hq.u.core.thisCore != 0)
    for (i = 0; i < diagCfg.secCoreStartDelay; i++);

} /* dsiuAutoRunStall */
  


/**********************************************************************
 * FUNCTION PURPOSE: Clean the DSP and spin
 **********************************************************************
 * DESCRIPTION: Caches are disabled, peripherals with DMA to the core
 *              are disabled, and the spin code is entered.
 **********************************************************************/
void dsiu_auto_run_spin (void)
{
  halUtopiaControl_t utCtl;
  halGmacControl_t   gmacCtl;
  halTdmControl_t    tCtl;
  halMemConfig_t     memCfg;
  halGlobAddr_t      gAddr;
  tulong             mSize;
  dmsg_t            *msg;
  tint               zsize;
  tint i;

  /* Disable network and tdm peripherals DMA to this core */
  if (dsiuContext.hal->utopia)  {

    utCtl.mode = HAL_UTOPIA_RX_TMP_DISABLE_CORE;

    for (i = 0; i < dsiuContext.utLogicAlloc; i++)
      (*dsiuContext.hal->utopia->halUtopiaControl)(dsiuContext.utopiaPort[i].halId, &utCtl);

  }

  if (dsiuContext.hal->gmac)  {

    gmacCtl.code = HAL_GMAC_CTL_CODE_TMP_DISABLE;

    for (i = 0; i < dsiuContext.macLogicAlloc; i++) {
      gmacCtl.u.halId = dsiuContext.macPort[i].halId;
      (*dsiuContext.hal->gmac->halGmacControl)(&gmacCtl);
    }
  }

  for (i = 0; i < diagCfg.nTdmPorts; i++)  {

    if (DREG_PERIPH_READ_STATUS(dsiuContext.tdmPort[i].status->idNumStatus) == DREG_PERIPH_STATUS_CFG_ENABLED)  {

      tCtl.mode = HAL_TDM_HALT_TDM;
      tCtl.u.haltRestore.operation = HAL_TDM_CTL_HALT_LOCAL;
      (*dsiuContext.hal->tdm->halTdmControl)(dsiuContext.tdmPort[i].halId, &tCtl);

    }

  }


  /* Disable L1D */
  memCfg.memCfgFunction = HAL_MEMCFGFUNC_CACHECONFIG;
  memCfg.u.cacheCfg.cacheType = HAL_CACHE_L1D;
  memCfg.u.cacheCfg.cacheMode = 0;
  (*dsiuContext.hal->halMemConfig)(&memCfg);

  /* Disable L1P */
  memCfg.memCfgFunction = HAL_MEMCFGFUNC_CACHECONFIG;
  memCfg.u.cacheCfg.cacheType = HAL_CACHE_L1P;
  memCfg.u.cacheCfg.cacheMode = 0;
  (*dsiuContext.hal->halMemConfig)(&memCfg);

  /* Disable L2 */
  memCfg.memCfgFunction = HAL_MEMCFGFUNC_CACHECONFIG;
  memCfg.u.cacheCfg.cacheType = HAL_CACHE_L2;
  memCfg.u.cacheCfg.cacheMode = 0;
  (*dsiuContext.hal->halMemConfig)(&memCfg);

  /* Interrupts must be disabled, since the vector table is about to be destroyed */
  (*dsiuContext.hal->halIntrLevel)(HAL_INT_LEVEL_DISABLE);

  /* Relocate the remaining code and new stack. This is required if the code
   * section resides in a memory area that is cache during download or if
   * it resides in a memory test block used for scratch */
  if ((tulong)&dsiu_arun_run != (tulong)&dsiu_arun_load)
    memcpy ((void *)&dsiu_arun_run, 
            (void *)&dsiu_arun_load, 
            (tulong)&dsiu_arun_size);


  /* Relocate the memory clear table into the reserved area. */
  if ((tulong)&dregMemClearLoad != (tulong)&dregMemClearRun)
    memcpy ((void *)&dregMemClearRun, (void *)dregMemClearLoad, (tulong)&dsiu_reg_mem_clear_size);

  /* Initialize the new stack to verify depth used */
  memset (dsiuAstack, 0xce, SIU_BYTES_TO_WORDS(DSIU_ASTACK_SIZE_BYTES));


  /* Zero out memory that isn't used by the diag code. This is done here just to reduce the
   * the amount of code in the reserved section. All sections that MUST be cleared through DMA
   * (L1P) must currently be marked unused so it can be cleared using HAL (before hal is cleared).
   * It's ok for each core to clear memory marked as global, as long as it is also marked as unused */

  for (i = 0; dregMemClearRun[i].flags & DREG_MEM_CLEAR_FLAG_VALID; i++)  {

    if (dregMemClearRun[i].flags & DREG_MEM_CLEAR_FLAG_UNUSED)  {

      if (dregMemClearRun[i].flags & DREG_MEM_CLEAR_FLAG_DMA)  {

        gAddr = (*dsiuContext.hal->halMapLocalToGlobal)((halLocAddr_t)dregMemClearRun[i].sAddr);

        if ((dregMemClearRun[i].sizeBytes & 0x3) == 0)
          zsize = 4;
        else
          zsize = 1;

        (*dsiuContext.hal->halGlobMemSet) (gAddr, dregMemClearRun[i].sizeBytes, 0, zsize);

      }  else  {

        memset (dregMemClearRun[i].sAddr, SIU_BYTES_TO_WORDS(dregMemClearRun[i].sizeBytes), 0);

      }
    }
  }


  /* Core 0 can clear the DDR if required */
  if (dsiuAutoContext.core == 0)  {
    
    msg = (dmsg_t *)&dregMsgBlock[DSIU_ARUN_PREDEF_MSG_MEMTEST_GLOBAL];

    mSize = (((tulong)msg->msg_body.diag_code_control.ddr_size_msw) << 16) | msg->msg_body.diag_code_control.ddr_size_lsw;

    if ((mSize & 3) == 0)
      zsize = 4;
    else
      zsize = 0;
     
    gAddr = (((tulong)msg->msg_body.diag_code_control.ddr_start_addr_msw) << 16) | msg->msg_body.diag_code_control.ddr_start_addr_lsw;

    (*dsiuContext.hal->halGlobMemSet) (gAddr, mSize, 0, zsize);

  }

  /* Switch to the new stack. The old stack pointer is not saved,
   * so this is irreversable */
  dsiuArunSetStack ((tulong)(dsiuAstack + DSIU_ASTACK_SIZE_BYTES));

  /* Branch to the finishing code */
  dsiuSpinDry (dsiuAutoContext.core);


} /* dsiu_auto_run_spin */


/**********************************************************************
 * FUNCTION PURPOSE: Indicate restart message has been sent.
 **********************************************************************
 * DESCRIPTION: The stat reguster is updated to show restart has been
 *              sent.
 **********************************************************************/
void dsiuAutoRunSetStat (tuint v)
{

  /* Only wait for stat to clear if any commands have been processed */
  if (dsiuAutoContext.waitStatClear == TRUE)
    while (dregAutoCtl.stat != 0);

  /* Update the value */
  dregAutoCtl.stat = v;

} /* dsiuAutoRunRestartSent */


/**********************************************************************
 * FUNCTION PURPOSE: Instruct other cores to sleep
 **********************************************************************
 * DESCRIPTION: All other cores are instructed to run the idle
 *              message.
 **********************************************************************/
void dsiu_nmsgs_idle (tuint n)
{
  dregBlockMsgCntl_t  bcmd;
  halMemMove_t        mv;
  tint i;

  bcmd.firstMsg = DSIU_AURN_PREDEF_MSG_IDLE;
  bcmd.nMsg     = n;
  mv.src = (*dsiuContext.hal->halMapLocalToGlobal) ((halLocAddr_t)&bcmd);
  mv.size = sizeof(dregBlockMsgCntl_t);
   

  for (i = 0; i < dsiuAutoContext.ncores; i++)  {

    if (i != dsiuAutoContext.core)  {

      mv.dest = (*dsiuContext.hal->halFormGlobalAddr) ((halLocAddr_t)&dregBlockMsgCntlMem.u.cntrl, i);
      (*dsiuContext.hal->halGlobMemMove)(1, &mv, NULL, NULL, TRUE);
      
    }

  }

} /* dsiu_other_cores_sleep */


void dsiu_other_cores_sleep (void)
{
  dsiu_nmsgs_idle(1);

} /* dsiu_other_cores_sleep */

    
/***********************************************************************
 * FUNCTION PURPOSE: Wake up other cores
 ***********************************************************************
 * DESCRIPTION: All other cores are given a wakeup interrupt
 ***********************************************************************/
void dsiu_other_cores_wake (void)
{
  halControl_t ctl;
  volatile tuint i;
  dsiu_nmsgs_idle(0);
  
  for(i=0;i<100;i++);
  
  ctl.u.intCore.coreMask  = (1 << dsiuAutoContext.ncores) - 1;
  ctl.u.intCore.coreMask &= ~(1 << dsiuAutoContext.core);
  ctl.cmd = HAL_COMMAND_INT_OTHER_CORE;
  ctl.u.intCore.info = 0;

  (*dsiuContext.hal->halControl) (&ctl);

} /* dsiu_other_cores_wake */

/**********************************************************************
 * FUNCTION PURPOSE: check if other cores have claimed memory test
 **********************************************************************
 * DESCRIPTION: Returns TRUE if any other cores have claimed the memory
 *              test.
 **********************************************************************/
bool dsiu_ocore_mcheck (void)
{
  halControl_t hCtl;
  tint         i;

  hCtl.cmd = HAL_COMMAND_IPC_READ;

  /* Proceed only if all other cores indicate they are not running the test */
  for (i = 0; i < dsiuAutoContext.ncores; i++) {

    if (i != dsiuAutoContext.core)  {
      hCtl.u.ipcRead.core = i;
      (*dsiuContext.hal->halControl)(&hCtl);

      if (hCtl.u.ipcRead.msg & diagCfg.memTestMsg)  
        return (TRUE);

    }
  }

  return (FALSE);

} /* dsiu_ocore_mcheck */


/************************************************************************
 * FUNCTION PURPOSE: Check if this core has claimed the memory test
 ************************************************************************
 * DESCRIPTION: Returns true if this core has claimed the memory test
 ************************************************************************/
bool dsiu_thiscore_mcheck(void)
{

  halControl_t hCtl;

  hCtl.cmd = HAL_COMMAND_IPC_READ;
  hCtl.u.ipcRead.core = dsiuAutoContext.core;
  (*dsiuContext.hal->halControl)(&hCtl);

  if (hCtl.u.ipcRead.msg & diagCfg.memTestMsg)
    return (TRUE);

  return (FALSE);

} /* dsiu_thiscore_mcheck */

/*********************************************************************************************
 * FUNCTION PURPOSE: Begin memory test
 *********************************************************************************************
 * DESCRIPTION: The memory test is started if no other cores are trying to run the test.
 *********************************************************************************************/
void dsiu_auto_run_begin_memtest (void)
{
  dmsg_t       *msg;
  dmsg_t        msgMtest;
  halControl_t  hCtl;
  tulong        blkSize;
  volatile tint i;
 
  if (dsiuAutoContext.core == 0)
    msg = (dmsg_t *)&dregMsgBlock[DSIU_ARUN_PREDEF_MSG_MEMTEST_GLOBAL];
  else
    msg = (dmsg_t *)&dregMsgBlock[DSIU_ARUN_PREDEF_MSG_MEMTEST_LOCAL];


  /* Copy the message from the auto run block so values in the block itself will not change */
  msgMtest = *msg;

 
 /* Bypassing the ipc logic for memory bypass case setting the status done even for bypass test*/
 if(msg->msg_body.diag_code_control.control_code==0) {

    dsiuAutoContext.state = DSIU_AUTO_RUN_STATE_MEMTEST;
    dsiuContext.dregStatus->memTest.status = DREG_MEM_STATUS_DONE;

    return;
  
 }
 

  if (dsiu_thiscore_mcheck()) {
      hCtl.cmd = HAL_COMMAND_IPC_WRITE;
      hCtl.u.ipcWrite.core      = dsiuAutoContext.core;
      hCtl.u.ipcWrite.setMask   = 0;
      hCtl.u.ipcWrite.clearMask = diagCfg.memTestMsg;
      (*dsiuContext.hal->halControl) (&hCtl);
      return;
  }
  

  /* If another core has claimed the memory test just return. The other core has sent 
   * a sleep command which will be picked up on the next message polling pass */
  if (dsiu_ocore_mcheck ())
    return;


  /* Claim the memory test */
  do {
  
    hCtl.cmd = HAL_COMMAND_IPC_WRITE;
    hCtl.u.ipcWrite.core      = dsiuAutoContext.core;
    hCtl.u.ipcWrite.setMask   = diagCfg.memTestMsg;
    hCtl.u.ipcWrite.clearMask = 0;
    (*dsiuContext.hal->halControl) (&hCtl);
  } while(!dsiu_thiscore_mcheck());  

  /* Verify that no other cores have also claimed the memory test */
  if (dsiu_ocore_mcheck ())  {

    do  {


      hCtl.u.ipcWrite.setMask   = 0;
      hCtl.u.ipcWrite.clearMask = diagCfg.memTestMsg;
      (*dsiuContext.hal->halControl) (&hCtl);

    } while (dsiu_thiscore_mcheck());

    return;

  }  

  /* Ask the other cores to idle, give them some time to comply */
  dsiu_other_cores_sleep ();
  for (i = 0; i < 10000; i++);


  blkSize = ((tulong)msg->msg_body.diag_code_control.block_size_msw << 16) |
                     msg->msg_body.diag_code_control.block_size_lsw;

  /* If the block test size is 0 then don't do the external memory test. Store
   * the DDR size, then set it to zero to start the test, and restore it when
   * the test is complete */
  if (blkSize == 0)  {

    msgMtest.msg_body.diag_code_control.ddr_size_msw = 0;    
    msgMtest.msg_body.diag_code_control.ddr_size_lsw = 0;    

  }  else  {

    /* If blkSize is 1 run the memory test with the block size equal to the ddr size */
    msgMtest.msg_body.diag_code_control.block_size_msw = msg->msg_body.diag_code_control.ddr_size_msw;
    msgMtest.msg_body.diag_code_control.block_size_lsw = msg->msg_body.diag_code_control.ddr_size_lsw;

  }

  /* Must record in the context that the memory test is in progress because the memory test
   * returns to the main poll function to look for messages */
  dsiuAutoContext.state = DSIU_AUTO_RUN_STATE_MEMTEST;

  /* Run the memory test */
  dsiuProcessMessage (&msgMtest);

  /* Release the memory test hold */
  do  {

    hCtl.cmd                  = HAL_COMMAND_IPC_WRITE;
    hCtl.u.ipcWrite.setMask   = 0;
    hCtl.u.ipcWrite.clearMask = diagCfg.memTestMsg;
    (*dsiuContext.hal->halControl) (&hCtl);

  } while (dsiu_thiscore_mcheck());

  /* Wake up other cores */
  dsiu_other_cores_wake ();

  /* Return without changing the state. When the memory test is complete (detected through
   * status registers) the remaining commands are run */

} /* dsiu_auto_run_begin_memtest */



/**********************************************************************
 * FUNCTION PURPOSE: Process commands
 **********************************************************************
 * DESCRIPTION: Commands received through the command register are
 *              processed.
 **********************************************************************/
void dsiu_auto_run_get_cmd (void)
{
  volatile dmsg_t *msg;
  volatile tuint   i;
           tulong  blkSize;

  /* If any commands are received through the auto control, enable the wait
   * on stat clear. */
  if (dregAutoCtl.msg != 0)
    dsiuAutoContext.waitStatClear = TRUE;

  if ((dregAutoCtl.msg & 0xFF00) == (DSIU_AUTOCTL_CMD_HW_CONFIG & 0xFF00))  {

    dregAutoCtl.msg = 0;

    if (dsiuAutoContext.core == 0) 
      dsiuProcessMessage ((dmsg_t *)&dregMsgBlock[DSIU_ARUN_PREDEF_MSG_EMIF_CONFIG]);

    /* Core 0 MUST do tsip config before other cores. Add delay to make sure if a 
     * simultaneous start occurs core 0 will run through the test first. Otherwise
     * if core X runs before core 0, it releases the memory test, and core 0 will start.
     * But core 0 may get to the mem test before core X turns on the TDM. Then core 0
     * will test the local memory for core X when the TSIP is on, which will fail. */
    else
      for (i = 0; i < 30000; i++);  

    dsiuProcessMessage ((dmsg_t *)&dregMsgBlock[DSIU_ARUN_PREDEF_MSG_HW_CONFIG]);


    /* The DDR block test size must be either 0 or 1. If not either of those values
     * then wait */
    if (dsiuAutoContext.core == 0)
      msg = (volatile dmsg_t *)&dregMsgBlock[DSIU_ARUN_PREDEF_MSG_MEMTEST_GLOBAL];
    else
      msg = (volatile dmsg_t *)&dregMsgBlock[DSIU_ARUN_PREDEF_MSG_MEMTEST_LOCAL];

    do  {
      blkSize = ((tulong)msg->msg_body.diag_code_control.block_size_msw << 16) |
                         msg->msg_body.diag_code_control.block_size_lsw;

    } while ((blkSize != 0) && (blkSize != 1));


    /* Set the auto-run state to memtest begin. The auto-run code must exit so that message
     * processing occurrs. This allows for sleep messages to be processed */
    dsiuAutoContext.state = DSIU_AUTO_RUN_STATE_BEGIN_MEMTEST;


  }  else if (dregAutoCtl.msg == DSIU_AUTOCTL_CMD_SPIN) {

    dsiu_auto_run_spin ();

  }  else  {

    dregAutoCtl.msg = 0;
  }


} /* dsiu_auto_run_get_cmd */


/**********************************************************************
 * FUNCTION PURPOSE: Poll for auto run commands
 **********************************************************************
 * DESCRIPTION: The auto run commands are processed
 **********************************************************************/
void dsiuAutoRunPoll (void)
{

  /* Look for command if not currently processing a command sequence */
  if (dsiuAutoContext.state == DSIU_AUTO_RUN_STATE_WAIT)  {

    if (dregAutoCtl.msg != 0)
      dsiu_auto_run_get_cmd ();

  } else if (dsiuAutoContext.state == DSIU_AUTO_RUN_STATE_BEGIN_MEMTEST)  {

    /* Wait for the host to clear the ready indication */
    if (dregAutoCtl.stat == DSIU_AUTOCTL_STAT_READY)
      return;

    
    /* This delay is to ensure that core 0 gets to begin the memory test first */
    if(dsiuAutoContext.core!=0) {
      volatile tulong i;
      for(i=0;i<10000;i++);
    }

    dsiu_auto_run_begin_memtest ();

  } else if (dsiuAutoContext.state == DSIU_AUTO_RUN_STATE_MEMTEST)  {


    if (dregStatus.memTest.status != DREG_MEM_STATUS_IN_PROGRESS)  {

      dsiuProcessMessage ((dmsg_t *)&dregMsgBlock[DSIU_ARUN_PREDEF_MSG_PORTCFG]);

      dsiuAutoContext.state = DSIU_AUTO_RUN_STATE_WAIT;

      /* Restore the reboot information in case it resides in a scratch memory */
      dregReboot.zeroStatus  = DREG_REBOOT_ZERO_STATUS_ZERO_PENDING;
      dregReboot.bootAddress = 0;

    }

  }

} /* dsiuAUtoRunPoll */




/**********************************************************************
 * DEFINITION: The auto run call table
 **********************************************************************/
dsiuAutoRun_t dsiuAutoRun = {

  dsiuAutoRunStall,
  dsiuAutoRunInit,
  dsiuAutoRunSetStat,
  dsiuAutoRunPoll

};


