/**************************************************************************************
 * FILE PURPOSE: System level interface to the full application BERT unit
 **************************************************************************************
 * FILE NAME: dsiubert.c
 *
 * DESCRIPTION: Provides the layer interfacing the diagnostic code to the full 
 *				application BERT.
 *
 * Copyright (C) 2005, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include "types.h"
#include "dsiuloc.h"
#include "bert.h"
#include "diagcfg.h"
#include "memmgm.h"
#include "diagappif.h"
#include "dsiuotbrun.h"

#if 0
#define DEBUG_PRINT_BERT

#ifdef DEBUG_PRINT_BERT
#include <stdio.h>
#endif
#endif

/**************************************************************************************
 * Full application call table declaration
 **************************************************************************************/
gg_bert_ctable_t bertCallTable =
{
  bertReceiveIn,
  bertSendIn,
  bertPatternGenTel,
  bertPatternDetectTelFcn
};


/**************************************************************************************
 * FUNCTION PURPOSE: Allocate channel BERT instance memory.
 **************************************************************************************
 * DESCRIPTION: Takes heap memory to create a BERT channel. 
 **************************************************************************************/
void *dsiuCreateBert (tint ecoId)
{
  void *inst       = NULL;
  bertNewConfig_t    cfg;


  /* Initialize the config. The size config is a dummy value */
  cfg.ID            = ecoId;
  cfg.sizeCfg.dummy = 0;


  /* Call the full application interface function to setup the ECO. On error
   * inst will point to NULL */
  siuFullAppEcoNewEco ((dsiuEcoGetSizes_t)bertGetSizes, (void *)&cfg.sizeCfg, (dsiuEcoNew_t)bertNew,
                        (void *)&cfg, &inst, ecoId);

  return (inst);

} /* dsiuCreateBert */


/***************************************************************************************
 * FUNCTION PURPOSE: Receive a BERT report from the BERT module
 ***************************************************************************************
 * DESCRIPTION: Receives a report request, then gets the stats from the module
 ***************************************************************************************/
void dsiuBertReport (void *vinst, bool reset, tuint trans_id, tuint reason,
                     tuint priority, tuint priorityValidFlag)
{
  dsiuInst_t *inst = (dsiuInst_t *)vinst;
  bertStats_t *bstats;
  tuint stateChange;
  tuint syncState;
  

  DSIU_DUMMY_ARG(reset);
  DSIU_DUMMY_ARG(trans_id);
  DSIU_DUMMY_ARG(reason);
  DSIU_DUMMY_ARG(priority);
  DSIU_DUMMY_ARG(priorityValidFlag);

  /* Get the bert stats block from the dsiu instance ID */
  bstats = (bertStats_t *)&(dsiuContext.dregBertStats[inst->Id - 1]);

  bertGetStats (inst->bertInst, FALSE, bstats);

  /* report BERT stats only if there is a state change (sync<-->not sync)*/
  stateChange=FALSE;

  /*current sync state*/
  syncState = bstats->telRcvMode & MGB_BERT_TEL_RX_SYNC;
  if( syncState =! inst->bertTelRxSync )
  {
  #ifdef PA_SS
    if(inst->bertTelRxSync==FALSE)
      /*State changed from (out of sync)==>(sync)*/
      dsiuNwLog("BERT State Change for Channel %d: TEL BERT in SYNC\n",inst->Id-1);
    else
      /*State changed from (sync)==>(out of sync)*/
      dsiuNwLog("BERT State Change for Channel %d: TEL BERT out of SYNC\n",inst->Id-1);
  #endif
   
    /*update state*/
    inst->bertTelRxSync = syncState; 
    stateChange=TRUE;
  }

  /*current sync state*/
  syncState = bstats->netRcvMode & MGB_BERT_NET_RX_SYNC;
  if( syncState =! inst->bertNetRxSync )
  {
  #ifdef PA_SS
    if(inst->bertNetRxSync==FALSE)
      /*State changed from (out of sync)==>(sync)*/
      dsiuNwLog("BERT State Change for Channel %d: NET BERT in SYNC\n",inst->Id-1);
    else
      /*State changed from (sync)==>(out of sync)*/
      dsiuNwLog("BERT State Change for Channel %d: NET BERT out of SYNC\n",inst->Id-1);
  #endif
    
    /*update state*/
    inst->bertNetRxSync = syncState; 
    stateChange=TRUE;
  }
  
  if(stateChange || dregBertTestCntl.periodicReport)
  { 
  #ifdef PA_SS
    dsiuNwLog ("Stats for Channel %d:\n"         ,inst->Id - 1);
    dsiuNwLog ("Tel Sync Loss Num: %d\n",bstats->telSyncLostNum);
    dsiuNwLog ("Tel BER: %d\n"          ,bstats->telBerCurrent);
    dsiuNwLog ("Tel Receive Bits: %d\n" ,bstats->telRcvBits);
    dsiuNwLog ("Tel Error Bits: %d\n"   ,bstats->telErrBits);
    dsiuNwLog ("Tel Error Seconds: %d\n",bstats->telErrSeconds);
    dsiuNwLog ("NetSyncLostNum: %d\n"   ,bstats->netSyncLostNum);
    dsiuNwLog ("Net BER: %d\n"          ,bstats->netBerCurrent);
    dsiuNwLog ("Net Receive Bits: %d\n" ,bstats->netRcvBits);
    dsiuNwLog ("Net Error Bits: %d\n"   ,bstats->netErrBits);
    dsiuNwLog ("Net Error Seconds: %d\n",bstats->netErrSeconds);
    dsiuNwLog ("\n");
    dsiuNwLog ("\n");
  #else
   #ifdef DEBUG_PRINT_BERT
    printf ("Stats for Channel %d:\n"         ,inst->Id - 1);
    printf ("Tel Sync Loss Num: %d\n",bstats->telSyncLostNum);
    printf ("Tel BER: %d\n"          ,bstats->telBerCurrent);
    printf ("Tel Receive Bits: %d\n" ,bstats->telRcvBits);
    printf ("Tel Error Bits: %d\n"   ,bstats->telErrBits);
    printf ("Tel Error Seconds: %d\n",bstats->telErrSeconds);
    printf ("NetSyncLostNum: %d\n"   ,bstats->netSyncLostNum);
    printf ("Net BER: %d\n"          ,bstats->netBerCurrent);
    printf ("Net Receive Bits: %d\n" ,bstats->netRcvBits);
    printf ("Net Error Bits: %d\n"   ,bstats->netErrBits);
    printf ("Net Error Seconds: %d\n",bstats->netErrSeconds);
    printf ("\n");
    printf ("\n");
   #endif 
  #endif
  }

} /* dsiuBertReport */

/***************************************************************************************
 * FUNCTION PURPOSE: Clear stats
 ***************************************************************************************
 * DESCRIPTION: Clear BERT stats for a channel
 ***************************************************************************************/
void dsiuBertClearStats (void *vinst)
{
  dsiuInst_t *inst = (dsiuInst_t *)vinst;

  bertStats_t *bstats;

  /* Get the bert stats block from the dsiu instance ID */
  bstats = (bertStats_t *)&(dsiuContext.dregBertStats[inst->Id - 1]);
  bertGetStats (inst->bertInst, TRUE, bstats);

} /* dsiuBertClearStats */
  



/***************************************************************************************
 * FUNCTION PURPOSE: Open a BERT channel
 ***************************************************************************************
 * DESCRIPTION: A BERT channel is opened. It is not inserted into the flow path. That's
 *              done in the channel configuration registers.
 ***************************************************************************************/
dsiuReturn_t dsiuOpenBert (void *vinst, tint mode)
{
  dsiuInst_t *inst = (dsiuInst_t *)vinst;
  bertOpenConfig_t   cfg;
  tint               bPattern;

  /* Standard bert configs. These are currently hard coded and unchangable. This
   * configuration is designed for QT testing with the tdm and network interface
   * in loopback mode. The bert pattern is generated to the TDM where it is expected
   * to be looped back. It is then sent to the network where it is expected to be
   * looped back, where it is received and the bert pattern is checked */
  cfg.moduleConfig = MGB_BERT_TELE_TX_ENABLE | MGB_BERT_TELE_RX_ENABLE |
                     MGB_BERT_NET_TX_ENABLE | MGB_BERT_NET_RX_ENABLE;

  cfg.measureInterval = 1;  /* A very short interval for QT testing */

  cfg.telUpperThreshold = 10000;
  cfg.telLowerThreshold = 1000;

  cfg.netUpperThreshold = 10000;
  cfg.netLowerThreshold = 1000;

  cfg.statsReportMode = MGB_BERT_STATS_PERIODIC;

/*
#define MGB_BERT_PATTERN_RANDOM     0x0000
#define MGB_BERT_PATTERN_255        0x0001
#define MGB_BERT_PATTERN_RANDOM_ID  0x0002  */
  switch (mode)  {

    case DREG_ROUTE_OUT_SRC_BERT_RANDOM:
         bPattern = 0;
         break;

    case DREG_ROUTE_OUT_SRC_BERT_255:
         bPattern = 1;
         break;

    case DREG_ROUTE_OUT_SRC_BERT_RND_ID:
         bPattern = 2;
         break;

    default: return (DSIU_FAIL);

  }

  cfg.netTestPattern = bPattern;  /* Standard 32768 pattern */
  cfg.telTestPattern = bPattern;  /* Standard 32768 pattern */

  cfg.bitsPerSample = 0;   /* 0 is for 8 bits per sample */

  cfg.reportInterval = 2;  /* Seconds for reports */

  cfg.netChannelID = inst->Id;
  cfg.telChannelID = inst->Id;

  cfg.statsReportFn = dsiuBertReport;
  cfg.statsReportTargetInst = vinst;

  bertOpen (inst->bertInst, &cfg);

  return (DSIU_SUCCESS);

} /* dsiuBertOpen */


/**********************************************************************************
 * FUNCTION PURPOSE: Close a BERT channel
 **********************************************************************************
 * DESCRIPTION: Passes the close to the BERT module.
 **********************************************************************************/
dsiuReturn_t dsiuCloseBert (void *vinst)
{
  dsiuInst_t *inst = (dsiuInst_t *)vinst;

  bertClose (inst->bertInst);

  return (DSIU_SUCCESS);

} /* dsiuCloseBert */



