/************************************************************************
:*                                                                      *
:*     @ Copyright (C) 2009  Texas Instruments Incorporated.            *
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

/**************************************************************************************************
 * FILE PURPOSE: Perform block message processing
 **************************************************************************************************
 * FILE NAME: dsiublkmsg.c
 *
 * DESCRIPTION: Block message processing
 *
 **************************************************************************************************/
#include "types.h"
#include "dsiuloc.h"
#include "diagreg.h"
#include "hal.h"
#include "diagcfg.h"


/**************************************************************************************************
 * FUNCTION PURPOSE: Peform block message processing
 **************************************************************************************************
 * DESCRIPTION: Process block messages. Because this allows one core to send a message to another
 *              the cache is managed.
 **************************************************************************************************/
void dsiuBlockProc (void)
{
  halCacheControl_t ctl;
  bool              wb;

  /* The block message allows a core to have another core execute a message.
   * Because of this the cache must be invalidated before checking the message count.
   * When this core makes any changes to the message count the cache must be 
   * written back. */
  if (diagCfg.blkMsgCacheFlush)  {
    ctl.cacheCtrl       = HAL_CACHE_CTRL_INVALIDATE;
    ctl.u.wbi.addr      = (tulong)&dregBlockMsgCntlMem;
    ctl.u.wbi.sizeBytes = SIU_WORDS_TO_BYTES(sizeof(dregBlockMsgCntlMem_t));
    (*dsiuContext.hal->halCacheControl) (&ctl);
  }

  if (dregBlockMsgCntlMem.u.cntrl.nMsg != 0)
    wb = TRUE;
  else
    wb = FALSE;

  /* If a message sequence is active do not process messages from
   * other sources until the entire sequence is complete. Note that
   * the memory test message will re-enter the polling function. This
   * should work, but message after mem test will complete before the
   * mem test does. */
  while (dregBlockMsgCntlMem.u.cntrl.nMsg != 0)  {

    if (dregBlockMsgCntlMem.u.cntrl.firstMsg > diagCfg.nBlockMessages)
      dregBlockMsgCntlMem.u.cntrl.nMsg = 0;

    else  {

      if (diagCfg.blkMsgCacheFlush)  {
        ctl.cacheCtrl       = HAL_CACHE_CTRL_INVALIDATE;
        ctl.u.wbi.addr      = (tulong)&dregMsgBlock[dregBlockMsgCntlMem.u.cntrl.firstMsg];
        ctl.u.wbi.sizeBytes = SIU_TUINT_TO_BYTES(sizeof(DREG_MSG_BLOCK_SIZE_TUINT));
        (*dsiuContext.hal->halCacheControl) (&ctl);
      }

      dsiuProcessMessage ((dmsg_t *)&dregMsgBlock[dregBlockMsgCntlMem.u.cntrl.firstMsg++]);

    }

  }

  /* Do not writeback the block message count unless it has been changed in this
   * function. If it is always written back this would result in a race with
   * other cores trying to write the value. There is still a race if two cores
   * try to write to one core at the same time, and no cross core semaphore is used. */
  if (wb && diagCfg.blkMsgCacheFlush)  {
    ctl.cacheCtrl       = HAL_CACHE_CTRL_WRITEBACK;
    ctl.u.wbi.addr      = (tulong)&dregBlockMsgCntlMem;
    ctl.u.wbi.sizeBytes = SIU_WORDS_TO_BYTES(sizeof(dregBlockMsgCntlMem_t));
    (*dsiuContext.hal->halCacheControl) (&ctl);
  }

} /* dsiuBlockProc */



/******************************************************************************************
 * FUNCTION PURPOSE: Initialize the block message parameters
 ******************************************************************************************
 * DESCRIPTION: The block message parameters are initialized to show no pending
 *              block messages.
 ******************************************************************************************/
void dsiuBlockInit (void)
{
  halCacheControl_t ctl;


  dregBlockMsgCntlMem.u.cntrl.firstMsg = 0;
  dregBlockMsgCntlMem.u.cntrl.nMsg     = 0;
  

  /* Flush the master control from the cache. This must be done since the
   * cache will be invalidated before the first read */
  ctl.cacheCtrl       = HAL_CACHE_CTRL_WRITEBACK;
  ctl.u.wbi.addr      = (tulong)&dregBlockMsgCntlMem;
  ctl.u.wbi.sizeBytes = SIU_WORDS_TO_BYTES(sizeof(dregBlockMsgCntlMem_t));
  (*dsiuContext.hal->halCacheControl) (&ctl);

} /* dsiuBlockInit */





