/**********************************************************************************
 * FILE PURPOSE: Generic peripheral boot support functions
 **********************************************************************************
 * FILE NAME: dsiuboot.c
 *
 * DESCRIPTION: Contains functions common to multiple boot modes
 *
 * TABS: None
 *
 * Copyright (C) 2004, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include "types.h"
#include "dsiuloc.h"
#include "diagapp.h"
#include "diagreg.h"
#include "diagver.h"
#include "diagcfg.h"
#include <ti/mas/neu/neuproto.h>
#include "dsiuport.h"
#include "utlrepck_m.h"
#include "dsiuotbrun.h"
#include <stdlib.h>

/*****************************************************************************
 * FUNCTION PURPOSE: Copy the configuration values to memory
 *****************************************************************************
 * DESCRIPTION: Parameters from the boot config cell are stored in memory
 *              to be accessed by the application.
 *****************************************************************************/
void dsiu_boot_copy_info (void)
{
  halLocAddr_t    dst, src;
  halMemMove_t    memMove;

  /* Copy the boot configuration to memory */
  /* The source is always in data memory */
  src = (halLocAddr_t)&dsiuContext.diagApp;
  src = hal.halMapLocalToGlobal(src);
  /*  HAL_MEM_ADDRESS_DATA(src); */

  /* The destination is always in shared program memory */
  dst = (halLocAddr_t)&diagAppMem;
/*  HAL_MEM_SHARED_ADDR(dst);
  HAL_MEM_ADDRESS_PROG(dst); */
  dst = hal.halMapLocalToGlobal(dst);

  /* Invoke the HAL to copy the data */
  memMove.dest = dst;
  memMove.src  = src;
  memMove.size = sizeof(diagApp_t);
  hal.halGlobMemMove(1, &memMove, NULL, NULL, TRUE);

} /* dsiu_ubt_copy_info */


/****************************************************************************
 * FUNCTION PURPOSE: Perform a ones complement addition
 ****************************************************************************
 * DESCRIPTION: The ones complement sum of two 16 bit values is returned.
 ****************************************************************************/
tuint dsiu_onesc_add (tuint value1, tuint value2)
{
  tulong result;

  result = (tulong)value1 + (tulong)value2;

  result = (result >> 16) + (result & 0xFFFF); /* add in carry   */
  result += (result >> 16);                    /* maybe one more */
  return (tuint)result;

} /* dsiu_onesc_add */


/*****************************************************************************
 * FUNCTION PURPOSE: Read boot configuration from global memory
 *****************************************************************************
 * DESCRIPTION: This function is used to simulate the application. It
 *              reads the configuration stored in shared memory by the 
 *              bootloader.
 ****************************************************************************/
#ifdef __TMS320C55X__
void lpdmaReset(void);  /* Should be in the hal call table */
#endif
dsiuReturn_t dsiuSimuAppConfig (void)
{
#ifdef __TMS320C55X__
  halLocAddr_t    dst, src;
  halMemMove_t    memMove;
  halControl_t    hctl;
#endif
  tint            offset;
  tint            offsetc;
  tint            coreId;
  tint            shift;
  tuint           chksum;
  tuint           port;
  word *hdr_image;

  /**************************************************************************
   * The 55x doesn't have a struct align pragma, so verify alignment is
   * correct.
   ***************************************************************************/
#ifdef __TMS320C55X__
  if ((tulong)&(dsiuContext.diagApp) & 1)
  dsiuPanic (DSIU_FAIL_DIAGAPP_ALIGN);

  /*********************************************************************
   * The lpdma must be reset - this is a direct call that should be
   * call tabled, but it is to work around a Janus initialization bug
   **********************************************************************/
  lpdmaReset();
#endif

  /***************************************************************************
   * Boot progress track: Indicate in dsiuSimuAppConfig
   ***************************************************************************/
  dsiuBootTrack (DREG_BOOT_TRACK_SIMU_APP_CONFIG);

#ifndef __TMS320C55X__
    memcpy(&dsiuContext.diagApp,&diagAppMem,sizeof(diagApp_t));
#else
    /* Reset the DMAs. There is no chance the DMA has been setup for a background
     * operation at this point */
    hctl.cmd = HAL_COMMAND_DMA_RESET;
     (*hal.halControl)(&hctl);
  
    /* Copy the boot configuration from shared program memory to local memory */
    /* The destination is always in data memory */
    dst = hal.halMapLocalToGlobal((halLocAddr_t)&dsiuContext.diagApp);
    HAL_MEM_ADDRESS_DATA(dst);

    /* The source is always in shared program memory */
    src = hal.halMapLocalToGlobal((halGlobAddr_t)&diagAppMem);
  //  HAL_MEM_SHARED_ADDR(src);
    HAL_MEM_ADDRESS_PROG(src);

    /* Invoke the HAL to copy the data. HAL will stall until the transfer is complete */
    memMove.dest = dst;
    memMove.src  = src;
    memMove.size = sizeof(diagApp_t);
    (*dsiuContext.hal->halGlobMemMove)(1, &memMove, NULL, NULL, TRUE);
#endif
  
  /* For utopia boot, check that the cell size can be handled by this program */
  if (dsiuContext.diagApp.route == DIAGAPP_ROUTE_UTOPIA)  {
    if ((dsiuContext.diagApp.u.utopia.cellSize != 58) && 
        (dsiuContext.diagApp.u.utopia.cellSize != 54) &&
        (dsiuContext.diagApp.u.utopia.cellSize != 0) )
      return (DSIU_FAIL);
  }

  /* For mac boot the mac address and destination of the default header
   * must be replaced with the core specific version */
  if (dsiuContext.diagApp.route == DIAGAPP_ROUTE_GMAC)  {
   
    coreId = dsiuContext.coreId;
	hdr_image=(word *)&dsiuContext.diagApp.u.mac.msg.hdrImage[0];
    offset = dsiuContext.diagApp.u.mac.msg.ethOffsetBytes + neuproto_ETH_BYTE_OFFSET_SRC_MAC;
    if (dsiuContext.diagApp.u.mac.msg.ethOffsetBytes < 0)
      offset = neuproto_ETH_BYTE_OFFSET_SRC_MAC;

	utlWrite16bits_m(hdr_image,offset,dsiuContext.diagApp.u.mac.macAddr[coreId].macAddr[0]);
	utlWrite16bits_m(hdr_image,offset+2,dsiuContext.diagApp.u.mac.macAddr[coreId].macAddr[1]);
	utlWrite16bits_m(hdr_image,offset+4,dsiuContext.diagApp.u.mac.macAddr[coreId].macAddr[2]);

    /* The udp checksum must be updated. The pseudo header checksum is already included,
     * so the current source IP address must be subtracted, and the new IP address
     * added. */
    if ( (dsiuContext.diagApp.u.mac.msg.ipOffsetBytes >= 0)  && (dsiuContext.diagApp.u.mac.msg.udpOffsetBytes >= 0) ) {
      offsetc = dsiuContext.diagApp.u.mac.msg.udpOffsetBytes + neuproto_UDP_BYTE_OFFSET_CHKSUM;
      chksum  = utlRead16bits_m(hdr_image,offsetc);
      offset  = dsiuContext.diagApp.u.mac.msg.ipOffsetBytes + neuproto_IP_BYTE_OFFSET_SRC_ADDR;
      chksum  = dsiu_onesc_add (chksum, ~(utlRead16bits_m(hdr_image,offset)));
      chksum  = dsiu_onesc_add (chksum, ~(utlRead16bits_m(hdr_image,offset+2)));

      /* Change the default udp source port based on the core number, if enabled */
      if (diagCfg.neuUdpSrcMask)  {
        offset = dsiuContext.diagApp.u.mac.msg.udpOffsetBytes + neuproto_UDP_BYTE_OFFSET_SRC_PORT;
        port   = utlRead16bits_m(hdr_image,offset);
        port   = port & ~diagCfg.neuUdpSrcMask;
        for (shift = 0; (((diagCfg.neuUdpSrcMask >> shift) & 1) == 0); shift++);
        port   = port | (dsiuContext.coreId << shift);
        utlWrite16bits_m(hdr_image,offset,port);
      }
    }

#ifdef IP_BOOT_SUPP
#else /* IP_BOOT_SUPP */
    if (dsiuContext.diagApp.u.mac.msg.ipOffsetBytes >= 0)  {
      offset = dsiuContext.diagApp.u.mac.msg.ipOffsetBytes + neuproto_IP_BYTE_OFFSET_SRC_ADDR;
      utlWrite16bits_m(hdr_image,offset,dsiuContext.diagApp.u.mac.ipAddr[coreId].srcIpHigh);
      utlWrite16bits_m(hdr_image,offset+2,dsiuContext.diagApp.u.mac.ipAddr[coreId].srcIpLow);
    }
#endif /* IP_BOOT_SUPP */

    if ( (dsiuContext.diagApp.u.mac.msg.ipOffsetBytes >= 0)  && (dsiuContext.diagApp.u.mac.msg.udpOffsetBytes >= 0) ) {
      chksum = dsiu_onesc_add (chksum, utlRead16bits_m(hdr_image,offset));
      chksum = dsiu_onesc_add (chksum, utlRead16bits_m(hdr_image,offset+2));
      utlWrite16bits_m(hdr_image,offsetc,chksum);
    }


  }
  

  return (DSIU_SUCCESS);

} /* dsiuSimuAppConfig */

/******************************************************************************
 * FUNCTION PURPOSE: Generate a boot acknowledgement message
 ******************************************************************************
 * DESCRIPTION: After the system has configured the system must send a boot
 *              acknowledgment to the host through the system message
 *              interface.
 *
 *              entry_code is 0 if the boot was successful. An error 
 *              indication is sent if the value is non-zero.
 *****************************************************************************/
void dsiuBootAck (tint entry_code)
{

  /* For utopia send an extra first cell since the first one seems to be eaten */
  if ( (dsiuContext.dregMsgCfgShadow->route == DIAGAPP_ROUTE_UTOPIA) && (diagCfg.ctable.siuUtNdu) )  {

    dsiuContext.msgTxBuf->header.length   = DSIU_MSG_HEADER_SIZE;
    dsiuContext.msgTxBuf->header.chnum    = 0;
    dsiuContext.msgTxBuf->header.msg_id   = DMSG_DH_MSG_ID_DEBUG_TRACE;
    dsiuContext.msgTxBuf->header.trans_id = 0;
  
    (*diagCfg.ctable.siuUtNdu->nduPushFifo)((void *)dsiuContext.msgTxBuf);

  }


  if (!entry_code)  {
    /* Generate the message in the system message assembly buffer */
    dsiuContext.msgTxBuf->header.length   = DSIU_MSG_HEADER_SIZE + 
                                            DSIU_MSG_BOOT_ACK_SIZE;
    dsiuContext.msgTxBuf->header.chnum    = 0;
    dsiuContext.msgTxBuf->header.msg_id   = DMSG_DH_MSG_ID_BOOT_ACK;
    dsiuContext.msgTxBuf->header.trans_id = 0;
  
    /* Generate the message body */
    dsiuContext.msgTxBuf->msg_body.boot_ack.version = DIAG_VERSION;

  } else {

    /* Generate and error indication message */
    dsiuContext.msgTxBuf->header.length = DSIU_MSG_HEADER_SIZE +
                                          DSIU_MSG_ERROR_INDICATION_SIZE;
    dsiuContext.msgTxBuf->header.chnum  = 0;
    dsiuContext.msgTxBuf->header.msg_id = DMSG_DH_MSG_ID_ERROR_INDICATION;
    dsiuContext.msgTxBuf->msg_body.error_indication.error_code =
      MGB_ERR_CORE_BOOT_FAIL;
    dsiuContext.msgTxBuf->header.trans_id = 0;
  }

  /* Send the message through the appropriate interface */
  if ( (dsiuContext.dregMsgCfgShadow->route == DIAGAPP_ROUTE_UTOPIA) && (diagCfg.ctable.siuUtNdu) )
    (*diagCfg.ctable.siuUtNdu->nduPushFifo)((void *)dsiuContext.msgTxBuf);
  else if ( (dsiuContext.dregMsgCfgShadow->route == DIAGAPP_ROUTE_GMAC) && (diagCfg.ctable.siuGmacNdu) )
    (*diagCfg.ctable.siuGmacNdu->nduPushFifo)((void *)dsiuContext.msgTxBuf);
  else
    dsiuPanic (DSIU_FAIL_NO_BOOT_REPLY_IF);

} /* dsiuBootAck */

