/*****************************************************************************
 * FILE PURPOSE: Diagnostic code initialization routines
 *****************************************************************************
 * FILE NAME: dsiudini.c
 *
 * DESCRIPTION: Functions required to initialize the diagnostic code
 * 
 * TABS: None
 *
 * Copyright (C) 2002, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include "types.h"
#include "diagcfg.h"
#include "dsiuloc.h"
#include "dsiumsg.h"
#include "hal.h"
#include "dsiuubt.h"

#include "diagbld.h"
#include "gghwcfg.h"
#include "diagreg.h"
#include "dsiumem.h"
#include <stdlib.h>
#include <string.h>

/****************************************************************************
 * FUNCTION PURPOSE: Allocate memory for the diagnostic DSIU modules
 ****************************************************************************
 * DESCRIPTION: Allocates and initializes memory for the diagnostic
 *              dsiuInst instance pointers.
 ****************************************************************************/
void dsiuAllocateDiagMemory (void)
{
  dpvpContextConfig_t dpvpCfg;
  tuint heapReq, *heap;

  /**************************************************************************
   * If Utopia ports or Gmac ports are used allocate the rx task buffer and 
   * cell/packet buffer.
   * The receive message fifo was allocated by the bootloader.
   **************************************************************************/
  if ( (diagCfg.ctable.siuUtNdu) || (diagCfg.ctable.siuGmacNdu) ) {
    dsiuContext.rxNetTasks = dsiuTaskAlloc (&dsiuContext.diagHeap, 
                                            4 * dsiuContext.dregStatus->nChannels);
    if (dsiuContext.rxNetTasks == NULL)
      dsiuPanic (DSIU_MEM_ALLOC_FAIL_RX_UTOPIA_TASKS);

    dsiuContext.netFifo = (dsiuFifo_t *)dsiuFifoAlloc (&dsiuContext.diagHeap, 
                                        diagCfg.netFifoSize);
    if (dsiuContext.netFifo == NULL)
      dsiuPanic (DSIU_MEM_ALLOC_FAIL_NET_AAL1_FIFO);

    dsiuFifoReset (dsiuContext.netFifo);
  }

  /*************************************************************************
   * If TDM ports are used allocate TDM tx and rx task buffers
   *************************************************************************/
  if (diagCfg.ctable.dpiu)  {
    dsiuContext.txPcmTasks = dsiuTaskAlloc (&dsiuContext.diagHeap, 
                                            10 * dsiuContext.dregStatus->nChannels);
    if (dsiuContext.txPcmTasks == NULL)
      dsiuPanic (DSIU_MEM_ALLOC_FAIL_TX_PCM_TASKS);  /* No Return */

    dsiuContext.rxPcmTasks = dsiuTaskAlloc (&dsiuContext.diagHeap,
                                            10 * dsiuContext.dregStatus->nChannels);
    if (dsiuContext.rxPcmTasks == NULL)
      dsiuPanic (DSIU_MEM_ALLOC_FAIL_RX_PCM_TASKS);  /* No Return */
  }
  else  {
    dsiuContext.txPcmTasks = dsiuContext.rxPcmTasks = NULL;
  }

  /**************************************************************************
   * The utopia and message buffers were initialized by the bootloader
   **************************************************************************/

  /**************************************************************************
   * Allocate memory for channel instance array
   **************************************************************************/
  dsiuInst = (dsiuInst_t *)dsiuHeapAlloc(&dsiuContext.diagHeap, 
                   dsiuContext.dregStatus->nChannels * sizeof (dsiuInst_t));
  if (dsiuInst == NULL)
    dsiuPanic (DSIU_MEM_ALLOC_FAIL_SIU_INST);

  /* Clear Context errors */
  memset (&dsiuContext.errs, 0, sizeof (dsiuContextErrs_t));

#if 0
  /***************************************************************************
   * Allocate memory for dneu if included 
   ***************************************************************************/
  if (diagCfg.ctable.dneu)  {
    dneuCfg.maxTxPacketSizeWords = DSIU_MAX_PKT_SIZE_WORDS;
    heapReq = (*diagCfg.ctable.dneu->dneuGetContextSizes)(&dneuCfg);
    heap    =  (void *)dsiuHeapAlloc (&dsiuContext.diagHeap, heapReq);
    if (heap == NULL)
      dsiuPanic (DSIU_MEM_ALLOC_FAIL_DNEU_CONTEXT_HEAP);
    (*diagCfg.ctable.dneu->dneuContextInit) (&dneuCfg, heap);
  }
#endif

  /***************************************************************************
   * Allocate memory for dpvp if included
   ***************************************************************************/
  if (diagCfg.ctable.dpvp)  {
    /* No size configuration required for pvp context */
    heapReq = (*diagCfg.ctable.dpvp->dpvpGetContextSizes)(&dpvpCfg);
    heap    = (void *)dsiuHeapAlloc (&dsiuContext.diagHeap, heapReq);
    if (heap == NULL)
      dsiuPanic (DSIU_MEM_ALLOC_FAIL_DPVP_CONTEXT_HEAP);
    (*diagCfg.ctable.dpvp->dpvpContextInit)(&dpvpCfg, heap);
  }

} /* dsiuAllocateDiagMemory */

/****************************************************************************
 * FUNCTION PURPOSE: Allocate memory for the channel and module instances
 ****************************************************************************
 * DESCRIPTION: Allocates and initializes instance and module memory
 ****************************************************************************/
void dsiuInitChannel (tint ch)
{
  dsiuInst_t  *inst;
  dmemBuffer_t *memb;
  tint         nbufs;
  union {
    dpiuNewConfig_t dpiuNewConfig;
    dvpuNewConfig_t dvpuNewConfig;
    dtguNewConfig_t dtguNewConfig;
    dsiuNduConfig_t dsiuNduConfig;
    dpvpNewConfig_t dpvpNewConfig;
 } u;
  
  inst = &dsiuInst[ch];

  /* Initialize channel state */
  inst->Id    = ch+1;     /* Ids are indexed from 1 */
  inst->state = DSIU_STATE_CLOSED;
  inst->chan_bfield2 = 0;

  /* Allocate memory for the channel error stats */
  inst->errs = (dsiuChErrs_t *)dsiuHeapAlloc (&dsiuContext.diagHeap, 
                                               sizeof (dsiuChErrs_t));
  if (inst->errs == NULL)
    dsiuPanic (DSIU_MEM_ALLOC_FAIL_CH_ERRS);
  memset (inst->errs, 0, sizeof (dsiuChErrs_t));

  /* Initialize subinstance pointers */
  inst->dvpuInst  = NULL;
  inst->dpiuInst  = NULL;
  inst->bertInst  = NULL;
  inst->dpvpInst  = NULL;
  inst->dtguInst  = NULL;
  inst->neuInst   = NULL; 

  /* Allocate memory for modules */
  /**************************************************************************
   *                          DPIU                                          *
   **************************************************************************/
  if (diagCfg.ctable.dpiu)  {
    u.dpiuNewConfig.sizeCfg.tdmBufSize = diagCfg.tdmBufSize;
    (*diagCfg.ctable.dpiu->dpiuGetSizes) (&nbufs, &memb, 
                                          &u.dpiuNewConfig.sizeCfg);
    if (dsiuAllocBufs (&dsiuContext.diagHeap, nbufs, memb))
      dsiuPanic (DSIU_MEM_ALLOC_DPIU_ALLOC_FAIL);
    u.dpiuNewConfig.Id = ch + 1;
    u.dpiuNewConfig.taskInst = inst;
    u.dpiuNewConfig.txTaskCall = dsiuPostDpiuTxTask;
    u.dpiuNewConfig.rxTaskCall = dsiuPostDpiuRxTask;
    inst->dpiuInst = (*diagCfg.ctable.dpiu->dpiuNew) (&u.dpiuNewConfig, memb);
  }

  /***************************************************************************
   *                        DVPU                                             *
   ***************************************************************************/
  if (diagCfg.ctable.dvpu)  {
    u.dvpuNewConfig.sizeCfg.circBufSize = diagCfg.vpuBufSize;
    (*diagCfg.ctable.dvpu->dvpuGetSizes) (&nbufs, &memb, 
                                         &u.dvpuNewConfig.sizeCfg);
    if (dsiuAllocBufs (&dsiuContext.diagHeap, nbufs, memb))
      dsiuPanic (DSIU_MEM_ALLOC_DVPU_ALLOC_FAIL);
    u.dvpuNewConfig.Id = ch + 1;
    inst->dvpuInst = (*diagCfg.ctable.dvpu->dvpuNew) (&u.dvpuNewConfig, memb);
  }
  
  /***************************************************************************
   *                       DTGU                                              *
   ***************************************************************************/
  if (diagCfg.ctable.dtgu)  {
    /* No size configuration info required */
    (*diagCfg.ctable.dtgu->dtguGetSizes) (&nbufs, &memb, 
                                          &u.dtguNewConfig.sizeCfg);
    if (dsiuAllocBufs (&dsiuContext.diagHeap, nbufs, memb))
      dsiuPanic (DSIU_MEM_ALLOC_DTGU_ALLOC_FAIL);
    u.dtguNewConfig.Id = ch + 1;
    inst->dtguInst = (*diagCfg.ctable.dtgu->dtguNew)(&u.dtguNewConfig, memb);
  }

  /***************************************************************************
   *                       NEU                                               *
   ***************************************************************************/
  if ( (diagCfg.ctable.neu)  && (diagCfg.ctable.dsiu->createVoiceNeu) ) {
    inst->neuInst = (*diagCfg.ctable.dsiu->createVoiceNeu)(ch+1);
  } 

  /***************************************************************************
   *                       BERT                                              *
   ***************************************************************************/
  if (  (diagCfg.ctable.bert)  &&  (*diagCfg.ctable.dsiu->createBert) )  {
    inst->bertInst = (*diagCfg.ctable.dsiu->createBert)(ch+1);
    inst->bertTelRxSync=FALSE; /*BERT is not in sync at creation*/
    inst->bertNetRxSync=FALSE; /*BERT is not in sync at creation*/
  }
    
  /****************************************************************************
   *                            DPVP                                          *
   ****************************************************************************/
  if (diagCfg.ctable.dpvp)  {
    /* No size configuration required */
    (*diagCfg.ctable.dpvp->dpvpGetSizes) (&nbufs, &memb,
                                          &u.dpvpNewConfig.sizeCfg);
    if (dsiuAllocBufs(&dsiuContext.diagHeap, nbufs, memb))
      dsiuPanic (DSIU_MEM_ALLOC_DPVP_ALLOC_FAIL);
    u.dpvpNewConfig.Id = ch + 1;
    inst->dpvpInst = (*diagCfg.ctable.dpvp->dpvpNew)(&u.dpvpNewConfig, memb);
  }

  /***************************************************************************
   * Initialize NDU to call neuReceiveInUt when a data cell is received
   * The utopia interface will be the default for channel data
   ***************************************************************************/
  if (diagCfg.ctable.siuUtNdu)  {
    /* The callout instance is the dsiu instance pointer */
    u.dsiuNduConfig.recInterface            = SIU_NDU_TYPE_UT;
    u.dsiuNduConfig.RxOutSpec.targetInst    = inst->neuInst;
    u.dsiuNduConfig.RxOutSpec.nduReceiveOut = diagCfg.ctable.neu->neuReceiveInUt;
    (*diagCfg.ctable.siuUtNdu->nduOpen)(inst->Id, &u.dsiuNduConfig);
  }

    
} /* dsiuInitChannel */


