/******************************************************************************
 * FILE PURPOSE: System FIFO implementation
 ******************************************************************************
 * FILE NAME: dsiufifo.c
 *
 * DESCRIPTION: Implements a basic FIFO
 *
 * TABS: None
 *
 * Copyright (C) 2002, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include "dsiufifo.h"

/******************************************************************************
 * FUNCTION PURPOSE: Return the amount of space available in the fifo
 ******************************************************************************
 * DESCRIPTION: The amount of space available in the fifo is one element less
 *              the fifo capacity. The fifo is not allowed to "fill" since
 *              no mechanism is included to then distinguish between an empty
 *              fifo and a full fifo.
 ******************************************************************************/
tuint dsiuFifoSizeAvail (dsiuFifo_t *fifo)
{
  tuint avail;

  if (fifo->inIndex >= fifo->outIndex)
    avail = fifo->size - (fifo->inIndex - fifo->outIndex) - 1;
  else
    avail = fifo->outIndex - fifo->inIndex - 1;

  return (avail);

} /* dsiuFifoSizeAvail */

/*****************************************************************************
 * FUNCTION PURPOSE: Return the number of elements in the fifo
 *****************************************************************************
 * DESCRIPTION: The number of elements in the fifo is computed.
 *****************************************************************************/
tuint dsiuFifoNelems(dsiuFifo_t *fifo)
{
  tuint n;

  n = fifo->size - dsiuFifoSizeAvail(fifo) - 1;
  return (n);

} /* dsiuFifoNelems */

/******************************************************************************
 * FUNCTION PURPOSE: Enter data into the fifo
 ******************************************************************************
 * DESCRIPTION: Places data into the fifo if enough space is available.
 *              Returns a pointer to the address of the 1st element in the
 *              fifo. Returns NULL if not enough space was available in
 *              the fifo.
 ******************************************************************************/
tuint *dsiuFifoIn (dsiuFifo_t *fifo, globData_t *data)
{
  tuint avail;
  tuint size1, size2;
  tuint *ibase, *tdata;

  tdata = (tuint *)data->data;

  /* Check that there is enough space in the fifo */
  avail = dsiuFifoSizeAvail (fifo);
  if (avail < data->dataSize)
    return (NULL);

  /* Store the address of the 1st address of stored block for return */
  ibase = &(fifo->base[fifo->inIndex]);

  /* Compute write sizes, including wrap around */
  size1 = fifo->size - fifo->inIndex;
  if (size1 >= data->dataSize)  {
    size1 = data->dataSize;
    size2 = 0;
  }
  else  {
    size2 = data->dataSize - size1;
  } 

  /* Write the 1st block of data to the fifo, update the fifo pointer */
  memcpy (&(fifo->base[fifo->inIndex]), tdata, size1 * sizeof(tint));
  fifo->inIndex += size1;
  if (fifo->inIndex >= fifo->size)
    fifo->inIndex = 0;

  if(size2) {
  /* Copy the 2nd block of data to the fifo, update the fifo pointer */
  memcpy (&(fifo->base[fifo->inIndex]), &(tdata[size1]), size2 * sizeof(tint));
  fifo->inIndex += size2;
  }
  /* This second wrap check should not be necessary, since the second block
   * cannot cause an address wrap */
  if (fifo->inIndex >= fifo->size)
    fifo->inIndex = 0;
  
  /* Return the address where the 1st element of the fifo lives */
  return (ibase);

} /* dsiuFifoIn */


/*****************************************************************************
 * FUNCTION PURPOSE: Extract data from the fifo
 *****************************************************************************
 * DESCRIPTION: Extracts data from the fifo. The return pointer is the
 *              same as that in the global block. Returns NULL if 
 *              a request was made for more data then is in the fifo
 *****************************************************************************/
tuint *dsiuFifoOut (dsiuFifo_t *fifo, globData_t *data)
{
  tuint avail;
  tuint size1, size2;
  tuint *tdata;

  tdata = (tuint *)data->data;

  /* Check that the requested number of element is in the fifo */
  avail = dsiuFifoNelems(fifo);
  if (data->dataSize > avail)
    return (NULL);

  /* Compute write sizes, including wrap around */
  size1 = fifo->size - fifo->outIndex;
  if (size1 >= data->dataSize)  {
    size1 = data->dataSize;
    size2 = 0;
  }
  else {
    size2 = data->dataSize - size1;
  }

  /* Extract the 1st block of the fifo, update the fifo index */
  memcpy (tdata, &(fifo->base[fifo->outIndex]), size1 * sizeof(tint));
  fifo->outIndex += size1;
  if (fifo->outIndex >= fifo->size)
    fifo->outIndex = 0;

  if(size2) {  
  /* Extract the 2nd block of the fifo */
  memcpy (&(tdata[size1]), &(fifo->base[fifo->outIndex]), size2 * sizeof(tint));
  fifo->outIndex += size2;
  }
  /* The wrap check should not be necessary, since the second write cannot
   * case a fifo wrap. */
  if (fifo->outIndex >= fifo->size)
    fifo->outIndex = 0;

  return (data->data);

} /* dsiuFifoOut */

/******************************************************************************
 * FUNCTION PURPOSE: Back data out from a fifo
 ******************************************************************************
 * DESCRIPTION: Data is removed from the TAIL of the fifo. This is required
 *              when a single block of data must be entered in multiple
 *              fifo writes. If the first write succeeds, but the second
 *              fails due to lack of space in the fifo, it is necessary to
 *              back the first write out. Essentially this backs up the
 *              in pointer in the fifo.
 *
 *              !!! No check for wrap past the out pointer is made !!!
 *****************************************************************************/
void dsiuFifoBackout (dsiuFifo_t *fifo, tuint backoutLen)
{
  if (fifo->inIndex < backoutLen)  
    fifo->inIndex = fifo->size - backoutLen + fifo->inIndex;
  else
    fifo->inIndex = fifo->inIndex - backoutLen;
} /* dsiuFifoBackout */
   

/*****************************************************************************
 * FUNCTION PURPOSE: Reset a fifo
 *****************************************************************************
 * DESCRIPTION: Resets the fifo in and out indices to 0. Used when fifo
 *              error is encountered.
 *****************************************************************************/
void dsiuFifoReset (dsiuFifo_t *fifo)
{
   fifo->inIndex = fifo->outIndex = 0;
} /* dsiuFifoReset */

