/****************************************************************************
 * FILE PURPOSE: Host to DSP message processing
 ****************************************************************************
 * FILE NAME: dsiuhdm.c
 *
 * DESCRIPTION: Functions that process the messages received from the host.
 *
 * TABS: None
 *
 * Copyright (C) 2002, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include "types.h"
#include "dsiuloc.h"
#include "dsiumsg.h"
#include "diagcfg.h"
#include "hal.h"
#include "dsiuport.h"
#include "cmisc.c"
#include "diagapp.h"
#include <ti/mas/neu/neu.h>
#include <stdlib.h>
#include <string.h>
#include "dsiuutop.h"
#include "dsiuotbrun.h"

#ifdef PA_SS
extern Int dsiuNwGlobClean (void);
#endif

/*****************************************************************************
 * FUNCTION PURPOSE: Route message to the host
 *****************************************************************************
 * DESCRIPTION: Routes an outgoing message to either the utopia interface,
 *              gmac interface, or the HPI message area based on the diag 
 *              register configuration values.
 ****************************************************************************/
void dsiu_send_msg (dmsg_t *msg)
{

  if ((dsiuContext.dregMsgCfgShadow->route == DIAGAPP_ROUTE_UTOPIA) &&
      (diagCfg.ctable.siuUtNdu))  {
    (*diagCfg.ctable.siuUtNdu->nduPushFifo) ((void *)msg);
    return;
  }
  else if ((dsiuContext.dregMsgCfgShadow->route == DIAGAPP_ROUTE_DPRAM) &&
           (diagCfg.ctable.siuHpiNdu))  {
    (*diagCfg.ctable.siuHpiNdu->nduPushFifo)((void*)msg);
    return;
  }
  else if ((dsiuContext.dregMsgCfgShadow->route == DIAGAPP_ROUTE_GMAC) &&
           (diagCfg.ctable.siuGmacNdu))  {
    (*diagCfg.ctable.siuGmacNdu->nduPushFifo)((void *)msg);
    return;
  }


} /* dsiu_send_msg */

/*****************************************************************************
 * FUNCTION PURPOSE: Generate an ack message
 *****************************************************************************
 * DESCRIPTION: Populates the input message pointer with an ack message. 
 *              The message is not sent to the host. The ack code is 
 *              assigned a value of message ack. 
 *****************************************************************************/
dmsg_t *dsiu_msg_generate_ack (dmsg_t *msg, tuint tId, tuint msgId, tuint ack_type)
{
    msg->header.length   = DSIU_MSG_HEADER_SIZE + DSIU_MSG_STATE_ACK_SIZE;
    msg->header.chnum    = 0;
    msg->header.msg_id   = DMSG_DH_MSG_ID_STATE_ACK;
    msg->header.trans_id = tId;
    msg->msg_body.state_ack.state_info = DSIU_MSG_STATE_ACK_MSG_ACK;
    msg->msg_body.state_ack.msg_id     = msgId;
    msg->msg_body.state_ack.ack_type   = ack_type;
  
    return (msg);
} /* dsiu_msg_generate_ack */

/******************************************************************************
 * FUNCTION PURPOSE: Generate an error message
 ******************************************************************************
 * DESCRIPTION: This function sends to the host the message it received 
 *              along with an error code describing why it was rejected.
 ******************************************************************************/
void dsiuMsgError (dmsg_t *msg, tuint ecode)
{
  tuint copyLength;
  tuint diff;

  /* If there is not enough room for the entire message to be copied back
   * to the host, truncate the message */
  copyLength = msg->header.length;

  /* Form the header */
  dsiuContext.msgTxBuf->header.length = msg->header.length   + 
                            DSIU_MSG_HEADER_SIZE + DSIU_MSG_MSG_ERR_SIZE;
  if (dsiuContext.msgTxBuf->header.length > diagCfg.msgTxBufferSize)  {
    dsiuContext.msgTxBuf->header.length = diagCfg.msgTxBufferSize;
    diff = dsiuContext.msgTxBuf->header.length - diagCfg.msgTxBufferSize;
    copyLength -= diff;
  }
  dsiuContext.msgTxBuf->header.chnum    = 0;
  dsiuContext.msgTxBuf->header.msg_id   = DMSG_DH_MSG_ID_MSG_ERROR;
  dsiuContext.msgTxBuf->header.trans_id = msg->header.trans_id;

  /* Enter the error code */
  dsiuContext.msgTxBuf->msg_body.msg_error.ecode = ecode;

  /* Copy the offending message */
  memcpy (&(dsiuContext.msgTxBuf->msg_body.msg_error.msg_first_word), msg,
          copyLength*sizeof (dsiuContext.msgTxBuf->msg_body.msg_error.msg_first_word) );

  /* Send the message to the host */
  dsiu_send_msg (dsiuContext.msgTxBuf);
} /* dsiuMsgError */

/******************************************************************************
 * FUNCTION PURPOSE: Send system message to host
 ******************************************************************************
 * DESCRIPTION: This function is invoked from the HAL after a memory move
 *              has been completed, and passes to this function a pointer
 *              to the message to be sent.
 *****************************************************************************/
tint dsiu_msg_ready (void *vmsg)
{
  dsiuContext.memOpPending = FALSE;
  DSIU_DUMMY_ARG(vmsg);
  return (0);
} /* dsiu_msg_ready */

/*****************************************************************************
 * FUNCTION PURPOSE: Shift the message body data up 1 word and send to host
 *****************************************************************************
 * DESCRIPTION: This function is called when a memory read is done and the
 *              resulting data has been shifted up one address due to 
 *              HAL memory alignment requirements. This function shifts the
 *              data back into the location which is correct for the message
 *              and sends the message.
 ******************************************************************************/
tint dsiu_msg_ready_read_shift (void *vmsg)
{
  dmsg_t *msg = (dmsg_t *)vmsg;
  tint i;
  tuint *p;


  p = &(msg->msg_body.get_mem_read.first_word);

  for (i = 0; i < SIU_SIZE_WORDS_TO_TUINT(msg->msg_body.get_mem_read.length_words); i++, p++)
    *p = *(p+1);

  dsiuContext.memOpPending = FALSE;

  return (0);
} /* dsiu_msg_ready_shift */
  


/******************************************************************************
 * FUNCTION PURPOSE: Process memory read request
 ******************************************************************************
 * DESCRIPTION: Processes a memory read request from the host
 ******************************************************************************/
void dsiuMsgReqMemRead (dmsg_t *msg)
{
  halLocAddr_t dest, src;    /* Unsigned 32 bit ints                   */
  tint (*read_done)(void *); /* Function Pointer to read done function */
  tulong length_words_l;
  tuint length_words;
  halMemMove_t memMove;
  length_words_l = DSIU_DWORD_CONSTRUCT (msg->msg_body.req_mem_read.length_words_msw,
                                         msg->msg_body.req_mem_read.length_words_lsw);
  length_words = SIU_SIZE_WORDS_TO_TUINT((tuint)length_words_l);

  
  /* Verify that the message is complete. The message is allowed to have
   * extra parameters */
  if (msg->header.length < DSIU_MSG_HEADER_SIZE + DSIU_MSG_REQ_MEM_READ_SIZE)  {
    /* Send a copy of the entire message back to the host with a size error
     * indication */
    dsiuMsgError (msg, DSIU_MSG_ERR_INVALID_MSG_SIZE);
    return;
  }

  /* Check that the number of words requested can fit in the available message
   * buffer */
  if (length_words_l > (diagCfg.msgTxBufferSize - DSIU_MSG_HEADER_SIZE))  {
    dsiuMsgError (msg, DSIU_MSG_ERR_REQ_SIZE_VIOLATION);
    return;
  }

  /* Check that the memory space requested is valid */
  if ((msg->msg_body.req_mem_read.mem_type != DMSG_MEM_TYPE_DATA) &&
      (msg->msg_body.req_mem_read.mem_type != DMSG_MEM_TYPE_PROG) &&
      (msg->msg_body.req_mem_read.mem_type != DMSG_MEM_TYPE_DMA))  {
    dsiuMsgError (msg, DSIU_MSG_ERR_MEM_TYPE_INVALID);
    return;
  }

  /* Create the message response */
  /* The header */
  dsiuContext.msgTxBuf->header.length = DSIU_MSG_HEADER_SIZE +
                                        SIU_SIZE_WORDS_TO_TUINT(length_words) +
                                      DSIU_MSG_GET_MEM_READ_BODY_SIZE;
  dsiuContext.msgTxBuf->header.chnum    = 0;
  dsiuContext.msgTxBuf->header.msg_id   = DMSG_DH_MSG_ID_GET_MEM_READ;
  dsiuContext.msgTxBuf->header.trans_id = msg->header.trans_id;

  /* The body */
  dsiuContext.msgTxBuf->msg_body.get_mem_read.addr_msw =
    msg->msg_body.req_mem_read.addr_msw;
  dsiuContext.msgTxBuf->msg_body.get_mem_read.addr_lsw =
    msg->msg_body.req_mem_read.addr_lsw;
  dsiuContext.msgTxBuf->msg_body.get_mem_read.mem_type =
    msg->msg_body.req_mem_read.mem_type;
  dsiuContext.msgTxBuf->msg_body.get_mem_read.length_words =
    SIU_SIZE_WORDS_TO_TUINT(length_words);

  /* Invoke the Hardware Abstraction Layer to copy the data to the message */
  src = DSIU_DWORD_CONSTRUCT (msg->msg_body.req_mem_read.addr_msw,
                              msg->msg_body.req_mem_read.addr_lsw);
  dest = (halLocAddr_t)&dsiuContext.msgTxBuf->msg_body.get_mem_read.first_word;

#if 0
  /* The destination address is garuanteed to be in data memory. The most
   * significant bit of the address specifies the memory space */
  HAL_MEM_ADDRESS_DATA (dest);

  /* Setup the memory space for the source */
  if ((msg->msg_body.req_mem_read.mem_type == DMSG_MEM_TYPE_PROG) ||
      (msg->msg_body.req_mem_read.mem_type == DMSG_MEM_TYPE_DMA))  {
    src = (*dsiuContext.hal->halMapLocalToGlobal) ((halLocAddr_t)src);
    HAL_MEM_ADDRESS_PROG (src);
  } else {
    HAL_MEM_ADDRESS_DATA (src);
  }

  /* Make the call. Have the HAL make the transfer and invoke the function
   * dsiu_msg_ready function when the transfer is complete */
  if (msg->msg_body.req_mem_read.mem_type == DMSG_MEM_TYPE_DMA)  {

    /* The HAL layer can only handle evenly word aligned addresses. If 
     * the source address is odd then generate an error. If the destination
     * is odd then load the data on the next higher even address, and shift
     * the data when the copy is complete. */
    if (src & 0x1)  {
      dsiuMsgError (msg, DSIU_MSG_ERR_INVALID_ALIGN);
      return;
    }

    if (dest & 0x1)  {
      dest += 1;
      read_done = dsiu_msg_ready_read_shift;
    } else {
      read_done = dsiu_msg_ready;
    }

    (*dsiuContext.hal->halGlobMemMove) (dest, src,  
        length_words, read_done, (void *)dsiuContext.msgTxBuf);
  
  }  else
    (*dsiuContext.hal->halLocalMemMove) (dest, src,  
        length_words, dsiu_msg_ready, (void *)dsiuContext.msgTxBuf);
#else
    if (dest & 0x1)  {
      dest += 1;
      read_done = dsiu_msg_ready_read_shift;
    } else {
      read_done = dsiu_msg_ready;
    }

#endif
    dsiuContext.memOpPending = TRUE;

    memMove.dest = (*dsiuContext.hal->halMapLocalToGlobal)(dest);
    memMove.src  = (*dsiuContext.hal->halMapLocalToGlobal)(src),
    memMove.size = length_words;

    /* Do the memory move. Have the HALL stall until the transfer is complete */    
    (*dsiuContext.hal->halGlobMemMove) (1, &memMove, read_done, (void *)dsiuContext.msgTxBuf, FALSE);

  /* Wait for the transfer to complete */
  while (dsiuContext.memOpPending == TRUE);


  /* Send the reply */
  dsiu_send_msg (dsiuContext.msgTxBuf);
  
} /* dsiuMsgReqMemRead */

/******************************************************************************
 * FUNCTION PURPOSE: Process memory write request
 ******************************************************************************
 * DESCRIPTION: Processes a memory write request from the host
 ******************************************************************************/
void dsiuMsgReqMemWrite (dmsg_t *msg)
{
  halLocAddr_t dest, src;   /* Unsigned 32 bit ints */  /* Define max mem write size */
  tuint totalLength;
  tuint lengthSave;
  halMemMove_t memMove;
  dmsg_t *response;

#if defined( __TMS320C55X__ )
  tuint i;
  tuint *copySource, *copyDest;
  halMemMoveComplete_t msgSend;
#endif


  /* Check that the message is complete. The message is allowed to contain
   * extra parameter */
  if (msg->header.length < DSIU_MSG_HEADER_SIZE + DSIU_MSG_REQ_MEM_WRITE_SIZE) {
    /* Send a copy of the entire message back to the host with a size error
     * indication */
    dsiuMsgError (msg, DSIU_MSG_ERR_INVALID_MSG_SIZE);
    return;
  }
  /* Check that the write length matches the message size */
  totalLength = DSIU_MSG_HEADER_SIZE + DSIU_MSG_REQ_MEM_WRITE_SIZE +
                SIU_SIZE_WORDS_TO_TUINT(msg->msg_body.req_mem_write.length_words);
  if (msg->header.length < totalLength)  {
    dsiuMsgError (msg, DSIU_MSG_ERR_INVALID_MSG_SIZE);
    return;
  }

  /* Check that the memory space requested is valid */
  if ((msg->msg_body.req_mem_write.mem_type != DMSG_MEM_TYPE_DATA) &&
      (msg->msg_body.req_mem_write.mem_type != DMSG_MEM_TYPE_PROG) &&
      (msg->msg_body.req_mem_write.mem_type != DMSG_MEM_TYPE_DMA))  {
    dsiuMsgError (msg, DSIU_MSG_ERR_MEM_TYPE_INVALID);
    return;
  }

  /* If the transaction ID of the message was non-zero a response must be
   * generated. If there is a pending response, wait until it is complete. */
  dsiuContext.memOpPending = TRUE;
  if (msg->header.trans_id != 0)  {

    response = dsiu_msg_generate_ack (dsiuContext.msgTxBuf, 
               msg->header.trans_id, msg->header.msg_id, MGB_ACK_TYPE_DONE);

  }
  else {
    response = NULL;
  }

#if defined( __TMS320C55X__ )

  msgSend = dsiu_msg_ready;

  /* The HAL can only write in blocks of 32 bits. If an odd word length was
   * specified, pad it out with a 0. This section of code will always leave
   * the length with an even value */
  if (msg->msg_body.req_mem_write.length_words & 0x0001)  {
    copySource = (tuint *)&msg->msg_body.req_mem_write.first_word;
    copySource[msg->msg_body.req_mem_write.length_words] = 0;
    msg->msg_body.req_mem_write.length_words += 1;
  }


  /* The HAL requires that the source data be aligned on a 32 bit boundry.
   * That will not be the case for the diag program. If the data is currently
   * odd aligned, shift it up 1 word. This will delete the length field, which
   * must be stored prior to the shift.  */
  lengthSave = msg->msg_body.req_mem_write.length_words;

  src  = (halLocAddr_t)&msg->msg_body.req_mem_write.first_word;
  if (src & 0x2)  {
    copySource = (tuint *)src;
    copyDest   = copySource-1;
    for (i = 0; i < lengthSave; i++)
      *copyDest++ = *copySource++;
    src -= 1;
  }

#elif defined ( _TMS320C6X )
  lengthSave = msg->msg_body.req_mem_write.length_words;

  src  = (halLocAddr_t)&msg->msg_body.req_mem_write.first_word;
#else
#error - Unkown processor
#endif
  /* Invoke the Hardware Abstraction Layer to copy the data to the message */
  dest = DSIU_DWORD_CONSTRUCT (msg->msg_body.req_mem_write.addr_msw,
                               msg->msg_body.req_mem_write.addr_lsw);

#ifdef __TMS320C55X__
  /* The source address is garuanteed to be in data memory. The most
   * significant bit of the address specifies the memory space */
  HAL_MEM_ADDRESS_DATA (src);

  /* Setup the memory space for the destination */
  if (msg->msg_body.req_mem_write.mem_type != DMSG_MEM_TYPE_DATA)
    HAL_MEM_ADDRESS_PROG (dest);
  else
    HAL_MEM_ADDRESS_DATA (dest);

  /* Make the call. Have the HAL make the transfer and invoke the function
   * dsiu_msg_ready function when the transfer is complete, if a reply
   * is required */
  if (msg->msg_body.req_mem_write.mem_type == DMSG_MEM_TYPE_DATA)
    (*dsiuContext.hal->halLocalMemMove) (dest, src,  
        lengthSave, msgSend, (void *)response);
  else
#endif

  memMove.dest = (*dsiuContext.hal->halMapLocalToGlobal)(dest);
  memMove.src  = (*dsiuContext.hal->halMapLocalToGlobal)(src);
  memMove.size = lengthSave;

  /* Do the memory move and stall until complete */
  (*dsiuContext.hal->halGlobMemMove) (1, &memMove, NULL, NULL, TRUE);

  if (msg->header.trans_id != 0) 
    dsiu_send_msg (response);

} /* dsiuMsgReqMemWrite */

/*****************************************************************************
 * FUNCTION PURPOSE: Process memory copy request
 *****************************************************************************
 * DESCRIPTION: Processes a memory copy request from the host
 *****************************************************************************/
void dsiuMsgReqMemCopy (dmsg_t *msg)
{
  halLocAddr_t  dest, src;   /* Unsigned 32 bit ints */
  halSAE_t      len;
  dmsg_t       *response;
  halMemMove_t  memMove;

#if defined( __TMS320C55X__ )
  halMemMoveComplete_t msgSend;
#endif

  /* Check that the message is complete. The message is allowed to contain
   * extra parameter */
  if (msg->header.length < DSIU_MSG_HEADER_SIZE + DSIU_MSG_REQ_MEM_COPY_SIZE) {
    /* Send a copy of the entire message back to the host with a size error
     * indication */
    dsiuMsgError (msg, DSIU_MSG_ERR_INVALID_MSG_SIZE);
    return;
  }

  /* Check that the memory spaces requested are valid */
  if ((msg->msg_body.req_mem_copy.source_mem_type != DMSG_MEM_TYPE_DATA) &&
      (msg->msg_body.req_mem_copy.source_mem_type != DMSG_MEM_TYPE_PROG) &&
      (msg->msg_body.req_mem_copy.source_mem_type != DMSG_MEM_TYPE_DMA)) {
    dsiuMsgError (msg, DSIU_MSG_ERR_MEM_TYPE_INVALID);
    return;
  }
  if ((msg->msg_body.req_mem_copy.dest_mem_type != DMSG_MEM_TYPE_DATA) &&
      (msg->msg_body.req_mem_copy.dest_mem_type != DMSG_MEM_TYPE_PROG) &&
      (msg->msg_body.req_mem_copy.dest_mem_type != DMSG_MEM_TYPE_DMA)) {
    dsiuMsgError (msg, DSIU_MSG_ERR_MEM_TYPE_INVALID);
    return;
  }

  /* If the transaction ID of the message was non-zero a response must be
   * generated. */
  dsiuContext.memOpPending = TRUE;
  if (msg->header.trans_id != 0)  {

    response = dsiu_msg_generate_ack (dsiuContext.msgTxBuf, 
               msg->header.trans_id, msg->header.msg_id, MGB_ACK_TYPE_DONE);

  }
  else {
    response = NULL;
  }
  /* Invoke the Hardware Abstraction Layer to copy the data to the message */
  src = DSIU_DWORD_CONSTRUCT  (msg->msg_body.req_mem_copy.source_addr_msw,
                               msg->msg_body.req_mem_copy.source_addr_lsw);
  dest = DSIU_DWORD_CONSTRUCT (msg->msg_body.req_mem_copy.dest_addr_msw,
                               msg->msg_body.req_mem_copy.dest_addr_lsw);
  len = DSIU_DWORD_CONSTRUCT (msg->msg_body.req_mem_copy.length_words_msw,
                               msg->msg_body.req_mem_copy.length_words_lsw);

  /* Make the call. Have the HAL make the transfer and invoke the function
   * dsiu_msg_ready function when the transfer is complete, if a reply
   * is required */
#if defined( _TMS320C6X ) /* Temporary to test code download with janus type micro */


    /* Note that if the source and dest are already global addresses they
     * are not changed */
    memMove.dest = (*dsiuContext.hal->halMapLocalToGlobal)(dest);
    memMove.src  = (*dsiuContext.hal->halMapLocalToGlobal)(src);
    memMove.size = len;

    /* Move the memory, stall until complete */
    (*dsiuContext.hal->halGlobMemMove) (1, &memMove, NULL, NULL, TRUE);


#elif defined( __TMS320C55X__ )

  /* Setup the memory space for the source */
  if (msg->msg_body.req_mem_copy.source_mem_type == DMSG_MEM_TYPE_PROG)
    HAL_MEM_ADDRESS_PROG (src);
  else
    HAL_MEM_ADDRESS_DATA (src);

  /* Setup the memory space for the dest */
  if (msg->msg_body.req_mem_copy.dest_mem_type == DMSG_MEM_TYPE_PROG)
    HAL_MEM_ADDRESS_PROG (dest);
  else
    HAL_MEM_ADDRESS_DATA (dest);

  memMove.dest = dest;
  memMove.src  = src;
  memMove.size = len;

  msgSend = dsiu_msg_ready;
  if ((msg->msg_body.req_mem_copy.source_mem_type == DMSG_MEM_TYPE_DMA) ||
      (msg->msg_body.req_mem_copy.dest_mem_type   == DMSG_MEM_TYPE_DMA))
    (*dsiuContext.hal->halGlobMemMove) (1, &memMove, NULL, NULL, TRUE);
  else
    (*dsiuContext.hal->halLocalMemMove) (dest, src,  
        len, msgSend, (void *)response);
#else
#error -unkown compile time CPU def
#endif

    if (msg->header.trans_id != 0)  
      dsiu_send_msg (response);


} /* dsiuMsgReqMemCopy */

/******************************************************************************
 * FUNCTION PURPOSE: Checksum copy poll function 
 ******************************************************************************
 * DESCRIPTION: This function is used in conjunction with dsiuMsgReqMemChecksum.
 *              In the checksum computation memory is copied into a buffer
 *              using the HAL. The copy can occur using a DMA asynchronously.
 *              When the transfer is done, the HAL calls this function, which
 *              sets the global variable dsiuGlobalChecksumFlag. This variable
 *              is polled in the checksum computation routine.
 *****************************************************************************/
tint dsiuGlobalChecksumFlag;
tint dsiu_checksum_poll (void *context)
{
  dsiuGlobalChecksumFlag = 1;
  DSIU_DUMMY_ARG(context);
  return (0);
} /* dsiu_checksum_poll */


/******************************************************************************
 * FUNCTION PURPOSE: Configure a channel to use a physical packet port
 ******************************************************************************
 * DESCRIPTION: Configures a channel to use a specified ndu interface for
 *              packet reception and transmission.
 ******************************************************************************/
dsiuReturn_t dsiu_ch_cfg_phys_port (dmsg_t *msg, dsiuInst_t *inst)  
{
  /* Physical port configuration for outgoing data */
  switch (msg->msg_body.ch_config.physPort)  {

    case DMSG_PHYS_PORT_UTOPIA:

      /* Verify the utopia NDU exits */
      if (!diagCfg.ctable.siuUtNdu)  {
        dsiuMsgError (msg, DSIU_MSG_ERR_INVALID_IFACE);
        return (DSIU_FAIL);
      }
      
      inst->nduInst.recInterface = SIU_NDU_TYPE_UT;
      inst->nduInst.RxOutSpec.nduReceiveOut = diagCfg.ctable.neu->neuReceiveInUt;
      break;

    case DMSG_PHYS_PORT_MAC:

      /* Verify the mac NDU exists */
      if (!diagCfg.ctable.siuGmacNdu)  {
        dsiuMsgError (msg, DSIU_MSG_ERR_INVALID_IFACE);
        return (DSIU_FAIL);
      }

      inst->nduInst.recInterface = SIU_NDU_TYPE_GMAC;
      inst->nduInst.RxOutSpec.nduReceiveOut = diagCfg.ctable.neu->neuReceiveInGmac;
      break;

    default:

      /* Invalid physical port specified */
      dsiuMsgError (msg, DSIU_MSG_ERR_INVALID_IFACE);
      return (DSIU_FAIL);

  } /* end switch */

  /* Setup the channel ndu interface to point to neu */
  inst->nduInst.RxOutSpec.targetInst    = inst->neuInst;

  return (DSIU_SUCCESS);

} /* dsiu_ch_cfg_phys_port */


/*******************************************************************************
 * FUNCTION PURPOSE: Reconfigure NEU
 *******************************************************************************
 * DESCRIPTION: Closes the existing NEU channel, then reconfigures with the
 *              parameters from the message.
 *******************************************************************************/
void dsiu_ch_cfg_neu (dmsg_t *msg, dsiuInst_t *inst)
{
  neuCfg_t   cfg;
  neuCtrl_t  ctrl;
  tint       count;
  dmsg_hd_ch_config_t *ch;



  /* Shorthand to the message parameters */
  ch = &(msg->msg_body.ch_config);

  /* Remove the current channel from the portmap. It's OK if the call is
   * not successful - this just insures that the instance is not already
   * mapped */
  ctrl.code = neu_CTRL_RMV_PORTMAP;
  ctrl.u.portMap.inst = (void *)inst;
  (*diagCfg.ctable.neu->neuControl) (inst->neuInst, &ctrl);

  /* Close the channel. It may already be in a closed state if no utopia
   * interface is supported, but that is OK. */
  (*diagCfg.ctable.neu->neuClose) (inst->neuInst);

  /* Provide the function pointer for send out. The validity of the 
   * interface was already verified in function dsiu_ch_cfg_phys_port */
  switch (msg->msg_body.ch_config.physPort)  {

    case DMSG_PHYS_PORT_UTOPIA:
      cfg.neuSendOut.neuSendOut = diagCfg.ctable.siuUtNdu->nduSendIn;
      break;

    case DMSG_PHYS_PORT_MAC:
      cfg.neuSendOut.neuSendOut = diagCfg.ctable.siuGmacNdu->nduSendIn;
      break;

  } /* end switch - no default required */

  cfg.neuSendOut.targetInst = inst;

  /* Leave NEU enabled */
  cfg.parm.chnl.enable = TRUE;

  /* Flags */
  cfg.parm.chnl.cfg_flag1 = ch->flags1;
  cfg.parm.chnl.cfg_flag2 = ch->flags2;

  /* Alternate ports */
  cfg.parm.chnl.alt_local_port  = ch->alt_local_port;
  cfg.parm.chnl.alt_remote_port = ch->alt_remote_port;

  count = 0;
  while(count < DMSG_IP_ADDR_MAX_NUM_TUINT)
  {
      cfg.parm.chnl.alt_remote_addr[count] = 
          ch->alt_remote_addr[count];
      count++;
  }

  /* Report control - not implemented in diag */
  cfg.parm.chnl.report_ctrl          = ch->report_ctrl;
  cfg.parm.chnl.report_throttle_time = ch->report_throttle_time;
  cfg.parm.chnl.sysInst              = (void *)inst;

  /* Second protocol type - not implemented in diag */
  cfg.parm.chnl.second_prototype = ch->tel2pkt_second_protocol;

  /* Header parameters */
  cfg.parm.chnl.rx_offset_bytes = ch->pkt2tel_start_offset;
  cfg.parm.chnl.rx_payload_type = ch->pkt2tel_first_protocol;

  /* Header offsets */
  if (ch->tel2pkt_ip_offset > 0)
    cfg.parm.chnl.tx_ip_offset_bytes = ch->tel2pkt_ip_offset;
  else
    cfg.parm.chnl.tx_ip_offset_bytes = -1;

  if (ch->tel2pkt_udp_offset > 0)
    cfg.parm.chnl.tx_udp_offset_bytes = ch->tel2pkt_udp_offset;
  else
    cfg.parm.chnl.tx_udp_offset_bytes = -1;

  if (ch->tel2pkt_802_3_offset > 0)
    cfg.parm.chnl.tx_802_3_offset_bytes = ch->tel2pkt_802_3_offset;
  else
    cfg.parm.chnl.tx_802_3_offset_bytes = -1;

  /* Min lengths, header lengths */
  cfg.parm.chnl.tx_min_cell_bytes = ch->tel2pkt_min_len;
  cfg.parm.chnl.tx_total_hdr_len_bytes = ch->tel2pkt_total_hdr_len;

  /* Copy the transmit header */
  memcpy (cfg.parm.chnl.header, ch->hdr_image, SIU_BYTES_TO_WORDS(ch->tel2pkt_total_hdr_len));

  /* Open NEU */
  (*diagCfg.ctable.neu->neuOpen) (inst->neuInst, &cfg);

  /* Pass the parameters again to enabled sendout */
  ctrl.code   = neu_CTRL_CHANNEL_CFG;
  ctrl.u.parm = cfg.parm;
  (*diagCfg.ctable.neu->neuControl) (inst->neuInst, &ctrl);

  /* Register the UDP port, if provided */
  ctrl.code         = neu_CTRL_GET_SRC_UDP_PORT;
    
  if ((*diagCfg.ctable.neu->neuControl)(inst->neuInst, &ctrl) == neu_NOERR)  {
        ctrl.code = neu_CTRL_ADD_PORTMAP;
        ctrl.u.portMap.inst = inst;
        
        if ((*diagCfg.ctable.neu->neuControl) (inst->neuInst, &ctrl) != neu_NOERR)  {
    /* Verify that the registration was successful */
      dsiuErrorIndication (msg->header.trans_id, 0, MGB_ERR_CORE_NEU_PORT_REG);
    }
  }
} /* dsiu_ch_cfg_neu */



/*******************************************************************************
 * FUNCTION PURPOSE: Configure pvp and neu for a channel
 *******************************************************************************
 * DESCRIPTION: This function is used to setup a channel for network transmit
 *              and receive. Only symmetric configuration is supported. 
 *              The message indicates which interface is used for outgoing
 *              network traffic, what encapsulation type to use, and
 *              the neu paramters used.
 *******************************************************************************/
void dsiuMsgChConfig (dmsg_t *msg)
{
  dsiuInst_t     *inst;
  halIntrLevel_t  iLevel;

  /* Extract the channel instance from the message header. Verify the ID
   * matches the channel number. */
  inst = &dsiuInst[msg->header.chnum - 1];
  if (inst->Id != msg->header.chnum)  {
    dsiuMsgError (msg, DSIU_MSG_ERR_INVALID_CHNUM);
    return;
  }

  /* A part of NEU can operate in an ISR context, so disable interrupts
   * before any configuration */
  iLevel = (*dsiuContext.hal->halIntrLevel) (HAL_INT_LEVEL_DISABLE);

  /* Perform physical port selection */
  if (dsiu_ch_cfg_phys_port (msg, inst) != DSIU_SUCCESS)  {
    (*dsiuContext.hal->halIntrLevel) (iLevel);
    return;
  }

  /* Perform pvp configuration */
  /* TODO */

  /* Perform NEU configuration */
  dsiu_ch_cfg_neu (msg, inst);


  /* Restore interrupts */
  (*dsiuContext.hal->halIntrLevel) (iLevel);

}

/******************************************************************************
 * FUNCTION PURPOSE: Process memory checksum request
 ******************************************************************************
 * DESCRIPTION: Processes a memory checksum request from the host
 ******************************************************************************/
void dsiuMsgReqMemChecksum (dmsg_t *msg)
{
  halLocAddr_t dest, src;   /* Unsigned 32 bit ints                    */
  tuint checksum;           /* The computed checksum                   */
  tuint dblock[DSIU_CHKSUM_BLOCK_SIZE];  /* temporary block        */
  tint remain;              /* Remaining words in checksum calculation */
  tint copysize;
  tint i;
  halMemMove_t memMove;
  
  /* Check that the message is complete. The message is allowed to contain
   * extra parameter */
  if (msg->header.length < DSIU_MSG_HEADER_SIZE + DSIU_MSG_REQ_MEM_CHECKSUM_SIZE) {
    /* Send a copy of the entire message back to the host with a size error
     * indication */
    dsiuMsgError (msg, DSIU_MSG_ERR_INVALID_MSG_SIZE);
    return;
  }

  /* Check that the memory spaces requested are valid */
  if ((msg->msg_body.req_mem_checksum.mem_type != DMSG_MEM_TYPE_DATA) &&
      (msg->msg_body.req_mem_checksum.mem_type != DMSG_MEM_TYPE_PROG) &&
      (msg->msg_body.req_mem_checksum.mem_type != DMSG_MEM_TYPE_DMA)) {
    dsiuMsgError (msg, DSIU_MSG_ERR_MEM_TYPE_INVALID);
    return;
  }

  /* The checksum will be computed one block at a time. Memory will be copied
   * into the dblock memory area for checksum calculation */
  dest = (halLocAddr_t)dblock;
  src = DSIU_DWORD_CONSTRUCT  (msg->msg_body.req_mem_checksum.addr_msw,
                               msg->msg_body.req_mem_checksum.addr_lsw);
  /* The destination address is always in data memory */
  HAL_MEM_ADDRESS_DATA (dest);

  /* Setup the memory space for the source */
  if (msg->msg_body.req_mem_checksum.mem_type == DMSG_MEM_TYPE_PROG)
    HAL_MEM_ADDRESS_PROG (src);
  else
    HAL_MEM_ADDRESS_DATA (src);

  /* Remain contains the number of words remaining in the calculation */
  remain   = msg->msg_body.req_mem_checksum.length_words;
  checksum = 0;
  while (remain > 0)  {
    copysize = 
        (remain > DSIU_CHKSUM_BLOCK_SIZE ? DSIU_CHKSUM_BLOCK_SIZE : remain);
    /* Clear the polling flag */
    dsiuGlobalChecksumFlag = 0;

    /* HAL handles the memory transfer */
    if (msg->msg_body.req_mem_checksum.mem_type == DMSG_MEM_TYPE_DMA)  {
      memMove.dest = dest;
      memMove.src  = src;
      memMove.size = copysize;
      (*dsiuContext.hal->halGlobMemMove) (1, &memMove, dsiu_checksum_poll, NULL, FALSE);
    }  else  {
      (*dsiuContext.hal->halLocalMemMove) (dest, src, copysize,
          dsiu_checksum_poll, NULL);
    }

    /* Once the transfer is complete, HAL will call dsiu_checksum_poll, which
     * will set dsiuGlobalChecksumFlag to 1. */
    while (dsiuGlobalChecksumFlag == 0);

    /* Continue checksum calculation */
    for (i = 0; i < copysize; i++)
      checksum += dblock[i];

    /* Update the source pointer, decrement the remain size */
    src    += copysize;
    remain -= copysize;
  } /* while (remain > 0 ) */

  /* Generate the checksum report */
  dsiuContext.msgTxBuf->header.length   = DSIU_MSG_HEADER_SIZE + 
                                          DSIU_MSG_GET_MEM_CHECKSUM_BODY_SIZE;
  dsiuContext.msgTxBuf->header.chnum    = 0;
  dsiuContext.msgTxBuf->header.msg_id   = DMSG_HD_MSG_ID_REQ_MEM_CHECKSUM;
  dsiuContext.msgTxBuf->header.trans_id = msg->header.trans_id;
  dsiuContext.msgTxBuf->msg_body.get_mem_checksum.checksum = checksum;

  /* Send the message */
  dsiu_send_msg (dsiuContext.msgTxBuf);
  
} /* dsiuMsgReqMemChecksum */

/******************************************************************************
 * FUNCTION PURPOSE: Process boot core control message
 ******************************************************************************
 * DESCRIPTION: Processes a boot core control request from the host
 ******************************************************************************/
void dsiuMsgBootCoreControl (dmsg_t *msg)
{
  tint i;
  dsiuCoreBootReturn_t  ret;
  dsiuCoreResetReturn_t rret;
  halControl_t          ctl;
  halWdTimerControl_t   wdctl;
  tulong entry;
  halIntrLevel_t  iLevel;

  /* Check that the message is complete. The message is allowed to contain
   * extra parameter */
  if (msg->header.length < (DSIU_MSG_HEADER_SIZE + 
                            DSIU_MSG_BOOT_CORE_CONTROL_MIN_SIZE)) {
    /* Send a copy of the entire message back to the host with a size error
     * indication */
    dsiuMsgError (msg, DSIU_MSG_ERR_INVALID_MSG_SIZE);
    return;
  }

  /* Use the entry point in the message, if provided */
  if (msg->header.length >= (DSIU_MSG_HEADER_SIZE + 
                             DSIU_MSG_BOOT_CORE_CONTROL_SIZE))  {
    entry = DSIU_DWORD_CONSTRUCT  (msg->msg_body.boot_core_control.addr_msw,
                                   msg->msg_body.boot_core_control.addr_lsw);
#ifdef __TMS320C55X__
    entry <<= 1;  /* Convert the word address to a byte address */
#endif
  }
  else
    entry = diagCfg.entry;

#ifdef PA_SS
  /* Cleanup for PA and associated subsystems */
  dsiuNwGlobClean ();
#endif

  /***************************************************************************
   * Boot the cores. Core 0 must be done last, since there is no return
   * from the boot termination callout. This function call is processor
   * specific.
   ***************************************************************************/
  if (!diagCfg.systemBoot)
    return;     /* No boot call provided by system */


  /* Check if the specified bit should be taken out of reset. If core 0
   * is specified this call terminates the diagnostic/bootloader code */
  for (i = 15; i >= 0; i--)  {
    if (msg->msg_body.boot_core_control.enable_bitmask & (1 << i))  {
      ret = (*diagCfg.systemBoot) (i, entry); 
      if (ret != DSIU_CORE_BOOT_SUCCESS)
        dsiuErrorIndication (msg->header.trans_id, 0, MGB_ERR_CORE_BOOT_FAIL);
    }
  }

  /* Check for cores to be placed into reset. A core can't put itself into reset */
  if (diagCfg.systemReset)  {
    for (i = 15; (i >= 0) && (i != dsiuContext.coreId); i--)  {
      if (msg->msg_body.boot_core_control.reset_bitmask & (1 << i))  {
        rret = (*diagCfg.systemReset) (i);
        if (rret != DSIU_CORE_RESET_SUCCESS)
          dsiuErrorIndication (msg->header.trans_id, 0, MGB_ERR_CORE_BOOT_FAIL);
      }
    }
  }

  if (msg->header.length >= (DSIU_MSG_HEADER_SIZE + 
                             DSIU_MSG_BOOT_CORE_SLEEP_WAKE_SIZE))  {

    /* Sleep */
    if (msg->msg_body.boot_core_control.sleep != 0)  {
      ctl.cmd = HAL_COMMAND_SLEEP;
      ctl.u.sleep.wakeMode = HAL_COMMAND_SLEEP_WAKE_OCORE_INT;
      (*dsiuContext.hal->halControl) (&ctl);
    }


    /* Wake other cores */
    if (msg->msg_body.boot_core_control.wake_bitmask)  {
      ctl.cmd = HAL_COMMAND_INT_OTHER_CORE;
      ctl.u.intCore.coreMask = msg->msg_body.boot_core_control.wake_bitmask;
      ctl.u.intCore.info     = 0;
      (*dsiuContext.hal->halControl) (&ctl);
    }

    if (msg->msg_body.boot_core_control.host_int == DSIU_MSG_BOOT_CORE_CONTROL_HOST_INT_HOUT)  {
      ctl.cmd = HAL_COMMAND_INT_HOST;
      (*dsiuContext.hal->halControl) (&ctl);

    }  else if (msg->msg_body.boot_core_control.host_int == DSIU_MSG_BOOT_CORE_CONTROL_HOST_INT_WD)  {

      /* Setup the watch dog to fire almost immediately */
      if (dsiuContext.hal->halWdTimerControl)  {
        iLevel = (*dsiuContext.hal->halIntrLevel) (HAL_INT_LEVEL_DISABLE);
        wdctl.operation   = HAL_WDTIMER_ENABLE;
        wdctl.periodCount = 1000;
        (*dsiuContext.hal->halWdTimerControl)(&wdctl);

        dsiuSafeWait (2000);  /* This function is different for each architecture
                               * and allows a safe wait that won't result in
                               * an invalid processor state when interrupted by the watch dog */
        (*dsiuContext.hal->halIntrLevel) (iLevel);
      }
    }
  }

           
} /* dsiuMsgBootCoreControl */

/*****************************************************************************
 * FUNCTION PURPOSE: Process a diag code control message
 *****************************************************************************
 * DESCRIPTION: Processes a diag control request from the host
 *****************************************************************************/
void dsiuMsgDiagCodeControl (dmsg_t *msg)
{
  dmsg_t readMem;

  /* Check that the message is complete. The message is allowed to contain
   * extra parameter */
  if (msg->header.length < DSIU_MSG_HEADER_SIZE + DSIU_MSG_DIAG_CODE_CONTROL_SIZE) {
    /* Send a copy of the entire message back to the host with a size error
     * indication */
    dsiuMsgError (msg, DSIU_MSG_ERR_INVALID_MSG_SIZE);
    return;
  }

  /* If the all test bit has been set, mark each test type as valid */
  if (msg->msg_body.diag_code_control.control_code & DMSG_CODE_CTRL_MEM_TEST_ALL)  {
    msg->msg_body.diag_code_control.control_code |= ( DMSG_CODE_CTRL_MARCH  |
                                                      DMSG_CODE_CTRL_ADDR   |
                                                      DMSG_CODE_CTRL_55     |
                                                      DMSG_CODE_CTRL_PSEUDO |
                                                      DMSG_CODE_CTRL_PROG   );
                                                    
  }
    


  /* Memory tests are handled in a device specific way */
  if (msg->msg_body.diag_code_control.control_code & (DMSG_CODE_CTRL_MEM_TEST_ALL |
                                                      DMSG_CODE_CTRL_STOP_TEST    |
                                                      DMSG_CODE_CTRL_LOCAL        |
                                                      DMSG_CODE_CTRL_MARCH        |
                                                      DMSG_CODE_CTRL_ADDR         |
                                                      DMSG_CODE_CTRL_55           |
                                                      DMSG_CODE_CTRL_PSEUDO       |
                                                      DMSG_CODE_CTRL_PROG) )   {
    if (diagCfg.dsiuTestMemory)  {
      (*diagCfg.dsiuTestMemory)(msg);

      if (msg->msg_body.diag_code_control.control_code & DMSG_CODE_CTRL_END_REPORT)  {
        readMem.header.echo     = 0;
        readMem.header.length   = 11;
        readMem.header.chnum    = 0;
        readMem.header.msg_id   = DMSG_HD_MSG_ID_REQ_MEM_READ;
        readMem.header.trans_id = msg->header.trans_id;

        readMem.msg_body.req_mem_read.addr_msw = ((tulong)&dregStatus.memTest) >> 16;
        readMem.msg_body.req_mem_read.addr_lsw = ((tulong)&dregStatus.memTest) & 0xffff;
        readMem.msg_body.req_mem_read.mem_type = DMSG_MEM_TYPE_DATA;
        readMem.msg_body.req_mem_read.length_words_msw = (sizeof(dregMemTest_t)) >> 16;
        readMem.msg_body.req_mem_read.length_words_lsw = (sizeof(dregMemTest_t)) & 0xffff;
        readMem.msg_body.req_mem_read.msg_rate = 0;


        dsiuMsgReqMemRead (&readMem);
      }
    }
  }

  /* RAMSET test if requested */
  if (msg->msg_body.diag_code_control.control_code & DMSG_CODE_CTRL_RAMSET_TEST)  {
    if (diagCfg.dsiuTestRamset)  {
      (*diagCfg.dsiuTestRamset)();
    }
  }

} /* disuMsgDiagCodeControl */

/******************************************************************************
 * FUNCTION PURPOSE: Process status request message
 ******************************************************************************
 * DESCRIPTION: Generates a status response
 ******************************************************************************/
void dsiuMsgStatusRequest (dmsg_t *msg)
{
  tint   i;
  tuint *p;
  tuint *q;

  /* Generate the response */
  dsiuContext.msgTxBuf->header.length   = msg->header.length;
  dsiuContext.msgTxBuf->header.chnum    = 0;
  dsiuContext.msgTxBuf->header.msg_id   = DMSG_DH_MSG_ID_STATUS_RESPONSE;
  dsiuContext.msgTxBuf->header.trans_id = msg->header.trans_id;

  /* Echo back any parameters in the message body */
  p = (tuint *)&(msg->msg_body);
  q = (tuint *)&(dsiuContext.msgTxBuf->msg_body);
  for (i = 0; i < msg->header.length - MGB_HEADER_SIZE; i++)
    *q++ = *p++;

  /* Send the message */
  dsiu_send_msg (dsiuContext.msgTxBuf);
} /* dsiuMsgStatusRequest */

/******************************************************************************
 * FUNCTION PURPOSE: Process a system checksum request
 ******************************************************************************
 * DESCRIPTION: Calls the system checksum function and replies. There are
 *              two replies possible. A general ack for a sys mem clear request,
 *              and a checksum reply for a check sum computation request.
 ******************************************************************************/

void dsiuMsgSysChecksum (dmsg_t *msg)
{
  tint result;

  /* If system checksum is not enabled return an error indication */
  if (!diagCfg.dsiuChksum)
    dsiuErrorIndication (msg->header.trans_id, msg->header.chnum, MGB_ERR_INVD_MSG_FUNC_NOT_SUPPORTED);

  /* Call the checksum handler. This function will generate the checksum response */
  result = (*diagCfg.dsiuChksum)(&msg->msg_body.sys_checksum, 
                                 &dsiuContext.msgTxBuf->msg_body.sys_checksum_ret);

  /* Send reply based on the result */
  switch (result)  {

    /* If a memory init was performed, send an ack when complete */
    case DMSG_MEM_INIT: 
      if (msg->header.trans_id)  {
        dsiu_msg_generate_ack (dsiuContext.msgTxBuf, 
                             msg->header.trans_id, msg->header.msg_id, MGB_ACK_TYPE_DONE);
        dsiu_send_msg (dsiuContext.msgTxBuf);
      }
      break;

    /* If a memory checksum was performed, send the checksum result */
    case DMSG_MEM_CHKSUM:
      dsiuContext.msgTxBuf->header.length   = DSIU_MSG_HEADER_SIZE + DSIU_MSG_SYS_CHKSUM_RET_SIZE;
      dsiuContext.msgTxBuf->header.chnum    = 0;
      dsiuContext.msgTxBuf->header.msg_id   = DMSG_HD_MSG_ID_SYS_CHKSUM;
      dsiuContext.msgTxBuf->header.trans_id = msg->header.trans_id;
      dsiu_send_msg (dsiuContext.msgTxBuf);
      break;

    /* Any other result code is due to an error */
    default:
      dsiuErrorIndication (msg->header.trans_id, msg->header.chnum, MGB_ERR_INVLD_STAT_REQUEST);
  }
} /* dsiuMsgSysChecksum */
 
/**************************************************************************
 * FUNCTION PURPOSE: Perform EMIF configuration
 **************************************************************************
 * DESCRIPTION: The EMIF configuration is passed through to the HAL
 **************************************************************************/
void dsiuMsgEmifCfg (dmsg_t *msg)
{
  halMemConfig_t memCfg;
  halReturn_t    halReturn;
  dmsg_t        *response;

  memCfg.memCfgFunction = HAL_MEMCFGFUNC_ECFG;
  memCfg.u.ecfg.u.sdram.valid_params = msg->msg_body.emif_config.u.sdram.valid_params;
  memCfg.u.ecfg.u.sdram.pll          = msg->msg_body.emif_config.u.sdram.pll;
  memCfg.u.ecfg.u.sdram.configMsw    = msg->msg_body.emif_config.u.sdram.configMsw;
  memCfg.u.ecfg.u.sdram.configLsw    = msg->msg_body.emif_config.u.sdram.configLsw;
  memCfg.u.ecfg.u.sdram.refreshMsw   = msg->msg_body.emif_config.u.sdram.refreshMsw;
  memCfg.u.ecfg.u.sdram.refreshLsw   = msg->msg_body.emif_config.u.sdram.refreshLsw;
  memCfg.u.ecfg.u.sdram.timing1Msw   = msg->msg_body.emif_config.u.sdram.timing1Msw;
  memCfg.u.ecfg.u.sdram.timing1Lsw   = msg->msg_body.emif_config.u.sdram.timing1Lsw;
  memCfg.u.ecfg.u.sdram.timing2Msw   = msg->msg_body.emif_config.u.sdram.timing2Msw;
  memCfg.u.ecfg.u.sdram.timing2Lsw   = msg->msg_body.emif_config.u.sdram.timing2Lsw;
  memCfg.u.ecfg.u.sdram.timing3Msw   = msg->msg_body.emif_config.u.sdram.timing3Msw;
  memCfg.u.ecfg.u.sdram.timing3Lsw   = msg->msg_body.emif_config.u.sdram.timing3Lsw;
  memCfg.u.ecfg.u.sdram.phycMsw      = msg->msg_body.emif_config.u.sdram.phycMsw;
  memCfg.u.ecfg.u.sdram.phycLsw      = msg->msg_body.emif_config.u.sdram.phycLsw;
  memCfg.u.ecfg.u.sdram.ddr_pll      = msg->msg_body.emif_config.u.sdram.ddr_pll;

  if (dsiuContext.hal->halMemConfig)  {
    halReturn = (*dsiuContext.hal->halMemConfig) (&memCfg);
    if (halReturn != HAL_SUCCESS)  {
      dsiuErrorIndication (msg->header.trans_id, 0, MGB_ERR_HAL);
	} else  {
      response = dsiu_msg_generate_ack (dsiuContext.msgTxBuf, 
               msg->header.trans_id, msg->header.msg_id, MGB_ACK_TYPE_DONE);
      dsiu_send_msg (response);
	}
  }

} /* dsiuMsgEmifCfg */

/**  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! **
 * The following functions are used only in the simulation of application code
 **  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! **/
#ifdef DSIU_SIMULATE_APPLICATION
/*****************************************************************************
 * FUNCTION PURPOSE: Generate a restart message
 *****************************************************************************
 * DESCRIPTION: Sends a simulated restart message to the host
 *****************************************************************************/
void dsiu_simu_restart (void)
{
  extern int dspImageMap[];

  /* Message header */
  dsiuContext.msgTxBuf->header.length   = DSIU_MSG_HEADER_SIZE + 
                                          DSIU_MSG_RESTART_SIZE;
  dsiuContext.msgTxBuf->header.chnum    = 0;
  dsiuContext.msgTxBuf->header.msg_id   = DMSG_DH_MSG_ID_RESTART;
  dsiuContext.msgTxBuf->header.trans_id = 0;

  /* The message body is copied from the DSP image section directly */
  memcpy ((void *)&dsiuContext.msgTxBuf->msg_body.restart, dspImageMap, DSIU_MSG_RESTART_SIZE);
  
  /***************************************************************************
   * Boot track progress. Indicate that the restart has been sent. This should
   * always be the value in the boot track location on successful operation.
   ***************************************************************************/
  dsiuBootTrack (DREG_BOOT_TRACK_SEND_RESTART);
  
  /* Send the message */
  dsiu_send_msg (dsiuContext.msgTxBuf);

  /* Update auto Run status */
  if (diagCfg.ctable.dsiuArun)
    (*diagCfg.ctable.dsiuArun->setStat) (DSIU_AUTOCTL_STAT_RESTART);


} /* dsiu_simu_restart */



/*****************************************************************************
 * FUNCTION PURPOSE: Allocate and initialize a tdm port
 *****************************************************************************
 * DESCRIPTION: Completes port setup given a complete configuration. Returns
 *              -1 on any hal failure
 *****************************************************************************/
tint dsiu_create_tdm_port (halTdmInit_t *tdmInit, dsiuPort_t *tdmPort, tint cfgIndex)
{
  halMemBuffer_t *bufs;
  tint            nbufs;
  void           *bstart;
  tulong          bsize;
  tuint           alignUsed;
  halReturn_t     halReturn;

  /* Indicate that the port status is allocated, but not initialized */
  DREG_PERIPH_SET_PORT   (tdmPort->status->idNumStatus, tdmInit->physPort);
  DREG_PERIPH_SET_STATUS (tdmPort->status->idNumStatus, DREG_PERIPH_STATUS_UNINITIALIZED);

  /* Get the size requirements for the specified configuration */
  halReturn = (*dsiuContext.hal->tdm->halTdmGetSizes) (&nbufs, &bufs, tdmInit);
  if (halReturn != HAL_SUCCESS)  
    return (-1);


  /* Allocate the buffers */
  if (bufs[HAL_TDM_HEAP_BUFN].base == NULL)  {
    bstart = dsiuBufferAlign (diagCfg.halHeapBufStart + dsiuContext.halHeapUsed, 
                              diagCfg.halHeapBufEnd, bufs[HAL_TDM_HEAP_BUFN].log2align,
                              &bsize, &alignUsed);
    if (bsize < bufs[HAL_TDM_HEAP_BUFN].size)  
      return (-1);

    bufs[HAL_TDM_HEAP_BUFN].base = bstart;
    dsiuContext.halHeapUsed += bufs[HAL_TDM_HEAP_BUFN].size + alignUsed;;
   }

#ifndef TSIP_LLD
  /* Allocate the tx buffer */
  if (bufs[HAL_TDM_TX_BUFN].base == NULL)  {
    bstart = dsiuBufferAlign (diagCfg.tdmConfig->tdmDmaMem[cfgIndex].txStart,
                              diagCfg.tdmConfig->tdmDmaMem[cfgIndex].txEnd,
                              bufs[HAL_TDM_TX_BUFN].log2align, &bsize, &alignUsed);
    if (bsize < bufs[HAL_TDM_TX_BUFN].size)  
      return (-1);

    bufs[HAL_TDM_TX_BUFN].base = bstart;
  }

  /* Allocate the rx buffer */
  if (bufs[HAL_TDM_RX_BUFN].base == NULL)  {
    bstart = dsiuBufferAlign (diagCfg.tdmConfig->tdmDmaMem[cfgIndex].rxStart,
                              diagCfg.tdmConfig->tdmDmaMem[cfgIndex].rxEnd,
                              bufs[HAL_TDM_RX_BUFN].log2align, &bsize, &alignUsed);
    if (bsize < bufs[HAL_TDM_RX_BUFN].size)  
      return (-1);
    bufs[HAL_TDM_RX_BUFN].base = bstart;
  }
#endif

  /* Initialize the port */
  halReturn = (*dsiuContext.hal->tdm->halTdmInit) (tdmInit, nbufs, bufs);
  if (halReturn != HAL_SUCCESS)  
    return (-1);

  /* Store returned information */
  tdmPort->physPort = tdmInit->physPort;
  tdmPort->halId    = tdmInit->halId;
  DREG_PERIPH_SET_STATUS (tdmPort->status->idNumStatus, DREG_PERIPH_STATUS_CFGD_DISABLED);

  return (0);

} /* dsiu_create_tdm_port */


/*****************************************************************************
 * FUNCTION PURPOSE: Enable a serial port
 *****************************************************************************
 * DESCRIPTION: The port is enabled from the parameters avaialable from
 *              the hardware config message.
 *****************************************************************************/
tint dsiu_hwcfg_enable_port (dmsg_t *msg, tint idx, tint port)
{
  halTdmControl_t tdmCtl;
  halTdmInit_t    tdmInit;
  halReturn_t     halReturn;
  tulong          srate;
  tuint           clkCfg;

  /* Configure the port */
  tdmInit.physPort = port;
  tdmInit.fcn = HAL_TDM_AS_TDM;

  tdmInit.sizes.maxChannels  = diagCfg.maxChannels;
  tdmInit.sizes.subFrameSize = diagCfg.tdmConfig->subFrameSize[idx];
  tdmInit.sizes.wordSize     = diagCfg.tdmConfig->wordSize[idx];
  tdmInit.sizes.buffIncr     = diagCfg.tdmConfig->buffIncr[idx];

  tdmInit.tdmCompanding   = msg->msg_body.hw_config.companding;
  tdmInit.start           = HAL_TDM_ALWAYS_ON;
  tdmInit.maxPhase        = diagCfg.tdmConfig->maxPhase[idx];
  tdmInit.maxMegaPhase    = diagCfg.tdmConfig->maxMegaPhase[idx];
  tdmInit.mcbsp_block     = diagCfg.opBlock;
  tdmInit.initTs          = NULL;
  tdmInit.aicInst         = NULL;
  tdmInit.subFrameCallout = diagCfg.dsiuSubRateFcn;
  tdmInit.cxt             = &dsiuContext.tdmPort[idx];

  clkCfg = msg->msg_body.hw_config.tdm_clock_config;
    tdmInit.tx.dataDelay = clkCfg & 0x7;
  tdmInit.tx.bdxDelay  = msg->msg_body.hw_config.bdx_delay_control;
  tdmInit.tx.tsPerFrame = msg->msg_body.hw_config.no_fsx_fsr;


  switch (tdmInit.tdmCompanding)  {
    case DMSG_ALAW_COMPANDING:
      tdmInit.tx.idleDrive = DSIU_COMPAND_ULAW_SILENCE;
      break;
    case DMSG_MULAW_COMPANDING:
      tdmInit.tx.idleDrive = DSIU_COMPAND_ALAW_SILENCE;
      break;
    case DMSG_LINEAR_COMPANDING:
      tdmInit.tx.idleDrive = DSIU_COMPAND_LINEAR_SILENCE;
      break;
  }
  tdmInit.tx.fsyncPol    = (clkCfg >> 4) & 1;
  tdmInit.tx.fsyncClkPol = 1;
  tdmInit.tx.clkPol      = (clkCfg >> 3) & 1;
  tdmInit.tx.clkRate     = 1;

  /* The clk rate isn't provided directly, and is only required
   * for tsip. For TSIP it can be derived from the data rate. Because
   * it's currently only used for TSIP, the sample size in bit is
   * set to 8, instead of using the hw config parameter. */
  if (msg->msg_body.hw_config.sampling_frequency == DMSG_SAMP_FREQ_8KHZ)
    srate = 8000;
  else if (msg->msg_body.hw_config.sampling_frequency == DMSG_SAMP_FREQ_16KHZ)
    srate = 16000;
  else
    srate = 48000;

  tdmInit.tx.dataRate  = (srate * msg->msg_body.hw_config.no_fsx_fsr * 8) / 1024000;
  tdmInit.tx.clkRedund = 0;
  tdmInit.tx.clkSrc    = 0;

  /* Most of the receive configuration matches the transmit */
    tdmInit.tx.clkRate     = 1;
    tdmInit.tx.tsPerFrame  = msg->msg_body.hw_config.no_fsx_fsr;
    tdmInit.tx.dataRate    = (8000 * tdmInit.tx.tsPerFrame * 8) / 1000000;;
    tdmInit.tx.clkRedund   = 0;
    tdmInit.tx.clkSrc      = 0;
    tdmInit.tx.idleDrive   = 1;
#if 0    
    if (tdmInit.physPort ==0)
        tdmInit.tx.idleDrive   = (msg->msg_body.hw_config.serial_port_assignment  >> 8) | 0x3; //msg->msg_body.port_config.u.rtInit.tx.idleDrive;
        else
      tdmInit.tx.idleDrive   = (msg->msg_body.hw_config.serial_port_assignment  >> 10) | 0x3; //msg->msg_body.port_config.u.rtInit.tx.idleDrive;
#endif           
    
    /* The receive configuration matches the transmit except one*/
  tdmInit.rx = tdmInit.tx;
    tdmInit.rx.clkPol      =  (tdmInit.tx.clkPol ==0)? 1:0;

  tdmInit.rx.fsyncPol    = (clkCfg >> 12) & 1;
  tdmInit.rx.fsyncClkPol = 1;
  tdmInit.rx.clkPol      = (clkCfg >> 11) & 1;
  tdmInit.rx.dataDelay   = (clkCfg >> 8) & 0x7;


  /* TDM port config, like mcbsp in himalaya, has 3 steps
     1. allocate memory and initialise TDM params: halTdmMcBspInit()
     2. enable the TDM port using halTdmControl : calls halStartupTdm()
  */
  /* 1. create port */
  if (dsiu_create_tdm_port (&tdmInit, &dsiuContext.tdmPort[idx], idx) != 0)  {
    dsiuErrorIndication (msg->header.trans_id, 0, MGB_ERR_HAL);
    return (-1);
  }

  /* 2.  Enable the port. Currently only core 0 can do this */
  tdmCtl.mode = HAL_TDM_PORT_ENABLE;
  halReturn = (*dsiuContext.hal->tdm->halTdmControl)(tdmInit.halId, &tdmCtl); 
  if (halReturn != HAL_SUCCESS)  {
    dsiuErrorIndication (msg->header.trans_id, 0, MGB_ERR_HAL);
    return (-1);
  }

  return (0);

} /* dsiu_hwcfg_enable_port */

/*****************************************************************************
 * FUNCTION PURPOSE: Generate ready indication message
 *****************************************************************************
 * DESCRIPTION: This function handles the hardware config message from the
 *              host. The TDM parameters are used to configure the TDM port.
 *****************************************************************************/
void dsiuMsgHwConfig (dmsg_t *msg)
{
#ifndef ITDM
  tint            i;
  tint            porta;
  tint            portb;
#endif
  volatile tuint iwait = 0;

  while (iwait);

  /* Program clock PLL */
  if (*dsiuContext.hal->halPllControl != NULL) {
    (*dsiuContext.hal->halPllControl) (msg->msg_body.hw_config.clk_multplier,1);
  }

  /*initialize the variable that stores the CPU frequency*/
  dsiuContext.core_mhz = msg->msg_body.hw_config.core_mhz;
  
#ifndef ITDM

  if (diagCfg.nTdmPorts > 0)  {

    porta = msg->msg_body.hw_config.serial_port_assignment & 0x000f;
    portb = (msg->msg_body.hw_config.serial_port_assignment & 0x00f0) >> 4;

    /* If the primary port is not assigned, then do not configure the port */
    if (porta != DMSG_PORT_UNUSED)  {

      /* Panic if there is no tdm */
      if (!dsiuContext.hal->tdm)
        dsiuMsgError (msg, DSIU_MSG_ERR_NO_TDM);

      /* Find an un-allocated port */
      for (i = 0; i < diagCfg.nTdmPorts; i++)  {
        if (DREG_PERIPH_READ_STATUS((dsiuContext.tdmPort[i]).status->idNumStatus) == DREG_PERIPH_STATUS_UNALLOCATED)
          break;
      }

      if (i == diagCfg.nTdmPorts)  {
        dsiuMsgError (msg, DSIU_MSG_ERR_NO_FREE_PORTS);
        return;
      }

      if (dsiu_hwcfg_enable_port (msg, i, porta))
        return;

      DREG_PERIPH_SET_STATUS(dsiuContext.tdmPort[i].status->idNumStatus, DREG_PERIPH_STATUS_CFG_ENABLED);
      dsiuContext.tdmLogicAlloc |= (1 << i);

      /* Flag the system to check enable timeslots that may be on the port */
      dsiuContext.masterCntrl->tdmCntrl.command = DREG_CNTRL_CONFIG;
    }

    if ( (portb != porta) && (portb != DMSG_PORT_UNUSED) )  {

      /* Panic if there is no tdm */
      if (!dsiuContext.hal->tdm)
        dsiuMsgError (msg, DSIU_MSG_ERR_NO_TDM);

      /* Find an un-allocated port */
      for (i = 0; i < diagCfg.nTdmPorts; i++)  {
        if (DREG_PERIPH_READ_STATUS((dsiuContext.tdmPort[i]).status->idNumStatus) == DREG_PERIPH_STATUS_UNALLOCATED)
          break;
      }

      if (i == diagCfg.nTdmPorts)  {
        dsiuMsgError (msg, DSIU_MSG_ERR_NO_FREE_PORTS);
        return;
      }

      if (dsiu_hwcfg_enable_port (msg, i, portb))
        return;

      DREG_PERIPH_SET_STATUS(dsiuContext.tdmPort[i].status->idNumStatus, DREG_PERIPH_STATUS_CFG_ENABLED);
      dsiuContext.tdmLogicAlloc |= (1 << i);

      /* Flag the system to check enable timeslots that may be on the port */
      dsiuContext.masterCntrl->tdmCntrl.command = DREG_CNTRL_CONFIG;

    }


  } /* if (diagCfg.nTdmPorts > 0) */

#endif



  /* Enter diagnostic mode */
  dsiuContext.mode = DSIU_MODE_DIAG;
#ifdef RELMSG_SUPPORT
  dsiuContext.msg_burst_size =  msg->msg_body.hw_config.msg_burst_size;
#endif
  
  /* Return the ready indication */
  /* Message header */
  dsiuContext.msgTxBuf->header.length   = DSIU_MSG_HEADER_SIZE;
  dsiuContext.msgTxBuf->header.chnum    = 0;
  dsiuContext.msgTxBuf->header.msg_id   = DMSG_DH_MSG_ID_READY;
  dsiuContext.msgTxBuf->header.trans_id = msg->header.trans_id;

  while (iwait);


  /* Send the message */
  dsiu_send_msg (dsiuContext.msgTxBuf);

  /* Update auto Run status */
  if (diagCfg.ctable.dsiuArun)
    (*diagCfg.ctable.dsiuArun->setStat) (DSIU_AUTOCTL_STAT_READY);

#ifdef RELMSG_SUPPORT  
  DSIU_SET_MSG_SEQ_NUM(0);
#endif  
  dsiuContext.masterCntrl->tdmCntrl.command = DREG_CNTRL_CONFIG;

}  /* dsiuMsgHwConfig */

#ifdef RELMSG_SUPPORT
/*****************************************************************************
 * FUNCTION PURPOSE: Message Ack processing
 *****************************************************************************
 * DESCRIPTION: Send GENERAL_ACK for each valid message recieved.
 *              Drop the message if necessary by returning TRUE
 *              
 *****************************************************************************/
bool siuProcMsgAck(mgb_t *message)
{
  mgb_t* p_msg = dsiuContext.msgTxBuf;
  mgbHeader_t* p_hdr = &message->header;
  tuint*       p_body = (tuint*)&message->msg_body;
  tuint seq_num, seq_num_expected;
  bool  drop = FALSE;
  bool  ack = FALSE;

  /* The message ack processing should be done only if it is enabled and
   * the download is complete
   */
  if((dsiuContext.mode != DSIU_MODE_DIAG) || !dsiuContext.msg_burst_size )
    return(FALSE);
    
  /* Verify the message sequence number */
  seq_num_expected = DSIU_GET_MSG_SEQ_NUM();
  seq_num = p_body[p_hdr->length - MGB_HEADER_SIZE - 1]; 
  
  if (seq_num_expected == seq_num) {
     /* Expected seq num matches, ack and process the message, update the expected seq num */
     ack = TRUE;
     if(++seq_num_expected > DSIU_MAX_MSG_SEQ_NUM)
       seq_num_expected = 0;
       
     DSIU_SET_MSG_SEQ_NUM(seq_num_expected);
     
     /* restore the currect message length */
     p_hdr->length--;  
       
  }
  else if (((seq_num_expected >= dsiuContext.msg_burst_size) && ((seq_num_expected - seq_num) <= dsiuContext.msg_burst_size)) ||    
           ((seq_num_expected < dsiuContext.msg_burst_size) && ((seq_num < seq_num_expected) ||
                                       ((DSIU_MAX_MSG_SEQ_NUM - seq_num + seq_num_expected) < dsiuContext.msg_burst_size))))
  {
     /* seq_num is in the valid range, the previous ack was lost
      * drop, but ack the message */
     ack = TRUE;
     drop = TRUE;  
     
  }
  else {
    /* message with invalid ack is received, send Error Message and drop the message */
    drop = TRUE; 
  }
  
  /* Prepare and send the message ack */      
  if (ack) {
    
    p_msg->header.length   = MGB_HEADER_SIZE + DSIU_MSG_STATE_ACK_SIZE;
    /* Channel number indicates the which channel underwent state change */
    p_msg->header.chnum    = 0;
    p_msg->header.msg_id   = DMSG_DH_MSG_ID_STATE_ACK;
    p_msg->header.trans_id = p_hdr->trans_id; 

    p_msg->msg_body.state_ack.ack_type   = MGB_ACK_TYPE_ACK;
    p_msg->msg_body.state_ack.msg_id = p_hdr->msg_id;
    p_msg->msg_body.state_ack.seq_num = seq_num;
    p_msg->msg_body.state_ack.priority =  MGB_MSG_PRIORITY_NORMAL;
    
    /* Send the message */
    dsiu_send_msg (p_msg);
    
  }
  return (drop);
}
#endif /* RELMSG_SUPPORT */
#endif /* DSIU_SIMULATE_APPLICATION */

/*****************************************************************************
 * FUNCTION PURPOSE: Generate an error indication message
 *****************************************************************************
 * DESCRIPTION: Generates an error indication message with the specified
 *              error code.
 *****************************************************************************/
void dsiuErrorIndication (tuint transId, tuint ch, tuint ecode)
{
  /* Message header */
  dsiuContext.msgTxBuf->header.length   = DSIU_MSG_HEADER_SIZE +
                                          DSIU_MSG_ERROR_INDICATION_SIZE;
  dsiuContext.msgTxBuf->header.chnum    = ch;
  dsiuContext.msgTxBuf->header.msg_id   = DMSG_DH_MSG_ID_ERROR_INDICATION;
  dsiuContext.msgTxBuf->header.trans_id = transId;

  /* Message body */
  dsiuContext.msgTxBuf->msg_body.error_indication.error_code = ecode;

  /* Send the message */
  dsiu_send_msg (dsiuContext.msgTxBuf);
} /* dsiuErrorIndication */
  

/*******************************************************************************
 * FUNCTION PURPOSE: Intialize a TDM port
 *******************************************************************************
 * DESCRIPTION: Configures a TDM port as specified
 *******************************************************************************/
#ifdef DSIU_SIMULATE_APPLICATION
void dsiuMsgTdmPortConfig (dmsg_t *msg)
{
  halTdmInit_t    tdmInit;
  halTdmControl_t tdmCtl;
  halReturn_t     halReturn;
  tint            i;

  /* Panic if there is no tdm */
  if (!dsiuContext.hal->tdm)
    dsiuMsgError (msg, DSIU_MSG_ERR_NO_TDM);


  /* Check to see if the port has already been initialized. */
  for (i = 0; i < diagCfg.nTdmPorts; i++)  {
    if (DREG_PERIPH_READ_PORT(dsiuContext.tdmPort[i].status->idNumStatus) == msg->msg_body.port_config.physPort)  {
      if (DREG_PERIPH_READ_STATUS(dsiuContext.tdmPort[i].status->idNumStatus) != DREG_PERIPH_STATUS_UNALLOCATED)  {
        dsiuMsgError (msg, DSIU_MSG_ERR_PORT_REALLOC);
        return;  /* This port was already initialized */
      }
    }
  }
  
  
  
  /* Find an un-allocated port */
  for (i = 0; i < diagCfg.nTdmPorts; i++)  {
    if (DREG_PERIPH_READ_STATUS((dsiuContext.tdmPort[i]).status->idNumStatus) == DREG_PERIPH_STATUS_UNALLOCATED)
      break;
  }
  
  if (i == diagCfg.nTdmPorts)  {
    dsiuMsgError (msg, DSIU_MSG_ERR_NO_FREE_PORTS);
    return;
  }


  if (msg->msg_body.port_config.mode == DMSG_RUN_TIME_INIT_START)  {

    tdmInit.physPort  = msg->msg_body.port_config.physPort;
    tdmInit.fcn   = HAL_TDM_AS_TDM;
    tdmInit.start = HAL_TDM_ALWAYS_ON;

    tdmInit.subFrameCallout = diagCfg.dsiuSubRateFcn;
    tdmInit.cxt             = &dsiuContext.tdmPort[i];

    tdmInit.sizes.maxChannels  = diagCfg.maxChannels;
    tdmInit.sizes.subFrameSize = diagCfg.tdmConfig->subFrameSize[i];
    tdmInit.sizes.wordSize     = diagCfg.tdmConfig->wordSize[i];
    tdmInit.sizes.buffIncr     = diagCfg.tdmConfig->buffIncr[i];

    tdmInit.maxPhase     = diagCfg.tdmConfig->maxPhase[i];
    tdmInit.maxMegaPhase = diagCfg.tdmConfig->maxMegaPhase[i];



    tdmInit.tdmCompanding = msg->msg_body.port_config.companding;


    tdmInit.tx.dataDelay   = msg->msg_body.port_config.u.rtInit.tx.dataDelay;
    tdmInit.tx.bdxDelay    = msg->msg_body.port_config.u.rtInit.tx.bdxDelay;
    tdmInit.tx.idleDrive   = msg->msg_body.port_config.u.rtInit.tx.idleDrive;
    tdmInit.tx.fsyncPol    = msg->msg_body.port_config.u.rtInit.tx.fsyncPolarity;
    tdmInit.tx.fsyncClkPol = msg->msg_body.port_config.u.rtInit.tx.fsyncClkPolarity;
    tdmInit.tx.clkPol      = msg->msg_body.port_config.u.rtInit.tx.clkPolarity;
    tdmInit.tx.clkRate     = msg->msg_body.port_config.u.rtInit.tx.clkRate;
    tdmInit.tx.tsPerFrame  = msg->msg_body.port_config.u.rtInit.tx.tsPerFrame;
    tdmInit.tx.dataRate    = msg->msg_body.port_config.u.rtInit.tx.dataRate;
    tdmInit.tx.clkRedund   = msg->msg_body.port_config.u.rtInit.tx.clkRedundancy;
    tdmInit.tx.clkSrc      = msg->msg_body.port_config.u.rtInit.tx.clkSource;
    tdmInit.sizes.wordSize = msg->msg_body.port_config.u.rtInit.tx.wordSize;

    tdmInit.rx.dataDelay   = msg->msg_body.port_config.u.rtInit.rx.dataDelay;
    tdmInit.rx.bdxDelay    = msg->msg_body.port_config.u.rtInit.rx.bdxDelay;
    tdmInit.rx.idleDrive   = msg->msg_body.port_config.u.rtInit.rx.idleDrive;
    tdmInit.rx.fsyncPol    = msg->msg_body.port_config.u.rtInit.rx.fsyncPolarity;
    tdmInit.rx.fsyncClkPol = msg->msg_body.port_config.u.rtInit.rx.fsyncClkPolarity;
    tdmInit.rx.clkPol      = msg->msg_body.port_config.u.rtInit.rx.clkPolarity;
    tdmInit.rx.clkRate     = msg->msg_body.port_config.u.rtInit.rx.clkRate;
    tdmInit.rx.tsPerFrame  = msg->msg_body.port_config.u.rtInit.rx.tsPerFrame;
    tdmInit.rx.dataRate    = msg->msg_body.port_config.u.rtInit.rx.dataRate;
    tdmInit.rx.clkRedund   = msg->msg_body.port_config.u.rtInit.rx.clkRedundancy;
    tdmInit.rx.clkSrc      = msg->msg_body.port_config.u.rtInit.rx.clkSource;

    if (dsiu_create_tdm_port (&tdmInit, &dsiuContext.tdmPort[i], i) != 0)  {
      dsiuErrorIndication (msg->header.trans_id, 0, MGB_ERR_HAL);
      return;
    }


    /* Enable the port */
      tdmCtl.mode = HAL_TDM_PORT_ENABLE;
      halReturn = (*dsiuContext.hal->tdm->halTdmControl)(tdmInit.halId, &tdmCtl); 
      if (halReturn != HAL_SUCCESS)  {
        dsiuErrorIndication (msg->header.trans_id, 0, MGB_ERR_HAL);
        return;
      }

    DREG_PERIPH_SET_STATUS(dsiuContext.tdmPort[i].status->idNumStatus, DREG_PERIPH_STATUS_CFG_ENABLED);
    dsiuContext.tdmLogicAlloc |= (1 << i);
    dsiuContext.tdmPort[i].status->hwId = tdmInit.hwId;

    /* Flag the system to check enable timeslots that may be on the port */
    dsiuContext.masterCntrl->tdmCntrl.command = DREG_CNTRL_CONFIG;

  }

} /* dsiuMsgTdmPortConfig */



#endif /* DSIU_SIMULATE_APPLICATION */

