/*************************************************************************************
 * FILE PURPOSE: Full application ECO interface handler
 *************************************************************************************
 * FILE NAME: dsiuif.c
 *
 * DESCRIPTION: Provides an interface between full application ECOs and
 *				the diagnostic/bootloader.
 *
 *
 * Copyright (C) 2003 Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include "types.h"
#include "dsiuloc.h"
#include "memmgm.h"

/*************************************************************************************
 * FUNCTION PURPOSE: Provides an interface to the ECO memory allocation and new.
 *************************************************************************************
 * DESCRIPTION: Calls the full application get sizes function. Translates the
 *              buffer requirements to the diagnostic format and allocates the
 *              buffers. Calls the ECOs new function to assign the buffer and
 *              returns a pointer to the newly created eco.
 *************************************************************************************/
void siuFullAppEcoNewEco (dsiuEcoGetSizes_t getSizes_fcn, void *sizeCfg,
                          dsiuEcoNew_t newEco_fcn, void *newCfg, void **inst, tuint id)
{
  memBuffer_t *sizeBufs;
  dmemBuffer_t diagBufs[DSIU_MAX_NBUFS];
  tint  stat;
  tint  ecoNbufs;
  tint  i;


  DSIU_DUMMY_ARG(id);

  /* Call the module's GetSizes function. Check validity of response */
  stat = (*getSizes_fcn) (&ecoNbufs, (const memBuffer_t **)&sizeBufs, sizeCfg);
  if (stat || (ecoNbufs > DSIU_MAX_NBUFS))  {
    *inst = NULL;
    return;
  }

  /* Convert the buffer information to the diagnostic style */
  for (i = 0; i < ecoNbufs; i++)  {
    diagBufs[i].log2align = sizeBufs[i].log2align;
    diagBufs[i].size      = sizeBufs[i].size;
  }

  /* Allocate the buffers */
#if defined (DSIU_SIMULATE_APPLICATION)
  if (dsiuAllocBufs (&dsiuContext.diagHeap, ecoNbufs, diagBufs))  {
    *inst = NULL;
    return;
  }
#else
  if (dsiuAllocBufs (&dsiuContext.bootHeap, ecoNbufs, diagBufs))  {
    *inst = NULL;
    return;
  }
#endif


  /* Convert the buffer bases to the ECO format */
  for (i = 0; i < ecoNbufs; i++)  {
    sizeBufs[i].base = diagBufs[i].base;
  }

  /* Call the ECOs new function */
  (*newEco_fcn) (inst, ecoNbufs, sizeBufs, newCfg);

  /* The new function has returned the pointer in inst */

}












