/****************************************************************************
 * FILE PURPOSE: Perfrom application initialization
 ****************************************************************************
 * FILE NAME: dsiuini.c
 *
 * DESCRIPTION: This file contains functions that are used to initialize
 *              the application. They are called only once at system startup
 *              and so can be overlayed with an uninitialized data section.
 *
 * TABS: None
 *
 * Copyright (C) 2002 Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include "types.h"
#include "diagcfg.h"
#include "dsiuloc.h"
#include "dsiumsg.h"
#include "hal.h"
#include "dsiuubt.h"
#include "diagreg.h"
#include "dsiumem.h"
#include "cmisc.h"
#include "dsiuutop.h"
#include "dsiuport.h"
#include "utlbitfield.h"
#include "dsiuotbrun.h"
#include <stdlib.h>
#include <string.h>

#ifdef HAL_INCLUDE_AIC
#include "cslaic20.h"
#endif

extern volatile unsigned int cregister TSCL;

/****************************************************************************
 * Declaration: Holds dummy args. This only prevents compiler warnings.
 ****************************************************************************/
void *dsiuDummy;

/****************************************************************************
 * FUNCTION PURPOSE: Boot track progress indicator
 ****************************************************************************
 * DESCRIPTION: This function is used to help locate the location of 
 *              secondary core boot failures. Since an error message cannot be
 *              sent until the utopia has been configured properly. The
 *              location of the tracker is setup in the linker command file
 *              and loaded into the diagCfg parameters.
 *              The progress codes are listed in dsiuloc.h.
 ****************************************************************************/
void dsiuBootTrack (tulong trackCode)
{
#ifndef DSP_ON_MIPS
  dsiuContext.dregBootTrack->location = trackCode;
#endif
} /* dsiuBootTrack */

/****************************************************************************
 * FUNCTION PURPOSE: Memory allocation Failure trap
 ****************************************************************************
 * DESCRIPTION: Freezes the CPU in an infinite loop. The argument to the
 *              function indicates the source of the memory failure.
 *              The boot track progress will reflect the failure, and the
 *              error code will be placed as well.
 ****************************************************************************/
void dsiuPanic (dsiuFailCode_t code)
{
  DSIU_DUMMY_ARG (code);

  for (;;); 

} /* dsiuPanic */


/******************************************************************************
 * FUNCTION PURPOSE: Return core information
 ******************************************************************************
 * DESCRIPTION: Returns the core ID. The direction hal call is used since
 *              this function can be called prior to hal initialization
 ******************************************************************************/
tint dsiu_get_core_num (void)
{
  halQuery_t qry;

  qry.periph = HAL_QUERY_NCORES;
  (*hal.halQuery) (&qry);
  return (qry.u.core.thisCore);

} /* dsiu_get_core_num */

/*****************************************************************************
 * FUNCTION PURPOSE: Configure and enable the utopia ports
 *****************************************************************************
 * DESCRIPTION: Allocates utopia memory, configures and enables the port.
 *****************************************************************************/
void dsiu_utport_config(tint physPort, tint logPort)
{
  halMemBuffer_t      *bp;
  halMemBuffer_t      *bufs; 
  halQuery_t           utQuery;
  void                *bstart;
  tulong               bsize;
  halUtopiaInit_t      utInit;
  halUtopiaControl_t   utCtl;
  halReturn_t          ret;
  tint                 nbufs;
  tuint                alignUsed;

  /* Return if the buffers for this port have already be used */
  if (dsiuContext.utLogicAlloc & (1 << logPort))
    return;


  /* Query if there is a utopia port */
  utQuery.periph  = HAL_QUERY_UTOPIA;
  utQuery.portNum = physPort;
  (*dsiuContext.hal->halQuery) (&utQuery);

  /* Proceed if there is a utopia port present */
  if ( (utQuery.isPresent == TRUE) && (utQuery.isAvail == TRUE) )  {

    /* Get the memory requirements. */
    utInit.sizes.xmtCellSizeBytes = dsiuContext.diagApp.u.utopia.cellSize;
    utInit.sizes.rcvCellSizeBytes = dsiuContext.diagApp.u.utopia.cellSize;

    utInit.sizes.txBufSizeBytes   = SIU_WORDS_TO_BYTES((tulong)(diagCfg.utConfig->utopiaDmaMem[logPort].txEnd) - (tulong)(diagCfg.utConfig->utopiaDmaMem[logPort].txStart));
    utInit.sizes.rxBufSizeBytes   = SIU_WORDS_TO_BYTES((tulong)(diagCfg.utConfig->utopiaDmaMem[logPort].rxEnd) - (tulong)(diagCfg.utConfig->utopiaDmaMem[logPort].rxStart));
    utInit.sizes.rxCABufSizeBytes = SIU_WORDS_TO_BYTES((tulong)(diagCfg.utConfig->caEnd[logPort]) - (tulong)(diagCfg.utConfig->caStart[logPort]));


    ret = (*dsiuContext.hal->utopia->halUtopiaGetSizes) (&nbufs, &bufs, &utInit.sizes);
    if (ret != HAL_SUCCESS)
      dsiuPanic (DSIU_FAIL_UTOPIA_GETSIZES);


    /* Allocate the Heap buffer */
    bp = &bufs[HAL_HEAP_BUFN];
    if (bp->base == NULL)  {
      bstart = dsiuBufferAlign (diagCfg.halHeapBufStart + dsiuContext.halHeapUsed, diagCfg.halHeapBufEnd, 
                                bp->log2align, &bsize, &alignUsed);
      if (bsize < bp->size)
        dsiuPanic (DSIU_MEM_ALLOC_FAIL_HAL);

      bp->base = (void *)bstart;
      dsiuContext.halHeapUsed += (bp->size + alignUsed);
      DSIU_PORT_ALIGN_HEAPBASE(dsiuContext.halHeapUsed);
    }

    /* Allocate the tx buffer */
    bp = &bufs[HAL_UT_TX_BUFN];
    if (bp->base == NULL)  {
      bstart = dsiuBufferAlign (diagCfg.utConfig->utopiaDmaMem[logPort].txStart, 
                                diagCfg.utConfig->utopiaDmaMem[logPort].txEnd,
                                bp->log2align, &bsize, &alignUsed);
      if (bsize < bp->size)
        dsiuPanic (DSIU_MEM_ALLOC_FAIL_HAL); bp->base = (void *)bstart;
    }

    /* Allocate the rx buffer */
    bp = &bufs[HAL_UT_RX_BUFN];
    if (bp->base == NULL)  {
      bstart = dsiuBufferAlign (diagCfg.utConfig->utopiaDmaMem[logPort].rxStart, 
                                diagCfg.utConfig->utopiaDmaMem[logPort].rxEnd,
                                bp->log2align, &bsize, &alignUsed);
      if (bsize < bp->size)
        dsiuPanic (DSIU_MEM_ALLOC_FAIL_HAL);
      bp->base = (void *)bstart;
    }

    /* Allocate the catch all buffer */
    bp = &bufs[HAL_UT_CA_BUFN];
    if (bp->base == NULL)  {
      bstart = dsiuBufferAlign (diagCfg.utConfig->caStart[logPort], 
                                diagCfg.utConfig->caEnd[logPort],
                                bp->log2align, &bsize, &alignUsed);

      if (bsize < bp->size)
        dsiuPanic (DSIU_MEM_ALLOC_FAIL_HAL);
      bp->base = (void *)bstart;
    }


    /* Indicate that the port status is allocated, but uninitialized */
    DREG_PERIPH_SET_PORT   (dsiuContext.utopiaPort[logPort].status->idNumStatus, physPort);
    DREG_PERIPH_SET_STATUS (dsiuContext.utopiaPort[logPort].status->idNumStatus, DREG_PERIPH_STATUS_UNINITIALIZED);

    /* Configure the port */
    utInit.portId   = physPort;
    utInit.busWidth = utQuery.u.utopia.busWidth;
    utInit.slaveId  = dsiuContext.diagApp.u.utopia.slaveId;
    utInit.mode     = diagCfg.utConfig->routeMode[logPort];
    utInit.rcvCell  = diagCfg.ctable.siuUtNdu->nduReceiveInInt;
    utInit.rcvCxt   = &dsiuContext.utopiaPort[logPort];
    utInit.rcvCellBufFull = NULL;

    ret = (*dsiuContext.hal->utopia->halUtopiaInit) (&utInit, nbufs, bufs);
    if (ret != HAL_SUCCESS)
      dsiuPanic (DSIU_FAIL_UTOPIA_INIT);

    /* Store context information */
    dsiuContext.utopiaPort[logPort].physPort = physPort;
    dsiuContext.utopiaPort[logPort].halId    = utInit.halId;
    DREG_PERIPH_SET_STATUS (dsiuContext.utopiaPort[logPort].status->idNumStatus, DREG_PERIPH_STATUS_CFGD_DISABLED);


    /* Enable the port */
    utCtl.mode = HAL_UTOPIA_PORT_ENABLE;
    if ((*dsiuContext.hal->utopia->halUtopiaControl)(dsiuContext.utopiaPort[logPort].halId, &utCtl) != 0)
      dsiuPanic (DSIU_FAIL_UTOPIA_ENABLE);

    DREG_PERIPH_SET_STATUS (dsiuContext.utopiaPort[logPort].status->idNumStatus, DREG_PERIPH_STATUS_CFG_ENABLED);

  } /* ( (utQuery.isPresent == TRUE) && (utQuery.isAvail == TRUE) ) */

  dsiuContext.utLogicAlloc |= (1 << logPort);

} /* dsiu_utport_config */

#ifdef NDU_GMAC
/*****************************************************************************
 * FUNCTION PURPOSE: Configure and enable the mac ports
 *****************************************************************************
 * DESCRIPTION: Allocates mac memory, configures and enables the ports.
 *              physPort is the physical port number, indexed from 0.
 *              logPort is the logical port, and it identifies which 
 *              configuration (stored in diagcfg) will be used.
 *****************************************************************************/
void dsiu_macport_config (tint physPort, tint logPort)
{
  halGmacInit_t gmacInit;
  halGmacSizes_t    sizes;
  halMemBuffer_t      *bp;
  halMemBuffer_t      *bufs; 
  halReturn_t          ret;
  void                *bstart;
  tulong               bsize;
  dsiuPort_t          *dport;
  tuint                alignUsed;
  tint nbufs;
  tint              j;

  /* Return if the buffers for this port have already be used */
  if (dsiuContext.macLogicAlloc & (1 << logPort))
    return;

  dport = &(dsiuContext.macPort[logPort]);

  /************************/
  /* GMAC initialization  */
  /************************/

    /* Transmit */
  sizes.tx.bufferUnitSizeWords  = diagCfg.gmacBufsizeWordsTx[logPort];
  sizes.tx.bufferTotalSizeWords = (tulong)diagCfg.gmacDmaMem[logPort].txEnd - (tulong)diagCfg.gmacDmaMem[logPort].txStart;
  sizes.tx.startAddr            = diagCfg.gmacDmaMem[logPort].txStart;

  /* Receive */
  sizes.rx.bufferUnitSizeWords  = diagCfg.gmacBufsizeWordsRx[logPort];
  sizes.rx.bufferTotalSizeWords = (tulong)diagCfg.gmacDmaMem[logPort].rxEnd - (tulong)diagCfg.gmacDmaMem[logPort].rxStart;
  sizes.rx.startAddr            = diagCfg.gmacDmaMem[logPort].rxStart;

  /* Assembly */
  sizes.maxRxPacketSizeWords = diagCfg.gmacRxAssemSizeWords[logPort];


  /* Get memory requirements from HAL  */
  ret = (*dsiuContext.hal->gmac->halGmacGetSizes) (&nbufs, &bufs, &sizes);
  if (ret != HAL_SUCCESS)
    dsiuPanic (DSIU_FAIL_MAC_GETSIZES);

  /* Allocate the Heap buffer */
  bp = &bufs[HAL_GMAC_HEAP_BUFN];
  if (bp->base == NULL)  {
    bstart = dsiuBufferAlign (diagCfg.halHeapBufStart + dsiuContext.halHeapUsed, diagCfg.halHeapBufEnd, 
                              bp->log2align, &bsize, &alignUsed);
    if (bsize < bp->size)
      dsiuPanic (DSIU_MEM_ALLOC_FAIL_HAL);

    bp->base = (void *)bstart;
    dsiuContext.halHeapUsed += (bp->size + alignUsed);
    DSIU_PORT_ALIGN_HEAPBASE(dsiuContext.halHeapUsed);
  }


  /* Tx buffer */
  bp = &bufs[HAL_GMAC_TX_BUFN];
  if (bp->base == NULL)  {
    bstart = dsiuBufferAlign (diagCfg.gmacDmaMem[logPort].txStart, diagCfg.gmacDmaMem[logPort].txEnd,
                              bp->log2align, &bsize, &alignUsed);

    if (bsize < bp->size)
      dsiuPanic (DSIU_MEM_ALLOC_FAIL_HAL);

    bp->base = (void *)bstart;
  }

  /* Rx buffer */
  bp = &bufs[HAL_GMAC_RX_BUFN];
  if (bp->base == NULL)  {
    bstart = dsiuBufferAlign (diagCfg.gmacDmaMem[logPort].rxStart, diagCfg.gmacDmaMem[logPort].rxEnd,
                              bp->log2align, &bsize, &alignUsed);

    if (bsize < bp->size)
      dsiuPanic (DSIU_MEM_ALLOC_FAIL_HAL);

    bp->base = (void *)bstart;
  }

  /* Assembly buffer */
  bp = &bufs[HAL_GMAC_ASSEM_BUFN];
  if (bp->base == NULL)  {
    bstart = dsiuBufferAlign (diagCfg.gmacRxAssemMem[logPort].start, diagCfg.gmacRxAssemMem[logPort].end,
                              bp->log2align, &bsize, &alignUsed);

  if (bsize < bp->size)
    dsiuPanic (DSIU_MEM_ALLOC_FAIL_HAL);

    bp->base = (void *)bstart;

  }

  /* Provide the memory to hal, get the handle */
  gmacInit.halId = dport->halId = (*dsiuContext.hal->gmac->halGmacAlloc)(nbufs, bufs, &sizes);

  /* The physical port number and status information are configured */
  dport->physPort   = physPort;
  gmacInit.physPort = physPort;
  gmacInit.cfg.port = physPort;
  gmacInit.cxt = (void *)dport;
  DREG_PERIPH_SET_PORT(dport->status->idNumStatus, physPort);
    
  /* Receive callout */
  gmacInit.rcvFrame = diagCfg.ctable.siuGmacNdu->nduReceiveInInt;

  for (j = 1; j < CSL_MAC_MAX_MAC_ADDR; j++)
    gmacInit.cfg.mac[j].core = -1;

  for (j = 0; j < DIAGAPP_MAX_MAC_ADDR; j++)  {
    gmacInit.cfg.mac[j].mac_addr[0] = dsiuContext.diagApp.u.mac.macAddr[j].macAddr[0];
    gmacInit.cfg.mac[j].mac_addr[1] = dsiuContext.diagApp.u.mac.macAddr[j].macAddr[1];
    gmacInit.cfg.mac[j].mac_addr[2] = dsiuContext.diagApp.u.mac.macAddr[j].macAddr[2];
    gmacInit.cfg.mac[j].core = j;
  }

  /* For the bootloader/diag, 400 words are used. This covers 128 word writes for the
   * bootloader, and 10ms pcm for the diag */
  gmacInit.cfg.maxRxPacketSizeBytes = SIU_WORDS_TO_BYTES(diagCfg.gmacRxAssemSizeWords[logPort]);  /* Maximum size accepted  */

  /* port setup - gigabit mode */
  if (dsiuContext.diagApp.u.mac.configFlags & DIAGAPP_MAC_CFG_GE)
    gmacInit.cfg.speed = CSL_MAC_MODE_GIG;
  else
    gmacInit.cfg.speed = CSL_MAC_MODE_10_100;

  /* port setup - duplex */
  if (dsiuContext.diagApp.u.mac.configFlags & DIAGAPP_MAC_CFG_HD)
    gmacInit.cfg.duplex = CSL_MAC_DUPLEX_HALF;
  else
    gmacInit.cfg.duplex = CSL_MAC_DUPLEX_FULL;

  /* port setup - flow control */
  if (dsiuContext.diagApp.u.mac.configFlags & DIAGAPP_MAC_CFG_FC)
    gmacInit.cfg.flow = CSL_MAC_FLOW_CONTROL_ON;
  else
    gmacInit.cfg.flow = CSL_MAC_FLOW_CONTROL_OFF;

  /* port setup - rgmii */
  if (dsiuContext.diagApp.u.mac.configFlags & DIAGAPP_MAC_CFG_RGMII)
    gmacInit.cfg.rgmii = CSL_MAC_RGMII_ON;
  else
    gmacInit.cfg.rgmii = CSL_MAC_RGMII_OFF;

  /* Port setup - rmii 10 */
  if (dsiuContext.diagApp.u.mac.configFlags & DIAGAPP_MAC_CFG_RMII_10) {
    gmacInit.cfg.rmii = CSL_MAC_RMII_ON;
	gmacInit.cfg.rmiiSpeed = CSL_MAC_RMII_SPEED_10MBS;
  }  else  {
    gmacInit.cfg.rmii = CSL_MAC_RMII_OFF;
  }

  /* Port setup - rmii 100 */
  if (dsiuContext.diagApp.u.mac.configFlags & DIAGAPP_MAC_CFG_RMII_100) {
    gmacInit.cfg.rmii = CSL_MAC_RMII_ON;
	gmacInit.cfg.rmiiSpeed = CSL_MAC_RMII_SPEED_100MBS;
  }  else  {
    gmacInit.cfg.rmii = CSL_MAC_RMII_OFF;
  }

  /* port setup - smii */
  if (dsiuContext.diagApp.u.mac.configFlags & DIAGAPP_MAC_CFG_SMII)
    gmacInit.cfg.smii = CSL_MAC_SMII_ON;
  else
    gmacInit.cfg.smii = CSL_MAC_SMII_OFF;

  /* port setup s3mii */
  if (dsiuContext.diagApp.u.mac.configFlags & DIAGAPP_MAC_CFG_S3MII)
    gmacInit.cfg.s3mii = CSL_MAC_S3MII_ON;
  else
    gmacInit.cfg.s3mii = CSL_MAC_S3MII_OFF;

  DREG_PERIPH_SET_STATUS(dport->status->idNumStatus, DREG_PERIPH_STATUS_UNINITIALIZED);

  /* Start up the peripheral - Only core 0 should do the startup of the peripheral*/
  if (dsiuContext.coreId == 0)
    gmacInit.operation = HAL_GMAC_INIT_OPERATION_ENABLE;
  else 
    gmacInit.operation = HAL_GMAC_INIT_OPERATION_CHANNEL;

  ret = (*dsiuContext.hal->gmac->halGmacInit) (&gmacInit);
  if (ret != HAL_SUCCESS)
    dsiuPanic (DSIU_FAIL_MAC_INIT);

 
  dport->status->hwId = gmacInit.hwId;
  DREG_PERIPH_SET_STATUS(dport->status->idNumStatus, DREG_PERIPH_STATUS_CFG_ENABLED);
  dsiuContext.macLogicAlloc |= (1 << logPort);



} /* dsiu_macport_config */
#endif


/******************************************************************************
 * FUNCTION PURPOSE: Initialize HAL (Hardware Abstraction Layer)
 ******************************************************************************
 * DESCRIPTION: The HAL unit is configured and memory allocated.
 *****************************************************************************/
dsiuReturn_t dsiu_hal_init (void)
{
  halNewConfig_t       cfg;  
  halMemBuffer_t      *bp;
  halMemBuffer_t      *bufs; 
  halReturn_t          ret;
  void                *bstart;
  tulong               bsize;
  tint                 nbufs;
  tuint                alignUsed;


  /***************************************************************************
   * Boot track progress: Indicate in hal initialization
   ***************************************************************************/
  dsiuBootTrack (DREG_BOOT_TRACK_HAL_INIT_BEGIN);

  /* Configure HAL using build time parameters */
  cfg.nTdmPorts  = diagCfg.nTdmPorts + diagCfg.nItdmPorts;
  cfg.nPktPorts  = diagCfg.nUtopiaPorts;
  cfg.nGmacPorts = diagCfg.nGmacPorts;
  cfg.nAicPorts  = diagCfg.nAicPorts;


  #ifdef TSIP_LLD
  cfg.tsipSizeCfg.maxChannels =diagCfg.maxChannels;
  cfg.tsipSizeCfg.subFrameSize=diagCfg.tdmConfig->subFrameSize[0]; 
  cfg.tsipSizeCfg.wordSize    =diagCfg.tdmConfig->wordSize[0];
  #endif

  /* Get memory requirements from HAL  */
  ret = (*dsiuContext.hal->halGetSizes) (&nbufs, &bufs, &cfg);
  if (ret != HAL_SUCCESS)
    dsiuPanic (DSIU_HAL_GETSIZES_FAIL);

  #ifndef TSIP_LLD
  /* HAL should request only a single buffer. Other buffers are allocated
   * when required for use. The dsiu context stores the amount of 
   * heap used by hal, and further requests requiring context memory
   * are appended to currently allocated requests */
  if (nbufs != 1)
    dsiuPanic (DSIU_HAL_INVALID_NBUFS);
  #else
  /* TSIP DMA buffers are also allocated at this point in time
     when TSIP LLD is used*/
  if (nbufs != 3)
    dsiuPanic (DSIU_HAL_INVALID_NBUFS);
  #endif

  bp = &bufs[HAL_HEAP_BUFN];
  if (bp->base == NULL)  {
    bstart = dsiuBufferAlign (diagCfg.halHeapBufStart, diagCfg.halHeapBufEnd, 
                              bp->log2align, &bsize, &alignUsed);
    if (bsize < bp->size)
      dsiuPanic (DSIU_MEM_ALLOC_FAIL_HAL);

    bp->base = (void *)bstart;
    dsiuContext.halHeapUsed = bp->size + alignUsed;
    DSIU_PORT_ALIGN_HEAPBASE(dsiuContext.halHeapUsed);
  }

  #ifdef TSIP_LLD
  /* Setting up the DMA buffers base for the TSIP LLD*/

  /* Allocate the tx buffer */
  if (bufs[HAL_LLD_TDM_TX_BUFN].base == NULL)  {
    bstart = dsiuBufferAlign (diagCfg.tdmConfig->tdmDmaMem[0].txStart,
                              diagCfg.tdmConfig->tdmDmaMem[0].txEnd,
                              bufs[HAL_LLD_TDM_TX_BUFN].log2align, &bsize, &alignUsed);
    if (bsize < bufs[HAL_LLD_TDM_TX_BUFN].size)  
      dsiuPanic (DSIU_MEM_ALLOC_FAIL_HAL);
    
    bufs[HAL_LLD_TDM_TX_BUFN].base = bstart;
  }

  /* Allocate the rx buffer */
  if (bufs[HAL_LLD_TDM_RX_BUFN].base == NULL)  {
    bstart = dsiuBufferAlign (diagCfg.tdmConfig->tdmDmaMem[0].rxStart,
                              diagCfg.tdmConfig->tdmDmaMem[0].rxEnd,
                              bufs[HAL_LLD_TDM_RX_BUFN].log2align, &bsize, &alignUsed);
    if (bsize < bufs[HAL_LLD_TDM_RX_BUFN].size)  
      dsiuPanic (DSIU_MEM_ALLOC_FAIL_HAL);

    bufs[HAL_LLD_TDM_RX_BUFN].base = bstart;
  }
  #endif

  /* Initialize the HAL unit */
  if ((*dsiuContext.hal->halNew)(nbufs, bufs, &cfg) != HAL_SUCCESS)
    dsiuPanic (DSIU_HAL_NEW_FAIL);

  /* Store the returned core Id */
  dsiuContext.coreId = cfg.coreId;

  /****************************************************************************
   * Boot track progress: Indicate hal new completed successfully
   ***************************************************************************/
  dsiuBootTrack (DREG_BOOT_TRACK_HAL_NEW_SUCCESS);

  return (DSIU_SUCCESS);

} /* dsiu_hal_init */


/*****************************************************************************
 * FUNCTION PURPOSE: Enable peripherals setup at start time
 *****************************************************************************
 * DESCRIPTION: Brings up peripherals that require only build time
 *              configuration to be brought up.
 *****************************************************************************/
dsiuReturn_t dsiu_periph_setup (void)
{

  /* Memory and cache configuration */
  if (diagCfg.ctable.dsiu->memConfig != NULL)
    (*diagCfg.ctable.dsiu->memConfig)();


#ifndef PA_SS
  /* Initial setup always uses logical port 0, with the physical port from 
   * the diagApp information */

  /* Utopia port enable */
  if ( (diagCfg.nUtopiaPorts > 0) && (dsiuContext.diagApp.u.utopia.cellSize != 0) ) 
    dsiu_utport_config(dsiuContext.diagApp.portNum, 0);


 /* Mac port enable */
#ifdef NDU_GMAC
  if (diagCfg.nGmacPorts > 0)  {
    /* Panic if the logical port number makes no sense */
/* The below error check not needed, as it is not meaningful to compare buffer Vs Ports */
/*
    if (dsiuContext.diagApp.u.mac.macBufferAssign > diagCfg.nGmacPorts)
      dsiuPanic (DSIU_FAIL_NO_MAC);
*/      
#if !(defined(DSIU_SIMULATE_APPLICATION) || defined(DSIU_DAUGHTER_TEST))
    dsiu_macport_config(dsiuContext.diagApp.portNum, 0);
#else
    dsiuContext.diagApp.u.mac.macBufferAssign=0;/* Temporary */
    dsiu_macport_config(dsiuContext.diagApp.portNum, dsiuContext.diagApp.u.mac.macBufferAssign);
#endif
  }
#endif
#endif

  return (DSIU_SUCCESS);

} /* dsiu_periph_setup */


/****************************************************************************
 * FUNCTION PURPOSE: Allocate memory for the DSIU module
 ****************************************************************************
 * DESCRIPTION: Allocates and initializes memory for dsiuContext and for the
 *              dsiuInst instance pointers.
 ****************************************************************************/
void dsiuAllocateContextMemory (void)
{

#ifdef DSP_ON_MIPS
   dsiuContext.dreg->tdmFirstTimeslot = 1;
   dsiuContext.dreg->nChannels = 2;
   dsiuContext.dreg->clocksPerFs = 24;
   dsiuContext.dreg->chCfg[0] = 0x202;
   dsiuContext.dreg->chCfg[1] = 0x101;
#endif

  /*************************************************************************
   * Allocate a scratch buffer
   *************************************************************************/
  dsiuContext.scratch = (tuint *)dsiuHeapAlloc (&dsiuContext.bootHeap, 
                                             diagCfg.apRate0 * sizeof (tuint));
  if (dsiuContext.scratch == NULL)                                              
    dsiuPanic (DSIU_MEM_ALLOC_FAIL_SCRATCH);

  /**************************************************************************
   * If Utopia ports or Mac ports are used allocate the received message fifo.
   **************************************************************************/
  if ( (diagCfg.ctable.siuUtNdu) || (diagCfg.ctable.siuGmacNdu) ) {
    /* The fifo structures are staticly allocated. Only the fifo space 
     * must be allocated. */
    dsiuContext.siuNduContext.msgNormalFifo.base = 
                      (tuint *)dsiuHeapAlloc (&dsiuContext.bootHeap, 
                                              diagCfg.msgRxFifoSize);
    if (dsiuContext.siuNduContext.msgNormalFifo.base == NULL)
      dsiuPanic (DSIU_MEM_ALLOC_FAIL_MSG_RX_FIFO);
    dsiuContext.siuNduContext.msgNormalFifo.size = diagCfg.msgRxFifoSize/2;/* Size was in bytes,defined in
	terms of aal5 cells, which are counted in bytes. But this is an tint FIFO */
    dsiuContext.siuNduContext.msgNormalFifo.inIndex  = 0;
    dsiuContext.siuNduContext.msgNormalFifo.outIndex = 0;

    /* There are no other priority messages in diag */
    dsiuContext.siuNduContext.msgHighFifo.size     = 0;
    dsiuContext.siuNduContext.msgHighFifo.inIndex  = 0;
    dsiuContext.siuNduContext.msgHighFifo.outIndex = 0;

    dsiuContext.siuNduContext.msgLowFifo.size      = 0;
    dsiuContext.siuNduContext.msgLowFifo.inIndex   = 0;
    dsiuContext.siuNduContext.msgLowFifo.outIndex  = 0;
  }


  /************************************************************************
   * Initialize the rx network task fifo to NULL. It will be created
   * if required during channel setup.
   ************************************************************************/
  dsiuContext.rxNetTasks = NULL;

  /**************************************************************************
   * Allocate memory for message processing
   **************************************************************************/
  dsiuContext.msgRxBuf  = (dmsg_t *)dsiuHeapAlloc (&dsiuContext.bootHeap, 
                                                    diagCfg.msgRxBufferSize);
  if (dsiuContext.msgRxBuf == NULL)
    dsiuPanic (DSIU_MEM_ALLOC_FAIL_MSG_RX_BUF);

  dsiuContext.msgTxBuf  = (dmsg_t *)dsiuHeapAlloc (&dsiuContext.bootHeap, 
                                                    diagCfg.msgTxBufferSize);
  if (dsiuContext.msgTxBuf == NULL)
    dsiuPanic (DSIU_MEM_ALLOC_FAIL_MSG_TX_BUF);

} /* dsiuAllocateContextMemory */

/******************************************************************************
 * FUNCTION PURPOSE: Configure the global message table interface
 ******************************************************************************
 * DESCRIPTION: Stores a pointer to the global message routing table in
 *              dsiuContext. Initializes the routing table.
 ******************************************************************************/
void dsiuMsgTableInit (void)
{
  dsiuContext.msgTable.table = globMsgTableElement;
  dsiuContext.msgTable.n     = dsiuInitMsgRouteTable (dsiuContext.msgTable.table);
} /* dsiuMsgTableInit */

/***************************************************************************
 * FUNCTION PURPOSE: Setup hard coded test configuration
 ***************************************************************************
 * DESCRIPTION: Hard coded values for the utopia configuration are set.
 *              The TDM configuration parameters are reset to cross
 *              connect the TDM ports. This is used for testing the 
 *              single janus daughter cards after assembly.
 ****************************************************************************/
#if 0
void dsiuTestMode(tuint dsp_id)
{
  diagReg_t     *dreg;
  tint          i;
  
  dreg = (diagReg_t *)diagCfg.rtConfigBlock;

  dreg->utopiaCellSize    = 58;
  dreg->routeTagHeader[0] = 0xEFA0;
  dreg->routeTagHeader[1] = 0;
  dreg->atmHeader[0]      = 0;
  dreg->atmHeader[1]      = 0;
  dreg->atmHeader[1]      = 0;
  dreg->dspId             = 1;
  dreg->utopiaConfig      = 0;
  dreg->messageRoute      = DREG_MSG_ROUTE_UTOPIA;

  dreg->nChannels         = 24;
  dreg->interleave        = 0;
  dreg->tdmFirstTimeslot  = (24 * dsp_id) + 1;
  dreg->clocksPerFs       = 24;

  /* Cross Loopback TDM ports */
  for (i = 0; i < DREG_MAX_CHANNELS-1; i += 2)  {
    diagRegs.chCfg[i]   =   (((i+2)<<8)|(i+2));
    diagRegs.chCfg[i+1] =   (((i+1)<<8)|(i+1));
  }
} /* dsiuTestMode */
#endif


#ifdef HAL_INCLUDE_AIC
/* Pasted from Longmont sample code aic20.c, etc. */
#define IODIR			((T_IODIR	*)(0x3400))			
#define IODATA			((T_IODATA 	*)(0x3401))			
typedef volatile struct DSP_IODIR_IODATA_BITS
{
	tuint 		: 8;
	tuint gpio7	: 1;
	tuint gpio6	: 1;
	tuint gpio5	: 1;
	tuint gpio4	: 1;
	tuint gpio3	: 1;
	tuint gpio2	: 1;
	tuint gpio1	: 1;
	tuint gpio0	: 1;

}T_IODIR_IODATA_BITS;

#define GPIO_INPUT	0
#define	GPIO_OUTPUT	1

typedef volatile ioport struct DSP_IODIR
{
	union
	{
		T_IODIR_IODATA_BITS	bit;
		tuint					all;
	} reg;
}T_IODIR;

typedef volatile ioport struct DSP_IODATA
{
	union
	{
		T_IODIR_IODATA_BITS	bit;
		tuint					all;
	} reg;
}T_IODATA;

void AIC20EnableReset(void)
{
	tint i;
	//put AIC20 in reset
	IODIR->reg.bit.gpio7 = GPIO_OUTPUT;
	IODATA->reg.bit.gpio7 = 0;
	for(i=0; i<10000; i++);
}

void AIC20DisableReset(void)
{
	IODIR->reg.bit.gpio7 = GPIO_OUTPUT;
	IODATA->reg.bit.gpio7 = 1;
}

void AIC20EnablePowerDown(void)
{
	//put AIC20 in power down mode
	IODIR->reg.bit.gpio6 = GPIO_OUTPUT;
	IODATA->reg.bit.gpio6 = 0;
}	

void AIC20DisablePowerDown(void)
{
	//take AIC20 out of power down mode
	IODIR->reg.bit.gpio6 = GPIO_OUTPUT;
	IODATA->reg.bit.gpio6 = 1;
}	
/* Pasted from Longmont sample code aic20.c, etc. */
	

void AIC20Disable (void)
{
  tint i;

  //AIC20 powered down and in reset
  IODIR->reg.bit.gpio6 = GPIO_OUTPUT;
  IODIR->reg.bit.gpio7 = GPIO_OUTPUT;
  IODATA->reg.bit.gpio6 = 0;
  IODATA->reg.bit.gpio7 = 0;
  for(i=0; i<10000; i++);
  IODATA->reg.bit.gpio7 = 1;
  for(i=0; i<10000; i++);
  IODATA->reg.bit.gpio6 = 1;
}	
#endif

/***************************************************************************
 * FUNCTION PURPOSE: Copy memory
 ***************************************************************************
 * DESCRIPTION: Performs a basic memory copy. Broken out of main so the
 *              stack would not be impacted.
 ***************************************************************************/
void dsiu_mem_cpy (halGlobAddr_t dest, halGlobAddr_t src, halSAE_t size)
{
  halMemMove_t memMove;

  memMove.dest = dest;
  memMove.src  = src;
  memMove.size = size;

  /* Do the copy, stall until complete */
  (*dsiuContext.hal->halGlobMemMove)(1, &memMove, NULL, NULL, TRUE);

} /* disu_mem_cpy */



/****************************************************************************
 * FUNCTION PURPOSE: Application Initialization
 ****************************************************************************
 * DESCRIPTION: Allocates memory for and opens modules as configured.
 *              On entry to this function the diagCfg structure is complete,
 *              as is the user configuration register bank, diagReg.
 ***************************************************************************/
#ifdef DSP_ON_MIPS
void siuMain (void)
{
#else
void main (void)
{
#endif

  tint  entry_code = 0; /* Provide info about boot process to system main */

  TSCL = 1;  /* Init internal cycle counter */


  /**************************************************************************
   * Initial DSIU context setup
   **************************************************************************/
  dsiuContext.tdmPort    = diagCfg.dcfgTdmPorts_p;
  dsiuContext.utopiaPort = diagCfg.dcfgUtopiaPorts_p;
  dsiuContext.macPort    = diagCfg.dcfgMacPorts_p;

  dsiuContext.dregStatus       = &dregStatus;
  dsiuContext.dregBootTrack    = &dregBootTrack;
  dsiuContext.dregMsgCfg       = &dregMsgCfg;
  dsiuContext.dregMsgCfgShadow = &dregMsgCfgShadow;
  dsiuContext.masterCntrl      = &dregMasterCntrl;
#ifdef ITDM
  dsiuContext.itdmStreamCfg    = itdmStreamCfg; 
#else 
  dsiuContext.itdmStreamCfg    = NULL;
#endif

  dsiuContext.memOpPending  = FALSE;

#if (defined(DSIU_SIMULATE_APPLICATION) || defined(DSIU_DAUGHTER_TEST))
  dsiuContext.dregTdmCfg          = dregTdmCfg;
  dsiuContext.dregVchCfgBlk       = dregVchCfgBlk;
  dsiuContext.dregRouterCfg       = dregRouterCfg;
  dsiuContext.dregBertStats       = dregBertStats;
  dsiuContext.dregOpStats         = &dregOpStats;
  dsiuContext.dregTdmCfgShadow    = dregTdmCfgShadow;
  dsiuContext.dregVchCfgShadow    = dregVchCfgShadow;
  dsiuContext.dregRouterCfgShadow = dregRouterCfgShadow;
#else
  dsiuContext.dregTdmCfg          = NULL;
  dsiuContext.dregVchCfgBlk       = NULL;
  dsiuContext.dregRouterCfg       = NULL;
  dsiuContext.dregBertStats       = NULL;
  dsiuContext.dregOpStats         = NULL;
  dsiuContext.dregTdmCfgShadow    = NULL;
  dsiuContext.dregVchCfgShadow    = NULL;
  dsiuContext.dregRouterCfgShadow = NULL;
#endif

  dsiuContext.tdmLogicAlloc = 0;
  dsiuContext.utLogicAlloc  = 0;
  dsiuContext.macLogicAlloc = 0;

  /**************************************************************************
   * There is only one hal, but if there were more then one, the one to be
   * used would be assigned to the siu context allowing a single build to
   * operate on multiple hardware platforms. 
   **************************************************************************/
  dsiuContext.hal = &hal;

  /**************************************************************************
   * Boot progress track: Indicate that main has been entered
   **************************************************************************/
  dsiuBootTrack (DREG_BOOT_TRACK_IN_MAIN);

#ifdef DIAG_INCLUDE_OTBRUN
  /*initialize diag app mem structure*/
  if(dsiuSimConfig.opt_diag_otb_BERT)
  {
    dsiuInitDiagAppMem();
  }
#endif

  /**************************************************************************
   * If auto run is enabled call the auto run stall. This stalls secondary
   * cores so that if all cores are started at once the primary core can
   * configure shared peripherals before secondary cores do local configuration.
   * dsiuContext.hal must be assigned before this call, but dsiu_hal_init
   * must be called after.
   **************************************************************************/
  if (diagCfg.ctable.dsiuArun)
    (*diagCfg.ctable.dsiuArun->stall)();


  /**************************************************************************
   * Provide Initialization for the HAL. No peripherals are setup.
   **************************************************************************/
  dsiu_hal_init ();

  /**************************************************************************
   * For the simulated application, read the configuration provided by
   * the boot loader.
   **************************************************************************/
#if defined(DSIU_SIMULATE_APPLICATION)

  if (diagCfg.bootSimuAppConfig)
    if ((*diagCfg.bootSimuAppConfig)())
      dsiuPanic (DSIU_PSEUDO_APP_CONFIG_FAIL);
  dsiuContext.mode = DSIU_MODE_BOOT; /* Remain in boot mode until hwconfig */
  dsiuBootTrack (DREG_BOOT_TRACK_PSEUDO_APP_INIT);

#endif

#if defined(DSIU_DAUGHTER_TEST)

  /************************************************************************ 
   * The Daughter card version uses pre-defined values so the system is
   * put directly into a known state - no messages required.
   ************************************************************************/
//  dsiuTestMode(dsiuContext.coreId); /* Must call after dsiuDefaultRegValues */
  dsiuContext.mode = DSIU_MODE_DIAG; /* Immediately enter diagnostic mode */
#endif

#ifndef DSIU_SIMULATE_APPLICATION
#ifndef DSIU_DAUGHTER_TEST
  /**************************************************************************
   * Device specific configuration for the bootloader. Each device may have 
   * it's own method of boot loading.
   **************************************************************************/
  if (diagCfg.bootConfig)
    entry_code = (*diagCfg.bootConfig)();
  dsiuContext.mode = DSIU_MODE_BOOT; /* Enter bootloader mode */

  /* Re-Initialise HAL to make gmacInst NULL so that it can be reconfigured: 
     new HAL structure will give No port available error if this is not done */
  dsiu_hal_init ();
  
#endif /* DSIU_DAUGHTER_TEST        */
#endif /* DSIU_SIMULATE_APPLICATION */

  /*************************************************************************
   * The diagApp configuration is complete. Bring up any peripherals
   * which required this information. 
   *
   * This setp up cache, so it should be done before default register
   * values are setup.
   *************************************************************************/
  dsiu_periph_setup ();

  /*********************************************************************
   * Optional automated control initialiation
   *********************************************************************/
  if (diagCfg.ctable.dsiuArun)
    (*diagCfg.ctable.dsiuArun->init)();

  /*********************************************************************
   * Start with valid set of registers. Will be replaced with user
   * programmed values in the diag registers. 
   *********************************************************************/
  dsiuDefaultRegValues (dsiuContext.coreId);

  /***************************************************************************
   * Boot progress track: Indicate default values have been set
   ***************************************************************************/
  dsiuBootTrack (DREG_BOOT_TRACK_DEFAULT_INIT);

  /**************************************************************************
   * The processor specific booting set the value of the dsp ID in the
   * diag configuration registers. It must be copied to the dsiuContext
   * structure for compatibility with the application NDU.
   **************************************************************************/
  dsiuContext.dspId = dsiuContext.diagApp.dspId;

  /*************************************************************************
   * Initialize CPU/fsync clock check as disabled
   *************************************************************************/
#if 0
  dsiuContext.eiClock.validData = TRUE;
  dsiuContext.dreg->clkRatio    = DREG_CLOCK_RATIO_NOT_RUN;
#endif


  /**************************************************************************
   * Initialize heap memory control
   **************************************************************************/
  dsiuContext.bootHeap.base = (word *)diagCfg.bootHeapBufStart;
  dsiuContext.bootHeap.size = (tulong)diagCfg.bootHeapBufEnd - (tulong)diagCfg.bootHeapBufStart;
  dsiuContext.bootHeap.used = 0;

  dsiuContext.diagHeap.base = (word *)diagCfg.diagHeapBufStart;
  dsiuContext.diagHeap.size = (tulong)diagCfg.diagHeapBufEnd - (tulong)diagCfg.diagHeapBufStart;
  dsiuContext.diagHeap.used = 0;

  /**************************************************************************
   * Allocate memory. Memory is allocated for modules based on values 
   * in diagCfg. 
   **************************************************************************/
  /* System Context Memory */
  dsiuAllocateContextMemory ();

  /**************************************************************************
   * If required, copy code from shared to local memory. If a load/run copy
   * is not required define the following values to 0 in the linker:
   *   _boverlay_load_start
   *   _boverlay_run_start
   *   _boverlay_load_size
   **************************************************************************/
  if ((tulong)&boverlay_load_start != 0l)  {
    dsiu_mem_cpy ( (*dsiuContext.hal->halMapLocalToGlobal)(SIU_BYTES_TO_WORDS((tulong)&boverlay_run_start)),
        (*dsiuContext.hal->halMapLocalToGlobal)(SIU_BYTES_TO_WORDS((tulong)&boverlay_load_start)),
        SIU_BYTES_TO_WORDS((tulong)&boverlay_load_size));
  }

#ifdef PA_SS  
    /*Initializing PA, QM, CPPI/NWAL*/
    if (dsiuNwInit () == FALSE)  {
        dsiuNwLog ("ERROR: dsiuNwInit FAILED \n");
        return;
    }
    else
    {
        dsiuNwLog ("dsiuNwInit Passed \n");
    }
#endif


  /*************************************************************************
   * Initialize the NDU and message interface 
   *************************************************************************/
  if (diagCfg.ctable.siuUtNdu)
    (*diagCfg.ctable.siuUtNdu->nduControl) (SIU_NDU_CTRL_INIT);

  if (diagCfg.ctable.siuHpiNdu)
    (*diagCfg.ctable.siuHpiNdu->nduControl) (SIU_NDU_CTRL_INIT);

  if (diagCfg.ctable.siuGmacNdu)
    (*diagCfg.ctable.siuGmacNdu->nduControl) (SIU_NDU_CTRL_INIT);

  /***************************************************************************
   * Configure the message tables
   ***************************************************************************/
  dsiuMsgTableInit ();

  /***************************************************************************
   * Any processor specific initializations
   ***************************************************************************/
  if (diagCfg.dsiuProcStartup)
    (*diagCfg.dsiuProcStartup)();

  /******************************************************************************/
  /* Initialize isiu_isr_clock                                                  */
  /******************************************************************************/
  dsiuContext.isiu_isr_clock = 0;
 
  /* Begin main loop execution */
  dsiuMain (entry_code);

} /* main */
 
  
