#ifndef _DSIULOC_H
#define _DSIULOC_H
/******************************************************************************
 * FILE PURPOSE: Diagnostic code local definitions
 ******************************************************************************
 * FILE NAME: dsiuloc.h
 * 
 * FILE PURPOSE: Definitions and structures used to manage the diag code
 *
 * TABS: None
 *
 * Copyright (C) 2002, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include "types.h"
#include "utlxferpkt.h"
#include "hal.h"
#include "diagreg.h"
#include "dsiutask.h"
#include "dsiumsg.h"
#include "dglob.h"
#include "dsiufifo.h"
#include "dsiumem.h"
#include "dsiuport.h"
#include "memmgm.h"
#include "diagapp.h"

#ifdef PA_SS
#include "dsiunwal.h"
#endif


/*****************************************************************************
 * DEFINITION: Local DSIU return values
 *****************************************************************************/
typedef enum {
  DSIU_SUCCESS = 0,
  DSIU_FAIL
} dsiuReturn_t;

/*****************************************************************************
 * DEFINITION: Main channel state
 *****************************************************************************/
typedef enum {
 DSIU_STATE_CLOSED = 0,
 DSIU_STATE_OPEN
} dsiuState_t;

/*****************************************************************************
 * DEFINITION: Companding types
 *****************************************************************************/
typedef enum {
 DSIU_COMPAND_ALAW = 0,
 DSIU_COMPAND_ULAW,
 DSIU_COMPAND_LINEAR
} dsiuCompand_t;

/*****************************************************************************
 * DEFINITION: Return values from core boot. Values for every type of 
 *             processor are used here.
 *****************************************************************************/
typedef enum {
  DSIU_CORE_BOOT_SUCCESS = 0,   /* Core boot successfull                */
  DSIU_CORE_BOOT_INIT_FAIL      /* C5561 did not find 0 in boot address */
} dsiuCoreBootReturn_t;

/*****************************************************************************
 * DEFINITION: Return values from core reset.
 *****************************************************************************/
typedef enum {
  DSIU_CORE_RESET_SUCCESS = 0,
  DSIU_CORE_RESET_FAIL
} dsiuCoreResetReturn_t;

/*****************************************************************************
 * DEFINITION: Errors tracked per channel
 *****************************************************************************/
typedef struct {
  tuint pcmTxTask;            /* Data read failures from PIU                */
  tuint pcmRxTask;            /* Data write failures from PIU               */
  tuint pcmInvalidNetRoute;   /* Route PCM to network selected, but invalid */
  tuint pcmInvalidRxBert;     /* BERT selected but not in build             */
  tuint pcmInvalidTxBert;     /* BERT selected but not in build             */
  tuint pcmInvalidTxTone;     /* PCM Tx Tone selected, but not in build     */
  tuint pcmInvalidRxTone;     /* PCM Tx Tone selected, but not in build     */
  tuint netRxTask;            /* Data read failures from network            */
  tuint netInvalidPcmRoute;   /* Route net to PCM selected, but invalid     */
  tuint rxNoPvp;              /* Receive data packet, no pvp                */
  tuint rxPvpFail;            /* Receive pvp failure                        */
  tuint invalidRouteCh;       /* Invalid route channel                      */
} dsiuChErrs_t;


/*****************************************************************************
 * DEFINITION: System Level errors
 ****************************************************************************/
typedef struct {
  tuint txPcmTaskQueueFull;
  tuint rxPcmTaskQueueFull;
  tuint msgTaskQueueFull;
  tuint msgBufferErr;         /* Message to large for message buffer        */
  tuint utopiaTypeErr;        /* Unkown type received                       */
  tuint utopiaCellSizeErr;    /* Utopia cell received with illegal size     */
  tuint netInvalidChannel;    /* Invalid channel from network               */
  tuint utopiaFifoExtractErr; /* Utopia cell fifo extraction error          */
  tuint msgRxFifoFull;        /* Message received but fifo full             */
  tuint msgRxFifoEmpty;       /* Message Task found empty fifo              */
  tuint msgRxFifoReset;       /* Message fifo read error, causing reset     */
  tuint rxNetTaskQueueFull;   /* Unable to post a task in rx network fifo   */
  tuint rxNetFifoFull;        /* The network cell/packet fifo full          */
  tuint rxNetFifoResync;      /* Network Fifo error                         */
} dsiuContextErrs_t;

/******************************************************************************
 * DEFINITION: Message routing table
 *****************************************************************************/
typedef struct {
  dsiuMsgTableElement_t *table;   /* Message table array         */
  tint n;                         /* Number of elements in table */
} dsiuMsgTable_t;

/*****************************************************************************
 * DEFINITION: Network driver unit definitions. These definitions must
 *             be code compatible with the definitions in the application
 *             file siuloc.h
 ****************************************************************************/
/* NDU output specification */
typedef struct {
  tint (*nduReceiveOut) (void *targetInst, xferPktInfo_t *data);
  void *targetInst;
} dsiuNduRxOutSpec_t;

/* A channel can be configured to receive packets from only one interface.
 * Messages can be received from any enabled interface */
typedef enum {
  SIU_NDU_TYPE_NONE,    /* Packet reception disabled */
  SIU_NDU_TYPE_HPI,     /* HPI packet reception      */
  SIU_NDU_TYPE_UT,      /* Utopia packet reception   */
  SIU_NDU_TYPE_GMAC     /* Gmac packet reception     */
} dsiuNduPacketRec_t;

/* NDU configuration paramameters */
typedef struct {
  dsiuNduPacketRec_t recInterface;
  dsiuNduRxOutSpec_t RxOutSpec; /* Name must match siuloc.h */
} dsiuNduConfig_t;
typedef dsiuNduConfig_t tsiuNduConfig_t;  /* to code match siu and dsiu */

/* NDU instance structure */
typedef struct {
  dsiuNduPacketRec_t recInterface;  /* Which interface is setup for reception */
  dsiuNduRxOutSpec_t RxOutSpec;     /* Name must match siuloc.h               */
} dsiuNduInst_t;
#define tsiuNduInst_t dsiuNduInst_t  /* to code match siu and dsiu */

/****************************************************************************
 * DATA DEFINITION: Non voice channel instance definition. Non voice channels
 *                  are an abstraction that is used to allow messages and
 *                  other data paths to use ecos.
 ****************************************************************************/
typedef enum {
  SIU_NVCH_MSG_UT_NORMAL = 1,    /* Normal priority utopia message channel  */
  SIU_NVCH_MSG_GM_NORMAL = 100   /* Normal priority gmac message channel    */
} siuNvChId_t;

/* Non voice "channel" instance definition - used for messages */
typedef struct siuNvInst_s {
  siuNvChId_t     ID;       /* Channel identifier                    */
  tint            vcIdRx;   /* Associated voice channel ID from host */
  tint            vcIdTx;   /* Associated voice channel ID to host   */
  void           *neuInst;  /* Full application neu eco instance     */
} dsiuNvInst_t;
  
/* NDU Context */
typedef struct {
  dsiuFifo_t msgNormalFifo;  /* Fifo names must match siuloc.h */
  dsiuFifo_t msgHighFifo;   
  dsiuFifo_t msgLowFifo;
  
  /* Message interface - Utopia */
  dsiuNvInst_t siuMsgNormalUt;  /* Normal priority message instance */
  tuint   cellSize;      /* UtopiaCellSize, bytes (54/58)            */
  
  /* Message interface - Mac */ 
  dsiuNvInst_t siuMsgNormalGm; /* Normal priority message instance */
#ifdef PA_SS
  nwal_Handle    udpHandleHiPrioMsg;
  nwal_Handle    udpHandleMedPrioMsg;
  nwal_Handle    udpHandleLowPrioMsg;
#endif
  
} dsiuNduContext_t;

/* NDU statistics, must match the application stats */
typedef struct {
  tuint RxErrState;         /* Rx pkts dropped since NDU in invalid state     */
  tuint RxErrLength;        /* Rx pkts dropped due to invalid length          */
  tuint RxErrChnum;         /* Rx pkts dropped due to invalid channel number  */
  tuint RxErrPktcnt;        /* Rx pkts dropped due to invalid packet count    */
  tuint RxErrInIndex;       /* Rx pkts dropped due to invalid in_index        */
  tuint RxErrOutIndex;      /* Rx pkts dropped due to invalid out_index       */
  tuint RxInvalidDspCoreId; /* Rx messages/packets with incorrect DSP/Core ID */
  tuint RxCheckSum;         /* Rx pkts dropped due to invalid checksum        */
  tuint TxErrPktcnt;        /* Tx pkts dropped due to invalid packet count    */
  tuint TxErrInIndex;       /* Tx pkts dropped due to invalid in_index        */
  tuint TxErrOutIndex;      /* Tx pkts dropped due to invalid out_index       */
  tuint TxErrDrop;          /* Tx pkts dropped to insufficient buffer space   */
  tuint TxResyncCnt;        /* Number of re-syncs initiated by Host           */
  tuint RxResyncCnt;        /* Number of re-syncs initiated by DSP            */
  tuint TxSigMsgDrop;       /* # of to-Host signaling messages dropped        */
  tuint TxCtrlMsgDrop;      /* # of to-Host control messages dropped          */
  tuint TxLowMsgDrop;       /* # of to-Host low priority messages dropped     */
  tuint RxSigMsgDrop;       /* # of from-Host signaling messages dropped      */
  tuint RxCtrlMsgDrop;      /* # of from-Host control messages dropped        */
  tuint RxLowMsgDrop;       /* # of from-Host low priority messages dropped   */
  tuint TxSigMsgCnt;        /* # of signaling messages sent to Host           */
  tuint TxCtrlMsgCnt;       /* # of control messages sent to Host             */
  tuint TxLowMsgCnt;        /* # of low priority messages sent to Host        */
  tuint RxSigMsgCnt;        /* # of signaling messages received from Host     */
  tuint RxCtrlMsgCnt;       /* # of control messages received from Host       */
  tuint RxLowMsgCnt;        /* # of low priority messages received from Host  */
  tuint RxSigMsgFifoErr;    /* # of rx signalling messages with fifo errors   */
  tuint RxCtrlMsgFifoErr;   /* # of rx normal messages with fifo errors       */
  tuint RxLowMsgFifoErr;    /* # of rx low priority messages with fifo errors */
  tuint TxInvldNvCh;        /* # of messages for an invalid non voice channel */
  tuint RxInvldNvCh;        /* # of messages for an invlaid non voice channel */
} dsiuNduStats_t;

/* Messaging type (signalling or regular) selection. Must match application
 * definition */
typedef enum {
   SIU_DPR_SIGNAL_MSG,
   SIU_DPR_NORMAL_MSG,
   SIU_DPR_LOW_MSG    
} siuDPRMsgType_t;

/* Errors during boot configuration */
typedef struct {
  tuint invldMagic;  /* Received cell with invalid magic value          */
  tuint invldId;     /* Extended boot cell with invalid ID              */
  tuint extBoot;     /* Count of extra extened mode boot cells received */
  tuint invldType;   /* Count of boot cells for a type marked unused    */
} siuBootErrs_t;


/*****************************************************************************
 * DEFINITION: Diagnostic channel instance. Defines channel specific 
 *             persistant data
 *****************************************************************************/
typedef struct {
  tuint         Id;       /* Module ID                           */
  dsiuState_t   state;    /* Channel State                       */
  dsiuChErrs_t *errs;     /* Register bank of errors             */

  tuint tdmFrameSize;         /* Frame size to/from TDM       */
  dsiuCompand_t compandLaw;   /* Companding on TDM            */
  tuint silencePattern;       /* Bit pattern for silence      */

  dsiuNduInst_t nduInst;      /* Network driver instance      */
  tuint chan_bfield2;
  void *dvpuInst;             /* Pointer to DVPU module data  */
  void *dpiuInst;             /* Pointer to DPIU module data  */
  void *ipiuInst;             /* Pointer to IPIU module data  */
  void *ippuInst;             /* Pointer to IPPU module data  */
  void *bertInst;             /* Pointer to BERT module data  */
  void *dpvpInst;             /* Pointer to DPVP module data  */
  void *dtguInst;             /* Pointer to DTGU module data  */
  void *neuInst;              /* Pointer to NEU module data   */
  tuint halRxID;              /* Used to close or modify this flow */
  tuint halTxID;              /* Used to close or modify this flow */
  itdmMultiTsMap_t mdebugTsMultiMap;
  itdmMultiTsMap_t mdebugTsMultiMapb;
  tuint txTaskPostTimeStamp;
  
  tint  bertSampleCount;      /* 1 if there is a left over BERT sample */
  tuint bertSample;           /* The left over bert sample             */
  tuint bertTelRxSync;        /* Used to detect (and report) a Telephony BERT state transition (sync<-->out of sync) */
  tuint bertNetRxSync;        /* Used to detect (and report) a Net BERT state transition (sync<-->out of sync) */

#ifdef PA_SS
  nwal_Handle    udpHandle;
#endif
  
} dsiuInst_t;
#define tsiuInst_t dsiuInst_t /* To code match application */

/* Main macros for accessing configuration bit fields       *
 * Input parameter a is tuint containing bit field.         *
 * b is bit offset withing bit field. c is number of bits   *
 * used by that parameter. x is new value of parameter that *
 * is packed in this bit field.                             */
#define SIU_READ_XXX(a,b,c)  ((a>>b) & ((1<<c)-1))
#define SIU_SET_XXX(a,x,b,c)  a &=~((((tuint)1<<c)-(tuint)1)<<b); a |=((tuint)x<<b)

#define SIU_READ_PRIMARY_DATA_PAGE(a)  SIU_READ_XXX(((dsiuInst_t *)a)->chan_bfield2, 0, 1)
#define SIU_SET_PRIMARY_DATA_PAGE(a,b) SIU_SET_XXX(((dsiuInst_t *)a)->chan_bfield2, b, 0, 1)
  
extern dsiuInst_t *dsiuInst;
#define tsiuInst dsiuInst   /* To code match application */


/****************************************************************************
 * DEFINITION: Structure used to calculate the ratio of the system clock to
 *             the frame sync clock.
 ****************************************************************************/
typedef struct {
           tulong lastEntryTick;   /* System timer tick on last entry */
           tulong lastDelta;       /* Last delta measurement          */
  volatile bool   validData;       /* If TRUE then lastDelta is valid */
} dsiuExtIntClock_t;

/****************************************************************************
 * DEFINITION: System operation mode definition. After startup the code
 *             enters bootloader mode. It can be put into diagnostic mode
 *             through the diagnostic control message.
 ****************************************************************************/
typedef enum {
  DSIU_MODE_BOOT,   /* bootloader mode */
  DSIU_MODE_DIAG    /* diagnostic mode */
} dsiuMode_t;


/******************************************************************************
 * DEFINITION: Possible TDM enable status states
 ******************************************************************************/
enum {
  DSIU_TDM_STATUS_DISABLED,         /* Port is disabled                             */
  DSIU_TDM_STATUS_PENDING_ENABLE,   /* HAL has started the port, no channels opened */
  DSIU_TDM_STATUS_ENABLED           /* Port is up, channels enabled                 */
};


/******************************************************************************
 * DEFINITION: Port management. Links the physical port number, the hal ID,
 *             the current hal status, and the system run time port status.
 ******************************************************************************/
typedef struct dsiuPort_s {

  tint physPort;         /* Physical port number, -1 if unassigned */
  tint halId;            /* Associated hal handle                  */
  dregPeriph_t *status;  /* Pointer to visible status register set */
  
} dsiuPort_t;
  
  
  

/******************************************************************************
 * DEFINITION: Diagnostic main structure which contains channel independent
 *             data.
 *****************************************************************************/
typedef struct {
  tuint dspId;      /* DSP ID  */
  tuint coreId;     /* Core ID */

  dsiuMode_t mode;  /* Operation mode - bootloader or diagnostic */

  halApi_t        *hal;          /* Hardware abstraction interface call table   */
  
  /* Pointers to control and status registers */
  dregStatus_t       *dregStatus;    /* Pointer to status register structure */
  dregMasterCntrl_t  *masterCntrl;   /* Master control registers             */
  dregTdmCfg_t       *dregTdmCfg;    /* Pointer to tdm config registers      */ 
  dregVchCfgBlk_t    *dregVchCfgBlk; /* Pointer to network config registers  */
  dregRouterCfg_t    *dregRouterCfg; /* Pointer to router config registers   */
  dregBootTrack_t    *dregBootTrack; /* Tracks progress before messaging     */
  dregNetCfg_t       *dregMsgCfg;    /* Message network configuration        */
  itdmStreamCfg_t    *itdmStreamCfg; 
  
  /* Pointers to statistics registers */
  dregBertStats_t    *dregBertStats; /* BERT module statistics registers     */
  dregOpStats_t      *dregOpStats;   /* Operational stats                    */
  
  /* Pointers to the shadow config registers. These reflect the actual
   * current system state */
  dregTdmCfg_t       *dregTdmCfgShadow;    /* TDM shadow config     */
  dregNetCfg_t       *dregVchCfgShadow;    /* Network shadow config */
  dregRouterCfg_t    *dregRouterCfgShadow; /* Router shadow config  */
  dregNetCfg_t       *dregMsgCfgShadow;    /* Message shadow config */
  
  
  /* Multiple tdm, utopia, and mac ports are supported. Each port has
   * a physical port number, which is associated with the hardware,
   * and a logical port config, which is assocated with software. The
   * logical port config is used to distinguish between ports setup
   * at build time through diagcfg.c and ggswcfg.h. For example, 
   * ggswcfg.h may define two mac ports. One port has 15k of buffering
   * for packets and is intended for packets, the other has 1k and
   * is intended for messages. The association of which buffer set
   * is assigned to which physical port is done through the port
   * config message. The logicAlloc fields are initialized to 0
   * at system start, and a bit is set to one when that logical
   * port config has been used by the system. */
  
  /* Array of TDM port information */
  dsiuPort_t     *tdmPort;  
  tuint           tdmLogicAlloc;
  
  /* Array of Network port information */
  dsiuPort_t     *utopiaPort; 
  tuint           utLogicAlloc;             
  
  /* Array of Mac port information */
  dsiuPort_t     *macPort;
  tuint           macLogicAlloc;

  /* Heap for bootloader and diagnostics */
  dsiuHeap_t bootHeap;  /* Heap memory for bootloader  */
  dsiuHeap_t diagHeap;  /* Heap memory for diagnostics */
  
  /* System scratch buffer */
  tuint *scratch;
  
  
#ifdef RELMSG_SUPPORT  
  /* Message seqence number */
  tuint seqNum;
#define DSIU_GET_MSG_SEQ_NUM()             (dsiuContext.seqNum & 0x00FF)
#define DSIU_SET_MSG_SEQ_NUM(newSeqNum)    dsiuContext.seqNum = ((newSeqNum) & 0x00FF)
#define DSIU_MAX_MSG_SEQ_NUM               255 

  UINT16 msg_burst_size;           /* The maxmium Micro-to-DSP burst size 
                                    * Note: It is used for the reliable message protocol
                                    *       0: Disable the reliable message protocol
                                    */
#endif
  /* System task queues */
  dsiuTask_t *txPcmTasks;
  dsiuTask_t *rxPcmTasks;
  dsiuTask_t *rxNetTasks;

  /* Message Buffers */
  dmsg_t     *msgRxBuf;      /* Global received message Process buffer   */
  dmsg_t     *msgTxBuf;      /* Global transmit message Process buffer   */

  /* Network receive processing fifos, aal1 and ethernet */
  dsiuFifo_t *netFifo;

  /* NDU Context, and error stats */
  dsiuNduContext_t siuNduContext;  /* Name must match siuloc.h */
  dsiuNduStats_t   nduStats;       /* Name must match siuloc.h */
  
  /* Frame sync/cpu clock ratio control */
  dsiuExtIntClock_t  eiClock;

  /* Message Route table */
  dsiuMsgTable_t msgTable;

  /* System errors */
  dsiuContextErrs_t errs;
  
  /* Boot operation errors */
  siuBootErrs_t bootErrs;
  
  /* Tracks run time usage of the hal heap buffer used */
  tulong halHeapUsed;
  
  /* Tracks pending memory write/copy requests */
  volatile bool memOpPending;
  
  diagApp_t diagApp;

  tuint     halId;

  tulong    isiu_isr_clock;
  
  tuint     core_mhz; /*CPU frequency in MHz*/

} dsiuContext_t;

extern dsiuContext_t dsiuContext;
#define tsiuContext dsiuContext  /* For compatibility with app code */

/****************************************************************************
 * DEFINITION: Returns boot information contained from an earlier boot loader
 ****************************************************************************/
enum {
  DSIU_PREV_MAC_IF_MII,
  DSIU_PREV_MAC_IF_RMII,
  DSIU_PREV_MAC_IF_GMII,
  DSIU_PREV_MAC_IF_RGMII,
  DSIU_PREV_MAC_IF_SMII,
  DSIU_PREV_MAC_IF_S3MII,
  DSIU_PREV_MAC_IF_RMII_10,
  DSIU_PREV_MAC_IF_RMII_100,
  DSIU_PREV_MAC_IF_SGMII
};

enum {
  DSIU_PREV_BOOT_PHYS_IFACE_UTOPIA,
  DSIU_PREV_BOOT_PHYS_IFACE_MAC
};
  
typedef struct dsiuPrevMacInfo_s {
  tuint iftype;     /* Physical interface type */
  bool  halfDuplex;
  bool  flowControl;
  tuint macAddr[3];
} dsiuPrevMacInfo_t;

typedef struct dsiuPrevUtInfo_s  {
  tuint slaveId;
  tuint cellSizeBytes;
} dsiuPrevUtInfo_t;
 
typedef struct dsiuPrevBootInfo_s {

  tint physIf;     /* Interface     */
  tint physPort;   /* Physical port */
  
  union {
  
    dsiuPrevMacInfo_t macInfo;
    dsiuPrevUtInfo_t  utInfo;
    
  } u;
  
} dsiuPrevBootInfo_t;
  
    
/****************************************************************************
 * DEFINITION: This structure is used to store some fixed values for IP 
 *             version. This is done to reduce the code size and logic
 ****************************************************************************/
typedef struct dsiuIpDetails_s {
  tuint byteOffsetSrcAddr;
  tuint addrLen;
} dsiuIpDetails_t;




/****************************************************************************
 * DEFINITION: A Stop compiler warnings. Kills unavoidable warnings.
 ***************************************************************************/
extern void *dsiuDummy;
#define DSIU_DUMMY_ARG(x)  dsiuDummy = (void *)(x)

/****************************************************************************
 * DEFINITION: System level utopia cell type definition
 ****************************************************************************/
typedef void *dsiuCell_t;


/******************************************************************************
 * DEFINITION: Macros for incrementing rx message fifo errors. Macrod for
 *             compatibility with system code.
 ******************************************************************************/
#define SIU_SIGNAL_MSG_FIFO_FAIL()   dsiuContext.nduStats.RxSigMsgFifoErr += 1
#define SIU_NORMAL_MSG_FIFO_FAIL()   dsiuContext.nduStats.RxCtrlMsgFifoErr += 1
#define SIU_LOW_MSG_FIFO_FAIL()      dsiuContext.nduStats.RxLowMsgFifoErr += 1

/******************************************************************************
 * DEFINITION: For the diagnostic code all messages are normal priority. The
 *             following macro is replaced with a function in the application
 *             code that computes the message level based on the message id.
 *****************************************************************************/
#define siuMessageLevel(x)  SIU_DPR_NORMAL_MSG

/*****************************************************************************
 * DEFINITION: Failure locations
 ****************************************************************************/
typedef enum {
  DSIU_MEM_ALLOC_FAIL_TX_PCM_TASKS=1,
  DSIU_MEM_ALLOC_FAIL_RX_PCM_TASKS,
  DSIU_MEM_ALLOC_FAIL_TX_UTOPIA_TASKS,
  DSIU_MEM_ALLOC_FAIL_RX_UTOPIA_TASKS,
  DSIU_MEM_ALLOC_FAIL_SIU_INST,
  DSIU_MEM_ALLOC_FAIL_SCRATCH,
  DSIU_MEM_ALLOC_FAIL_CH_ERRS,
  DSIU_MEM_ALLOC_FAIL_HAL,
  DSIU_MEM_ALLOC_DPIU_ALLOC_FAIL,
  DSIU_MEM_ALLOC_IPIU_ALLOC_FAIL,
  DSIU_MEM_ALLOC_IPPU_ALLOC_FAIL,
  DSIU_MEM_ALLOC_DVPU_ALLOC_FAIL,
  DSIU_MEM_ALLOC_DTGU_ALLOC_FAIL,
  DSIU_MEM_ALLOC_DNEU_ALLOC_FAIL,
  DSIU_MEM_ALLOC_DPVP_ALLOC_FAIL,
  DSIU_HAL_GETSIZES_FAIL,
  DSIU_HAL_NEW_FAIL,
  DSIU_HAL_INVALID_NBUFS,
  DSIU_HAL_CONTROL_FAIL,
  DSIU_UTOPIA_INVALID_CELL_SIZE,
  DSIU_MEM_ALLOC_FAIL_UTOPIA,
  DSIU_MEM_ALLOC_FAIL_NET_AAL1_FIFO,
  DSIU_MEM_ALLOC_FAIL_MSG_TASKS,
  DSIU_MEM_ALLOC_FAIL_MSG_RX_BUF,
  DSIU_MEM_ALLOC_FAIL_MSG_TX_BUF,
  DSIU_MEM_ALLOC_FAIL_MSG_RX_FIFO,
  DSIU_MEM_ALLOC_FAIL_DNEU_CONTEXT_HEAP,
  DSIU_MEM_ALLOC_FAIL_DPVP_CONTEXT_HEAP,
  DSIU_PSEUDO_APP_CONFIG_FAIL,
  DSIU_HAL_TDM_DEFAULT,
  DSIU_MEM_ALLOC_FAIL_RX_ETH_TASKS,
  DSIU_MEM_ALLOC_FAIL_NET_ETH_FIFO,
  DSIU_FAIL_NO_BOOT_INTERFACE,
  DSIU_FAIL_NO_BOOT_REPLY_IF,
  DSIU_FAIL_NO_MAC,
  DSIU_FAIL_HDR_ALIGN,
  DSIU_FAIL_HEAP_ODD_ALIGN,
  DSIU_FAIL_BOOT_CFG_ERR,
  DSIU_FAIL_UTNDU_NO_NEU,
  DSIU_FAIL_NEU_FAIL,
  DSIU_FAIL_NEU_PORTMAP_FAIL,
  DSIU_FAIL_NDU_COM_CELL_SIZE,
  DSIU_FAIL_GMNDU_NO_NEU,
  DSIU_FAIL_NEU_GM_FAIL,
  DSIU_FAIL_GM_INVALID_UDP_OFFSET,
  DSIU_FAIL_DIAGAPP_ALIGN,
  DSIU_FAIL_PREV_BOOT_INFO,
  DSIU_FAIL_UTOPIA_GETSIZES,
  DSIU_FAIL_UTOPIA_INIT,
  DSIU_FAIL_UTOPIA_ENABLE,
  DSIU_FAIL_UTOPIA_DISABLE,
  DSIU_FAIL_UTOPIA_CELL_SIZE_BECAME_0,
  DSIU_FAIL_MAC_GETSIZES,
  DSIU_FAIL_MAC_INIT,
  DSIU_FAIL_NO_PERIPH_REGS,                 /* Not enough peripheral memory  */
  DSIU_FAIL_NO_BOOT_INFO,                   /* Can't configure boot          */
  DSIU_FAIL_INVALID_BOOT_INFO,              /* Didn't understand boot config */
  DSIU_FAIL_NEU_NO_GMP                      /* No GMP in Diag. NEU's request failed. */
} dsiuFailCode_t;

/*****************************************************************************
 * DEFINITION: Maximum sizes of a packet/cell 
 *****************************************************************************/
#define DSIU_MAX_PKT_SIZE_WORDS   40    /* 80 byte packet over udp */
#define DSIU_MAX_PKT_SIZE_BYTES   80

/*****************************************************************************
 * DEFINITION: Silence patterns for various codings
 *****************************************************************************/
#define DSIU_COMPAND_ALAW_SILENCE   0x5555
#define DSIU_COMPAND_ULAW_SILENCE   0x7F7F
#define DSIU_COMPAND_LINEAR_SILENCE 0x00

/*****************************************************************************
 * DEFINITION: NEU header buffer sizes for 54 and 58 byte cells, and packets
 *****************************************************************************/
#define DSIU_DNEU_HEADER_BUFFER_SIZE_58    5  /* 5 words for 58 byte cells */
#define DSIU_DNEU_RX_OFFSET_BYTES_58      10  /* 10 byte offset            */
#define DSIU_DNEU_TX_HEADER_LEN_BYTES_58  10  /* 10 byte header            */
#define DSIU_DNEU_HEADER_BUFFER_SIZE_54    3  /* 3 words for 54 byte cells */
#define DSIU_DNEU_RX_OFFSET_BYTES_54       6  /* 6 byte offset             */
#define DSIU_DNEU_TX_HEADER_LEN_BYTES_54   6  /* 6 byte offset             */
#define DSIU_DNEU_TX_HEADER_LEN_BYTES_0    0  /* 0 byte offset (eth)       */

/****************************************************************************
 * DEFINITION: Payload size in an aal1 packet. The full 47 bytes do not
 *             need to be used.
 ****************************************************************************/
#define DSIU_AAL1_PAYLOAD_VALID_BYTES               47
#define DSIU_AAL1_PAYLOAD_VALID_BYTES_16BITDATA     40

/*****************************************************************************
 * DEFINITION: Payload size of a raw packet. 
 *****************************************************************************/
#define DSIU_RAW_PAYLOAD_SIZE_BYTES       80

/*****************************************************************************
 * DEFINITION: The destination phy route tag default value for aal1 cells
 *****************************************************************************/
#define DSIU_AAL1_RTAG_PHY    24

/*****************************************************************************
 * DEFINITION: Stack buffer size used for checksum calculation
 *****************************************************************************/
#define DSIU_CHKSUM_BLOCK_SIZE      64

/****************************************************************************
 * DEFINITION: Channel index base.  Name must match application code
 ****************************************************************************/
#define SIU_FIRSTCH   1
#define SIU_LASTCH    diagCfg.maxChannels

/****************************************************************************
 * DEFINITION: Message priorities. Type names must match application code
 ****************************************************************************/
typedef enum {
 SIU_MSG_PRIORITY_LOW,
 SIU_MSG_PRIORITY_NORMAL,
 SIU_MSG_PRIORITY_HIGH
} dsiuMsgPriority_t;
#define tsiuMsgPriority_t dsiuMsgPriority_t

/****************************************************************************
 * DEFINITION: The following definitions were added to enable the use of full
 *             applications in the diagnostic code.
 ****************************************************************************/
#define DSIU_MAX_NBUFS  6
#define DSIU_MSG_UT_NEU_ECO_ID 0x8000
#define DSIU_MSG_GM_NEU_ECO_ID 0x4000
typedef tint (*dsiuEcoGetSizes_t)(tint *modNbufs, const memBuffer_t **bufs,
                                  void *modSizeCfg);
typedef tint (*dsiuEcoNew_t)(void **modInst, tint modNbufs, memBuffer_t *modBufs,
                             void *modNewCfg);
                             
                             
/*****************************************************************************
 * Definition: A function/macro to increment an unsigned. Only required for
 *             janus devices with the pc lockup issue
 *****************************************************************************/
#ifdef PC_LOCKUP_WORK_AROUND
void dsiuInc (tuint *v);
#else
#define dsiuInc(v)      ((*(v)) += 1)
#endif

/*****************************************************************************
 * Prototypes
 *****************************************************************************/
/* dsiumain.c */
tint dsiuPostDpiuTxTask (void *inst);
tint dsiuPostDpiuRxTask (void *inst);
tint isiuPostIpiuTxTask (void *inst);
void dsiuSubRateFcn (void *cxt, tuint ts);
void *dsiu_buffer_align (void *bstart, void *bend, tint log2a, tuint *rsize);

/* dsiumsg.c */
void dsiuProcessMessage (dmsg_t *msg);
/* The following two defines are required for application compatibility */
#define siuProcessNormalMessage     dsiuProcessMessage
#define siuProcessInterruptMessage  dsiuProcessMessage
tint dsiuInitMsgRouteTable (dsiuMsgTableElement_t *msgTbl);

/* siunducom.c */
void siuNduComOpen (tint chnum, tsiuNduConfig_t *config);
void siuNduComClose (tsiuInst_t *inst);
void siu_ndu_com_open_neu (void *neuInst, dregNetCfg_t *netCfg, void *tInst, 
                         tint (*neuSendOut) (void *, xferPktInfo_t *), tint cellSize, tint halId);
void siu_ndu_com_rxcall_neu (void *inst, tint (*fcn)(void *, xferPktInfo_t *), void *rcvInst);
void siuNdComGetNormalMessage (void);
void siuNdComGetInterruptMessage (void);
void *siu_ndu_com_modify_vc_msg_header (siuDPRMsgType_t mtype, mgb_t *msg);
void siuNduComMSG_test (void);


/* siugmmdu.c */
tint siuGmWriteMessageToDPR (mgb_t *message, bool block, siuDPRMsgType_t type);
tint siuGmPushFifo (void *vmsg);
tint siu_gmmdu_msg_send (void *inst, xferPktInfo_t *pkt);
tint siu_gmmdu_rcv_message (void *inst, xferPktInfo_t *pkt);

/* siugmndu.c */
tint siuGmNduControl (tint flag);
tint siuGmNduSendIn (void *inst, xferPktInfo_t *pktInfo);
tint siuGmNduDummy (void);
void siuGmNduIntReceiveIn (void *cxt, word *packet, tuint sizeBytes);


/* siuutndu.c */
tint siuUtNduControl (tint flag);
tint siuUtNduSendIn (void *inst, xferPktInfo_t *pktInfo);
void siuUtNduReceiveIn (void);
void siuUtNduIntReceiveIn (void *rcvCxt, word *cell, tuint size);
void siuUtHeartBeat (tint mode);
tuint siuUtGetClockMul (void);

/* siuutmdu.c */
tint siuUtPushFifo (void *vmsg);
tint siuUtWriteMessageToDPR (mgb_t *message, bool block, siuDPRMsgType_t type);
tint siuUtFifo2DPR (tint dpr_id);
void siuUtMsgInit (void);

/* siundu.c */
void siuNduOpen (tint chnum, tsiuNduConfig_t *config);
void siuNduClose (tsiuInst_t *inst);
tint siuNduControl (tint flag);
tint siuNduSendIn (void *inst, xferPktInfo_t *pktInfo);
void siuNduReceiveIn (void);
void siuNduIntReceiveIn (void *cxt, word *data, tuint sizeBytes);
void siuHeartBeat (tint mode);
tuint siuGetClockMul (void);

/* siumdu.c */
void siuGetNormalMessage (void);
void siuGetInterruptMessage (void);
tint siuPushFifo (void *vmsg);
tint siuWriteMessageToDPR (mgb_t *message, bool block, siuDPRMsgType_t type);
tint siuFifo2DPR (tint dpr_id);
void siuMsgInit (void);
void siuMSG_test (void);
tint siu_utmdu_rcv_message (void *inst, xferPktInfo_t *pkt);
tint siu_utmdu_msg_send (void *inst, xferPktInfo_t *pkt);

/* dsiumain.c */
void dsiuMain (tint entry_code);
void siuTaskScheduler (void);
void dsiuTaskScheduler (tint entry_code);
void dsiuPollTasks (void);



/* dsiuini.c */
void dsiuBootTrack (tulong trackCode);
void dsiuPanic (dsiuFailCode_t code);

/* dsiumtest.c */
tint dsiuTestMemory (dregMemLoc_t *report, tint maxReport); 


/* dsiumtst64x.c */
void dsiumtst64x (dmsg_t *msg);

/* dsiumchk.c */
tint dsiuMemChecksum (dmsg_hd_sys_mem_chksum_t *ctl, dmsg_dh_sys_chksum_ret_t *ret);

/* dsiu5561s.c */
void c5561ProcStartup (void);

/* cache tests */
tint dsiuTestRamset (void);

/* dsiuaic.c */
void dsiuAnalogCodecConfig (void);

/* dsiuneu.c */
void *siuNeuCreate (void *vneuCfgSize, tint ecoId);
void dsiuNeuDebugInfo (void *ID, tuint msgType, tuint msgCode, tuint msgLength, 
                       tuint *supportingData);
tint dsiuNeuSendOut (void *v, xferPktInfo_t *p);
tulong dsiuNeuGetTime (void);
void dsiuNeuPktViol (tuint ID, void *d);
void *dsiuNeuCreateVoiceNeu (tint ecodId);
dsiuReturn_t dsiuNeuOpenVoiceNeu (void *inst);

/* dsiubert.c */
void *dsiuCreateBert (tint ecodId);
dsiuReturn_t dsiuOpenBert (void *vinst, tint mode);
dsiuReturn_t dsiuCloseBert (void *vinst);
void dsiuBertClearStats (void *vinst);

/* dsiurun */
tint dsiuRxNet(void *inst, xferPktInfo_t *data);
void dsiuRun (void);
void dsiuMeasureClockRatio (void);
void dsiuCheckChannels (void);

/* isiuRun */
void isiuRun (void);
void* isiuNeuGetInst(void *neuInst, tint pktSize, word *pktIn);
tint isiuRxItdmNet(void *inst, xferPktInfo_t *data);
void isiuPrdTask(void *inst, tuint t);
void halEnableItdmToTdmStream (dsiuInst_t *inst);
void halEnableItdmFromTdmStream (dsiuInst_t *inst);
tint halUpdateItdmFromTdmStream (void *vInst, word **buffer);
tint halUpdateItdmToTdmStream (void *inst);
dsiuReturn_t isiuUpdateChannel (dsiuInst_t *inst);

/* dsiuif.c */
void siuFullAppEcoNewEco (dsiuEcoGetSizes_t getSizes_fcn, void *sizeCfg,
                          dsiuEcoNew_t newEco_fcn, void *newCfg, void **inst, tuint id);
                          
/* dsiuhdm.c */
bool siuProcMsgAck(mgb_t *message);                          
void dsiu_send_msg (dmsg_t *msg);
                          
/* dsiuubt.c */                          
void dsiuGetUtBootConfig (halUtopiaInit_t *ul2Init, halQuery_t *utQuery);
void dsiuSetUtopiaBootBuffers (halMemBuffer_t *bufs, tuint port);

/* dsiuumbt.c */
tint dsiuBootUtGmac (void);

/* dsiumacbt1.c */
tint dsiuBootGmac (void);

/* dsiuboot.c */
void dsiu_boot_copy_info (void);
dsiuReturn_t dsiuSimuAppConfig (void);
void dsiuBootAck (tint entry_code);

/* dsiumacbt.c */
dsiuReturn_t dsiuGmacSimuAppConfig (void);
void dsiuGetMacBootSizes (halGmacSizes_t *size);
void dsiuGetMacBootConfig (halGmacInit_t *macInit, halGmacSizes_t *sizes);
void dsiuSetMacBootBuffers (halMemBuffer_t *bufs, tuint port);

/* dsiudini.c */
void dsiuAllocateDiagMemory (void);
void dsiuInitChannel (tint ch);

/* dsiumchk */
tint dsiu_clear_block (tuint *base, tulong size, tuint val);
tuint dsiu_chksum (tuint *base, tulong size);

void isiuAllocateDiagMemory (void);
void isiuInitStream (tint ch, itdmMode_t mode);
void ItdmRxIsr (void *vinst, itdmMultiTsMap_t *map, tuint **dataBuffer, tuint ts);
void ItdmTxIsr (void *vinst, itdmMultiTsMap_t *map, tuint **dataBuffer, tuint ts);

/* dsiutmhwkc */
dsiuReturn_t tmhwkMemConfig (void);
dsiuReturn_t kailashMemConfig (void);
dsiuReturn_t c64lcMemConfig (void);
void dsiuSafeWait (tuint count);
dsiuReturn_t shannonMemConfig (void);

/* dsiublkmsg.c */
void dsiuBlockProc (void);
void dsiuBlockInit (void);

/*log function*/
void dsiuNwLog ( const char* format, ... );

/******************************************************************************
 * The following files must be included after the definitions of dsiuContext
 * and dsiuInst. This is for compatibility with application code.
 *****************************************************************************/
#include "siundu.h"




#endif /* _DSIULOC_H */



