/************************************************************************************
 * FILE PURPOSE: Boot loader mac configuration
 ************************************************************************************
 * FILE NAME: dsiumacbt.c
 *
 * DESCRIPTION: Configures the mac to receive the initial boot config packet
 *
 * TABS: None
 *
 * Copyright (C) 2004, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include "types.h"
#include "dsiuloc.h"
#include "hal.h"
#include "dsiumacbt.h"
#include "diagcfg.h"
#include "dsiumsg.h"
#include "diagver.h"
#include "diagapp.h"
#include "dsiuutop.h"
#include "dsiumem.h"
#include "dsiuport.h"
#include "tiboot.h"
#include <ti/mas/neu/neuproto.h>
#include "dsiuport.h"
#include <stdlib.h>
#include "diagbld.h"
#include "util.h"
#include "utlrepck.h"

#ifdef NDU_GMAC
extern bool dsiuBootMacPacketReceived;

#ifdef IP_BOOT_SUPP
dsiuIpDetails_t dsiuIpDetails[GG_NUM_IP_VER_SUPPORTED] = 
{
  {neuproto_IP_BYTE_OFFSET_SRC_ADDR,
    neuproto_IP_ADDR_SIZE_IN_BYTES},
  {neuproto_IPV6_BYTE_OFFSET_SRC_ADDR,
    neuproto_IPV6_ADDR_SIZE_IN_BYTES}
};
#endif /* IP_BOOT_SUPP */
/**************************************************************************************
 * Declaration: The mac chip boot loader leaves configuration information in memory.
 *              This information will be used to configure the mac address for 
 *              reception of the boot config packet.
 **************************************************************************************/

void dsiu_copy_mac_config (diagAppHeader_t *da, dsiuMacbtNeu_t *mac)
{
  da->flags1              = mac->flags1;
  da->flags2              = mac->flags2;
  da->rxPayloadType       = mac->payloadType;
  da->ipOffsetBytes       = mac->ipOffsetBytes;
  da->udpOffsetBytes      = mac->udpOffsetBytes;
  da->ethOffsetBytes      = mac->ethOffsetBytes;
  da->totalHeaderLenBytes = mac->totalHeaderLenBytes;
  da->rxOffsetBytes       = mac->rxOffsetBytes;
  da->priDiffTag          = mac->priDiffTag;
  da->txMinCellBytes      = SIU_WORDS_TO_BYTES(mac->txMinCellWords);
  da->rxOffsetBytes       = mac->rxOffsetBytes;
  memcpy (da->hdrImage, mac->header, SIU_BYTES_TO_WORDS(mac->totalHeaderLenBytes));
}
/*************************************************************************************
 * FUNCTION PURPOSE: Verify the boot packet structure
 *************************************************************************************
 * DESCRIPTION: Checks the magic values in the boot config packet. Returns the 
 *              input parameter if they match, NULL otherwise.
 *************************************************************************************/
dsiuMacBootPacket_t *dsiu_boot_pkt_verify (dsiuMacBootPacket_t *pkt)
{
  if ( (pkt->magic0 == DSIU_MAC_BOOT_MAGIC_0) && (pkt->magic1 == DSIU_MAC_BOOT_MAGIC_1) )
    return (pkt);

  return (NULL);
} /* dsiu_boot_pkt_verify */


/**************************************************************************************
 * FUNCTION PURPOSE: Extract the source mac address from a mac header
 **************************************************************************************
 * DESCRIPTION: Copies the source mac address from the packet header to the
 *              mac address array. 0 is returned if the mac address is not
 *              found. 1 is returned if the mac address is returned successfully.
 **************************************************************************************/
tint dsiu_extract_mac (diagAppMacAddr_t *imac, diagAppHeader_t *hdr)
{
  tint p;

  /* An ethernet offset of < 0 indicates that the mac address is at the top of
   * the header. */
  if (hdr->ethOffsetBytes <= 0)
    p = SIU_BYTES_TO_TUINT(neuproto_ETH_BYTE_OFFSET_SRC_MAC);
  else  {
     if (hdr->ethOffsetBytes & 1)
       dsiuPanic (DSIU_FAIL_HDR_ALIGN);
     else
       p = SIU_BYTES_TO_TUINT(hdr->ethOffsetBytes+neuproto_ETH_BYTE_OFFSET_SRC_MAC);
  }

  imac->macAddr[0] = utlRead16bits_m((word *)(&hdr->hdrImage[p++]),0);
  imac->macAddr[1] = utlRead16bits_m((word *)(&hdr->hdrImage[p++]),0);
  imac->macAddr[2] = utlRead16bits_m((word *)(&hdr->hdrImage[p++]),0);
  
  return (1);

} /* dsiu_extract_mac */

/***************************************************************************************
 * FUNCTION PURPOSE: Extract the source IP address from the network header
 ***************************************************************************************
 * DESCRIPTION: Returns 1 if the IP source address is extracted correctly. Returns
 *              0 if the IP header is not word aligned.
 ***************************************************************************************/
tint dsiu_extract_ip (diagAppIpAddr_t *ip, diagAppHeader_t *hdr
#ifdef IP_BOOT_SUPP
,tint *ipVerFlag
#endif /* IP_BOOT_SUPP */
)
{
#ifdef IP_BOOT_SUPP
  tuint ipVer = 0;
  tuint i;
#endif /* IP_BOOT_SUPP */

  tint p;

  /* Verify there is an IP header */
  if (hdr->ipOffsetBytes < 0)
    return (0);

  /* Currently only a word aligned ip offset will work */
  if (hdr->ipOffsetBytes & 1)
    dsiuPanic (DSIU_FAIL_HDR_ALIGN);

#ifdef IP_BOOT_SUPP
  ipVer = (utlRead16bits_m((word *)&hdr->hdrImage[0],
                         (hdr->ipOffsetBytes + neuproto_IP_BYTE_OFFSET_VER_HLEN_TOS))
                          & neuproto_IP_VER_MASK);
  if (ipVer == neuproto_IP_VER_VALUE)
  {
    *ipVerFlag = DIAG_APP_IP_VER_IPV4_FLAG;
  }
  else if (ipVer == neuproto_IPV6_VER_VALUE)
  {
    *ipVerFlag = DIAG_APP_IP_VER_IPV6_FLAG;
  }
  else
    return (0);

  p = SIU_BYTES_TO_TUINT(hdr->ipOffsetBytes + 
                               dsiuIpDetails[*ipVerFlag].byteOffsetSrcAddr);
  for (i=0;i<SIU_BYTES_TO_TUINT(dsiuIpDetails[*ipVerFlag].addrLen);i++) {
    ip->IpAddr[i] = utlRead16bits_m((word *)(&hdr->hdrImage[p++]),0);;
  }

  //memcpy((word*)&(ip->IpAddr[0]), (word*)&(hdr->hdrImage[offset]),
  //        UTL_BYTES_TO_WORDS(dsiuIpDetails[*ipVerFlag].addrLen));
#else
  p = SIU_BYTES_TO_TUINT(hdr->ipOffsetBytes+neuproto_IP_BYTE_OFFSET_SRC_ADDR);

  ip->srcIpHigh = utlRead16bits_m((word *)(&hdr->hdrImage[p++]),0);
  ip->srcIpLow  = utlRead16bits_m((word *)(&hdr->hdrImage[p++]),0);
#endif /* IP_BOOT_SUPP */
  return (1);
} /* dsiu_extract_ip */



/**************************************************************************************
 * FUNCTION PURPOSE: Compare two mac addresses
 **************************************************************************************
 * DESCRIPTION: Returns TRUE if the two macs are equal.
 **************************************************************************************/
bool dsiu_mac_equal(diagAppMacAddr_t *m1, diagAppMacAddr_t *m2)
{
  if (m1->macAddr[0] != m2->macAddr[0])
    return (FALSE);

  if (m1->macAddr[1] != m2->macAddr[1])
    return (FALSE);

  if (m1->macAddr[2] != m2->macAddr[2])
    return (FALSE);

  return (TRUE);
} /* dsiu_mac_equal */

/**************************************************************************************
 * FUNCTION PURPOSE: Re-align the received packet by removing the first byte
 **************************************************************************************
 * DESCRIPTION: Performs a simple repack. The very last word is not packed
 *              because the last magic value can't be in the last word, or
 *              it would have been found in the pre-pack search.
 **************************************************************************************/
void dsiu_repack (word *packet, tuint sizeBytes)
{
  tint i;
  
  for (i = 0; i < SIU_BYTES_TO_WORDS(sizeBytes)-1; i++)  {
    packet[i] = ((packet[i] & 0x00ff) << 8) | ((packet[i+1] & 0xff00) >> 8);
  }
} /* dsiu_repack */

/**************************************************************************************
 * FUNCTION PURPOSE: Receive a packet
 **************************************************************************************
 * DESCRIPTION: Packets received by the HAL are passed to this function. The packet
 *              is checked for the magic tags to verify it is a boot packet, then
 *              the packet is processed.
 **************************************************************************************/
void dsiuBootRcvPacket (void *cxt, word *packet, tuint sizeBytes)
{
  dsiuMacBootPacket_t *pkt;
  diagApp_t           *diagApp;
#ifdef IP_BOOT_SUPP
  tuint addrLenBytes = 0, addrLenTuint = 0;
  tint ipVerFlag = -1, ipVer = 0;
  tint j, k, l, p, i1;
  tint max;
  halQuery_t           coreQuery;
  tuint nMacAddr = 0, nIpAddr = 0;
#endif /* IP_BOOT_SUPP */



  tint i;
  tint maxSearchLen;
  tuint bits;
  bool  gige = FALSE;

  DSIU_DUMMY_ARG(cxt);
  /* The maximum search length prevents a search beyond the size of the
   * input packet. This works only if the magic values are the first and
   * the last values in the boot packet structure  */
  maxSearchLen = SIU_BYTES_TO_TUINT(sizeBytes) - SIU_BYTES_TO_TUINT(sizeof(dsiuMacBootPacket_t)) + 1;


  /* Search for the magic0 pattern */
  pkt = NULL;
  for (i = 0; i < maxSearchLen; i++)  {
    if (((tuint *)packet)[i] == DSIU_MAC_BOOT_MAGIC_0)  {
      if ( (pkt = dsiu_boot_pkt_verify ((dsiuMacBootPacket_t *)&(((tuint *)packet)[i]))) != NULL)
        break;
    }
  }
  /* If not found, re-align the bytes/words and check again */
  if (pkt == NULL)  {
    dsiu_repack (packet, sizeBytes);
    for (i = 0; i < maxSearchLen-1; i++)  {
      if ((tuint)packet[i] == DSIU_MAC_BOOT_MAGIC_0)
        if ( (pkt = dsiu_boot_pkt_verify ((dsiuMacBootPacket_t *)&(((tuint *)packet)[i]))) != NULL)
          break;
    }
  }


  /* Return if this was not a valid packet */
  if (pkt == NULL)
    return;   /* There should be a stat */

  /* The packet has been verified. Process the data. */
  diagApp = &dsiuContext.diagApp;
  dsiuContext.dregMsgCfg->route = DIAGAPP_ROUTE_GMAC;

  /* Route messages through the mac interface since the boot config was received
   * through the mac */
  diagApp->route = DIAGAPP_ROUTE_GMAC;

  /* Dsp ID not provided used by mac interface. Set it to 0 */
  diagApp->dspId = pkt->dsp_id;

  diagApp->tag_desc= pkt->tag_desc;

  /* Populate the mac configuration structure for messages and block requests */
  dsiu_copy_mac_config (&(diagApp->u.mac.msg), &(pkt->msg));
#ifdef IP_BOOT_SUPP
  dsiu_copy_mac_config (&(diagApp->u.mac.block), &(pkt->blk));
  dsiu_copy_mac_config (&(diagApp->u.mac.nonVoice), &(pkt->nvmsg));

  /* Populate the Non Voice control parameters */
  diagApp->nvmsgOnOff          = pkt->nvmsgOnOff;
  diagApp->max_nvmsg           = pkt->max_nvmsg;

  /* Copy the mac and ip address configuration. If any mac addresses are specified
   * in the boot config mac address, all valid macs must be specified. If 
   * no macs are specified then the source mac addresses from the message and block
   * headers are used. */
  nMacAddr = pkt->nMacAddr;
  max = nMacAddr > DIAGAPP_MAX_MAC_ADDR ? DIAGAPP_MAX_MAC_ADDR : nMacAddr;

  if (nMacAddr > 0)  {
    for (i = 0; i < max; i++)  {
      diagApp->u.mac.macAddr[i].core       = pkt->macAddr[i].core;
      diagApp->u.mac.macAddr[i].macAddr[0] = utlRead16bits_m((word *)&pkt->macAddr[i].macHigh, 0);
      diagApp->u.mac.macAddr[i].macAddr[1] = utlRead16bits_m((word *)&pkt->macAddr[i].macMed, 0);
      diagApp->u.mac.macAddr[i].macAddr[2] = utlRead16bits_m((word *)&pkt->macAddr[i].macLow, 0);
    }

  }  else  {
    
    /* Extract the source macs from the message and blocks */
    i  = 0;
    i += dsiu_extract_mac (&(diagApp->u.mac.macAddr[i]), &(diagApp->u.mac.msg));
    i += dsiu_extract_mac (&(diagApp->u.mac.macAddr[i]), &(diagApp->u.mac.block));
    diagApp->u.mac.macAddr[0].core = 0;
    diagApp->u.mac.macAddr[1].core = 0;

    if (i == 0)
      dsiuPanic (DSIU_FAIL_NO_MAC);

    if (i == 2)  {
      if (dsiu_mac_equal(&(diagApp->u.mac.macAddr[0]), &(diagApp->u.mac.macAddr[1])))
        i = 1;
    }

    /* For each mac address specified, add 1 to the ls byte to get the mac
     * for any secondary cores */
    coreQuery.periph = HAL_QUERY_NCORES;
    hal.halQuery (&coreQuery);

    /* k indexes the core number                               */
    /* j indexes the mac address                               */
    /* i is the number of mac addresses to carry accross cores */
    for (p = i, j = 0; j < i; j++)  {
      for (k = 1; k < coreQuery.u.core.nCores; k++, p++)  {
        diagApp->u.mac.macAddr[p].core       = k;
        diagApp->u.mac.macAddr[p].macAddr[0] = utlRead16bits_m((word *)&diagApp->u.mac.macAddr[j].macAddr[0], 0);
        diagApp->u.mac.macAddr[p].macAddr[1] = utlRead16bits_m((word *)&diagApp->u.mac.macAddr[j].macAddr[1], 0);
        diagApp->u.mac.macAddr[p].macAddr[2] = utlRead16bits_m((word *)&diagApp->u.mac.macAddr[j].macAddr[2], 0) + k;
      }
    }

    /* Set i equal to the number of mac addresses used to complete the
     * array of mac addresses */
    i = p;
  }

  /* Fill the rest of the mac structure with unmarked macs */
  for ( /* no init */ ; i < DIAGAPP_MAX_MAC_ADDR; i++)
    diagApp->u.mac.macAddr[i].core = -1;

    
  /* Repeat for the IP addresses */
  nIpAddr = pkt->nIpAddr;
  max = nIpAddr > DIAGAPP_MAX_IP_ADDR ? DIAGAPP_MAX_IP_ADDR : nIpAddr;

  /* Various scenario:
     1) If nIpAddr > 0
        --The address could be IPv4 or IPv6 address and is specified in typeIPverCore
        --The Msg header is not used for core IP address
        --Could support dual stack
     2) nIpAddr == 0
        -- Msg header will be used to assign core's IP address
        -- Core support only one IP version during download for Ping.
        
     Voice traffic can be either IPv4 or IPv6 and is independent of the download
  */
  if (nIpAddr > 0)  {
    for (i = 0; i < max; i++)  {
      diagApp->u.mac.ipAddr[i].typeIPverCore  = pkt->ipAddr[i].typeIPverCore;
      ipVerFlag = DIAG_APP_GET_IP_VER_FLAG(diagApp->u.mac.ipAddr[i].typeIPverCore);
      for (i1=0;i1<SIU_BYTES_TO_TUINT(dsiuIpDetails[ipVerFlag].addrLen);i1++) {
       diagApp->u.mac.ipAddr[i].IpAddr[i1] = utlRead16bits_m((word *)(&pkt->ipAddr[i].IpAddr[i1]),0);
     }
    }

  }  else  {

    /* For each ip address specified, add 1 to the ls byte to get the ip 
     * for any secondary cores */
    coreQuery.periph = HAL_QUERY_NCORES;
    hal.halQuery (&coreQuery);

    /* Extract the source IP addresses from the message structure */
    i = dsiu_extract_ip (&(diagApp->u.mac.ipAddr[0]), &(diagApp->u.mac.msg), &ipVerFlag);
    /* i==0 --> Error condition. No valid IP address found
       i==1 --> One IP address found                       */
    if(i) {
      addrLenBytes = dsiuIpDetails[ipVerFlag].addrLen;
      addrLenTuint = addrLenBytes >> 1;
      diagApp->u.mac.ipAddr[0].typeIPverCore = 0;
      DIAG_APP_SET_IP_VER_FLAG(diagApp->u.mac.ipAddr[0].typeIPverCore, ipVerFlag);
    }

    /* Modify the message IP address for each core */
    /* k indexes the core number                              */
    /* j indexes the ip address                               */
    /* i is the number of ip addresses to carry accross cores */
    for (p = i, j = 0; j < i; j++)  {
      for (k = 1; k < coreQuery.u.core.nCores; k++, p++)  {
        diagApp->u.mac.ipAddr[p].typeIPverCore  = k;
        DIAG_APP_SET_IP_VER_FLAG(diagApp->u.mac.ipAddr[0].typeIPverCore, ipVerFlag);
        DIAG_APP_SET_IP_TYPE(diagApp->u.mac.ipAddr[0].typeIPverCore, ipVerFlag);

        for (i1=0;i1<SIU_BYTES_TO_TUINT(addrLenBytes);i1++) {
          diagApp->u.mac.ipAddr[p].IpAddr[i1] = utlRead16bits_m((word *)(&pkt->ipAddr[j].IpAddr[i1]),0);
        }
        diagApp->u.mac.ipAddr[p].IpAddr[addrLenTuint-1] +=k;
      }
    }

    /* Repeat for the block request information */
    i = dsiu_extract_ip (&(diagApp->u.mac.ipAddr[l = p++]), &(diagApp->u.mac.block), &ipVerFlag);
    if(i) {
      addrLenBytes = dsiuIpDetails[ipVerFlag].addrLen;
      addrLenTuint = addrLenBytes >> 1;
      diagApp->u.mac.ipAddr[l].typeIPverCore = 0;
      DIAG_APP_SET_IP_VER_FLAG(diagApp->u.mac.ipAddr[l].typeIPverCore, ipVerFlag);
      DIAG_APP_SET_IP_TYPE(diagApp->u.mac.ipAddr[l].typeIPverCore, DIAG_APP_IP_TYPE_BLOCK);
    }

    /* k indexes the core number                              */
    /* j indexes the ip address                               */
    /* i is the number of ip addresses to carry accross cores */
    for (j = 0; j < i; j++)  {
      for (k = 1; k < coreQuery.u.core.nCores; k++, p++)  {
        diagApp->u.mac.ipAddr[p].typeIPverCore  = k ;
        DIAG_APP_SET_IP_VER_FLAG(diagApp->u.mac.ipAddr[p].typeIPverCore, ipVerFlag);
        DIAG_APP_SET_IP_TYPE(diagApp->u.mac.ipAddr[p].typeIPverCore, DIAG_APP_IP_TYPE_BLOCK);
        for (i1=0;i1<SIU_BYTES_TO_TUINT(addrLenBytes);i1++) {
          diagApp->u.mac.ipAddr[p].IpAddr[i1] = utlRead16bits_m((word *)(&pkt->ipAddr[j].IpAddr[i1]),0);
        }
        diagApp->u.mac.ipAddr[p].IpAddr[addrLenTuint-1] +=k;
      }
    }
   
    /* Repeat for the Non Voice request information */
    i = dsiu_extract_ip (&(diagApp->u.mac.ipAddr[l = p++]), &(diagApp->u.mac.nonVoice), &ipVerFlag);
    if(i) {
      addrLenBytes = dsiuIpDetails[ipVerFlag].addrLen;
      addrLenTuint = addrLenBytes >> 1;
      diagApp->u.mac.ipAddr[l].typeIPverCore = 0;
      DIAG_APP_SET_IP_VER_FLAG(diagApp->u.mac.ipAddr[l].typeIPverCore, ipVerFlag);
      DIAG_APP_SET_IP_TYPE(diagApp->u.mac.ipAddr[l].typeIPverCore, DIAG_APP_IP_TYPE_NV);
    }

    /* k indexes the core number                              */
    /* j indexes the ip address                               */
    /* i is the number of ip addresses to carry accross cores */
    for (j = 0; j < i; j++)  {
      for (k = 1; k < coreQuery.u.core.nCores; k++, p++)  {
        diagApp->u.mac.ipAddr[p].typeIPverCore  = k ;
        DIAG_APP_SET_IP_VER_FLAG(diagApp->u.mac.ipAddr[p].typeIPverCore, ipVerFlag);
        DIAG_APP_SET_IP_TYPE(diagApp->u.mac.ipAddr[p].typeIPverCore, DIAG_APP_IP_TYPE_NV);
        for (i1=0;i1<SIU_BYTES_TO_TUINT(addrLenBytes);i1++) {
          diagApp->u.mac.ipAddr[p].IpAddr[i1] = utlRead16bits_m((word *)(&pkt->ipAddr[j].IpAddr[i1]),0);
        }
        diagApp->u.mac.ipAddr[p].IpAddr[addrLenTuint-1] +=k;
      }
    }
    i = p;

  }   

  for ( /* no init */ ; i < DIAGAPP_MAX_IP_ADDR; i++) {
    diagApp->u.mac.ipAddr[i].typeIPverCore = -1;
  }
#else /* IP_BOOT_SUPP */

  /* Extract the source macs from the message and blocks */
  dsiu_extract_mac (&(diagApp->u.mac.macAddr[0]), &(diagApp->u.mac.msg));

  /* Create the default mac addresses for any secondary cores by adding
   * 1 to the lsb of the received mac */
  for (i = 1; i < DIAGAPP_MAX_MAC_ADDR; i++)  {
    diagApp->u.mac.macAddr[i].macAddr[0] = diagApp->u.mac.macAddr[0].macAddr[0];
    diagApp->u.mac.macAddr[i].macAddr[1] = diagApp->u.mac.macAddr[0].macAddr[1];
    diagApp->u.mac.macAddr[i].macAddr[2] = diagApp->u.mac.macAddr[0].macAddr[2] + i;
    if (diagApp->u.mac.macAddr[i].macAddr[2] < diagApp->u.mac.macAddr[0].macAddr[2])
      diagApp->u.mac.macAddr[i].macAddr[1] += 1;
    if (diagApp->u.mac.macAddr[i].macAddr[0] < diagApp->u.mac.macAddr[0].macAddr[0])
      diagApp->u.mac.macAddr[i].macAddr[0] += 1;
  }

  
    
  /* Repeat for the IP addresses */
  /* Extract the source IP addresses from the message structure */
  dsiu_extract_ip (&(diagApp->u.mac.ipAddr[0]), &(diagApp->u.mac.msg));

  /* Create the default IP addresses for any secondary cores by adding
   * 1 to the lsb of the received ip */
  for (i = 1; i < DIAGAPP_MAX_MAC_ADDR; i++)  {
    diagApp->u.mac.ipAddr[i].srcIpHigh = diagApp->u.mac.ipAddr[0].srcIpHigh;
    diagApp->u.mac.ipAddr[i].srcIpLow  = diagApp->u.mac.ipAddr[0].srcIpLow + i;
    if (diagApp->u.mac.ipAddr[i].srcIpLow < diagApp->u.mac.ipAddr[0].srcIpLow)
      diagApp->u.mac.ipAddr[i].srcIpHigh += 1;
  }

  /* Logical buffer boot information */
  diagApp->u.mac.macBufferAssign = pkt->macBufferAssign;
#endif /* IP_BOOT_SUPP */

  /* Mac configFlags setup. Defaults were setup when the port was configured,
   * these can be overridden here. */

  /* Gigabit, 10/100 mode */
  bits = (pkt->portConfig1 & (0x3 << 4)) >> 4;
  switch (bits)  {

    case DSIU_MAC_BOOT_PORTCONFIG_SPEED_1G:
      diagApp->u.mac.configFlags |= DIAGAPP_MAC_CFG_GE;
	  gige = TRUE;
      break;

    case DSIU_MAC_BOOT_PORTCONFIG_SPEED_10_100:
      diagApp->u.mac.configFlags &= ~(DIAGAPP_MAC_CFG_GE);
      break;

    /* default operation is already setup */
    
  }

  /* Duplex mode */
  bits = (pkt->portConfig1 & 0x03);
  switch (bits)  {

    case DSIU_MAC_BOOT_PORTCONFIG_DUPLEX_HALF:
      diagApp->u.mac.configFlags |= DIAGAPP_MAC_CFG_HD;
      break;

    case DSIU_MAC_BOOT_PORTCONFIG_DUPLEX_FULL:
      diagApp->u.mac.configFlags &= ~(DIAGAPP_MAC_CFG_HD);
      break;

    /* default operation is already setup */
  }

  /* Flow control */
  bits = (pkt->portConfig1 & (0x03<<2)) >> 2;
  switch (bits)  {

    case DSIU_MAC_BOOT_PORTCONFIG_FLOW_CTRL_ENABLE:
      diagApp->u.mac.configFlags |= DIAGAPP_MAC_CFG_FC;
      break;
  
    case DSIU_MAC_BOOT_PORTCONFIG_FLOW_CTRL_DISABLE:
      diagApp->u.mac.configFlags &= ~(DIAGAPP_MAC_CFG_FC);
      break;

    /* default operation is already setup */

  }

  /* RGMII/RMII - only 100mb RMII supported at the moment */
  bits = (pkt->portConfig1 >> 6) & 0x3;
  switch (bits)  {

    case DSIU_MAC_BOOT_PORTCONFIG_RMII_ENABLE:
	  if (gige)
        diagApp->u.mac.configFlags |= DIAGAPP_MAC_CFG_RGMII;
	  else  
        diagApp->u.mac.configFlags |= DIAGAPP_MAC_CFG_RMII_100;
      break;

    case DSIU_MAC_BOOT_PORTCONFIG_RMII_DISABLE:
	  if (gige)
        diagApp->u.mac.configFlags &= ~(DIAGAPP_MAC_CFG_RGMII);
	  else
        diagApp->u.mac.configFlags &= ~(DIAGAPP_MAC_CFG_RMII_100);
      break;

    /* default operation is already setup */
  }


  /* Copy the config information to be used by the application */
  dsiu_boot_copy_info ();
  dsiuBootMacPacketReceived = TRUE;

} /* dsiuBootRcvPkt */

/**************************************************************************************
 * FUNCTION PURPOSE: Allocate the mac buffers
 **************************************************************************************
 * DESCRIPTION: Allocates and verifies the buffers based on size requirements
 *              and available buffer size
 **************************************************************************************/
void dsiuSetMacBootBuffers (halMemBuffer_t *bufs, tuint logicalPort)
{
  halMemBuffer_t *bp;
  tuint  *bstart;
  tulong  bsize;
  tuint   alignUsed;

  /* Allocate the Heap buffer */
  bp = &bufs[HAL_GMAC_HEAP_BUFN];
  if (bp->base == NULL)  {
    bstart = dsiuBufferAlign (diagCfg.halHeapBufStart + dsiuContext.halHeapUsed, diagCfg.halHeapBufEnd, 
                              bp->log2align, &bsize, &alignUsed);
    if (bsize < bp->size)
      dsiuPanic (DSIU_MEM_ALLOC_FAIL_HAL);

    bp->base = (void *)bstart;
    dsiuContext.halHeapUsed += (bp->size + alignUsed);
    DSIU_PORT_ALIGN_HEAPBASE(dsiuContext.halHeapUsed);
  }


  /* Tx buffer */
  bp = &bufs[HAL_GMAC_TX_BUFN];
  if (bp->base == NULL)  {
    bstart = dsiuBufferAlign (diagCfg.gmacDmaMem[logicalPort].txStart, diagCfg.gmacDmaMem[logicalPort].txEnd,
                              bp->log2align, &bsize, &alignUsed);

    if (bsize < bp->size)
      dsiuPanic (DSIU_MEM_ALLOC_FAIL_HAL);

    bp->base = (void *)bstart;
  }

  /* Rx buffer */
  bp = &bufs[HAL_GMAC_RX_BUFN];
  if (bp->base == NULL)  {
    bstart = dsiuBufferAlign (diagCfg.gmacDmaMem[logicalPort].rxStart, diagCfg.gmacDmaMem[logicalPort].rxEnd,
                              bp->log2align, &bsize, &alignUsed);

    if (bsize < bp->size)
      dsiuPanic (DSIU_MEM_ALLOC_FAIL_HAL);

    bp->base = (void *)bstart;
  }

  /* Assembly buffer */
  bp = &bufs[HAL_GMAC_ASSEM_BUFN];
  if (bp->base == NULL)  {
    bstart = dsiuBufferAlign (diagCfg.gmacRxAssemMem[logicalPort].start, diagCfg.gmacRxAssemMem[logicalPort].end,
                              bp->log2align, &bsize, &alignUsed);

  if (bsize < bp->size)
    dsiuPanic (DSIU_MEM_ALLOC_FAIL_HAL);

    bp->base = (void *)bstart;

  }

} /* dsiuSetMacBootBuffers */


/**************************************************************************************
 * FUNCTION PURPOSE: Provide mac specific buffer utilization
 **************************************************************************************
 * DESCRIPTION: Loads the mac sizes structure with boot specific
 *              parameters.
 **************************************************************************************/
void dsiuGetMacBootSizes (halGmacSizes_t *sizes)
{

  /* Transmit */
  sizes->tx.bufferUnitSizeWords  = 200;
  sizes->tx.bufferTotalSizeWords = (tulong)diagCfg.gmacDmaMem[0].txEnd - (tulong)diagCfg.gmacDmaMem[0].txStart;
  sizes->tx.startAddr            = diagCfg.gmacDmaMem[0].txStart;

  /* Receive */
  sizes->rx.bufferUnitSizeWords  = 200;
  sizes->rx.bufferTotalSizeWords = (tulong)diagCfg.gmacDmaMem[0].rxEnd - (tulong)diagCfg.gmacDmaMem[0].rxStart;
  sizes->rx.startAddr            = diagCfg.gmacDmaMem[0].rxStart;

  /* This size must be large enough to handle the largest received packet. During
   * boot this will be memory write requests. */
  sizes->maxRxPacketSizeWords = MAX_PACKET_SIZE_IN_BYTES;  // this is in BYTES

} /* dsiuGetMacBootSizes */

/*************************************************************************************
 * FUNCTION PURPOSE: Get mac configuration information
 *************************************************************************************
 * DESCRIPTION: Mac boot information from a previous boot loader, or hard coded
 *              base on device, is retrieved and loaded into the mac configuration
 *              structure.
 *************************************************************************************/
void dsiuGetMacBootConfig (halGmacInit_t *macInit, halGmacSizes_t *sizes)
{
  tint i;
  dsiuPrevBootInfo_t pInfo;

  /* Get boot information. Some devices will get this from a ROM boot process,
   * others will hard code the information */
  if (!dsiuCallTable.prevBootInfo)
    dsiuPanic (DSIU_FAIL_NO_BOOT_INFO);

  if ((*dsiuCallTable.prevBootInfo)(&pInfo) != DSIU_SUCCESS)
    dsiuPanic (DSIU_FAIL_INVALID_BOOT_INFO);
  
  /* Receive callout */
  macInit->rcvFrame = dsiuBootRcvPacket;
  macInit->cxt      = NULL;

  /* The physical ports are assigned in sequence */
  macInit->physPort = pInfo.physPort;

  /* The mac address to use is found from the chip boot loader configuration structure.
   * Only 1 mac address is used. */
  macInit->cfg.mac[0].core = 0;
  /* For kailash the boot parameters are not used */
  macInit->cfg.mac[0].mac_addr[0] = pInfo.u.macInfo.macAddr[0];
  macInit->cfg.mac[0].mac_addr[1] = pInfo.u.macInfo.macAddr[1];
  macInit->cfg.mac[0].mac_addr[2] = pInfo.u.macInfo.macAddr[2];


  /* Don't accept any other mac addresses */
  for (i = 1; i < CSL_MAC_MAX_MAC_ADDR; i++)
    macInit->cfg.mac[i].core = -1;


  /* For the bootloader/diag, 400 words are used. This covers 128 word writes for the
   * bootloader, and 10ms pcm for the diag */
  macInit->cfg.maxRxPacketSizeBytes = sizes->maxRxPacketSizeWords;

  /* Parameters received from the mac boot loader mode. These can be overridden
   * by the received boot config packet. */
  dsiuContext.diagApp.u.mac.configFlags = 0;

  if ( (pInfo.u.macInfo.iftype == DSIU_PREV_MAC_IF_GMII)  ||
       (pInfo.u.macInfo.iftype == DSIU_PREV_MAC_IF_RGMII)  )  {
    macInit->cfg.speed = CSL_MAC_MODE_GIG;
    dsiuContext.diagApp.u.mac.configFlags |= DIAGAPP_MAC_CFG_GE;
  }  else  {
    macInit->cfg.speed = CSL_MAC_MODE_10_100;
  }

  if (pInfo.u.macInfo.halfDuplex)  {
    macInit->cfg.duplex = CSL_MAC_DUPLEX_HALF;
    dsiuContext.diagApp.u.mac.configFlags |= DIAGAPP_MAC_CFG_HD;
  }  else  {
    macInit->cfg.duplex = CSL_MAC_DUPLEX_FULL;
  }

  if (pInfo.u.macInfo.flowControl)  {
    macInit->cfg.flow = CSL_MAC_FLOW_CONTROL_ON;
    dsiuContext.diagApp.u.mac.configFlags |= DIAGAPP_MAC_CFG_FC;
  }  else  {
    macInit->cfg.flow = CSL_MAC_FLOW_CONTROL_OFF;
  }

  if ( (pInfo.u.macInfo.iftype == DSIU_PREV_MAC_IF_RMII)     ||
       (pInfo.u.macInfo.iftype == DSIU_PREV_MAC_IF_RMII_100)  )  {
    macInit->cfg.rmii      = CSL_MAC_RMII_ON;
    macInit->cfg.rmiiSpeed = CSL_MAC_RMII_SPEED_100MBS;
    dsiuContext.diagApp.u.mac.configFlags |= DIAGAPP_MAC_CFG_RMII_100;

  } else if (pInfo.u.macInfo.iftype == DSIU_PREV_MAC_IF_RMII_10)  {
    macInit->cfg.rmii      = CSL_MAC_RMII_ON;
    macInit->cfg.rmiiSpeed = CSL_MAC_RMII_SPEED_10MBS;
    dsiuContext.diagApp.u.mac.configFlags |= DIAGAPP_MAC_CFG_RMII_10;

  } else  {
    macInit->cfg.rmii      = CSL_MAC_RMII_OFF;
  }

  if (pInfo.u.macInfo.iftype == DSIU_PREV_MAC_IF_RGMII) {
    macInit->cfg.rgmii = CSL_MAC_RGMII_ON;
    dsiuContext.diagApp.u.mac.configFlags |= DIAGAPP_MAC_CFG_RGMII;
  }  else  {
    macInit->cfg.rgmii = CSL_MAC_RGMII_OFF;
  }


} /* dsiuGetMacBootConfig */

/************************************************************************************
 * FILE PURPOSE: Load simulated app configurations
 ************************************************************************************
 * DESCRIPTION: Loads the system configuration information and adjusts for 
 *              mac specific information.
 ************************************************************************************/
dsiuReturn_t dsiuGmacSimuAppConfig (void)
{
  dsiuReturn_t ret;

  if ((ret = dsiuSimuAppConfig ()) != DSIU_SUCCESS)
    return (ret);

  return (DSIU_SUCCESS);

} /* dsiuGmacSimuAppConfig */


#endif


