#ifndef _DSIUMACBT_H
#define _DSIUMACBT_H
/**********************************************************************************
 * FILE PURPOSE: MAC boot configuration definitions
 **********************************************************************************
 * DESCRIPTION: Definitions used to implement reception of a boot config packet.
 *
 * TABS: None
 *
 * Copyright (C) 2004 Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include "types.h"
#ifdef IP_BOOT_SUPP
/* Present size is 1162 bytes */
#define MAX_PACKET_SIZE_IN_BYTES 1200
#else
#define MAX_PACKET_SIZE_IN_BYTES 700
#endif /* IP_BOOT_SUPP */
/*********************************************************************************
 * Definition: Mac address configuration. No mac addresses are required if 
 *             the flags indicate internally hard configured mac(s) are to be used.
 *             If a core does not have a mac address specified the mac address is
 *             derived from the mac address for core 0. If core 0 has no mac 
 *             address specified it is derived from the header information.
 *********************************************************************************/
typedef struct dsiuMacbtMacAddr_s {
  
  INT16 core;       /* Core number of associated mac address. A value of -1
                     * indicates the structure does not contain a valid mac */

  UINT16 macHigh;   /* Most significant 16 bits  */
  UINT16 macMed;    /* Middle 16 bits            */
  UINT16 macLow;    /* Least significant 16 bits */
 
} dsiuMacbtMacAddr_t;

#ifdef IP_BOOT_SUPP
/*********************************************************************************
 * Definition: IP address configuration. Can contain either IPv4 or IPv6 address
 *             Each core can be assigned a unique IP
 *             address, and a different IP address for messages and block requests.
 *             If a core does not have an IP assigned, the IP is derived based on
 *             the IP address for core 0. If core 0 does not have an IP address
 *             assigned the value used is from the header information.
 *
 *   Bit values for typeIPverCore
 *
 *       /-----------------+----------+----+-------------------\
 *       | 15   rsvd   9   |  8  7  6 | 5  | 4              0  |
 *       \-----------------+----------+----+-------------------/
 *                              |        |     \->  5 bits specify core number
 *                              |        |
 *                              |         \-->IP protocol 0 - IPv4
 *                              |                         1 - IPv6
 *                              \--> Message/block flag: 0 - IP for message
 *                                                       1 - IP for block
 *                                                       2 - IP for Non Voice
 *                                                       3 - IP for Processor
 *
 *********************************************************************************/
#define DSIU_MAC_BOOT_IPV6_ADDR_LEN_IN_TUINT    8   
typedef struct dsiuMacbtIpAddr_s {
  INT16   typeIPverCore;   
  UINT16  IpAddr[DSIU_MAC_BOOT_IPV6_ADDR_LEN_IN_TUINT];    /* Source IP  */
} dsiuMacbtIpAddr_t;
#else
/*********************************************************************************
 * Definition: IP address configuration. Each core can be assigned a unique IP
 *             address, and a different IP address for messages and block requests.
 *             If a core does not have an IP assigned, the IP is derived based on
 *             the IP address for core 0. If core 0 does not have an IP address
 *             assigned the value used is from the header information.
 *
 *   Bit values for typeCore
 *
 *       /-----------------+----------+-----------------------\
 *       | 15   rsvd   9   |    8     |   7                0  |
 *       \-----------------+----------+-----------------------/
 *                              |         \->  8 bits specify core number
 *                              |
 *                              \--> Message/block flag: 0 - IP for message
 *                                                       1 - IP for block
 *
 *********************************************************************************/
typedef struct dsiuMacbtIpAddr_s {
  INT16   typeCore;   
  UINT16  srcIpHigh;    /* Most significant bits of source IP  */
  UINT16  srcIpLow;     /* Least significant bits of source IP */
} dsiuMacbtIpAddr_t;
#endif /* IP_BOOT_SUPP */

/*********************************************************************************
 * Definition: Basic NEU configuration. One configuration is provided for
 *             messages and another for block requests.
 *********************************************************************************/
#ifdef IP_BOOT_SUPP
#define DSIU_MAC_BOOT_MAX_HEADER_LEN    42   /* 42 words */
#else
#define DSIU_MAC_BOOT_MAX_HEADER_LEN    30   /* 30 words */
#endif /* IP_BOOT_SUPP */
typedef struct dsiuMacbtNeu_s {
  UINT16    flags1;              /* Flags 1 used by NEU                  */
  UINT16    flags2;              /* Flags 2 used by NEU                  */
  UINT16    payloadType;         /* NEU payload type                     */
  UINT16    ipOffsetBytes;       /* Offset to the IP header              */
  UINT16    udpOffsetBytes;      /* Offset to the UDP header             */
  UINT16    ethOffsetBytes;      /* Offset to the ethernet header        */
  UINT16    totalHeaderLenBytes; /* Total length of attached header      */
  UINT16    priDiffTag;          /* Message priority differentiation tag */
  UINT16    txMinCellWords;      /* Min tx cell size                     */
  UINT16    rxOffsetBytes;       /* Receive offset, bytes                */
  UINT16    rsvd_c[8];           /* Reserved                             */
  
  UINT16    header[DSIU_MAC_BOOT_MAX_HEADER_LEN];  /* Header info        */
  
  UINT16    rsvd_d[32];          /* Reserved */
} dsiuMacbtNeu_t;

/*********************************************************************************
 * Definition: Magic values
 *********************************************************************************/
#define DSIU_MAC_BOOT_MAGIC_0  0x544e
#define DSIU_MAC_BOOT_MACIC_1  0x2164

/**********************************************************************************
 * Definition: Bit values for portConfig1
 *
 *    /-----------------------------------+-------+--------+--------\
 *    | 15          rsvd          | 7   6 | 5  4  |  3  2  |  1  0  |
 *    \-----------------------------------+-------+--------+--------/
 *                                    |       |        |      \-> Duplex: 00 - boot rom value
 *                                    |       |        |                  01 - full duplex
 *                                    |       |        |                  10 - half duplex
 *                                    |       |        |                  11 - reserved
 *                                    |       |        |
 *                                    |       |        \-> Flow control: 00 - boot rom value
 *                                    |       |                          01 - disabled
 *                                    |       |                          10 - enabled
 *                                    |       |                          11 - reserved
 *                                    |       |
 *                                    |       \-> Gig Mode: 00 - based on clock mode
 *                                    |                     01 - force 10/100
 *                                    |                     10 - force Gigabit
 *                                    |
 *                                    \-> RMII Mode: 00 - based on boot rom value
 *                                                   01 - rmii disabled
 *                                                   10 - rmii enabled
 *
 **********************************************************************************/
 
 #define DSIU_MAC_BOOT_PORTCONFIG_SPEED_BOOT_ROM      0x00
 #define DSIU_MAC_BOOT_PORTCONFIG_SPEED_10_100        0x01
 #define DSIU_MAC_BOOT_PORTCONFIG_SPEED_1G            0x02
 
 #define DSIU_MAC_BOOT_PORTCONFIG_DUPLEX_BOOT_ROM     0x00
 #define DSIU_MAC_BOOT_PORTCONFIG_DUPLEX_FULL         0x01
 #define DSIU_MAC_BOOT_PORTCONFIG_DUPLEX_HALF         0x02
 
 #define DSIU_MAC_BOOT_PORTCONFIG_FLOW_CTRL_BOOT_ROM  0x00
 #define DSIU_MAC_BOOT_PORTCONFIG_FLOW_CTRL_DISABLE   0x01
 #define DSIU_MAC_BOOT_PORTCONFIG_FLOW_CTRL_ENABLE    0x02
 
 #define DSIU_MAC_BOOT_PORTCONFIG_RMII_BOOT_ROM       0x00
 #define DSIU_MAC_BOOT_PORTCONFIG_RMII_DISABLE        0x01
 #define DSIU_MAC_BOOT_PORTCONFIG_RMII_ENABLE         0x02
 


#ifdef IP_BOOT_SUPP
/*********************************************************************************
 * The boot config packet is designed to be decoded without regard to headers.
 * The information is contained between two tags. The DSP scans received packets
 * for the two magic values.
 * 
 * The message parameters describe the values used for messages to/from the host.
 *********************************************************************************/
typedef struct {

  /* The first magic value is used to find the start of the parameters. The
   * magic value is included in the checksum */
  UINT16    magic0;                /* 0x544e */

  UINT16    dsp_id;                /* DSP ID  (logic DSP number for core A) */

  /* Hardware configuration parameters */
  UINT16    portConfig1;           /* Port config flags                   */
  
  UINT16    rsvd_a[4];              /* Reserved                      */
  
  UINT16    nMacAddr;               /* Number of valid elements in macAddr */
  dsiuMacbtMacAddr_t macAddr[16];   /* Mac address configuration           */
  
  UINT16    nIpAddr;                /* Number of valid elements in ipAddr */
  dsiuMacbtIpAddr_t  ipAddr[16];    /* IPv4 or v6 addresses               */

  UINT16    rsvd_b[32];             /* Reserved                           */
  
  /* NEU parameters for messages */ 
  dsiuMacbtNeu_t msg;
  dsiuMacbtNeu_t blk;
  dsiuMacbtNeu_t nvmsg;

  UINT16         nvmsgOnOff;       /* 1: Forward the messages,0: Drop the packets*/

  UINT16         max_nvmsg;        /* Maximum number of packets to forward */

  
  UINT16    magic1;                /* 0x2164 */
  
} dsiuMacBootPacket_t;
#else
/*********************************************************************************
 * The boot config packet is designed to be decoded without regard to headers.
 * The information is contained between two tags. The DSP scans received packets
 * for the two magic values.
 * 
 * The message parameters describe the values used for messages to/from the host.
 *********************************************************************************/
typedef struct {

  /* The first magic value is used to find the start of the parameters. The
   * magic value is included in the checksum */
  UINT16    magic0;                /* 0x544e */
  
  UINT16    dsp_id;                /* DSP ID  (logic DSP number for core A) */
  
  /* Hardware configuration parameters */
  UINT16    portConfig1;           /* Port config flags                   */
  
#define DSIU_MAC_BOOT_BUFFER_MSG     0
#define DSIU_MAC_BOOT_BUFFER_VCH     1
#define DSIU_MAC_BOOT_BUFFER_SINGLE  2
  UINT16    macBufferAssign;       /* Mac assocatated memory for boot     */

/*
    ** tag_desc: Tag descriptor
    **
    **     15  14  13  12  11  10   9   8   7   6   5   4   3   2   1   0
    **   +---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+
    **   |Field Len  | Byte offset       |Bit offset |     Tag Len       |
    **   +---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+
    **   
    **   Byte offset: Byte offset to the protocol identifier tag in the PDT from the start of the Ethernet frame
    **   Bit offset: Bit offset to the Least significant bit of the protocol identifier tag in the PDT
    **   Field Len:	Length of the protocol identifier tag in the PDT in bits
    **   Tag Len: Total Length of the PDT tag in bits (channel Identifier + protocol identifier)
    **   
    **   Length of the Channel identifier tag (in bits) = (Tag Len - Field Len)
    **   Assumption/Restriction: protocol identifier tag is always preceded by the Channel identifier bits
    **
*/
  UINT16    tag_desc;

  UINT16    rsvd[38];              /* Reserved */
  
  /* NEU parameters for messages */ 
  dsiuMacbtNeu_t msg;
  
  UINT16    magic1;              /* 0x2164 */
  
} dsiuMacBootPacket_t;
#endif /* IP_BOOT_SUPP */
  
  
/*****************************************************************************
 * DEFINITION: Magic values for boot packets
 *****************************************************************************/
#define DSIU_MAC_BOOT_MAGIC_0   0x544e
#define DSIU_MAC_BOOT_MAGIC_1   0x2164


#endif




