/*************************************************************************************
 * FILE PURPOSE: Bootup configuration through gmac
 *************************************************************************************
 * FILE NAME: dsiumacbt1.c
 *
 * DESCRIPTION: Configures the gmac port to receive data containing
 *              system peripheral parameters.
 *
 * TABS: None
 *
 * Copyright (C) 2004, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include "types.h"
#include "dsiuloc.h"
#include "hal.h"
#include "diagcfg.h"
#include "dsiumem.h"
#include "dsiuport.h"
#include "dsiumacbt.h"
#include <stdlib.h>

#ifdef PA_SS
#include "dsiunwal.h"
nwal_Handle  dsiuAddMac    (nwalMacParam_t     *pMacInfo);
#endif

/************************************************************************************
 * Definition: Global variables. These variables are set when a complete boot cell
 *             sequence or boot packet sequence have been received. Note that these
 *             variable names are shared between this file and the individual mac
 *             and utopia boot loaders.
 ************************************************************************************/
volatile bool dsiuBootMacPacketReceived;    /* Set in dsiumbt.c */

#define BRINGUP_WORKAROUND

#ifdef BRINGUP_WORKAROUND
static tuint debug_no_rom_boot =0;
#endif
/*************************************************************************************
 * FUNCTION PURPOSE: Configure for boot cell/packet reception
 *************************************************************************************
 * DESCRIPTION: Checks the device for utopia and gmac peripheral, configures
 *              them for reception of a boot cell/packet.
 *************************************************************************************/
tint dsiuBootGmac (void)
{
#ifndef PA_SS
  halQuery_t        macQuery;
  halGmacInit_t     macInit;
  halGmacSizes_t    sizes;
  halMemBuffer_t   *bufs;
  halReturn_t       ret;
  tint              nbufs;
  tint              halId;
  dsiuPort_t       *dport;

  tuint      timestamp = 0;
  volatile tuint i;

  /* Initialize the globals */
  dsiuBootMacPacketReceived  = FALSE;



  /* Hal main struct is already configured in dsiuMain() */
  
  /* configure the required boot port and wait for boot config msg */

  /* Query hardware for mac configuration */
  macQuery.periph  = HAL_QUERY_MAC;
  macQuery.portNum = 0;
  hal.halQuery (&macQuery);

  /* If neither of the interfaces was present, panic */
  if  ( (macQuery.isPresent == FALSE) || (macQuery.isAvail == FALSE) ) {
    dsiuPanic (DSIU_FAIL_NO_BOOT_INTERFACE);
  }

  /* Get information left over from the chip boot loader. The port
   * number must be stored. It is not sent in the mac config packet */
  dsiuGetMacBootSizes (&sizes);

  /* Get memory requirements from HAL  */
  ret = (*dsiuContext.hal->gmac->halGmacGetSizes) (&nbufs, &bufs, &sizes);
  if (ret != HAL_SUCCESS)
    dsiuPanic (DSIU_FAIL_MAC_GETSIZES);

  
  /* Allocate the gmac buffers. Logical port number 0 is always used for boot */
  dsiuSetMacBootBuffers (bufs, 0);
  halId =  (*dsiuContext.hal->gmac->halGmacAlloc) (nbufs, bufs, &sizes);

  /* Get mac physical configuration info */
  macInit.operation = HAL_GMAC_INIT_OPERATION_ENABLE;
  dsiuGetMacBootConfig (&macInit, &sizes);
  macInit.halId = halId;
  dsiuContext.diagApp.portNum = macInit.physPort;


  /* Start the mac interface. Logical mac port 0 is always used
   * if mac is enabled in the boot. */
  dport = &(dsiuContext.macPort[0]);
  dport->physPort = macInit.physPort;
  dport->halId    = halId;
  ret = (*dsiuContext.hal->gmac->halGmacInit) (&macInit);
  if (ret != HAL_SUCCESS)
    dsiuPanic (DSIU_FAIL_MAC_INIT);


  /* Wait for configuration information */
  while ( (dsiuBootMacPacketReceived == FALSE)  )  {
    for (i = 0; i < 60000; i++);
    if (hal.halPoll)
      (*hal.halPoll) (timestamp++, 4);
  }

#else
  volatile tuint i;

  /*Initializing PA, QM, CPPI/NWAL*/
    if (dsiuNwInit () == FALSE)  {
        dsiuNwLog ("ERROR: dsiuNwInit FAILED \n");
        /* Block for Error */
        while(1);
        dsiuPanic (DSIU_FAIL_MAC_INIT);
    }
    else
    {
        paEthInfo_t         ethInfo; 
        dsiuPrevBootInfo_t  pInfo;
        nwalMacParam_t      nwalMacParam;         
        nwal_Handle         nwalMacHandle;

        dsiuNwLog ("dsiuNwInit Passed \n");

        /* Configure MAC address for receiving Bootconfig packet */
        if (!dsiuCallTable.prevBootInfo)
            dsiuPanic (DSIU_FAIL_NO_BOOT_INFO);

        /* Get MAC address information from the Chip Bootloader */
        if ((*dsiuCallTable.prevBootInfo)(&pInfo) != DSIU_SUCCESS)
            dsiuPanic (DSIU_FAIL_INVALID_BOOT_INFO);
#ifdef BRINGUP_WORKAROUND
        if(debug_no_rom_boot)
        {
            pInfo.u.macInfo.macAddr[0] = 0x2A2B;
            pInfo.u.macInfo.macAddr[1] = 0x2C2D;
            pInfo.u.macInfo.macAddr[2] = 0x2E2F;
        }
#endif
        memset(&ethInfo,0,sizeof(paEthInfo_t));

        for (i = 0; i < 3; i++)
        {
           ethInfo.dst[2*i] = pInfo.u.macInfo.macAddr[i] >> 8;
           ethInfo.dst[2*i+1] = pInfo.u.macInfo.macAddr[i] & 0xFF;
        }

        memset(&nwalMacParam, 0, sizeof(nwalMacParam_t));
        memcpy((void*)&nwalMacParam.macAddr, &(ethInfo.dst[0]), sizeof(nwalMacAddr_t));
        nwalMacParam.nextRtfail = NWAL_NEXT_ROUTE_FAIL_ACTION_HOST; 

        nwalMacHandle = dsiuAddMac(&nwalMacParam);
        if(nwalMacHandle == NULL)
        {
            /* Block for Error */
            while(1);
        }
    }

    /* Wait for configuration information */
  while ( (dsiuBootMacPacketReceived == FALSE)  )  {
    for (i = 0; i < 60000; i++);
    dsiuNwalPoll();
  }

#endif
  
  return (DSIU_SUCCESS);

} /* dsiuBootGmac */
    




