/*****************************************************************************
 * FILE PURPOSE: Main Diagnositic execution loop
 *****************************************************************************
 * FILE NAME: dsiumain.c
 *
 * DESCRIPTION: Provides the main execution loop for the diagnostic code.
 *               
 * TABS: None
 *
 * Copyright (C) 2002 Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include "types.h"
#include "dsiu.h"
#include "dsiuloc.h"
#include "dsiuutop.h"
#include "dsiutask.h"
#include "diagreg.h"
#include "diagcfg.h"
#include "hal.h"
#include "dsiuport.h"
#include "util.h"
#include <stdlib.h>

#ifdef ITDM
#include "ipiuloc.h" /* This should be removed */
#endif

#include "dsiuotbrun.h"


#ifdef PA_SS
#include "dsiunwal.h"  
#endif


/****************************************************************************
 * Global Variable declaration
 ****************************************************************************/
dsiuContext_t dsiuContext;
dsiuInst_t   *dsiuInst;
tuint         globalTimeStamp = 0 ;
tuint         numActiveChannels = 4;
tuint         shouldIPoll = 0 ;
tuint         sync_error;
tulong        siuMIPSIsrCycles    = 0;
tuint         isrTimerLastTime    = 0;
/*****************************************************************************
 * FUNCTION PURPOSE: Process Tasks that need Polling
 *****************************************************************************
 * DESCRIPTION: Processes Polling tasks. 
 * These are tasks that are done on a polling basis, this is like an Idle Task.
 * Right now we support the Poll Tasks for Utopia Send and Receive.
 *  This function is required in the bootloader mode.
 *****************************************************************************/
void dsiuPollTasks (void)
{
#ifdef PA_SS
    dsiuNwalPoll ();
#else
  if (dsiuContext.hal->halPoll !=NULL)
     (*dsiuContext.hal->halPoll) (globalTimeStamp, numActiveChannels);
#endif

  /* Check for block message processing */
  if (diagCfg.ctable.dsiu->blockMsgProc)
    (*diagCfg.ctable.dsiu->blockMsgProc) ();
   
  /* Check for messages from the utopia */
  if (diagCfg.ctable.siuUtNdu)
    (*diagCfg.ctable.siuUtNdu->nduGetNormalMessage)();

  /* Check for message from the HPI */
  if (diagCfg.ctable.siuHpiNdu)
    (*diagCfg.ctable.siuHpiNdu->nduGetNormalMessage)();

  /* Check for messages from gmac */
  if (diagCfg.ctable.siuGmacNdu)
    (*diagCfg.ctable.siuGmacNdu->nduGetNormalMessage)();

  /* Check and process messages received from emulator */
  if (diagCfg.ctable.siuUtNdu)
    (*diagCfg.ctable.siuUtNdu->nduMSG_test)();
  
    /* Process messages from the emulator - gmac interface */
    if (diagCfg.ctable.siuGmacNdu)
      (*diagCfg.ctable.siuGmacNdu->nduMSG_test)();  

  /* Check Auto-run commands */
  if (diagCfg.ctable.dsiuArun)
    (*diagCfg.ctable.dsiuArun->poll) ();

} /* End of dsiuPollTasks() */

/*****************************************************************************
 * FUNCTION PURPOSE: Increment an unsigned int
 *****************************************************************************
 * DESCRIPTION: This function used to get around a read/modify/write bug
 *              seen on janus. If PC_LOCKUP_WORKAROUND is not defined
 *              this becomes a macro through dsiuloc.h
 *****************************************************************************/
#ifdef PC_LOCKUP_WORK_AROUND
void dsiuInc (tuint *v)
{
  volatile tuint vtl;

  vtl = *v + 1;
  *v = vtl;

}
#endif /* PC_LOCKUP_WORKAROUND */

/*****************************************************************************
 * FUNCTION PURPOSE: UTOPIA driver loop
 *****************************************************************************/
halUtopiaData_t testPkt;
tuint testBuff[64], sndTstPkt = 1, tstPktSize = 54;

void utopTest()
{
  tuint i, *bufPtr[2];
  while(sndTstPkt) {
    /* Fill header */
    testBuff[0] = 0x0010;
    testBuff[1] = 0x4030;
    testBuff[2] = 0x0000;
/*
    testBuff[0] = 0x0000;
    testBuff[1] = 0x1040;
    testBuff[2] = 0x3000;
    */
    for (i=3; i<=63; i++)
      testBuff[i] = 0xFACE;
    
    bufPtr[0] = testBuff;  
    testPkt.nSegs   = 1;
    testPkt.type    = 0;
    testPkt.sizes   = &tstPktSize;
    testPkt.segs    = (tuint **)bufPtr;
  //  tsiuContext.hal->halUtopiaSendCell (0, 0, &testPkt);
  }
}

#ifdef ITDM
extern cregister  volatile unsigned int DNUM;

/* fromTdm -> toNet */
void ItdmRxIsr (void *vinst, itdmMultiTsMap_t *map, tuint **dataBuffer, tuint ts)
{
    ipiuInst_t *inst = (ipiuInst_t *)vinst;
    itdmFifoEntry_t *inpBuffer = inst->inpBuffer;

    if (inst->state == IPIU_STATE_CLOSED)
        return;

    inpBuffer->dataValid = 1;

    diagCfg.ctable.ipiu->ipiuTdmRxExchange((void *)inst,map,(itdmFifoEntry_t **)&inpBuffer,\
               dsiuContext.isiu_isr_clock,1);

    *dataBuffer = (tuint *)(&(inpBuffer->buf[0]));

    inst->inpBuffer = inpBuffer;

}

/* toTdm  <- fromNet */
void ItdmTxIsr (void *vinst, itdmMultiTsMap_t *map, tuint **dataBuffer, tuint ts)
{
    ipiuInst_t *inst = (ipiuInst_t *)vinst;
    itdmFifoEntry_t *outBuffer = inst->outBuffer;
    
    if(inst->state == IPIU_STATE_CLOSED)
        return;
    
    diagCfg.ctable.ipiu->ipiuTdmTxExchange((void *)inst,map,(itdmFifoEntry_t **)&outBuffer,\
               dsiuContext.isiu_isr_clock,1);

    *dataBuffer = (tuint *)(&(outBuffer->buf[0]));

    outBuffer->dataValid = 0;

    inst->outBuffer = outBuffer;

}
#endif

/*****************************************************************************
 * FUNCTION PURPOSE: Main execution loop
 ***************************************************************************** 
 * DESCRIPTION: The main execution loop handles processing of foreground
 *              tasks and tracks configuration changes.
 *  This function is required in the bootloader mode.
 *  The entry_code paramter provides information about the success of the 
 *  boot process from the initializtion code
 *****************************************************************************/
void dsiuMain (tint entry_code)
{
  DSIU_DUMMY_ARG(entry_code);

  (*dsiuContext.hal->halIntrLevel) (HAL_INT_LEVEL_ENABLE);

  
#if defined(DSIU_SIMULATE_APPLICATION)
  dsiu_simu_restart();  /*- mdebug - temporary to stop any messages */
#elif defined(DSIU_DAUGHTER_TEST)
  /* Do nothing in daugther test mode */
#else
 /* Generate a diagnostic boot acknowledgement if configured to do so */
  if (diagCfg.bootAck)
    (*diagCfg.bootAck)(entry_code);
#endif


  /**************************************************************************
   * Bootloader main loop.
   * The polled tasks check the utopia buffer and for posted messages.
   * The diagnostic mode message will change the mode to DSIU_MODE_DIAG.
   **************************************************************************/
  while (dsiuContext.mode == DSIU_MODE_BOOT)  {
    /* Process Polled Tasks */

    shouldIPoll = 1;
    dsiuPollTasks();
  }

  /**************************************************************************
   * When leaving the boot mode the DSP enters the diagnostic mode. This
   * mode is not supported in the bootloader.
   **************************************************************************/
#if (defined(DSIU_SIMULATE_APPLICATION) || defined(DSIU_DAUGHTER_TEST))

#ifdef ITDM
  isiuRun();
#else
  dsiuRun(); 
#endif

#endif

} /* dsiuMain */

