/*********************************************************************************
 * FILE PURPOSE: Perform memory clear and checksum
 *********************************************************************************
 * FILE NAME: dsiumchk.c
 *
 * DESCRIPTION: Performs the memory clear and memory checksum functions on
 *				memory blocks defined in the linker command file.
 *
 *              This file is specific to the c6415 processor
 *
 * TABS: None
 *
 * Copyright (C) 2003, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include "diagcfg.h"
#include "diagreg.h"
#include "dsiuloc.h"
#include "dsiumsg.h"
#include "dsiuport.h"
#include "hal.h"

/************************************************************************************
 * FUNCTION PURPOSE: Initialize memory
 ************************************************************************************
 * DESCRIPTION: Initializes the blocks of memory defined through the linker command
 *              file and diagcfg.
 ************************************************************************************/
tint dsiu_mem_clear (dmsg_hd_sys_mem_chksum_t *ctl)
{
  tint i;
  tint core;

  for (i = 0; i < diagCfg.sysChkNblocks; i++)  {

    /* Clear the block if it is a shared block */
    if (DCFG_SYS_CHKSUM_ATTR_IS_SHARED (diagCfg.sysChkBlockFlags[i]))
      dsiu_clear_block ( (tuint *)(diagCfg.sysChkBlocks[i]),
                          SIU_SIZE_WORDS_TO_TUINT(diagCfg.sysChkBlockSizesWords[i]),
                          ctl->init_val);

    core = diagCfg.sysChkBlockFlags[i];
    if (ctl->bit_core & (1 << core))
      dsiu_clear_block ( (tuint *)diagCfg.sysChkBlocks[i], 
                          SIU_SIZE_WORDS_TO_TUINT(diagCfg.sysChkBlockSizesWords[i]),
                          ctl->init_val);

  }

  return (ctl->mode);

} /* dsiu_mem_clear */


/*******************************************************************************
 * FUNCTION PURPOSE: Checksum the memory
 *******************************************************************************    
 * DESCRIPTION: Performs the checksum over the shared memory and local
 *              memory of selected cores.
 *******************************************************************************/
tint dsiu_mem_chksum (dmsg_hd_sys_mem_chksum_t *ctl, dmsg_dh_sys_chksum_ret_t *ret)
{
  tint i;
  tint core;

  /* Clear the checksum fields in the returned structure */
  ret->chksum_shared = 0;
  for (i = 0; i < DMSG_MAX_CORES; i++)
    ret->chksum_local[i] = 0;


  for (i = 0; i < diagCfg.sysChkNblocks; i++)  {

    /* Compute the checksum over and add to the appropriate block */
    if (DCFG_SYS_CHKSUM_ATTR_IS_SHARED (diagCfg.sysChkBlockFlags[i]))
      ret->chksum_shared ^= dsiu_chksum ( (tuint *)(diagCfg.sysChkBlocks[i]),
                                           SIU_SIZE_WORDS_TO_TUINT(diagCfg.sysChkBlockSizesWords[i]));

    core = diagCfg.sysChkBlockFlags[i];
    if (ctl->bit_core & (1 << core))
      ret->chksum_local[core] ^= dsiu_chksum ( (tuint *)diagCfg.sysChkBlocks[i], 
                                                SIU_SIZE_WORDS_TO_TUINT(diagCfg.sysChkBlockSizesWords[i]));

  }

  return (ctl->mode);

} /* dsiu_mem_chksum */




/********************************************************************************
 * FILE PURUPOSE: Main memory clear/checksum handler
 ********************************************************************************
 * DESCRIPTION: This is the interface function to the system. The input 
 *              structure is used to determine the function to be performed
 *              as well as returning the results.
 *
 *              The return value is the value in the mode field of the input
 *              structure, -1 on error. 
 ********************************************************************************/
tint dsiuMemChecksum (dmsg_hd_sys_mem_chksum_t *ctl, dmsg_dh_sys_chksum_ret_t *ret)
{
  tint retval;
  halIntrLevel_t level;

  /* Because the memory init and checksum use most of the stack memory
   * disable interrupts to prevent further stack depth increases. The
   * only peripheral enabled when this function is mapped is the utopia,
   * and there are no valid cells which can arrive during the memory test,
   * so losing cells during the rather long memory test is not a problem */
  level = (*dsiuContext.hal->halIntrLevel) (HAL_INT_LEVEL_DISABLE);
 
  /* Branch to handler based on mode */
  switch (ctl->mode)  {
    case DMSG_MEM_INIT:
      retval = dsiu_mem_clear (ctl);
      break;

    case DMSG_MEM_CHKSUM:
      retval = dsiu_mem_chksum (ctl, ret);
      break;

    default:
      retval = -1;
      break;
  }

  /* Restore original interrupt level */
  (*dsiuContext.hal->halIntrLevel) (level);
  return (retval);

}










