/*****************************************************************************
 * FILE PURPOSE: Implement system heap memory allocation
 *****************************************************************************
 * FILE NAME: dsiumem.c
 *
 * DESCRIPTION: Contains funcitons used to align and allocate heap memory.
 *
 * TABS: None
 *
 * Copyright (C) 2002, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include "types.h"
#include "dsiumem.h"
#include "dsiutask.h"
#include "dsiufifo.h"
#include "dsiu.h"
#include "dsiuloc.h"
#include "dsiuport.h"

/*****************************************************************************
 * FUNCTION PURPOSE: Find buffer alignment
 *****************************************************************************
 * DESCRIPTION: Returns the 1st address in a buffer that meets the required
 *              log2 buffer requirement. 
 *****************************************************************************/
void *dsiuBufferAlign (void *bstart, void *bend, tint log2a, tulong *rsize, tuint *alignUsed)
{
  tint   i;
  tulong mask = 0;
  tulong ubstart;
  tulong ubend;
  tulong umask;

  ubstart = (tulong)bstart;
  ubend   = (tulong)bend;

  /* Form the alignment bit mask */
  for (i = 0; i < log2a; i++)
    mask = (mask << 1) | 1;

  /* Check buffer alignment */
  umask = ubstart & mask;
  if (!umask)  {
    *rsize     = ubend - ubstart;
    *alignUsed = 0;
  } 
  else  {
    /* Alignment condition not met - find 1st buffer address that is aligned */
    umask   = ubstart & ~mask; 
    umask   = umask + (mask + 1);
    ubstart = umask;
    if (ubend < ubstart)  {
      *rsize = 0;
      ubstart = 0;
    }
    else  {
      *rsize    = ubend - ubstart;
      *alignUsed = ubstart - (tulong)bstart;
    }
  }

  return ((void *)ubstart);
} /* dsiuBufferAlign */

/****************************************************************************
 * FUNCTION PURPOSE: Allocate heap memory
 ****************************************************************************
 * DESCRIPTION: Claims memory from the heap if the requested size is 
 *              available.
 ***************************************************************************/
void *dsiuHeapAlloc (dsiuHeap_t *heap, tulong size)
{
  word *base = (word *) (DIAG_BASE_ADDRESS_MOD(heap->base,heap->used));	  
  /* If size is odd, increase it by one word to keep even alignment. Panic if
   * the base address is odd */
  if (size & 0x1)
    size += 1;

  if (((tulong)heap->base) & 0x1)
    dsiuPanic (DSIU_FAIL_HEAP_ODD_ALIGN);

  heap->used += size;
  if (heap->used >= heap->size)
    return (NULL);

  while ((heap->used & DSIU_MEM_ALIGN_MASK) != 0)  /* align block to a proper boundary */
    heap->used++;
  
  return ((void *)base);
} /* dsiuHeapAlloc */

/*****************************************************************************
 * FUNCTION PURPOSE: Allocate memory buffers from application heap
 *****************************************************************************/
dsiuMemReturn_t dsiuAllocBufs (dsiuHeap_t *heap, tint nbufs, dmemBuffer_t *memb)
{
  tint   i;
  tulong avail;
  tuint  al;
  void  *bstart;
  void  *bend;

  for (i = 0; i < nbufs; i++)  {
    bstart  = (void *)(&(heap->base[heap->used]));
    bend    = (void *)(&(heap->base[heap->size]));
    avail   = heap->size - heap->used;
    dsiuBufferAlign (bstart, bend, memb[i].log2align, &avail, &al);
    heap->used = heap->size - avail;
    memb[i].base =  dsiuHeapAlloc (heap, memb[i].size);
    if (memb[i].base == NULL)
      return (DSIU_MEM_FAIL);
  }

  return (DSIU_MEM_SUCCESS);
} /* dsiuAllocBufs */
  
/******************************************************************************
 * FUNCTION PURPOSE: Allocate a fifo from heap
 ******************************************************************************
 * DESCRIPTION: Allocates a fifo structure and fifo memory from the heap
 ******************************************************************************/
dsiuFifo_t *dsiuFifoAlloc (dsiuHeap_t *heap, tuint size)
{
  dsiuFifo_t *fifo;

  /* Allocate space for the fifo structure */
  fifo = (dsiuFifo_t *)dsiuHeapAlloc (heap, sizeof (dsiuFifo_t));
  if (fifo == NULL)
    return (NULL);

  /* Allocate space for the fifo base memory */
  fifo->base = (tuint *)dsiuHeapAlloc (heap, size * sizeof (tuint));
  if (fifo->base == NULL)
    return (NULL);

  /* Initialize fifo structure */
  fifo->size = size;
  fifo->inIndex = 0;
  fifo->outIndex = 0;

  return (fifo);

} /* dsiuFifoAlloc */

/****************************************************************************
 * FUNCTION PURPOSE: Allocate memory for a system task
 ****************************************************************************
 * DESCRIPTION: Requests memory for a task structure. 
 ****************************************************************************/
dsiuTask_t *dsiuTaskAlloc (dsiuHeap_t *heap, tint depth)
{
  dsiuTask_t *ret;

  /* Allocate memory for the task structure */
  ret = (dsiuTask_t *)dsiuHeapAlloc (heap, sizeof(dsiuTask_t));
  if (ret == NULL)
    return (NULL);

  /* Initialize In and out indices */
  ret->inIndex = ret->outIndex = 0;
  ret->size = depth;

  /* Allocate memory for the task queue */
  ret->tasks = (void **)dsiuHeapAlloc (heap, sizeof (void *) * depth);
  if (ret->tasks == NULL)
    return (NULL);

  return (ret);

} /* dsiuTaskAlloc */
