/******************************************************************************
 * FILE PURPOSE: Diagnostic code host message handler
 ******************************************************************************
 * FILE NAME: dsiumsg.c
 *
 * DESCRIPTION: Contains functions which format messages to be passed to and
 *              from the host.
 *
 * TABS: None
 *
 * Copyright (C) 2002, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include "types.h"
#include "dsiuloc.h"
#include "dsiumsg.h"
#include "diagcfg.h"
#include <stdlib.h>

/*****************************************************************************
 * FUNCTION PURPOSE: Initialize the message route table
 *****************************************************************************
 * DESCRIPTION: Insures that the message table is sorted in assending order,
 *              and returns the number of elements in the route table.
 *****************************************************************************/
tint dsiuInitMsgRouteTable (dsiuMsgTableElement_t *msgTbl)
{
  tint i, n;
  bool done;
  dsiuMsgTableElement_t tmp;

  /* Determine the number of elements in the table */
  n = 0;
  while (msgTbl[n].msgId != 0) n++;

  /* Sort the table in ascending order */
  do  {
    done = TRUE;
    for (i = 0; i < n-1; i++)  {
      if (msgTbl[i].msgId > msgTbl[i+1].msgId)  {
        tmp         = msgTbl[i];
        msgTbl[i]   = msgTbl[i+1];
        msgTbl[i+1] = tmp;
        done        = FALSE;
      }
    }
  } while (!done);

  /* Return the number of elements in the table */
  return (n);
}

/******************************************************************************
 * FUNCTION PURPOSE: Searche the global message table
 ******************************************************************************
 * DESCRIPTION: Searches the global message table for a message associated with
 *              the specified message ID.
 *****************************************************************************/
dsiuMsgTableElement_t *dsiu_msg_table_search (tuint msgId)
{
  dsiuMsgTableElement_t *element;
  tint i    = dsiuContext.msgTable.n / 2;
  tint step = dsiuContext.msgTable.n / 4;
  tint lastCheck = 0;

  /* For 3 or fewer elements in the table ensure at least 1 step */
  if (step == 0)  {
    step = 1;
  }
   
  /* Search until match is found or step reaches 0 */
  for (;;)  {
    element = &dsiuContext.msgTable.table[i];
    if (element->msgId == msgId)
      return (element);

    /* Update next search index */
    if (element->msgId < msgId)
      i += step;
    else 
      i -= step;

    /* Reduce search range by factor of 2. Three searches by 1 indices without 
     * a match indicate failure */
    step = step / 2;
    if (step == 0) {
      step = 1;
      if (lastCheck++ >= 3)
        return (NULL);
    }
  }
} /* dsiu_msg_table_search  */

  

/*****************************************************************************
 * FUNCTION PURPOSE: Process host messages
 *****************************************************************************
 * DESCRIPTION: Process messages received from the host. 
 *              Errors in processing messages are reported back to the host.
 *****************************************************************************/
void dsiuProcessMessage (dmsg_t *msg)
{
  dsiuMsgTableElement_t *msge;

  /* Messages are routed based on message ID. The routing of messages is
   * handled through a routing table. This table is a table of message IDs and
   * function pointers that are searched using a binary search algorithm */
  msge = dsiu_msg_table_search (msg->header.msg_id);
  if (msge == NULL)
    return;   /* Illegal message - should send message to host */

  (*msge->processFcn)(msg);
} /* dsiuProcessmessage */
   





