#ifndef _DSIUMSG_H
#define _DSIUMSG_H
/*****************************************************************************
 * FILE PURPOSE: Diagnostic code messaging definitions
 *****************************************************************************
 * FILE NAME: dsiumsg.h
 *
 * DESCRIPTION: Defines structures used to handle messaging between the host
 *              processor and the DSP
 *
 * TABS: None
 *
 * Copyright (C) 2002, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include "types.h"

/*****************************************************************************
 * DEFINITION: Header format for messages to/from the host.
 *****************************************************************************/
typedef struct {
  tuint echo;     /* Unused                        */
  tuint length;   /* Length of message with header */
  tuint chnum;    /* Channel number                */
  tuint msg_id;   /* Message identifier            */
  tuint trans_id; /* Transaction Identifier        */
} dmsgHeader_t;
#define mgbHeader_t dmsgHeader_t
#define DSIU_MSG_HEADER_SIZE  5   /* All sizes are in 16 bit word units       */

#ifdef MGB_HEADER_SIZE
#if (MGB_HEADER_SIZE != DSIU_MSG_HEADER_SIZE)
#error diagnostic message header size does not match application header size
#endif
#endif

#ifndef MGB_HEADER_SIZE
#define MGB_HEADER_SIZE   DSIU_MSG_HEADER_SIZE  /* application compatibility  */
#endif


#define DSIU_MSG_SUB_HEADER_SIZE 3  /* echo + length + channel are sub-header */

/****************************************************************************
 * DEFINITION: For 54 byte cells (VC demux) the 1st two words of the message
 *             body reconstructed from the aal5 payloads contain dsp core and
 *             channel information. This header is converted to a standard
 *             message header by the utopia message router.
 ***************************************************************************/
typedef struct {
  tuint dsp_id;                /* DSP ID                                    */
  tuint priority_core_channel; /* Message channel, priority fields          */
  tuint payload_first_word;    /* First word of remaining message           */
} dmsgVcDemuxMsgHeader_t;

#define DSIU_MSG_VCDEMUX_CHANNEL_MASK 0x001F

/*****************************************************************************
 * DEFINITION: Host to DSP message bodies.
 *****************************************************************************/
/* Request Memory Read */
typedef struct {
  tuint addr_msw;     /* Read address, most significant word  */
  tuint addr_lsw;     /* Read address, least significant word */
  tuint mem_type;     /* Memory type to read                  */
#define DMSG_MEM_TYPE_DATA    0   /* Data memory       */
#define DMSG_MEM_TYPE_PROG    1   /* Program memory    */
#define DMSG_MEM_TYPE_DMA     2   /* DMA address space */
  tuint length_words_msw; /* Number of 16 bit words to read       */
  tuint length_words_lsw; /* Number of 16 bit words to read       */
  tuint msg_rate;         /* Must be 0 for diag */
} dmsg_hd_req_mem_read_t;
#define DSIU_MSG_REQ_MEM_READ_SIZE  4


/* Request Memory Write */
typedef struct {
  tuint addr_msw;     /* Write address, most significant word                 */
  tuint addr_lsw;     /* Write address, least significant word                */
  tuint mem_type;     /* Memory type - values defined in dmsg_hd_req_mem_read */
  tuint length_words; /* Number of 16 bit words to write                      */
  tuint first_word;   /* The 1st word of the data to write. The remaining     *
                       * words follow in memory sequentially                  */
} dmsg_hd_req_mem_write_t;
#define DSIU_MSG_REQ_MEM_WRITE_SIZE 4  /* Size less write data */

/* Request Memory Copy */
typedef struct {
  tuint source_addr_msw;  /* Source memory address, most significant word  */
  tuint source_addr_lsw;  /* Source memory address, least significant word */
  tuint source_mem_type;  /* Source memory type - defined in dmsg_hd_req_mem_read */
  tuint dest_addr_msw;    /* Destination memory address, most significat word */
  tuint dest_addr_lsw;    /* Destination memory address, least significant word */
  tuint dest_mem_type;    /* Destination memory type                          */
  tuint length_words_lsw; /* Number of 16 bit words to copy       */
  tuint length_words_msw; /* Number of 16 bit words to copy       */  
} dsmg_hd_req_mem_copy_t;
#define DSIU_MSG_REQ_MEM_COPY_SIZE  7  /* Message body size */

/* Request Memory Checksum */
typedef struct {
  tuint addr_msw;     /* Memory address, most significant word   */
  tuint addr_lsw;     /* Memory address, least significant word  */
  tuint mem_type;     /* Memory type - values defined in dmsg_hd_req_mem_read */
  tuint length_words; /* Number of 16 bit words to compute checksum           */
} dmsg_hd_req_mem_checksum_t;
#define DSIU_MSG_REQ_MEM_CHECKSUM_SIZE   4 /* Message body size */

/* Boot Core Control */
typedef struct {
  tuint enable_bitmask; /* Set bits indicate cores to be taken out of reset */
  tuint reset_bitmask;  /* Set bits indicate cores to remain in reset       */
  tuint addr_msw;       /* Boot address, msw if provided */
  tuint addr_lsw;       /* Boot address, lsw if provided */
  bool  sleep;          /* If non-zero this core executes an idle until interrupted 
                           by another core */
  tuint wake_bitmask;   /* Sends an inter-processor interrupt to the cores specified */                          
  tint  host_int;       /*  Send a pulse to the host */
#define DSIU_MSG_BOOT_CORE_CONTROL_HOST_INT_NONE   0  /* do not pulse host */
#define DSIU_MSG_BOOT_CORE_CONTROL_HOST_INT_HOUT   1  /* pulse host through hout */
#define DSIU_MSG_BOOT_CORE_CONTROL_HOST_INT_WD     2  /* pulse host through watchdog */
} dmsg_hd_boot_core_control_t;
#define DSIU_MSG_BOOT_CORE_CONTROL_SIZE     4  /* Message body size for boot    */
#define DSIU_MSG_BOOT_CORE_SLEEP_WAKE_SIZE  6  /* Size including sleep and wake */
#define DSIU_MSG_BOOT_CORE_CONTROL_MIN_SIZE 2  /* Required message body size */

/* Diag Code Control */
typedef struct {
  tuint control_code;
#define DMSG_CODE_CTRL_MEM_TEST_ALL    (1<<0)   /* Run all memory tests          */
#define DMSG_CODE_CTRL_RAMSET_TEST (1<<1)   /* Run cache test      */
#define DMSG_CODE_CTRL_STOP_TEST       (1<<2)   /* Stop running tests            */
#define DMSG_CODE_CTRL_LOCAL           (1<<3)   /* Run local memory tests        */
#define DMSG_CODE_CTRL_MARCH           (1<<4)   /* Run marching 1s/0s test       */
#define DMSG_CODE_CTRL_ADDR            (1<<5)   /* Run the 0x55/0xAA tests       */
#define DMSG_CODE_CTRL_55              (1<<6)   /* Run the 0x55/0xAA tests       */
#define DMSG_CODE_CTRL_PSEUDO          (1<<7)   /* Run the pseudo random test    */
#define DMSG_CODE_CTRL_PROG            (1<<8)   /* Test memory as program        */
#define DMSG_CODE_CTRL_STATUS          (1<<9)   /* Status reporting              */
#define DMSG_CODE_CTRL_END_REPORT      (1<<10)  /* Mem read of status regs       */
#define DMSG_CODE_CTRL_NO_OCORE_LOCAL  (1<<11)  /* Do not test other cores local */

  tuint ddr_size_msw;       /* DDR test size, MSW */
  tuint ddr_size_lsw;       /* DDR test size, LSW */
  
  tuint ddr_start_addr_msw; /* DDR test start address, MSW */
  tuint ddr_start_addr_lsw; /* DDR test start address, LSW */
  
  tuint block_size_msw;     /* DDR block test size, MSW */
  tuint block_size_lsw;     /* DDR block test size, LSW */
  
  tuint ddr_only;           /* If set only the DDR memory test is run */
  
  

} dmsg_hd_diag_code_control_t;

#define DSIU_MSG_DIAG_CODE_CONTROL_SIZE 8   /* Message body size */

struct DIAG_MGB_MDS_AIC_FREQ {
  tuint  sampling_frequency;     /* Sampling frequency for IP Phone */
  /* zero is invalid frequency setting */
#define  MGB_SAMP_FREQ_8KHZ     8
#define  MGB_SAMP_FREQ_16KHZ    16
  tuint  crystal_frequency;	  /* crystal frequency input to AICx */
  /* zero is invalid frequency setting */
#define  MGB_CRSTL_32MHZ        32  /* 32.768 MHz clock */
#define  MGB_CRSTL_24MHZ        24  /* 24.576 MHz clock */
};

/* Memory checksum */
typedef struct {
  tuint mode;
#define DMSG_MEM_INIT 0
#define DMSG_MEM_CHKSUM 1
  tuint bit_core;  /* Bit mapped value of cores to check */
  tuint init_val;  /* Mem init value */
} dmsg_hd_sys_mem_chksum_t;
#define DSIU_MSG_SYS_MEM_CHKSUM_SIZE 3

/* Hardware Config */
typedef struct {
  tuint clk_multplier;              /* unused */
  tuint no_fsx_fsr;                 /* used for tdm configuration */
  tuint DPR_VDB_mapping;            /* unused */
  tuint timer_period;               /* unused */
  tuint wake_up_interrupt_mask;     /* unused */
  tuint companding;                 /* Port Level Companding */
  tuint serial_port_config;         /* unused */
  tuint sync_interrupt_config;      /* unused */
  tuint clkout_config;              /* unused */
  tuint hint_control;               /* unused */
  tuint bdx_delay_control;          /* used for tdm configuration */
  tuint hw_gain_control;            /* unused */
  tuint reserved1;                  /* unused */
  tuint pcktisr_timeslot_offset;    /* unused */
  tuint sampling_frequency;         /* unused */
  tuint serial_port_type;           /* unused */
  tuint serial_port_assignment;     /* unused */
  tuint analog_codec_type;          /* unused */
  tuint reserved2;                  /* unused */
  tuint reserved3;                  /* unused */
  tuint hpi_format;                 /* unused */
  tuint reserved4;                  /* unused */
  tuint reserved5;                  /* unused */
  tuint reserved6;                  /* unused */
  tuint tdm_clock_config;           /* used for tdm configuration */
  tuint keep_alive;
  tuint core_mhz;
  tuint pktif_config;
  tuint msg_burst_size;           /* The maxmium Micro-to-DSP burst size 
                                    * Note: It is used for the reliable message protocol
                                    *       0: Disable the reliable message protocol
                                    */
  struct DIAG_MGB_MDS_AIC_FREQ aic_freq;
} dmsg_hd_hw_config_t;

#define DMSG_PORT_UNUSED    0x000f

#define DMSG_ALAW_COMPANDING   0
#define DMSG_MULAW_COMPANDING  1
#define DMSG_LINEAR_COMPANDING 2

#define DMSG_SAMP_FREQ_8KHZ     0
#define DMSG_SAMP_FREQ_16KHZ    1
#define DMSG_SAMP_FREQ_48KHZ    2


/* Port pin configuration */
typedef struct dmsg_tdm_pin_cfg_s
{
  tuint dataDelay;      /* Time delta between frame sync and first bit of tdm stream, in tdm clock cycles */
  tint  bdxDelay;       /* 1st bit drive delay enabled(1) or disabled(0), only valid for tx */
  tint  idleDrive;      /* What to drive on idle timeslots, only valid for tx */
#define DMSG_DRIVE_0   0
#define DMSG_DRIVE_1   1
#define DMSG_DRIVE_Z   2

  tint  fsyncPolarity;     /* Use 1 to invert the frame sync to the DSP, 0 for no-invert */
  tint  fsyncClkPolarity;  /* Use 1 to invert the frame sync clock to the DSP, 0 for no-invert */
  tint  clkPolarity;       /* Use 1 to invert the data clock polarity to the DSP */
  tint  clkRate;           /* Use 1 for single rate clock, 2 for double rate */
  
  tint  tsPerFrame;        /* Number of timeslots per frame */
  tint  dataRate;          /* Data rate - varies by peripheral */
#define DMSG_DATA_RATE_TSIP_8   0       /* Valid only for TSIP devices */
#define DMSG_DATA_RATE_TSIP_16  1
#define DMSG_DATA_RATE_TSIP_32  2

  tint clkRedundancy;
#define DMSG_CLOCK_REDUNDANT    0
#define DMSG_CLOCK_INDEPENDENT  1

  tint  clkSource;         /* Clock source, not used for independent clocks */
  
  tint  wordSize;          /* Size of word from the port   */

} dmsg_tdm_pin_cfg_t;

/* Run time initialization for the port */
typedef struct dmsg_tdm_port_mode_rt_init_s {

  dmsg_tdm_pin_cfg_t tx;  /* Transmit config */
  dmsg_tdm_pin_cfg_t rx;  /* Receive config  */
  
} dmsg_tdm_port_mode_rt_init_t;

typedef struct dmsg_tdm_port_mode_clk_src_s
{
  tint clkSourceTx;
  tint clkSourceRx;
#define DMSG_CLK_SRC_A             0
#define DMSG_CLK_SRC_B             1
#define DMSG_CLK_SRC_NO_SWITCH   100
#define DMSG_CLK_SRC_SWITCH      101

} dmsg_tdm_port_mode_clk_src_t;


/* TDM Port config */
typedef struct dmsg_tdm_port_config_s {
  tint physPort;        /* Physical port number  */
  tint companding;      /* Port level companding */
  
  tint mode;            /* What operation is to be completed */
#define DMSG_RUN_TIME_INIT_START    0
#define DMSG_RUN_TIME_DISABLE       1
#define DMSG_CLOCK_SWITCH           2

  union   {
    dmsg_tdm_port_mode_rt_init_t rtInit;
    dmsg_tdm_port_mode_clk_src_t clkSrc;
  } u;
  
} dmsg_tdm_port_config_t;
  



/*  Channel configuration 
 *  This message controls the pvp and neu module and the physical port used. 
 *  The pvp parameter is unique to the diagnostic, the neu configuration is 
 *  the same as the full application neu channel config message */
 
typedef struct {

  /* Physical port for output */
  tuint physPort;
#define DMSG_PHYS_PORT_UTOPIA   0
#define DMSG_PHYS_PORT_MAC      1

  /* pvp parameters */
  tuint pvpType;    
#define DMSG_PVP_TYPE_AAL1      1       /* Aal1 encap on transmit and receive       */
#define DMSG_PVP_TYPE_RAW80     2       /* 80 byte raw data on transmit and receive */

  /* neu parameters */
  
  tuint flags1;     /*****************/
  tuint flags2;     /* Flags defined */
  tuint flags3;     /* in neu.h      */
  tuint flags4;     /*****************/
  
  tuint hd_pkt_routing_type;
  
  tuint alt_local_port;
  tuint alt_remote_port;
#define DMSG_IP_ADDR_MAX_NUM_TUINT      8
  tuint alt_remote_addr[DMSG_IP_ADDR_MAX_NUM_TUINT];
  
  tuint report_ctrl;
  tuint report_throttle_time;
  
  tuint pkt2tel_first_protocol;
  tuint tel2pkt_second_protocol;
  tuint pkt2tel_start_offset;
  
  tuint tel2pkt_min_len;
  tint  tel2pkt_ip_offset;
  tint  tel2pkt_udp_offset;
  tint  tel2pkt_802_3_offset;
  
  tint  rsvd1;
  tint  rsvd2;
  tuint tel2pkt_aal5_hdr_len;
  tuint tel2pkt_total_hdr_len;
  tuint hdr_image[40];
  
} dmsg_hd_ch_config_t;

/* Emif Config */

typedef struct {
  tuint valid_params;
#define MGB_SDRAM_DOTEST  (1<<0)
#define MGB_SDRAM_CPU_PLL (1<<1)
#define MGB_SDRAM_CONFIG  (1<<2)
#define MGB_SDRAM_REFRESH (1<<3)
#define MGB_SDRAM_TIMING1 (1<<4)
#define MGB_SDRAM_TIMING2 (1<<5)
#define MGB_SDRAM_DDR_PLL (1<<6)
#define MGB_SDRAM_TIMING3 (1<<7)
#define MGB_SDRAM_PHYC    (1<<8)


  tuint pll;       /* CPU pll. CPU pll may be required for ddr2 access */
  
  tuint configMsw;
  tuint configLsw; /* Bit 22: ddr2 differential dqs enable 
                     * Bit 21: ddr2 termination value 0 = 75 ohm, 1 = 150 ohm 
                     * Bit 20: ddr2 enabled
                     * Bit 18: drive strength: 0 = normal, 1 = weak
                     * Bit 17: ddr enable: 0 = single rate, 1 = double rate
                     * Bit 16: sdram enable
                     * Bit 14: narrow mode: 0 for narry mode
                     * Bits 11:9: CAS latency (valid values are 2 - 6)
                     * Bits 6:4: Internal sdram bank setup
                     * Bit 3: External chip select setup
                     * Bits 2:0: page size */
                       
  tuint refreshMsw;
  tuint refreshLsw; /* Bit 31: Self refresh enable 
                      * Bit 30: mclk stop enable
                      * Bits 15:0: Refresh rate */
 
  tuint timing1Msw;
  tuint timing1Lsw; /* Bits 31:25: load to activate cycle timing 
                      * Bits 24:22: Precharge to activate cycles
                      * Bits 21:19: Active to read or write cycles
                      * Bits 18:16: Write to pre-charge cycles
                      * Bits 15:11: Activate to pre-charge cycles
                      * Bits 10:6:  Active to activate cycles
                      * Bits 5:3:   Active to activate (different bank) cycles
                      * Bits 1:0:   Write to read cycles */
                      
  tuint timing2Msw;
  tuint timing2Lsw;  /* Bits 24:23: EDT enable to write cycles 
                       * Bits 22:16: Self refresh exit to any command but read cycles
                       * Bits 15:8:  Self refresh exti to read command cycles
                       * Bits 7:5:   Read to pre-charge cycles
                       * Bits 4:0:   pado mcke change cycles */
  tuint ddr_pll;     /* DDR pll. */
  tuint timing3Msw;
  tuint timing3Lsw;   /* Bits 24:23: EDT enable to write cycles 
                       * Bits 22:16: Self refresh exit to any command but read cycles
                       * Bits 15:8:  Self refresh exti to read command cycles
                       * Bits 7:5:   Read to pre-charge cycles
                       * Bits 4:0:   pado mcke change cycles */
  tuint phycMsw;
  tuint phycLsw;                     
                       

} emif_sdram_t;
  
typedef struct {
  
  union {
    emif_sdram_t sdram;
  } u;
  
} dmsg_hd_emif_config_t;

/*****************************************************************************
 * DEFINITION: DSP to host messages
 *****************************************************************************/
/* Get Memory Read */
typedef struct {
  tuint addr_msw;     /* Memory address, most significant word  */
  tuint addr_lsw;     /* Memory address, least significant word */
  tuint mem_type;     /* Memory type - values defined in dmsg_hd_req_mem_read */
  tuint length_words; /* Number of 16 bit words read            */
  tuint first_word;   /* The 1st word read                      */ 
} dmsg_dh_get_mem_read_t;
#define DSIU_MSG_GET_MEM_READ_BODY_SIZE 4 /* Size less data */

/* Get Memory Checksum */
typedef struct {
  tuint checksum;     /* The checksum  */
} dmsg_dh_get_mem_checksum_t;
#define DSIU_MSG_GET_MEM_CHECKSUM_BODY_SIZE 1

/* State Ack */
typedef struct {
  tuint state_info;   /* State information                   */
  tuint msg_id;       /* ID of message that triggerd the ack */
  tuint ack_type;     
#define MGB_ACK_TYPE_PROC 0x0001    /* Message processing complete */
#define MGB_ACK_TYPE_DONE 0x0002    /* Message action is complete  */
#define MGB_ACK_TYPE_ACK  0x0004    /* Message is received */

  /* The following parameters are noly valid when acktype including MGB_ACK_TYPE_ACK */
  tuint seq_num;                   /* Sequence number of received message */
  
#define MGB_MSG_PRIORITY_HIGH   0
#define MGB_MSG_PRIORITY_NORMAL 1
#define MGB_MSG_PRIORITY_LOW    2
  
  tuint priority;                  /* Priority of the received message */
} dmsg_dh_state_ack_t;
#define DSIU_MSG_STATE_ACK_SIZE   5  /* Message body size */

/* List of state info ack codes */
#define DSIU_MSG_STATE_ACK_MSG_ACK  0x1000

/* Message Error */
typedef struct {
  tuint ecode;             /* Message Error Code */
  tuint msg_first_word;    /* First word of the message which caused error */
} dmsg_dh_msg_error_t;
#define DSIU_MSG_MSG_ERR_SIZE 1 /* Size less header and returned message */

/* Boot Ack */
typedef struct {
  tuint version;          /* Diagnostic code version */
} dmsg_dh_boot_ack_t;
#define DSIU_MSG_BOOT_ACK_SIZE  1 /* Size less header */

/* Error Indication */
typedef struct {
  tuint error_code;       /* Error Codes - must match mgmicdsp.h    */
} dmsg_dh_error_indication_t;
#define DSIU_MSG_ERROR_INDICATION_SIZE         1  /* Size less header */
#define MGB_ERR_INVLD_HW_CONFIG                0
#define MGB_ERR_CORE_BOOT_FAIL                60
#define MGB_ERR_INVD_MSG_FUNC_NOT_SUPPORTED  192
#define MGB_ERR_INVLD_STAT_REQUEST           197
#define MGB_ERR_TDM_INVALID_TDM_CONFIG       210
#define MGB_ERR_NEU_BUFFERS                  600
#define MGB_ERR_CORE_NEU_PORT_REG            1000
#define MGB_ERR_HAL                          1001
#define MGB_ERR_MEMTEST                      1002

/* System memory checksum return */
typedef struct {
  tuint chksum_shared;                      /* Checksum over shared memory */
#define DMSG_MAX_CORES   8
  tuint chksum_local[DMSG_MAX_CORES];  /* Checksum over local memory */
} dmsg_dh_sys_chksum_ret_t;
#define DSIU_MSG_SYS_CHKSUM_RET_SIZE sizeof(dmsg_dh_sys_chksum_ret_t)

/* Debug Info */
typedef struct {
  tuint trace_data_first_word;   /* First word of debug trace data */
} dmsg_dh_debug_trace_t;


/* Memory test error report */
typedef struct {

  tuint address_msw;      /* The failed memory test address, MSW */
  tuint address_lsw;      /* The failed memory test address, LSW */
  
  tuint expected_msw;     /* The expected memory value, MSW */
  tuint expected_lsw;     /* The expected memory value, LSW */
  
  tuint actual_msw;       /* The actual value, MSW */
  tuint actual_lsw;       /* The actual value, LSW */
  
} mem_test_failure_t;

/* Memory test status */
#define DSIU_MSG_MEM_TEST_STATUS_FAIL_ARRAY_SIZE    4
typedef struct  {

  tuint block_addr_msw;  /* Memory block under test, MSW */
  tuint block_addr_lsw;  /* Memory block under test, LSW */
  
  tuint block_size_msw;  /* Memory block size, MSW */
  tuint block_size_lsw;  /* Memory block size, LSW */
  
  tuint blocks_complete; /* Number of blocks completed */
  
  tuint bytes_complete_msw;  /* Number of bytes in block already tested, MSW */
  tuint bytes_complete_lsw;  /* Number of bytes in block already tested, LSW */
  
  tuint blocks_remaining; /* Number of blocks left to test */
  
  tuint bytes_remaining_msw; /* Number of bytes in block left to test, MSW */
  tuint bytes_remaining_lsw; /* Number of bytes in block left to test, LSW */
  
  tuint fail_count;      /* The number of failures detected so far */

  mem_test_failure_t failures[DSIU_MSG_MEM_TEST_STATUS_FAIL_ARRAY_SIZE];

} dmsg_dh_mem_test_status_t;
#define DSIU_MSG_MEM_TEST_REPLY_SIZE (11 + (DSIU_MSG_MEM_TEST_STATUS_FAIL_ARRAY_SIZE * 6))


/****************************************************************************
 * The following DSP to host messages are used to simulate the application
 * only.
 ****************************************************************************/
typedef struct {
  tuint DSP_capabilities;
  tuint dsp_aer_tail_len;
  tuint dsp_rel;
  tuint dsp_subrel;
  tuint dsp_make_rev;
  tuint dsp_make_subrev;
  tuint dsp_make_subrev1;
  tuint dsp_make_subrev2;
  tuint dsp_proc;
  tuint dsp_platform;
  tuint DSP_capabilities_2;
  tuint DSP_capabilities_3;
  tuint number_of_channels;
  tuint ECU_tail_length;
  tuint number_of_PCM_only_channels;
  tuint DSP_capabilities_4;
  tuint DSP_capabilities_5;
  tuint DSP_capabilities_6;
  tuint FIFO_size;
  tuint reserved20;
  tuint reserved21;
  tuint reserved22;
  tuint reserved23;
  tuint reserved24;
  tuint reserved25;
  tuint reserved26;
  tuint reserved27;
  tuint reserved28;
  tuint reserved29;
  tuint reserved30;
  tuint reserved31;
  tuint reserved32;
  tuint reserved33;
  tuint reserved34;
  tuint reserved35;
  tuint reserved36;
  tuint reserved37;
  tuint reserved38;
  tuint reserved39;
  tuint reserved40;
  tuint reserved41;
  tuint reserved42;
  tuint reserved43;
  tuint reserved44;
  tuint reserved45;
  tuint reserved46;
  tuint reserved47;
  tuint reserved48;
} dmsg_dh_msg_restart_t;
#define DSIU_MSG_RESTART_SIZE (48*sizeof(tuint)) /* Size less header */





/****************************************************************************
 * DEFINITION: Union of message types with message header 
 ****************************************************************************/
typedef struct {
  dmsgHeader_t header;
  union {
    dmsg_hd_req_mem_read_t        req_mem_read;
    dmsg_hd_req_mem_write_t       req_mem_write;
    dsmg_hd_req_mem_copy_t        req_mem_copy;
    dmsg_hd_req_mem_checksum_t    req_mem_checksum;
    dmsg_hd_boot_core_control_t   boot_core_control;
    dmsg_hd_diag_code_control_t   diag_code_control;
    dmsg_hd_hw_config_t           hw_config;
    dmsg_hd_sys_mem_chksum_t      sys_checksum;
    dmsg_dh_get_mem_read_t        get_mem_read;
    dmsg_dh_get_mem_checksum_t    get_mem_checksum;
    dmsg_hd_emif_config_t         emif_config;
    dmsg_dh_state_ack_t           state_ack;
    dmsg_dh_msg_error_t           msg_error;
    dmsg_dh_msg_restart_t         restart;
    dmsg_dh_boot_ack_t            boot_ack;
    dmsg_dh_error_indication_t    error_indication;
    dmsg_dh_sys_chksum_ret_t      sys_checksum_ret;
    dmsg_dh_debug_trace_t         debug_trace;
    dmsg_hd_ch_config_t           ch_config;
    dmsg_tdm_port_config_t        port_config;
    dmsg_dh_mem_test_status_t     mem_test_status;
    tuint                         raw_data[20];   /* Used for gels. */
  } msg_body;
} dmsg_t;
#define mgb_t dmsg_t    /* System compatibility */
  

/****************************************************************************
 * DEFINITION: Message ID values - host to DSP messages 
 ****************************************************************************/
enum {
  DMSG_HD_MSG_ID_STATUS_REQUEST = 8,      /*    8 */
  DMSG_HD_MSG_ID_REQ_MEM_WRITE = 55,      /*   55 */
  DMSG_HD_MSG_ID_REQ_MEM_READ  = 72,      /*   72 */
  DMSG_HD_MSG_ID_REQ_MEM_COPY  = 1043,    /* 1043 */
  DMSG_HD_MSG_ID_REQ_MEM_CHECKSUM,        /* 1044 */
  DMSG_HD_MSG_ID_BOOT_CORE_CONTROL,       /* 1045 */
  DMSG_HD_MSG_ID_DIAG_CODE_CONTROL,       /* 1046 */
  DMSG_HD_MSG_ID_HW_CONFIG = 80,          /*   80 */
  DMSG_HD_MSG_ID_SYS_CHKSUM = 2082,       /* 2082 */
  DMSG_HD_MSG_ID_EMIF_CONFIG = 2102,      /* 2102 */
  DMSG_HD_MSG_ID_TDM_PORT_CFG = 2217,     /* 2217 */
  DMSG_HD_MSG_ID_CH_CONFIG  = 3000       /* 3000 */
};

/*****************************************************************************
 * DEFINITION: Message ID values - DSP to host messages
 *****************************************************************************/
enum {
  DMSG_DH_MSG_ID_READY            = 80,                               /*   80 */
  DMSG_DH_MSG_ID_GET_MEM_READ     = DMSG_HD_MSG_ID_REQ_MEM_READ,      /*   72 */
  DMSG_DH_MSG_ID_STATUS_RESPONSE  = 101,                              /*  101 */
  DMSG_DH_MSG_ID_STATE_ACK        = 117,                              /*  117 */
  DMSG_DH_MSG_ID_MSG_ERROR        = 126,                              /*  126 */
  DMSG_DH_MSG_ID_BOOT_ACK         = 127,                              /*  127 */
  DMSG_DH_MSG_ID_GET_MEM_CHECKSUM = DMSG_HD_MSG_ID_REQ_MEM_CHECKSUM,  /* 1044 */
  DMSG_DH_MSG_ID_RESTART          = 100,                              /*  100 */
  DMSG_DH_MSG_ID_ERROR_INDICATION = 105,                              /*  105 */
  DMSG_DH_MSG_ID_SYS_CHKSUM       = DMSG_HD_MSG_ID_SYS_CHKSUM,        /* 2082 */
  DMSG_DH_MSG_ID_DEBUG_TRACE      = 81,                               /*   81 */
  DMSG_DH_MSG_ID_TDM_PORT_RSP     = DMSG_HD_MSG_ID_TDM_PORT_CFG,      /* 2217 */
  DMSG_DH_MSG_ID_MEM_TEST_STATUS  = 156                               /*  156 */
};

/*****************************************************************************
 * DEFINITION: Message processing table element. Messages are processed based
 *             on message ID value. The following table is used to 
 *             associate message IDs with the processing function.
 ****************************************************************************/
typedef struct {
  tuint msgId;                        /* Message Identifier value */
  void (*processFcn)(dmsg_t *msg);    /* Message Process function */
} dsiuMsgTableElement_t;

/*****************************************************************************
 * Function prototypes for host message processing functions 
 *****************************************************************************/
void dsiuMsgReqMemRead      (dmsg_t *msg);
void dsiuMsgReqMemWrite     (dmsg_t *msg);
void dsiuMsgReqMemCopy      (dmsg_t *msg);
void dsiuMsgReqMemChecksum  (dmsg_t *msg);
void dsiuMsgBootCoreControl (dmsg_t *msg);
void dsiuMsgDiagCodeControl (dmsg_t *msg);
void dsiuMsgHwConfig        (dmsg_t *msg);
void dsiuMsgStatusRequest   (dmsg_t *msg);
void dsiuMsgSysChecksum     (dmsg_t *msg);
void dsiuMsgChConfig        (dmsg_t *msg);
void dsiuMsgTdmPortConfig   (dmsg_t *msg);
void dsiuMsgEmifCfg         (dmsg_t *msg);

/*****************************************************************************
 * Function prototypes for generating messages to host
 *****************************************************************************/
void dsiu_simu_restart (void);
void dsiuErrorIndication (tuint transId, tuint ch, tuint ecode);


/* External declaration for the global message table */
extern dsiuMsgTableElement_t globMsgTableElement[];

/*****************************************************************************
 * DEFINITION: Error codes returned with rejected messages
 *****************************************************************************/
#define DSIU_MSG_ERR_INVALID_MSG_SIZE   1   /* Not enough parameters     */
#define DSIU_MSG_ERR_REQ_SIZE_VIOLATION 2   /* Mem read - too many words */
#define DSIU_MSG_ERR_MEM_TYPE_INVALID   3   /* Mem read/write/copy - invalid mem type */
#define DSIU_MSG_ERR_INVALID_HW_CONFIG  4   /* HAL did not like hw config */
#define DSIU_MSG_ERR_INVALID_ALIGN      5   /* Invalid mem read/write alignment request */
#define DSIU_MSG_ERR_INVALID_CHNUM      6   /* Invalid channel number */
#define DSIU_MSG_ERR_INVALID_IFACE      7   /* Invalid channel interface specified */
#define DSIU_MSG_ERR_NO_FREE_PORTS      8   /* No free tdm port found */
#define DSIU_MSG_ERR_NO_TDM             9   /* Tdm command, no tdm in build */
#define DSIU_MSG_ERR_PORT_REALLOC      10   /* Tried to initialize an already initialized port */





/*****************************************************************************
 * DEFINITION: automated system contrl
 *****************************************************************************/
#define DSIU_AUTOCTL_CMD_CHKSUM_BYPASS  0x2300
#define DSIU_AUTOCTL_CMD_HW_CONFIG      0x2600
#define DSIU_AUTOCTL_CMD_SPIN           0x3b00

#define DSIU_AUTOCTL_STAT_RESTART       0x0100
#define DSIU_AUTOCTL_STAT_READY         0x0200




#endif /* _DSIUMSG_H */

