/*************************************************************************************
 * FILE PURPOSE: System level interface to full application NEU unit for voice channels
 *************************************************************************************
 * FILE NAME: dsiuneuv.c
 *
 * DESCRIPTION: Provides the layer interfacing the diagnostic code to the full
 *				application NEU for diag voice channels.
 *
 * Copyright (C) 2003, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include "types.h"
#include "dsiuloc.h"
#include "dsiuutop.h"
#include "dsiuport.h"
#include <ti/mas/neu/neu.h>
#include <ti/mas/neu/neuproto.h>
#include "diagcfg.h"
#include "memmgm.h"
#include "string.h"
#include "utlrepck.h"
#ifdef PA_SS
#include "dsiunwal.h"
#endif

/***************************************************************************************
 * FUNCTION PURPOSE: Allocates memory for an NEU instance for a voice channel
 ***************************************************************************************
 * DESCRIPTION: Takes heap memory to create a voice channel NEU instance. Currently
 *              only raw voice (no rtp/aal1/aal2 headers) through NEU is supported. 
 ***************************************************************************************/
void *dsiuNeuCreateVoiceNeu (tint ecoId)
{
  void          *inst = NULL;
  neuNewCfg_t    neuCfgNew;
  neuSizeCfg_t   neuCfgSize;

  /* The buffer sizes are stored in diagCfg. */
  if ((diagCfg.voiceAssemblySize == 0) || (diagCfg.voiceSegSize == 0))  {
    dsiuErrorIndication (0, 0, MGB_ERR_NEU_BUFFERS);
    return (NULL);
  }
    
  /* Buffer sizes are specified in diagbld.h */
  neuCfgSize.rxAssemSize   = diagCfg.voiceAssemblySize;
  neuCfgSize.txScratchSize = SIU_BYTES_TO_WORDS(neuproto_MAX_DATA_SIZE_BYTES_POSTAAL5) + diagCfg.voiceSegSize;

  /* DIAG never uses SSSAR */
  neuCfgSize.sssar_required = FALSE;
  /* The max tx header size is configured for to allow switching from aal1 to 
   * voice over udp */
  neuCfgSize.txHeaderSize = SIU_SIZE_TUINT_TO_WORDS(DREG_ENCAP_MAX_HEADER_SIZE_TUINT);

  neuCfgNew.ID      = ecoId;
  neuCfgNew.sizeCfg = neuCfgSize;

  /* Call the full application interface function to setup the ECO. On error
   * inst will point to NULL. */
  siuFullAppEcoNewEco ((dsiuEcoGetSizes_t)neuGetSizes, &neuCfgSize, (dsiuEcoNew_t)neuNew,
                        &neuCfgNew, &inst, ecoId);

  return (inst);

}

/********************************************************************************
 * FUNCTION PURPOSE: Open and enable an neu voice channel
 ********************************************************************************
 * DESCRIPTION: Configures the NEU parameters to enable a voice channel. 
 *              Configuration paramters are from the diagnostic registers.
 ********************************************************************************/
dsiuReturn_t dsiuNeuOpenVoiceNeu (void *vinst)
{
  dsiuInst_t       *inst = (dsiuInst_t *)vinst;
  dregNetCfg_t     *vch;

  neuCfg_t     cfg;
  neuCtrl_t    ctrl;
  tint         count;

  /* Extract the configuration information from the ID */
  vch = &((dsiuContext.dregVchCfgBlk[inst->Id - 1]).vch);
  
  /* Return if the system is not configured */
  if (vch->encap == DREG_NET_ENCAP_NO_VOICE)
    return (DSIU_FAIL);

  /* Provide the function pointer and instance for NEU to make on send out */
  if (vch->route == DIAGAPP_ROUTE_UTOPIA)   {
    if (diagCfg.ctable.siuUtNdu)  {
      cfg.neuSendOut.neuSendOut = diagCfg.ctable.siuUtNdu->nduSendIn;
      cfg.neuSendOut.targetInst = vinst;
      /* bug fix: sar_type is not set in this fcn but used inside called fcn */
      cfg.parm.chnl.sar_type = neuproto_SAR_NONE;

    } else {
      return (DSIU_FAIL);
    }

  } else if (vch->route == DIAGAPP_ROUTE_GMAC)  {
    if (diagCfg.ctable.siuGmacNdu)  {
#ifdef PA_SS
      cfg.neuSendOut.neuSendOut = dsiuPktSendOut;
#else
      cfg.neuSendOut.neuSendOut = diagCfg.ctable.siuGmacNdu->nduSendIn;
#endif
      cfg.neuSendOut.targetInst = vinst;
      /* bug fix: sar_type is not set in this fcn but used inside called fcn */
      cfg.parm.chnl.sar_type = neuproto_SAR_NONE;
    } else {
      return (DSIU_FAIL);
    }
  }

  /* Enable NEU to send out data */
  cfg.parm.chnl.enable = TRUE;

  /* Flags are set to 0 since only the aal 1 header will be used */
  cfg.parm.chnl.cfg_flag1 = vch->cfg_flag1;
  cfg.parm.chnl.cfg_flag2 = vch->cfg_flag2;

#ifdef ITDM
  cfg.parm.chnl.itdmMode = 1;
#else
  cfg.parm.chnl.itdmMode = 0;
#endif

  /* Alternate ports unused */
  cfg.parm.chnl.alt_local_port  = vch->alt_local_port;
  cfg.parm.chnl.alt_remote_port = vch->alt_remote_port;

  count = 0;
  while(count < ifneu_IP_ADDR_MAX_NUM_WORDS){
        cfg.parm.chnl.alt_remote_addr[count] = vch->alt_remote_addr[count];
        count++;
  }  

  /* Disable report control */
  cfg.parm.chnl.report_ctrl          = 0;
  cfg.parm.chnl.report_throttle_time = 0;
  cfg.parm.chnl.sysInst              = NULL;

  /* No second protocol allowed */
  cfg.parm.chnl.second_prototype = vch->second_prototype;

  /* Header parameters */
  cfg.parm.chnl.rx_offset_bytes = vch->rx_offset_bytes;
  cfg.parm.chnl.rx_payload_type = vch->rx_payload_type;

  /* IP and udp */
  cfg.parm.chnl.tx_ip_offset_bytes    = vch->tx_ip_offset_bytes;
  cfg.parm.chnl.tx_udp_offset_bytes   = vch->tx_udp_offset_bytes;
  cfg.parm.chnl.tx_802_3_offset_bytes = vch->tx_802_3_offset_bytes;
  cfg.parm.chnl.tx_min_cell_bytes     = vch->tx_min_cell_bytes;

  /* Disable udp port fixup */
  cfg.parm.chnl.tx_udp_src_mask  = 0;
  cfg.parm.chnl.tx_udp_src_val   = 0;

  /* Copy the header */
  cfg.parm.chnl.tx_total_hdr_len_bytes = vch->tx_total_hdr_len_bytes;
  memcpy ((void *)cfg.parm.chnl.header, (void *)vch->header, SIU_SIZE_TUINT_TO_WORDS(DREG_ENCAP_MAX_HEADER_SIZE_TUINT));
  
  /* Open NEU */
  (*diagCfg.ctable.neu->neuOpen) (inst->neuInst, &cfg);

  /* Re-pass the parameters to enable transmit */
  ctrl.code   = neu_CTRL_CHANNEL_CFG;
  ctrl.u.parm = cfg.parm;
  (*diagCfg.ctable.neu->neuControl) (inst->neuInst, &ctrl);

  /* Setup the receive out pointer */
  ctrl.code = neu_CTRL_RX_RECEIVE_OUT_CFG;
#ifdef ITDM
  ctrl.u.neuReceiveOutSpec.neuReceiveOut = isiuRxItdmNet;
#else 
  ctrl.u.neuReceiveOutSpec.neuReceiveOut = dsiuRxNet;
#endif
  ctrl.u.neuReceiveOutSpec.targetInst = (void *)inst;

  (*diagCfg.ctable.neu->neuControl) (inst->neuInst, &ctrl);


#ifdef PA_SS
      if(!(dsiuConfigL2L3L4(inst->neuInst,
                            &cfg.parm.chnl,
                            &inst->udpHandle,
                            NULL,
                            NULL)))
      {
          return (DSIU_FAIL);
      }
#else
  /* The UDP port must be registered if the mac interface is used and the udp port
   * offset is valid */
  /* Port map is also registered for ETHERNET PDT packets on GMAC interface */ 
  if (vch->route == DIAGAPP_ROUTE_GMAC) 
  {
    /* Register the UDP port, if provided */
    ctrl.code           = neu_CTRL_GET_SRC_UDP_PORT;    
    
    if ((*diagCfg.ctable.neu->neuControl)(inst->neuInst, &ctrl) == neu_NOERR)  {
        
      ctrl.code = neu_CTRL_ADD_PORTMAP;
    ctrl.u.portMap.inst = (void *)inst;

      if ((*diagCfg.ctable.neu->neuControl) (inst->neuInst, &ctrl) != neu_NOERR)  {
        /* Verify that the registration was successful */
      dsiuPanic (DSIU_FAIL_NEU_PORTMAP_FAIL);
  }
    }  
  }
#endif
  return (DSIU_SUCCESS);

} /* dsiuNeuOpenVoiceNeu */

