/************************************************************************
:*                                                                      *
:*     @ Copyright (C) 2009  Texas Instruments Incorporated.            *
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

/**************************************************************************************************
 * FILE PURPOSE: Out of the Box Run 
 **************************************************************************************************
 * FILE NAME: dsiuotbrun.c
 *
 * DESCRIPTION: Setup functions for Out of the Box Run for Diagnostics
 *
 **************************************************************************************************/
#include "types.h"
#include "dsiuloc.h"
#include "diagreg.h"
#include "hal.h"
#include "diagcfg.h"
#include "dsiuotbrun.h"
#include "diagbld.h"
#include "utlrepck_m.h"

#ifdef DIAG_INCLUDE_OTBRUN // comes from diagbld.h
/**************************************************************************************************
 * FUNCTION PURPOSE: Configures Out of the box BERT run
 **************************************************************************************************
 * DESCRIPTION: Configures Out of the box BERT run. Can also be used to send a pattern instead of BERT. 
 *              
 **************************************************************************************************/
void dsiuCfgOtbBert (void)
{

  /*setup OTB bert only if the option is enabled*/
  if(!dsiuSimConfig.opt_diag_otb_BERT)
    return;

  /*select the messages that are spoofed to enable OTB run*/
  dregBlockMsgCntlMem.u.cntrl.firstMsg=0;
  dregBlockMsgCntlMem.u.cntrl.nMsg=DIAG_PREDEF_OTB_BERT_N_MSGS-1;

  /*In order to send only 'nMsg' messages in the bulkmsg processing, need to set nBlockMessages*/
  diagCfg.nBlockMessages=dregBlockMsgCntlMem.u.cntrl.nMsg-1;
  
  /*copy predefined OTB BERT messages into dregMsgBlock[] which is used to store the bulk msgs
    to be spoofed*/
  memcpy (dregMsgBlock, dregMsgOTBBertInit, DIAG_PREDEF_OTB_BERT_N_MSGS*sizeof(dregMsgBlock_t));

}
  
/**************************************************************************************************
 * FUNCTION PURPOSE: BERT setup
 **************************************************************************************************
 * DESCRIPTION: Sets up BERT mode for a given channel 
 *              
 **************************************************************************************************/
void dsiuBertToTdm(tlong chan,tlong  mode)
{
  dregRouterCfg[2*chan].status = DREG_ROUTE_STATUS_ENABLED;
  dregRouterCfg[2*chan].outSrc = mode;
  dregMasterCntrl.routerCntrl.command = DREG_CNTRL_CONFIG;
}

/**************************************************************************************************
 * FUNCTION PURPOSE: Network Loopback   
 **************************************************************************************************
 * DESCRIPTION: Set up a network loopback for a given channel 
 *              
 **************************************************************************************************/
void dsiuSetNetLoopBack(tlong chan)
{
  /* Set the Destination Mac address equal to the source mac address */
  dregVchCfgBlk[chan].vch.header[0] = dregVchCfgBlk[chan].vch.header[3];
  dregVchCfgBlk[chan].vch.header[1] = dregVchCfgBlk[chan].vch.header[4];
  dregVchCfgBlk[chan].vch.header[2] = dregVchCfgBlk[chan].vch.header[5];

  /* IP Headers */
  dregVchCfgBlk[chan].vch.header[15] = dregVchCfgBlk[chan].vch.header[13];
  dregVchCfgBlk[chan].vch.header[16] = dregVchCfgBlk[chan].vch.header[14];

  
  /* Disable UDP checksum- To be Fixed !!!! */
  dregVchCfgBlk[chan].vch.cfg_flag1=dregVchCfgBlk[chan].vch.cfg_flag1 & 0xFFFC;


  dregMasterCntrl.netCntrl.command = DREG_CNTRL_CONFIG;
}

/**************************************************************************************************
 * FUNCTION PURPOSE: TDM NET connection
 **************************************************************************************************
 * DESCRIPTION: Set up a TDM<--->NET connection for a given channel 
 *              
 **************************************************************************************************/
void dsiuTdmNetConnect(tlong chan)
{
  /* Configure the router that routes data from the TDM to the network */
  dregRouterCfg[2*chan].status = DREG_ROUTE_STATUS_ENABLED;
  dregRouterCfg[2*chan].outCh  = chan;
  dregRouterCfg[2*chan].outDir = DREG_ROUTE_OUTDIR_NET;

  /* Configure the router that routes data from the network to the TDM */
  dregRouterCfg[(2*chan)+1].status = DREG_ROUTE_STATUS_ENABLED;
  dregRouterCfg[(2*chan)+1].outCh  = chan;
  dregRouterCfg[(2*chan)+1].outDir = DREG_ROUTE_OUTDIR_TDM;

  dregMasterCntrl.routerCntrl.command = DREG_CNTRL_CONFIG;

}

/**************************************************************************************************
 * FUNCTION PURPOSE: BERT detector setup
 **************************************************************************************************
 * DESCRIPTION: Enables BERT detection from TDM for a given channel 
 *              
 **************************************************************************************************/
void dsiuBertDetectFromTdm(tlong chan)
{
  dregRouterCfg[2*chan].bertDetect = DREG_ROUTE_BERT_DETECT_ON;
  dregMasterCntrl.routerCntrl.command = DREG_CNTRL_CONFIG;
}

/**************************************************************************************************
 * FUNCTION PURPOSE: BERT detector setup
 **************************************************************************************************
 * DESCRIPTION: Enables BERT detection from NET for a given channel  
 *              
 **************************************************************************************************/
void dsiuBertDetectFromNet(tlong chan)
{
  dregRouterCfg[(2*chan)+1].bertDetect = DREG_ROUTE_BERT_DETECT_ON;
  dregMasterCntrl.routerCntrl.command = DREG_CNTRL_CONFIG;
}


/**************************************************************************************************
 * FUNCTION PURPOSE: BERT Test Setup
 **************************************************************************************************
 * DESCRIPTION: BERT Test Setup 
 *              Sets up bert test based on the bert test control register
 **************************************************************************************************/
void dsiuBertTestSetup (void)
{
  tuint i;
  
  for(i=dregBertTestCntl.firstCh;i<dregBertTestCntl.firstCh+dregBertTestCntl.nCh;i++)
  {  
    dsiuBertToTdm(i, dregBertTestCntl.bertMode);
    dsiuSetNetLoopBack(i);
    dsiuTdmNetConnect(i);
    dsiuBertDetectFromTdm(i);
    dsiuBertDetectFromNet(i);
  }
}

/**************************************************************************************************
 * FUNCTION PURPOSE: Initialize Diag App structure
 **************************************************************************************************
 * DESCRIPTION:  Initialize Diag App structure 
 *              
 **************************************************************************************************/
void dsiuInitDiagAppMem(void)
{
  tint              count=0;
  tuint             offset_b;

  diagAppMem.dspId = 1;

  /* Message route for mac - doesn't really matter for emulator start */
  diagAppMem.route   = 2; /* 0:UTOPIA, 2:EMAC, 1:HPI */
  diagAppMem.portNum = 1;

  /* Mac Config - make up the mac info - gig interface */
  diagAppMem.u.mac.configFlags        = 0x400;
  diagAppMem.u.mac.macBufferAssign    = 0;
  diagAppMem.u.mac.msg.flags1         = 0x0983;
  diagAppMem.u.mac.msg.flags2         = 1;
  diagAppMem.u.mac.msg.rxPayloadType  = 3;
  diagAppMem.u.mac.msg.ipOffsetBytes  = 14;
  diagAppMem.u.mac.msg.udpOffsetBytes = 34;
  diagAppMem.u.mac.msg.ethOffsetBytes = -1;
  diagAppMem.u.mac.msg.txMinCellBytes = 64;
  diagAppMem.u.mac.msg.totalHeaderLenBytes = 42;
  diagAppMem.u.mac.msg.rxOffsetBytes  = 0;
  diagAppMem.u.mac.msg.priDiffTag     = 0;

  utlWrite16bits_m((word*)diagAppMem.u.mac.msg.hdrImage,neuproto_ETH_BYTE_OFFSET_DEST_MAC,0x2A2B);/* dest mac msw */
  utlWrite16bits_m((word*)diagAppMem.u.mac.msg.hdrImage,(neuproto_ETH_BYTE_OFFSET_DEST_MAC+2),0x2C2D); /* dest mac mw  */
  utlWrite16bits_m((word*)diagAppMem.u.mac.msg.hdrImage,(neuproto_ETH_BYTE_OFFSET_DEST_MAC+4),0x2E2F);/* dest mac lsw */

  utlWrite16bits_m((word*)diagAppMem.u.mac.msg.hdrImage,neuproto_ETH_BYTE_OFFSET_SRC_MAC,0x2A2B); /* source mac msw */
  utlWrite16bits_m((word*)diagAppMem.u.mac.msg.hdrImage,(neuproto_ETH_BYTE_OFFSET_SRC_MAC+2),0x2C2D); /* source mac mw  */
  utlWrite16bits_m((word*)diagAppMem.u.mac.msg.hdrImage,(neuproto_ETH_BYTE_OFFSET_SRC_MAC+4),0x2E2F); /* source mac lsw */

  utlWrite16bits_m((word*)diagAppMem.u.mac.msg.hdrImage,(neuproto_ETH_BYTE_OFFSET_TYPE_LEN),neuproto_ETH_TYPE_IP);/* ethernet type */

  offset_b = diagAppMem.u.mac.msg.ipOffsetBytes;
  utlWrite16bits_m((word*)diagAppMem.u.mac.msg.hdrImage,(offset_b + neuproto_IPV6_BYTE_OFFSET_VER),0x45f0); /* ip version    */
  offset_b += 2;
  utlWrite16bits_m((word*)diagAppMem.u.mac.msg.hdrImage, offset_b,0x001c);/* ip length     */

  offset_b += 2;
  utlWrite16bits_m((word*)diagAppMem.u.mac.msg.hdrImage, offset_b,0x0048);/* ip ident      */
  
  offset_b += 2;
  utlWrite16bits_m((word*)diagAppMem.u.mac.msg.hdrImage, offset_b,0x0000); /* flags and fragment offset */
 
  offset_b += 2;
  utlWrite16bits_m((word*)diagAppMem.u.mac.msg.hdrImage, offset_b,0x4011); /* ip time to live protocol */

  offset_b += 2;
  utlWrite16bits_m((word*)diagAppMem.u.mac.msg.hdrImage, offset_b,0x36de); /* ip header checksum */

  offset_b += 2;
  utlWrite16bits_m((word*)diagAppMem.u.mac.msg.hdrImage, offset_b,0x6465); /* ip source addr msw  */

  offset_b += 2;
  utlWrite16bits_m((word*)diagAppMem.u.mac.msg.hdrImage, offset_b,0x6667); /* ip source addr lsw  */

  offset_b += 2;
  utlWrite16bits_m((word*)diagAppMem.u.mac.msg.hdrImage, offset_b,0x6465); /* ip dest addr msw */

  offset_b += 2;
  utlWrite16bits_m((word*)diagAppMem.u.mac.msg.hdrImage, offset_b,0x6667); /* ip dest addr lsw */

  offset_b += 2;
  utlWrite16bits_m((word*)diagAppMem.u.mac.msg.hdrImage, offset_b,0x0200); /* udp source port  */

  offset_b += 2;
  utlWrite16bits_m((word*)diagAppMem.u.mac.msg.hdrImage, offset_b,0x0200); /* udp dest port    */

  offset_b += 2;
  utlWrite16bits_m((word*)diagAppMem.u.mac.msg.hdrImage, offset_b,0x0008); /* udp length       */

  offset_b += 2;
  utlWrite16bits_m((word*)diagAppMem.u.mac.msg.hdrImage, offset_b,0); /* udp checksum	   */

  for(count=0;count < CPU_NCORES;count++)
  {
      diagAppMem.u.mac.macAddr[count].macAddr[0] = 0x2a2b;
      diagAppMem.u.mac.macAddr[count].macAddr[1] = 0x2c2d;
      diagAppMem.u.mac.macAddr[count].macAddr[2] = (0x2e2f + count);
      
      diagAppMem.u.mac.ipAddr[count].srcIpHigh = 0x6465;
      diagAppMem.u.mac.ipAddr[count].srcIpLow  = (0x6667 + count);
  }
}

#endif /* DIAG_INCLUDE_OTBRUN */

/* Nothing past this point */

