/*
 *
 * Copyright (C) 2010 Texas Instruments Incorporated - http://www.ti.com/ 
 * 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/



/*************************************************************************************
 * FILE PURPOSE: Ramset memory test
 *************************************************************************************
 * FILE NAME: dsiurmtst.c
 *
 * DESCRIPTION: Performs a simple test of ramset memories
 **************************************************************************************/
#include "diagcfg.h"
#include "diagreg.h"
#include "dsiuloc.h"
#include "hal.h"

tulong dsiuRunCacheTest (tulong (*ctest)());
tulong test_00FF (void);
tulong test_FF00 (void);
tulong test_AA55 (void);
tulong test_55AA (void);
tulong test_00FF_offset (void);
tulong test_FF00_offset (void);
tulong test_AA55_offset (void);
tulong test_55AA_offset (void);

tulong test_00FF_expect (void);
tulong test_FF00_expect (void);
tulong test_AA55_expect (void);
tulong test_55AA_expect (void);
tulong test_FF00_offset_expect (void);
tulong test_00FF_offset_expect (void);
tulong test_AA55_offset_expect (void);
tulong test_55AA_offset_expect (void);

void r0_place_fnc (void);


/* The number of tests is processor specific, and thus local to this file */
#define DSIU_RMTEST_NUM_TESTS 8


/***********************************************************************************
 * FUNCTION PURPOSE: Provide a delay while the RAMSET loads
 ***********************************************************************************/
void dsiu_rdelay(void)
{
  tint i;

  /* Wait a few cycles for the ramsets to load */
  for (i = 0; i < 1000; i++)  {
    asm ("  nop");
  }
} /* dsiu_rdelay */

/*************************************************************************************
 * FUNCTION PURPOSE: Test the cache and RAMSET
 *************************************************************************************
 * DESCRIPTION: Runs the memory test in various cache configurations
 *************************************************************************************/
tint dsiuTestRamset (void)
{
  halMemConfig_t halMemCfg;
  tulong         expected[DSIU_RMTEST_NUM_TESTS];
  tulong         (*rFnc[DSIU_RMTEST_NUM_TESTS])(void);
  tint           i;


  /* Clear the code status flag for the ramset test */
  dsiuContext.dregStatus->cacheTest = DREG_STATUS_CACHE_TEST_NOT_RUN;


  /* First run the test with all cache disabled */
  halMemCfg.memCfgFunction     = HAL_MEMCFGFUNC_CACHECONFIG;
  halMemCfg.u.cacheCfg.cacheMode = (tuint)HAL_CACHE_DISABLE;
  if (*dsiuContext.hal->halMemConfig)  {
    (*dsiuContext.hal->halMemConfig) (&halMemCfg);
  }

  /* Initialize the function arrays */
  expected[0] = test_00FF_expect();
  expected[1] = test_FF00_expect();
  expected[2] = test_AA55_expect();
  expected[3] = test_55AA_expect();
  expected[4] = test_00FF_offset_expect();
  expected[5] = test_FF00_offset_expect();
  expected[6] = test_AA55_offset_expect();
  expected[7] = test_55AA_offset_expect();

  rFnc[0] = test_00FF;
  rFnc[1] = test_FF00;
  rFnc[2] = test_AA55;
  rFnc[3] = test_55AA;
  rFnc[4] = test_00FF_offset;
  rFnc[5] = test_FF00_offset;
  rFnc[6] = test_AA55_offset;
  rFnc[7] = test_55AA_offset;


  /* Test with cache disabled */
  for (i = 0; i < DSIU_RMTEST_NUM_TESTS; i++)  {
    if (dsiuRunCacheTest(rFnc[i]) != expected[i])  {
      dsiuContext.dregStatus->cacheTest = DREG_STATUS_CACHE_TEST_FAIL;
      return (-1);  
    }
  }

  /* Run each of the tests after clearing the 2 way set associative */
  for (i = 0; i < DSIU_RMTEST_NUM_TESTS; i++)  {
    halMemCfg.memCfgFunction      = HAL_MEMCFGFUNC_CACHECONFIG;
    halMemCfg.u.cacheCfg.cacheMode  = HAL_CACHE_MODE2WAY;
    halMemCfg.u.cacheCfg.numRamsets = 0;
    if (*dsiuContext.hal->halMemConfig)  {
      (*dsiuContext.hal->halMemConfig) (&halMemCfg);
    }
    if (dsiuRunCacheTest(rFnc[i]) != expected[i])  {
      dsiuContext.dregStatus->cacheTest = DREG_STATUS_CACHE_TEST_FAIL;
      return (-1); 
    }
  }

  /* Run each of the tests in the first RAMSET */
  for (i = 0; i < DSIU_RMTEST_NUM_TESTS; i++)  {
    halMemCfg.memCfgFunction      = HAL_MEMCFGFUNC_CACHECONFIG;
    halMemCfg.u.cacheCfg.cacheMode  = HAL_CACHE_MODE2WAY;
    halMemCfg.u.cacheCfg.numRamsets = 1;
    halMemCfg.u.cacheCfg.ramsetTags[0] = (halCacheTag_t)rFnc[i];
    if (*dsiuContext.hal->halMemConfig)  {
      (*dsiuContext.hal->halMemConfig) (&halMemCfg);
    }
    dsiu_rdelay(); /* Wait a bit for RAMSET to load */
    if (dsiuRunCacheTest(rFnc[i]) != expected[i])  {
      dsiuContext.dregStatus->cacheTest = DREG_STATUS_CACHE_TEST_FAIL;
      return (-1); 
    }

  }


  /* Run each of the tests in the second RAMSET */
  for (i = 0; i < DSIU_RMTEST_NUM_TESTS; i++)  {
    halMemCfg.memCfgFunction      = HAL_MEMCFGFUNC_CACHECONFIG;
    halMemCfg.u.cacheCfg.cacheMode  = HAL_CACHE_MODE2WAY;
    halMemCfg.u.cacheCfg.numRamsets = 2;
    halMemCfg.u.cacheCfg.ramsetTags[0] = (halCacheTag_t)r0_place_fnc;
    halMemCfg.u.cacheCfg.ramsetTags[1] = (halCacheTag_t)rFnc[i];
    if (*dsiuContext.hal->halMemConfig)  {
      (*dsiuContext.hal->halMemConfig) (&halMemCfg);
    }
    dsiu_rdelay(); /* Wait a bit for RAMSET to load */
    if (dsiuRunCacheTest(rFnc[i]) != expected[i])  {
      dsiuContext.dregStatus->cacheTest = DREG_STATUS_CACHE_TEST_FAIL;
      return (-1); 
    }
  }
    
  /* Indicate the test was completed successfully */
  dsiuContext.dregStatus->cacheTest = DREG_STATUS_CACHE_TEST_SUCCESS;

  return (0);

} /* dsiuTestRamset */






