/*****************************************************************************
 * FILE PURPOSE: Main Diagnositic execution loop
 *****************************************************************************
 * FILE NAME: dsiumain.c
 *
 * DESCRIPTION: Provides the main execution loop for the diagnostic code.
 *               
 * TABS: None
 *
 * Copyright (C) 2002 Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include "types.h"
#include "dsiu.h"
#include "dsiuloc.h"
#include "dsiuutop.h"
#include "dsiutask.h"
#include "diagreg.h"
#include "diagcfg.h"
#include "dpiu.h"
#include "dtgu.h"
#include "dbert.h"
#include "dpvp.h"
#include <ti/mas/neu/neu.h>
#include <ti/mas/neu/neuproto.h>
#include "hal.h"
#include "dsiuport.h"
#include "diagappif.h"
#include "utlrepck.h"
#include "dmiscport.h"
#include "utlrepck.h"
#include <stdlib.h>
#include <string.h>
#include "dsiuotbrun.h"
#include "diagbld.h"

/****************************************************************************
 * Global Variable declaration
 ****************************************************************************/
extern tuint shouldIPoll;

/*****************************************************************************
 * FUNCTION PURPOSE: Dummy function
 *****************************************************************************
 * DESCRIPTION: This function is called periodically at the TDM0 sub frame
 *              rate. 
 *  This function is only required in diagnostic mode.
 *****************************************************************************/
void dsiuSubRateFcn (void *cxt, tuint ts)
{

  dsiuPort_t       *port = (dsiuPort_t *)cxt;

  DSIU_DUMMY_ARG((void *)ts);
  dsiuInc (&port->status->intCount);
  port->status->rxDataCount = ts;
  port->status->txDataCount = ts;

#if 0
  halTimerControl_t tCtl;
  tulong            delta;
  dsiuPort_t       *port = (dsiuPort_t *)cxt;


  /* Measure the period between entries into this function */
  if (dsiuContext.eiClock.validData == FALSE)  {

    /* Read the timer */
    tCtl.operation = HAL_TIMER_GET_COUNT;
    (*dsiuContext.hal->halTimerControl) (0, &tCtl);

    /* If this is the first read store the value. lastDelta is
     * used to flag that */
    if (dsiuContext.eiClock.lastDelta == 0) {
      dsiuContext.eiClock.lastDelta = 1;
      dsiuContext.eiClock.lastEntryTick = tCtl.timerperiod;

    } else {

      /* Get the time delta between the two readings, in cpu cycles,
       * and convert the the units for display in the status register */
      tCtl.operation = HAL_TIMER_DELTA;
      tCtl.tickrate = dsiuContext.eiClock.lastEntryTick;
   /* tCtl.periodCount has already been set in the timer get count call above */
      (*dsiuContext.hal->halTimerControl) (0, &tCtl);


      delta = tCtl.timerperiod;
      delta = delta / diagCfg.reportDivisor;
      delta = delta / diagCfg.subFrameRate0;

      dsiuContext.dreg->clkRatio    = (tuint)delta;
      dsiuContext.eiClock.validData = TRUE;
    }

  }
#endif
   

} /* dsiuSubRateFcn */

/*****************************************************************************
 * FUNCTION PURPOSE: Send Data to the network
 *****************************************************************************
 * DESCRIPTION: TDM Data is to be sent to the network.
 *  This function is only required in the diagnostic mode.
 *****************************************************************************/
void dsiuSendDataToNet (dsiuInst_t *inst, globData_t *data)
{
  xferPktInfo_t  pktInfo;         /* Utopia NDU data structure     */
  tint           dataSizeBytes;   

  /**************************************************************************
   * Send the data to dpvp. If the function returns with a non-zero data
   * length there are packets available for transmission
   *
   *  !!!!!!!
   *  The data sent to dpvp is sized in units of samples (tint),
   *  but returns in units of bytes.
   *  !!!!!!!
   *
   **************************************************************************/
  (*diagCfg.ctable.dpvp->dpvpSendIn)(inst->dpvpInst, data);
  

  while (data->dataSize)  {
    /*********************************************************************
     * Handle network encapsulation.
     *********************************************************************/
    if (diagCfg.ctable.neu)  {
      dataSizeBytes   = data->dataSize;
      pktInfo.type    = XFER_PKT_TYPE_NULL;
      pktInfo.npkts   = 1;
      pktInfo.pktSize = &dataSizeBytes;
      pktInfo.pktIn   = &data->data;
      (*diagCfg.ctable.neu->neuSendIn)(inst->neuInst, (xferpktInfo_t *)&pktInfo);
    }

    /*************************************************************************
     * Query DPVP to see if more packets are ready for sendout
     *************************************************************************/
    data->dataSize = 0;
    (*diagCfg.ctable.dpvp->dpvpSendIn) (inst->dpvpInst, data);
  } /* (data->dataSize) */

} /* dsiuSendDataToNet */


/*****************************************************************************
 * FUNCTION PURPOSE: Process tx TDM data
 *****************************************************************************
 * DESCRIPTION: Processes data received on the TDM port. The data can
 *              be dropped or loopback back to a second channel.
 *  This function is only required in the diagnostic mode.
 *****************************************************************************/
void dsiuProcessTxPcm (dsiuInst_t *inst)
{
  globData_t         data;
  dsiuInst_t        *destInst;
  dregRouterCfg_t   *rtr;
/*  dbertPatternLock_t dbertLock;
  tuint              dbertCount;
  */
  /* Get the data from the PCM port */
  if ((*diagCfg.ctable.dpiu->dpiuTx)(inst->dpiuInst, &data) != DPIU_SUCCESS)  {
    inst->errs->pcmTxTask += 1;
    return;
  }

  /*************************************************************
   * Get the associated router, exit if disabled.
   *************************************************************/
  rtr = &(dsiuContext.dregRouterCfgShadow[DREG_ROUTER_TDM(inst->Id-1)]);
  if (rtr->status == DREG_ROUTE_STATUS_DISABLED)
    return;


  /*************************************************************
   * Do BERT detection if enabled. This must be done
   * before any data replacement.
   *************************************************************/

  if (diagCfg.ctable.bert)  {
    if ( rtr->bertDetect == DREG_ROUTE_BERT_DETECT_ON  )  {
      (*diagCfg.ctable.bert->bertPatternDetectTelFcn)(inst->bertInst, data.data, inst->tdmFrameSize);
    }
  }

  /************************************************************
   * The TDM input can be replaced by tone
   ************************************************************/
  if (rtr->outSrc == DREG_ROUTE_OUT_SRC_TONE)  {
    if (diagCfg.ctable.dtgu)
      (*diagCfg.ctable.dtgu->dtguGenTone) (inst->dtguInst, &data);
    else
      dsiuInc (&(inst->errs->pcmInvalidTxTone));
  }

  /*************************************************************
   * The TDM input canbe replaced by BERT
   *************************************************************/
  if ( (rtr->outSrc == DREG_ROUTE_OUT_SRC_BERT_RANDOM)  ||
       (rtr->outSrc == DREG_ROUTE_OUT_SRC_BERT_255)     ||
       (rtr->outSrc == DREG_ROUTE_OUT_SRC_BERT_RND_ID)  )  {
  
    if (diagCfg.ctable.bert)  {
       (*diagCfg.ctable.bert->bertPatternGenTelFcn) (inst->bertInst, 
                                                      inst->tdmFrameSize, data.data);
    }  
    else
      dsiuInc (&(inst->errs->pcmInvalidTxBert)); 
    }


  /************************************************************
   * Route the data
   ************************************************************/
  if (rtr->outCh >= diagCfg.maxChannels)  {
    dsiuInc (&(inst->errs->invalidRouteCh));
    return;
  }
  destInst = &(dsiuInst[rtr->outCh]);

  if (rtr->outDir == DREG_ROUTE_OUTDIR_TDM)  {
    if (diagCfg.ctable.dvpu)  {
      (*diagCfg.ctable.dvpu->dvpuReceiveIn)(destInst->dvpuInst, &data);
    }
  
  } else if (rtr->outDir == DREG_ROUTE_OUTDIR_NET)  {
    dsiuSendDataToNet (destInst, &data);
  }


} /* dsiuProcessTxPcm  */

/******************************************************************************
 * FUNCTION PURPOSE: Replace input data with silence
 ******************************************************************************
 * DESCRIPTION: Replaces data with silence
 *   This function is only required in the diagnostic mode.
 ******************************************************************************/
void dsiuGenerateSilence (dsiuInst_t *inst, globData_t *data)
{
  utlLsMemSet (data->data, inst->silencePattern, data->dataSize);
}
         
/***************************************************************************
 * FUNCTION PURPOSE: Process Rx data (to TDM)
 ***************************************************************************
 * DESCRIPTION: The TDM unit has requested data. If data is available in
 *              the channels rx buffer, use it, otherwise generate
 *              silence.
 *  This function is only required in the diagnostic mode.
 ***************************************************************************/
void dsiuProcessRxPcm (dsiuInst_t *inst)
{
  globData_t   data;
  dvpuReturn_t vpur;

  /* If there is no vpu there is no way to get any data */
  if (!diagCfg.ctable.dvpu)
    return;

  /* Request data from the vpu */
  data.dataSize = inst->tdmFrameSize;
  vpur = (*diagCfg.ctable.dvpu->dvpuReceiveOut)(inst->dvpuInst, &data);

  /* If not data was available, send silence */
  if (vpur == DVPU_ERR_NO_DATA)  {
    data.dataSize = inst->tdmFrameSize;
    data.data     = dsiuContext.scratch;
    data.ts       = 0;
    dsiuGenerateSilence (inst, &data);
  }

  /* Send the data to the TDM */
  if ((*diagCfg.ctable.dpiu->dpiuRx)(inst->dpiuInst, &data) != DPIU_SUCCESS) 
    inst->errs->pcmRxTask += 1;

} /* dsiuProcessRxPcm */  

/***************************************************************************
 * FUNCTION PURPOSE: Load data into a fifo, add a task to a queue
 ***************************************************************************
 * DESCRIPTION: This function is called from an isr context. It contains
 *              a pointer to one or more received packets or cells which contain
 *              voice data. The cell/packet is placed into the rx net fifo
 *              and a task is posted. 
 *
 *              The Rx net fifo is structured as follows: The 1st element
 *              is a single word that contains the size of the remaining
 *              elements in the block. 
 *  This function is only required in the diagnostic mode.
 ***************************************************************************/
tint dsiuRxNet (void *inst, xferPktInfo_t *data)
{
  tuint      *buff_ptr;
  tuint       sizeV;
  bool        anErr = FALSE;
  globData_t  gdata, sdata;
  dsiuTaskReturn_t tret;

  /* Size information pushed into the fifo */
  /* The input size is in bytes! */
  sizeV          = SIU_BYTES_TO_TUINT (*(data->pktSize));  
  sdata.data     = &sizeV;             /* Pointer to number of elements */
  sdata.dataSize = 1;                  /* Size of size pointer          */

  /* Network data */
  gdata.data     = *(data->pktIn);
  gdata.dataSize = SIU_BYTES_TO_TUINT(*(data->pktSize));

  /* Make sure that there is room to post the task and the data. For the
   * receive network data these two queues must remain in sync */
  if (dsiuFifoSizeAvail (dsiuContext.netFifo) >= (gdata.dataSize+1))  {
    if (dsiuTaskSizeAvail (dsiuContext.rxNetTasks))  {
      /* Push the element word size into the fifo */
      buff_ptr = dsiuFifoIn (dsiuContext.netFifo, &sdata);
      if (buff_ptr == NULL)
        anErr = TRUE; /* Unexplained error */

      /* Push the data */
      buff_ptr = dsiuFifoIn (dsiuContext.netFifo, &gdata);
      if (buff_ptr == NULL)
        anErr = TRUE;  /* Unexplained error - flag for resync */
      
      /* Post the task */
      tret = dsiuPostTask (dsiuContext.rxNetTasks, inst);
      if (tret == DSIU_TASK_FAIL)
        anErr = TRUE; /* Unexplained error */
    }
    else  { /* There was fifo space, but the task queue was full */
      dsiuContext.errs.rxNetTaskQueueFull += 1;
    }
  }
  else { /* There was not room in the fifo */
    dsiuContext.errs.rxNetFifoFull += 1;
  }

  /***************************************************************************
   * The fifo and task queues both indicated there was space, but an 
   * unkown error occured. Clear both the task queue and the packet fifo.
   * Interrupts should be currently disabled - this may change when using
   * BIOS. 
   ***************************************************************************/
  if (anErr)  {
    dsiuFifoReset (dsiuContext.netFifo);
    dsiuTaskClear (dsiuContext.rxNetTasks);
    dsiuContext.errs.rxNetFifoResync += 1;
  }

  /* Return 0. The symbolic returns are not used since this function interfaces
   * to the full application neu. */
  return (0);
    
} /* dsiuRxNet */

/***************************************************************************
 * FUNCTION PURPOSE: Process Rx data (received from net)
 ***************************************************************************
 * DESCRIPTION: Data received from the net can be dropped, looped back
 *              to the network, or passed to the TDM
 * This function is only required in the diagnostic mode.
 ****************************************************************************/
void dsiuProcessRxNet (dsiuInst_t *inst)
{
  globData_t        data, sdata;     /* Packet data, packet size data */
  tuint             sizeV;           /* Packet Size                   */
  bool              anErr = FALSE;   /* Set true on fifo error        */
  halIntrLevel_t    iLevel;          /* Saved interrupt level         */
  xferPktInfo_t     pktInfo;         /* Utopia NDU data structure     */
  tint              dataSizeBytes;   
  dregRouterCfg_t  *rtr;
  dsiuInst_t       *destInst;        /* Loopback dsiu instance        */
  dpvpReturn_t      pvpr;            /* Return value from dpvp        */
  dregNetCfg_t     *sourceCfg;
  dregNetCfg_t     *destCfg;
  tuint             pktData[DSIU_MAX_PKT_SIZE_WORDS];
  tint              i, j, k;
  tuint             v, w;
  tuint             bertData[(2*DSIU_MAX_PKT_SIZE_WORDS)+1];

  /**************************************************************************
   * Packets/cells are currently in the netFifo. Network headers have already
   * been removed. Data is extracted from the cell/packet, and routed
   * through the system. 
   **************************************************************************/
  /* The 1st element in the rx network fifo is the size of the packet to be
   * extracted */
  sdata.dataSize = 1;        /* Extract 1 element        */
  sdata.data     = &sizeV;   /* Where to put the element */
  if (dsiuFifoOut (dsiuContext.netFifo, &sdata))  {
    /* Check that the size is within range */
    if ((sizeV > 0) && (sizeV <= SIU_BYTES_TO_WORDS(DSIU_MAX_PKT_SIZE_BYTES)))  {
      data.dataSize = sizeV;
      data.data     = pktData;
      if (!dsiuFifoOut(dsiuContext.netFifo, &data))
        anErr = TRUE;
    }
    else  /* (sizeV<0) */
      anErr = TRUE;
  }
  else  /* (dsiuFifoOut) */
   anErr = TRUE;

  /**************************************************************************
   * If an error was detected reading the size and packet from the queue
   * reset the queue and the task. Interrupts must be disabled during 
   * reset since elements are placed in the queue from an interrupt context.
   **************************************************************************/
  if (anErr)  {
    if((*dsiuContext.hal->halIntrLevel) != NULL)
    {
      iLevel = HAL_INT_LEVEL_DISABLE;
      iLevel = (*dsiuContext.hal->halIntrLevel)(iLevel);
      dsiuFifoReset (dsiuContext.netFifo);
      dsiuTaskClear (dsiuContext.rxNetTasks);
      dsiuContext.errs.rxNetFifoResync += 1;
      (*dsiuContext.hal->halIntrLevel)(iLevel);
    }
  } 

  /***************************************************************************
   * Route the data
   ***************************************************************************/
  rtr = &(dsiuContext.dregRouterCfgShadow[DREG_ROUTER_NET(inst->Id-1)]);

  /************************************************************
   * Drop data if the router is disabled
   ************************************************************/
  if (rtr->status == DREG_ROUTE_STATUS_DISABLED)
    return;

  /**************************************************************************
   * Routing to net is handled as a special case. Although a true packet
   * loopback is not supported (switch source/dest phys/mac/ip) the
   * data can be looped back without sending it to pvp first. The payload
   * will be looped back but the headers will be those configured
   * for the out bound channel.
   *
   * If routing from non-similar types of network encapsulations (raw to aal1)
   * then the data is passed back to the network interface through the standard
   * data callout.
   ***************************************************************************/
  if ((rtr->outSrc == DREG_ROUTE_OUT_SRC_INMUX) && 
     (rtr->outDir == DREG_ROUTE_OUTDIR_NET)) {
    if (rtr->outCh >= diagCfg.maxChannels)  {
      dsiuInc (&(inst->errs->invalidRouteCh));
      return;
    }
    destInst  = &dsiuInst[rtr->outCh];
    sourceCfg = &(dsiuContext.dregVchCfgShadow[inst->Id - 1]);
    destCfg   = &(dsiuContext.dregVchCfgShadow[rtr->outCh]);

    if (sourceCfg->encap == destCfg->encap)  {
      if (sourceCfg->encap == DREG_NET_ENCAP_AAL1)
        pktInfo.type = XFER_PKT_TYPE_AAL1;

      else  /* Type RAW or NONE */
        pktInfo.type = XFER_PKT_TYPE_NULL;

      /*********************************************************************
       * Handle network encapsulation.
       *********************************************************************/
      if (diagCfg.ctable.neu)  {

        dataSizeBytes   = SIU_TUINT_TO_BYTES(data.dataSize);
        pktInfo.npkts   = 1;
        pktInfo.pktSize = &dataSizeBytes;
        pktInfo.pktIn   = &data.data;
        (*diagCfg.ctable.neu->neuSendIn)(destInst->neuInst, (xferpktInfo_t *)&pktInfo);
      }
      
      return;
    }

  }

   /**************************************************************************
    * Extract the data from the packet 
    **************************************************************************/
   if (inst->dpvpInst)  {
     pvpr = (*diagCfg.ctable.dpvp->dpvpReceiveIn)(inst->dpvpInst, &data);
     if ((pvpr != DPVP_SUCCESS) || (data.dataSize == 0))  {
       dsiuInc (&(inst->errs->rxPvpFail));
       return;
     }
   }
   else { /* No pvp instance for this channel, simply return */
     dsiuInc (&(inst->errs->rxNoPvp));
     return;
   }

  /***************************************************************************
   * Check for BERT detection/verification
   * For Detection from the net, the samples must be packed two bytes per
   * word. Pack the received samples into a scratch buffer and provide
   * to BERT.
   *
   * BERT only accepts an even number of bytes, so for aal1 a one byte pad
   * is used.
   ***************************************************************************/
  
  if ( rtr->bertDetect == DREG_ROUTE_BERT_DETECT_ON  )  {
    if (diagCfg.ctable.bert)  {

      /* Get any left over samples. */
      if (inst->bertSampleCount)  {
        bertData[0] = inst->bertSample;
        j = 1;
        inst->bertSampleCount = 0;

      }  else
        j = 0;


      for (i = 0; i < data.dataSize; i++)
        bertData[j++] = ((tuint *)data.data)[i];

      /* If there are an odd number of samples, store 1 */
      if (j & 1)  {
        j -= 1;
        inst->bertSample = bertData[j];
        inst->bertSampleCount = 1;
      }


      for (i = k = 0; i < j; i += 2, k++)  {
        v = bertData[i+0] & 0x00ff;
        w = bertData[i+1] & 0x00ff;
        bertData[k] = (v << 8) | w;
      }

      //verify the NET received samples for BERT
      (*diagCfg.ctable.bert->bertReceiveInFcn) (inst->bertInst, VCA_PACKET_MORE_ON, j << 3,
                                              bertData, 0, 0, 0, NULL);
    }
    else
    dsiuInc (&(inst->errs->pcmInvalidTxBert)); 
  }

  /***************************************************************************
   * Check BERT replacement:
   *  The GenTelFcn is required regardless of the eventual destination
   ***************************************************************************/
  if ( (rtr->outSrc == DREG_ROUTE_OUT_SRC_BERT_RANDOM)  ||
       (rtr->outSrc == DREG_ROUTE_OUT_SRC_BERT_255)     ||
       (rtr->outSrc == DREG_ROUTE_OUT_SRC_BERT_RND_ID)  )  {
    if (diagCfg.ctable.bert)  {
        (*diagCfg.ctable.bert->bertPatternGenTelFcn) (inst->bertInst, 
                                                      data.dataSize, data.data);
      }
    
    else
      dsiuInc (&(inst->errs->pcmInvalidTxBert)); 
  }

  /***************************************************************************
   * Check tone replacement 
   ***************************************************************************/
  if (rtr->outSrc == DREG_ROUTE_OUT_SRC_TONE)  {
    if (diagCfg.ctable.dtgu)
      (*diagCfg.ctable.dtgu->dtguGenTone) (inst->dtguInst, &data);
    else
      dsiuInc (&(inst->errs->pcmInvalidRxTone));
  }

  /***************************************************************************
   * Route the output data
   ***************************************************************************/
  if (rtr->outCh >= diagCfg.maxChannels)  {
    dsiuInc (&(inst->errs->invalidRouteCh));
    return;
  }
  destInst = &dsiuInst[rtr->outCh];

  if (rtr->outDir == DREG_ROUTE_OUTDIR_TDM)  {
    if (diagCfg.ctable.dvpu)
      (*diagCfg.ctable.dvpu->dvpuReceiveIn) (destInst->dvpuInst, &data);
    else
      dsiuInc (&(inst->errs->netInvalidPcmRoute));

  } else if (rtr->outDir == DREG_ROUTE_OUTDIR_NET)  {
    dsiuSendDataToNet (destInst, &data);
  }

} /* dsiuProcessRxNet */
  
/*****************************************************************************
 * FUNCTION PURPOSE: Post a Tx PCM task.
 *****************************************************************************
 * DESCRIPTION: This function is called from an ISR context by DPIU. It 
 *              posts a tx pcm task.
 *   This function is only required in the diagnostic mode.
 *****************************************************************************/
tint dsiuPostDpiuTxTask (void *inst)
{
  if (dsiuPostTask (dsiuContext.txPcmTasks, inst) != DSIU_TASK_SUCCESS)
    dsiuContext.errs.txPcmTaskQueueFull += 1;

  return (DSIU_SUCCESS);
} /* DsiuPostDpiuTxTask */

/*****************************************************************************
 * FUNCTION PURPOSE: Post an Rx PCM task.
 *****************************************************************************
 * DESCRIPTION: This function is called from an ISR context by DPIU. It 
 *              posts an rx pcm task.
 *   This function is only required in the diagnostic mode.
 *****************************************************************************/
tint dsiuPostDpiuRxTask (void *inst)
{
  if (dsiuPostTask (dsiuContext.rxPcmTasks, inst) != DSIU_TASK_SUCCESS)
    dsiuContext.errs.rxPcmTaskQueueFull += 1;

  return (DSIU_SUCCESS);
} /* DsiuPostDpiuRxTask */


/*****************************************************************************
 * FUNCTION PURPOSE: Process posted tasks
 *****************************************************************************
 * DESCRIPTION: Processes posted tasks. Tx PCM tasks have the highest priority, 
 *              followed by Rx PCM, Rx Net.
 *  This function is only required in the diagnostic mode.
 *****************************************************************************/
void dsiuProcessTasks(void )
{
  bool taskProcessed;
  dsiuInst_t *inst;

  /* Data pump tasks */
  do  {
    taskProcessed = FALSE;

    /* Tx PCM tasks */
    if ((diagCfg.ctable.dpiu) && 
              (inst = (dsiuInst_t *)dsiuGetTask (dsiuContext.txPcmTasks)))  {
      taskProcessed = TRUE;
      dsiuProcessTxPcm (inst);
    }
    /* Rx PCM tasks */
    else if ((diagCfg.ctable.dpiu) && 
              (inst = (dsiuInst_t *)dsiuGetTask (dsiuContext.rxPcmTasks)))  {
      taskProcessed = TRUE;
      dsiuProcessRxPcm (inst);
    }

    /* Rx Net tasks */
    else if(dsiuContext.rxNetTasks) {
      if (inst = (dsiuInst_t *)dsiuGetTask (dsiuContext.rxNetTasks))  {
        taskProcessed = TRUE;
        dsiuProcessRxNet (inst);
      }
    }

  } while (taskProcessed);

} /* dsiuProcessTasks */

/*****************************************************************************
 * FUNCTION PURPOSE: Opens channels
 *****************************************************************************
 * DESCRIPTION: Opens TDM and network channels as configured
 *  This function is only required in the diagnostic mode.
 *****************************************************************************/
dsiuReturn_t dsiuOpenChannel (dsiuInst_t *inst)
{
  union {
    dvpuOpenConfig_t dvpuOpenCfg;
    dtguOpenConfig_t dtguOpenCfg;
    dpvpOpenConfig_t dpvpOpenCfg;
  } u;
  dpiuOpenConfig_t     dpiuOpenCfg;
  dregNetCfg_t        *vch;
  dregTdmCfg_t        *tch;

  /* Setup the pointer to the channel network and configuration info */
  vch = &(dsiuContext.dregVchCfgShadow[inst->Id - 1]);
  tch = &(dsiuContext.dregTdmCfgShadow[inst->Id - 1]);


  /* Setup main instance values */
  inst->tdmFrameSize = diagCfg.apRate0;
  switch (tch->compand)  {
    case DREG_TDM_COMPAND_ULAW:
      inst->compandLaw     = DSIU_COMPAND_ULAW;
      inst->silencePattern = DSIU_COMPAND_ULAW_SILENCE;
      break;

    case DREG_TDM_COMPAND_ALAW:
      inst->compandLaw     = DSIU_COMPAND_ALAW;
      inst->silencePattern = DSIU_COMPAND_ALAW_SILENCE;
      break;

    case DREG_TDM_COMPAND_LINEAR:
      inst->compandLaw     = DSIU_COMPAND_LINEAR;
      inst->silencePattern = DSIU_COMPAND_LINEAR_SILENCE;
      break;
  }


  /* Open TDM side */

  /**************************************************
   *   Open DVPU                                    *
   **************************************************/
  if (diagCfg.ctable.dvpu)  {
    u.dvpuOpenCfg.opDelay = (diagCfg.vpuBufSize/sizeof(tuint)) >> 1;
    u.dvpuOpenCfg.minDelay = 0;
    if ((*diagCfg.ctable.dvpu->dvpuOpen)(inst->dvpuInst, 
                                         &u.dvpuOpenCfg) != DVPU_SUCCESS)
      return (DSIU_FAIL);
  }

  /***************************************************
   *     Open DTGU                                   *
   ***************************************************/
  if (diagCfg.ctable.dtgu)  {
    switch (inst->compandLaw)  {
      case DSIU_COMPAND_ALAW:   u.dtguOpenCfg.compandLaw = DTGU_COMPAND_ALAW;
        break;
      case DSIU_COMPAND_ULAW:   u.dtguOpenCfg.compandLaw = DTGU_COMPAND_ULAW;
        break;
      case DSIU_COMPAND_LINEAR: u.dtguOpenCfg.compandLaw = DTGU_COMPAND_LINEAR;
        break;
    }
    if ((*diagCfg.ctable.dtgu->dtguOpen)(inst->dtguInst, 
                                         &u.dtguOpenCfg) != DTGU_SUCCESS)
      return (DSIU_FAIL);
  }
    
  /**************************************************
  *    Open DPIU                                    *
  *   The timeslot must be enabled                  *
  *   After DPIU                                    *
  ***************************************************/
  if (diagCfg.ctable.dpiu)  {
    dpiuOpenCfg.tdmFrameSize = inst->tdmFrameSize;
    dpiuOpenCfg.rxInitVal = inst->silencePattern;
    if ((*diagCfg.ctable.dpiu->dpiuOpen) (inst->dpiuInst, 
                                         &dpiuOpenCfg) != DPIU_SUCCESS)
      return (DSIU_FAIL);
  }

  /**************************************************
   *    Open NEU, default to utopia interface       *
   **************************************************/
  if (diagCfg.ctable.neu && diagCfg.ctable.dsiu->openVoiceNeu)  {
    if ((*diagCfg.ctable.dsiu->openVoiceNeu)(inst) != DSIU_SUCCESS)
      return (DSIU_FAIL);
  }

  /**************************************************
   *    Open DPVP                                   *
   **************************************************/
switch (vch->encap)  {

    case DREG_NET_ENCAP_AAL1:
    u.dpvpOpenCfg.encap = DPVP_ENCAP_AAL1;
    u.dpvpOpenCfg.txValidDataSizeBytes = DSIU_AAL1_PAYLOAD_VALID_BYTES;
    u.dpvpOpenCfg.rxValidDataSizeBytes = DSIU_AAL1_PAYLOAD_VALID_BYTES;
    u.dpvpOpenCfg.txFill               = 0;
      break;

    case DREG_NET_ENCAP_AAL1_16BIT:
      u.dpvpOpenCfg.encap = DPVP_ENCAP_AAL1_16BIT;
      u.dpvpOpenCfg.txValidDataSizeBytes = DSIU_AAL1_PAYLOAD_VALID_BYTES_16BITDATA;
      u.dpvpOpenCfg.rxValidDataSizeBytes = DSIU_AAL1_PAYLOAD_VALID_BYTES_16BITDATA;
      u.dpvpOpenCfg.txFill               = 0;
      break;

    case DREG_NET_ENCAP_RAW:
    u.dpvpOpenCfg.encap = DPVP_ENCAP_RAW;
    u.dpvpOpenCfg.txValidDataSizeBytes = DSIU_RAW_PAYLOAD_SIZE_BYTES;
    u.dpvpOpenCfg.rxValidDataSizeBytes = DSIU_RAW_PAYLOAD_SIZE_BYTES;
    u.dpvpOpenCfg.txFill               = 0;
      break;

    case DREG_NET_ENCAP_RAW_16BIT:
      u.dpvpOpenCfg.encap = DPVP_ENCAP_RAW_16BIT;
      u.dpvpOpenCfg.txValidDataSizeBytes = DSIU_RAW_PAYLOAD_SIZE_BYTES;
      u.dpvpOpenCfg.rxValidDataSizeBytes = DSIU_RAW_PAYLOAD_SIZE_BYTES;
      u.dpvpOpenCfg.txFill               = 0;
      break;

    default:
      u.dpvpOpenCfg.encap = DPVP_ENCAP_NO_VOICE;
      break;

  }

  if(diagCfg.ctable.dpvp) {
    if ((*diagCfg.ctable.dpvp->dpvpOpen) (inst->dpvpInst,
                                          &u.dpvpOpenCfg) != DPVP_SUCCESS)
      return (DSIU_FAIL);
  }

  /* Clear BERT left over bit */
  inst->bertSampleCount = 0;

  /* Set channel state */
  inst->state = DSIU_STATE_OPEN;

  return (DSIU_SUCCESS);
} /* dsiuOpenChannel */


/****************************************************************************
 * FUNCTION PURPOSE: Measure the ratio between the system clock and the
 *                   sub-frame rate of the enabled mcbsp.
 ****************************************************************************/
void dsiuMeasureClockRatio (void)
{
  halTimerControl_t    tCtl;
  halIntStateControl_t iCtl;
  halTdmControl_t      mCtl;
  tulong               i;

  /* Initialize the clock ratio to failure. The value will be changed only
   * if the polling loop detects a successful measurement. Note that as long
   * as message processing is done in the foreground the value cannot be
   * read during the test. */
  dsiuContext.dregStatus->clkRatio = 0;

  /* Setup the timer to tick at the build time configured number of cpu clocks */
  tCtl.operation   = HAL_TIMER_OP_SET_PERIOD | HAL_TIMER_ENABLE;
  tCtl.timerperiod = diagCfg.timerPeriod;
  if ((*dsiuContext.hal->halTimerControl) (0, &tCtl) != HAL_SUCCESS)  {
    return;
  }

  /* Retrieve the current interrupt state */
  iCtl.operation = HAL_INT_RETRIEVE;
  (*dsiuContext.hal->halIntState) (&iCtl);

  /* Disable all interrupts */
  iCtl.operation = HAL_INT_CLEAR_ALL;
  (*dsiuContext.hal->halIntState) (&iCtl);

  /* Enable only the mcbsp interrupt */
  mCtl.mode = HAL_TDM_ENABLE_INT;
  (*dsiuContext.hal->tdm->halTdmControl) (dsiuContext.tdmPort->halId, &mCtl);

  /* Initialize the valid data flag and last delta value to enable the 
   * rate tracking */
  dsiuContext.eiClock.lastDelta = 0;
  dsiuContext.eiClock.validData = FALSE;

  /* Wait for the measurement to complete */
  for (i = 0; i < diagCfg.timerTimeout; i++)  {
  
    if (dsiuContext.eiClock.validData == TRUE)  {
      break;
    }

  }

  /* Restore interrupts to their original states */
  iCtl.operation = HAL_INT_SET;
  (*dsiuContext.hal->halIntState) (&iCtl);

  /* Disable the timer */
  tCtl.operation = HAL_TIMER_DISABLE;
  (*dsiuContext.hal->halTimerControl) (0, &tCtl);

} /* dsiuMeasureClockRatio */

/****************************************************************************
 * FUNCTION PURPOSE: Configure dpvp
 ****************************************************************************
 * DESCRIPTION: Reconfigures dpvp. Returns a non-zero value on failure
 ****************************************************************************/
tint dsiuSetPvp (dregNetCfg_t *vcfg, dsiuInst_t *inst) 
{
  dpvpControl_t   dpvpCtl;

  /* Inform dpvp of the new encapsulation */
  dpvpCtl.code = DPVP_CTL_SET_ENCAP;
  switch (vcfg->encap)  {
    case DREG_NET_ENCAP_NO_VOICE:
         dpvpCtl.u.setEncap.encap = DPVP_ENCAP_NO_VOICE;
         dpvpCtl.u.setEncap.txValidDataSizeBytes = 0;
         dpvpCtl.u.setEncap.rxValidDataSizeBytes = 0;
         break;
  
    case DREG_NET_ENCAP_AAL1:
         dpvpCtl.u.setEncap.encap = DPVP_ENCAP_AAL1;
         dpvpCtl.u.setEncap.txValidDataSizeBytes = DSIU_AAL1_PAYLOAD_VALID_BYTES;
         dpvpCtl.u.setEncap.rxValidDataSizeBytes = DSIU_AAL1_PAYLOAD_VALID_BYTES;
         break;

    case DREG_NET_ENCAP_RAW:
         dpvpCtl.u.setEncap.encap = DPVP_ENCAP_RAW;
         dpvpCtl.u.setEncap.txValidDataSizeBytes = DSIU_RAW_PAYLOAD_SIZE_BYTES;
         dpvpCtl.u.setEncap.rxValidDataSizeBytes = DSIU_RAW_PAYLOAD_SIZE_BYTES;
         break;
    }

  if ((*diagCfg.ctable.dpvp->dpvpControl)(inst->dpvpInst, &dpvpCtl) != DPVP_SUCCESS)  {
    return (-1);
  }

  return (0);

} /* dsiuSetPvp */

/***************************************************************************
 * FUNCTION PURPOSE: Compare NEU config paramters
 ***************************************************************************
 * DESCRIPTION: Returns TRUE if the two neu configs are different
 ***************************************************************************/
bool dsiuCompareNeu (dregNetCfg_t *vcfg, dregNetCfg_t *vcfgShadow)
{
  tint i;

  if (  (vcfg->cfg_flag1              != vcfgShadow->cfg_flag1)               ||
        (vcfg->cfg_flag2              != vcfgShadow->cfg_flag2)               ||
        (vcfg->alt_local_port         != vcfgShadow->alt_local_port)          ||
        (vcfg->second_prototype       != vcfgShadow->second_prototype)        ||
        (vcfg->rx_offset_bytes        != vcfgShadow->rx_offset_bytes)         ||
        (vcfg->rx_payload_type        != vcfgShadow->rx_payload_type)         ||
        (vcfg->tx_min_cell_bytes      != vcfgShadow->tx_min_cell_bytes)       ||
        (vcfg->tx_ip_offset_bytes     != vcfgShadow->tx_ip_offset_bytes)      ||
        (vcfg->tx_udp_offset_bytes    != vcfgShadow->tx_udp_offset_bytes)     ||
        (vcfg->tx_802_3_offset_bytes  != vcfgShadow->tx_802_3_offset_bytes)   ||
        (vcfg->tx_total_hdr_len_bytes != vcfgShadow->tx_total_hdr_len_bytes)  ||
        (vcfg->tx_udp_src_mask        != vcfgShadow->tx_udp_src_mask)         ||
        (vcfg->tx_udp_src_val         != vcfgShadow->tx_udp_src_val)          )  {

    return (TRUE);
  }

  for (i = 0; i < DREG_IP_ADDR_MAX_NUM_TUINT; i++)  {
    if (vcfg->alt_remote_addr[i] != vcfgShadow->alt_remote_addr[i])
      return (TRUE);
  }

  /* If an odd number of bytes is in the header even the unused byte will
   * be compared here. This would cause a new config to occur, but it
   * won't make any difference */
  for (i = 0; i < SIU_BYTES_TO_TUINT(vcfg->tx_total_hdr_len_bytes); i++)  {
    if (vcfg->header[i] != vcfgShadow->header[i])
      return (TRUE);
  }

  return (FALSE);

} /* dsiuCompareNeu */


/***************************************************************************
 * FUNCTION PURPOSE: Configure neu
 ***************************************************************************
 * DESCRIPTION: Provides the specified configuration to neu. Returns
 *              a non-zero value on failure.
 ***************************************************************************/
tint dsiuSetNeu (dregNetCfg_t *vcfg, dsiuInst_t *inst)
{
  neuCtrl_t       ctrl;
  neuParms_t     *ncfg;
  tint            count;
  tint            i;

  /* Peform the NEU configuration */
  ncfg = &(ctrl.u.parm);

  /* There is no mode for channel disable */
  ncfg->chnl.enable = TRUE;

  /* Get configuration from the registers */
  ncfg->chnl.cfg_flag1 = vcfg->cfg_flag1;
  ncfg->chnl.cfg_flag2 = vcfg->cfg_flag2;

  ncfg->chnl.itdmMode = 0;

  /* SAR type is currently determined only by route and encap */
  if ((vcfg->route == DIAGAPP_ROUTE_UTOPIA) && (vcfg->encap == DREG_NET_ENCAP_RAW))
    ncfg->chnl.sar_type = neuproto_SAR_AAL5;
  else
    ncfg->chnl.sar_type = neuproto_SAR_NONE;
  

  /* Alt ports/address */
  ncfg->chnl.alt_local_port  = vcfg->alt_local_port;
  ncfg->chnl.alt_remote_port = vcfg->alt_remote_port;
  count = 0;
  while(count < DMSG_IP_ADDR_MAX_NUM_TUINT){
    ncfg->chnl.alt_remote_addr[count] = vcfg->alt_remote_addr[count];
    count++;
  }

  /* Report control is always disabled */
  ncfg->chnl.report_ctrl          = 0;
  ncfg->chnl.report_throttle_time = 0;
  ncfg->chnl.sysInst              = 0;

  /* Second protocol type */
  ncfg->chnl.second_prototype = vcfg->second_prototype;
  
  /* Header parameters */
  ncfg->chnl.rx_offset_bytes = vcfg->rx_offset_bytes;
  ncfg->chnl.rx_payload_type = vcfg->rx_payload_type;

  /* IP/UDP */
  ncfg->chnl.tx_ip_offset_bytes    = vcfg->tx_ip_offset_bytes;
  ncfg->chnl.tx_udp_offset_bytes   = vcfg->tx_udp_offset_bytes;
  ncfg->chnl.tx_802_3_offset_bytes = vcfg->tx_802_3_offset_bytes;
  ncfg->chnl.tx_min_cell_bytes     = vcfg->tx_min_cell_bytes;

  /* Disable udp port fixup */
  ncfg->chnl.tx_udp_src_mask = 0;
  ncfg->chnl.tx_udp_src_val  = 0;

  /* Copy the header */
  ncfg->chnl.tx_total_hdr_len_bytes = vcfg->tx_total_hdr_len_bytes;
  memcpy(ncfg->chnl.header, vcfg->header, SIU_SIZE_TUINT_TO_WORDS(DREG_ENCAP_MAX_HEADER_SIZE_TUINT)); 

  /* The hal ID */
  if (vcfg->route == DIAGAPP_ROUTE_UTOPIA) {
    for (i = 0; i < dsiuContext.utLogicAlloc; i++)  {
      if (dsiuContext.utopiaPort[i].physPort == vcfg->port) {
        ncfg->chnl.halId =  dsiuContext.utopiaPort[i].halId;
        break;
      }
    }

  } else if (vcfg->route == DIAGAPP_ROUTE_GMAC)  {
    for (i = 0; i < dsiuContext.macLogicAlloc; i++)  {
      if (dsiuContext.macPort[i].physPort == vcfg->port)  {
        ncfg->chnl.halId = dsiuContext.macPort[i].halId;
        break;
      }
    }

  }
  /* Perform the reconfigure */
  ctrl.code = neu_CTRL_CHANNEL_CFG;
  if ((*diagCfg.ctable.neu->neuControl) (inst->neuInst, &ctrl))  {
    return (-1);
  }

  /* un-register any udp ports mapped to this channels neu instance */
  ctrl.code = neu_CTRL_RMV_PORTMAP;
  ctrl.u.portMap.inst = (void *)inst;
  (*diagCfg.ctable.neu->neuControl) (inst->neuInst, &ctrl);
  
  /* Remap the new udp port */
  /* Register the UDP port, if provided */
  ctrl.code           = neu_CTRL_GET_SRC_UDP_PORT; 
  
  if ((*diagCfg.ctable.neu->neuControl)(inst->neuInst, &ctrl) == neu_NOERR)  {
      
    ctrl.code = neu_CTRL_ADD_PORTMAP;
    ctrl.u.portMap.port = utlRead16bits ( (word *)vcfg->header, SIU_BYTES_TO_WORDS(vcfg->tx_udp_offset_bytes + neuproto_UDP_BYTE_OFFSET_SRC_PORT));
    ctrl.u.portMap.inst = (void*)inst; 
        
    if ((*diagCfg.ctable.neu->neuControl) (inst->neuInst, &ctrl) != neu_NOERR)  {
      /* Verify that the registration was successful */
      dsiuInc (&(dsiuContext.dregOpStats->neuPortMapFail));
      return (-1);
    }
  }

  return (0);

} /* dsiuSetNeu */
  

/****************************************************************************
 * FUNCTION PURPOSE: Monitor changes in network configuraion registers
 ****************************************************************************
 * DESCRIPTION: Validates the settings in the network configuration registers
 *              and implements any changes.
 ****************************************************************************/
void dsiuCheckNetChannels (void)
{
  dregNetCfg_t   *vcfg;
  dregNetCfg_t   *vcfgShadow;
  dsiuInst_t     *inst;
  bool            cfgFail = FALSE;
  bool            sdwCopy = FALSE;
  tint i;

  if (dsiuContext.masterCntrl->netCntrl.command == DREG_CNTRL_NO_CMD)
    return;

  dsiuContext.masterCntrl->netCntrl.lastCommand = dsiuContext.masterCntrl->netCntrl.command;


  if (dsiuContext.masterCntrl->netCntrl.command == DREG_CNTRL_CONFIG)  {

    for (i = 0; i < diagCfg.maxChannels; i++)  {
      vcfg       = &(dsiuContext.dregVchCfgBlk[i].vch);
      vcfgShadow = &(dsiuContext.dregVchCfgShadow[i]);
      inst       = &(dsiuInst[i]);

      /* Check the routing */
      if (vcfg->route != vcfgShadow->route)  {

        if (vcfg->route == DIAGAPP_ROUTE_UTOPIA)  {
          if (vcfg->port >= diagCfg.nUtopiaPorts)  {
            dsiuInc (&(dsiuContext.dregOpStats->chPortFail));
            cfgFail = TRUE;
            break;
          }

        } else if (vcfg->route == DIAGAPP_ROUTE_GMAC)  {
          if (vcfg->port >= diagCfg.nGmacPorts)  {
            dsiuInc (&(dsiuContext.dregOpStats->chPortFail));
            cfgFail = TRUE;
            break;
          }
        }  else  {
          dsiuInc (&(dsiuContext.dregOpStats->chRouteFail));
          cfgFail = TRUE;
          break;
        }

      }

      /* Reconfigure dpvp */
      if (vcfg->encap != vcfgShadow->encap)  {
        if (dsiuSetPvp (vcfg, inst))  {
          dsiuInc (&(dsiuContext.dregOpStats->dpvpEncapFail));
          cfgFail = TRUE;
          break;
        }
      }

      /* Reconfigure neu */
      if (dsiuCompareNeu (vcfg, vcfgShadow))  {
        if (dsiuSetNeu (vcfg, inst))  {
          dsiuInc (&(dsiuContext.dregOpStats->neuCfgFail));
          cfgFail = TRUE;
          break;
        }
      }
    }  
  


    /* On configuration failure restore the settings in the shadow registers */
    if (cfgFail)  {
      
     for (i = 0; i < diagCfg.maxChannels; i++)  {
       vcfg       = &(dsiuContext.dregVchCfgBlk[i].vch);
       vcfgShadow = &(dsiuContext.dregVchCfgShadow[i]);
       inst       = &(dsiuInst[i]);

        if (dsiuSetPvp (vcfgShadow, inst))  {
         dsiuInc (&(dsiuContext.dregOpStats->dpvpResetFail));
       }

        if (dsiuSetNeu (vcfgShadow, inst))  {
         dsiuInc (&(dsiuContext.dregOpStats->neuResetFail));
       }
     }

      dsiuContext.masterCntrl->netCntrl.lastCommandStatus = DREG_CNTLSTAT_FAIL;
  
   }  else  {

    sdwCopy = TRUE;
    dsiuContext.masterCntrl->netCntrl.lastCommandStatus = DREG_CNTLSTAT_SUCCESS;
  
   }

  }  else if (dsiuContext.masterCntrl->netCntrl.command == DREG_CNTRL_RETRIEVE)  {
   
    sdwCopy = TRUE;

  }  else   {

    /* An unknown command */
    dsiuContext.masterCntrl->netCntrl.lastCommandStatus = DREG_CNTLSTAT_FAIL;

  }

  /* Copy the config registers to the shadow registers */
  if (sdwCopy)  {

      /* Copy the new config to the shadow registers */
     for (i = 0; i < diagCfg.maxChannels; i++)  {
       vcfg        = &(dsiuContext.dregVchCfgBlk[i].vch);
       vcfgShadow  = &(dsiuContext.dregVchCfgShadow[i]);
       *vcfgShadow = *vcfg;
     }
  }

  dsiuContext.masterCntrl->netCntrl.command = DREG_CNTRL_NO_CMD;

} /* dsiuCheckNetChannels */



/****************************************************************************
 * FUNCTION PURPOSE: Enable a tdm timeslot
 ****************************************************************************
 * DESCRIPTION: Enables a timeslot
 ****************************************************************************/
dsiuReturn_t dsiuEnableTs(dregTdmCfg_t *tdmCfg, dsiuInst_t *inst)
{
  halTdmControl_t  ctl;
  halReturn_t      ret;
  dpiuControl_t    pctl;
  halIntrLevel_t   iLevel;

  /**************************************************
   * Reset piu
   **************************************************/
  iLevel = HAL_INT_LEVEL_DISABLE;
  iLevel = (*dsiuContext.hal->halIntrLevel)(iLevel);
  if (diagCfg.ctable.dpiu)  {
    pctl.mode = DPIU_CTL_MODE_RESET;
    if ((*diagCfg.ctable.dpiu->dpiuControl)(inst->dpiuInst, &pctl) != DPIU_SUCCESS)  {
      dsiuInc (&(dsiuContext.dregOpStats->dpiuFail));
      return (DSIU_FAIL);
    }
  }
  iLevel = (*dsiuContext.hal->halIntrLevel)(iLevel);

  /***************************************************
   *   Enable Timeslot                               *
   ***************************************************/
  ctl.mode = HAL_TDM_TIMESLOT_CONFIG;

  ctl.u.tdmConfig.toTdmEnable   = TRUE;
  ctl.u.tdmConfig.fromTdmEnable = TRUE;
  ctl.u.tdmConfig.txTimeslot    = tdmCfg->txTimeslot;
  ctl.u.tdmConfig.rxTimeslot    = tdmCfg->rxTimeslot;
  ctl.u.tdmConfig.initVal     = inst->silencePattern;
  ctl.u.tdmConfig.txFrameSize = inst->tdmFrameSize;
  ctl.u.tdmConfig.rxFrameSize = inst->tdmFrameSize;
  ctl.u.tdmConfig.phase = 0;
  ctl.u.tdmConfig.txCallout = diagCfg.ctable.dpiu->dpiuTdmTxExchange;
  ctl.u.tdmConfig.rxCallout = diagCfg.ctable.dpiu->dpiuTdmRxExchange;
  typeSet_tfPtr(&ctl.u.tdmConfig.txContext, (void *)inst->dpiuInst, 
                (tuint)SIU_READ_PRIMARY_DATA_PAGE(inst));
  typeSet_tfPtr(&ctl.u.tdmConfig.rxContext, (void *)inst->dpiuInst, 
                (tuint)SIU_READ_PRIMARY_DATA_PAGE(inst));
  ctl.u.tdmConfig.txBuffer  = pctl.u.reset.txBuf;
  ctl.u.tdmConfig.rxBuffer  = pctl.u.reset.rxBuf;
  ctl.u.tdmConfig.txBufSize = inst->tdmFrameSize;
  ctl.u.tdmConfig.rxBufSize = inst->tdmFrameSize;
  ctl.u.tdmConfig.phase     = 0;

  /* The TDM is always configured as linear. The compand law in the
   * instance is used only for tone generation */
  ctl.u.tdmConfig.compand = HAL_BIT_EXACT;

  
  /* The port field is included in the timeslot value */
  if (dsiuContext.hal->tdm)  {
    ret = (*dsiuContext.hal->tdm->halTdmControl) (0, &ctl);
    
    /* It's not an error to enable a timeslot that is already on */
    if (ret == HAL_TDM_TS_ALREADY_ENABLED)  
      return (DSIU_SUCCESS);

    if (ret != HAL_SUCCESS)
      return (DSIU_FAIL);
  }

  return (DSIU_SUCCESS);

} /* dsiuEnableTs */

/****************************************************************************
 * FUNCTION PURPOSE: Verify TDM parameters
 ****************************************************************************
 * DESCRIPTION: The input TDM values are checked. A non-zero value is 
 *              returned if the parameters are invalid
 ****************************************************************************/
tint dsiuVerifyTdmCfg (dregTdmCfg_t *tdmCfg)
{
  /* Any timeslot value is allowed, so no check is made for them.The compand,
   * status, and control values are checked */
  if ( (tdmCfg->compand != DREG_TDM_COMPAND_ALAW)    &&
       (tdmCfg->compand != DREG_TDM_COMPAND_ULAW)    &&
       (tdmCfg->compand != DREG_TDM_COMPAND_LINEAR) )
    return (-1);

  if ( (tdmCfg->control != DREG_TDM_CONTROL_DISABLE_TS) &&
       (tdmCfg->control != DREG_TDM_CONTROL_ENABLE_TS)   ) 
    return (-1);

  return (0);

} /* dsiuVerifyTdmCfg */


/****************************************************************************
 * FUNCTION PURPOSE: Set the TDM state as configured
 ****************************************************************************
 * DESCRIPTION: Checks for a command change in the config registers.
 *              If a change is requested the register state is verified
 *              and implemented.
 ****************************************************************************/
void dsiuCheckTdmChannels (void)
{
  dregTdmCfg_t    *tdmCfg;
  dregTdmCfg_t    *tdmCfgB;
  dregTdmCfg_t    *tdmShadow;
  halReturn_t      ret;
  halTdmControl_t  ctl;
  tint             i, j;

  /* Check for requested change */
  if (dsiuContext.masterCntrl->tdmCntrl.command == DREG_CNTRL_CONFIG)  {

    dsiuContext.masterCntrl->tdmCntrl.lastCommand  = DREG_CNTRL_CONFIG;

    for (i = 0; i < diagCfg.maxChannels; i++)  {

      tdmCfg    = &(dsiuContext.dregTdmCfg[i]);
      tdmShadow = &(dsiuContext.dregTdmCfgShadow[i]);

      /* If the configuration changed check the validity of the new configuration */
      if (  (tdmCfg->txTimeslot != tdmShadow->txTimeslot)   ||
            (tdmCfg->rxTimeslot != tdmShadow->rxTimeslot)   ||
            (tdmCfg->control    != tdmShadow->control)       )  {
      
        /* If the requested config is invalid stop checks */        
        if (dsiuVerifyTdmCfg (tdmCfg))  {
          dsiuContext.masterCntrl->tdmCntrl.lastCommandStatus = DREG_CNTLSTAT_FAIL;
          dsiuContext.masterCntrl->tdmCntrl.command           = DREG_CNTRL_NO_CMD;
          return;
        }
      }

      /* Check that the configuration doesn't confict with another channel */
      if (tdmCfg->control == DREG_TDM_CONTROL_ENABLE_TS)  {

        for (j = i+1; j < diagCfg.maxChannels; j++)  {

          tdmCfgB = &(dsiuContext.dregTdmCfg[j]);
          if (tdmCfgB->control == DREG_TDM_CONTROL_ENABLE_TS)  {

            if (tdmCfg->txTimeslot == tdmCfgB->txTimeslot)  {
              dsiuContext.masterCntrl->tdmCntrl.lastCommandStatus = DREG_CNTLSTAT_FAIL;
              dsiuContext.masterCntrl->tdmCntrl.command           = DREG_CNTRL_NO_CMD;
              return;
            }
          }
        }
      }
    }

    /* All checks passed. Implement the new configuration. Do this in two 
     * passes. First disable any channels that had been enabled, then
     * on the second pass do any enables */
    for (i = 0; i < diagCfg.maxChannels; i++)  {
      
      tdmCfg    = &(dsiuContext.dregTdmCfg[i]);
      tdmShadow = &(dsiuContext.dregTdmCfgShadow[i]);

     
      if (  (tdmCfg->txTimeslot != tdmShadow->txTimeslot)      ||
            (tdmCfg->rxTimeslot != tdmShadow->rxTimeslot)      ||
            (tdmCfg->control == DREG_TDM_CONTROL_DISABLE_TS)    )  {

        if ( (tdmShadow->control == DREG_TDM_CONTROL_ENABLE_TS)  &&
             (tdmShadow->status  == DREG_TDM_STATUS_ENABLED)     ) { 

          ctl.mode = HAL_TDM_TIMESLOT_CONFIG;
          ctl.u.tdmConfig.toTdmEnable = FALSE;
          ctl.u.tdmConfig.fromTdmEnable = FALSE;
          ctl.u.tdmConfig.txTimeslot    = tdmShadow->txTimeslot;
          ctl.u.tdmConfig.rxTimeslot    = tdmShadow->rxTimeslot;

          /* Continue on failure to close a timeslot */
          if (dsiuContext.hal->tdm)  {
            ret = (*dsiuContext.hal->tdm->halTdmControl) (-1, &ctl);
            if (ret != HAL_SUCCESS)  {
              dsiuInc (&(dsiuContext.dregOpStats->tdmCfgErr));
            }
            tdmShadow->status  = DREG_TDM_STATUS_DISABLED;
            tdmShadow->control = tdmCfg->control;
          }
        }
      }
    }

    /* Perform enables */
    for (i = 0; i < diagCfg.maxChannels; i++)  {

      tdmCfg    = &(dsiuContext.dregTdmCfg[i]);
      tdmShadow = &(dsiuContext.dregTdmCfgShadow[i]);

      if  (  ((tdmCfg->txTimeslot != tdmShadow->txTimeslot)        ||
              (tdmCfg->rxTimeslot != tdmShadow->rxTimeslot)        ||
              (tdmCfg->control    != tdmShadow->control) )         &&
              (tdmCfg->control    == DREG_TDM_CONTROL_ENABLE_TS) )  {

        *tdmShadow = *tdmCfg;

        /* If the timeslot was already enabled don't mark the timeslot as
         * disabled. This shouldn't really happen since the checks above
         * should have detected a duplicated timeslot */
        if (dsiuEnableTs (tdmShadow, &dsiuInst[i]) != DSIU_SUCCESS)  {
          tdmShadow->status = DREG_TDM_STATUS_DISABLED;
          dsiuInc (&(dsiuContext.dregOpStats->tdmTsEnableFail));
        } else {
          tdmShadow->status = DREG_TDM_STATUS_ENABLED;
        }
        *tdmCfg    = *tdmShadow;

      /* Also check for an unchanged timeslot that is requested enabled
       * but is not enabled. This can happen because the system has not
       * yet received the tdm port config message, or when a previously
       * disabled timeslot is enabled on the same channel */
      }  else if (   (tdmCfg->txTimeslot   == tdmShadow->txTimeslot)       &&
                     (tdmCfg->rxTimeslot   == tdmShadow->rxTimeslot)       &&
                     (tdmCfg->control      == DREG_TDM_CONTROL_ENABLE_TS)  &&
                     (tdmShadow->status    == DREG_TDM_STATUS_DISABLED)     )  {

        *tdmShadow = *tdmCfg;
        if (dsiuEnableTs(tdmShadow, &dsiuInst[i]) != DSIU_SUCCESS)  {
          tdmShadow->status = DREG_TDM_STATUS_DISABLED;
          dsiuInc (&(dsiuContext.dregOpStats->tdmTsEnableFail));
        } else {
          tdmShadow->status = DREG_TDM_STATUS_ENABLED;
        }
        *tdmCfg    = *tdmShadow;
      }

    }

    dsiuContext.masterCntrl->tdmCntrl.lastCommandStatus = DREG_CNTLSTAT_SUCCESS;
    dsiuContext.masterCntrl->tdmCntrl.command           = DREG_CNTRL_NO_CMD;

  

  } else if (dsiuContext.masterCntrl->tdmCntrl.command == DREG_CNTRL_RETRIEVE)  {


    /* Copy the currently active configuration back to the config registers */
    dsiuContext.masterCntrl->tdmCntrl.lastCommand  = DREG_CNTRL_RETRIEVE;

    for (i = 0; i < diagCfg.maxChannels; i++)  {
      
      tdmCfg    = &(dsiuContext.dregTdmCfg[i]);
      tdmShadow = &(dsiuContext.dregTdmCfgShadow[i]);
      *tdmCfg = *tdmShadow;

    }

    dsiuContext.masterCntrl->tdmCntrl.lastCommandStatus = DREG_CNTLSTAT_SUCCESS;
    dsiuContext.masterCntrl->tdmCntrl.command           = DREG_CNTRL_NO_CMD;

  }  else  if (dsiuContext.masterCntrl->tdmCntrl.command != DREG_CNTRL_NO_CMD)  {

    /* An unkown command was received */
    dsiuContext.masterCntrl->tdmCntrl.lastCommand       = dsiuContext.masterCntrl->tdmCntrl.command;
    dsiuContext.masterCntrl->tdmCntrl.lastCommandStatus = DREG_CNTLSTAT_FAIL;
    dsiuContext.masterCntrl->tdmCntrl.command           = DREG_CNTRL_NO_CMD;

  }

} /* dsiuCheckTdmChannels */

/****************************************************************************
 * FUNCTION PURPOSE: Check basic router settings
 ****************************************************************************
 * DESCRIPTION: Verifies that the output channel and direction for the
 *              specified router are valid. Returns TRUE if the router is
 *              invalid.
 ****************************************************************************/
bool dsiu_router_invalid (dregRouterCfg_t *rtr)
{
  if (rtr->outCh >= diagCfg.maxChannels)
    return (TRUE);

  if ( (rtr->outDir != DREG_ROUTE_OUTDIR_TDM) && (rtr->outDir != DREG_ROUTE_OUTDIR_NET) )
    return (TRUE);

  return (FALSE);

} /* dsiu_router_invalid */


/****************************************************************************
 * FUNCTION PURPOSE: Set the command status.
 ****************************************************************************
 * DESCRIPTION: Sets the state of the specified command register, clears
 *              the command.
 ****************************************************************************/
void dsiu_cmd_stat (dregCntrl_t *ctl, tint status)
{
  ctl->lastCommand       = ctl->command;
  ctl->lastCommandStatus = status;
  ctl->command           = DREG_CNTRL_NO_CMD;
} /* dsiu_cmd_stat */ 

/****************************************************************************
 * FUNCTION PURPOSE: Check for a bert configuration change
 ****************************************************************************
 * DESCRIPTION: Compares the two router configurations, and returns TRUE
 *              if the bert configuration is different.
 ****************************************************************************/
bool dsiu_bert_route_change (dregRouterCfg_t *rtra, dregRouterCfg_t *rtrb)
{
  if (rtra->bertDetect != rtrb->bertDetect)
    return (TRUE);

  if (rtra->outSrc != rtrb->outSrc)  {

    if ( (rtra->outSrc == DREG_ROUTE_OUT_SRC_BERT_RANDOM) ||
         (rtra->outSrc == DREG_ROUTE_OUT_SRC_BERT_255)    ||
         (rtra->outSrc == DREG_ROUTE_OUT_SRC_BERT_RND_ID) ||
         (rtrb->outSrc == DREG_ROUTE_OUT_SRC_BERT_RANDOM) ||
         (rtrb->outSrc == DREG_ROUTE_OUT_SRC_BERT_255)    ||
         (rtrb->outSrc == DREG_ROUTE_OUT_SRC_BERT_RND_ID)  )
      return (TRUE);

  }

  return (FALSE);

} /* dsiu_bert_route_change */

/****************************************************************************
 * FUNCTION PURPOSE: Determine if the router BERT configuration is valid
 ****************************************************************************
 * DESCRIPTION: A BERT configuration is valid if both routers are not generating
 ****************************************************************************/
bool dsiu_bert_is_valid (tint channel)
{
  dregRouterCfg_t *rtrTdm;
  dregRouterCfg_t *rtrNet;

  bool tdmGen = FALSE;
  bool netGen = FALSE;
  
  rtrTdm = &(dsiuContext.dregRouterCfg[DREG_ROUTER_TDM(channel)]);
  rtrNet = &(dsiuContext.dregRouterCfg[DREG_ROUTER_NET(channel)]);

  if ( (rtrTdm->outSrc == DREG_ROUTE_OUT_SRC_BERT_RANDOM)  ||
       (rtrTdm->outSrc == DREG_ROUTE_OUT_SRC_BERT_255)     ||
       (rtrTdm->outSrc == DREG_ROUTE_OUT_SRC_BERT_RND_ID)   )
    tdmGen = TRUE;

  if ( (rtrNet->outSrc == DREG_ROUTE_OUT_SRC_BERT_RANDOM)  ||
       (rtrNet->outSrc == DREG_ROUTE_OUT_SRC_BERT_255)     ||
       (rtrNet->outSrc == DREG_ROUTE_OUT_SRC_BERT_RND_ID)   )
    netGen = TRUE;

  if (netGen && tdmGen)
    return (FALSE);

  return (TRUE);

} /* dsiu_bert_is_valid */

/****************************************************************************
 * FUNCTION PURPOSE: Determine the bert mode and enable sequence
 ****************************************************************************
 * DESCRIPTION: Determine the sequence (close, open, close then open)
 *              required to change from the shadow configuration to the
 *              new configuration.
 *
 *              It is already certain that a BERT change has been requested,
 *              but this change could be nothing but a router change.
 *          
 *              The return value is the generation pattern.
 ****************************************************************************/
tint dsiu_bert_get_seq (tint channel, bool *bertDisable, bool *bertEnable)
{
  dregRouterCfg_t *rtrTdm;
  dregRouterCfg_t *rtrShadow;
  dregRouterCfg_t *rtrNet;

  /* Initialize returns to no action */
  *bertDisable = FALSE;
  *bertEnable  = FALSE;

  /* If the new configuration has no BERT, disable only. This is relying on 
   * the fact that a change was made. */
  rtrTdm = &(dsiuContext.dregRouterCfg[DREG_ROUTER_TDM(channel)]);
  rtrNet = &(dsiuContext.dregRouterCfg[DREG_ROUTER_NET(channel)]);

  if ( (rtrTdm->outSrc != DREG_ROUTE_OUT_SRC_BERT_RANDOM)   &&
       (rtrTdm->outSrc != DREG_ROUTE_OUT_SRC_BERT_255)      &&
       (rtrTdm->outSrc != DREG_ROUTE_OUT_SRC_BERT_RND_ID)   &&
       (rtrNet->outSrc != DREG_ROUTE_OUT_SRC_BERT_RANDOM)   &&
       (rtrNet->outSrc != DREG_ROUTE_OUT_SRC_BERT_255)      &&
       (rtrNet->outSrc != DREG_ROUTE_OUT_SRC_BERT_RND_ID)    )  {
    *bertDisable = TRUE;
    return (0);

  }

  /* BERT is enabled in the new configuration. If it has the same
   * generation pattern as the shadow, then only a detector may
   * have changed. */
  rtrTdm    = &(dsiuContext.dregRouterCfg[DREG_ROUTER_TDM(channel)]);
  rtrShadow = &(dsiuContext.dregRouterCfgShadow[DREG_ROUTER_TDM(channel)]);

  if (rtrTdm->outSrc == rtrShadow->outSrc)  {

    rtrNet    = &(dsiuContext.dregRouterCfg[DREG_ROUTER_NET(channel)]);
    rtrShadow = &(dsiuContext.dregRouterCfgShadow[DREG_ROUTER_NET(channel)]);

    if (rtrNet->outSrc == rtrShadow->outSrc)
      return (0);  /* Only a detector change was configured */

  }
  

  /* BERT is enabled in the new configuration. If it was enabled in the shadows
   * then the old configuration must be closed. */
  rtrTdm = &(dsiuContext.dregRouterCfgShadow[DREG_ROUTER_TDM(channel)]);
  rtrNet = &(dsiuContext.dregRouterCfgShadow[DREG_ROUTER_NET(channel)]);

  if ( (rtrTdm->outSrc == DREG_ROUTE_OUT_SRC_BERT_RANDOM)  ||
       (rtrTdm->outSrc == DREG_ROUTE_OUT_SRC_BERT_255)     ||
       (rtrTdm->outSrc == DREG_ROUTE_OUT_SRC_BERT_RND_ID)  ||
       (rtrNet->outSrc == DREG_ROUTE_OUT_SRC_BERT_RANDOM)  ||
       (rtrNet->outSrc == DREG_ROUTE_OUT_SRC_BERT_255)     ||
       (rtrNet->outSrc == DREG_ROUTE_OUT_SRC_BERT_RND_ID)  )  
    *bertDisable = TRUE;
  

  /* BERT is enabled and in a valid configuration. Determine which
   * mode to operate in */
  rtrTdm = &(dsiuContext.dregRouterCfg[DREG_ROUTER_TDM(channel)]);
  rtrNet = &(dsiuContext.dregRouterCfg[DREG_ROUTER_NET(channel)]);

  *bertEnable = TRUE;

  if ( (rtrTdm->outSrc == DREG_ROUTE_OUT_SRC_BERT_RANDOM) ||
       (rtrNet->outSrc == DREG_ROUTE_OUT_SRC_BERT_RANDOM) )
    return (DREG_ROUTE_OUT_SRC_BERT_RANDOM);

  if ( (rtrTdm->outSrc == DREG_ROUTE_OUT_SRC_BERT_255) ||
       (rtrNet->outSrc == DREG_ROUTE_OUT_SRC_BERT_255) )
    return (DREG_ROUTE_OUT_SRC_BERT_255);

  if ( (rtrTdm->outSrc == DREG_ROUTE_OUT_SRC_BERT_RND_ID) ||
       (rtrNet->outSrc == DREG_ROUTE_OUT_SRC_BERT_RND_ID) )
    return (DREG_ROUTE_OUT_SRC_BERT_RND_ID);

  /* The code should never get here. Inc the stat to indicate a logic
   * error in the code. */
  dsiuInc (&(dsiuContext.dregOpStats->bertCtlFail));
  
  return (0);

} /* dsiu_bert_get_seq */



/****************************************************************************
 * FUNCTION PURPOSE: Check for route changes
 ****************************************************************************
 * DESCRIPTION: Checks for command changes to the router channels. A router
 *              configuration is invalid if two channels route to the same
 *              output, or if an output channel is out of range.
 ****************************************************************************/
void dsiuCheckRouterChannels (void)
{
  tint i, j;
  tint countT, countN;
  tint mode;

  dregRouterCfg_t *rtr;
  dregRouterCfg_t *rtrShadow;

  bool bertChange;
  bool bertEnable;
  bool bertDisable;

  if (dsiuContext.masterCntrl->routerCntrl.command == DREG_CNTRL_NO_CMD)
    return;


  if (dsiuContext.masterCntrl->routerCntrl.command == DREG_CNTRL_CONFIG)  {

    /* Check that all output channels are valid */
    for (i = 0; i < diagCfg.maxChannels; i++)  {
      rtr = &(dsiuContext.dregRouterCfg[DREG_ROUTER_TDM(i)]);
      if (dsiu_router_invalid (rtr))  {
        dsiu_cmd_stat (&dsiuContext.masterCntrl->routerCntrl, DREG_CNTLSTAT_FAIL);
        return;
      }
      
      rtr = &(dsiuContext.dregRouterCfg[DREG_ROUTER_NET(i)]);
      if (dsiu_router_invalid (rtr))  {
        dsiu_cmd_stat (&dsiuContext.masterCntrl->routerCntrl, DREG_CNTLSTAT_FAIL);
        return;
      }

    }

    /* Verify that no two sources are routed to the same channel */
    for (i = 0; i < diagCfg.maxChannels; i++)  {
      
      for (j = i; j < diagCfg.maxChannels; j++)  {
        /* Count the number of channels routed to this channels net and tdm output */
        countT = countN = 0;

        /* The validity of outDir was verified above */
        rtr = &(dsiuContext.dregRouterCfg[DREG_ROUTER_TDM(j)]);
        if ( (rtr->status == DREG_ROUTE_STATUS_ENABLED) && (rtr->outCh == i)  ) {
          if (rtr->outDir == DREG_ROUTE_OUTDIR_TDM)
            countT += 1;
          else
            countN += 1;
        }

        rtr = &(dsiuContext.dregRouterCfg[DREG_ROUTER_NET(j)]);
        if ( (rtr->status == DREG_ROUTE_STATUS_ENABLED) && (rtr->outCh == i)  ) {
          if (rtr->outDir == DREG_ROUTE_OUTDIR_TDM)
            countT += 1;
          else
            countN += 1;
        }
      }
            
      if ( (countT > 1) || (countN > 1) )  {
        dsiu_cmd_stat (&dsiuContext.masterCntrl->routerCntrl, DREG_CNTLSTAT_FAIL);
        return;
      }
    }

    /* Check if the bert pattern has changed. The two routers (from net and from tdm)
     * both share the same BERT instance, so if both directions are enabled they
     * must be configured to use the same pattern. */
    for (i = 0; i < diagCfg.maxChannels; i++)  {
 
      bertChange = FALSE;
      
      rtr       = &(dsiuContext.dregRouterCfg[DREG_ROUTER_TDM(i)]);
      rtrShadow = &(dsiuContext.dregRouterCfgShadow[DREG_ROUTER_TDM(i)]);

      if (dsiu_bert_route_change (rtr, rtrShadow))
        bertChange = TRUE;

      /* Repeat for the network side */
      rtr       = &(dsiuContext.dregRouterCfg[DREG_ROUTER_NET(i)]);
      rtrShadow = &(dsiuContext.dregRouterCfgShadow[DREG_ROUTER_NET(i)]);

      if (dsiu_bert_route_change (rtr, rtrShadow))
        bertChange = TRUE;

      /* If BERT has changed check the validity of the configuration and determine
       * if the module must be closed, opened, or closed then opened */
      if (bertChange)  {

        if (!dsiu_bert_is_valid(i))  {
          dsiu_cmd_stat (&dsiuContext.masterCntrl->routerCntrl, DREG_CNTLSTAT_FAIL);
          return;
        }

        /*  Determine if BERT must be opened or closed, or both, or neither (detector enabled) */
        mode = dsiu_bert_get_seq (i, &bertDisable, &bertEnable);

        if (bertDisable)
          (*diagCfg.ctable.dsiu->closeBert)(dsiuInst[i].bertInst);

        if (bertEnable)  {
          if ((*diagCfg.ctable.dsiu->openBert)(&dsiuInst[i], mode) != DSIU_SUCCESS) 
            dsiuInc (&(dsiuContext.dregOpStats->bertCtlFail));
        }
#ifdef DIAG_INCLUDE_BERT
        if ((rtr->bertDetect == TRUE) && (rtrShadow->bertDetect == FALSE))
          dsiuBertClearStats ((void *)(&dsiuInst[i]));
#endif
      }
    }
          

    /* The new configuration is valid. Copy it to the shadow registers */
    for (i = 0; i < 2*diagCfg.maxChannels; i++)  {
      dsiuContext.dregRouterCfgShadow[i] = dsiuContext.dregRouterCfg[i];
    }
    dsiu_cmd_stat (&dsiuContext.masterCntrl->routerCntrl, DREG_CNTLSTAT_SUCCESS);

  } else if (dsiuContext.masterCntrl->routerCntrl.command == DREG_CNTRL_RETRIEVE)  {

    for (i = 0; i < 2*diagCfg.maxChannels; i++)  {
      dsiuContext.dregRouterCfg[i] = dsiuContext.dregRouterCfgShadow[i];
    }

    dsiu_cmd_stat (&dsiuContext.masterCntrl->routerCntrl, DREG_CNTLSTAT_SUCCESS);

  } else {  /* An unknown command */

    dsiu_cmd_stat (&dsiuContext.masterCntrl->routerCntrl, DREG_CNTLSTAT_FAIL);

  }

} /* dsiuCheckRouterChannels */


/****************************************************************************
 * FUNCTION PURPOSE: Set the channel state
 ****************************************************************************
 * DESCRIPTION: Checks the channel configuration registers and modifies
 *              the channel state based on changes and current status.
 ****************************************************************************/
void dsiuCheckChannels (void)
{
#ifdef DIAG_INCLUDE_BERT
  tint i;
#endif

  /* Check TDM configuration */
  dsiuCheckTdmChannels ();

  /* Check Network configuration */
  dsiuCheckNetChannels ();

  /* Check router configuration */
  dsiuCheckRouterChannels ();
#ifdef DIAG_INCLUDE_BERT
  /* Bert clear */
  if (dregMasterCntrl.bertClearCntrl.channel == DREG_BERTCLEAR_CNTL_CLEAR_ALL)  {

    for (i = 0; i < diagCfg.maxChannels; i++) 
      dsiuBertClearStats ((void *)(&dsiuInst[i]));

    dregMasterCntrl.bertClearCntrl.channel = DREG_BERTCLEAR_CNTL_IDLE;

  } else if (dregMasterCntrl.bertClearCntrl.channel != DREG_BERTCLEAR_CNTL_IDLE)  {

    if ( (dregMasterCntrl.bertClearCntrl.channel >= 0) && 
         (dregMasterCntrl.bertClearCntrl.channel < diagCfg.maxChannels) )
      dsiuBertClearStats ((void *)(&dsiuInst[dregMasterCntrl.bertClearCntrl.channel]));

    dregMasterCntrl.bertClearCntrl.channel = DREG_BERTCLEAR_CNTL_IDLE;
  }
#endif /* DIAG_INCLUDE_BERT*/


} /* dsiuCheckChannels */

/******************************************************************************
 DDR2<->L1D memory move using EDMA
******************************************************************************/
#if 0 //dsiuTestDdr2Paging
/* ---- Timer stuffs for Profiling ---------------- */
volatile tuint DDR2_DIRECT_ACCESS = 1;
#define START_TIMER_TOTAL start_time_totalDDR = TSCL;
#define STOP_TIMER_TOTAL  end_time_totalDDR   = TSCL;
#define TIME_TAKEN_TOTAL   (end_time_totalDDR - start_time_totalDDR)
extern far cregister volatile unsigned int TSCL;
int start_time_totalDDR;
int end_time_totalDDR;
tuint profile_frameDDR;
tuint cycles_per_frame_readDDR[100];
tuint cycles_per_frame_writeDDR[100];
volatile tuint ddr2Error = 1;
tulong ddrDebug[3];
/* END---- Timer stuffs for Profiling ---------------- */
#define DDR2_SIZE        0x80000     /* in 32-bit */
#pragma DATA_SECTION(ddr_buffer, ".ddr_buffers")
volatile tulong ddr_buffer[DDR2_SIZE];

#define L1D_BUF_SIZE     0xF00     /* in 32-bit */
#pragma DATA_SECTION(l1d_buffer, ".l1d_buffers")
volatile tulong l1d_buffer[L1D_BUF_SIZE];

void dsiuTestDdr2Paging (tuint *ch)
{
  tulong index, i;
  tulong seg_size = L1D_BUF_SIZE/2;
  tulong wr_value,rd_value;
 
  /* Fill L1D buffer with a pattern  */  
  for (i = 0; i < seg_size ;i++) 
  {
    l1d_buffer[i] = ( i + 0X100000*profile_frameDDR );    /* Write value of Index aded with 10000000 */
  }                                       

  
  if(dsiuContext.hal->qdma) {
    START_TIMER_TOTAL;

  if (!DDR2_DIRECT_ACCESS){
    /* transfer L1D buffer contents to DDR2 using EDMA */
    (*dsiuContext.hal->qdma->halQdmaStartTransfer) ((halGlobAddr_t)ddr_buffer, 
                  (halGlobAddr_t)l1d_buffer, (halSAE_t)seg_size*4, ch[0] );
    (*dsiuContext.hal->qdma->halWaitTillQDMATransferComplete) ( ch[0]);
  }else{
    for (i = 0; i < seg_size ;i++) 
    {
      ddr_buffer[i] = l1d_buffer[i];    
    }                                       
  }
    STOP_TIMER_TOTAL;

  asm (" NOP");
  asm (" NOP");
  asm (" NOP");

  cycles_per_frame_writeDDR[profile_frameDDR] = TIME_TAKEN_TOTAL;

  /* clear L1D buffer */
  for (i = 0; i < seg_size ;i++) 
  {
    l1d_buffer[i] = 0;    
  }                                       
  
  /* Get the buffer from DDR2 to L1D using EDMA */
  START_TIMER_TOTAL;
  if (!DDR2_DIRECT_ACCESS){
    (*dsiuContext.hal->qdma->halQdmaStartTransfer) ((halGlobAddr_t)l1d_buffer,
                  (halGlobAddr_t)ddr_buffer, (halSAE_t)seg_size*4, ch[1] );
    (*dsiuContext.hal->qdma->halWaitTillQDMATransferComplete) ( ch[1]);
  }else{
    for (i = 0; i < seg_size ;i++) 
    {
      l1d_buffer[i] = ddr_buffer[i];    
    }                                       
  }
  STOP_TIMER_TOTAL;

    asm (" NOP");
    asm (" NOP");
    asm (" NOP");

    cycles_per_frame_readDDR[profile_frameDDR] = TIME_TAKEN_TOTAL;

    if(profile_frameDDR > 399) {
      asm (" NOP");
      asm (" NOP");
//      while (profile_frameDDR);
      profile_frameDDR = 0;
      asm (" NOP");
    }  
    
  }


  /* Verify the contents */
  for (i = 0; i < seg_size ;i++)
  {
    wr_value = ( i + 0X100000*profile_frameDDR );
    rd_value = l1d_buffer[i];

    if (rd_value != wr_value )                  /* Check for data correctness */
    {
      ddrDebug[0] = (tulong)&ddr_buffer[i];
      ddrDebug[1] = wr_value;
      ddrDebug[2] = rd_value;
      while(ddr2Error);
    }
  }
  profile_frameDDR++;
}
#endif

extern far cregister volatile unsigned int TSCL;


/* mdebug function. Disable/enable the specicied timeslot. This happens
 * about once every second at 500 MHz */
unsigned int lastCount = 0;
int mdChannel = -1;
int lastState = 0;
#define TR_DELTA  0x20000000

void mdebugToggle (void)
{
  unsigned int count;
  unsigned int delta;

  if (mdChannel < 0)  {
    lastCount = TSCL;
    return;
  }

  count = TSCL;

  delta = count - lastCount;

  if (delta > TR_DELTA)  {

    if (lastState == 0)  {
      dregTdmCfg[mdChannel].control = 1;  /* enable the timeslot */
      lastState = 1;

    } else {

      dregTdmCfg[mdChannel].control = 0;  /* disable the timeslot */
      lastState = 0;
    }

    dregMasterCntrl.tdmCntrl.command = 1;

    lastCount = count;

  }

}
    
/*****************************************************************************
 * FUNCTION PURPOSE: Main execution loop
 ***************************************************************************** 
 * DESCRIPTION: The main execution loop handles processing of foreground
 *              tasks and tracks configuration changes.
 *  This function is required in the bootloader mode.
 *  The entry_code paramter provides information about the success of the 
 *  boot process from the initializtion code
 *****************************************************************************/

#if (defined(C6416)) || (defined(c6416))
#define GIE              0   
extern int savedIER;      /* Needed for c64x */
cregister unsigned int CSR;
cregister unsigned int IER;    
#endif
void dsiuRun (void)
{
  tint i;

#if 1
  /****************** TIMER FOR PROFILING */
  //Enable the Timer
  TSCL = 1;
#endif


  /***************************************************************************
   * Diagnostic setup. The diagnostic configuration registers must already
   * be set for execution to proceed properly.
   ***************************************************************************/
  dsiuAllocateDiagMemory ();  /* Allocate diag context heap memory */
  for (i = 0; i < diagCfg.maxChannels; i++)
    dsiuInitChannel (i); /* Allocate channel heap memory */

  /* Initialize and start an AIC Instance */
  if(dsiuContext.hal->aic) {
    /* set the sampling frequency */
    (*dsiuContext.hal->aic->halAicInitStart) (0, MGB_AIC_10, 32, 16);
  }

  /**************************************************************************
   * If configured, measure the number of CPU clocks between sub-frames.
   * This is used to verify both the system and frame sync clocks.
   **************************************************************************/
   if (diagCfg.reportDivisor)  {

     dsiuMeasureClockRatio ();
   }

  /* Configure and open the channels. Timeslots are not enabled until the
   * main execution loop */
  for (i = 0; i < diagCfg.maxChannels; i++) 
    dsiuOpenChannel (&dsiuInst[i]);

#ifdef DIAG_INCLUDE_OTBRUN
  /*setup bert test if number of bert channels > 0*/
  if(dregBertTestCntl.nCh>0)
  {
    dsiuBertTestSetup();
  }
#endif

  /* Main loop */
  for (;;)  {

    /* Track the number of times through the main loop */
    dsiuInc (&(dsiuContext.dregStatus->pLoopCounter));

    /* Check for requested changes in channel state */
    dsiuCheckChannels ();

    /* Process posted tasks */
    dsiuProcessTasks ();

    /* Process Polled Tasks */
    dsiuPollTasks ();

#ifdef DIAG_INCLUDE_TDMSIM
    if(dsiuSimConfig.opt_simulate_tsip_isr)
    {
      dsiuTDMIsr();
    }  
#endif

#if defined(C5472)
    if(config == 1)
    {
      config = 0;
      dsiuAnalogCodecConfig();
    }
#endif
  }
} /* dsiuRun */


#ifdef DIAG_INCLUDE_TDMSIM
/*****************************************************************************
 * FUNCTION PURPOSE: Simulate TDM ISR (Interrupt Service Routine)
 ***************************************************************************** 
 * DESCRIPTION: Used mainly in silicon or simulator where TDM ISR (TSIP) is not present
 *              Used to drive the TDM TX and RX tasks
 *              For BERT test, BERT samples are introduced later in the TX task
 *              If BERT is not enabled, user can choose a pattern to input to TX task
 *****************************************************************************/
void dsiuTDMIsr(void)
{
  tuint      ch,i;
  dsiuInst_t *inst;
  linSample *locFromTdm;
  linSample *locToTdm;
  tulong nowTSCL;
  static tulong pastTSCL=0; /*last recorded time stamp*/
  static tuint firstCh=0;/*first channel to run in this ISR*/
  tuint lastCh;/*last channel to run in this ISR*/
  static tulong isrPeriod=0; /*ISR period*/
  
  /*get new time*/
  nowTSCL=TSCL;
  
  /*compute ISR period only once*/
  if(!isrPeriod)
  {
     /*Period for the simulated TDM ISR (interrupt service routine). 
       In other words, it is the interval between TDM ISRs.*/
     /* How to compute the period: */
     /* 1ms = 10^3*(cpu_mhz)  CPU cycles*/
     /* Frame period in ms = DIAG_APP_FRAME_RATE_0/8 */
     /* ISR period in CPU cycles = (frame period in CPU cycles)/(number of ISR slots)*/
     isrPeriod = (1000*dsiuContext.core_mhz)*(DIAG_APP_FRAME_RATE_0/MGB_SAMP_FREQ_8KHZ)/DSIU_SIM_NUM_TDM_ISR_SLOTS;
  }      

  /*Only run next set of channels if ISR period has passed*/
  if  ((nowTSCL - pastTSCL) < isrPeriod)
  {
    /*one ISR period has not passed*/
    return;
  }   
  /*update recorded timestamp*/
  pastTSCL=nowTSCL;
  
  /*compute the last channel that should be processed in this ISR*/
  if((dsiuSimConfig.tsipIsrNumCh%DSIU_SIM_NUM_TDM_ISR_SLOTS)==0)
  {
    /*each ISR slot has the same number of channels*/
    lastCh = firstCh + dsiuSimConfig.tsipIsrNumCh/DSIU_SIM_NUM_TDM_ISR_SLOTS - 1;
  }
  else
  {
    /*add 1 channel to the ISR slots so that the last slot won't have extra channels*/
    lastCh = firstCh + dsiuSimConfig.tsipIsrNumCh/DSIU_SIM_NUM_TDM_ISR_SLOTS;
  } 
  
  /* channels are 0-based, so the last channel must be < tsipIsrNumCh */
  if(lastCh >= dsiuSimConfig.tsipIsrNumCh)
  {
    lastCh=dsiuSimConfig.tsipIsrNumCh-1;
  }
  
  /*simulates TDM ISR for channels that belong to this ISR slot*/
  for(ch = firstCh; ch <= lastCh; ch++) 
  {
  
    inst = &dsiuInst[ch];
 
    if(inst->state == DSIU_STATE_CLOSED)
    {
      dsiuPanic (DSIU_MEM_ALLOC_FAIL_CH_ERRS);
    }

    /*data from app-->TDM*/
    dpiuTdmTxExchange (inst->dpiuInst, 
                       &locToTdm, 
		       NULL,
                       0 /*time stamp ts*/, 
                       inst->tdmFrameSize);

    /*data from TDM-->app*/
    dpiuTdmRxExchange (inst->dpiuInst, 
                       &locFromTdm, 
                       &locFromTdm, 
                       0 /*time stamp ts*/,
                       inst->tdmFrameSize);

    /*if we are inputting a TDM pattern we need to fill the PIU buffer now
    and when the TX task is processed it will pick up this buffer*/
    if(dsiuSimConfig.opt_tdm_input_pattern==DSIU_SIM_TDM_PATTERN_ARRAY_PATTERN)
    {
      for(i=0;i<inst->tdmFrameSize;i++)
      {
        locFromTdm[i]=dsiuSimConfig.inputPattern[i];
      }
    }
    else if(dsiuSimConfig.opt_tdm_input_pattern==DSIU_SIM_TDM_PATTERN_CH_NUM_PATTERN)
    {
      for(i=0;i<inst->tdmFrameSize;i++)
      {
        locFromTdm[i]=ch;
      }
    }
  }
  
  /*update firstCh*/
  if(lastCh == dsiuSimConfig.tsipIsrNumCh - 1)
  {
    firstCh=0;
  }
  else
  {
    firstCh=lastCh+1;
  }
  
}

#endif /* DIAG_INCLUDE_TDMSIM */

/* Nothing past this point */

