/**************************************************************************** 
 * FILE PURPOSE: Implement system task queues
 ****************************************************************************
 * FILE NAME: dsiutask.c
 *
 * DESCRIPTION: Implements tasks queue
 *
 * TABS: None
 *
 * Copyright (C) 2002, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include "types.h"
#include "dsiutask.h"
#include <stdlib.h>

/*****************************************************************************
 * FUNCTION PURPOSE: Get element from task queue
 *****************************************************************************
 * DESCRIPTION: Returns head of task queue. Returns NULL if no tasks are in
 *              the queue
 *****************************************************************************/
void *dsiuGetTask (dsiuTask_t *task)
{
   void *retv;

   /* Return NULL on empty queue */
   if (task->inIndex == task->outIndex)
     return (NULL);

   /* Extract the top element from the queue */
   retv = task->tasks[task->outIndex];

   /* Advance the queue pointer */
   task->outIndex += 1;
   if (task->outIndex >= task->size)
     task->outIndex = 0;

   return (retv);
} /* dsiuGetTask */
   
   
/*****************************************************************************
 * FUNCTION PURPOSE: Post a task
 *****************************************************************************
 * DESCRIPTION: Posts a task into the specified queue. Returns FAIL if 
 *              the queue was full.
 *****************************************************************************/
dsiuTaskReturn_t dsiuPostTask (dsiuTask_t *task, void *inst)
{
  tuint newInIndex;
  /* Return failure if the task queue is full */
  newInIndex = task->inIndex + 1;
  if (newInIndex >= task->size)
    newInIndex = 0;
  if (newInIndex == task->outIndex)
    return (DSIU_TASK_FAIL);

  /* Place the task in the queue */
  task->tasks[task->inIndex] = inst;
  task->inIndex = newInIndex;

  return (DSIU_TASK_SUCCESS);
}


/*****************************************************************************
 * FUNCTION PURPOSE: Return the number of free tasks in the queue
 *****************************************************************************
 * DESCRIPTION: The number of available tasks in the queue is computed.
 *****************************************************************************/
tuint dsiuTaskSizeAvail (dsiuTask_t *task)
{
  tuint avail;

  if (task->inIndex >= task->outIndex)
    avail = task->size - (task->inIndex - task->outIndex) - 1;
  else
    avail = task->outIndex - task->inIndex - 1;

  return (avail);
} /* dsiuTaskSizeAvail */
  

/******************************************************************************
 * FUNCTION PURPOSE: Reset the task queue
 ******************************************************************************
 * DESCRIPTION: The task pointers are reset, deleting any pending tasks.
 ******************************************************************************/
void dsiuTaskClear (dsiuTask_t *task)
{
  task->inIndex = task->outIndex = 0;
}
   


