/*****************************************************************************
 * FILE PURPOSE: Bootup configuration through utopia driver
 *****************************************************************************
 * FILE NAME: dsiuubt.c
 *
 * DESCRIPTION: Contains code to setup system parameters based on a boot
 *              cell received from the utopia interface. The initial 
 *              configuration of the utopia is pre-determined. 
 * 
 * TABS: None
 *
 * Copyright (C) 2002, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include "types.h"
#include "dsiuloc.h"
#include "hal.h"
#include "dsiuubt.h"
#include "diagcfg.h"
#include "dsiumsg.h"
#include "diagver.h"
#include "diagapp.h"
#include "dsiuutop.h"
#include "dsiumem.h"
#include "dsiuport.h"
#include <stdlib.h>

/*****************************************************************************
 * DEFINITION: Cell receptions by the utopia are interrupt events. The 
 *             following global varible is set to true from an isr process
 *             when the boot cell has been received. This flags the foreground
 *             application (which is frozen in a loop) that the boot config
 *             cell has been received.
 *****************************************************************************/
extern volatile bool dsiuBootUtopiaCellReceived;

/*****************************************************************************
 * Definition: Global structure used to track the progress of received 
 *             boot cells.
 *****************************************************************************/
dsiuBCellTrack_t dsiuBCellTrack;
diagApp_t        *uDiagApp;


/*****************************************************************************
 * FUNCTION PURPOSE: Process received legacy cells.
 *****************************************************************************
 * DESCRIPTION: This function processes data in the legacy cell format.
 *****************************************************************************/
bool dsiu_boot_rcv_legacy_cell (dsiuBootCell_t *bootCell)
{
  dsiuRouteTagHeader_t *rth;

  /* If this cell is not flagged as an extended cell then verify no extended
   * boot cells were received prior to this one */
  if (dsiuBCellTrack.msg_rcvd)
    dsiuPanic (DSIU_FAIL_BOOT_CFG_ERR);


  /**************************************************************************
   * Legacy configuration for messages 
   **************************************************************************/
  uDiagApp->u.utopia.msg.rxPayloadType  = DIAGAPP_PROTO_NULL;
  uDiagApp->u.utopia.msg.ipOffsetBytes  = -1;
  uDiagApp->u.utopia.msg.udpOffsetBytes = -1;
  uDiagApp->u.utopia.msg.ethOffsetBytes = -1;


  /**************************************************************************
   * Flags 
   **************************************************************************/
  uDiagApp->u.utopia.msg.flags1 = 0;
  uDiagApp->u.utopia.msg.flags2 = 0;
  if ((tuint)bootCell->utopiaConfig & DSIU_UTOPIA_CONFIG_AAL5_CRC_GEN_MASK)
    uDiagApp->u.utopia.msg.flags2 |= DIAGAPP_F2_AAL5_GEN_CRC_BIT;

  if ((tuint)bootCell->utopiaConfig & DSIU_UTOPIA_CONFIG_AAL5_CRC_CHECK_MASK)
    uDiagApp->u.utopia.msg.flags2 |= DIAGAPP_F2_AAL5_CHK_CRC_BIT;



  /**************************************************************************
   * Copy the boot config information into the application setup structure
   **************************************************************************/
  uDiagApp->u.utopia.cellSize = ((tint)bootCell->u.cell0.utopiaCellSize);
  if (uDiagApp->u.utopia.cellSize == 58)  {

    rth = (dsiuRouteTagHeader_t *)&(bootCell->u.cell0.routeTagHeader[0]);
    uDiagApp->dspId = DSIU_UTOPIA_RT_EXTRACT_JANUS_ID(rth);

          /*  ------- 58 Byte Cells  --------  */
    uDiagApp->u.utopia.msg.totalHeaderLenBytes = 10;
    uDiagApp->u.utopia.msg.hdrImage[0] = (tuint)bootCell->u.cell0.routeTagHeader[0];
    uDiagApp->u.utopia.msg.hdrImage[1] = (tuint)bootCell->u.cell0.routeTagHeader[1];
    uDiagApp->u.utopia.msg.hdrImage[2] = (tuint)bootCell->u.cell0.atmHeader[0];
    uDiagApp->u.utopia.msg.hdrImage[3] = (tuint)bootCell->u.cell0.atmHeader[1];
    uDiagApp->u.utopia.msg.hdrImage[4] = (tuint)bootCell->u.cell0.atmHeader[2];


  }  else  { 

          /*  ------- 54 Byte Cells  --------- */
    uDiagApp->dspId = (tuint)bootCell->u.cell0.dspId;
    uDiagApp->u.utopia.msg.totalHeaderLenBytes = 6;

    uDiagApp->u.utopia.msg.hdrImage[0] = (tuint)bootCell->u.cell0.atmHeader[0];
    uDiagApp->u.utopia.msg.hdrImage[1] = (tuint)bootCell->u.cell0.atmHeader[1];
    uDiagApp->u.utopia.msg.hdrImage[2] = (tuint)bootCell->u.cell0.atmHeader[2];

    /* Details on Keep Alive cells from this DSP */
    uDiagApp->u.utopia.kpalive.kpAlive_gfc_vp_vcih = (tuint)bootCell->u.cell0.kpAlive_gfc_vp_vcih;
    uDiagApp->u.utopia.kpalive.kpAlive_vcil_pt_clp = (tuint)bootCell->u.cell0.kpAlive_vcil_pt_clp;  
    uDiagApp->u.utopia.kpalive.kpAliveAddr_hi = (tuint)bootCell->u.cell0.kpAliveAddr_hi;
    uDiagApp->u.utopia.kpalive.kpAliveAddr_lo = (tuint)bootCell->u.cell0.kpAliveAddr_lo;
  }
  tsiuContext.dspId = uDiagApp->dspId;

  /* Return indicating the boot information is complete */
  return (TRUE);

} /* dsiu_boot_rcv_legacy_cell */


/*************************************************************************** 
 * FUNCTION PURPOSE: Extended boot cell primary cell handler
 ***************************************************************************
 * DESCRIPTION: Processes the primary boot config cell in extended mode.
 *              This cell is typically the first one received, however
 *              this is not required.
 ***************************************************************************/
void dsiu_extboot_icell (dsiuBootCell_t *bootCell)
{

  /* Flag if this cell has already been processed */
  if (dsiuBCellTrack.boot0_rcvd)  {
    dsiuContext.bootErrs.extBoot += 1;
    return;
  }

  /* Basic configuration values */
  uDiagApp->dspId               = bootCell->u.cell0.dspId;
  tsiuContext.dspId             = bootCell->u.cell0.dspId;

  /* Keep Alive Address for this DSP */
  uDiagApp->u.utopia.kpalive.kpAliveAddr_hi = bootCell->u.cell0.kpAliveAddr_hi;
  uDiagApp->u.utopia.kpalive.kpAliveAddr_lo = bootCell->u.cell0.kpAliveAddr_lo;
  uDiagApp->u.utopia.kpalive.kpAlive_gfc_vp_vcih = (tuint)bootCell->u.cell0.kpAlive_gfc_vp_vcih;
  uDiagApp->u.utopia.kpalive.kpAlive_vcil_pt_clp = (tuint)bootCell->u.cell0.kpAlive_vcil_pt_clp;

  uDiagApp->u.utopia.cellSize   = bootCell->u.cell0.utopiaCellSize;

  /* Mark any sections that are listed as unconfigured */
  /* Messages */
  if (!(DSIU_UTOPIA_GET_MSG_FLAG(bootCell->utopiaConfig)))  {
    uDiagApp->u.utopia.msg.rxPayloadType = DIAGAPP_PROTO_NULL;
    if (dsiuBCellTrack.msg_rcvd)
      dsiuContext.bootErrs.invldType += 1;
    dsiuBCellTrack.msg_rcvd = DSIU_UTOPIA_BOOT_TYPE_UNUSED;
  }

} /* dsiu_extboot_icell */

/****************************************************************************
 * FUNCTION PURPOSE: Process data from the first cell in a group
 ****************************************************************************
 * DESCRIPTION: Verifies that the cell has not already been received. 
 *              Stores the data from the cell, and marks it as received.
 ****************************************************************************/
void dsiu_boot_rcv_fcell (diagAppHeader_t *hdr, dsiuBootCell_t *bootCell, tuint *mark)
{
  tuint unusedStart;
  tuint lenWords;
  tuint i;

  /* Do not process a second copy of the cell. The first cell in a group 
   * is flagged in bit position 0 */
  if (*mark & 0x0001)  {
    dsiuContext.bootErrs.extBoot += 1;
    return;
  }

  /* Copy the data from the cell to the header */
  hdr->flags1              = bootCell->u.cell1.flags1;
  hdr->flags2              = bootCell->u.cell1.flags2; 
  hdr->rxPayloadType       = bootCell->u.cell1.payloadType;
  hdr->ipOffsetBytes       = bootCell->u.cell1.ipOffsetBytes;
  hdr->udpOffsetBytes      = bootCell->u.cell1.udpOffsetBytes;
  hdr->ethOffsetBytes      = bootCell->u.cell1.ethOffsetBytes;
  hdr->totalHeaderLenBytes = bootCell->u.cell1.totalHeaderLenBytes;
  hdr->priDiffTag          = bootCell->u.cell1.priDiffTag;
  hdr->txMinCellBytes      = (bootCell->u.cell1.txMinCellWords << 1);
  hdr->rxOffsetBytes       = bootCell->u.cell1.rxOffsetBytes;

  /* Based on the header length, mark any header cells that are not required */
  lenWords = SIU_BYTES_TO_WORDS(hdr->totalHeaderLenBytes);
  unusedStart = ((lenWords + DSIU_UTOPIA_CELL_HDR_SIZE - 1) /
                  DSIU_UTOPIA_CELL_HDR_SIZE) + 1;
  for (i = unusedStart; i < DSIU_UTOPIA_MAX_CELL_INDEX; i++)
    *mark |= (1 << i);
  

  /* Record that the cell has been processed. The first cell in a group
   * is flagged in bit position 0 */
  *mark = *mark | 0x0001;

} /* dsiu_boot_rcv_fcell */

/******************************************************************************
 * FUNCTION PURPOSE: Process data from secondary boot cells
 ******************************************************************************
 * DESCRIPTION: Secondary boot cells contain header information. Verify that
 *              the cell has not already been received, then copy the header
 *              information.
 ******************************************************************************/
void dsiu_boot_rcv_cell (diagAppHeader_t *hdr, dsiuBootCell_t *bootCell, tint cIdx, tuint *mark)
{
  tuint mask;
  tint  start;
  tint  end;
  tint  size;

  /* The bit mask position is the index number */
  mask = (1 << cIdx);

  /* Verify that a cell with this ID has not already been processed */
  if (*mark & mask) {
    dsiuContext.bootErrs.extBoot += 1;
    return;
  }

  /* Determine where to put the data and how many words to save */
  start = (cIdx - 1) * DSIU_UTOPIA_CELL_HDR_SIZE;
  end   = start + DSIU_UTOPIA_CELL_HDR_SIZE;
  if (end > DIAGAPP_MAX_HDR_LEN_WORDS) 
    size = DIAGAPP_MAX_HDR_LEN_WORDS - start;
  else
    size = DSIU_UTOPIA_CELL_HDR_SIZE;

  /* Copy the data */
  memcpy (&(hdr->hdrImage[start]), bootCell->u.cell2.image, size);

  /* Mark the cell as read */
  *mark = *mark | mask;

} /* dsiu_boot_rcv_cell */

/****************************************************************************
 * FUNCTION PURPOSE: Check if all cells required for an operation are received
 ****************************************************************************
 * DESCRIPTION: Returns true if the status indicates that the group is unused
 *              or if used, that all required cells are received.
 *****************************************************************************/
bool dsiu_boot_group_complete (tuint group)
{
  /* Return true if the group is unused */
  if (group & DSIU_UTOPIA_BOOT_TYPE_UNUSED)
    return (TRUE);


  /* Return true if the group is complete */
  if ((group & DSIU_UTOPIA_BOOT_TYPE_COMPLETE) == DSIU_UTOPIA_BOOT_TYPE_COMPLETE)
    return (TRUE);

  /* Otherwise still waiting for cells or to be marked as unused */
  return (FALSE);

} /* dsiu_boot_group_complete */


/****************************************************************************
 * FUNCTION PURPOSE: Process an extended boot cell
 ****************************************************************************
 * DESCRIPTION: The extended boot cell is routed to the correct cell
 *              handler.
 *
 *              Returns TRUE when all required boot cells have been received
 ****************************************************************************/
bool dsiu_boot_rcv_extended_cell (dsiuBootCell_t *bootCell)
{
  diagAppHeader_t *hdr   = NULL;
  tuint           *mark;
  tint             cIdx;

  /* Identify which header the cell is for. Each cell contains a "serial number"
   * which is divided into a cell ID (cell group - messages, blocks, or init) and
   * an index number. */
  switch (DSIU_UTOPIA_GET_CELL_ID(bootCell->utopiaConfig))  {
   
    case DSIU_UTOPIA_CELL_ID_INIT:
      dsiu_extboot_icell (bootCell);
      dsiuBCellTrack.boot0_rcvd = 1;
      break; 

    case DSIU_UTOPIA_CELL_ID_MESSAGE:
      hdr  = &(uDiagApp->u.utopia.msg);
      mark = &(dsiuBCellTrack.msg_rcvd);
      break;

    default: /* Unknown cell ID. Drop the cell */
      dsiuContext.bootErrs.invldId += 1;
      return (FALSE);
  }

  /* Identify the sequence number and route the cell. The first cell in
   * each group contains header offset information and is handled seperately. */
  if (hdr)  {
    cIdx = DSIU_UTOPIA_GET_CELL_INDEX(bootCell->utopiaConfig);
    if (cIdx == 0) 
      dsiu_boot_rcv_fcell (hdr, bootCell, mark);
    else
      dsiu_boot_rcv_cell (hdr, bootCell, cIdx, mark);
  }

  /* Return true if all expected cells have been received */
  if ( dsiuBCellTrack.boot0_rcvd                              && 
      (dsiu_boot_group_complete (dsiuBCellTrack.msg_rcvd)) )
    return (TRUE);


  /* Still waiting for more cells */
  return (FALSE);

} /* dsiu_boot_rcv_extended_cell */


/*****************************************************************************
 * FUNCTION PURPOSE: Process received cells.
 *****************************************************************************
 * DESCRIPTION: This function is the hal callback when a cell is recieved.
 *              The cell is verified as a boot cell by checking the 
 *              magic values. Once confirmed as a boot cell the cell is
 *              routed to a handler based on the flags. 
 *
 *              There are two types of boot configuration - legacy in which
 *              a single boot cell is used, and mii, in which a network
 *              header is provided for messages and bulk memory read/write
 *              requests.
 *
 *              The boot config parameters are then copied to a pre-defined
 *              location in shared memory for use by the application when
 *              the bootloader has finished.
 *****************************************************************************/
void dsiuBootRcvCell (void *cxt, word *cell, tuint size)
{
  dsiuBootCell_t *bootCell;
  bool            complete;

  bootCell = (dsiuBootCell_t *)cell;

  DSIU_DUMMY_ARG(cxt);
  DSIU_DUMMY_ARG(size);


  /* Keep count of cells/interrupts */
  dsiuContext.utopiaPort[0].status->intCount    += 1;
  dsiuContext.utopiaPort[0].status->rxDataCount += 1;


  /* Check the magic IDs, return if not valid. This will also handle the
   * case where a second cell arrives at the utopia port before the system
   * has re-configured the HAL utopia cell handler */
  if ((bootCell->magic[0] != DSIU_UTOPIA_BOOT_MAGIC_0) ||
      (bootCell->magic[1] != DSIU_UTOPIA_BOOT_MAGIC_1))  {
      dsiuContext.bootErrs.invldMagic += 1;
      return;
  }

  /* Check for the enhanced boot cell format, verify and route */
  if (DSIU_UTOPIA_GET_TYPE(bootCell->utopiaConfig) == DSIU_UTOPIA_TYPE_EXTENDED) {
    complete = dsiu_boot_rcv_extended_cell (bootCell);

  } else {

    /* Otherwise this is a legacy boot config cell. */
    complete = dsiu_boot_rcv_legacy_cell (bootCell);
  }

  /* When all boot cells have been received, copy the information for the
   * application, and flag that the boot parameters have been received */
  if (complete)  {
    dsiu_boot_copy_info ();
    dsiuBootUtopiaCellReceived = TRUE;
  }

} /* dsiuBootRcvCell */


/*****************************************************************************
 * FUNCTION PURPOSE: Provide utopia specific configuration
 *****************************************************************************
 * DESCRIPTION: Loads the utopia initialization structure with boot specific
 *              parameters.
 *****************************************************************************/
void dsiuGetUtBootConfig (halUtopiaInit_t *ul2Init, halQuery_t *utQuery)
{
  dsiuPrevBootInfo_t info;

  if (diagCfg.ctable.dsiu->prevBootInfo)  {
    if ((*diagCfg.ctable.dsiu->prevBootInfo)(&info) != DSIU_SUCCESS)
      dsiuPanic (DSIU_FAIL_PREV_BOOT_INFO);
  }  else  {
      dsiuPanic (DSIU_FAIL_PREV_BOOT_INFO);
  }

  ul2Init->portId         = 0;
  ul2Init->busWidth       = utQuery->u.utopia.busWidth;
  ul2Init->slaveId        = info.u.utInfo.slaveId;
  ul2Init->mode           = HAL_UTOPIA_PASS_THRU;
  ul2Init->rcvCell        = dsiuBootRcvCell;
  ul2Init->rcvCxt         = NULL;
  ul2Init->rcvCellBufFull = NULL;

  ul2Init->sizes.xmtCellSizeBytes = info.u.utInfo.cellSizeBytes;
  ul2Init->sizes.rcvCellSizeBytes = info.u.utInfo.cellSizeBytes;
  ul2Init->sizes.txBufSizeBytes   = SIU_BYTES_TO_WORDS((tulong)diagCfg.utConfig->utopiaDmaMem->txEnd -
                                    (tulong)diagCfg.utConfig->utopiaDmaMem->txStart);
  ul2Init->sizes.rxBufSizeBytes   = SIU_BYTES_TO_WORDS((tulong)diagCfg.utConfig->utopiaDmaMem->rxEnd -
                                    (tulong)diagCfg.utConfig->utopiaDmaMem->rxStart);
 
  /***************************************************************************
   * The Utopia is now configured. Poll the Utopia until a cell is receved.
   ***************************************************************************/
  dsiuBootUtopiaCellReceived  = FALSE;
  dsiuBCellTrack.msg_rcvd     = 0;
  dsiuBCellTrack.boot0_rcvd   = 0;
  uDiagApp                    = &dsiuContext.diagApp;

  uDiagApp->u.utopia.slaveId  = info.u.utInfo.slaveId;

} /* dsiuGetUtBootConfig */

/*****************************************************************************
 * FUNCTION PURPOSE: Allocate the utopia buffers
 *****************************************************************************
 * DESCRIPTION: Allocates and verifies the buffers based on size 
 *              requirements and buffer availability.
 *****************************************************************************/
void dsiuSetUtopiaBootBuffers (halMemBuffer_t *bufs, tuint port)
{
  halMemBuffer_t *bp;
  tuint *bstart; 
  tulong bsize;
  tuint  alignUsed;


  /* Allocate the Heap buffer */
  bp = &bufs[HAL_HEAP_BUFN];
  if (bp->base == NULL)  {
    bstart = dsiuBufferAlign (diagCfg.halHeapBufStart + dsiuContext.halHeapUsed, diagCfg.halHeapBufEnd, 
                              bp->log2align, &bsize, &alignUsed);
    if (bsize < bp->size)
      dsiuPanic (DSIU_MEM_ALLOC_FAIL_HAL);

    bp->base = (void *)bstart;
    dsiuContext.halHeapUsed += (bp->size + alignUsed);
    DSIU_PORT_ALIGN_HEAPBASE(dsiuContext.halHeapUsed);
  }

  /* Allocate the tx buffer */
  bp = &bufs[HAL_UT_TX_BUFN];
  if (bp->base == NULL)  {
    bstart = dsiuBufferAlign (diagCfg.utConfig->utopiaDmaMem[port].txStart, 
                              diagCfg.utConfig->utopiaDmaMem[port].txEnd,
                              bp->log2align, &bsize, &alignUsed);
    if (bsize < bp->size)
      dsiuPanic (DSIU_MEM_ALLOC_FAIL_HAL); 
    bp->base = (void *)bstart;
  }

  /* Allocate the rx buffer */
  bp = &bufs[HAL_UT_RX_BUFN];
  if (bp->base == NULL)  {
    bstart = dsiuBufferAlign (diagCfg.utConfig->utopiaDmaMem[port].rxStart, 
                              diagCfg.utConfig->utopiaDmaMem[port].rxEnd,
                              bp->log2align, &bsize, &alignUsed);
    if (bsize < bp->size)
      dsiuPanic (DSIU_MEM_ALLOC_FAIL_HAL);
    bp->base = (void *)bstart;
  }

} /* dsiuSetUtopiaBootBuffers */


/*****************************************************************************
 * FUNCTION PURPOSE: Read boot configuration from global memory
 *****************************************************************************
 * DESCRIPTION: This function is used to simulate the application. It
 *              reads the configuration stored in shared memory by the 
 *              bootloader.
 ****************************************************************************/
dsiuReturn_t dsiuUtSimuAppConfig (void)
{

  dsiuSimuAppConfig ();

  return (DSIU_SUCCESS);

} /* dsiuSimuAppConfig */


  
  
  
