#ifndef _DSIUUBT_H
#define _DSIUUBT_H
/****************************************************************************
 * FILE PURPOSE: Utopia boot configuration definitions
 ****************************************************************************
 * FILE NAME: dsiuubt.h
 *
 * DESCRIPTION: Contains definitions used to implement reception of a boot
 *              config cell. These definitions are taken directly from the
 *              Janus Diagnostics & Bootloader Code Software Design Document,
 *              version 1.42.
 *
 * TABS: None
 *
 * Copyright (C) 2002, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include "types.h"

/***************************************************************************
 * DEFINITION: Body of the initial boot cell
 ***************************************************************************/
typedef struct {
  tint utopiaCellSize;      /* Utopia cell size              */
  tint routeTagHeader[2];   /* Route Tag Header              */
  tint atmHeader[3];        /* ATM Header                    */
  tint dspId;               /* DSP ID                        */
  tint kpAlive_gfc_vp_vcih; /* First 16bit of header for 
                             * keep alive AAL5 cells from DSP. This will
                             * comprise of GFC,VPI and high 4 bits 
                             * of VCI
                             */
  tint kpAlive_vcil_pt_clp; /* Second word of header for keep alive 
                             * AAL5 cells from DSP.This will comprise of low 
                             * 12 bits of VCI field
                             */
  tint kpAliveAddr_hi;      /* 16bit MSB
                             * Address in the payload for keep alive cells
                             * from the DSP 
                             */
  tint kpAliveAddr_lo;      /* 16bit LSB
                             * Address in the payload for keep alive cells
                             * from the DSP
                             */
  tint rsvd[4];             /* Reserved                      */
} dsiuBootCell0_t;

/*****************************************************************************
 * DEFINITION: Body of the Header 1 (H1) type cell
 *****************************************************************************/
typedef struct {
  tint flags1;               /* Flags 1 used by NEU           */
  tint flags2;               /* Flags 2 used by NEU           */
  tint payloadType;          /* Payload type used by NEU      */
  tint ipOffsetBytes;        /* IP header offset, bytes       */
  tint udpOffsetBytes;       /* UDP header offset, bytes      */
  tint ethOffsetBytes;       /* Ethernet header offset, bytes */
  tint totalHeaderLenBytes;  /* Total header length, bytes    */
  word priDiffTag;           /* Priority differentiation tag  */
  word txMinCellWords;       /* Min tx cell size, words       */
  word rxOffsetBytes;        /* Receive offset, bytes         */
  word rsvd[5];              /* Reserved                      */
} dsiuBootCellH1_t;

/*****************************************************************************
 * DEFINITION: Body of the Header 2 (H2) type cell
 *****************************************************************************/
#define DSIU_UTOPIA_CELL_HDR_SIZE   15
typedef struct {
  tint image[DSIU_UTOPIA_CELL_HDR_SIZE];   /* Tx Header data */
} dsiuBootCellH2_t;



/*****************************************************************************
 * DEFINITION: Format of the initial boot config cell
 *****************************************************************************/
typedef struct {
  tint atmHeader1[3];     /* ATM header                    */
  tint padding[6];        /* Padding                       */
  tint utopiaConfig;      /* Utopia configuration bit map  */
  union {
    dsiuBootCell0_t  cell0;
    dsiuBootCellH1_t cell1;
    dsiuBootCellH2_t cell2;
  } u;
  tint magic[2];
} dsiuBootCell_t;
  
  

/*************************************************************************************
 * UtopiaConfig Bits for cell 0
 *
 *      15      |     14     |    13 12 11 10 9 8 7 6 5 4 3 |        2         |      1         |   0
 *  <type_flag> | <msg_flag> |           <rsvd>             | <aal5_crc_check> | <aal5_crc_gen> | <mphy>
 *
 *
 *  type_flag: Set to 1 for MII mode, 0 for legacy mode
 *  msg_flag:  If set the bootloader expects up to 4 message config cells
 *
 *  aal5_crc_check: If set the DSP will check the aal5 crc on on incoming assemblies.
 *  
 *  aal5_crc_gen: If set the DSP will generate the aal5crc on outgoing assemblies
 *
 *  mphy:  0 = single phy mode
 *         1 = multi phy mode
 *
 *************************************************************************************
 * UtopiaConfig bits for other cells
 *
 *     15      | 14 13 12 11 10 9 8 | 7 6 5 4 3 2 |     1 0
 * <type_flag> |    <rsvd>          |  <cell_id>  | <cell_index>
 *
 * type_flag: Always 1 for cells other then the legacy boot cell
 *
 *     cell_id   cell_index    Boot cell                Cell type
 *       0          XX        Initial boot cell             0
 *       1          00        message config cell 1        H1
 *       1          01        message config cell 2        H2
 *       1          10        message config cell 3        H2
 *       1          11        message config cell 4        H2
 * 
 ****************************************************************************************/
#define DSIU_UTOPIA_GET_TYPE(x)          ((x)>>15)
#define DSIU_UTOPIA_GET_CELL_ID(x)      (((x)>>2) & 0x003F)
#define DSIU_UTOPIA_GET_CELL_INDEX(x)    ((x)&0x0003)
#define DSIU_UTOPIA_GET_FLAGS(x)        ((x)&0x0007)

#define DSIU_UTOPIA_GET_MSG_FLAG(x)      (((x)>>14)&0x0001)
#define DSIU_UTOPIA_GET_BLK_FLAG(x)      (((x)>>13)&0x0001)

#define DSIU_UTOPIA_TYPE_EXTENDED       1

#define DSIU_UTOPIA_MAX_CELL_INDEX      4

#define DSIU_UTOPIA_CONFIG_AAL5_CRC_GEN_MASK   (1<<1)
#define DSIU_UTOPIA_CONFIG_AAL5_CRC_CHECK_MASK (1<<2)


/* Cell ID definitions */
#define DSIU_UTOPIA_CELL_ID_INIT     0
#define DSIU_UTOPIA_CELL_ID_MESSAGE  1
#define DSIU_UTOPIA_CELL_ID_BLOCK    2

/*****************************************************************************
 * Boot cell reception progress
 *****************************************************************************/
#define DSIU_UTOPIA_BOOT_TYPE_COMPLETE  0x000F
#define DSIU_UTOPIA_BOOT_TYPE_UNUSED    0x8000
typedef struct {
  tuint      boot0_rcvd;
  tuint      msg_rcvd;
} dsiuBCellTrack_t;


/*****************************************************************************
 * DEFINITION: Magic values for boot cells
 *****************************************************************************/
#define DSIU_UTOPIA_BOOT_MAGIC_0   0x544e
#define DSIU_UTOPIA_BOOT_MAGIC_1   0x2164

/*****************************************************************************
 * DEFINITION: Utopia configuration parameters
 *****************************************************************************/
#define DSIU_UTOPIA_BOOT_CELL_SIZE     54
#define DSIU_UTOPIA_BOOT_CELL_BUFF_SIZE  10 


/* The boot function and boot ack prototypes */
tint dsiuBootUtopia (void);
dsiuReturn_t dsiuUtSimuAppConfig (void);
 


#endif /* _DSIUUBT_H */
