/*************************************************************************************
 * FILE PURPOSE: Utopia boot control
 *************************************************************************************
 * FILE NAME: dsiuubt1.c
 *
 * DESCRIPTION: Performs utopia boot control
 *
 * TABS: None
 * 
 * Copyright (C) 2004, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include "types.h"
#include "dsiuloc.h"
#include "hal.h"
#include "diagcfg.h"
#include "dsiumem.h"
#include "dsiuport.h"
#include "dsiuubt.h"
#include <stdlib.h>


/**************************************************************************************
 * Definition: Global variable. This variable is set when a complete boot cell
 *             sequence has been received. Note that the name of this variable
 *             is shared between this file and the combined mac/utopia boot loader
 **************************************************************************************/
volatile bool dsiuBootUtopiaCellReceived;   /* Set in dsiuubt.c */


/************************************************************************************
 * FUNCTION PURPOSE: Boot through utopia interface
 ************************************************************************************
 * DESCRIPTION: Checks for the utopia peripheral and configures for reception of
 *              a boot cell. 
 ************************************************************************************/
tint dsiuBootUtopia (void)
{
  halQuery_t          utQuery;
  halUtopiaInit_t     utInit;
  halUtopiaControl_t  utCtl;
  halReturn_t         ret;
  halMemBuffer_t     *bufs;
  tuint               origHeapSize;
  tint                nbufs;

  tuint      timestamp = 0;
  volatile tuint i;

  /* Intialize the global */
  dsiuBootUtopiaCellReceived = FALSE;

  /* Query for utopia configuration */
  utQuery.periph  = HAL_QUERY_UTOPIA;
  utQuery.portNum = 0;
  (*dsiuContext.hal->halQuery) (&utQuery);

  /* Panic if there is no utopia available */
  if ( (utQuery.isPresent == FALSE) || (utQuery.isAvail == FALSE) )  {
    dsiuPanic (DSIU_FAIL_NO_BOOT_INTERFACE);
  }

  origHeapSize = dsiuContext.halHeapUsed;

  /* Get Utopia specific configuration */
  dsiuGetUtBootConfig (&utInit, &utQuery);

  ret = (*dsiuContext.hal->utopia->halUtopiaGetSizes) (&nbufs, &bufs, &utInit.sizes);
  if (ret != HAL_SUCCESS)
    dsiuPanic (DSIU_FAIL_UTOPIA_GETSIZES);

  /* Allocate the HAL buufers */
  dsiuSetUtopiaBootBuffers (bufs, utInit.portId);


  /* Indicate that the port status is allocated, but uninitialized */
  UTL_SET_BITFIELD(dsiuContext.utopiaPort[0].status->idNumStatus, utInit.portId, 11, 8);
  UTL_SET_BITFIELD(dsiuContext.utopiaPort[0].status->idNumStatus, DREG_PERIPH_STATUS_UNINITIALIZED, 7, 0);

  /* Configure the port */
  ret = (*dsiuContext.hal->utopia->halUtopiaInit) (&utInit, nbufs, bufs);
  if (ret != HAL_SUCCESS)
    dsiuPanic (DSIU_FAIL_UTOPIA_INIT);

  /* Store the context information */
  dsiuContext.utopiaPort[0].physPort = utInit.portId;
  dsiuContext.utopiaPort[0].halId    = utInit.halId;
  UTL_SET_BITFIELD(dsiuContext.utopiaPort[0].status->idNumStatus, DREG_PERIPH_STATUS_CFGD_DISABLED, 7, 0);

  /* Enable the port */
  utCtl.mode = HAL_UTOPIA_PORT_ENABLE;
  if ((*dsiuContext.hal->utopia->halUtopiaControl)(dsiuContext.utopiaPort[0].halId, &utCtl) != 0)
    dsiuPanic (DSIU_FAIL_UTOPIA_ENABLE);

  UTL_SET_BITFIELD(dsiuContext.utopiaPort[0].status->idNumStatus, DREG_PERIPH_STATUS_CFG_ENABLED, 7, 0);


  /* Wait for the boot cell */
  while (dsiuBootUtopiaCellReceived == FALSE)  {
    for (i = 0; i < 60000; i++); 
    (*dsiuContext.hal->halPoll) (timestamp++, 4);
  }

  /****************************************************************************
   * A valid boot cell has been received, and the boot config parameters 
   * have been stored. Disable the utopia port and return the hal
   * heap memory used.
   ****************************************************************************/
  utCtl.mode = HAL_UTOPIA_PORT_DISABLE;
  if ((*dsiuContext.hal->utopia->halUtopiaControl)(dsiuContext.utopiaPort[0].halId, &utCtl) != 0)
    dsiuPanic (DSIU_FAIL_UTOPIA_DISABLE);

  UTL_SET_BITFIELD(dsiuContext.utopiaPort[0].status->idNumStatus, DREG_PERIPH_STATUS_UNALLOCATED, 7, 0);
  dsiuContext.utopiaPort[0].status->intCount    = 0;
  dsiuContext.utopiaPort[0].status->rxDataCount = 0;
  dsiuContext.utopiaPort[0].status->txDataCount = 0;
  dsiuContext.utopiaPort[0].status->rxDrop      = 0;
  dsiuContext.utopiaPort[0].status->txDrop      = 0;
  dsiuContext.halHeapUsed = origHeapSize;

  /****************************************************************************
   * Due to a problem in the reset of PG2 Janus, a work around is required
   * on bootup. This function is not call tabled - the define will only be
   * used for Janus.
   ****************************************************************************/
#ifdef DSIU_JANUS_PG2_BOOT_ISSUE
  return (dsiuCheckSecondaryBoot());
#else
  return (DSIU_SUCCESS);
#endif

} /* dsiuBootUtopia */

