/*************************************************************************************
 * FILE PURPOSE: Bootup configuration through either utopia or gmac
 *************************************************************************************
 * FILE NAME: dsiuumbt.c
 *
 * DESCRIPTION: Configures the utopia and gmac ports to receive data containing
 *              system peripheral parameters.
 *
 * TABS: None
 *
 * Copyright (C) 2004, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include "types.h"
#include "dsiuloc.h"
#include "hal.h"
#include "diagcfg.h"
#include "dsiumem.h"
#include "dsiuport.h"
#include "dsiuubt.h"
#include <stdlib.h>

/************************************************************************************
 * Definition: Global variables. These variables are set when a complete boot cell
 *             sequence or boot packet sequence have been received. Note that these
 *             variable names are shared between this file and the individual mac
 *             and utopia boot loaders.
 ************************************************************************************/
volatile bool dsiuBootUtopiaCellReceived;   /* Set in dsiuubt.c */
volatile bool dsiuBootMacPacketReceived;    /* Set in dsiumbt.c */


/*************************************************************************************
 * FUNCTION PURPOSE: Configure for boot cell/packet reception
 *************************************************************************************
 * DESCRIPTION: Checks the device for utopia and gmac peripheral, configures
 *              them for reception of a boot cell/packet.
 *************************************************************************************/
tint dsiuBootUtGmac (void)
{
  halQuery_t          utQuery;
  halQuery_t          macQuery;
  halUtopiaInit_t     utInit;
  halGmacInit_t       macInit;
  halGmacSizes_t      sizes;
  halMemBuffer_t     *bufs;
  halUtopiaControl_t  utCtl;
  halReturn_t         ret;
  tint                nbufs;
  dsiuPort_t         *dport;

  tuint timestamp = 0;
  volatile tuint i;

  /* Initialize the globals */
  dsiuBootUtopiaCellReceived = FALSE;
  dsiuBootMacPacketReceived  = FALSE;

  /* Hal main struct is already configured in dsiuMain() */
  
  /* configure the required boot port and wait for boot config msg */

  /* Query hardware for utopia configuration */
  utQuery.periph  = HAL_QUERY_UTOPIA;
  utQuery.portNum = 0;
  hal.halQuery (&utQuery);

  /* Query hardware for mac configuration */
  macQuery.periph  = HAL_QUERY_MAC;
  macQuery.portNum = 0;
  hal.halQuery (&macQuery);

  /* If neither of the interfaces was present, panic */
  if  (   ( (utQuery.isPresent  == FALSE) || (utQuery.isAvail  == FALSE) )   &&
          ( (macQuery.isPresent == FALSE) || (macQuery.isAvail == FALSE) )      )  {
    dsiuPanic (DSIU_FAIL_NO_BOOT_INTERFACE);
  }

  /* Setup the utopia port if it exists and is enabled */
  if ( (utQuery.isPresent) && (utQuery.isAvail) && (dsiuContext.hal->utopia != NULL) )  {
    dsiuGetUtBootConfig (&utInit, &utQuery);

    /* Get memory requirements */
    ret = (*dsiuContext.hal->utopia->halUtopiaGetSizes) (&nbufs, &bufs, &utInit.sizes);
    if (ret != HAL_SUCCESS)
      dsiuPanic (DSIU_FAIL_UTOPIA_GETSIZES);
    
    /* Allocate the utopia buffers */
    dsiuSetUtopiaBootBuffers (bufs, utInit.portId);

    /* Init and Start the utopia interface */
    ret = (*dsiuContext.hal->utopia->halUtopiaInit) (&utInit, nbufs, bufs);
    if (ret != HAL_SUCCESS)
      dsiuPanic (DSIU_FAIL_UTOPIA_INIT);

    /* Store context information */
    dsiuContext.utopiaPort[i].physPort = utInit.portId;
    dsiuContext.utopiaPort[i].halId    = utInit.halId;
    DREG_PERIPH_SET_STATUS (dsiuContext.utopiaPort[i].status->idNumStatus, DREG_PERIPH_STATUS_CFGD_DISABLED);


    /* Enable the port */
    utCtl.mode = HAL_UTOPIA_PORT_ENABLE;
    if ((*dsiuContext.hal->utopia->halUtopiaControl)(dsiuContext.utopiaPort[i].halId, &utCtl) != 0)
      dsiuPanic (DSIU_FAIL_UTOPIA_ENABLE);

    DREG_PERIPH_SET_STATUS (dsiuContext.utopiaPort[i].status->idNumStatus, DREG_PERIPH_STATUS_CFG_ENABLED);
  }

  /* Setup the mac port if it exists and is enabled */
  if ( (macQuery.isPresent) && (macQuery.isAvail) && (dsiuContext.hal->gmac != NULL) )  {
    dsiuGetMacBootSizes (&sizes);

    /* Get memory requirements from HAL  */
    ret = (*dsiuContext.hal->gmac->halGmacGetSizes) (&nbufs, &bufs, &sizes);
    if (ret != HAL_SUCCESS)
      dsiuPanic (DSIU_FAIL_MAC_GETSIZES);
  
    /* Allocate the gmac buffers. Logical port 0 is always used for boot */
    dsiuSetMacBootBuffers (bufs, 0);
    macInit.halId = (*dsiuContext.hal->gmac->halGmacAlloc) (nbufs, bufs, &sizes);

    dsiuGetMacBootConfig (&macInit, &sizes);
    dsiuContext.diagApp.portNum = macInit.physPort;

    /* Start the mac interface */
    macInit.operation = HAL_GMAC_INIT_OPERATION_ENABLE;
    dport = &(dsiuContext.macPort[0]);
    dport->physPort = macInit.physPort;
    dport->halId    = macInit.halId;
    ret = (*dsiuContext.hal->gmac->halGmacInit) (&macInit);
    if (ret != HAL_SUCCESS)
      dsiuPanic (DSIU_FAIL_MAC_INIT);

  }


  /* Wait for configuration information */
  while ( (dsiuBootUtopiaCellReceived == FALSE) && (dsiuBootMacPacketReceived == FALSE)  )  {
    for (i = 0; i < 60000; i++);
    if (hal.halPoll)
      (*hal.halPoll) (timestamp++, 4);
  }

  return (DSIU_SUCCESS);

} /* dsiuBootUtGmac */
    










