/******************************************************************************
 * FILE PURPOSE: System Level Utopia handling functions
 ******************************************************************************
 * FILE NAME: dsiuutop.c
 *
 * DESCRIPTION: Provides system level interface to the utopia port
 *
 * TABS: None
 *
 * Copyright (C) 2002, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include "dsiuloc.h"
#include "dsiuutop.h"
#include "hal.h"
#include "diagcfg.h"
#include "dsiumsg.h"
#include <stdlib.h>

dsiuUtopiaCallTable_t dsiuUtopiaCallTable = {
  dsiuUtopiaTxCell,
  dsiuUtopiaRxCell,
  dsiuUtopiaSendMessageToHost,
  dsiuUtopiaCellRcvd,
  dsiuUtopiaInit
};

/*****************************************************************************
 * FUNCTION PURPOSE: Initialization of utopia routing
 *****************************************************************************
 * DESCRIPTION: In the diagnostic code, only a single copy of the route tags
 *              and ATM header is stored for all channels. The route tag type
 *              (for route tag mode) or VCI field (VC demux mode). This
 *              function initializing this routing information to be for
 *              messages. No other function modifies these global fields.
 *****************************************************************************/
void dsiuUtopiaInit (void)
{
  dsiuRouteTagHeader_t *rth;
  dsiuVcDemuxHeader_t  *vcd;

  /* Initialization depends on routing mode */
  if (dsiuContext.dreg->utopiaCellSize == 58)  {
    /* Route tag mode */
    rth = (dsiuRouteTagHeader_t *)dsiuContext.dreg->routeTagHeader;
    DSIU_UTOPIA_RT_SET_TYPE(rth,DSIU_UTOPIA_RT_TYPE_MSG);
  }
  else if (dsiuContext.dreg->utopiaCellSize == 54)  {
    /* VC demux mode */
    vcd = (dsiuVcDemuxHeader_t *)dsiuContext.dreg->atmHeader;
    DSIU_UTOPIA_VC_SET_VCI_CHANNEL(vcd,DSIU_UTOPIA_VC_MSG_CHANNEL);
  }
  /* Else there is an invalid configuration */
} /* dsiuUtopiaInit */
    
    


/*****************************************************************************
 * FUNCTION PURPOSE: Receive aal5 message cells
 *****************************************************************************
 * DESCRIPTION: When the final cell has been received a message task is
 *              posted. Note that a message queue is not used, so the host
 *              must be sure that the message has been processed before 
 *              the next message is sent.
 *****************************************************************************/
void dsiu_receive_message_cell (globData_t *data, tint channel)
{
  tint       pktSizeBytes;
  void      *pktIn;
  tuint      header[DSIU_MSG_SUB_HEADER_SIZE]; /* msg length, channel field */
  globData_t mdata;
  dmsgVcDemuxMsgHeader_t *vcdh;
  dmsgHeader_t           *msgh;

  /* Format parameters for aal5 cell re-assembly */
  pktSizeBytes = data->dataSize * 2;  /* Convert word length to byte length */
  pktIn        = data->data;

  /* Perform re-assembly */
  if (diagCfg.ctable.daal5->aal5_receive_process(dsiuContext.aal5Inst, 
      &pktSizeBytes, &pktIn, dsiuContext.aal5CrcCheck, NULL, NULL))  {
    /* A complete message has been received. The recomputed message does not
     * contain the full message header, though. The length and channel are not
     * present. Push the message into the message processing fifo */

    /* Create the message structure an put the packet into the message fifo */
    if (dsiuContext.dreg->utopiaCellSize == 54)  {
      /* In 54 byte mode (VC demux) the channel information is contained
       * in the 1st two words of the cell. These 1st two words will be
       * replaced with the standard message  */
      vcdh = (dmsgVcDemuxMsgHeader_t *)pktIn;
      msgh = (dmsgHeader_t *)pktIn;
  
      /* The total message length is the same as the total aal5 packet */
      msgh->length  = (pktSizeBytes+1) / 2;
      msgh->channel =  vcdh->priority_core_channel & 
                       DSIU_MSG_VCDEMUX_CHANNEL_MASK;

      /* Place the packet into the fifo */
      mdata.dataSize = msgh->length;
      mdata.data     = (void *)msgh;

      if (dsiuFifoIn (dsiuContext.msgRxFifo, &mdata) == NULL)  {
        /* There was no room in the fifo, exit */
        dsiuContext.errs.msgRxFifoFull += 1;
        return;
      }
    } 
    else  { /* else 58 byte cells, route tag mode, are used */
      /* Header 0 is the complete length of the message, in words. The
         two word sub header (length and channel) must be added */
      header[0] = ((pktSizeBytes+1) / 2) + DSIU_MSG_SUB_HEADER_SIZE;
      header[1] = channel;

      /* Put the sub-header into the message rx fifo */
      mdata.dataSize = DSIU_MSG_SUB_HEADER_SIZE;
      mdata.data     = header;
      if (dsiuFifoIn (dsiuContext.msgRxFifo, &mdata) == NULL)  {
        /* There was no room in the fifo, exit */
        dsiuContext.errs.msgRxFifoFull += 1;
        return;
      }

      /* Put the remaining header and the message body into the fifo */
      mdata.dataSize = (pktSizeBytes + 1) >> 1;
      mdata.data     = pktIn;
      if (dsiuFifoIn (dsiuContext.msgRxFifo, &mdata) == NULL)  {
        /* There was no room in the fifo. Back out the header and exit */
        dsiuContext.errs.msgRxFifoFull += 1;
        dsiuFifoBackout (dsiuContext.msgRxFifo, DSIU_MSG_SUB_HEADER_SIZE);
        return;
      }
    } /* end cell size if */
    
    /* Post a task to process the message */
    dsiuPostTask (dsiuContext.msgTasks, dsiuContext.msgRxBuf);
  } /* end aal5 complete */

} /* dsiu_receive_message_cell */

/******************************************************************************
 * FUNCTION PURPOSE: Format a message to be passed to host
 ******************************************************************************
 * DESCRIPTION: For the diag code, messages can only be passed through the
 *              utopia interface. Message are therefore always passed as one
 *              or more AAL5 cells. 
 *
 *              There are no channel specific messages in the diag code. 
 *              because of this, the route tags, if used, and ATM header
 *              are unchanged for messages.
 *****************************************************************************/
dsiuReturn_t dsiuUtopiaSendMessageToHost (dmsg_t *msg)
{
  tint pktSizeBytes;
  void *pktIn;
  tuint *p, *q;
  halUtopiaData_t data;
  tuint sizes[1] ;

  /* The message header passed from the DSP does not contain a length and
   * channel field. Convert the message length to bytes less the length and
   * channel fields. */
  pktSizeBytes = (msg->header.length - 2) * 2;
  pktIn = (void *) &(msg->header.msgId);

  /* Segment the message into multiple utopia cells. The aal5inst has
   * already been configured with the correct route tags and atm header. */
  diagCfg.ctable.daal5->aal5_send_process (dsiuContext.aal5Inst, 
       &pktSizeBytes, &pktIn, dsiuContext.aal5CrcGen);

  /* The segmented packets are now stored as consecutive aal5 cells. The total
   * length of the cells is now in pktSizeBytes, and the cells are located
   * at *pktIn. Route the cells to the network interface. Note that this
   * next block of code relies on the fact that an even number of bytes is
   * in each cell. */
  p = (tuint *)pktIn;
  data.nSegs    = 1;
  data.type     = 0;  /* What should this be? */

    sizes[0]    = pktSizeBytes ;
    data.sizes  = &sizes[0];
    q = &p[0];
    data.segs = &q;
    dsiuContext.hal->halUtopiaSendCell (0, 0, &data);

  return (DSIU_SUCCESS);
} /* dsiuSendMessageToHost */
    

/*****************************************************************************
 * FUNCTION PURPOSE: Format a cell and send to the utopia driver
 *****************************************************************************
 * DESCRIPTION: Places header onto the input packet and sends it to the 
 *              utopia driver. This function is used only to send aal1
 *              cells.
 *****************************************************************************/
dsiuReturn_t dsiuUtopiaTxCell (dsiuInst_t *inst, globData_t *data)
{
  halUtopiaData_t haldata;
  dsiuRouteTagHeader_t *rth;
  dsiuVcDemuxHeader_t  *vcd;
  tuint  cell[DSIU_UTOPIA_MAX_CELL_SIZE];  /* assembly buffer */
  tuint  *cell_p;
  tuint  size;
  tint  p = 0;

  /* Copy the header to the assembly buffer */
  switch (dsiuContext.dreg->utopiaCellSize)  {
    case 58:  /* Route Tag mode */
      memcpy (&cell[p], dsiuContext.dreg->routeTagHeader, 
              DREG_RTAG_HEADER_LEN_WORDS);
      p += DREG_RTAG_HEADER_LEN_WORDS;

      /* Set the type, core ID and channel id fields */
      rth = (dsiuRouteTagHeader_t *)cell;
      DSIU_UTOPIA_RT_SET_TYPE(rth, DSIU_UTOPIA_RT_TYPE_AAL1);
      DSIU_UTOPIA_RT_SET_CORE_ID(rth, dsiuContext.dreg->dspId);
      DSIU_UTOPIA_RT_SET_CHANNEL(rth, inst->Id);
      
      /* Copy the ATM header without changes */
      memcpy (&cell[p], dsiuContext.dreg->atmHeader,
              DREG_ATM_HEADER_LEN_WORDS);
      p += DREG_ATM_HEADER_LEN_WORDS;
      break;
   
    case 54:  /* VC Demux mode */
      memcpy (&cell[p], dsiuContext.dreg->atmHeader,
               DREG_ATM_HEADER_LEN_WORDS);
      p += DREG_ATM_HEADER_LEN_WORDS;

      /* Set the vci core and vci channel fields */
      vcd = (dsiuVcDemuxHeader_t *)cell;
      DSIU_UTOPIA_VC_SET_VCI_CORE(vcd, dsiuContext.dreg->dspId);
      DSIU_UTOPIA_VC_SET_VCI_CHANNEL(vcd, inst->Id);
      break;

    /* Default case not required */
  } /* end  switch (dsiuContext.dreg->utopiaCellSize) */

  /* Copy the payload */
  memcpy (&cell[p], data->data, data->dataSize*sizeof (cell[0]));


  /* Setup HAL format */
  haldata.nSegs = 1;
  cell_p        = cell;
  haldata.segs  = &cell_p;
  size          = dsiuContext.dreg->utopiaCellSize;
  haldata.sizes = &size;
  haldata.type  = 0; /* Huh? */

  /* Send the Cell */
  if(dsiuContext.hal->halUtopiaSendCell)
    dsiuContext.hal->halUtopiaSendCell (0, 0, &haldata);

  return (DSIU_SUCCESS);
} /* dsiuUtopiaTxCell */

/******************************************************************************
 * FUNCTION PURPOSE: HAL driver cell received handler
 ******************************************************************************
 * DESCRIPTION: This function is called by the HAL driver when a utopia cell
 *              has been received. This function decodes the routing
 *              information and posts a task for the appropriate processing
 *              function (message/aal1)
 *****************************************************************************/
void dsiuUtopiaCellRcvd (tuint utopiaChannelId, word *cell, tuint size)
{
  tuint *buff_ptr;
  globData_t data;

  /* Verify that the size is valid, then post a utopia rx cell task */
  if (size != dsiuContext.dreg->utopiaCellSize) 
    dsiuContext.errs.utopiaCellSizeErr += 1;
  else {
    /* Place the Cell into the rx net fifo */
    data.data     = (tuint *)cell;
    data.dataSize = (size + 1) >> 1;   /* Convert size to words */
    buff_ptr = dsiuFifoIn (dsiuContext.netFifo, &data);
    if (buff_ptr == NULL)
      dsiuContext.errs.rxNetFifoFull += 1;
    else
      dsiuPostTask (dsiuContext.rxUtopiaTasks, (void *)buff_ptr);
  }
} /* dsiuUtopiaCellRcvd */


/****************************************************************************
 * FUNCTION PURPOSE: Process received utopia cell
 ****************************************************************************
 * DESCRIPTION: This is the rx cell task handler.  The cell size has already
 *              been verified before the task was posted. The cell parameter
 *              recevied does point to the base of the cell, but the cell is
 *              in the rxNet fifo, and could be wrapped.
 ****************************************************************************/
dsiuReturn_t dsiuUtopiaRxCell (dsiuCell_t *cell)
{
  dsiuRouteTagHeader_t *rth;
  dsiuVcDemuxHeader_t  *vcd;
  dsiuInst_t           *inst;
  dsiuInst_t           *loopInst;
  globData_t            data, cellData;
  tuint                 block[DSIU_UTOPIA_MAX_CELL_SIZE];
  tuint type;
  tuint channel;
  tuint chConfig;

  /* Extract the Cell From the FIFO, process extracted cell */
  cellData.dataSize = (dsiuContext.dreg->utopiaCellSize + 1) >> 1; /* words */
  cellData.data     = (void *)block;
  cell = (dsiuCell_t *)dsiuFifoOut (dsiuContext.netFifo, &cellData);
  if (cell == NULL)  {
    dsiuContext.errs.utopiaFifoExtractErr += 1;
    return (DSIU_FAIL);
  }

  /* The location of the routing information depends on the cell size,
   * (route tag vs vc demux mode) */
  switch (dsiuContext.dreg->utopiaCellSize)  {
    case 58: 
      /* Route tags are used - routing information is in the type field */
      rth     = (dsiuRouteTagHeader_t *) cell;
      type    = DSIU_UTOPIA_RT_EXTRACT_TYPE(rth);
      channel = DSIU_UTOPIA_RT_EXTRACT_CHANNEL(rth);

      switch (type)  {
        case DSIU_UTOPIA_RT_TYPE_MSG:
          /* Pass the entire cell to the message handler */
          data.dataSize = dsiuContext.dreg->utopiaCellSize;
          data.data     = (void *)cell;
          /* dsiu_receive_message_cell will post a message task when the entire
           * message has been received */
          dsiu_receive_message_cell (&data, channel);
          return (DSIU_SUCCESS);
   
       case DSIU_UTOPIA_RT_TYPE_AAL1:
         /* Strip the route tag and ATM header, flag the pvp handler */
         data.data     = &(rth->payload_first_word);
         data.dataSize = DSIU_UTOPIA_PAYLOAD_SIZE_WORDS;
         break;

       default:
         /* An unkown type was received */
         dsiuContext.errs.utopiaTypeErr += 1;
         return (DSIU_FAIL);
       } /* end switch (type) */
    break;  /* End case 58 */

    case 54:
      /* VC demux mode used.  */
      vcd      = (dsiuVcDemuxHeader_t *) cell;
      channel  = DSIU_UTOPIA_VC_EXTRACT_VCI_CHANNEL(vcd);
      if (channel == DSIU_UTOPIA_VC_MSG_CHANNEL)  {
        /* Pass the entire cell to the message handler */
        data.dataSize = dsiuContext.dreg->utopiaCellSize;
        data.data     = (void *)cell;
        /* dsiu_receive_message_cell will post a message task when the entire
         * message has been received. No channel information is available. */
        dsiu_receive_message_cell (&data, 0);
        return (DSIU_SUCCESS);
      }
      else if ((channel >=6) && (channel < dsiuContext.dreg->nChannels + 6))  {
        /* Strip the route tag and ATM header, flag the pvp handler */
        channel      -= 5;  /* index the channel from 1 */
        data.data     = &(vcd->payload_first_word);
        data.dataSize = DSIU_UTOPIA_PAYLOAD_SIZE_WORDS;
      }
      else {
        dsiuContext.errs.utopiaTypeErr += 1;
        return (DSIU_FAIL);
      }
    break;

    /* No default case is required since the cell size was verified prior
     * to posting the rx cell task */
  } /* end switch (size) */

  /* The payload is now in the globData structure. Route the cell based
   * on the system configuration */
  if ((channel == 0) || (channel > dsiuContext.dreg->nChannels))  {
    dsiuContext.errs.netInvalidChannel += 1;
    return (DSIU_FAIL);
  }

  inst = &dsiuInst[channel-1];  /* Dsiu Instance extraction from channel */
  chConfig = DSIU_GET_NET_CONFIG(inst->chStatus);

  /***************************************************************************
   * Configured to drop received cells
   ***************************************************************************/
  if (chConfig == DREG_CHCFG_NET_DROP)
    return (DSIU_SUCCESS);

  /***************************************************************************
   *       NET Loopback
   *  Config values from 1 to nchannels specify a network loopback
   ***************************************************************************/
  else if (chConfig <= dsiuContext.dreg->nChannels)  {
    loopInst = &dsiuInst[chConfig - 1];
    return (dsiuUtopiaTxCell (loopInst, &data));
  }

  /***************************************************************************
   *      PCM Route
   * Pass the Cell to the depacketizer and then to the vpu
   ***************************************************************************/
   else if (chConfig == DREG_CHCFG_NET_TO_TDM)  {
     /* Todo - pvp, vpu route */
   }

  return (DSIU_SUCCESS);
 
} /* dsiuUtopiaRxCell */
      
    







