/***********************************************************************************
 * FILE PURPOSE: Message interface for Mac
 ***********************************************************************************
 * FILE NAME: siugmndu.c
 *
 * FILE PURPOSE: Handle messages to and from the mac interface
 *
 * TABS: None
 *
 * Copyright (C) 2004, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include "types.h"
#include "hal.h"
#include "dsiuloc.h"
#include "diagcfg.h"
#include "dsiuport.h"
#include "dsiuutop.h"

#ifdef PA_SS
#include "dsiunwal.h"
#endif

#ifdef NDU_GMAC
/**********************************************************************************
 * FUNCTION PURPOSE: Send a formatted message to the mac driver
 **********************************************************************************
 * DESCRIPTION: A message has been formatted through neu and is ready to be
 *              sent to the hal driver.
 **********************************************************************************/
tint siu_gmmdu_msg_send (void *inst, xferPktInfo_t *pkt)
{
  dsiuNvInst_t     *nvInst = (dsiuNvInst_t *)inst;
#ifndef PA_SS
  halGmacData_t     data;
  tint              halId;
#endif
  tint              i;

  /* Verify the message channel */
  if (nvInst->ID != SIU_NVCH_MSG_GM_NORMAL)  {
    tsiuContext.nduStats.TxInvldNvCh += 1;
    return (DSIU_FAIL);
  }

  /* Increment the transmitted message stat */
  tsiuContext.nduStats.TxCtrlMsgCnt++;

#ifdef PA_SS
  for (i = 0; i < pkt->npkts; i++)  {
      if((nwal_sendRaw(dsiuNwGlobContext.nwalInstHandle,
                       (UInt16)(*pkt->pktSize),
                       (UInt8*)(pkt->pktIn[i]))) != nwal_OK)
      {
          dsiuContext.nduStats.TxCtrlMsgDrop += 1;
      }
  }
#else

  data.nSegs = pkt->npkts;
  data.type  = 0;
  data.sizes = (tuint *)pkt->pktSize;
  data.segs  = (tuint **)pkt->pktIn;

  /* Get the halId for the port used by this channel */
  for (i = 0; i < diagCfg.nGmacPorts; i++)  {
    if (dsiuContext.macPort[i].physPort == dsiuContext.dregMsgCfg->port)  {
      halId   = dsiuContext.macPort[i].halId;
    }
  }

  if ((*tsiuContext.hal->gmac->halGmacSendPacket) (halId, &data) == HAL_PKT_TX_PKT_OVERFLOW)
    dsiuContext.nduStats.TxErrDrop += 1;
#endif

  return (DSIU_SUCCESS);

} /* siu_gmmdu_msg_send */



/**********************************************************************************
 * FUNCTION PURPOSE: Format a message to be sent to the host
 **********************************************************************************
 * DESCRIPTION: A message is passed through the neu layer. The neu instance
 *              will then pass the packet with header to function
 *              siu_gmmdu_msg_send which passes it to hal.
 **********************************************************************************/
tint siu_gm_send_message (void *vmsg, siuDPRMsgType_t mtype)
{
  mgb_t *msg = (mgb_t *)vmsg;
  xferpktInfo_t     pkt;
  void             *data;
  dsiuNduContext_t *cxt;
  dsiuNvInst_t     *nvInst;
  tint              pktSizeBytes;
  tint              ret;

  /* Shorthand to the NDU context */
  cxt = &(tsiuContext.siuNduContext);

  /* Store the message ID for the return value */
  ret = msg->header.msg_id;

  /* Only one message priority is supported */
  nvInst = &(cxt->siuMsgNormalGm);

  /* If the message instance is NULL then messaging is not supported in this
   * interface. */
  if (!nvInst)
    return (0);  /* A return of 0 indicates failure to send the message */

  /* Compute the packet size. This must be done before the header translation, since
   * the size field is lost.
   *
   *        NOTE:
   *    The format of the message header is from the vcdemux cell format, this does
   *    not imply that the message will actually BE in vcdemux format. Just the
   *    message header is formatted to the vc demux msg header type which is
   *    also used by mac.
   */
  pktSizeBytes = SIU_TUINT_TO_BYTES(msg->header.length - MGB_HEADER_SIZE +
                                    SIU_VC_DEMUX_MSG_HEADER_LENGTH);
  data = siu_ndu_com_modify_vc_msg_header (mtype, msg);

  /* Pass the formatted message to NEU. Once the message headers are
   * applied it will be sent to siu_gmndu_msg_send */
  pkt.type    = xferpkt_TYPE_MSG_NORMAL;
  pkt.npkts   = 1;
  pkt.pktSize = &pktSizeBytes;
  pkt.pktIn   = &data;

  (*diagCfg.ctable.neu->neuSendIn) (nvInst->neuInst, (xferpktInfo_t *)&pkt);

  return (ret);

} /* siu_gm_send_message */

/***********************************************************************************
 * FUNCTION PURPOSE: Send a message to the host
 ***********************************************************************************
 * DESCRIPTION: A message is formatted for transmit through the mac interface.
 ***********************************************************************************/
tint siuGmPushFifo (void *vmsg)
{
  siuDPRMsgType_t mtype;

  /* Extract the message type from the message ID */
  mtype = siuMessageLevel (msg->header.msg_id);

  /* Send the message to the host */
  return (siu_gm_send_message (vmsg, mtype));

} /* siuGmPushFifo */


/***********************************************************************************
 * FUNCTION PURPOSE: Send a message to the host.
 ***********************************************************************************
 * DESCRIPTION: Functionally nearly the same as suGmPushFifo. The only difference
 *              is that the priority of the message is input and not extracted
 *              from the message ID.
 ***********************************************************************************/
tint siuGmWriteMessageToDPR (mgb_t *message, bool block, siuDPRMsgType_t type)
{
  DSIU_DUMMY_ARG(block);
  return (siu_gm_send_message ((void *)message, type));

} /* siuGmWriteMessageToDPR */


/**********************************************************************************
 * FUNCTION PURPOSE: Format and queue a received message
 **********************************************************************************
 * DESCRIPTION: This function receives a message that has been processed through
 *              NEU. The standard message header is created and the message is
 *              placed into the appropriate queue.
 **********************************************************************************/
tint siu_gmmdu_rcv_message (void *inst, xferPktInfo_t *pkt)
{
  tsiuVcDemuxMsgPayload_t *vcmsg;

  dsiuNvInst_t     *nvInst = (dsiuNvInst_t *)inst;
  dsiuFifo_t       *msgFifo;
  dsiuNduStats_t   *stats;
  dsiuNduContext_t *cxt;

  mgbHeader_t      header;
  globData_t       data;

  tuint           *rxMsgDropPtr;
  tuint           *rxMsgCntPtr;
  tuint           *body;


  /* Shorthand to the stats and ndu context */
  cxt   = &tsiuContext.siuNduContext;
  stats = &tsiuContext.nduStats;

  /* Only normal priority messages are supported */
  if (nvInst->ID != SIU_NVCH_MSG_GM_NORMAL)  {
    stats->RxInvldNvCh += 1;
    return (DSIU_FAIL);
  }

  /* Pointers to the fifo and stats of interest. Although there is only one possible
   * message priority this mimics the full application version of the function */
  msgFifo      = &(cxt->msgNormalFifo);
  rxMsgDropPtr = &(stats->RxCtrlMsgDrop);
  rxMsgCntPtr  = &(stats->RxCtrlMsgCnt);

  /* The received payload has the message with the VC demux message header.
   * THIS DOES NOT IMPLY CELLS HAVE BEEN RECEIVED */
  vcmsg           = (tsiuVcDemuxMsgPayload_t *)(pkt->pktIn[0]);
  header.length   = SIU_BYTES_TO_TUINT(pkt->pktSize[0]) + MGB_HEADER_SIZE - SIU_VC_DEMUX_MSG_HEADER_LENGTH;
  header.chnum    = DSIU_UTOPIA_VC_DEMUX_MSG_EXTRACT_CHANNEL(vcmsg);
  header.msg_id   = vcmsg->msg_id;
  header.trans_id = vcmsg->trans_id;
  body            = &(vcmsg->msg_first_word);

  /* Put the message into the appropriate fifo */
  if (dsiuFifoSizeAvail (msgFifo) < header.length)  {
    *rxMsgDropPtr += 1;
    return (DSIU_FAIL);
  }

  /* Put the message header into the file */
  data.dataSize = MGB_HEADER_SIZE;
  data.data     = (void *)&header;

  if (dsiuFifoIn (msgFifo, &data) == NULL)  {
    *rxMsgDropPtr += 1;
    dsiuFifoReset (msgFifo);  /* Fifo out of sync */
    return (DSIU_FAIL);
  }

  /* Put the message body into the fifo */
  data.dataSize = header.length - MGB_HEADER_SIZE;
  data.data     = (void *)body;

  if (dsiuFifoIn (msgFifo, &data) == NULL)  {
    *rxMsgDropPtr += 1;
    dsiuFifoReset (msgFifo);  /* Fifo out of sync */
    return (DSIU_FAIL);
  }

  /* The message is successfully in the fifo */
  *rxMsgCntPtr += 1;

  return (DSIU_SUCCESS);

} /* siu_gmmdu_rcv_message */


#endif



