/*******************************************************************************
 * FILE PURPOSE: Implement the NDU call table for the GMAC interface
 *******************************************************************************
 * FILE NAME: siugmndu.c
 *
 * DESCRIPTION: Implements
 *              nduOpen:    Channel specific open
 *              nduColose:  Channel specific close
 *              nduControl: System level initialization
 *
 *              nduSendIn:       Packet level input
 *              nduReceiveIn:    Polled function to find packets 
 *              nduReceiveInInt: Interrupt function for received packets 
 *
 *              nduGetNormalMessage:    searches from host message fifo
 *              nduGetInterruptMessage: searches from host message fifo
 *              nduPushFifo:            queues message to the host
 *              nduFifo2DPR:            sends queued message to the host
 *              nduWriteMessageToDPR:   sends message to host, no queue
 *              nduMsgInit:             initializes message interface
 *              nduMSG_test:            message spoof utility
 *
 *              nduHeartBeat:   system stability check
 *              nduGetClockMul: Returns clock multiplier stored in NDU memory
 *
 *
 * TABS: None
 *
 * Copyright (C) 2004 Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include "types.h"
#include "hal.h"
#include "dsiuloc.h"
#include "dsiuport.h"
#include "diagcfg.h"
#include <ti/mas/neu/neuproto.h>
#include <ti/mas/neu/neu.h>
#include "utlrepck_m.h"
#include "utlrepck.h"
#include <string.h>

#ifdef NDU_GMAC
siuNduCallTable_t siuGmacNduCallTable = {
  siuNduComOpen,                  /* nduOpen                */
  siuNduComClose,                 /* nduClose               */
  siuGmNduSendIn,                 /* nduSendIn              */
  (void (*)(void))siuGmNduDummy,  /* nduReceiveIn           */
  siuGmNduIntReceiveIn,           /* nduReceiveInInt        */
  siuGmNduControl,                /* nduControl             */

  siuNdComGetNormalMessage,       /* nduGetNormalMessage    */
  siuNdComGetInterruptMessage,    /* nduGetInterruptMessage */
  siuGmWriteMessageToDPR,         /* nduWriteMessageToDPR   */
  siuNduComMSG_test,              /* nduMSG_test            */
  (void (*)(tint))siuGmNduDummy,  /* nduHeartBeat           */
  (tuint (*)(void))siuGmNduDummy, /* nduGetClockMul         */
  (void (*)(void))siuGmNduDummy,  /* nduMsgInit             */
  siuGmPushFifo,                  /* nduPushFifo            */
  (tint (*)(tint))siuGmNduDummy   /* dnuFifo2DPR            */
};

/****************************************************************************** 
 * FUNCTION PURPOSE: Provide a generic dummy function
 ******************************************************************************
 * DESCRIPTION: The dummy function simply returns. It's used to populate
 *              unsupported or unused functions in the NDU call table.
 ******************************************************************************/
tint siuGmNduDummy (void)
{
  return (0);

} /* siuGmNduDummy */

/******************************************************************************
 * FUNCTION PURPOSE: Initialize the dsiu-mac interface
 ******************************************************************************
 * DESCRIPTION: This function initializes the application to handle the mac
 *              packets. The input parameter is for compatibility with the
 *              legacy HPI.
 ******************************************************************************/
tuint mdebug1, mdebug2, mdebug3;
tint siuGmNduControl (tint ctlCode)
{
  dsiuNduContext_t *cxt;
  neuSizeCfg_t      neuCfgSize;
  neuCtrl_t         neuCtrl;

#ifndef PA_SS
  /* Verify that the HAL interface supports mac send out */
  /* first verify if HAL GMAC is defined */
  if (!tsiuContext.hal->gmac)
    return (-1);
  if (!tsiuContext.hal->gmac->halGmacSendPacket)
    return (-1);
#endif

  /* Shorthand reference to the NDU control */
  cxt = &(tsiuContext.siuNduContext);

  /* The mac NDU requires that neu be in the build */
  if (!diagCfg.ctable.neu)
    dsiuPanic (DSIU_FAIL_GMNDU_NO_NEU);

  /* If a total header length of 0 is specified then the mac ndu will not be used
   * for messages */
  if (!dsiuContext.diagApp.u.mac.msg.totalHeaderLenBytes)  {
    cxt->siuMsgNormalGm.neuInst = NULL;
    return (0);
  }

  /* In initialization create the neu instance */
  if (ctlCode == SIU_NDU_CTRL_INIT)  {

    /* Create the neu for normal priority messages */
    neuCfgSize.rxAssemSize    = 0;
    neuCfgSize.txScratchSize  = diagCfg.neuSegSize;
    neuCfgSize.txHeaderSize   = SIU_BYTES_TO_WORDS(dsiuContext.diagApp.u.mac.msg.totalHeaderLenBytes);
    neuCfgSize.sssar_required = FALSE;    /* Diag/bootloader never use SSSAR */
    cxt->siuMsgNormalGm.ID    = SIU_NVCH_MSG_GM_NORMAL;
    cxt->siuMsgNormalGm.neuInst = siuNeuCreate (&neuCfgSize, DSIU_MSG_GM_NEU_ECO_ID);

    /* Panic if the neu creation failed */
    if (!(cxt->siuMsgNormalGm.neuInst))
      dsiuPanic (DSIU_FAIL_NEU_GM_FAIL);


    /* Open the newly created neu. Logical port 0 is used for messages */
    siu_ndu_com_open_neu (cxt->siuMsgNormalGm.neuInst, dsiuContext.dregMsgCfgShadow, &(cxt->siuMsgNormalGm),
                          siu_gmmdu_msg_send, 0, dsiuContext.macPort[0].halId);


    /* Enable the NEU channel receive callouts */
    siu_ndu_com_rxcall_neu (cxt->siuMsgNormalGm.neuInst, siu_gmmdu_rcv_message, &(cxt->siuMsgNormalGm));

    /* Register the UDP port, if provided */
    neuCtrl.code           = neu_CTRL_GET_SRC_UDP_PORT;
    
    if ((*diagCfg.ctable.neu->neuControl) (cxt->siuMsgNormalGm.neuInst, &neuCtrl) != neu_NOERR)  {
      dsiuPanic (DSIU_FAIL_GM_INVALID_UDP_OFFSET);
    }

    mdebug1 = SIU_BYTES_TO_WORDS(neuproto_UDP_BYTE_OFFSET_SRC_PORT);
    mdebug2 = SIU_BYTES_TO_WORDS(dsiuContext.diagApp.u.mac.msg.udpOffsetBytes);
    mdebug3 = neuCtrl.u.portMap.port;

    /* The lower two bits of the udp port specify the message priority. Force the
     * priority to be normal for the diag code */
    /* For Ethernet PDT the priority is determined from the PDT */ 
    if (dsiuContext.diagApp.u.mac.msg.rxPayloadType != neuproto_ETHERNET_PDT)    
    {
    neuCtrl.u.portMap.port = (neuCtrl.u.portMap.port & 0xfffc) + 1;
    }

    neuCtrl.code = neu_CTRL_ADD_PORTMAP;
    neuCtrl.u.portMap.inst = (void *)&(cxt->siuMsgNormalGm);
    
    (*diagCfg.ctable.neu->neuControl) (cxt->siuMsgNormalGm.neuInst, &neuCtrl);

    /* Verify that NEU was successful */
    if (neuCtrl.u.portMap.retCode != neu_NOERR)
      dsiuPanic (DSIU_FAIL_NEU_PORTMAP_FAIL);

  }

  /* Nothing else to do for other control codes */

  return (0);

} /* siuGmNduControl  */

/******************************************************************************
 * FUNCTION PURPOSE: Send formatted packets to the network
 ******************************************************************************
 * DESCRIPTION: Fully formatted packets are sent to the hal layer
 ******************************************************************************/
tint siuGmNduSendIn (void *vinst, xferPktInfo_t *pktInfo)
{
#if (defined(DSIU_SIMULATE_APPLICATION) || defined(DSIU_DAUGHTER_TEST))
  halGmacData_t  haldata;
  halIntrLevel_t level;
  dsiuInst_t     *inst = (dsiuInst_t *)vinst;
  dregPeriph_t   *pStatus;
  tint            halId = -1;
  tint            i;
  tuint           size;

  /* Translate the data into hal format */
  haldata.nSegs = (tuint)   pktInfo->npkts;
  haldata.segs  = (tuint **)pktInfo->pktIn;
  haldata.sizes = (tuint *) pktInfo->pktSize;
  haldata.type  = 0;  /* types don't match */

  level = (*tsiuContext.hal->halIntrLevel) (HAL_INT_LEVEL_DISABLE);


  /* Get the halId for the port used by this channel */
  for (i = 0; i < diagCfg.nGmacPorts; i++)  {
    if (dsiuContext.macPort[i].physPort == dregVchCfgShadow[(inst->Id) - 1].port)  {
      halId   = dsiuContext.macPort[i].halId;
      pStatus = dsiuContext.macPort[i].status;
    }
  }

  for (i = 0, size = 0; i < pktInfo->npkts; i++)
    size += pktInfo->pktSize[i]; 


  /* Send the packets. Existance of the funciton was verified in nduControl */
  if (halId != -1)  {
    if (((*tsiuContext.hal->gmac->halGmacSendPacket) (halId, &haldata)) != HAL_SUCCESS)
      pStatus->txDrop += 1;
    else
      pStatus->txDataCount += size;
  }
   

  tsiuContext.hal->halIntrLevel (level);

#else
  DSIU_DUMMY_ARG(vinst);
  DSIU_DUMMY_ARG(pktInfo);
#endif

  return (0);

} /* siuGmNduSendIn */
    

/*******************************************************************************
 * FUNCTION PURPOSE: Interrupt handler for packets
 *******************************************************************************
 * DESCRIPTION: This function is called from the hal driver when packets are
 *              are available. The type of packet (message, voice data) is
 *              unknown.
 *
 *              The packet is sent to NEU with a NULL instance. NEU will
 *              attempt to extract the packet type from it's hash table,
 *              which contains the NEU instances, which contain routing
 *              information.
 *******************************************************************************/
void siuGmNduIntReceiveIn (void *cxt, word *packet, tuint sizeBytes)
{
  xferpktInfo_t pktOut;
  void         *vInst;
  dsiuNvInst_t *msgInst;
  dsiuPort_t   *dsiuPort = (dsiuPort_t *)cxt;
  dregPeriph_t *port     = dsiuPort->status;
  neuValidationInfo_t   vInfo;

  /* Increment the interrupt stat. This is somewhat misleading since this function
   * can receive a packet through a poll */
  dsiuInc (&(port->intCount));
  port->rxDataCount += sizeBytes;

  
  /* Wrap up the packet for shipout */
  pktOut.pktSize = (tint *)&sizeBytes;
  pktOut.pktIn   = (void *)&packet;
  pktOut.type    = XFER_PKT_TYPE_AAL1;  /* Always use aal1 */
  pktOut.npkts   = 1;
  memset(&vInfo,0,sizeof(neuValidationInfo_t));
  pktOut.suppInfo =  &vInfo;

  /* Send the packet to NEU with a NULL instance. Neu will return with
   * a matching instance if one if found */
  vInst = (*diagCfg.ctable.neu->neuGetInstance) ((xferpktInfo_t *)&pktOut);

  /* For no matching instance return */
  if (vInst == NULL)  {
    /* The lookup failed - can't route the packet */
    dsiuInc (&(dsiuContext.dregOpStats->pkt_lookup_err));
//    port->rxDrop++;
    return;
  }

//  port->rxDataCount += sizeBytes;

  /* If the instance was the message instance repass the packet to NEU  */
  msgInst = &(dsiuContext.siuNduContext.siuMsgNormalGm);
  if (vInst == (void *)msgInst)  {
    (*diagCfg.ctable.neu->neuReceiveInGmac) (msgInst->neuInst, (xferpktInfo_t *)&pktOut);
    return;
  }

  /* Otherwise the matching instance was a voice channel instance. Repass
   * the packet */
  (*diagCfg.ctable.neu->neuReceiveInGmac) (((dsiuInst_t *)vInst)->neuInst, (xferpktInfo_t *)&pktOut);
  

} /* siuGmNduIntReceiveIn */

#endif


  


