/***********************************************************************
 * FILE PURPOSE: Implement HPI message interface
 ***********************************************************************
 * FILE NAME: siumdu.c
 *
 * FILE PURPOSE: Handle messages to and from the HPI interface
 *
 * Copyright (C) 2003, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include "types.h"
#include <string.h>

#ifdef DSIU
#include "dsiuloc.h"
#include "ddpram.h"
#endif


/************************************************************************
 * FUNCTION PURPOSE: Check for messages
 ************************************************************************
 * DESCRIPTION: The receive from host mail box is checked for messages, and
 *              any messages found are processed.
 ************************************************************************/
void siuGetNormalMessage (void)
{
  tuint msg_id;

#ifdef DSP_ON_MIPS
  /* Check the echo field, a zero value indicates a message */
  if (!hpiMem->host_to_dsp.msg.header.echo)  {
    msg_id = hpiMem->host_to_dsp.msg.header.msg_id;
    siuProcessNormalMessage (&hpiMem->host_to_dsp.msg);
    /* Put the transaction id in the echo field to indicate the message was processed */
    hpiMem->host_to_dsp.msg.header.echo = msg_id;
  }
#else
  /* Check the echo field, a zero value indicates a message */
  if (!dpram.host_to_dsp.msg.header.echo)  {
    /* Put the transaction id in the echo field to indicate the message was processed */
    msg_id = dpram.host_to_dsp.msg.header.msg_id;
    dpram.host_to_dsp.msg.header.echo = msg_id;
    siuProcessNormalMessage (&dpram.host_to_dsp.msg);
  }
#endif

} /* siuGetNormalMessage */
  

/************************************************************************
 * FUNCTION PURPOSE: Check for high priority messages.
 ************************************************************************
 * DESCRIPTION: There are no high priority messages in diag
 ************************************************************************/
void siuGetInterruptMessage (void)
{
  /* empty */
} /* siuGetInterruptMessage */


/*************************************************************************
 * FUNCTION PURPOSE: Send a message to the host
 *************************************************************************
 * DESCRIPTION: Puts the message into the to host mailbox. No check is made
 *              to see if the previous message has been processed.
 **************************************************************************/
tint siuPushFifo (void *vmsg)
{
  mgb_t *msg;

  msg = (mgb_t *)vmsg;

#ifdef DSP_ON_MIPS
  /* Copy the message to the HPI memory area */
  memcpy (&hpiMem->dsp_to_host.msg, vmsg, (msg->header.length)*sizeof(tint));

  /* Clear the echo field to signal the host */
  hpiMem->dsp_to_host.msg.header.echo = 0;
#else
  /* Copy the message to the HPI memory area */
  memcpy (&dpram.dsp_to_host.msg, vmsg, msg->header.length * sizeof(tuint));

  /* Clear the echo field to signal the host */
  dpram.dsp_to_host.msg.header.echo = 0;
#endif

  return (msg->header.msg_id);
} /* siuPushFifo */


/***************************************************************************
 * FUNCTION PURPOSE: Send a message to the host
 ***************************************************************************
 * DESCRIPTION: Sends a message to the host.
 ***************************************************************************/
tint siuWriteMessageToDPR (mgb_t *message, bool block, siuDPRMsgType_t type)
{
  DSIU_DUMMY_ARG(block);
  DSIU_DUMMY_ARG(type);
  return (siuPushFifo ((void *)message));
} /* siuWriteMessageToDPR */

/****************************************************************************
 * FUNCTION PURPOSE: Transfer message in fifo to host
 ****************************************************************************
 * DESCRIPTION: Dummy function. There are no message to host fifos in diag.
 ****************************************************************************/
tint siuFifo2DPR (tint dpr_id)
{
  DSIU_DUMMY_ARG(dpr_id);
  return (0);
} /* siuFifo2DPR */

/****************************************************************************
 * FUNCTION PURPOSE: Message interface initialization
 ****************************************************************************
 * DESCRIPTION: Message initialation. Dummy function for HPI
 ****************************************************************************/
void siuMsgInit (void)
{
  /* empty */
} /* siuMsgInit */

/*****************************************************************************
 * FUNCTION PURPOSE: spoof a message
 *****************************************************************************
 * DESCRIPTION: A dummy function - a message can be spoofed by using the 
 *              mailboxes
 *****************************************************************************/
void siuMSG_test (void)
{
  /* empty */
} /* siuMSG_test */
