/****************************************************************************
 * FILE PURPOSE: Implement the NDU call table for the HPI interface
 ****************************************************************************
 * FILE NAME: siundu.c
 *
 * DESCRIPTION: Implements the ndu call table defined as:
 *              nduOpen:    Channel specific open
 *              nduClose:   Channel specific close
 *              nduControl: System level initialization
 *
 *              nduSendIn:       Packet level input
 *              nduReceiveIn:    Polled function to find packets
 *              nduReceiveInInt: Interrupt function for received packets
 *         
 *
 *              nduGetNormalMessage:    Checks for message from host
 *              nduGetInterruptMessage: Checks for message from host
 *              nduPushFifo:            sends message to host from queue
 *              nduFifo2DPR:            sends message to host from queue
 *              nduWriteMessageToDPR:   sends message to host, no queue
 *              nduMsg_test:            message spoof utility
 *
 *              nduHeartBeat:      system stability check
 *              nduGetClockMul:    Return clock multiplier stored in NDU memory
 *
 * Copyright (C) 2003, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include "types.h"

#ifdef DSIU
#include "dsiuloc.h"
#include "ddpram.h"
#endif

/***************************************************************************
 * For the diagnostic code the call table is instantiated 
 ***************************************************************************/
#ifdef DSIU
siuNduCallTable_t siuHpiNduCallTable = {
  siuNduOpen,
  siuNduClose,
  siuNduSendIn,
  siuNduReceiveIn,
  siuNduIntReceiveIn,
  siuNduControl,

  siuGetNormalMessage,
  siuGetInterruptMessage,
  siuWriteMessageToDPR,
  siuMSG_test,
  siuHeartBeat,
  siuGetClockMul,
  siuMsgInit,
  siuPushFifo,
  siuFifo2DPR
};
#endif /* DSIU */
  

/***************************************************************************
 * FUNCTION PURPOSE: Opens a network driver channel
 ***************************************************************************
 * DESCRIPTION: Specifies the output function to call with network data
 ***************************************************************************/
void siuNduOpen (tint chnum, tsiuNduConfig_t *config)
{
  DSIU_DUMMY_ARG(chnum);
  DSIU_DUMMY_ARG(config);
  /* empty */
} /* siuNduOpen */

/***************************************************************************
 * FUNCTION PURPOSE: Closes the network driver channel
 ***************************************************************************
 * DESCRIPTION: Disables the network data path
 ***************************************************************************/
void siuNduClose (tsiuInst_t *inst)
{
  DSIU_DUMMY_ARG(inst);
  /* empty */
} /* siuNduClose */

/****************************************************************************
 * FUNCTION PURPOSE: Initializes the HPI interface
 ****************************************************************************
 * DESCRIPTION: Initializes the application to manage the HPI interface.
 ****************************************************************************/
tint siuNduControl (tint flag)
{
  DSIU_DUMMY_ARG(flag);
  /* empty */
  return (0);
} /* siuNduControl */

/****************************************************************************
 * FUNCTION PURPOSE: Send non-message data
 ****************************************************************************
 * DESCRIPTION: Sends non-message data to the network interface
 ****************************************************************************/
tint siuNduSendIn (void *inst, xferPktInfo_t *pktInfo)
{
  DSIU_DUMMY_ARG(inst);
  DSIU_DUMMY_ARG(pktInfo);
  /* empty */
  return (0);
} /* siuNduSendIn */

/****************************************************************************
 * FUNCTION PURPOSE: Check for non-message data from network interface
 ****************************************************************************
 * DESCRIPTION: This function is a polling function called to see if
 *              there is any data available from the network.
 ****************************************************************************/
void siuNduReceiveIn (void)
{
 /* empty */
} /* siuNduReceiveIn */

/****************************************************************************
 * FUNCTION PURPOSE: Interrupt handler for network data
 ****************************************************************************
 * DESCRIPTION: The function that is called when network data is signalled
 *              as available through an interrupt.
 ****************************************************************************/
void siuNduIntReceiveIn (void *cxt, word *data, tuint sizeBytes)
{
  DSIU_DUMMY_ARG(cxt);
  DSIU_DUMMY_ARG(data);
  DSIU_DUMMY_ARG(sizeBytes);
  /* empty */
} /* siuNduIntReceiveIn */

/****************************************************************************
 * FUNCTION PURPOSE: System heartbeat update
 ****************************************************************************
 * DESCRIPTION: Updates the heart beat system check.
 ****************************************************************************/
void siuHeartBeat (tint mode)
{
  DSIU_DUMMY_ARG(mode);
  /* empty */
} /* siuHeartBeat */

/****************************************************************************
 * FUNCTION PURPOSE: Return system clock multiplier
 ****************************************************************************
 * DESCRIPTION: Gets clock multiplier from the NDU data fields.
 ****************************************************************************/
tuint siuGetClockMul (void)
{
  return (0);
} /* siuGetClockMul */

  


