/****************************************************************************
 * FILE PURPOSE: Functions common to two or more NDU types
 ****************************************************************************
 * FILE NAME: nducom.c
 *
 * DESCRIPTION: Provides functions in common to two or mroe NDU interfaces.
 *
 * TABS: None
 *
 * Copyright (C) 2004, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include "types.h"
#include "hal.h"
#include <ti/mas/neu/neu.h>
#include <ti/mas/neu/neuproto.h>
#include "diagcfg.h"
#include "dsiuport.h"
#include "dsiuutop.h"
#include "util.h"
#include "utlrepck.h"
#include "dmiscport.h"
#include <string.h>


#define BRINGUP_WORKAROUND

/******************************************************************************
 * FUNCTION PURPOSE: Opens a network driver channel
 ******************************************************************************
 * DESCRIPTION: Specifies the output function to call with network data cells.
 *****************************************************************************/
void siuNduComOpen (tint chnum, tsiuNduConfig_t *config)
{
  tsiuInst_t *inst = &tsiuInst[chnum - SIU_FIRSTCH];
  inst->nduInst.RxOutSpec    = config->RxOutSpec;
  inst->nduInst.recInterface = config->recInterface;
} /* siuUtNduOpen */

/*****************************************************************************
 * FUNCTION PURPOSE: Closes a network driver channel
 *****************************************************************************
 * DESCRIPTION: Sets the output function call to NULL, which prevents
 *              callout.
 *****************************************************************************/
void siuNduComClose (tsiuInst_t *inst)
{
  inst->nduInst.RxOutSpec.nduReceiveOut = NULL;
  inst->nduInst.RxOutSpec.targetInst    = NULL;
  inst->nduInst.recInterface            = SIU_NDU_TYPE_NONE;
}  /* siuUtNduClose */


/*****************************************************************************
 * FUNCTION PURPOSE: Setup the NEU rx callout
 *****************************************************************************
 * DESCRIPTION: Sets up the callout for the non voice channel NEU for
 *              a completed reception.
 *****************************************************************************/
void siu_ndu_com_rxcall_neu (void *inst, tint (*fcn)(void *, xferPktInfo_t *), void *rcvInst)
{
  neuCtrl_t cfg;

  cfg.code = neu_CTRL_RX_RECEIVE_OUT_CFG;
  cfg.u.neuReceiveOutSpec.neuReceiveOut = fcn;
  cfg.u.neuReceiveOutSpec.targetInst    = rcvInst;

  (*diagCfg.ctable.neu->neuControl) (inst, &cfg);

} /* siu_ndu_com_rxcall_neu */


/*****************************************************************************
 * FUNCTION PURPOSE: Open an NEU instance
 *****************************************************************************
 * DESCRIPTION: Opens the NEU instance for the message interface with the
 *              information provided in the diagApp structure. Only normal
 *              priority messages exist.
 *****************************************************************************/
void siu_ndu_com_open_neu (void *neuInst, dregNetCfg_t *netCfg, void *tInst, 
                           tint (*neuSendOut) (void *, xferPktInfo_t *), tint cellSize,
                           tint halId)
{
  neuCfg_t    cfg;
  neuCtrl_t   ctrl;
  tint        shift;
  tuint       val,count;
  tint        tagOffset;
  tint        flen;

  /* Provide the function pointer and instance NEU uses for the formatted message */
  cfg.neuSendOut.neuSendOut = neuSendOut;
  cfg.neuSendOut.targetInst = tInst;

  /* Leave NEU enabled */
  cfg.parm.chnl.enable = TRUE;

  /* Flags. Bit exactness of the desired flags is required. It is not checked here,
   * but the full application does perform the check */
  cfg.parm.chnl.cfg_flag1 = netCfg->cfg_flag1;

#ifdef BRINGUP_WORKAROUND
  /* Disabling the UDP checksum as received messages were lost */
    cfg.parm.chnl.cfg_flag1 = 0;
#endif

  cfg.parm.chnl.cfg_flag2 = netCfg->cfg_flag2;

  /* Alternate ports unused */
  cfg.parm.chnl.alt_local_port  = 0;
  cfg.parm.chnl.alt_remote_port = 0;

  count = 0;
  while(count < ifneu_IP_ADDR_MAX_NUM_WORDS){
        cfg.parm.chnl.alt_remote_addr[count] = 0;
        count++;
  }

  /* report control must be disabled */
  cfg.parm.chnl.report_ctrl          = 0;  /* The discard definition is in either mgmicdsp or neuloc.h */
  cfg.parm.chnl.report_throttle_time = 0;
  cfg.parm.chnl.sysInst              = NULL;

  /* No second protocol allowed */
  cfg.parm.chnl.second_prototype = neuproto_PROTOCOL_NULL;

  /* Parameters from the header */
  cfg.parm.chnl.rx_offset_bytes   = netCfg->rx_offset_bytes;
  cfg.parm.chnl.rx_payload_type   = netCfg->rx_payload_type;
  cfg.parm.chnl.tx_min_cell_bytes = netCfg->tx_min_cell_bytes;

  /* Only AAL5 SAR supported for diagnostic */
  if (cellSize)
    cfg.parm.chnl.sar_type = neuproto_SAR_AAL5;
  else
    cfg.parm.chnl.sar_type = neuproto_SAR_NONE;

  /* Tx parameters */
  cfg.parm.chnl.tx_ip_offset_bytes    = netCfg->tx_ip_offset_bytes;
  cfg.parm.chnl.tx_udp_offset_bytes   = netCfg->tx_udp_offset_bytes;
  cfg.parm.chnl.tx_802_3_offset_bytes = netCfg->tx_802_3_offset_bytes;

  if ( (cellSize != 54) && (cellSize != 58) && (cellSize != 0) )
    dsiuPanic (DSIU_FAIL_NDU_COM_CELL_SIZE);

  /* For the utopia NDU with MII the core ID is placed into the UDP source address
   * as defined in the ggswcfg.h */
  if ( (cfg.parm.chnl.tx_udp_offset_bytes >= 0) && (diagCfg.neuUdpSrcMask) )  {
    val = diagCfg.neuUdpSrcMask;
    for (shift = 0; !(val & 0x1); shift++, val >>= 1);
    cfg.parm.chnl.tx_udp_src_mask = diagCfg.neuUdpSrcMask;
    cfg.parm.chnl.tx_udp_src_val  = dsiuContext.coreId << shift;

  } else {
    cfg.parm.chnl.tx_udp_src_mask = 0;
    cfg.parm.chnl.tx_udp_src_val  = 0;
  }

  /* Configure the priority tag. The values in the priority tag are 
   * overlayed with the alt_remote_addr_lsw field */
  if (netCfg->cfg_flag1 & (neuproto_FLAGS1_PDT_RX_CHK | neuproto_FLAGS1_PDT_TX_CHK)) {
    memset(&cfg.parm.chnl.alt_remote_addr[0], 0, DMSG_IP_ADDR_MAX_NUM_TUINT * sizeof(tuint));
    cfg.parm.chnl.alt_remote_addr[0] = netCfg->alt_remote_addr[0];
    cfg.parm.chnl.alt_remote_addr[1] = netCfg->alt_remote_addr[1];
    cfg.parm.chnl.alt_remote_addr[2] = netCfg->alt_remote_addr[2];
  }

  /* Transmit header */
  cfg.parm.chnl.tx_total_hdr_len_bytes = netCfg->tx_total_hdr_len_bytes;
  memcpy (cfg.parm.chnl.header, netCfg->header, UTL_BYTES_TO_WORDS(netCfg->tx_total_hdr_len_bytes));

  /* The core ID field is added into the header. The size of the dsp field is hardcoded to 3 */
  if ((cfg.parm.chnl.cfg_flag1 & neuproto_FLAGS1_PDT_TX_CHK) &&
     (neu_PDT_TYPE(cfg.parm.chnl.alt_remote_addr[0]) != neu_PDT_TYPE_EXT)) {
      tagOffset = (netCfg->tx_total_hdr_len_bytes - (tint)neu_PDT_FIELD_TAG_LENGTH(cfg.parm.chnl.alt_remote_addr));
      flen = (tint) neu_PDT_FIELD_LEN(cfg.parm.chnl.alt_remote_addr);
    utlCopyBits ((word *)&dsiuContext.coreId, (word *)cfg.parm.chnl.header, TYP_WORD_SIZE - 3, 
                    (tint) (neu_PDT_FIELD_BYTE_OFFSET(cfg.parm.chnl.alt_remote_addr) * 8) +
                            (8 - neu_PDT_FIELD_BIT_OFFSET(cfg.parm.chnl.alt_remote_addr)) +
                          (tagOffset * 8) - flen - 3,
                   3);
  }

  /* Hal Id */
  cfg.parm.chnl.halId = halId;

  /* Open NEU */
  (*diagCfg.ctable.neu->neuOpen) (neuInst, &cfg);

  /* The parameters must be re-passed to enable sending of cells */
  ctrl.code   = neu_CTRL_CHANNEL_CFG;
  ctrl.u.parm = cfg.parm;
  (*diagCfg.ctable.neu->neuControl) (neuInst, &ctrl);


#ifdef PA_SS
  dsiuConfigL2L3L4(neuInst,
                   &cfg.parm.chnl,
                   &tsiuContext.siuNduContext.udpHandleHiPrioMsg,
                   &tsiuContext.siuNduContext.udpHandleMedPrioMsg,
                   &tsiuContext.siuNduContext.udpHandleLowPrioMsg);

#endif

} /* siu_ndu_com_open_neu */

/*****************************************************************************
 * FUNCTION PURPOSE: Extract message from the system fifos, process it
 *****************************************************************************
 * DESCRIPTION: Extracts a message from the specified fifo and processes it.
 *****************************************************************************/
tint siu_process_message (dsiuFifo_t *fifo, void (*processFunc)(mgb_t *))
{
#ifdef DSIU
  mgb_t *msg = dsiuContext.msgRxBuf;
#else
  mgb_t  msgBuf;
  mgb_t *msg = &msgBuf;
  tuint  size = sizeof(mgb_t);
#endif

  globData_t data;
  data.data     = (void *)msg;
  data.dataSize = MGB_HEADER_SIZE;
  /* Extract the message header to the linear buffer */
  if (dsiuFifoOut (fifo, &data) == NULL)  {
    dsiuFifoReset (fifo);
    return (-1);  
  }

  /* Extract the message body */
  if (msg->header.length > MGB_HEADER_SIZE)  {
    data.dataSize = msg->header.length - MGB_HEADER_SIZE;
    data.data     = (void *)&msg->msg_body;
    if (dsiuFifoOut (fifo, &data) == NULL)  {
      dsiuFifoReset (fifo);
      return (-1);
    }
  }

  /* Process the message */
  (*processFunc)(msg);

  return (0);
} /* siu_process_message */

/*****************************************************************************
 * FUNCTION PURPOSE: Poll for messages
 *****************************************************************************
 * DESCRIPTION: The normal and low priority message queues are polled, and
 *              if a message is found, it is processed.
 *****************************************************************************/
void siuNdComGetNormalMessage (void)
{
  dsiuFifo_t *fifo;
  tint         pr;

  /* Check the normal priority message fifo first. If it is empty check
   * the low priority message fifo */
  if (dsiuFifoNelems(&tsiuContext.siuNduContext.msgNormalFifo)) {
    pr = 1;
    fifo = &tsiuContext.siuNduContext.msgNormalFifo;
  }
  else if (dsiuFifoNelems (&tsiuContext.siuNduContext.msgLowFifo)) {
    pr = 0;
    fifo = &tsiuContext.siuNduContext.msgLowFifo;
  }
  else
    return;  /* No messages pending */

  if (siu_process_message (fifo, siuProcessNormalMessage)) {
    if (pr)
      SIU_NORMAL_MSG_FIFO_FAIL();
    else
      SIU_LOW_MSG_FIFO_FAIL();
  }
    
} /* siuNdComGetNormalMessage */


/****************************************************************************
 * FUNCTION PURPOSE: Process the high priority message queue
 ****************************************************************************
 * DESCRIPTION: If the high priority message fifo is not empty the top 
 *              message is extracted and processed.
 ****************************************************************************/
void siuNdComGetInterruptMessage (void)
{
  if (dsiuFifoNelems (&tsiuContext.siuNduContext.msgHighFifo))
    if (siu_process_message (&tsiuContext.siuNduContext.msgHighFifo,
                             siuProcessInterruptMessage))
      SIU_SIGNAL_MSG_FIFO_FAIL();

} /* siuNdComGetInterruptMessage */

/*****************************************************************************
 * FUNCTION PURPOSE: Format a VC Demux message header
 *****************************************************************************
 * DESCRIPTION: Re-formats a message header the format used by vc demux as well
 *              as mii. The length field is lost.
 *****************************************************************************/
void *siu_ndu_com_modify_vc_msg_header (siuDPRMsgType_t mtype, mgb_t *msg)
{
  tsiuVcDemuxMsgPayload_t *vcd;
  tuint ttype;

  vcd = (tsiuVcDemuxMsgPayload_t *)&msg->header.length;

  /* The length field of the message type is replaced by the DSP ID */ 
  vcd->dsp_id = (dsiuContext.diagApp.dspId + tsiuContext.coreId);

  /* Translate the proirity into route tag type field */
  switch (mtype)  {
    case SIU_DPR_SIGNAL_MSG:
      ttype = SIU_UTOPIA_VC_DEMUX_MSG_PRIORITY_HIGH;
      break;
    case SIU_DPR_NORMAL_MSG:
      ttype = SIU_UTOPIA_VC_DEMUX_MSG_PRIORITY_NORMAL;
      break;
    case SIU_DPR_LOW_MSG:
      ttype = SIU_UTOPIA_VC_DEMUX_MSG_PRIORITY_LOW;
      break;
  } /* end switch */
  
  /* The channel field of the message type is appended with the core ID
   * and the message priority */
  SIU_UTOPIA_VC_DEMUX_MSG_SET_PRIORITY(vcd, ttype);
  SIU_UTOPIA_VC_DEMUX_MSG_SET_COREID(vcd, tsiuContext.coreId);

  /* Return a pointer the vc demux message payload */
  return ((void *) vcd);

} /* siu_ndu_com_modify_vc_msg_header */

/*****************************************************************************
 * FUNCTION PURPOSE: Spoof a message.
 *****************************************************************************
 * DESCRIPTION: Processes the message in the global test message structure.
 ****************************************************************************/
mgb_t msgTest;
tint msgTest_flag = 0;
tint sigTest_flag = 0;
void siuNduComMSG_test (void)
{
  if (msgTest_flag) {
    msgTest_flag = 0;
    siuProcessNormalMessage (&msgTest);
  }
  else if (sigTest_flag)  {
    sigTest_flag = 0;
    siuProcessInterruptMessage (&msgTest);
  }
} /* siuNduComMSG_test */

