/*****************************************************************************
 * FILE PURPOSE: Message interface unit for utopia 
 *****************************************************************************
 * FILE NAME: siuutmdu.c
 *
 * FILE PURPOSE: Handle messages to and from the utopia interface
 *
 * TABS: None
 *
 * Copyright (C) 2002, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include "types.h"
#include "utlxferpkt.h"
#include <ti/mas/iface/ifpkt/xferpkt.h>
#include "hal.h"
#ifdef DSIU
#include "dsiuloc.h"
#include <ti/mas/neu/aal5.h>
#include "diagcfg.h"
#include "dsiuutop.h"
#include "dsiuport.h"
#else
#include "siuloc.h"
#include "siuport.h"
#endif

/*****************************************************************************
 * FUNCTION PURPOSE: Extract message from the system fifos, process it
 *****************************************************************************
 * DESCRIPTION: Extracts a message from the specified fifo and processes it.
 *****************************************************************************/
tint siu_ut_process_message (dsiuFifo_t *fifo, void (*processFunc)(mgb_t *))
{
#ifdef DSIU
  mgb_t *msg = dsiuContext.msgRxBuf;
#else
  mgb_t  msgBuf;
  mgb_t *msg = &msgBuf;
  tuint  size = sizeof(mgb_t);
#endif

  globData_t data;
  data.data     = (void *)msg;
//  data.dataSize = MGB_HEADER_SIZE*2; Removed as fifo is now in TUINTs
  data.dataSize = MGB_HEADER_SIZE;
  /* Extract the message header to the linear buffer */
  if (dsiuFifoOut (fifo, &data) == NULL)  {
    dsiuFifoReset (fifo);
    return (-1);  
  }

  /* Extract the message body */
  if (msg->header.length > MGB_HEADER_SIZE)  {
//    data.dataSize = (msg->header.length - MGB_HEADER_SIZE)*2; Removed as fifo is now in TUINTs */
	data.dataSize = (msg->header.length - MGB_HEADER_SIZE);
    data.data     = (void *)&msg->msg_body;
    if (dsiuFifoOut (fifo, &data) == NULL)  {
      dsiuFifoReset (fifo);
      return (-1);
    }
  }
  
#if defined(DSIU_SIMULATE_APPLICATION) && defined(RELMSG_SUPPORT)
  if(siuProcMsgAck(msg))
    return (0);
#endif  

  /* Process the message */
  (*processFunc)(msg);

  return (0);
} /* siu_ut_process_message */

/******************************************************************************
 * FUNCTION PURPOSE: Pass the formatted cells to the HAL Utopia driver
 ******************************************************************************
 * DESCRIPTION: Receives segmented and formatted cells. Each of these cells
 *              will have its top level header modified.
 ******************************************************************************/
tint siu_utmdu_msg_send (void *inst, xferPktInfo_t *pkt)
{
  dsiuNvInst_t          *nvInst = (dsiuNvInst_t *)inst;
  tsiuRouteTagHeader_t  *rth;
  tsiuVcDemuxHeader_t   *vcd;
  tsiuVc53DemuxHeader_t *vcd53;
  dsiuNduContext_t      *cxt;
  halUtopiaData_t        data;

  tuint   cellSizeTuint;
  tuint   pktSizeBytes;
  tint    pktSizeTuint;
  tint    rtype;
  tint    vtype;
  tuint   i;
  tuint  *p;

  /* Verify the message priority. Only normal priority messages are allowed */
  if (nvInst->ID != SIU_NVCH_MSG_UT_NORMAL)  {
    tsiuContext.nduStats.TxInvldNvCh += 1;
    return (DSIU_FAIL);
  }

  /* Shorthand to the NDU context */
  cxt = &tsiuContext.siuNduContext;

  /* The message is verified as a normal priority message */
  rtype = SIU_UTOPIA_RT_TYPE_NORMAL_MSG;
  vtype = DSIU_UTOPIA_VC_MSG_CHANNEL;
  tsiuContext.nduStats.TxCtrlMsgCnt++;

  /* The cells are grouped into a single "block" */
  p = (tuint *)(pkt->pktIn[0]);

  /* Modify the header for each cell */
  cellSizeTuint = SIU_BYTES_TO_TUINT (cxt->cellSize);
  pktSizeTuint  = SIU_BYTES_TO_TUINT (pkt->pktSize[0]);
  for (i = 0; i < pktSizeTuint; i += cellSizeTuint)  {

    /* The cell headers are modified based on the cell mode (route tag or vc demux)
     * which is determined by the cell size */
    switch (cxt->cellSize)  {

      case 58:
        rth = (tsiuRouteTagHeader_t *)&p[i];
        /* Put the route tag type into the route tag field */
        SIU_UTOPIA_RT_SET_TYPE(rth, rtype);
        /* Set the channel number */
        SIU_UTOPIA_RT_SET_CHANNEL(rth, nvInst->vcIdTx);
        /* Add the core ID */
        DSIU_UTOPIA_RT_SET_CORE_ID(rth, dsiuContext.coreId);
        break;

      case 54:
        vcd = (tsiuVcDemuxHeader_t *)&p[i];
        /* the channel field of the message type is appended with the core ID
         * and the message priority */
        DSIU_UTOPIA_VC_SET_VCI_CHANNEL(vcd, vtype);
        /* Add the core Id */
        DSIU_UTOPIA_VC_SET_VCI_CORE(vcd, dsiuContext.coreId);
        break;

      case 53:
        vcd53 = (tsiuVc53DemuxHeader_t *)&p[i];
        /* the channel field of the message type is appended with the core ID
         * and the message priority */
        DSIU_UTOPIA_VC53_SET_VCI_CHANNEL(vcd53, vtype);
        /* Add the core Id */
        DSIU_UTOPIA_VC53_SET_VCI_CORE(vcd53, dsiuContext.coreId);
        break;


      /* No default required. The cell size has already been verified */

    } /* end switch */
  
  } /* end for */

  /* Send the group of cells to the HAL utopia driver */
  pktSizeBytes = pkt->pktSize[0];
  data.nSegs   = 1;
  data.type    = 0;
  data.sizes   = &pktSizeBytes;
  data.segs    = (tuint **)pkt->pktIn;
  if (tsiuContext.hal->utopia->halUtopiaSendCell (0, 0, &data) == HAL_PKT_TX_PKT_OVERFLOW)
    dsiuContext.nduStats.TxErrDrop += 1;

  return (DSIU_SUCCESS);

} /* tint siu_utmdu_msg_send */


/*****************************************************************************
 * FUNCTION PURPOSE: Send a message to the host
 *****************************************************************************
 * DESCRIPTION: A message is segmented into aal5 cells and sent to the hal
 *              utopia interface.
 *****************************************************************************/
tint siu_ut_send_message (void *vmsg, siuDPRMsgType_t mtype)
{
  mgb_t *msg = (mgb_t *)vmsg;
  xferPktInfo_t      pkt;
  void              *data;
  dsiuNduContext_t  *cxt;
  dsiuNvInst_t      *nvInst;
  tint               pktSizeBytes;
  tint               ret;

  /* Shorthand to the NDU context */
  cxt= &(tsiuContext.siuNduContext);

  /* Store the message ID for the return value */
  ret = msg->header.msg_id;

  /* Only one message type is supported */
  nvInst = &(cxt->siuMsgNormalUt);

  /* If the instance is null this interface does not support messaging */
  if (!nvInst)
    return (0);  /* 0 indicates message failed */

  /* Store the channel number. In route tag mode this information is lost
   * in the segmentation process and must be post-fed into the header
   * after segmentation */
  nvInst->vcIdTx = msg->header.chnum;

  switch (cxt->cellSize) {

    case 58:
      data = (void *)&(msg->header.msg_id);
      pktSizeBytes = SIU_TUINT_TO_BYTES(msg->header.length - 
                     MGB_HEADER_SIZE + SIU_ROUTE_TAG_MSG_HEADER_LENGTH);
      break;

    /* In VC Demux mode, the message header must be changed to the
     * vc demux message header. The VCI channel may have to be modfied as well*/
    case 53:
    case 54:
      /* The packet size must be computed before modifiing the header */
      pktSizeBytes = (msg->header.length - MGB_HEADER_SIZE +
                     SIU_VC_DEMUX_MSG_HEADER_LENGTH)<<1;
      data = siu_ndu_com_modify_vc_msg_header (mtype, msg);
      break;
  } /* end switch */

  /* Pass the formatted message to NEU. Once segmented it will be sent to
   * siu_utmdu_msg_send. */
  pkt.type     = XFER_PKT_TYPE_MSG_NORMAL;  
  pkt.npkts    = 1;
  pkt.pktSize  = &pktSizeBytes;
  pkt.pktIn    = &data;

  (*diagCfg.ctable.neu->neuSendIn) (nvInst->neuInst, (xferpktInfo_t *)&pkt);
  
  /* Return the message id */
  return (ret);

} /* siu_ut_send_message */

/*****************************************************************************
 * FUNCTION PURPOSE: Send a message to the host
 *****************************************************************************
 * DESCRIPTION: A message is segmented into aal5 cells and sent to the hal
 *              utopia interface.
 *****************************************************************************/
tint siuUtPushFifo (void *vmsg)
{
  siuDPRMsgType_t mtype;

  /* Extract the message type from the message ID */
  mtype = siuMessageLevel (msg->header.msg_id);
 
  /* Send the message to the host */
  return (siu_ut_send_message (vmsg, mtype));
} /* siuUtPushFifo */


/*****************************************************************************
 * FUNCTION PURPOSE: Send a message to the host
 *****************************************************************************
 * DESCRIPTION: Functionally nearly the same as siuUtPushFifo. The only 
 *              difference is that the priority of the message is input
 *              and not extracted from the message ID.
 *****************************************************************************/
tint siuUtWriteMessageToDPR (mgb_t *message, bool block, siuDPRMsgType_t type)
{
  DSIU_DUMMY_ARG(block);
  return (siu_ut_send_message ((void *)message, type));
} /* siuUtWriteMessageToDPR */


/*****************************************************************************
 * FUNCTION PURPOSE: Dummy function
 *****************************************************************************
 * DESCRIPTION: Dummy function for nudFifo2DPR. This function is not required
 *              for the utopia interface.
 *****************************************************************************/
tint siuUtFifo2DPR (tint dpr_id)
{
  DSIU_DUMMY_ARG(dpr_id);
  return (0);
} /* siuUtFifo2DPR */

/******************************************************************************
 * FUNCTION PURPOSE: Message initialization
 ******************************************************************************
 * DESCRIPTION: A dummy function for the utopia interface.
 *****************************************************************************/
void siuUtMsgInit (void)
{
  /* empty */
} /* siuUtMsgInit */


/*****************************************************************************
 * FUNCTION PURPOSE: Format and queue a received message
 *****************************************************************************
 * DESCRIPTION: The function is called with a message that has been
 *              processed through NEU. The standard message header is
 *              created and the message is placed into the appropriate
 *              queue.
 *****************************************************************************/
tint siu_utmdu_rcv_message (void *inst, xferPktInfo_t *pkt)
{
  tsiuRouteTagMsgPayload_t *rtmsg;
  tsiuVcDemuxMsgPayload_t  *vcmsg;

  dsiuNvInst_t     *nvInst = (dsiuNvInst_t *)inst;
  dsiuFifo_t       *msgFifo;
  dsiuNduContext_t *cxt;
  dsiuNduStats_t   *stats;

  mgbHeader_t       header;
  globData_t        data;

  tuint            *rxMsgDropPtr;
  tuint            *rxMsgCntPtr;
  tuint            *body;

  /* Shorthand to the NDU context and stats */
  cxt   = &tsiuContext.siuNduContext;
  stats = &tsiuContext.nduStats;

  /* Only normal priority messages are supported */
  if (nvInst->ID != SIU_NVCH_MSG_UT_NORMAL)  {
    stats->RxInvldNvCh += 1;
    return (DSIU_FAIL);
  }

  msgFifo      = &(cxt->msgNormalFifo);
  rxMsgDropPtr = &(stats->RxCtrlMsgDrop);
  rxMsgCntPtr  = &(stats->RxCtrlMsgCnt);

  /* The format of arriving messages varies by the cell size, which defines
   * the transfer mode (route tag vs vc demux) */
  switch (cxt->cellSize)  {

    case 58:  /* Route Tag mode */
      rtmsg           = (tsiuRouteTagMsgPayload_t *)(pkt->pktIn[0]);
      header.length   = SIU_BYTES_TO_WORDS(pkt->pktSize[0]) + MGB_HEADER_SIZE - SIU_ROUTE_TAG_MSG_HEADER_LENGTH;
      header.chnum    = nvInst->vcIdRx;
      header.msg_id   = rtmsg->msg_id;
      header.trans_id = rtmsg->trans_id;
      body            = &(rtmsg->msg_first_word);
      break;

    case 53:
    case 54:  /* VC demux */
      vcmsg           = (tsiuVcDemuxMsgPayload_t *)(pkt->pktIn[0]);
      header.length   = SIU_BYTES_TO_WORDS(pkt->pktSize[0]) + MGB_HEADER_SIZE - SIU_VC_DEMUX_MSG_HEADER_LENGTH;
      header.chnum    = DSIU_UTOPIA_VC_DEMUX_MSG_EXTRACT_CHANNEL(vcmsg);
      header.msg_id   = vcmsg->msg_id;
      header.trans_id = vcmsg->trans_id;
      body            = &(vcmsg->msg_first_word);
      break;

    /* No default required. The cell size has been verified */

  }

  /* Put the message into the appropriate fifo */
  if (dsiuFifoSizeAvail (msgFifo) < header.length)  {
    *rxMsgDropPtr += 1;
    return (DSIU_FAIL);
  }

  /* Put the message header into the fifo */
  data.dataSize = MGB_HEADER_SIZE;
  data.data = (void *)&header;

  if (dsiuFifoIn (msgFifo, &data) == NULL)  {
    *rxMsgDropPtr += 1;
    dsiuFifoReset (msgFifo); /* Fifo out of sync */
    return (DSIU_FAIL);
  }

  /* Put the message body into the fifo */
  data.dataSize = (header.length - MGB_HEADER_SIZE);
  data.data     = (void *)body;

  if (dsiuFifoIn (msgFifo, &data) == NULL)  {
    *rxMsgDropPtr += 1;
    dsiuFifoReset (msgFifo);  /* Fifo out of sync */
    return (DSIU_FAIL);
  }

  /* The message is successfully in the fifo */
  *rxMsgCntPtr += 1;

  return (DSIU_SUCCESS);

} /* siu_utmdu_rcv_message */



