/*****************************************************************************
 * FILE PURPOSE: Implement the NDU call table for Utopia interface
 *****************************************************************************
 * FILE NAME: siuutndu.c
 *
 * DESCRIPTION: Implements the ndu call table defined as:
 *              nduOpen:    Channel specific open
 *              nduColose:  Channel specific close
 *              nduControl: System level initialization
 *
 *              nduSendIn:       Cell level input
 *              nduReceiveIn:    Polled function to find cells
 *              nduReceiveInInt: Interrupt function for received cells
 *
 *              nduGetNormalMessage:    searches from host message fifo
 *              nduGetInterruptMessage: searches from host message fifo
 *              nduPushFifo:            queues message to the host
 *              nduFifo2DPR:            sends queued message to the host
 *              nduWriteMessageToDPR:   sends message to host, no queue
 *              nduMsgInit:             initializes message interface
 *              nduMSG_test:            message spoof utility
 *
 *              nduHeartBeat:   system stability check
 *              nduGetClockMul: Returns clock multiplier stored in NDU memory
 *
 *
 * TABS: None
 *
 * Copyright (C) 2002 Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include "types.h"
#include "utlxferpkt.h"
#include <ti/mas/iface/ifpkt/xferpkt.h>
#include "hal.h"
#include <ti/mas/neu/neu.h>

/* Different include files are used for the diagnostic code. The include
 * file typedefs the name tsiuInst_t to be either dsiuInst_t (for diag)
 * or siuInst_t (for application). The siu context is also typedefed */
#ifdef DSIU
#include "dsiuloc.h"
#include "dsiuutop.h"
#include "dsiuport.h"
#include <ti/mas/neu/aal5.h>
#include "diagapp.h"
#include "diagcfg.h"
#include <ti/mas/neu/neuproto.h>
#else
#include "siuloc.h"
#endif

/*****************************************************************************
 * For the diagnostic code, the call table instantiated 
 *****************************************************************************/
#ifdef DSIU
siuNduCallTable_t siuUtopiaNduCallTable = {
  siuNduComOpen,              /* Common NDU function */
  siuNduComClose,             /* Common NDU function */
  siuUtNduSendIn,
  siuUtNduReceiveIn,
  siuUtNduIntReceiveIn,
  siuUtNduControl,

  siuNdComGetNormalMessage,
  siuNdComGetInterruptMessage,
  siuUtWriteMessageToDPR,
  siuNduComMSG_test,
  siuUtHeartBeat,
  siuUtGetClockMul,
  siuUtMsgInit,
  siuUtPushFifo,
  siuUtFifo2DPR
};
#endif /* DSIU */

/******************************************************************************
 * FUNCTION PURPOSE: Initializes the Utopia interface.
 ******************************************************************************
 * DESCRIPTION: This function initializes the application to handle the utopia 
 *              cells. The input parameter is for compatibility with the
 *              legacy HPI.
 *****************************************************************************/
tint siuUtNduControl (tint ctlCode)
{
  dsiuNduContext_t   *cxt;
  neuSizeCfg_t        neuCfgSize;

  /* Verify that the HAL interface supports utopia send out */
  if (!tsiuContext.hal->utopia)
    return (-1);

  /* Shorthand reference to NDU control structure */
  cxt = &(tsiuContext.siuNduContext);

  /* The utopia NDU requires that neu be in build */
  if (!diagCfg.ctable.neu)
    dsiuPanic (DSIU_FAIL_UTNDU_NO_NEU);

  /* If the cellsize is 0 then the utopia interface is not used for messges */
  if (!dsiuContext.diagApp.u.utopia.cellSize)  {
    cxt->siuMsgNormalUt.neuInst = NULL;
    return (0);
  }

  /* In initialization create the neu instance */
  if (ctlCode == SIU_NDU_CTRL_INIT)  {

    /* Create the neu for normal priority messages */
    /* The aal5 sizes are defined in diagbld.h*/
    neuCfgSize.rxAssemSize       = diagCfg.aal5AssemblySize;
    neuCfgSize.txScratchSize     = diagCfg.neuSegSize;
    neuCfgSize.txHeaderSize      = SIU_BYTES_TO_WORDS(dsiuContext.diagApp.u.utopia.msg.totalHeaderLenBytes);
    neuCfgSize.sssar_required    = FALSE;    /* Diag/bootloader never use SSSAR */
    cxt->siuMsgNormalUt.ID       = SIU_NVCH_MSG_UT_NORMAL;
    cxt->siuMsgNormalUt.neuInst  = siuNeuCreate (&neuCfgSize, DSIU_MSG_UT_NEU_ECO_ID);

    /* Panic if the neu creation failed */
    if (!(cxt->siuMsgNormalUt.neuInst))
      dsiuPanic (DSIU_FAIL_NEU_FAIL);

    /* Open the newly created neu. Logical port 0 is used. */
    siu_ndu_com_open_neu (cxt->siuMsgNormalUt.neuInst, dsiuContext.dregMsgCfgShadow, &(cxt->siuMsgNormalUt), 
                          siu_utmdu_msg_send, dsiuContext.diagApp.u.utopia.cellSize, dsiuContext.utopiaPort[0].halId);

    /* Enable the NEU channel receive callouts */
    siu_ndu_com_rxcall_neu (cxt->siuMsgNormalUt.neuInst, siu_utmdu_rcv_message, &(cxt->siuMsgNormalUt));

  } 

  /* Nothing to do for other control codes */
 
  return (0);

} /* siuUtNduControl */

/******************************************************************************
 * FUNCTION PURPOSE: Send formatted cells to the network.
 ******************************************************************************
 * DESCRIPTION: Formatted cells (with route tags and/or ATM headers are
 *              passed to the hal layer to be sent to the network
 *****************************************************************************/
tint siuUtNduSendIn (void *inst, xferPktInfo_t *pktInfo)
{
  halUtopiaData_t haldata;
  halIntrLevel_t  level;

  DSIU_DUMMY_ARG(inst);

  /* Translate the data into hal format */
  haldata.nSegs = (tuint)   pktInfo->npkts;
  haldata.segs  = (tuint **)pktInfo->pktIn;
  haldata.sizes = (tuint *) pktInfo->pktSize;
  haldata.type  = 0; /* types don't match */

  level = tsiuContext.hal->halIntrLevel (HAL_INT_LEVEL_DISABLE);

  /* Send the Cell. Existance of the function call was verified in nduControl */
  if ( (*tsiuContext.hal->utopia->halUtopiaSendCell) (dsiuContext.utopiaPort[0].halId, 0, &haldata) != HAL_SUCCESS)  {
    dsiuContext.utopiaPort[0].status->txDrop += 1;
    tsiuContext.hal->halIntrLevel (level);
    return (-1);
  }

  if (dsiuContext.diagApp.u.utopia.cellSize == 0)
    dsiuPanic (DSIU_FAIL_UTOPIA_CELL_SIZE_BECAME_0);

  dsiuContext.utopiaPort[0].status->txDataCount += (pktInfo->npkts * dsiuContext.diagApp.u.utopia.cellSize);
  tsiuContext.hal->halIntrLevel (level);

  return (0);
} /* siuUtNduSendIn */
  
/*****************************************************************************
 * FUNCTION PURPOSE: Poll for packets.
 *****************************************************************************
 * DESCRIPTION: Function called from the application to poll for packets.
 *              For the utopia interface this is unused, since arriving cells
 *              are serviced through an interrupt. A dummy function is
 *              required to populate the call table
 *****************************************************************************/
void siuUtNduReceiveIn (void)
{
  /* empty */
} /* siuUtNduReceiveIn */

/*****************************************************************************
 * FUNCTION PURPOSE: Interrupt handler for cells
 *****************************************************************************
 * DESCRIPTION: This function is called from the hal driver when cells are
 *              available. This function determines the cell type. Messages
 *              are routed directly to the aal5 library. All other types are
 *              sent to the ndu receive out function pointer.
 *****************************************************************************/
volatile tuint wait_ndu =1;

void siuUtNduIntReceiveIn (void *vcxt, word *cell, tuint sizeBytes)
{
  tsiuRouteTagHeader_t *rth;
  tsiuVcDemuxHeader_t  *vcd;
  tsiuVc53DemuxHeader_t  *vcd53;
  xferPktInfo_t         pktOut;
  dsiuNvInst_t         *msgp;
  tuint                *pkt_p;
  dsiuNduContext_t     *cxt;

  tuint type;
  tuint cellSizeWords;
  tuint cellSizeBytes;
  tint channel;
  tsiuInst_t *inst;
  tint p = 0;

  halUtopiaControl_t utctl;


  /* Inc the interrupt stat, even though this function can be
   * called through a poll */
  dsiuInc (&(dsiuContext.utopiaPort[0].status->intCount));
  

  /* Shorthand to the NDU context */
  cxt = &(dsiuContext.siuNduContext);

  /* Convert the individual cell size to words */
  cellSizeWords = SIU_BYTES_TO_WORDS(tsiuContext.siuNduContext.cellSize);
  cellSizeBytes = tsiuContext.siuNduContext.cellSize;

  /* Process all cells received */
  while (sizeBytes >= tsiuContext.siuNduContext.cellSize)  {
    msgp = NULL; /* Set to non-null for message types */
    dsiuContext.utopiaPort[0].status->rxDataCount += tsiuContext.siuNduContext.cellSize;

    switch (tsiuContext.siuNduContext.cellSize) {
      case 58:
        /* Route tags used - routing information is in the type field */
        rth     = (tsiuRouteTagHeader_t *)&cell[p];
        channel = SIU_UTOPIA_RT_EXTRACT_CHANNEL(rth);
        type    = SIU_UTOPIA_RT_EXTRACT_TYPE(rth);
        if (type == SIU_UTOPIA_RT_TYPE_NORMAL_MSG)  {
          msgp = &(cxt->siuMsgNormalUt);
        }
        break;

      case 54: 
        /* VC Demux used - routing information is in the atm header */
        vcd = (tsiuVcDemuxHeader_t *)&cell[p];
        channel = DSIU_UTOPIA_VC_EXTRACT_VCI_CHANNEL(vcd);
        if (channel == DSIU_UTOPIA_VC_MSG_CHANNEL) {
          msgp     = &(cxt->siuMsgNormalUt);
        }
        else 
          channel -= 5;  /* Index channels from 1 */
        break;

      case 53:
        /* VC Demux used - routing information is in the atm header */
        vcd53 = (tsiuVc53DemuxHeader_t *)&cell[p];
        channel = DSIU_UTOPIA_VC53_EXTRACT_VCI_CHANNEL(vcd53);
        if (channel == DSIU_UTOPIA_VC_MSG_CHANNEL) {
          msgp     = &(cxt->siuMsgNormalUt);
        }
        else 
          channel -= 5;  /* Index channels from 1 */
        break;

      /* No default needed - cell size must be verified before utopia starts */
    } /* end switch (cellSize) */

    /* Package the cell for shipout */
    pkt_p          = (tuint *)&cell[p];
    pktOut.pktSize = (tint *)&cellSizeBytes;
    pktOut.pktIn   = (void *) &(pkt_p);
    pktOut.type    =  XFER_PKT_TYPE_AAL1; /* Always use aal1 */
    pktOut.npkts   = 1;
   
    /* Route message/packets */     
    if (msgp)  { /* If non-NULL then the cell is a message cell */
      /* Pass the entire cell to the message handler */
      msgp->vcIdRx = channel;
      (*diagCfg.ctable.neu->neuReceiveInUt) (msgp->neuInst, (xferpktInfo_t *)&pktOut);
    }
    else { /* The cell is not a message */  
      /* Extract siu instance from channel number */
      if ((channel <= 0) || (channel > SIU_LASTCH))  {
        tsiuContext.nduStats.RxErrChnum += 1;

        utctl.mode = HAL_UTOPIA_BAD_RX_CELL;
        (*tsiuContext.hal->utopia->halUtopiaControl)(0, &utctl);

        break;
      }  

/*****************************************************************************
* Perform even/odd channel translation for vpbx external utopia loopback 
******************************************************************************/
#ifdef NDU_DBG4
      if (channel & 0x0001)
        channel += 1;
      else
        channel -= 1;
#endif
/****************************************************************************/

      /* Extract the channel instance */
      inst = &tsiuInst[channel - SIU_FIRSTCH];
   
      /* Send the cell if the channel is expecting non-message data from utopia */
      if ( (inst->nduInst.RxOutSpec.nduReceiveOut) &&
           (inst->nduInst.recInterface == SIU_NDU_TYPE_UT) )
        (*inst->nduInst.RxOutSpec.nduReceiveOut) (
          inst->nduInst.RxOutSpec.targetInst, &pktOut);
       
    } /* end else - non-message handler */         

    /* Advance to the next cell */
    sizeBytes -= tsiuContext.siuNduContext.cellSize;
    p         += cellSizeWords;

  } /* end while (size >= tsiuContext.nduContext.cellSize) */
        
} /* siuUtIntReceiveIn */



/****************************************************************************
 * FUNCTION PURPOSE: System Heart Beat Update
 ****************************************************************************
 * DESCRIPTION: A dummy function for the utopia interface.
 ****************************************************************************/
void siuUtHeartBeat (tint mode)
{
  DSIU_DUMMY_ARG(mode);
  /* empty */
} /* siuUtHeartBeat */

/*****************************************************************************
 * FUNCTION PURPOSE: Return system clock multiplier
 *****************************************************************************
 * DESCRIPTION: A dummy function for the utopia interface.
 *****************************************************************************/
tuint siuUtGetClockMul (void)
{
  return (0);  /* empty */
}  /* siuUtGetClockMul */




