/************************************************************************/
/*                                                                      */
/*     Copyright (C) 2000  Texas Instruments Incorporated.             
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
/*                                    Revision:   1.0                   */
/************************************************************************/
/* FILE PURPOSE: TARGET DSP Hardware Configuration                      */
/************************************************************************/
/*                                                                      */
/* File Name  : targethw.c                                              */
/*                                                                      */
/* DESCRIPTION: Defines memory map, and structures which describe the   */
/*              TARGET hardware.                                        */
/*                                                                      */
/* REQUIRES:    targethw.h  (target hardware Defs)                      */
/*              c55xint.h   (TMS320C55x interrupt definitions)          */
/*              c55xreg.h   (TMS320C55x register definitions)           */
/*                                                                      */
/*                                                                      */
/*                                                                      */
/* Tools Version : 1.60                                                 */
/*                                                                      */
/* Date : 8th November 2000                 OWNERSHIP : Ebenezer V.     */
/************************************************************************/

#include "types.h"
cregister unsigned int CSR ;
cregister unsigned int IER ;
#define GIE         0

/*****************************************************************************
 * DESCRIPTION: This variable is a Debug option for critical sections        
 *              embeddement implementation.
 *****************************************************************************/
/* #define THW_BIT_STACK_DBG */
#ifdef THW_BIT_STACK_DBG
  volatile tuint thwBitStack = 0;    /* It is used for stack depth measurement */
  volatile tint thwMaxBitStack = 0;  /* Max. bit-stack level */
  volatile tint thwMinBitStack = 0;  /* Min. bit-stack level */
#endif

/*****************************************************************************
 * DESCRIPTION: This variable is specific bit-stack implementation. It is 
 *            manipulated by thwCriticalBegin() and thwCriticalEnd() functions
 *            in order to properly recover status of INTM bit after 
 *            completion of critical section. Up to 16 bits can be shifted in
 *            and out, meaning that 15-level nesting of critical sections
 *            is allowed (highly unlikely).
 *****************************************************************************/          
volatile tuint thwCriticalStatusSaved = 0;

/******************************************************************************
 * FUNCTION PURPOSE: Disable All Interrupts
 ******************************************************************************
 * DESCRIPTION: Disable all interrupts without checks.
 *
 ******************************************************************************/
void thwDisableAllInt (void)
{
    IER = 0;
    CSR &= ~(0x1<<GIE);
} /* thwDisableAllInt */



/******************************************************************************
 * FUNCTION PURPOSE: Start of critical section (up to 15 sections can be nested)
 *                   
 ******************************************************************************
 * DESCRIPTION:  Save current interrupt enable/disable status, and disable 
 *               all interrupts. Bit ST1_INTM is pushed in global variable.
 *               IER0 and IER1 are saved and restored
 *****************************************************************************/
#ifdef HW_USE_C_VERSION
void thwCriticalBegin (void)
{

  tulong ierStore;


  /* Disable interrupts by clearing the IER0 */
  ierStore = IER;
  IER = 0;
  /* Disable interrupts by clearing the IER1 */

  if(!(CSR & (0x1<<GIE))) /* Is global interrupt enabled ? */
  {
    CSR &= ~(0x1<<GIE);
//    asm("	bit(st1, #st1_intm) = #1"); /* Always disable interrupt on exit */
    asm ("  NOP");
    asm ("  NOP");
    asm ("  NOP");
    thwCriticalStatusSaved <<= 1;  /* Bit stack prepared */
    thwCriticalStatusSaved |= 1; /* It is disabled ! */
  }
  else {                           /* else ... means it is enabled */
    CSR &= ~(0x1<<GIE);
  //  asm("	bit(st1, #st1_intm) = #1"); /* Always disable interrupt on exit */
    asm ("  NOP");
    asm ("  NOP");
    asm ("  NOP");
    thwCriticalStatusSaved <<= 1;  /* Bit stack prepared */
  }
#ifdef THW_BIT_STACK_DBG  
  thwBitStack ++;                  /* Increase depth of stack */
  if(thwMaxBitStack < thwBitStack)
    thwMaxBitStack = thwBitStack;  /* Record max bit-stack depth */
  if(thwMaxBitStack > 10) {
     for (;;) {
       if(thwMaxBitStack == 0)     /* Set me to zero, and then do the single
                                    * stepping to find out who does the mess */
         break;
     }
  }  
#endif
  IER = ierStore;
}
#endif
/******************************************************************************
 * FUNCTION PURPOSE: End of critical section (up to 15 sections can be nested)
 *                   
 ******************************************************************************
 * DESCRIPTION: Restore interrupt enable/disable status encountered on entry to
 *              critical section. Bit ST1_INTM is poped from the global variable.
 *****************************************************************************/
#ifdef HW_USE_C_VERSION
void thwCriticalEnd (void)
{
#ifdef THW_BIT_STACK_DBG
  thwBitStack --;                   /* Decrease depth of stack */
  if(thwMinBitStack > thwBitStack)  
    thwMinBitStack = thwBitStack;   /* Record min bit-stack underflow */
#endif

  if(thwCriticalStatusSaved & 0x0001) /* Was the last global interrupt 
                                       * disabled ? */
    thwCriticalStatusSaved >>= 1;     /* Drop the last bit saved !
                                       * It was already disabled, prior to
                                       * critical section entry */
  else { 
    thwCriticalStatusSaved >>= 1;     /* Drop the last bit saved ! */
    asm ("   NOP");
    asm ("   NOP");
    asm ("   NOP");
//    asm("	bit(st1, #st1_intm) = #0"); /* It was enabled prior to critical
//                                         * section entry */
    CSR |= (0x1<<GIE);
  }
}
#endif
/******************************************************************************
 * FUNCTION PURPOSE: Begin critical section; returns old value
 *****************************************************************************/
tulong thwCriticalBeginFastFcn(void) {
  return (tulong)_disable_interrupts();
}
/******************************************************************************
 * FUNCTION PURPOSE: End critical section; uses old value
 *****************************************************************************/
void thwCriticalEndFastFcn(tulong old_val) {
  _restore_interrupts(old_val);
}

/* nothing after this point */
