#ifndef _DVPU_H
#define _DVPU_H
/*****************************************************************************
 * FILE PURPOSE: Defines and API for the Diagnostic Voice Playout Unit (DVPU)
 *****************************************************************************
 * FILE NAME: dvpu.h
 *
 * DESCRIPTION: Defines the API used to interact with the DVPU module
 * 
 * TABS: None
 *
 * Copyright (C) 2002 Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include "types.h"
#include "dglob.h"

/*****************************************************************************
 * DEFINITION: Return values from DVPU
 *****************************************************************************/
typedef enum {
  DVPU_SUCCESS=0,              /* Operation Successful             */
  DVPU_ERR_STATE_CLOSED,       /* Operation failed, DVPU is closed */
  DVPU_ERR_NO_DATA,            /* Request for data failed          */
  DVPU_ERR_DATASIZE,           /* Input data size too large        */
  DVPU_ERR_INVALID_OPEN,       /* Open DVPU failed                 */
  DVPU_ERR_INVALID_OP_DELAY,   /* Invalid operating delay at open  */
  DVPU_ERR_INVALID_MIN_DELAY   /* Invalid min delay at open        */
} dvpuReturn_t;


/*****************************************************************************
 * Structure used by DVPU to determine its size requirements
 *****************************************************************************/
typedef struct {
  tuint circBufSize;    /* Number of elements in circular buffer */
} dvpuSizeConfig_t;

/*****************************************************************************
 * Structure used by DVPU to create a new instance
 *****************************************************************************/
typedef struct {
  tuint Id;                 /* Instance Id */
  dvpuSizeConfig_t sizeCfg;
} dvpuNewConfig_t;

/*****************************************************************************
 * Structure used to configure DVPU when the instance is opened
 *****************************************************************************/
typedef struct {
  tuint opDelay;
  tuint minDelay;
} dvpuOpenConfig_t;

/*****************************************************************************
 * DVPU function for placing data into a channels VPU fifo
 *****************************************************************************/
dvpuReturn_t dvpuReceiveIn (void *dvpuInst, globData_t *data);

/*****************************************************************************
 * DVPU function for removing data from a channels VPU fifo
 *****************************************************************************/
dvpuReturn_t dvpuReceiveOut (void *dvpuInst, globData_t *data);

/*****************************************************************************
 * DVPU new and configuration function calls 
 *****************************************************************************/
dvpuReturn_t dvpuGetSizes (tint *nbufs, dmemBuffer_t **bufs,
                           dvpuSizeConfig_t *cfg);
void *dvpuNew (dvpuNewConfig_t *cfg, dmemBuffer_t *bufs);
dvpuReturn_t dvpuOpen (void *dvpuInst, dvpuOpenConfig_t *cfg);



/****************************************************************************
 * DVPU API call table. The dvpu API is referenced by the system through a
 * pointer to this call table.
 ****************************************************************************/
typedef struct {
  dvpuReturn_t (*dvpuReceiveIn)  (void *, globData_t *);
  dvpuReturn_t (*dvpuReceiveOut) (void *, globData_t *);
  dvpuReturn_t (*dvpuGetSizes)   (tint *, dmemBuffer_t **, dvpuSizeConfig_t *);
  void *       (*dvpuNew)        (dvpuNewConfig_t *, dmemBuffer_t *);
  dvpuReturn_t (*dvpuOpen)       (void *, dvpuOpenConfig_t *);
} dvpuCallTable_t;

extern dvpuCallTable_t dvpuCallTable;



#endif /* _DVPU_H */



