#ifndef _ipiu_h
#define _ipiu_h

/******************************************************************************
 * FILE PURPOSE: Defines API for diagnostic PIU (PCM input/output Unit)
 ******************************************************************************
 * FILE NAME: ipiu.h
 *
 * DESCRIPTION: Defines the API used by the system to exchange data with IPIU
 *
 * TABS: None
 *
 * Copyright (C) 2007, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include "types.h"
#include "dglob.h"

/****************************************************************************
 * DEFINITION: Return values from ipiu
 *****************************************************************************/
typedef enum {
  IPIU_SUCCESS = 0,  
  IPIU_ERR_STATE_CLOSED,
  IPIU_ERR_INVALID_OPEN,
  IPIU_ERR_INVALID_DATA_LENGTH,
  IPIU_ERR_INVALID_CTL
} ipiuReturn_t;

/*****************************************************************************
 * IPIU size configuration structure. Used by IPIU to determine its memory
 * requirements.
 *****************************************************************************/
typedef struct {
  itdmMode_t  mode;
  tint        tdmBufTxSize;   /* Size, in words of the IPIU TDM buffers */
  tint        tdmBufRxSize;
} ipiuSizeConfig_t;

/****************************************************************************
 * IPIU New configuration structure. 
 ****************************************************************************/
typedef struct {
  tuint ID;                   /* Instance ID                                 */
  itdmMode_t  mode;
  void *taskInst;             /* Task posting instance                       */
  tint (*txTaskCall)(void *); /* Function to call when tdm tx data needed    */
  tint (*txHalUpdate)(void *, word **); /* update tslot map for hal */
  tint (*rxHalUpdate)(void *);
  tuint *txTaskPostTimeStamp;
  ipiuSizeConfig_t sizeCfg;
} ipiuNewConfig_t;

/*****************************************************************************
 * IPIU Open configuration structre.
 *****************************************************************************/
typedef struct {
  tuint  rxInitVal;       /* Initial value sent to TDM out                   */ 
  word   *rxInitBuf;      /* Output - 1st two rx (to tdm) buffer locations   */
  word   *txInitBuf;      /* Output - 1st two tx (from tdm) buffer locations */
  tulong curr_isr_clock;
} ipiuOpenConfig_t;

/*****************************************************************************
 * IPIU control
 *****************************************************************************/
typedef struct ipiuReset_s {
  linSample *txBuf;
  linSample *rxBuf;
} ipiuReset_t;

enum {
  IPIU_CTL_MODE_RESET  = 0
};

typedef struct ipiuControl_s {
  tuint            tslot;
  itdmChanUpdate_t action;
  itdmDirection_t  dir;
  itdmChanUpdateTime_t updateType;
  bool             reset;

//  union {
//    ipiuReset_t reset;
//  } u;
} ipiuControl_t;

/*****************************************************************************
 * IPIU data transfer API. ipiuTx gets data received from the TDM, ipiuRx
 * passes data to the TDM.
 *****************************************************************************/
ipiuReturn_t ipiuTx (void *ipiuInst, ITDM_DATA_EXCHANGE_T *tx);
ipiuReturn_t ipiuRx (void *ipiuInst, ITDM_DATA_EXCHANGE_T *rx, tint ipiuMsg);

/*****************************************************************************
 * IPIU TDM data transfer API. The application gives a pointer to this function
 * to the TDM driver of the HAL unit. This function is called when the TDM
 * driver is ready to exchange data. This function is called from an ISR
 * context.
 *****************************************************************************/
void ipiuTdmTxExchange (void *ipiuInst, itdmMultiTsMap_t *map, itdmFifoEntry_t **toTdmdata, 
                        tulong ts, tuint nsamples);
void ipiuTdmRxExchange (void *ipiuInst, itdmMultiTsMap_t *map, itdmFifoEntry_t **fromTdmdata, 
                        tulong ts, tuint nsamples);

/*****************************************************************************
 * IPIU Initialization and configuration
 *****************************************************************************/
ipiuReturn_t ipiuGetSizes (tint *nbufs, dmemBuffer_t **bufs, 
                           ipiuSizeConfig_t *cfg);
void *ipiuNew (ipiuNewConfig_t *cfg, dmemBuffer_t *bufs);
ipiuReturn_t ipiuOpen (void *ipiuInst, ipiuOpenConfig_t *cfg);
ipiuReturn_t ipiuControl (void *vinst, ipiuControl_t *ctl);
ipiuReturn_t ipiuGetTslotMap (void *vInst, itdmMapStatus_t status, itdmDirection_t dir, 
                              itdmMultiTsMap_t *map);
ipiuReturn_t ipiuSetStaggering(void *, tulong *curr_isr_clock);

/*****************************************************************************
 * IPIU API call table. The dpiu API is referenced by the system through a 
 * pointer to this call table.
 *****************************************************************************/
typedef struct {
  ipiuReturn_t (*ipiuGetSizes) (tint *, dmemBuffer_t **,  ipiuSizeConfig_t *);
  void *(*ipiuNew) (ipiuNewConfig_t *, dmemBuffer_t *);
  ipiuReturn_t (*ipiuOpen) (void *, ipiuOpenConfig_t *);
  ipiuReturn_t (*ipiuControl) (void *, ipiuControl_t *);
  ipiuReturn_t (*ipiuSetStaggering)(void *, tulong *curr_isr_clock);
  ipiuReturn_t (*ipiuTx) (void *, ITDM_DATA_EXCHANGE_T *);
  ipiuReturn_t (*ipiuRx) (void *, ITDM_DATA_EXCHANGE_T *, tint);
  void (*ipiuTdmTxExchange)(void *, itdmMultiTsMap_t *, itdmFifoEntry_t **, tulong, tuint);
  void (*ipiuTdmRxExchange)(void *, itdmMultiTsMap_t *, itdmFifoEntry_t **, tulong, tuint);
  ipiuReturn_t (*ipiuGetTslotMap) (void *vInst, itdmMapStatus_t status, itdmDirection_t dir, 
                                   itdmMultiTsMap_t *map);
} ipiuCallTable_t;
  
extern ipiuCallTable_t ipiuCallTable;
 
#endif
