#ifndef _IPIULOC_H
#define _IPIULOC_H

#include "types.h"
#include "targethw.h"

/****************************************************************************
 * FILE PURPOSE: Local definitions for ITDM piu
 ****************************************************************************
 * FILE NAME: ipiuloc.h
 *
 * DESCRIPTION: Local structures and definitions private to ipiu.
 *
 * TABS: None
 *
 * Copyright (C) 2007 Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#define MAX_TSLOTS_PER_PKT_1MILLISEC_MODE   148
#define MAX_TSLOTS_PER_PKT_125MICROSEC_MODE 512

/* fromTdm -> Net is RX buffers */
#define MAX_NUM_RX_1MSEC_BUFFS        10
#define MAX_NUM_RX_125MICROSEC_BUFFS   3
/* toTdm  <-  From Net is Tx buffers */
#define MAX_NUM_TX_1MSEC_BUFFS        26
#define MAX_NUM_TX_125MICROSEC_BUFFS   7

/*****************************************************************************
 * IPIU Circular buffer definition. This is not implemented as a true 
 * circular buffer, but as a segment buffer. Data can be placed into or
 * removed from this buffer only in frameLength amounts. The value lastIndex
 * is computed to be the last value at which the next index value will be 0
 * after data is added or removed. The element size represents the true length
 * of the data buffer.
 *
 * There are 2 buffers used by IPIU
 *  for toTdm:
 *    inIndex  - Application puts data here
 *    outIndex - Application gives data to HAL from this pointer
 *
 * for fromTdm:
 *    inIndex  - Application gives this pointer to HAL to put data here
 *    outIndex - Application gets data from here
 *****************************************************************************/
typedef struct ipiuCircBuf_s {
  itdmFifoEntry_t *fifo; 
  tuint  inIndex;   /* Location to put data into buffer    */
  tuint  outIndex;  /* Location to remove data from buffer */
  tuint  lastIndex; /* Last valid start location           */
} ipiuCircBuf_t;

/*****************************************************************************
 * Definition: IPIU Channel States
 *****************************************************************************/
typedef enum {
  IPIU_STATE_CLOSED,
  IPIU_STATE_OPEN
} ipiuState_t;

/* Critical sections for the IPIU */
/* For control path */
#define IPIU_STATE_BEGIN() (thwCriticalBegin())
#define IPIU_STATE_END()   (thwCriticalEnd())


/*****************************************************************************
 * IPIU Statistics
 *****************************************************************************/
typedef struct {
  tuint txAppMiss; /* number of time the application did not process tx data
                    * buffer before interrupted by TDM port                */
  tuint rxAppMiss; /* number of time the application did not process rx data
                    * buffer before interrupted by TDM port                */
  tuint overwrite; /* number of times the out pointer runs past the in 
                    * pointer in the circular buffers                      */
  tuint circErr;   /* number of times circular buffer exceeded bounds      */
  tuint appLengthErr; /* Number of times the amount of data received from the
                    * application was not what was expected                */
  tuint numTxResyncs;
  tuint toTdmSampleLoss; /* loss of 125 micro second sample */
  tuint fromTdmSampleLoss;
} ipiuStats_t;

typedef struct ipiuInfo_s {
    ipiuCircBuf_t    circBuf;
    itdmMultiTsMap_t activeMap;
    itdmMultiTsMap_t pendingMap;
    tuint            halUpdatePending;
}ipiuInfo_t;

/*****************************************************************************
 * IPIU instance definition. Persistant flow specific data values are
 * stored. The Diag code generally refers to the Tx and Rx direction. These
 * names are in reference to the network. So Tx data means data from the DSP
 * to the network. When data is to or from the TDM port, Tx and Rx notations
 * are not used, fromTdm and toTdm are used. The notation can be confusing: 
 * data fromTdm is Tx data.
 *****************************************************************************/
typedef struct ipiuInst_s {
  tuint              ID;                   /* Instance ID                      */
  ipiuState_t        state;                /* Instance state                   */
  itdmMode_t         mode;                 /* 1 msec or 125 microsec mode      */
  tuint              state_bitfield;
  ipiuInfo_t         fromTdm;              /* Buffers data from the TDM port   */
  ipiuInfo_t         toTdm;                /* Buffers data to the TDM port     */
  tint               txTaskPostCounter;
  bool               txResyncPending;
  tuint              *txTaskPostTimeStamp; /* 1 msec timestamp */
  tuint              RxFirstItdmPacket;    /* Non zero if first packet is received */
  void               *taskInst; /* (USE siuINST ID instead) Task channel identifier */
  tint               (*txTaskCall)(void *); /* Function to call to post Tx task */
  tint               (*txHalUpdate)(void *, word **); /* update tslot map for hal */
  tint               (*rxHalUpdate)(void *);
  ipiuStats_t        stats;
/* Delete the following two eventually, hacked for GLUE w/ current HAL */
  itdmFifoEntry_t    *inpBuffer; 
  itdmFifoEntry_t    *outBuffer;
} ipiuInst_t;

/****************************************************************************
 * DEFINITION: Defines for local circular-like buffer handling
 ****************************************************************************/
typedef enum {
  IPIU_IN,    /* Refer to the in index  */
  IPIU_OUT    /* Refer to the out index */
} ipiuCbPoint_t;

typedef enum {
  IPIU_PRE_ADVANCE,   /* Pre-advance pointer for return value  */
  IPIU_POST_ADVANCE   /* Post-advance pointer for return value */
} ipiuCbAdvance_t;

/****************************************************************************
 * Prototypes
 ****************************************************************************/
itdmFifoEntry_t *ipiu_advance_pointer (ipiuCircBuf_t *buf, ipiuCbPoint_t dir, 
                                       ipiuCbAdvance_t adv);

void ipiuPendingBecomesActive (ipiuInfo_t *info);

#endif

