/************************************************************************
:*                                                                      *
:*     @ Copyright (C) 2009  Texas Instruments Incorporated.            *
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#ifndef _ippu_h
#define _ippu_h

#include "types.h"
#include "dglob.h"
#include "utlxferpkt.h"

/* IPPU error codes */
typedef enum {
    IPPU_SUCCESS       = 0,        /* success */
    IPPU_FAILURE       = 1,         /* error */
    IPPU_MPLS_MISMATCH = 2,
    IPPU_DUPLICATE_SEQNUM = 3
}ippuReturn_t;

typedef struct ippuSizeConfig_s {
    itdmMode_t mode;          /* ITDM 125 micro sec or 1 milli sec */
    tuint      max_chanIDs;
} ippuSizeConfig_t;

typedef struct ippuNewConfig_s {
    tuint      ID;
    itdmMode_t mode;
    tulong     *currIsrTime;
    ippuSizeConfig_t sizeCfg;
} ippuNewConfig_t;

typedef struct {
    tuint bad_stats;
    tuint mplsLabelMismatch;
    tuint duplicatePkts;
    tuint jitter;
    tuint outOfOrderPkts;
    tulong tx_ITDM_packets;
    tulong rx_ITDM_packets;
    tulong noMatchingChanIdMap;
}ippuStats_t;

typedef struct ippuFlowConfig_s {
    tulong         mpls_label; /* 4 Bytes */
    tuint          timestamp;  /* 2 Bytes */
    word           seq_num;    /* 1 Byte */
    tuint          flags;
    /* pkt length and check sum aren't configured. They are calculated */
    word           UID_MSB;          
    tuint          UID_LSB;    /* FlowID - 3 Bytes */
    tuint          num_chan;
    tuint          fill_pattern;
} ippuFlowConfig_t;

typedef struct ippuChanCtrl_s {
    tuint            chanID_MSB;
    tuint            chanID_LSB;
    itdmChanUpdate_t action;
    itdmDirection_t  dir;
    itdmChanUpdateTime_t updateType;
} ippuChanCtrl_t;

typedef struct ippuCmdCtrl_s {
    tuint  cmd_id;
    tulong chan_id;
    tuint  chan_loc1;
    tuint  chan_loc2;
}ippuCmdCtrl_t;


/******************************************************************************
* DATA DEFINITION: Get IPPU Instance
*****************************************************************************/
//typedef void* (*ippuGetInstFcn_t) (word *pktBuf);

/******************************************************************************
* IPPU context definition. Non channel specific variables are stored here.
******************************************************************************/
//typedef struct ippuContext_s{
/* Debug Trace Function */
/* debugInfo_t debugInfo; */
/* Call out to update  the TDM timeslot list */
/* void *updateTslotFcn (tuint ID, ippuTdmCtrl_t *p_tdmCtrl); */
/* Call out function to report events */
/* void *reportFcn (tuint ID, tuint report_code); */
/* ippuGetInstFcn_t      getsiuInst;  */
//}ippuContext_t;

/*****************************************************************************
* IPPU Open configuration structre.
*****************************************************************************/
typedef struct ippuOpenConfig_s {
    ippuFlowConfig_t tx_flow_cfg;
    ippuFlowConfig_t rx_flow_cfg;
    tuint            max_cmd_retries;
    void             *tx_scratch_buffer;
}ippuOpenConfig_t;

typedef struct ippuControl_s{
    ippuCmdCtrl_t    cmdCtrl;
    ippuChanCtrl_t   chanCtrl;
} ippuControl_t;


ippuReturn_t ippuGetSizes(tint *nbufs, dmemBuffer_t **bufs, ippuSizeConfig_t *cfg);
void *ippuNew (ippuNewConfig_t *cfg, dmemBuffer_t *bufs);
ippuReturn_t ippuOpen (void *ippuInst, ippuOpenConfig_t *cfg);
ippuReturn_t ippuControl (void *vinst, ippuControl_t *ctl);
ippuReturn_t ippuClose (void *ippuInst);
ippuReturn_t ippuDelete (void **ippuInst);
void ippuGetStats (void *ippuInst, ippuStats_t *stats);
tint ippuReceiveIn(void *ippuInst, xferPktInfo_t *pktInfo, ITDM_DATA_EXCHANGE_T *p_tdm_data);
void ippuSendIn(void *ippuInst, globData_t *data, ITDM_DATA_EXCHANGE_T *p_tdm_data, tuint taskPostTimeStamp);
bool ippuCompareUID(void *ippuInst, word* pktbuf);
tint ippuProcessRxItdmHeader(void *ippuInst, xferPktInfo_t* pktInfo, tint *ipiuMsg);

/*****************************************************************************
* IPPU API call table. The dpiu API is referenced by the system through a 
* pointer to this call table.
*****************************************************************************/
typedef struct {
    ippuReturn_t (*ippuGetSizes) (tint *, dmemBuffer_t **,  ippuSizeConfig_t *);
    void *(*ippuNew) (ippuNewConfig_t *, dmemBuffer_t *);
    ippuReturn_t (*ippuOpen) (void *, ippuOpenConfig_t *);
    ippuReturn_t (*ippuControl) (void *, ippuControl_t *);
    ippuReturn_t (*ippuClose) (void *ippuInst);
    ippuReturn_t (*ippuDelete) (void **ippuInst);
    void (*ippuGetStats) (void *ippuInst, ippuStats_t *stats);
    void (*ippuSendIn)(void *ippuInst, globData_t *data, ITDM_DATA_EXCHANGE_T *p_tdm_data, tuint taskPostTimeStamp);
    tint (*ippuReceiveIn)(void *ippuInst, xferPktInfo_t *pktInfo, ITDM_DATA_EXCHANGE_T *p_tdm_data);
    bool (*ippuCompareUID)(void *ippuInst, word* pktBuf);
    tint (*ippuProcessRxItdmHeader)(void *ippuInst, xferPktInfo_t* pktInfo, tint *ipiuMsg);
} ippuCallTable_t;

extern ippuCallTable_t ippuCallTable;

#endif

