/************************************************************************
:*                                                                      *
:*     @ Copyright (C) 2009  Texas Instruments Incorporated.            *
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

/********************************************************************************
 * FILE PURPOSE: TSIP application interface driver control.
 ********************************************************************************
 * FILE NAME: tsipEmul.c
 *
 * DESCRIPTION: Drivers to implement the hal level interface. Some of the 
 *              functions are executed from an ISR context. These functions
 *              are only used during a timeslot state transition, so are
 *              not part of the isr file.
 *
 ********************************************************************************/
#include "gghwcfg.h"
#include "types.h"
#include "hal.h"
#include "halloc.h"
#include "halport.h"
#include "tsipEmul.h"
#include "cpuutils.h"
#include "cslisr.h"
#include "device.h"
#include <stdio.h>
#include <string.h>

#ifdef EMU_PROFILE
#include "emureg.h"
#endif

/*************************************************************************************************************
 * FUNCTION PURPOSE: Configure timer 6
 *************************************************************************************************************
 * DESCRIPTION: Timer 6 is configured to run as a 32 bit counter with max count.
 *************************************************************************************************************/
void tsip_emul_configureTimer6 (unsigned int period)
{

  /* Check if timer is set already, by another core. To do that
  check whether ENAMODE of TCR12 is set && TimerPeriod is set to its
  proper value in PRD12 */
 

  if( ( *((unsigned int *)0x2280018)== period ) && ( (*((unsigned int *)0x2280020) & 0x80) !=0 )  ) /* Period */
  {
    return;/* Timer values already initialized */
  }
 
  *((unsigned int *)0x2280024) = 0;  /* Disable the timer */

  *((unsigned int *)0x2280018) = period;  /* Period */

  *((unsigned int *)0x2280010) = 0; /* TIM */

#ifdef HAL_TIMER_BASED_TDM_EXT
  *((unsigned int *)0x2280020) = 0x180;  /* TCR12
                                            ENAMODE=10b,continously enabled timer,
                                            CLKSRC=1,Timer input pin.
                                             */
#else
  *((unsigned int *)0x2280020) = 0x80;  /* TCR12
                                            ENAMODE=10b,continously enabled timer,
                                            CLKSRC=0,Internal VBUS triggered.
                                      */
#endif

  *((unsigned int *)0x2280024) = 5;  /* TGCR - dual 32 bit unchained, TIM12 is enabled*/

} /* configureTimer6 */

 
halTsipEmulContext_t halTsipEmulContext;
/**********************************************************************************
 * FUNCTION PURPOSE: Locate the tsip structure associated with a port
 **********************************************************************************
 * DESCRIPTION: Finds the tsip structure with the requested port number. Returns
 *              NULL is that port is not found.
 **********************************************************************************/
void siuSubRateFcn(void *,tuint);
void tsipEmulInit (tuint core_mhz)
{
unsigned int period;

halTsipEmulContext.loc_count=0;
halTsipEmulContext.subFrameCallout=siuSubRateFcn;

/* check if the timer is not configured earlier, by any other core */
/* Configure the Timer to get interrupt every 1ms.
TODO! Support may be added later to get interrupt every 0.5 for
 staggering across cores */
#ifdef HAL_TIMER_BASED_TDM_EXT
  tsip_emul_configureTimer6(19440);/* TODO, remove the hardcoding */
                     /* The timer is fed using external timer,which is 19.44Mhz
                        for radisys.
                            So, 19.44, i.e 19.44 ticks per microsecond,
                            = 19.44 * 1000 ticks per millisecond. So,
                             the timer period should be 19.44*1000=19440
                             */
#else
  period = (unsigned int)((core_mhz*1000)/6);
  tsip_emul_configureTimer6(period); /* The timer is fed using SYSCLK8,which is CPU/6.
                            So, 500/6 Mhz, i.e 500/6 ticks per microsecond,
                            = (500/6)*1000 ticks per millisecond. So,
                             the timer period should be (500/6)*1000=
                             83333.333 rounded = 83333 */
#endif

/* Route the edma memory to memory interrupt to the edma interrupt handler */
hal_route_interrupt (CSL_DEVICE_ISR_TIMER_8, halTsipEmulIsr, &halTsipEmulContext);

csl_enable_interrupt (CSL_DEVICE_ISR_TIMER_8);

return;

} /* tsipEmulInit  */

void halTsipEmulIsr (void *context)
{
  halTsipEmulContext_t *tsipEmulContext = (halTsipEmulContext_t *)context;
#ifdef PROFILE_IDLE_MIPS
  extern cregister volatile unsigned int TSCL;
  tulong TSCL_Begin = TSCL;
#endif

#ifdef EMU_PROFILE
  EMU_PROFILE_BEGIN (EMU_TASK_ID_MCBSP(0),0)
#endif

#ifndef C_ISR
  siuHaltCheck();
#endif

 /* Make the sub-frame callout */
 if (tsipEmulContext->subFrameCallout != NULL)
 {
    (*tsipEmulContext->subFrameCallout) (context,0);/* Why should context be passed?,
                                                       it wont be used anyway */
 }
 halTsipEmulContext.loc_count++;

#ifdef PA_SS
    dsiuNwalPoll ();
#endif        

#ifdef EMU_PROFILE
 EMU_PROFILE_END()
#endif
#ifndef C_ISR /* DIAG */
#ifdef PROFILE_IDLE_MIPS
  siuIsrCycles(TSCL_Begin, TSCL);
#endif
#endif
 return;
} /* halTsipEIsr */

/* Nothing beyond this point */

