/*****************************************************************************
 * FILE PURPOSE: HAL Call table setup
 *****************************************************************************
 * FILE NAME: hal.c
 *
 * DESCRIPTION: Sets up the HAL call table for GEM devices.
 *
 * TABS: None
 *
 * Copyright (C) 2002, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include "hal.h"
#include "halloc.h"
#include "haledtpcc.h"
#include "devqu.h"
#include "device.h"
#include "icache.h"
#include "halport.h"
#include "gghwcfg.h"
#include "tcslipc.h"
#include <stdlib.h>

#ifdef GG_HAL_INCLUDE_UTOPIA
#include "ul2loc.h"
#endif

#ifdef GG_HAL_INCLUDE_GMAC
  #include "halgmac.h"
#endif

#ifdef GG_HAL_INCLUDE_TDM
 #if defined (CSL_DEVICE_TDM_IS_MCBSP)
  #include "tdm_dma.h"
 #elif defined (CSL_DEVICE_TDM_IS_TSIP)
  #include "tsip_app.h"
 #endif
#endif

#ifdef TSIP_LLD 
  #include "tsip_app.h"
#endif


#ifdef GG_HAL_INCLUDE_ITDM
 #if defined (CSL_DEVICE_TDM_IS_TSIP)
  #include "tsip_itdm.h"
 #endif
#endif

/* Prototypes */
halReturn_t halWdTimerControl (halWdTimerControl_t *ctl);

/************************************************************************************
 * FUNCTION PURPOSE: Generic hal control for 64x
 ************************************************************************************
 * DESCRIPTION: Provides a generic interface for interacting with HAL.
 ************************************************************************************/
halReturn_t halControl (halControl_t *ctl)
{

   switch (ctl->cmd)  {

                     /* HAL_HALT is used only if the DSP is bringing itself down
                      * in a controlled fasion. The transmit portion of the 
                      * utopia interrupt is enabled to allow any queued 
                      * messages to exit on the next interrupt. */
     case HAL_COMMAND_HALT:
       break;


     case HAL_COMMAND_PERIPH_RESET:
       break;

#ifdef DEVICE_CORE_BOOT
     case HAL_COMMAND_CORE_BOOT:
       if (cslDeviceCoreBoot (ctl->u.coreBoot.core, ctl->u.coreBoot.entryAddr))
         return (HAL_FAIL);
        break;
#endif

#ifdef DEVICE_CORE_RESET
     case HAL_COMMAND_CORE_RESET:
       if (cslDeviceCoreReset (ctl->u.coreReset.core))
         return (HAL_FAIL);
       break;
#endif

    case HAL_COMMAND_SLEEP:
        cslDeviceSleep (ctl->u.sleep.wakeMode);
        break;

    case HAL_COMMAND_INT_OTHER_CORE:
        return (cslDeviceIntOcore (&ctl->u.intCore));

    case HAL_COMMAND_INT_HOST:
        cslDeviceIntHost ();
        break;

#ifdef GG_INCLUDE_IPCMSG
    case HAL_COMMAND_IPC_WRITE:
        tcslIpcBitSet (ctl->u.ipcWrite.core, ctl->u.ipcWrite.setMask, ctl->u.ipcWrite.clearMask);
        break;

    case HAL_COMMAND_IPC_READ:
        ctl->u.ipcRead.msg = tcslIpcRead (ctl->u.ipcRead.core);
        break;
#endif

  }

  return (HAL_SUCCESS);

} /* halGenericCtrl */

/*********************************************************************************
 * FUNCTION PURPOSE: Perform polling for both mac and utopia
 *********************************************************************************
 * DESCRIPTION: Calls both polling functions.
 *********************************************************************************/
#if ( defined(GG_HAL_INCLUDE_UTOPIA) && defined(GG_HAL_INCLUDE_GMAC) )
bool halUtopiaMacPoll (tuint time, tuint activeChnls)
{
  halUtopiaPoll (time, activeChnls);
  halMacPoll    (time, activeChnls);

  return (TRUE);
}
#endif

#if defined(GG_HAL_INCLUDE_TDM)

halTdmApi_t halTdmApi = {

    halTdmGetSizes,   /* halTdmGetSizes */
    halTdmInit,       /* halTdmInit     */

  #if defined (CSL_DEVICE_TDM_IS_MCBSP)
    halTdmControl     /* halTdmControl  */
  #elif defined (CSL_DEVICE_TDM_IS_TSIP)
    halTsipControl    /* halTdmControl  */
  #else
    #error device.h must define CSL_DEVICE_TDM_IS_MCBSP or CSL_DEVICE_TDM_IS_TSIP
  #endif

};

#elif defined(TSIP_LLD)

halTdmApi_t halTdmApi = {

    halTsipGetSizes,        /* halTdmGetSizes */
    halTsipInit,            /* halTdmInit     */
    halTsipControl          /* halTdmControl  */

};

#elif defined(GG_HAL_INCLUDE_ITDM)

halTdmApi_t halTdmApi = {

    halTdmGetSizes,        /* halTdmGetSizes */
    halTdmInit,            /* halTdmInit     */
    halTdmTsipItdmControl  /* halTdmControl  */

};

#endif


#ifdef GG_HAL_INCLUDE_HPI
halHpiApi_t halHpiApi = {
  halHpiRead,               /* halHpiRead           */
  halHpiWrite,              /* halHpiWrite          */
  halHpiControl             /* halHpiControl        */
};
#endif

#ifdef GG_HAL_INCLUDE_IPC
halIpcApi_t halIpcApi = {
  halIpcControl             /* halHpiControl        */
};
#endif

#ifdef GG_HAL_INCLUDE_UTOPIA
halUtopiaApi_t halUtopiaApi = {  
  halUtopiaGetSizes,   /* halUtopiaGetSizes   */
  halUtopiaInit,       /* halUtopiaInit       */
  halUtopiaControl,    /* halUtopiaControl    */
  halUtopiaSendCell    /* halUtopiaSendCell   */
};
#endif

#ifdef GG_HAL_INCLUDE_GMAC
halGmacApi_t halGmacApi = {
  halGmacGetSizes,
  halGmacAlloc,
  halGmacInit,
  halGmacControl,           /* halGmacControl */
  halGmacSendPacket         /* halGmacSendPacket */
};
#endif

halApi_t hal = {
  halGetSizes,     /* halGetSizes         */
  halNew,          /* halNew              */
  halQuery,        /* halQuery            */
  halControl,      /* halControl          */   


#if ( defined(GG_HAL_INCLUDE_UTOPIA) && defined(GG_HAL_INCLUDE_GMAC) )
  halUtopiaMacPoll,

#elif defined (GG_HAL_INCLUDE_UTOPIA)
  halUtopiaPoll,   /* halPoll             */

#elif defined (GG_HAL_INCLUDE_GMAC)
  halMacPoll,      /* halPoll             */

#else
  NULL,            /* halPoll             */
#endif


  NULL,            /* halStatsRequest     */

#if ( defined(GG_HAL_INCLUDE_TDM) || defined(GG_HAL_INCLUDE_ITDM) || defined(TSIP_LLD)) 
  &halTdmApi,
#else
  NULL,
#endif

#ifdef GG_HAL_INCLUDE_HPI
  &halHpiApi,
#else
  NULL,            
#endif

#ifdef GG_HAL_INCLUDE_IPC
  &halIpcApi,
#else
  NULL,            
#endif

#ifdef GG_HAL_INCLUDE_PLL_CTL
  cslPllControl,
#else
  NULL,            /* halPllControl       */
#endif



#ifdef GG_HAL_INCLUDE_UTOPIA
  &halUtopiaApi,
#else
  NULL,  
#endif

#ifdef GG_HAL_INCLUDE_GMAC
  &halGmacApi,
#else
  NULL,
#endif

  NULL,   /* halGpioApi */


  halIntrLevel,             /* halIntrLevel         */
  NULL,                     /* halExternalInt       */
  NULL,                     /* halIntState          */


#ifdef GG_HAL_INCLUDE_TIMER
  halTimerControl,  /* halTimerControl     */
#else
  NULL,             /* halTimerControl     */
#endif


#ifdef GG_HAL_INCLUDE_WATCHDOG
  halWdTimerControl,
#else
  NULL,                     /* Watchdog timer       */
#endif

#ifdef GG_HAL_INCLUDE_DMA_CONTEXT
  halEdmaMemMoveChain,      /* halGlobMemMove       */
  (halReturn_t (*)(halLocAddr_t, halLocAddr_t, halSAE_t, halMemMoveComplete_t, void *))halEdmaMemMove,           /* halLocalMemMove      */
#else
  halMemMoveChain,          /* halGlobMemMove       */
  halMemMove,               /* halLocalMemMove      */
#endif
  halMapLocal2Global,       /* halMapLocalToGlobal  */
  halFormGlobAddr,          /* halFormGlobalAddr      */
#ifdef GG_HAL_INCLUDE_DMA_CONTEXT
  halEdmaMemSet,            /* halGlobMemSet        */
  halEdmaMMStatus,          /* halMMStatus          */
#else
  halMemSet,            /* halGlobMemSet        */
  halMMStatus,              /* halMMStatus          */
#endif


  NULL,             /* halAicApi */

  NULL,             /* halMessage */

  halMemConfig,     /* halMemConfig        */

  halCacheControl,  /* halCacheControl     */

  NULL,              /* halDmaApi */

};
  
/*****************************************************************************
 * Declaration of the intialization call table used by halinit. This table
 * can often be overlayed with the utopia tx buffer since it is used
 * only once.
 *****************************************************************************/
halInitCtable_t halInitCtable = {

#if defined(GG_HAL_INCLUDE_TDM)
  #if defined (CSL_DEVICE_TDM_IS_MCBSP)
    halTdmGetHeapSize,        /* halTdmGetHeapSize   */
    halTdmGetDmaBufInfo,      /* halTdmGetDmaBufInfo */
    halTdmGetDmaBufInfo,      /* halTdmGetDmaBufInfo */
    halTdmAllocAndInit,       /* halTdmAllocAndInit  */
    halTdmInitStart,          /* halTdmInitStart     */
    halTdmResourceInit,       /* halTdmResourceInit  */
    NULL,                     /* halTdmShGetHeapSize */
    NULL,                     /* halTdmShInit        */
  #elif defined (CSL_DEVICE_TDM_IS_TSIP)
      halTsipGetHeapSize,        /* halTdmGetHeapSize     */
      halTsipGetDmaBufInfo,      /* halTdmGetTxDmaBufInfo */
      halTsipGetDmaBufInfo,      /* halTdmGetRxDmaBufInfo */
      halTsipAllocAndInit,       /* halTdmAllocAndInit    */
      halTsipInitStart,          /* halTdmInitStart       */
      halTsipResourceInit,       /* halTdmResourceInit    */
    #ifdef GG_HAL_SHARED_TDM_RESOURCES
      halTsipGetSharedHeapSize,  /* halTdmShGetHeapSize   */
      halTsipSharedInit,         /* halTdmShInit          */
    #else
      NULL,
      NULL,
    #endif
  #endif
#elif defined (TSIP_LLD)
    NULL,                      /* halTdmGetHeapSize     */
    NULL,                      /* halTdmGetTxDmaBufInfo */
    NULL,                      /* halTdmGetRxDmaBufInfo */
    halTsipAllocAndInit,       /* halTdmAllocAndInit    */
    NULL,                      /* halTdmInitStart       */
    NULL,                      /* halTdmResourceInit    */
    NULL,                      /* halTdmShGetHeapSize   */
    NULL,                      /* halTdmShInit          */
#elif defined (GG_HAL_INCLUDE_ITDM)
  #if defined (CSL_DEVICE_TDM_IS_TSIP)
    halTsipItdmGetHeapSize,    /* halTdmGetHeapSize     */
    halTsipItdmGetDmaBufInfo,  /* halTdmGetTxDmaBufInfo */
    halTsipItdmGetDmaBufInfo,  /* halTdmGetRxDmaBufInfo */
    halTsipItdmAllocAndInit,   /* halTdmAllocAndInit    */
    NULL,                      /* halTdmInitStart       */
    NULL,                      /* halTdmResourceInit    */
    NULL,                      /* halTdmShGetHeapSize   */
    NULL,                      /* halTdmShInit          */
  #else
    NULL,                      /* halTdmGetHeapSize     */
    NULL,                      /* halTdmGetTxDmaBufInfo */
    NULL,                      /* halTdmGetRxDmaBufInfo */
    NULL,                      /* halTdmAllocAndInit    */
    NULL,                      /* halTdmInitStart       */
    NULL,                      /* halTdmResourceInit    */
    NULL,                      /* halTdmShGetHeapSize   */
    NULL,                      /* halTdmShInit          */
  #endif
#else
  NULL,                     /* halTdmGetHeapSize     */
  NULL,                     /* halTdmGetTxDmaBufInfo */
  NULL,                     /* halTdmGetRxDmaBufInfo */
  NULL,                     /* halTdmAllocAndInit    */
  NULL,                     /* halTdmInitStart       */
  NULL,                     /* halTdmResourceInit    */
  NULL,                     /* halTdmShGetHeapSize   */
  NULL,                     /* halTdmShInit          */
#endif

#ifdef GG_HAL_INCLUDE_UTOPIA
  halPktGetHeapSize,        /* halPktGetHeapSize   */
  halPktGetTxBufInfo,       /* halPktGetTxBufInfo  */
  halPktGetRxBufInfo,       /* halPktGetRxBufInfo  */
  halPktGetCaBufInfo,       /* halPktGetCaBufInfo  */
  halPktAllocAndInit,       /* halPktAllocAndInit  */
  halPktInitStart,          /* halPktInitStart     */
#else
  NULL,                     /* halPktGetHeapSize   */
  NULL,                     /* halPktGetTxBufInfo  */
  NULL,                     /* halPktGetRxBufInfo  */
  NULL,                     /* halPktGetCaBufInfo  */
  NULL,                     /* halPktAllocAndInit  */
  NULL,                     /* halPktInitStart     */
#endif

#ifdef GG_HAL_INCLUDE_GMAC
  halGmacGetHeapSize,       /* halGmacGetHeapSize  */
  halGmacGetTxBufInfo,      /* halGmacGetTxBufInfo */
  halGmacGetRxBufInfo,      /* halGmacGetRxBufInfo */
  halGmacBufAlloc,          /* halGmacBufAlloc     */
  halGmacInit,              /* halGmacInit         */
#else
  NULL,                     /* halGmacGetHeapSize  */
  NULL,                     /* halGmacGetTxBufInfo */
  NULL,                     /* halGmacGetRxBufInfo */
  NULL,                     /* halGmacBufAlloc     */
  NULL,                     /* halGmacInit         */
#endif 

#ifdef GG_HAL_INCLUDE_DMA_CONTEXT
  halEdIsrGetSizes,         /* halDmaGetSizes      */
  halEdIsrInit              /* HalDmaInit          */
#else
  NULL,                     /* halDmaGetSizes      */
  NULL                      /* halDmaInit          */
#endif

};
  

halContext_t *halContext;


