/************************************************************************/
/*                                                                      */
/* Copyright (C) 2001   Texas Instruments Incorporated.        
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
/*                      Revision:   1.0                                 */
/************************************************************************/
/*                                                                      */
/* File Name    : icache.c                                              */
/*                                                                      */
/* Description  : Shannon ICache API implementation.                   */
/*                                                                      */
/* Tools Ver    :                                                       */
/*                                                                      */
/* Date         :                                      */
/************************************************************************/

#include "ti/csl/csl_cacheAux.h"
#include "gghwcfg.h"
#include "hal.h"                               

/************************************************************************************
 * UMC Memory attribute registers
 ************************************************************************************/
#define DEVICE_UMC_MAR_BASE     0x1848000

#define CSL_UMC_MAR_WRITE(x, v) \
    *((volatile unsigned int *)(DEVICE_UMC_MAR_BASE + ((x)*4))) = (v)
    
    
#define CSL_UMC_MAR_READ(x) \
    *((volatile unsigned int *)(DEVICE_UMC_MAR_BASE + ((x)*4)))
    
/******************************************************************************
* FUNCTION PURPOSE: Configures the L1D/L1P/L2 Cache                          *
******************************************************************************
* DESCRIPTION    : This funtion configures a the cache by specifying         *
*                  information such as associativity and number of ramsets   *
*                  used.                                                     *
*                                                                            *
******************************************************************************/
void halCacheConfig(halCacheConfig_t *cacheConfig)
{
  CACHE_L1Size L1Size;
  CACHE_L2Size L2Size;

#ifdef HAL_MAR_REGS
  /*enable MAR registers if needed for Shannon*/
  /*can use CACHE_enableCaching()*/

  tint mregs[] = HAL_MAR_REGS;
  tint mvals[] = HAL_MAR_VALS;
  tint i;
#else
#ifdef HAL_MAR_DYNAMIC_CFG
  tint *mregs = cacheConfig->marRegs;
  tint *mvals = cacheConfig->marVals;
  tint i;
#endif
#endif

  /* The SIU passes cache size as 0-7 matching the HW */
  L1Size = (CACHE_L1Size)cacheConfig->cacheMode; 
  L2Size = (CACHE_L2Size)cacheConfig->cacheMode; 
  switch (cacheConfig->cacheType)
  {
    case HAL_CACHE_L2:
      /* Clean L2 */
      CACHE_wbInvAllL2(CACHE_WAIT);/*wait for the operation to complete. */

      /* configure L2 cache by writing to CCFG */
      CACHE_setL2Size(L2Size);
      while(CACHE_getL2Size() != L2Size); /* Force CCFG modification via read from CCFG. */
      break;
      
    case HAL_CACHE_L1D:

      /* write the desired cache mode to L1DMODE field in L1DCFG */
      CACHE_setL1DSize(L1Size);
      /* read back L1DCFG. thsi stalls the cpu until the mode change is complete */
      while(CACHE_getL1DSize() != L1Size); 
      break;  

    case HAL_CACHE_L1P:

      /* write the desired cache mode to L1DMODE field in L1DCFG */
      CACHE_setL1PSize(L1Size);
      /* read back L1DCFG. thsi stalls the cpu until the mode change is complete */
      while(CACHE_getL1PSize() != L1Size); 
      break;  


    case HAL_CACHE_L2_FDEF:

      /* Clean L2 */
      CACHE_wbInvAllL2(CACHE_WAIT);/*wait for the operation to complete. */
      L2Size = (CACHE_L2Size)HAL_L2_CACHE_MODE;
      CACHE_setL2Size(L2Size);
      while(CACHE_getL2Size() != L2Size); /* Force CCFG modification via read from CCFG. */
      break;


    case HAL_CACHE_L1D_FDEF:
      L1Size = (CACHE_L1Size)HAL_L1D_CACHE_MODE;
      CACHE_setL1DSize(L1Size);
      /* read back L1DCFG. thsi stalls the cpu until the mode change is complete */
      while(CACHE_getL1DSize() != L1Size); 
      break;


    case HAL_CACHE_L1P_FDEF:
      L1Size = (CACHE_L1Size)HAL_L1P_CACHE_MODE;
      CACHE_setL1PSize(L1Size);
      /* read back L1DCFG. thsi stalls the cpu until the mode change is complete */
      while(CACHE_getL1PSize() != L1Size); 
      break;

    case HAL_CACHE_MAR:
#if (defined(HAL_MAR_REGS) || defined(HAL_MAR_DYNAMIC_CFG))
      /*enable MAR registers if needed for Shannon*/
      /*can use CACHE_enableCaching()*/
      for (i = 0; mregs[i] >= 0; i++)  
        CSL_UMC_MAR_WRITE (mregs[i], mvals[i]);
#endif
      break;

    default:
     /* Unknown. No need for cache configuration.Return */   
      return; 
  } /* switch */
} /* halCacheConfig */

void hal_cache_flush (halCacheControl_t *cacheControl)
{
  switch (cacheControl->cacheType)
  {
    case HAL_CACHE_L2:
      CACHE_wbInvAllL2(CACHE_WAIT);
      break;
      
    case HAL_CACHE_L1D:
      CACHE_wbInvAllL1d (CACHE_WAIT);
      break;  

    case HAL_CACHE_L1P:
      CACHE_invAllL1p (CACHE_WAIT);
     break;
     
    case HAL_CACHE_ALL:
      CACHE_wbInvAllL2(CACHE_WAIT);
      CACHE_wbInvAllL1d (CACHE_WAIT);
      CACHE_invAllL1p (CACHE_WAIT);
      break;
    
    default:
     /* Unknown. No need for cache configuration.Return */   
      return; 
  } /* switch */

} /* hal_cache_flush */


/************************************************************************************** 
 * FUNCTION PURPOSE: Handle cache lines
 **************************************************************************************
 * DESCRIPTION: Cache invalidates, writebacks, and writeback-invalidates are performed
 **************************************************************************************/
void hal_cache_linectl (halCacheControl_t *cacheControl)
{
 /*stub for shannon implementation*/

} /* hal_cache_linectl */
    


void halCacheControl(halCacheControl_t *cacheControl)
{

  if (cacheControl->cacheCtrl == HAL_CACHE_CTRL_FLUSH)  {
    hal_cache_flush(cacheControl);
    return;
  }

  if ( (cacheControl->cacheCtrl == HAL_CACHE_CTRL_INVALIDATE)   ||
       (cacheControl->cacheCtrl == HAL_CACHE_CTRL_WRITEBACK)    ||
       (cacheControl->cacheCtrl == HAL_CACHE_CTRL_WB_INV)  )  {
    hal_cache_linectl (cacheControl);
    return;
  }
                     

} /* halCacheControl */

/* Nothing Beyond this point */  


