/**************************************************************************************
 * FILE PURPOSE: Provide support for hal query
 **************************************************************************************
 * FILE NAME: devqu.c
 *
 * DESCRIPTION: Provides responses for hal query
 *
 * Copyright (C) 2005, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include "types.h"
#include "hal.h"
#include "cpuutils.h"
#include "devqu.h"
#include "device.h"
#include "cslpower_api.h"
#include "halloc.h"
#include "gghwcfg.h"

#ifdef GG_HAL_INCLUDE_GMAC
  #include "cslmacapi.h"
#endif

#include <ti/csl/csl_tsip.h>
#include <ti/csl/csl_tsipAux.h>
                
/**************************************************************************************
 * FUNCTION PURPOSE: Boot pin information
 **************************************************************************************
 * DESCRIPTION: Reads and returns the device latched specific information
 **************************************************************************************/
halReturn_t halBootPinQuery (halQuery_t *query)
{
  tulong devstat;

  devstat = *((volatile unsigned int *)0x2a80000);

  query->isPresent = 1;
  query->isAvail   = 1;
  query->periphId  = 0;

  query->u.bootpin.bootmode = (tuint)HAL_READ_BITFIELD(devstat,3,0);
  query->u.bootpin.cfg      = (tuint)HAL_READ_BITFIELD(devstat,21,17);

  return (HAL_SUCCESS);

} /* halBootPinQuery */

/**************************************************************************************
 * FUNCTION PURPOSE: Provide utopia query
 **************************************************************************************
 * DESCRIPTION: Returns info about the utopia port
 **************************************************************************************/
halReturn_t halUtopiaQuery (halQuery_t *query)
{
  if (query->portNum == 0)  {
    query->isPresent = 1;
    query->isAvail   = 1;
    query->u.utopia.busWidth = 8;
    query->periphId  = 0x0000DEAD;

  }  else  {
    query->isPresent = 0;
    query->isAvail   = 0;

  }

  return (HAL_SUCCESS);

}  /* halUtopiaQuery */


/**************************************************************************************
 * FUNCTION PURPOSE: Provide mac query
 **************************************************************************************
 * DESCRIPTION: Returns the status of the specified mac port
 **************************************************************************************/
halReturn_t halMacQuery (halQuery_t *query)
{
  if (query->portNum <= 1)  {
    query->isPresent = 1;
    query->isAvail   = 1;
    query->u.mac.speed = HAL_MAC_SPEED_1G;

    #ifdef GG_HAL_INCLUDE_GMAC
    /* Check if the device is powered up */
    if (cslPowerStateGet(DEVICE_POWERID_GMAC(query->portNum)) != CSL_POWER_STATE_POWER_ENABLE)  {
      query->isEnabled = FALSE;
      query->periphId = 0;

    }  else  {
      query->isEnabled = cslCpmacIsEnabled ((tuint)(query->portNum));
      query->periphId = cslCpmacReadPeriphId (query->portNum);
    }
    #else
      query->isEnabled = FALSE;
      query->periphId = 0;
    #endif
    
  
  }  else  {
    query->isPresent = 0;
    query->isAvail   = 0;
    query->isEnabled = FALSE;

  }

  return (HAL_SUCCESS);

} /* halMacQuery */

/*************************************************************************************
 * FUNCTION PURPOSE: Provides a core query
 *************************************************************************************
 * DESCRIPTION: Returns information about the core number
 *************************************************************************************/
halReturn_t halCoreQuery (halQuery_t *query)
{
  query->u.core.nCores   = 8;
  query->u.core.thisCore = whoAmI();
  return (HAL_SUCCESS);

} /* halCoreQuery */

/*************************************************************************************
 * FUNCTION PURPOSE: Return the silicon revision ID
 *************************************************************************************
 * DESCRIPTION: The silicon revision ID is read from the efuse farm, and coverted
 *              into a 16 bit pg value.
 *************************************************************************************/
halReturn_t halPgQuery (halQuery_t *query)
{
  tint pgver;

  pgver = readPgVer ();

  switch (pgver)  {
    default: query->u.pg.pg = 0x10;
      break;
  }


  return (HAL_SUCCESS);

} /* halPgQuery */

/*************************************************************************************
 * FUNCTION PURPOSE: Return mcbsp info
 *************************************************************************************
 * DESCRIPTION: There are no mcbsps on this device. This is indicated on return
 *************************************************************************************/
halReturn_t halMcBspQuery (halQuery_t *query)
{
  query->isPresent = FALSE;
  query->isAvail   = FALSE;
  return (HAL_SUCCESS);
}

/**************************************************************************************
 * FUNCTION PURPOSE: Return TSIP info
 **************************************************************************************
 * DESCRIPTION: Returns information about the 2 TSIP peripherals
 **************************************************************************************/
halReturn_t halTsipQuery (halQuery_t *query)
{
#ifdef TSIP_LLD
  CSL_TsipHandle  hTsip;

  /* There are 2 tsips, they are always enabled */
  if (query->portNum < 2)  {
    /* Open the TSIP module */
    hTsip  = CSL_tsipOpen(query->portNum);
    
    query->isPresent = TRUE;
    query->isAvail   = TRUE;
    query->periphId  = CSL_tsipGetPid(hTsip);   
  } else {
    query->isPresent = FALSE;
    query->isAvail   = FALSE;
  }
#endif

  return (HAL_SUCCESS);

} /* halTsipQuery */

/*************************************************************************************
 * FUNCTION PURPOSE: Converts a timeslot, port, and link value into a combined
 *                   timeslot value
 *************************************************************************************/
halReturn_t halTsEncQuery (halQuery_t *query)
{
#ifdef TSIP_LLD 
  query->u.ts.port_link_ts = 0;
  query->u.ts.port_link_ts = HAL_SET_BITFIELD(query->u.ts.port_link_ts, query->u.ts.port, GG_TSIP_PORT_MSB, GG_TSIP_PORT_LSB);
  query->u.ts.port_link_ts = HAL_SET_BITFIELD(query->u.ts.port_link_ts, query->u.ts.link, GG_TSIP_LINK_MSB, GG_TSIP_LINK_LSB);
  query->u.ts.port_link_ts = HAL_SET_BITFIELD(query->u.ts.port_link_ts, query->u.ts.ts,   GG_TSIP_TS_MSB, GG_TSIP_TS_LSB);
#endif
  return (HAL_SUCCESS);

} /* halTsEncQuery */

/*************************************************************************************
 * FUNCTION PURPOSE: Decode a timeslot
 *************************************************************************************
 * DESCRIPTION: Converts a combined timeslot value into port, link and timeslot
 *************************************************************************************/
halReturn_t halTsDecQuery (halQuery_t *query)
{
#ifdef TSIP_LLD 
  query->u.ts.port = HAL_READ_BITFIELD(query->u.ts.port_link_ts, GG_TSIP_PORT_MSB, GG_TSIP_PORT_LSB);
  query->u.ts.link = HAL_READ_BITFIELD(query->u.ts.port_link_ts, GG_TSIP_LINK_MSB, GG_TSIP_LINK_LSB);
  query->u.ts.ts   = HAL_READ_BITFIELD(query->u.ts.port_link_ts, GG_TSIP_TS_MSB, GG_TSIP_TS_LSB);
#endif
  return (HAL_SUCCESS);

} /* halTsDecQuery */


/*************************************************************************************
 * FUNCTION PURPOSE: Return the status of memory - local, shared, nonlocal
 *************************************************************************************
 * DESCRIPTION: The global memory address is examined and the type is returned.
 *************************************************************************************/
halReturn_t halMemQuery (halQuery_t *query)
{
  tulong localAddr;
  tulong addr;
  tint   globAddr;

  addr = (tulong)(query->u.mem.address);
  localAddr = addr & 0x00ffffff;

  if ((addr & 0xf0000000) == 0x10000000)
    globAddr = (addr >> 24) & 0xf;
  else
    globAddr = DNUM;

  query->u.mem.core = globAddr;

  /* check for shared */
  if (addr > 0xe0000000)  {
    query->u.mem.type = HAL_MEM_SHARED;
    query->u.mem.core = -1;
  }

 
  else if ((localAddr >= 0x200000) && (localAddr < 0x2c0000))
    query->u.mem.type = HAL_MEM_SHARED;

  else if ((localAddr >= 0x800000) && (localAddr < 0x898000))  {
    if (globAddr == DNUM)
      query->u.mem.type = HAL_MEM_LOCAL;
    else
      query->u.mem.type = HAL_MEM_OTHER_CORE;
   }

   else if ((localAddr >= 0xe00000) && (localAddr < 0xe08000))  {
    if (globAddr == DNUM)
      query->u.mem.type = HAL_MEM_LOCAL;
    else
      query->u.mem.type = HAL_MEM_OTHER_CORE;
   }

   else if ((localAddr >= 0xf00000) && (localAddr < 0xf08000))  {
    if (globAddr == DNUM)
      query->u.mem.type = HAL_MEM_LOCAL;
    else
      query->u.mem.type = HAL_MEM_OTHER_CORE;
   }

   else  {
      query->u.mem.type = HAL_MEM_PERIPH;
      if (localAddr > 0x1c00000)  /* Below this are gem local regs */
        query->u.mem.core = -1;
   }

  return (HAL_SUCCESS);

} /* halMemQuery */


/**************************************************************************************
 * FUNCTION PURPOSE: Return response to query
 **************************************************************************************
 * DESCRIPTION: Replies to requests about hardware information for the system
 **************************************************************************************/
halReturn_t halQuery (halQuery_t *query)
{

  switch (query->periph)  {

    case HAL_QUERY_UTOPIA:
         return (halUtopiaQuery (query));

    case HAL_QUERY_MAC:
         return (halMacQuery (query));

    case HAL_QUERY_NCORES:
         return (halCoreQuery (query));

    case HAL_QUERY_PG:
         return (halPgQuery (query));

    case HAL_QUERY_MCBSP:
         return (halMcBspQuery (query));

    case HAL_QUERY_TSIP:
         return (halTsipQuery (query));

    case HAL_QUERY_TS_ENCODE:
         return (halTsEncQuery (query));

    case HAL_QUERY_TS_DECODE:
         return (halTsDecQuery (query));

    case HAL_QUERY_BOOTPINS:
         return (halBootPinQuery (query));

    case HAL_QUERY_MEMORY:
         return (halMemQuery (query));

  }

  return (HAL_FAIL);

} /* halQuery */




