#ifndef _CSLISR_DEV_H
#define _CSLISR_DEV_H
/*****************************************************************************
 * FILE PURPOSE: Provide device specific inlines
 *****************************************************************************
 * FILE NAME: cslisr_dev.h
 *
 * DESCRIPTION: Provides device specific inlines and macros for csl and
 *			    hal functions.
 *
 * Copyright (C) 2005, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/



extern cregister  volatile unsigned int CSR;
extern cregister  volatile unsigned int IER;
extern cregister  volatile unsigned int TSR ; /* 16 May 2006: GEM Exception handling*/

#define ONE32 ((UINT32)1)
#define NMIE  (ONE32<<1)
#define GIE   (ONE32<<0)
#define XEN   (ONE32<<3)
#define GEE   (ONE32<<2)

/******************************************************************************
* FUNCTION PURPOSE: Globally Disable All Interrupts
******************************************************************************
* DESCRIPTION: The function resets NMIE bit and GIE bits
*
* void cslGlobDisableInt ( void )
*****************************************************************************/
static inline void cslGlobDisableInt ( void )
{

  IER |= ( unsigned int )( NMIE ) ;  /*Resetting of GIE works only if NMIE=1*/
  CSR &= ( unsigned int )(~( GIE )) ;

} /* cslGlobDisableAllInt */



/******************************************************************************
 * FUNCTION PURPOSE: Globally Enable All Interrupts
 ******************************************************************************
 * DESCRIPTION: The function sets NMIE and GIE bits
 *****************************************************************************/
static inline void cslGlobEnableInt ( void )
{

  IER |= ( int ) ( NMIE ) ;
  CSR |= ( int ) ( ( GIE ) ) ;
/******************************************************************************
EXCEP is the maskable external exception input to the CPU. It is enabled by
the XEN bit in TSR. For this exception to be recognized, the XEN bit must be
set to 1, the GEE bit must be set to 1, and the NMIE bit must be set to 1.
*****************************************************************************/
  /* 16 May 2006 : GEM Exceptions*/
  TSR |= ( int )( XEN ) ;  
  TSR |= ( int )( GEE ) ;  

} /* cslGlobEnableInt */


/* Define the context used to disable/re-enable interrupts */
typedef struct cslIntContext_s {
  UINT32 ierStore;
} cslIntContext_t;

/*******************************************************************************
 * FUNCTION PURPOSE: Restore the interrupt context
 *******************************************************************************
 * DESCRIPTION: Restores the previously saved individual interrupt enable
 *              values.
 *******************************************************************************/
static inline void cslIntRestore (cslIntContext_t *cxt)
{
  IER = cxt->ierStore;
  
} /* cslIntRestore */  


/**********************************************************************************
 * FUNCTION PURPOSE: Perform a preliminary interrupt disable, return global enable
 **********************************************************************************
 * DESCRIPTION: This function Disables all currently enabled interrupts using the
 *              individual interrupt enable bits. Returned in the context
 *              is the required restore value.
 *
 *              This function returns 1 if interrupts are globally enabled,
 *              0 if disabled.
 **********************************************************************************/
static inline tint cslIntChange (cslIntContext_t *cxt)
{
  /* Disable all interrupts by clearing all individual interrupt enable bits.
   * This method is used due to the difficulty in checking the value of the global
   * enable bit when interrupts are enabled */
  cxt->ierStore = IER;
  IER = 0;

  /* Return 1 if interrupts are enabled, 0 if disabled. This is just the
   * value of the GIE bit in CSR */
  return ((tint)CSR & (GIE));

} /* cslIntChange */

  

#endif /* _CSLISR_DEV_H */
