/*****************************************************************************/
/*                                                                           */
/*     Copyright (C) 2004  Texas Instruments Incorporated.                   
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
/*                                    Revision:   1.0                        */
/*****************************************************************************/
/*  FILE PURPOSE: Memory Move function for HAL.                              */
/*  File Name   : memmvhal.c                                                 */
/*                                                                           */
/*  Description: Memory Move functions are implemented here                  */
/*                                                                           */
/*  TABS : None                                                              */
/*                                                                           */
/*                                                                           */
/******************************************************************************/
#include "hal.h"
#include "device.h"
#include "cpuutils.h"
#include <stdlib.h>
#include <string.h>

extern void halCacheControl (halCacheControl_t *cacheControl);

/********************************************************************************
 * FUNCTION PRUPOSE: Form a global address
 ********************************************************************************
 * DESCRIPTION: A global address is formed based on a local address and 
 *              a global identifier
 ********************************************************************************/
halGlobAddr_t halFormGlobAddr (halLocAddr_t addr, tint globalId)
{
  halGlobAddr_t glb_addr;

  if ((addr & (halLocAddr_t)0xFF000000) != 0)
    return (addr); /* The address is already a global address */

  glb_addr = CSL_LOCAL_TO_GLOBAL(globalId, addr);

  return (glb_addr);

} /* halFormGlobAddr */


/******************************************************************************
 * FUNCTION PURPOSE: map a address from local memory space to global
 ******************************************************************************
 * DESCRIPTION: A local address is mapped to a global address by the following
 *              map:
 *
 *          global_addr = local_addr + 0x10000000 + (core_num << 24)
 *              
 * halGlobAddr_t halMapLocal2Global (
 *     halLocAddr_t addr -- address
 *     )
 *  Returns    : Address mapped to global memory space.
 *
 *  Note: If the address is already a global address nothing is done!
 *****************************************************************************/
extern cregister volatile unsigned int DNUM;
halGlobAddr_t halMapLocal2Global (halLocAddr_t addr)
{

  return (halFormGlobAddr (addr, (tint)DNUM));
  
}/* End of halMapLocal2Global() */

#ifndef GG_HAL_INCLUDE_DMA_CONTEXT
/***********************************************************************************
 * FUNCTION PURPOSE: Perform one or more memory moves
 ***********************************************************************************
 * DESCRIPTION: One or more memory moves are done using memcpy
 ***********************************************************************************/
tint halMemMoveChain (tint nXfers, halMemMove_t *xfers, halMemMoveComplete_t complete,
                          void *data, bool stallToCompletion)
{
  tint i;
  halCacheControl_t ctl;

   /* Call back and stall are not compatible */
  if ((complete != NULL) && (stallToCompletion == TRUE))
    return (-2);

  for (i = 0; i < nXfers; i++)  {
      memcpy ((void *)xfers[i].dest, (void *)xfers[i].src, (unsigned int)(xfers[i].size));
  }

  ctl.cacheCtrl         = HAL_CACHE_CTRL_FLUSH;
  ctl.cacheType        = HAL_CACHE_ALL;
  halCacheControl(&ctl);

  if(complete)
  {
      (complete)(data);
  }

  return(HAL_SUCCESS);
}


/*************************************************************************************
 * FUNCTION PURPOSE: Perform a memory transfer
 *************************************************************************************
 * DESCRIPTION: The first free channel is used to perform a memory transfer. 
 *              An error is returned if no channels are available. This 
 *              function is typically linked to the hal call table functions
 *              halGlobMemMove and halLocalMemMove.
 *************************************************************************************/

halReturn_t halMemMove (halGlobAddr_t dest, halGlobAddr_t src, halSAE_t size,
                            halMemMoveComplete_t complete, void *data)
{
    halCacheControl_t ctl;

    memcpy ((void *)dest, (void *)src, (unsigned int)(size));

    ctl.cacheCtrl       = HAL_CACHE_CTRL_FLUSH;
    ctl.cacheType      = HAL_CACHE_ALL;
    halCacheControl(&ctl);

    if(complete)
    {
      (complete)(data);
    }
    
    return(HAL_SUCCESS);
}

/*************************************************************************************************************
 * FUNCTION PURPOSE: Perform memset
 *************************************************************************************************************
 * DESCRIPTION: A 1 or 4 byte value is copied to memory
 *************************************************************************************************************/
halReturn_t halMemSet (halGlobAddr_t addr, tulong sizeBytes, tulong v, tint tsizeBytes)
{
    UINT32         count=0;
    halCacheControl_t ctl;


    if(tsizeBytes == 1)
    {
        memset((void *)addr,v,sizeBytes);
    }
    else
    {
        while(count<sizeBytes)
        {
            *((unsigned char *)addr) = v;
            addr=addr+4;
            count =count+4;
        }
    }

    ctl.cacheCtrl       = HAL_CACHE_CTRL_FLUSH;
    ctl.cacheType      = HAL_CACHE_ALL;
    halCacheControl(&ctl);

    return(HAL_SUCCESS);
}

/************************************************************************************
 * FUNCTION PURPOSE: Return status of a memory move
 ************************************************************************************
 * DESCRIPTION: Returns 1 indicating memory move is complete
 ************************************************************************************/
tint halMMStatus (tint handle)
{
    return(1);
}
#endif
/* Nothing past this point */






