/*****************************************************************************
 * FILE PURPOSE: Common utility for XMC, SES, SMS MPAX  configuration
 *****************************************************************************
 * FILE NAME: mpax_util.c
 *
 * DESCRIPTION: Common utility for XMC, SES, SMS MPAX  configuration
 *               
 * TABS: None
 *
 * (C) Copyright 2009 Texas Instruments.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include "mpax_util.h"
#include "hal.h"

/****************************************************************************************
*  Function Name    : halMpaxXmcSetup
*  Description      : This configures the MPAXL and MPAXH registers in the GEM XMC MPAX unit
* 					  with the desired base address, replacement address and segment size
*
*  Parameters   : halMpaxCfg_t cfg which has the following fields:
*
*                    tulong base_addr 
*                       Represents the virtual address (alias)
*                       Base Address Upper bits of address range to match in CGEM's native 32-bit address space
*                    
*                    tulong replace_addr_lower
*                    tulong replace_addr_upper
*                       Represents the 36bit system address (physical address), 
*                       Replacement Address Bits that replace and extend the upper address bits matched by base_addr
*
*                       replace_addr_upper must have one of the following values:
*                       0000 -> MSMC routes these to long distance port which further routes to SoC endpoints through SCR.
*                       0001 -> DDR3 config space
*                       1000 and 1001 -> DDR3 memory space.
*
*                    SEG_SIZE seg_size
*                       Segment size
*                    
*                    tlong xmc_index
*                       Index in the MPAX table.
*                       xmc_index < 16 
*                    
*  Return          : None
****************************************************************************************/
void halMpaxXmcSetup(const halMpaxCfg_t *cfg, tint nEnts)
{
  CSL_XMC_XMPAXH xmc_mpaxh;
  CSL_XMC_XMPAXL xmc_mpaxl;
  tulong         base_addr;
  tint           segsize_power2;
  tulong         replace_addr;
  tulong         perms;
  int            i;
  extern cregister volatile unsigned int DNUM;
  halCacheControl_t cacheControl;

  /* Flush all caches, because caches operate on logical addresses, which
   * are about to change!
   */
  cacheControl.cacheCtrl = HAL_CACHE_CTRL_FLUSH;
  cacheControl.cacheType = HAL_CACHE_ALL;
  halCacheControl(&cacheControl);

  for (i = 0; i < nEnts; i++) {
    /*
     *        The replace_addr is the upper 32 bit of the 36bit address.
     *        For example, in order to input 8:0000_0000 we have:
     *        replace_addr = (0x8 << 28) + (0000_0000 >> 4)
     *        For NySh we can have a system address DDR memory space as either 8:xxxx_xxxx or 9:xxxx_xxxx
     *        So replace_addr = 0x8000_0000+ (xxxx_xxxx >> 4)
     *                          or
     *                          0x9000_0000+ (xxxx_xxxx >> 4)
     *
     *        We could also have 0:xxxx_xxxx or 1:xxxx_xxxx 
     */
    replace_addr   = (cfg[i].replace_addr_upper << 28) + 
                     (cfg[i].replace_addr_lower >> 4);
    base_addr      = cfg[i].base_addr;
    perms          = cfg[i].permissions;
    segsize_power2 = cfg[i].segsize_power2;

    if (cfg[i].privAlloc) {
      tulong corenum_adjust = (1 << segsize_power2) * DNUM;
      replace_addr += (corenum_adjust >> 4);
    }

    /*Configure GEM XMC MPAXL register*/
    /*Set permissions*/
    xmc_mpaxl.sr      = (perms & HAL_MPAX_PERM_SR) ? 1 : 0;
    xmc_mpaxl.sw      = (perms & HAL_MPAX_PERM_SW) ? 1 : 0;
    xmc_mpaxl.sx      = (perms & HAL_MPAX_PERM_SX) ? 1 : 0;
    xmc_mpaxl.ur      = (perms & HAL_MPAX_PERM_UR) ? 1 : 0;
    xmc_mpaxl.uw      = (perms & HAL_MPAX_PERM_UW) ? 1 : 0;
    xmc_mpaxl.ux      = (perms & HAL_MPAX_PERM_UX) ? 1 : 0;
    xmc_mpaxl.rAddr   = (replace_addr >> 8);

    CSL_XMC_setXMPAXL (cfg[i].index, &xmc_mpaxl); 
    
    /*Configure GEM XMC MPAXH register*/
    xmc_mpaxh.bAddr   = (base_addr >> 12);
    if (segsize_power2) {
      xmc_mpaxh.segSize = segsize_power2 - 1;
    } else {
      xmc_mpaxh.segSize = 0;
    }
    CSL_XMC_setXMPAXH (cfg[i].index, &xmc_mpaxh);
  }
} /* halMpaxXmcSetup */

/****************************************************************************************
*  Function Name    : halMpaxSesSetup
*  Description      : This configures the MPAXL and MPAXH registers in the MSMC's SES MPAX unit
* 					  with the desired base address, replacement address, segment size
* 					  and desired master privID for address extension to external DDR3.
*
*
*  Parameters   :     halMpaxCfg_t cfg which has the following fields:
*
*                     tulong base_addr, 
*                     tulong replace_addr_lower, 
*                     tulong replace_addr_upper, 
*                     SEG_SIZE seg_size,
*                     tulong privID,
*                     tlong xmc_index
*                     
*  Return          : None
****************************************************************************************/
void halMpaxSesSetup(const halMpaxCfg_t *cfg, tint nEnts)
{
  CSL_MSMC_SESMPAXH ses_mpaxh;
  CSL_MSMC_SESMPAXL ses_mpaxl;
  tulong            base_addr;
  tint              segsize_power2;
  tulong            replace_addr;
  tulong            perms;
  tulong            startPrivID, endPrivID, privID;
  tulong            stepSize;
  int               i;

  for (i = 0; i < nEnts; i++) {
    replace_addr   = (cfg[i].replace_addr_upper << 28) + 
                     (cfg[i].replace_addr_lower >> 4);
    base_addr      = cfg[i].base_addr;
    perms          = cfg[i].permissions;
    segsize_power2 = cfg[i].segsize_power2;
    stepSize       = 1 << (segsize_power2 - 4);
    startPrivID    = cfg[i].startPrivID;
    endPrivID      = cfg[i].endPrivID;
    if (! cfg[i].privAlloc) {
      stepSize     = 0;
    }


    /*Configure GEM XMC MPAXL register*/
    /*Set permissions*/
    ses_mpaxl.sr      = (perms & HAL_MPAX_PERM_SR) ? 1 : 0;
    ses_mpaxl.sw      = (perms & HAL_MPAX_PERM_SW) ? 1 : 0;
    ses_mpaxl.sx      = (perms & HAL_MPAX_PERM_SX) ? 1 : 0;
    ses_mpaxl.ur      = (perms & HAL_MPAX_PERM_UR) ? 1 : 0;
    ses_mpaxl.uw      = (perms & HAL_MPAX_PERM_UW) ? 1 : 0;
    ses_mpaxl.ux      = (perms & HAL_MPAX_PERM_UX) ? 1 : 0;
    if (segsize_power2) {
      ses_mpaxh.segSz = segsize_power2 - 1;
    } else {
      ses_mpaxh.segSz = 0;
    }

    for (privID = startPrivID; privID <= endPrivID; privID++, replace_addr += stepSize) {
      ses_mpaxl.raddr = (replace_addr >> 8);
      ses_mpaxh.baddr = (base_addr >> 12);

      CSL_MSMC_setSESMPAXL (privID, cfg[i].index, &ses_mpaxl); 
      CSL_MSMC_setSESMPAXH (privID, cfg[i].index, &ses_mpaxh);
    }
  }
} /* halMpaxSesSetup */

/****************************************************************************************
*  Function Name    : halMpaxSmsSetup
*  Description      : This configures the MPAXL and MPAXH registers in the MSMC's SMS MPAX unit
* 					  with the desired base address, replacement address, segment size
* 					  and desired master privID for address extension to MSMC RAM
*
*  Parameters      :  halMpaxCfg_t cfg which has the following fields:
*                     tulong base_addr, 
*                     tulong replace_addr_lower, 
*                     tulong replace_addr_upper, 
*                     SEG_SIZE seg_size,
*                     tulong privID
*                     tlong xmc_index
*
*  Return          : None
****************************************************************************************/
void halMpaxSmsSetup(const halMpaxCfg_t *cfg, tint nEnts)
{
  CSL_MSMC_SMSMPAXH sms_mpaxh;
  CSL_MSMC_SMSMPAXL sms_mpaxl;
  tulong            base_addr;
  tint              segsize_power2;
  tulong            replace_addr;
  tulong            perms;
  tulong            startPrivID, endPrivID, privID;
  tulong            stepSize;
  int               i;

  for (i = 0; i < nEnts; i++) {
    replace_addr   = (cfg[i].replace_addr_upper << 28) + 
                     (cfg[i].replace_addr_lower >> 4);
    base_addr      = cfg[i].base_addr;
    perms          = cfg[i].permissions;
    segsize_power2 = cfg[i].segsize_power2;
    stepSize       = 1 << (segsize_power2 - 4);
    startPrivID    = cfg[i].startPrivID;
    endPrivID      = cfg[i].endPrivID;
    if (! cfg[i].privAlloc) {
      stepSize     = 0;
    }

    /*Configure GEM XMC MPAXL register*/
    /*Set permissions*/
    sms_mpaxl.sr      = (perms & HAL_MPAX_PERM_SR) ? 1 : 0;
    sms_mpaxl.sw      = (perms & HAL_MPAX_PERM_SW) ? 1 : 0;
    sms_mpaxl.sx      = (perms & HAL_MPAX_PERM_SX) ? 1 : 0;
    sms_mpaxl.ur      = (perms & HAL_MPAX_PERM_UR) ? 1 : 0;
    sms_mpaxl.uw      = (perms & HAL_MPAX_PERM_UW) ? 1 : 0;
    sms_mpaxl.ux      = (perms & HAL_MPAX_PERM_UX) ? 1 : 0;
    if (segsize_power2) {
      sms_mpaxh.segSz = segsize_power2 - 1;
    } else {
      sms_mpaxh.segSz = 0;
    }

    for (privID = startPrivID; privID <= endPrivID; privID++, replace_addr += stepSize) {
      sms_mpaxl.raddr = (replace_addr >> 8);
      sms_mpaxh.baddr = (base_addr >> 12);

      CSL_MSMC_setSMSMPAXL (privID, cfg[i].index, &sms_mpaxl); 
      CSL_MSMC_setSMSMPAXH (privID, cfg[i].index, &sms_mpaxh);
    }
  }
} /* halMpaxSmsSetup */

/* Nothing past this point */

