/*****************************************************************************
 * FILE PURPOSE: Common utility for XMC, SES, SMS MPAX  configuration
 *****************************************************************************
 * FILE NAME: mpax_util.h
 *
 * DESCRIPTION: Common utility for XMC, SES, SMS MPAX  configuration
 *               
 * TABS: None
 *
 * (C) Copyright 2009 Texas Instruments.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#ifndef _MPAX_UTIL_H_
#define _MPAX_UTIL_H_

#include "ti/csl/csl_msmcAux.h"
#include "ti/csl/csl_xmcAux.h"
#include "ti/csl/csl_chip.h"
#include "types.h"

#include <stdlib.h>

typedef struct { 
    /* Index in the MPAX table.
     *             
     * For XMC index:
     *              index < 16 
     *              index should be > 2  because by default the
     * XMC configures MPAX segments 0 and 1 so that CGEM can access system memory.
     * The power up configuration is that segment 1 remaps 8000_0000 - FFFF_FFFF in
     * CGEM's address space to 8:0000_0000 - 8:7FFF_FFFF in the system address map. This
     * corresponds to the first 2GB of address space dedicated to EMIF by the MSMC
     * controller.
     *
     * For SMS and SES index:
     *              index < 8
     *              index > 1
     */
    tlong index;

    /* Private (per core) allocation
     *
     * For XMC (only 1 entry is created)
     * privAlloc = 1: create unique memory region per core such that
     *                the physical address is 
     *                replace_addr + DNUM * (2 << segsize_power2)
     * privAlloc = 0; create commen memory region for all cores
     *
     * For SMS and SES:
     * privAlloc = 1: Create unique memory for each privID
     *                physical address is
     *                for (N = 0; N < (endPrivID - startPrivID); N++) {
     *                  if (replace_addr) 
     *                    phys_addr[N] = replace_addr + N * (2 << segsize_power2)
     *                  else
     *                    phys_addr[N] = NULL
     *                }
     * privAlloc = 0: Same as privAlloc = 1.
     */
    tulong privAlloc;

    /* For SES and SMC: which privID will be set (there is one entry per indx per privID) */
    /* For XMC: ignored */
    tulong startPrivID; 
    tulong endPrivID; 

    /* base_addr: Represents the 32bit virtual address (alias)
     */
    tulong base_addr; 
    
    /* replace_addr: Represents the 36bit system address (physical address), 
     * Replacement Address Bits that replace and extend the upper address bits matched by base_addr
     */
    tulong replace_addr_upper; 
    tulong replace_addr_lower; 
    
    /* Segment size as power of 2 (eg, length = 2^segsize_power2) */
    tlong segsize_power2; 
#define HAL_MPAX_PERM_SR 0x40
#define HAL_MPAX_PERM_SW 0x20 
#define HAL_MPAX_PERM_SX 0x10
#define HAL_MPAX_PERM_UR 0x04
#define HAL_MPAX_PERM_UW 0x02
#define HAL_MPAX_PERM_UX 0x01
    /* Permissions in "unix like" format sr sw sx ur uw ux */
    tulong permissions;
} halMpaxCfg_t;

void halMpaxXmcSetup(const halMpaxCfg_t *cfg, tint nEnts);
void halMpaxSesSetup(const halMpaxCfg_t *cfg, tint nEnts);
void halMpaxSmsSetup(const halMpaxCfg_t *cfg, tint nEnts);

#endif

/* Nothing past this point */

