/************************************************************************************
 * FILE PURPOSE: Provide control for the power sleep controller (PSC)
 ************************************************************************************
 * FILE NAME: psc.c
 *
 * DESCRIPTION: Enables or disables peripherals through the power sleep controller.
 *				Implements the api defined in cslpower_api.h
 *
 *  Copyright (C) 2005, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include "types.h"
#include "halloc.h"
#include "ti/csl/cslr_device.h"
#include "ti/csl/csl_pscAux.h"

/***************************************************************************************
 * FUNCTION PURPOSE: Configure power domain
 ***************************************************************************************
 * DESCRIPTION: function used to power up a power domain and the module clock.
 *              Power domain is OFF by default. It
 *              needs to be turned on before doing any  device
 *              register access.
 *
 * Input: 
 *        tuint psc_pd    : PSC Power Domain Assignment (PSC_PD)
 *        tuint psc_lpsc  : PSC LPSC Module Assignment (PSC_LPSC)
 *
 *        e.g. halDomainPowerUp(CSL_PSC_PD_PASS, CSL_PSC_LPSC_PKTPROC)
 ***************************************************************************************/
tuint halDomainPowerUp(tuint psc_pd, tuint psc_lpsc)
{

  /* Set Power domain to ON */        
  CSL_PSC_enablePowerDomain (psc_pd);

  /* Enable the clocks */
  CSL_PSC_setModuleNextState (psc_lpsc, PSC_MODSTATE_ENABLE);

  /* Start the state transition */
  CSL_PSC_startStateTransition (psc_pd);

  /* Wait until the state transition process is completed. */
  while (!CSL_PSC_isStateTransitionDone (psc_pd));

#if 0
  /* Return PSC status */
  if ((CSL_PSC_getPowerDomainState(psc_pd) == PSC_PDSTATE_ON) &&
      (CSL_PSC_getModuleState (psc_lpsc) == PSC_MODSTATE_ENABLE))
  {
      /* Power ON. Ready for use */            
      return HAL_SUCCESS;
  }
  else
  {
      /* Power on failed. Return error */            
      return HAL_FAIL;            
  }
#else
  return HAL_SUCCESS;

#endif
}


/***************************************************************************************
 * FUNCTION PURPOSE: Power up PASS domains
 ***************************************************************************************
 * DESCRIPTION: function used to power up all necessary PASS domains
 ***************************************************************************************/
tuint halPowerUpPass(void)
{
  tuint retVal=HAL_SUCCESS;
  
  if(halDomainPowerUp(CSL_PSC_PD_PASS, CSL_PSC_LPSC_PKTPROC) == HAL_FAIL)
  {
    retVal=HAL_FAIL;
  }
  if(halDomainPowerUp(CSL_PSC_PD_PASS, CSL_PSC_LPSC_CPGMAC) == HAL_FAIL)
  {
    retVal=HAL_FAIL;
  }
#if 0 // some chips don't have crypto and this fails...
  if(halDomainPowerUp(CSL_PSC_PD_PASS, CSL_PSC_LPSC_Crypto) == HAL_FAIL)
  {
    retVal=HAL_FAIL;
  }
#endif
  
  return retVal;
}


/************************************************************************************
 * FUNCTION PURPOSE: Provide a tiny delay
 ************************************************************************************
 * DESCRIPTION: A simple time waster 
 ************************************************************************************/
UINT32 cslPscDelay (void)
{
  volatile UINT32 i;

  for (i = 0; i < 100; i++);

  return (1);

} /* cslPscDelay */


/************************************************************************************
 * FUNCTION PURPOSE: Wait for the end of a transitional power state
 ************************************************************************************
 * DESCRIPTION: Polls ptstat for the selected domain and waits for transitions
 *              to be complete.
 ************************************************************************************/
tint cslPscWait (UINT32 domainNum)
{
  return cslPscDelay();
} /* cslPscWait */



/************************************************************************************
 * FUNCTION PURPOSE: Set the power state of a peripheral (domain)
 ************************************************************************************
 * DESCRIPTION: Writes to the power sleep control registers to enable or disable
 *              a peripheral.
 ************************************************************************************/
void cslPowerStateSet (tuint devInfo, tint newState)
{


} /* cslPowerStateSet */

/*********************************************************************************
 * FUNCTION PURPOSE: Read the power state of a module
 *********************************************************************************
 * DESCRIPTION: Reads and returns the power state. Waits for transitional 
 *              states to be complete.
 *********************************************************************************/
tint cslPowerStateGet (tuint devInfo)
{
  return 0; /* stub */
} /* cslPowerStateGet */
  

   
