#ifndef _HAL_H
#define _HAL_H
/*****************************************************************************
 * FILE PURPOSE: Define the application programming interface to HAL
 *****************************************************************************
 * DESCRIPTION: The Hardware Abstraction Layer (HAL) defines an API which 
 *              is conistent accross hardware platforms. This file defines
 *              the C-level HAL API.
 *
 *              HAL is designed to exist as a seperate application which can
 *              be seperately linked and loaded into DSP memory. 
 *
 * TABS: NONE
 *
 * Copyright (C) 2002, Texas Instruments Incorporated. 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include "types.h"
#ifdef ITDM
#include "dglob.h"
#endif

#include "ecoif/halapi.h"
#include "ecoif/aiccfg.h" /* contains AIC related message structure */
#include "ecoif/gains.h" /* contains gain control message structure */

#include "cslif/cslmac.h"



#ifdef TSIP_LLD 
#include <ti/drv/tsip/tsip.h> 
#endif


/****************************************************************************/
/* Memory buffer information returned to application for inital start       */
/****************************************************************************/
#ifdef TSIP_LLD
 #define HAL_HEAP_NBUFS              3
 #define HAL_HEAP_BUFN               0
 #define HAL_LLD_TDM_TX_BUFN         1
 #define HAL_LLD_TDM_RX_BUFN         2
#else
#define HAL_HEAP_NBUFS              1
#define HAL_HEAP_BUFN               0
#endif

/*****************************************************************************
 * Memory buffer information for utopia
 *****************************************************************************/
#define HAL_UT_NBUFS                3
#define HAL_UT_HEAP_BUFN            0
#define HAL_UT_TX_BUFN              1
#define HAL_UT_RX_BUFN              2
#define HAL_UT_CA_BUFN              3

/******************************************************************************
 * Memory buffer requirements for tdm 
 ******************************************************************************/
#ifdef TSIP_LLD
 #define HAL_TDM_NBUFS               1
 #define HAL_TDM_HEAP_BUFN           0
#else
#define HAL_TDM_NBUFS               3
#define HAL_TDM_HEAP_BUFN           0
#define HAL_TDM_TX_BUFN             1
#define HAL_TDM_RX_BUFN             2
#endif

/******************************************************************************
 * Memory buffer information for mac
 ******************************************************************************/
#define HAL_GMAC_NBUFS              4
#define HAL_GMAC_HEAP_BUFN          0
#define HAL_GMAC_TX_BUFN            1
#define HAL_GMAC_RX_BUFN            2
#define HAL_GMAC_ASSEM_BUFN         3

/*****************************************************************************
 * Max number of buffers
 *****************************************************************************/
#define HAL_NBUFS                   HAL_GMAC_NBUFS

/*****************************************************************************
 * DEFINITION: HAL return values to application.
 *****************************************************************************/
typedef enum {
  HAL_SUCCESS = 0,
  HAL_FAIL,
  HAL_HEAP_MEMORY_ERR,           /* Error encountered allocating heap memory */
  HAL_DETECT_NO_CLOCKS,          /* No clocks detected on TDM port           */
  HAL_ILLEGAL_CLOCK_CONFIG,      /* Clocks < nTimeslots or Clocks > Max      */
  HAL_DMA_UNAVAILABLE,           /* DMA resource unavailable                 */
  HAL_TDM_ISR_UNAVAILABLE,       /* TDM isr resource unavailable             */
  HAL_TDM_NO_PORTS,              /* No ports available for allocation        */
  HAL_TDM_INVALID_HAL_TDM_ID,    /* Invalid tdm port specified               */
  HAL_TDM_INVALID_TDM_ON_IO,     /* Attempted tdm on IO port                 */
  HAL_TDM_INVALID_TDM_CONFIG,    /* Invalid tdm enable/disable config        */
  HAL_TDM_INVALID_TS,            /* Invalid timeslot specified               */
  HAL_TDM_INVALID_TX_RX_TS_PAIR, /* Invalid tx rx timeslot pair              */
  HAL_TDM_TS_ALREADY_ENABLED,    /* Asked to enable an already enabled ts    */
  HAL_TDM_NO_FREE_TS,            /* To many timeslots requested              */
  HAL_TDM_INVALID_DISABLE_TS,    /* Asked to disable an unenabled timeslot   */
  HAL_TDM_INVALID_SUBFRAME_LEN,  /* Invalid Sub frame length                 */ 
  HAL_PKT_INVALID_TXRX_CELLSIZE, /* Invalid tx rx Cell Sizes                 */
  HAL_PKT_PHY_PINS_MISMATCH,     /* The Slave ID and UPHY Pins Mismatch      */
  HAL_PKT_NO_FREE_TS,            /* Reserved - Clean UP                      */
  HAL_PKT_INVALID_HAL_UL2_ID,    /* Invalid Utopia Port Specified            */
  HAL_PKT_TX_PKT_OVERFLOW,       /* No room in tx buffer for packet          */
  HAL_HPI_INVALID_HAL_OFFSET,    /* Invalid Offset in HPI Memory Speciifed   */
  HAL_TMR_INVALID_HAL_TMR_ID,    /* Invalid Timer Port Specified.            */
  HAL_GPIO_INVALID_HAL_DEV_ID,   /* Unsupported GPIO Device Specified.       */
  HAL_GPIO_INVALID_HAL_PIN_ID,   /* Unsupported GPIO Pin Specified.          */
  HAL_GPIO_INVALID_HAL_CPU_ID,   /* GPIO Pin cannot be set from this Core    */
  HAL_MEMMOVE_INVALID_SRC_ADDR,  /* Source address is not even aligned       */
  HAL_MEMMOVE_INVALID_DST_ADDR,  /* Destination address is not even aligned  */
  HAL_MEMMOVE_INVALID_SIZE,      /* Size is not 32 bit word aligned          */
  HAL_MEMMOVE_INVALID_OPERATION, /* Operation is invalid                     */
  HAL_TDM_INVALID_COMBINE,       /* Invalid tdm resource combine             */
  HAL_WDTIMER_INVALID_OP,        /* Invalid operation to watchdog timer      */
  HAL_AIC_INST_TOO_SMALL,        /* can't fit into the heap provided         */
  HAL_AIC_FATAL,
  HAL_AIC_INVALID_TYPE,          /* the AIC type doesn't exist               */
  HAL_INTOSEL_FAIL,              /* Invalid intosel routing selected         */
  HAL_MAC_CPPI_CREATE_FAIL,      /* Failed to create a cppi                  */
  HAL_MAC_TICSL_PKT_CREATE_FAIL, /* Failed to create a packet chain for CSL  */
  HAL_MAC_TX_FAIL_MEMORY,        /* Mac transmit failed, no memory           */
  HAL_SW_CPPI_CREATE_FAIL,       /* Failed to create a cppi                  */
  HAL_SW_TX_FAIL_MEMORY,         /* Ethernet Switch transmit failed,no memory*/
  HAL_MAC_TX_CSL_FAIL,           /* Mac transmit failed by CSL sendPacket () failed */
  HAL_PKT_NO_PORTS,              /* Utopia port allocation failed            */
  HAL_GMAC_NO_PORTS,             /* Gmac port allocation failed              */
  HAL_SW_NO_PORTS,               /* Ethernet Switch port allocation failed   */
  HAL_GMAC_NO_ADDR,              /* Gmac match addresses space full          */
  HAL_MEMMOVE_INVALID_NO_OF_CHS, /* Invalid number of channels: reduce it    */
  HAL_TDM_QUEUE_FULL_OPEN,       /* TDM Enable Fail - Queue full */
  HAL_TDM_QUEUE_FULL_CLOSE,      /* TDM Disable Fail - Queue full */
  HAL_TDM_INVALID_TDM_ON_ITDM,   /* Invalid tdm mode received in itdm mode   */
  HAL_TDM_INVALID_PORT,          /* Invalid tdm port request                 */
  HAL_TDM_INVALID_CHAN,          /* Invalid tdm channel                      */
  HAL_TDM_NO_FREE_CHAN,          /* No free channels remain                  */
  HAL_TDM_TSIP_LLD_ERROR         /* TSIP LLD returned not OK                 */
} halReturn_t;

/*****************************************************************************
 * DEFINITION: Memory buffer allocation structures. The structures used by
 *             HAL to request memory.
 ****************************************************************************/
typedef struct {
  tint  class;      /* Memory class. It is used to describe kind of memory
                     *  that is requested or returned. For example: external,
                     *  internal, or similar. One should use MEM_CLASS_xxx
                     *  constants. On RETURN, class may be changed if original
                     *  memory class is exhausted. */
  tuint log2align;  /* Alignment information (>= 0). If zero, no alignment is
                     *  is required nor provided. Otherwise, memory buffer is
                     *  aligned on appropriate power of 2 (e.g. if log2align=3,
                     *  buffer is aligned modulo-8). */
  tulong size;      /* Number of words that are requested or supplied. Word
                     *  length depends on the platform, and corresponds to
                     *  the shortest element that can be addressed by the CPU.
                     *  Word storage length in bits is defined in types.h as
                     *  TYP_WORD_SIZE. The maximum size depends on the
                     *  platform. */
  bool  volat;      /* TRUE: Memory buffer must be restored before and saved
                     *  after it is used.
                     * FALSE: Save/restore are not necessary.
                     *  On RETURN, volatile flag may be changed if original
                     *  request could not be satisfied. */
  void  *base;      /* Base address of the requested buffer. */
} halMemBuffer_t;

/* Memory class definitions */
enum {
  HAL_MEM_CLASS_EXTERNAL = 0,   /* generic (bulk) external memory */
  HAL_MEM_CLASS_INTERNAL = 1    /* generic (bulk) internal memory */
};

/*****************************************************************************
 * DEFINITION: Some TDM peripherals can be configured as gpio
 *****************************************************************************/
typedef enum {
  HAL_TDM_AS_TDM,        /* Port is used as a TDM port */
  HAL_TDM_AS_GPIO,       /* Port is used as GPIO       */
  HAL_TDM_AS_TDM_INIT,   /* Port is used at TDM - perform hardware initialization */
  HAL_TDM_AS_TDM_NO_INIT /* Port is used as TDM - no hardware initialization      */
} halSportFcn_t;

/*****************************************************************************
 * DEFINITION: The TDM ports can be configured to start up when HAL is
 *             initialized, or to start up when the 1st timeslot is enabled
 *             and shutdown when the last timeslot is disabled.
 *****************************************************************************/
typedef enum {
  HAL_TDM_ALWAYS_ON,     /* TDM port is always on                          */
  HAL_TDM_ON_AS_NEEDED   /* TDM port is on only when a timeslot is enabled */
} halTdmStart_t;

/*****************************************************************************
 * Definition: McBSP type. This only is used for the C54x to distinguish the
 *             McBSP from the eMcBSP
 *****************************************************************************/
typedef enum {
  HAL_TDM_EMCBSP,
  HAL_TDM_MCBSP
} halMcBspType_t;

/******************************************************************************
 * DEFINITION: Configuration of TDM port in GPIO mode
 *****************************************************************************/
typedef enum {
  HAL_PIN_AS_INPUT,     /* Specified pin used as input  */
  HAL_PIN_AS_OUTPUT,    /* Specified pin used as output */
  HAL_PIN_UNUSED        /* Specified pin not used       */
} halPinUse_t;

typedef struct {
  halPinUse_t clkx;     /* TDM xmit clock pin      */
  halPinUse_t fsx;      /* TDM xmit frame sync pin */
  halPinUse_t dx;       /* TDM xmit data pin       */
  halPinUse_t clkr;     /* TDM rcv clock pin       */
  halPinUse_t fsr1;     /* TDM rcv frame sync pin  */
  halPinUse_t dr;       /* TDM rcv data pin        */
  halPinUse_t aux_clk;  /* TDM auxillary clock pin */
} halTdmGpio_t;

/*****************************************************************************
 * DEFINITION: Configuration of TDM port in TDM mode
 ****************************************************************************/
#define HAL_TDM_AUTO_DETECT_CLOCKS  0   /* Auto detect clocks per frame */

typedef enum {
   HAL_BIT_EXACT,
   HAL_MULAW,
   HAL_ALAW
} halCompand_t;

typedef enum {
   HAL_MONO,
   HAL_STEREO
} halTsType_t;

typedef enum {
   HAL_TDM_PORT_ENABLE,
   HAL_TDM_TDM_CONFIG,
   HAL_TDM_TIMESLOT_CONFIG,
   HAL_TDM_PHASE_CONFIG,
   HAL_TDM_PHASE_QUERY,
   HAL_TDM_RESOURCE_COMBINE,
   HAL_TDM_SET_CALLOUT,
   HAL_TDM_GET_CALLOUT,
   HAL_TDM_MEGA_STAGGER,
   HAL_TDM_HALT_TDM,
   HAL_TDM_RESTORE_TDM,
   HAL_TDM_ENABLE_INT,
   HAL_TDM_ITDM_CONFIG,
   HAL_TDM_GET_TX_DMA_POS_CALLBACK
} halTdmControlMode_t;

/*****************************************************************************
 * tdmClockCfg bit mapped parameters - must match hardware config bit mapping
 *****************************************************************************/
#define HAL_TDM_TDM_CLOCK_CFG_TX_DAT_DELAY(x)      ((x) & 0x0007)
#define HAL_TDM_TDM_CLOCK_CFG_TX_CLK_POLARITY(x)   (((x)>>3)&0x0001)
#define HAL_TDM_TDM_CLOCK_CFG_TX_FSYNC_POLARITY(x) (((x)>>4)&0x0001)
#define HAL_TDM_TDM_CLOCK_CFG_RX_DAT_DELAY(x)      (((x)>>8)&0x0007)
#define HAL_TDM_TDM_CLOCK_CFG_RX_CLK_POLARITY(x)   (((x)>>11)&0x0001)
#define HAL_TDM_TDM_CLOCK_CFG_RX_FSYNC_POLARITY(x) (((x)>>12)&0x0001)
#define HAL_TDM_TDM_CLOCK_CFG_DELAY_MASK           0x0007
#define HAL_TDM_TDM_CLOCK_DAT_DELAY_OFFSET         1

#define HAL_TDM_TDM_IDLE_DRIVE_0       0
#define HAL_TDM_TDM_IDLE_DRIVE_1       1
#define HAL_TDM_TDM_IDLE_DRIVE_Z       2

#if 0
typedef struct {
  tuint wordSize;       /* Length of TDM word, in bits                       */
  bool  bdxDelay;       /* 1st bit delay enable - for wired or'd ports       */
  tuint clocksPerFrame; /* Number of bit clocks received per frame sync      */
  tuint tdmBitMask;     /* Bit Mask for which has to be updated. NOT USED    */
  tuint tdmClockCfg;    /* TDM clocking configuration - must match value
                         * from hardware config message.                     */
  tuint physPort;       /* Physical TDM port number (indexed from 0)         */
  halIdleDrive_t idleDrive;  /* Idle drive pattern to use                    */
  tuint tdmCompanding;                     
} halTdmTdmControl_t;
#endif


typedef struct halTdmConfigDir_s
{
  tint dataDelay;       /* Amount of delay between fsync and 1st data bit */
  tint bdxDelay;        /* 1st bit delay enable - for wire or'd ports     */
  tint idleDrive;       /* Value to drive on an idle timeslot (tx only)   */
  tint fsyncPol;        /* Frame sync Polarity   (1 to invert)            */
  tint fsyncClkPol;     /* Frame sync clock polarity                      */
  tint clkPol;          /* Clock polarity                                 */
  tint clkRate;         /* Clock rate                                     */
  tint tsPerFrame;      /* Number of timeslots per frame                  */
  tint dataRate;        /* Data rate                                      */
  tint clkRedund;       /* Clock redundancy flag                          */
  tint clkSrc;          /* Clock Source                                   */
} halTdmConfigDir_t;

typedef struct halTdmSizes_s { 
 
  tint  maxChannels;    /* Maximum number of channels supported */  
  tint  subFrameSize;   /* Number of frames per sub frame       */
  tint  wordSize;       /* The size of one sample, in bits      */
  tuint buffIncr;      /* The offset where the next buffer will be located  */
 
 
} halTdmSizes_t;


typedef struct halTdmInit_s
{
  tint physPort;       /* Physical port number, 0 indexed */
  tint halId;          /* Returned halId                  */
  tulong hwId;         /* Returned hardware ID            */
  halSportFcn_t fcn;   /* Port use, TDM or GPIO           */

  halTdmSizes_t sizes;  /* Size configuration info   */
  
  tuint tdmCompanding;  /* Port level companding config              */
  halTdmStart_t start;  /* Startup/shutdown status of TDM port               */
  
  tint  maxPhase;       /* Maximum number of phases port tracks              */
  tint  maxMegaPhase;   /* Maximum number of mega phases tracked             */
  
  tint  mcbsp_block;    /* Which block of 32 timeslots the TDM "sees". This is
                         * only the McBSP. The eMcBSP can see the full range
                         * of timeslots.                                     */
  tuint *initTs;        /* If non NULL, this array must contain maxChannels 
                         * values which are the timeslots to bring up (tx and
                         * rx) at startup.                                    */
  void *aicInst;        /* aic Instance pointer */
  tuint silence;        /* Initial buffer fill value         */
  
                         
  /* Periodic Function to call at every subframe, along with callout context */
  void (*subFrameCallout) (void *, tuint);
  void *cxt;        

  halTdmConfigDir_t tx;  /* Transmit config */
  halTdmConfigDir_t rx;  /* Receive config  */
  
} halTdmInit_t;

/*******************************************************************************
 * MGB_MDS_SRIO_INIT_CONFIG
 *******************************************************************************/
#define HAL_SRIO_PORT_INIT_CFG_VALID_SERDES_CFGRX_CNTL   (1<<0)
#define HAL_SRIO_PORT_INIT_CFG_VALID_SERDES_CFGTX_CNTL   (1<<1)
#define HAL_SRIO_PORT_INIT_CFG_VALID_PF_8B_CNTL          (1<<2)
#define HAL_SRIO_PORT_INIT_CFG_VALID_PF_16B_CNTL         (1<<3)


typedef struct halSrioPortInitCfg_s {
  tuint  physPort;       /* Physical port number, 0 indexed */
  tuint  valid_params;
  tulong  serdesCfgRxCntl;  /* See register SERDES_CFGRXn_CNTL */
  tulong  serdesCfgTxCntl;  /* See register SERDES_CFGTXn_CNTL */
  tulong  pf8bCntl;       /* See register PF_8Bn_CNTL */
  tulong  pf16bCntl;       /* See register PF_16Bn_CNTL */
} halSrioPortInitCfg_t;

#define HAL_SRIO_MAX_NO_OF_PORTS  4

/* All configuration parameters must  be provided  to configure SRIO*/
#define HAL_SRIO_INIT_CONFIG_VALID_BASEID_REG      (1<<0)
#define HAL_SRIO_INIT_CONFIG_VALID_DEVICEID_REG1   (1<<1)
#define HAL_SRIO_INIT_CONFIG_VALID_DEVICEID_REG2   (1<<2)
#define HAL_SRIO_INIT_CONFIG_VALID_DEVICEID_REG3   (1<<3)
#define HAL_SRIO_INIT_CONFIG_VALID_DEVICEID_REG4   (1<<4)
#define HAL_SRIO_INIT_CONFIG_VALID_SERDES_CFG0_CNTL     (1<<5)

typedef struct halSrioInitCfg_s {
  tuint  valid_params;
  tulong baseidReg;    /* See register BASE_ID_REG */
  tulong deviceidReg1; /* See register DEVICEID_REG1 */
  tulong deviceidReg2; /* See register DEVICEID_REG2 */
  tulong deviceidReg3; /* See register DEVICEID_REG3 */
  tulong deviceidReg4; /* See register DEVICEID_REG4 */
  tulong serdesCfg0Cntl;  /* See register SERDES_CFGn_CNTL */
  tuint  numPorts;
  halSrioPortInitCfg_t srioPortInitCfg[HAL_SRIO_MAX_NO_OF_PORTS];
} halSrioInitCfg_t;

typedef struct {
  tint  recip;      /* Hal tdm Id of port receiveing memory */
  tint  donor;      /* Hal tdm Id of port donating memory   */
} halTdmCombine_t;

typedef struct {
  bool         toTdmEnable;    /* Enables or disables tdm xmit             */
  bool         fromTdmEnable;  /* Enables or disables tdm rcv              */
  tint         txTimeslot;     /* TDM Tx timeslot number                   */
  tint         rxTimeslot;     /* TDM Rx timeslot number                   */
  halCompand_t compand;        /* Companding                               */
  tuint        initVal;        /* initial value of TDM                     */
  tuint        txFrameSize;    /* Must be multiple of subFrameRate         */
  tuint        rxFrameSize;    /* Must be multiple of subFrameRate         */
  tuint        phase;          /* Channel staggering phase                 */
  /* Callout Functions when frame received */
  void (*txCallout) (void *, linSample **, tuint, tuint);
  void (*rxCallout) (void *, linSample **, tuint, tuint);
  tfPtr        txContext;      /* 1st argument to pass back in txCallout   */
  tfPtr        rxContext;      /* 1st argument to pass back in rxCallout   */
  void        *tecInst;        /* Pointer to associated TEC instance       */
  linSample   *txBuffer;       /* Initial tx data to send out              */
  linSample   *rxBuffer;       /* Place to store initial rx data           */
  tuint        txBufSize;      /* Size of tx buffers                       */
  tuint        rxBufSize;      /* Size of rx buffers                       */
  void        *hId;            /* Restore ID used for mega-staggering      */
  void        *hIdb;           /* Second mega-staggering ID                */
  bool        duplicate;       /* If this is associated with a Timeslot    */
                               /*   and ACTIVE then it has a DUPLICATE TS  */
  tint       duplicateTS;      /* if associated with an ACTIVE TS and      */
                               /* duplicate == TRUE then this is the       */
                               /* DUPLICATE timeslot                       */
  halTsType_t tstype;          /* Time slot can be mono or a stereo        */

  /* following are added to keep hw_config message processing intact */
  tuint wordSize;       /* Length of TDM word, in bits                       */
  tint  bdxDelay;       /* 1st bit delay enable - for wired or'd ports       */
  tuint clocksPerFrame; /* Number of bit clocks received per frame sync      */
  tuint tdmClockCfg;    /* TDM clocking configuration - must match value
                         * from hardware config message.                     */
  tuint physPort;       /* Physical TDM port number (indexed from 0)         */
  halIdleDrive_t idleDrive;  /* Idle drive pattern to use                    */
  
  /* Output params */
  void  *txTsContext;   /* Timeslot context pointer (o)                      */
  bool         timeslotPrimary;  /* The timeslot is taken as a primary timeslot
                                    which performs TDM-PIU callouts in 16-bit
                                    linear companing mode for Kailash. 
                                    Callouts will not be executed for a timeslot
                                    if it is not a primary timeslot           */
} halTdmConfig_t;                              

#ifdef ITDM
/*********************************************************************************
 * Multiple timeslot channel configuration - used for iTdm. A single
 * channel can contain multiple timeslots.
 *
 * DO NOT MODIFY THE VALUES FOR THESE ENUMERATIONS! THE HAL CODE USES THESE
 * VALUES FOR TABLE INDICIES!
 *********************************************************************************/
enum {
  HAL_TDM_ITDM_ENABLE_CHANNEL  = 0,
  HAL_TDM_ITDM_MODIFY_CHANNEL  = 2,
  HAL_TDM_ITDM_DISABLE_CHANNEL = 4
};

enum {
  HAL_TDM_ITDM_CHAN_TO_TDM   = 0,
  HAL_TDM_ITDM_CHAN_FROM_TDM = 1
};

typedef struct haliTdmConfig_s {

  tint opMode;      /* enable, disable, modify map */
  tint direction;   /* HAL_TDM_ITDM_CHAN_TO_TDM or HAL_TDM_ITDM_CHAN_FROM_TDM */
  tint port;        /* Actually redundant for enables and disables, but used
                     * as a sanity check */
  
  tint halId;       /* returned by hal for channel enable, provided for channel
                     * disable and modify map */
                     
  /* Function to call with received data or to get data to be transmitted */                       
  void (*callout) (void *vinst, itdmMultiTsMap_t *map, tuint **dataBuffer, tuint ts);                     
  void *vinst;
  
  tuint *dataBuffer; /* Initial buffer for rx data. Not required for tx */
  
  itdmMultiTsMap_t *tsMap;  /* Timeslot map. Not required for closing a channel */
  
} haliTdmConfig_t;
#endif

typedef enum {
  HAL_TDM_CTL_HALT_LOCAL,
  HAL_TDM_CTL_HALT_GLOBAL,
  HAL_TDM_CTL_RESTORE_LOCAL,
  HAL_TDM_CTL_RESTORE_GLOBAL
} halTdmHaltRestoreOp_t;
typedef struct halTdmHaltRestore_s  {

  halTdmHaltRestoreOp_t operation;
  
  
  tulong state[48];  /* Saved state information */
  
} halTdmHalRestore_t;

typedef struct {
  halTdmControlMode_t mode;    /* Control Mode: Timeslot or TDM Config     */

  union {
    halTdmConfig_t      tdmConfig;   /* Run time timeslot config                 */
    halTdmCombine_t     tdmCombine;  /* McBSP memory consolidation               */
    tint              (*tdmTxDmaPosFcn) (void *cxt); /* Current TX DMA offset */
#ifdef ITDM
    haliTdmConfig_t     itdmConfig;  /* Multi timeslot configuration             */
#endif
    halTdmHalRestore_t  haltRestore; /* Temporary disable/enable a tdm port      */
  } u;
  
} halTdmControl_t;

/*****************************************************************************
 * DEFINTION: Structure definition used to control HPI
 *****************************************************************************/
#define  HPI_CTRL_FCN_GENINT   0x1   /* Generate HPI Interrupt        */
#define  HPI_CTRL_FCN_RCVINT   0x2   /* Initialize revceive intr fcn  */
#define  HPI_CTRL_FCN_SETADDR  0x4   /* Set address and size for HPI  */


typedef struct {
  tuint  hpiCtrlFcn;   /* Bitmap to indicate what has to be done.        */
  void   (*hpiInt)();  /* Function to call on received HPI interrupt     */
  void   *context;     /* Context to be passed back with HPI interrupt   */
  tint   *address;     /* Specifies the absolute starting address of HPI */
  tuint  size;         /* Specifies the size of the HPI area, in words   */
  tint   intNum;       /* Interrupt number of the HPI as per HD HPI's API */
} halHpiControl_t;

/*****************************************************************************
 * DEFINTION: Structure definition used to control IPC
 *****************************************************************************/
#define  IPC_CTRL_FCN_GENINT   0x1   /* Generate IPC Interrupt       */
#define  IPC_CTRL_FCN_RCVINT   0x2   /* Initialize receive intr fcn  */

typedef struct {
  tuint  ipcCtrlFcn;   /* Bitmap to indicate what has to be done.        */
  tuint  oCore;		   /* Other core to be interrupted                   */
  void   (*ipcInt)();  /* Function to call on received IPC interrupt     */
  void   *context;     /* Context to be passed back with IPC interrupt   */
} halIpcControl_t;

/*****************************************************************************/
typedef struct {
  tuint  bufferUnitSizeWords;  /* Unit buffer size, in CPU addressable words   */
  tulong bufferTotalSizeWords; /* Total buffer size, in CPU addressable words  */
  tuint *startAddr;            /* Buffer start addr, for alignment */
} halEthBuf_t;

typedef struct halGmacSizes_s {
  halEthBuf_t tx;                    /* Buffer config packets out of dsp */
  halEthBuf_t rx;                    /* Buffer config packets into dsp   */
  tuint       maxRxPacketSizeWords;  /* Assembly buffer requirements     */
} halGmacSizes_t;

enum {
  HAL_GMAC_INIT_OPERATION_ENABLE,  /* Powers up and configures the port    */
  HAL_GMAC_INIT_OPERATION_CHANNEL  /* Does not alter global configurations */
};

typedef struct halGmacInit_s {
  tint  halId;            /* Hal Id associated with the memory buffers    */
  tint  operation;        /* Allocate, Allocate and Init, Run time config */
  tint  physPort;         /* The physical port number, 0 indexed          */
  tulong hwId;            /* Returned hardware ID of the port             */
  void (*rcvFrame)(void *, word *, tuint);  /* Rx routine for the application.    */
  void *cxt;              /* Receive callout context             */
  cslMacConfig_t cfg;     /* Port configuration                  */
} halGmacInit_t;

enum {
 HAL_GMAC_CTL_CODE_ID_FROM_PORT,
 HAL_GMAC_CTL_CODE_TX_CPPI_AVAIL_BUF,
 HAL_GMAC_CTL_CODE_TMP_DISABLE,
 HAL_GMAC_CTL_CODE_RE_ENABLE,
 HAL_GMAC_CTL_CODE_GET_CALLOUT_CXT,
 HAL_GMAC_CTL_CODE_SET_CALLOUT_CXT,
 HAL_GMAC_CTL_CODE_ADD_MAC_ADDR,
 HAL_GMAC_CTL_CODE_DEL_MAC_ADDR
};

typedef struct {
  tint physPort;  /* Input physical port              */
  tint halId;    /* Returned hal Id, -1 if not found */
} halGmacCtlPortId_t;

typedef struct {
  tint  halId;      /* The HAL ID (logical port num) */
  void *rcvFcn;     /* The receive packet call back  */
  void *rcvCxt;     /* The receive packet context    */
} halGmacCalloutCxt_t;

typedef struct {
  tuint halId;
  tuint macAddr[3];
} halGmacMacAddr_t;

typedef struct {
  tint tx_cppi_bytes_available;
  tint tx_cppi_buffs_available;
} halGmacTxCPPIAvail_t;

typedef struct {
  tuint code;
  union {
    halGmacCtlPortId_t   portId;
    halGmacCalloutCxt_t  calloutCxt;
	halGmacTxCPPIAvail_t txCppiAvail;
    halGmacMacAddr_t     addr;
    tint halId;
  } u;
} halGmacControl_t;  

typedef struct {
  tuint nSegs;          /* Number of segments          */
  tuint **segs;         /* Array of segment pointers   */
  tuint *sizes;         /* Array of segment sizes      */
  tuint type;           /* Transfer type identifier    */
} halGmacData_t;

/********************************************************************************
 * Utopia
 ********************************************************************************/
 
typedef enum {
  HAL_UTOPIA_PASS_THRU,
  HAL_UTOPIA_USER_ROUTE
} halUtopiaMode_t;

typedef struct halUtopiaGetSizes_s {

  tuint xmtCellSizeBytes;     /* Size of cells, in bytes, the DSP xmits     */
  tuint rcvCellSizeBytes;     /* Size of cells, in bytes, received by DSP   */
  tuint txBufSizeBytes;       /* Size of the transmit buffer, in bytes      */
  tuint rxBufSizeBytes;       /* Size of the receive buffer, in bytes       */
  tuint rxCABufSizeBytes;     /* Size of the catch all buffer, in bytes     */
  
} halUtopiaSizes_t;


typedef struct {
  tint  portId;          /* Port Number (indexed from 0)      */
  tint  halId;           /* Returned by HAL after new         */
  tint  busWidth;        /* 8 or 16 bit external bus          */
  tint slaveId;          /* Slave ID address, -1 to read pins */
  halUtopiaMode_t mode;  /* Pass Through Mode or User Defined Routing  */
  void (*rcvCell)(void *, word *, tuint); /* Function to call when a cell is received   */
  void *rcvCxt;           /* Context passed with received cells */
  tuint *rcvCellBufFull; /* Pointer to (optional) fifo full counter    */
  halUtopiaSizes_t sizes; /* Size configuration   */
} halUtopiaInit_t;
 
  
  
typedef struct {
  tuint xmtCellSize;     /* Size of cells, in bytes, the DSP xmits     */
  tuint rcvCellSize;     /* Size of cells, in bytes, received by DSP   */
  halUtopiaMode_t mode;  /* Pass Through Mode or User Defined Routing  */
  tuint pktBitMask;      /* Bit Mask for which has to be updated. NOT USED    */
} halUtopiaPktControl_t;

typedef enum {
   HAL_UTOPIA_PORT_ENABLE,
   HAL_UTOPIA_PORT_DISABLE,
   HAL_UTOPIA_ROUTE_CONFIG,
   HAL_UTOPIA_PKT_CONFIG,
   HAL_UTOPIA_GET_FREE_CELLS,
   HAL_UTOPIA_RX_TMP_DISABLE_CORE,  /* Temporarily disable receive for this core */
   HAL_UTOPIA_RX_TMP_DISABLE_ALL,   /* Temporarily disable receive for all cores */
   HAL_UTOPIA_RX_RE_ENABLE_CORE,    /* Restore after temporary disable this core */
   HAL_UTOPIA_RX_RE_ENABLE_ALL,     /* Restore after temporary disable all cores */
   HAL_UTOPIA_BAD_RX_CELL           /* A bad cell was received                   */
} halUtopiaControlMode_t;

typedef struct {
  tuint match;     /* Match value for connection */
  tuint mask;      /* Mask value for connection  */
} halRecords_t;
  
#define HAL_UTOPIA_MAX_RECORDS     8
typedef struct {
  tuint routeSource;     /* bits 0-2:  start bit position within a byte *
                          * bits 3-11: start byte position within a cell */
  tuint nRecords;        /* Number of routing records                    */
  halRecords_t records[HAL_UTOPIA_MAX_RECORDS];  /* the routing records  */
} halRouteDef_t;

typedef struct {
  tuint nSegs;          /* Number of segments          */
  tuint **segs;         /* Array of segment pointers   */
  tuint *sizes;         /* Array of segment sizes      */
  tuint type;           /* Transfer type identifier    */
} halUtopiaData_t;

typedef struct {
  tuint *seg;           /* Pointer to the ocation of the Segment */
  tuint size;           /* Segment Size in Bytes                 */
} halUtopiaRxData_t;

typedef struct {

  halUtopiaControlMode_t mode ; /* Mode of Control to be done  */
  
  union {
    halRouteDef_t          route; /* Routing Details for the DSP */
    halUtopiaPktControl_t  pkt  ; /* Re-Config Params for Utopia */
    tuint                  freeCells ; /* Number of free cells in buffer */
  } u;
  
} halUtopiaControl_t;

/*****************************************************************************
 * DEFINITION: AIC configuration structure for HAL. 
 *****************************************************************************/
typedef struct {
  tuint       typeId;   /* AIC type */
  tint        numAics;   /* Number of AICs connected to this TDM in different timeslots
                           Note: presently it is restricted and we should have only one
                           type of AIC sharing same TDM bus */
  tint        tdmPort;
  tuint       instSize;
} halAicInit_t; 

/*****************************************************************************
 * DEFINITION: Structure used by HAL to determine its size requirements
 *****************************************************************************/
typedef struct {
  tint nTdmPorts;        /* Number of elements in halTdmInit array  */
  tint nPktPorts;        /* Number of elements in halPktInit array  */
  tint nAicPorts;        /* Number of elements in halAicInit array  */
  tint nGmacPorts;       /* Number of elements in halGmacInit array */
  tint coreId;           /* Returns the core ID                     */
  tuint revId;           /* Returns the chip revision ID            */

#ifdef TSIP_LLD
  tsipSizeInfo_t tsipSizeCfg;
#endif

} halNewConfig_t;

/*****************************************************************************
 * DEFINITION: Structures used to configure GPIO
 *****************************************************************************/
typedef enum {
  HAL_GPIO_0,      /* Dedicated GPIO Pin 0                 */
  HAL_GPIO_1,      /* Dedicated GPIO Pin 1                 */
  HAL_GPIO_2,      /* Dedicated GPIO Pin 2                 */
  HAL_GPIO_3,      /* Dedicated GPIO Pin 3                 */
  HAL_GPIO_4,      /* Dedicated GPIO Pin 4                 */
  HAL_GPIO_5,      /* Dedicated GPIO Pin 5                 */
  HAL_GPIO_6,      /* Dedicated GPIO Pin 6                 */
  HAL_GPIO_7,      /* Dedicated GPIO Pin 7                 */
  HAL_GPIO_BIO,    /* System bio pin, output               */
  HAL_GPIO_XF,     /* System xf pin, input                 */
  HAL_GPIO_CLKX,   /* TDM port xmit clock pin, I/O         */
  HAL_GPIO_FSX,    /* TDM port xmit frame sync pin, I/O    */
  HAL_GPIO_DX,     /* TDM port xmit data pin, output       */
  HAL_GPIO_CLKR,   /* TDM port rcv clock pin, I/O          */
  HAL_GPIO_FSR,    /* TDM port rcv frame sync pin, I/O     */
  HAL_GPIO_DR,     /* TDM port rcv data pin, input         */
  HAL_GPIO_CLKS    /* TDM port clks pin, input             */
} halGpioPin_t;

typedef enum {
  HAL_DEVICE_GPIO,
  HAL_DEVICE_TDM
} halGpioDeviceType_t;

typedef struct {
  halGpioDeviceType_t device; /* The pin multiplexed device */
  tint                id;     /* The device Id              */
} halGpioDevice_t;

typedef struct {
  halGpioPin_t pin;         /* The GPIO pin to control                        */
  halGpioDevice_t  device;  /* The device for mutliplexed pins                */
  bool         state;       /* Used to set the state for tx, return it for rx */
} halGpio_t;

/*****************************************************************************
 * DEFINITION: Structures used to control interrupt handling
 *****************************************************************************/
typedef enum {
  HAL_INT_LEVEL_DISABLE,   /* Disable interrupts  */
  HAL_INT_LEVEL_ENABLE,    /* Enable interrupts   */
  HAL_INT_LEVEL_NO_CHANGE  /* Interrupt unchanged */
} halIntrLevel_t;

typedef struct {
  tint intNum;             /* External interrupt number                */
  void (*intFcn)();        /* Function to call when interrupt received */
} halExternalIntControl_t;

typedef enum {
  HAL_INT_RETRIEVE,
  HAL_INT_SET,
  HAL_INT_CLEAR_ALL
} halIntrOp_t;

#define HAL_INT_STATE_SIZE  2

typedef struct {
  halIntrOp_t   operation;
  tuint         state[HAL_INT_STATE_SIZE];
} halIntStateControl_t;
  
/*****************************************************************************
 * DEFINTION: Structure definition used for I-Cache config.
 *****************************************************************************/

#define HAL_CACHE_RAMSETS 2

/* Values used by ramsetTagSelect in structure halCacheCtrlRamset_t */
#define HAL_CACHE_RAMSET_TAG_1 0x1
#define HAL_CACHE_RAMSET_TAG_2 0x2 

typedef enum {
  HAL_CACHE_MODE2WAY =  2,
  HAL_CACHE_DISABLE  = -1
} halCacheMode_e;

typedef enum {
  HAL_CACHE_L2,       /* L2 setup runtime  */
  HAL_CACHE_L1P,      /* L1P setup runtime */
  HAL_CACHE_L1D,      /* L1D setup runtime */
  HAL_CACHE_ALL,      /* For Cache Flush only */ 
  
  HAL_CACHE_L2_FDEF,   /* L2 setup from gghwcfg.h  */
  HAL_CACHE_L1P_FDEF,  /* L1P setup from gghwcfg.h */
  HAL_CACHE_L1D_FDEF,  /* L1D setup from gghwcfg.h */
  HAL_CACHE_MAR        /* MAR setup */
} halCacheType_e;


typedef void (*halCacheTag_t)(void);

typedef struct {
  tuint cacheMode;
  tuint numRamsets;
  halCacheTag_t ramsetTags[HAL_CACHE_RAMSETS];
  halCacheType_e cacheType;
  tint *marRegs;
  tint *marVals;
} halCacheConfig_t;


typedef enum {
  HAL_CACHE_CTRL_DISABLE  = 0,
  HAL_CACHE_CTRL_ENABLE   = 1,
  HAL_CACHE_CTRL_FLUSH    = 2,
  HAL_CACHE_CTRL_RAMSET     = 3,
  HAL_CACHE_CTRL_INVALIDATE = 4,
  HAL_CACHE_CTRL_WRITEBACK  = 5,
  HAL_CACHE_CTRL_WB_INV     = 6
} halCacheCtrl_e;

typedef struct {
  tuint ramsetTagSelect; /* Bit field indicating ramset tags to modify */
  halCacheTag_t tag[HAL_CACHE_RAMSETS];
} halCacheCtrlRamset_t;
/* invalidate, writeback, writeback and invalidate configuration */
typedef struct {
  tulong addr;
  tulong sizeBytes;
} halCacheCtrlWI_t;
  

typedef union {
  int disable;
  int enable;
  int flush;
  halCacheCtrlRamset_t ramset;
  halCacheCtrlWI_t  wbi;
}halCacheCtrl_u;


typedef struct {
  halCacheCtrl_e cacheCtrl;
  halCacheCtrl_u    u;
  halCacheType_e cacheType;
} halCacheControl_t;

                        
/*****************************************************************************
 * DEFINTION: Structure definition used for memory config.
 *****************************************************************************/
#define HAL_MEMCFGFUNC_EMIFCONFIG  0x1  /* Configure EMIF  */
#define HAL_MEMCFGFUNC_CACHECONFIG 0x2  /* Configure Cache */
#define HAL_MEMCFGFUNC_SMCCONFIG   0x3  /* Configure Cache */
#define HAL_MEMCFGFUNC_EMIFACONFIG 0x4  /* Configure EMIFA */
#define HAL_MEMCFGFUNC_ECFG        0x8  /* Configure EMIF run time. 
                                         * No bits in common with other modes. */
#define HAL_MEMCFGFUNC_XMC_MPAX_CONFIG 0x10 /* Configure XMC MPAX. 
                                         * No bits in common with other modes. */
#define HAL_MEMCFGFUNC_SMS_MPAX_CONFIG 0x20 /* Configure XMC MPAX. 
                                         * No bits in common with other modes. */
#define HAL_MEMCFGFUNC_SES_MPAX_CONFIG 0x40 /* Configure XMC MPAX. 
                                         * No bits in common with other modes. */

typedef struct {
  tuint memCfgFunction;           /* Mem Config Function */
  union {
    halCacheConfig_t cacheCfg;        /* Parameters for cache config */
    struct MGB_MDS_EMIF_CONFIG ecfg;  /* Run time configuration      */
  } u;
} halMemConfig_t;


/*****************************************************************************
 * DEFINITION: Structure used to configure timers
 *****************************************************************************/
#define HAL_TIMER_OP_SET_PERIOD     (1 << 0)   /* Set timer period        */
#define HAL_TIMER_ENABLE            (1 << 1)   /* Enable timer            */
#define HAL_TIMER_DISABLE           (1 << 2)   /* Disable timer           */
#define HAL_TIMER_GET_COUNT         (1 << 3)   /* Get timer count         */
#define HAL_TIMER_NUMBER            (1 << 4)   /* Return number of timers */
#define HAL_TIMER_DELTA             (1 << 5)   /* Timer delta             */

typedef struct {
  tuint  operation;   /* Bitmapped operation                             */
  tulong timerperiod; /* Timer period or returned counter */
  tuint  tickrate;    /* Rate at which timer increments with respect to 
                         CPU clock.This is always at a rate less than or
                         equal to CPU clock */
} halTimerControl_t;

/*****************************************************************************
 * DEFINITION: Structure used to configure/operate watchdog timer
 *****************************************************************************/
#define HAL_WDTIMER_ENABLE         (1 << 0)   /* Enable watchdog      */
#define HAL_WDTIMER_FEED           (1 << 1)   /* Feed the watchdog    */
#define HAL_WDTIMER_CHECK          (1 << 2)   /* Checks for failure   */

typedef struct {
  tuint operation;       /* Bit mapped operation        */
  tuint status;          /* non-zero if operation is check and watch dog failed */
  tulong periodCount;    /* Timer period, clock cycles  */
} halWdTimerControl_t;

/*****************************************************************************
 * DEFINITION: Structures used to configure memory transfer 
 *****************************************************************************/
typedef tulong halLocAddr_t;
typedef tulong halGlobAddr_t;
typedef tulong  halSAE_t; /* Size of transfer may be greater than 65535 */
typedef tint (*halMemMoveComplete_t)(void *);

/* These defines set the priority for DMA transfer.Multiple transfers may be 
   simultaneously in progress and hence require priority.c64x supports
   four levels of priority for EDMA transfers.
   Of these 
      a) Urgent priority  is reserved for L2 cache controller.
      b) High priority  is reserved for TDM-DMA transfer.
   Remaining two priorities are  available for user programming.
   If processor does not support these priorities the priority may be ignored */
#define HAL_MEMMV_PRIORITY_MEDIUM   0x2u
#define HAL_MEMMV_PRIORITY_LOW      0x3u


/******************************************************************************
 * DEFINITION: Messages used to handle AIC operations
 ******************************************************************************/
/* halAicControl cntrl typedefs */
typedef enum {
  HAL_AIC_CFG  = 0x0001,
  HAL_AIC_FREQ = 0x0002,
  HAL_AIC_CHECK= 0x0004
} halCtrl_e;

typedef union {
  struct MGB_MDS_ANALOG_CODEC_CONFIG  config;
} aicCtrl_t;

/***************************************/

typedef struct MGB_MDS_GAIN_CTRL  aicGainCtrl_t;

typedef struct {
  tuint clipval; /* clip Value */
  tuint time; /* time when the actual gain setting happens */
} aicRet_t;

/***************************************/

typedef struct MGB_MDS_ANALOG_CODEC_REGISTER_CTRL aicRegCtrl_t;

/***************************************/
typedef enum {
  HAL_AIC_GET_GAINS = 0x0001
} halGetCtrl_e;

typedef union {
  struct MGB_DMS_GET_GAINS  gains;
} aicGet_t;
/******************************************************************************
 * DEFINITION: The hal memory move instructions use the most significant bit
 *             of the source and destination addresses to distinguish
 *             program and data memory.
 *****************************************************************************/
//#ifdef GG_HAL_GLOBAL_ADDR_BIT_24
#ifdef __TMS320C55X__
#define HAL_MEM_ADDRESS_DATA(x)   ((x)&=0x7FFFFFFFl)  /* MSB=0 for data */
#define HAL_MEM_ADDRESS_PROG(x)   ((x)|=0x80000000l)  /* MSB=1 for prog */
#define HAL_MEM_SHARED_ADDR(x)    ((x)|=(0x0F000000l>>1))  /* Shared prefix address */
#else
#define HAL_MEM_ADDRESS_DATA(x)
#define HAL_MEM_ADDRESS_PROG(x)
#define HAL_MEM_SHARED_ADDR(x)
#endif

/*****************************************************************************
 * DEFINITION: Structures used for HAL Control Functions 
 *****************************************************************************/
#define HAL_MAX_TDM_PORTS 4    /* 4 TDM ports allowed as follows */
/* CAUTION: do not change the definitions of HAL_TDM_0 through HAL_TDM_4 */
/*          if change is necessary, change halControl code appropriately */
typedef enum {
  HAL_TDM,       /* TDM Ports          */
  HAL_HPI,       /* HPI Peripheral.    */
  HAL_TMR,       /* TIMERS             */
  HAL_EMIF,      /* EMIF Configuration */
  HAL_UL2,       /* UTOPIA Port        */
  HAL_PCI,       /* PCI Peripheral     */
  HAL_GMAC       /* GMAC Peripheral    */
} halPeriph_t;

/****************************************************************************
 * DEFINITION: Peripheral query definitions
 ****************************************************************************/
typedef enum {
  HAL_QUERY_UTOPIA,
  HAL_QUERY_MAC,
  HAL_QUERY_NCORES,
  HAL_QUERY_PG,
  HAL_QUERY_MCBSP,
  HAL_QUERY_TSIP,
  HAL_QUERY_TS_DECODE,    /* Timeslot (port/link/timeslot) decode */
  HAL_QUERY_TS_ENCODE,    /* Timeslot encode */
  HAL_QUERY_BOOTPINS,
  HAL_QUERY_MEMORY        /* Memory type (local to this core, shared, other core */
} halQueryPeriph_t;

typedef struct halQueryUtopia_s {
  tuint busWidth;       /* 8 or 16 bit utopia bus */
} halQueryUtopia_t;

typedef enum {
  HAL_MAC_SPEED_10_100,     /* 10/100 mac */
  HAL_MAC_SPEED_1G          /* 1g mac     */
} halMacSpeed_t;

typedef struct halQueryMac_s {
  halMacSpeed_t speed;
} halQueryMac_t;

typedef struct halQueryCores_s {
  tint nCores;
  tint thisCore;
} halQueryCores_t;

typedef struct halQueryPg_s {
  tuint pg;
} halQueryPg_t;

typedef enum {
  HAL_MCBSP_TYPE_MCBSP,
  HAL_MCBSP_TYPE_EMCBSP,
  HAL_MCBSP_TYPE_MCBSP512,
  HAL_MCBSP_TYPE_MCBSP256,
  HAL_MCBSP_TYPE_MCBSP128
} halMcbspType_t;

typedef struct halQueryMcbsp_s {
  halMcbspType_t type;
} halQueryMcbsp_t;

typedef struct halQueryTs_s {
  tint port;    /* port only     */
  tint link;    /* timeslot only */
  tint ts;      /* link only     */
  tuint port_link_ts;  /* combined port/link/ts - varies by build */
} halQueryTs_t;

typedef struct halQueryBootPin_s  {
  tuint bootmode;       /* Boot mode pins */
  tuint cfg;            /* cfg pins       */
} halQueryBootPin_t;

typedef enum {
  HAL_MEM_SHARED,      /* Memory address is shared memory */
  HAL_MEM_LOCAL,       /* Memory address is local to this core */
  HAL_MEM_OTHER_CORE,  /* Memory belongs to another core */
  HAL_MEM_PERIPH       /* Memory belongs to a peripheral */
} halQueryMemType_t;

typedef struct halQueryMem_s  {
  word              *address;    /* Address to be queried */
  halQueryMemType_t  type;       /* Returned memory info */
  tint               core;       /* If type is this core or other core, this is the core num */
} halQueryMem_t;

typedef struct halQuery_s {
  halQueryPeriph_t periph;  /* The port to query */
  tint   portNum;            /* Port number       */
  bool   isPresent;          /* True if the requested port is on the device */
  bool   isAvail;            /* True if the port is available through configuration */
  bool   isEnabled;          /* True if the device is powered up and running */
  tulong periphId;           /* Peripheral hardware ID  */
  union {
    halQueryUtopia_t utopia;
    halQueryMac_t    mac;
    halQueryCores_t  core;
    halQueryPg_t     pg;
    halQueryMcbsp_t  mcbsp;
    halQueryTs_t     ts;
    halQueryBootPin_t bootpin; 
    halQueryMem_t     mem;
  } u;
} halQuery_t;  
  
/*****************************************************************************
 * DEFINITION: HAL Control
 *****************************************************************************/
typedef enum {
  HAL_COMMAND_HALT,           /* Unrecoverable system halt                 */
  HAL_COMMAND_DMA_RESET,      /* DMA reset                 */
  HAL_COMMAND_PERIPH_RESET,   /* Puts system peripherals into a know state */
  HAL_COMMAND_LOW_POWER,      /* Enter Low Power mode */
  HAL_COMMAND_CORE_BOOT,      /* Release a core from reset */
  HAL_COMMAND_CORE_RESET,     /* Put a core into reset */
  HAL_COMMAND_SLEEP,          /* Execute an idle */
  HAL_COMMAND_INT_OTHER_CORE, /* Interrupt another core */
  HAL_COMMAND_INT_HOST,       /* Send interrupt to the host */
  HAL_COMMAND_IPC_WRITE,      /* Send message to a core, no interrupt */
  HAL_COMMAND_IPC_READ        /* Read IPC message from a core */

} halControlCmd_t;

typedef enum {
  HAL_COMMAND_SLEEP_WAKE_ANY_INT,    /* Wake on any interrupt */
  HAL_COMMAND_SLEEP_WAKE_OCORE_INT   /* Wake only on interrupt from another core */
} halCmdWakeOn_t;


typedef struct {
  tint portId;   /* HAL assigned port id */
} halCtlPeriphReset_t;

typedef struct {
  tint  core;        /* Core number, indexed from 0 */
  tulong entryAddr;  /* Boot entry point            */
} halCtlCoreBoot_t;

typedef struct {
  tint core;
} halCtlCoreReset_t;

typedef struct {
  halCmdWakeOn_t wakeMode;
} halCtlSleep_t;


typedef struct  {
  tuint  coreMask;  /* One bit per core to wake   */
  tulong info;      /* Info to pass to other core */
} halCtlIntCore_t;

typedef struct {
  tuint  core;
  tulong setMask;    
  tulong clearMask;
} halIpcWrite_t;

typedef struct {
  tuint  core;
  tulong msg;
} halIpcRead_t;
typedef struct halControl_s  {
  halControlCmd_t  cmd;         /* Command code */
  
  union {
    halCtlPeriphReset_t periphReset;
    halCtlCoreBoot_t    coreBoot;
    halCtlCoreReset_t   coreReset;
    halCtlSleep_t       sleep;
    halCtlIntCore_t     intCore;
    halIpcWrite_t       ipcWrite;
    halIpcRead_t        ipcRead;
  } u;
  
} halControl_t;


typedef enum {
   HAL_TID_CONTROL_GET_INTERRUPT_COUNT=1,
   HAL_TID_CONTROL_CLR_INTERRUPT_COUNT,
   HAL_TID_CONTROL_SET_OPERATION_STATUS,
   HAL_TID_CONTROL_GET_OPERATION_STATUS,
   HAL_TID_CONTROL_CLR_OPERATION_STATUS

} halTidControlMode_t;

typedef struct {
  halTidControlMode_t mode;    /* Control Mode: Timeslot or TDM Config     */
  
} halTidControl_t;

/*****************************************************************************
 * DEFINITION: Structure used for Memory tests on the CPU
 *****************************************************************************/
#define TOTAL_NO_TESTS      10

typedef struct {
    tuint *startaddr;  /*Points to beginning of memory block*/
    tlong size;        /*Size of the memory block*/
    tuint test_mask;   /*Indicates which test has to be carried out*/
    tuint test_result; /*Indicates result of the tests */
    tuint *FailedLocns[TOTAL_NO_TESTS];
    /*Gives a location where a particular test failed very first time*/
}memBlock_t;

/*****************************************************************************
 * TDM call table
 *****************************************************************************/
typedef struct halTdmApi_s {

  halReturn_t (*halTdmGetSizes)(tint *nbufs, halMemBuffer_t **bufs,  
                                halTdmInit_t *sizes);
                                
  halReturn_t (*halTdmInit)    (halTdmInit_t *cfg, tint nbufs, halMemBuffer_t *bufs);
  halReturn_t (*halTdmControl) (tint halId, halTdmControl_t *ctl);

} halTdmApi_t;
 
/******************************************************************************
 * HPI call table
 ******************************************************************************/
typedef struct halHpiApi_s {

  tint (*halHpiRead)    (tuint hpiOffset, tuint size, word *data);
  tint (*halHpiWrite)   (tuint hpiOffset, tuint size, word *data, tint mode);
  tint (*halHpiControl) (halHpiControl_t *ctl);
  
} halHpiApi_t;

/******************************************************************************
 * IPC call table
 ******************************************************************************/
typedef struct halIpcApi_s {

  tint (*halIpcControl) (halIpcControl_t *ctl);
  
} halIpcApi_t;

/******************************************************************************
 * Utopia call table
 ******************************************************************************/
typedef struct halUtopiaApi_s {

  halReturn_t (*halUtopiaGetSizes) (tint *nbufs, halMemBuffer_t **bufs, 
                                    halUtopiaSizes_t *sizes);
  halReturn_t (*halUtopiaInit)     (halUtopiaInit_t *cfg, tint nbufs, halMemBuffer_t *bufs);
  tint (*halUtopiaControl)  (tint halId, halUtopiaControl_t *ctl);
  tint (*halUtopiaSendCell) (tint halId, tuint channelId, halUtopiaData_t *data);
  
} halUtopiaApi_t;

/*******************************************************************************
 * Gmac call table
 *******************************************************************************/
typedef struct halGmacApi_s  {

  halReturn_t (*halGmacGetSizes)   (tint *nbufs, halMemBuffer_t **bufs,
                                    halGmacSizes_t *sizes);
  tint        (*halGmacAlloc)      (tint nbufs, halMemBuffer_t *bufs, 
                                    halGmacSizes_t *sizes);                                   
  halReturn_t (*halGmacInit)       (halGmacInit_t *cfg);
  halReturn_t (*halGmacControl)    (halGmacControl_t *ctl);
  halReturn_t (*halGmacSendPacket) (tint halId, halGmacData_t *data);

} halGmacApi_t;

/******************************************************************************
 * GPIO call table
 ******************************************************************************/
typedef struct halGpioApi_s {

  void (*halGpioInit) (void);
  bool (*halGpioTx) (halGpio_t *gpio);
  bool (*halGpioRx) (halGpio_t *gpio);

} halGpioApi_t;

/******************************************************************************
 * AIC call table
 ******************************************************************************/
typedef struct halAicApi_s  {
  
  /* Analog Codec Control */
  tint (*halAicControl) (tint timeslot, halCtrl_e cntrl, aicCtrl_t *aicCtrl);
  /* reads/writes value in AIC registers */
  /* returns non zero value if it reads otherwise must return 0 */
  tint (*halAicRegister) (aicRegCtrl_t *ctrl); 

  /* Reads Analog Codec configuration and parameters */
  tint (*halAicGet) (tint timeslot, halGetCtrl_e cntrl, aicGet_t *ctrl);

  /* Analog Codec Instance creation at Hardware Config message */
  halReturn_t (*halAicInitStart) (tuint portId, tuint aicType, tuint crstl_freq, tuint samp_freq);

  /* Analog Codec Gain */
  tint (*halAicGainCtrl) (tint timeslot, aicGainCtrl_t *ctrl, aicRet_t *ret);

  /* Analog Codec Gain Correction*/
  tint (*halGetAicTxGainCorrection) (tint tx_pga_gain);
  tint (*halGetAicRxGainCorrection) (tint rx_pga_gain);

} halAicApi_t;

/******************************************************************************
 * DMA call table
 ******************************************************************************/
typedef struct halDmaApi_s  {
  
  halReturn_t (*halStartTransfer)  (halGlobAddr_t dst_addr, halGlobAddr_t src_addr,
                      halSAE_t size, halMemMoveComplete_t call_back,
                      void *data, tuint *chnum_priority);
  halReturn_t (*halWaitTillDMATransferComplete)(tuint ch_num);
  halReturn_t (*halAbortDMATransfer)(tuint ch_num);
  
 } halDmaApi_t;
 
/*****************************************************************************
 * Dma API
 *****************************************************************************/
typedef struct halMemMove_s {
  halGlobAddr_t dest;
  halGlobAddr_t src;
  halSAE_t      size;
} halMemMove_t;

enum {
  HAL_MM_INVALID_HANDLE = -1,
  HAL_MM_COMPLETE,
  HAL_MM_IN_PROGRESS
 };
typedef struct halTidControlApi_s {

  halReturn_t (*halTidControlInit)    (void);
  halReturn_t (*halTidControlControl) (halTidControl_t *ctl, tint *retVal);

} halTidControlApi_t;

/*****************************************************************************
 * DEFINITION: The HAL API is defined by a structure of function calls. HAL
 *             creates an instance of this structure and populates it with
 *             the required function pointers. A pointer to this structure 
 *             is provided to the application to access the HAL functions.
 *****************************************************************************/
typedef struct halApi_s {
  /* Memory Initialization */
  halReturn_t (*halGetSizes) (tint *nbufs, halMemBuffer_t **bufs, 
                              halNewConfig_t *cfg);
  halReturn_t (*halNew)      (tint  nbufs, halMemBuffer_t *bufs, halNewConfig_t *cfg);

  /* Query peripheral status */
  halReturn_t (*halQuery) (halQuery_t *query);
  
  /* Starting the Peripherals */
  halReturn_t (*halControl)(halControl_t *ctl);

  /* Polling */
  bool (*halPoll) (tuint time, tuint activeChnls);

  /* Stats request */
  void (*halStatsRequest) (tuint registerBank, tuint offset, tint nRegs);

  /* TDM port */
  halTdmApi_t *tdm;

  /* HPI */
  halHpiApi_t *hpi;

  /* IPC */
  halIpcApi_t *ipc;

  /* PLL */
  tint (*halPllControl) (tuint multplier, tuint divider);

  /* Utopia */
  halUtopiaApi_t *utopia;
  
  /* GMAC */
  halGmacApi_t *gmac;

  /* GPIO */
  halGpioApi_t *gpio;

  /* Interrupt */
  halIntrLevel_t (*halIntrLevel) (halIntrLevel_t level);
  tint (*halExternalInt) (halExternalIntControl_t *ctl);
  halReturn_t (*halIntState) (halIntStateControl_t *istate);

  /* Timer */
  tulong (*halTimerControl) (tint timerId, halTimerControl_t *ctl);

  /* Watch dog timer */
  halReturn_t (*halWdTimerControl) (halWdTimerControl_t *ctl);

  /* Memory */
  tint (*halGlobMemMove) (tint nXfers, halMemMove_t *xfers, halMemMoveComplete_t complete, void *data, bool stallToCompletion);
  halReturn_t (*halLocalMemMove) (halLocAddr_t dest, halLocAddr_t src, halSAE_t size,
                           halMemMoveComplete_t complete, void *data);
  halGlobAddr_t (*halMapLocalToGlobal) (halLocAddr_t addr);
  halGlobAddr_t (*halFormGlobalAddr) (halLocAddr_t addr, tint globalId);
  halReturn_t (*halGlobMemSet) (halGlobAddr_t addr, tulong sizeBytes, tulong v, tint elemSizeBytes);
  tint (*halMMStatus) (tint handle);
  

  /* Analog Codec Control */
  halAicApi_t *aic;
  

  /* Host to Driver control */
  tint (*halMessage) (tint msgId, void *msg, tint (*msgReply)(void *, tint, tint, tint), 
                      tint header_size, tint trans_id);

  /* Memory Config */
  halReturn_t (*halMemConfig) (halMemConfig_t *halMemCfg);

  /* Cache Control */
  void (*halCacheControl) (halCacheControl_t *cacheControl);

  /* DMA control */
  halDmaApi_t *dma; 

  /* TID control */
  halTidControlApi_t *tidControl; 

  halReturn_t (*halSrioInit)(halSrioInitCfg_t *ctl);
  
} halApi_t;
  
/* For now declare an external HAL type */
extern halApi_t hal;

#endif /* _HAL_H */


