/*****************************************************************************
 * FILE PURPOSE: Hardware Abstraction Layer (HAL) initialization
 *****************************************************************************
 * FILE NAME: halinit.c
 * 
 * DESCRIPTION: Provides HAL API functions halGetSizes and halNew
 *
 * TABS: None
 *
 * Copyright (C) 2002, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include "hal.h"
#include "csl_gg.h" /* File renamed to avoid conflict with existing csl.h
                      * (Part of BIOS).Cleaner to have different filename 
                      *  than setting vpath in makefile */
#include "gghwcfg.h"
#include "types.h"
#include "halloc.h"

#if GG_TDM_PORTS_USED
#include "tdm_dma.h"
#endif

#include "cslintc.h"
#include "cslcore.h"
#ifdef GG_HAL_INCLUDE_UTOPIA
#include "ul2loc.h"
#endif

#ifdef HAL_INCLUDE_AIC
#include "aic.h"
#endif

#include "cpuutils.h"
#include "halport.h"
#include <stdlib.h>
#include <string.h>


/*TSIP_LLD and (HAL TDM or HAL TDM SHARED) are mutually exclusive*/
#if ( ( defined(GG_HAL_INCLUDE_TDM) || defined(GG_HAL_SHARED_TDM_RESOURCES) ) && ( defined (TSIP_LLD) ) )
#error TSIP_LLD and (GG_HAL_INCLUDE_TDM or GG_HAL_SHARED_TDM_RESOURCES) are mutually exclusive
#endif

static halMemBuffer_t memBuf[HAL_NBUFS] = {
   /* class                                 log2align       size        volat       base  */
  HAL_MEM_CLASS_INTERNAL,          0,           0,          FALSE,     NULL,
  HAL_MEM_CLASS_INTERNAL,          0,           0,          FALSE,     NULL,
  HAL_MEM_CLASS_INTERNAL,          0,           0,          FALSE,     NULL,
  HAL_MEM_CLASS_INTERNAL,          0,           0,          FALSE,     NULL
};  

/******************************************************************************
 * FUNCTION PURPOSE: Returns buffer requirements for HAL
 ******************************************************************************
 * DESCRIPTION: Returns the memory requirements for HAL based on the 
 *              information provided in the sizeConfig structure.
 *
 *              Returns only the requirements for halContext with pointers
 *              to peripheral contexts. The peripheral contexts are not
 *              allocated (they are done when they are used).
 *
 *****************************************************************************/
halReturn_t halGetSizes (tint *nbufs, halMemBuffer_t **bufs,
                         halNewConfig_t *cfg)
{
  tuint memSize;

 #ifdef TSIP_LLD 
  tsipSizeInfo_t sizeCfg;
  int   sizes[tsip_N_BUFS_SHARED];
  int   aligns[tsip_N_BUFS_SHARED];

  sizeCfg = cfg->tsipSizeCfg;
 #endif

  /* Determine the size of the heap buffer. */
  memSize = sizeof(halContext_t);

  HAL_ALIGN_STRUCT(memSize);

#if ( defined(GG_HAL_INCLUDE_TDM) || defined(GG_HAL_INCLUDE_ITDM) )
  memSize += cfg->nTdmPorts * sizeof (void *);
  HAL_ALIGN_STRUCT(memSize);
#endif

#ifdef GG_HAL_SHARED_TDM_RESOURCES
  memSize += sizeof (void *);
  HAL_ALIGN_STRUCT(memSize);
  memSize += (*halInitCtable.halTdmShGetHeapSize)();
  HAL_ALIGN_STRUCT(memSize);
#endif  /* GG_HAL_SHARED_TDM_RESOURCES */

#ifdef TSIP_LLD
  Tsip_getBufferReqShared (&sizeCfg, sizes, aligns);

  memSize += sizes[TSIP_BUFFNUM_INSTANCE_DRIVER];
  HAL_ALIGN_STRUCT(memSize);

  memSize += sizes[TSIP_BUFFNUM_HEAP_TIMESLOT];
  HAL_ALIGN_STRUCT(memSize);

  memSize += sizes[TSIP_BUFFNUM_HEAP_SHARED];
  HAL_ALIGN_STRUCT(memSize);
#endif



#ifdef GG_HAL_INCLUDE_UTOPIA
  /* Memory required by halContext Pkt pointers */
  memSize += cfg->nPktPorts * sizeof (void *);

  HAL_ALIGN_STRUCT(memSize);
#endif

#ifdef GG_HAL_INCLUDE_AIC
  /* Memory required by halContext Aic pointers */
  memSize += cfg->nAicPorts * sizeof (void *);

  HAL_ALIGN_STRUCT(memSize);
#endif

#if ((defined(GG_HAL_INCLUDE_GMAC)) || (defined(GG_HAL_INCLUDE_EMAC)) || (defined(GG_HAL_INCLUDE_3PSW)))
  /* Memory required by halContext Gmac pointers */
  memSize += cfg->nGmacPorts * sizeof (void *);
  HAL_ALIGN_STRUCT(memSize);
#endif

#ifdef GG_HAL_INCLUDE_DMA_CONTEXT
  /* Get the DMA requirements since the dma must be setup at halNew */
  memSize += (*halInitCtable.halDmaGetSizes) ();
  HAL_ALIGN_STRUCT(memSize);
#endif

#ifdef GG_HAL_INCLUDE_TID_CONTROL
  /* Get the DMA requirements since the dma must be setup at halNew */
  memSize += (*halInitCtable.halTidControlGetSizes) ();
  HAL_ALIGN_STRUCT(memSize);
#endif

  /* Complete the 1st buffer. The first buffer is always heap, but it will
   * return a size request of 0 if none is needed */
  memBuf[HAL_HEAP_BUFN].class     = HAL_MEM_CLASS_INTERNAL; 
  memBuf[HAL_HEAP_BUFN].log2align = 0;    /* no alignment requirements */    
  memBuf[HAL_HEAP_BUFN].size      = memSize;
  memBuf[HAL_HEAP_BUFN].volat     = FALSE;
  memBuf[HAL_HEAP_BUFN].base      = NULL;

#ifdef TSIP_LLD 
  memBuf[HAL_LLD_TDM_TX_BUFN].class     = HAL_MEM_CLASS_INTERNAL; 
  memBuf[HAL_LLD_TDM_TX_BUFN].log2align = 4;        
  memBuf[HAL_LLD_TDM_TX_BUFN].size      = sizes[TSIP_BUFFNUM_TX_DMA_BUFFER];
  memBuf[HAL_LLD_TDM_TX_BUFN].volat     = FALSE;
  memBuf[HAL_LLD_TDM_TX_BUFN].base      = NULL;

  memBuf[HAL_LLD_TDM_RX_BUFN].class     = HAL_MEM_CLASS_INTERNAL; 
  memBuf[HAL_LLD_TDM_RX_BUFN].log2align = 4;        
  memBuf[HAL_LLD_TDM_RX_BUFN].size      = sizes[TSIP_BUFFNUM_RX_DMA_BUFFER];
  memBuf[HAL_LLD_TDM_RX_BUFN].volat     = FALSE;
  memBuf[HAL_LLD_TDM_RX_BUFN].base      = NULL;
#endif
  
  /* Return the buffer information */
  *nbufs = HAL_HEAP_NBUFS;
  *bufs  = memBuf;

  return (HAL_SUCCESS);

} /* halGetSizes */


#ifdef GG_HAL_INCLUDE_UTOPIA
/*******************************************************************************
 * FUNCTION PURPOSE: Get utopia memory requirements
 *******************************************************************************
 * DESCRIPTION: Returns context, tx and rx dma buffer requirements
 *******************************************************************************/
halReturn_t halUtopiaGetSizes (tint *nbufs, halMemBuffer_t **bufs, halUtopiaSizes_t *sizeCfg)
{
  tuint align;
  tuint size;

  *nbufs = HAL_UT_NBUFS;

  /* The heap requirements */
  memBuf[HAL_UT_HEAP_BUFN].class     = HAL_MEM_CLASS_INTERNAL;
  memBuf[HAL_UT_HEAP_BUFN].log2align = GG_UTOPIA_HEAP_BUF_ALIGN;
  memBuf[HAL_UT_HEAP_BUFN].size      = (*halInitCtable.halPktGetHeapSize)(sizeCfg);
  memBuf[HAL_UT_HEAP_BUFN].volat     = FALSE;
  memBuf[HAL_UT_HEAP_BUFN].base      = NULL;

  /* Transmit buffer requirements */
  (*halInitCtable.halPktGetTxBufInfo) (sizeCfg, &align, &size);

  memBuf[HAL_UT_TX_BUFN].class       = HAL_MEM_CLASS_INTERNAL;
  memBuf[HAL_UT_TX_BUFN].log2align   = align;
  memBuf[HAL_UT_TX_BUFN].size        = size;
  memBuf[HAL_UT_TX_BUFN].volat       = FALSE;
  memBuf[HAL_UT_TX_BUFN].base        = NULL;

  /* Receive buffer requirements */
  (*halInitCtable.halPktGetRxBufInfo) (sizeCfg, &align, &size);

  memBuf[HAL_UT_RX_BUFN].class       = HAL_MEM_CLASS_INTERNAL;
  memBuf[HAL_UT_RX_BUFN].log2align   = align;
  memBuf[HAL_UT_RX_BUFN].size        = size;
  memBuf[HAL_UT_RX_BUFN].volat       = FALSE;
  memBuf[HAL_UT_RX_BUFN].base        = NULL;

  /* Catch all buffer requirements */
  if (halInitCtable.halPktGetCaBufInfo)  {

    (*halInitCtable.halPktGetCaBufInfo) (sizeCfg, &align, &size);

  }  else  {

     align  = 0;
     size   = 0;
    *nbufs -= 1;

  }

  memBuf[HAL_UT_CA_BUFN].class       = HAL_MEM_CLASS_INTERNAL;
  memBuf[HAL_UT_CA_BUFN].log2align   = align;
  memBuf[HAL_UT_CA_BUFN].size        = size;
  memBuf[HAL_UT_CA_BUFN].volat       = FALSE;
  memBuf[HAL_UT_CA_BUFN].base        = NULL;

  *bufs  = memBuf;

  return (HAL_SUCCESS);


} /* halUtopiaGetSize */
#endif

#if ((defined(GG_HAL_INCLUDE_GMAC)) || (defined(GG_HAL_INCLUDE_EMAC)) || (defined(GG_HAL_INCLUDE_3PSW)))
/*********************************************************************************
 * FUNCTION PURPOSE: Get Mac memory requirements
 *********************************************************************************
 * DESCRIPTION: Returns context. tx and rx dma buffer requirements 
 *********************************************************************************/
halReturn_t halGmacGetSizes (tint *nbufs, halMemBuffer_t **bufs, halGmacSizes_t *sizeCfg)
{
  tuint  align;
  tulong size;

  /* The heap requirements */
  memBuf[HAL_GMAC_HEAP_BUFN].class     = HAL_MEM_CLASS_INTERNAL;
  memBuf[HAL_GMAC_HEAP_BUFN].log2align = GG_GMAC_HEAP_BUF_ALIGN;
  memBuf[HAL_GMAC_HEAP_BUFN].size      = (*halInitCtable.halGmacGetHeapSize)(sizeCfg);
  memBuf[HAL_GMAC_HEAP_BUFN].volat     = FALSE;
  memBuf[HAL_GMAC_HEAP_BUFN].base      = NULL;

  /* Transmit buffer requirements */
  (*halInitCtable.halGmacGetTxBufInfo) (sizeCfg, &align, &size);

  memBuf[HAL_GMAC_TX_BUFN].class       = HAL_MEM_CLASS_INTERNAL;
  memBuf[HAL_GMAC_TX_BUFN].log2align   = align;
  memBuf[HAL_GMAC_TX_BUFN].size        = size;
  memBuf[HAL_GMAC_TX_BUFN].volat       = FALSE;
  memBuf[HAL_GMAC_TX_BUFN].base        = NULL;

  /* Receive buffer requirements */
  (*halInitCtable.halGmacGetRxBufInfo) (sizeCfg, &align, &size);

  memBuf[HAL_GMAC_RX_BUFN].class       = HAL_MEM_CLASS_INTERNAL;
  memBuf[HAL_GMAC_RX_BUFN].log2align   = align;
  memBuf[HAL_GMAC_RX_BUFN].size        = size;
  memBuf[HAL_GMAC_RX_BUFN].volat       = FALSE;
  memBuf[HAL_GMAC_RX_BUFN].base        = NULL;

  /* Assembly buffer requirements */
  memBuf[HAL_GMAC_ASSEM_BUFN].class     = HAL_MEM_CLASS_INTERNAL;
  memBuf[HAL_GMAC_ASSEM_BUFN].log2align = 1;
  memBuf[HAL_GMAC_ASSEM_BUFN].size      = sizeCfg->maxRxPacketSizeWords;
  memBuf[HAL_GMAC_ASSEM_BUFN].volat     = FALSE;
  memBuf[HAL_GMAC_ASSEM_BUFN].base      = NULL;


  *nbufs = HAL_GMAC_NBUFS;
  *bufs  = memBuf;

  return (HAL_SUCCESS);

} /* halGmacGetSizes */
#endif

#if ((defined(GG_HAL_INCLUDE_GMAC)) || (defined(GG_HAL_INCLUDE_EMAC)) || (defined(GG_HAL_INCLUDE_3PSW)))
/**********************************************************************************
 * FUNCTION PURPOSE: Allocate memory for a single mac port
 **********************************************************************************
 * DESCRIPTION: Memory for a mac port is allocated. The halId for this memory
 *              is returned. The memory is not associated with a physical port.
 *              The halId for the memory is returned. A negative value is
 *              returned on error.
 **********************************************************************************/
tint halGmacAlloc (tint nbufs, halMemBuffer_t *bufs, halGmacSizes_t *sizes)

{
  tint          i;
  halReturn_t   ret;
  void         *gmac;

  /* The halId is the index of ports currently allocated */
  for (i = 0; i < halContext->nGmacPorts; i++)  {
    gmac = halContext->gmacPorts[i];
    if (gmac == NULL)
      break;
  }

  if (i == halContext->nGmacPorts)
    return ((tint)HAL_GMAC_NO_PORTS);


  halContext->gmacPorts[i] =
    (*halInitCtable.halGmacBufAlloc) (bufs[HAL_GMAC_HEAP_BUFN].base, sizes,
                                     bufs[HAL_GMAC_TX_BUFN].base, bufs[HAL_GMAC_TX_BUFN].size,
                                     bufs[HAL_GMAC_RX_BUFN].base, bufs[HAL_GMAC_RX_BUFN].size,
                                     bufs[HAL_GMAC_ASSEM_BUFN].base, &ret);

  if (ret != HAL_SUCCESS)
    return ((tint)-ret);

  return (i);

} /* halGmacAlloc */


#endif

#ifdef GG_HAL_INCLUDE_UTOPIA
/**********************************************************************************
 * FUNCTION PURPOSE: Allocate memory for and initialize a utopia instance
 **********************************************************************************
 * DESCRIPTION: Creates a utopia instance
 **********************************************************************************/
halReturn_t halUtopiaInit (halUtopiaInit_t *cfg, tint nbufs, halMemBuffer_t *bufs)
{
  tint         i;
  halReturn_t  ret;
  void        *pkt;

  /* The halId is the index of ports currently allocated */
  for (i = 0; i < halContext->nPktPorts; i++)  {
    pkt = halContext->pktPorts[i];
    if (pkt == NULL)
      break;
  }

  if (i == halContext->nPktPorts)
    return (HAL_PKT_NO_PORTS);

  cfg->halId = i;

  halContext->pktPorts[i] = 
    (*halInitCtable.halPktAllocAndInit) (bufs[HAL_UT_HEAP_BUFN].base, cfg, 
                               bufs[HAL_UT_TX_BUFN].base, bufs[HAL_UT_TX_BUFN].size,
                               bufs[HAL_UT_RX_BUFN].base, bufs[HAL_UT_RX_BUFN].size,
                               bufs[HAL_UT_CA_BUFN].base, bufs[HAL_UT_CA_BUFN].size,
                               &ret);
  return (ret);

} /* halUtopiaInit */
#endif

#if ( defined(GG_HAL_INCLUDE_TDM) || defined(GG_HAL_INCLUDE_ITDM) )
/****************************************************************************
 * FUNCTION PURPOSE: Get TDM size requirements
 ****************************************************************************
 * DESCRIPTION: Returns context, tx and rx dma buffer requirements
 ****************************************************************************/
halReturn_t halTdmGetSizes (tint *nbufs, halMemBuffer_t **bufs, halTdmInit_t *tdmInit)
{
  tuint align;
  tuint size;

  /* The heap requirements */
  memBuf[HAL_TDM_HEAP_BUFN].class     = HAL_MEM_CLASS_INTERNAL;
  memBuf[HAL_TDM_HEAP_BUFN].log2align = 3;
  memBuf[HAL_TDM_HEAP_BUFN].size      = (*halInitCtable.halTdmGetHeapSize)(tdmInit);
  memBuf[HAL_TDM_HEAP_BUFN].volat     = FALSE;
  memBuf[HAL_TDM_HEAP_BUFN].base      = NULL;

  /* Transmit buffer requirements */
  (*halInitCtable.halTdmGetTxDmaBufInfo) (tdmInit, &align, &size);

  memBuf[HAL_TDM_TX_BUFN].class       = HAL_MEM_CLASS_INTERNAL;
  memBuf[HAL_TDM_TX_BUFN].log2align   = align;
  memBuf[HAL_TDM_TX_BUFN].size        = size;
  memBuf[HAL_TDM_TX_BUFN].volat       = FALSE;
  memBuf[HAL_TDM_TX_BUFN].base        = NULL;

  /* Receive buffer requirements */
  (*halInitCtable.halTdmGetRxDmaBufInfo) (tdmInit, &align, &size);
  memBuf[HAL_TDM_RX_BUFN].class       = HAL_MEM_CLASS_INTERNAL;
  memBuf[HAL_TDM_RX_BUFN].log2align   = align;
  memBuf[HAL_TDM_RX_BUFN].size        = size;
  memBuf[HAL_TDM_RX_BUFN].volat       = FALSE;
  memBuf[HAL_TDM_RX_BUFN].base        = NULL;

  *nbufs = HAL_TDM_NBUFS;
  *bufs  = memBuf;

  return (HAL_SUCCESS);

} /* halTdmGetSizes */

/******************************************************************************
 * FUNCTION PURPOSE: Allocate memory for and initialize a TDM instance
 ******************************************************************************
 * DESCRIPTION: Creates a tdm instance
 ******************************************************************************/
halReturn_t halTdmInit (halTdmInit_t *cfg, tint nbufs, halMemBuffer_t *bufs)
{
  tint          i;
  halReturn_t   ret;
  void         *tdm;

  /* The halID is the index of the port allocated */
  for (i = 0; i < halContext->nTdmPorts; i++)  {
    tdm = halContext->tdmPorts[i];
    if (tdm == NULL)
      break;
  }

  if (i == halContext->nTdmPorts)
    return (HAL_TDM_NO_PORTS);

  cfg->halId = i;

  memset (bufs[HAL_TDM_TX_BUFN].base, 0x7F7F, bufs[HAL_TDM_TX_BUFN].size);
  halContext->tdmPorts[i] = 
    (*halInitCtable.halTdmAllocAndInit) (bufs[HAL_TDM_HEAP_BUFN].base, bufs[HAL_HEAP_BUFN].size, cfg, 
                               bufs[HAL_TDM_TX_BUFN].base, bufs[HAL_TDM_TX_BUFN].size,
                               bufs[HAL_TDM_RX_BUFN].base, bufs[HAL_TDM_RX_BUFN].size,
                               &ret);
  return (ret);
} /* halTdmInit */

#endif


/*****************************************************************************
 * FUNCTION PURPOSE: Create HAL
 *****************************************************************************
 * DESCRIPTION: Creates and initializes new hal structures.
 *****************************************************************************/
halReturn_t halNew (tint nbufs, halMemBuffer_t *bufs, halNewConfig_t *cfg)
{
  word   *bufBase;
  tint    i;
  tuint   used = 0;

  extern void siuDebugInfo (tuint moduleID, tuint msgType, tuint messageCode, 
                       tuint msgLength, tuint *messageData);


#ifdef TSIP_LLD 
  void* bases [tsip_N_BUFS_SHARED];
  int   sizes [tsip_N_BUFS_SHARED];
  int   aligns[tsip_N_BUFS_SHARED];
  tsipConfig_t tsipConfig;
  tsipSizeInfo_t sizeCfg;

  memset(&tsipConfig, 0, sizeof(tsipConfig_t));
  memset(&sizeCfg,    0, sizeof(tsipSizeInfo_t));
 
  tsipConfig.sizeCfg = &sizeCfg;
  sizeCfg            = cfg->tsipSizeCfg;
#endif
  
  /* Put chip peripherals into a known state */
  cslReset();

  /* Allocate and initialize heap amoung the parts of hal */
  bufBase = (word *)bufs[HAL_HEAP_BUFN].base;

  /***************************************************************************
   *       Heap memory required by halContext                                *
   ***************************************************************************/
  /* The halContext strucure itself */
  if (bufs[HAL_HEAP_BUFN].size < sizeof (halContext_t))
    return (HAL_FAIL);
  halContext = (halContext_t *)bufBase;
  used = sizeof (halContext_t);

#ifdef GG_HAL_INCLUDE_UTOPIA
  /* utopia ports */
  halContext->nPktPorts = cfg->nPktPorts;
  halContext->pktPorts = (void **)&(bufBase[used]);
  used += cfg->nPktPorts * sizeof (void *);
  for (i = 0; i < cfg->nPktPorts; i++)
    halContext->pktPorts[i] = NULL;
#endif

#if (defined(GG_HAL_INCLUDE_TDM) || defined(GG_HAL_INCLUDE_ITDM))
  /* tdm ports */
  halContext->tdmPorts = (void **)&(bufBase[used]);
  halContext->nTdmPorts = cfg->nTdmPorts;
  used += cfg->nTdmPorts * sizeof (void *);
  for (i = 0; i < cfg->nTdmPorts; i++)
    halContext->tdmPorts[i] = NULL;
#endif

#ifdef TSIP_LLD
  /* tdm ports */
  halContext->nTdmPorts = cfg->nTdmPorts;
  for (i = 0; i < cfg->nTdmPorts; i++)
    halContext->tsipLldContext.tsipLldPort[i] = NULL;
#endif

#ifdef GG_HAL_SHARED_TDM_RESOURCES
  halContext->tdmShared = (void *)&(bufBase[used]);
  used += (*halInitCtable.halTdmShGetHeapSize)();
  (*halInitCtable.halTdmShInit)(halContext->tdmShared);
  HAL_ALIGN_STRUCT(used);
#endif

#ifdef TSIP_LLD
  Tsip_getBufferReqShared (&sizeCfg, sizes, aligns);

  /* TSIP buffers that are allocated from HAL HEAP buffer*/
  bases[TSIP_BUFFNUM_INSTANCE_DRIVER] = (void *)&(bufBase[used]);
  used += sizes[TSIP_BUFFNUM_INSTANCE_DRIVER];
  HAL_ALIGN_STRUCT(used);

  bases[TSIP_BUFFNUM_HEAP_SHARED] = (void *)&(bufBase[used]);
  used += sizes[TSIP_BUFFNUM_HEAP_SHARED];
  HAL_ALIGN_STRUCT(used);

  bases[TSIP_BUFFNUM_HEAP_TIMESLOT] = (void *)&(bufBase[used]);
  used += sizes[TSIP_BUFFNUM_HEAP_TIMESLOT];
  HAL_ALIGN_STRUCT(used);

  bases[TSIP_BUFFNUM_TX_DMA_BUFFER] = bufs[HAL_LLD_TDM_TX_BUFN].base;
  bases[TSIP_BUFFNUM_RX_DMA_BUFFER] = bufs[HAL_LLD_TDM_RX_BUFN].base;

  Tsip_createShared(&tsipConfig , bases, &(halContext->tsipLldContext.tsipLldDrv));  
#endif


#ifdef GG_HAL_INCLUDE_AIC
  /* aic ports */
  halContext->aicPorts = (void **)&(bufBase[used]);
  halContext.nAicPorts = cfg->nAicPorts;
  used += cfg->nAicPorts * sizeof (void *);
  for (i = 0; i < cfg->nAicPorts; i++)
    halContext->aicPorts[i] = NULL;
#endif

#if ((defined(GG_HAL_INCLUDE_GMAC)) || (defined(GG_HAL_INCLUDE_EMAC)) || (defined(GG_HAL_INCLUDE_3PSW)))
  /* gmac ports */
  halContext->gmacPorts = (void **)&(bufBase[used]);
  halContext->nGmacPorts = cfg->nGmacPorts;
  used += cfg->nGmacPorts * sizeof (void *);
  for (i = 0; i < cfg->nGmacPorts; i++)
    halContext->gmacPorts[i] = NULL;
#endif

#ifdef GG_HAL_INCLUDE_DMA_CONTEXT
  halContext->dmaContext = (void *)&(bufBase[used]);
  used += (*halInitCtable.halDmaGetSizes) ();
  (*halInitCtable.halDmaInit) (halContext->dmaContext, !(whoAmI()));
#endif
  
#ifdef GG_HAL_INCLUDE_TID_CONTROL
  halContext->tidControlContext = (void *)&(bufBase[used]);
  used += (*halInitCtable.halTidControlGetSizes) ();
  (*halInitCtable.halTidControlAllocInit) (halContext->tidControlContext, !(whoAmI()));
#endif

  /*************************************************************************** 
   *      Return the core number
   ***************************************************************************/
  cfg->coreId = whoAmI();  

  /***************************************************************************
   * Return the chip revision
   ***************************************************************************/
  cfg->revId = cslChipRevision();

#ifndef C_ISR /* DIAG */
  halContext->debugInfo=siuDebugInfo;
#endif  
  return (HAL_SUCCESS);

} /* halNew */



   
/* Nothing Past This Point */

