/************************************************************************
:*                                                                      *
:*     @ Copyright (C) 2009  Texas Instruments Incorporated.            *
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

/******************************************************************************
 * FILE PURPOSE: Common ECO creation/relocation routines.
 ******************************************************************************
 * FILE NAME:   ecorelocutil.c
 ******************************************************************************/
#include <stdio.h>
#include <stddef.h> /* for offsetof macro usage */
#include "types.h"
#include "memmgm.h"

/* Macro to verify requested buffer alignment */
#define eco_chk_base_l2(base,l2a)    \
        (((tulong)(base)&(((l2a)==TYP_TULONG_SIZE)?~0UL:~(~0UL<<(l2a))))==0UL)

/******************************************************************************
 * DESCRIPTION: This function relocates the requested buffers in XXX. 
 ******************************************************************************
 *  void ecoNewErrCheckUtil (
 *    void  **ecoInst, - pointer to location where eco instance pointer is stored (INPUT)
 *    tuint    nBufs,  - number of provided buffers (INPUT)
 *    tuint  ecoNbufs, - number of expected buffers (requested buffers) (INPUT)
 *    memBuffer_t *bufs - list of descriptors pf provided buffers (INPUT)
 *    memBuffer_t *bufReq - list of descriptors of requested buffers (INPUT)
 *    word *bufAddrs[]   - list of buffer base addresses (OUTPUT)
 *
 * RETURN VALUE: ECO_NOERR in case of no error
 *               ECO_ERR_INST_NON_NULL if eco instance pointer is NULL
 *               ECO_ERR_BAD_NBUFS if number of provided buffers is not as requested
 *               ECO_ERR_BAD_BUF_SIZE if a buffer size is smaller than requested
 *               ECO_ERR_BAD_LOG2ALIGN if buffer base is not aligned as requested.
 *
 * Note:         Error checking should be minimal in this API due
 *               to the real-time usage.
 *
 *****************************************************************************/
tint ecoNewErrCheckUtil(void *ecoInst, tuint nBufs, tuint ecoNbufs, 
                      memBuffer_t *bufs, memBuffer_t *bufReq, word *bufAddrs[])
{
  tuint i;

  if (ecoInst != NULL) {
    return(ECO_ERR_INST_NON_NULL);
  }

  if (nBufs != ecoNbufs) {
    return(ECO_ERR_BAD_NBUFS);
  }

  for (i=0; i < nBufs; i++, bufs++, bufReq++)
  {
	/* Check buffer base address: this test can be more complex and CPU dependent! */
    if(bufs->base == NULL) {
      return(ECO_ERR_BAD_BUF_ADDR);
    }
	/* Check buffer size */
    if (bufs->size < bufReq->size) {
	    return(ECO_ERR_BAD_BUF_SIZE);
    }
    /* Check alignment */
    if (!eco_chk_base_l2 (bufs->base, bufReq->log2align))
    {
      return(ECO_ERR_BAD_ALIGN);
    }
    bufAddrs[i] = (word *) bufs->base; /* assign new buffer value */
  }
  return(ECO_NOERR);
}
/******************************************************************************
 * DESCRIPTION: This function assigns ECO buffers pointers.
 *				Generality is achieved by providing the list of offsets to buffer pointer
 *              locations. Assignment is done if provided buffer pointer is not NULL.
 ******************************************************************************
 *  void ecoAssignBufs (
 *    void  **ecoInst, - pointer to location where eco instance pointer is stored (OUTPUT)
 *    tuint    nBufs,  - number of provided buffers (INPUT)
 *    word *bufAddrs[]  - list of buffer base addresses (INPUT)
 *    tuint *bufOffsets - offsets to locations of basic buffer pointers (instance offsets) (INPUT)
 *
 * RETURN VALUE: None
 *
 *****************************************************************************/
void ecoAssignBufs(void **ecoInst, tuint nBufs, word *bufAdrs[CGEN_ASSIST_RESTRICT], 
                   tuint *CGEN_ASSIST_RESTRICT bufOffsets)
{
  word *ptr;
  word **ptrloc;
  int  i;

  if (bufAdrs[0]) {
    *ecoInst = (word *)bufAdrs[0];
  }

  ptr  = (word *) *ecoInst;

  for(i = 1; i < nBufs; i++) {
    if (bufAdrs[i]) {
      ptrloc = (word **)&ptr[bufOffsets[i-1]];
      *ptrloc = bufAdrs[i];
    }
  }
}
/******************************************************************************
 * DESCRIPTION: This function creates list of ECO buffers addresses.
 *				Generality is achieved by providing the list of offsets to buffer pointer
 *              locations. 
 ******************************************************************************
 *  void ecoGetBufs (
 *    void  **ecoInst, - pointer to location where eco instance pointer is stored (INPUT)
 *    tuint    nBufs,  - number of provided buffers (INPUT)
 *    word *bufAddrs[]  - list of buffer base addresses (INPUT)
 *    tuint *bufOffsets - offsets to locations of basic buffer pointers (instance offsets) (OUTPUT)
 *
 * RETURN VALUE: None
 *
 *****************************************************************************/
void ecoGetBufs(void **ecoInst, tuint nBufs, word *bufAdrs[CGEN_ASSIST_RESTRICT], 
                tuint *CGEN_ASSIST_RESTRICT bufOffsets)
{
  int i;
  word *ptr;
  word **ptrloc;

  bufAdrs[0] = *ecoInst;
  ptr        = (word *)*ecoInst;

  for(i = 1; i < nBufs; i++) {
    ptrloc     = (word **)&ptr[bufOffsets[i-1]];
    bufAdrs[i] = *ptrloc;
  }
}
/* nothing past this point  */
