/******************************************************************************
 * FILE PURPOSE: Multiple heap manager.
 ******************************************************************************
 * FILE NAME:   mhm.c
 *
 * DESCRIPTION: Contains routines for multiple heap management. These functions
 *              may be used in allocating memory from several independent
 *              memory pools.
 *
 *              NOTE: Current implementation does NOT provide deallocation!
 *
 * TABS: NONE
 *
 * $Id: mhm.c 1.1 1997/06/25 11:40:55 BOGDANK Exp $
 *
 * REVISION HISTORY:
 *
 * $Log: mhm.c $
 * Revision 1.1  1997/06/25 11:40:55  BOGDANK
 * Initial revision
 *
 * (C) Copyright 1997, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include <stdlib.h>
#include <string.h>

#include "types.h"

#include "mhm.h"
#include "mhmport.h"

/* Heap header that is placed at the beginning of a memory pool. */
typedef struct {
  uint  total_size;   /* total number of words in a memory pool */
  uint  used_size;    /* index of the first available word */
} mhmHead_t;

/* The minimum and maximum number of words in a memory pool. Align gap and
 *  maximum size value are specified in mhmport.h. */
#define MHM_MIN_POOLSIZE  (sizeof(mhmHead_t)+MHM_MAX_ALIGNGAP+1)
#define MHM_MAX_POOLSIZE  MHM_MAX_SIZEVAL

/******************************************************************************
 * FUNCTION PURPOSE: Create and initialize a heap.
 ******************************************************************************
 * DESCRIPTION: Creates and initializes a heap. If successful returns a heap
 *              handle, otherwise returns NULL.
 *
 *  void *mhmCreate (
 *    void *base,         - base address of a memory pool (!= NULL)
 *    uint size,          - number of words within the memory pool
 *    word initPattern)   - initializing pattern for the heap
 *
 *  RETURN VALUE: A heap handle, or NULL in case of error.
 *
 *****************************************************************************/

void *mhmCreate (void *base, uint size, word initPattern)
{
  uint      used_size;
  mhmHead_t *head;
  word      *w_base = (word *)base;
  word      *block;

  /* initialize the raw space */
  memset (w_base, initPattern, size);
  
  /* In the first part we align the base address of the memory pool */
  used_size = 0;
  while (((mhmAddress_t)w_base & MHM_ALIGN_MASK) != 0) { /* align base address */
    w_base++; used_size++;
  }
  if (size < used_size+MHM_MIN_POOLSIZE || size-used_size > MHM_MAX_POOLSIZE ||
      base == NULL)
    return(NULL);

  /* Skip the pool alignment and start fresh. Do not forget to reset used size. */
  head = (mhmHead_t*)w_base;
  head->total_size = size - used_size;  /* record total heap size */

  used_size = sizeof(mhmHead_t);    /* reserve space for the header */
  block     = &w_base[used_size];   /* skip the header */
  while (((mhmAddress_t)(block++) & MHM_ALIGN_MASK) != 0) {  /* align the data block */
    used_size++;
  }
  head->used_size = used_size;    /* record an index to empty space */
  return(w_base);                 /* return heap handle */
} /* mhmCreate */

/******************************************************************************
 * FUNCTION PURPOSE: Allocate a block of memory from a heap.
 ******************************************************************************
 * DESCRIPTION: Allocates a block of memory from a heap. Block size must be
 *              bigger than zero. It returns a pointer to block start or a
 *              NULL if unsuccessful.
 *
 *  void *mhmAlloc (
 *    void *handle,     - heap handle obtained in mhmCreate()
 *    uint size)        - number of words to allocate for the block
 *
 *  RETURN VALUE: A pointer to block start, or NULL when unsuccessful.
 *
 *****************************************************************************/

void *mhmAlloc (void *handle, uint size)
{
  uint      left;
  mhmHead_t *head = (mhmHead_t*)handle;
  word      *w_base = (word *)handle;
  word      *block;

  if (size == 0 || size > MHM_MAX_SIZE || head->total_size <= head->used_size)
    return(NULL);

  while ((size & MHM_ALIGN_MASK) != 0)  /* align block to a proper boundary */
    size++;

  left = head->total_size - head->used_size;
  if (left < (uint)size)      /* do we have space for this block? */
    return(NULL);
  block = &w_base[head->used_size];
  head->used_size += (uint)size;      /* increase the used space */
  return(block);                      /* return a pointer to start */
} /* mhmAlloc */

/******************************************************************************
 * FUNCTION PURPOSE: Deallocate a memory block from a heap.
 ******************************************************************************
 * DESCRIPTION: Deallocates a memory block from a heap.
 *
 *  void mhmFree (
 *    void *handle,     - a heap handle
 *    void *base)       - base address of a block
 *
 *****************************************************************************/

void mhmFree (void *handle, void *base)
{
  return;           /* currently NOT supported */
} /* mhmFree */
/* nothing past this point */
