#ifndef __FRACT_H
#define __FRACT_H
/******************************************************************************
 * FILE PURPOSE: Fractional type macros (C54x version).
 ******************************************************************************
 * FILE NAME:   fract.h
 *
 * DESCRIPTION: Contains macros for manipulation of fractional types.
 *
 *              NOTE: PRIVATE constants and macros may not exist in all
 *                    implementations. Hence, they should never be used
 *                    explicitely in the source code.
 *
 *              CAUTION: frctMul() uses r = (LFract)a * b; which may not
 *                       result in efficient code for some versions of
 *                       tools.
 *
 *              TOOLS: V3.50 or higher.
 *
 * TABS: NONE
 *
 * $Id: fract.h 1.16 1999/11/08 18:02:00 bogdank Exp $
 *
 * REVISION HISTORY:
 *
 * $Log: fract.h $
 * Revision 1.16  1999/11/08 18:02:00  bogdank
 *     - frctMul() fixed, requires tools V3.50 or higher
 *    frctMul() fixed to work for ANSI C compilers (tools 3.50 or higher)
 * Revision 1.15  1998/04/24 16:11:04  nkumar
 *      Fixed a bug in all MultbyP2 and DivbyP2, that was potentially
 *      causing loss of significant bits.
 * Revision 1.14  1997/06/30 17:02:34  BOGDANK
 *     - long constants used again when necessary
 *    Long constants used when necessary.
 * Revision 1.13  1997/05/21 15:53:34  BOGDANK
 *     - new types used
 *    New types used.
 * Revision 1.12  1997/01/13 16:59:13  BOGDANK
 *     - frctMul() changed to accomodate bugs in optimizer (v1.16b)
 *    frctMul() changed to accomodate a bug in tools v1.16b.
 * Revision 1.11  1996/12/30 17:49:26  BOGDANK
 *     - several macros added (ULFract based, division-by-p2)
 *    Adding frctMulU(), frctMulULFbyUF(), frctMulULFbyULF(), power-of-2
 *    multiply/divide functions, frctULF2LFS().
 * Revision 1.10  1996/12/27 12:01:55  BOGDANK
 *     - bug fixes, improvements, additions prompted by ECU
 *    frctSaturateF() added.
 * Revision 1.9  1996/12/09 17:48:33  BOGDANK
 *     - frctAbs(), frctLF2FS() added
 *    frctAbs(), frctLF2FS() added.
 * Revision 1.8  1996/12/09 14:49:54  BOGDANK
 *     - UFract & ULFract added
 *    Adding frctRoundULF(), frctRoundUF(), frctRoundU(), frctMulLFbyF(),
 *    frctMulLFbyLF().
 * Revision 1.7  1996/10/16 19:15:40  BOGDANK
 *     - bug fixed in frctRoundLF() for Qa<=0
 *    Bug fix in frctRound[L]F() functions to work with Qa <= 0!!!
 * Revision 1.6  1996/10/07 17:35:08  BOGDANK
 *     - round functions improved
 *    Improving round functions.
 * Revision 1.5  1996/10/07 09:32:58  BOGDANK
 *     - float to fract conversions
 *    Float to Fract conversions added.
 * Revision 1.4  1996/10/02 14:19:23  BOGDANK
 *     - round functions added
 *    Round functions added.
 * Revision 1.3  1996/09/20 15:05:24  BOGDANK
 *     - bug fixed in frctMul().
 *    Putting more info and comments. Bugs fixed in frctMul().
 * Revision 1.2  1996/09/19 16:28:24  BOGDANK
 *    Minor bug in frctMul().
 * Revision 1.1  1996/09/19 10:00:37  BOGDANK
 * Initial revision
 *
 * (C) Copyright 1996, 1997, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include <ti/mas/fract/fract.h>

/* Mask that extracts Fract from LFract, e.g. 0x0000FFFF on 16-bit DSP CPU's */
#define FRCT_FRACT_MASK           (((LFract)1<<TYP_FRACT_PREC)-1)
/* Maximum positive fractional number is 0x7FFF on 16-bit DSP CPU's */
#define FRCT_MAX                  ((Fract)((~0)^(1U<<(TYP_FRACT_PREC-1))))
/* Minimum negative fractional number is 0x8000 on 16-bit DSP CPU's */
#define FRCT_MIN                  ((Fract)(1<<(TYP_FRACT_PREC-1)))
/* Maximum positive long fractional number is 0x7FFFFFFF on 16-bit DSP CPU's */
#define LFRCT_MAX                 ((LFract)((~0U)^(1U<<(TYP_LFRACT_PREC-1))))
/* Minimum negative long fractional number is 0x80000000 on 16-bit DSP CPU's */
#define LFRCT_MIN                 ((LFract)(1<<(TYP_LFRACT_PREC-1)))

#define frctNormLF(x) (_norm(x))
#endif  /* _FRACT_H */
/* nothing past this point */
