#ifndef __TYPES_H
#define __TYPES_H
/******************************************************************************
 * FILE PURPOSE: Basic Integral and Derived DSP ECO types for a C54x.
 ******************************************************************************
 * FILE NAME:   types.h
 *
 * DESCRIPTION: This file contains definitions of basic integral and
 *              derived DSP ECO types for a C54x platform. DSP ECO's should not
 *              use any other types. If additional types are needed, they could
 *              be defined within local and platform specific source code.
 *              However, types listed in this file must be used in all external
 *              API's.
 *
 *              Inline capability is also treated by this file.
 *
 * NOTE:        It is IMPORTANT to note that bit sizes of integral types
 *              correspond to the C54x C compiler.
 *
 * TABS: NONE
 *
 * $Id: types.h 1.8 1998/09/30 10:36:24 DLI Exp $
 *
 * REVISION HISTORY:
 *
 * $Log: types.h $
 * Revision 1.8  1998/09/30 10:36:24  DLI
 *   - added #define type and macro for structure alignment
 * Revision 1.7  1997/05/21 15:28:46  BOGDANK
 *     - telogy integer types added, precision definitions added
 *    Telogy integer types added. TYP_xxx_PREC added.
 * Revision 1.6  1996/12/27 12:01:55  BOGDANK
 *     - bug fixes, improvements, additions prompted by ECU
 *    TYP_LINSAMPLE_MAG added.
 * Revision 1.5  1996/12/09 14:49:54  BOGDANK
 *     - UFract & ULFract added
 *    Adding UFract and ULFract.
 * Revision 1.4  1996/09/18 11:06:02  ZORANM
 * Added definition for LFract type
 * Revision 1.3  1996/09/11 15:36:01  BOGDANK
 *    'inline' treatment added.
 * Revision 1.2  1996/09/08 14:32:18  BOGDANK
 *    Word type added.
 * Revision 1.1  1996/09/06 12:15:40  BOGDANK
 * Initial revision
 *
 * (C) Copyright 1996, 1997, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include <ti/mas/types/types.h>

/* Please note that the definitions that are remaining below should eventually
   disappear after all the code is compliant with the types.h defined in RTSC. */

/* If we do not use "inline" capability, get rid of it */
#ifndef _INLINE
#define inline
#endif


/******************************************************************************
 * DATA TYPE DEFINITION: Basic Integral Data Types.
 ******************************************************************************
 * DESCRIPTION: The following are basic integral data types, with descriptions
 *              of their suggested usage.
 *
 *****************************************************************************/

typedef tuint          uint;   /* ANSI C unsigned integer. Same as int but unsigned. */
typedef tulong         ulong;  /* ANSI C unsigned long integer. Same as long but unsigned. */
typedef unsigned short int  uchar;  /*! ANSI C unsigned char. Same as char but unsigned. */

/* a signed 16-bit integer */
#ifndef INT16
#define INT16 tint
#endif

/* an unsigned 16-bit integer */
#ifndef UINT16
#define UINT16 tuint
#endif

/* a signed 32-bit integer */
#ifndef INT32
#define INT32 tlong
#endif

/* an unsigned 32-bit integer */
#ifndef UINT32
#define UINT32 tulong
#endif

/*------------------*/
/* TELOGY DSP Types */
/*------------------*/
typedef unsigned char    word;   /* the smallest addressable _and_ dynamically
                                  *  allocatable integral element is a 16-bit
                                  *  word on this platform. */
typedef word             wordio; /* ioport is a keyword for c5x to indicate
                                    I/O address but is undefined for other 
                                    processors.This type maps ioport address
                                    to word for c6x and word ioport for c5x */

/* Bit storage sizes for non-ANSI integral types */
#define TYP_WORD_SIZE    TYP_CHAR_SIZE


/* Bit precision sizes for non-ANSI integral types */
#define TYP_WORD_PREC    TYP_CHAR_PREC

#if (!defined(BIG_ENDIAN) && !defined(LITTLE_ENDIAN))
#ifdef _BIG_ENDIAN
  #define BIG_ENDIAN
#else /* Little endian by default */
  #define LITTLE_ENDIAN
#endif
#endif
/******************************************************************************
 * DATA TYPE DEFINITION: Derived Integral Data Types
 ******************************************************************************
 * DESCRIPTION: The following are integral data types derived from the basic
 *              integral types, with descriptions of their suggested usage.
 *
 *****************************************************************************/

typedef tint bool;    /* logical variable */

typedef tint Cosine;      /* linear sample of a cosine/sine function in [-1,1]
                           *  range. On 16-bit fixed point platforms it is in
                           *  Q14 format. */

/* Bit storage sizes for derived integral types */
#define TYP_BOOL_SIZE       TYP_TINT_SIZE
#define TYP_COSINE_SIZE     TYP_TINT_SIZE

/* Bit precision sizes for derived integral types */
#define TYP_BOOL_PREC       TYP_TINT_PREC
#define TYP_COSINE_PREC     TYP_TINT_PREC

/* Linear sample magnitude is always [-32768,32767]=[-2^15,2^15-1] */
#define TYP_LINSAMPLE_MAG   15

typedef  void* tfPtr;  /* far pointer for data */

static inline void typeSet_tfPtr(tfPtr *x, void *ptr, tuint dummy) 
{ 
 *x = ptr;
 dummy = dummy; /* optimizer will optimize this out */

}

#endif /* __TYPES_H */
/* nothing after this point */
