/******************************************************************************
 * FILE PURPOSE: Provides APIs for copying bits. Mainly used to support Tandem
 *               Free Operation (TFO).
 ******************************************************************************
 * FILE NAME: utlbitcopy.c
 *
 * DESCRIPTION: TFO requires bits to be extracted and embedded from/into PCM 
 *              samples and also packed/unpacked with different frame structures
 *              This file has utilities taking care of these requirements.
 * 
 * (C) Copyright 2006, Texas Instruments Incorporated. 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

/* System level header files */
#include "types.h"

void utlUnpackLsbs (void *lsbs, void *lsbs_out, tint num_lsbs, tint length);
void utlCopyBits (tuint *src, tuint *dest, tint src_offset, tint dest_offset,
                  tint num_bits);
void utlCopyBits2Bits (void *src, void *dest, tint src_offset, tint num_bits);
void utlExtractAllLSBs (tuint *lsbs_ts, tint *tx_ptr, tint frame_size,
                                                        tint num_lsbs);

/* buffer that will be used between VCU and PIU has a header that will have 
   currently one tuint (16 bits). The header will be bitmap 
   with the following fields :

    MSB     14     13     12     11     10      9     8   
  +------+------+------+------+------+------+------+------+
  | STAT2| STAT1| EXTR3| EXTR2| EXTR1| EMBD3| EMBD2| EMBD1|
  +------+------+------+------+------+------+------+------+
  | NXT7 | NXT6 | NXT5 | NXT4 | NXT3 | NXT2 | NXT1 | NXT0 |
  +------+------+------+------+------+------+------+------+
     7      6      5      4      3      2      1     LSB
     
  STAT2-1 (2 bits)  : Status of TFO.
                      0 - Disabled
                      1 - Configured as IPE
                      2 - Configured as Active Equipment (AE)
  
  EXTR3-1 (3 bits)  : IPE: No significance
                      AE : Number of bits to Extract per PCM sample
  
  EMBD3-1 (3 bits)  : IPE: Number of bits to embed for this frame
                      AE : Number of bits to Embed per PCM sample
                           (1 for GSM-HR, 2 for rest of the GSM NB
                           codecs, 4 for some modes of WB-AMR)
  
  NXT7-0  (8 bits)  : IPE: Next phase (PCM phase position) where the first
                           bit needs to be embedded
                      AE : Next bit position in the body to be
                           embedded (with every frame, only a few bits of
                           the body are sent over the TDM. Typically
                           body contains 20ms worth of bits and only
                           5ms worth is embedded every time)
                      
  =============================================================
*/
#define UTL_TFO_GET_STAT(bitmap) ((bitmap >> 14) & 3)
#define UTL_TFO_GET_EXTR(bitmap) ((bitmap >> 11) & 7)
#define UTL_TFO_GET_EMBD(bitmap) ((bitmap >>  8) & 7)
#define UTL_TFO_GET_NXT(bitmap)  (bitmap & 0x00FF)

#define MASK_ONE_LSB  1  /* binary - 0000 0001 */
#define MASK_TWO_LSB  3  /* binary - 0000 0011 */
#define MASK_FOUR_LSB 15 /* binary - 0000 1111 */

#define MASK_X_ONE_LSB  0xFFFE /* binary - 1111 1111 1111 1110 */
#define MASK_X_TWO_LSB  0xFFFC /* binary - 1111 1111 1111 1100 */
#define MASK_X_FOUR_LSB 0xFFF0 /* binary - 1111 1111 1111 0000 */

/******************************************************************************
 * FUNCTION PURPOSE: Unpack the LSBs of PCM samples.
 ******************************************************************************
 * DESCRIPTION: PCM samples are packed into an array of words. The first
 *              The first sample's LSB (considering only one LSB is extracted)
 *              is placed as the MSB of the first word and so on. This routine
 *              unpacks the LSBs. Each word of the output will have the unpacked
 *              LSBs as its own LSBs and rest of the upper bits will be zeros.
 *
 * Fract utlUnpackLsbs(
 *       void      *lsbs,       - Pointer to output of unpack
 *       void      *lsbs_out,   - Pointer to packed LSBs
 *       tint      num_lsbs,    - Number of LSBs (per sample) packed
 *       tint      length)      - Frame length (number of samples from where the
 *                                LSBs were extracted) 
 *
 *   !!! IMPORTANT : This utl routine is not tested for any values other than 1
 *                   for num_lsbs !!!
 * 
 * Return Value: None
 *
 *****************************************************************************/

void utlUnpackLsbs (void *lsbs, void *lsbs_out, tint num_lsbs, tint length)
{
  tuint *lsbs_tmp = (tuint *) lsbs;
  tuint *lsbs_out_tmp = (tuint *) lsbs_out;
  tuint i, j, shift_val, extracted_bits;
  tuint mask;

  shift_val = (num_lsbs == 1) ? 4 
                              : ((num_lsbs == 2)
                                ? 3
                                : 2);
  mask = (num_lsbs == 1) ? MASK_ONE_LSB
                         : ((num_lsbs == 2) 
                           ? MASK_TWO_LSB
                           : MASK_FOUR_LSB);

  for (i=0; i<length; i++)
  {
    j = i * num_lsbs;
    extracted_bits = lsbs_tmp[i>>shift_val] >> (16 - (num_lsbs + (j%TYP_TINT_SIZE)));
    lsbs_out_tmp[i] = extracted_bits & mask;
  }
}
/******************************************************************************
 * FUNCTION PURPOSE: Unpack the LSBs of PCM samples.
 ******************************************************************************
 * DESCRIPTION: PCM samples are packed into an array of bytes. The first
 *              The first sample's LSB (considering only one LSB is extracted)
 *              is placed as the MSB of the first byte and so on. This routine
 *              unpacks the LSBs. Each byte of the output will have the unpacked
 *              LSBs as its own LSBs and rest of the upper bits will be zeros.
 *
 * Fract utlUnpackLsbsByte(
 *       void      *lsbs,       - Pointer to output of unpack
 *       void      *lsbs_out,   - Pointer to packed LSBs
 *       tint      num_lsbs,    - Number of LSBs (per sample) packed
 *       tint      length)      - Frame length (number of samples from where the
 *                                LSBs were extracted) 
 *
 *   !!! IMPORTANT : This utl routine is not tested for any values other than 1
 *                   for num_lsbs !!!
 * 
 * Return Value: None
 *
 *****************************************************************************/

void utlUnpackLsbsByte (void *lsbs, void *lsbs_out, tint num_lsbs, tint length)
{
  tuint *lsbs_tmp = (tuint *) lsbs;
  tuint *lsbs_out_tmp = (tuint *) lsbs_out;
  tuint i, j, shift_val, extracted_bits;
  tuint mask;

  shift_val = (num_lsbs == 1) ? 4 
                              : ((num_lsbs == 2)
                                ? 3
                                : 2);
  mask = (num_lsbs == 1) ? MASK_ONE_LSB
                         : ((num_lsbs == 2) 
                           ? MASK_TWO_LSB
                           : MASK_FOUR_LSB);

  for (i=0; i<length/2; i++)
  {
    j = (i*2) * num_lsbs;
    extracted_bits = lsbs_tmp[(i*2)>>shift_val] >> (16 - (num_lsbs + (j%TYP_TINT_SIZE)));
    lsbs_out_tmp[i] = (extracted_bits & mask) << 8;
    
    j = (i*2+1) * num_lsbs;
    extracted_bits = lsbs_tmp[(i*2+1)>>shift_val] >> (16 - (num_lsbs + (j%TYP_TINT_SIZE)));
    lsbs_out_tmp[i] |= extracted_bits & mask;
  }
}

/******************************************************************************
 * FUNCTION PURPOSE: Extract and pack every 16th LSB from PCM sample frame
 ******************************************************************************
 * DESCRIPTION: IS (Inband Signaling) messages are embedded into every 16th LSB
 *              of PCM samples. This routine extractes every 16th LSB in the 
 *              frame and packs it into an array of word, such that the first 
 *              of the 16th LSB extracted will the MSB of the first word. This
 *              routine also appends extracted bits into the packed array.
 *
 * tint  utlExtract16thLSB(
 *       void  *input,             - Pointer to the input frame of PCM samples
 *       tint  input_frame_length, - Input frame length
 *       tuint input_samp_offset,  - Sample offset of the first of the 16th
 *                                   LSB in the current frame
 *       void  *output,            - Pointer to where output array of extracted
 *                                   and packed LSBs are to be stored
 *       tint  output_bit_offset)  - Bit offset where the next bit needs to be
 *                                   placed (value 16 means the next bit has to
 *                                   be placed at the MSB of the second word)
 *
 *   !!! IMPORTANT : This utl routine is not tested for any values other than 40
 *                   for input_frame_length !!!
 * 
 * Return Value: Number of bits extracted
 *
 *****************************************************************************/

tint utlExtract16thLSB (void *input, tint input_frame_length, tuint input_samp_offset, 
                             void *output, tint outbitoff)
{
  tuint num_extracts, samp_comp, frame_comp, samp_diff, delta_bits, i;
  tuint in_index, out_index, out_bit_pos;
  tint  *input_buf = (tint *) input;
  tuint *output_buf = (tuint *) output;
  tuint output_bit_offset = (tuint) outbitoff;
  tuint extracted_bit;

  frame_comp = (input_frame_length / 16) + ((input_frame_length % 16) ? 1 : 0);
  samp_comp = input_samp_offset + (16 * frame_comp);
  samp_diff = samp_comp - input_frame_length;
  delta_bits = samp_diff / 16; /* Integer division */
  num_extracts = frame_comp - delta_bits; /* number of bits to be extracted in
                                             the current frame */

  for (i=0; i<num_extracts; i++, output_bit_offset++)
  {
    in_index = input_samp_offset + (i * 16);
    extracted_bit = input_buf[in_index] & 0x01;

    /*out_index = output_bit_offset / TYP_TINT_SIZE;*/
    out_index = output_bit_offset >> 4; /* Previous line optimized */
    out_bit_pos = output_bit_offset % TYP_TINT_SIZE;

    extracted_bit = extracted_bit << ((TYP_TINT_SIZE - 1) - out_bit_pos);

    output_buf[out_index] |= extracted_bit;
  }

  return (num_extracts);
}

/******************************************************************************
 * FUNCTION PURPOSE: Prepare a bitstream that needs to be embedded in every
 *                   16th LSB of the current frame
 ******************************************************************************
 * DESCRIPTION: This routine selects the bits that need to be sent out embedded
 *              into every 16th LSB of the current frame. The input is a bit
 *              stream that contains all the bits that are scheduled to be 
 *              transmitted over several frames. 
 *
 * Fract utlPrepare16thLSBs(
 *       void  *input,           - Pointer to the input bitstream
 *       tint  input_bit_offset, - Offset of the bit that needs to go next
 *       void  *output_ptr,      - Pointer to the output array.
 *                                 output_ptr[0] - Header (refer to tfo.h) 
 *                                 output_ptr[1] - Actual bits to be transmitted
 *                                                 with the current frame
 *       tint  samp_offset,      - Sample offset where the first of the 16th LSB
 *                                 should be embedded
 *       tint  frame_length)     - Frame length of the current frame
 *
 *   !!! IMPORTANT : This utl routine is not tested for any values other than 40
 *                   for input_frame_length !!!
 * 
 * Return Value: Number of bits to be embedded in the current frame
 *
 *****************************************************************************/

tint utlPrepare16thLSBs (void *input, tint input_bit_offset, void *output_ptr,
                                          tint samp_offset, tint frame_length)
{
  tint  num_embeds, samp_comp, frame_comp, samp_diff, delta_bits;
  tuint *output = (tuint *) output_ptr;

  frame_comp = (frame_length / 16) + ((frame_length % 16) ? 1 : 0);
  samp_comp = samp_offset + (16 * frame_comp);
  samp_diff = samp_comp - frame_length;
  delta_bits = samp_diff / 16; /* Integer division */
  num_embeds = frame_comp - delta_bits; /* number of bits to be extracted in
                                             the current frame */

  output[1] = 0;
  utlCopyBits2Bits ((tuint *) input, (tuint *) &output[1], input_bit_offset, num_embeds);

  return num_embeds;
}

/******************************************************************************
 * FUNCTION PURPOSE: Embed the bits into every 16th LSB
 ******************************************************************************
 * DESCRIPTION: This routine takes the input that was created by another util
 *              API - utlPrepare16thLSBs, and embeds them into a frame of PCM
 *              samples
 *
 * Fract utlEmbed16thLSB(
 *       void  *output_ptr,     - Pointer to the output PCM samples
 *       void  *input_ptr,      - Pointer to the input array. This is the array
 *                                that is created (output_ptr) in the routine
 *                                utlPrepare16thLSBs
 *       tint  num_bits,        - Number of bits to be embed in this frame
 *       tint  offset)          - Offset in output_ptr where the first bit needs
 *                                to be embedded
 *
 * Return Value: None
 *
 *****************************************************************************/

void utlEmbed16thLSB (void *output_ptr, void *input_ptr, tint num_bits, 
                                                                tint offset)
{
  tuint *input  = (tuint *) input_ptr;
  tuint *output = (tuint *) output_ptr;
  tint  i;
  tuint extracted_bit;

  for (i=0; i<num_bits; i++)
  {
    extracted_bit = input[0] >> (TYP_TINT_SIZE - 1 - i);
    extracted_bit &= 0x01;

    output[offset + (i * 16)] &= 0x00FE;
    output[offset + (i * 16)] |= extracted_bit;
  }
}

/******************************************************************************
 * FUNCTION PURPOSE: Copy bits from any offset to a word
 ******************************************************************************
 * DESCRIPTION: This routine copies any number of bits from any offset (even 
 *              spanning across words) to the beginning of a word
 *
 * Fract utlCopyBits2Bits (
 *       void  *src,       - Pointer to the source bitstream
 *       void  *dest,      - Pointer to the destination
 *       tint  src_offset, - Offset of the first bit to be copied in the source
 *                           bitstream
 *       tint  num_bits)   - Number of bits to be copied
 *
 *   !!! IMPORTANT : This utl routine is not tested for any values greater than
 *                   16 for num_bits !!!
 *
 * Return Value: None
 *
 *****************************************************************************/

void utlCopyBits2Bits (void *src, void *dest, tint src_offset, tint num_bits)
{
  tuint *input  = (tuint *) src;
  tuint *output = (tuint *) dest;
  tuint extracted_bit, extracted_word;
  tuint num_bits_packed = 0, i;

  for (i=src_offset; i<(src_offset+num_bits); i++, num_bits_packed++)
  {
    extracted_word = input[i/TYP_TINT_SIZE];
    extracted_word >>= (TYP_TINT_SIZE - 1 - (i%16));
    extracted_bit  = extracted_word & 0x0001;
    extracted_bit <<= (TYP_TINT_SIZE - 1 - (num_bits_packed % 16));
    output[num_bits_packed/TYP_TINT_SIZE] |= extracted_bit;
  }
}

void utlUnpackPCM (void *dest, void *source, tint frame_length)
{
  tuint *input  = (tuint *) source; /* 20 words for 5ms */
  tuint *output = (tuint *) dest;   /* 40 words for 5ms */
  tint  i;

  for (i=0; i<(frame_length / 2); i++)
  {
    output[i*2]     = (input[i] >> 8) & 0x00FF;
    output[(i*2)+1] = input[i] & 0x00FF;
  }
}

void utlPackPCM (void *dest, void *source, tint frame_length)
{
  tuint *input  = (tuint *) source; /* 40 words for 5ms */
  tuint *output = (tuint *) dest;   /* 20 words for 5ms */
  tint  i;

  for (i=0; i<(frame_length / 2); i++)
  {
    output[i] =  (input[i*2] << 8) & 0xFF00;
    output[i] |= ((input[(i*2)+1]) & 0x00FF);
  }
}



/*****************************************************************************
 * FUNCTION PURPOSE: Extract the specified LSBs and pack the bits
 *****************************************************************************
 * DESCRIPTION: Extract the specified LSBs and pack the bits
 *
 * void utlExtractAllLSBs(
 *   word *lsbs_ts,    - Pointer to memory where extracted and packed LSBs are
 *                       Stored. IMPORTANT - word datatype should be 16-bit
 *   linSample *tx_ptr,- Pointer to incoming PCM samples
 *   tint frame_size,  - Size of the frame
 *   tint num_lsbs)    - Number of LSBs to be extracted per PCM sample
 *
 ****************************************************************************/

void utlExtractAllLSBs (tuint *lsbs_ts, tint *tx_ptr, tint frame_size,
                                                        tint num_lsbs)
{
  tuint i, extracted_bits, shift_by, mask;

  if (num_lsbs == 1)
  {
    shift_by = 4;
    mask = MASK_ONE_LSB;
  }
  else if (num_lsbs == 2)
  {
    shift_by = 3;
    mask = MASK_TWO_LSB;
  }
  else
  {
    shift_by = 2;
    mask = MASK_FOUR_LSB;
  }
  
  for (i=0; i<frame_size; i++)
  {
    extracted_bits = tx_ptr[i] & mask;
    lsbs_ts[i>>shift_by] <<= num_lsbs;
    lsbs_ts[i>>shift_by] |= extracted_bits;
  } /* for */

} /* piu_tfo_extract */

/*****************************************************************************
 * FUNCTION PURPOSE: Embed the specified # of LSBs into PCM samples
 *****************************************************************************
 * DESCRIPTION: Embed the specified # of LSBs into PCM samples
 *
 * void utlExtractAllLSBs(
 *   word *lsbs_ts,    - Pointer to memory where extracted and packed LSBs are
 *                       Stored. IMPORTANT - word datatype should be 16-bit
 *   linSample *tx_ptr,- Pointer to PCM samples where LSBs are to be embedded
 *   tint frame_size,  - Size of the frame
 *   tint num_lsbs)    - Number of LSBs to be embedded per PCM sample
 *
 ****************************************************************************/

void utlEmbedAllLSBs (tuint *lsbs_ts, tint *tx_ptr, tint frame_size,
                                  tint num_lsbs_packed, tint num_lsbs_to_embed)
{
  tuint i, embed_bits, shift_by, mask_embed, mask_pcm;
  tuint num_bits_embedded = 0;

  shift_by = (num_lsbs_packed == 1) ? 4 : ((num_lsbs_packed == 2) ? 3 : 2);
  mask_embed = (num_lsbs_to_embed == 1) ? 0x1 : ((num_lsbs_to_embed == 2) ? 0x3 : 0xF);
  mask_pcm = (num_lsbs_to_embed == 1) ? 0xFE : ((num_lsbs_to_embed == 2) ? 0xFC : 0xF0);

  for (i=0; i<frame_size; i++)
  {
    embed_bits = (lsbs_ts[i>>shift_by] >> (15 - num_bits_embedded - (num_lsbs_packed - 1))) & mask_embed;

    tx_ptr[i] &= mask_pcm;
	tx_ptr[i] |= embed_bits;

    num_bits_embedded += num_lsbs_packed;
	if (num_bits_embedded == 16)
	{
	  num_bits_embedded = 0;
	}
  }
} /* utlEmbedAllLSBs */

void utlByteReverse (void *output_ptr, void *input_ptr, tint frame_length)
{
  tuint *input = (tuint *) input_ptr;
  tuint *output = (tuint *) output_ptr;
  tint  i;

  for (i=0; i<frame_length; i++)
  {
    output[i] = 0;
    output[i] |= ((input[i] >> 7) & 0x01);  /* Output LSB = Input MSB */
    output[i] |= ((input[i] >> 5) & 0x02);  /* Output 1b  = Input 6b  */
    output[i] |= ((input[i] >> 3) & 0x04);  /* Output 2b  = Input 5b  */
    output[i] |= ((input[i] >> 1) & 0x08);  /* Output 3b  = Input 4b  */
    output[i] |= ((input[i] << 1) & 0x10);  /* Output 4b  = Input 3b  */
    output[i] |= ((input[i] << 3) & 0x20);  /* Output 5b  = Input 2b  */
    output[i] |= ((input[i] << 5) & 0x40);  /* Output 6b  = Input 1b  */
    output[i] |= ((input[i] << 7) & 0x80);  /* Output MSB = Input LSB */
  }
}


/*****************************************************************************
 * FUNCTION PURPOSE: Embed number of LSBs into the output PCM samples
 *****************************************************************************
 * DESCRIPTION: Embed LSBs into the output PCM samples from the packed
 *              input bit stream
 *
 * void utlExtractAllLSBs(
 *   void *output_ptr, - Pointer to memory where output PCM samples are stored
 *                       and where the LSBs need to be embedded into
 *   void *input_ptr,  - Pointer to the packed bits which needs to be embedded
 *                       into the PCM samples
 *   tint num_lsbs_per_sample,  - Number of LSBs per sample
 *   tint bit_offset,   - Bit offset in the input bit stream. This value is
 *                        in the form of number of PCM samples (eg: if the
 *                        offset is 40 and num_lsbs_per_sample is 2, the actual
 *                        offset in bits is 80 (40*2))
 *   tint frame_length) - Frame length 
 *
 ****************************************************************************/

void utlEmbedLSBs (void *output_ptr, void *input_ptr, tint num_lsbs_per_sample,
                                            tint bit_offset, tint frame_length)
{
  tuint *input  = (tuint *) input_ptr;
  tuint *output = (tuint *) output_ptr;
  tuint extracted_bits, mask1, mask2;
  tint  i, count_bits, shift_by;
  tint actual_offset_in_bits = num_lsbs_per_sample * bit_offset;
  tint actual_offset_in_words = actual_offset_in_bits / TYP_TUINT_SIZE;
  tint bit_offset_reminder = actual_offset_in_bits % TYP_TUINT_SIZE;

  count_bits = bit_offset_reminder;

  mask1 =  (num_lsbs_per_sample == 1) ? MASK_ONE_LSB
                   : ((num_lsbs_per_sample == 2) ? MASK_TWO_LSB
                                                 : MASK_FOUR_LSB);
  mask2 =  (num_lsbs_per_sample == 1) ? MASK_X_ONE_LSB
                   : ((num_lsbs_per_sample == 2) ? MASK_X_TWO_LSB
                                                 : MASK_X_FOUR_LSB);

  for (i=0; i<frame_length; i++)
  {
    shift_by = TYP_TUINT_SIZE - (num_lsbs_per_sample + count_bits);
    extracted_bits = input[actual_offset_in_words] >> shift_by;

    extracted_bits &= mask1;

    output[i] &= mask2;
    output[i] |= extracted_bits;

    count_bits += num_lsbs_per_sample;
    if ((count_bits % TYP_TUINT_SIZE) == 0)
    {
      count_bits = 0;
      actual_offset_in_words++;
    }
  }
}

/******************************************************************************
 * FUNCTION PURPOSE: Prepare a bitstream that needs to be embedded in every
 *                   PCM sample
 ******************************************************************************
 * DESCRIPTION: This routine selects the bits that need to be sent out embedded
 *              into LSBs of every PCM sample of the current frame. The input 
 *              is a bitstream that contains all the bits that are scheduled 
 *              to be transmitted every 16th LSB only
 *              
 *
 * tint utlPrepareLsbFrame (
 *       void  *output_ptr,   - Pointer to the output array
 *       void  *input_ptr,    - Pointer to the input bit stream
 *       tint  frame_size,    - Frame length of the current frame
 *       tint  dest_offset,   - Sample offset where the first of the LSBs
 *                              should be embedded
 *       tint  src_offset,    - Offset of the next bit to embed in input_ptr
 *       tint num_lsbs)       - Number of LSBs per sample
 *
 * 
 * Return Value: Number of bits to be embedded in the current frame
 *
 *****************************************************************************/

tint utlPrepareLsbFrame (void *output_ptr, void *input_ptr, tint frame_size,
                               tint dest_offset, tint src_offset, tint num_lsbs)
{
  tuint *output = (tuint *) output_ptr;
  tuint *input  = (tuint *) input_ptr;
  tint  index;
  tint  num_input_bits_packed = src_offset;
  tuint bit_to_be_packed, bit_field;
  tint  next_bit_pos = dest_offset + (num_lsbs - 1); /* Initial bit position */

  for (index=0; index<(frame_size/16); index++) /* 16 here signifies every 16th LSB */
  {
    bit_to_be_packed = ((input[num_input_bits_packed/TYP_TUINT_SIZE] >>
                                 (TYP_TUINT_SIZE - 1)) - 
                                 (num_input_bits_packed % TYP_TUINT_SIZE));
    bit_to_be_packed &= 0x0001;
    bit_field = bit_to_be_packed << ((TYP_TUINT_SIZE - 1) -
                                            (next_bit_pos % TYP_TUINT_SIZE));

    output[next_bit_pos / TYP_TUINT_SIZE] = 0;
    output[next_bit_pos / TYP_TUINT_SIZE] |= bit_field;
    num_input_bits_packed++;
    next_bit_pos += (16 * num_lsbs); /* 16 here signifies every 16th LSB */
  }

  return (num_input_bits_packed);
} /* utlPrepareLsbFrame */

/* Nothing beyond this point */

