/******************************************************************************
 * FILE PURPOSE: Packing/unpacking functions for the C54x
 ******************************************************************************
 * FILE NAME:   utlrepck.h
 *
 * DESCRIPTION: Contains functions which deal with packing/unpacking on the 
 *              C54x (or any other 16-bit-word architecture)
 *
 * TABS: NONE
 *
 * $Id: pack.c 1.5 1999/03/12 14:22:37 jdowdal Exp $
 *
 * REVISION HISTORY:
 *
 * $Log: pvppack.c $
 * Revision 1.5  1999/03/12 14:22:37  jdowdal
 *    Got rid of debug #includes
 * Revision 1.4  1999/03/05 22:48:40  jdowdal
 *    Converted rtp and s3 to use packed cwsegs.
 * Revision 1.3  1999/03/04 23:47:11  jdowdal
 *    Made packing routines more general.
 * Revision 1.2  1999/03/03 23:56:20  jdowdal
 *    S3 SendIn doesnt unpack cwsegs any more
 *        plus necessary changes so debugging works before all packing is fixed
 * Revision 1.1  1999/03/03 21:18:53  jdowdal
 * Initial revision
 *
 * (C) Copyright 1999 Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

/* Module Includes */
#include "types.h"
#include "utlrepck.h"
#include "string.h"
#include "utlport.h"

/******************************************************************************
 * FUNCTION PURPOSE: pack tuints into tulongs
 ******************************************************************************
 * DESCRIPTION: This utility is used to pack 16 bit words from a data buffer into
 *              an array of 32 bit words.
 *
 * void utlPackTuintToTulong (
 *      tuint   *src,     -  Pointer to dataBuffer with packet 16 bit words
 *      tulong  *dest,    -  Pointer to byte array to pack 16 bit words to
 *      tint   NumTuints) -  Number of 16 bit NumTuints to unpack
 *
 *****************************************************************************/
void utlPackTuintToTulong (tuint *src, tulong *dest, tint NumTuints)
{
  tint i, shift;

  for (i = 0, shift = -1, dest--;i < NumTuints; i++, shift -= 16)
  {
	 if (shift < 0)
	 {
	 	shift = 16; 
	 	*(++dest) = *src++ << shift; 
	 }
	 else
	 {
	 	*dest |= *src++ << shift; 
	 }
  }
} /* utlPackTuintToTulong() */


/******************************************************************************
 * FUNCTION PURPOSE: pack bytes into words
 ******************************************************************************
 * DESCRIPTION: This utility is used to pack bytes from a data buffer into
 *              an array of 16 bit words.
 *
 * void utlPackWordToTuint (
 *      word   *src,     -  Pointer to dataBuffer with packet data bytes
 *      tuint  *dest,    -  Pointer to byte array to pack data bytes to
 *      tint   NumBytes) -  Number of data bytes to unpack
 *
 *****************************************************************************/
void utlPackWordToTuint (word *src, tuint *dest, tint NumBytes)
{
  tint i, shift;

  for (i = 0, shift = -1, dest--;i < NumBytes; i++, shift -= 8)
  {
	 if (shift < 0)
	 {
	 	shift = 8; 
	 	*(++dest) = *src++ << shift; 
	 }
	 else
	 {
	 	*dest |= *src++ << shift; 
	 }
  }
} /* utlPackWordToTuint() */

/******************************************************************************
 * FUNCTION PURPOSE: Unpack words into bytes
 ******************************************************************************
 * DESCRIPTION: UnPacks words (which are stored two-byte-per word) into bytes
 *
 * void utlPackBytesIntoWords (
 *     word *src,         # Source word 
 *     word *dest,        # Destination unpacked words
 *     tint NumBytes,     # Number of bytes to unpack
 *                         
 *****************************************************************************/
void utlUnpackBuffer (word *src, word *dest, tint NumBytes)
{
  tint i;

  for (i = 0; i < NumBytes; i++)
  {
	  *dest++ = *src++;
  }
} /* utlUnpackBuffer() */

/******************************************************************************
 * FUNCTION PURPOSE: Read n bit value from 8 bit word (macro version)
 ******************************************************************************
 * DESCRIPTION: copies n bit value from an word.
 *              Assumes the reading from MSB to LSB
 * tuint utlRead8bits_m (
 *    word  *base,  - Base of byte array
 *    tuint byte,   - Byte offset to read
 *    tint  nbits); - Number of bits to read
 * 
 *****************************************************************************/
word utlReadnbits (word *base, tuint byte,tint nbits)
{
	return utlReadnbits_m (base,byte,nbits);
}

/******************************************************************************
 * FUNCTION PURPOSE: Convert bytes to words, rounding up
 ******************************************************************************
 * DESCRIPTION: Converts bytes to words, rounding up
 *
 * tuint utlBytesToWords (
 *     tuint bytes)        # number of bytes
 *
 *****************************************************************************/

tuint utlBytesToWords (tuint bytes) 
{
  return bytes;
} /* utlBytesToWords */

/******************************************************************************
 * FUNCTION PURPOSE: Read 8 bit value from 16 bit word (function version)
 ******************************************************************************
 * DESCRIPTION: Returns 8 bit value from 16 bit word.  Assumes nothing.
 * 
 * tuint utlRead8bits (
 *    word  *base,  - Base of byte array
 *    tuint byte); - Byte offset to read
 * 
 *****************************************************************************/
tuint utlRead8bits (word *base, tuint byte) 
{
  return utlRead8bits_m (base, byte);
} /* utlRead8bits */

/******************************************************************************
 * FUNCTION PURPOSE: Write 8 bit value into 16 bit word (function version)
 ******************************************************************************
 * DESCRIPTION: Writes 8 bit value into 16 bit word; nothing assumed.
 * 
 * void utlWrite8bits (
 *    word  *base,  - Base of byte array
 *    tuint byte,   - byte offset to write
 *    tuint val)    - Byte in low 8 bits of val
 * 
 *****************************************************************************/
void utlWrite8bits (word *base, tuint byte, tuint val)
{
  utlWrite8bits_m (base, byte, val);
} /* utlWrite8bits */

/******************************************************************************
 * FUNCTION PURPOSE: Write 16 bit value into 16 bit word (function version)
 ******************************************************************************
 * DESCRIPTION: Writes 16 bit value into 16 bit word.  No assumptions
 * 
 * void utlWrite16bits (
 *    word  *base,  - Base of byte array
 *    tuint byte,   - byte offset to write; assumed to be even
 *    tuint val)    - 16 bit val
 * 
 *****************************************************************************/

void utlWrite16bits (word *base, tuint byte, tuint val) 
{
  utlWrite16bits_m (base, byte, val);
} /* utlWrite16bits */


void utlWrite16bitsSwap (word *base, tuint byte, tuint val) 
{
  val = ( (val <<8) | ((val >>8)&0xff)  );
  utlWrite16bits_m (base, byte, val);
} /* utlWrite16bitsSwap */


/******************************************************************************
 * FUNCTION PURPOSE: Read 16 bit value from 16 bit word (function version)
 ******************************************************************************
 * DESCRIPTION: Returns 16 bit value from 16 bit word.  No assumptions.
 * 
 * tuint utlRead16bits (
 *    word  *base,  - Base of byte array
 *    tuint byte); - Byte offset to read, assumed to be even
 * 
 *****************************************************************************/
tuint utlRead16bits (word *base, tuint byte) 
{
  return utlRead16bits_m (base, byte);
} /* utlRead16bits */

/******************************************************************************
 * FUNCTION PURPOSE: Write 32 bit value into 16 bit words (function version)
 ******************************************************************************
 * DESCRIPTION: Writes 32 bit value into 16 bit word; No
 *              alignment assumed
 * 
 * void utlWrite32bits (
 *    word  *base,  - Base of byte array
 *    tuint byte,   - byte offset to write; assumed to be even.
 *    tulong val)   - 32 bit val
 * 
 *****************************************************************************/
void utlWrite32bits (word *base, tuint byte, tulong val) 
{
  utlWrite32bits_m (base, byte, val);
} /* utlWrite32bits */

/******************************************************************************
 * FUNCTION PURPOSE: Read 32 bit value from 16 bit words (function version)
 ******************************************************************************
 * DESCRIPTION: Read 32 bit value from 16 bit words; No
 *              alignment assumed
 * 
 * tulong utlRead32bits (
 *    word  *base,  - Base of byte array
 *    tuint byte)   - byte offset to write; assumed to be even.
 * 
 *****************************************************************************/
tulong utlRead32bits (word *base, tuint byte) 
{
  return utlRead32bits_m (base, byte);
} /* utlRead32bits */


/*********************************************************************************
 * FUNCTION PURPOSE: Pack the lower 8 bits of a 16-bit word whose MSB is zero into
 *                    words
 *********************************************************************************
 * DESCRIPTION: This function is identical to ultPackBytesIntoWords in c55x.
 *              Needed only for c64x
 *              
 *
 * 
 * void utlPackLSBof16bitsIntoWords(
 *    linSample *src,   - Base of byte array 
 *    word      *dst,   - Destination word array
 *    tint       nbytes - number of bytes to write
 *    tint       offset - byte offset to write
 * 
 *********************************************************************************/
void utlPackLSBof16bitsIntoWords(linSample *src, word *words, tint numbytes, tint skip)
{
   char *dst = (char *)words;
   /* Word is currently 16-bits.Hence bytes are stored into array of 16-bit
      values,in which only the lower 8 bits have the desired byte value */
   int i;
   dst += skip;
   for(i=0;i<numbytes;i++){
     char byteToPack = (char)((tuint)(*src++) & 0xFF);
     *dst++ = byteToPack; 
   }
}

/*********************************************************************************
 * FUNCTION PURPOSE: Unpacks the bytes of a byte-array in to the LSBs of a 16 bit
 *                    16-bit word array, whose MSBs would be zero.
 *********************************************************************************
 * DESCRIPTION: Unpacks the bytes of a byte-array in to the LSBs of a 16 bit
 *                    16-bit word array, whose MSBs would be zero.
 * 
 * void utlUnpackBytesTo16bitWords(
 *    word  *packed_bytes,   - Base of byte array 
 *    linSample      *dest,   - Destination 16-bit array
 *    tint       nbytes - number of bytes to write
 *    tint       offset - byte offset to write from
 *********************************************************************************/
void utlUnpackBytesTo16bitWords (word *packed_bytes,linSample *dest, tint numbytes,tint src_offset)
{
   int i;
   for(i=0;i<numbytes;i++)
   {
     *dest++=(utlRead8bits_m(packed_bytes,(src_offset+i)) & 0xFF);
   }
}

#if defined(LITTLE_ENDIAN) 
/*********************************************************************************
 * FUNCTION PURPOSE: Tranforms an array of 16-bit entities from LITTLE_ENDIAN to 
 *                   big endian
 *********************************************************************************
 * DESCRIPTION: This function tranforms the input array into big endian format
 *              irrespective of the native endian-ness of the processor.Needs
 *              to be implemented only for little endian processors 
 *
 * 
 * void utlBigEndianTransform (
 *    tuint    *src,     - Base of array to be transformed into big endian format
 *    tuint    count     - size of array in tuints
 * 
 *********************************************************************************/
void utlBigEndianTransform (tuint *src,tuint count)
{
  int i; 
  tuint temp;
  for (i = 0;i < count;i++,src++) {
    temp = *src;
    utlWrite16bits_m ((word *)src,0,temp);
  }
}
#endif

/*********************************************************************************
 * FUNCTION PURPOSE: OR operation in a endian portable form
 *                   
 *********************************************************************************
 * DESCRIPTION: Performs OR operation in an endian protable way.The value to be 
 *              ORed is specified assuming big endian archtecture.The function will 
 *              take care byte swapping for little endian architectures 
 * 
 * void utlOR16bits(
 *    word *src,   - Base of byte array 
 *    tint       offset - byte offset to for OR operation
 *    tint       val    - value to be ORed 
 * 
 *********************************************************************************/
void utlOR16bits (word *src, tuint bytes, tuint val)
{
  utlOR16bits_m (src,bytes,val);
}


/*********************************************************************************
 * FUNCTION PURPOSE: AND operation in a endian portable form
 *                   
 *********************************************************************************
 * DESCRIPTION: Performs AND operation in an endian protable way.The value to be 
 *              ANDed is specified assuming big endian archtecture.The function will 
 *              take care byte swapping for little endian architectures 
 * 
 * void utlAND16bits(
 *    linSample *src,   - Base of byte array 
 *    tint       offset - byte offset to for AND operation
 *    tint       val    - value to be ANDed 
 * 
 *********************************************************************************/
void utlAND16bits (word *src, tuint bytes, tuint val)
{
  utlAND16bits_m (src,bytes,val);
}

void utl_prog_to_data (void (*prog_tbl)(void), tint *data_tbl, tint tbl_size)
{
  memcpy(data_tbl, prog_tbl, tbl_size * sizeof(tint));
}

/* Nothing past this point */
