#ifndef __UTLREPCK_M_H__
#define __UTLREPCK_M_H__
/******************************************************************************
 * FILE PURPOSE: C54X platform specific macros and definitions
 ******************************************************************************
 * FILE NAME: utlrepck_m.h  
 *
 * DESCRIPTION: This contains 8 bit byte in 16 bit word manipulation via
 *              macros.  It is important to check size-optimization quality
 *              of compiler, and tune these macros to minimize size given
 *              existing alignments.  It might be smaller to test proper
 *              alignment with an if, and directly assign it than have
 *              optimizer optimize shifting and masking.
 *
 * (C) Copyright 2002, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include "types.h"

/* Define explicit endian mappings for stupid compilers that can't optimize */
#ifdef STUPID_COMPILER
#ifdef LITTLE_ENDIAN
#define UTL_ENDIAN_16BIT_LSB  1
#define UTL_ENDIAN_16BIT_MSB  0
#define UTL_ENDIAN_32BIT_LSB  3
#define UTL_ENDIAN_32BIT_MSB1 2
#define UTL_ENDIAN_32BIT_MSB2 1
#define UTL_ENDIAN_32BIT_MSB  0
#else
#define UTL_ENDIAN_16BIT_LSB  0
#define UTL_ENDIAN_16BIT_MSB  1
#define UTL_ENDIAN_32BIT_LSB  0
#define UTL_ENDIAN_32BIT_MSB1 1
#define UTL_ENDIAN_32BIT_MSB2 2
#define UTL_ENDIAN_32BIT_MSB  3
#endif
#endif

/******************************************************************************
 * FUNCTION PURPOSE: Read n bit value from 8 bit word (macro version)
 ******************************************************************************
 * DESCRIPTION: copies n bit value from an word. Assumes nothing
 * tuint utlRead8bits_m (
 *    word  *base,  - Base of byte array
 *    tuint byte,   - Byte offset to read
 *    tuint nbits); - Number of bits to read
 * 
 *****************************************************************************/
static inline word utlReadnbits_m (word *base, tulong byte,tint nbits) 
{
  char *src = (char *)base;
  char wvalue = *(src + byte);
  tuint readByte = (tuint)(wvalue & 0xFF);
  word  temp_bit;

  if (nbits > 0)
	temp_bit = readByte >> nbits; 
  else
	temp_bit = readByte << (-nbits); 
  return (temp_bit);
} /* utlReadnbits_m */

/******************************************************************************
 * FUNCTION PURPOSE: Read 8 bit value from 16 bit word (macro version)
 ******************************************************************************
 * DESCRIPTION: Returns 8 bit value from 16 bit word.  Assumes nothing.
 * 
 * tuint utlRead8bits_m (
 *    word  *base,  - Base of byte array
 *    tuint byte); - Byte offset to read
 * 
 *****************************************************************************/
static inline tuint utlRead8bits_m (word *base, tulong byte) 
{
  char *src = (char *)base;
  char wvalue = *(src + byte);
  tuint readByte = (tuint)(wvalue & 0xFF);
  return readByte;
} /* utlRead8bits_m */

/******************************************************************************
 * FUNCTION PURPOSE: Write 8 bit value into 16 bit word (macro version)
 ******************************************************************************
 * DESCRIPTION: Writes 8 bit value into 16 bit word; nothing assumed.
 * 
 * void utlWrite8bits_m (
 *    word  *base,  - Base of byte array
 *    tuint byte,   - byte offset to write
 *    tuint val)    - Byte in low 8 bits of val
 * 
 *****************************************************************************/
static inline void utlWrite8bits_m (word *base, tulong byte, tuint val)
{
  char *wptr = ((char *)base + byte);
  *wptr = (char)(val & 0xFF);
} /* utlWrite8bits_m */

/******************************************************************************
 * FUNCTION PURPOSE: Write 16 bit value into 16 bit word (macro version)
 ******************************************************************************
 * DESCRIPTION: Writes 16 bit value into 16 bit word.  No assumptions
 * 
 * void utlWrite16bits_m (
 *    word  *base,  - Base of byte array
 *    tuint byte,   - byte offset to write; assumed to be even
 *    tuint val)    - 16 bit val
 * 
 *****************************************************************************/

static inline void utlWrite16bits_m (word *base, tulong byte, tuint val) 
{
  char *wptr = ((char *)base + byte);

#ifdef STUPID_COMPILIER
  /* Explicit endian might be more efficient */
  word *srcp = (word *)&val;
  wptr[0] = srcp[UTL_ENDIAN_16BIT_MSB];
  wptr[1] = srcp[UTL_ENDiAN_16BIT_LSB];
#else
  /* Shift/mask is endian-portable, but look out for stupid compilers */
  wptr[0] = (char)(val>>8);
  wptr[1] = (char)(val & 0xff);
#endif
} /* utlWrite16bits_m */

/******************************************************************************
 * FUNCTION PURPOSE: Read 16 bit value from 16 bit word (macro version)
 ******************************************************************************
 * DESCRIPTION: Returns 16 bit value from 16 bit word.  No assumptions.
 * 
 * tuint utlRead16bits_m (
 *    word  *base,  - Base of byte array
 *    tuint byte); - Byte offset to read, assumed to be even
 * 
 *****************************************************************************/
static inline tuint utlRead16bits_m (word *base, tulong byte) 
{
  char *wptr = ((char *)base + byte);
  tuint ret;
#ifdef STUPID_COMPILIER
  /* Explicit endian might be more efficient */
  word *dptr = &ret;
  dptr[UTL_ENDIAN_16BIT_MSB] = wptr[0];
  dptr[UTL_ENDIAN_16BIT_LSB] = wptr[1];
#else
  /* Shift/mask is endian-portable, but look out for stupid compilers */
  ret = (((tuint)wptr[0]) << 8) | (wptr[1] & 0xFF);
#endif

  return ret;
} /* utlRead16bits_m */

/******************************************************************************
 * FUNCTION PURPOSE: Write 32 bit value into 16 bit words (macro version)
 ******************************************************************************
 * DESCRIPTION: Writes 32 bit value into 16 bit word; No
 *              alignment assumed
 * 
 * void utlWrite32bits_m (
 *    word  *base,  - Base of byte array
 *    tuint byte,   - byte offset to write; assumed to be even.
 *    tulong val)   - 32 bit val
 * 
 *****************************************************************************/
static inline void utlWrite32bits_m (word *base, tulong byte, tulong val) 
{
#ifdef STUPID_COMPILIER
  /* Explicit endian might be more efficient */
  word *sptr = (word *)&val; 
  word *dptr = (base + byte);
  dptr[0] = sptr[UTL_ENDIAN_32BIT_MSB];
  dptr[1] = sptr[UTL_ENDIAN_32BIT_MSB2];
  dptr[2] = sptr[UTL_ENDIAN_32BIT_MSB1];
  dptr[3] = sptr[UTL_ENDIAN_32BIT_LSB];
#else
  /* Shift/mask is endian-portable, but look out for stupid compilers */
  utlWrite16bits_m (base, byte, (tuint)(val>>16));
  utlWrite16bits_m (base, byte+2, (tuint)(val&0xffff));
#endif
} /* utlWrite32bits_m */

/******************************************************************************
 * FUNCTION PURPOSE: Read 32 bit value from 16 bit words (macro version)
 ******************************************************************************
 * DESCRIPTION: Read 32 bit value from 16 bit words; No
 *              alignment assumed
 * 
 * tulong utlRead32bits_m (
 *    word  *base,  - Base of byte array
 *    tuint byte)   - byte offset to write; assumed to be even.
 * 
 *****************************************************************************/
static inline tulong utlRead32bits_m (word *base, tulong byte) 
{
  tulong ret;
  
#ifdef STUPID_COMPILIER
  /* Explicit endian might be more efficient */
  word *sptr = (base + byte);
  word *dptr = (word *)&ret; 
  dptr[0] = sptr[UTL_ENDIAN_32BIT_MSB];
  dptr[1] = sptr[UTL_ENDIAN_32BIT_MSB2];
  dptr[2] = sptr[UTL_ENDIAN_32BIT_MSB1];
  dptr[3] = sptr[UTL_ENDIAN_32BIT_LSB];
#else
  /* Shift/mask is endian-portable, but look out for stupid compilers */
  ret = (((tulong)utlRead16bits_m (base, byte)) << 16);
  ret |= (tulong)utlRead16bits_m (base, byte + 2);
#endif
         
  return ret;
} /* utlRead32bits_m */

/* Prototype  needs to be here because macro below calls this functions
   Is it better to do 
   #define utlPackLSBof16bitsIntoWords_m  utlPackLSBof16bitsIntoWords
   instead of a static inline function? */
void utlPackLSBof16bitsIntoWords(linSample *src, word *words, tint numbytes, tint skip);

/*********************************************************************************
 * FUNCTION PURPOSE: Pack the lower 8 bits of a 16-bit word whose MSB is zero into
 *                    words
 *********************************************************************************
 * DESCRIPTION: This function is identical to ultPackBytesIntoWords in c55x.
 *              Needed only for c64x
 *              
 *
 * 
 * void utlPackLSBof16bitsIntoWords_m(
 *    linSample *src,   - Base of byte array 
 *    word      *dst,   - Destination word array
 *    tint       nbytes - number of bytes to write
 *    tint       offset - byte offset to write
 * 
 *********************************************************************************/
static inline void utlPackLSBof16bitsIntoWords_m (linSample *src,word *dst,
                                                  tint nbytes,tint offset)
{

  utlPackLSBof16bitsIntoWords (src,dst,nbytes,offset);
  return;
}

void utlUnpackBytesTo16bitWords (word *packed_bytes,linSample *dest, tint numbytes,tint src_offset);

/*********************************************************************************
 * FUNCTION PURPOSE: OR operation in a endian portable form
 *                   
 *********************************************************************************
 * DESCRIPTION: Performs OR operation in an endian protable way.The value to be 
 *              ORed is specified assuming big endian archtecture.The function will 
 *              take care byte swapping for little endian architectures 
 * 
 * void utlOR16bits(
 *    word *src,   - Base of byte array 
 *    tint       offset - byte offset to for OR operation
 *    tint       val    - value to be ORed 
 * 
 *********************************************************************************/
static inline void utlOR16bits_m (word *src, tuint bytes, tuint val)
{
  tuint existing_val = utlRead16bits_m (src,bytes);
  existing_val |=  val;
  utlWrite16bits_m (src,bytes,existing_val);
}


/*********************************************************************************
 * FUNCTION PURPOSE: AND operation in a endian portable form
 *                   
 *********************************************************************************
 * DESCRIPTION: Performs AND operation in an endian protable way.The value to be 
 *              ANDed is specified assuming big endian archtecture.The function will 
 *              take care byte swapping for little endian architectures 
 * 
 * void utlAND16bits(
 *    linSample *src,   - Base of byte array 
 *    tint       offset - byte offset to for AND operation
 *    tint       val    - value to be ANDed 
 * 
 *********************************************************************************/
static inline  void utlAND16bits_m (word *src, tuint bytes, tuint val)
{
  tuint existing_val = utlRead16bits_m (src,bytes);
  existing_val &=  val;
  utlWrite16bits_m (src,bytes,existing_val);
}
 /******************************************************************************
 * FUNCTION PURPOSE: Pack bytes into words
 ******************************************************************************
 * DESCRIPTION: Packs bytes (which are stored one-byte-per word in the least
 *              significant 8 bits of the words) into words in big-endian
 *              format (meaning most-significant bytes are stored in the
 *              least significant bits of the words).
 *
 * void utlPackBytesIntoWords (
 *     word *bytes,        # Source bytes (stored in LS Bits of words)
 *     word *words,        # Destination packed words
 *     tint nbytes,        # Number of bytes to pack
 *     tint skip )         # # of bytes in 'words' to skip over
 *                         # In 24 bit word | 0 | 1 | 2 |
 *                         
 *****************************************************************************/

static inline void utlPackBytesIntoWords(word *bytes, word *words, tint nbytes, tint offset) 
{
   char *dst = (char *)words;
   /* Word is currently 16-bits.Hence bytes are stored into array of 16-bit
      values,in which only the lower 8 bits have the desired byte value */
   dst += offset;
   memcpy(dst,bytes,nbytes);
}

 /******************************************************************************
 * FUNCTION PURPOSE: Unpack words into bytes.
 ******************************************************************************
 * DESCRIPTION: Unpack words containing one or more bytes into individual bytes.
 *              The most significant bits of word go into the lowest indexed
 *              bytes (big endian format).
 *
 *              On the C54x, assume that nbytes is even and that offset is 0
 *              
 * void utlUnpackWordsIntoBytes (
 *     word *bytes,        # Source packed words 
 *     word *words,        # Destination unpacked bytes
 *     tint nbytes,        # Number of bytes to unpack
 *     tint offset )       # Offset into first word to start unpacking
 *                         # In 24 bit word | 0 | 1 | 2 |
 *                         
 *****************************************************************************/

static inline void utlUnpackWordsIntoBytes(word *words, word *bytes, tint nbytes,
                             tint offset)
{
   char *src = (char *)words;
   src += offset;
   memcpy(bytes,src,nbytes);
}

/******************************************************************************
 * FUNCTION PURPOSE: Repack words into words
 ******************************************************************************
 * DESCRIPTION: Copies words from source to destination starting at the 
 *              'offset' byte within the first word.
 *
 * void utlRepackWordsIntoWOrds (
 *     word *src,          # Source packed words 
 *     word *dst,          # Destination packed words
 *     tint sskip,         # Number of bytes in source to skip
 *     tint nbytes,        # Number of bytes to copy
 *     tint skip )         # Number of bytes in dst to skip over
 *                         # In 24 bit word | 0 | 1 | 2 |
 *                         
 *****************************************************************************/

static inline void utlRepackWordsIntoWords (word *src, word *dst, tint sskip, tint nbytes, 
                              tint skip) 
{
   char *source = (char *)src;
   char *dest   = (char *)dst;
   source += sskip;
   dest   += skip;
   memcpy (dest, source, nbytes);
}

static inline void utlXor32bits_m (word *base1, tuint byte1, word *base2, 
                                                                  tuint byte2)
{
  tulong xorVal = (utlRead32bits_m (base1, byte1)) ^ 
                                       (utlRead32bits_m (base2, byte2));
  utlWrite32bits_m (base2, byte2, xorVal);
  
} /* utlXor32bits_m */


static inline void utlXor16bits_m (word *base1, tuint byte1, word *base2, 
                                                                  tuint byte2)
{
  tuint xorVal = (utlRead16bits_m (base2, byte2)) ^ (utlRead16bits_m (base1, byte1));
  utlWrite16bits_m (base2, byte2, xorVal);
} /* utlXor16bits_m */

static inline void utlXor8bits_m (word *base1, tuint byte1, word *base2, 
                                                                  tuint byte2)
{
  tuint op2 = utlRead8bits_m (base2, byte2);
  tuint op1 = utlRead8bits_m (base1, byte1);
  tuint xorVal = op1 ^ op2;
  utlWrite8bits_m (base2, byte2, xorVal);
} /* utlXor8bits_m */


static inline void utlXorWordsIntoWords (word *src, word *dst, 
                                         tint sskip, tint nbytes, tint skip) 
{
  char *wptr1 = ((char *)src + sskip);
  char *wptr2 = ((char *)dst + skip);
  tint i;
  for (i=0; i<nbytes; i++)
  {
    wptr2[i] = wptr1[i] ^ wptr2[i];
  }
}

#endif
/* Nothing past this point */
