#ifndef __RTTRACE_H__
#define __RTTRACE_H__
/******************************************************************************
 * FILE PURPOSE: Functions and definitions used by RTTRACE functionality
 ******************************************************************************
 * FILE NAME:   rttrace.h
 *
 * DESCRIPTION: Functions and definitions used by RTTRACE functionality
 *
 * TABS: NONE
 *
 * $Id: $
 *
 * REVISION HISTORY:
 *
 * $Log: $
 *
 * (C) Copyright 2004 Texas Instruments Incorporated. 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include "types.h"
#include "util.h"
#include "ecoif/rttraceapi.h"

#define UTL_RTTRACE_NUM_BUFS      2  /* Assumed to be power of 2 below */
#define UTL_RTTRACE_MSGHEADER     (5 * sizeof(tuint))  /* sizeof(mgbHeader_t) */

/* PCM/PKT trace buffering (per buffer) */
typedef struct utlRtTraceBuff_s {
  tuint index;         /* Index into buffer, size if buffer is full */
  word  *data;
} utlRtTraceBuff_t;

typedef struct utlRtTraceEnable_s {
  tuint enabled_bitfield;                /* Per-channel enable bitfield;
                                          * bit possition is stream ID */
} utlRtTraceEnable_t;

/* PCM/PKT trace root structure */
typedef struct utlRtTraceContext_s {
  utlRtTraceEnable_t *enable;            /* Enable fields */
  utlRtTraceBuff_t   buffers[UTL_RTTRACE_NUM_BUFS];
  tuint              seqn;               /* Trace packet/msg seqn */
  tuint              bufferNumber;       /* Double/N Buffer Selector */
  tuint              clock;              /* Time until buffer halves should
                                          * switch */
} utlRtTraceContext_t;

/* This value can be enum since it is never stored */
typedef enum {
  UTL_RTTRACE_UNPACKED,  /* Data is one byte per word */
  UTL_RTTRACE_PACKED     /* Data is packed into words */
} utlRtTracePacked_e;

extern utlRtTraceContext_t utlRtTraceContext;
void siuRtTraceFcn (tuint ID, tuint dataSizeBytes, void *data, tuint streamId,
                    utlRtTracePacked_e packed);

/*
 * If build-by-build enabling of traces is required, define this from the
 * makefile and add cdefdep to all modules that use tracing to it will
 * recompile everything build-to-build
 */
#define UTL_RTTRACE_BUILDMASK 0xffffu
/******************************************************************************
 * FUNCTION PURPOSE: Realtime trace logging
 ******************************************************************************
 * DESCRIPTION: Provides realtime data streaming to the host.
 *
 * This wrapper checks if the trace is allowed in the build (at compile time
 * if streamId is a constant), and checks if the trace is enabled at runtime.
 * If enabled, call the function to record the trace (to save mips due to
 * calling functions that just return)
 *
 * utlRtTrace(tuint ID,                 - MID + CID of module recording trace
 *            tuint dataSizeBytes,      - Record length in bytes
 *            void *data,               - Record
 *            tuint streamId,           - MGB_RTTRACE_STREAM_ID_*
 *            utlRtTracePacked_e packed - Data one byte per word or packed 
 *
 *****************************************************************************/
static inline void utlRtTrace (tuint ID, tuint dataSizeBytes, void *data, 
                               tuint streamId, utlRtTracePacked_e packed) 
{
  tuint streamId_bit = 1u << streamId;

  if ((streamId_bit & UTL_RTTRACE_BUILDMASK) == 0) {
    /* This trace isn't allowed in the build.  Assuming streamId is a constant
     * in the call to utlRtTrace, it will compile out with optimization.
     */
    return;
  }

  /* Check for enable without calling any functions to save mips */
  if (utlRtTraceContext.enable[UTL_EXTRACT_CHNUM(ID) - 1].enabled_bitfield & 
      streamId_bit) {
#ifndef HDCT
    siuRtTraceFcn(ID, dataSizeBytes, data, streamId, packed); 
#endif
  }
}

/*****************************************************************************
 * FUNCTION PURPOSE: Switch to next double/N buffer
 ****************************************************************************/
static inline void utlRtTraceNextBuf() {
  utlRtTraceContext.bufferNumber = 
    (utlRtTraceContext.bufferNumber + 1) & (UTL_RTTRACE_NUM_BUFS - 1);
  utlRtTraceContext.buffers[utlRtTraceContext.bufferNumber].index =
    UTL_RTTRACE_MSGHEADER;
}

#endif /* __RTTRACE_H__ */

/* Nothing past this point */
