/******************************************************************************
 * FILE PURPOSE: Provides functions for large additions/multiplications 
 ******************************************************************************
 * FILE NAME: utlop.c  
 *
 * DESCRIPTION: Provides functions to use for long number addition and 
 *              multiplication.
 *
 * TABS: NONE
 *
 * $Id$
 *
 * REVISION HISTORY:
 *
 * $Log$
 *
 * (C) Copyright 2002, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include "types.h"
#include "utlport.h"
/******************************************************************************
 * FUNCTION PURPOSE: Calculates dst = dst + (src << shift)  for a arbitrary 
 *                   long of length size 16 bits.  Max shift: 15 bits.
 ******************************************************************************
 * DESCRIPTION: Calculates dst = dst + (src << shift)  for a arbitrary long 
 *              of length size 16 bits.  Max shift: 15 bits.
 *              dst and src cannot be the same.
 *
 * void utlLongAdd(
 *          tuint *dst,  - A pointer to result array
 *          tuint *src,  - A pointer to operand array
 *          tuint shift, - Shift value
 *          tuint size)  - Size of the array
 *
 *****************************************************************************/

void utlLongAdd (tuint *dst, tuint *src, tuint shift, tuint size)
{
  tint i;
  tulong temp;
  tulong shifted_src;

  temp        = 0; /* Init carry */
  shifted_src = 0;
  for (i = size - 1; i>=0; i--)
  {
    shifted_src >>= 16;
    shifted_src  |= ((tulong)src[i]) << shift;
    temp        >>= 16; /* Adjust carry */
    temp          = temp + (shifted_src & 0xFFFF) + ((tulong)dst[i]);
    dst[i] = (tuint)(temp & 0xFFFF);
  } 
} /* utlLongAdd */

/******************************************************************************
 * FUNCTION PURPOSE: Calculates dst = src * k for a arbitrary long of length
 *                   size 16 bits. dst and src cannot be same.
 ******************************************************************************
 * DESCRIPTION: Calculates dst = src * k for a arbitrary long of length
 *              size 16 bits. dst and src cannot be same.
 *
 * void utlLongMult(
 *          tuint *dst, - A pointer to result array
 *          tuint *src, - A pointer to operand array
 *          tuint k,    - Number to be mult by src array
 *          tuint size) - Size of the array
 *
 *****************************************************************************/

void utlLongMult (tuint *dst, tuint *src, tuint k, tuint size)
{
  tint shift; 
  tint i;

  for (i=0;i<size;i++)
     dst[i] = 0;

  shift = 0;
  while (k) {
    if (k & 1) {
       utlLongAdd (dst, src, shift, size);
    }

    k>>=1;
    shift++;
  }

} /* utlLongMult */

/* Nothing past this point */
