#!/bin/bash

# (C) Copyright 2001-2011 Texas Instruments Incorporated.
# * 
# *  Redistribution and use in source and binary forms, with or without 
# *  modification, are permitted provided that the following conditions 
# *  are met:
# *
# *    Redistributions of source code must retain the above copyright 
# *    notice, this list of conditions and the following disclaimer.
# *
# *    Redistributions in binary form must reproduce the above copyright
# *    notice, this list of conditions and the following disclaimer in the 
# *    documentation and/or other materials provided with the   
# *    distribution.
# *
# *    Neither the name of Texas Instruments Incorporated nor the names of
# *    its contributors may be used to endorse or promote products derived
# *    from this software without specific prior written permission.
# *
# *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
# *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
# *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
# *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
# *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
# *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
# *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
# *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
# *
# */

# Note: ccperl cannot run the reflection page script 
# (clrcase/refpage/refpage.pl) becaues it has a bug in its regexp engine
# that crashes on some of the complicated regexp in the script.

# If you wish to run perl from your local hard drive, you must relocate
# it there.  To relocate perl, go to t:/gen/perl/activestate/5_14_2_1402/bin,
# and run
#
# reloc_perl -t -a c:/tools/gen/perl/activestate/5_14_2_1402 t:/gen/perl/activestate/5_14_2_1402 
#
# This will relocate (install) perl in c:/tools for you.
#
# You can then change the following two paths point to c:/tools/gen/perl/activestate/5_14_2_1402.

# Regex to convert between mingw and DOS drive letters (/c/ vs c:/)
make_dos_sed_cmd='sed s/^\/\([a-zA-Z]\)\//\1:\//'

function invalid_argument {
cat << MSGEND
============================== INVALID ARGUMENT ===============================
Option "$1" is invalid.  Valid options are:
"bypass":       configure environment without checking tools
"enable_write": *****ERASE***** all files in local tools repository
and replace with correct tools
============================== INVALID ARGUMENT ===============================
MSGEND
}

# Argument is the shell variable name containing the path to convert
# Result is returned in same variable
function make_shortname {
  # Compute shortname
  local results=`"$MAKE_SHORTNAME" "\${!1}"`
  # Convert drive letter
  results=`echo "$results" | sed 's/^\([a-zA-Z]\):/\/\U\1/'`
  eval $1=\"$results\"
}

function rtsc_verify_msg {
cat << MSGEND
******************************************************************************
Note: Each dependant RTSC package will also do $1 tools.  To stop this, 
re-run this script with "bypass" argument
******************************************************************************
MSGEND
}

function check_exist {
  local pathname="${!1}$2";

  if [ ! -e "$pathname" ] 
  then
    let "errors=$errors+1"
    echo "Variable $1=${!1} doesn't point to a valid tool ($2)"
  fi
}

# This sets up the MINGW/MSYS path and version.  This must be done
# before this script can use any unix functionality like if [ ] or 
# sed.
#
# Since this function operates before tools are copied, it must put
# the SRCPATH into the PATH.
function set_msys_path {
  errors=0

  # Note that MINGW_RUNPATH should NOT have spaces in the name.  The mingw
  # installer puts it in c:\MinGW by default.
  MINGW_VERSION="20111117"
  MINGW_RUNPATH="/c/mingw"
  MINGW_SRCPATH="$MINGW_RUNPATH"
  # Set a temporary path pointing only at mingw.  The rest of the path will
  # be built later.  This allows [.exe and sed.exe to be found for
  # the rest of this function to run
  export PATH="$MINGW_SRCPATH/msys/1.0/bin"
  export PATH="$PATH:$MINGW_SRCPATH/mingw32/bin"
  export PATH="$PATH:$MINGW_SRCPATH/bin"
  # Put windows in path for cmd.exe but need to change from C:\WINDOWs to
  # /c/WINDOWS.
  WINDIR_UNIX=`echo $WINDIR | sed -e 's/\\\\/\//g' -e 's/^\([a-zA-Z]\):/\/\1/'`
  export PATH="$PATH:$WINDIR_UNIX:$WINDIR_UNIX/system32"
  check_exist "MINGW_SRCPATH" "/bin/gcc.exe"
  check_exist "MINGW_SRCPATH" "/msys/1.0/bin/make.exe"

  # Note that [ requires the path to be set its actually a program [.exe
  if [ $errors -eq 0 ] 
  then
    return 0 # success -- also imples [.exe found in path
  fi
  # could say 0 errors found if [.exe not in path even though
  # that is also an error
  echo "Environment not configured.  $errors errors found"
  exit 1
}

function set_tools_versions {
  errors=0
  # Factor out a common installation location.  TOOLSBASE# just
  # represents different common locations, the # itself doesn't
  # mean anything
  TOOLSBASE1="/c/Program Files/Texas Instruments"

  # Tools can be automatically synchronized locally in order to improve build
  # time.  For each tool there are two variables:
  #
  # XXX_RUNPATH : location where the tool should run from
  # XXX_SRCPATH : location where the tool should be copied from
  #
  # If XXX_RUNPATH==XXX_SRCPATH the tool is used "in-place" and will never
  # be copied.  If XXX_RUNPATH!=XXX_SRCPATH, then XXX_SRCPATH will be
  # copied/synchronized with XXX_RUNPATH.  In this case XXX_RUNPATH would
  # be a local storage location.
  
  # Start with blank list of tools that could be copied
  COPY_TOOLS_LIST=

  # For each tool, if SRCPATH != RUNPATH then the tool will be copied 
  # from SRCPATH to RUNPATH to improve build time, if argument "enable_write"
  # is supplied to command line
  COPY_TOOLS_LIST="$COPY_TOOLS_LIST MINGW"

  # PERL doesn't support relocation by simple copy, see instructions above
  # therefore exclude PERL from COPY_TOOLS_LIST (and SRCPATH)
  PERL_VERSION="5_14_2_1402"
  PERL_RUNPATH="/c/perl/bin/perl"
  check_exist "PERL_RUNPATH" ".exe"

  # XDC build tools
  XDC_VERSION="xdctools_3_22_04_46"
  XDC_SRCPATH="$TOOLSBASE1/$XDC_VERSION"
  MAKE_SHORTNAME="$XDC_SRCPATH/packages/xdc/services/io/release/path2dos.exe"
  make_shortname "XDC_SRCPATH"
  XDC_RUNPATH="$XDC_SRCPATH"
  check_exist "XDC_SRCPATH" "/xdc.exe"
  # Used to find windows short names.  This gets rid of the spaces in 
  # c:\program files which makes mingw happy with the filenames.
  COPY_TOOLS_LIST="$COPY_TOOLS_LIST XDC"

  # Path/version for SYS/BIOS 6
  BIOS6_VERSION="bios_6_32_05_54"
  BIOS6_RUNPATH="$TOOLSBASE1/$BIOS6_VERSION"
  make_shortname "BIOS6_RUNPATH"
  BIOS6_SRCPATH="$BIOS6_RUNPATH"
  check_exist "BIOS6_SRCPATH" "/packages/ti/sysbios/BIOS.c"
  COPY_TOOLS_LIST="$COPY_TOOLS_LIST BIOS6"

  # Path/version for IPC
  IPC_VERSION="ipc_1_24_00_16"
  IPC_RUNPATH="$TOOLSBASE1/$IPC_VERSION"
  make_shortname "IPC_RUNPATH"
  IPC_SRCPATH="$IPC_RUNPATH"
  check_exist "IPC_SRCPATH" "/packages/ti/ipc/package.xdc"
  COPY_TOOLS_LIST="$COPY_TOOLS_LIST IPC"

  # Path/version for EDMA3 LLD
  EDMA3_LLD_VERSION="edma3_lld_02_11_03_02"
  EDMA3_LLD_RUNPATH="$TOOLSBASE1/$EDMA3_LLD_VERSION"
  make_shortname "EDMA3_LLD_RUNPATH"
  EDMA3_LLD_SRCPATH="$EDMA3_LLD_RUNPATH"
  check_exist "EDMA3_LLD_SRCPATH" "/packages/ti/sdo/edma3/rm/package.xdc"
  COPY_TOOLS_LIST="$COPY_TOOLS_LIST EDMA3_LLD"

  # C6000 code generation tools (compiler)
  # This is different from the other tools, because the t: drive location
  # has different version specifier than c:\program files
  C6XCGEN_VERSION="C6000 Code Generation Tools 7.2.4"
  C6XCGEN_RUNPATH="$TOOLSBASE1/$C6XCGEN_VERSION"
  make_shortname "C6XCGEN_RUNPATH"
  C6XCGEN_SRCPATH="$C6XCGEN_RUNPATH"
  check_exist "C6XCGEN_SRCPATH" "/bin/cl6x.exe"
  COPY_TOOLS_LIST="$COPY_TOOLS_LIST C6XCGEN"

  FC_VERSION="framework_components_3_22_00_05"
  FC_RUNPATH="$TOOLSBASE1/$FC_VERSION"
  make_shortname "FC_RUNPATH"
  FC_SRCPATH="$FC_RUNPATH"
  check_exist "FC_SRCPATH" "/packages/ti/sdo/fc/utils/package.xdc"
  COPY_TOOLS_LIST="$COPY_TOOLS_LIST FC"

  XDAIS_VERSION="xdais_7_20_00_07"
  XDAIS_RUNPATH="$TOOLSBASE1/$XDAIS_VERSION"
  make_shortname "XDAIS_RUNPATH"
  XDAIS_SRCPATH="$XDAIS_RUNPATH"
  check_exist "XDAIS_SRCPATH" "/packages/ti/xdais/package.xdc"
  COPY_TOOLS_LIST="$COPY_TOOLS_LIST XDAIS"

  # Drivers for the C667X
  BIOS_MCSDK_PDK6678_VERSION="pdk_C6678_1_0_0_17"
  BIOS_MCSDK_PDK6678_RUNPATH="$TOOLSBASE1/$BIOS_MCSDK_PDK6678_VERSION"
  make_shortname "BIOS_MCSDK_PDK6678_RUNPATH"
  BIOS_MCSDK_PDK6678_SRCPATH="$BIOS_MCSDK_PDK6678_RUNPATH"
  check_exist "BIOS_MCSDK_PDK6678_SRCPATH" "/packages/ti/drv/pdksetupenv.bat"
  COPY_TOOLS_LIST="$COPY_TOOLS_LIST BIOS_MCSDK_PDK6678"

  # Drivers for NWAL
  NWAL_C66X_VERSION="nwal_c66x_1_0_0_1"
  NWAL_C66X_RUNPATH="$TOOLSBASE1/$NWAL_C66X_VERSION"
  make_shortname "NWAL_C66X_RUNPATH"
  NWAL_C66X_SRCPATH="$NWAL_C66X_RUNPATH"
  check_exist "NWAL_C66X_SRCPATH" "/ti/drv/nwal/setupenv.bat"
  COPY_TOOLS_LIST="$COPY_TOOLS_LIST NWAL_C66X"

  #####################################################
  # Video Codecs start                                #
  #####################################################
  # specify the base directory for video codecs
  VIDEOBASE="/c/Program Files/Texas Instruments/Codecs"

  # MPEG4 encoder
  VIDEO_MPEG4_ENC_VERSION="C66x_mp4venc_02_04_00_00_ELF"
  VIDEO_MPEG4_ENC_RUNPATH="$VIDEOBASE/$VIDEO_MPEG4_ENC_VERSION"
  make_shortname "VIDEO_MPEG4_ENC_RUNPATH"
  VIDEO_MPEG4_ENC_SRCPATH="$VIDEO_MPEG4_ENC_RUNPATH"
  check_exist "VIDEO_MPEG4_ENC_SRCPATH" "/packages/ti/sdo/codecs/mp4venc/imp4venc.h"
  COPY_TOOLS_LIST="$COPY_TOOLS_LIST VIDEO_MPEG4_ENC"

  # MPEG4 decoder
  VIDEO_MPEG4_DEC_VERSION="C66x_m4h3dec_02_03_00_01_ELF"
  VIDEO_MPEG4_DEC_RUNPATH="$VIDEOBASE/$VIDEO_MPEG4_DEC_VERSION"
  make_shortname "VIDEO_MPEG4_DEC_RUNPATH"
  VIDEO_MPEG4_DEC_SRCPATH="$VIDEO_MPEG4_DEC_RUNPATH"
  check_exist "VIDEO_MPEG4_DEC_SRCPATH" "/packages/ti/sdo/codecs/m4h3dec/im4h3dec.h"
  COPY_TOOLS_LIST="$COPY_TOOLS_LIST VIDEO_MPEG4_DEC"

  # MPEG2 decoder
  VIDEO_MPEG2_DEC_VERSION="C66x_mpeg2vdec_02_02_00_02_ELF"
  VIDEO_MPEG2_DEC_RUNPATH="$VIDEOBASE/$VIDEO_MPEG2_DEC_VERSION"
  make_shortname "VIDEO_MPEG2_DEC_RUNPATH"
  VIDEO_MPEG2_DEC_SRCPATH="$VIDEO_MPEG2_DEC_RUNPATH"
  check_exist "VIDEO_MPEG2_DEC_SRCPATH" "/packages/ti/sdo/codecs/mpeg2vdec/impeg2vdec.h"
  COPY_TOOLS_LIST="$COPY_TOOLS_LIST VIDEO_MPEG2_DEC"

  # H264BP encoder
  VIDEO_H264_ENC_VERSION="C66x_h264venc_01_24_00_01_ELF"
  VIDEO_H264_ENC_RUNPATH="$VIDEOBASE/$VIDEO_H264_ENC_VERSION"
  make_shortname "VIDEO_H264_ENC_RUNPATH"
  VIDEO_H264_ENC_SRCPATH="$VIDEO_H264_ENC_RUNPATH"
  check_exist "VIDEO_H264_ENC_SRCPATH" "/packages/ti/sdo/codecs/h264venc/ih264venc.h"
  COPY_TOOLS_LIST="$COPY_TOOLS_LIST VIDEO_H264_ENC"

  # H264BP-MP decoder
  VIDEO_H264_DEC_VERSION="C66x_h264vdec_02_04_00_03_ELF"
  VIDEO_H264_DEC_RUNPATH="$VIDEOBASE/$VIDEO_H264_DEC_VERSION"
  make_shortname "VIDEO_H264_DEC_RUNPATH"
  VIDEO_H264_DEC_SRCPATH="$VIDEO_H264_DEC_RUNPATH"
  check_exist "VIDEO_H264_DEC_SRCPATH" "/packages/ti/sdo/codecs/h264vdec/ih264vdec.h"
  COPY_TOOLS_LIST="$COPY_TOOLS_LIST VIDEO_H264_DEC"

  # H264HP decoder
  VIDEO_H264HP_DEC_VERSION="C66x_h264hpvdec_01_01_00_01_ELF"
  VIDEO_H264HP_DEC_RUNPATH="$VIDEOBASE/$VIDEO_H264HP_DEC_VERSION"
  make_shortname "VIDEO_H264HP_DEC_RUNPATH"
  VIDEO_H264HP_DEC_SRCPATH="$VIDEO_H264HP_DEC_RUNPATH"
  check_exist "VIDEO_H264HP_DEC_SRCPATH" "/packages/ti/sdo/codecs/h264hpvdec/ih264hpvdec.h"
  COPY_TOOLS_LIST="$COPY_TOOLS_LIST VIDEO_H264HP_DEC"
  #####################################################
  # Video Codecs end                                  #
  #####################################################

  DOXYGEN_STUB="YES"

  if [ "$DOXYGEN_STUB" = "YES" ]
  then
    DOXYGEN_RUNPATH=`echo $(readlink -m ../clrcase/doxygen_stub) | $make_dos_sed_cmd`
    DOXYGEN_SRCPATH="$DOXYGEN_RUNPATH"
    make_shortname "DOXYGEN_RUNPATH"
    if ! gcc -o $DOXYGEN_RUNPATH/bin/doxygen.exe $DOXYGEN_RUNPATH/bin/doxygen.c
    then
      echo "Failed to compile doxygen stub"
      exit
    fi
    TITEMPLATES_RUNPATH="$DOXYGEN_RUNPATH" 
  else
    # Following tool versions are used for Doxygen support
    DOXYGEN_VERSION="1.5.1-p1"
    DOXYGEN_RUNPATH="/t/Doxygen/doxygen/$DOXYGEN_VERSION"
    DOXYGEN_SRCPATH="$DOXYGEN_RUNPATH"
    check_exist "DOXYGEN_SRCPATH" "/bin/doxygen.exe"
    COPY_TOOLS_LIST="$COPY_TOOLS_LIST DOXYGEN"

    GRAPHVIZ_VERSION="2.12"
    GRAPHVIZ_RUNPATH="/t/Doxygen/Graphviz/$GRAPHVIZ_VERSION"
    GRAPHVIZ_SRCPATH="$GRAPHVIZ_RUNPATH"
    check_exist "GRAPHVIZ_SRCPATH" "/bin/GVUI.exe"
    COPY_TOOLS_LIST="$COPY_TOOLS_LIST GRAPHVIZ"

    HTMLHELP_VERSION="10-01-2007"
    HTMLHELP_RUNPATH="/t/Doxygen/HTML_Help_Workshop/$HTMLHELP_VERSION"
    HTMLHELP_SRCPATH="$HTMLHELP_RUNPATH"
    check_exist "HTMLHELP_SRCPATH" "/hhw.exe"
    COPY_TOOLS_LIST="$COPY_TOOLS_LIST HTMLHELP"

    TITEMPLATES_VERSION="10-01-2007"
    TITEMPLATES_RUNPATH="/t/Doxygen/TI_Templates/$TITEMPLATES_VERSION"
    TITEMPLATES_SRCPATH="$TITEMPLATES_RUNPATH"
    check_exist "TITEMPLATES_SRCPATH" "/tilogo.gif"
    COPY_TOOLS_LIST="$COPY_TOOLS_LIST TITEMPLATES"
  fi



  CPYTOOLS="../clrcase/cpytools/cpytools.pl"

  if [ $errors -gt 0 ] 
  then
    echo "Environment not configured.  $errors errors found"
    return 1
  fi
  return 0
}

# This function is called after all tools have been copied (if necessary).
# This means that all paths should depend on RUNPATH
function set_public_paths {
  # Wrapper perl script for sending xdc command properly to RTSC component
  export RTSC_INTEGRATE="$PERL_RUNPATH ../../clrcase/rtsc_integrate/rtsc.pl"

  # Wrapper perl script for sending xdc command properly to Unzip the Video Codec Releases
  export EXTRACT_ZIP_PERL_SCRIPT="$PERL_RUNPATH ../../clrcase/extract_zip/extractZip.pl"
  export UNZIPEXE="$XDC_RUNPATH/bin/unzip.exe"
  
  # Set a defined path which has no other compilers or unix tools
  # the makefiles will specify tools with their whole path, but the tools
  # require themselves to be in the path to work right.
  #
  # This is done because windows does not hash the path, so path search
  # performance is very poor.  Lets avoid path searches.
  #
  export PATH="$MINGW_RUNPATH/msys/1.0/bin"
  export PATH="$PATH:$MINGW_RUNPATH/mingw32/bin"
  export PATH="$PATH:$MINGW_RUNPATH/bin"
  # Put windows in path for cmd.exe
  # WINDIR_UNIX comes from set_msys_path
  export PATH="$PATH:$WINDIR_UNIX:$WINDIR_UNIX/system32"
  export PATH="$PATH:$C6XCGEN_RUNPATH/bin"
  export PATH="$PATH:$COV_RUNPATH/bin"
  export PATH="$PATH:$XDC_RUNPATH"
  export PATH="$PATH:$XDC_RUNPATH/bin"
  # Add Doxygen path (used by ti/mas)
  export PATH="$PATH:$DOXYGEN_RUNPATH/bin"
  export PATH="$PATH:$GRAPHVIZ_RUNPATH/bin"
  export PATH="$PATH:$HTMLHELP_RUNPATH/$HTML_HELP_WORKSHOP_BASE"
  export PATH="$PATH:$INSTALLJAMMER_RUNPATH"

  # XDCPATH for BIOS 6
  export XDC='xdc.exe $*'
  export XDCPATH="`echo $BIOS6_RUNPATH | $make_dos_sed_cmd`/packages"
  export XDCPATH="$XDCPATH;`echo $IPC_RUNPATH | $make_dos_sed_cmd`/packages"
  export XDCPATH="$XDCPATH;`echo $XDC_RUNPATH | $make_dos_sed_cmd`/packages"
  export XDCPATH="$XDCPATH;`echo $FC_RUNPATH | $make_dos_sed_cmd`/packages"
  export XDCPATH="$XDCPATH;`echo $EDMA3_LLD_RUNPATH | $make_dos_sed_cmd`/packages"

  # XDAIS
  export XDCPATH="$XDCPATH;`echo $XDAIS_RUNPATH | $make_dos_sed_cmd`/packages"

  # Clean up variables which affect the tools to avoid side effects
  export C54X_A_DIR=
  export C54X_C_DIR=
  export C55X_A_DIR=
  export C55X_C_DIR=
  export C6X_C_DIR=
  export C6X_A_DIR=
  export A_DIR=
  export C_DIR=

  # Path to tools, in unix format
  export TOOLSC6XCGEN="$C6XCGEN_RUNPATH"

  # BIOS tools UNIX format
  export TOOLSBIOS="$BIOS6_RUNPATH/packages"
  export TOOLSBIOSTCF="$BIOS6_RUNPATH/xdctools"
  export TOOLSBIOS6XS="$XDC_RUNPATH"

  # ###################################################################
  # rem DO NOT MERGE TO OTHER VERTICALS -- PICK YOUR OWN SERVER/PRODUCT
  export covhost="158.218.108.140"
  export covproduct="HDGW"
  export covport="30401"
  # rem ###############################################################

  # Make DOS versions of variables that are interpreted by non-mingw tools
  # These tools require drive letters with "d:" instead of /d/.  Mingw tools
  # can't accept the d: because : is the search path delimiter

  # This regex converts /d/ to d:/
  export TOOLSXDCDOS="`echo $XDC_RUNPATH | $make_dos_sed_cmd`"
  export TOOLSC6XCGENDOS="`echo $C6XCGEN_RUNPATH | $make_dos_sed_cmd`"
  export TOOLSBIOSDOS="`echo $BIOS6_RUNPATH | $make_dos_sed_cmd`/packages"

  # paths for PERL
  export PERL="$PERL_RUNPATH"
  export PERLDOS="`echo $PERL_RUNPATH | $make_dos_sed_cmd`"

  # paths for XDAIS
  export XDAISPATHDOS="`echo $XDAIS_RUNPATH | $make_dos_sed_cmd`/packages"

  # paths for PDK
  export BIOS_MCSDK_PDK6678_DOS="`echo $BIOS_MCSDK_PDK6678_RUNPATH | $make_dos_sed_cmd`/packages"

  # paths for NWAL
  export NWAL_C66X_DOS="`echo $NWAL_C66X_RUNPATH | $make_dos_sed_cmd`"

  # Definitions of same tools above for ti/mas use
  # codegen same for c6xx
  export C64CODEGENTOOL=$TOOLSC6XCGENDOS
  export C66CODEGENTOOL=$TOOLSC6XCGENDOS

  # Find dsps_gtmas for ti/mas xdc scripts
  MAS_INSTALL_DIR=`echo $(readlink -m ../../components) | $make_dos_sed_cmd`
  make_shortname "MAS_INSTALL_DIR"
  export MAS_INSTALL_DIR  
  export XDCBUILDCFG="$MAS_INSTALL_DIR/ti/mas/swtools/config.bld"
  export XDCPATH="$XDCPATH;$MAS_INSTALL_DIR"

  # TI doxygen templates for ti/mas
  export TI_DOXYGEN_TEMPLATES="`echo $TITEMPLATES_RUNPATH | $make_dos_sed_cmd`"

  # paths for video codecs
  export VIDEO_MPEG4_ENC_DOS="`echo $VIDEO_MPEG4_ENC_RUNPATH | $make_dos_sed_cmd`/packages"
  export VIDEO_MPEG4_DEC_DOS="`echo $VIDEO_MPEG4_DEC_RUNPATH | $make_dos_sed_cmd`/packages"
  export VIDEO_MPEG2_DEC_DOS="`echo $VIDEO_MPEG2_DEC_RUNPATH | $make_dos_sed_cmd`/packages"
  export VIDEO_H264_ENC_DOS="`echo $VIDEO_H264_ENC_RUNPATH | $make_dos_sed_cmd`/packages"
  export VIDEO_H264_DEC_DOS="`echo $VIDEO_H264_DEC_RUNPATH | $make_dos_sed_cmd`/packages"
  export VIDEO_H264HP_DEC_DOS="`echo $VIDEO_H264HP_DEC_RUNPATH | $make_dos_sed_cmd`/packages"

}

function tools_copy {
  # ************************* Tools Copy/Validation **************************
  # If PERL is not available, treat as bypass
  # *********************************************************************
  if [ "$PERL_RUNPATH" = "" ]; then
    export RTSC_EANBLE_WRITE=
    return 0
  fi

  # ************************* Tools Copy/Validation **************************
  # If the "bypass" option is not provided, validate/copy the tools
  # *********************************************************************
  if [ "$1" = "bypass" ] 
  then
    export RTSC_EANBLE_WRITE=
    return 0
  elif [ "$1" = "" ] 
  then
    # check, but don't copy tools
    export RTSC_ENABLE_WRITE="verify"
    rtsc_verify_msg verify
  elif [ "$1" = "enable_write" ]
  then
    export RTSC_ENABLE_WRITE="enable_write"
    rtsc_verify_msg enable_write
  else
    invalid_argument $1
    return 1
  fi

  # Iterate over each tool in COPY_TOOLS_LIST.  If the tool's _RUNPATH !=
  # _SRCPATH, then invoke the copy script for the tool.
  fail=0
  for tool in $COPY_TOOLS_LIST
  do
    # the ! "dereferences" a variable containing name of another variable
    this_rundir="$tool"_RUNPATH
    this_rundir=${!this_rundir}
    this_srcdir="$tool"_SRCPATH
    this_srcdir=${!this_srcdir}
    if [ "$this_rundir" != "$this_srcdir" ]
    then
      # convert to DOS drive letter
      this_rundir=`echo $this_rundir | $make_dos_sed_cmd`
      this_srcdir=`echo $this_srcdir | $make_dos_sed_cmd`
      $PERL_RUNPATH $CPYTOOLS $this_srcdir $this_rundir $1
    fi
  done
  return 0
}

# Set path to msys/mingw so the shell dependancies like [.exe, cat.exe, sed.exe
# can be found
set_msys_path
echo "Please wait... configuring environment"

# If script is not sourced, complain
if [[ `basename $0` != *bash* ]] 
then 
  cat <<END
Must source this script from inside bash with optional argument on command line
or invoke as in init script with optional argument in SETUPENV_ARG env var

Sourcing example:
bash-3.1$ . setupenvMsys.sh [arg]

Init example:
$ set SETUPENV_ARG=[arg]
$ bash --init-file setupenvMsys.sh
======================== FAIL TO SETUP ENVIRONMENT !!!  ========================
END
exit
fi

# Back up the PATH so it can be restored on failure
# The path will be temporarily pointed at only MSYS/MINGW by 
# set_tools_versions
ORIGPATH=$PATH

# Set the tool versions (locally, not seen by make)
if set_tools_versions 
then

  # If SETUPENV_ARG is undefined (but not blank) then use script argument
  # instead
  # This allows for two invocation methods
  # . setupenv.sh arg : arg is taken from command line
  # bash --init_file setupenv.sh : arg is taken from SETUPENV_ARG

  if [ "${SETUPENV_ARG-foo}" = "foo" ] ; then SETUPENV_ARG=$1 ; fi
  if ! tools_copy ${SETUPENV_ARG-$1}
  then
  cat << MSGEND
================================== BAD TOOLS ==================================
* Build environment is NOT configured.  You may rerun script with:
*
* $0 bypass
*    configure environment without checking tools.  This option is
*    is used when the user will manually configure the tools.
*
* $0 enable_write
*    [*** DANGEROUS OPTION ***]
*    This option
*        1) *** DANGER *** Recursively deletes all files in local tools
*        2) Copies correct tools from network share
*
================================== BAD TOOLS ==================================
MSGEND
  # Failed - restore path
  export PATH=$ORIGPATH
  else
    # export variables
    set_public_paths
    echo '===================== ENVIRONMENT SUCESSFULLY CONFIGURED ======================'
  fi
  unset SETUPENV_ARG

  # Makefiles don't use any predefined rules.  Thus disable them which
  # greatly speeds things up (each implicit rule is tested against everything 
  # in vpath!)
  alias make='make -r'
else
  # Failed - restore path
  export PATH=$ORIGPATH
fi
  
# Don't need backed up path any more
unset ORIGPATH

# Normal build doesn't use RTSC_TARREP
export RTSC_TARREP=

# Normal build doesn't use RTSC_RELATIVEPATH
export RTSC_RELATIVEPATH=

# Point to RTSC tar repository 
export RTSC_RELATIVEPATH=../components
