/****************************************************************************
 * FILE PURPOSE: Perfrom hardware abstraction layer initialization
 ****************************************************************************
 * FILE NAME: siuhal.c
 *
 * DESCRIPTION: This file contains functions that are used to initialize
 *              the HAL. They are called only once at system startup
 *              and so can be overlayed with an uninitialized data section.
 *
 * TABS: None
 *
 * Copyright (C) 2002 Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include "types.h"
#include "ggcfg.h"
#include "ggmbuf.h"
#include "common/siuloc.h"
#include "hal.h"
#include "halloc.h"
#include <stdlib.h>
#include "gghwcfg.h"
#include "ndu.h"

/* low-level hardware files */
#include "targethw.h"

#include <ti/csl/csl_cpIntcAux.h>

/* CSL EMAC include */
#include <ti/csl/csl_cpsw.h>
#include <ti/csl/csl_cpsgmii.h>
#include <ti/csl/csl_cpsgmiiAux.h>
#include <ti/csl/cslr_cpsgmii.h>

extern halCacheTag_t *_external_mem_ramset1_start;
extern halCacheTag_t *_external_mem_ramset2_start;
void siu_macport_config(void);
void siu_invalidate_cache (void);
//void siuIntcConfig(void);

#define SIU_PORT_ALIGN_HEAPBASE(x)   (x)=(((x)+3)&0xfffc)

/******************************************************************************
 * FUNCTION PURPOSE: Initialize HAL APIs and mem config.
 ******************************************************************************
 * DESCRIPTION: The HAL APIs are initialized and memory config is performed.
 *****************************************************************************/
siuReturn_t siu_init_hal (const tint *marRegs, const tint *marVals)
{
 
  halMemConfig_t halMemCfg;
  thwCriticalBegin();

  siuContext.halApi = &hal;
  
  if (*siuContext.halApi->halMemConfig != NULL)
  {
    halMemCfg.memCfgFunction       = HAL_MEMCFGFUNC_CACHECONFIG;
    halMemCfg.u.cacheCfg.cacheType = HAL_CACHE_L1D;
    halMemCfg.u.cacheCfg.cacheMode = HAL_L1D_CACHE_MODE; /* gghwcfg.h specified cache config
                                                            for the build */
  
    (*siuContext.halApi->halMemConfig)(&halMemCfg);

    halMemCfg.memCfgFunction       = HAL_MEMCFGFUNC_CACHECONFIG;
    halMemCfg.u.cacheCfg.cacheType = HAL_CACHE_L1P;
    halMemCfg.u.cacheCfg.cacheMode = HAL_L1P_CACHE_MODE; /* gghwcfg.h specified cache config
                                                            for the build */
  
    (*siuContext.halApi->halMemConfig)(&halMemCfg);

    halMemCfg.memCfgFunction       = HAL_MEMCFGFUNC_CACHECONFIG;
    halMemCfg.u.cacheCfg.cacheType = HAL_CACHE_L2;
    halMemCfg.u.cacheCfg.cacheMode = HAL_L2_CACHE_MODE;/* gghwcfg.h specified cache config
                                                          for the build */
    (*siuContext.halApi->halMemConfig)(&halMemCfg);
    
    halMemCfg.memCfgFunction       = HAL_MEMCFGFUNC_CACHECONFIG;
    halMemCfg.u.cacheCfg.cacheType = HAL_CACHE_MAR;
#ifdef HAL_MAR_DYNAMIC_CFG
    halMemCfg.u.cacheCfg.marRegs   = (tint *)marRegs;
    halMemCfg.u.cacheCfg.marVals   = (tint *)marVals;
#else
    halMemCfg.u.cacheCfg.marRegs   = NULL;
    halMemCfg.u.cacheCfg.marVals   = NULL;
#endif
    (*siuContext.halApi->halMemConfig)(&halMemCfg);

/*  The following has been moved to startup_reset() to configure MPAX as earliest as possible */   
/*  halMemCfg.memCfgFunction       = HAL_MEMCFGFUNC_XMC_MPAX_CONFIG;
    if (DNUM == 0) {
      halMemCfg.memCfgFunction    |= HAL_MEMCFGFUNC_SES_MPAX_CONFIG;
      halMemCfg.memCfgFunction    |= HAL_MEMCFGFUNC_SMS_MPAX_CONFIG;
    }
    (*siuContext.halApi->halMemConfig)(&halMemCfg);
*/
    
  }  
  else
  {
    return(SIU_FAIL);
  }
  
  thwCriticalEnd();

  return (SIU_SUCCESS);
} /* siu_init_hal */


siuReturn_t siu_set_mar (const tint *marRegs, const tint *marVals)
{
  halMemConfig_t halMemCfg;
  siuContext.halApi = &hal;

  thwCriticalBegin();

  if (*siuContext.halApi->halMemConfig != NULL)
  {
    halMemCfg.memCfgFunction       = HAL_MEMCFGFUNC_CACHECONFIG;
    halMemCfg.u.cacheCfg.cacheType = HAL_CACHE_MAR;
#ifdef HAL_MAR_DYNAMIC_CFG
    halMemCfg.u.cacheCfg.marRegs   = (tint *)marRegs;
    halMemCfg.u.cacheCfg.marVals   = (tint *)marVals;
#else
    halMemCfg.u.cacheCfg.marRegs   = NULL;
    halMemCfg.u.cacheCfg.marVals   = NULL;
#endif
    (*siuContext.halApi->halMemConfig)(&halMemCfg);
  }

  thwCriticalEnd();

  return (SIU_SUCCESS);
}

void siu_invalidate_cache (void)
{
 
  halCacheControl_t halCacheControl;

  halCacheControl.cacheCtrl  = HAL_CACHE_CTRL_FLUSH;
  halCacheControl.cacheType = HAL_CACHE_ALL;

  if(*siuContext.halApi->halCacheControl != NULL)
  {
    (*siuContext.halApi->halCacheControl)(&halCacheControl);
   
  }
} /* siu_invalidate_cache */

/**************************************************************************** 
* This function should be called after diagApp structure is ready by micro
* and copied to local memory, ie after siuNduStart() is over. 
*****************************************************************************/   
siuReturn_t siu_periph_setup (void)
{
#ifdef NDU_GMAC
  /* Mac port enable */
  if (ggConfig.nGmacPorts > 0)
    siu_macport_config();
#endif

  return (SIU_SUCCESS);

} /* siu_periph_setup */

#ifdef NDU_GMAC
/*****************************************************************************
 * FUNCTION PURPOSE: Configure and enable the first mac port
 *****************************************************************************
 * DESCRIPTION: Allocates mac memory, configures and enables the first 
 *              (message) port.
 *****************************************************************************/
void siu_macport_config(void)
{
  /* TBD: Still need to confiure PA/NEU for Message Port */
#warn Need to configure message port routing

} /* dsiu_macport_config */

/** ============================================================================
 *   @n@b Init_SGMII
 *
 *   @b Description
 *   @n SGMII peripheral initialization code.
 *
 *   @param[in]  
 *   @n macPortNum      MAC port number for which the SGMII port setup must
 *                      be performed.
 * 
 *   @return
 *   @n None
 * =============================================================================
 */
Int32 Init_SGMII (UInt32 macPortNum, UInt16 fLoopback)
{  
    CSL_SGMII_ADVABILITY    sgmiiCfg;
	CSL_SGMII_STATUS        sgmiiStatus;
    
#warn need to configure SGMII port based on the configuarion parameters    
    
    if (macPortNum == 0)
    {
        /* Shannon EVM back end: MAC-to-MAC force link */

        /* Reset the port before configuring it */
        CSL_SGMII_doSoftReset (macPortNum);        
        while (CSL_SGMII_getSoftResetStatus (macPortNum) != 0);   

        /* Hold the port in soft reset and set up
        * the SGMII control register:
        *      (1) Enable Master Mode (default)
        */
        CSL_SGMII_startRxTxSoftReset (macPortNum);  
        CSL_SGMII_enableMasterMode (macPortNum);
        if (fLoopback)
        {
	        CSL_SGMII_enableLoopback (macPortNum);
        }    
        
	    /* Setup the Advertised Ability register for this port:
        *      (1) Enable Full duplex mode
        *      (2) Speed = 1000M
        *      (3) Force the Link
        */
        sgmiiCfg.bLinkUp        =   1;
        sgmiiCfg.linkSpeed      =   CSL_SGMII_1000_MBPS;
        sgmiiCfg.duplexMode     =   CSL_SGMII_FULL_DUPLEX;
        CSL_SGMII_setAdvAbility (macPortNum, &sgmiiCfg);
        
        CSL_SGMII_endRxTxSoftReset (macPortNum);   
    
	    /* Wait for SGMII Link */
	    do
	    {
	        CSL_SGMII_getStatus(macPortNum, &sgmiiStatus);
	    } while (sgmiiStatus.bIsLinkUp != 1);
    }
    else 
    {
        /* Reset the port before configuring it */
        CSL_SGMII_doSoftReset (macPortNum);        
        while (CSL_SGMII_getSoftResetStatus (macPortNum) != 0);   

        /* Hold the port in soft reset and set up
        * the SGMII control register:
        *      (1) Enable Master Mode (default)
        *      (2) Enable Auto-negotiation
        */
        CSL_SGMII_startRxTxSoftReset (macPortNum);  
        CSL_SGMII_enableMasterMode (macPortNum);
        if (fLoopback)
        {
	        CSL_SGMII_enableLoopback (macPortNum);
        }    
    
	    /* Setup the Advertised Ability register for this port:
        *      (1) Enable Full duplex mode
        *      (2) Enable Auto Negotiation
        */
        sgmiiCfg.linkSpeed      =   CSL_SGMII_1000_MBPS;
        sgmiiCfg.duplexMode     =   CSL_SGMII_FULL_DUPLEX;
        CSL_SGMII_setAdvAbility (macPortNum, &sgmiiCfg);
    
        CSL_SGMII_enableAutoNegotiation (macPortNum);
        CSL_SGMII_endRxTxSoftReset (macPortNum);   
    
	    /* Wait for SGMII Link */
	    do
	    {
	        CSL_SGMII_getStatus(macPortNum, &sgmiiStatus);
	    } while (sgmiiStatus.bIsLinkUp != 1);
	
	    /* Wait for SGMII Autonegotiation to complete without error */
	    do
	    {
	        CSL_SGMII_getStatus(macPortNum, &sgmiiStatus);
	        if (sgmiiStatus.bIsAutoNegError != 0)
	            return -1;
	    } while (sgmiiStatus.bIsAutoNegComplete != 1);
    }

    /* All done with configuration. Return Now. */
    return 0;
}

/** ============================================================================
 *   @n@b Switch_update_addr
 *
 *   @b Description
 *   @n This API add/delete entries in the Address Lookup Engine (ALE) in "Switch" mode.
 *
 *   @param[in]  
 *   @n portNum         Switch port number.
 
 *   @param[in]  
 *   @n macAddress      MAC address to configure on the switch.
 * 
 *   @param[in]  
 *   @n add             0:add; 1:delete.
 *
 *   @return
 *   @n None
 *
 *   @Note  It supports "add" operation only now.           
 * =============================================================================
 */
int Switch_update_addr (UInt32 portNum, UInt8 macAddress[6], UInt16 add, UInt16 loopback)
{
    UInt32                              i;
    CSL_CPSW_3GF_ALE_PORTCONTROL        alePortControlCfg;
    CSL_CPSW_3GF_ALE_UNICASTADDR_ENTRY  ucastAddrCfg;


    /* Configure the address in "Learning"/"Forward" state */
    alePortControlCfg.portState             =   ALE_PORTSTATE_FORWARD;
    alePortControlCfg.dropUntaggedEnable    =   0;
    alePortControlCfg.vidIngressCheckEnable =   0;
    alePortControlCfg.noLearnModeEnable     =   (loopback)?1:0;
    alePortControlCfg.mcastLimit            =   0;
    alePortControlCfg.bcastLimit            =   0;

    CSL_CPSW_3GF_setAlePortControlReg (portNum, &alePortControlCfg);

    if (loopback)
    {
        /* Program the ALE with the MAC address.
        *
        * The ALE entries determine the switch port to which any
        * matching received packet must be forwarded to.
        */
        /* Get the next free ALE entry to program */
        for (i = 0; i < CSL_CPSW_3GF_NUMALE_ENTRIES; i++)
        {
            if (CSL_CPSW_3GF_getALEEntryType (i) == ALE_ENTRYTYPE_FREE)
            {
                /* Found a free entry */
                break;                    
            }
        }
        if (i == CSL_CPSW_3GF_NUMALE_ENTRIES)
        {
            /* No free ALE entry found. return error. */            
            return -1;            
        }
        else
        {
            /* Found a free ALE entry to program our MAC address */            
            memcpy (ucastAddrCfg.macAddress, macAddress, 6);    // Set the MAC address
            ucastAddrCfg.ucastType      =      ALE_UCASTTYPE_UCAST_NOAGE;   // Add a permanent unicast address entryALE_UCASTTYPE_UCAST_NOAGE.
            ucastAddrCfg.secureEnable   =      FALSE;   
            ucastAddrCfg.blockEnable    =      FALSE;   
            ucastAddrCfg.portNumber     =      portNum;   // Add the ALE entry for this port

            /* Setup the ALE entry for this port's MAC address */
            CSL_CPSW_3GF_setAleUnicastAddrEntry (i, &ucastAddrCfg);            
        }
    }

    /* Done with upading address */
    return 0;
}


/*************************************************************************************
 * FUNCTION PURPOSE: Perform initial MAC port configuration
 *************************************************************************************
 * DESCRIPTION: Link a physical port and logical port and configure with supplied
 *              MAC addresses
 *************************************************************************************/
bool siu_pktio_mac_init (struct MGB_MDS_PKTIO_IF_MAC_PARAMS *pktio_params)
{

  UInt16  fLoopback = pktio_params->cfg_bfield & MGB_MD_PKTIO_IF_MAC_LOOPBACK_EN;
  

  if(pktio_params->num_mac_addr > CSL_MAC_MAX_MAC_ADDR)
    return FALSE;
  
  Init_SGMII ((UInt32)pktio_params->port_num, fLoopback);
  
  if (fLoopback)
  {
      CSL_CPSW_3GF_enableAleBypass();
  }    
  
  return TRUE;
} /* siu_pktio_macport_config */


/****************************************************************************
 * FUNCTION PURPOSE: Add a MAC address for packet reception
 ****************************************************************************
 * DESCRIPTION: MAC addresses are added to the accept table for the port
 ****************************************************************************/
bool siu_pktio_mac_addr_mod (struct MGB_MDS_PKTIO_IF_MAC_PARAMS *pktio_params, tuint code)
{
  int              i, j;
  UInt8            macAddr[6];
  UInt32           macPort = pktio_params->port_num + 1;
  UInt16           fLoopback = pktio_params->cfg_bfield & MGB_MD_PKTIO_IF_MAC_LOOPBACK_EN;
  UInt16           fAdd = (code == HAL_GMAC_CTL_CODE_ADD_MAC_ADDR); 

  for (i = 0; i < pktio_params->num_mac_addr; i++)  {
    
    for (j = 0; j < 3; j++)
    {
        macAddr[2*j] = pktio_params->mac_addr[i].addr[j] >> 8;
        macAddr[2*j + 1] = pktio_params->mac_addr[i].addr[j] & 0xFF;
    }
    
    Switch_update_addr (macPort, macAddr, fAdd, fLoopback);
  }

  return (TRUE);
} /* siu_pktio_mac_addr_add */
    
/*****************************************************************************
 * FUNCTION PURPOSE: Configure the MAC port based on PKTIO_IF_CONFIG message
 *****************************************************************************
 * DESCRIPTION:Configures and enables the ports. Returns SUCCESS if there is
 *             no error in the configuration. Otherwise returns FALSE
 *****************************************************************************/
bool siu_pktio_macport_config(struct MGB_MDS_PKTIO_IF_MAC_PARAMS*   pktio_params)
{
  /* Allocate the memory if required */
  if (pktio_params->cfg_bfield & MGB_MD_PKTIO_IF_MAC_CFG_INIT)  {

    return (siu_pktio_mac_init (pktio_params));
  }

  if (pktio_params->cfg_bfield & MGB_MD_PKTIO_IF_MAC_CFG_ADDR)  {

    /* Make sure delete is not also specified */
    if (pktio_params->cfg_bfield & MGB_MD_PKTIO_IF_MAC_DEL_ADDR)
      return (FALSE);

    return (siu_pktio_mac_addr_mod (pktio_params, HAL_GMAC_CTL_CODE_ADD_MAC_ADDR));

  }

  if (pktio_params->cfg_bfield & MGB_MD_PKTIO_IF_MAC_DEL_ADDR)  {

    return (siu_pktio_mac_addr_mod (pktio_params, HAL_GMAC_CTL_CODE_DEL_MAC_ADDR));

  }
  
  return (TRUE);

} /* siu_pktio_macport_config */

#endif

/******************************************************************************
 * FUNCTION PURPOSE: Initialize HAL (Hardware Abstraction Layer)
 ******************************************************************************
 * DESCRIPTION: The HAL unit is configured and memory allocated. The
 *              PG revision is returned in revId, in Q.4 format
 *****************************************************************************/
siuReturn_t siu_new_hal (void)
{
  halNewConfig_t       cfg;  
  halMemBuffer_t      *bp;
  halMemBuffer_t      *bufs; 
  halReturn_t          ret;
  void                *bstart,*addr_unaligned;
  tuint               bsize;
  tint                 nbufs;

  /* Configure HAL using build time parameters */
  cfg.nTdmPorts  = ggConfig.nTdmPorts;
  cfg.nPktPorts  = ggConfig.nUtopiaPorts;
  cfg.nGmacPorts = ggConfig.nGmacPorts;
  cfg.nAicPorts  = ggConfig.nAicPorts;

#ifdef TSIP_LLD
  cfg.tsipSizeCfg.maxChannels  = ggConfig.max_channels + ggConfig.secChannels;
  cfg.tsipSizeCfg.subFrameSize = ggConfig.tdmConfig->subFrameSize[0]; 
  cfg.tsipSizeCfg.wordSize     = ggConfig.tdmConfig->wordSize[0];
#endif

  /* Get memory requirements from HAL  */
  ret = (*siuContext.halApi->halGetSizes) (&nbufs, &bufs, &cfg);
  siu_exc_assert(ret == HAL_SUCCESS,SIUHAL_EXC_HALGETSIZES,&siuContext);

#ifdef TSIP_LLD
  /* When using TSIP LLD, hal will always request the common TSIP DMA
   * buffers 
   */
  if (nbufs != 3)
    return (SIU_FAIL);
#endif

  bp = &bufs[HAL_HEAP_BUFN];
  if (bp->base == NULL)  {
    addr_unaligned=ggConfig.halHeapBufStart;
    bstart = siu_buffer_align (addr_unaligned, ggConfig.halHeapBufEnd, 
                               bp->log2align, &bsize);
    if (bsize < bp->size) {
      return SIU_FAIL;
    }

    bp->base = (void *)bstart;
    siuContext.halHeapUsed = bp->size+(int)((int)bstart-(int)addr_unaligned);
    SIU_PORT_ALIGN_HEAPBASE(siuContext.halHeapUsed);
  }

#ifdef TSIP_LLD
  /* Setting up the DMA buffers base for the TSIP LLD*/
  bp = &bufs[HAL_LLD_TDM_TX_BUFN];
  /* Allocate the tx buffer (only 1 buffer used) */
  if (bp->base == NULL)  {
    bstart = siu_buffer_align (ggConfig.tdmConfig->tdmDmaMem[0].txStart,
                               ggConfig.tdmConfig->tdmDmaMem[0].txEnd,
                               bp->log2align, &bsize);
    if (bsize < bp->size) {
      return SIU_FAIL;
    }
    bp->base = (void *)bstart;
  }

  /* Allocate the rx buffer (only 1 buffer used) */
  bp = &bufs[HAL_LLD_TDM_RX_BUFN];
  if (bp->base == NULL)  {
    bstart = siu_buffer_align (ggConfig.tdmConfig->tdmDmaMem[0].rxStart,
                               ggConfig.tdmConfig->tdmDmaMem[0].rxEnd,
                               bp->log2align, &bsize);
    if (bsize < bp->size) {
      return SIU_FAIL;
    }
    bp->base = (void *)bstart;
  }
#endif

  /* Initialize the HAL unit */
  siu_exc_assert((*siuContext.halApi->halNew)(nbufs, bufs, &cfg) == 0,SIUHAL_EXC_HALNEW,
      &siuContext)

  /* Store the returned core Id */
  siuContext.coreId = cfg.coreId;

  return (SIU_SUCCESS);

} /* siu_hal_init */
/*****************************************************************************
 * FUNCTION PURPOSE: Allocate and initialize a tdm port
 *****************************************************************************
 * DESCRIPTION: Completes port setup given a complete configuration. Returns
 *              -1 on any hal failure
 *****************************************************************************/
tint siu_create_tdm_port (halTdmInit_t *tdmInit, tint cfgIndex)
{
  halMemBuffer_t *bufs;
  tint            nbufs;
  void           *bstart,*addr_unaligned;
  tuint          bsize;
  halReturn_t     halReturn;

  /* Get the size requirements for the specified configuration */
  halReturn = (*siuContext.halApi->tdm->halTdmGetSizes) (&nbufs, &bufs, tdmInit);
  if (halReturn != HAL_SUCCESS)  
    return (-1);


  /* Allocate the buffers */
  if (bufs[HAL_TDM_HEAP_BUFN].base == NULL)  {
  	addr_unaligned=ggConfig.halHeapBufStart + (siuContext.halHeapUsed/sizeof(tuint));
    bstart = siu_buffer_align (addr_unaligned, 
                              ggConfig.halHeapBufEnd, bufs[HAL_TDM_HEAP_BUFN].log2align,
                              &bsize);
    if (bsize < bufs[HAL_TDM_HEAP_BUFN].size)  
      return (-1);

    bufs[HAL_TDM_HEAP_BUFN].base = bstart;
    siuContext.halHeapUsed += bufs[HAL_TDM_HEAP_BUFN].size+ (int)((int)bstart-(int)addr_unaligned);
   }

  /* Initialize the port */
  halReturn = (*siuContext.halApi->tdm->halTdmInit) (tdmInit, nbufs, bufs);
  if (halReturn != HAL_SUCCESS)  
    return (-1);

  return (0);

}  /* siu_create_tdm_port */

void siuMsgTdmPortConfig (mgb_t *msg)
{
  halTdmInit_t    tdmInit;
  halTdmControl_t tdmCtl;
  halReturn_t     halReturn;
  tint            port;

  /* Panic if there is no tdm */
  if (!siuContext.halApi->tdm)
      siuErrorIndication (msg->header.trans_id, 0, MGB_ERR_TDM_INVALID_TDM_CONFIG);
      
   port =   msg->msg_body.port_config.physPort;
   
  if (port >= ggConfig.nTdmPorts)  {
      siuErrorIndication (msg->header.trans_id, 0, MGB_ERR_TDM_INVALID_TDM_CONFIG);
    return; 
  }

  if (msg->msg_body.port_config.mode == MSG_RUN_TIME_INIT_START)  {

    tdmInit.fcn = HAL_TDM_AS_TDM;

    if (port == 0){
      tdmInit.subFrameCallout = siuSubRateFcn;
      tdmInit.cxt             = NULL ; // &tdm_port[port];
      tdmInit.sizes.maxChannels  = (ggConfig.max_channels + ggConfig.secChannels);
    } else {
      tdmInit.subFrameCallout = NULL ;
      tdmInit.cxt             = NULL ; // &tdm_port[port];
      tdmInit.sizes.maxChannels  = 0;
    }
    
    tdmInit.sizes.subFrameSize = ggConfig.tdmConfig->subFrameSize[port];
    tdmInit.sizes.wordSize     = ggConfig.tdmConfig->wordSize[port];
    tdmInit.sizes.buffIncr     = ggConfig.tdmConfig->buffIncr[port];


    tdmInit.physPort      = msg->msg_body.port_config.physPort;
    tdmInit.tdmCompanding = msg->msg_body.port_config.companding;
    tdmInit.start         = HAL_TDM_ALWAYS_ON;
    tdmInit.initTs        = NULL;
    tdmInit.aicInst       = NULL;

    tdmInit.maxMegaPhase   =  SIU_HAL_MAX_MEGA_PHASE;
    tdmInit.maxPhase       =  ggConfig.bsp_max_phase;
    tdmInit.tx.dataDelay   = msg->msg_body.port_config.u.rtInit.tx.dataDelay;
    tdmInit.tx.bdxDelay    = msg->msg_body.port_config.u.rtInit.tx.bdxDelay;
    tdmInit.tx.idleDrive   = msg->msg_body.port_config.u.rtInit.tx.idleDrive;
    tdmInit.tx.fsyncPol    = msg->msg_body.port_config.u.rtInit.tx.fsyncPolarity;
    tdmInit.tx.fsyncClkPol = msg->msg_body.port_config.u.rtInit.tx.fsyncClkPolarity;
    tdmInit.tx.clkPol      = msg->msg_body.port_config.u.rtInit.tx.clkPolarity;
    tdmInit.tx.clkRate     = msg->msg_body.port_config.u.rtInit.tx.clkRate;
    tdmInit.tx.tsPerFrame  = msg->msg_body.port_config.u.rtInit.tx.tsPerFrame;
    tdmInit.tx.dataRate    = msg->msg_body.port_config.u.rtInit.tx.dataRate;
    tdmInit.tx.clkRedund   = msg->msg_body.port_config.u.rtInit.tx.clkRedundancy;
    tdmInit.tx.clkSrc      = msg->msg_body.port_config.u.rtInit.tx.clkSource;
    tdmInit.sizes.wordSize = msg->msg_body.port_config.u.rtInit.tx.wordSize;

    tdmInit.rx.dataDelay   = msg->msg_body.port_config.u.rtInit.rx.dataDelay;
    tdmInit.rx.bdxDelay    = msg->msg_body.port_config.u.rtInit.rx.bdxDelay;
    tdmInit.rx.idleDrive   = msg->msg_body.port_config.u.rtInit.rx.idleDrive;
    tdmInit.rx.fsyncPol    = msg->msg_body.port_config.u.rtInit.rx.fsyncPolarity;
    tdmInit.rx.fsyncClkPol = msg->msg_body.port_config.u.rtInit.rx.fsyncClkPolarity;
    tdmInit.rx.clkPol      = msg->msg_body.port_config.u.rtInit.rx.clkPolarity;
    tdmInit.rx.clkRate     = msg->msg_body.port_config.u.rtInit.rx.clkRate;
    tdmInit.rx.tsPerFrame  = msg->msg_body.port_config.u.rtInit.rx.tsPerFrame;
    tdmInit.rx.dataRate    = msg->msg_body.port_config.u.rtInit.rx.dataRate;
    tdmInit.rx.clkRedund   = msg->msg_body.port_config.u.rtInit.rx.clkRedundancy;
    tdmInit.rx.clkSrc      = msg->msg_body.port_config.u.rtInit.rx.clkSource;

    if (siu_create_tdm_port (&tdmInit, port) != 0)  {
      siuErrorIndication (msg->header.trans_id, 0, MGB_ERR_TDM_INVALID_TDM_CONFIG);
      return;
    }

    /* Enable the port */
    tdmInit.halId = 0;
    tdmCtl.mode = HAL_TDM_PORT_ENABLE;
    halReturn = (*siuContext.halApi->tdm->halTdmControl)(tdmInit.halId, &tdmCtl); 
    if (halReturn != HAL_SUCCESS)  {
      siuErrorIndication (msg->header.trans_id, 0, MGB_ERR_TDM_INVALID_TDM_CONFIG);
      return;
    }

  }
} /* siuMsgTdmPortConfig */

#if (!defined(HDVIDEO) && !defined(HDCT))
tint siuCreateAndConfigTdmPort(siuTdmHw_t *hwcfg)
{
  halTdmInit_t    tdmInit;
  tulong		  srate;

  tdmInit.physPort = hwcfg->physPort;
  tdmInit.fcn      = HAL_TDM_AS_TDM;

  tdmInit.sizes.maxChannels  = (ggConfig.max_channels + ggConfig.secChannels);
  tdmInit.sizes.subFrameSize = ggConfig.BSP_segment_length;
#ifdef AUDIO
  tdmInit.sizes.subFrameSize *= ggConfig.audioOverSamplingFactor;
#endif

#if (defined(HAL_TIMESLOT_SIZE_32))
  tdmInit.sizes.wordSize = 32;
#elif (defined(HAL_TIMESLOT_SIZE_16))
  tdmInit.sizes.wordSize = 16;
#else
  tdmInit.sizes.wordSize = 8;
#endif

  tdmInit.sizes.buffIncr = ggConfig.tdmConfig->buffIncr[hwcfg->logPort];

  tdmInit.tdmCompanding   = hwcfg->companding;
  tdmInit.start           = HAL_TDM_ALWAYS_ON;
  tdmInit.maxPhase        = ggConfig.bsp_max_phase;
  tdmInit.maxMegaPhase	  = SIU_HAL_MAX_MEGA_PHASE;
  tdmInit.mcbsp_block     = 0;
  tdmInit.initTs          = NULL;
  tdmInit.aicInst         = NULL;
  tdmInit.subFrameCallout = hwcfg->subFrameCallout;
  tdmInit.cxt			  = NULL;
  tdmInit.tx.dataDelay    = hwcfg->txDelay;
  tdmInit.tx.bdxDelay     = hwcfg->bdxDelay;
  tdmInit.tx.tsPerFrame   = hwcfg->tsPerFrame;
  tdmInit.tx.idleDrive    = hwcfg->txIdleDrive;
  tdmInit.tx.fsyncPol     = hwcfg->txFsyncPol;
  tdmInit.tx.fsyncClkPol  = 1;
  tdmInit.tx.clkPol       = hwcfg->txClkPol;
  tdmInit.tx.clkRate      = 1;

  if (hwcfg->sfreq == MGB_SAMP_FREQ_8KHZ)
    srate = 8000;
  else if (hwcfg->sfreq == MGB_SAMP_FREQ_16KHZ)
    srate = 16000;
  else
    srate = 48000;

  tdmInit.tx.dataRate  = (srate * hwcfg->tsPerFrame * 8) / 1024000;
  tdmInit.tx.clkRedund = 0;
  tdmInit.tx.clkSrc    = 0;

  tdmInit.rx = tdmInit.tx;

  tdmInit.rx.fsyncPol    = hwcfg->rxFsyncPol;
  tdmInit.rx.fsyncClkPol = 1;
  tdmInit.rx.clkPol      = hwcfg->rxClkPol;
  tdmInit.rx.dataDelay   = hwcfg->rxDelay;

  if (halInitCtable.halTdmResourceInit)
    (*halInitCtable.halTdmResourceInit)();
    
    siu_exc_assert(siu_create_tdm_port (&tdmInit, hwcfg->logPort) == 0, SIUHAL_EXC_TDM_ALLOC,
      &siuContext)

  return (tdmInit.halId);

} /* siuMsgTdmPortConfig */
#endif

#ifdef GG_HAL_INCLUDE_SRIO

/*************************************************************************************
 * FUNCTION PURPOSE: Perform initial SRIO port configuration
 *************************************************************************************
 * DESCRIPTION: Initialise and configure SRIO port
 *************************************************************************************/
void siu_srio_init (mgb_t *message)
{
  halSrioInitCfg_t srioInit;
  halReturn_t ret;
  struct MGB_MDS_SRIO_INIT_CONFIG *srio_params= &(message->msg_body.srio_init_config);
  tuint i;
  /* Pass Glob configuration */

  srioInit.numPorts= srio_params->num_ports;
  srioInit.valid_params = srio_params->valid_params;
  srioInit.baseidReg= (srio_params->baseid_reg_msb<< 16) | (srio_params->baseid_reg_lsb);
  srioInit.deviceidReg1= (srio_params->deviceid_reg1_msb<< 16) | (srio_params->deviceid_reg1_lsb);
  srioInit.deviceidReg2= (srio_params->deviceid_reg2_msb<< 16) | (srio_params->deviceid_reg2_lsb);
  srioInit.deviceidReg3= (srio_params->deviceid_reg3_msb<< 16) | (srio_params->deviceid_reg3_lsb);
  srioInit.deviceidReg4= (srio_params->deviceid_reg4_msb<< 16) | (srio_params->deviceid_reg4_lsb);
  srioInit.serdesCfg0Cntl= (srio_params->serdes_cfg0_cntl_msb << 16) |
                                                srio_params->serdes_cfg0_cntl_lsb;

  for(i=0; i< srio_params->num_ports; i++) {
    /* Pass Port specific configuration */
    srioInit.srioPortInitCfg[i].physPort= srio_params->srio_port_init_cfg[i].phys_port;
    srioInit.srioPortInitCfg[i].valid_params = srio_params->srio_port_init_cfg[i].valid_params;
    srioInit.srioPortInitCfg[i].serdesCfgRxCntl= (srio_params->srio_port_init_cfg[i].serdes_cfgrx_cntl_msb<< 16) |
                                                  srio_params->srio_port_init_cfg[i].serdes_cfgrx_cntl_lsb;
    srioInit.srioPortInitCfg[i].serdesCfgTxCntl= (srio_params->srio_port_init_cfg[i].serdes_cfgtx_cntl_msb << 16) |
                                                  srio_params->srio_port_init_cfg[i].serdes_cfgtx_cntl_lsb;
    srioInit.srioPortInitCfg[i].pf8bCntl= (srio_params->srio_port_init_cfg[i].pf_8b_cntl_msb<< 16) |
                                                  srio_params->srio_port_init_cfg[i].pf_8b_cntl_lsb;
    srioInit.srioPortInitCfg[i].pf16bCntl= (srio_params->srio_port_init_cfg[i].pf_16b_cntl_msb<< 16) |
                                                  srio_params->srio_port_init_cfg[i].pf_16b_cntl_lsb;
  }

  /* Start up the peripheral */
  if (*siuContext.halApi->halSrioInit) {
    ret = (*siuContext.halApi->halSrioInit) (&srioInit);
    if (ret != HAL_SUCCESS) {
      /* Return error indication */  
      siuErrorIndication (siuContext.ID, message->header.trans_id, MGB_ERR_SRIO_INIT_FAILURE);
    }
  }
  return;
}
#endif

void tsipEmulInit();
void siuTsipEmulConfig(mgb_t *msg)
{
  /*TODO-This has to be through a call table,also counter 'period' as a paramter */
  tsipEmulInit(msg->msg_body.hw_config.core_mhz);
  /*TODO-This has to be through a call table */
}

/* Nothing past this point */

