/************************************************************************
:*                                                                      *
:*     @ Copyright (C) 2009  Texas Instruments Incorporated.            *
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

/****************************************************************************
 * FILE PURPOSE: Tomahawk (6486) specific functions
 ****************************************************************************
 * FILE NAME: siu6486.c
 *
 * DESCRIPTION: Contains device specific functions
 *
 *****************************************************************************/
#include "common/siuloc.h"
#include "types.h"
#include "hal.h"
#include "ggcfg.h"
#include "mgmicdsp.h"

/***************************************************************************
 * FUNCTION PURPOSE: Boot a core
 ***************************************************************************
 * DESCRIPTION: Secondary cores are booted by writing to boot complete.
 *			    Local reset recovery is not yet implemented.
 ***************************************************************************/
tint siuC6486CoreBoot (tuint core, tulong entry, tint *ecode)
{
  void (*goodbye)(void);
  halControl_t ctl;

  /* There are only 6 cores */
  if (core > 5)  {
    *ecode = MGB_ERR_INVD_WARMBOOT;
    return (-1);
  }

  /* If this core is being booted, just branch */
  if (core == siuContext.coreId)  {
    goodbye = (void (*)())(entry);
	(*goodbye)();
  }

  /* Direct hal to boot the other core */
  ctl.cmd                  = HAL_COMMAND_CORE_BOOT;
  ctl.u.coreBoot.core      = core;
  ctl.u.coreBoot.entryAddr = entry;

  if ( ((*siuContext.halApi->halControl)(&ctl)) == HAL_SUCCESS)  
    return (0);

  *ecode = MGB_ERR_INVD_WARMBOOT;
  return (-1);

} /* siuC6486CoreBoot */
/***********************************************/
void siuC6486_l2wbinv_all (void) 
{
  volatile tulong *L2WBINV = (tulong *)0x1845004;

  *L2WBINV = 1;
  while ((*L2WBINV) & 1);

}
/***********************************************/
void siuC6486_l2inv_all (void) 
{
  volatile tulong *L2INV = (tulong *)0x1845008;

  *L2INV = 1;
  while ((*L2INV) & 1);

}
/***********************************************/
void siuC6486_cache_writeback_invalidate (void *base, tulong length, bool block)
{
   volatile tulong *L2WIBAR = (tulong *)0x1844010;
   volatile tulong *L2WIWC = (tulong *)0x1844014;
   tulong base_masked = ((tulong)base) & (~0x7f);
   tulong leftover_length = (((tulong)base) & 0x7f) + 3;
   tulong word_length = (length + leftover_length) >> 2;
   tulong this_word_length;
   bool this_block;

   while (word_length > 0) {
     if (word_length > 0xffc0) {
       this_word_length = 0xffc0;
       this_block = TRUE;
     } else {
       this_word_length = word_length;
       this_block = block;
     }
     *L2WIBAR = base_masked;
     *L2WIWC = this_word_length;
     if (this_block) 
       while(*L2WIWC);
     word_length -= this_word_length;
     base_masked += (this_word_length * 4);
   }
}
/***********************************************/
void siuC6486_cache_invalidate (void *base, tulong length, bool block)
{
   volatile tulong *L2IBAR = (tulong *)0x1844018;
   volatile tulong *L2IWC = (tulong *)0x184401C;
   tulong base_masked = ((tulong)base) & (~0x7f);
   tulong leftover_length = ((tulong)base) & 0x7f + 3;
   tulong word_length = (length + leftover_length) >> 2;
   tulong this_word_length;
   bool this_block;

   while (word_length > 0) {
     if (word_length > 0xffc0) {
       this_word_length = 0xffc0;
       this_block = TRUE;
     } else {
       this_word_length = word_length;
       this_block = block;
     }
     *L2IBAR = base_masked;
     *L2IWC = this_word_length;
     if (this_block) 
       while(*L2IWC);
     word_length -= this_word_length;
     base_masked += (this_word_length * 4);
   }
}
/*****************************************************************************
 * FUNCTION PURPOSE: Configure mips tables for core mhz
 *****************************************************************************
 * DESCRIPTION: 
 *****************************************************************************/
void siuInitCoreMhz6486 (tuint mhz)
{
  if(mhz < 500) mhz   = 500;
  siuContext.core_mhz = mhz;  
} /* siuInitCoreMhz6486 */

/*** nothing past this point ***/












