/************************************************************************************
 * FILE PURPOSE: Processor specific stack check for keep alives
 ************************************************************************************
 * FILE NAME: siuali55.c
 *
 * DESCRIPTION: Performs keep alive stack initializations and checks.
 *
 * TABS: None
 *
 * Copyright (C) 2002, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include "types.h"
#include "common/siuloc.h"
#ifdef HAL_INCLUDE_UTOPIA
#include "siuutop.h"
#endif
#include <ti/mas/neu/aal5.h>
#include "common/siualive.h"
#include "siuport.h"
#include "gghwcfg.h"
#include "ndu.h"
#include "targethw.h"

/**************************************************************************************
 * Externally defined symbols from the boot.s and the linker command file, required
 * for stack integrity check.
 **************************************************************************************/
extern tulong _stack;
extern tulong _STACK_SIZE;

/**************************************************************************************
 * FUNCTION PURPOSE: Initialize stack state for checks
 **************************************************************************************
 * DESCRIPTION: The lowest five elements of the stack are set to an initial value.
 **************************************************************************************/
void siuHealthStackInit (void)
{
  tuint *bsp ;
  tint   i;

  bsp =  (tuint *)&_stack;

  for (i = 0; i < SIU_HEALTH_STACK_CHECK_DEPTH; i++)
    *bsp++ = SIU_HEALTH_STACK_CHECK_VALUE;
} /* siuHealthStackInit */

/****************************************************************************************
 * FUNCTION PURPOSE: Cursory check of stack validity
 ****************************************************************************************
 * DESCRIPTION: Checks that the stack pointers are in range, and that the last
 *              values in the stack match the initially set up values.
 ****************************************************************************************/
tint siuHealthStackCheck (void)
{
  void * base;
  void * spv;
  tulong size;
  tuint *bsp;
  tint   i;

  /* Read the stack and system stack pointer values. These should never vary, since
   * this function is called from the foreground, but check that they are in 
   * a valid range */
  spv = thwReadSP();
 
 /* spv  = ((tulong)SIU_HEALTH_C55X_SPH_REG << 16) | (tulong)SIU_HEALTH_C55X_SP_REG;*/
  /* The stack sizes are stored as addresses, but in bytes */
  size  = ((tulong)&_STACK_SIZE);
  base = (void *) &_stack;
  if (((tulong)spv < (tulong)base) || ((tulong)spv >= ((tulong)base + size)))
    return (SIU_ERROR);

  /* Check the lowest 5 values in the stacks to be sure they have not been
   * overwritten */
  bsp  = (tuint *)&_stack;

  for (i = 0; i < SIU_HEALTH_STACK_CHECK_DEPTH; i++) {
    if (*bsp++ != SIU_HEALTH_STACK_CHECK_VALUE)
      return (SIU_ERROR);
  }

 return (SIU_NOERR);
} /* siuHealthStackCheck */


/***********************************************************************************
 * FUNCTION PURPOSE: Check the processor status
 ***********************************************************************************
 * DESCRIPTION: C55X Status register bits are checked to see that a valid C
 *              state is setup.
 *
 *              Returns 0 if the processor is in the correct state.
 ***********************************************************************************/
tint siuHealthProcStat (void)
{
  int amr;

  amr = thwReadAMR();
/* Check for AMR */
  /* The C54x mode must be disabled */
  if ( amr !=0)
    return (1);  /* Direct accessing mode was enabled return an error */


  /* All checks passed */
  return (0);

} /* siuHealthProcStat */


/**************************************************************************************
 * FUNCTION PURPOSE: Create and send a keep alive cell
 **************************************************************************************
 * DESCRIPTION: A temporary aal5 instance is created. This is made a copy of the aal5
 *              message instance. A custom header is made, and the payload is created.
 *              The aal5 tx sar is hijacked, which requires interrupts to be disabled
 *              during this operation, since there is a message path through the aal5
 *              instance via interrupts. The function returns the number of cells
 *              sent.
 **************************************************************************************/
extern siuHealthContext_t siuHealthContext;
tint siu_send_keep_alive (void)
{
#ifdef HAL_INCLUDE_UTOPIA
 // xferPktInfo_t    pktOut;

//  pktOut.type = XFER_PKT_TYPE_ALIVE;
//  NDU_utSendData(1, &pktOut);
  
  siuNduHealthSend(siuHealthContext.stats.sendCount);

#if 0
  aal5Inst_t       ka_aal5;     /* A copy of the message aal5 instance */
  aal5Inst_t      *sys_aal5;    /* Pointer to the system aal5 instance */
  halUtopiaData_t  data;
  word             header[SIU_UTOPIA_RT_HEADER_SIZE_WORDS];
  ka_payload_t     payload;
  void            *payload_p;
  tint             i;
  tint             sizeBytes;

  /* make the aal5 instance a copy of the message aal5 instance. This will 
   * hijack the tx sar buffer */
  sys_aal5 = (aal5Inst_t *)siuContext.siuNduContext.aal5Inst;
  ka_aal5  = *sys_aal5;

  /* Handle route tags and VC demux mode seperately */
  if (siuContext.cellSize == 58)  {    

    /* Copy the system aal5 instance header to the local header, modify it,
     * then attach it to the local aal5 instance. */
    for (i = 0; i < SIU_UTOPIA_RT_HEADER_SIZE_WORDS; i++) 
      header[i] = sys_aal5->header_buf[i];

    /* Set the route tag type, destination phy, and channel */
    SIU_UTOPIA_RT_SET_TYPE((tsiuRouteTagHeader_t *)header, SIU_UTOPIA_RT_TYPE_HEALTH);
    SIU_UTOPIA_RT_SET_DEST_PHY((tsiuRouteTagHeader_t *)header, SIU_UTOPIA_RT_DEST_PHY_MISC);
    SIU_UTOPIA_RT_SET_CHANNEL((tsiuRouteTagHeader_t *)header, 0);

    /* Point the correct header */
    ka_aal5.header_buf = header;
    
    /* The source phy is already set from the message interface. */
  } else {
    /* Currently there is no support for VC demux mode */
    return (0);
  }

  /* Form the payload */
  payload.seqNum = 0;
  payload.chip_core = 0;
  SIU_KEEP_ALIVE_SET_CHIP_ID(&payload, siuContext.dspId);
  SIU_KEEP_ALIVE_SET_CORE_ID(&payload, siuContext.coreId);

  /* Perform the aal5 segmentation. Since the message tx sar buffer is hijacked,
   * interrupts must be disabled */
  SIU_STATE_BEGIN();
  sizeBytes = SIU_KEEP_ALIVE_PAYLOAD_SIZE_BYTES;
  payload_p = (void *)&payload;
  aal5_send_process (&ka_aal5, &sizeBytes, &payload_p, 
                      siuContext.siuNduContext.aal5CrcGen);

  /* The keep alive is now a single aal5 cell. Pass this to the network interface */
  data.nSegs = 1;
  data.type  = 0;
  data.sizes = (tuint *)&sizeBytes;
  data.segs  = (tuint **)&payload_p;

  if (siuContext.halApi->halUtopiaSendCell != NULL)
    siuContext.halApi->halUtopiaSendCell(0,0,&data);

  SIU_STATE_END();

  return (1); /* One cell sent */
#endif
#else
  return (0); /* No utopia, there is no keep alive message */
#endif  

} /* siu_send_keep_alive */

/* FUNCTION to send error message, none for C55x if keep alive is done through utopia */
tint mdebugEsent = 0;
tint siu_send_error_message (tuint errorCode)
{
#ifndef HAL_INCLUDE_UTOPIA
    if (mdebugEsent == 0)  {
      siuErrorIndication(siuContext.ID, 0, errorCode);
	  mdebugEsent = 1;
    }
    return(1);
#else
    return(0); /* Utopia, no error message reporting */
#endif    
}



