/*****************************************************************************
 * FILE PURPOSE: DSIU file containing OSAL functions for NWAL
 *****************************************************************************
 * FILE NAME: dsiunwosal.c
 *
 * DESCRIPTION: OSAL functions for running NWAL
 *               
 * TABS: None
 *
 * Copyright (C) 2010 Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#ifdef PA_SS

#include "dsiunwal.h"

/* CSL RL includes */
#include <ti/csl/cslr_device.h>
#include <ti/csl/csl_pscAux.h>
#include <ti/csl/csl_chip.h>
#include <ti/csl/csl_semAux.h>
#include <ti/csl/csl_cacheAux.h>
#include <ti/csl/csl_xmcAux.h>
#include <c6x.h>
#include <stdlib.h>
#include <stdio.h>

#include "targethw.h" /* for thwCritical */

UInt32              Osal_qmss_MallocCounter =0;
UInt32              Osal_qmss_FreeCounter =0;
UInt32              Osal_cppi_MallocCounter =0;
UInt32              Osal_cppi_FreeCounter =0;

#define COMMON_HW_SEM           3
/* Hardware Semaphore to synchronize access 
 * across different cores to the QMSS library.
 */
#define     QMSS_HW_SEM         4 

/* Hardware Semaphore to synchronize access 
 * across different cores to the CPPI library.
 */
#define     CPPI_HW_SEM         COMMON_HW_SEM 

/* Hardware Semaphore to synchronize access
 * across different cores to the PA library.
 */
#define     PA_HW_SEM           COMMON_HW_SEM 

/* Hardware Semaphore to synchronize access
 * across different cores to the SA library.
 */
#define     SA_HW_SEM           COMMON_HW_SEM 

/* Hardware Semaphore to synchronize access
 * across different cores for NWAL library.
 * Semaphore should be different from the one allocated for
 * CPPI/QMSS/PA/SA
 */
#define     NWAL_HW_SEM           2 

UInt32    cpswPaMemProtNestedLevel =   0;


extern cregister volatile unsigned int DNUM;

unsigned int Osal_nwalLocToGlobAddr(unsigned int x);

UInt32 globalCritkey;

/*****************************************************************************
 * FUNCTION PURPOSE: Cache Invalidation Routine
 ***************************************************************************** 
 * DESCRIPTION: Cache Invalidation Routine
 *****************************************************************************/
void Osal_invalidateCache (void *blockPtr, UInt32 size) 
{
    /* Disable Interrupts */
    thwCriticalBegin();

    /* Cleanup the prefetch buffer also. */
    CSL_XMC_invalidatePrefetchBuffer();

    size = size + (((UInt32)blockPtr) &(CACHE_L2_LINESIZE-1));
    /* Invalidate the cache. */
    CACHE_invL2 (blockPtr,size ,CACHE_FENCE_WAIT);

    /* Reenable Interrupts. */
    thwCriticalEnd();

    return;
}

/*****************************************************************************
 * FUNCTION PURPOSE: Cache Writeback Routine
 ***************************************************************************** 
 * DESCRIPTION: Cache Invalidation Routine
 *****************************************************************************/
void Osal_writeBackCache (void *blockPtr, UInt32 size) 
{
    /* Disable Interrupts */
    thwCriticalBegin();

    size = size + (((UInt32)blockPtr) &(CACHE_L2_LINESIZE-1));

    /* Invalidate the cache. */
    CACHE_wbL2 (blockPtr, size,CACHE_FENCE_WAIT);

    /* Reenable Interrupts. */
    thwCriticalEnd();

    return;
}


Void *  Osal_qmssMtCsEnter()
{
     /* Get the hardware semaphore for protection against multiple core access */
    while ((CSL_semAcquireDirect (QMSS_HW_SEM)) == 0);       
    return NULL;
}


Void Osal_qmssMtCsExit(void *key)
{
    /* Release the hardware semaphore */ 
    CSL_semReleaseSemaphore (QMSS_HW_SEM);        
    return;
}

Void Osal_nwalCsEnter(UInt32 *key)
{
    /* Get the hardware semaphore for protection against multiple core access */
    while ((CSL_semAcquireDirect (NWAL_HW_SEM)) == 0); 
    return;
}

Void Osal_nwalCsExit(UInt32 key)
{
    /* Release the hardware semaphore */ 
    CSL_semReleaseSemaphore (NWAL_HW_SEM); 
    return;
}


Void Osal_qmssLog ( String fmt, ... )
{
}


Void Osal_cppiCsEnter (UInt32 *key)
{
    /* Get the hardware semaphore for protection against multiple core access */
    while ((CSL_semAcquireDirect (CPPI_HW_SEM)) == 0); 
    return;
}

Void Osal_cppiCsExit (UInt32 key)
{
    /* Release the hardware semaphore */ 
    CSL_semReleaseSemaphore (CPPI_HW_SEM);    
    return;
}

Void Osal_cppiLog ( String fmt, ... )
{
}

void Osal_paBeginMemAccess (Ptr addr, UInt32 size)
{
    /* Get the hardware semaphore. 
     *
     * Acquire Multi core PA synchronization lock at fisrt nested level only
     */
    if(!cpswPaMemProtNestedLevel) 
        while ((CSL_semAcquireDirect (PA_HW_SEM)) == 0);
        
    cpswPaMemProtNestedLevel++; 
    
    Osal_invalidateCache(addr,size);

}

void Osal_paEndMemAccess (Ptr addr, UInt32 size)
{      
    cpswPaMemProtNestedLevel--; 
    Osal_writeBackCache(addr,size);

     /* Release the hardware semaphore only if the nexted level reach 0. */
    if(!cpswPaMemProtNestedLevel)
        CSL_semReleaseSemaphore (PA_HW_SEM);
}

void*  Osal_qmssCsEnter ()
{
    
    /* Disable all interrupts */
    while ((CSL_semAcquireDirect (QMSS_HW_SEM)) == 0);
    return(NULL);
}

Void Osal_qmssCsExit (void *  key)
{
    /* Enable all interrupts */
    CSL_semReleaseSemaphore (QMSS_HW_SEM);  
    return;
}

Ptr Osal_qmssMalloc (UInt32 num_bytes)
{
    Ptr ret;
   
    Osal_qmss_MallocCounter++;
    ret = malloc (num_bytes);    
    if(ret==NULL)
    {
      dsiuNwLog("\nERROR! QMSS Malloc failed!\n");
    }    
    
    return ((Ptr)(Osal_nwalLocToGlobAddr((unsigned int)ret)));
}

Void Osal_qmssFree (Ptr ptr, UInt32 size)
{
    /* Increment the free counter. */
    Osal_qmss_FreeCounter++;	
    free(ptr);
}

Ptr Osal_cppiMalloc (UInt32 num_bytes)
{
    Ptr ret;
    
    Osal_cppi_MallocCounter++;
    ret = malloc (num_bytes);

    if(ret==NULL)
    {
      dsiuNwLog("\nERROR! CPPI Malloc failed!\n");
    }   
    
    return ((Ptr)(Osal_nwalLocToGlobAddr((unsigned int)ret)));
}

Void Osal_cppiFree (Ptr ptr, UInt32 size)
{
    /* Increment the free counter. */
    Osal_cppi_FreeCounter++;	
    free(ptr);    
}

void Osal_qmssBeginMemAccess (void *blockPtr, UInt32 size)
{
    Osal_invalidateCache(blockPtr,size);
    return;
}

void  Osal_qmssEndMemAccess (void *blockPtr, UInt32 size)
{
    Osal_writeBackCache(blockPtr,size);
    return;
}

void Osal_cppiBeginMemAccess (void *blockPtr, UInt32 size)
{
    Osal_invalidateCache(blockPtr,size);
    return;
}

void Osal_cppiEndMemAccess (void *blockPtr, UInt32 size)
{
    Osal_writeBackCache(blockPtr,size);
    return;
}

void Osal_nwalInvalidateCache (void *blockPtr, UInt32 size)
{
    Osal_invalidateCache(blockPtr,size);
    return;
}

void Osal_nwalWriteBackCache (void *blockPtr, UInt32 size)
{
    Osal_writeBackCache(blockPtr,size);
    return;
}

UInt32 Osal_nwalGetCacheLineSize (void )
{
    /* By default assumes L2 cache line is enabled. If not return CACHE_L1D_LINESIZE */
    return (CACHE_L2_LINESIZE);
}

/********************************************************************
 * FUNCTION PURPOSE: Convert local address to global
 ********************************************************************
 * DESCRIPTION: Returns global address
 ********************************************************************/
unsigned int Osal_nwalLocToGlobAddr(unsigned int x)
{
  if ((x >= 0x800000) && (x < 0x900000))
  {
    x = (1 << 28) | (DNUM << 24) | x;
  }
    return (x);
}

UInt16 Osal_nwalGetProcId (void )
{
    return DNUM;
}

#else
int dummy; /* no warning */
#endif
