/************************************************************************
:*                                                                      *
:*     @ Copyright (C) 2009  Texas Instruments Incorporated.            *
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

/******************************************************************************
 * FILE PURPOSE: Performs siu initialization right after download.
 ******************************************************************************
 * FILE NAME:  siuinit.c  */
#include <stdlib.h>
#include <string.h>
#if (!defined(HDVIDEO) && !defined(HDCT))
#include "const.h"
#endif
#include "types.h"
#include "vctypes.h"
#include "memmgm.h"
#include "ggmbuf.h"
#include "ggswcfg.h"
#include "ggcfg.h"
#include "globstats.h"
/* SIU header files */
#include "common/siu.h"
#include "common/dsiunwal.h"
#include "siuloc.h"
#include "siuport.h"
#include "ndu.h"
#include "osal/siuIpcOsal.h"
#include "osal/siuIpcMsgOsal.h"
#ifndef HDCT
#include "common/siuctbl.h"
#endif
#ifdef GMP
#include "siugmp.h"
#endif
#include "util.h"
#include "utlrepck.h"
#ifdef GG_INCLUDE_HEALTH_CHECK
#include "common/siualive.h"
#endif
/* low-level hardware files */
#include "ggcfg.h"

/* dual-port RAM definitions */
#include "micdspdp.h"
#ifndef RESIZABLEHPI
#include "dpriram.h"
#endif
#include "vbmicdsp.h"
#include "dprutil.h"
#include "mgmicdsp.h"
#include "micdspdp.h"
#include "rttrace.h"

#include "video/siuVigdkIbl.h"

#define loop    for(;;)     /* endless loop */

/******************************************************************************
 * FUNCTION PURPOSE: Initialize SIU right after download.
 ******************************************************************************
 * DESCRIPTION: It initializes the internal SIU context, and SIU instance
 *              structure for operation.  After execution, SIU state is
 *              set to SIU_ST_START.
 * 
 *****************************************************************************/
void siuFrameworkInit (void)
{
  extern word *gg_rttrace_buffers[];
  extern utlRtTraceEnable_t gg_rttrace_enable[];
  int   i;
  tint  numChannels;

  /* use chnum 0 at this time */
  siuContext.ID                = siuMakeID (SIU_MID_SIU, 0);
  siuContext.debugInfo         = siuDebugInfo;
  siuContext.maxNumberChannels = (ggConfig.max_channels + 
                                  ggConfig.aux_pcm_channels +
                                  ggConfig.pkt_channels);
  siuContext.maxFullChannels   = (ggConfig.max_channels + ggConfig.aux_pcm_channels);

  siuContext.msgDelayCnt       = 0;
  siuContext.FreeMIPS          = FALSE;
  
  /* Error reporting boolean - used in Broadcast messages */
  siuContext.errorReport = TRUE;
  /* initialize dynamic buffer allocation parameters */
  siuContext.dynBufAlloc.dynBufAllocEnable  = ggDynBufAlloc.dynBufAllocEnable;
  siuContext.dynBufAlloc.numBitmapWord      = ggDynBufAlloc.numBitmapWord;
  siuContext.dynBufAlloc.maxChannelsPerWord = ggDynBufAlloc.ggMaxChannelsPerWord;
  siuContext.dynBufAlloc.bitmapBufIndexPerWord= ggDynBufAlloc.ggBitmapBufIndexPerWord;
  siuContext.dynBufAlloc.siuCh2Slot         = ggDynBufAlloc.ggCh2Slot;
  siuContext.dynBufAlloc.siuCh2SlotLBRC     = ggDynBufAlloc.ggCh2SlotLBRC;
  siuContext.dynBufAlloc.maxLBRCBuffers     = &ggDynBufAlloc.maxLBRCBuffers;
  siuContext.dynBufAlloc.bitmapLBRCBufIndexPerWord
    = ggDynBufAlloc.bitmapLBRCBufIndexPerWord;
  siuContext.dynBufAlloc.bitmapLBRCBufIndexPerWord[0] = 0;
  numChannels = siuContext.maxFullChannels;
  for (i = 0; i < siuContext.dynBufAlloc.numBitmapWord; i++) {
    siuContext.dynBufAlloc.bitmapBufIndexPerWord[i] = 0;
    if (numChannels > 16)
      siuContext.dynBufAlloc.maxChannelsPerWord[i] = 16;
    else
      siuContext.dynBufAlloc.maxChannelsPerWord[i] = numChannels;
    numChannels -= 16;
  }

#ifndef HDCT
  /* Task queue initializetions */
  siu_init_task();
#endif

  /* Initialize the 5ms staggered channel array. Indicates which channels
   * are processed at what sub-rate point. This is done here and stored
   * since the division is too costly to perform every 0.5ms */
  for (i = 0; i <= ggConfig.pcmIntPer5ms; i++)  {
    ggPrdCh[i] = (i * siuContext.maxNumberChannels) / ggConfig.pcmIntPer5ms;
  }

  siuContext.numActiveChannels =  0; /* no active channels (all closed) */

  for (i=0;i<UTL_RTTRACE_NUM_BUFS;i++) {
    utlRtTraceContext.buffers[i].data  = gg_rttrace_buffers[i];
  }
  utlRtTraceContext.bufferNumber = 0;
  utlRtTraceContext.enable = gg_rttrace_enable;
  utlRtTraceNextBuf();
  utlCheckSize(UTL_RTTRACE_MSGHEADER == sizeof(mgbHeader_t));

  memset (&siuContext.readMemPacing, 0, sizeof(siuContext.readMemPacing));

  /* Timer control initilization for power management */
  siuContext.timer_period = 0;
  siuContext.wake_up_interrupt_mask = 0;

  /* Initialize DSP-to-Micro signaling and normal message FIFO queue */
  /* normal message */
  siuContext.msgFifo.inIndex     = 0;
  siuContext.msgFifo.outIndex    = 0;
  siuContext.msgFifo.freeSpace   = ggConfig.siuMsgFifoLen;
  siuContext.msgFifo.dropCounter = 0;

  /* signaling message */
  siuContext.sigFifo.inIndex     = 
  siuContext.sigFifo.outIndex    = 0;
  siuContext.sigFifo.freeSpace   = ggConfig.siuSigFifoLen;
  siuContext.sigFifo.dropCounter = 0;

  /* Low priority messages */
  siuContext.lowFifo.inIndex     = 0;
  siuContext.lowFifo.outIndex    = 0;
  siuContext.lowFifo.freeSpace   = ggConfig.siuLowFifoLen;
  siuContext.lowFifo.dropCounter = 0;

  /* 5ms isr task counter */
  siuContext.pcm5msCounter = 0;
  siuContext.pcmMsgTaskIsrCounter = 0;
  siuContext.prdLastPhase  = 0;
  siuContext.phaseWrap     = 0;
  siuContext.msgTaskCounter = 0;         /* number of msg tasks */

  if (ggDynBufAlloc.dynBufAllocEnable) {
    for (i = 0; i < ggConfig.max_channels; i++) {
      siuContext.dynBufAlloc.siuCh2Slot[i]     = -1;
      siuContext.dynBufAlloc.siuCh2SlotLBRC[i] = -1;
    }
  } else {
    for (i = 0; i < (ggConfig.max_channels + ggConfig.pkt_channels); i++) {
      siuContext.dynBufAlloc.siuCh2Slot[i] = i;
    }
  }

  /* Core speed initialization */
  siuContext.siuInitCoreMhz = ggConfig.initCoreMhz;

  /* Initialize other global SIU variables */
  siu_isr_clock = siu_timer_tick = siu_stagger_clock = siu_2_5msIsrDecimate = 0;

  /* Initialize SIU pointers */
  /* Initialize all _allocated_ channels and "make" them "closed" */
  for (i = 0; i < siuContext.maxNumberChannels; i++)
  { /* Initialize statically allocated instances immediatelly */
    siuInitSiu (i);
  }

  /* Initialize keep alivesin disabled state */
#ifdef GG_INCLUDE_HEALTH_CHECK
    siuHealthInit(siu_timer_tick, 0, 0);
#endif
  /* Initialize global debug utility variables */
  siuDebugBufferInit();

  /* Is there enough memory to store siu heap data ? */
  siu_exc_assert(gg_siuheap_inst_len >= sizeof(siuHeapInst_t), SIUINIT_EXC_HEAPINST_MEMORY, &siuContext);

  siu_exc_assert(gg_siuheap_inst_ext_len >= sizeof(siuHeapInstExt_t), 
                 SIUINIT_EXC_HEAPINST_MEMORY, &siuContext);
  siuContext.gmpFormatState = SIU_GMPFORMAT_UNFORMATTED;
  siuContext.numLateTxSends = siuContext.numLatePlayouts = 0;
#ifndef HDCT
  /* check if task queue length meets requirement */
  siu_exc_assert( (SIU_TASK_QUEUE_LENGTH >= (ggConfig.max_channels + ggConfig.pkt_channels + 1)),
                 SIUTASK_EXC_INSUFFICIENT_QUEUE_LEN, &siuContext);
#endif
  /* Default BuildID to zero */
  siuContext.buildID = 0;

  /* Set flag for RMAN configuration */
  siuContext.rmanConfigFlag = 1;

} /* siuFrameworkInit */

/******************************************************************************
 * FUNCTION PURPOSE: Initialize SIU instance to known state
 ******************************************************************************
 * DESCRIPTION: Reset all ECO pointers, indicate closed state, etc...
 *****************************************************************************/
void siuInitSiu (tuint i)
{
   tuint chnum = SIU_FIRSTCH + i;
   tuint ID = siuMakeID (SIU_MID_SIU, chnum);
   siuInst_t *inst = SIUINSTp(i);

    memset (inst, 0, sizeof(siuInst_t));  

    /* set SIU channel state to CLOSED */
    inst->state          = SIU_CHST_CLOSED;
    inst->pending_state  = SIU_CHST_CLOSED; 

    inst->ID                 = ID;
    inst->chHeapID           = NULL;

    inst->state_transition_msg_id = 0;    
    inst->neuInst            = NULL;
#ifndef HDCT
    inst->msuRtpInst         = NULL;
    inst->msuRtcpInst        = NULL;
#endif
    SIU_SET_TX_ENABLED(inst, FALSE);    
    /* NDU Instance */
    inst->amInst.RxOutSpec.nduReceiveOut = NULL;
    inst->amInst.RxOutSpec.targetInst    = NULL;
    
} /* siuInitSiu() */

#ifdef GG_ECO_DATA_MEMORY_ANALYSIS
#define SIU_HEAP_BUFN    0
#define SIU_TX_NET_BUFN (SIU_HEAP_BUFN + 1)
#define SIU_NBUFS       (SIU_TX_NET_BUFN + 1)

memBuffer_t siuBufs[SIU_NBUFS] = {
  {MEM_CLASS_EXTERNAL, 0, sizeof(siuHeapInst_t), FALSE, NULL},  /* heap buffer */
  {MEM_CLASS_EXTERNAL, 0, sizeof(siuTxNetwork_t)*GG_MAX_NUM_SLOTS,TRUE, NULL},   /* scratch */ 
};

tint siuGetSizes(tint *nbufs, const memBuffer_t **bufs, void *sizeCfg)
{
  *nbufs = SIU_NBUFS;
  *bufs  = &siuBufs[0];
  
  return(SIU_NOERR); 
}
#endif /* GG_ECO_DATA_MEMORY_ANALYSIS */
/******************************************************************************
 * FUNCTION PURPOSE: Computes Checksums required for download check
 ******************************************************************************
 * DESCRIPTION: Computes the checksums and returns them for micro to validate
 *              code download.
 *****************************************************************************/
#ifdef GG_ENABLE_CHKSUM_CHECK
void siuChecksumCompute ( void )
{
  extern int dspImageMap[];
  tuint  checksum;       /* DSP calculated checksum */
  tulong progStartAddr;
  tuint  *localStartAddr;
  tuint  size, k;
  tint   i;
  tuint  local_mem_checksum,   /* Core local memory checksum     */
         shared_prog_checksum; /* Shared program memory checksum */

  /* initialize checksum */
  checksum = 0; 

  /* Shared Program Pages Checksum */
  if (ggChkSum_ProgBlck[0].blck_size != 0) {
    /* Compute program pages/blocks checksum */
    for (i=0; i<ggConfig.progblcknum; i++) {
      /* Extract Block info */
      progStartAddr = ggChkSum_ProgBlck[i].blck_start;
      size          = ggChkSum_ProgBlck[i].blck_size;

      /* Compute checksum for each block */
      /* Checksum is updated at each call - computing the total program pages
       * checksum. */
      utlCmpPrChksum( progStartAddr, size, &checksum ); 
    }
    /* Record shared program memory checksum */
    shared_prog_checksum = checksum;
  }
  else
    shared_prog_checksum = 0;   

  /* initialize checksum */
  checksum = 0; 

  /* Local memory Checksum */
  if (ggChkSum_LocalBlck[0].blck_size != 0) {
    /* Compute local memory checksum */
    for (i=0; i<ggConfig.localblcknum; i++) {
      /* Extract Block info */
      localStartAddr = (tuint *) ggChkSum_LocalBlck[i].blck_start;
      size           = ggChkSum_LocalBlck[i].blck_size;

      /* Compute checksum for each block */
      /* Checksum is updated at each call - computing the total local memory
       * checksum. */
      for (k=0; k < size; k++)
        checksum = checksum ^ localStartAddr[k];
    }
    /* Record shared program memory checksum */
    local_mem_checksum = checksum;
  }
  else
    local_mem_checksum = 0;   
  
  /* Report Checksum to micro */
  dspImageMap[30] = shared_prog_checksum;
  dspImageMap[31] = local_mem_checksum; 

} /* siuChecksumCompute */
#endif

/************************************************************************************
 * FUNCTION PURPOSE: Program memory copy
 ************************************************************************************
 * DESCRIPTION: Performs a one time copy of memory. 
 ************************************************************************************/
void siu_init_prg_copy (void)
{
  halLocAddr_t dest, src;
  halMemMove_t memMove;

  if (!ggConfig.loadSize)
    return;  /* nothing to copy */

  /* Convert addresses and sizes for use by HAL */
  dest = (halLocAddr_t) UTL_BYTES_TO_WORDS((tulong)ggConfig.runStart);
  memMove.dest = (*siuContext.halApi->halMapLocalToGlobal)(dest);

  src  = (halLocAddr_t) UTL_BYTES_TO_WORDS(ggConfig.loadStart);
  memMove.src = (*siuContext.halApi->halMapLocalToGlobal) (src);
#ifdef GG_HAL_GLOBAL_ADDR_BIT_24  
  memMove.src = HAL_MEM_ADDRESS_PROG(memMove.src);
#endif  

  memMove.size = (halSAE_t) UTL_BYTES_TO_WORDS(ggConfig.loadSize);

  /* Copy the data */
  (*siuContext.halApi->halGlobMemMove) (1, &memMove, NULL, NULL, TRUE);

} /* siu_init_prg_copy */

/*********************************************************************************
 * FUNCTION PURPOSE: System Hardware Initialization
 *********************************************************************************
  DESCRIPTION:      This function is called during initialization of the system.
                    It does platform specific initialization, calls Hardware Abstraction
                    Layer (HAL) initialization and sets up the peripherals.
  Parameters :      Inputs: None
                    Output: Revision ID.
 *********************************************************************************/
extern cregister volatile unsigned int DNUM;
tuint siuHwInit (void)
{
  tuint revId = 0; /* Not supported yet */
  siuReturn_t halRet,halRetNew;
  tuint platformInitError;
  extern const vigdkMemoryContext_t gg_supported_builds[];
  const vigdkMemoryContext_t *activeBuild;

  /* Default configuration: Build 0 */
  activeBuild = &gg_supported_builds[0]; 

  /* Don't die while processing interrupts */
  siuContext.halt.halt.halt_if_set = siuContext.halt.halt.halted_if_set = 0;

  /* Initialize HAL APIs and memory config */
  /* Assertion cannot be done here so store result and assert after ndu is 
     initialized */
/*  !!!! Warning:  Don't do anything else before this function 
  for example in case of titan this initialises the emif access .  
  so it is critical that this is executed before any external memory program execution*/
  halRet = siu_init_hal(activeBuild->marRegs, activeBuild->marVals);

  /* There seems to be memory protection issues in NWAL. 
   * This is a temporary fix to initialize each core sequentially.
   */
  while(siu_is_ibl() && siuNumCoreInit<DNUM) {
    siu_osal_inv_cache((void *)&siuNumCoreInit, sizeof(siuNumCoreInit), TRUE);
  }

  /* Platform-specific initialization */
  platformInitError = (platformInit() == GGCFG_NOERR);
  
  /* 
   * Warning: Be careful if merging the following code!!!
   * Global interrupt should be enabled within HAL and platform initialization.
   * But it was not at this branch.
   * It is only a temporary fix to enable global interrupt and serves as a reminder.
   * If the global interrupt is enabled at this point, do not merge this code.
   * If it is not, we may want to find a better way to do it. 
   */
  thwCriticalReset();

  /* NDU Start must be the first function to be called.If any of the functions below
     go to exception , it will try to send a message to micro before NDU is intialised
     and crash the DSP */
   
   /* This is required by NDU for NV ecos, in siumgrv.c */
  siuContext.cellSize          = SIU_DEFAULT_CELL_SIZE;
  /* Initialize hardware configuration */
  halRetNew = siu_new_hal(); //(SIU_DEFAULT_CELL_SIZE, &revId);
    
#ifdef PA_SS  
  /*Initializing PA, QM, CPPI/NWAL*/
  if (dsiuNwInit () == FALSE)  {
    dsiuNwLog ("ERROR: dsiuNwInit FAILED \n");
    return;
  }
  else
  {
    dsiuNwLog ("dsiuNwInit Passed \n");
  }
#endif

  /*  Following will initialize GMAC or Utopia. This function must be called
   *  before siuNduStart to get the halId of the outgoing message peripheral */
  siu_periph_setup();

  siu_exc_assert(halRet == SIU_SUCCESS,SIUHAL_EXC_HALINIT,&siuContext);
  siu_exc_assert(halRetNew ==  SIU_SUCCESS,SIUHAL_EXC_HALNEW,&siuContext);
  siu_exc_assert(platformInitError,SIUGG_EXC_GGPLATINIT,&siuContext);

  return (revId);
} /* siuHwInit */

/*********************************************************************************
 * FUNCTION PURPOSE: System Software Initialization
 *********************************************************************************
  DESCRIPTION:      This function is called during initialization of the system.
                    It brings up the network interface (which is essential for the
                    message path for future communication between DIM and DSP) and 
                    a random seed in initialized.
  Parameters :      Inputs: None
                    Output: None
 *********************************************************************************/
void siuSwInit(void) 
{
  tuint tmp;
   /* Bring up the network Driver Interface */
   siuNduStart();

#ifdef GMP
  /* Initialization of Global Memory Pool support -- one area for messages
   * (restart/hwconfig/ready); other area(s) can be overlayed with startup
   * code; once hwconfig received, reset of GMP will be added 
   */
  siugmpInit(ggConfig.gmcSize, gg_gmp_no_segments, (void *)gg_gmp_segments, TRUE);
#endif

  /* Copy any program memory sections */
  siu_init_prg_copy ();

  /* Initialize random seed in the DSP */
  tmp = (siuContext.dspId << 8) + siuContext.coreId;
  srand(tmp); 

#ifdef HDVIDEO
  /* Does the inter-processor initialization for core-to-core transcoding */
  /* Initialize IPC internals such as barrier and gate */
  siu_ipc_osal_init();
  /* Intialize IPC messageQ internals, such as local messageQ and remote MessageQ IDs */
  siu_ipc_osal_msg_init();
#endif
  return;

 } /* siuSwInit */
