/************************************************************************
:*                                                                      *
:*     @ Copyright (C) 2009  Texas Instruments Incorporated.            *
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include <stdlib.h>
#include "types.h"
#include "mgmicdsp.h"  /* ICD defintions */
#include "vctypes.h"

/* SIU header files */
#include "common/siu.h"
#include "siuloc.h"
#include "siuport.h"
#ifndef HDCT
#include "common/siuctbl.h"  /* SIU internal call table */
#endif
#include "mhm.h"

#include "ggmbuf.h"   /* Defines memory configuration data structures */

#include "ggcfg.h"    /* Defines Golden Gateway configuration data */

#include "vmmcomp.h"  /* vmm compression/decompression of buffer information */

#include "siundu.h"
/******************************************************************************
 * FUNCTION PURPOSE: Allocate module memory buffers
 ******************************************************************************
 * DESCRIPTION: returns SIU_ERROR if unsuccessful,
 *                      SIU_NOERR if successful
 *
 *****************************************************************************/
tint siuBufferAlloc (tint ecoNbufs, siuInst_t *inst, const memBuffer_t *reqBuffer,
                     memBuffer_t *assignedBuffer, void *EcoBufTbl, tuint mid,
                     bool allocFromHeap)
{
  tint  i, chnum;
  ggbufDesc_t *MemBufferTbl = (ggbufDesc_t *) EcoBufTbl;
  
  chnum = inst->ID & SIU_CID_MASK;

  for (i = 0; i < ecoNbufs; i++)
  {
    if( (allocFromHeap == TRUE) && (reqBuffer[i].volat == MEM_PERSISTANT) )
    {  /* This way, we are forcing the non-scratch buffers to be allocated 
          from heap if allocFromHeap is true. */
      MemBufferTbl[i].buffAddr = NULL;
    }

    assignedBuffer[i] = reqBuffer[i];
    if (MemBufferTbl[i].buffAddr == NULL)
    {
      if (reqBuffer[i].size > 0)
      {
        assignedBuffer[i].volat  = FALSE;
#ifdef GMP
        if(reqBuffer[i].volat == MEM_GMP_TYPE) {
           /* This buffer may fit into global memory pool ? */
           if (reqBuffer[i].size <= ggConfig.gmcSize) {
             assignedBuffer[i].base  = gmpAllocGmc(mid);
             assignedBuffer[i].volat = MEM_GMP_TYPE;
           }
        }
#endif
        if((assignedBuffer[i].base == NULL) && ((reqBuffer[i].volat == MEM_PERSISTANT) ||
           (reqBuffer[i].volat == MEM_BYPASS)))
        {
          assignedBuffer[i].base   = siuChAlloc (chnum, reqBuffer[i].size);
        }
        siu_exc_assert (assignedBuffer[i].base != NULL, SIUMGR_EXC_BUFALLOC, inst);
      }
    }
    else
    {
      if (MemBufferTbl[i].buffSize < reqBuffer[i].size) { /* Dynamic Heap */
        siuDebugInfo(mid, SIU_DEBUG_FATAL, SIUMGR_EXC_WRONG_BUFF_SIZE, 0, NULL);
      }
      else {
        assignedBuffer[i].size = MemBufferTbl[i].buffSize;
        /* If buffer is in a multicore shared data area, remap it based on
         * current core ID, else just copy the pointer 
         */
        assignedBuffer[i].base = siu_remap_mcs_data(MemBufferTbl[i].buffAddr);
      }
    }
  }

  return(SIU_NOERR);
} /* siuBufferAlloc */

#ifdef SIU_MULTICORE_SHARED_DATA
/******************************************************************************
 * FUNCTION PURPOSE: Remap per-core shared data to core specific area
 ******************************************************************************
 * DESCRIPTION:  Remaps multi-core shared data (MCS) pointers
 *
 *               This function is used when MCS_DATA=YES in mkrel/makefile
 *
 * void *siu_remap_mcs_data (
 *        void *input_ptr)    - Pointer to remap
 * 
 * RETURN: Remapped value, if applicaple, else original pointer
 ******************************************************************************/
void *siu_remap_mcs_data (void *input_ptr)
{
  extern const vigdkMultiCoreSharedBuild_t gg_mcs_builds[];
  const vigdkMultiCoreSharedBuild_t *activeBuild = &(gg_mcs_builds[siuContext.buildID]);
  ggMultiCoreShared_t *area_p = activeBuild->build;
  word *remap_ptr = (word *)input_ptr;
  
  while(area_p->area_end) {
    word *begin = area_p->area_begin;
    word *end = area_p->area_end;
    if ((remap_ptr >= begin) && (remap_ptr < end)) {
      remap_ptr += (end - begin) * siuContext.coreId;
      /* Since normal case is no remapping, and areas should not overlap,
       * keep going to allow this to sploop on c64x, and rptlocal on 55x.
       */
      /* break; */
    }
    area_p ++;
  }
  return remap_ptr;
} /* siu_remap_mcs_data */
#else
  /* Function is #define in siuloc.h for zero overhead on single core
   * devices and multicore devices where there is no shared per-core
   * data
   * 
   * MCS_DATA=NO in mkrel/makefile
   */
#endif

/******************************************************************************
 * FUNCTION PURPOSE: Create a generic ECO
 ******************************************************************************
 * DESCRIPTION: Performs getsizes/alloc/new for an ECO
 *
 * siuNewEco (
 *    siuInst_t *inst,               - siu instance pointer (= NULL for NVECO)
 *    siuEcoGetSizes_t getsizes_fcn, - eco ecoGetSizes() function pointer
 *    void *sizeCfg,                 - ecoGetSizes config parameters
 *    siuEcoNew_t new_fcn,           - eco ecoNew() function pointer
 *    void *ecoNewCfg,               - ecoNew()'s config parameters
 *    void **ecoInst,                - new instance returned by ecoNew()
 *    memBuffer_t *ecoBUfs,          - eco's (for caller's use)
 *    tuint moduleId)                - module identifier (=NVECO_ID for NVECO)
 * NOTE: channel number is retrieved from SIU ID (lower 8 bits)
 *****************************************************************************/\
tint siuNewEco (siuInst_t *inst, siuEcoGetSizes_t getsizes_fcn, void *sizeCfg, 
                siuEcoNew_t new_fcn, void *ecoNewCfg, void **ecoInst, 
                memBuffer_t *ecoBufs, tuint moduleId) 
{
   const memBuffer_t *sizeBufs;
   tint  stat, ecoNbufs;
   tint  slot_num, memSlot;
   ggbufDesc_t memBufInfo[SIU_MAX_NBUFS];
   bool forceAllocFromHeap = FALSE; /* Forcing network lite buffers from heap */
   tuint buffNum, chnum, i;

   /* Call the module's GetSizes function */
   stat = getsizes_fcn (&ecoNbufs, &sizeBufs, sizeCfg); 
   if (stat || (ecoNbufs > SIU_MAX_NBUFS)) 
     return SIU_ERROR;
     
   if (inst != NULL) { /* channel dependent */
      chnum = (inst->ID & SIU_CID_MASK) - SIU_FIRSTCH;
      memSlot = chnum;

      if (siuContext.dynBufAlloc.dynBufAllocEnable) {
         memSlot = siuContext.dynBufAlloc.siuCh2Slot[chnum];
      }

      slot_num = memSlot; 

     /* Check if all buffer descriptors are correctly extracted ? */
     siu_exc_assert(vmmGetBuffInfo ((tulong)ggmbufStartAddrInPRAM,
                                   moduleId,
                                   VMM_IDT_NONCODEC_ECO,
                                   slot_num, &buffNum,
                                   (void *)&memBufInfo) >= 0, SIUMGR_EXC_NEWECO, inst);                           

     if (buffNum < ecoNbufs) return SIU_ERROR;   

     /* Assign the module's buffers */
     stat = siuBufferAlloc (ecoNbufs,
                          inst,
                          &sizeBufs[0],
                          &ecoBufs[0], 
                          (void *)&memBufInfo[0],
                          siuMakeID (moduleId, chnum + SIU_FIRSTCH),
                          forceAllocFromHeap);
   } else { /* channel independent */
     /* Find the buffer structure based on the moduleId, which is the index to the table */
     stat = siuNvEcoGetBuffInfo (moduleId, &buffNum, memBufInfo);
     siu_exc_assert (stat != SIU_ERROR, SIUMGR_EXC_NEWECO, &siuContext);
     if (buffNum != ecoNbufs) return SIU_ERROR;   

     /* Assign the module's buffers */
     for (i = 0; i < ecoNbufs; i++) {
       if (sizeBufs[i].size > memBufInfo[i].buffSize)
         return (SIU_ERROR);
       ecoBufs[i].base      = (void *)memBufInfo[i].buffAddr;
       ecoBufs[i].size      = (tuint) memBufInfo[i].buffSize;
       ecoBufs[i].log2align = 0;
       ecoBufs[i].volat     = FALSE;
     }   
  }
   
   /* Return error if buffer allocation failed */
   if (stat == SIU_ERROR) return(SIU_ERROR);

   /* Invoke the module's new() function */
   return new_fcn (ecoInst, ecoNbufs,
                   ecoBufs, ecoNewCfg);
}
/******************************************************************************
 * FUNCTION PURPOSE: Return Non-voice ECO buffer information
 ******************************************************************************
 * DESCRIPTION: Extracts the buffer information based on the non-voice
 *              ECO index. 
 ******************************************************************************/
tint siuNvEcoGetBuffInfo (tint nvId, tuint *buffNum, ggbufDesc_t *memBufInfo)
{
  const ggbuffDescNEU_t *neuBuf;
  tint i;

  /* Currently the only ECO used this way is NEU. If other ECOS are added to
   * non-voice channels the extraction will have to be updated to handle
   * more then one ECO type */

  /* Verify the index range */
  if (nvId >= ggConfig.nvEcoBufNum)
    return (SIU_ERROR);

  /* With only one ECO type, extraction is simply the index to the NEU buffer */
  neuBuf = &ggNvBuffDesc[nvId].buffDescNEU;


  *buffNum = neuBuf->numBuffs;
  for (i = 0; i < *buffNum; i++)
    memBufInfo[i] = neuBuf->bufDesc[i];

  return (SIU_NOERR);

} /* siuNvEcoGetBuffInfo */


