/************************************************************************
:*                                                                      *
:*     @ Copyright (C) 2009  Texas Instruments Incorporated.            *
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include <stdlib.h>
#include <string.h>

#include "types.h"
#if (!defined(HDVIDEO) && !defined(HDCT))
#include "const.h"
#endif

#include "mgmicdsp.h"  /* Message structures */

/* SIU definitions */
#include "common/siu.h"
#include "siuloc.h"
#include "siuport.h"
#ifndef HDCT
#include "common/siuctbl.h"     /* SIU internal call table */
#endif
#include "ggcfg.h"
#include "common/siucmpnd.h"

#include "utlswap.h"   /* Prog to Data swap utility */
#include "utlrepck.h"  /* Utl pack functions */
#include "targethw.h"  /* Hardware configuration - clock */

#include "hal.h"
#include "rttrace.h"
#ifdef GMP
#include "gmp.h"
#endif
#include "utlrepck_m.h"
#include <ti/mas/iface/ifpkt/xferpkt.h>
#include "util_s.h"

#include "siuCommonMpu.h"

#ifdef SIUTBLPRG
tuint           *dispatchTblStatsReqChst;
siuFcnResetCh_t *dispatchTblResetChState;
fcnGetStats_t   *dispatchTblGetStats;
#else
const tuint           *dispatchTblStatsReqChst;
const siuFcnResetCh_t *dispatchTblResetChState;
const fcnGetStats_t   *dispatchTblGetStats;
#endif

/******************************************************************************
 * FUNCTION PURPOSE: Executes a message processing function
 ******************************************************************************
 * DESCRIPTION: This function select the message processing function to 
 *  execute by matching the MGB_MD_* message type with the first element of
 *  the table entry and calls the processing function in the table. If no
 *  match is found, it generates an error message using the code supplied in
 *  the parameters.
 *
 *****************************************************************************/

bool siuMsgDispatch (long dtable_padr, int errorCode, siuInst_t *inst, mgb_t *msg)
{
  union {
    siuDispatch_t dtable;
    siuIndepDispatch_t idtable;
  } d;

  typedef void (*vfnptr)(void);
  
  tlong dt_ptr;
  bool done = FALSE;
  bool error = FALSE;
  
  /* make sure that at least the size of the two dispatch types match
   * (in fact they must match except for the prototype, but no way to 
   * test that). */
  utlCheckSize(sizeof(siuDispatch_t) == sizeof(siuIndepDispatch_t));

  dt_ptr = ((tlong) dtable_padr) << SIU_BYTE_ADDRESS_SHIFT ;
  while (!done) {
    utl_prog_to_data ((vfnptr) dt_ptr, (tint *) &d, sizeof (siuDispatch_t)/sizeof(tint));
    if (d.dtable.msgId == 0) {
      error = TRUE;
      done = TRUE;
    }
    else  if (d.dtable.msgId == msg->header.msg_id) {
      if (inst) {
        d.dtable.proc_fn (inst, msg);
      } else {
        d.idtable.proc_fn (msg);
      }
      error = FALSE;
      done = TRUE;
    }
    dt_ptr += (sizeof (siuDispatch_t) << SIU_BYTE_ADDRESS_SHIFT);
  }
  
  if (error && (errorCode != MGB_ERR_MSG_ID_UNDEF))
#ifdef HDMOVE
    if(!NFE_ReceiveInMsg(msg))
        error = FALSE;
#else
    siuErrorIndication (inst->ID, msg->header.trans_id, errorCode);
#endif

  return error;
} /* siuMsgDispatch */



/*-------- Supporting routines for executing DSP-to-MIC Messages ----------*/

/******************************************************************************
 * FUNCTION PURPOSE: Execute an alarm indication message.
 ******************************************************************************
 * DESCRIPTION: Sends an alarm indication message to MIC.  Present
 *              implementation sends only DSP_FATAL_ERROR indication and
 *              sends the message directly to DPR bypassing the message queue.
 *
 *****************************************************************************/

void siuAlarm (tuint module_id)
{
  mgb_t send_message;

  send_message.header.length = MGB_HEADER_SIZE + 2;
  send_message.header.chnum  = (tint)(module_id & SIU_CID_MASK);
  send_message.header.msg_id = MGB_DM_ALARM_INDICATION;
  send_message.header.trans_id = 0; /* not a response to a message */

  send_message.msg_body.alarm_indication.alarm_bits = MGB_ALM_DSP_FATAL_ERROR;
  send_message.msg_body.alarm_indication.numeric_data = module_id;

  /* send message to DPR directly, blocking on echo field */
  siuNduSendMsg(&send_message);
  
} /* siuAlarm */  

/******************************************************************************
 * FUNCTION PURPOSE:  Execute a debug information message. 
 ******************************************************************************
 * DESCRIPTION: Sends a debug info message to MIC.  The calling routine has
 *              the flexibility to either bypass the message fifo or not.
 *
 *****************************************************************************/

void siuDebugReport (tuint id, tuint *str, bool queue)
{
  tint i;
  mgb_t send_message;
  
  send_message.header.length = MGB_HEADER_SIZE + MGB_MSG_MAXNPAR*MGB_MSGPAR_SIZE;
  send_message.header.chnum = (tint)(id & SIU_CID_MASK);
  send_message.header.msg_id = MGB_DM_DEBUG_INFO;
  send_message.header.trans_id = 0; /* not a response to a message */
  
  /* copy debug information to message */
  for (i=0;i<MGB_MSG_MAXNPAR*MGB_MSGPAR_SIZE - 1 ;i++ )
    send_message.msg_body.debug_info.ascii_data[i] = (tuint)str[i];  

  /* Make sure the debug message is NULL terminated */
  send_message.msg_body.debug_info.ascii_data[MGB_MSG_MAXNPAR*MGB_MSGPAR_SIZE -1]
       = (tuint) NULL;

  siuNduSendMsg(&send_message);

} /* siuDebugReport */ 


/******************************************************************************
 * FUNCTION PURPOSE:  Execute Status Response Message.
 ******************************************************************************
 * DESCRIPTION:  Sends a status response message to MIC. 
 *
 *****************************************************************************/
void siuStatusResponse(mgb_t *message)
{
  mgb_t send_message;

  send_message.header.length   = message->header.length;
  send_message.header.chnum    = 0;  /* use channel number 0 - not channel specific */
  send_message.header.msg_id   = MGB_DM_STATUS_RESPONSE;
  send_message.header.trans_id = message->header.trans_id;

  /* Clamp length to message size */
  if (send_message.header.length > (MGB_HEADER_SIZE + MGB_MSG_MAXNPAR))
    send_message.header.length = MGB_HEADER_SIZE + MGB_MSG_MAXNPAR;

  memcpy (&send_message.msg_body.status_response.payload,
          &message->msg_body.status_request.payload,
          (send_message.header.length - MGB_HEADER_SIZE) * sizeof (UINT16));

  /* send message to FIFO */  
  siuNduSendMsg(&send_message);
} /* siuStatusResponse */ 

/******************************************************************************
 * FUNCTION PURPOSE:  Send GENERAL_ACK message
 ******************************************************************************
 * DESCRIPTION:  Sends a status report upon completion of an event such as
 *               channel state changed or redundant packet transmission
 *               complete.
 *
 * void siuGeneralAck(
 *    mgbHeader_t *msghdr, - Header of message requesting ack (channel,
 *                           trans_id, msg_id are used)
 *    tuint ack_condition) - Condition code for ack (new state) change.   
 *****************************************************************************/
void siuGeneralAck (tuint msgId, tuint chnum, tuint transId, tuint ackType, 
                    tuint gen_param, tuint priority)
{
  mgb_t send_message;

  send_message.header.length   = MGB_HEADER_SIZE + 3;
  /* Channel number indicates the which channel underwent state change */
  send_message.header.chnum    = chnum;
  send_message.header.msg_id   = MGB_DM_GENERAL_ACK;
  send_message.header.trans_id = transId; /* response to the message */

  /* Is message just processed, or action is done? */
  send_message.msg_body.general_ack.ack_type   = ackType;
  /* Message which caused state change is provided as well */
  send_message.msg_body.general_ack.message_id = msgId;
  /* Gen param -- used for send packet */
  send_message.msg_body.general_ack.gen_param  = gen_param;

  /* send message to FIFO */  
  siuNduSendMsg(&send_message);
} /* siuGeneralAck */ 


/******************************************************************************
 * FUNCTION PURPOSE:  Call out function after memory write is over.
 ******************************************************************************/
tint siu_memory_write_done (void *flag)
{
  /* DMA Has completed */
  *(bool *)flag = TRUE;
  return 0;
}
/******************************************************************************
 * FUNCTION PURPOSE: Process memory write request from host
 ******************************************************************************
 * DESCRIPTION: Writes specified write pattern to specified memory location.
 *
 *****************************************************************************/
tint siu_memory_read_done (void *flag)
{
  /* DMA Has completed */
  *(bool *)flag = TRUE;
  return 0;
}

void siuMemoryWrite (mgb_t *message)
{
  tuint *data,write_value;
  wordio *io; /* wordio defined in types.h.ioport will not compile on c6x */
  halMemMove_t  memMove;
  tulong addr;
  int    i, j;
  tint   write_len, write_pattern_len;
  tint   write_len_b, write_len_t;
  tint   trans_id_copy;
  tulong source_l;
  tuint *source_p;

  addr = (((tulong)message->msg_body.mem_write.adr_msw) << TYP_TINT_SIZE) | 
          message->msg_body.mem_write.adr_lsw;
  /* write_pattern_len is now in units of tuint */
  write_pattern_len = message->header.length - MGB_HEADER_SIZE - MGB_MDS_MEM_WRITE_HEADER_SIZE;

  /* write_len is now in units of words */
  write_len = message->msg_body.mem_write.fill_length;

  /* Store the transaction ID. This could be overwritten by the non-data type copies */
  trans_id_copy     = message->header.trans_id;

  if (message->msg_body.mem_write.type == MEM_TYPE_DATA)
  {

     /* convert write_len from words to tuint. write_pattern_len is already in tuint */
     write_len_b = UTL_WORDS_TO_BYTES(write_len);
     write_len_t = UTL_BYTES_TO_TINT(write_len_b);

     /* data memory write */
     data = (tuint *) addr;
     for (j = 0; j < write_pattern_len; j++)
     {
        write_value = (tuint) message->msg_body.mem_write.write_val[j];
        for (i = j; i < write_len_t; i = i + write_pattern_len)
        {
           data[i] = write_value;
        }
     }
  }
  else if (message->msg_body.mem_write.type == MEM_TYPE_IO) 
  {
    /* io memory write */
    io= (wordio *)((tuint)addr);
    for (j = 0; j < write_pattern_len; j++)
    {
      write_value = (word) message->msg_body.mem_write.write_val[j];
      for (i = j; i < write_len; i += write_pattern_len)
      {
        io[i] = write_value;
      }
    }
  }
  else if ((message->msg_body.mem_write.type == MEM_TYPE_PROGRAM) ||
           (message->msg_body.mem_write.type == MEM_TYPE_DMA))
  {
    /* HAL requires that the source data be 32 bit aligned */
    source_p = &(message->msg_body.mem_write.write_val[0]);
    source_l = (tulong)source_p; 
    if (source_l & 0x0001)  {
      /* The source is on an odd address. Shift this upwords 1 word. This will
       * destroy the transaction ID of the header, which was saved */
      source_p -= 1;
      for (i = 0; i < write_len; i++)
        source_p[i] = source_p[i+1];
    }

    /* Put the source and destination addresses in HAL format */
    memMove.src  = siuContext.halApi->halMapLocalToGlobal ((halGlobAddr_t)source_p);
    memMove.dest = (halGlobAddr_t) addr;
    memMove.size = write_len;
    
    /* Invoke HAL to perform the move. Wait until the move is complete */
    siuContext.halApi->halGlobMemMove (1, &memMove, NULL, NULL, TRUE);
  }
  else /* An invalid type has been specified. Create an error indication */
  {    /* message in place of the memory write request */
     siuErrorIndication (siuContext.ID, message->header.trans_id, 
                         MGB_ERR_INVD_MEM_WRITE); 
     return;
  }

  /* Return an Ack if the transaction ID was non-zero */
  if (trans_id_copy)  {
    /* Form the message on top of the original message. This is valid only 
     * because the source message length is garaunteed to be larger then 
     * the ack */
    i = message->header.msg_id;
    message->header.length   = MGB_HEADER_SIZE + 3;
    message->header.chnum    = 0;
    message->header.msg_id   = MGB_DM_GENERAL_ACK;
    message->header.trans_id = trans_id_copy;

    message->msg_body.general_ack.gen_param  = 0;
    message->msg_body.general_ack.message_id = i;
    message->msg_body.general_ack.ack_type   = MGB_ACK_TYPE_DONE;

    /* send message to FIFO */
    siuNduSendMsg(message);
  }
   /*  siuErrorIndication (siuContext.ID, message->header.trans_id, 
                         MGB_ERR_INVD_MEM_WRITE); */
    
} /* siuMemoryWrite() */

/******************************************************************************
 * FUNCTION PURPOSE: Process memory read request from host
 ******************************************************************************
 * DESCRIPTION: Returns current memory dump at specified location
 *
 *****************************************************************************/
void siuMemoryRead (mgb_t *message) 
{
  siuReadMemPacing_t read_mem;

  read_mem.cur_addr = 
         (((tulong)message->msg_body.get_mem_read.adr_msw) << TYP_TINT_SIZE) | 
          message->msg_body.get_mem_read.adr_lsw;
  read_mem.rem_len =
         (((tulong)message->msg_body.get_mem_read.len_msw) << TYP_TINT_SIZE) |
          message->msg_body.get_mem_read.len_lsw;

  read_mem.type = message->msg_body.get_mem_read.type;

  read_mem.msg_rate = message->msg_body.get_mem_read.msg_rate;

  read_mem.trans_id = message->header.trans_id;

  read_mem.isr_counter = 0;

  /* Convert units from per ms to per isr */
  read_mem.msg_rate = 5 * read_mem.msg_rate / ggConfig.pcmIntPer5ms;
  if (!read_mem.msg_rate) {
    read_mem.msg_rate++; /* 0 not allowed */
  }

  /* Use untranslated rate to see if its instant or paced */
  if (message->msg_body.get_mem_read.msg_rate) {
    /* If paced, store it */
    siuContext.readMemPacing = read_mem;
  } else {
    /* Else execute it now */
    siu_memory_read_exec (&read_mem);
  }
}

/******************************************************************************
 * FUNCTION PURPOSE: Process memory read request from host or isr
 ******************************************************************************
 * DESCRIPTION: Processes both paced and single read requests.
 *
 *****************************************************************************/
#define SIU_MEM_READ_PAYLOAD (MGB_MDS_MAX_MEM_RW_BLOCK_SIZE-4)
#define SIU_MEM_READ_STREAM  1460
tuint   vDebugStreamMemSlowDown = 0;
tulong  vDebugStreamMemSeqNum   = 0;
tuint   vDebugStreamMemSlowDownFactor = 25;
#pragma DATA_SECTION(vDebugNeuStreamerScratch, ".vDebugNeuStreamerScratch")
word vDebugNeuStreamerScratch[1500];
void siu_memory_read_exec (void *vread_mem)
{
  siuReadMemPacing_t *read_mem = (siuReadMemPacing_t *)vread_mem;
  tuint  *data;
  wordio *io;
  int i;
  tint read_length;
  halGlobAddr_t halsrc;
  halMemMove_t memMove;
  volatile bool done_flag;
  tuint responses_left;
  mgb_t  send_message;
  tulong dest_addr;
  union {
    tulong align_me;
    tuint data[SIU_MEM_READ_PAYLOAD+2];
    /* +2 for alignment required by DMA */
  } dma_buffer;
  tuint SIU_MEM_MAX;
  bool retVal = TRUE;

  if (read_mem->type == MEM_TYPE_STREAM) { 
    if (vDebugStreamMemSlowDown == 0) {
      vDebugStreamMemSlowDown = vDebugStreamMemSlowDownFactor;
    }
    vDebugStreamMemSlowDown --;
    if (vDebugStreamMemSlowDown > 1)
      return;
  }

  responses_left = read_mem->msg_rate;

  while(responses_left && read_mem->rem_len) {
    if (read_mem->type == MEM_TYPE_STREAM) {
        SIU_MEM_MAX = SIU_MEM_READ_STREAM;
    } else {
        SIU_MEM_MAX = SIU_MEM_READ_PAYLOAD;
    }
    if (read_mem->rem_len > SIU_MEM_MAX) {
      read_length = SIU_MEM_MAX;
    } else {
      read_length = (tint)read_mem->rem_len; /* Must fit in tint! */
    }
  
    send_message.msg_body.mem_read.memory_address_MSW = 
      siu_get_umsw(read_mem->cur_addr);
    send_message.msg_body.mem_read.memory_address_LSW = 
      siu_get_ulsw(read_mem->cur_addr);
    send_message.msg_body.mem_read.memory_control = read_mem->type;
    send_message.msg_body.mem_read.read_length = read_length;
    
    /* Mark the last message to be sent */
    if(read_length == read_mem->rem_len)
        send_message.msg_body.mem_read.memory_control |= 
            MGB_MDS_MEM_CTRL_MEM_READ_END_FLAG;    


    send_message.header.length   = MGB_HEADER_SIZE + 4 + read_length;
    send_message.header.chnum    = 0;  /* not channel specific */
    send_message.header.msg_id   = MGB_DM_MEM_READ;
    send_message.header.trans_id = read_mem->trans_id;

    switch(read_mem->type) {
      case MEM_TYPE_DATA:
        /* Data memory read */
        data = (tuint *)read_mem->cur_addr;  

        for (i = 0; i < read_length; i++)
          send_message.msg_body.mem_read.data[i] = data[i];
        break;
      case MEM_TYPE_IO:
        /* I/O memory read */
        io = (wordio *)((tuint)read_mem->cur_addr);
        for (i = 0; i < read_length; i++)
          send_message.msg_body.mem_read.data[i] = io[i];
        break;
      case MEM_TYPE_PROGRAM:
      case MEM_TYPE_DMA:
      {
        tuint dma_read_length = read_length;
        word *copy_start = (word *)&dma_buffer;
        /* halsrc = siuContext.halApi->halMapLocalToGlobal (addr); */
        halsrc = (halGlobAddr_t)read_mem->cur_addr;
        if (halsrc & 1) {
          halsrc --;
          dma_read_length ++;
          copy_start ++;
        }
        if (dma_read_length & 1) {
          dma_read_length ++;
        }
    
        /* The HAL api requires that the source and destination addresses be 32
         * bit aligned. No error is returned if this is not the case, but the
         * data is placed in the wrong location. Force the mem move to put the
         * resulting data into an even address, then adjust it up if required */
        dest_addr = (tulong) &dma_buffer;
        memMove.dest = siuContext.halApi->halMapLocalToGlobal (dest_addr);
        memMove.src  = halsrc;
        memMove.size = dma_read_length;
        siuContext.halApi->halGlobMemMove (1, &memMove, NULL, NULL, TRUE);

        memcpy (send_message.msg_body.mem_read.data, copy_start, read_length);
        break;
      }
      case MEM_TYPE_STREAM:
      {
        xferPktInfo_t pkt;
        void      *data;
        extern    NDU_gmacContext_t NDU_gmacContext;
        void      *payloadBegin;
        tint      pktLength;

        /* Send only 1 packet */
        if ((NDU_gmacContext.nduTrace).neuInst != NULL) {
           pkt.srcPort = 0;
           pkt.npkts   = 1;
           payloadBegin   = (void *)(read_mem->cur_addr);
           memcpy(vDebugNeuStreamerScratch,&vDebugStreamMemSeqNum,4);
           memcpy(vDebugNeuStreamerScratch+4,payloadBegin,read_length);
           vDebugStreamMemSeqNum++;
           pktLength   = read_length+4;
           data = (void *)(vDebugNeuStreamerScratch);
           pkt.pktSize = &pktLength;
           pkt.pktIn   = &data;
           pkt.type    = XFER_PKT_TYPE_NULL;
           SIU_STATE_BEGIN();
             (*gg_neu_ctable.neuSendIn) ((NDU_gmacContext.nduTrace).neuInst, &pkt);
           SIU_STATE_END();
         }
        break;
      }
      default:
        /* Bad request: Invlaid type */
        read_mem->msg_rate = 0;
        siuErrorIndication (siuContext.ID, read_mem->trans_id,
                            MGB_ERR_INVD_MEM_READ);
        return;
    }

    /* send message to FIFO */
    if (read_mem->type != MEM_TYPE_STREAM) {
        retVal = siuNduSendMsg(&send_message);
    }

    if (retVal) {
      responses_left --;
      read_mem->rem_len -= read_length;
      if (!read_mem->rem_len) {
        /* The request is completely done.  Shut down scheduling */
        read_mem->msg_rate = 0;
      }
      read_mem->cur_addr += read_length;
    } else {
      break; /* Failed to send the message, try again next isr */
    }
  }
} /* siuMemoryRead */


 /******************************************************************************
 * FUNCTION PURPOSE:  Report NDU (Network) statistics  
 ******************************************************************************
 * DESCRIPTION: Report NDU statistics.
 *****************************************************************************/

void siuGetNetworkStats (void *siuInst, bool reset, tuint trans_id, tuint priority)
{
  /* Note: siuInst isn't valid -- just match API */
  mgb_t send_message;
#ifdef SIU_NDU_HETERO_PKT_IFACE
 tuint           nduCallTblIdx;
#endif

  send_message.header.length   = MGB_HEADER_SIZE + 30 * MGB_MSGPAR_SIZE;
  send_message.header.chnum    = 0;  /* use channel number 0 - not channel specific */
  send_message.header.msg_id   = MGB_DM_GET_NETWORK_STAT;
  send_message.header.trans_id = trans_id;
   
#ifdef SIU_NDU_HETERO_PKT_IFACE
 nduCallTblIdx = 
     SIU_PKT_GET_IDX_IFACE(siuContext.msg_pkt_iface_idx,
                           SIU_CTL_PKT_IDX_MASK,
                           SIU_CTL_PKT_IDX_SHIFT);
  NDU_GetStats(siuContext.ifaceNduCallTable[nduCallTblIdx],&send_message.msg_body.network_stat, reset);
#else
  NDU_GetStats(NDU_pCallTable,&send_message.msg_body.network_stat, reset);
#endif

  /* send message to FIFO */
  siuNduSendMsgPriority(&send_message,(NDU_DPRMsgType_t)priority); 
} /* siuGetNetworkStats */ 
/******************************************************************************
 * FUNCTION PURPOSE:  Sends an error indication to MIC.
 ******************************************************************************
 * DESCRIPTION: Reports an error indication to MIC.  These are non-fatal errors.
 *              The calling routine provides a transaction ID if this error
 *              is associated with a particular message received from MIC. Else,
 *              if the error is not associated with a particular message the
 *              calling routine should set the trans_id to zero.  
 *
 *****************************************************************************/
void siuErrorIndication(tuint moduleID, tuint trans_id, tuint error_code)
{
  mgb_t send_message;
  siuMsgAck_t *msgAck;

  /* Send error message only if allowed */
  if (siuContext.errorReport == TRUE) {
    send_message.header.length = MGB_HEADER_SIZE + 2;
    send_message.header.chnum  = (tint)(moduleID & SIU_CID_MASK);
    send_message.header.msg_id = MGB_DM_ERROR_INDICATION;
    send_message.header.trans_id = trans_id;
    send_message.msg_body.error_indication.error_code = MGB_ERR_REPORT_CODE(moduleID, error_code);

    /* Stick message ID of last message processed by DSP */
    if(siuInterruptMsgAck.msgId == MGB_ERR_MSG_ID_UNDEF) 
      msgAck = &siuNormalMsgAck;
    else
      msgAck = &siuInterruptMsgAck;

    /* Send the error for the msg, and stop the ack */
    send_message.msg_body.error_indication.msg_id = msgAck->msgId;
    msgAck->ackType = 0;

    /* send message to FIFO */  
    siuNduSendMsg(&send_message);  
  } /* siuContext.errorReport == TRUE */
} /* siuErrorIndication() */ 


/******************************************************************************
 * FUNCTION PURPOSE:  Sends a Channel Exception message to MIC.
 ******************************************************************************
 * DESCRIPTION: Reports a channel exception to MIC.  
 *
 *****************************************************************************/
void siuChannelException(tuint moduleID, tuint error_code)
{
  mgb_t send_message;
  
  send_message.header.length = MGB_HEADER_SIZE + 2;
  send_message.header.chnum  = (tint)(moduleID & SIU_CID_MASK);
  send_message.header.msg_id = MGB_DM_CHANNEL_EXCEPTION;
  send_message.header.trans_id = 0;
  send_message.msg_body.chan_exception.module_id = SIU_EXTRACT_MID(moduleID);
  send_message.msg_body.chan_exception.exception_code = error_code;

   /* send message to FIFO */
   siuNduSendMsg(&send_message);
     
} /* siuChannelException() */ 
/******************************************************************************
 * FUNCTION PURPOSE: Sends Packet Violation Reports from NEU to Host.
 ******************************************************************************
 * DESCRIPTION: Sends Packet Violation Reports from NEU to Host.
 *
 *  void siuNeuViolReport (
 *
 *    (void  *targetInst,        - pointer to SIU instance) old
 *    tuint ID,                 - ID of calling instance 
 *    void  *vdata)             - pointer to reason info
 *
 *****************************************************************************/
 void siuNeuViolReport ( tuint ID, void *vdata )
 {
   siuInst_t *inst = (siuInst_t *) SIUINSTp((ID & SIU_CID_MASK) - SIU_FIRSTCH);
   mgb_t     send_message;
   struct MGB_DMS_NEU_RX_PACKET_VIOLATION *data = 
       (struct MGB_DMS_NEU_RX_PACKET_VIOLATION *)vdata;
   bool allowPktViol = FALSE;

   /* See if a packet violation is allowed yet based on clock */
   if (inst->neuInst) { 
     allowPktViol = gg_neu_ctable.pktViolThrottle (inst->neuInst);
   }
 
   if (!allowPktViol)
     return;

   /* prepare the message */
   send_message.header.length   = MGB_HEADER_SIZE + (sizeof(*data)/sizeof(dprword));
   send_message.header.chnum    = (tint) (inst->ID & SIU_CID_MASK);
   send_message.header.trans_id = 0;    /* Not a responce to a MICRO message */
   send_message.header.msg_id   = MGB_DM_NEU_RX_PACKET_VIOLATION;

   send_message.msg_body.neu_rx_packet_violation = *data;

   /* send message to FIFO */
   siuNduSendMsg(&send_message);  
 } /* siuNeuViolReport */
/******************************************************************************
 * FUNCTION PURPOSE: Realtime Trace Request
 ******************************************************************************
 * DESCRIPTION: Realtime Trace Request Processing
 *
 *****************************************************************************/
void siuGetRtTrace (siuInst_t *inst, mgb_t *message)
{
  SIU_RTTRACE_BEGIN();
    utlRtTraceContext.enable[message->header.chnum - 1].enabled_bitfield &=
      ~message->msg_body.get_rttrace.mask;  
    utlRtTraceContext.enable[message->header.chnum - 1].enabled_bitfield |=  
      (message->msg_body.get_rttrace.enable & message->msg_body.get_rttrace.mask);
  SIU_RTTRACE_END();
} /* siuGetPcmPktTrace */

/******************************************************************************
 * FUNCTION PURPOSE: Send realtime traces to the host
 *****************************************************************************/
void siuRtTraceFlush () {
  tuint otherHalf;
  mgb_t *msg;
  tuint freespace;
  NDU_DPRMsgType_t queue_id;

  /* Is there something to flush in the non-busy buffer half? */
  otherHalf = (utlRtTraceContext.bufferNumber + 1) & 1;
  if (utlRtTraceContext.buffers[otherHalf].index > sizeof(mgbHeader_t)) {
  SIU_RTTRACE_BEGIN();
    /* Is there enough free space for this message? */
    queue_id = siuMessageLevel (MGB_DM_RTTRACE);     /* determine the queue into */
    freespace = siuNduGetFree (queue_id);            /* which this msg will go   */    
    if (freespace >= ggConfig.rtTracePktFree) {
      /* Ship out the message */
      msg = (mgb_t *)utlRtTraceContext.buffers[otherHalf].data;

      msg->header.length   = (utlRtTraceContext.buffers[otherHalf].index)/sizeof(dprword);
      msg->header.trans_id = 0;
      msg->header.chnum    = 0;
      msg->header.msg_id   = MGB_DM_RTTRACE;
      utlRtTraceContext.buffers[otherHalf].index = 0; /* Mark empty */

      /* send message to FIFO , only for utopia interface*/
      siuNduSendMsg(msg); 
    }
  SIU_RTTRACE_END();
  }
} /* siuRtTraceFlush */

/******************************************************************************
 * FUNCTION PURPOSE: Realtime Trace Processing
 ******************************************************************************
 * DESCRIPTION: Collects realtime traces
 *
 *****************************************************************************/
void siuRtTraceFcn (tuint ID, tuint dataSize, void *data, tuint streamId, 
                    utlRtTracePacked_e packed)
{
#if defined(VOICE) || defined(VOICE_FAX)
  tuint chnum         = UTL_EXTRACT_CHNUM(ID);
  tuint dataSizeWords;
  tuint index;
  tuint newIndex;
  struct MGB_DMS_RTTRACE_HDR *perMsgHdr;
  struct MGB_DMS_RTTRACE_RECHDR *perTraceHdr;
  word *payloadPtr;
  tulong now, deltaT;
  extern const tuint gg_rttrace_bufsize;
  tuint isDataSizeInBytes = 1, streamIdIsPCM, compandingIsLinear;

  streamIdIsPCM = ( (streamId == MGB_RTTRACE_STREAM_ID_PCM_IN) || 
                    (streamId == MGB_RTTRACE_STREAM_ID_PCM_OUT)
                  );

  compandingIsLinear = (SIU_READ_COMPANDING_LAW(SIUINSTp(chnum)) == CONST_COMP_LINEAR);

  if (streamIdIsPCM && compandingIsLinear) {
    isDataSizeInBytes = 0;
    packed = UTL_RTTRACE_PACKED;
  }

  dataSizeWords = (isDataSizeInBytes) ? UTL_BYTES_TO_WORD(dataSize): dataSize;

  if (dataSizeWords > (gg_rttrace_bufsize - 
                      sizeof(struct MGB_DMS_RTTRACE_HDR) - 
                      sizeof(struct MGB_DMS_RTTRACE_RECHDR) - 
                      sizeof(mgbHeader_t))) {
    /* Increment sequence number to show data loss */
    utlRtTraceContext.seqn++;
    return;
  }

  SIU_RTTRACE_BEGIN(); /* Could be processing pkts and get pcm int */
    now = siu_isr_clock * 4;

    /* Does the new frame/packet fit in the buffer? */
    index      = utlRtTraceContext.buffers[utlRtTraceContext.bufferNumber].index;
    newIndex   = index + dataSizeWords + sizeof(struct MGB_DMS_RTTRACE_HDR);
    payloadPtr = (word *)utlRtTraceContext.buffers[utlRtTraceContext.bufferNumber].data;
    perMsgHdr  = (struct MGB_DMS_RTTRACE_HDR *)(payloadPtr + sizeof(mgbHeader_t));
    if (index > sizeof(mgbHeader_t)) {
      /* Extract the header timestamp and find the relative difference */
      deltaT = (((tulong)(perMsgHdr->sqn_tsmsw & MGB_RTTRACE_TSMSW_MASK) >> 
                 MGB_RTTRACE_TSMSW_SHIFT) << 16) |
               perMsgHdr->tslsw;
      deltaT = (now - deltaT) & 0xFFFFFFUL;
    } else {
      deltaT = 0;
    }

    if ((newIndex > gg_rttrace_bufsize) || (deltaT > 0xff)) {
      /* Move to next buffer in double buffer or N buffer */
      utlRtTraceNextBuf();
      siuRtTraceFlush(); /* Attempt to send traces to host */
      deltaT     = 0;
      index      = sizeof(mgbHeader_t);
      newIndex   = index + dataSizeWords + sizeof(struct MGB_DMS_RTTRACE_HDR);
      payloadPtr = (word *)utlRtTraceContext.buffers[utlRtTraceContext.bufferNumber].data;
    }

    payloadPtr += index;

    /* Put on a per-msg header */
    if (index == sizeof(mgbHeader_t)) {
      /* Reset minimum scheduling clock */
      utlRtTraceContext.clock = ggConfig.rtTraceInterval;
      perMsgHdr->sqn_tsmsw = ((utlRtTraceContext.seqn << MGB_RTTRACE_SQN_SHIFT) &
                              MGB_RTTRACE_SQN_MASK) |
                             ((((now >> 16) & 0xff) << MGB_RTTRACE_TSMSW_SHIFT) &
                              MGB_RTTRACE_TSMSW_MASK);
      perMsgHdr->tslsw = now & 0xffff;
      newIndex   += sizeof(struct MGB_DMS_RTTRACE_HDR);
      payloadPtr += sizeof(struct MGB_DMS_RTTRACE_HDR);
      utlRtTraceContext.seqn ++;
    }

    /* Put on a per-trace header */
    perTraceHdr = (struct MGB_DMS_RTTRACE_RECHDR *)payloadPtr;
    perTraceHdr->streamId_chan = ((streamId << MGB_RTTRACE_STREAMID_SHIFT) &
                                  MGB_RTTRACE_STREAMID_MASK) |
                                 ((chnum << MGB_RTTRACE_CHNUM_SHIFT) &
                                  MGB_RTTRACE_CHNUM_MASK);
    perTraceHdr->ts_len = ( (((tuint)deltaT) << MGB_RTTRACE_TS_SHIFT) &
                            MGB_RTTRACE_TS_MASK ) |
                          ( (UTL_WORDS_TO_BYTES(dataSizeWords) << MGB_RTTRACE_LEN_SHIFT) &
                            MGB_RTTRACE_LEN_MASK);
    /* Point to the payload slot */
    payloadPtr += sizeof(struct MGB_DMS_RTTRACE_HDR);

    /* Put the payload into the buffer */
    if (packed == UTL_RTTRACE_UNPACKED) {
      utlPackLSBof16bitsIntoWords_m ((linSample *)data, payloadPtr, dataSize, 0);
    } else {
      memcpy (payloadPtr, data, dataSizeWords); 
    }

    /* Adjust the index */
    utlRtTraceContext.buffers[utlRtTraceContext.bufferNumber].index = newIndex;
  SIU_RTTRACE_END();
#else
  return;
#endif
} /* utlRtTrace */
/******************************************************************************
 * FUNCTION PURPOSE: Control of Tx and Rx packet flow is done in RCU/ACU
 ******************************************************************************
 * DESCRIPTION: Bitfields indicating packet flow for each of event spaces in
 *      both to-Pkt and to-Tele directions, are passed to RCU/ACU.
 ******************************************************************************/
void siuChannelState (siuInst_t *inst, mgb_t *message)
{
#ifdef GG_INCLUDE_AAL2
  acuControl_t acuCtrl;
#endif

#ifndef HDCT
  rcuCtrl_t rcuCtrl;
  
#ifndef HDVIDEO
  /* Protocol independent actions */
  if(message->msg_body.channel_state.pkt2tele_mask & MGB_CHANNEL_STATE_MUTE) {
    /* Muting from PKT side changed */
    if(message->msg_body.channel_state.pkt2tele_value & MGB_CHANNEL_STATE_MUTE) {
      SIU_SET_RX_MUTE(inst, 1);
    } else {
      SIU_SET_RX_MUTE(inst, 0);
    }
  }

  if(message->msg_body.channel_state.tele2pkt_mask & MGB_CHANNEL_STATE_MUTE) {
    /* Muting from TELE side changed */
    if(message->msg_body.channel_state.tele2pkt_value & MGB_CHANNEL_STATE_MUTE) {
      SIU_SET_TX_MUTE(inst, 1);
    } else {
      SIU_SET_TX_MUTE(inst, 0);
    }
  }
#endif /* HDVIDEO */
#endif /* HDCT */
#ifdef GG_INCLUDE_AAL2
  if (SIU_IS_PROTO_AAL2(inst)) {
    acuCtrl.code = ACU_CONTROL_PACKET_FLOW;
    acuCtrl.u.pkt_flow = &message->msg_body.channel_state;
    gg_acu_ctable.acuControl (inst->net.acuInst, &acuCtrl);
  }
#endif
#ifndef HDCT
  if(SIU_IS_PROTO_RTP(inst) ||
     (SIU_IS_PROTO_IU(inst) && SIU_IS_IU_OVER_IP(inst))){
    rcuCtrl.code = rcu_CTRL_PKT_FLOW;
    rcuCtrl.u.pkt_flow = &message->msg_body.channel_state;
    gg_rcu_ctable.rcuControl (inst->net.rcuInst, &rcuCtrl);
  }
#endif
#if (!defined(HDVIDEO) && !defined(HDCT))
  if (SIU_IS_PROTO_AAL1(inst)) {
    gg_siu_ctable.Aal1.channel_state (inst, message);
  }
  if (SIU_IS_PROTO_IU(inst)) {
    gg_siu_ctable.Iuup.channel_state (inst, message);
  }  

  /* Put channel into standby mode if requested */
  if(message->msg_body.channel_state.tele2pkt_mask & MGB_CHANNEL_STATE_STANDBY)
  {
    SIU_SET_STANDBY(inst, 
    ((message->msg_body.channel_state.tele2pkt_value & MGB_CHANNEL_STATE_STANDBY) ? 1 : 0));
  }
#endif
} /* Channel State message processing function */
/******************************************************************************
 * FUNCTION PURPOSE: Return message priority
 ******************************************************************************
 * DESCRIPTION: Returns the message priority for a message ID.
 ******************************************************************************/
NDU_DPRMsgType_t siuMessageLevel (tuint msgId)
{
  switch (msgId)  {
    case MGB_DM_RECEIVE_PACKET:
    case MGB_DM_SIG_TRANSFER:
    case MGB_DM_SECURITY_KEY_REQUEST: /* Security Key Request is High Priority Message */
      return (NDU_DPR_SIGNAL_MSG);

    case MGB_DM_GET_ERROR_STAT:    
    case MGB_DM_GET_RXTX_STAT:
    case MGB_DM_GET_FAX_MODEM_STAT:
    case MGB_DM_GET_FAX_STAT:
    case MGB_DM_GET_FAX_CALL_STAT:
    case MGB_DM_GET_EC_DBGSTAT:
    case MGB_DM_GET_MODEM_RXTX_STAT:
    case MGB_DM_GET_MODEM_RELAY_STAT:
    case MGB_DM_GET_CLEAR_DATA_STAT:
    case MGB_DM_GET_MAG_DBGSTAT:
    case MGB_DM_GET_NETWORK_STAT:
    case MGB_DM_GET_DSIG_STAT:
    case MGB_DM_GET_VP_DELAY:
    case MGB_DM_GET_LEVELS:
    case MGB_DM_GET_VQM_STAT:
      return (NDU_DPR_LOW_MSG);

    case MGB_DM_STATUS_RESPONSE:
      return (NDU_DPR_NORMAL_MSG);

    default:
      return (NDU_DPR_NORMAL_MSG);
  
  } /* switch */
} /* siuMessageLevel */


/******************************************************************************
 * FUNCTION PURPOSE: Perform ID to channel translation for messages
 ******************************************************************************
 * DESCRIPTION: ID to channel translation is performed if the message ID
 *              is a channel specific message.
 ******************************************************************************/
tuint siu_translate_dynamic_chnum (tuint msg_id, tuint chan) 
{
  tuint chnum = chan;

  if (siuContext.dynBufAlloc.dynBufAllocEnable ) {
#if (!defined(HDVIDEO) && !defined(HDCT))
     if (siuContext.RdndcMode != SIU_RDNDC_MODE_SHADOW)) {
#endif
    /* Buffer to channel translations are NOT done for the following messages */
    if (!(msg_id == MGB_DM_DSIG_REPORT_STATE   ||
          msg_id == MGB_DM_DSIG_CADENCE_DONE   ||
          msg_id == MGB_DM_DSIG_DIGIT_EVENT    ||
#if 0
          msg_id == MGB_DM_DSIGTST_REPORT      ||
          msg_id == MGB_DM_DSIGTST_RET_SIGDEF  ||
#endif
          msg_id == MGB_DM_GET_DSIG_STAT       ||
          msg_id == MGB_DM_AIC_ATTENTION       ||
          msg_id == MGB_DM_GENERAL_ACK         ||
          msg_id == MGB_DM_STATUS_RESPONSE     ||
          msg_id == MGB_DM_RESTART_INDICATION  ||
          msg_id == MGB_DM_READY_INDICATION    ||
          msg_id == MGB_DM_GET_NETWORK_STAT    ||
          msg_id == MGB_DM_MEM_READ            ||
          msg_id == MGB_DM_GET_GLOBAL_PKT_STAT ||
          msg_id == MGB_DM_RTTRACE             ||
          msg_id == MGB_DM_MACR_STATS_REPLY    ||
          msg_id == MGB_DM_MACR_TABLE_READ_RESP ||
     #if ( (defined(NDU_DBG4)) || (defined(NDU_DBG5)) )
     msg_id == MGB_MD_OPEN_CHANNEL       ||
     #endif

          msg_id == MGB_DM_DEBUG_TRACE))
          ;
#if (!defined(HDVIDEO) && !defined(HDCT))
     }
#endif
  }
  return chnum;
}

/******************************************************************************
 * FUNCTION PURPOSE: Suppress all unsolicited events in STANDBY mode.
 ******************************************************************************
 * DESCRIPTION: If transaction ID is 0, message is susspended.
 ******************************************************************************/
bool siu_susspend_unsolicited (siuInst_t *inst, tuint id) {
  if (id == 0)
    return TRUE;
  else
    return FALSE;
}


/******************************************************************************
 * FUNCTION PURPOSE: Halt DSP, to allow core dump
 ******************************************************************************
 * DESCRIPTION: Halts DSP in infinite loop with interrupts off.
******************************************************************************/
void siuHaltDsp (mgb_t *message) 
{
  halControl_t halControl;

  if ((message->header.length == MGB_HEADER_SIZE) ||
      (message->msg_body.halt_dsp.core == siuContext.coreId)) {
    /* Halt myself */
    thwDisableAllInt ();  /* disable all interrupts */
    siu_preserve_registers(SIU_CPU_REGISTERS_HALT_MSG, siuContext.siu_common_scratch);
    /* No channel translation needed -- channel indep message */
    siuGeneralAck (message->header.msg_id, message->header.chnum,
                   message->header.trans_id, MGB_ACK_TYPE_DONE, 0, 
                   SIU_DPR_NORMAL_MSG);

    siuContext.halt.halt.halted_if_set = 1;
    /* Flush the message through HAL */
    halControl.cmd = HAL_COMMAND_HALT;
    halControl.u.periphReset.portId =  NULL;
  
    (*siuContext.halApi->halControl) (&halControl);
    while(1);
  } else {
    /* Halt the other guy */
    siuHalt_t halt_local;
    halMemMove_t memMove;
    tuint *source_p;

    halt_local.halt.halt_if_set = 1;
    halt_local.halt.halted_if_set = 0;
    source_p = (tuint *)&halt_local;

    /* Put the source and destination addresses in HAL format */
    memMove.src  = siuContext.halApi->halMapLocalToGlobal ((halGlobAddr_t)source_p);
    memMove.dest = (*siuContext.halApi->halFormGlobalAddr)((halLocAddr_t)&siuContext.halt, (tint)message->msg_body.halt_dsp.core);
    memMove.size = sizeof(halt_local);
    
    /* Invoke HAL to perform the move. Have HAL wait until the move is complete */
    (*siuContext.halApi->halGlobMemMove) (1, &memMove, NULL, NULL, TRUE);

  }
} /* siuHaltDsp */

/******************************************************************************
 * FUNCTION PURPOSE: Send back response with the run time state information of DSP
 ******************************************************************************
 * DESCRIPTION: 
 *                  Function to get the run time state information of DSP
 *                  Currently only Security related state information 
 *                  is sent back as response.
******************************************************************************/
void siuGetDspStateInfo (mgb_t *message) 
{
#ifdef SIU_SECURITY
    siuMsuGetDspStateResp(message);
#endif
}

/******************************************************************************
 * FUNCTION PURPOSE: Update the 1:1 redundancy state information
 ******************************************************************************
 * DESCRIPTION: Function to set the redundancy state information. Currently only
 *              MSU state is being updated.
 *                  
******************************************************************************/
void siuSetDspStateInfo (mgb_t *message) 
{

#ifdef SIU_SECURITY
    siuMsuSetDspStateReq(message);    
#endif
}

/******************************************************************************
 * FUNCTION PURPOSE: Configure PKTIO interface for the DSP core
 ******************************************************************************
 * DESCRIPTION: 
 *                  The function will configure the PKTIO interfaces for the
 *                  DSP
******************************************************************************/
void siuPktioIfCfg (mgb_t *message) 
{
    bool    retStatus = TRUE;

    if(message->msg_body.pktio_config.pkt_if_type == MGB_PKTIO_IF_GMAC){
#ifdef NDU_GMAC
    retStatus = siu_pktio_macport_config(&message->msg_body.pktio_config.params.mac);
#endif
    }

    if(!retStatus){
        siuErrorIndication (siuContext.ID, 
                            message->header.trans_id, 
                            MGB_ERR_INVD_PKTIO_IF_CONFIG); 
    }
}
    
/******************************************************************************
 * FUNCTION PURPOSE: Support for multiple statistics request from the host
 ******************************************************************************
 * DESCRIPTION: Bit field is provided on the input, and array of function 
 *              pointers. Each function has same api, and it is invoked if
 *              related bit in the bit-field is set.
 * void siu_stats_request_single_word (
 *   tuint stat_req_mask,             - bit-field indicating stats requests
 *   fcnGetStats_t *array_get_stats,  - pointer to an array of call-out functions
 *   void *siuInst,                   - channel instance
 *   bool reset,                      - Statistics to be resetted or not
 *   tuint trans_id                   - transaction ID of this request
 * )
 *****************************************************************************/
static void siu_stats_request_single_word (tuint stat_req_mask, 
                                    const fcnGetStats_t *array_get_stats, 
                                    void *siuInst, bool reset, tuint trans_id,
                                    tuint priority)
{
tuint i;
fcnGetStats_t get_stats;
siuInst_t *inst = (siuInst_t *)siuInst;
  for (i = 0; stat_req_mask; i ++)
  {
    if(stat_req_mask & 0x0001) {
      get_stats = array_get_stats[i];
      if(get_stats)
        (*get_stats) (inst, reset, trans_id, priority);
        else
          break; /* delimiter encountered */
    }
    stat_req_mask >>= 1;
  }
}
/******************************************************************************
 * FUNCTION PURPOSE: Server function for multiple stats request
 ******************************************************************************
 * DESCRIPTION: All stats request cannot fit into a single word bit-field.
 *              So, it is iterrated for low and high 16-bit bit-fields.
 *              Statistics reset indication is common for all individual stat
 *              requests.
 ******************************************************************************/
void siuStatsRequest (siuInst_t *inst, mgb_t *message) {
  bool reset     = (message->msg_body.stats_req.stat_req_reset & 0x0001);
  tuint priority;
  tuint trans_id = message->header.trans_id;
  tuint bit_cnt  = 0;
  tuint bfield, bfield_lsw, bfield_msw, bfield2_lsw, bfield2_msw;
#ifdef SIUTBLPRG
  union {
    tuint           PtrStatsReqChst[SIU_MAX_NUM_CHST];
    fcnGetStats_t   PtrarrayGetStats[SIU_MAX_NUM_CHST];
  } scratch_u;
#endif

  if(siuContext.msg_priority_config_support == FALSE) 
    /* The priority level of error stat is taken as representative of all the stats priority */
    priority =siuMessageLevel(MGB_DM_GET_ERROR_STAT);
  else
    priority = ((message->msg_body.stats_req.stat_req_reset>>8) & 0x00FF);

  siuGetDispatchTblStatsReqChst(dispatchTblStatsReqChst,
                                siuLoadsiuStatsReqChstMaskLsw,
                                siuStatsReqChstMaskLsw);
  bfield_lsw = (message->msg_body.stats_req.stat_req_mask_lsw
                & dispatchTblStatsReqChst[inst->state]);
  siuGetDispatchTblStatsReqChst(dispatchTblStatsReqChst,
                                siuLoadsiuStatsReqChstMaskMsw,
                                siuStatsReqChstMaskMsw);
  bfield_msw = (message->msg_body.stats_req.stat_req_mask_msw
                & dispatchTblStatsReqChst[inst->state] 
                & (~MGB_STATS_GLOBAL_MASK));

  siuGetDispatchTblStatsReqChst(dispatchTblStatsReqChst,
                                siuLoadsiuStatsReqChstMask2Lsw,
                                siuStatsReqChstMask2Lsw); 
  bfield2_lsw = (message->msg_body.stats_req.stat_req_mask2_lsw
                 & dispatchTblStatsReqChst[inst->state]);

  siuGetDispatchTblStatsReqChst(dispatchTblStatsReqChst,
                                siuLoadsiuStatsReqChstMask2Msw,
                                siuStatsReqChstMask2Msw);
  bfield2_msw = (message->msg_body.stats_req.stat_req_mask2_msw
                 & dispatchTblStatsReqChst[inst->state] 
                 & (~MGB_STATS_GLOBAL_MASK));

  if(gg_cmpnd_ctable.msgToMicroBegin) {
    /* Calculate number of simple stats requests */
    /* First bitfield */
    for (bfield = bfield_lsw; bfield; bfield >>= 1)
    {
      bit_cnt += (bfield & 0x0001);
    }     
    /* Second bitfield */  
    for (bfield = bfield_msw; bfield; bfield >>= 1)
    {
      bit_cnt += (bfield & 0x0001);
    }    
    /* Second 32bit word - lower bitfield */
    for (bfield = bfield2_lsw; bfield; bfield >>= 1)
    {
      bit_cnt += (bfield & 0x0001);
    }  
    /* Second 32bit word - upper bitfield */
    for (bfield = bfield2_msw; bfield; bfield >>= 1)
    {
      bit_cnt += (bfield & 0x0001);
    }  

    /* Low-priority messages are collected into a single compound message,
    * beginning from here ... If number of simple messages is greater 
    * then one.
    */  
    if(bit_cnt > 1) {  
      gg_cmpnd_ctable.msgToMicroBegin(priority, inst->ID & 0x00ff, trans_id);
    }
  }  /* end of if compound msg support exist */


  siuGetDispatchTblarrayGetStats(dispatchTblGetStats,
                                 siuLoadarrayGetStatsLSW,arrayGetStatsLSW);
  siu_stats_request_single_word(bfield_lsw, dispatchTblGetStats, inst, reset,
                                trans_id, priority);
  siuGetDispatchTblarrayGetStats(dispatchTblGetStats,
                                 siuLoadarrayGetStatsMSW,arrayGetStatsMSW);
  siu_stats_request_single_word(bfield_msw, dispatchTblGetStats, inst, reset,
                                trans_id, priority);
  siuGetDispatchTblarrayGetStats(dispatchTblGetStats,
                                 siuLoadarrayGetStats2LSW,arrayGetStats2LSW);
  siu_stats_request_single_word(bfield2_lsw, dispatchTblGetStats, inst, reset,
                                trans_id, priority);
  siuGetDispatchTblarrayGetStats(dispatchTblGetStats,
                                 siuLoadarrayGetStats2MSW,arrayGetStats2MSW);
  siu_stats_request_single_word(bfield2_msw, dispatchTblGetStats, inst, reset,
                                trans_id, priority);

 /* ... to here. All simple low-priority messages in between are grouped
  * into one or more compound messages. Next function is a kind of flush
  * function (If number of simple messages is greater then one).
  */  
  if(bit_cnt > 1) {
    /* bit_cnt != 0, only if real compound message is assembled */
    gg_cmpnd_ctable.msgToMicroEnd(priority);
  }
}
/******************************************************************************
 * FUNCTION PURPOSE: Called when some statistic request is not valid.
 ******************************************************************************
 * DESCRIPTION: 
 ******************************************************************************/
void siu_get_stats_unavail (void *siuInst, bool reset, tuint trans_id, tuint priority) {
  siuInst_t *inst = (siuInst_t *)siuInst;
  siuErrorIndication (inst->ID, trans_id, MGB_ERR_INVLD_STAT_REQUEST);
}

/******************************************************************************
 * FUNCTION PURPOSE: Support for multiple state/statistics reset request from
 *                   the host
 ******************************************************************************
 * DESCRIPTION: Bit field is provided on the input, and array of function 
 *              pointers. Each function has same api, and it is invoked if
 *              related bit in the bit-field is set.
 * void siu_channel_resetRqst_single_word (
 *   tuint stat_reset_mask,               - bit-field indicating stat reset requests
 *   siuFcnResetStat_t *array_reset_stat, - pointer to an array of call-out functions
 *   void *siuInst,                       - channel instance
 *   tuint trans_id                       - transaction ID of this request
 * )
 *****************************************************************************/
static void siu_channel_resetRqst_single_word ( tuint ch_reset_mask, 
                                                const siuFcnResetCh_t *array_reset_ch, 
                                                void *siuInst, tuint trans_id)
{
  tuint i;
  siuFcnResetCh_t reset_ch;
  siuInst_t *inst = (siuInst_t *)siuInst;
  for (i = 0; ch_reset_mask; i ++)
  {
    if(ch_reset_mask & 0x0001) {
      reset_ch = array_reset_ch[i];
      if(reset_ch)
        (*reset_ch) (inst, trans_id);
      else
        break; /* delimiter encountered */
    }
    ch_reset_mask >>= 1;
  }
} /* siu_channel_resetRqst_single_word */

/******************************************************************************
 * FUNCTION PURPOSE: Server function for multiple channel state/stats reset
 ******************************************************************************
 * DESCRIPTION: Channel State reset bitfield is divided into 2 single word
 *              bitfields. Same reasoning applies for Statistics reset as well.
 *              If a reset request is not satisfied, it is ignored.
 ******************************************************************************/
void siuChannelResetRqst (siuInst_t *inst, mgb_t *message) 
{
  tuint trans_id = message->header.trans_id;
  tuint bfield_lsw, bfield_msw, bfield2_lsw, bfield2_msw;
#ifdef SIUTBLPRG
  union {
  tuint           PtrStatsReqChst[SIU_MAX_NUM_CHST];
  siuFcnResetCh_t PtrarrayResetChState[SIU_MAX_NUM_CHST];
  fcnGetStats_t   PtrarrayGetStats[SIU_MAX_NUM_CHST];
  } scratch_u;
#endif
  
  /* Process State Reset Requests */
  siuGetDispatchTblarrayResetChState(dispatchTblResetChState,
                                     siuLoadsiuArrayResetChStateLSW,
                                     siuArrayResetChStateLSW);
  siu_channel_resetRqst_single_word(
      message->msg_body.channel_rst_req.state_rst_mask_lsw,
      dispatchTblResetChState, inst, trans_id);
  siuGetDispatchTblarrayResetChState(dispatchTblResetChState,
                                     siuLoadsiuArrayResetChStateMSW,
                                     siuArrayResetChStateMSW);
  siu_channel_resetRqst_single_word(
      message->msg_body.channel_rst_req.state_rst_mask_msw,
      dispatchTblResetChState, inst, trans_id);

  /* Process Statistics Reset Requests */
  siuGetDispatchTblStatsReqChst(dispatchTblStatsReqChst,
                                siuLoadsiuStatsReqChstMaskLsw,
                                siuStatsReqChstMaskLsw);
  bfield_lsw = (message->msg_body.channel_rst_req.stats_rst_mask_lsw
                & dispatchTblStatsReqChst[inst->state]);
  siuGetDispatchTblStatsReqChst(dispatchTblStatsReqChst,
                                siuLoadsiuStatsReqChstMaskMsw,
                                siuStatsReqChstMaskMsw);
  bfield_msw = (message->msg_body.channel_rst_req.stats_rst_mask_msw
                & dispatchTblStatsReqChst[inst->state]);

  siuGetDispatchTblStatsReqChst(dispatchTblStatsReqChst,
                                siuLoadsiuStatsReqChstMask2Lsw,
                                siuStatsReqChstMask2Lsw);
  bfield2_lsw = (message->msg_body.channel_rst_req.stats_rst_mask2_lsw
                 & dispatchTblStatsReqChst[inst->state]);

  siuGetDispatchTblStatsReqChst(dispatchTblStatsReqChst,
                                siuLoadsiuStatsReqChstMask2Msw,
                                siuStatsReqChstMask2Msw);
  bfield2_msw = (message->msg_body.channel_rst_req.stats_rst_mask2_msw
                 & dispatchTblStatsReqChst[inst->state]);

  /* Process Get Stats Request */
  siuGetDispatchTblarrayGetStats(dispatchTblGetStats,
                                 siuLoadarrayGetStatsLSW,arrayGetStatsLSW);
  siu_stats_request_single_word(bfield_lsw, dispatchTblGetStats, inst, TRUE,
                                trans_id, SIU_DPR_IGNORE_MSG);
  siuGetDispatchTblarrayGetStats(dispatchTblGetStats,
                                 siuLoadarrayGetStatsMSW,arrayGetStatsMSW);
  siu_stats_request_single_word(bfield_msw, dispatchTblGetStats, inst, TRUE,
                                trans_id, SIU_DPR_IGNORE_MSG);
  siuGetDispatchTblarrayGetStats(dispatchTblGetStats,
                                 siuLoadarrayGetStats2LSW,arrayGetStats2LSW);
  siu_stats_request_single_word(bfield2_lsw, dispatchTblGetStats, inst, TRUE,
                                trans_id, SIU_DPR_IGNORE_MSG);
  siuGetDispatchTblarrayGetStats(dispatchTblGetStats,
                                 siuLoadarrayGetStats2MSW,arrayGetStats2MSW);
  siu_stats_request_single_word(bfield2_msw, dispatchTblGetStats, inst, TRUE,
                                trans_id, SIU_DPR_IGNORE_MSG);
} /* siuChannelResetRqst */

/******************************************************************************
 * FUNCTION PURPOSE: Called when some state/stats reset request is not valid.
 ******************************************************************************
 * DESCRIPTION: 
 ******************************************************************************/
void siu_reset_channel_unavail (void *siuInst, tuint trans_id) 
{
  siuInst_t *inst = (siuInst_t *)siuInst;
  siuErrorIndication (inst->ID, trans_id, MGB_ERR_INVLD_CHANNEL_RESET_RQST);
} /* siu_rest_channel_unavail */

/******************************************************************************
 * FUNCTION PURPOSE: Message Processing Wrapper
 ******************************************************************************
 * DESCRIPTION:  Handles message acknowledgments and routes interrupt/normal
 *               priority messages.
 *****************************************************************************/
void siuProcessMessage (NDU_msgPriority_e priority, mgb_t *message)
{
  siuMsgAck_t *msgAck; 
  void (*processFcn) (mgb_t *message, siuMsgAck_t *msgAck);
  tuint chnum, transId;
  siuDPRMsgType_t outPriority;

  /* Choose correct ack type and processing function */
  switch (priority) {
    case NDU_NORMAL_MESSAGE:
      outPriority = SIU_DPR_NORMAL_MSG;
      msgAck     = &siuNormalMsgAck;
      processFcn = siuProcessNormalMessage;
      break;
    case NDU_LOW_MESSAGE:
      outPriority = SIU_DPR_LOW_MSG;
      msgAck     = &siuNormalMsgAck;
      processFcn = siuProcessNormalMessage;
      break;
    case NDU_INTERRUPT_MESSAGE:
      outPriority = SIU_DPR_SIGNAL_MSG;
      msgAck     = &siuInterruptMsgAck;
      processFcn = siuProcessInterruptMessage;
      break;
    default:
      siuErrorIndication (0, message->header.trans_id, 
                          MGB_ERR_INVD_MESSAGE_CONFIG);
      return;
  }

  /* Store message id for error indications */
  msgAck->msgId = message->header.msg_id;

  /* Store the message header for ack including untranslated channum */
  chnum   = message->header.chnum;
  transId = message->header.trans_id;
  if (message->header.msg_id == MGB_MD_STATS_REQ) {
    /* keep the reset configuration */
    message->msg_body.stats_req.stat_req_reset &= 0x0001; 
    /* use higher 8 bits for priority*/
    message->msg_body.stats_req.stat_req_reset |= (outPriority << 8); 
  }
  if (((transId & MGB_TRANS_ACK_BIT)               || 
      (msgAck->msgId == MGB_MD_DSIG_OPEN_CHANNEL) ||
      (msgAck->msgId == MGB_MD_DSIG_CLOSE_CHANNEL)) &&
      (msgAck->msgId != MGB_MD_RTCP_SEND_PACKET) ) {
    /* Assume an ack is needed if the ack bit is set */
    /* DSIG OPEN/CLOSE always get ack.  Regular open/close also always get
     * ack, but use commit logic for state transition.  DSIG does not use
     * commit.
     *   Ack for RTCP_SEND_PACKET is sent from message processing function
     * internally.
     */
                                                          /* ACK mesage type */
    if ((msgAck->msgId == MGB_MD_TONE_ON)              || /* MGB_DM_DIGITSIG_TONE_GENERATION_DONE */
        (msgAck->msgId == MGB_MD_DIGITSIG_RQST)        || /* MGB_DM_DIGITSIG_TONE_GENERATION_DONE */
        (msgAck->msgId == MGB_MD_SEND_PACKET)          || /* GENERAL_ACK */
        (msgAck->msgId == MGB_MD_CALLER_ID_CTRL)       || /* MGB_DMS_CALLER_ID_DETECT !! */
        (msgAck->msgId == MGB_MD_DSIG_CAS_CADENCE_GEN) || /* MGB_DM_DSIG_CADENCE_DONE */
        (msgAck->msgId == MGB_MD_ANNOUNCE_BEGIN)       || /* MGB_DM_ANNOUNCE_COMPLETE */
        (msgAck->msgId == MGB_MD_AUTO_COT_RQST)        || /* MGB_DM_AUTO_COT_RESP     */         
        (msgAck->msgId == MGB_MD_ENCODE_CONFIG)           /* No special ack message for this one
                                                             ack is given after delayed message
                                                             is executed in siucsu.c:siuTxSwapEncoder */
      /*|| (msgAck->msgId == MGB_MD_AIC_CONFIG)*/) {
      /* Assume an ack is only a "PROC" if delayed message */
      msgAck->ackType = MGB_ACK_TYPE_PROC;
    } else {
      msgAck->ackType = MGB_ACK_TYPE_PROC | MGB_ACK_TYPE_DONE;
    }
  } else {
    msgAck->ackType = 0;
  }
  /* Process the message */
  processFcn (message, msgAck);

  if (msgAck->ackType) {
    /* Notify micro of state change */
    siuGeneralAck (msgAck->msgId, chnum, transId, msgAck->ackType, 0, outPriority);
  }
  /* Indicate to error indications that we exit context of message processing */
  msgAck->msgId = MGB_ERR_MSG_ID_UNDEF;

#ifdef GMP
  if (siuContext.gmpFormatState == SIU_GMPFORMAT_DOFORMAT) {
    /* The init code is done.  Format the rest of GMP (which will destroy
     * the init code if its overlayed) */
    siugmpInit(ggConfig.gmcSize, gg_gmp_no_segments, (void *)gg_gmp_segments, FALSE);
    siuContext.gmpFormatState = SIU_GMPFORMAT_FORMATTED;
  }
#endif
} /* siuProcessMessage */


#if defined(RELMSG_SUPPORT)

#define RELMSG_DEBUG        0
#if RELMSG_DEBUG
tuint siu_dbg_msg_drop_thresh = 7;
tuint siu_dbg_msg_cnt = 0;
tuint siu_dbg_ack_drop_thresh = 10;
tuint siu_dbg_ack_cnt = 0;
#endif

/*****************************************************************************
 * FUNCTION PURPOSE: Message Ack processing
 *****************************************************************************
 * DESCRIPTION: Send GENERAL_ACK for each valid message recieved.
 *              Drop the message if necessary by returning TRUE
 *              
 *****************************************************************************/
bool siuProcMsgAck(NDU_msgPriority_e priority, mgb_t *message)
{
  mgb_t send_message;
  mgbHeader_t* p_hdr = &message->header;
  tuint*       p_body = (tuint*)&message->msg_body;
  tuint seq_num, seq_num_expected;
  bool  drop = FALSE;
  bool  ack = FALSE;
  bool  high_prio = (priority == NDU_INTERRUPT_MESSAGE);
  

  /* The message ack processing should be done only if it is enabled and
   * the download is complete
   */
  if(!siuContext.msg_burst_size || !siuContext.hw_config_received)
    return(FALSE);
    
#if RELMSG_DEBUG
  /* message drop */
  if (siu_dbg_msg_drop_thresh)
  {
    if (++siu_dbg_msg_cnt > siu_dbg_msg_drop_thresh)
    {
        siu_dbg_msg_cnt = 0;
        return(TRUE);
    } 
  }  
#endif    
    
  /* Verify the message sequence number */
  seq_num_expected = high_prio? SIU_GET_MSG_SEQ_NUM_HI():
                                SIU_GET_MSG_SEQ_NUM();
  seq_num = p_body[p_hdr->length - MGB_HEADER_SIZE - 1]; 
  
  if (seq_num_expected == seq_num) {
     /* Expected seq num matches, ack and process the message, update the expected seq num */
     ack = TRUE;
     if(++seq_num_expected > SIU_MAX_MSG_SEQ_NUM)
       seq_num_expected = 0;
       
     if(high_prio)
       SIU_SET_MSG_SEQ_NUM_HI(seq_num_expected);
     else
       SIU_SET_MSG_SEQ_NUM(seq_num_expected);
     
     /* restore the currect message length */
     p_hdr->length--;  
       
  }
  else if (((seq_num_expected > seq_num) && ((seq_num_expected - seq_num) <= siuContext.msg_burst_size)) ||    
           ((seq_num_expected < seq_num) && ((SIU_MAX_MSG_SEQ_NUM - seq_num + seq_num_expected) < siuContext.msg_burst_size)))
  {
     /* seq_num is in the valid range, the previous ack was lost
      * drop, but ack the message */
     ack = TRUE;
     drop = TRUE;  
     
  }
  else {
    /* message with invalid ack is received, send Error Message and drop the message */
    drop = TRUE; 
  }
  
#if RELMSG_DEBUG
  /* ack drop */
  if (ack && siu_dbg_ack_drop_thresh)
  {
    if (++siu_dbg_ack_cnt > siu_dbg_ack_drop_thresh)
    {
        siu_dbg_ack_cnt = 0;
        return(drop);
    } 
  }  
#endif   
  
  /* Prepare and send the message ack */      
  if (ack) {
    send_message.header.length   = MGB_HEADER_SIZE + 5;
    /* Channel number indicates the which channel underwent state change */
    send_message.header.chnum    = 0;
    send_message.header.msg_id   = MGB_DM_GENERAL_ACK;
    send_message.header.trans_id = p_hdr->trans_id; 

    send_message.msg_body.general_ack.ack_type   = MGB_ACK_TYPE_ACK;
    send_message.msg_body.general_ack.message_id = p_hdr->msg_id;
    send_message.msg_body.general_ack.gen_param  = 0;
    send_message.msg_body.general_ack.seq_num = seq_num;
    send_message.msg_body.general_ack.priority = high_prio?MGB_MSG_PRIORITY_HIGH:
                                                           MGB_MSG_PRIORITY_NORMAL;
    /* send message to FIFO */  
    siuNduSendMsg(&send_message);
  }
  return (drop);
}

#endif /* RELMSG_SUPPORT */

/* nothing past this point */
