/**************************************************************************************
 * FILE PURPOSE: Create and send keep alive cells to the host
 **************************************************************************************
 * FILE NAME: siualive.c
 *
 * DESCRIPTION: Creates keep alive cells to be sent to the hardware 
 *
 * TABS: None
 *
 * Copyright 2002, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include "types.h"
#include "siuloc.h"
#include "siualive.h"
#include "mgmicdsp.h"
#include "hal.h"
#include <string.h>

/**************************************************************************************
 * Global Declarations of the keep alive call table and context
 **************************************************************************************/
siuHealthCtbl_t siuHealthCtbl = {
  siuHealthMain,
  siuHealthProc,
  siuHealthInit
};

siuHealthContext_t siuHealthContext;
/*******************************************************************************
 * FUNCTION PURPOSE: Trap function for illegal interrupts
 *******************************************************************************
 * DESCRIPTION: Infinite loop for illegal interrupts.  Saves IER registers
 *              for debugging.  MUST BE MADE PORTABLE FOR MERGING OUT
 *              OF GGXHD_R100100
 ******************************************************************************/
void invld_interrupt (void)
{
  for (;;);
}

/***********************************************************************************
 * FUNCTION PURPOSE: A trap function for the watchdog timer
 ***********************************************************************************
 * DESCRIPTION: This function implements an endless loop. If the watchdog expires
 *              the soft reset will cause the DSP to branch to this function. This
 *              has no effects on hard reset or downloads since the start address
 *              is provided through other means.
 ***********************************************************************************/
void siu_alive_timeout (void)
{
  for (;;);
} /* siu_alive_timeout */

/************************************************************************************
 * FUNCTION PURPOSE: Initialize the health check context.
 ************************************************************************************
 * DESCRIPTION: Called from hardware config to initialize the health check feature.
 *              The function checks for a valid configuration.
 ************************************************************************************/
tint siuHealthInit (tulong curTime, tuint config, tuint coreFreq)
{
  tuint  per_ms;     /* Keep alive period, ms      */
  tuint  options;    /* Keep alive options         */

  halWdTimerControl_t wdControl;  /* Watchdog timer control */

  per_ms  = SIU_HEALTH_EXTRACT_PERIOD (config);
  options = SIU_HEALTH_EXTRACT_OPTIONS(config);

  /* Check the validity of the options. For now only option 0 supported. */
  if ((options != SIU_HEALTH_MODE_CELL) && (options != SIU_HEALTH_MODE_WATCHDOG))
    return (SIU_ERROR);
  siuHealthContext.mode = options;

  /* Check the validity of the period */
  if (per_ms == SIU_HEALTH_DISABLE_PERIOD)  {
    siuHealthContext.period = 0;
    return (SIU_NOERR);
  }

  if (per_ms < SIU_HEALTH_MIN_PERIOD)
    return (SIU_ERROR);

  /* Initialize the context */
  siuHealthContext.lastSendTime = curTime;
  siuHealthContext.period       = per_ms;
  #if 1
  /* Disable Task check until we can determine when and how it should be performed in all combinations of 
   * operating modes such as announcement and Nx64 mode */
  siuHealthContext.eventBitMap = 0;
  #else
  siuHealthContext.eventBitMap  = SIU_HEALTH_TXRX_TASK | SIU_HEALTH_TX_TASK;
  #endif
  memset (&siuHealthContext.stats, 0, sizeof(siuHealthStats_t));

  /* Place initial values in the stack. Stack modifications are clearly processor
   * dependent, so this function lives in the processor specific directories */
  siuHealthStackInit();

  /* For watch dog timer mode the timer must be initialized. The timeout
   * period is set to be two an a half times the specified refresh period */
  if (siuHealthContext.mode == SIU_HEALTH_MODE_WATCHDOG)  {

    /* The watchdog is only valid if a non-zero sample frequency is specified */
    if (!coreFreq)
      return (SIU_ERROR);

    /* Convert the time in ms into timer ticks, multiply by the slip factor (2.5)
     * to get the timeout timer value. */
    wdControl.periodCount = (tulong)per_ms * coreFreq * 1000;
    wdControl.periodCount = SIU_HEALTH_WD_SLIP(wdControl.periodCount);
    wdControl.operation   = HAL_WDTIMER_ENABLE;
    if ((*siuContext.halApi->halWdTimerControl) (&wdControl) != HAL_SUCCESS)
      return (SIU_ERROR); 
  }

  return (SIU_NOERR);
} /* siuHealthInit */
  

/*************************************************************************************
 * FUNCTION PURPOSE: Health Check task check
 *************************************************************************************
 * DESCRIPTION: Checks if the task is relevent to health check keep alives, and 
 *              updates the keep alive status.
 *************************************************************************************/
void siuHealthProc (tulong curTime, tuint queue_id)
{
  /* Return if health check keep alives are disabled, or the event mask is already clear */
  if ((!siuHealthContext.period) || (!siuHealthContext.eventBitMap)) return;
} /* siuHealthProc */

/**************************************************************************************
 * FUNCTION PURPOSE: Feed the watchdog timer
 **************************************************************************************
 * DESCRIPTION: Feeds the watchdog keeping it from timing out
 **************************************************************************************/
void siu_feed_watchdog (void)
{
  halWdTimerControl_t wdControl;

  wdControl.operation = HAL_WDTIMER_FEED;
  (*siuContext.halApi->halWdTimerControl)(&wdControl);
} /* siu_feed_watchdog */
  

/**************************************************************************************
 * FUNCTION PURPOSE: Health Check send packet check
 **************************************************************************************
 * DESCRIPTION: This function is called to check if it is time to send a health
 *              check message. If the health check packet cannot be sent at the
 *              expected time, then statistics are updated to indicate why.
 **************************************************************************************/
void siuHealthMain (tulong curTime, tuint state)
{
  tulong deltaTime;

  /* Return if health check is disabled */
  if (!siuHealthContext.period)
    return;

  /* Compute the required send time, compare it to current time */
  deltaTime = curTime - siuHealthContext.lastSendTime;
  if (deltaTime >= siuHealthContext.period)  {

    /* Perform the stack check, returns non-zero on error. This is a processor
     * specific function. Inc the stat on failure, but there is never a recovery */
    if (siuHealthStackCheck()) {
      siuHealthContext.stats.stackFail += 1; 
      siuHealthContext.stats.sendCount += siu_send_error_message (MGB_ERR_INVD_STACK);
      return;  /* stack check failed */
    }

    /* Check the processor status bits to be sure the C state is in effect. This
     * is also processor specific. */

    if (siuHealthProcStat())  {
      siuHealthContext.stats.stxFail += 1;
      return;  /* process status check failed */
    }

    /* In the ready state tasks are not checked */
    if ((state == SIU_ST_READY) || (!siuHealthContext.eventBitMap) || 
        (siuHealthContext.stateCheck))  {
      if (siuHealthContext.mode == SIU_HEALTH_MODE_WATCHDOG)
        siu_feed_watchdog ();
      else
        siuHealthContext.stats.sendCount += siu_send_keep_alive ();
      #if 0
      /* Disable Task check until we can determine when and how it should be performed in all combinations of 
       * operating modes such as announcement and Nx64 mode */
      siuHealthContext.eventBitMap      = SIU_HEALTH_TXRX_TASK | SIU_HEALTH_TX_TASK;
      #endif
      siuHealthContext.lastSendTime     = curTime;

    } else {

      /* The keep alive criteria failed. Record why. */
      if (siuHealthContext.eventBitMap & SIU_HEALTH_TXRX_TASK) 
      {
        siuHealthContext.stats.txrxFail += 1;
        siuHealthContext.stats.sendCount += siu_send_error_message (MGB_ERR_TXRX_TASK_FAIL);        
      }  
      if (siuHealthContext.eventBitMap & SIU_HEALTH_TX_TASK)
      {
        siuHealthContext.stats.txFail += 1;
        siuHealthContext.stats.sendCount += siu_send_error_message (MGB_ERR_TX_TASK_FAIL);
      }
    } /* end else */


    /* If the current state is IDLE then set the stateCheck to true. By doing this
     * the transition from idle to active state will not require the eventBitMap to
     * be clear on the first active state. This is necessary since a channel could be
     * opened with only 1 ms before the health check cell is due. It will still be up
     * to 5ms before any tasks are processed, and the health packet must not be delayed. */
    if (state == SIU_ST_READY)
      siuHealthContext.stateCheck = SIU_HEALTH_STATE_COUNT; 
    else  { /* state == SIU_ST_WORKING */
      if (siuHealthContext.stateCheck)
        siuHealthContext.stateCheck -= 1;
    }

  } /* (deltaTime >= siuHealthContext.period)  */

} /* siuHealthMain */






