#ifndef SIUALIVE_H
#define SIUALIVE_H
/***************************************************************************************
 * FILE PURPOSE: Definitions required for generating keep alive cells to hardware
 ***************************************************************************************
 * FILE NAME: siualive.h
 *
 * DESCRIPTION: Definitions and macros used to generate keep alive cells
 *
 * TABS: None
 *
 * Copyright (C) 2002, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include "types.h"
#include "siuport.h"

/************************************************************************************
 * Definition: System call table used to interface to the keep alive generator 
 ************************************************************************************/
typedef struct {
  void (*siuHealthMain)(tulong, tuint);  
  void (*siuHealthProc)(tulong, tuint);
  tint (*siuHealthInit)(tulong, tuint, tuint);
} siuHealthCtbl_t;

/*************************************************************************************
 * Definition: Keep alive context
 *************************************************************************************/
typedef struct {
  tuint  sendCount;     /* Number of keep alives sent                 */
  tuint  txrxFail;      /* Number of failures due to no txrx tasks    */
  tuint  txFail;        /* Number of failures due to no tx tasks      */
  tuint  stackFail;     /* Number of failures due to stack check      */
  tuint  stxFail;       /* Number of failures due to status registers */
} siuHealthStats_t;

typedef struct {
  tulong lastSendTime;     /* When last keep alive cell was sent       */
  tuint  mode;             /* Keep alive mode, cell or watchdog        */
  tuint  period;           /* Rate at which keep alives are generated  */
  tuint  eventBitMap;      /* Event bit map - cleared as events arrive */
  tuint  stateCheck;       /* Idle to active state change tracking     */
  siuHealthStats_t stats;  /* Module info                              */
} siuHealthContext_t;
#define SIU_HEALTH_TXRX_TASK    (1<<0)
#define SIU_HEALTH_TX_TASK      (1<<1)

/************************************************************************************
 * Definition: Macros for extracting the fields in the keep alive configuration field
 ************************************************************************************/
#define SIU_HEALTH_PERIOD_MASK  0x03FF
#define SIU_HEALTH_PERIOD_SHIFT 0
#define SIU_HEALTH_EXTRACT_PERIOD(x) \
        (((x) & SIU_HEALTH_PERIOD_MASK) >> SIU_HEALTH_PERIOD_SHIFT)

#define SIU_HEALTH_OPTIONS_MASK 0xFC00
#define SIU_HEALTH_OPTIONS_SHIFT 10
#define SIU_HEALTH_EXTRACT_OPTIONS(x) \
        (((x) & SIU_HEALTH_OPTIONS_MASK) >> SIU_HEALTH_OPTIONS_SHIFT)

/*************************************************************************************
 * Definition: Keep alive options
 *************************************************************************************/
#define SIU_HEALTH_MODE_CELL       0   /* Cell type keep alives */
#define SIU_HEALTH_MODE_WATCHDOG   1   /* Watch dog mode        */

/*************************************************************************************
 * Definition: Boundries for input keep alive timeout periods
 *************************************************************************************/
#define SIU_HEALTH_DISABLE_PERIOD   0
#define SIU_HEALTH_MIN_PERIOD       6

/*************************************************************************************
 * Definition: Stack check depth and check value
 *************************************************************************************/
#define SIU_HEALTH_STACK_CHECK_DEPTH    5
#define SIU_HEALTH_STACK_CHECK_VALUE    0xCEC1

/*************************************************************************************
 * Definition: Watch dog timer slip compution. The watchdog will be refreshed
 *             at a given rate. The watchdog itself will be configured to timemout
 *             at a somewhat greater rate. The macro computes this rate, which
 *             is set to a factor of 2.5.
 *************************************************************************************/
#define SIU_HEALTH_WD_SLIP(x)   (((x)*5)>>1)

/*************************************************************************************
 * Definition: On transition from system idle to system active state (no channels
 *             open to at least one channel open) there must be some foregiveness
 *             in not finding tasks. If a channel is opened 1ms before the send
 *             time, the keep alive will not be sent since now there is an active
 *             channel and txrx and tx tasks must be posted. These could be delayed
 *             up to 9.5ms due to sub-frame staggering and the initial frame fill.
 *************************************************************************************/
#define SIU_HEALTH_STATE_COUNT  1


/*************************************************************************************
 * Definition: Structure definition defining the payload used for keep alive data
 *************************************************************************************/
typedef struct {
  tuint seqNum;              /* Sequence number - currently unused */
  tuint chip_core;           /* DSP and core ID values             */
} ka_payload_rt_t;

/* Macros to manipulate ka_payload */
#define SIU_KEEP_ALIVE_CHIP_MASK       0xF800
#define SIU_KEEP_ALIVE_CHIP_SHIFT      11
#define SIU_KEEP_ALIVE_SET_CHIP_ID(x,y) \
  ((x)->chip_core) = (((x)->chip_core) & ~SIU_KEEP_ALIVE_CHIP_MASK)  | \
                     ((y) << SIU_KEEP_ALIVE_CHIP_SHIFT)
                     
#define SIU_KEEP_ALIVE_CORE_MASK       0x01C0
#define SIU_KEEP_ALIVE_CORE_SHIFT      6
#define SIU_KEEP_ALIVE_SET_CORE_ID(x,y)  \
  ((x)->chip_core) = (((x)->chip_core) & ~SIU_KEEP_ALIVE_CORE_MASK) | \
                     ((y) << SIU_KEEP_ALIVE_CORE_SHIFT)

#define SIU_KEEP_ALIVE_PAYLOAD_SIZE_BYTES  UTL_WORDS_TO_BYTES(sizeof(ka_payload_rt_t))


/*************************************************************************************
 * Definition: Definition of keep alive payload for mii cells
 *************************************************************************************/
typedef struct {
  tuint transId;          /* Transaction ID                    */
  tuint opCode_count;     /* Opcode and count field            */
  tuint addrMsw;          /* Most significant word of DSP ID   */
  tuint addrLsw;          /* Least significant word of DSP ID  */
  tuint valMsw;           /* Most significant word of core ID  */
  tuint valLsw;           /* Least significant word of core ID */
} ka_payload_vc_t;

/* There is only one valid value for the opCode_count field */
#define SIU_KEEP_ALIVE_VC_OP_COUNT  0x5001

#define SIU_KEEP_ALIVE_SET_ADDR(x,y) \
        (x)->addrMsw = 0; (x)->addrLsw = y
        
#define SIU_KEEP_ALIVE_SET_VAL(x,y) \
        (x)->valMsw = 0; (x)->valLsw = y


/* Function prototypes */
tint siuHealthInit (tulong curTime, tuint config, tuint sampleFreq);
void siuHealthProc (tulong curTime, tuint queue_id);
void siuHealthMain (tulong curTime, tuint state);

tint siuHealthProcStat (void);
void siuHealthStackInit (void);
tint siuHealthStackCheck (void);
tint siu_send_keep_alive (void);
tint siu_send_error_message (tuint errorCode);

extern siuHealthCtbl_t siuHealthCtbl;

#endif /* SIUALIVE_H */
