/******************************************************************************
 * FILE PURPOSE: Library functions for handling compound messages
 *****************************************************************************
 * FILE NAME: siucmpnd.c
 *
 * DESCRIPTION: Contains the routines for receiving and sending compound
 *              messages. This feature can be removed from the second stage.
 *
 * TABS: NONE
 *
 * (C) Copyright 2000, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include "types.h"
#include "mgmicdsp.h"
#include "siuloc.h"
#include "ggmbuf.h"

#include "siucmpnd.h"

/* Type of compound message in progress (low, normal or high priority) */
static NDU_DPRMsgType_t compound_mtype = NDU_DPR_NONE_MSG;
/* Channel number of compound message in progress.
 * Zero indicates, no compound message is in progress */
tuint compound_channel = 0;
/* Transaction ID of a compound message */
tuint compound_trans_id = 0;
/* Counter of simple messages in single compound message */
/* used for stats update. */
tuint compound_simple_messages = 0;
static bool  compound_write_result    = FALSE;
dprword *compound_msg_ptr;
tuint compound_msg_space;
/******************************************************************************
 * FUNCTION PURPOSE: Parse and process compound messages 
 ******************************************************************************
 * DESCRIPTION: Compound messages are parsed into single messages and
 *              processed sequentially
 *****************************************************************************/
static void siu_process_compound_message (void *vdp, NDU_msgPriority_e priority)
{
  mgb_t   singleMsg;   /* constructed single message                */
  tuint  *dp = (tuint *)vdp; 
  tuint   i, submsglen;
  bool    bad_msg = FALSE;

  i = MGB_HEADER_SIZE;           /* 1st element past common header */

  /* Sanitize the compound message */
  while (i < ((mgb_t *)vdp)->header.length)  {
    submsglen = dp[i + MGB_COMPOUND_MSG_LENGTH_OFFSET];
    if ((submsglen < MGB_COMPOUND_HDR_SIZE) ||
        (submsglen + MGB_HEADER_SIZE - MGB_COMPOUND_HDR_SIZE) > 
         sizeof(singleMsg)) {
      /* Single message doesn't fit in single message buffer; would overflow */
      bad_msg = TRUE;
      break;
    }
    i += submsglen;
  }

  if (i > ((mgb_t *)vdp)->header.length) {
    /* Compound message wasn't integral number of single messages; corrupt */
    bad_msg = TRUE;
  }

  if (bad_msg) {
//    siuContext.nduStats.RxCtrlMsgDrop++;
    siuErrorIndication(siuContext.ID, 0, MGB_ERR_CORRUPT_COMPOUND_MSG);
    return;
  }

  i = MGB_HEADER_SIZE;           /* 1st element past common header */
  while (i < ((mgb_t *)vdp)->header.length)  {
    /* Construct the single message header from the compound message header
     * and the sub-message header */
    singleMsg.header.echo     = 0;
    singleMsg.header.length   = dp[i+MGB_COMPOUND_MSG_LENGTH_OFFSET] + 
                                MGB_HEADER_SIZE - MGB_COMPOUND_HDR_SIZE;
    singleMsg.header.chnum    = ((mgb_t *)vdp)->header.chnum;
    singleMsg.header.msg_id   = dp[i+MGB_COMPOUND_MSG_ID_OFFSET];
    singleMsg.header.trans_id = dp[i+MGB_COMPOUND_MSG_TRANS_ID_OFFSET];

    /* Copy the message body */
    memcpy (&singleMsg.msg_body, (void *)&dp[i+MGB_COMPOUND_MSG_BODY_OFFSET],
             (singleMsg.header.length - MGB_HEADER_SIZE) * sizeof(dprword));

    /* process the message */
    siuProcessMessage (priority, &singleMsg);

    /* Set index to point to start of next message */
    i = i + dp[i + MGB_COMPOUND_MSG_LENGTH_OFFSET];
  
  } /* end of while */
} /* siuProcessCompoundMessage */
/*****************************************************************************/
void siuCmpndMsgFromMicro(NDU_msgPriority_e priority, void *buf) {
    /* Pass off simple messages directly and return. */
  if (((mgb_t *)buf)->header.msg_id != MGB_MD_COMPOUND_MSG)  {
    /* process message */
    siuProcessMessage (priority, buf);
    return;
  }
  /* compound messages are handled by breaking them into multiple single
   * messages and processing them sequentially */
  siu_process_compound_message ((void *)buf, priority);
}

/******************************************************************************
 *              Support for compound messages in DSP->HOST direction          *
 ******************************************************************************/
static void siu_compound_write_to_dpr (void) {
mgb_t *msg = (mgb_t *)siuContext.siu_compound_scratch;

  if(compound_msg_space == 0)
    return; /* no payload collected */

  if(compound_simple_messages == 1) {
    /* No need to create compound message for a single simple message * 
     * Therefore, simple message header has to be recreated.          */
     tuint simple_msg_id, simple_length;
     tuint *msg_ptr = 
       (tuint *)((dprword *)siuContext.siu_compound_scratch + MGB_HEADER_SIZE);
       
     simple_msg_id   = *msg_ptr ++;
                       *msg_ptr ++; /* trans_id is same as compound_trans_id */
     simple_length   = *msg_ptr;
     msg             = (mgb_t *)(&((tuint *)msg)[MGB_COMPOUND_HDR_SIZE]);
     msg->header.length = simple_length - MGB_COMPOUND_HDR_SIZE + MGB_HEADER_SIZE;
     msg->header.chnum  = compound_channel;
     msg->header.msg_id = simple_msg_id;
     msg->header.trans_id = compound_trans_id;
    siuNduSendSimpleMsgPriority(msg, (NDU_DPRMsgType_t)compound_mtype);
  }
  else {
     /* True compound message */
     msg->header.length   = compound_msg_space + MGB_HEADER_SIZE;
     msg->header.chnum    = compound_channel;
     msg->header.msg_id   = MGB_DM_COMPOUND_MSG;
     msg->header.trans_id = compound_trans_id;  
  /* write compound message into HPI */
  if(siuNduSendMsgPriority (msg,(NDU_DPRMsgType_t)compound_mtype) == 0)
    compound_write_result |= 1;
  }
}

/******************************************************************************/
static void siu_compound_write_init (void) {
    compound_msg_space = compound_simple_messages = 0;  
    compound_msg_ptr   = (dprword *)siuContext.siu_compound_scratch + MGB_HEADER_SIZE;
} 
/******************************************************************************
 * FUNCTION PURPOSE: Collection of a new compound message is started with this
 *                   call.
 ******************************************************************************
 * DESCRIPTION: Current implementation has limitation of collecting one
 * compound message at a time. All simple messages stored in a compound message
 * should be of a same type. If stream of simple messages of same type is 
 * interrupted by simple message of different type, it is passed as simple one
 * (compound transport mechanism is not used).
 * Additional constraint applies: collection of a compound message on only one
 * channel is supported.
 * If message from different channel arrives, it will be sent as simple message.
 * Free state (available) of this collection mechanism is indicated with
 * compound_channel variable being set to 0.
 * void siuHdCompoundMsgToMicroBegin (
 *   tuint mtype,   - message type (normal, high or low)
 *   tuint channel, - channel (1-based)
 *   tuint trans_id - transaction ID that initiated collection of this compound
 * )                  message.
 ******************************************************************************/
void siuCmpndMsgToMicroBegin (tuint mtype, tuint channel, tuint trans_id)
{
 /* Generate exception if nesting of compound messages is detected -
  * This is development-time bug, better to be caught immediatelly !!! */
  siu_exc_assert (compound_channel == 0, SIUHDNDU_EXC_NOT_SUPPORTED, 
                  SIUINSTp(channel));     

  /* From now on, only messages of this type are collected */
  compound_mtype        = (NDU_DPRMsgType_t)mtype;
  compound_trans_id     = trans_id;
  compound_write_result = FALSE;  /* no errors */
  compound_channel      = channel;    

  /* initialize some variables */
  siu_compound_write_init();
}
/******************************************************************************
 * FUNCTION PURPOSE: Add simple message to a compound message in progress.
 ******************************************************************************
 * DESCRIPTION: Simple message is padded to the end of payload of a compound
 *  message. If size of a compound message is exceeded, previous compound
 *  message of a same type is flushed, and collection of new one is initiated.
 ******************************************************************************/
bool siuCmpndMsgToMicroAdd (NDU_DPRMsgType_t mtype, void *vmsg) {
mgb_t *msg = (mgb_t *)vmsg;
tuint payload_size;
tuint overall_message_size;

  if(mtype != compound_mtype)
     return FALSE;
        
  if(compound_channel != msg->header.chnum)
     return FALSE;

  payload_size         = msg->header.length - MGB_HEADER_SIZE;
  overall_message_size = payload_size + MGB_COMPOUND_HDR_SIZE;
  if((compound_msg_space + overall_message_size) > HD_COMPOUND_MSG_MAX_LENGTH) {
    /* No more space in this compound message - flush it, and get prepared for new
     * one !!! */   
    siu_compound_write_to_dpr ();
    
    /* prepare for new one - initialize some vars */
    siu_compound_write_init();
  } 
  *compound_msg_ptr++ = msg->header.msg_id;
  *compound_msg_ptr++ = msg->header.trans_id;
  *compound_msg_ptr++ = overall_message_size;
  /* Copy the message body */
  memcpy (compound_msg_ptr, &msg->msg_body, payload_size * sizeof(dprword));
  compound_msg_ptr   += payload_size;
  compound_msg_space += overall_message_size;
  compound_simple_messages ++;    
  return TRUE;
}
/******************************************************************************
 * FUNCTION PURPOSE: End of compound message collection.
 ******************************************************************************
 * DESCRIPTION: Collection of compound message is ended. It is sent towards
 *  HPI interface, and compound message collection server is set to free state.
 ******************************************************************************/
void siuCmpndMsgToMicroEnd (tuint mtype) {
  if(mtype != compound_mtype)  /* only one compound message at a time */
    return;
    
  siu_compound_write_to_dpr ();
  /* indicate end of compound message collection */
  compound_channel = 0;
  compound_mtype   = NDU_DPR_NONE_MSG;
/* TODO:
  return compound_write_result;
 */  
}
/* nothing past this point */

