/******************************************************************************
 * FILE PURPOSE:  SIU Real-time Debug Tracing Utility
 ******************************************************************************
 * FILE NAME: siudbg.c  
 *
 * DESCRIPTION: Contains all routines for implementing real-time debug tracing
 *              utility for voice/fax software.
 *
 *
 * TABS: NONE
 *
 * $Id: siudbg.c 1.4 1999/09/23 14:27:50 jdowdal Exp $
 *
 * REVISION HISTORY:
 *
 * $Log: siudbg.c $
 * Revision 1.4  1999/09/23 14:27:50  jdowdal
 *    Added a header file
 * Revision 1.3  1999/09/22 15:40:16  jdowdal
 *    Workaround RPTB bugs.
 * Revision 1.2  1998/12/18 08:48:15  BONAL
 *   - Bug in message length check corrected.
 * Revision 1.1  1998/11/25 15:57:07  BONAL
 * Initial revision
 *
 * (C) Copyright 1998, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include <string.h>
#include "types.h"
#include "siuloc.h"
#include "siuport.h"
#include "mgmicdsp.h"
#ifndef HDCT
#include "siutmr.h"
#endif
#include <ti/mas/util/utl.h> /* for utlDummy() */

/* Definition of debug parameters */
#define SIU_DBG_BUFFER_SIZE              60   /* Trace records */
#define SIU_DBG_TRACE_PLAYOUT_INTERVAL   100
#define SIU_DBG_TRC_MAX_RECORDS_MSG      14
#define SIU_DBG_TRC_RECORD_SIZE          4    /* length of trace record 4 words */

#if ((SIU_DBG_TRC_MAX_RECORDS_MSG * SIU_DBG_TRC_RECORD_SIZE) > \
     (MGB_MSG_MAXNPAR))
# error Maximum debug trace message exceeds allowable maximum message length
#endif

/* Debug Trace record is comprised of four 16-bit parameters */
typedef struct {
  tuint  moduleID;     /* Module and channel identifier */
  tuint  messageCode;  /* Message code */
  tuint  suppData1;    /* Supporting data */
  tuint  suppData2;    /* Supporting data */
} siuDbgTrcRec_t;

/* Global Debug Utility Variables */
siuDbgTrcRec_t  siuDbgBuf[SIU_DBG_BUFFER_SIZE];
tuint  siuDbgBuf_InPtr       = 0;                   
tuint  siuDbgBuf_OutPtr      = 0;
tint   siuDbgBuf_count       = 0;
tuint  siuDbgBuf_dropCnt     = 0;
tulong siuTrcLastPlayoutTime = 0;

/******************************************************************************
 * FUNCTION PURPOSE: Initializes Debug Utility
 ******************************************************************************
 * DESCRIPTION: Initializes global debug utility variables.
 *
 *****************************************************************************/
void siuDebugBufferInit (void)
{

  siuDbgBuf_InPtr       = 0;
  siuDbgBuf_OutPtr      = 0;
  siuDbgBuf_count       = 0;
  siuDbgBuf_dropCnt     = 0;
  siuTrcLastPlayoutTime = siuTmrGetTime();

} /* siuDebugInit() */

/******************************************************************************
 * FUNCTION PURPOSE: Writes a trace data record to the debug buffer.
 ******************************************************************************
 * DESCRIPTION: 
 *
 *****************************************************************************/
void siuWriteDebugTrace (tuint moduleID, tuint messageCode, tuint msgLength, 
                         tuint *messageData)
{

  /* Check that there is room in the debug buffer for the new trace data */
  if (siuDbgBuf_count >= SIU_DBG_BUFFER_SIZE)
  {
     /* Trace data could not fit in buffer - drop trace and increment statistics */
     siuDbgBuf_dropCnt++;
     return;
  }
  else
  {
     /* Store new trace record in the debug buffer */
     siuDbgBuf[siuDbgBuf_InPtr].moduleID = moduleID;
     siuDbgBuf[siuDbgBuf_InPtr].messageCode = messageCode;
     siuDbgBuf[siuDbgBuf_InPtr].suppData1 = *messageData++;
     siuDbgBuf[siuDbgBuf_InPtr].suppData2 = *messageData;

     /* Increment in-pointer */
     siuDbgBuf_InPtr = (siuDbgBuf_InPtr + 1) % SIU_DBG_BUFFER_SIZE;

     /* Increment debug buffer counter */
     siuDbgBuf_count++;
    
  }

  /* Record indication of lost trace data due to debug buffer overflows */
  if ((siuDbgBuf_count < SIU_DBG_BUFFER_SIZE) && (siuDbgBuf_dropCnt > 0))
  {
     /* Insert lost trace data record */
     siuDbgBuf[siuDbgBuf_InPtr].moduleID = 0x00;
     siuDbgBuf[siuDbgBuf_InPtr].messageCode = 0x01;
     siuDbgBuf[siuDbgBuf_InPtr].suppData1 = siuDbgBuf_dropCnt;
     siuDbgBuf[siuDbgBuf_InPtr].suppData2 = 0x00;

     /* Increment in-pointer */
     siuDbgBuf_InPtr = (siuDbgBuf_InPtr + 1) % SIU_DBG_BUFFER_SIZE;

     /* Increment debug buffer counter */
     siuDbgBuf_count++;

     /* Reset dropCnt statistic */
     siuDbgBuf_dropCnt = 0;
  }


} /* siuDebugWriteTrace () */

/******************************************************************************
 * FUNCTION PURPOSE: Debug info function
 ******************************************************************************
 * DESCRIPTION: Processes DebugInfo calls by software modules.  If the module
 *              indicate a FATAL ERROR, the SIU calls an exception and provides
 *              a debug string specifing the FATAL ERROR code.  If the module
 *              provides INFORMATION data, the SIU sends the data to the debug
 *              task.
 *             
 *****************************************************************************/
static inline void siu_tuint_to_hex (char *dest, tuint val)
{
  tint i;
  
  dest[4] = 0; /* Null terminate */
  
  for (i=3; i>=0; i--) {
    dest[i] = (val & 0xF);
    if (dest[i] > 9)
      dest[i] += 'A' - 10;
    else
      dest[i] += '0';
    val >>= 4;
  }
}

#ifdef NDU_DBG4
tuint siu_dbg_array[2];
#endif /* NDU_DBG4 */
#if (defined(HDVIDEO) || defined(HDCT))
tuint siu_dbg_array[2];
#endif /* (defined(HDVIDEO) || defined(HDCT)) */
void siuDebugInfo (tuint moduleID, tuint msgType, tuint messageCode, 
                   tuint msgLength, tuint *messageData)
{
  char exc_string[25];

  switch (msgType)
  {
     case SIU_DEBUG_CHAN_EXCEP:
        /* Used for channel-based exception */
        siuProcChanExcep (moduleID, messageCode);
        break;
  
     case SIU_DEBUG_FATAL:
#ifdef NDU_DBG4
        siu_dbg_array[0] = (moduleID & 0xFF00) >> 8;
        siu_dbg_array[1] = messageCode & 0x3FF;
#endif /* NDU_DBG4 */

#if (defined(HDVIDEO) || defined(HDCT))
        siu_dbg_array[0] = (moduleID & 0xFF00) >> 8;
        siu_dbg_array[1] = messageCode & 0x3FF;
#endif /* (defined(HDVIDEO) || defined(HDCT)) */

        /* Convert message code to a string -- call SIU exception handler */
        strcpy (exc_string, "Exception 0x");
        /* Tack exception code after previous string */
        siu_tuint_to_hex (exc_string+12, 
                          ((moduleID & 0xFF00) << 2) | (messageCode & 0x3FF));
        #if 0                  
        return;
        #endif                   
        siuException (moduleID, exc_string); 
        break;
     case SIU_DEBUG_WARNING:
        /* Used for non-fatal error indications - messageCode must be less than 512 */
        siuErrorIndication (moduleID, 0, (messageCode | MGB_ERR_INTERNAL_MASK));
        break;

     case SIU_DEBUG_INFO:
        /* Post debug trace to debug task */
        siuWriteDebugTrace (moduleID, messageCode, msgLength, messageData);
        break;

     default:
        break;  /* Ignore invalid message types */
  }
} /* siuDebugInfo() */

/******************************************************************************
 * FUNCTION PURPOSE: Translate between RTSC void * and siu ID
 ******************************************************************************
 *
 *****************************************************************************/
void siuRtscDebugInfo (void *handle, tuint msgType, tuint messageCode, 
                       tuint msgLength, tuint *messageData) {
  siuDebugInfo((tuint)handle, msgType, messageCode, msgLength, messageData);
}

/******************************************************************************
 * FUNCTION PURPOSE: Debug Task
 ******************************************************************************
 * DESCRIPTION: Prints debug trace data records from the debug buffer to an
 *              external sink.
 *
 *****************************************************************************/
void siuDebugTask (void)
{
  tulong currentTime, delta;
  tint i, numTrcRecords, status, count;
  tuint Outptr;
  mgb_t send_message;

  thwCriticalState_t origState = SIU_DBGCNT_BEGIN();
  /* If debug buffer is empty, return immediately */
  if (siuDbgBuf_count == 0)
  {
    goto end;
  }

  /* Ensure a minimum time between trace messages to the microprocessor */
  currentTime = siuTmrGetTime();
  delta = siuTmrCalcDTime (currentTime, siuTrcLastPlayoutTime);
  if (siuDbgBuf_count < SIU_DBG_TRC_MAX_RECORDS_MSG) {
    if (delta < SIU_DBG_TRACE_PLAYOUT_INTERVAL)
    {
      goto end;
    }
  }

  siuTrcLastPlayoutTime = currentTime;
  /* Copy trace data to message body */
  count = siuDbgBuf_count;
  numTrcRecords = 0;
  Outptr = siuDbgBuf_OutPtr;
  i = 0;
  while ((numTrcRecords < SIU_DBG_TRC_MAX_RECORDS_MSG) && (count > 0))
  {
     send_message.msg_body.debug_trace_data.trace_data[i++] = siuDbgBuf[Outptr].moduleID;
     send_message.msg_body.debug_trace_data.trace_data[i++] = siuDbgBuf[Outptr].messageCode;
     send_message.msg_body.debug_trace_data.trace_data[i++] = siuDbgBuf[Outptr].suppData1;
     send_message.msg_body.debug_trace_data.trace_data[i++] = siuDbgBuf[Outptr].suppData2;

     /* Increment out-pointer */
     Outptr = (Outptr + 1) % SIU_DBG_BUFFER_SIZE;
     numTrcRecords++;
     count--;

#ifdef C54X_BRAF_BUG
     utlDummy();  /* RPTB bug */
#endif
  }

  /* Create message header */
  send_message.header.chnum = 0;  /* Channel independent message */
  send_message.header.msg_id = MGB_DM_DEBUG_TRACE;
  send_message.header.length = MGB_HEADER_SIZE + i;
  send_message.header.trans_id = 0; /* Not a response to a message */

  /* Send message to MPU */
  status = siuNduSendMsg(&send_message);

  /* If message accepted, update out pointer and count */
  if (status != 0)
  {
     siuDbgBuf_OutPtr = Outptr;
        siuDbgBuf_count = count;
  }
end:
  SIU_DBGCNT_END(origState);
  
} /* siuDebugTask() */
