/*****************************************************************************************
 * FILE PURPOSE: Siu functions called from an ISR context
 *****************************************************************************************
 * FILE NAME: siumisr.c
 *
 * DESCRIPTION: Contains functions called from an ISR context. This is done to
 *				improve performance in systems with cached memory and ram set.
 *
 * TABS: None
 *
 * Copyright (C) 2003, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
 #include "types.h"
 #include "siuloc.h"
 #include "ggcfg.h"
 #include "common/siuCommonMain.h"
 #include <ti/mas/util/pkt.h>

 #include <string.h>


/*********************************************************************************/
void siuHaltCheck() {
  if (siuContext.halt.halt.halt_if_set) {
    siuContext.halt.halt.halted_if_set = 1;
    thwCriticalBegin();
    siu_preserve_registers(SIU_CPU_REGISTERS_HALT_CHECK, siuContext.siu_common_scratch);
    while(1);
  }
}

#ifdef PROFILE_IDLE_MIPS
/*********************************************************************************
 * FUNCTION PURPOSE: Accumulate cycles spent in ISR
 *********************************************************************************
  DESCRIPTION:      This function is called from all the ISRs to account for all 
                    the cycles spent in ISRs. This is subtracted from the cycles 
                    measured in idle-loop so that the idle cycles are accurate
  Parameters :      Inputs: begin_time      : entry point time into ISR
                            end_time        : exit point time from ISR
                    Output: none
 *********************************************************************************/
void siuIsrCycles(tulong begin_time, tulong end_time) {
    tulong cycles;
    cycles = end_time - begin_time;
    siuContext.siuAccIsrCycles += cycles;
    return;
}

/*********************************************************************************
 * FUNCTION PURPOSE: Update the IDLE report to be sent out of DSP
 *********************************************************************************
  DESCRIPTION:      This function is called every 'n'-millisec where n is configurable.
                    The IDLE report is updated exactly every 'n'-millisec. There can
                    be some jitter in actually when the report goes out
  Parameters :      Inputs: None
                    Output: None
 *********************************************************************************/
void siuUpdateIdleReport (void) {
  SIU_STATE_BEGIN();
  if (siuContext.idleReport.reportPending == TRUE) {
    siuContext.idleReport.numReportsMissed++;
  }
  siuContext.idleReport.idleMIPSCycles = siuContext.siuIdleMIPSCycles;
  siuContext.idleReport.isr_clock      = siu_isr_clock;
  siuContext.idleReport.numRxTasksProcessed = siuContext.numRxTasksProcessed;
  siuContext.idleReport.numRxPkts      = vppuContext.rxPkts;
  siuContext.idleReport.numTxPkts      = vppuContext.txPkts;
  siuContext.idleReport.reportPending  = TRUE;
  /* reset siuContext values */
  siuContext.siuIdleMIPSCycles = 0;
  siuContext.numRxTasksProcessed = 0;
  //siuContext.numRxPkts = 0;
  //siuContext.numTxPkts = 0;
  SIU_STATE_END();
  return;
}

/*********************************************************************************
 * FUNCTION PURPOSE: sends IDLE report over UDP-IP-ETH
 *********************************************************************************
  DESCRIPTION:      This function is called to send the IDLE MIPS report to PC
  Parameters :      Inputs: None
                    Output: None
 
 * MIPS REPORT FORMAT
 ****************************************************************************** 
 *  0                   1                   2                   3
 *  0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
 * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 * |          seq_num              |   data_sampled@_isr_clock     |
 * +=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+
 * |   data_shipped@_isr_clock     |        idle_cycles            |
 * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 * |  #of rx_tasks processed       |     #of rx_pkts in VPPU       |
 * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 * |  #of tx_pkts in VPPU          |     #of missed_reports        |
 * +=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+
 *********************************************************************************/

#define SIU_IDLE_MIPS_REPORT_SEQNUM_OFF              0
#define SIU_IDLE_MIPS_REPORT_DATA_SAMPLED_ISR_OFF    4
#define SIU_IDLE_MIPS_REPORT_DATA_SHIPPED_ISR_OFF    8
#define SIU_IDLE_MIPS_REPORT_IDLE_CYCLES_OFF        12
#define SIU_IDLE_MIPS_REPORT_NUM_RX_TASKS_PROC_OFF  16 
#define SIU_IDLE_MIPS_REPORT_NUM_RX_PKTS_OFF        20
#define SIU_IDLE_MIPS_REPORT_NUM_TX_PKTS_OFF        24
#define SIU_IDLE_MIPS_REPORT_NUM_MISSED_REPORTS_OFF 28
#define SIU_IDLE_MIPS_REPORT_SIZE                  SIU_IDLE_MIPS_REPORT_NUM_MISSED_REPORTS_OFF+4

void siu_send_idle_report (void) {
  extern word vDebugNeuStreamerScratch[1500];
  xferPktInfo_t pkt;
  void      *data;
  word      *payload;
  tint      pktLength;
  extern    NDU_gmacContext_t NDU_gmacContext;

  if ((NDU_gmacContext.nduTrace).neuInst != NULL) {
     pkt.srcPort = 0;
     pkt.npkts   = 1;
     payload = (word *)(vDebugNeuStreamerScratch);
     pktWrite32bits (payload, SIU_IDLE_MIPS_REPORT_SEQNUM_OFF,             siuContext.idleReport.seq_num);
     pktWrite32bits (payload, SIU_IDLE_MIPS_REPORT_DATA_SAMPLED_ISR_OFF,   siuContext.idleReport.isr_clock);
     pktWrite32bits (payload, SIU_IDLE_MIPS_REPORT_DATA_SHIPPED_ISR_OFF,   siu_isr_clock);
     pktWrite32bits (payload, SIU_IDLE_MIPS_REPORT_IDLE_CYCLES_OFF,        siuContext.idleReport.idleMIPSCycles);
     pktWrite32bits (payload, SIU_IDLE_MIPS_REPORT_NUM_RX_TASKS_PROC_OFF,  siuContext.idleReport.numRxTasksProcessed);
     pktWrite32bits (payload, SIU_IDLE_MIPS_REPORT_NUM_RX_PKTS_OFF,        siuContext.idleReport.numRxPkts);
     pktWrite32bits (payload, SIU_IDLE_MIPS_REPORT_NUM_TX_PKTS_OFF,        siuContext.idleReport.numTxPkts);
     pktWrite32bits (payload, SIU_IDLE_MIPS_REPORT_NUM_MISSED_REPORTS_OFF, siuContext.idleReport.numReportsMissed);

     siuContext.idleReport.seq_num++;
     pktLength   = SIU_IDLE_MIPS_REPORT_SIZE;
     data = (void *)(vDebugNeuStreamerScratch);
     pkt.pktSize = &pktLength;
     pkt.pktIn   = &data;
     pkt.type    = XFER_PKT_TYPE_NULL;
     SIU_STATE_BEGIN();
       (*gg_neu_ctable.neuSendIn) ((NDU_gmacContext.nduTrace).neuInst, &pkt);
     SIU_STATE_END();
  }
  siuContext.idleReport.reportPending = FALSE;
  return;
}
#endif /* PROFILE_IDLE_MIPS */
/* Nothing past this point */
