/******************************************************************************
 * FILE PURPOSE: Contains functions necessary to integrate RTCP 
 ******************************************************************************
 * FILE NAME: siurtcp.c
 *
 * DESCRIPTION: Contains functions necessary to integrate RTCP (RTP Control 
 *              protocol) into system, such that it can be call tabled out 
 *              of the system
 *
 * (C) Copyright 2002, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
 
/* System utility files */
#include "types.h"
#include "mgmicdsp.h"
#include "ggcfg.h"
#include "utlrepck.h"
#include <ti/mas/util/pkt.h>
//#include "rtcpxr.h"
#include <ti/mas/iface/ifrcu/ifrcu.h>

/* SIU files */
#include "siuloc.h"
#ifndef HDCT
#include "common/siuctbl.h"     /* SIU internal call table */
#endif
#include "siurtcp.h"
 
/* Module files */
#include <ti/mas/rcu/rcu.h> 
#include <ti/mas/rcu/rcurtcp.h>
#ifdef GG_INCLUDE_VQM
#include "vqm.h"
#endif

/* defines for gg_dummy to match with those in makegg */
#define GG_TELCHEMY_ENABLED_MASK 256
extern const tint gg_dummy;


/******************************************************************************
 * FUNCTION PURPOSE: controls RTCP operation
 ******************************************************************************
 * DESCRIPTION: Process the RTCP Control message. 
 *****************************************************************************/
void siu_rtcp_control (siuInst_t *inst, mgb_t *msg) 
{
   rcuRtcpCfg_t rtcpCfg;
   bool config_error = FALSE;
#if (defined(VQM_RFC3611) || defined(VQM_H248XNQ))
   tuint new_xr_type;
#endif
   bool XR_config_error = FALSE;
   
   /* Bitmap for parameter extraction */
   rtcpCfg.bitmap  = msg->msg_body.rtcp_control.ctrlBitmap;

   /* RTCP transmission repeat interval in ms */
   /* Valid range is 200ms-40000ms so make sure this is correct */
   rtcpCfg.txRptInt = msg->msg_body.rtcp_control.txRptInt;

   /* Event Report Bitmap */
   rtcpCfg.reportBitmap = msg->msg_body.rtcp_control.reportBitmap;

   /* NTP Timestamp */
   rtcpCfg.NTP_timestampMS1 = msg->msg_body.rtcp_control.NTP_timestampMS1;
   rtcpCfg.NTP_timestampMS2 = msg->msg_body.rtcp_control.NTP_timestampMS2;
   rtcpCfg.NTP_timestampMS3 = msg->msg_body.rtcp_control.NTP_timestampMS3;
   rtcpCfg.NTP_timestampMS4 = msg->msg_body.rtcp_control.NTP_timestampMS4;

   /* Session timeout Multiplier. */
   rtcpCfg.timeoutMult = msg->msg_body.rtcp_control.timeoutMult;    

   /* Timeout timer Control field */
   rtcpCfg.timerCtrl = msg->msg_body.rtcp_control.timerCtrl;

   /* Accept XR configuration only when telchemy is in the build. */
   if (gg_dummy & GG_TELCHEMY_ENABLED_MASK) 
   {
     /* XR VoIP metrics report Control */
     rtcpCfg.xrConfig      = msg->msg_body.rtcp_control.xrConfig;
     rtcpCfg.xr_tx_mult    = msg->msg_body.rtcp_control.xr_tx_mult;
     rtcpCfg.xr_host_thr   = msg->msg_body.rtcp_control.xr_host_throttle;
     rtcpCfg.xrConfigPiqua = msg->msg_body.rtcp_control.xrConfigPiqua;
   }
   else if (rtcpCfg.bitmap & MGB_RTCP_CTRL_XR_CFG_VALID) {
       if(msg->msg_body.rtcp_control.xrConfig != MGB_RTCP_XR_DISABLE){
           XR_config_error     = TRUE;
       }
   }
#if (defined(VQM_RFC3611) || defined(VQM_H248XNQ))
   /* Select VQM type and if H248, configure severe error seconds */
   new_xr_type = VQM_GET_XR_TYPE(inst->heapInst->stats.vqmStats);
   if (rtcpCfg.bitmap & MGB_RTCP_CTRL_XR_TYPE_VALID) {
     new_xr_type = msg->msg_body.rtcp_control.rtcp_xr_type;   
   }
   if (new_xr_type == MGB_RTCP_XR_TYPE_H248XNQ) {
#ifdef VQM_H248XNQ
     if (inst->vqmInst) {
       /* Memory overlayed; get rid of telchmy vqm */
       gg_vqm_ctable.vqmClose(inst->vqmInst, 0);
     }
     if (rtcpCfg.bitmap & MGB_RTCP_CTRL_XR_XNQ_SPLT_VALID) {
       VQM_SET_XR_SEVERE_THRESH(inst->heapInst->stats.vqmStats, 
                                msg->msg_body.rtcp_control.severe_degrad_threshold);
     }
#else
     config_error = TRUE; /* No H248XNQ VQM */
#endif
   }
#endif   
   if (!config_error && !XR_config_error)  {
#if (defined(VQM_RFC3611) || defined(VQM_H248XNQ))
     VQM_SET_XR_TYPE(inst->heapInst->stats.vqmStats, new_xr_type);
#endif
     config_error = (*gg_rcuRtcp_ctable.rtcpConfig) (inst->net.rcuInst, &rtcpCfg);
   }

   if (config_error || XR_config_error) {
     siuErrorIndication (inst->ID, msg->header.trans_id, 
                         MGB_ERR_INVALID_RTCP_CONFIG );
   }
} /* siu_rtcp_control */

/******************************************************************************
 * FUNCTION PURPOSE: Specifies the SDES STRING for RTCP
 ******************************************************************************
 * DESCRIPTION: Gets the SDES STRING for RTCP 
 *****************************************************************************/
void siu_rtcp_sdes_string (siuInst_t *inst, mgb_t *msg) 
{
  tuint bitmap, typeLength, sdes_type, sdes_usage, length, total_len, msglen; 
  word *asciiPtr;
   
  /* Extract the string size from msg size */
  msglen   = msg->header.length - (MGB_HEADER_SIZE + (6 * MGB_MSGPAR_SIZE));
  msglen <<= 2;
   
  typeLength = msg->msg_body.rtcp_sdes_string.typeLength;
  bitmap     = msg->msg_body.rtcp_sdes_string.bitmap;
  
  /* Extract SDES type */
  sdes_type = (typeLength >> SIU_RTCP_SDES_TYPE_OFF) & SIU_RTCP_SDES_TYPE_MASK;

  /* Extract length of string in the packet */
  length = (typeLength >> SIU_RTCP_SDES_LEN_OFF) & SIU_RTCP_SDES_LEN_MASK;  

  /* Input string can not be longer than allowed in the message */
  if (length > msglen) length = msglen;

  asciiPtr = (word *) msg->msg_body.rtcp_sdes_string.ASCII_char;

  /* Global SDES string */
  if (msg->header.chnum == 0) { /* Global SDES String manipulation */

    /* Handle XR stuff first */

    if (msg->msg_body.rtcp_sdes_string.xrBitmap & MGB_RTCP_EXT_R_FACTOR_VALID) {
      /* Get the External R Factor */
      rcuRtcpContext.extRfactor = msg->msg_body.rtcp_sdes_string.extRfactor;
    }

    if (msg->msg_body.rtcp_sdes_string.xrBitmap & MGB_RTCP_QA_INIT_TIME_VALID) {
      /* Get the QA init time */
      rcuRtcpContext.qaInitTime =  msg->msg_body.rtcp_sdes_string.qaInitTime;
    }

    if (msg->msg_body.rtcp_sdes_string.xrBitmap & MGB_RTCP_XR_PIQUA_BLKTYPE_VALID) {
      /* Get the XR Piqua Block Type */
      rcuRtcpContext.piquaBlkType = msg->msg_body.rtcp_sdes_string.piquaBlkType;
    }

    rcuRtcpContext.sdesType = sdes_type; /* Store the type of Global SDES string */
    if (bitmap & MGB_RTCP_APPEND_FLAG) { /* Append to global sdes string */
      total_len = length + rcuRtcpContext.sdesString_len; /* Compute total length */ 
      if (total_len > rcu_RTCP_MAX_GLOBAL_SDES_L_B) { /* Check for the MAX size 
                                                   * of global SDES String */         
        length = rcu_RTCP_MAX_GLOBAL_SDES_L_B - rcuRtcpContext.sdesString_len;
      }

      utlRepackWordsIntoWords(asciiPtr, rcuRtcpContext.sdesString, 0, length, rcuRtcpContext.sdesString_len);
      rcuRtcpContext.sdesString_len += length;
    }
    else { /* Overwrite the global sdes string with the current string */
      utlRepackWordsIntoWords(asciiPtr, rcuRtcpContext.sdesString, 0, length, 0);
      rcuRtcpContext.sdesString_len = length;
    }    
    return;       
  } /* Global SDES String manipulation */  

  /* Channel specific SDES String manipulation */  
  if (length > rcu_RTCP_MAX_LOCAL_SDES_L_B)
    length = rcu_RTCP_MAX_LOCAL_SDES_L_B;

  if (bitmap & MGB_RTCP_STRSLCT_FLAG) { /* String Selection */
    sdes_usage = rcu_RTCP_SDES_LOCAL_N_GLOBAL;
  }
  else { /* Just use local string for SDES computation. */
    sdes_usage = rcu_RTCP_SDES_LOCAL;
  }

  (*gg_rcuRtcp_ctable.rtcp_sdes_update)(inst->net.rcuInst, asciiPtr, length, sdes_usage, sdes_type);
  
} /* siu_rtcp_sdes_string */

/******************************************************************************
 * FUNCTION PURPOSE: RTCP send packet message
 ******************************************************************************
 * DESCRIPTION: Process the RTCP sends packet message. 
 *****************************************************************************/
void siu_rtcp_send_packet (siuInst_t *inst, mgb_t *msg) 
{
  tuint ctrlBitmap, str_len, transid;
  word *str; 
  
  transid    = msg->header.trans_id;
  ctrlBitmap = msg->msg_body.rtcp_send_packet.ctrlBitmap;
  str        = (word *) msg->msg_body.rtcp_send_packet.packet;
  str_len    = msg->header.length - (MGB_HEADER_SIZE + (4 * MGB_MSGPAR_SIZE));
  str_len  <<= 1; /* in bytes */

  /* Send MGB_ACK_TYPE_PROC to host. This Ack type needs to be done before calling            */
  /* rtcp_send_hpkt() function because the function may internally send out MGB_ACK_TYPE_DONE */
  if (transid & MGB_TRANS_ACK_BIT) {
    siuGeneralAck (msg->header.msg_id, msg->header.chnum, transid, MGB_ACK_TYPE_PROC, 
                   0, SIU_DPR_NORMAL_MSG);
  }
  (*gg_rcuRtcp_ctable.rtcp_send_hpkt)( inst->net.rcuInst, ctrlBitmap, str, str_len,
                                       transid );
 
} /* siu_rtcp_send_packet */

/******************************************************************************
 * FUNCTION PURPOSE: RTCP request statistics
 ******************************************************************************
 * DESCRIPTION: Process the RTCP request statistics message. 
 *  void siu_rtcp_req_stats (siuInst_t *inst,          - SIU instance
 *                           tuint TxRx_Report,        - report type
 *                           rcuRtcpStatReq_t *reqmsg) - Statistics request data
 *****************************************************************************/
void siu_rtcp_req_stats (siuInst_t *inst, tuint trans_id, 
                         tuint TxRx_Report, rcuRtcpStatReq_t *reqmsg, tuint priority) 
{
  rcuRtcpReport_t  rptmsg;  /* statistics report */
  mgb_t send_message;   
  rcuRtcpRxInf_t  *rx_rr;

  /* Set everything to 0 before preparing message */
  memset (&send_message, 0, sizeof(mgb_t));

  /* Prepare response to stats message */
  send_message.header.length   = MGB_HEADER_SIZE + 
    (sizeof(struct MGB_DMS_RTCP_REQ_STATS)) * MGB_MSGPAR_SIZE;
  send_message.header.chnum    = (int)(inst->ID & SIU_CID_MASK);
  send_message.header.msg_id   = MGB_DM_RTCP_REQ_STATS;
  send_message.header.trans_id = trans_id;
   
  (*gg_rcuRtcp_ctable.rtcp_statsReq)( inst->net.rcuInst, reqmsg, &rptmsg );   

  send_message.msg_body.rtcp_stats.reportType = 
                                        reqmsg->packetType | (TxRx_Report << 8);
  send_message.msg_body.rtcp_stats.delaySince = reqmsg->delaySince; 

  send_message.msg_body.rtcp_stats.roundTripTime = reqmsg->roundTripTime;   

  send_message.msg_body.rtcp_stats.rxRtcpPkts = reqmsg->rxRtcpPkts;
  send_message.msg_body.rtcp_stats.txRtcpPkts = reqmsg->txRtcpPkts;
 
  if (reqmsg->packetType == rcu_RTCP_PACKET_TYPE_SR) {
    send_message.msg_body.rtcp_stats.sr_SSRC_MSW = siu_get_umsw (reqmsg->ssrc);
    send_message.msg_body.rtcp_stats.sr_SSRC_LSW = siu_get_ulsw (reqmsg->ssrc);

    send_message.msg_body.rtcp_stats.sr_NTP_MS1 = 
                                        siu_get_umsw (rptmsg.sr.sender.NTP_MSL);
    send_message.msg_body.rtcp_stats.sr_NTP_MS2 = 
                                        siu_get_ulsw (rptmsg.sr.sender.NTP_MSL);
    send_message.msg_body.rtcp_stats.sr_NTP_MS3 = 
                                        siu_get_umsw (rptmsg.sr.sender.NTP_LSL);
    send_message.msg_body.rtcp_stats.sr_NTP_MS4 = 
                                        siu_get_ulsw (rptmsg.sr.sender.NTP_LSL);

    send_message.msg_body.rtcp_stats.sr_pktcnt_MSW = 
                            siu_get_umsw (rptmsg.sr.sender.txPktCnt);
    send_message.msg_body.rtcp_stats.sr_pktcnt_LSW = 
                            siu_get_ulsw (rptmsg.sr.sender.txPktCnt);
    send_message.msg_body.rtcp_stats.sr_octcnt_MSW = 
                            siu_get_umsw (rptmsg.sr.sender.txOctCnt);
    send_message.msg_body.rtcp_stats.sr_octcnt_LSW = 
                            siu_get_ulsw (rptmsg.sr.sender.txOctCnt);
  }   

  if (reqmsg->packetType == rcu_RTCP_PACKET_TYPE_SR) 
    rx_rr = &(rptmsg.sr.receiver);
  else
    rx_rr = &(rptmsg.rr);

  send_message.msg_body.rtcp_stats.rr_SSRC_MSW = 
                          siu_get_umsw (rx_rr->ssrc);
  send_message.msg_body.rtcp_stats.rr_SSRC_LSW = 
                          siu_get_ulsw (rx_rr->ssrc);

  send_message.msg_body.rtcp_stats.rr_fracPkt_lost = 
                          siu_get_umsw (rx_rr->lostInf);
  send_message.msg_body.rtcp_stats.rr_pkt_lost =
                          siu_get_ulsw (rx_rr->lostInf);

  send_message.msg_body.rtcp_stats.rr_xHseqNum_MSW = 
                          siu_get_umsw (rx_rr->xHseqNum);
  send_message.msg_body.rtcp_stats.rr_xHseqNum_LSW = 
                          siu_get_ulsw (rx_rr->xHseqNum);

  send_message.msg_body.rtcp_stats.rr_jitter_MSW = 
                          siu_get_umsw (rx_rr->rx_jitter);
  send_message.msg_body.rtcp_stats.rr_jitter_LSW = 
                          siu_get_ulsw (rx_rr->rx_jitter);

  send_message.msg_body.rtcp_stats.rr_LSRstamp_MSW =
                          siu_get_umsw (rx_rr->last_SR_ts);
  send_message.msg_body.rtcp_stats.rr_LSRstamp_LSW =
                          siu_get_ulsw (rx_rr->last_SR_ts);

  send_message.msg_body.rtcp_stats.rr_DLSR_MSW = 
                          siu_get_umsw (rx_rr->DLSR);
  send_message.msg_body.rtcp_stats.rr_DLSR_LSW = 
                          siu_get_ulsw (rx_rr->DLSR);

  /* send message to FIFO */
  siuNduSendMsgPriority(&send_message,(NDU_DPRMsgType_t)priority); 

} /* siu_rtcp_req_stats */


void siuRtcpGetStatsToPkt (void *siuInst, bool reset, tuint trans_id, tuint priority) 
{
  siuInst_t *inst = (siuInst_t *) siuInst;
  rcuRtcpStatReq_t reqmsg;

  /* Check whether active protocol is RTP */
  if(SIU_IS_PROTO_RTP(inst) ||
     (SIU_IS_PROTO_IU(inst) && SIU_IS_IU_OVER_IP(inst))){
    /* Do we need to reset the stats */
    reqmsg.reset = reset; 
    /* Send TX statistics */
    reqmsg.code = rcu_RTCP_LAST_TX_REPORT;
    siu_rtcp_req_stats (inst, trans_id, SIU_RTCP_TX_REPORT, &reqmsg, priority);
  }
  else {
    /* Current active protocol is NOT RTP. This message is not valid 
     * for this protocol. Report error */
     siu_get_stats_unavail ( inst, reset, trans_id, priority );
  }    

}
   
void siuRtcpGetStatsFromPkt (void *siuInst, bool reset, tuint trans_id, tuint priority) 
{
  siuInst_t *inst = (siuInst_t *) siuInst;
  rcuRtcpStatReq_t reqmsg;

  /* Check whether active protocol is RTP */
  if(SIU_IS_PROTO_RTP(inst) ||
     (SIU_IS_PROTO_IU(inst) && SIU_IS_IU_OVER_IP(inst))){
    /* Do we need to reset the stats */
    reqmsg.reset = reset; 
    /* Send TX statistics */
    reqmsg.code = rcu_RTCP_LAST_RX_REPORT;
    siu_rtcp_req_stats (inst, trans_id, SIU_RTCP_RX_REPORT, &reqmsg, priority);
  }
  else {
    /* Current active protocol is NOT RTP. This message is not valid 
     * for this protocol. Report error */
    siu_get_stats_unavail ( inst, reset, trans_id, priority );
  }    

}

/******************************************************************************
 * FUNCTION PURPOSE: Report XR VOIP Block to host
 ******************************************************************************
 * DESCRIPTION:  
 *****************************************************************************/
void siuRtcpReportXRVoip (void *siuInst,   tuint trans_id, 
                          tuint priority,  tuint location, 
                          tuint valid_lsw, tuint valid_msw, 
                          rcuRtcpXRStats_t *xrVoipStats, 
                          tuint lisnRfactor)
{                        
  siuInst_t *inst = (siuInst_t *) siuInst;
  tuint level;
  mgb_t send_message;   
  
 /* Set everything to 0 before preparing message */
  memset (&send_message, 0, sizeof(mgb_t));

  /* Prepare response to stats message */
  send_message.header.length   = MGB_HEADER_SIZE + 
    (sizeof(struct MGB_DMS_RTCP_XRVOIP_STATS)) * MGB_MSGPAR_SIZE;
  send_message.header.chnum    = (int)(inst->ID & SIU_CID_MASK);
  send_message.header.msg_id   = MGB_DM_RTCP_XRVOIP_STAT;
  send_message.header.trans_id = trans_id;   

  send_message.msg_body.rtcp_xrvoip_stats.location      = location; 
  send_message.msg_body.rtcp_xrvoip_stats.validPrms_MSW = valid_msw; 
  send_message.msg_body.rtcp_xrvoip_stats.validPrms_LSW = valid_lsw; 

  send_message.msg_body.rtcp_xrvoip_stats.lossRate     = 
                                               xrVoipStats->lossDscrdRate >> 8;  
  send_message.msg_body.rtcp_xrvoip_stats.discardRate  = 
                                               xrVoipStats->lossDscrdRate & 0xFF;
  send_message.msg_body.rtcp_xrvoip_stats.burstDensity =  
                                               xrVoipStats->brstGapDnsty >> 8;
  send_message.msg_body.rtcp_xrvoip_stats.gapDensity   =    
                                               xrVoipStats->brstGapDnsty & 0xFF;

  send_message.msg_body.rtcp_xrvoip_stats.burstDuration = xrVoipStats->brstDuration;
  send_message.msg_body.rtcp_xrvoip_stats.gapDuration   = xrVoipStats->gapDuration;

  send_message.msg_body.rtcp_xrvoip_stats.roundTripDelay =
                                                     xrVoipStats->roundTripDelay;
  send_message.msg_body.rtcp_xrvoip_stats.endSysDelay    =    
                                                     xrVoipStats->endSysDelay;

  /*
   * Both Signal and Noise level are 8-bit signed number
   * we need to extend them to 16-bit signed number
   */
  level = (xrVoipStats->sigNoiseLev >> 8) & 0x00FF;
  level = (level & 0x80)?(level|0xFF00):level;
  send_message.msg_body.rtcp_xrvoip_stats.signalLev =  (INT16)level;
  
  level = xrVoipStats->sigNoiseLev & 0xFF;
  level = (level & 0x80)? (level|0xFF00):level;
  send_message.msg_body.rtcp_xrvoip_stats.noiseLev  =  (INT16)level;

  send_message.msg_body.rtcp_xrvoip_stats.RERL = xrVoipStats->RERLGmin >> 8;
  send_message.msg_body.rtcp_xrvoip_stats.gmin = xrVoipStats->RERLGmin & 0xFF;

  send_message.msg_body.rtcp_xrvoip_stats.RFactor    =        
                                                 xrVoipStats->RfacExtRfac >> 8;
  send_message.msg_body.rtcp_xrvoip_stats.extRFactor =    
                                                 xrVoipStats->RfacExtRfac & 0xFF;

  send_message.msg_body.rtcp_xrvoip_stats.MOS_LQ = xrVoipStats->MOS_LQ_CQ >> 8;
  send_message.msg_body.rtcp_xrvoip_stats.MOS_CQ = xrVoipStats->MOS_LQ_CQ & 0xFF;

  send_message.msg_body.rtcp_xrvoip_stats.rxCfg  = xrVoipStats->rxConfig >> 8;

  send_message.msg_body.rtcp_xrvoip_stats.JB_nom    = xrVoipStats->jbNom;
  send_message.msg_body.rtcp_xrvoip_stats.JB_max    = xrVoipStats->jbMax;
  send_message.msg_body.rtcp_xrvoip_stats.JB_absMax = xrVoipStats->jbAbsMax;

  send_message.msg_body.rtcp_xrvoip_stats.lisnRFactor = lisnRfactor;        

  /* send message to FIFO */
  siuNduSendMsgPriority(&send_message,(NDU_DPRMsgType_t)priority);
} /* siuRtcpReportXRVoip */

/******************************************************************************
 * FUNCTION PURPOSE: Report Local XR VOIP statistics
 ******************************************************************************
 * DESCRIPTION: Reports locally generated XR VOIP statistics 
 *****************************************************************************/

void siuRtcpGetXRVoipLoc (void *siuInst, bool reset, tuint trans_id, tuint priority) 
{
  siuInst_t *inst = (siuInst_t *) siuInst;
  rcuRtcpXRStats_t xrVoipStats;
  tuint valid_lsw = 0, valid_msw = 0;
  tuint lisnRFactor = 0;

 /* Set everything to 0 before preparing message */
  memset (&xrVoipStats,  0, sizeof(rcuRtcpXRStats_t));

  /* Check whether active protocol is RTP */
  if ((inst->protocol_type != MGB_PT_RTP) || !(gg_dummy & GG_TELCHEMY_ENABLED_MASK)) {
    /* Current active protocol is NOT RTP. This message is not valid 
     * for this protocol. Report error */
    siu_get_stats_unavail ( inst, reset, trans_id, priority );
    return;
  }    

  /* Get the stats from RTCP */
  (*gg_rcuRtcp_ctable.getXRVoip)( inst->net.rcuInst, &valid_lsw, &valid_msw,
                                  &xrVoipStats, rcu_RTCP_XR_VOIP_LOC, 
                                  &lisnRFactor );

  
  siuRtcpReportXRVoip (siuInst, trans_id, priority,  
                       rcu_RTCP_XR_VOIP_LOC, 
                       valid_lsw, valid_msw, &xrVoipStats, lisnRFactor);
 
} /* siuRtcpGetXRVoipLoc */

/******************************************************************************
 * FUNCTION PURPOSE: Report remotely received XR VOIP message
 ******************************************************************************
 * DESCRIPTION:  
 *****************************************************************************/

void siuRtcpGetXRVoipRem (void *siuInst, bool reset, tuint trans_id, tuint priority) 
{
  siuInst_t *inst = (siuInst_t *) siuInst;
  rcuRtcpXRStats_t xrVoipStats;
  tuint valid_lsw = 0, valid_msw = 0;
  tuint lisnRFactor = 0;

 /* Set everything to 0 before preparing message */
  memset (&xrVoipStats,  0, sizeof(rcuRtcpXRStats_t));

  /* Check whether active protocol is RTP */
  if ((inst->protocol_type != MGB_PT_RTP) || !(gg_dummy & GG_TELCHEMY_ENABLED_MASK)) {
    /* Current active protocol is NOT RTP. This message is not valid 
     * for this protocol. Report error */
    siu_get_stats_unavail ( inst, reset, trans_id, priority );
    return;
  }    

  /* Get the stats from RTCP */
  (*gg_rcuRtcp_ctable.getXRVoip)( inst->net.rcuInst, &valid_lsw, &valid_msw,
                                  &xrVoipStats, rcu_RTCP_XR_VOIP_REM,
                                  &lisnRFactor );
  
  siuRtcpReportXRVoip (siuInst, trans_id, priority, 
                       rcu_RTCP_XR_VOIP_REM,
                       valid_lsw, valid_msw, &xrVoipStats, 0);
 
} /* siuRtcpGetXRVoipRem */

#ifdef VQM_H248XNQ
/******************************************************************************
 * FUNCTION PURPOSE: Converts XR VOIP2 packet format into mgmicdsp.h format
 ******************************************************************************
 * DESCRIPTION: Populates a struct MGB_DMS_RTCP_XRVOIP_STATS2 structure
 *****************************************************************************/
void siuRtcpReportXRVoip2 (siuInst_t *inst, bool reset, tuint trans_id, 
                           tuint priority, tuint type) 
{
  mgb_t send_message;   
  word packet[rcu_RTCP_PKT_XR_VOIP2_L_W];
  tulong temp32;
  struct MGB_DMS_RTCP_XRVOIP_STATS2 *body = &send_message.msg_body.rtcp_xrvoip_stats2;

  /* Check whether active protocol is RTP and H248 is enabled */
  if ((inst->protocol_type != MGB_PT_RTP) || 
      (VQM_GET_XR_TYPE(inst->heapInst->stats.vqmStats) != VQM_TYPE_H248XNQ)) {
    siu_get_stats_unavail ( inst, reset, trans_id, priority );
    return;
  }    

  /* Get the appropriate packet (no calltable needed; inside ifdef) */
  rcuRtcp_getXRVoip2 (inst->net.rcuInst, packet, type, FALSE);

  body->type                      = type;
  body->max_differential_pdv      = pktRead16bits (packet, rcu_RTCP_H248XNQ_VMAXDIFF_OFF);
  body->pdv_range                 = pktRead16bits (packet, rcu_RTCP_H248XNQ_VRANGE_OFF);
  temp32                          = pktRead32bits (packet, rcu_RTCP_H248XNQ_VSUM_OFF);
  body->differential_pdv_sum_MSW  = siu_get_umsw(temp32);
  body->differential_pdv_sum_LSW  = siu_get_ulsw(temp32);
  body->meas_cycles               = pktRead16bits (packet, rcu_RTCP_H248XNQ_C_OFF);
  body->jitter_buffer_events      = pktRead16bits (packet, rcu_RTCP_H248XNQ_JBEVENTS_OFF);
  temp32                          = pktRead32bits (packet, rcu_RTCP_H248XNQ_TDEGNET_OFF);
  body->net_degrade_period_MSW    = siu_get_umsw(temp32);
  body->net_degrade_period_LSW    = siu_get_ulsw(temp32);
  temp32                          = pktRead32bits (packet, rcu_RTCP_H248XNQ_TDEGJIT_OFF);
  body->jitter_degrade_period_MSW = siu_get_umsw(temp32);
  body->jitter_degrade_period_LSW = siu_get_ulsw(temp32);
  temp32                          = pktRead32bits (packet, rcu_RTCP_H248XNQ_TDEGES_OFF);
  body->affected_secs_MSW         = siu_get_umsw(temp32);
  body->affected_secs_LSW         = siu_get_ulsw(temp32);
  temp32                          = pktRead32bits (packet, rcu_RTCP_H248XNQ_TDEGSES_OFF);
  body->severe_affected_secs_MSW  = siu_get_umsw(temp32);
  body->severe_affected_secs_LSW  = siu_get_ulsw(temp32);
  
  body->rtt_cur                   = inst->heapInst->stats.vqmStats->u.h248xnq.rtdnow;
  body->rtt_min                   = inst->heapInst->stats.vqmStats->u.h248xnq.rtdmin;
  body->rtt_max                   = inst->heapInst->stats.vqmStats->u.h248xnq.rtdmax;

  /* do the reset if needed */
  if (reset) {
    rcuRtcp_getXRVoip2 (inst->net.rcuInst, NULL, type, TRUE);
  }

  send_message.header.length      = MGB_HEADER_SIZE + 
    (sizeof(struct MGB_DMS_RTCP_XRVOIP_STATS2)) * MGB_MSGPAR_SIZE;
  send_message.header.chnum       = (int)(inst->ID & SIU_CID_MASK);
  send_message.header.msg_id      = MGB_DM_RTCP_XRVOIP2_STAT;
  send_message.header.trans_id    = trans_id;   

  /* send message to FIFO */
  siuNduSendMsgPriority(&send_message,(NDU_DPRMsgType_t)priority); 
  
} /* siuRtcpReportXRVoip2 */

/******************************************************************************
 * FUNCTION PURPOSE: Report Local XR VOIP statistics
 ******************************************************************************
 * DESCRIPTION: Reports locally generated XR VOIP statistics 
 *****************************************************************************/

void siuRtcpGetXRVoip2Loc (void *siuInst, bool reset, tuint trans_id, tuint priority) 
{
  siuInst_t *inst = (siuInst_t *)siuInst;

  siuRtcpReportXRVoip2 (inst, reset, trans_id, priority, MGB_RTCP_STATS_LOCAL);
} /* siuRtcpGetXRVoip2Loc */

/******************************************************************************
 * FUNCTION PURPOSE: Report remotely received XR VOIP message
 ******************************************************************************
 * DESCRIPTION:  
 *****************************************************************************/

void siuRtcpGetXRVoip2Rem (void *siuInst, bool reset, tuint trans_id, tuint priority) 
{
  siuInst_t *inst = (siuInst_t *)siuInst;

  siuRtcpReportXRVoip2 (inst, reset, trans_id, priority, MGB_RTCP_STATS_REMOTE);
} /* siuRtcpGetXRVoip2Rem */
#endif

/******************************************************************************
 * FUNCTION PURPOSE: Configure RTCP QA operation
 ******************************************************************************
 * DESCRIPTION: Process the QA Config message. 
 *****************************************************************************/
void siu_rtcp_qa_config (siuInst_t *inst, mgb_t *msg) 
{
  rcuCtrl_t         rcuCtrl;
  
  if (SIU_IS_PROTO_RTP(inst))
  {
    rcuCtrl.code = rcu_CTRL_QA_CONFIG;
    rcuCtrl.u.qaCfg = &msg->msg_body.qa_config;

    if (!gg_rcu_ctable.rcuControl (inst->net.rcuInst, &rcuCtrl)) {
        siuErrorIndication (inst->ID, msg->header.trans_id, 
                            MGB_ERR_INVALID_QA_CONFIG );
    }                        
  }
  
  /* Ignore the message for non-RTP channel */
   

} /* siu_rtcp_qa_config */

/******************************************************************************
 * FUNCTION PURPOSE: Reset RTCP QA state machine
 ******************************************************************************
 * DESCRIPTION: Reset the QA state machine. 
 *****************************************************************************/
void siu_rtcp_qa_reset (void *siuInst, tuint trans_id) 
{
  siuInst_t *inst = (siuInst_t *)siuInst;
  rcuCtrl_t         rcuCtrl;
  
  if (SIU_IS_PROTO_RTP(inst))
  {
    rcuCtrl.code = rcu_CTRL_QA_RESET;
    gg_rcu_ctable.rcuControl (inst->net.rcuInst, &rcuCtrl);
  }
  
  /* Ignore the message for non-RTP channel */

} /* siu_rtcp_qa_reset */



/******************************************************************************
 * FUNCTION PURPOSE: RTCP Status Report
 ******************************************************************************
 * DESCRIPTION: Prepares and sends RTCP status report message. 
 *****************************************************************************/
void siu_rtcp_status_rpt (void *targetInst, tulong SSRC, tulong CNAME_hash,
                          tuint status_code, tuint str_len, word *CNAME_string,
                          tuint skip, tuint report_code) 
{
  mgb_t send_message;
  siuInst_t *inst = (siuInst_t *)targetInst;
  tuint segnum, tmp_str_len, loc_skip, str_skip;
  bool  str_done; 

  str_skip = skip;

  memset (&send_message, 0, sizeof(mgb_t));
 
  send_message.header.chnum    = (int)(inst->ID & SIU_CID_MASK);
  send_message.header.msg_id   = MGB_DM_RTCP_STATUS_REPORT;
  send_message.header.trans_id = 0; /* not a response to a message */

  send_message.msg_body.rtcp_status_report.statusCode = status_code;
  
  if ((status_code & (MGB_RTCP_STS_USERJOIN | MGB_RTCP_STS_USERLEFT | 
                      MGB_RTCP_STS_SSRCCLSION)) &
      (report_code & (MGB_RTCP_RPT_USERJOIN | MGB_RTCP_RPT_USERLEFT |
                      MGB_RTCP_RPT_SSRCCLSION)))
  {
    send_message.msg_body.rtcp_status_report.user.event.SSRC_MSW =
                              siu_get_umsw( SSRC );
    send_message.msg_body.rtcp_status_report.user.event.SSRC_LSW =
                              siu_get_ulsw( SSRC );

    send_message.msg_body.rtcp_status_report.user.event.stringLen = str_len;

    send_message.msg_body.rtcp_status_report.user.event.CNAMEhash_MSW = 
                              siu_get_umsw( CNAME_hash );
    send_message.msg_body.rtcp_status_report.user.event.CNAMEhash_LSW = 
                              siu_get_ulsw( CNAME_hash );

    /* send more than one message if necessary */
    segnum = 1; loc_skip = 0;
    
    do {
      if (str_len > SIU_RTCP_STAT_RPT_MAX_STR_LEN) {
        tmp_str_len = SIU_RTCP_STAT_RPT_MAX_STR_LEN;
        str_len    -= SIU_RTCP_STAT_RPT_MAX_STR_LEN;
        str_done    = FALSE;
      }
      else {
        tmp_str_len = str_len;
        str_done    = TRUE;
      }

      send_message.msg_body.rtcp_status_report.user.event.segnum = segnum++;
 
      utlRepackWordsIntoWords(
                       CNAME_string, 
                       (word *)send_message.msg_body.rtcp_status_report.user.event.string,
                       loc_skip + str_skip,
                       tmp_str_len,
                       0);
      loc_skip += tmp_str_len; str_skip = 0;

      send_message.header.length = MGB_HEADER_SIZE + 7 + ((tmp_str_len + 1)/2);

      /* send message to FIFO */
      siuNduSendMsg(&send_message); 

    } while ( !str_done );
    return;
  }
  else if ((status_code & MGB_RTCP_STS_USERTIMEOUT) &
           (report_code & MGB_RTCP_RPT_USERTIMEOUT))
  {
    send_message.msg_body.rtcp_status_report.user.timeout.SSRC_MSW =
                              siu_get_umsw( SSRC );
    send_message.msg_body.rtcp_status_report.user.timeout.SSRC_LSW =
                              siu_get_ulsw( SSRC );

    send_message.msg_body.rtcp_status_report.user.timeout.CNAMEhash_MSW = 
                              siu_get_umsw( CNAME_hash );
    send_message.msg_body.rtcp_status_report.user.timeout.CNAMEhash_LSW = 
                              siu_get_ulsw( CNAME_hash );

    send_message.header.length = MGB_HEADER_SIZE + 5;
    
  }
  else if ((status_code & MGB_RTCP_STS_SESSIONTIMEOUT) &
           (report_code & MGB_RTCP_RPT_SESSIONTIMEOUT))
  {
    send_message.header.length = MGB_HEADER_SIZE + 1;
  }
  else {
   report_code = 0;
  }
  
  /* send message to FIFO only if required */
  if (report_code)
    siuNduSendMsg(&send_message); 
 
} /* siu_rtcp_status_rpt */

/******************************************************************************
 * FUNCTION PURPOSE: Send acknowledgement when a packet received from host is
 *                   sent to network
 ******************************************************************************
 * DESCRIPTION: Sends GENERAL_ACK message when host requested packet is sent 
 *              to network
 *****************************************************************************/
void siu_rtcp_send_pkt_ack (void *siuInst, tuint trans_id, tuint stm) 
{
  siuInst_t *inst = (siuInst_t *)siuInst;
  tuint origChNum;

  origChNum = inst->ID & SIU_CID_MASK;

  siuGeneralAck (MGB_MD_RTCP_SEND_PACKET, origChNum, trans_id, 
                 MGB_ACK_TYPE_DONE, stm, SIU_DPR_NORMAL_MSG);
} /* siu_rtcp_send_pkt_ack */

/******************************************************************************
 * FUNCTION PURPOSE: Reset user timeout in RTCP.
 ******************************************************************************
 * DESCRIPTION: Resets user timeout counter in RTCP.
 *****************************************************************************/
void siuRtcpRstTimeout (void *siuInst, tuint trans_id) 
{
  siuInst_t *inst = (siuInst_t *)siuInst;
  rcuRtcpCfg_t rtcpCfg;
  
  /* Check whether active protocol is RTP */
  if(SIU_IS_PROTO_RTP(inst) ||
     (SIU_IS_PROTO_IU(inst) && SIU_IS_IU_OVER_IP(inst))){
    /* Set bitmap for session timeout reset */
    rtcpCfg.bitmap = rcu_RTCP_CFG_RST_SSN_TO;
     
    (*gg_rcuRtcp_ctable.rtcpConfig) (inst->net.rcuInst, &rtcpCfg);
  }
} /* siuRtcpRstTimeout */

/******************************************************************************
 * FUNCTION PURPOSE: Routes packets to the micro 
 ******************************************************************************
 * DESCRIPTION: 
 *
 * tint siu_rtcp_to_host (
 *    void  *siuInst,    - a pointer to instance structure
 *    tuint packet_type  - XR RX, XR TX or APP
 *    tuint packetLength - length of packet in bytes 
 *    void  *pkt)        - Pointer to packet 
 *
 *****************************************************************************/
tint siu_rtcp_to_host (void *siuInst, tuint packetType, tuint packetLength, 
                       void *pkt)
{
  siuInst_t *inst = (siuInst_t *)siuInst;
  tuint length_bytes, length_words;
  mgb_t msg; 

  if (packetLength == 0) return(0);

  memset (&msg, 0, sizeof(mgb_t));

  if (packetType == rcu_RTCP_XR_VOIP_LOC) 
    packetType = MGB_RECEIVE_RTCPXR_TX;
  else if (packetType == rcu_RTCP_XR_VOIP_REM)
    packetType = MGB_RECEIVE_RTCPXR_RX;
  else if (packetType == rcu_RTCP_REPORT_APP_PKT)
    packetType = MGB_RECEIVE_RTCPAPP_PKT;
  else if (packetType == rcu_RTCP_REPORT_RTPFB_PKT)
    packetType = MGB_RECEIVE_RTCPRTPFB_PKT;
  else if (packetType == rcu_RTCP_REPORT_PSFB_PKT)
    packetType = MGB_RECEIVE_RTCPPSFB_PKT;
  else if (packetType == rcu_RTCP_REPORT_SR_RR_RX)
    packetType = MGB_RECEIVE_RTCP_SR_RR_RX;
  else if (packetType == rcu_RTCP_REPORT_SR_RR_TX)
    packetType = MGB_RECEIVE_RTCP_SR_RR_TX;
  else if (packetType == rcu_RTCP_XR_PIQUA_RX)
    packetType = MGB_RECEIVE_RTCPXR_PIQUA_RX;
  else
    return (0);

  msg.msg_body.receive_packet.layer = MGB_MDS_LAYER_RTCP;

  msg.msg_body.receive_packet.hdr.proto.rtcp.packetLength = packetLength;
  msg.msg_body.receive_packet.hdr.proto.rtcp.packetType   = packetType;  

  length_bytes = msg.msg_body.receive_packet.hdr.proto.rtcp.packetLength; 
  length_words = UTL_BYTES_TO_WORDS(length_bytes);
  
  /* Protect from overflow */
  if (length_words > sizeof(msg.msg_body.receive_packet.payload))
    length_words = sizeof(msg.msg_body.receive_packet.payload);
    
  /* copy the message body into the local variable */
  memcpy ((void *)&msg.msg_body.receive_packet.payload, pkt, length_words);
  
  msg.header.length   = MGB_HEADER_SIZE + 
                        (length_words+sizeof(dprword)-1)/sizeof(dprword) +
                        (sizeof(MGB_DMS_RECEIVE_PACKET_HEADER)+sizeof(dprword)-1)/sizeof(dprword) + 1; 
  msg.header.chnum    = inst->ID & SIU_CID_MASK;
  msg.header.msg_id   = MGB_DM_RECEIVE_PACKET;
  msg.header.trans_id = 0;
  
  return (siuNduSendMsg((void *)&msg));
} /* siu_rtcp_to_host */

/******************************************************************************
 * FUNCTION PURPOSE: RTCP QA Report
 ******************************************************************************
 * DESCRIPTION: Prepares and sends RTCP QA report message. 
 *****************************************************************************/
void siu_rtcp_qa_rpt (void *targetInst, rcuRtcpQaReport_t *report)
{
  mgb_t send_message;
  siuInst_t *inst = (siuInst_t *)targetInst;

  memset (&send_message, 0, sizeof(mgb_t));
 
  send_message.header.chnum    = (int)(inst->ID & SIU_CID_MASK);
  send_message.header.msg_id   = MGB_DM_QA_REPORT;
  send_message.header.length   = MGB_HEADER_SIZE + 3;
  send_message.header.trans_id = 0; /* not a response to a message */

  send_message.msg_body.qa_report = *report;
  
  /* send message to FIFO */
  siuNduSendMsg((void *)&send_message);   
 
} /* siu_rtcp_qa_rpt */


/* Nothing past this point */
