/******************************************************************************
 * FILE PURPOSE: Contains functions necessary to integrate RTP protocol
 ******************************************************************************
 * FILE NAME: siurtp.c
 *
 * DESCRIPTION: Contains functions necessary to integrate RTP protocol into
 *              system, such that it can be call tabled out of the system
 *
 * (C) Copyright 2001, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#ifdef RTP /* Allow #ifdef in siuloc.h w/o error */
 
/* System utility files */
#include "types.h"
#include "mgmicdsp.h"
#include "ggcfg.h"
#include "vctypes.h"

#include "globstats.h"

/* SIU files */
#include "siuloc.h"
#ifndef HDCT
#include "common/siuctbl.h"     /* SIU internal call table */
#endif
#include "siurtcp.h"
 
/* Module files */
#include <ti/mas/rcu/rcu.h>
#include "siundu.h"
#ifndef HDVIDEO
#include <ti/mas/fiu/fiu.h>
#endif

/******************************************************************************
 * FUNCTION PURPOSE: Create RTP Common Unit (RCU) module 
 ******************************************************************************
 * DESCRIPTION: Creates an instance of RCU. Tables of locations and sizes for
 *              RCU memory buffers with special requirements are utilzed. 
 *
 *  siu_new_rcu (
 *    tint  chnum)  - channel number
 *
 *****************************************************************************/
void siu_new_rcu (tint chnum)
{
  tint         stat;
  memBuffer_t  rcuBufs[SIU_MAX_NBUFS];
  rcuSizeCfg_t rcuCfgSize;
  rcuNewCfg_t  rcuCfgNew;
  siuInst_t    *inst;
  
  /* initialize the siu instance */
  inst = &siuInst[chnum-SIU_FIRSTCH];
  
  /* configure the instance */
  rcuCfgSize.rtcpCallTable = rcuRtcpCallTable;

  rcuCfgNew.ID          = siuMakeID (SIU_MID_RCU, chnum);
  rcuCfgNew.sizeCfg     = rcuCfgSize;
  
  stat = siuNewEco (inst, (siuEcoGetSizes_t)gg_rcu_ctable.rcuGetSizes, &rcuCfgSize, 
                    (siuEcoNew_t)gg_rcu_ctable.rcuNew, &rcuCfgNew, 
                    &(inst->net.rcuInst),  rcuBufs, SIU_MID_RCU);
  
  siu_exc_assert (stat == SIU_NOERR, SIURTP_EXC_NEW_RCU, inst);
} /* siu_new_rcu */ 

/******************************************************************************
 * FUNCTION PURPOSE:  Open RCU module
 ******************************************************************************
 * DESCRIPTION: Opens one instance of a RCU module
 *
 * siu_open_rcu (
 *   tint *chnum)         - instance pointer 
 *
 *****************************************************************************/
void siu_open_rcu (tint chnum)
{
  rcuCfg_t rcuCfg;
  siuInst_t *inst;
  siuInstRTP_t *p_rtp_params;

  /* initialize the siu instance */
  inst = &siuInst[chnum-SIU_FIRSTCH];
#ifndef HDVIDEO
  /* Initialize the RTP parameters pointer based on the Encapsulation type */
  /* RTP or IUUP over IP */
  if(SIU_IS_PROTO_RTP(inst)){
    p_rtp_params = &inst->proto.u.rtp;
  } 
  else  {
    /* IUUP over IP */
    /* Note: This function will only be called for IUUP over IP (not ATM)
     *       and the GMP buffer should be allocated already. 
     *       Therefore, it is not necessary to perform additional checks.
     */
    p_rtp_params = inst->proto.u.iu.p_rtp_params;
  }
#else
  if(SIU_IS_PROTO_RTP(inst)){
    p_rtp_params = &inst->rtp;
  } 
#endif
  /* default configuration of RCU is open receive out */
  rcuCfg.rcuRxOut.rcuReceiveOut.rcuReceiveOut = NULL;
  rcuCfg.rcuRxOut.rcuReceiveOut.targetInst    = NULL; 
  /* 
   * newApi is set to TRUE so that P2P packets such as CAS Delay can be handled 
   * at Idle state
   */
  rcuCfg.rcuRxOut.newApi                      = TRUE;

  if (gg_neu_ctable.neuSendIn) {
    rcuCfg.rcuSendOut.rcuSendOut           = gg_neu_ctable.neuSendIn;
    rcuCfg.rcuSendOut.targetInst           = inst->neuInst;
  }
  else {
    rcuCfg.rcuSendOut.rcuSendOut           = siuNduSendPkt;
    rcuCfg.rcuSendOut.targetInst           = inst;
  } 

  /* RTCP related definitions */
  rcuCfg.rtcpCallTable            = rcuRtcpCallTable;
//  rcuCfg.rtcpReportOut.fcn        = gg_siu_ctable.Rtcp.status_rpt;
//  rcuCfg.rtcpReportOut.targetInst = inst;

  rcuCfg.sysInst                  = inst;
  rcuCfg.rcuCasOutInst            = NULL;
  
  /* Pass msuInst pointer to the rcu config */
  rcuCfg.rcuMsuRtpInst  = &inst->msuRtpInst;   
  rcuCfg.rcuMsuRtcpInst = &inst->msuRtcpInst;   

  
  /* Start off with RCU control of timestamp */
  rcuCfg.rcuTxParms.tsIncrPerTick = 5*8;

  /* TBR: Should the following parameters be initialized during rcuOpen
   *      It has not been configured yet.
   *      should the default values be used here????
   */      
  rcuCfg.rcuTxParmsF.ssrc    = p_rtp_params->TxSyncSrc;
  rcuCfg.rcuTxParms.numCsrc = p_rtp_params->HeaderFirstOct;
  rcuCfg.rcuTxParmsF.ts      = p_rtp_params->TxInitTimeStamp;

  rcuCfg.rcuTxParmsF.seqn    = 0;
  
  rcuCfg.rcuRxParms.ssrc              = p_rtp_params->RxSyncSrc;
  rcuCfg.rcuRxParms.RxSSRCControl     = (rcuSSRCControl_t)p_rtp_params->RxSSRCControl;
  rcuCfg.rcuRxParms.rcuCasOutInst     = NULL;
  rcuCfg.rcuRxParms.rcuHostPktMinTime = 0;
  rcuCfg.rcuRxParms.rcuPktViolMinTime = 1;
  rcuCfg.rcuRxParms.rcuPktViolBurstCfg = 1;

  rcuCfg.stats = &inst->heapInst->stats.pktStats.proto.rtp;

  /* open rcu channel */
  if ((*gg_rcu_ctable.rcuOpen) (inst->net.rcuInst,&rcuCfg) != rcu_NOERR)
     siu_exc_assert (0, SIURTP_EXC_OPEN_RCU, inst);
} /* siu_open_rcu */ 

/******************************************************************************
 * FUNCTION PURPOSE: close an instance of RCU module                     
 ******************************************************************************
 * DESCRIPTION: 
 *                         
 *****************************************************************************/
void siu_close_rcu (siuInst_t *inst)
{  
  (*gg_rcu_ctable.rcuClose) (inst->net.rcuInst); 
} /* siu_close_rcu */


/******************************************************************************
 * FUNCTION PURPOSE: Deletes a RCU channel                     
 ******************************************************************************
 * DESCRIPTION: This routine deletes all modules for RCU operation. 
 *
 *                         
 *****************************************************************************/
void siu_delete_rcu (siuInst_t *inst)
{
  (*gg_rcu_ctable.rcuDelete) (&inst->net.rcuInst);
  inst->protocol_type = MGB_PT_NONE;
} /* siu_delete_rcu */

/******************************************************************************
 * FUNCTION PURPOSE:  Message processing function:
 *                    Modification of encapsulation parameters TX SSRC,
 *                    TX initial timestamp, RTP TX first byte (ver., CC, X),
 *                    RX SSRC, RX initial timestamp and RX SSRC incoming packet
 *                    packet gating.
 ******************************************************************************
 * DESCRIPTION: Each of above mentioned parameters may be modified or not 
 *              according to bit in valid_params bit field, which indicates              
 *              fields that will be modified in this transaction.
 *              TX SSRC and RX SSRC and RX SSRC CTRL are used by main (voice)
 *              application. 
 *****************************************************************************/
void siu_rtp_encap_config (siuInst_t *inst, struct MGB_MDS_ENCAP_RTP *parms) {   
siuInstRTP_t *p_rtp_params;
#if (defined (VOICE) || defined (VOICE_FAX))
pvpControl_t pvpCtrl;
#endif
rcuCtrl_t rcuCtrlTx, rcuCtrlRx;
tuint bitmap;

  SIU_STATE_BEGIN();
#ifndef HDVIDEO
  /* Initialize the RTP parameters pointer based on the Encapsulation type */
  /* RTP or IUUP over IP */
  if(SIU_IS_PROTO_RTP(inst)){
    p_rtp_params = &inst->proto.u.rtp;
  }
  else  {
    /* IUUP over IP */
    /* Note: This function will only be called for IUUP over IP (not ATM)
     *       and the GMP buffer should be allocated already. 
     *       Therefore, it is not necessary to perform additional checks.
     */
    p_rtp_params = inst->proto.u.iu.p_rtp_params;
  }
#else
  if(SIU_IS_PROTO_RTP(inst)){
    p_rtp_params = &inst->rtp;
  } 
#endif
  /* Always set some RCU fields */
  rcuCtrlTx.u.tx.bitmap = 0;
  rcuCtrlTx.code        = rcu_CTRL_TX;
  rcuCtrlRx.u.rx.bitmap = 0;
  rcuCtrlRx.code        = rcu_CTRL_RX;   

  bitmap = parms->valid_params;
  /*-----------------------------------------------------------------------*/
  /* Change channelwise network parameters (this operation must be atomic) */
  /*-----------------------------------------------------------------------*/   

  /* Is Tx SyncSrc modified ? */
  if(bitmap & MGB_ENCAP_CONFIG_VALID_RTP_TX_SSRC) {
     p_rtp_params->TxSyncSrc = 
       siu_conv_2_tulong(parms->tx_ssrc_lsw,
                         parms->tx_ssrc_msw);       
     rcuCtrlTx.u.tx.pF.ssrc = p_rtp_params->TxSyncSrc;
     rcuCtrlTx.u.tx.bitmap |= rcu_CTRL_TX_SSRC;        
  }
  /* Is Tx Start Timestamp modified ? */     
  if(bitmap & MGB_ENCAP_CONFIG_VALID_RTP_TX_TIMESTAMP) {
     p_rtp_params->TxInitTimeStamp = 
       siu_conv_2_tulong(parms->tx_start_timestamp_lsw,
                         parms->tx_start_timestamp_msw);
     rcuCtrlTx.u.tx.pF.ts = p_rtp_params->TxInitTimeStamp;
     rcuCtrlTx.u.tx.bitmap |= rcu_CTRL_TX_TS;
#ifdef HDVIDEO
    if(inst->stateInst.video.vppuInst) 
    { /* Set the initial RTP timestamp */
      vppuCtrl_t vppuCtrl;
       vppuCtrl.code = VPPU_SET_RTP_BASE_TS;
       vppuCtrl.u.rtpBaseTs = p_rtp_params->TxInitTimeStamp;
       vppuControl(inst->stateInst.video.vppuInst, &vppuCtrl);
    }
#endif
  }
  /* Is First Byte modified ? */
  if(bitmap & MGB_ENCAP_CONFIG_VALID_RTP_TX_FIRST_OCTET) {
     p_rtp_params->HeaderFirstOct =
       parms->tx_first_octet;   
     rcuCtrlTx.u.tx.p.numCsrc = p_rtp_params->HeaderFirstOct & 0xF;
     rcuCtrlTx.u.tx.bitmap |= rcu_CTRL_TX_NUMCSRC;        
  }   
  /* Is Rx SyncSrc modified ? */
  if(bitmap & MGB_ENCAP_CONFIG_VALID_RTP_RX_SSRC) {
     p_rtp_params->RxSyncSrc = 
       siu_conv_2_tulong ( parms->rx_ssrc_lsw,
                           parms->rx_ssrc_msw );  
     /* Pass info to RCU Config */
     rcuCtrlRx.u.rx.p.ssrc  = p_rtp_params->RxSyncSrc;
     rcuCtrlRx.u.rx.bitmap |= rcu_CTRL_RX_SSRC;        
  }
  /* Is Rx Start Timestamp modified ? */
  if(bitmap & MGB_ENCAP_CONFIG_VALID_RTP_RX_TIMESTAMP) {
     p_rtp_params->RxInitTimeStamp =        
       siu_conv_2_tulong(parms->rx_start_timestamp_lsw,
                         parms->rx_start_timestamp_msw);
  }   
  /* Is RxSSRC control modified ? */
  if(bitmap & MGB_ENCAP_CONFIG_VALID_RTP_RX_SSRC_CTRL) {
     p_rtp_params->RxSSRCControl = 
       (RtpSSRCControl_t)parms->rx_ssrc_ctrl;   
     /* Pass info to RCU Config */
     rcuCtrlRx.u.rx.p.RxSSRCControl = (rcuSSRCControl_t)p_rtp_params->RxSSRCControl;
     rcuCtrlRx.u.rx.bitmap         |= rcu_CTRL_RX_SSRCACC;       
  }
  /* Is Tx Start SEQuence Number is non Zero ? */     
  if(bitmap & MGB_ENCAP_CONFIG_VALID_RTP_TX_SEQ_NUMBER) {
     rcuCtrlTx.u.tx.pF.seqn  = 
        parms->tx_start_seq_number;
     rcuCtrlTx.u.tx.bitmap |= rcu_CTRL_TX_SEQN;
  }

  /* Set RCU TX */
  if (rcuCtrlTx.u.tx.bitmap)
    gg_rcu_ctable.rcuControl (inst->net.rcuInst, &rcuCtrlTx); 
  /* Set RCU RX */
  if (rcuCtrlRx.u.rx.bitmap)
    gg_rcu_ctable.rcuControl (inst->net.rcuInst, &rcuCtrlRx);

#if (defined (VOICE) || defined (VOICE_FAX))
  /* Send PVP control if PVP instance has been created and if protocol is
   * RTP not Iu/IP. */
  if (inst->pvpInst && inst->protocol_type == MGB_PT_RTP) {
    /* Voice and Transparent: PVP is not "new-ed" in idle or any other mode */
    pvpCtrl.u.rtp.validBitMap = bitmap;
    pvpCtrl.code = PVP_CTRL_PROTOCOL_MODIFY;
    pvpControl (inst->pvpInst, &pvpCtrl);
  }
#endif

  SIU_STATE_END();
}

/******************************************************************************
 * FUNCTION PURPOSE: Inquire the RCU instance from the host
 ******************************************************************************
 * DESCRIPTION: Inquire the RCU instance from the host based on the channel ID
 *
 *  void* siuGetRcuInst (
 *    tuint chan_num)         - channel number
 *
 * Return NULL if channel is not at an valid state (such as VOICE)
 *        neuInst for the specific channel  
 *                         
 *****************************************************************************/
void* siuGetRcuInst (tuint instance_ID)
{
  siuInst_t *inst = SIUINSTp(((instance_ID & SIU_CID_MASK) - SIU_FIRSTCH));
  return(inst->net.rcuInst);

} /* siuGetNetEncapInst */ 
/******************************************************************************
 * FUNCTION PURPOSE:  Report receive end and transmit end statistics  
 ******************************************************************************
 * DESCRIPTION: Report receive end and transmit end statistics for a DSP
 *              channel.
 *
 *****************************************************************************/
void siuRtpGetTxRxStats (void *siuInst, bool reset, tuint trans_id, tuint priority)
{
  siuInst_t *inst = (siuInst_t *)siuInst;
  tulong rxLostPackets = 0;
  mgb_t send_message;
  siuStats_t *siuStats;
  pktStats_t *pktStats;

  siuStats = (siuStats_t *)inst->heapInst;
  pktStats = (pktStats_t *)&siuStats->pktStats;

  send_message.header.length  = MGB_HEADER_SIZE + 
    (sizeof(struct MGB_DMS_GET_RXTX_STAT)) * MGB_MSGPAR_SIZE;
  send_message.header.chnum   = (int)(inst->ID & SIU_CID_MASK);
  send_message.header.msg_id  = MGB_DM_GET_RXTX_STAT;
  send_message.header.trans_id = trans_id;
  
  /* Clear the message, so unsupportated statics get zeroed w/o bloating code */
  memset (&send_message.msg_body.rxtx_stat, 0, sizeof(send_message.msg_body.rxtx_stat));

  SIU_STATE_BEGIN();  
 
  send_message.msg_body.rxtx_stat.RxNetworkPkts_MSW = 
     siu_get_umsw(pktStats->proto.rtp.txrx.rxPkts);
  send_message.msg_body.rxtx_stat.RxNetworkPkts_LSW = 
     siu_get_ulsw(pktStats->proto.rtp.txrx.rxPkts);
  send_message.msg_body.rxtx_stat.TxNetworkPkts_MSW = 
     siu_get_umsw(pktStats->proto.rtp.txrx.txPkts);
  send_message.msg_body.rxtx_stat.TxNetworkPkts_LSW = 
     siu_get_ulsw(pktStats->proto.rtp.txrx.txPkts);

  send_message.msg_body.rxtx_stat.RxOctets_MSW = 
    siu_get_umsw(pktStats->proto.rtp.txrx.rxOctets);
  send_message.msg_body.rxtx_stat.RxOctets_LSW = 
    siu_get_ulsw(pktStats->proto.rtp.txrx.rxOctets);
  send_message.msg_body.rxtx_stat.TxOctets_MSW = 
    siu_get_umsw(pktStats->proto.rtp.txrx.txOctets);
  send_message.msg_body.rxtx_stat.TxOctets_LSW = 
    siu_get_ulsw(pktStats->proto.rtp.txrx.txOctets);
    
  send_message.msg_body.rxtx_stat.pToPFilteredPkts_MSW =
    siu_get_umsw(pktStats->proto.rtp.txrx.rxRedDrops);
  send_message.msg_body.rxtx_stat.pToPFilteredPkts_LSW =
    siu_get_ulsw(pktStats->proto.rtp.txrx.rxRedDrops);        
      
  send_message.msg_body.rxtx_stat.pToPHostPkts_MSW =
        siu_get_umsw(pktStats->proto.rtp.txrx.rxHostPkts);
  send_message.msg_body.rxtx_stat.pToPHostPkts_LSW =
        siu_get_ulsw(pktStats->proto.rtp.txrx.rxHostPkts);
        
  send_message.msg_body.rxtx_stat.pToPSquelchedPkts_MSW =
        siu_get_umsw(pktStats->proto.rtp.txrx.voiceSquelch);
  send_message.msg_body.rxtx_stat.pToPSquelchedPkts_LSW =
        siu_get_ulsw(pktStats->proto.rtp.txrx.voiceSquelch);
        
  send_message.msg_body.rxtx_stat.txLastSequenceNumber = 
        pktStats->proto.rtp.txrx.lastSeqn;
  send_message.msg_body.rxtx_stat.txExtendedSequenceNumber = 
        pktStats->proto.rtp.txrx.extendedSeqn;

  /* Avoid returning negative numbers when sampling with no lost packets  *
   * and lots of out of order jitter                                      */
  if (!(pktStats->proto.rtp.dual.lostPackets > (0UL -32UL))) {
       rxLostPackets += pktStats->proto.rtp.dual.lostPackets;
  }

  if(reset) {
    memset (&pktStats->proto.rtp.txrx, 0, sizeof(pktStats->proto.rtp.txrx));
    memset (&pktStats->proto.rtp.dual, 0, sizeof(pktStats->proto.rtp.dual));
  }
  SIU_STATE_END();

  send_message.msg_body.rxtx_stat.packetsLostNtwk_MSW = (uint)(rxLostPackets >> 16);
  send_message.msg_body.rxtx_stat.packetsLostNtwk_LSW = (uint)(rxLostPackets & 0xFFFF);

  /* send message to FIFO */
  siuNduSendMsgPriority(&send_message,(NDU_DPRMsgType_t)priority); 
} /* siuGetTxRxStats */ 
#else
int siurtp_dummy; /* Shut off the empty file warning */
#endif /* #ifdef RTP */

/* Nothing past this point */
