/******************************************************************************
 * FILE PURPOSE: Some utility functions used by SIU.
 ******************************************************************************
 * FILE NAME:   siuutil.c
 *
 * DESCRIPTION: Utility functions for SIU.
 *  
 *
 * TABS: NONE
 *
 * $Id: siuutil.c 1.3 1998/11/09 16:03:28 BONAL Exp $
 *
 * REVISION HISTORY:
 *
 * $Log: siuutil.c $
 * Revision 1.3  1998/11/09 16:03:28  BONAL
 *    - Removed siu_queue_read() and siu_queue_write() functions.
 * Revision 1.2  1998/11/07 16:14:46  BONAL
 *    - g723loc.h removed. 
 * Revision 1.1  1998/11/04 18:39:41  BONAL
 * Initial revision
 *
 *
 * (C) Copyright 1997, 1998, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include "types.h"

/* SIU header files */
#include "common/siu.h"
#include "siuloc.h"
#include "siuport.h"
/* Low level hardware files */
#include "ggcfg.h"
#include "targethw.h"
#include "hal.h"

#ifdef SIU_SNIFF
siuSniff_t siuSniff[SIUSNIFF_BUFSIZE];
int        siuSniffer_flag = 0;
int        siuSniffCnt     = 0;
#endif


tuint SIU_TIMER_VAL = 0;
tuint SIU_MAX_PCM_ISR_TIME = 0;
tuint SIU_PCM_SKIP_CNTR = 0;

/******************************************************************************
 * FUNCTION PURPOSE: Do nothing function.
 ******************************************************************************
 * DESCRIPTION: Does not nothing at all, just returns! Used to avoid checking
 *              on NULL for Tx_process() and Rx_process() functions.
 *
 *****************************************************************************/

void siuDoNothing ( void *siuInst, tuint chIndx )
{
  return;
} /* siuDoNothing */


/*****************************************************************************
 * FUNCTION PURPOSE: Find buffer alignment
 *****************************************************************************
 * DESCRIPTION: Returns the 1st address in a buffer that meets the required
 *              log2 buffer requirement.
 * void *siu_buffer_align (
 *                          void *bstart  - Start Address of the buffer
 *                          void *bend    - End Address of the buffer
 *                          tint log2a    - Log2 alignment required 
 *                          tuint *rsize  - Pointer size of buffer.The 
 *                                          size of the buffer will be
 *                                          in terms of Minimum Addressable
 *                                          Unit (MAU) of the processor  
 * 
 *****************************************************************************/
void *siu_buffer_align (void *bstart, void *bend, tint log2a, tuint *rsize)
{
  tuint  mask = 0;
  word* bufstart = (word *)bstart;
  word* bufend   = (word *)bend;
  
  /* Form the alignment bit mask */
  mask = (1 << log2a) - 1;
  /* Check buffer alignment */
  while((tulong)bufstart & mask)
    bufstart++;
  
  *rsize = bufend - bufstart;

  return ((void *)bufstart);
} /* siu_buffer_align */

/*****************************************************************************
 * FUNCTION PURPOSE: Find buffer alignment for large buffes
 *****************************************************************************
 * DESCRIPTION: Returns the 1st address in a buffer that meets the required
 *              log2 buffer requirement.
 * void *siu_buffer_align_large (
 *                          void *bstart  - Start Address of the buffer
 *                          void *bend    - End Address of the buffer
 *                          tint log2a    - Log2 alignment required 
 *                          tulong *rsize - Pointer size of buffer.The 
 *                                          size of the buffer will be
 *                                          in terms of Minimum Addressable
 *                                          Unit (MAU) of the processor  
 * 
 *****************************************************************************/
void *siu_buffer_align_large (void *bstart, void *bend, tint log2a, tulong *rsize)
{
  tuint  mask = 0;
  word* bufstart = (word *)bstart;
  word* bufend   = (word *)bend;
  
  /* Form the alignment bit mask */
  mask = (1 << log2a) - 1;
  /* Check buffer alignment */
  while((tulong)bufstart & mask)
    bufstart++;
  
  *rsize = bufend - bufstart;

  return ((void *)bufstart);
} /* siu_buffer_align_large */

/*****************************************************************************
 * FUNCTION PURPOSE: Get silence pattern based on companding law
 *****************************************************************************
 * DESCRIPTION: Returns the silence pattern for configured companding law
 *****************************************************************************/
#if (!defined(HDVIDEO) && !defined(HDCT))
tuint siuGetSilencePattern(siuInst_t *inst)
{
  switch (SIU_READ_COMPANDING_LAW(inst))
  {
  case MGB_ALAW_COMPANDING:
     return((tuint)((tuint)THW_ALAW_SILENCE << 8) | (tuint)THW_ALAW_SILENCE);

  case MGB_MULAW_COMPANDING:
     return((tuint)((tuint)THW_MULAW_SILENCE << 8) | (tuint)THW_MULAW_SILENCE);

  case MGB_LINEAR_COMPANDING:
     return((tuint)THW_LINEAR_SILENCE);
  }

  return 0; /* Shouldn't get here */
}
#endif
/******************************************************************************
 * FUNCTION PURPOSE: Sniffer function for debugging.
 ******************************************************************************
 * DESCRIPTION: 
 *
 *****************************************************************************/
#ifdef SIU_SNIFF
void siuSniffer( void *siu_Inst, siuSniff_t *sniffBuf, tint *sniffCnt, 
                 tint code, tint value, tint section )
{
  siuInst_t *inst = (siuInst_t *) siu_Inst;

//  if (SIUINSTp(1)->state == SIU_CHST_VOICE) 
//    siuSniffer_flag = 1;
//
//  if (siuSniffer_flag == 0) return;

  if (*sniffCnt >= SIUSNIFF_BUFSIZE)
    *sniffCnt = 0;
//    return;

  sniffBuf[*sniffCnt].timestamp = siu_isr_clock;
  sniffBuf[*sniffCnt].code      = code;
  sniffBuf[*sniffCnt].value     = value;
  sniffBuf[*sniffCnt].section   = section;

  (*sniffCnt)++;
}
#endif  

void* siuGetModuleHandle (tuint chId, tuint moduleInstanceOffset)
{ 
  return *( (void**)((char*)(SIUINSTp(chId)) + moduleInstanceOffset) );
}  

/******************************************************************************
 * FUNCTION PURPOSE: To copy a table from shared memory to local memory.
 ******************************************************************************
 * DESCRIPTION: This function moves table from shared memory to the local
 *              memory
 *
 *  void siutbl_copy_from_sharedmem (
 *    tulong *src               ; Source Address in shared memory
 *    tulong *dest              ; Destination Address in local memory
 *    tint size )               ; size in words to be moved
 *
 * Return Value : None.
 ******************************************************************************/
#if (!defined(HDVIDEO) && !defined(HDCT))
void siutbl_copy_from_sharedmem(tulong src,tulong dest,tint size)
{ 
  halGlobAddr_t dest_global, src_global;
  halMemMove_t memMove;

  /* Must be even-aligned source */
  siu_exc_assert ((src & 0x1) == 0, SIUTABRELOC_EXC_UNEVEN, &siuContext);

  src_global = (( halGlobAddr_t )src) << 1; /* This will be a byte address */

  /* Append 0x0F to the MSB of the source byte */
  src_global |= 0x0F000000;

  /* Extract the word address out of the global byte source address */
  src_global >>= 1 ;

  /* Get the destination global address */
  dest_global = (*siuContext.halApi->halMapLocalToGlobal)((halLocAddr_t)dest);

  /* issuing the macros */
  HAL_MEM_ADDRESS_PROG(src_global);
  HAL_MEM_ADDRESS_DATA(dest_global);

  memMove.src  = src_global;
  memMove.dest = dest_global;
  memMove.size = (halSAE_t)size;

  (*siuContext.halApi->halGlobMemMove) (1, &memMove, NULL,NULL, TRUE);

}/* siutbl_copy_from_sharedmem */
#endif


/* Nothing after this point. */
