/******************************************************************************
 * FILE PURPOSE: Contains functions necessary to integrate ATK
 ******************************************************************************
 * FILE NAME: siuatk.c
 *
 * DESCRIPTION: Contains functions necessary to integrate ATK into
 *              system, such that it can be call tabled out of the system
 *
 * (C) Copyright 2008, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
 
/* System utility files */
#include "types.h"
#include "mgmicdsp.h"
#include "ggcfg.h"

/* SIU files */
#include "common/siuloc.h"
#include "ividmc/siuVidMc.h"
#include "vctypes.h"

/* Module files */
#include "atk.h"
#include "util.h"
#include "utlrepck.h" /* for utlPackTuintToTulong */
#include "ggmbuf.h"

/******************************************************************************
 * FUNCTION PURPOSE: Create Video Protocol Processing Unit (ATK) module 
 ******************************************************************************
 * DESCRIPTION: 
 *
 *  siu_new_atk (
 *    tint  chnum)  - channel number
 *
 *****************************************************************************/
void siu_new_atk (tint chnum) {
  tint         stat;
  memBuffer_t  atkBufs[SIU_MAX_NBUFS];
  atkNewCfg_t atkCfgNew;
  atkSizeCfg_t atkCfgSize;
  siuInst_t    *inst;
  
  /* initialize the siu instance */
  inst = &siuInst[chnum-SIU_FIRSTCH];
  
  /* configure the instance */
  atkCfgNew.ID             = siuMakeID (SIU_MID_ATK, chnum);
  
  stat = siuNewEco (inst, (siuEcoGetSizes_t)atkGetSizes, &atkCfgSize, 
                    (siuEcoNew_t)atkNew, &atkCfgNew, 
                    &(inst->stateInst.video.atkInst),  atkBufs, SIU_MID_ATK);
  
  siu_exc_assert (stat == SIU_NOERR, SIUATK_EXC_NEW_ATK, inst);
} /* siu_new_atk */ 

/******************************************************************************
 * FUNCTION PURPOSE:  Open ATK module
 ******************************************************************************
 * DESCRIPTION: Opens one instance of a ATK module
 *
 * siu_open_atk (
 *   tint chnum            - Channel number
 *   tint codingType       - Coding Type
 *   )
 *
 *****************************************************************************/
void siu_open_atk (tint chnum, tuint codingType) {
  atkCfg_t atkCfg;
  siuInst_t *inst;
  tint      retval = ATK_NOERR;

  /* initialize the siu instance */
  inst = &siuInst[chnum-SIU_FIRSTCH];
  
  /* default configuration of ATK is open receive out */
  memset(&atkCfg, 0, sizeof(atkCfg_t));

  switch(codingType) {
    case EVCT_AUDIO_MP3_DEC:
      atkCfg.codecName       = "MP3DEC";
      break;
    case EVCT_AUDIO_AAC_ENC:
      atkCfg.codecName       = "AACLEENC";
      break;
    default:
      siu_exc_assert(0,SIUATK_EXC_OPEN_ATK,inst);
  }

  atkCfg.sysInst         = (void *)inst;

  /* open atk channel */
  retval = atkOpen (inst->stateInst.video.atkInst,&atkCfg);
  siu_exc_assert ( (retval == ATK_NOERR), SIUATK_EXC_OPEN_ATK, inst);
} /* siu_open_atk */

/******************************************************************************
 * FUNCTION PURPOSE:  siu_commit_atk
 ******************************************************************************
 * DESCRIPTION: Commits one instance of a ATK module
 *
 * siu_commit_atk (
 *   siuInst_t     *inst         - instance pointer 
 *   siuCodecCfg_t *siuCodecCfg  - retrieve codec configuration
 *                )
 *****************************************************************************/
void siu_commit_atk (siuInst_t *inst) {
  atkCtrl_t ctrl;

  /* commit ATK and get codec configuration from ATK */
  ctrl.code  = ATK_COMMIT_CODEC;
  siu_exc_assert ((atkControl(inst->stateInst.video.atkInst, &ctrl) == ATK_NOERR), 
                   SIUATK_EXC_COMMIT_ATK | SIU_EXCEPTION_TYPE_CHANNEL, inst);
  return;

}

/******************************************************************************
 * DESCRIPTION: Get ATK stats 
 *
 * siu_atk_get_stats (
 *  (void *siuInst,          - instance pointer 
 *   bool reset, 
 *   tuint trans_id, 
 *   tuint priority)
 *
 *****************************************************************************/
void siu_atk_get_stats (void *siuInst, bool reset, tuint trans_id, tuint priority) {
  siuInst_t *inst = (siuInst_t *)siuInst;
  mgb_t send_message;
  atkCtrl_t ctrl;
 
#if 0
  send_message.header.length   = MGB_HEADER_SIZE + sizeof(struct ifatk_GET_STAT)
                                               / sizeof(UINT16);
  send_message.header.chnum    = (int)(inst->ID & SIU_CID_MASK);
  send_message.header.msg_id   = MGB_DM_GET_ATK_STAT;
  send_message.header.trans_id = trans_id;

  chType    = (siuVideoChannelOperation_t) SIU_GET_VIDEO_CHANNEL_TYPE(inst);
#endif

  ctrl.code = ATK_GET_CFG;
 
  /* Not using ecoif for stats due to endian translation */
  if (atkControl(inst->stateInst.video.atkInst, &ctrl) == ATK_ERROR) {
    /* codec didn't receive static/dynamic configuration yet: atk not opened */
    siuErrorIndication(inst->ID, trans_id, MGB_ERR_VIDEO_INVD_STATS_REQUEST);
    return;
  }

  /* send message to FIFO */
  siuNduSendMsgPriority(&send_message, (NDU_DPRMsgType_t)priority); 

}

/* Nothing past this point */
