/******************************************************************************
 * FILE PURPOSE: Contains functions necessary to integrate MMCU
 ******************************************************************************
 * FILE NAME: siummcu.c
 *
 * DESCRIPTION: Contains functions necessary to integrate MMCU into
 *              system, such that it can be call tabled out of the system
 *
 * (C) Copyright 2008, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
 
/* System utility files */
#include "types.h"
#include "mgmicdsp.h"
#include "ggcfg.h"
/* SIU files */
#include "common/siuloc.h"
#include "siummcu.h"
#include "siutftp.h"
#include "vctypes.h"
 
/* Module files */
#include <ti/mas/mmcu/mmcu.h>
#include <ti/mas/neu/neu.h>
#include "vtk.h"
#include "util.h"
#include "ggmbuf.h"
#include "utlrepck_m.h" 




/******************************************************************************
 * FUNCTION PURPOSE: Create MultiMedia Container Unit (MMCU) module 
 ******************************************************************************
 * DESCRIPTION: 
 *
 *  siu_new_mmcu (
 *    tint  chnum)  - channel number
 *
 *****************************************************************************/
void siu_new_mmcu (tint chnum)
{
  tint          stat;
  memBuffer_t   mmcuBufs[SIU_MAX_NBUFS];
  mmcuNewCfg_t  mmcuCfgNew;
  mmcuSizeCfg_t mmcuCfgSize; /* Currently not used. */
  siuInst_t     *inst;
  
  /* initialize the siu instance */
  inst = SIUINSTp(chnum-SIU_FIRSTCH);
  
  /* configure the instance */
  mmcuCfgNew.ID             = siuMakeID (SIU_MID_MMCU, chnum);

  stat = siuNewEco (inst, (siuEcoGetSizes_t)mmcuGetSizes, &mmcuCfgSize, 
                    (siuEcoNew_t)mmcuNew, &mmcuCfgNew, 
                    &(inst->stateInst.video.mmcuInst),  mmcuBufs, SIU_MID_MMCU);
  
  siu_exc_assert (stat == SIU_NOERR, SIUMMCU_EXC_NEW_MMCU, inst);
} /* siu_new_mmcu */ 

/******************************************************************************
 * FUNCTION PURPOSE:  Open MMCU module
 ******************************************************************************
 * DESCRIPTION: Opens one instance of a MMCU module
 *
 * siu_open_mmcu (
 *   siuInst_t *inst                 - instance pointer 
 *   )
 *****************************************************************************/
void siu_open_mmcu (siuInst_t *inst)
{
  mmcuCfg_t mmcuCfg;

  /* memset mmcu Config to zero */
  memset(&mmcuCfg,0,sizeof(mmcuCfg_t));

  mmcuCfg.mmcName             = "AVIDEMUX";
  mmcuCfg.dataIO.targetInst   = (void *)inst;
  mmcuCfg.dataIO.start        = siu_start_tftp;
  mmcuCfg.dataIO.req          = siu_get_tftp;
  mmcuCfg.dataIO.stop         = siu_stop_tftp;
  mmcuCfg.maxFifoNodes        = 20;
  mmcuCfg.minFifoNodes        = 10;

  /* open mmcu channel */
  if (mmcuOpen (inst->stateInst.video.mmcuInst,&mmcuCfg) != mmcu_NOERR)
     siu_exc_assert (0, SIUMMCU_EXC_OPEN_MMCU, inst);

} /* siu_open_mmcu */ 


/******************************************************************************
 * FUNCTION PURPOSE:  Calls MMCU module to parse the media container header.
 ******************************************************************************
 * DESCRIPTION: Parses the media file header and selects a stream of interest.
 *
 * siu_open_mmcu (
 *   siuInst_t *inst                 - instance pointer 
 *   )
 *****************************************************************************/
tuint debug_mmcu_mux_insert_pcr = 1;
void siu_mmcu_send_header(siuInst_t *inst)
{
  tsMuxCtrl_t         muxCtrl;
  tint                retval;
  tuint               videoCodingType = SIU_GET_VIDEO_CODING_TYPE(inst);
  tuint               audioCodingType = SIU_GET_AUDIO_CODING_TYPE(inst);

  memset(&muxCtrl,0,sizeof(tsMuxCtrl_t));
  muxCtrl.code = tsMux_CTRL_ADD_PROGRAM;

  muxCtrl.u.newProg.PMT_PID       = SIU_PMT_PID;
  muxCtrl.u.newProg.PCR_PID       = SIU_PCR_PID;
  muxCtrl.u.newProg.PCRInAF_Flag  = debug_mmcu_mux_insert_pcr;
  
  muxCtrl.u.newProg.nPIDs = 1;
  muxCtrl.u.newProg.es[0].PID = SIU_VIDEO_PID;

  //muxCtrl.u.newProg.es[0].StreamType = SIU_VIDEO_STREAM_TYPE;
  switch(videoCodingType) 
  {
    case EVCT_VIDEO_H264BP_ENC:
    case EVCT_VIDEO_H264MP_ENC:
      muxCtrl.u.newProg.es[0].StreamType = ts_STREAM_TYPE_H264_VIDEO;
      break;
    case EVCT_VIDEO_MPEG4SP_ENC:
    case EVCT_VIDEO_H263P_ENC:
      muxCtrl.u.newProg.es[0].StreamType = ts_STREAM_TYPE_MPEG4_VIDEO;
      break;
    default:
      siu_exc_assert(0,SIUMMCU_EXC_MUX_HDR,inst);
  }
    
  muxCtrl.u.newProg.es[1].PID = SIU_AUDIO_PID;
  switch (audioCodingType) {
    case EVCT_AUDIO_AAC_ENC:
      muxCtrl.u.newProg.nPIDs++;
      muxCtrl.u.newProg.es[1].StreamType = SIU_AUDIO_STREAM_TYPE;
      break;
  }

  retval = mmcuControl(inst->stateInst.video.mmcuInst,(mmcuCtrl_t *)&muxCtrl);
  siu_exc_assert((retval == ts_NOERR),SIUMMCU_EXC_MUX_CTRL,inst);

  retval = mmcuSendHeader(inst->stateInst.video.mmcuInst);
  siu_exc_assert((retval == ts_NOERR),SIUMMCU_EXC_MUX_HDR,inst);

  SIU_SET_MMCU_HDR_PARSED(inst);
}
/******************************************************************************
 * FUNCTION PURPOSE:  Calls MMCU module to parse the media container header.
 ******************************************************************************
 * DESCRIPTION: Parses the media file header and selects a stream of interest.
 *
 * siu_open_mmcu (
 *   siuInst_t *inst                 - instance pointer 
 *   )
 *****************************************************************************/
void siu_mmcu_parse_header(siuInst_t *inst)
{
  void          *mmcuInst = inst->stateInst.video.mmcuInst;
  tint          i;
  tsDemuxStreamInfo_t streamInfo;
  tuint PIDs[tsDemux_MAX_TS_PIDS];
  tint pmtPID, nPIDs = 1;

  mmcuGetStreamInfo(mmcuInst, &streamInfo);

  pmtPID = streamInfo.esInfo[0].PMT_PID;

  for ( i=0; i<streamInfo.nStreams; i++) 
  {
    if( streamInfo.esInfo[i].PMT_PID == pmtPID ) 
    {
      /*
      PIDs[i] = streamInfo.esInfo[i].PID;

      muxCtrl.u.newProg.es[muxPIDs].PID           = streamInfo.esInfo[i].PID;
      */
      switch (streamInfo.esInfo[i].StreamType) {
        case ts_STREAM_TYPE_MPEG1_VIDEO: 
        case ts_STREAM_TYPE_MPEG2_VIDEO: 
        case ts_STREAM_TYPE_MPEG4_VIDEO: 
        case ts_STREAM_TYPE_H264_VIDEO:
          //muxCtrl.u.newProg.es[muxPIDs].PID = SIU_VIDEO_PID;
          //muxCtrl.u.newProg.es[muxPIDs].StreamType = streamInfo.esInfo[i].StreamType;
          
          PIDs[0] = streamInfo.esInfo[i].PID;

          break;
        case ts_STREAM_TYPE_MPEG1_AUDIO:
        case ts_STREAM_TYPE_MPEG2_AUDIO:
        case ts_STREAM_TYPE_AAC_AUDIO:
        case ts_STREAM_TYPE_AC3_AUDIO:
          //muxCtrl.u.newProg.es[muxPIDs].PID = SIU_AUDIO_PID;
          //muxCtrl.u.newProg.es[muxPIDs].StreamType = streamInfo.esInfo[i].StreamType;
          if(SIU_GET_AUDIO_CODING_TYPE(inst) != EVCT_AUDIO_NONE) {
            PIDs[1] = streamInfo.esInfo[i].PID;
            nPIDs = 2;
          }
          break;

        default:
          /* ignore rest of stream types (e.g: PCRs etc) */
          break;
      }
    }
  }
  //mmcuSelectStream(mmcuInst,streamInfo.nStreams,PIDs);
  mmcuSelectStream(mmcuInst,nPIDs,PIDs);  
  
  SIU_SET_MMCU_FPS(inst,25);
  SIU_SET_MMCU_HDR_PARSED(inst);
}

mmcMapping_t mmcMuxMapping[] = {
  { MGB_MDS_VIDEO_MMCU_CONTAINER_AVI   ,  "NULL"},
  { MGB_MDS_VIDEO_MMCU_CONTAINER_MOV   ,  "NULL"},
  { MGB_MDS_VIDEO_MMCU_CONTAINER_MPEGTS,  "TSMUX"},
  { -1                                 ,  "NULL"}
};

mmcMapping_t mmcDemuxMapping[] = {
  { MGB_MDS_VIDEO_MMCU_CONTAINER_AVI   ,  "AVIDEMUX"    },
  { MGB_MDS_VIDEO_MMCU_CONTAINER_MOV   ,  "MOVDEMUX"    },
  { MGB_MDS_VIDEO_MMCU_CONTAINER_MPEGTS,  "TSDEMUX" },
  { -1                                 ,  "NULL"        }
};

char *siuMmcuGetMmcName(tuint mmcType, mmcMapping_t *mapPtr)
{
  while((mapPtr->mmcType != mmcType) && (mapPtr->mmcType != -1)) {
    mapPtr++;
  }
  return((char *)mapPtr->mmcName);
}

tulong  debug_mmcu_mux_bitrate = 0;
void siuMmcuCfg(siuInst_t *inst, mgb_t *message)
{
  siuVideoChannelOperation_t chType = (siuVideoChannelOperation_t)SIU_GET_VIDEO_CHANNEL_TYPE(inst);
  void        *mmcuInst = inst->stateInst.video.mmcuInst;
  mmcuCfg_t   mmcuCfg;

  if(message->msg_body.video_mmcu_cfg.validParams & MGB_MDS_VIDEO_MMCU_CFG_VALID_CONTAINER)
  {
    switch(chType) {
      case SIU_CHTYPE_DECODER:
        mmcuCfg.mmcName = siuMmcuGetMmcName(message->msg_body.video_mmcu_cfg.container, mmcDemuxMapping);
        break;
      case SIU_CHTYPE_ENCODER:
        mmcuCfg.mmcName = siuMmcuGetMmcName(message->msg_body.video_mmcu_cfg.container, mmcMuxMapping);
        break;
      default:
        siuErrorIndication (inst->ID, message->header.trans_id, MGB_ERR_VIDEO_INVALID_MMCU_CONFIG);
    }
  }

  mmcuCfg.dataIO.targetInst   = (void *)inst;
  mmcuCfg.dataIO.start        = siu_start_tftp;
  mmcuCfg.dataIO.req          = siu_get_tftp;
  mmcuCfg.dataIO.stop         = siu_stop_tftp;
  mmcuCfg.maxFifoNodes        = 50; 
  mmcuCfg.minFifoNodes        = 40;
  mmcuCfg.mtuSize             = inst->maxPayloadSize;
  mmcuCfg.bitRate             = debug_mmcu_mux_bitrate;
  mmcuCfg.shipoutPeriod       = 90;

  if(SIU_IS_PROTO_TFTP(inst)) {
    mmcuCfg.sendOut.sendOut     = siu_send_tftp;
    mmcuCfg.sendOut.targetInst  = (void *)inst;
  } else if (inst->protocol_type == MGB_PT_NONE) {
    mmcuCfg.sendOut.sendOut     = neuSendIn;
    mmcuCfg.sendOut.targetInst  = inst->neuInst;
  }

  if( mmcuOpen(mmcuInst, &mmcuCfg) != mmcu_NOERR) {
    siuErrorIndication (inst->ID, message->header.trans_id, MGB_ERR_VIDEO_INVALID_MMCU_CONFIG);
  }
  
  SIU_SET_MMCU_CFG_DONE(inst);

  return;
}
/* nothing past this point */
