/******************************************************************************
 * FILE PURPOSE: Contains functions necessary to integrate NEU
 ******************************************************************************
 * FILE NAME: siuneu.c
 *
 * DESCRIPTION: Contains functions necessary to integrate NEU into
 *              system, such that it can be call tabled out of the system
 *
 * (C) Copyright 2001, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
 
/* System utility files */
#include "types.h"
#include "mgmicdsp.h"
#include "ggcfg.h"
#include "ndu.h"
#include "ndugmac.h"
#ifndef HDCT
#include "common/siuctbl.h"
#endif
#include "siuneu.h"

/* SIU files */
#include "common/siuloc.h"
#ifndef HDCT
#include "common/siutmr.h"
#endif
#ifdef GG_INCLUDE_ANNOUNCEMENTS
#include "siuanu.h"
#endif
/* Module files */
#include <ti/mas/neu/neu.h>
#include <ti/mas/neu/aal5.h>
#ifdef GG_INCLUDE_ANNOUNCEMENTS
#include "anu.h"
#endif
#include "common/siundu.h"
#include "common/dsiunwal.h"
#include <ti/mas/neu/neuproto.h>
#include "ggmbuf.h"
#include <ti/mas/util/pkt.h>
#include <ti/mas/iface/ifpkt/xferpkt.h>

#include "diagapp.h"

#ifdef UTOPIA_8BIT
#define SIU_NEU_DUMMY_BIT_OFFSET 1
#else
#define SIU_NEU_DUMMY_BIT_OFFSET 0
#endif 
/******************************************************************************
 * FUNCTION PURPOSE: Create NET ENCAP Unit (NEU) module 
 ******************************************************************************
 * DESCRIPTION: Creates an instance of NEU. Tables of locations and sizes for
 *              NEU memory buffers with special requirements are utilzed. 
 *
 *  siu_new_neu (
 *    tint  chnum)  - channel number
 *
 *****************************************************************************/
void siu_new_neu (tint chnum)
{
  tint         stat;
  memBuffer_t  neuBufs[SIU_MAX_NBUFS];
  neuNewCfg_t  neuCfgNew;
  neuSizeCfg_t neuCfgSize;
  siuInst_t    *inst;
  
  /* initialize the siu instance */
  inst = SIUINSTp(chnum-SIU_FIRSTCH);

  neuCfgSize.txHeaderSize  = UTL_BYTES_TO_WORDS(neuproto_MAX_HEADER_SIZE_BYTES);
  
  /* The max. size = 7 cell + 1 cell for trailer */
  if (ggConfig.neu_sar & GGCFG_NEU_SAR_AAL5) {
    neuCfgSize.rxAssemSize   = (UTL_BYTES_TO_WORDS(neuproto_MAX_AAL5_PAYLOAD_BYTES) *(neuproto_MAX_CELLS+1));
    neuCfgSize.txScratchSize = UTL_BYTES_TO_WORDS(neuproto_MAX_DATA_SIZE_BYTES_POSTAAL5);
  }
  else  {
    neuCfgSize.rxAssemSize   = 0;
    neuCfgSize.txScratchSize = neuCfgSize.txHeaderSize + neuproto_MAX_PAYLOAD_BYTES; //review
  }

  neuCfgSize.sssar_required = (ggConfig.neu_sar & GGCFG_NEU_SAR_SSSAR) ? TRUE : FALSE;

  /* Copy the cell size from the NDU context to the NEU context. */
  neuContext.cellSize = siuContext.cellSize;

  /* configure the instance */
  neuCfgNew.ID          = siuMakeID (SIU_MID_NEU, chnum);
  neuCfgNew.sizeCfg     = neuCfgSize;
  neuCfgNew.chanIndep   = FALSE;

  stat = siuNewEco (inst, (siuEcoGetSizes_t)neuGetSizes, &neuCfgSize, 
                    (siuEcoNew_t)neuNew, &neuCfgNew, 
                    &(inst->neuInst),  neuBufs, SIU_MID_NEU);
  
  siu_exc_assert (stat == SIU_NOERR, SIUNEU_EXC_NEW_NEU, inst);
} /* siu_new_neu */ 

/******************************************************************************
 * FUNCTION PURPOSE: Initialize an neu instance for a non-voice channel.
 ******************************************************************************
 * DESCRIPTION: Performs a non-voice channel eco "new" and then opens the
 *              non-voice channel eco. nvEcoId identifies the non-voice 
 *              "channel".
 ******************************************************************************/
void *siu_new_nv_neu (tint nvEcoId, tint txHdrLenWords)
{
  tint           stat;
  tint           size;
  tint           max_header_size_bytes = neuproto_MAX_HEADER_SIZE_BYTES;
  memBuffer_t    neuBufs[SIU_MAX_NBUFS];
  neuNewCfg_t    neuCfgNew;
  neuSizeCfg_t   neuCfgSize;
  void          *inst = NULL;


    /* The size of the required rx assembly buffer depends on which eco is created */
    switch (nvEcoId)  {

      case GG_NVECO_MSG:

        size = UTL_TINTS_TO_BYTE(MGB_HD_COMPOUND_MSG_MAX_LENGTH)   +           /* message parameters    */
               max_header_size_bytes            +           /* network encapsulation */
               UTL_TINTS_TO_BYTE(SIU_UTOPIA_MAX_MSG_HEADER_LENGTH + 1);        /* DSP message header    */
        break;

  
      case GG_NVECO_BLK:
#ifdef GG_INCLUDE_ANNOUNCEMENTS
        size = UTL_TINTS_TO_BYTE((SIU_ANU_PCM_VIF_30MS / TYP_TINT_SIZE))  +     /* data                  */
               max_header_size_bytes                  +     /* network encapsulation */
               UTL_TINTS_TO_BYTE(ANU_MAX_RQST_HEADER_LENGTH + 1);  /* request header        */
#else
        size = 0;
#endif
        break;


      case GG_NVECO_HLT:

        size = 0;
        break;

      case GG_NVECO_TRACE:
        size = 0;
        break;

      } /* end switch */

   if (ggConfig.neu_sar & GGCFG_NEU_SAR_AAL5)  {
      /* The size must be rounded up to the next aal5 payload size */
       size = ((size + neuproto_MAX_AAL5_PAYLOAD_BYTES - 1) / 
              neuproto_MAX_AAL5_PAYLOAD_BYTES) *
             neuproto_MAX_AAL5_PAYLOAD_BYTES;

       neuCfgSize.txScratchSize = UTL_BYTES_TO_WORDS(((size + neuproto_MAX_AAL5_PAYLOAD_BYTES- 1) /
                                neuproto_MAX_AAL5_PAYLOAD_BYTES) *   
                               siuContext.cellSize);
       neuCfgSize.rxAssemSize = UTL_BYTES_TO_WORDS(size);
                        

	} else {

       neuCfgSize.txScratchSize = UTL_BYTES_TO_WORDS(size);
       neuCfgSize.rxAssemSize = 0; /* Rx Assembly buffer is not used for gmac */
    }
       

  /* The message/anu neu's never use SSSAR */
  neuCfgSize.sssar_required = FALSE;

  neuCfgSize.txHeaderSize  = txHdrLenWords;

  neuCfgNew.ID      = siuMakeNvID (SIU_MID_NEU, nvEcoId);
  neuCfgNew.sizeCfg = neuCfgSize;
  neuCfgNew.chanIndep   = TRUE;

  stat = siuNewEco (NULL, (siuEcoGetSizes_t)neuGetSizes, &neuCfgSize, (siuEcoNew_t)neuNew,
                      &neuCfgNew, &inst, neuBufs, nvEcoId);

  siu_exc_assert (stat == SIU_NOERR, SIUNEU_EXC_NEW_NEU, &siuContext);

  return (inst);
} /* siu_new_nv_neu */
  

/******************************************************************************
 * FUNCTION PURPOSE:  Open NEU module
 ******************************************************************************
 * DESCRIPTION: Opens one instance of a neu module
 *
 * siu_open_neu (
 *   tint *chnum)         - instance pointer 
 *
 *****************************************************************************/
void siu_open_neu (tint chnum, neuParms_t *cfg)
{
  neuCfg_t neuCfg;
  siuInst_t *inst;

  /* initialize the siu instance */
  inst = SIUINSTp(chnum-SIU_FIRSTCH);
  
  /* default configuration of NEU is open receive out */
  neuCfg.neuSendOut.neuSendOut = siuNduSendPkt;
  neuCfg.neuSendOut.targetInst = inst;

  /* Pass Timer Function */
  neuCfg.getTime = siuTmrGetTime;

  /* configuration parameters */
  neuCfg.parm = *cfg;
  /* open neu channel */
  if (neuOpen (inst->neuInst,&neuCfg) != neu_NOERR)
     siu_exc_assert (0, SIUNEU_EXC_OPEN_NEU, inst);

} /* siu_open_neu */ 

/******************************************************************************
 * FUNCTION PURPOSE: Global configuration from host
 ******************************************************************************
 * DESCRIPTION: Global configuration from host
 *                         
 *****************************************************************************/
void siu_net_encap_glob_cfg (void *message) 
{
  neuGlobParms_t cfg;
  mgb_t *msg = (mgb_t *)message;
  cfg.min_ip_id = msg->msg_body.net_encap_control.parms.glob.ip_id_min;
  cfg.max_ip_id = msg->msg_body.net_encap_control.parms.glob.ip_id_max;
  neu_glob_config(&cfg); 
} /* siu_net_encap_cfg */
/******************************************************************************
 * FUNCTION PURPOSE: channel configuration from host
 ******************************************************************************
 * DESCRIPTION: channel configuration from host
 *                         
 *****************************************************************************/
void siu_net_encap_control (siuInst_t *inst, mgb_t *msg) 
{
  neuCtrl_t cfg;
  int retval = neu_ERROR;
  tint count;
#ifndef PA_SS   
  tint physPort;
  halGmacControl_t hctl;
#endif
#ifdef SIU_NDU_HETERO_PKT_IFACE
  tuint   hal_pkt_iface,ndu_call_tbl_idx;
#endif

  inst->amInst.hd_pkt_routing_type
    = msg->msg_body.net_encap_control.parms.channel.hd_pkt_routing_type;

  cfg.u.parm.chnl.cfg_flag1
    = msg->msg_body.net_encap_control.parms.channel.flags1;
  cfg.u.parm.chnl.cfg_flag2
    = msg->msg_body.net_encap_control.parms.channel.flags2;

  cfg.u.parm.chnl.alt_local_port
    = msg->msg_body.net_encap_control.parms.channel.alt_local_port;
  cfg.u.parm.chnl.alt_remote_port
    = msg->msg_body.net_encap_control.parms.channel.alt_remote_port;
  
  count = 0;
  while(count < MGB_IP_ADDR_MAX_NUM_WORDS)
  {
      cfg.u.parm.chnl.alt_remote_addr[count] = 
          msg->msg_body.net_encap_control.parms.channel.alt_remote_addr[count];
      count++;
  }

  cfg.u.parm.chnl.report_ctrl
    = msg->msg_body.net_encap_control.parms.channel.report_ctrl;
  cfg.u.parm.chnl.report_throttle_time
    = msg->msg_body.net_encap_control.parms.channel.report_throttle_time;
  cfg.u.parm.chnl.sysInst = inst;

  cfg.u.parm.chnl.second_prototype
    = msg->msg_body.net_encap_control.parms.channel.tel2pkt_second_protocol;
  cfg.u.parm.chnl.rx_payload_type
    = msg->msg_body.net_encap_control.parms.channel.pkt2tel_first_protocol;
  cfg.u.parm.chnl.rx_offset_bytes
    = msg->msg_body.net_encap_control.parms.channel.pkt2tel_start_offset;

  cfg.u.parm.chnl.tx_min_cell_bytes
    = msg->msg_body.net_encap_control.parms.channel.tel2pkt_min_len;

  cfg.u.parm.chnl.tx_ip_offset_bytes
    = msg->msg_body.net_encap_control.parms.channel.tel2pkt_ip_offset;
  
  cfg.u.parm.chnl.tx_udp_offset_bytes
    = msg->msg_body.net_encap_control.parms.channel.tel2pkt_udp_offset;
  
  cfg.u.parm.chnl.tx_802_3_offset_bytes
    = msg->msg_body.net_encap_control.parms.channel.tel2pkt_802_3_offset;


  cfg.u.parm.chnl.tx_total_hdr_len_bytes
    = msg->msg_body.net_encap_control.parms.channel.tel2pkt_total_hdr_len;

  /* ENHANCEMENT NEEDED - neu might need to support multiple output port
   * types in the future */
#ifndef PA_SS   
  physPort
      = ((msg->msg_body.net_encap_control.parms.channel.pkt_dev_handle 
         & NET_ENCAP_PKT_DEV_HNDL_PORT_ID_MASK)
         >> NET_ENCAP_PKT_DEV_HNDL_PORT_ID_SHIFT);

  if ( (msg->msg_body.net_encap_control.parms.channel.pkt_dev_handle & NET_ENCAP_PKT_DEV_HNDL_IFACE_MASK) ==
        NET_ENCAP_PKT_DEV_HNDL_IFACE_GMAC)  {
    hctl.code              = HAL_GMAC_CTL_CODE_ID_FROM_PORT;
    hctl.u.portId.physPort = physPort;
    (*siuContext.halApi->gmac->halGmacControl) (&hctl);
    if (hctl.u.portId.halId != -1)
      cfg.u.parm.chnl.halId = hctl.u.portId.halId;
    else
      cfg.u.parm.chnl.halId = 0;

  } else  
#endif
  {
    cfg.u.parm.chnl.halId = 0;
  }

  cfg.u.parm.chnl.sar_type
    = msg->msg_body.net_encap_control.parms.channel.sar_type;

  if(cfg.u.parm.chnl.tx_udp_offset_bytes > 0)
  {
      cfg.u.parm.chnl.u.rtp_param.loc_rtcp_port = 
          msg->msg_body.net_encap_control.parms.channel.proto.rtp.loc_rtcp_port;
      cfg.u.parm.chnl.u.rtp_param.rem_rtcp_port = 
          msg->msg_body.net_encap_control.parms.channel.proto.rtp.rem_rtcp_port;
  }else {
      cfg.u.parm.chnl.u.sar_param.cid = 
          msg->msg_body.net_encap_control.parms.channel.proto.sar.cid;
      cfg.u.parm.chnl.u.sar_param.ra_time_out = 
          msg->msg_body.net_encap_control.parms.channel.proto.sar.ra_time_out;
      cfg.u.parm.chnl.u.sar_param.max_cps_size = 
          msg->msg_body.net_encap_control.parms.channel.proto.sar.max_cps_size;
  }

/*  memcpy(cfg.u.parm.chnl.header,
         msg->msg_body.net_encap_control.parms.channel.hdr_image,
         UTL_BYTES_TO_WORDS(neuproto_MAX_HEADER_SIZE_BYTES)); 
*/ 
  {
#ifdef GG_INCLUDE_NEU_AAL5
/* This is to fix the dummy-byte positioning issue in the VCdemux format header */
    /* Copy the ATM Header */ 
    tuint byte_offset=SIU_NEU_DUMMY_BIT_OFFSET;
    for(i=0;i< neuproto_ATM_HEADER_SIZE_BYTES/2;i++)
    {
      pktWrite16bits(cfg.u.parm.chnl.header,byte_offset,
                     msg->msg_body.net_encap_control.parms.channel.hdr_image[i]);
      byte_offset+=2;
     }
     
    /* Copy the rest of the header */
    byte_offset=neuproto_ATM_HEADER_SIZE_BYTES;
    for(i=neuproto_ATM_HEADER_SIZE_BYTES/2;i<neuproto_MAX_HEADER_SIZE_BYTES/2;i++)
    {
      pktWrite16bits(cfg.u.parm.chnl.header,byte_offset,
                     msg->msg_body.net_encap_control.parms.channel.hdr_image[i]);
      byte_offset+=2;
     }
#else
   
   memcpy(cfg.u.parm.chnl.header,
         msg->msg_body.net_encap_control.parms.channel.hdr_image,
         UTL_BYTES_TO_WORDS(neuproto_MAX_HEADER_SIZE_BYTES));
#endif
  }   
  
  /* UDP source port modifications not required */
  cfg.u.parm.chnl.tx_udp_src_mask = 0;
  cfg.u.parm.chnl.tx_udp_src_val  = 0;

  /* Configure NEU to update remote UDP port if there is there is a mismatch.
   * This is required for TFTP
   */
  cfg.u.parm.chnl.cfg_flag2 |= neuproto_FLAGS2_UDP_USE_REMOTE;

  /* NEU gets enabled once it is configured for the first time */
  cfg.u.parm.chnl.enable = TRUE;

  cfg.code = neu_CTRL_CHANNEL_CFG;
  if (inst->neuInst) {
     retval = (*gg_neu_ctable.neuControl) (inst->neuInst, &cfg);
  }
  if (retval != neu_NOERR) {
    siuErrorIndication (inst->ID, msg->header.trans_id, MGB_ERR_NEU_BAD_CONFIG);
  }
#ifdef NEU_GMAC
#ifdef PA_SS
  if(!(cfg.u.parm.chnl.cfg_flag2 & NET_ENCAP_FLAGS2_DISABLE)) {
      if(!(dsiuConfigL2L3L4(inst->neuInst,
                            &cfg.u.parm.chnl,
                            &inst->udpHandle,
                            NULL,
                            NULL)))
      {
        siuErrorIndication (inst->ID, msg->header.trans_id, MGB_ERR_NEU_BAD_CONFIG);
      }
  }
#else
  /* un-register any udp ports mapped to channel */
  cfg.code = neu_CTRL_RMV_PORTMAP;
  cfg.u.portMap.inst = (void *)inst;
  (*gg_neu_ctable.neuControl) (inst->neuInst, &cfg);

  /* Map the new udp port */
  if (cfg.u.parm.chnl.tx_udp_offset_bytes >= 0)  {
    cfg.code = neu_CTRL_ADD_PORTMAP;
    cfg.u.portMap.port = pktRead16bits_m(&(cfg.u.parm.chnl.header[UTL_BYTES_TO_WORDS(
                         cfg.u.parm.chnl.tx_udp_offset_bytes + 
                         neuproto_UDP_BYTE_OFFSET_SRC_PORT)]),0);
    cfg.u.portMap.inst = (void *)inst;
    (*gg_neu_ctable.neuControl) (inst->neuInst, &cfg);

/* Register the corresponding RTCP port */
     cfg.code = neu_CTRL_ADD_PORTMAP;
    cfg.u.portMap.port = cfg.u.portMap.port +1;
    (*gg_neu_ctable.neuControl) (inst->neuInst, &cfg);
  }
#endif  
#endif

#ifdef SIU_NDU_HETERO_PKT_IFACE
  switch(msg->msg_body.net_encap_control.parms.channel.pkt_dev_handle 
         & NET_ENCAP_PKT_DEV_HNDL_IFACE_MASK)
  {
    case NET_ENCAP_PKT_DEV_HNDL_IFACE_UTOPIA:
         hal_pkt_iface = SIU_NDU_PKT_IFACE_UTOPIA;
         ndu_call_tbl_idx = SIU_NDU_CALL_TABLE_DEF_IDX;
         break;

    case NET_ENCAP_PKT_DEV_HNDL_IFACE_GMAC:
         hal_pkt_iface = SIU_NDU_PKT_IFACE_GMAC;
         ndu_call_tbl_idx = SIU_NDU_CALL_TABLE_GMAC_IDX;
         break;

    case NET_ENCAP_PKT_DEV_HNDL_IFACE_DEFAULT:
         hal_pkt_iface = SIU_NDU_PKT_IFACE_DEFAULT;
         ndu_call_tbl_idx = SIU_NDU_CALL_TABLE_DEF_IDX;
         break;
  }

  inst->pkt_iface_idx = 0;
  SIU_PKT_SET_IDX_IFACE(inst->pkt_iface_idx,
                       hal_pkt_iface,
                       SIU_CHAN_PKT_IFACE_MASK,
                       SIU_CHAN_PKT_IFACE_SHIFT);
  SIU_PKT_SET_IDX_IFACE(inst->pkt_iface_idx,
                       hal_pkt_iface,
                       SIU_CHAN_PKT_IDX_MASK,
                       SIU_CHAN_PKT_IDX_SHIFT);
#endif
} /* siu_net_encap_cfg */

/******************************************************************************
 * FUNCTION PURPOSE: Bandwidth Control Configuration from host
 ******************************************************************************
 * DESCRIPTION: Bandwidth Control  configuration from host
 *                         
 *****************************************************************************/
void siu_net_encap_bwc_config (siuInst_t *inst, mgb_t *msg) 
{
  neuCtrl_t cfg;
  int retval = neu_ERROR;

  cfg.u.bwc_config = (void *)(&(msg->msg_body.bwc_config));

  cfg.code = neu_CTRL_SET_BWC;
  if (inst->neuInst) {
     retval = (*gg_neu_ctable.neuControl) (inst->neuInst, &cfg);
  }
  if (retval != neu_NOERR) {
    siuErrorIndication (inst->ID, msg->header.trans_id, MGB_ERR_BWC_CONFIG);
  }
} /* siu_net_encap_bwc_config */

/******************************************************************************
 * FUNCTION PURPOSE: response to the statistics request from the host
 ******************************************************************************
 * DESCRIPTION: Statistics report
 *                         
 *****************************************************************************/
void siuGetNetEncapStats (void *siuInst, bool reset, tuint trans_id, tuint priority)
{
  siuInst_t *inst = (siuInst_t *)siuInst;
  mgb_t send_message;

  send_message.header.length  = MGB_HEADER_SIZE + (sizeof(struct MGB_DMS_GET_NETENCAP_STAT)/sizeof(dprword));
  send_message.header.chnum   = (int)(inst->ID & SIU_CID_MASK);
  send_message.header.msg_id  = MGB_DM_GET_NETENCAP_STAT;
  send_message.header.trans_id = trans_id;

  neuStatReq (inst->neuInst, reset, &send_message.msg_body.net_encap_stat);

  /* send message to FIFO */
  siuNduSendMsgPriority (&send_message,(NDU_DPRMsgType_t)priority); 
} /* siuGetTxRxStats */ 

/******************************************************************************
 * FUNCTION PURPOSE: Response to the GLOBAL Packet statistics request 
 *                   from the host
 ******************************************************************************
 * DESCRIPTION: Global Packet Statistics report
 *                         
 *****************************************************************************/
void siuGetGlobalPktStats (void *siuInst, bool reset, tuint trans_id, tuint priority)
{
  /* Note: siuInst isn't valid -- just match API */
  mgb_t send_message;
  neuGlblPktStats_t pkt_stats;

  send_message.header.length   = MGB_HEADER_SIZE + 10*MGB_MSGPAR_SIZE;
  send_message.header.chnum    = 0;  /* Channel Independent message */
  send_message.header.msg_id   = MGB_DM_GET_GLOBAL_PKT_STAT;
  send_message.header.trans_id = trans_id;

  neuGlobalPktStatReq (&pkt_stats, reset);

  send_message.msg_body.global_pkt_stat.txPkts_MSW = siu_get_umsw(pkt_stats.txPackets);
  send_message.msg_body.global_pkt_stat.txPkts_LSW = siu_get_ulsw(pkt_stats.txPackets);

  send_message.msg_body.global_pkt_stat.rxPkts_MSW = siu_get_umsw(pkt_stats.rxPackets);
  send_message.msg_body.global_pkt_stat.rxPkts_LSW = siu_get_ulsw(pkt_stats.rxPackets);

  send_message.msg_body.global_pkt_stat.rxErrPkts_L2_MSW = 
                               siu_get_umsw(pkt_stats.rxErrPktsL2);
  send_message.msg_body.global_pkt_stat.rxErrPkts_L2_LSW =
                               siu_get_ulsw(pkt_stats.rxErrPktsL2);

  send_message.msg_body.global_pkt_stat.rxErrPkts_L3_MSW = 
                               siu_get_umsw(pkt_stats.rxErrPktsL3);
  send_message.msg_body.global_pkt_stat.rxErrPkts_L3_LSW =
                               siu_get_ulsw(pkt_stats.rxErrPktsL3);

  send_message.msg_body.global_pkt_stat.rxErrPkts_L4_MSW = 
                               siu_get_umsw(pkt_stats.rxErrPktsL4);
  send_message.msg_body.global_pkt_stat.rxErrPkts_L4_LSW =
                               siu_get_ulsw(pkt_stats.rxErrPktsL4);
  send_message.msg_body.global_pkt_stat.rxErrPkts_TC_MSW = 
                               siu_get_umsw(pkt_stats.rxErrPktsTC);
  send_message.msg_body.global_pkt_stat.rxErrPkts_TC_LSW =
                               siu_get_ulsw(pkt_stats.rxErrPktsTC);

  

  /* send message to FIFO */
  siuNduSendMsg(&send_message); 
} /* siuGetGlobalPktStats */ 

/******************************************************************************
 * FUNCTION PURPOSE: Inquire the Neu instatance from the host
 ******************************************************************************
 * DESCRIPTION: Inquire the Neu instatance from the host based on the channel ID
 *
 *  void* siuGetNetEncapInst (
 *    tuint chan_num)         - channel number
 *
 * Return NULL if channel is not at an valid state (such as VOICE)
 *        neuInst for the specific channel  
 *                         
 *****************************************************************************/
void* siuGetNetEncapInst (tuint instance_ID)
{
  siuInst_t *inst = (siuInst_t *)SIUINSTp((instance_ID & SIU_CID_MASK) - SIU_FIRSTCH);
  /* if (inst->state == SIU_CHST_VOICE) */
  return(inst->neuInst);

} /* siuGetNetEncapInst */ 

/******************************************************************************
 * FUNCTION PURPOSE: Update Tx Statistics
 ******************************************************************************
 * DESCRIPTION: This function is called when ever a packet is sent via IP-IP 
 *              transparency connection in NEU. It does the conversion from
 *              neuInst ID to rcuInst and calls the RCU function to update the 
 *              global Heap Stats for the channel.
 *
 *  void siuUpdateTxStats (
 *    tuint chan_num          - channel number
 *    xferPktInfo_t *pktInfo) - Packet that is being sent out
 *                         
 *****************************************************************************/
void siuUpdateTxStats (tuint instance_ID, xferPktInfo_t *pktInfo, tint retval)
{
#ifndef HDCT
  siuInst_t *inst = (siuInst_t *)SIUINSTp((instance_ID & SIU_CID_MASK) - SIU_FIRSTCH);

  if (gg_rcu_ctable.rcuUpdateTxStats!= NULL)
      gg_rcu_ctable.rcuUpdateTxStats(inst->net.rcuInst, pktInfo, retval);
#endif
} /* siuUpdateTxStats */ 


static bool siu_is_valid_chn_neu (tuint chnum)
{
  return    ( chnum < siuContext.maxNumberChannels )
         && ( SIUINSTp(chnum)->neuInst != NULL);           
}


/******************************************************************************
 * FUNCTION PURPOSE:  processes an atom of connect command
 ******************************************************************************
 * DESCRIPTION: 
 *   Procesess a single element from a list from the connect message. Handles
 *   only NEU-related elements.
 *
 * RETURNS:
 *   The return value of TRUE means success.
 *****************************************************************************/
bool siu_conn_req_neu (struct MGB_MDS_CONN_ELEM_T *pCmd)
{
  tuint connType = pCmd->req_type & MGB_CONN_REQ_TYPE_MASK;
  neuCtrl_t ctrl;
  neuPktRoutingCfg_t*  routingReq = &ctrl.u.pktRoutingReq;
  tuint srcChan, dstChan;
  void *neuInst;
  tuint stream_option = 0, dstside_option = 0;


  srcChan = pCmd->source_chan; 
  dstChan = pCmd->dest_chan;

  if (!siu_is_valid_chn_neu (srcChan) || !siu_is_valid_chn_neu (dstChan))
    return FALSE;
    
  /* only ADD and DISC are supported at this moment */
  if(connType == MGB_CONN_REQ_CONN_ADD)
    routingReq->enable = TRUE; 
  else if (connType == MBG_CONN_REQ_DISC) 
    routingReq->enable = FALSE;
  else
    return FALSE; 
    
  /* NEU supports only connections 'from' packet, not from TDM */
  if  (!(pCmd->req_type & MGB_CONN_SRC_PKT_SIDE))
    return FALSE;

  routingReq->chanID = dstChan + SIU_FIRSTCH;
  stream_option = (pCmd->req_type & MGB_CONN_TRANSP_PRI_DOMAIN) ? (stream_option | neu_PKT_ROUTING_PRI_ON) : stream_option;
  stream_option = (pCmd->req_type & MGB_CONN_TRANSP_SEC_DOMAIN) ? (stream_option | neu_PKT_ROUTING_SEC_ON) : stream_option;
  stream_option = (pCmd->req_type & MGB_CONN_TRANSP_SEC_XR_DEL_RX) ? (stream_option | neu_PKT_ROUTING_SEC_XR_DEL_RX) : stream_option;
  stream_option = (pCmd->req_type & MGB_CONN_TRANSP_SEC_XR_INS_TX) ? (stream_option | neu_PKT_ROUTING_SEC_XR_INS_TX) : stream_option;
  dstside_option = (pCmd->req_type & MGB_CONN_DEST_PKT_SIDE) ? (dstside_option | neu_PKT_ROUTING_DST_SIDE_PKT) : dstside_option;
  routingReq->options = stream_option | (dstside_option << neu_PKT_ROUTING_DST_SIDE_SHIFT);   

  /* Identify the source NEU */
  neuInst = SIUINSTp(srcChan)->neuInst;
  
  ctrl.code =  neu_CTRL_SET_PKT_ROUTING;
  
  return ((*gg_neu_ctable.neuControl) (neuInst, &ctrl) == neu_NOERR);
  
}

#ifdef NEU_USE_GMPISRSPLIT
void siuProcessPktFifo(void)
{
    neuProcessRxPkts();
}

tuint siuNeuDropPkts(tuint siugmpGmcSize)
{
    return(freeGmpPkts(siugmpGmcSize));

}
#endif

/******************************************************************************
 * FUNCTION PURPOSE:  configures the streaming NEU
 ******************************************************************************
 * DESCRIPTION: Configures the streaming NEU. 
 *
 *****************************************************************************/
tint siuConfigStreamingNeu(void *configmsg) {
    NDU_config_t config;
    struct MGB_MDS_NET_ENCAP_CHANNEL_SET *chcfg = (struct MGB_MDS_NET_ENCAP_CHANNEL_SET *)configmsg;
#ifndef PA_SS
    halGmacControl_t hctl;
#endif
    tint retVal;

    config.pNvneuOpen    = gg_neu_ctable.neuOpen;
    config.pNvneuControl = gg_neu_ctable.neuControl;
    config.pNvNeuCreate  = siu_new_nv_neu;;
    config.pAnuXmt       = siuNduSendPkt;

#ifndef PA_SS
    hctl.code              = HAL_GMAC_CTL_CODE_ID_FROM_PORT;
    hctl.u.portId.physPort = ((chcfg->pkt_dev_handle) & NET_ENCAP_PKT_DEV_HNDL_PORT_ID_MASK) >>
                           NET_ENCAP_PKT_DEV_HNDL_PORT_ID_SHIFT;

    /* Extract the halId for the requested physical port */
    (*siuContext.halApi->gmac->halGmacControl) (&hctl);
    if (hctl.u.portId.halId == -1)
      return (-1);
    else
      config.halId = hctl.u.portId.halId;
#else
    config.halId = 0; 
#endif

    retVal = NDU_gmacConfigNeuStreamer(chcfg, &config);
    return(retVal);
}

/* Nothing past this point */
