/******************************************************************************
 * FILE PURPOSE: Contains functions necessary to integrate OVLY
 ******************************************************************************
 * FILE NAME: siuovly.c
 *
 * DESCRIPTION: Contains functions necessary to integrate OVLY into
 *              system, such that it can be call tabled out of the system
 *
 * (C) Copyright 2008, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
 
/* System utility files */
#include "types.h"
#include "fract.h"
#include "mgmicdsp.h"
#include "ggcfg.h"
#include "siuovly.h"
#include "common/siuloc.h"
#include <ti/mas/ovly/ovly.h>
#include <ti/mas/vppu/vppu.h>
#include <ti/mas/iface/ifrfc/iftextrfc.h>

/*********************************************************************************/
/* Overlay conferencing matrix: this is local to core.                           */
/* Currently it is not possible to connect overlay chanel on core N with video	 */
/* channel on core M.															 */
/*********************************************************************************/
siuVigdkOvlyConn_t ovlyConnectionArray[OVLY_MAX_CONNECTIONS] = 
{ /* srcChanID (1-based!!!) | dstChan bit field  */
     0,                       0x00000000, 
     0,                       0x00000000, 
     0,                       0x00000000, 
     0,                       0x00000000, 
};
/******************************************************************************
 * FUNCTION PURPOSE: Connect overlay channel with destination video channel
 ******************************************************************************
 * DESCRIPTION: Update overlay conferencing matrix.
 *              If we are adding connection, find a free record and store connection
 *              details. Free record has source and destination channels set to 0.
 *              Active video and overlay channels are 1-based.
 *              If we are disconnecting the connection, find the record and release
 *              it. This is done by reseting source and destination channels.
 *
 *  tint siuOvlyConnect (
 *    void *srcInst,	 - source (overlay channel) siu instance
 *    void *dstInst,     - destination (video channel) siu instance
 *    tuint connType)    - connection request
 *
 *****************************************************************************/
tint siuOvlyConnect (void *srcInst, void *dstInst, tuint connType)
{
  siuInst_t *src_inst = (siuInst_t *)srcInst;
  siuInst_t *dst_inst = (siuInst_t *)dstInst;
  tint i;
  tint retval   = SIU_NOERR;
  tint src_chan = src_inst->ID & 0x00FF;
  tint dst_chan = dst_inst->ID & 0x00FF;
  ovlyCtrl_t ctrl;   
  
  switch (connType) {
    case MGB_CONN_REQ_CONN_ADD:
      for (i = 0; i < OVLY_MAX_CONNECTIONS; i ++)
      { /* check if source overlay channel already has its slot */
        if(ovlyConnectionArray[i].srcChan == src_chan)
        { /* Free connection slot, populate with the connection */
          ovlyConnectionArray[i].dstChan_bf |= (1 << dst_chan);
          break;
        }
      }
      if(i < OVLY_MAX_CONNECTIONS) 
      { /* slot found and updated */
        break;
      }      
      for (i = 0; i < OVLY_MAX_CONNECTIONS; i ++)      
      { /* allocate new slot for this source overlay channel */
        if(ovlyConnectionArray[i].srcChan == 0)
        { /* Free connection slot, set the connection (any previous is erased) */
          ovlyConnectionArray[i].srcChan = src_chan;
          ovlyConnectionArray[i].dstChan_bf = (1 << dst_chan);
          break;
        }
      }
      if(i == OVLY_MAX_CONNECTIONS)
      { /* No more free overlay connections */
        retval = SIU_ERROR;  
      }
      break;
    case MBG_CONN_REQ_DISC:
      for (i = 0; i < OVLY_MAX_CONNECTIONS; i ++)
      {	/* Identify the connection and remove it, by setting source and destination
         * channels to 0. Active channels are 1-based */
        if(ovlyConnectionArray[i].srcChan == src_chan)
        {
          ovlyConnectionArray[i].dstChan_bf &= ~(1 << dst_chan);
          if(ovlyConnectionArray[i].dstChan_bf == 0)
          {	/* This was last to disconnect, release the slot */
            ovlyConnectionArray[i].srcChan = 0;
            ctrl.control = OVLY_RESET;
            ovlyControl (src_inst->stateInst.video.ovlyInst, &ctrl);      
          }
        }
      }    
      break;
    case MGB_CONN_REQ_DISC_ALL_SRC:
      for (i = 0; i < OVLY_MAX_CONNECTIONS; i ++)
      {	/* Disconnect all source for the destination channel */
        if(ovlyConnectionArray[i].srcChan)
        {
          ovlyConnectionArray[i].dstChan_bf &= ~(1 << dst_chan);
          if(ovlyConnectionArray[i].dstChan_bf == 0)
          {	/* This was last to disconnect, release the slot */
            ovlyConnectionArray[i].srcChan = 0;
            ctrl.control = OVLY_RESET;
            ovlyControl (src_inst->stateInst.video.ovlyInst, &ctrl);      
          }
      	}
      }
      break;
    default: /* Unsupported connect command */
      retval = SIU_ERROR;  
      break;
  }
  return retval;
}
/******************************************************************************
 * FUNCTION PURPOSE: Message processing for overlay channel configuration.
 ******************************************************************************
 * DESCRIPTION: Extract message details and store them into overlay conferencing
 *              matrix.
 * void siuOvlyCfgExt (
 *   void *siuInst,     - siu instance of overlay channel
 *   mgb_t *message)    - message providing overlay configuration details
 ******************************************************************************/ 
void siuOvlyCfg (void *siuInst, mgb_t *message)
{
siuInst_t *inst = (siuInst_t *)siuInst;
ovlyCtrl_t ctrl;

  if(message->msg_body.video_ovly_cfg.valid_params & MGB_VIDEO_OVLY_TYPE_PARAMS_OVLY_DEMO)
  { /* Detect legacy codes */
    inst->stateInst.video.ovly_legacy_bitmap = message->msg_body.video_ovly_cfg.ovly_res_demo;
    return;      
  }
  if(inst->stateInst.video.ovlyInst == NULL) return;
  ctrl.control        = OVLY_TTXT;        
  ctrl.u_in.ttxt      = message->msg_body.video_ovly_cfg;
  ctrl.buffer_en_flag = OVLY_BUFFER_ENDIAN_FIX;  
  ovlyControl (inst->stateInst.video.ovlyInst, &ctrl);
  if(message->msg_body.video_ovly_cfg.valid_params & MGB_VIDEO_OVLY_TYPE_PARAMS_CTRL_CODE)
  { /* SIU level processing */
    if(message->msg_body.video_ovly_cfg.ctrl_code == MGB_VIDEO_OVLY_TYPE_RESUME)
    { /* Release flow from server to overlay channel */
      /** Invalidate overlay buffer **/          
      if(ctrl.buffer) { /* check if valid overlay buffer is already assigned */
        siuYuvSetBufFree(inst, ctrl.buffer);
      }
    } else if(message->msg_body.video_ovly_cfg.ctrl_code == MGB_VIDEO_OVLY_TYPE_STOP)
    { /* Freeze the overlay content, drop any incoming update from the server */
      inst->yuvBuffer = NULL;
    }
  }
}
/******************************************************************************
 * FUNCTION PURPOSE: Overlay matrix update
 ******************************************************************************
 * DESCRIPTION: Run-time parameters are stored by overlay channel, to be used
 *              in overlay operation, by overlay process function, before the
 *              encoding of destination video channel.
 * void siuOvlyConfMatrixUpdate (
 *   void *siuInst,       - siu instance of overlay channel
 *   tlong ovly_buf_size) - size of overlay data in the buffer
 ******************************************************************************/ 
void siuOvlyConfUpdate (void *siuInst, void *inputBuf)
{
  siuInst_t *inst = (siuInst_t *)siuInst;
  ovlyCtrl_t ctrl;
  /* Update buffer state info for RAW and TTEXT overlay in same way */
  ctrl.control               = OVLY_UPDATE;
  ctrl.u_in.rawg.yuvBuffer   = inst->yuvBuffer;
  ctrl.u_in.rawg.yuvWidth    = inst->yuvWidth;
  ctrl.u_in.rawg.yuvHeight   = inst->yuvHeight;
  /* If updating is stopped, it will return immediately */
  if(ovlyControl (inst->stateInst.video.ovlyInst, &ctrl) == OVLY_ERROR) return;
  
  /* If this is overlay channel configured with timed text, pass it to OVLY to do the rendering */
  if (SIU_GET_VIDEO_CHANNEL_EXT_TYPE(inst) == SIU_CHTYPE_EXT_OVLY_TTXT)
  { /* Do the text rendering */
    rfcRxTText_t *ttext = (rfcRxTText_t *)inputBuf;

    ctrl.control                = OVLY_TTXT;
    ctrl.u_in.ttxt.valid_params = MGB_VIDEO_OVLY_TYPE_PARAMS_CHAR_ARRAY;
    ctrl.buffer                 = &ttext->textp[0];
    /* Make sure it is null terminated */
    ttext->textp[ttext->text_len] = 0;
    ctrl.buffer_en_flag         = OVLY_BUFFER_ENDIAN_OK;
    if(ttext->valid_bitmap & RFC_FIFO_VALID_TEXT_DUR)
    { /* Timed text duration is provisoned in-band */
      ctrl.u_in.ttxt.valid_params |= MGB_VIDEO_OVLY_TYPE_PARAMS_TIMER;
      ctrl.u_in.ttxt.timer         = ttext->text_dur;
    }
    ovlyControl (inst->stateInst.video.ovlyInst, &ctrl);
  }
}
/*********************************************************************************
 * FUNCTION PURPOSE: Run-time overlay processing
 *********************************************************************************
 * DESCRIPTION: This version currently does not support any complex mixing of overlay
 *        channel content and destination video channel content. Only 4:2:0 format is
 *        supported (this version). Absolute x- and y-offsets are calculated using relative
 *        x- and y-offset (0.0 - 1.0 range) as provisioned through DIM API.         
 * Parameters:         
 *   void siuOvlyProcess (
 *     siuInst_t *inst,      - siu instance
 *     word *yuvInputFrame,  - YUV frame buffer of destination video channel to be overlaid
 *     tuint yuvInputWidth,  - YUV frame width  of destination video channel to be overlaid
 *     tuint yuvInputHeight) - YUV frame height of destination video channel to be overlaid
 *********************************************************************************/
#if EQUALIZER_BAR
volatile tuint barWidth = 6;
volatile tword lumam = 200, chroma_r = 100, chroma_b = 30; 
#define NUM_BAR 7
volatile tuint numBar = NUM_BAR;
volatile tuint minBarHeight = 6;
volatile tuint BarHeightRange = 20;
volatile tuint barHeight[NUM_BAR] = {12, 16, 24, 20, 28, 20, 16};
volatile tuint xPosition = 8;
volatile tuint yPosition = 144;
volatile tuint numFrameUpdate = 3;
volatile tuint indexFrameUpdate = 0;
#endif
void siuOvlyProcess (void *dstInst, word *yuvInputFrame, tuint yuvInputWidth, tuint yuvInputHeight)
{
  tint i;
  siuInst_t *src_inst, *dst_inst = (siuInst_t *)dstInst;
  siuInst_t *inst = dst_inst;
  tuint dstChan = (dst_inst->ID & SIU_CID_MASK); //1-based
  tulong dstChan_mask = (1 << dstChan);

  /************************************************************************
   * Currently simple overlay and logo insertion is done at this location
   ************************************************************************/
  if(inst->stateInst.video.ovly_legacy_bitmap) {
    ovly_process_legacy (inst->stateInst.video.ovly_legacy_bitmap, siuContext.coreId,
                         ((inst->ID & 0x00ff) - 1), SIU_GET_VIDEO_CODING_TYPE(inst),
                         inst->frameRate/1000, inst->bitRate/1000,
                         yuvInputFrame, inst->yuvWidth, inst->yuvHeight);
  }

  if(inst->stateInst.video.ovly_legacy_bitmap & OVLY_LEGACY_BIT_BAR) {
#if EQUALIZER_BAR
    if((indexFrameUpdate % (numFrameUpdate*siuContext.numActiveChannels)) == 0)
    {
        for (i=0; i<numBar; i++) {
            barHeight[i] = rand() % BarHeightRange + minBarHeight;
        }
    }
    indexFrameUpdate++;

    /* equalizer bar for IBC demo */
    ovly_process_equalizer(yuvInputFrame, inst->yuvWidth, inst->yuvHeight,
                           barWidth, (tuint *)barHeight, numBar,
                           xPosition, yPosition,
                           lumam, chroma_r, chroma_b);
#endif
  }

  /* overlay process */
  for (i = 0; i < OVLY_MAX_CONNECTIONS; i ++)
  {
    if(ovlyConnectionArray[i].dstChan_bf & dstChan_mask)
    { /* ...yuvBuffer is set if overlay content arrived. Also verify that overlay 
       * content is intended for this video channel */
      src_inst = SIUINSTp(ovlyConnectionArray[i].srcChan - SIU_FIRSTCH);
      ovlyProcess (src_inst->stateInst.video.ovlyInst, yuvInputFrame, yuvInputWidth, yuvInputHeight);
    }  
  }
}
/******************************************************************************
 * FUNCTION PURPOSE: Executes Video Overlay Configuration
 ******************************************************************************/
void siu_new_ovly (tint chnum) {
  tint         stat;
  memBuffer_t  ovlyBufs[SIU_MAX_NBUFS];
  ovlyNewCfg_t ovlyCfgNew;
  ovlySizeCfg_t ovlyCfgSize;
  siuInst_t    *inst;
  
  /* initialize the siu instance */
  inst = &siuInst[chnum-SIU_FIRSTCH];
  
  /* configure the instance */
  ovlyCfgNew.ID             = siuMakeID (SIU_MID_OVLY, chnum);
  ovlyCfgNew.sizeCfg        = ovlyCfgSize;

  stat = siuNewEco (inst, (siuEcoGetSizes_t)ovlyGetSizes, &ovlyCfgSize, 
                    (siuEcoNew_t)ovlyNew, &ovlyCfgNew, 
                    &(inst->stateInst.video.ovlyInst),  ovlyBufs, SIU_MID_OVLY);
  
  siu_exc_assert (stat == SIU_NOERR, SIUOVLY_EXC_NEW_OVLY, inst);
}

/******************************************************************************
 * FUNCTION PURPOSE: Executes Video Overlay Configuration
 ******************************************************************************/
void siu_open_ovly (tint chnum, tuint codingType, tuint width, tuint height) {
  ovlyCfg_t ovlyCfg;
  siuInst_t *inst;

  /* initialize the siu instance */
  inst = &siuInst[chnum-SIU_FIRSTCH];
  
  ovlyCfg.coreId       = siuContext.coreId;
  ovlyCfg.codingType   = codingType;
  ovlyCfg.img_width    = width;
  ovlyCfg.img_height   = height;

  /* open custom ECO */
  if (gg_ovly_ctable.ovlyOpen) {
    if ((gg_ovly_ctable.ovlyOpen)(inst->stateInst.video.ovlyInst, &ovlyCfg) != OVLY_NOERR) 
    {
      siu_exc_assert (0, SIUOVLY_EXC_OPEN_OVLY, inst);
    }
  }
}
/*** nothing past this point ***/
