/******************************************************************************
 * FILE PURPOSE: Contains functions necessary to integrate TFTP
 ******************************************************************************
 * FILE NAME: siutftp.c
 *
 * DESCRIPTION: Contains functions necessary to integrate TFTP into
 *              system, such that it can be call tabled out of the system
 *
 * (C) Copyright 2009, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
 
/* System utility files */
#include "types.h"
#include "mgmicdsp.h"
#include "ggcfg.h"
/* SIU files */
#include "common/siuloc.h"
#include "siutftp.h"
 
/* Module files */
#include <ti/mas/tftp/tftp.h>
#include <ti/mas/iface/ifpkt/xferpkt.h>
#ifndef HDCT
#include <ti/mas/mmcu/mmcu.h>
#include "siummcu.h"
#endif
#include "util.h"
#include "ggmbuf.h"
#include "utlrepck_m.h" 

/******************************************************************************
 * FUNCTION PURPOSE: Create Video Protocol Processing Unit (TFTP) module 
 ******************************************************************************
 * DESCRIPTION: 
 *
 *  siu_new_tftp (
 *    tint  chnum)  - channel number
 *
 *****************************************************************************/
void siu_new_tftp (tint chnum)
{
  tint         stat;
  memBuffer_t  tftpBufs[SIU_MAX_NBUFS];
  tftpNewCfg_t tftpCfgNew;
  tftpSizeCfg_t tftpCfgSize;
  siuInst_t    *inst;
  
  /* initialize the siu instance */
  inst = &siuInst[chnum-SIU_FIRSTCH];
  
  /* configure the instance */
  tftpCfgNew.ID             = siuMakeID (SIU_MID_TFTP, chnum);
  tftpCfgSize.scratchSize   = 1600;
  tftpCfgNew.sizeCfg        = tftpCfgSize;

  stat = siuNewEco (inst, (siuEcoGetSizes_t)tftpGetSizes, &tftpCfgSize, 
                    (siuEcoNew_t)tftpNew, &tftpCfgNew, 
                    &(inst->net.tftpInst),  tftpBufs, SIU_MID_TFTP);
  
  siu_exc_assert (stat == SIU_NOERR, SIUTFTP_EXC_NEW_TFTP, inst);
} /* siu_new_tftp */ 

/******************************************************************************
 * FUNCTION PURPOSE:  Open TFTP module
 ******************************************************************************
 * DESCRIPTION: Opens one instance of a TFTP module
 *
 * siu_open_tftp (
 *   tint *chnum                 - instance pointer 
 *   siuCodecCfg_t *codecCfg)    - configuration for the codec
 *****************************************************************************/
void siu_open_tftp (siuInst_t *inst)
{
  tftpCfg_t tftpCfg;

  /* memset tftp Config to zero */
  memset(&tftpCfg,0,sizeof(tftpCfg_t));

  /* default configuration of TFTP is open receive out */
  tftpCfg.tftpSendOut.tftpSendOut = gg_neu_ctable.neuSendIn;
  tftpCfg.tftpSendOut.targetInst  = inst->neuInst;
  tftpCfg.MTUsize      = 0;  // Will be determined when configured for Get or Put operation.
  tftpCfg.ackfreq      = 1;
  /* open tftp channel */
  if (tftpOpen (inst->net.tftpInst,&tftpCfg) != TFTP_NOERR)
     siu_exc_assert (0, SIUTFTP_EXC_OPEN_TFTP, inst);

} /* siu_open_tftp */ 

/******************************************************************************
 * FUNCTION PURPOSE:  Send out data via TFTP
 ******************************************************************************
 * DESCRIPTION: Send out data via TFTP. Runs until the buffer is exhausted.
 *
 * siu_open_tftp (
 *   siuInst_t  *inst             - instance pointer 
 *   tword      *bufPtr           - Pointer to data buffer
 *   tulong     bufSize)          - Size of the data in the buffer
 *****************************************************************************/
tint siu_put_tftp (siuInst_t *inst, tword *bufPtr, tulong bufSize) 
{
  void        *tftpInst = inst->net.tftpInst;
  tftpCtrl_t  tftpCtrl;
  tulong      bytesConsumed;

  do {
    tftpCtrl.code = TFTP_CTRL_GET_STATUS;
    tftpControl( tftpInst, &tftpCtrl);
  } while(tftpCtrl.u.xferStatus == TFTP_FILE_XFER_WORKING);

  do {
    
    bytesConsumed = tftpPutData(tftpInst, bufPtr, bufSize);
    bufPtr  += bytesConsumed;
    bufSize -= bytesConsumed;

  } while( bufSize > 0);

  siu_invalidate_cache();

  return 0;
} /* siu_put_tftp */

/******************************************************************************
 * FUNCTION PURPOSE:  Send out data via TFTP
 ******************************************************************************
 * DESCRIPTION: Send out data via TFTP. Runs until the buffer is exhausted.
 *
 * siu_open_tftp (
 *   void           *siuInst      - instance pointer 
 *   xferpktInfo_t  *pktInfo      - Pointer to data buffer
 *****************************************************************************/
tint siu_send_tftp(void *siuInst, xferpktInfo_t *pktInfo)
{
  siuInst_t *inst = (siuInst_t *)siuInst;
  tint      i;

  for ( i=0; i<pktInfo->npkts; i++)
    siu_put_tftp(inst,pktInfo->pktIn[i],pktInfo->pktSize[i]);
  
  return 0;
}

tint siu_get_tftp(void *siuInst)
{
  siuInst_t *inst = (siuInst_t *)siuInst;

  tftpGetData(inst->net.tftpInst);
  return 0;
}

tint siu_start_tftp(void *siuInst)
{
  siuInst_t *inst     = (siuInst_t *)siuInst;
  neuCtrl_t neuCtrl;

  neuCtrl.code = neu_CTRL_SET_TX_HDR;
  neuCtrl.u.newTxHdr.remUdpPortNum  = 69;
  neuCtrl.u.newTxHdr.validParams    = neu_TX_HDR_VALID_REM_UDP;
  neuControl(inst->neuInst, &neuCtrl);

  tftpStart(inst->net.tftpInst);

  return 0;
}

/******************************************************************************
 * FUNCTION PURPOSE:  Stop a TFTP session.
 ******************************************************************************
 * DESCRIPTION: Stops the TFTP seesion for a channel. This will immediatel 
 *              stop a GET session while a PUT session will still need to send 
 *              out the last of the file.
 *
 * siu_open_tftp (
 *   siuInst_t  *inst)            - instance pointer 
 *
 *****************************************************************************/
tint siu_stop_tftp(void *siuInst)
{
  siuInst_t *inst = (siuInst_t *)siuInst;

  tftpStop(inst->net.tftpInst);

  return 0;
}

/******************************************************************************
 * FUNCTION PURPOSE:  Stop a TFTP session.
 ******************************************************************************
 * DESCRIPTION: 
 *
 * siuTftpCfg (
 *   siuInst_t  *inst             - instance pointer 
 *   mgb_t *message)
 *
 *****************************************************************************/
#ifndef HDCT
void siuTftpCfg (siuInst_t *inst,  mgb_t *message)
{
  struct MGB_MDS_ENCAP_TFTP *tftp_cfg =  &message->msg_body.encap_config.encap_params.tftp;
  siuVideoChannelOperation_t chType = (siuVideoChannelOperation_t) SIU_GET_VIDEO_CHANNEL_TYPE(inst);
  tftpCtrl_t tftpCtrl;
  tword      filename[MGB_MSG_MAXNPAR - 5];
  tint       i;

  if(tftp_cfg->valid_params & MGB_MDS_TFTP_CFG_VALID_FILE_NAME)
  {
    switch(chType) {
      case SIU_CHTYPE_DECODER:
        tftpCtrl.code = TFTP_CTRL_RRQ;

        tftpCtrl.u.req.receiveOut.tftpReceiveOut  = mmcuReceiveIn;
        tftpCtrl.u.req.receiveOut.targetInst      = inst->stateInst.video.mmcuInst;

        break;
      case SIU_CHTYPE_ENCODER:
        tftpCtrl.code = TFTP_CTRL_WRQ;
        tftpCtrl.u.req.receiveOut.tftpReceiveOut  = NULL;
        tftpCtrl.u.req.receiveOut.targetInst      = inst;
        break;
      default:
        //siuErrorIndication (inst->ID, message->header.trans_id, MGB_ERR_VIDEO_INVALID_TFTP_CONFIG);
    }

    i = 0; 
    while(tftp_cfg->file_name[i] && (i < MGB_MSG_MAXNPAR - 5 - 1)) 
    {
      filename[2*i] = (tword)(tftp_cfg->file_name[i] & 0xFF);
      filename[2*i+1] = (tword)(tftp_cfg->file_name[i] >> 8);
      i++;
    }
    filename[2*i] = 0;

    tftpCtrl.u.req.filename   = (char *)filename;
    tftpCtrl.u.req.MTUsize    = inst->maxPayloadSize;
    tftpCtrl.u.req.timeout    = 10000;
    
    tftpControl( inst->net.tftpInst, &tftpCtrl);
     
  }
  if(tftp_cfg->valid_params & MGB_MDS_TFTP_CFG_VALID_ACTION)
  {
    if(tftp_cfg->action == MGB_MDS_TFTP_CFG_ACTION_START) 
    {
      /* Assert MMCU is open */
      if( (SIU_GET_VIDEO_CHANNEL_TYPE(inst) == SIU_CHTYPE_DECODER) &&
          !SIU_GET_MMCU_CFG_DONE(inst)) 
      {
          siuErrorIndication(inst->ID, message->header.trans_id, MGB_ERR_VIDEO_MMCU_CFG_NOT_DONE);
          return;
      }

      siu_start_tftp((void *)inst);

      if(SIU_GET_VIDEO_CHANNEL_TYPE(inst) == SIU_CHTYPE_DECODER) 
        SIU_SET_MMCU_HDR_TASK_POST_PENDING(inst);

    }
    else if(tftp_cfg->action == MGB_MDS_TFTP_CFG_ACTION_STOP)
    {
      tftpStop(inst->net.tftpInst);
    }
  }

  return;
}
#endif
/* Nothing past this point */
