/******************************************************************************
 * FILE PURPOSE: Contains functions necessary to integrate TSU
 ******************************************************************************
 * FILE NAME: siutsu.c
 *
 * DESCRIPTION: Contains functions necessary to integrate TSU into
 *              system, such that it can be call tabled out of the system
 *
 * (C) Copyright 2009, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
 
/* System utility files */
#include "types.h"
#include "mgmicdsp.h"
#include "ggcfg.h"
/* SIU files */
#include "common/siuloc.h"
#include "siutsu.h"
#include "vctypes.h"
#include "video/siuVigdkGmp.h"

/* Module files */
#include <ti/mas/tsu/tsu.h>
#include "util.h"
#include "ggmbuf.h"
#include "utlrepck_m.h" 

/******************************************************************************
 * FUNCTION PURPOSE: Control Video Protocol Processing Unit (TSU) module 
 ******************************************************************************
 * DESCRIPTION: 
 *
 *  siu_tsu_control (
 *    tint  chnum,    - channel number
 *    mgb_t *message) - message from host
 *****************************************************************************/
void siu_tsu_control (tint chnum, mgb_t *message) {
  siuInst_t    *inst;
  tsuCtrl_t    ctrl;

  /* initialize the siu instance */
  inst = &siuInst[chnum-SIU_FIRSTCH];

  tsuControl(inst->stateInst.video.tsuInst, TSU_CTL_RESET_FILTER, &ctrl);
  return;
}

/******************************************************************************
 * FUNCTION PURPOSE: Create Video Protocol Processing Unit (TSU) module 
 ******************************************************************************
 * DESCRIPTION: 
 *
 *  siu_new_tsu (
 *    tint  chnum)  - channel number
 *
 *****************************************************************************/
void siu_new_tsu (tint chnum)
{
  tint         stat;
  memBuffer_t  tsuBufs[SIU_MAX_NBUFS];
  tsuNewCfg_t tsuCfgNew;
  tsuSizeCfg_t tsuCfgSize;
  siuInst_t    *inst;
  
  /* initialize the siu instance */
  inst = &siuInst[chnum-SIU_FIRSTCH];
  
  /* configure the instance */
  tsuCfgNew.ID             = siuMakeID (SIU_MID_TSU, chnum);
  tsuCfgSize.scratchSize   = 0;
  tsuCfgNew.sizeCfg        = tsuCfgSize;

  stat = siuNewEco (inst, (siuEcoGetSizes_t)tsuGetSizes, &tsuCfgSize, 
                    (siuEcoNew_t)tsuNew, &tsuCfgNew, 
                    &(inst->stateInst.video.tsuInst),  tsuBufs, SIU_MID_TSU);
  
  siu_exc_assert (stat == SIU_NOERR, SIUTSU_EXC_NEW_TSU, inst);
} /* siu_new_tsu */ 

/******************************************************************************
 * FUNCTION PURPOSE:  Open TSU module
 ******************************************************************************
 * DESCRIPTION: Opens one instance of a TSU module
 *
 * siu_open_tsu (
 *   tint *chnum                 - instance pointer 
 *   siuCodecCfg_t *codecCfg)    - configuration for the codec
 *****************************************************************************/
void siu_open_tsu (tint chnum, tuint targetImgWidth, tuint targetImgHeight)
{
  siuInst_t *inst;
  siuVideoChannelOperation_t chType;
  tsuCfg_t tsuCfg;

  /* initialize the siu instance */
  inst = &siuInst[chnum-SIU_FIRSTCH];
  chType       = (siuVideoChannelOperation_t) SIU_GET_VIDEO_CHANNEL_TYPE(inst);

  if (chType != SIU_CHTYPE_ENCODER) {
    /* TSU is opened only on Encoder Channels */
    return;
  }

  /* memset tsu Config to zero */
  memset(&tsuCfg,0,sizeof(tsuCfg_t));

  tsuCfg.mode       = TSU_POLYPHASE_INTERP;
  /* We don't know the source image resolution */
  tsuCfg.in_width   = 0;
  tsuCfg.in_height  = 0;

  /* Set the target channel's image resolution */
  tsuCfg.out_width  = targetImgWidth;
  tsuCfg.out_height = targetImgHeight;

  /* open tsu channel */
  if (tsuOpen (inst->stateInst.video.tsuInst,&tsuCfg) != TSU_NOERR)
     siu_exc_assert (0, SIUTSU_EXC_OPEN_TSU, inst);

} /* siu_open_tsu */ 
/******************************************************************************
 * FUNCTION PURPOSE:  Scan through TSU instances to find one with already created
 *                    coefficients (optimization only)
 ******************************************************************************
 * DESCRIPTION: Opens one instance of a TSU module
 *
 * siu_tsu_coeff_avail (
 *   tuint ID             Input: TSU instance ID
 *   tsuMode_t mode       Input: mode (bicubic or polyphase)
 *   tuint in_width       Input: input resolution, w   
 *   tuint in_height      Input: input resolution, h   
 *   tuint out_width      Input: output resolution, w   
 *   tuint out_height     Input: output resolution, h   
 *   void **partner_inst  Output - pointer to coefficients instance in another channel
 *   )                    Returns: TRUE  - if set of coefficients is found
 *                                 FALSE - coefficients are not found
 *****************************************************************************/
tbool siu_tsu_coeff_avail (tuint ID, tsuMode_t mode, tuint in_width, tuint in_height, tuint out_width, tuint out_height, void **partner_inst)
{
  tsuCtrl_t tsuCtl;
  siuInst_t *inst;
  void *src_inst;
  tint i;
  tint dst_chan = (ID & 0x00FF) - 1;

  for (i = 0; i < (tint)siuContext.maxNumberChannels; i++) 
  {
    if((i != dst_chan) && SIUINSTpNOTCLOSED(i))
    {
      inst = SIUINSTp(i);
      if (inst->state != SIU_CHST_VIDEO) continue;
      if (SIU_GET_VIDEO_CHANNEL_TYPE(inst) != SIU_CHTYPE_ENCODER) continue;
      src_inst = inst->stateInst.video.tsuInst;
      if(src_inst == NULL) continue;

      /* Query input and output resolutions */
      tsuControl(src_inst, TSU_CTL_QUERY_FILTER, &tsuCtl);
      if(tsuCtl.u.fltData.mode      == mode        &&
         tsuCtl.u.fltData.in_width  == in_width    && 
         tsuCtl.u.fltData.in_height == in_height   &&
         tsuCtl.u.fltData.out_width  == out_width  && 
         tsuCtl.u.fltData.out_height == out_height )
      {  /* Return pointer to coefficients area located in another channel (identical filter) */
         *partner_inst = tsuCtl.u.fltData.fltInstance;
         return TRUE;
      }
    } 
  }
  *partner_inst = NULL;
  return FALSE;
}

/******************************************************************************
 * FUNCTION PURPOSE:  TSU process
 ******************************************************************************
 * DESCRIPTION: Opens one instance of a TSU module
 *
 * siu_open_tsu (
 *   tint *chnum                 - instance pointer 
 *   siuCodecCfg_t *codecCfg)    - configuration for the codec
 *****************************************************************************/
tint siu_tsu_process (void *siuInst, word *yuvInputFrame, tuint yuvInputWidth, 
                      tuint yuvInputHeight, word **yuvTransSizedFrame)
{
    siuInst_t *inst = (siuInst_t *)siuInst;
    tulong dstYUVframeSize = (inst->yuvWidth * inst->yuvHeight * 3)>>1;
    bool skipEncode;
    tint retval;

    if(SIU_GET_VIDEO_CHANNEL_EXT_TYPE(inst) == SIU_CHTYPE_EXT_TSU_ONLY) {
        /* Get free buffer from YUV pool */
        skipEncode = siuVideoGetYUVBuffer(inst);
        *yuvTransSizedFrame = inst->yuvBuffer;
        if(skipEncode == TRUE) {
            inst->yuvBuffer = NULL;
            thwCriticalBegin();
            siuYuvSetBufFree(inst, *yuvTransSizedFrame);
            thwCriticalEnd();
            retval = TSU_NOMEMORY;
            inst->yuvDropCtr++;
            return(retval);
      }
    } else {
        /* Check if scratch buffer is already allocated that can be used, use strict greater than
           else, GMP memory will be allocated every frame and that is expensive */
        if (dstYUVframeSize > siuContext.max_video_tsu_scratch_size) {
          if (siuContext.siu_video_tsu_scratch != NULL) {
            /* If there is memory allocated already, free it and allocate bigger scratch */
            gmpFreeGmc(inst->ID,siuContext.siuYuvMemHandle,siuContext.siu_video_tsu_scratch);
          }
          siuContext.siu_video_tsu_scratch = (word *)gmpPartialAlloc(inst->ID, siuContext.siuYuvMemHandle,dstYUVframeSize);
          siu_exc_assert((siuContext.siu_video_tsu_scratch != NULL), SIUVIGDK_TX_YUV_ALLOC_ERROR, inst);
          siuContext.max_video_tsu_scratch_size = dstYUVframeSize;
        }
        *yuvTransSizedFrame = siuContext.siu_video_tsu_scratch;
    }

    retval = tsuProcess(inst->stateInst.video.tsuInst, yuvInputFrame,
                        yuvInputWidth, yuvInputHeight, *yuvTransSizedFrame);
    return(retval);

}


/* Nothing past this point */
