/******************************************************************************
 * FILE PURPOSE: Contains functions necessary to integrate VPPU
 ******************************************************************************
 * FILE NAME: siuvppu.c
 *
 * DESCRIPTION: Contains functions necessary to integrate VPPU into
 *              system, such that it can be call tabled out of the system
 *
 * (C) Copyright 2008, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
 
/* System utility files */
#include "types.h"
#include "mgmicdsp.h"
#include "ggcfg.h"
/* SIU files */
#include "common/siuloc.h"
#include "siuvppu.h"
#include "vctypes.h"
 
/* Module files */
#include <ti/mas/vppu/vppu.h>
#include "vtk.h"
#include "util.h"
#include "ggmbuf.h"
#include "utlrepck_m.h" 

#pragma DATA_SECTION (codecRfcMap, ".codecRfcMap");
codecRfcMapping_t codecRfcMap[] = {
    { EVCT_VIDEO_MPEG4SP_DEC, "RFC3016"},
    { EVCT_VIDEO_MPEG4SP_ENC, "RFC3016"},
    { EVCT_VIDEO_H263P_DEC,   "RFC2429"},
    { EVCT_VIDEO_H263P_ENC,   "RFC2429"},
    { EVCT_VIDEO_H264BP_MP_DEC,  "RFC3984"},
    { EVCT_VIDEO_H264HP_DEC,  "RFC3984"},
    { EVCT_VIDEO_H264BP_ENC,  "RFC3984"},
    { EVCT_VIDEO_MPEG2MP_DEC, "RFC2250"},
    { EVCT_VIDEO_BYPASS_DEC,  "RFC4175"},
    { EVCT_VIDEO_BYPASS_ENC,  "RFC4175"},
    { EVCT_VIDEO_OVLY_TEXT,   "RFC4396"},
    { -1                  ,   "NULL"   }
};

/******************************************************************************
 * FUNCTION PURPOSE: Return the RFC name given coding type
 ******************************************************************************
 * DESCRIPTION: Return the RFC name given coding type
 *
 *  siuVppuGetRfcName (
 *    tuint  codingType    - coding Type
 *    )
 *****************************************************************************/
char* siuVppuGetRfcName (tuint codingType) {
    codecRfcMapping_t *mapPtr = codecRfcMap;
    while ((mapPtr->codingType != codingType) && (mapPtr->codingType != -1)){
       mapPtr++;
    }
    return((char *)mapPtr->rfcName);
}

/******************************************************************************
 * FUNCTION PURPOSE: Control Video Protocol Processing Unit (VPPU) module 
 ******************************************************************************
 * DESCRIPTION: 
 *
 *  siu_vppu_control (
 *    tint  chnum,    - channel number
 *    mgb_t *message) - message from host
 *****************************************************************************/
void siu_vppu_control (tint chnum, mgb_t *message) {
  siuInst_t    *inst;
  vppuCtrl_t    ctrl;

  /* initialize the siu instance */
  inst = &siuInst[chnum-SIU_FIRSTCH];

  vppuControl(inst->stateInst.video.vppuInst, &ctrl);
  return;
}

/******************************************************************************
 * FUNCTION PURPOSE: Create Video Protocol Processing Unit (VPPU) module 
 ******************************************************************************
 * DESCRIPTION: 
 *
 *  siu_new_vppu (
 *    tint  chnum)  - channel number
 *
 *****************************************************************************/
void siu_new_vppu (tint chnum)
{
  tint         stat;
  memBuffer_t  vppuBufs[SIU_MAX_NBUFS];
  vppuNewCfg_t vppuCfgNew;
  vppuSizeCfg_t vppuCfgSize;
  siuInst_t    *inst;
  
  /* initialize the siu instance */
  inst = &siuInst[chnum-SIU_FIRSTCH];
  
  /* configure the instance */
  vppuCfgNew.ID             = siuMakeID (SIU_MID_VPPU, chnum);
  vppuCfgSize.txScratchSize = 1600; /* Max MTU Size */

  vppuCfgNew.sizeCfg        = vppuCfgSize;
  stat = siuNewEco (inst, (siuEcoGetSizes_t)vppuGetSizes, &vppuCfgSize, 
                    (siuEcoNew_t)vppuNew, &vppuCfgNew, 
                    &(inst->stateInst.video.vppuInst),  vppuBufs, SIU_MID_VPPU);
  
  siu_exc_assert (stat == SIU_NOERR, SIUVPPU_EXC_NEW_VPPU, inst);
} /* siu_new_vppu */ 

/******************************************************************************
 * FUNCTION PURPOSE:  Open VPPU module
 ******************************************************************************
 * DESCRIPTION: Opens one instance of a VPPU module
 *
 * siu_open_vppu (
 *   tint *chnum                 - instance pointer 
 *   siuCodecCfg_t *codecCfg)    - configuration for the codec
 *****************************************************************************/
void siu_open_vppu (siuInst_t *inst, siuCodecCfg_t *codecCfg)
{
  vppuCfg_t vppuCfg;
  siuVideoChannelOperation_t chType;
  tuint codingType;

  /* memset vppu Config to zero */
  memset(&vppuCfg,0,sizeof(vppuCfg_t));

  /* set up the VideoOut targetInst to post task from ISR */
  vppuCfg.vppuRxVideoOut.targetInst      = inst;
  
  /* default configuration of VPPU is open receive out */
  vppuCfg.vppuSendOut.vppuSendOut = gg_rcu_ctable.rcuSendIn;
  vppuCfg.vppuSendOut.targetInst = inst->net.rcuInst;
  vppuCfg.imgWidth   = codecCfg->maxImgWidth;
  vppuCfg.imgHeight  = codecCfg->maxImgHeight;
  vppuCfg.frameRate  = codecCfg->maxFrameRate;
  vppuCfg.maxBitRate = codecCfg->maxBitRate;
  vppuCfg.inputStreamFormat = codecCfg->inputStreamFormat;
  
  vppuCfg.rfcName = siuVppuGetRfcName(codecCfg->codingType);
  siu_exc_assert ((strcmp(vppuCfg.rfcName, "NULL")!=0), SIUVPPU_EXC_OPEN_VPPU, inst);

  chType     = (siuVideoChannelOperation_t) SIU_GET_VIDEO_CHANNEL_TYPE(inst);
  codingType = (tuint)SIU_GET_VIDEO_CODING_TYPE(inst);

  if (codingType == EVCT_VIDEO_BYPASS_ENC) {
    /* overwrite for Encoder Bypass, as codecCfg is invalid */
    vppuCfg.imgWidth  = inst->yuvWidth; 
    vppuCfg.imgHeight = inst->yuvHeight;
  }

  if (chType == SIU_CHTYPE_ENCODER) {
    vppuCfg.type = VPPU_TYPE_TX;
  } else {
    vppuCfg.type = VPPU_TYPE_RX;
    vppuCfg.dropPartialFrames = FALSE;
  }
  
  vppuCfg.payloadType = inst->payloadType;
  vppuCfg.MTUsize     = inst->maxPayloadSize;

  /* open vppu channel */
  if (vppuOpen (inst->stateInst.video.vppuInst,&vppuCfg) != VPPU_NOERR)
     siu_exc_assert (0, SIUVPPU_EXC_OPEN_VPPU, inst);

} /* siu_open_vppu */ 

/******************************************************************************
 * FUNCTION PURPOSE: Returns VPPU statistics
 ******************************************************************************
 * DESCRIPTION: 
 ******************************************************************************/
void siuVppuGetStats (void *siuInst, bool reset, tuint trans_id, tuint priority)
{
  siuInst_t *inst = (siuInst_t *)siuInst;
  vppuStats_t vppuStats;
  struct ifvppu_GET_RX_STAT   *rxStats;
  struct ifvppu_GET_TX_STAT   *txStats;
  vppuRxStats_t *vppuRxStats;
  vppuTxStats_t *vppuTxStats;
  siuVideoChannelOperation_t chType;
  mgb_t send_message;

  if(!SIU_IS_PROTO_RTP(inst)) {
    siu_get_stats_unavail(siuInst,reset,trans_id,priority);
    return;
  }
  
  send_message.header.length   = MGB_HEADER_SIZE + sizeof(struct ifvppu_GET_STAT)
                                               / sizeof(UINT16);
  send_message.header.chnum    = (int)(inst->ID & SIU_CID_MASK);
  send_message.header.msg_id   = MGB_DM_GET_VPPU_STAT;
  send_message.header.trans_id = trans_id;

  chType    = (siuVideoChannelOperation_t) SIU_GET_VIDEO_CHANNEL_TYPE(inst);

  /* Not using ecoif for stats due to endian translation */
  vppuGetStats(inst->stateInst.video.vppuInst, reset, &vppuStats);

  if (chType == SIU_CHTYPE_DECODER) {
     send_message.msg_body.vppu_stat.vppuType = ifvppu_GET_TYPE_RX;
     rxStats                                 = &(send_message.msg_body.vppu_stat.info.rx);
     rxStats->codingType                     = SIU_GET_VIDEO_CODING_TYPE(inst);
     vppuRxStats                             = &(vppuStats.stats.rx);
     rxStats->dupSeqNumDrops                 = vppuRxStats->dupSeqNumDrops;
     rxStats->numLostPkts                    = vppuRxStats->numLostPkts;
     rxStats->numStreamSwitches              = vppuRxStats->numStreamSwitches;
     rxStats->numPktsDroppedMSW              = siu_get_umsw(vppuRxStats->numPktsDropped);
     rxStats->numPktsDroppedLSW              = siu_get_ulsw(vppuRxStats->numPktsDropped);
     rxStats->numOutofOrderPktsMSW           = siu_get_umsw(vppuRxStats->numOutofOrderPkts);
     rxStats->numOutofOrderPktsLSW           = siu_get_ulsw(vppuRxStats->numOutofOrderPkts);
     rxStats->rxNetPktsMSW                   = siu_get_umsw(vppuRxStats->rxNetPkts);
     rxStats->rxNetPktsLSW                   = siu_get_ulsw(vppuRxStats->rxNetPkts);
     rxStats->rxNetFramesRecvdMSW            = siu_get_umsw(vppuRxStats->rxNetFramesRecvd);
     rxStats->rxNetFramesRecvdLSW            = siu_get_ulsw(vppuRxStats->rxNetFramesRecvd);
     rxStats->rxNetFramesRFCdoneMSW          = siu_get_umsw(vppuRxStats->rxNetFramesRFCdone);
     rxStats->rxNetFramesRFCdoneLSW          = siu_get_ulsw(vppuRxStats->rxNetFramesRFCdone);
     rxStats->minFrameInterArrivalTimeMSW    = siu_get_umsw(vppuRxStats->minFrameInterArrivalTime);
     rxStats->minFrameInterArrivalTimeLSW    = siu_get_ulsw(vppuRxStats->minFrameInterArrivalTime);
     rxStats->maxFrameInterArrivalTimeMSW    = siu_get_umsw(vppuRxStats->maxFrameInterArrivalTime);
     rxStats->maxFrameInterArrivalTimeLSW    = siu_get_ulsw(vppuRxStats->maxFrameInterArrivalTime);
     rxStats->maxPktsRcvdPerSecond           = vppuRxStats->maxPktsRcvdPerSecond;
     rxStats->currNumPendingFrames           = vppuRxStats->currNumPendingFrames;
     rxStats->maxNumPendingFrames            = vppuRxStats->maxNumPendingFrames;
     rxStats->currNumPendingPackets          = vppuRxStats->currNumPendingPackets;
     rxStats->maxNumPendingPackets           = vppuRxStats->maxNumPendingPackets;
     rxStats->playout_fps                    = vppuRxStats->playout_fps;
     rxStats->wire_fps                       = vppuRxStats->wire_fps;
     rxStats->lastSeqNum                     = vppuRxStats->lastSeqNum;
     rxStats->lastTimeStampMSW               = siu_get_umsw(vppuRxStats->lastTimeStamp);
     rxStats->lastTimeStampLSW               = siu_get_ulsw(vppuRxStats->lastTimeStamp);
  } else {
     send_message.msg_body.vppu_stat.vppuType = ifvppu_GET_TYPE_TX;
     txStats                       = &(send_message.msg_body.vppu_stat.info.tx);
     txStats->codingType           = SIU_GET_VIDEO_CODING_TYPE(inst);
     vppuTxStats                   = &(vppuStats.stats.tx);
     txStats->numTxFramesMSW       = siu_get_umsw(vppuTxStats->numTxFrames);
     txStats->numTxFramesLSW       = siu_get_ulsw(vppuTxStats->numTxFrames);
     txStats->txNetPktsMSW         = siu_get_umsw(vppuTxStats->txNetPkts);
     txStats->txNetPktsLSW         = siu_get_ulsw(vppuTxStats->txNetPkts);
     txStats->wire_fps             = vppuTxStats->wire_fps;
     txStats->wire_fps_jitter      = vppuTxStats->wire_fps_jitter;
     txStats->lastSeqNum           = vppuTxStats->lastSeqNum;
     txStats->lastTimeStampMSW     = siu_get_umsw(vppuTxStats->lastTimeStamp);
     txStats->lastTimeStampLSW     = siu_get_ulsw(vppuTxStats->lastTimeStamp);
  }

  /* send message to FIFO */
  siuNduSendMsgPriority(&send_message, (NDU_DPRMsgType_t)priority); 
} /* siuVppuGetStats */

/* Nothing past this point */
