/******************************************************************************
 * FILE PURPOSE: Contains functions necessary to integrate VTK
 ******************************************************************************
 * FILE NAME: siuvtk.c
 *
 * DESCRIPTION: Contains functions necessary to integrate VTK into
 *              system, such that it can be call tabled out of the system
 *
 * (C) Copyright 2008, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
 
/* System utility files */
#include "types.h"
#include "mgmicdsp.h"
#include "ggcfg.h"

/* SIU files */
#include "common/siuloc.h"
#include "ividmc/siuVidMc.h"
#include "vctypes.h"
/* Module files */
#include "vtk.h"
#include "util.h"
#include "utlrepck.h" /* for utlPackTuintToTulong */
#include "ggmbuf.h"

/******************************************************************************
 * FUNCTION PURPOSE: Create Video Protocol Processing Unit (VTK) module 
 ******************************************************************************
 * DESCRIPTION: 
 *
 *  siu_new_vtk (
 *    tint  chnum)  - channel number
 *
 *****************************************************************************/
void siu_new_vtk (tint chnum) {
  tint         stat;
  memBuffer_t  vtkBufs[SIU_MAX_NBUFS];
  vtkNewCfg_t vtkCfgNew;
  vtkSizeCfg_t vtkCfgSize;
  siuInst_t    *inst;
  
  /* initialize the siu instance */
  inst = &siuInst[chnum-SIU_FIRSTCH];
  
  /* configure the instance */
  vtkCfgNew.ID             = siuMakeID (SIU_MID_VTK, chnum);
  
  stat = siuNewEco (inst, (siuEcoGetSizes_t)vtkGetSizes, &vtkCfgSize, 
                    (siuEcoNew_t)vtkNew, &vtkCfgNew, 
                    &(inst->stateInst.video.vtkInst),  vtkBufs, SIU_MID_VTK);
  
  siu_exc_assert (stat == SIU_NOERR, SIUVTK_EXC_NEW_VTK, inst);
} /* siu_new_vtk */ 

/******************************************************************************
 * FUNCTION PURPOSE:  Open VTK module
 ******************************************************************************
 * DESCRIPTION: Opens one instance of a VTK module
 *
 * siu_open_vtk (
 *   tint chnum            - Channel number
 *   tint codingType       - Coding Type
 *   tint width            - Image Width
 *   tint height           - Image Height
 *   tint maxDisplayDelay) - Max Display Delay
 *
 *****************************************************************************/
void siu_open_vtk (tint chnum, tuint codingType, tuint width, tuint height, word maxDisplayDelay) {
  vtkCfg_t vtkCfg;
  siuInst_t *inst;
  vtkCtrl_t ctrl;

  /* initialize the siu instance */
  inst = &siuInst[chnum-SIU_FIRSTCH];
  
  /* default configuration of VTK is open receive out */
  memset(&vtkCfg, 0, sizeof(vtkCfg_t));
  vtkCfg.coreId          = siuContext.coreId;
  vtkCfg.codingType      = codingType;
  vtkCfg.img_width       = width;
  vtkCfg.img_height      = height;
  vtkCfg.sysInst         = (void *)inst;
  vtkCfg.maxDisplayDelay = maxDisplayDelay;
  vtkCfg.encOutBufSize   = siuContext.max_codec_frame_size;
  vtkCfg.decFlush        = FALSE;

  /* open vtk channel */
  if (vtkOpen (inst->stateInst.video.vtkInst,&vtkCfg) != VTK_NOERR) {
     siu_exc_assert (0, SIUVTK_EXC_OPEN_VTK, inst);
  }

  /* Initialization of multi-core params */
  ctrl.code = VTK_SET_MC_PARAMS;
  ctrl.u.vtkMcParams.swbarr        = siuVidMc_Swbarr;
  ctrl.u.vtkMcParams.shmmap        = siuVidMc_Map_Shmem;
  ctrl.u.vtkMcParams.shmmunmap     = siuVidMc_Unmap_Shmem;
  ctrl.u.vtkMcParams.shmmap_sync   = siuVidMc_Shmem_Sync;
  ctrl.u.vtkMcParams.Lock          = NULL;
  ctrl.u.vtkMcParams.UnLock        = NULL;
  ctrl.u.vtkMcParams.ncores        = inst->mcInst.nCores;
  ctrl.u.vtkMcParams.core_task_ID  = (SIU_IS_MCCODEC_MASTER(inst))?IVIDMC_TASK_MASTER:IVIDMC_TASK_SLAVE;
  ctrl.u.vtkMcParams.coreID        = siuContext.coreId;
  vtkControl(inst->stateInst.video.vtkInst, &ctrl);

} /* siu_open_vtk */

/******************************************************************************
 * FUNCTION PURPOSE:  siu_commit_vtk
 ******************************************************************************
 * DESCRIPTION: Commits one instance of a VTK module
 *
 * siu_commit_vtk (
 *   siuInst_t     *inst         - instance pointer 
 *   siuCodecCfg_t *siuCodecCfg  - retrieve codec configuration
 *                )
 *****************************************************************************/
void siu_commit_vtk (siuInst_t *inst, siuCodecCfg_t *siuCodecCfg) {
  vtkCtrl_t ctrl;

  /* commit VTK and get codec configuration from VTK */
  ctrl.code  = VTK_COMMIT_CODEC;
  siu_exc_assert ((vtkControl(inst->stateInst.video.vtkInst, &ctrl) == VTK_NOERR), 
                   SIUVTK_EXC_COMMIT_VTK | SIU_EXCEPTION_TYPE_CHANNEL, inst);

  ctrl.code = VTK_GET_CFG;
  vtkControl(inst->stateInst.video.vtkInst, &ctrl);

  /* copy the codec configuration to SIU */
  siuCodecCfg->codingType    = ctrl.u.vtkCfg.codingType;
  if (ctrl.u.vtkCfg.codingType == EVCT_VIDEO_BYPASS_DEC) {
    siuCodecCfg->maxImgWidth   = ctrl.u.vtkCfg.img_width;
    siuCodecCfg->maxImgHeight  = ctrl.u.vtkCfg.img_height;
  } else {
    siuCodecCfg->maxImgWidth   = ctrl.u.vtkCfg.maxImgWidth;
    siuCodecCfg->maxImgHeight  = ctrl.u.vtkCfg.maxImgHeight;
  }
  siuCodecCfg->maxFrameRate  = ctrl.u.vtkCfg.maxFrameRate;
  siuCodecCfg->maxBitRate    = ctrl.u.vtkCfg.maxBitRate;
  siuCodecCfg->inputStreamFormat = ctrl.u.vtkCfg.inputStreamFormat;

  return;

}

/******************************************************************************
 * DESCRIPTION: Get VTK stats 
 *
 * siu_open_vtk (
 *   tint *chnum)         - instance pointer 
 *
 *****************************************************************************/
void siuVtkGetStats (void *siuInst, bool reset, tuint trans_id, tuint priority) {
  siuInst_t *inst = (siuInst_t *)siuInst;
  vtkStats_t vtkStats;
  vtkEncDecStats_t *decStats;
  vtkEncDecStats_t *encStats;
  struct ifvtk_GET_RX_STAT *rxStats;
  struct ifvtk_GET_TX_STAT *txStats;
  siuVideoChannelOperation_t chType;
  mgb_t send_message;
  vtkCtrl_t ctrl;
 
  send_message.header.length   = MGB_HEADER_SIZE + sizeof(struct ifvtk_GET_STAT)
                                               / sizeof(UINT16);
  send_message.header.chnum    = (int)(inst->ID & SIU_CID_MASK);
  send_message.header.msg_id   = MGB_DM_GET_VTK_STAT;
  send_message.header.trans_id = trans_id;

  chType    = (siuVideoChannelOperation_t) SIU_GET_VIDEO_CHANNEL_TYPE(inst);

  ctrl.code = VTK_GET_CFG;
 
  /* Not using ecoif for stats due to endian translation */
  if (vtkControl(inst->stateInst.video.vtkInst, &ctrl) == VTK_ERROR) {
    /* codec didn't receive static/dynamic configuration yet: vtk not opened */
    siuErrorIndication(inst->ID, trans_id, MGB_ERR_VIDEO_INVD_STATS_REQUEST);
    return;
  }

  vtkGetStats(inst->stateInst.video.vtkInst, reset, &vtkStats);

  if (chType == SIU_CHTYPE_DECODER) {
    send_message.msg_body.vtk_stat.vtkType = ifvtk_GET_STAT_TYPE_RX;
    decStats                   = &(vtkStats.dec);
    rxStats                    = &(send_message.msg_body.vtk_stat.info.rx);
    rxStats->codingType        = ctrl.u.vtkCfg.codingType;
    rxStats->imgWidth          = inst->yuvWidth;
    rxStats->imgHeight         = inst->yuvHeight;
    rxStats->numIframesMSW     = siu_get_umsw(decStats->numIFrames);   /* received I-frames */
    rxStats->numIframesLSW     = siu_get_ulsw(decStats->numIFrames);
    rxStats->numPframesMSW     = siu_get_umsw(decStats->numPFrames);   /* received P-frames */
    rxStats->numPframesLSW     = siu_get_ulsw(decStats->numPFrames); 
    rxStats->numBframesMSW     = siu_get_umsw(decStats->numBFrames);   /* received B-frames */
    rxStats->numBframesLSW     = siu_get_ulsw(decStats->numBFrames); 
    rxStats->maxIframeSizeMSW  = siu_get_umsw(decStats->maxIFrameSize);/* maximum I-frame size */
    rxStats->maxIframeSizeLSW  = siu_get_ulsw(decStats->maxIFrameSize);
    rxStats->maxPframeSizeMSW  = siu_get_umsw(decStats->maxPFrameSize);/* maximum P-frame size */
    rxStats->maxPframeSizeLSW  = siu_get_ulsw(decStats->maxPFrameSize);
    rxStats->maxBframeSizeMSW  = siu_get_umsw(decStats->maxBFrameSize);/* maximum B-frame size */
    rxStats->maxBframeSizeLSW  = siu_get_ulsw(decStats->maxBFrameSize);
    rxStats->numDecBytesMSW    = siu_get_umsw(decStats->totalNumBytes);/* num of bytes dec consumed */
    rxStats->numDecBytesLSW    = siu_get_ulsw(decStats->totalNumBytes);
    rxStats->lastGOP           = decStats->lastGOP;      /* last inter I-frames gap (last GOP) */
    rxStats->bitRateMSW        = siu_get_umsw(decStats->bitrate);/* estimated average bps (Q4) */
    rxStats->bitRateLSW        = siu_get_ulsw(decStats->bitrate); 
    rxStats->numErrFramesMSW   = siu_get_umsw(decStats->errorFrames);/* counter of undecodable frames */
    rxStats->numErrFramesLSW   = siu_get_ulsw(decStats->errorFrames);    
    rxStats->decLastErrCodeMSW = siu_get_umsw(decStats->lastErrorCode);/* extended errors, decoder dependent */
    rxStats->decLastErrCodeLSW = siu_get_ulsw(decStats->lastErrorCode);
  } else {
    send_message.msg_body.vtk_stat.vtkType = ifvtk_GET_STAT_TYPE_TX;
    encStats                   = &(vtkStats.enc);
    txStats                    = &(send_message.msg_body.vtk_stat.info.tx);
    txStats->codingType        = ctrl.u.vtkCfg.codingType;
    txStats->imgWidth          = inst->yuvWidth;
    txStats->imgHeight         = inst->yuvHeight;
    txStats->numIframesMSW     = siu_get_umsw(encStats->numIFrames);   /* transmitted I-frames */
    txStats->numIframesLSW     = siu_get_ulsw(encStats->numIFrames);
    txStats->numPframesMSW     = siu_get_umsw(encStats->numPFrames);   /* transmitted P-frames */
    txStats->numPframesLSW     = siu_get_ulsw(encStats->numPFrames);
    txStats->numBframesMSW     = siu_get_umsw(encStats->numBFrames);   /* transmitted B-frames */
    txStats->numBframesLSW     = siu_get_ulsw(encStats->numBFrames);
    txStats->maxIframeSizeMSW  = siu_get_umsw(encStats->maxIFrameSize);/* maximum I-frame size */
    txStats->maxIframeSizeLSW  = siu_get_ulsw(encStats->maxIFrameSize);
    txStats->maxPframeSizeMSW  = siu_get_umsw(encStats->maxPFrameSize);/* maximum P-frame size */
    txStats->maxPframeSizeLSW  = siu_get_ulsw(encStats->maxPFrameSize);
    txStats->maxBframeSizeMSW  = siu_get_umsw(encStats->maxBFrameSize);/* maximum B-frame size */
    txStats->maxBframeSizeLSW  = siu_get_ulsw(encStats->maxBFrameSize);
    txStats->numEncBytesMSW    = siu_get_umsw(encStats->totalNumBytes);/* num of bytes dec consumed */
    txStats->numEncBytesLSW    = siu_get_ulsw(encStats->totalNumBytes);
    txStats->lastGOP           = encStats->lastGOP;    /* last inter I-frames gap (last GOP) */
    txStats->bitRateMSW        = siu_get_umsw(encStats->bitrate);/* estimated average bps (Q4) */
    txStats->bitRateLSW        = siu_get_ulsw(encStats->bitrate);
    txStats->skippedFramesMSW  = siu_get_umsw(encStats->skippedFrames);/* skipped frames due to rateControl */
    txStats->skippedFramesLSW  = siu_get_ulsw(encStats->skippedFrames);
    txStats->numErrFramesMSW   = siu_get_umsw(encStats->errorFrames);  /* counter of unencodable frames */
    txStats->numErrFramesLSW   = siu_get_ulsw(encStats->errorFrames);  /* counter of unencodable frames */
    txStats->encLastErrCodeMSW = siu_get_umsw(encStats->lastErrorCode);/* extended errors, encoder dependent */
    txStats->encLastErrCodeLSW = siu_get_ulsw(encStats->lastErrorCode);
  }

  /* send message to FIFO */
  siuNduSendMsgPriority(&send_message, (NDU_DPRMsgType_t)priority); 

}

/* Nothing past this point */
