/******************************************************************************
 * FILE PURPOSE: Multi-core functionality.
 ******************************************************************************
 * FILE NAME: siuVidMc.c
 *
 * DESCRIPTION: This file contains API implemenations for multi-core codecs.
 *              There are two major functionalities: 1) software barrier; 2)
 *              global from a shared region which need to be accessed by
 *              multiple cores
 *
 * TABS: NONE
 *
 *
 * (C) Copyright 2010, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include "siuVidMc.h"
#include "osal/siuIpcOsal.h"
#include "osal/siuOsal.h"
#include "common/siuloc.h"
#ifdef EMU_PROFILE
#include "emureg.h"
#endif

/*********************************************************************************
 * FUNCTION PURPOSE: Software barrier between multiple cores
 *********************************************************************************
  DESCRIPTION:      This function is a wrapper of software barrier which is 
                    implemented in BIOS IPC. Codec generates call-out and sends 
                    number (swbarr_cnt) of cores from the team (<ncores) that need
                    to reach the software barrier, before execution proceeds. 
                    It is possible to have multiple sync points in the system 
                    present simultaneosly. This is why codec needs to identify 
                    barrier with swbarr_id parameter 

                    In the current implemenation of software barrier based on 
                    BIOS IPC, swbarr_cnt has been passed during the create/open
                    time, while core_id is not needed.

  Parameters :      Inputs: core_id         : core ID
                            sync_point      : synchronization point
                            swbarr_cnt      : number of cores in barrier
                    Output: return value    : None
                                              
 *********************************************************************************/
XDAS_Void siuVidMc_Swbarr(XDAS_Int32 core_id, IVIDMC_SWBARR sync_point, XDAS_UInt32 swbarr_cnt)
{
  if(swbarr_cnt > 1) {
#ifdef EMU_PROFILE
    /*This profiling point is enabled only for getting barrier wait details of encode/decode */
    //EMU_PROFILE_BEGIN(EMU_TASK_ID_BARWAIT,0)
#endif
    siu_ipc_osal_barrier_wait(sync_point);
#ifdef EMU_PROFILE
    //EMU_PROFILE_END() /* EMU_TASK_ID_BARWAIT */
#endif
  }
}

/*********************************************************************************
 * FUNCTION PURPOSE: Initialize shared memory Tab 
 *********************************************************************************
  DESCRIPTION:      This function is called to intialize share memory Tab which 
                    can be accessed by multiple cores

  Parameters :      Inputs: Core_ID         : core ID
                    Output: return value    : XDM_EOK
                                              
 *********************************************************************************/
XDAS_Int32 siuVidMc_Init_ShmemTab(XDAS_Int32 Core_ID)
{ 
  siuVidShmHandle_t *codecShmHandle = &(siuContext.codecShmHandle);
  tuint i, gateKey;
  SIUVID_MC_SHMEM_MemTab *codecShm = (SIUVID_MC_SHMEM_MemTab *)siuContext.ipcContext.codecShm;
  tuint shareSize = sizeof(SIUVID_MC_SHMEM_MemTab);

  /* Enter critical section before accessing codecShm which is on a shared region */
  gateKey = siu_ipc_osal_enter_share((void*)codecShm, shareSize);

  /* Initialize codecShm */
  for(i=0; i<IVIDMC_SHMEM_NUM_KEYS;i++)
  {
    codecShm->attribute[Core_ID][i] = -1;
    codecShm->size[Core_ID][i] = -1;
    codecShm->baseptr[Core_ID][i] = NULL;
  }

  /* Specifying base address and size for sheared memory in SL2 and DDR2*/
  codecShm->shmem_SL2       = codecShmHandle->intShm;
  codecShm->shmemsize_SL2   = codecShmHandle->intShmSize;

  codecShm->shmem_DDR2      = codecShmHandle->extShm;
  codecShm->shmemsize_DDR2  = codecShmHandle->extShmSize;

  codecShm->outputBitStreamPtr[Core_ID]  = NULL;
  codecShm->outputBitStreamSize[Core_ID] = 0;

  /* Leave critical section after accessing codecShm which is on a shared region */
  siu_ipc_osal_leave_share((void*)codecShm, shareSize, gateKey);

  return XDM_EOK;
}


/*********************************************************************************
 * FUNCTION PURPOSE: Write output bit stream info in codecShm
 *********************************************************************************
  DESCRIPTION:      This function is called to write the output bit stream pointer
                    and size in the codec shared memory. It will be called by slave
                    cores. Then the infomation will be used by the master core to 
                    assemble the output from all participating cores.

  Parameters :      Inputs: Core_ID         : core ID
                            bitStreamPtr    : pointer of output bit steam
                            bitStreamSize   : size of output bit steam
                    Output: return value    : None
                                              
 *********************************************************************************/
XDAS_Void siuVidMc_Set_outputBitStreamInfo(XDAS_Int32 Core_ID, XDAS_UInt8 *bitStreamPtr, XDAS_Int32 bitStreamSize)
{ 
  SIUVID_MC_SHMEM_MemTab *codecShm = (SIUVID_MC_SHMEM_MemTab *)siuContext.ipcContext.codecShm;
  tuint shareSize = sizeof(SIUVID_MC_SHMEM_MemTab);
  tuint gateKey;

  /* Enter critical section before accessing codecShm which is on a shared region */
  gateKey = siu_ipc_osal_enter_share((void*)codecShm, shareSize);

  codecShm->outputBitStreamPtr[Core_ID]  = bitStreamPtr;
  codecShm->outputBitStreamSize[Core_ID] = bitStreamSize;
  
  /* Leave critical section after accessing codecShm which is on a shared region */
  siu_ipc_osal_leave_share((void*)codecShm, shareSize, gateKey);
}

/*********************************************************************************
 * FUNCTION PURPOSE: Read output bit stream info from codecShm
 *********************************************************************************
  DESCRIPTION:      This function is called to read the output bit stream pointer
                    and size from the codec shared memory. It will be called by
                    the master core for assembling the output from all participating
                    cores.

  Parameters :      Inputs: Core_ID         : core ID
                            bitStreamPtr    : pointer to pointer of output bit steam
                            bitStreamSize   : pointer to size of output bit steam
                    Output: return value    : None
                                              
 *********************************************************************************/

XDAS_Void siuVidMc_Get_outputBitStreamInfo(XDAS_Int32 Core_ID, XDAS_UInt8 **bitStreamPtr, XDAS_Int32 *bitStreamSize)
{ 
  SIUVID_MC_SHMEM_MemTab *codecShm = (SIUVID_MC_SHMEM_MemTab *)siuContext.ipcContext.codecShm;
  tuint shareSize = sizeof(SIUVID_MC_SHMEM_MemTab);
  tuint gateKey;

  /* Enter critical section before accessing codecShm which is on a shared region */
  gateKey = siu_ipc_osal_enter_share((void*)codecShm, shareSize);

  *bitStreamPtr = codecShm->outputBitStreamPtr[Core_ID];
  *bitStreamSize = codecShm->outputBitStreamSize[Core_ID];
  
  /* Leave critical section after accessing codecShm which is on a shared region */
  siu_ipc_osal_leave_share((void*)codecShm, shareSize, gateKey);
}

/*********************************************************************************
 * FUNCTION PURPOSE: Compute Alignment required for buffer
 *********************************************************************************
  DESCRIPTION:      This function is called to find the alignment bytes
  Parameters :      Inputs: address         : base address
                            alignment       : alignment requirement
                    Output: return value    : number of bytes offset needed to 
                                              meet the alignment requirement
 *********************************************************************************/
tulong siuVidMemAlign(tulong address, tulong alignment) 
{
  tulong alignBytes;
  
  alignBytes  = ((address  + (alignment - 1)) & ~ (alignment - 1));
  alignBytes -= address;
  return(alignBytes);
}


/*********************************************************************************
 * FUNCTION PURPOSE: Allocate codec shared memory region
 *********************************************************************************
  DESCRIPTION:      This function dynamically allocates shared memory region 
                    identified with global key "shmem_key". If region with same
                    global key "shmem_key" is already allocated, its base address
                    is provided. Attribute provided here indicates shared memory
                    region placement: SL2 or DDR. Meaning of alignment is identical
                    to ialg/xdais definition. Having alignment in the API allows
                    the Codec to request a shared buffer with required alignment

  Parameters :      Inputs: coreID       : core ID
                            shmem_key    : global shared memory key
                            attr         : shared memory in SL2 or DDR
                            alignment    : alignment of the buffer
                    Output: shMem        : base address of allocated shared memory
 *********************************************************************************/
XDAS_Int32 *siuVidMc_Map_Shmem(XDAS_Int32 coreID, IVIDMC_SHMEMKEY shmem_key, XDAS_Int32 shmem_size, 
                               IVIDMC_SHMEM_ATTRS attr, XDAS_Int32 alignment)
{
  siuInst_t *inst = SIUINSTp(siuContext.current_working_chnum);
  word *shMem = NULL;
  SIUVID_MC_SHMEM_MemTab *codecShm = (SIUVID_MC_SHMEM_MemTab *)siuContext.ipcContext.codecShm;
  tuint shareSize = sizeof(SIUVID_MC_SHMEM_MemTab);

  unsigned int  alignBytes;
  tuint i, coreIdAllocated, gateKey;

  /* Enter critical section before accessing codecShm which is on a shared region */
  gateKey = siu_ipc_osal_enter_share((void*)codecShm, shareSize);

  /* Check if shared memory with the same key has been allocated. 
     If so, record the coreID                                    */
  for (i = 0; i < inst->mcInst.nCores; i++) {
    if(codecShm->baseptr[inst->mcInst.coreTeamMapping[i]][shmem_key] != NULL) {
      coreIdAllocated = inst->mcInst.coreTeamMapping[i];
      break;
    }
  }

  if (i == inst->mcInst.nCores) { 
    /* Shared memory with the same key has not been allocated on any of the participating cores,
       so allocate shared memory for the first time */
    if( attr == IVIDMC_SHMEM_ATTRS_DDR_CACHED ) {
      /* Allocate shared DDR2*/
      alignBytes = siuVidMemAlign((tulong)(codecShm->shmem_DDR2), alignment);
      codecShm->shmem_DDR2 += alignBytes;
      codecShm->shmemsize_DDR2 -= alignBytes;
      
      if (codecShm->shmemsize_DDR2 > shmem_size) {
        shMem = codecShm->shmem_DDR2;
        codecShm->shmem_DDR2 += shmem_size;
        codecShm->shmemsize_DDR2 -= shmem_size;
      } else {
        siu_exc_assert (0, SIUMGR_EXC_EXT_SHM_OVERFLOW | SIU_EXCEPTION_TYPE_CHANNEL, inst);
      }
    }
    else { 
      /* Allocate shared SL2*/
      alignBytes = siuVidMemAlign((tulong)(codecShm->shmem_SL2), alignment);
      codecShm->shmem_SL2 += alignBytes;
      codecShm->shmemsize_SL2 -= alignBytes;
      
      if (codecShm->shmemsize_SL2 > shmem_size) {
        shMem = codecShm->shmem_SL2;
        codecShm->shmem_SL2 += shmem_size;
        codecShm->shmemsize_SL2 -= shmem_size;
      } else {
        siu_exc_assert (0, SIUMGR_EXC_INT_SHM_OVERFLOW | SIU_EXCEPTION_TYPE_CHANNEL, inst);
      }
    }
    codecShm->baseptr[coreID][shmem_key] = (XDAS_Int32 *)shMem;
    codecShm->size[coreID][shmem_key] = shmem_size;
  } else {
    /* Reuse shared memory with the same key allocated earlier */
    shMem = (XDAS_UInt8 *) codecShm->baseptr[coreIdAllocated][shmem_key];

    codecShm->baseptr[coreID][shmem_key] = (XDAS_Int32 *)shMem;
    codecShm->size[coreID][shmem_key] = shmem_size;
  }

  /* Leave critical section after accessing codecShm which is on a shared region */
  siu_ipc_osal_leave_share((void*)codecShm, shareSize, gateKey);

  return ((XDAS_Int32*)shMem);
}

/*********************************************************************************
 * FUNCTION PURPOSE: Deallocate shared memory region
 *********************************************************************************
  DESCRIPTION:      This function deallocates shared memory region which has been
                    allocated earlier. It is completely recycled after all clients
                    do unmap operation

  Parameters :      Inputs: coreID       : core ID
                            shmem_key    : global shared memory key
                            shmem_base   : base address of shared memory
                    Output: XDM_EOK      : memory deallocation is done
                            XDM_EFAIL    : no memory deallocation is done
 *********************************************************************************/
XDAS_Int32 siuVidMc_Unmap_Shmem(XDAS_Int32 coreID, IVIDMC_SHMEMKEY shmem_key, XDAS_Int32 *shmem_base)
{
  SIUVID_MC_SHMEM_MemTab *codecShm = (SIUVID_MC_SHMEM_MemTab *)siuContext.ipcContext.codecShm;
  tuint gateKey;
  tuint shareSize = sizeof(SIUVID_MC_SHMEM_MemTab);
  XDAS_Int32 retVal;

  /* Enter critical section before accessing codecShm which is on a shared region */
  gateKey = siu_ipc_osal_enter_share((void*)codecShm, shareSize);

  /* Deallocate shared memory */
  if (codecShm->baseptr[coreID][shmem_key] == shmem_base ) {
    codecShm->baseptr[coreID][shmem_key] = NULL;
    retVal = XDM_EOK;
  } else {
    retVal = XDM_EFAIL;
  }

  /* Leave critical section after accessing codecShm which is on a shared region */
  siu_ipc_osal_leave_share((void*)codecShm, shareSize, gateKey);

  return(retVal);
}

/*********************************************************************************
 * FUNCTION PURPOSE: Synchronize/flush shared memory region 
 *********************************************************************************
  DESCRIPTION:      This function synchronizes/flushs shared memory region. 
                    It contains explicit cache coherency operation.

  Parameters :      Inputs: coreID       : core ID
                            shmem_key    : global shared memory key
                            shmem_base   : base address of shared memory
                    Output: XDM_EOK      : synchronization or flush is done
                            XDM_EFAIL    : no cache operation is done
 *********************************************************************************/
XDAS_Int32 siuVidMc_Shmem_Sync(XDAS_Int32 coreID, IVIDMC_SHMEMKEY shmem_key, XDAS_Int32 *shmem_base)
{
  SIUVID_MC_SHMEM_MemTab *codecShm = (SIUVID_MC_SHMEM_MemTab *)siuContext.ipcContext.codecShm;
  tuint shareSize = sizeof(SIUVID_MC_SHMEM_MemTab);
  tuint gateKey;
  XDAS_Int32 retVal;

  /* Enter critical section before accessing codecShm which is on a shared region */
  gateKey = siu_ipc_osal_enter_share((void*)codecShm, shareSize);

  if(shmem_key == IVIDMC_SHMEM_KEY_CLEAN) 
  {
    /* cache writeback and invalidate for all */
    siu_osal_wbinv_cache_all();
    retVal = XDM_EOK;
  }
  else if(codecShm->baseptr[coreID][shmem_key] == shmem_base)
  {
    /* cache writeback and invalidate for the specified memory region */
    siu_osal_wbinv_cache(codecShm->baseptr[coreID][shmem_key], codecShm->size[coreID][shmem_key], TRUE);
    retVal = XDM_EOK;
  }
  else
    retVal = XDM_EFAIL;

  /* Leave critical section after accessing codecShm which is on a shared region */
  siu_ipc_osal_leave_share((void*)codecShm, shareSize, gateKey);

  return(retVal);
}

/*** nothing past this point ***/
