/******************************************************************************
 * FILE PURPOSE: Implement software barrier using BIOS IPC
 ******************************************************************************
 * FILE NAME:   GateMPBarrier.c
 *
 * DESCRIPTION: Contains BIOS IPC implementation of software barrier. Shared
                region, GateMP, and Name Server modules are used.
 *
 * TABS: NONE
 *
 *
 * (C) Copyright 2010, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include <stdio.h>
#include <xdc/std.h>

/* XDC runtime module headers */
#include <xdc/runtime/System.h>
#include <xdc/runtime/Error.h>
#include <xdc/runtime/Memory.h>

/* BIOS IPC headers */
#include <ti/ipc/MultiProc.h>
#include <ti/ipc/GateMP.h>
#include <ti/ipc/Ipc.h>
#include <ti/ipc/SharedRegion.h>
#include <ti/ipc/HeapBufMP.h>
#include <ti/sdo/utils/_NameServer.h>

/* BIOS6 module headers */
#include <ti/sysbios/family/c66/Cache.h>
#include <ti/sysbios/interfaces/ICache.h>

#include "GateMPBarrier.h"

/*********************************************************************************
 * FUNCTION PURPOSE: Create a barrier
 *********************************************************************************
  DESCRIPTION:      This function does memory allocation for a barrier and 
                    adds the barrier pointer to the name server
                    
  Parameters :      Inputs: name  : Name of the barrier
                            mas   : Number of cores to be in the barrier
                    Output: bar   : Pointer of the barrier
 *********************************************************************************/
Bar *Bar_create(String name, Int max, void* barrierTable)
{
    Bar *bar;

    /* Allocate shared memory for the barrier and initialize with zeros */
    bar = Memory_calloc(SharedRegion_getHeap(0), sizeof(Bar), 0, NULL);

    /* Save the max to shared memory */
    bar->max.nCoresInBar = max;

    /* Write back the entire barrier to shared memory */
    Cache_wbInv(bar, sizeof(Bar), Cache_Type_ALL, TRUE);

    /* Add the barrier instance to the NameServer */
    NameServer_addUInt32((NameServer_Handle)barrierTable, name, (UInt32)bar);

    return (bar);
}

/*********************************************************************************
 * FUNCTION PURPOSE: Open a barrier
 *********************************************************************************
  DESCRIPTION:      This function open a barrier with a specified name which has 
                    been create by looking for the name server 
                    
  Parameters :      Inputs: name  : Name of the barrier
                    Output: bar   : Pointer of the barrier
 *********************************************************************************/
Bar *Bar_open(String name, void* barrierTable)
{
    Int status;
    Bar *bar;

    /* Try to open the remotely/locally created barrier */
    status = NameServer_getUInt32((NameServer_Handle)barrierTable, name, (UInt32 *)&bar, NULL);
    if (status < 0) {
        /* Barrier not available yet */
        return (NULL);
    }
    /* Return the barrier ptr  */
    return (bar);
}


/*********************************************************************************
 * FUNCTION PURPOSE: Wait on a barrier
 *********************************************************************************
  DESCRIPTION:      This function does synchronization on a barrier
                    
  Parameters :      Inputs: Bar        : barrier pointer
                            syncPoint  : Synchronization point on the barrier
                    Output: None
 *********************************************************************************/
Bool Bar_wait(Bar *bar, UInt32 syncPoint, GateMP_Handle gateBarrier) 
{
    UInt key, retVal=TRUE;
    GateMP_Handle gate = gateBarrier;

    /* Check whether cores are being released from the barrier.  If this is the 
     * case, wait until the last thread has left the barrier before entering
     */
    do {
        Cache_inv(&(bar->release[syncPoint].barAvailFlag), sizeof(UInt32_padding), Cache_Type_ALL, TRUE);
    } while (bar->release[syncPoint].barAvailFlag == BARRIER_NOTAVAILABLE);


    key = GateMP_enter(gate);
    /* Make sure that the shared memory for the barrier instance isn't in cache */
    Cache_inv(&(bar->count[syncPoint].nCoresInBar), sizeof(UInt32_padding), Cache_Type_ALL, TRUE);
    bar->count[syncPoint].nCoresInBar++;
    if (bar->count[syncPoint].nCoresInBar > bar->max.nCoresInBar) {
       retVal= FALSE;
    }
    Cache_wb(&(bar->count[syncPoint].nCoresInBar), sizeof(UInt32_padding), Cache_Type_ALL, TRUE);

    /* If bar->count equals bar->max, all threads have reached the barrier. 
     * enter 'release' mode
     */
    if (bar->count[syncPoint].nCoresInBar == bar->max.nCoresInBar) {
        bar->release[syncPoint].barAvailFlag = BARRIER_NOTAVAILABLE;
        Cache_wb(&(bar->release[syncPoint].barAvailFlag), sizeof(UInt32_padding), Cache_Type_ALL, TRUE);
    }

    GateMP_leave(gate, key);

    /* Spin waiting for all the threads to reach the barrier */
    do {
        Cache_inv(&(bar->release[syncPoint].barAvailFlag), sizeof(UInt32_padding), Cache_Type_ALL, TRUE);
    } while (bar->release[syncPoint].barAvailFlag == BARRIER_AVAILABLE);

    /* At this point we have left the barrier */
    
    key = GateMP_enter(gate);
    /* Make sure that the shared memory for the barrier instance isn't in cache */
    Cache_inv(&(bar->count[syncPoint].nCoresInBar), sizeof(UInt32_padding), Cache_Type_ALL, TRUE);
    bar->count[syncPoint].nCoresInBar--;  /* Decrement the barrier count as the core leaves the barrier */
    if (bar->count[syncPoint].nCoresInBar < 0) {
       retVal= FALSE;
    }
    Cache_wb(&(bar->count[syncPoint].nCoresInBar), sizeof(UInt32_padding), Cache_Type_ALL, TRUE);

    /* If bar->count equals zero, then the last thread has left the barrier. 
     * Reset to non-'release' mode
     */
    if (bar->count[syncPoint].nCoresInBar == 0) {
        bar->release[syncPoint].barAvailFlag = BARRIER_AVAILABLE;
        Cache_wb(&(bar->release[syncPoint].barAvailFlag), sizeof(UInt32_padding), Cache_Type_ALL, TRUE);
    }

    GateMP_leave(gate, key);

    /* Spin waiting for all the threads to leave the barrier */
    do {
        Cache_inv(&(bar->release[syncPoint].barAvailFlag), sizeof(UInt32_padding), Cache_Type_ALL, TRUE);
    } while (bar->release[syncPoint].barAvailFlag == BARRIER_NOTAVAILABLE);

    /* Cache invalidate the count value after leaving the barrier */
    Cache_inv(&(bar->count[syncPoint].nCoresInBar), sizeof(UInt32_padding), Cache_Type_ALL, TRUE);

    return(retVal);
}


/*** nothing past this point ***/
