/******************************************************************************
 * FILE PURPOSE: Intercore functions using BIOS 6 IPC
 ******************************************************************************
 * FILE NAME:   siuIpcBios6.c
 *
 * DESCRIPTION: Contains BIOS IPC related functions
 *
 * TABS: NONE
 *
 *
 * (C) Copyright 2010, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
/* SIU header files */
#include "siuIpcBios6Loc.h"

/* IPC context */
#pragma DATA_SECTION (biosIpcContext, ".ipc_context");
biosIpcContext_t biosIpcContext;

/*********************************************************************************
 * FUNCTION PURPOSE: Set Local Processer ID
 *********************************************************************************
  DESCRIPTION:      This function is setting Local Processer ID. It will be called
                    during the runtime startup sequence as user-supplied "firsr
                    functions", as specified in BIOS 6 configuration file:

                    xdc_runtime_Startup.firstFxns.$add('&siuIpcBiosSetLocalId');

  Parameters :      Inputs: None
                    Output: None
 *********************************************************************************/
void siuIpcBiosSetLocalId(void)
{
    tuint procId;
    char procName[5];

    /* Determine the procName from DNUM */
    System_sprintf(procName, "CORE%d", DNUM);

    /* Translate from a procName to a procId */
    procId = MultiProc_getId(procName);
    
    /* Set the local ID */
    MultiProc_setLocalId(procId);
}

/*********************************************************************************
 * FUNCTION PURPOSE: Initialize BIOS IPC data variables
 *********************************************************************************
  DESCRIPTION:      This function is initializing BIOS IPC internal data variables. 
                    It is called once per node.

  Parameters :      Inputs: None
                    Output: None
 *********************************************************************************/
void siu_ipc_osal_init(void)
{
  /* Initialize barrier */
  biosIpcContext.ipcBarrier = NULL;
  /* Initialize gate */
  biosIpcContext.ipcGate = NULL;

}

/*********************************************************************************
 * FUNCTION PURPOSE:IPC attach between non-owner cores and the owner core
 *********************************************************************************
  DESCRIPTION:      Create IPC sync pair between the owner core and the other
                    cores. This is required because every core needs access
                    to the shared heap and shared gate. Ipc_start() must be called
                    before this.

  Parameters :      Inputs: None
                    Output: None
 *********************************************************************************/
void siu_ipc_osal_attach_owner(tuint nCores)
{
  SharedRegion_Entry entry;
  tint  status;
  tuint numCores;

  /* get region 0 information */
  SharedRegion_getEntry(0, &entry);

  if (nCores == NUM_CORE_FROM_CFG) {
    numCores = MultiProc_getNumProcessors();
  } else {
    numCores = nCores; 
  }

  /* Must attach to owner first to get default GateMP and HeapMemMP */
  if (MultiProc_self() != entry.ownerProcId) {
      do {
          status = Ipc_attach(entry.ownerProcId);
          if (status < 0) {
            simTaskSleep();
          }
      } while (status < 0);
  } else {
  /* Owner core needs to call Ipc_attach() for remote processors. */
      tuint i;
  
      for (i = 0; i < numCores; i++) {          
          /* Bypass itself */
          if (i == MultiProc_self())  {
              continue;
          }
          do {
              status = Ipc_attach(i);
              if (status < 0) {
                simTaskSleep();
              }
          } while (status < 0);
      }
  }
}


/*********************************************************************************
 * FUNCTION PURPOSE: Attach to a remote core
 *********************************************************************************
  DESCRIPTION:      This function performs Ipc_attach() to a remote core if they
                    are not attached already. 
  Parameters :      Inputs: remote_core: remote core ID
                    Output: None
 *********************************************************************************/
void siu_ipc_osal_attach_remote(tuint remote_core)
{
    SharedRegion_Entry entry;
    tint  status;
    tuint local_core = MultiProc_self();

    /* get region 0 information */
    SharedRegion_getEntry(0, &entry);
    if ( (remote_core != entry.ownerProcId) && (local_core != entry.ownerProcId)) {
        /* call Ipc_attach for the remote processor */
        do {
            status = Ipc_attach(remote_core);
            if (status < 0) {
                simTaskSleep();
            }
        } while (status < 0);
    }
}

/*********************************************************************************
 * FUNCTION PURPOSE: Create a barrier
 *********************************************************************************
  DESCRIPTION:      This function creates a barrier. It will be called only once
                    by a master core.
                    
  Parameters :      Inputs: nCoresInBarrier  : number of cores in the barrier
                    Output: None
 *********************************************************************************/
void siu_ipc_osal_barrier_create(tuint nCoresInBarrier)
{
  char barrierName[8];

  /* Determine the barrier Name: only master core does BAR create */
  System_sprintf(barrierName, "BARRIER%d", MultiProc_self());

  /* Create barrier */
  if (biosIpcContext.ipcBarrier == NULL) {
    /* barrierTable is defined in bios6ipc.cfg */
    biosIpcContext.ipcBarrier = Bar_create(barrierName, nCoresInBarrier, (void *)barrierTable);
  }
  siu_exc_assert (biosIpcContext.ipcBarrier != NULL, SIUVIGDK_EXC_IPC_BARRIER_CREATE, &siuContext);
}

/*********************************************************************************
 * FUNCTION PURPOSE: Open a barrier
 *********************************************************************************
  DESCRIPTION:      This function opens a barrier which has been created by a 
                    master core. It can be called by multiple slave cores.
                    
  Parameters :      Inputs: masterCoreId   : ID of master core
                    Output: None
 *********************************************************************************/
void siu_ipc_osal_barrier_open(tuint masterCoreId)
{
  char barrierName[8];
  char procName[5];
  tuint masterProcId;

  /* Determine the procName and then procId for the master core */
  System_sprintf(procName, "CORE%d", masterCoreId);
  masterProcId = MultiProc_getId(procName);

  /* Determine the barrier Name from master core ID */
  System_sprintf(barrierName, "BARRIER%d", masterProcId);

  /* Open barrier */
  while (biosIpcContext.ipcBarrier == NULL) {
    /* barrierTable is defined in bios6ipc.cfg */
    biosIpcContext.ipcBarrier = Bar_open(barrierName, (void *)barrierTable);
    if (biosIpcContext.ipcBarrier == NULL) {
      simTaskSleep();
    }
  }
}

/*********************************************************************************
 * FUNCTION PURPOSE: Wait on a barrier
 *********************************************************************************
  DESCRIPTION:      This function does barrier wait on a specified synchronization 
                    point.
                    
  Parameters :      Inputs: syncPoint:  synchronization point
                    Output: None
 *********************************************************************************/
void siu_ipc_osal_barrier_wait(tuint syncPoint)
{
  siu_exc_assert (biosIpcContext.ipcBarrier != NULL, SIUVIGDK_EXC_IPC_BARRIER_NOT_AVAILABLE, &siuContext);
  siu_exc_assert (Bar_wait(biosIpcContext.ipcBarrier, syncPoint, biosIpcContext.gateBarrier) == TRUE,
                  SIUVIGDK_EXC_IPC_BARRIER_NOT_AVAILABLE, &siuContext);
}

/*********************************************************************************
 * FUNCTION PURPOSE:Create shared global accessed by multiple cores
 *********************************************************************************
  DESCRIPTION:      Create global on a shared memory region which can be accessed
                    by multiple cores

  Parameters :      Inputs: shmSize      : size of shared global
                            shareBase    : pointer to base of shared global
                    Output: None
 *********************************************************************************/
void siu_ipc_osal_create_share(tuint shmSize, void**shareBase)
{
  void *share;

  /* Allocate memory for the shared global and initialize to zeros */
  share = (void *)Memory_calloc(SharedRegion_getHeap(0), shmSize, 0, NULL);
  /* Add the share instance to the NameServer: shmTable is defined in bios6ipc.cfg */
  NameServer_addUInt32((NameServer_Handle)shmTable, "SHARE", (UInt32)share);

  siu_exc_assert (share != NULL, SIUVIGDK_EXC_IPC_SHARE_CREATE, &siuContext);
  /* Return the share pointer to application */
  *shareBase = share;
}

/*********************************************************************************
 * FUNCTION PURPOSE: Open shared global created by a master core
 *********************************************************************************
  DESCRIPTION:      Open global on a shared memory region which has been created
                    by a master core

  Parameters :      Inputs: shareBase    : pointer to base of shared global
                            masterCoreId : ID of master core
                    Output: None
 *********************************************************************************/
void siu_ipc_osal_open_share(void**shareBase)
{
  void *share;
  tint  status;

  /* Open the share */
  do {
    /* Get the global share from the name server: shmTable is defined in bios6ipc.cfg */
    status = NameServer_getUInt32((NameServer_Handle)shmTable, "SHARE", (UInt32 *)&share, NULL);
    if (status < 0) {
      simTaskSleep();
    }
  } while (status < 0);
  siu_exc_assert (share != NULL, SIUVIGDK_EXC_IPC_SHARE_CREATE, &siuContext);
  /* Return the share pointer to application */
  *shareBase = share;
}


/*********************************************************************************
 * FUNCTION PURPOSE: Create a gate
 *********************************************************************************
  DESCRIPTION:      This function creates a gate
                    
  Parameters :      Inputs: None
                    Output: None
 *********************************************************************************/
void siu_ipc_osal_create_gate(void)
{
  GateMP_Params gparams;

  GateMP_Params_init(&gparams);
  gparams.localProtect = GateMP_LocalProtect_THREAD;
  gparams.remoteProtect = GateMP_RemoteProtect_SYSTEM;
  gparams.regionId = 0;

  /* Create gate for shared global */
  gparams.name = "ipcGate";
  biosIpcContext.ipcGate = GateMP_create(&gparams);

  /* Create gate for ipc barrier */
  gparams.name = "gateBarrier";
  biosIpcContext.gateBarrier = GateMP_create(&gparams);

}

/*********************************************************************************
 * FUNCTION PURPOSE: Open gates
 *********************************************************************************
  DESCRIPTION:      This function opens gates
                    
  Parameters :      Inputs: None
                    Output: None
 *********************************************************************************/
void siu_ipc_osal_open_gate(void)
{
  tint  status;
  /* Open gate for shared global */
  do {
    status = GateMP_open("ipcGate", &(biosIpcContext.ipcGate));
    if (status < 0) {
      simTaskSleep();
    }
  } while (status < 0);

  /* Open gate for barrier use */
  do {
    status = GateMP_open("gateBarrier", &(biosIpcContext.gateBarrier));
    if (status < 0) {
      simTaskSleep();
    }
  } while (status < 0);
}


/*********************************************************************************
 * FUNCTION PURPOSE: Close gates
 *********************************************************************************
  DESCRIPTION:      This function closes gates
                    
  Parameters :      Inputs: None
                    Output: None
 *********************************************************************************/
void siu_ipc_osal_close_gate(void)
{
  tint  status;
  /* Close gate for shared global */
  if(biosIpcContext.ipcGate != NULL) {
      do {
        status = GateMP_close(&(biosIpcContext.ipcGate));
        if (status < 0) {
          simTaskSleep();
        }
      } while (status < 0);
  }

  /* Close gate for barrier use */
  if(biosIpcContext.gateBarrier != NULL) {
      do {
        status = GateMP_close(&(biosIpcContext.gateBarrier));
        if (status < 0) {
          simTaskSleep();
        }
      } while (status < 0);
  }
}

/*********************************************************************************
 * FUNCTION PURPOSE: Delete gates
 *********************************************************************************
  DESCRIPTION:      This function closes gates
                    
  Parameters :      Inputs: None
                    Output: None
 *********************************************************************************/
void siu_ipc_osal_delete_gate()
{
  tint  status;
  /* Delete gate for shared global */
  do {
    status = GateMP_delete(&(biosIpcContext.ipcGate));
    if (status < 0) {
      simTaskSleep();
    }
  } while (status < 0);

  /* Delete gate for barrier use */
  do {
    status = GateMP_delete(&(biosIpcContext.gateBarrier));
    if (status < 0) {
      simTaskSleep();
    }
  } while (status < 0);
}

/*********************************************************************************
 * FUNCTION PURPOSE: Enter a shared global
 *********************************************************************************
  DESCRIPTION:      This function does gatemp_enter and cache invalidate before
                    acessing a shared global
                    
  Parameters :      Inputs: shareBase    : base address of shared global
                            shmSize      : size of shared global
                    Output: key:         : key when entering GateMP
 *********************************************************************************/
tuint siu_ipc_osal_enter_share(void* shareBase, tuint shareSize)
{
  tuint key;

  key = GateMP_enter(biosIpcContext.ipcGate);
  Cache_inv(shareBase, shareSize, Cache_Type_ALL, TRUE);

  return (key);
}

/*********************************************************************************
 * FUNCTION PURPOSE: Leave a shared global
 *********************************************************************************
  DESCRIPTION:      This function does cache writeback and gatemp_leave before
                    leaving a shared global
                    
  Parameters :      Inputs: shareBase    : base address of shared global
                            shmSize      : size of shared global
                            key          : key when entering GateMP last time
                    Output: None
 *********************************************************************************/
void siu_ipc_osal_leave_share(void* shareBase, tuint shareSize, tuint gateKey)
{
  Cache_wb(shareBase, shareSize, Cache_Type_ALL, TRUE);
  GateMP_leave(biosIpcContext.ipcGate, gateKey);
}

/*********************************************************************************
 * FUNCTION PURPOSE:IPC start on a core
 *********************************************************************************
  DESCRIPTION:      This function starts IPC by call Ipc_start() and then sync 
                    up the owner core with slave cores.

  Parameters :      Inputs: None
                    Output: None
 *********************************************************************************/
void siu_ipc_osal_start(void *ipcContext)
{
    /* Call Ipc_start() */
    Ipc_start();
    biosIpcContext.ipcContext = (ipcContext_t *)ipcContext;
}

/*** nothing past this point ***/
