/******************************************************************************
 * FILE PURPOSE: Intercore communication functions using BIOS 6 IPC
 ******************************************************************************
 * FILE NAME:   siuIpcMsgBios6.c
 *
 * DESCRIPTION: Contains BIOS IPC related functions for inter-core messaging
 *
 * TABS: NONE
 *
 *
 * (C) Copyright 2010, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
/* SIU header files */
#include "osal/siuIpcMsgOsal.h"
#include "osal/siuIpcOsal.h"
#include "siuIpcMsgBios6Loc.h"

/* IPC Messaging context */
#pragma DATA_SECTION (biosIpcMsgContext, ".ipc_context");
biosIpcMsgContext_t biosIpcMsgContext;

extern biosIpcContext_t biosIpcContext;

/*********************************************************************************
 * FUNCTION PURPOSE: Set priority for BIOS IPC related task
 *********************************************************************************
  DESCRIPTION:      This function is activating MessageQ control task by changing
                    its priority from -1 (inactive) in the configuration file to 
                    1. This is needed to place MessageQ control task stack in DDR.

  Parameters :      Inputs: None
                    Output: None
 *********************************************************************************/
void siu_ipc_osal_create_msgq_task(void)
{
  extern unsigned char _lcf_bios_message_q_stack_begin;
  extern unsigned char _lcf_bios_message_q_stack_end;
  Task_Params taskParams;

  Task_Params_init(&taskParams);
  taskParams.stackSize = 
    &_lcf_bios_message_q_stack_end - &_lcf_bios_message_q_stack_begin;    
  taskParams.stack = (Ptr)&_lcf_bios_message_q_stack_begin;
  taskParams.priority = 2;
  Task_create((Task_FuncPtr)siuIpcBiosMessageQControl, &taskParams, NULL);
}

/*********************************************************************************
 * FUNCTION PURPOSE: Initialize BIOS IPC Messaging data variables
 *********************************************************************************
  DESCRIPTION:      This function is initializing internal data variables related
                    to BIOS IPC messaging. It is called once per node.

  Parameters :      Inputs: None
                    Output: None
 *********************************************************************************/
void siu_ipc_osal_msg_init(void)
{
  tuint i;
  /* Initialize the array to store remote MessageQ ID  */
  for(i=0; i<CPU_NCORES; i++)
    biosIpcMsgContext.remoteQueueId[i] = MessageQ_INVALIDMESSAGEQ;;

  /* Initialize local MessageQ */
  biosIpcMsgContext.messageQ = NULL;

  /* Indicate HeapBufMP has not been opened for the core */
  biosIpcMsgContext.ipc_message_open_heap = FALSE;
}

/*********************************************************************************
 * FUNCTION PURPOSE:IPC Messging start on a core
 *********************************************************************************
  DESCRIPTION:      This function starts IPC messaging. It gets context information
                    from application and then post MessageQ create event to
                    get local MessageQ created.

  Parameters :      Inputs: None
                    Output: None
 *********************************************************************************/
void siu_ipc_osal_msg_start(void *ipcMsgContext)
{
    /* Get IPC Messaging context from application */
    biosIpcMsgContext.ipcMsgContext = (ipcMsgContext_t *)ipcMsgContext;

    /* MessageQ create */
    siuIpcBiosMessageQCreate();

}

/*********************************************************************************
 * FUNCTION PURPOSE: Post events for MessageQ control
 *********************************************************************************
  DESCRIPTION:      This function post events for 
                    MessageQ close (EVENT_ID_MESSAGEQ_CLOSE), 

  Parameters :      Inputs: application event ids for MessageQ create, open and
                    alloc, close; barrier create, open; and shared global create
                    and open

                    Output: None
 *********************************************************************************/
void siu_ipc_osal_event(tuint event)
{
  switch (event) {

  case EVENT_ID_MESSAGEQ_CLOSE:
      Event_post(siuMsgQEvent, Event_Id_00);
      break;
  }
}

/*********************************************************************************
 * FUNCTION PURPOSE: Create Local MessageQ and Register Heap with MessageQ
 *********************************************************************************
  DESCRIPTION:      This function is called after image download is done. 
                    It creates the local MessageQ and register the heap with it.

  Parameters :      Inputs: None
                    Output: None
 *********************************************************************************/
void siuIpcBiosMessageQCreate(void)
{
  SharedRegion_Entry entry;

  MessageQ_Params msgqParams;
  char localQueueName[5];
  HeapBufMP_Handle heapHandle;
  tuint appMsgSize = biosIpcMsgContext.ipcMsgContext->appMsgSize;

  SharedRegion_getEntry(0, &entry);

  if(biosIpcMsgContext.ipcMsgContext->sharedRegionType == SHARED_REGION_TYPE_SL2)
    biosIpcMsgContext.ipc_message_shared_region_id = 1;
  else if(biosIpcMsgContext.ipcMsgContext->sharedRegionType == SHARED_REGION_TYPE_DDR)
    biosIpcMsgContext.ipc_message_shared_region_id = 2;

  System_sprintf(localQueueName, "MSGQ%d", MultiProc_self());

  /* Create a message queue. */
  MessageQ_Params_init(&msgqParams);
  msgqParams.synchronizer = siuMessageQGetSynch;
  biosIpcMsgContext.messageQ = MessageQ_create(localQueueName, &msgqParams);
  siu_exc_assert (biosIpcMsgContext.messageQ != NULL, SIUVIGDK_EXC_IPC_MSGQ_CREATE, &siuContext);

  if (MultiProc_self() == entry.ownerProcId) {
    HeapBufMP_Params heapBufParams;
    /* Create the heap that will be used to allocate messages. */
    HeapBufMP_Params_init(&heapBufParams);
    heapBufParams.regionId       = biosIpcMsgContext.ipc_message_shared_region_id;
    heapBufParams.name           = HEAP_NAME;
    heapBufParams.align          = 128;
    heapBufParams.numBlocks      = HEAP_NUM_BLOCK;
    heapBufParams.blockSize      = sizeof(msgIpcBios_t) + appMsgSize;
    heapBufParams.gate           = GateMP_getDefaultRemote();
    heapHandle = HeapBufMP_create(&heapBufParams);
    siu_exc_assert (heapHandle != NULL, SIUVIGDK_EXC_IPC_HEAP_CREATE, &siuContext);
  }
}

/*********************************************************************************
 * FUNCTION PURPOSE:Open remote MessageQs 
 *********************************************************************************
  DESCRIPTION:      This function performs Ipc_attach() to a remote core if they
                    are not attached already. Then it opens the remote MessageQ.
  Parameters :      Inputs: remote_core: remote core ID
                    Output: None
 *********************************************************************************/
void siuIpcBiosMessageQOpen(tuint remote_core)
{
    char  remoteQueueName[5];
    tint  status;

    /* Attach to the remote core */
    siu_ipc_osal_attach_remote(remote_core);
    if(biosIpcMsgContext.remoteQueueId[remote_core] == MessageQ_INVALIDMESSAGEQ) {
      System_sprintf(remoteQueueName, "MSGQ%d", remote_core);
      /* Open the remote message queue. Spin until it is ready. */
      do {
        status = MessageQ_open(remoteQueueName, &biosIpcMsgContext.remoteQueueId[remote_core]);
        if (status != MessageQ_S_SUCCESS) {
          simTaskSleep();
        }
      } while (status != MessageQ_S_SUCCESS);
    }
}

/*********************************************************************************
 * FUNCTION PURPOSE:Open remote MessageQs according to inter-core connection
 *                  Also allocate messages from heap
 *********************************************************************************
  DESCRIPTION:      This function checks the current inter-core connection.
                    Then, it performs Ipc_attach() to the remote core, 
                    and open remote MessageQs which are created by partner 
                    encoding/decoding cores.

                    It also does MessageQ_Alloc so that run-time message allocation
                    can be avoided.

  Parameters :      Inputs: None
                    Output: None
 *********************************************************************************/
void siu_ipc_osal_msgq_open_alloc(void)
{
    tuint i;
    tuint remote_core;
    tint  status;
    tuint appMsgSize = biosIpcMsgContext.ipcMsgContext->appMsgSize;

    /* Open remote MessageQ for core-to-core transcoding */
    for (i = 0; i < MAX_SIU_INTERCORE_CONN; i ++) {
      remote_core = 0xFF;
      if (siuInterCoreConn[i].enc_intercore != siuInterCoreConn[i].dec_intercore) {
        if (SIU_INTER_CORE_CONN_ENC_COREMATCH(i, siuContext.coreId)) {
          remote_core = SIU_GET_INTER_CORE_CONN_DEC_COREID(i);
        } else if (SIU_INTER_CORE_CONN_DEC_COREMATCH(i, siuContext.coreId)) {
          remote_core = SIU_GET_INTER_CORE_CONN_ENC_COREID(i);
        }
      }
     
     if (remote_core != 0xFF) {
       siuIpcBiosMessageQOpen(remote_core);
     }
    }

    /* Open remote MessageQ on a slave core for a master core */
    for (i = 0; i < CPU_NCORES-1; i++) {
      remote_core = siuContext.slaveCoreConn[i];
      if (remote_core != 0xFF) {
        siuIpcBiosMessageQOpen(remote_core);
      }
    }

    if (!biosIpcMsgContext.ipc_message_open_heap) {
      HeapBufMP_Handle heapHandle;
      /* Open the heap created by owner core. Loop until opened. */
      do {
        status = HeapBufMP_open(HEAP_NAME, &heapHandle);
        if (status != HeapBufMP_S_SUCCESS) {
          simTaskSleep();
        }
      } while (status != HeapBufMP_S_SUCCESS);

      /* Register this heap with MessageQ */
      MessageQ_registerHeap((IHeap_Handle)heapHandle, HEAP_ID);
      biosIpcMsgContext.ipc_message_open_heap = TRUE;

      for (i=0; i<MAX_IPC_MSG_SEND_PER_CORE; i++) {
        /* Alloc a BIOS IPC message which can later be sent to remote message queues */
        biosIpcMsgContext.ipcMsg[i] = (void*) MessageQ_alloc(HEAP_ID, sizeof(msgIpcBios_t) + appMsgSize);
        siu_exc_assert (biosIpcMsgContext.ipcMsg[i] != NULL, SIUVIGDK_EXC_IPC_MSGQ_ALLOC, &siuContext);
        /* Initialize the message as consumed so that they can be used by MessageQ_Put*/
        biosIpcMsgContext.ipcMsg[i]->consumed = TRUE;
        /* Initialize application message location just after msgApp pointer */
        biosIpcMsgContext.ipcMsg[i]->msgApp = (void *)((Uint32)(&(biosIpcMsgContext.ipcMsg[i]->header)) + sizeof(msgIpcBios_t));
      }
      biosIpcMsgContext.ipcMsgIndex = 0;
      Cache_wbInvAll();
    }

}

/*********************************************************************************
 * FUNCTION PURPOSE:Close remote MessageQs according to inter-core connection
 *********************************************************************************
  DESCRIPTION:      This function checks the current inter-core connection,  
                    and close remote MessageQs if their corresponding remote cores
                    are no longer the partner encoding/decoding cores
                    
  Parameters :      Inputs: None
                    Output: None
 *********************************************************************************/
void siu_ipc_osal_msgq_close(void)
{
    tuint   i, j;
    tint  status;

    for (i = 0; i < CPU_NCORES; i++) {
      if(biosIpcMsgContext.remoteQueueId[i] != MessageQ_INVALIDMESSAGEQ) { /* remote messageQ on core i has been opened before */
        /* Search the inter core connection matrix to check if core i is still connected to the current core */
        for (j = 0; j < MAX_SIU_INTERCORE_CONN; j++) {
          if ((siuInterCoreConn[i].enc_intercore != siuInterCoreConn[i].dec_intercore) && /* connection i is not in reset*/
              (SIU_INTER_CORE_CONN_ENC_COREMATCH(j,i) || /* core i is not the encoder core, or */
               SIU_INTER_CORE_CONN_DEC_COREMATCH(j,i))) {/* core i is not the decoder core */
            break;
          }
        }
        /* Close remote MessageQ on core i if it is no longer connected to the current core */
        if (j == MAX_SIU_INTERCORE_CONN) {
          SharedRegion_Entry entry;
          /* Close the remote messageQ */
          do {
            status = MessageQ_close(&biosIpcMsgContext.remoteQueueId[i]);
            if (status != MessageQ_S_SUCCESS) {
              simTaskSleep();
            }
          } while (status != MessageQ_S_SUCCESS);

          /* get region 0 information */
          SharedRegion_getEntry(0, &entry);

          /* Detach the remote core only when the owner core is 
             neither the local core nor the remote core         */
          if ((MultiProc_self() != entry.ownerProcId) && 
              (i != entry.ownerProcId)) {
              do {
                status = Ipc_detach(i);
                if (status < 0) {
                  simTaskSleep();
                }
              } while (status < 0);
          }
        }
      }
    }
}

/*********************************************************************************
 * FUNCTION PURPOSE:Control function for MessageQ create, open, and close
 *********************************************************************************
  DESCRIPTION:      This is a BIOS task which control MessageQ operations,

                    It is pending on and processing the following BIOS Events
                    Event_Id_00: close remote MessageQs; event is posted in
                                 siuVideoConnReq() after disconnect core-core
                                 connections as well as in siu_msm_video_to_closed()

  Parameters :      Inputs: arg0, arg1, input parameters for BIOS tasks
                    Output: None
 *********************************************************************************/
void siuIpcBiosMessageQControl(xdc_UArg arg0, xdc_UArg arg1)
{
  tuint events;

  while (TRUE) {
    /* Event_pend for MessageQ close events */
    events = Event_pend(siuMsgQEvent, Event_Id_NONE, 
                        Event_Id_00,
                        BIOS_WAIT_FOREVER);

    if (events & Event_Id_00) {
    /* Process MessageQ close event */
      siu_ipc_osal_msgq_close();
    }

  }
}

/*********************************************************************************
 * FUNCTION PURPOSE:Send message to a remote core
 *********************************************************************************
  DESCRIPTION:      This function sends messages to MessageQ on a remote core. 
                    
  Parameters :      Inputs: msg: msg to be sent;
                            appMsgSize: size of application message
                            ipcChan: inter-core channel information
                    Output: None
 *********************************************************************************/
void siu_ipc_osal_msg_send(void *msg, tuint appMsgSize, ipcChannel_t *ipcChan)
{
  msgIpcBios_t* msgIpcBios;

  if(biosIpcMsgContext.ipcMsgIndex >= MAX_IPC_MSG_SEND_PER_CORE) {
    biosIpcMsgContext.ipcMsgIndex = 0;
  }
  msgIpcBios = biosIpcMsgContext.ipcMsg[biosIpcMsgContext.ipcMsgIndex];

  do {
    Cache_inv((void *)msgIpcBios, sizeof(msgIpcBios_t), Cache_Type_ALL, TRUE);
  } while(!msgIpcBios->consumed);  /* Spin so that the message is reused
                                    only after it has been consumed */
  msgIpcBios->consumed = FALSE; /* Re-initialize message as not consumed */
  Cache_wbInv((void *)msgIpcBios, sizeof(msgIpcBios_t), Cache_Type_ALL, TRUE);

  biosIpcMsgContext.ipcMsgIndex++;
  
  memcpy(msgIpcBios->msgApp, msg, appMsgSize);

  MessageQ_put(biosIpcMsgContext.remoteQueueId[ipcChan->remote_core], (MessageQ_Msg)msgIpcBios);
}

/*********************************************************************************
 * FUNCTION PURPOSE:Retrieve message from local MessageQ
 *********************************************************************************
  DESCRIPTION:      This function retrieves messages from local MessageQ. 
                    If there is message coming from other cores, its posts a Tx
                    task for encoder along with the received message. 
                    For the decoder, it consumes the message immediately and then
                    free the message back to the heap.
                    
  Parameters :      Inputs: None
                    Output: None
 *********************************************************************************/
void siuIpcBiosMessageQGet(void)
{
  msgIpcBios_t *pMsgRcv;
  tint  status;

  thwCriticalBegin();
  status = MessageQ_get(biosIpcMsgContext.messageQ, (MessageQ_Msg *)&pMsgRcv, MessageQ_FOREVER);
  siu_exc_assert (status == MessageQ_S_SUCCESS, SIUVIGDK_EXC_IPC_MSGQ_GET, &siuContext);
  biosIpcMsgContext.ipcMsgContext->msgRevCallout((void*)(pMsgRcv->msgApp));
  thwCriticalEnd();
}

/*********************************************************************************
 * FUNCTION PURPOSE:Set consumed flag for a message
 *********************************************************************************
  DESCRIPTION:      This function sets the consumed flag for a message which 
                    has been consumed by application
                    
  Parameters :      Inputs: msg: msg which has been consumed by the application;
                    Output: None
 *********************************************************************************/
void siu_ipc_osal_set_msg_consumed(void *msg)
{
  msgIpcBios_t* msgIpcBios;

  msgIpcBios = (msgIpcBios_t*) ((Uint32)msg - sizeof(msgIpcBios_t));
  msgIpcBios->consumed = TRUE;

  Cache_wbInv((void *)msgIpcBios, sizeof(msgIpcBios_t), Cache_Type_ALL, TRUE);
}

/*** nothing past this point ***/
