/******************************************************************************
 * FILE PURPOSE: Implement OS abstraction layer functions for BIOS6
 ******************************************************************************
 * FILE NAME:   siuOsalBios6.c
 *
 * DESCRIPTION: Contains OS abstraction layer functions when using BIOS6
 *
 * TABS: NONE
 *
 *
 * (C) Copyright 2010, Texas Instruments Incorporated.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include "osal/siuOsal.h"
#ifdef HDVIDEO
#include "osal/siuIpcMsgOsal.h"
#endif
/* BIOS6 module headers */
#include <ti/sysbios/BIOS.h>
#include <ti/sysbios/knl/Task.h>
#include <ti/sysbios/knl/Event.h>
#include <ti/sysbios/family/c66/Cache.h>
#include <ti/sysbios/interfaces/ICache.h>

/* Get globals from cfg header */
#include <xdc/cfg/global.h>

/*********************************************************************************
 * FUNCTION PURPOSE: Post signal
 *********************************************************************************
  DESCRIPTION:      This function post bios signals

  Parameters :      Inputs: None
                    Output: None

 *********************************************************************************/
#ifdef HDVIDEO
void siu_osal_post_signal(siuOSAL_SignalID_t signalId)
{
  if(signalId == SIU_OSAL_SIGNAL_HANDLE_TXRXSWI)
    /* Post siuTxRxSwi SWI */
    Swi_post (siuTxRxSwih);
  else if (signalId == SIU_OSAL_SIGNAL_HANDLE_TASKSCHEDULER)
    /* Post siuTaskScheduler SWI */
    Swi_post (siuTaskSchedulerh);
  else if (signalId == SIU_OSAL_SIGNAL_HANDLE_MESSAGE) {
    /* Post event for processing network message */
    Event_post (networkMessageEvent, Event_Id_00);
  }
}

/*********************************************************************************
 * FUNCTION PURPOSE: Message
 *********************************************************************************
  DESCRIPTION:      This function triggers siuTaskScheduler

  Parameters :      Inputs: arg0, arg1: BIOS task arguments
                    Output: None

 *********************************************************************************/
extern void siuGuaranteedMessageTaskProc (void);
void networkMessageProcess(xdc_UArg arg0, xdc_UArg arg1)
{
  tuint events;

  while (TRUE) {
    /* Event_pend for message process events */
    events = Event_pend(networkMessageEvent, Event_Id_NONE,
                        Event_Id_00, BIOS_WAIT_FOREVER);

    if (events & Event_Id_00) {
      /* Message process task */
      siuGuaranteedMessageTaskProc();
    }
  }
}

/*********************************************************************************
 * FUNCTION PURPOSE: Set priority for message processing task
 *********************************************************************************
  DESCRIPTION:      This function is activating networkMessageProcess task by changing
                    its priority from -1 (inactive) in the configuration file to
                    2. This is needed to place networkMessageProcess task stack in DDR.

  Parameters :      Inputs: None
                    Output: None
 *********************************************************************************/
void siuCreateNetworkMessageTask(void)
{
  extern unsigned char _lcf_bios_message_proc_stack_begin;
  extern unsigned char _lcf_bios_message_proc_stack_end;
  Task_Params taskParams;

  Task_Params_init(&taskParams);
  taskParams.stackSize =
    &_lcf_bios_message_proc_stack_end - &_lcf_bios_message_proc_stack_begin;
  taskParams.stack = (Ptr)&_lcf_bios_message_proc_stack_begin;
  taskParams.priority = 2;
  Task_create((Task_FuncPtr)networkMessageProcess, &taskParams, NULL);
}
#endif

/*********************************************************************************
 * FUNCTION PURPOSE: Start BIOS
 *********************************************************************************
  DESCRIPTION:      This function starts BIOS

  Parameters :      Inputs: None
                    Output: None

 *********************************************************************************/
void siu_osal_start(void)
{
  /* Create tasks */
#ifdef HDVIDEO
  siuCreateNetworkMessageTask();
  siu_ipc_osal_create_msgq_task();
#endif
  /* Start BIOS */
  BIOS_start();
}

/*********************************************************************************
 * FUNCTION PURPOSE: Cache writeback and invalidate for all
 *********************************************************************************
  DESCRIPTION:      This function does cache writeback and invalidate for all
                    using BIOS cache APIs

  Parameters :      Inputs: None
                    Output: None

 *********************************************************************************/
void siu_osal_wbinv_cache_all(void)
{
  Cache_wbInvAll();
}

/*********************************************************************************
 * FUNCTION PURPOSE: Cache writeback and invalidate for a region
 *********************************************************************************
  DESCRIPTION:      This function does cache writeback and invalidate for a
                    specified region using BIOS cache APIs

  Parameters :      Inputs: base     : base address of the memory
                            length   : size of the memory
                            wait     : flag for waiting
                    Output: None
 *********************************************************************************/
void siu_osal_wbinv_cache(void *base, tulong length, bool wait)
{
  Cache_wbInv(base, length, Cache_Type_ALL, wait);
}


/*********************************************************************************
 * FUNCTION PURPOSE: Cache writeback for a region
 *********************************************************************************
  DESCRIPTION:      This function does cache writeback for a
                    specified region using BIOS cache APIs

  Parameters :      Inputs: base     : base address of the memory
                            length   : size of the memory
                            wait     : flag for waiting
                    Output: None
 *********************************************************************************/
void siu_osal_wb_cache(void *base, tulong length, bool wait)
{
  Cache_wb(base, length, Cache_Type_ALL, wait);
}

/*********************************************************************************
 * FUNCTION PURPOSE: Cache invalidate for a region
 *********************************************************************************
  DESCRIPTION:      This function does cache invalidate for a
                    specified region using BIOS cache APIs

  Parameters :      Inputs: base     : base address of the memory
                            length   : size of the memory
                            wait     : flag for waiting
                    Output: None
 *********************************************************************************/
void siu_osal_inv_cache(void *base, tulong length, bool wait)
{
  Cache_inv(base, length, Cache_Type_ALL, wait);
}
/* nothing past this point */
