/******************************************************************************
* FILE PURPOSE: Client Wrapper to integrate XDM0.9 Compliant Encoder to TVCT
******************************************************************************
* FILE NAME:   vctaacEncClient.c
*
* DESCRIPTION: Client Wrapper to integrate XDM0.9 Compliant Encoder to TVCT
*
* TABS: NONE
*
*
* (C) Copyright 2010, Texas Instruments Incorporated
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include "vctAACEncClient.h"
#include <string.h>

void *aacGetStaticParams(void *aacEncInst) {
   aacEncInst_t *inst = (aacEncInst_t *)aacEncInst;
   return((void *)&inst->extn_params);
}

void *aacGetDynamicParams(void *aacEncInst){
   aacEncInst_t *inst = (aacEncInst_t *)aacEncInst;
   return((void *)&inst->extn_dynamicParams);
}

void *aacGetInArgs(void *aacEncInst) {
   aacEncInst_t *inst = (aacEncInst_t *)aacEncInst;
   IAACENC_Params *ext_params    = &(inst->extn_params);
   IAUDENC_Params *params        = &ext_params->audenc_params;
   IAACENC_InArgs *inArgs        = &(inst->inArgs);
   int nbFrameSamples;
   int nbChannels = 2;
   
   nbFrameSamples = (ext_params->outObjectType == AACENC_OBJ_TYP_LC)? 1024L : 2048L;
   inArgs->numInSamples = ((sizeof(XDAS_Int16)) * nbFrameSamples * nbChannels);
   return((void *)inArgs);
}

void *aacGetOutArgs(void *aacEncInst) {
   aacEncInst_t *inst = (aacEncInst_t *)aacEncInst;
   return((void *)&inst->outArgs);
}

void *aacGetStatus(void *aacEncInst) {
   aacEncInst_t *inst = (aacEncInst_t *)aacEncInst;
   return((void *)&inst->status);
}

void *aacGetFxns(void *aacEncInst) {
   aacEncInst_t *inst = (aacEncInst_t *)aacEncInst;
   return((void *)(inst->IALG_Fxns));
}

void aacInitInstance(void *aacEncInst)
{
    aacEncInst_t *inst = (aacEncInst_t *)aacEncInst;
    memset(inst, 0, sizeof(aacEncInst_t));

    inst->IALG_Fxns                    = &AACENC_TII_IAACENC;
    inst->status.audenc_status.size    = sizeof(IAACENC_Status);
    inst->inArgs.audenc_inArgs.size    = sizeof(IAACENC_InArgs);
    inst->outArgs.audenc_outArgs.size  = sizeof(IAACENC_OutArgs);
    return;
}

void aacSetStaticParams(void *aacEncInst, void *codecParams)
{
    aacEncInst_t *inst           = (aacEncInst_t *)aacEncInst;
    IAACENC_Params *ext_params    = &(inst->extn_params);
    IAUDENC_Params *params        = &ext_params->audenc_params;

    params->size                   = sizeof(IAACENC_Params);
    params->encodingPreset         = 0;
    params->maxSampleRate          = 96000;
    params->maxBitrate             = 576000;
    params->maxNoOfCh              = 1;
    params->dataEndianness         = 2;

    ext_params->outObjectType      = AACENC_OBJ_TYP_PS;   /*!< Output Object Type LC/HE/HEv2       */
    ext_params->outFileFormat      = AACENC_TT_ADTS;   /*!< Output File Format             */
    ext_params->useCRC             = AACENC_FALSE;   /*!< Flag for inserting CRC bits         */
    ext_params->useTns             = AACENC_TRUE;   /*!< Flag for activating TNS feature     */
    ext_params->usePns             = AACENC_TRUE;   /*!< Flag for activating PNS feature     */
    ext_params->downMixFlag        = AACENC_FALSE;   /*!< downmixing Flag                   */
    ext_params->bitRateMode        = AACENC_BR_MODE_CBR;   /*!< Flag for CBR / VBR coding          */
    /*! Replicated from Dynamic Params */
    ext_params->bitRate            = 64000;   /*!< Input Bit-rate                      */
    ext_params->sampleRate         = 48000;   /*!< Input Sampling Rate                 */
    ext_params->nbInChannels       = 1;   /*!< Number of Input Channels            */
    ext_params->dualMono           = AACENC_FALSE;   /*!< Flag to indicate Dual Mono stream   */
    ext_params->ancFlag            = AACENC_FALSE;   /*!< ancillary data Flag                 */
    ext_params->ancRate            = -1;   /*!< ancillary Data Rate                 */

    return;
}

void aacSetDynamicParams(void *aacEncInst, void *dynParams)
{
    aacEncInst_t *inst                   = (aacEncInst_t *)aacEncInst;
    IAACENC_DynamicParams *extn_dynpar   = &inst->extn_dynamicParams;
    IAUDENC_DynamicParams *dynamicParams = &extn_dynpar->audenc_dynamicparams;
    IAACENC_Params *params               = (IAACENC_Params *)dynParams;

    memset((void *)dynamicParams, 0, sizeof(IAACENC_DynamicParams));

    dynamicParams->size                   = sizeof(IAACENC_DynamicParams);
    dynamicParams->inputFormat            = 1;
    dynamicParams->bitRate                = 64000;
    dynamicParams->sampleRate             = 48000;
    dynamicParams->numChannels            = 1;
    dynamicParams->numLFEChannels         = 0;
    dynamicParams->inputBitsPerSample     = 16;

    extn_dynpar->dualMono           = AACENC_FALSE;   /*!< Flag to indicate Dual Mono stream   */
    /* For RESET Command */
    extn_dynpar->useTns             = AACENC_TRUE;   /*!< Flag for activating TNS feature     */
    extn_dynpar->usePns             = AACENC_TRUE;   /*!< Flag for activating PNS feature     */
    extn_dynpar->useCRC             = AACENC_FALSE;   /*!< Flag for inserting CRC bits        */
    extn_dynpar->downMixFlag        = AACENC_FALSE;   /*!< downmixing Flag                  */
    extn_dynpar->outObjectType      = AACENC_OBJ_TYP_PS;   /*!< Output Object Type LC/HE/HEv2     */
    extn_dynpar->outFileFormat      = AACENC_TT_ADTS;   /*!< Output File Format            */
    extn_dynpar->ancFlag            = AACENC_FALSE;   /*!< ancillary data Flag                */
    extn_dynpar->ancRate            = -1;   /*!< ancillary Data Rate                */

    return;
}

/* nothing past this point */
