/******************************************************************************
 * FILE PURPOSE: Parse the configuration file for video.
 ******************************************************************************
 * FILE NAME:   siuVctParse.c
 *
 * DESCRIPTION: Contains routines that parse configuration file
 *
 * TABS: NONE
 *
 *
 * (C) Copyright 2010, Texas Instruments Incorporated
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include <ti/mas/types/types.h>
#include "siuVctParse.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

siuVctParams_t siuVctParams;


/* This is the place to add configuration enhancements. */
siuVctParseMapping_t siuVctParseMap[] =
{
    {"serverIpAddress",         siuVctParams.serverIpAddress   ,SIU_VCT_PARAM_TYPE_ADDRESS},
    {"serverMacAddress",        siuVctParams.serverMacAddress  ,SIU_VCT_PARAM_TYPE_ADDRESS},
    {"localIpAddress",          siuVctParams.localIpAddress    ,SIU_VCT_PARAM_TYPE_ADDRESS},
    {"localMacAddress",         siuVctParams.localMacAddress   ,SIU_VCT_PARAM_TYPE_ADDRESS},
    {"codecName",              &siuVctParams.codecName         ,SIU_VCT_PARAM_TYPE_ADDRESS},
    {"decCodecName",           &siuVctParams.decCodecName      ,SIU_VCT_PARAM_TYPE_ADDRESS},
    {"encCodecName",           &siuVctParams.encCodecName      ,SIU_VCT_PARAM_TYPE_ADDRESS},
    {"codecType",              &siuVctParams.codecType         ,SIU_VCT_PARAM_TYPE_ADDRESS},
    {"encode_or_decode",       &siuVctParams.encode_or_decode  ,SIU_VCT_PARAM_TYPE_ADDRESS},
    {"xdmVersion",             &siuVctParams.xdmVersion        ,SIU_VCT_PARAM_TYPE_ADDRESS},
    {"encResolution",          &siuVctParams.encResolution     ,SIU_VCT_PARAM_TYPE_ADDRESS},
    {"nCores",                 &siuVctParams.nCores            ,SIU_VCT_PARAM_TYPE_INT_DEC},
    {"coreTeamMapping",        &siuVctParams.coreTeamMapping   ,SIU_VCT_PARAM_TYPE_INT_DEC},
    {NULL,                      NULL                           ,0}
};

#pragma DATA_SECTION (siuVctParseBuf, ".siuVctParseBuf");
static char siuVctParseBuf[20000];
//Reads the configuration file content in a buffer and returns the address 
//of the buffer
static char *siuVctGetConfigFileContent (FILE *fname)
{
    tlong FileSize;
    if (0 != fseek (fname, 0, SEEK_END))
    {
        return 0;
    }

    FileSize = ftell (fname);
    if (FileSize < 0 || FileSize > 20000)
    {
        return 0;

    }
    if (0 != fseek (fname, 0, SEEK_SET))
    {
        return 0;
    }

    /****************************************************************************/
    /* Note that ftell() gives us the file size as the file system sees it.     */
    /* The actual file size, as reported by fread() below will be often smaller */ 
    /* due to CR/LF to CR conversion and/or control characters after the dos    */
    /* EOF marker in the file.                                                  */
    /****************************************************************************/
    FileSize = fread (siuVctParseBuf, 1, FileSize, fname);
    siuVctParseBuf[FileSize] = '\0';
    fclose (fname);
    return siuVctParseBuf;
}


/*
***********************************************************************
* \brief
*    Returns the index number from siuVctParseMap for a given parameter name.
* \param s
*    parameter name string
* \return
*    the index number if the string is a valid parameter name,         \n
*    -1 for error
***********************************************************************
*/
static tlong siuVctParameterNameToMapIndex (const char *s)
{
  tlong i = 0;

  while (siuVctParseMap[i].TokenName != NULL)
    if (0==strcmp (siuVctParseMap[i].TokenName, s))
      return i;
    else
      i++;
  return -1;
}

void siuVctParseContent (char *buf, tlong bufsize)
{

  char *items[SIUVCT_PARSE_MAX_NUM_ITEMS];
  tlong MapIdx;
  tlong item = 0;
  tlong InString = 0, InItem = 0;
  char *p = buf;
  char *bufend = &buf[bufsize];
  tlong tlongContent;
  double DoubleContent;
  tlong i;
  FILE *fpErr = stderr;

// Stage one: Generate an argc/argv-type list in items[], without comments and whitespace.
// This is context insensitive and could be done most easily with lex(1).

  while (p < bufend)
  {
    switch (*p)
    {
      case 13:
        p++;
        break;
      case '#':                 // Found comment
        *p = '\0';              // Replace '#' with '\0' in case of comment immediately following tlongeger or string
        while (*p != '\n' && p < bufend)  // Skip till EOL or EOF, whichever comes first
          p++;
        InString = 0;
        InItem = 0;
        break;
      case '\n':
        InItem = 0;
        InString = 0;
        *p++='\0';
        break;
      case ' ':
      case '\t':              // Skip whitespace, leave state unchanged
        if (InString)
          p++;
        else
        {                     // Terminate non-strings once whitespace is found
          *p++ = '\0';
          InItem = 0;
        }
        break;

      case '"':               // Begin/End of String
        *p++ = '\0';
        if (!InString)
        {
          items[item++] = p;
          InItem = ~InItem;
        }
        else
          InItem = 0;
        InString = ~InString; // Toggle
        break;

      default:
        if (!InItem)
        {
          items[item++] = p;
          InItem = ~InItem;
        }
        p++;
    }
  }

  item--;

  for (i=0; i<item; i+= 3)
  {
//    printf ("[%d]%s\n", i/3, items[i]); fflush(stdout);
  }

  for (i=0; i<item; i+= 3)
  {
    if (0 > (MapIdx = siuVctParameterNameToMapIndex (items[i])))
    {
      fprintf(fpErr, " Parsing error in config file: Parameter Name '%s' not recognized.", items[i]);

    }
    if (strcmp ("=", items[i+1]))
    {
      fprintf(fpErr, " Parsing error in config file: '=' expected as the second token in each line.");
    }

    // Now tlongerprete the Value, context sensitive...

    switch (siuVctParseMap[MapIdx].Type)
    {
      case SIU_VCT_PARAM_TYPE_INT_DEC:           // Numerical
        if (1 != sscanf (items[i+2], "%d", &tlongContent))
        {
          fprintf(fpErr, " Parsing error: Expected numerical value for Parameter of %s, found '%s'.", items[i], items[i+2]);
        }
        if (strcmp ("coreTeamMapping", items[i]) == 0)
        {
            int jj = 0;
            i++; // skipping the '='
            while(jj < SIUVCT_PARSE_NCORES)
            {
                sscanf (items[i+1], "%d", &tlongContent);
                siuVctParams.coreTeamMapping[jj] = tlongContent;

                if(tlongContent < 0)
                    break;

                i++;
                jj++;
            }
            i--;
        }
        else
        {
        * (tlong *) (siuVctParseMap[MapIdx].Place) = tlongContent;
        }
        break;
      case SIU_VCT_PARAM_TYPE_FILENAME:
        strncpy ((char *) siuVctParseMap[MapIdx].Place, items [i+2], SIUVCT_PARSE_FILE_NAME_SIZE);
        break;
      case SIU_VCT_PARAM_TYPE_DOUBLE:           // Numerical double
        if (1 != sscanf (items[i+2], "%f", &DoubleContent))
        {
          fprintf(fpErr, " Parsing error: Expected numerical value for Parameter of %s, found '%s'.", items[i], items[i+2]);
        }
        * (double *) (siuVctParseMap[MapIdx].Place) = DoubleContent;
        break;
      case SIU_VCT_PARAM_TYPE_INT_HEX:           // Numerical hex
        if (1 != sscanf (items[i+2], "%x", &DoubleContent))
        {
          fprintf(fpErr, " Parsing error: Expected numerical value for Parameter of %s, found '%s'.", items[i], items[i+2]);
        }
        * (double *) (siuVctParseMap[MapIdx].Place) = DoubleContent;
        break;
      case SIU_VCT_PARAM_TYPE_ADDRESS:
        strncpy ((char *) siuVctParseMap[MapIdx].Place, items [i+2], SIUVCT_PARSE_ADDRESS_SIZE);
        break;

      default:
      fprintf(fpErr, "Unknown value type in the map definition of configfile.h");
       // assert ("Unknown value type in the map definition of configfile.h");
    }
  }
}

tlong siuVctReadParamFile(const char *fileName)
{
    FILE *f, *fpErr = stderr;
    char *FileBuffer = NULL ;

    if (NULL == (f = fopen (fileName, "r")))
    {
        fprintf(fpErr,"Cannot open configuration file %s.\n", fileName);
        return 0;
    }
    
    memset(&siuVctParams, 0, sizeof(siuVctParams_t));

    //read the content in a buffer
    FileBuffer = siuVctGetConfigFileContent(f);

    if(FileBuffer)
    {
        siuVctParseContent(FileBuffer,strlen(FileBuffer));
        return 0;
    }
    else
        return -1;
}

/* nothing past this point*/
