/*
 * Copyright (C) 2010 Texas Instruments Incorporated - http://www.ti.com/
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include <stdio.h>
#include <string.h>
#include "targethw.h"
#include "siuVctFileIO.h"
#include "siuVctTftp.h"
#include "osal/siuOsal.h"

siuVctFileIOMgr_t fileIO;

#define SIUVCT_FILE_READ_BUFFER_SIZE    0x1000000   /* 16 MB */
tword siuVctFileReadBuffer[SIUVCT_FILE_READ_BUFFER_SIZE];
#pragma DATA_SECTION( siuVctFileReadBuffer, ".siuVctFileReadBuffer");

/******************************************************************************
 * FUNCTION PURPOSE: Initialize DDR pointers
 ******************************************************************************
 * DESCRIPTION: Initialize DDR pointers
 *
 * void siuVct_DDR_Init(
 *   XDAS_Int8 *    readAddr,        - Input to the Codec
 *   XDAS_UInt32    inputDataAvail   - Input buffer size
 *   XDAS_Int8 *    writeAddr        - Output of the Codec
 *   XDAS_UInt32    outputSizeAvail  - Output buffer size
 * )
 *****************************************************************************/
XDAS_Void siuVct_DDR_Init(void)
{
    memset(&fileIO, 0, sizeof(siuVctFileIOMgr_t));
    fileIO.read.ptr_start        = &siuVctFileReadBuffer[0];
    fileIO.read.ptr_working      = &siuVctFileReadBuffer[0];
    fileIO.read.memorySizeLimit  = SIUVCT_FILE_READ_BUFFER_SIZE;

    return;
}


/******************************************************************************
 * FUNCTION PURPOSE: Start TFTP transfers
 ******************************************************************************
 * DESCRIPTION: Start TFTP get of input and TFTP put of output.
 *
 *              siuVctStartTftpGet() and siuVctStartTftpPut() send a request
 *              to the TFTP server and waits for the server to send an ack.
 *
 * void siuVct_FileIO_Start(void)
 *
 *****************************************************************************/
XDAS_Void siuVct_FileIO_Start(void)
{
  siuVctStartTftpGet();

  /* TFTP Put is not started here. It will be started when the first output
   * frame is ready. This is to help prevent the TFTP server from timing out,
   * since the time out duration is less before any data has been sent.
   */
  //siuVctStartTftpPut();
  return;
}


/******************************************************************************
 * FUNCTION PURPOSE: Stop TFTP transfers
 ******************************************************************************
 * DESCRIPTION: Stop TFTP get of input and TFTP put of output.
 *
 * void siuVct_FileIO_Stop(void)
 *
 *****************************************************************************/
XDAS_Void siuVct_FileIO_Stop(void)
{
  siuVctStopTftpGet();
  siuVctStopTftpPut();
  return;
}


/******************************************************************************
 * FUNCTION PURPOSE: Read input data.
 ******************************************************************************
 * DESCRIPTION: Reads data from input file buffer into codec input buffer.
 *              This function also manages the file buffer by using TFTP to
 *              keep enough data in the buffer.
 *
 * XDAS_Int32 siuVct_ReadInputData(
 *
 *    XDAS_Int8 *pBuf,            - pointer to codec input buffer.
 *    XDAS_UInt32 bufSizes,       - size of the codec input buffer.
 *    XDAS_UInt32 bytesConsumed   - number of bytes consumed from
 *                                  codec input buffer.
 *  )
 *
 *****************************************************************************/
XDAS_Int32 siuVct_ReadInputData(XDAS_Int8 *pBuf, XDAS_UInt32 bufSizes, XDAS_UInt32 bytesConsumed)
{
    siuVctIOmemDef_t *readPtr = &(fileIO.read);

    /*
     *  Update file buffer with the nuber of bytes consumed
     *  and set the buffer threshold to the number of bytes
     *  requested.
     */
    thwCriticalBegin();
    readPtr->numValidBytes -= bytesConsumed;
    readPtr->ptr_working   += bytesConsumed;
    readPtr->tftpThresh     = bufSizes;
    thwCriticalEnd();

    /*
     *  Use TFTP to transfer in data from a server. This
     *  function will wait until the threshold is met.
     */
    siuVctTftpGetData();

    thwCriticalBegin();
    if( readPtr->numValidBytes < bufSizes)
      bufSizes = readPtr->numValidBytes;

    siu_invalidate_cache();

    /*
     *  Copy data from file buffer to codec buffer
     */
    memcpy(pBuf, readPtr->ptr_working, bufSizes);
    thwCriticalEnd();
    siu_osal_wbinv_cache(pBuf, bufSizes, TRUE);

    if (readPtr->numValidBytes < 0) {
        return(0);
    }
    return(bufSizes);
} /* siuVct_ReadInputData() */


/******************************************************************************
 * FUNCTION PURPOSE: Write output data
 ******************************************************************************
 * DESCRIPTION: Writes data from codec output buffer into output file buffer.
 *              This function also manages the file buffer by using TFTP to
 *              send out the data in the buffer.
 *
 * void siuVct_WriteOutputData(
 *
 *    XDAS_Int8 *pBuf,            - pointer to codec input buffer.
 *    XDAS_UInt32 bufSizes,       - size of the codec input buffer.
 *  )
 *
 *****************************************************************************/
XDAS_Void siuVct_WriteOutputData(XDAS_Int8 *pBuf,
                                 XDAS_UInt32 bufSizes)
{
  siu_osal_wbinv_cache(pBuf, bufSizes, TRUE);
#if 0
  siuVctTftpPutData(pBuf,bufSizes);
#else
  siuVctVppuPutData(pBuf,bufSizes);
#endif

  return;
} /* siuVct_WriteOutputData() */
