/******************************************************************************
 * FILE PURPOSE: Implementation of Vigdk Initialization code
 ******************************************************************************
 * FILE NAME:   siuVigdkInit.c
 *
 * DESCRIPTION: This file contains the implementation for the initialization of
 *              SIU-VIGDK
 *
 * FUNCTION           DESCRIPTION
 * --------           -----------
 *
 * (C) Copyright 2008, Texas Instruments Inc.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include "common/siu.h"
#include "common/siuloc.h"
#include "common/dsiunwal.h"
#include "osal/siuIpcOsal.h"
#include "ggcfg.h"
#include "hal.h"
#include "utlrepck.h"
#include "rttrace.h"
#include "siutftp.h"
#include "setjmp.h"
#include "string.h"
#include "ividmc/siuVidMc.h"
#include "osal/siuFcOsal.h"
#include "gghwcfg.h"

#include "ggCodecTestCfg.h"      /* Defines Golden Gateway configuration data */

#include <ti/mas/rcu/rcu.h>
#include <ti/mas/vppu/vppu.h>
#include "vctypes.h"

volatile tint HwConfig_control = 0;

/*********************************************************************************
 * FUNCTION PURPOSE: System Initialization
 *********************************************************************************
  DESCRIPTION:      This function is called during initialization of the system.
                    It does the software, hardware initializations for the system
                    and also the application specific context and channel instances
  Parameters :      Inputs: None
                    Output: None
 *********************************************************************************/
void siuMain (void)
{

  /* This function is called during initialization of the system.           *
   * It does platform specific initialization, calls Hardware Abstraction   *
   * Layer (HAL) initialization and sets up the peripherals.                */
   siuHwInit();

  /* This function is called during initialization of the system.           *
   * It brings up the network interface (which is essential for the         *
   * message path for future communication between DIM and DSP) and         *
   * a random seed in initialized.                                          */
   siuSwInit();

  /* Initializes application independent portions of siuContext and         *
   * all the SIU Instances.                                                 */
   siuFrameworkInit();

  /* Initializes video application specific portions of siuContext, which   *
   * are not initialized in common.                                         */
   siuVigdkInit();

   siuContext.state = SIU_ST_READY;
   siuContext.hw_config_received = FALSE;

   return;

} /* siuMain */

#define INTERNAL_SHARED_DATA_MEM_SIZE 1024
#define EXTERNAL_SHARED_DATA_MEM_SIZE 0x2000000

#ifndef SINGLE_CORE_ONLY
/* Shared memory in SL2 */
#pragma DATA_SECTION(shared_mem_SL2, ".shared_mem_SL2")
XDAS_UInt8 shared_mem_SL2[INTERNAL_SHARED_DATA_MEM_SIZE];

/* Shared memory in DDR2 */
#pragma DATA_SECTION(shared_mem_DDR2, ".shared_mem_DDR2")
XDAS_UInt8 shared_mem_DDR2[EXTERNAL_SHARED_DATA_MEM_SIZE];
#endif

/*********************************************************************************
 * FUNCTION PURPOSE: Video Gateway Developer Kit Initialization
 *********************************************************************************
  DESCRIPTION:      This function is called during initialization of the system.
                    It does the software initializations for the video sub system
  Parameters :      Inputs: None
                    Output: None
 *********************************************************************************/
void siuVigdkInit(void)
{
  extern const vigdkMemoryContext_t gg_supported_builds[];
  const vigdkMemoryContext_t *activeBuild;
  siuVidShmHandle_t   *codecShmHandle = &(siuContext.codecShmHandle);
  extern cregister volatile unsigned int TSCL;
  extern tulong siuVctMsgs[];
  int i;
  mgb_t *storedMsgs = (mgb_t *)siuVctMsgs;
  tuint edmaInstances[CPU_NCORES] = GG_DMA_INSTANCE_ARRAY;
  tuint edmaQueues   [CPU_NCORES] = GG_DMA_QUEUE_ARRAY;

  TSCL = 0; /* Activate Timer */

  activeBuild = &gg_supported_builds[0];

  /* Codec Shared Memory */
  memset(codecShmHandle, 0, sizeof(siuVidShmHandle_t));

#ifndef SINGLE_CORE_ONLY
  codecShmHandle->intShm            = &shared_mem_SL2[0];
  codecShmHandle->intShmSize        = INTERNAL_SHARED_DATA_MEM_SIZE;

  codecShmHandle->extShm            = &shared_mem_DDR2[0];
  codecShmHandle->extShmSize        = EXTERNAL_SHARED_DATA_MEM_SIZE;
#endif

  /* Set the External memory cacheability based on Build ID */
  siu_invalidate_cache();
  siu_init_hal(activeBuild->marRegs, activeBuild->marVals);
  siu_invalidate_cache();

  /* RMAN initialization */
  if(siuContext.rmanConfigFlag) {
    siu_exc_assert (siu_osal_fc_resoure_init(edmaInstances[siuContext.coreId], edmaQueues[siuContext.coreId], siuContext.coreId, FALSE)
                    == TRUE, SIUVIGDK_EXC_RMAN_INIT_FAIL, &siuContext);
  }

  /* Execute messages */
   for(i=0;i<20;i++) {
      mgb_t *msg      = &storedMsgs[i];
      siuInst_t *inst = SIUINSTp(msg->header.chnum - SIU_FIRSTCH);

      switch(msg->header.msg_id) {
        case MGB_MD_HW_CONFIG:
            {
                if (!siuContext.hw_config_received) {
                   siuHwConfig (msg);
                   siuContext.hw_config_received = TRUE;
                   siuContext.seqNum = 0;
                }
            }
            break;

        case MGB_MD_NET_ENCAP_CONTROL:
            if(i!=9) {
                siu_net_encap_control(inst, msg);
            }
            break;

        case MGB_MD_ENCAP_CONFIG:
            siuEncapConfig(inst, msg);
            break;

        case MGB_MD_CHANNEL_STATE:
            siuChannelState(inst, msg);
            break;

        case MGB_MD_PKTIO_IF_CONFIG:
#ifndef PA_SS
            /*ignore PKTIO IF Config for PA*/
            siuPktioIfCfg(msg);
#endif
            break;

        case MGB_MD_OPEN_CHANNEL:
            siuOpenChannel(inst, msg);
            break;

        case MGB_MD_VIDEO_MODE:
            siuVideoMode(inst, msg);
            break;

        case MGB_MD_COMMIT_STATE:
            siuCommitState(inst, msg);
            break;

        case MGB_MD_TDM_PORT_CONFIG:
            /* ignore TDM Port Config */
            break;

        default:
            siu_exception(inst->ID, 1);
            break;
      }
      //siuProcessMessage(1, &storedMsgs[i]);
   }
}

/******************************************************************************
 * FUNCTION PURPOSE:  Process hardware configuration message from MIC
 ******************************************************************************
 * DESCRIPTION:   Processes a hardware configuration message from MIC.  If
 *                valid, DSP configures itself for the selected hardware
 *                configuration and reports a ready indication to MIC.  If
 *                any configuration parameters are invalid an error is reported.
 *
 *****************************************************************************/
// volatile tuint emuwait_1=1;
void siuHwConfig (mgb_t *message)
{

// while(emuwait_1);

  /* Check message parameters are valid -- if not ignore message and report
     error */
  if (((message->msg_body.hw_config.companding != MGB_MULAW_COMPANDING) &&
      (message->msg_body.hw_config.companding != MGB_ALAW_COMPANDING) &&
      (message->msg_body.hw_config.companding != MGB_LINEAR_COMPANDING)) ||
      ((message->msg_body.hw_config.bdx_delay_control != MGB_BDX_DELAY_ENABLE) &&
      (message->msg_body.hw_config.bdx_delay_control != MGB_BDX_DELAY_DISABLE)) ||
      ((message->msg_body.hw_config.clkout_config != MGB_CLKOUT_ENABLE) &&
       (message->msg_body.hw_config.clkout_config != MGB_CLKOUT_DISABLE)))
  {
     siuErrorIndication (siuContext.ID, message->header.trans_id,
                         MGB_ERR_INVLD_HW_CONFIG);
     return;
  }

  /* make sure we got a valid sampling rate for this build */
  /* Program clock PLL */
  if (message->msg_body.hw_config.clk_multplier != MGB_CLK_MULT_INVALID) {
    if (*siuContext.halApi->halPllControl != NULL) {
      (*siuContext.halApi->halPllControl) (message->msg_body.hw_config.clk_multplier,1);
    }
  }

  /* Get the values for sleep timer for power management */
  siuContext.timer_period = message->msg_body.hw_config.timer_period;
  siuContext.wake_up_interrupt_mask = message->msg_body.hw_config.wake_up_interrupt_mask;

  /* CLKOUT Pin configuration */
  if (message->msg_body.hw_config.clkout_config == MGB_CLKOUT_ENABLE) {
    thwResetCLKOFFPin();
  }
  else {
    if (message->msg_body.hw_config.clkout_config ==  MGB_CLKOUT_DISABLE) {
      thwSetCLKOFFPin();
    }
  }

  /* Keep alive - health check generation */
#ifdef GG_INCLUDE_HEALTH_CHECK
    if (siuHealthInit(siu_timer_tick,
            message->msg_body.hw_config.keep_alive,
            message->msg_body.hw_config.core_mhz) == SIU_ERROR)  {
       siuErrorIndication (siuContext.ID, message->header.trans_id,
                           MGB_ERR_INVLD_HW_CONFIG);
    }
#endif

  if (siuContext.siuInitCoreMhz)
  {
    siuContext.siuInitCoreMhz(message->header.length >= MGB_HEADER_SIZE + 27 ?
                              message->msg_body.hw_config.core_mhz : 0);
  }


#ifdef RELMSG_SUPPORT
  /* If check allows backwards compatibility with old micro */
  if (message->header.length >= 34) {
    siuContext.msg_burst_size =  message->msg_body.hw_config.msg_burst_size;
  } else {
    siuContext.msg_burst_size =  0;
  }
#endif

#ifdef SIU_TIMER_BASED_TDM
  siuTsipEmulConfig(message);
#endif

} /* siuHwConfig() */

/******************************************************************************
 * FUNCTION PURPOSE: Initialize SIU right after download.
 ******************************************************************************
 * DESCRIPTION: It initializes the internal SIU context, and SIU instance
 *              structure for operation.  After execution, SIU state is
 *              set to SIU_ST_START.
 *
 *****************************************************************************/
void siuFrameworkInit (void)
{
  int   i;

  /* use chnum 0 at this time */
  siuContext.ID                = siuMakeID (SIU_MID_SIU, 0);
  siuContext.debugInfo         = siuDebugInfo;
  siuContext.maxNumberChannels = (ggConfig.max_channels +
                                  ggConfig.aux_pcm_channels +
                                  ggConfig.pkt_channels);
  siuContext.maxFullChannels   = (ggConfig.max_channels + ggConfig.aux_pcm_channels);

  siuContext.numActiveChannels =  0; /* no active channels (all closed) */

  /* Initialize DSP-to-Micro signaling and normal message FIFO queue */
  /* normal message */
  siuContext.msgFifo.inIndex     = 0;
  siuContext.msgFifo.outIndex    = 0;
  siuContext.msgFifo.freeSpace   = ggConfig.siuMsgFifoLen;
  siuContext.msgFifo.dropCounter = 0;

  /* signaling message */
  siuContext.sigFifo.inIndex     =
  siuContext.sigFifo.outIndex    = 0;
  siuContext.sigFifo.freeSpace   = ggConfig.siuSigFifoLen;
  siuContext.sigFifo.dropCounter = 0;

  /* Low priority messages */
  siuContext.lowFifo.inIndex     = 0;
  siuContext.lowFifo.outIndex    = 0;
  siuContext.lowFifo.freeSpace   = ggConfig.siuLowFifoLen;
  siuContext.lowFifo.dropCounter = 0;

  /* 5ms isr task counter */

  /* Core speed initialization */
  siuContext.siuInitCoreMhz = ggConfig.initCoreMhz;

  /* Set flag for RMAN configuration */
  siuContext.rmanConfigFlag = 1;

  /* Initialize SIU pointers */
  /* Initialize all _allocated_ channels and "make" them "closed" */
  for (i = 0; i < siuContext.maxNumberChannels; i++)
  { /* Initialize statically allocated instances immediatelly */
    siuInitSiu (i);
  }

#if 0 /* Heap inst is only used for stats which is not needed in VCT */
  /* Is there enough memory to store siu heap data ? */
  siu_exc_assert(gg_siuheap_inst_len >= sizeof(siuHeapInst_t), SIUINIT_EXC_HEAPINST_MEMORY, &siuContext);

  siu_exc_assert(gg_siuheap_inst_ext_len >= sizeof(siuHeapInstExt_t),
                 SIUINIT_EXC_HEAPINST_MEMORY, &siuContext);
#endif

  /* Default BuildID to zero */
  siuContext.buildID = 0;

} /* siuFrameworkInit */

/******************************************************************************
 * FUNCTION PURPOSE: Initialize SIU instance to known state
 ******************************************************************************
 * DESCRIPTION: Reset all ECO pointers, indicate closed state, etc...
 *****************************************************************************/
void siuInitSiu (tuint i)
{
   tuint chnum = SIU_FIRSTCH + i;
   tuint ID = siuMakeID (SIU_MID_SIU, chnum);
   siuInst_t *inst = SIUINSTp(i);
    memset (inst, 0, sizeof(siuInst_t));
    /* set SIU channel state to CLOSED */
    inst->state          = SIU_CHST_CLOSED;
    inst->pending_state  = SIU_CHST_CLOSED;
    inst->ID                 = ID;
    inst->chHeapID           = NULL;
    inst->state_transition_msg_id = 0;
    inst->neuInst            = NULL;
    SIU_SET_TX_ENABLED(inst, FALSE);
    /* NDU Instance */
    inst->amInst.RxOutSpec.nduReceiveOut = NULL;
    inst->amInst.RxOutSpec.targetInst    = NULL;

} /* siuInitSiu() */

/************************************************************************************
 * FUNCTION PURPOSE: Program memory copy
 ************************************************************************************
 * DESCRIPTION: Performs a one time copy of memory.
 ************************************************************************************/
void siu_init_prg_copy (void)
{
  halLocAddr_t dest, src;
  halMemMove_t memMove;

  if (!ggConfig.loadSize)
    return;  /* nothing to copy */

  /* Convert addresses and sizes for use by HAL */
  dest = (halLocAddr_t) UTL_BYTES_TO_WORDS((tulong)ggConfig.runStart);
  memMove.dest = (*siuContext.halApi->halMapLocalToGlobal)(dest);

  src  = (halLocAddr_t) UTL_BYTES_TO_WORDS(ggConfig.loadStart);
  memMove.src = (*siuContext.halApi->halMapLocalToGlobal) (src);
#ifdef GG_HAL_GLOBAL_ADDR_BIT_24
  memMove.src = HAL_MEM_ADDRESS_PROG(memMove.src);
#endif

  memMove.size = (halSAE_t) UTL_BYTES_TO_WORDS(ggConfig.loadSize);

  /* Copy the data */
  (*siuContext.halApi->halGlobMemMove) (1, &memMove, NULL, NULL, TRUE);

} /* siu_init_prg_copy */

/*********************************************************************************
 * FUNCTION PURPOSE: System Hardware Initialization
 *********************************************************************************
  DESCRIPTION:      This function is called during initialization of the system.
                    It does platform specific initialization, calls Hardware Abstraction
                    Layer (HAL) initialization and sets up the peripherals.
  Parameters :      Inputs: None
                    Output: Revision ID.
 *********************************************************************************/
tuint siuHwInit (void)
{
  tuint revId = 0; /* Not supported yet */
  siuReturn_t halRet,halRetNew;
  tuint platformInitError;
  extern const vigdkMemoryContext_t gg_supported_builds[];
  const vigdkMemoryContext_t *activeBuild;

  /* Default configuration: Build 0 */
  activeBuild = &gg_supported_builds[0];

  /* Don't die while processing interrupts */
  siuContext.halt.halt.halt_if_set = siuContext.halt.halt.halted_if_set = 0;

  /* Initialize HAL APIs and memory config */
  /* Assertion cannot be done here so store result and assert after ndu is
     initialized */
/*  !!!! Warning:  Don't do anything else before this function
  for example in case of titan this initialises the emif access .
  so it is critical that this is executed before any external memory program execution*/
  halRet = siu_init_hal(activeBuild->marRegs, activeBuild->marVals);

  /* Platform-specific initialization */
  platformInitError = (platformInit() == GGCFG_NOERR);

  /*
   * Warning: Be careful if merging the following code!!!
   * Global interrupt should be enabled within HAL and platform initialization.
   * But it was not at this branch.
   * It is only a temporary fix to enable global interrupt and serves as a reminder.
   * If the global interrupt is enabled at this point, do not merge this code.
   * If it is not, we may want to find a better way to do it.
   */
  thwCriticalReset();

  /* NDU Start must be the first function to be called.If any of the functions below
     go to exception , it will try to send a message to micro before NDU is intialised
     and crash the DSP */

   /* This is required by NDU for NV ecos, in siumgrv.c */
  siuContext.cellSize          = SIU_DEFAULT_CELL_SIZE;
  /* Initialize hardware configuration */
  halRetNew = siu_new_hal(); //(SIU_DEFAULT_CELL_SIZE, &revId);

#ifdef PA_SS
  /*Initializing PA, QM, CPPI/NWAL*/
  if (dsiuNwInit () == FALSE)  {
    dsiuNwLog ("ERROR: dsiuNwInit FAILED \n");
    return;
  }
  else
  {
    dsiuNwLog ("dsiuNwInit Passed \n");
  }
#endif

  /*  Following will initialize GMAC or Utopia. This function must be called
   *  before siuNduStart to get the halId of the outgoing message peripheral */
  siu_periph_setup();

  siu_exc_assert(halRet == SIU_SUCCESS,SIUHAL_EXC_HALINIT,&siuContext);
  siu_exc_assert(halRetNew ==  SIU_SUCCESS,SIUHAL_EXC_HALNEW,&siuContext);
  siu_exc_assert(platformInitError,SIUGG_EXC_GGPLATINIT,&siuContext);

  return (revId);
} /* siuHwInit */

/*********************************************************************************
 * FUNCTION PURPOSE: System Software Initialization
 *********************************************************************************
  DESCRIPTION:      This function is called during initialization of the system.
                    It brings up the network interface (which is essential for the
                    message path for future communication between DIM and DSP) and
                    a random seed in initialized.
  Parameters :      Inputs: None
                    Output: None
 *********************************************************************************/
void siuSwInit(void)
{
  tuint tmp;
   /* Bring up the network Driver Interface */
   siuNduStart();

  /* Copy any program memory sections */
  siu_init_prg_copy ();

  /* Initialize random seed in the DSP */
  tmp = (siuContext.dspId << 8) + siuContext.coreId;
  srand(tmp);

#ifndef SINGLE_CORE_ONLY
  siu_ipc_osal_init();
#endif

  return;

 } /* siuSwInit */

/******************************************************************************
 * FUNCTION PURPOSE: Configure network for voice/fax/transparent mode
 ******************************************************************************
 * DESCRIPTION:  Opens NDU, ACU and/or SIOU
 *
 *  void siu_open_network (
 *    tint chnum)        - Channel #
 *    void (*netReceiveIn) (void *inst, tint cnt[], void *pkts[]),
 *    void *targetInst,
 *    bool rcuApi)       - TRUE : use new RCU api; FALSE use old NDU API
 *
 *****************************************************************************/

void siu_open_network (tint chnum, tint (*netReceiveIn) (void *inst, xferPktInfo_t *pktInfo),
                       void *targetInst, bool rcuApi)
{
  siuInst_t *inst = SIUINSTp(chnum-SIU_FIRSTCH);
  siuNduConfig_t   nduConfig;   /* Set up NDU ReceiveOut */
  neuCtrl_t        neuConfig;   /* Set up NEU ReceiveOut */

  SIU_RXINT_BEGIN();
  /* Configure NDU/ACU/RCU receiveout */
  if (SIU_IS_NEU_PRESENT(inst)){
    nduConfig.RxOutSpec.nduReceiveOut           = SIU_NDU_GET_NEU_RECEIVE_IN;
    nduConfig.RxOutSpec.targetInst              = inst->neuInst;
    neuConfig.u.neuReceiveOutSpec.neuReceiveOut = tftpReceiveIn;
    neuConfig.u.neuReceiveOutSpec.targetInst    = inst->net.tftpInst;
    neuConfig.code = neu_CTRL_RX_RECEIVE_OUT_CFG;
    (*gg_neu_ctable.neuControl) (inst->neuInst, &neuConfig);
    /* Configure ACU to point to neu for sendout, if present */
  }

  /* Open NDU */
  siuAmLinkNdu(inst, nduConfig.RxOutSpec.nduReceiveOut, nduConfig.RxOutSpec.targetInst);
  SIU_RXINT_END();
} /* siu_open_network */

/******************************************************************************
 * FUNCTION PURPOSE: Executes change to Video Mode.
 ******************************************************************************
 * DESCRIPTION: Implements start of transition to voice mode. Assumes channel is
 *              in an IDLE configuration upon entry.  Channel state is not changed
 *              to SIU_CHST_VIDEO, but pending_state is.
 *              All ECOs are opened at this point if there is sufficient information
 *              for its configuration, i.e. provided in open channel configuration.
 *
 *  void siuVideoMode (
 *    siuInst_t *inst         - pointer to channel information
 *    mgb_t     *message)     - pointer to micro message
 *
 *****************************************************************************/
volatile tulong TxSyncSrc = 0x00000064; /* Adjust this according to application */
void siuVideoMode (siuInst_t *inst, mgb_t *message)
{
  tint chnum      = (tint) (inst->ID & SIU_CID_MASK);

  /* Make Sure channel is in Idle Mode */
  if (inst->state != SIU_CHST_IDLE) {
    siuErrorIndication (inst->ID, message->header.trans_id, MGB_ERR_VIDEO_MODE_MSG_REJECTED);
    return;
  }

  /* PROTOCOL type should have been configured to non-MGB_PT_NONE by now, else
     reject the message */
  if(inst->protocol_type == MGB_PT_NONE) {
     siuErrorIndication (inst->ID, message->header.trans_id, MGB_ERR_NO_PROTOCOL_TYPE_CONFIGURED);
     return;
  }

  inst->maxPayloadSize  = SIU_DEFAULT_TX_MAX_PAYLOAD_SIZE;

  /* Invalidate last packet type, to force siu_stm_search to re-search */
  inst->lastPacketType = 0xFFFFU;
  /* Save current payload Type */
  inst->payloadType = message->msg_body.video_mode.payloadType;

  /* Open NDU or ACU or RCU w/ new API */
  if(chnum==1) {
    siu_open_network (chnum, NULL, NULL, TRUE);
  } else if (chnum==2) {
    siu_new_vppu(chnum);
    siu_open_network (chnum, gg_vppu_ctable.vppuReceiveIn, inst->stateInst.video.vppuInst, TRUE);
  }


  /* VIDEO mode is not entered */
  inst->pending_state = SIU_CHST_VIDEO;

  SIU_STATE_BEGIN();
  siuTxOpen (inst);   /*R9.2, do not open Tx Task until COMMIT state arrived*/
  SIU_STATE_END();

  if(chnum==1) {
    siu_open_tftp (inst);
  } else if(chnum==2) {
      siu_open_rcu (chnum);

      /* RTP Encap Config*/
      {
        rcuCtrl_t rcuCtrlTx;

        SIU_STATE_BEGIN();

        /* Always set some RCU fields */
        rcuCtrlTx.u.tx.bitmap = 0;
        rcuCtrlTx.code        = rcu_CTRL_TX;

        /* Configure Tx SSRC */
        rcuCtrlTx.u.tx.pF.ssrc = TxSyncSrc;
        rcuCtrlTx.u.tx.bitmap |= rcu_CTRL_TX_SSRC;

        gg_rcu_ctable.rcuControl (inst->net.rcuInst, &rcuCtrlTx);
        SIU_STATE_END();
      }
  }

} /* siuVideoMode */

/******************************************************************************
 * FUNCTION PURPOSE: Creates a full video channel
 ******************************************************************************
 * DESCRIPTION: This routine news all modules for Video operation. Creates
 *              all video modules - allocates memory resources for each module
 *              and initializes then to CLOSED state
 *
 *****************************************************************************/
void siuCreateVideoCh (tint chnum)
{
  siuInst_t *inst = SIUINSTp(chnum-SIU_FIRSTCH);
  /* Auxiliary channels share same address space - they are upper half of all channels */
  bool chan_full  = chnum < (SIU_FIRSTCH + ggConfig.max_channels) ? TRUE : FALSE;
  tuint heap_size = chan_full ? ggchVoiceHeapSize : ggchVoiceAuxHeapSize;
  tint slotnum; /* It is 0 based */
  /* Get a slot for this channel */
  slotnum = chnum-SIU_FIRSTCH;

  /* Initialize channel heap for voice */
  inst->chHeapID = mhmCreate ((void *)ggchVoiceHeapStartAddr[slotnum],
                                 heap_size, 0 /* heap init pattern */);

  /* Create NEU module */
  siu_new_neu(chnum);
  if(chnum==1) {
    /* Create TFTP module */
	siu_new_tftp(chnum);
  } else if(chnum==2) {
	  /* Create RCU module */
    siu_new_rcu(chnum);
  }
} /* siuCreateVideoCh */


/******************************************************************************
 * FUNCTION PURPOSE: Create channels at DSP start-up
 ******************************************************************************
 * DESCRIPTION: This routine is performed at DSP start-up and initializes all
 *              SIU channels.  For all channels voice modules are created.
 *              Upon return all channels are in CLOSED state.
 *
 *****************************************************************************/
void siuCreateChannels ( void )
{
#ifndef SIU_DYNSUBSLOT
  tint chnum,i;
  siuInst_t *inst;
  const tint n = siuContext.maxNumberChannels;

  for (i = 0, chnum = SIU_FIRSTCH; i < n ; i++, chnum++)
  {
    inst = SIUINSTp(i);
    /* set the data page register to point to channel's data page */
    thw_change_data_page( SIU_READ_PRIMARY_DATA_PAGE(inst) );
  }
#endif

  #ifdef SIUTBLPRG
  siu_init_msg_dispatch();
  #endif
}  /* siuCreateChannels */

/******************************************************************************
 * FUNCTION PURPOSE: Open DSP Channel
 ******************************************************************************
 * DESCRIPTION: This routine copies the channel information, and opens PIU
 *              module in IDLE channel state configuration to get ready for
 *              possible tone generation. In the IDLE state PIU is configured
 *              for Voice/DTMF operation. Channel state is set to SIU_CHST_IDLE
 *              before normal return.  It is assumed Channel is currently CLOSED.
 *
 *              NOTE: Will result in exception if invalid parameters are
 *                    found in message.
 *
 *****************************************************************************/
void siuOpenChannel (siuInst_t *inst, mgb_t *message)
{
  tint slotnum,heapInstSlot;
  word *siu_heap_inst_ext_ptr;
  tint chanInt = (inst->ID & SIU_CID_MASK) - SIU_FIRSTCH;

  /* Init heap instance */
  /* Get the slot for this channel */
  slotnum = chanInt;
  if (siuContext.dynBufAlloc.dynBufAllocEnable) {
    slotnum = siuContext.dynBufAlloc.siuCh2Slot[chanInt];
  }
  heapInstSlot = slotnum;

#if 0 /* Heap inst is only used for stats which is not needed in VCT */
  inst->heapInst = (siuHeapInst_t *)gg_siuheap_inst[heapInstSlot];

  /*********************************/
  /* Initialize channel statistics */
  /*********************************/
  memset ((void *)globStatsSiuGetPtr(inst->ID), 0, sizeof (siuStats_t));

  /* In some builds, a part of siu heap instance, comprising of profiles,
     error stats are kept in external memory. Pointers to these places
     are kept in gg_siuheap_inst_ext[]. Unlike its internal memory counterpart,
     (gg_siuheap_inst[]), this table of pointers is not accessed after
     opening a channel,so could be kept in external memory. Since this is a
     common table across cores and the location of where the pointers point to,
     are in external shared memory, they need to be mapped through remap_mcs
     just like every other buffer, before the pointer is assigned to
     the internal mem data structure */
  siu_heap_inst_ext_ptr=siu_remap_mcs_data(gg_siuheap_inst_ext[heapInstSlot]);
  /* Initialize the portion of stats in external memory */
  memset (siu_heap_inst_ext_ptr, 0, sizeof (siuHeapInstExt_t));

  inst->heapInst->stats.pktStats.proto.rtp.err=
          &(((siuHeapInstExt_t *)siu_heap_inst_ext_ptr)->stats.err.pktErrStats.proto.rtp);
#endif

  siuCreateVideoCh(message->header.chnum);

  /* Still not aware of protocol type */
  inst->protocol_type =  MGB_PT_NONE;

  /* Default configuration parameters are set. */
  /* increment # of active channels (those that are NOT closed). */
  /* Note: We need do it here, not at siuCommitStateOpen() since channel-based exception
   *       may occur between "OPEN_CHANNEL" and "COMMIT_STATE" */
  siuContext.numActiveChannels++;
  inst->pending_state = SIU_CHST_IDLE;
} /* siuOpenChannel */
/******************************************************************************
 * FUNCTION PURPOSE: Finish transition to idle state.
 ******************************************************************************
 * DESCRIPTION: Add necessary default parameters, according to second stage data,
 *  and already received configuration parameters from the HOST:
 * 1. If no encap config data are received between open_channel and commit_state,
 *    and build supports AAL1 mode, it defaults to it. But, if build does not
 *   support AAL1, error indication is generated.
 * 2. According to configured protocol type, data flow on the network side is
 *    initialized.
 * 3. NEU is opened and initialized w/ default configuration parameters.
 * 4. NDU or serial packet interface is opened
 * 5. If TDM configuration has not been initialized, we provide default with
 *    timeslots set to channel number.
 ******************************************************************************/
void siuCommitStateOpen (siuInst_t *inst, mgb_t *message) {
  siuNduConfig_t   nduConfig;   /* configuration structure for NDU */
  neuCtrl_t        neuConfig;   /* configuration data for NEU */
  tuint            count;

  memset(&nduConfig, 0, sizeof(siuNduConfig_t));
  memset(&neuConfig, 0, sizeof(neuCtrl_t));

  if (SIU_IS_NEU_PRESENT(inst)) {
    /* If NEU is in the build, initialize configuration parameters. */
    neuConfig.u.parm.chnl.cfg_flag1              = 0;
    neuConfig.u.parm.chnl.cfg_flag2              = 0;
    neuConfig.u.parm.chnl.sar_type               = neuproto_SAR_NONE;
    neuConfig.u.parm.chnl.u.sar_param.cid           = 0;
    neuConfig.u.parm.chnl.u.sar_param.ra_time_out   = 0;
    neuConfig.u.parm.chnl.u.sar_param.max_cps_size  = 45;
    neuConfig.u.parm.chnl.alt_local_port         = 0;
    neuConfig.u.parm.chnl.alt_remote_port        = 0;

    count = 0;
    while(count < MGB_IP_ADDR_MAX_NUM_WORDS)
    {
        neuConfig.u.parm.chnl.alt_remote_addr[count] = 0;
        count++;
    }

    neuConfig.u.parm.chnl.report_ctrl            = 0;
    neuConfig.u.parm.chnl.report_throttle_time   = 0;
    neuConfig.u.parm.chnl.sysInst                = inst;
    neuConfig.u.parm.chnl.second_prototype       = 0;
    neuConfig.u.parm.chnl.rx_offset_bytes        = 0;
    neuConfig.u.parm.chnl.rx_payload_type        = 0;
    neuConfig.u.parm.chnl.tx_min_cell_bytes      = 0;
    neuConfig.u.parm.chnl.tx_ip_offset_bytes     = -1;
    neuConfig.u.parm.chnl.tx_udp_offset_bytes    = -1;
    neuConfig.u.parm.chnl.tx_802_3_offset_bytes  = -1;
    neuConfig.u.parm.chnl.tx_total_hdr_len_bytes = 0;
    neuConfig.u.parm.chnl.tx_udp_src_mask        = 0;
    neuConfig.u.parm.chnl.tx_udp_src_val         = 0;
    neuConfig.u.parm.chnl.enable                 = FALSE;
    siu_open_neu (message->header.chnum, &neuConfig.u.parm);

    /* Insert NEU in receive path */
    neuConfig.u.neuReceiveOutSpec.neuReceiveOut = nduConfig.RxOutSpec.nduReceiveOut;
    neuConfig.u.neuReceiveOutSpec.targetInst    = nduConfig.RxOutSpec.targetInst;
    nduConfig.RxOutSpec.nduReceiveOut           = SIU_NDU_GET_NEU_RECEIVE_IN;
    nduConfig.RxOutSpec.targetInst              = inst->neuInst;
    neuConfig.code = neu_CTRL_RX_RECEIVE_OUT_CFG;
    (*gg_neu_ctable.neuControl)(inst->neuInst, &neuConfig);
  }

  /* HPI packet transfer: Open NDU */
  siuAmLinkNdu(inst, SIU_NDU_GET_NEU_RECEIVE_IN, inst->neuInst);

  /* Note: It is moved to siuOpenChannel()
   * siuContext.numActiveChannels++; */
  inst->state = SIU_CHST_IDLE;
}
/******************************************************************************
 * FUNCTION PURPOSE: General commit configuration state machine.
 ******************************************************************************
 * DESCRIPTION: Invoke function that does final transition to pending state.
 ******************************************************************************/
volatile tulong txInitTimeStamp = 0;
void siuCommitState (siuInst_t *inst,  mgb_t *message)
{
  tint      chnum = (tint)(inst->ID & SIU_CID_MASK);

  /* Check for Wrong Commit Message. Commit Should not arrive twice */
  if(inst->state == inst->pending_state) {
    siuErrorIndication (inst->ID, message->header.trans_id,
                      MGB_ERR_INVD_MSG_FUNC_NOT_SUPPORTED);
    return;
  }

  switch(inst->pending_state) {
  case SIU_CHST_IDLE:
    /* The only way to get to this case is by receiving a open message.
     * The idle mode message transitions directly to idle mode without
     * waiting in pending_state IDLE
     */
    if (inst->state == SIU_CHST_CLOSED) {
       siuCommitStateOpen (inst, message);
    } else {
       inst->state = SIU_CHST_IDLE;
    }
    break;

  case SIU_CHST_VIDEO:
    if(chnum==2) {
      siuCodecCfg_t siuCodecCfg;
      vppuCtrl_t vppuCtrl;

      /* Open VPPU if present. MMCU is opened with the MMCU CFG message */
      if(inst->stateInst.video.vppuInst) {
        siuCodecCfg.maxImgWidth  = 1920;
        siuCodecCfg.maxImgHeight = 1088;
        siuCodecCfg.maxFrameRate = 60;
        siuCodecCfg.maxBitRate   = 20000000;
        siuCodecCfg.inputStreamFormat = 0;
        siuCodecCfg.codingType   = EVCT_VIDEO_H264BP_ENC;

        siu_open_vppu (inst, &siuCodecCfg);

        /* Set the initial RTP timestamp */
        vppuCtrl.code = VPPU_SET_RTP_BASE_TS;
        vppuCtrl.u.rtpBaseTs = txInitTimeStamp;
        vppuControl(inst->stateInst.video.vppuInst, &vppuCtrl);
      }
    }
    inst->state = SIU_CHST_VIDEO;    /* declare this channel as in VIDEO mode */
    break;

  }/*switch(inst->pending_state)*/
}/*siuCommitState*/

/******************************************************************************
 * FUNCTION PURPOSE: Change encode or set protocol parameters on the fly
 ******************************************************************************
 * DESCRIPTION: Change encode or set protocol parameters on the fly from
 *
 *    void siuEncapConfig (
 *         siuInst_t *inst,    - pointer to siu's instance
 *         mgb_t *msg)         - msg from MIC
 *
 *****************************************************************************/
void siuEncapConfig (siuInst_t *inst, mgb_t *message)
{
  if(message->msg_body.encap_config.valid_params & MGB_ENCAP_CONFIG_VALID_ENCAPSULATION)
  {
    if(inst->state == SIU_CHST_IDLE)
    {
      /* We cannot reconfigure protocol on-the-fly to a different type!. Suppress the
       * error indication if the encapsulation supplied is same as the current type
       */
      if(inst->protocol_type != message->msg_body.encap_config.encapsulation)
      {
          siuErrorIndication (inst->ID, message->header.trans_id, MGB_ERR_MSG_REJECTED_FROM_THIS_STATE);
          return;
      }
    }
    else
    {
      inst->protocol_type = message->msg_body.encap_config.encapsulation;
      if(inst->protocol_type == MGB_PT_NONE) {
        /* Any time we use none, make sure that we get a RCU to implement
         * channel state.  If channel state is cleaned up this shoudl go away */
        inst->protocol_type = MGB_PT_RTP;
      }
    }
  }
} /* siuEncapConfig */

void siuErrorIndication(tuint moduleID, tuint trans_id, tuint error_code)
{
   return;
} /* siuErrorIndication() */

void siuChannelState (siuInst_t *inst, mgb_t *message)
{
   tint      chnum = (tint)(inst->ID & SIU_CID_MASK);
   if(chnum==2) {
       rcuCtrl_t rcuCtrl;
       rcuCtrl.code = rcu_CTRL_PKT_FLOW;
       rcuCtrl.u.pkt_flow = &message->msg_body.channel_state;
       gg_rcu_ctable.rcuControl (inst->net.rcuInst, &rcuCtrl);
   }
   return;
}

void siuPktioIfCfg (mgb_t *message)
{
    bool    retStatus = TRUE;

    if(message->msg_body.pktio_config.pkt_if_type == MGB_PKTIO_IF_GMAC){
#ifdef NDU_GMAC
    retStatus = siu_pktio_macport_config(&message->msg_body.pktio_config.params.mac);
#endif
    }

    if(!retStatus){
        siuErrorIndication (siuContext.ID,
                            message->header.trans_id,
                            MGB_ERR_INVD_PKTIO_IF_CONFIG);
    }
}

void siuChannelException(tuint moduleID, tuint error_code)
{
  return;
}

void siuAlarm (tuint module_id)
{
  return;
}

void siuDebugReport (tuint id, tuint *str, bool queue)
{
  return;
}

void siuNeuViolReport ( tuint ID, void *vdata )
{
  return;
}

NDU_DPRMsgType_t siuMessageLevel (tuint msgId)
{
  return (NDU_DPR_NORMAL_MSG);
}

bool siuProcMsgAck(NDU_msgPriority_e priority, mgb_t *message)
{
  return FALSE;
}

tuint siu_translate_dynamic_chnum (tuint msg_id, tuint chan)
{
  return(chan);
}

void siuHaltCheck()
{
    return;
}

tulong    siu_isr_clock;      /* incremented in every PCM interrupt */
tulong    siu_timer_tick;     /* system timer tick (1ms) */

tulong siuTmrGetTime (void)
{
  return (siu_timer_tick);
} /* siuTmrGetTime */

tulong siuTmrCalcDTime (const tulong new_time, const tulong old_time)
{
  return (new_time - old_time);   /* this should always work! */
} /* siuTmrCalcDTime */

void siuTmrReset (void)
{
  siu_timer_tick = 0;
} /* siuTmrReset */

static inline void siuTmrTick (void)
{
  siu_timer_tick++;
} /* siuTmrTick */

void siuSubRateFcn(void *cxt, tuint ts)
{
    siu_isr_clock++;
    siuTmrTick();
    return;
}

tuint siu_dbg_id;         /* debug ID used in debug and exception functions */
tuint siu_dbg_message[MGB_MSG_MAXNPAR*MGB_MSGPAR_SIZE];    /* debug message */
siuContext_t  siuContext;                 /* SIU Global Context */
utlRtTraceContext_t utlRtTraceContext;    /* Realtime trace context */

/*
 * The SIU context environment used by setjmp() and longjmp()
 * Note: the critical section in siuMain() will prevent any task
 *       from ever pre-empting any messages processing within BIOS.
 *       This allows us to share one setjmp buffer.
 * Note: message processing should store siu_env on the stack,
 *       and restore it when message processing is done.
 */
jmp_buf  siu_env;

/******************************************************************************
 * FUNCTION PURPOSE: Exception function
 ******************************************************************************
 * DESCRIPTION: Relays a fatal exception through the global exception
 *              pointer.
 *****************************************************************************/
void siu_exception(tuint ID, tuint code)
{
   siuDebugInfo(ID,
                SIU_IS_CHAN_EXCEP(code)? SIU_DEBUG_CHAN_EXCEP:SIU_DEBUG_FATAL,
                SIU_GET_EXCEPTION_CODE(code),
                0,
                NULL);
}

/******************************************************************************
 * FUNCTION PURPOSE: SIU Debug & exception functions.
 ******************************************************************************
 * DESCRIPTION: These functions put the id and debug information in global
 *              memory (i.e., can be read by an emulator) and send appropriate
 *              messages to MIC. They may also pack character strings into
 *              dprword's if necessary.
 *
 *****************************************************************************/


void siuDebugInternal (uint id, char *str, bool queue)
{
  int i;
  char *src;
  unsigned char c1, c2;
  tuint *dst;

  /* Copy id and message into a global buffer */
  siu_dbg_id = id;
  src = str; dst = (tuint *)&siu_dbg_message[0];
  for (i = 0; i < MGB_MSG_MAXNPAR*MGB_MSGPAR_SIZE; i++) {
#if (defined(BIGEND))
  /* BIG endian exception report */
    c1 = (unsigned char)(*src++);         /* no need to mask high bits */
    c2 = (unsigned char)(*src++ & 0xFFu); /*   since we shift them up */
#else
  /* LITTLE endian exception report */
    c2 = (unsigned char)(*src++);         /* no need to mask high bits */
    c1 = (unsigned char)(*src++ & 0xFFu); /*   since we shift them up */
#endif
    *dst++ = (tuint)(c1 << 8 | c2);        /*   when packing into words */
  }

  /* send debug message to MIC */
  siuDebugReport (id, &siu_dbg_message[0], queue);
} /* siuDebugInternal */


void siuException (uint id, char *s)
{
  halControl_t halControl;
  thwDisableAllInt ();  /* disable all interrupts */
  /* send debug message to DPR directly, bypassing the message fifo */
  siuDebugInternal (id, s, FALSE);

  /* Send alarm indication message to DPR */
  siuAlarm (id);

  /* Flush the message through HAL */
  halControl.cmd = HAL_COMMAND_HALT;
  halControl.u.periphReset.portId =  NULL;

  (*siuContext.halApi->halControl) (&halControl);

  siu_dsp_halt(id, s);     /* loop forever */

} /* siuException */

/******************************************************************************
 * FUNCTION PURPOSE: SIU Channel exception function.
 ******************************************************************************
 * DESCRIPTION: The function performs the following actions:
 *              - Set Channel State to EXCEPTION
 *              - Prepare and Send the "MGB_DM_CHANNEL_EXCEPTION" message to the host
 *              - Invoke longjmp() to resume the execution
 *
 *****************************************************************************/
void siuProcChanExcep (tuint moduleID, tuint error_code)
{

  siuInst_t *inst = (siuInst_t *)SIUINSTp((moduleID & SIU_CID_MASK) - SIU_FIRSTCH);

  /* Restore interrupt */
  thwCriticalReset();

  SIU_STATE_BEGIN();

  /*************  Warning ************
   * Note: Use pending_state instead for HDGW 11.0 and above
   */
    inst->state = SIU_CHST_EXCEPTION;    /* declare this channel as in EXCEPTION mode */

  /* Disable all tasks for this channel */
  SIU_STATE_END();

  /* Send channel exception message to DPR */
  siuChannelException(moduleID, error_code);

  /* Resume task or message execution */
  longjmp(siu_env, 1);

} /* siuProcChanExcep */


/* nothing after this point */
