/******************************************************************************
 * FILE PURPOSE: Contains functions necessary to integrate RTP protocol
 ******************************************************************************
 * FILE NAME: siurtp.c
 *
 * DESCRIPTION: Contains functions necessary to integrate RTP protocol into
 *              system, such that it can be call tabled out of the system
 *
 * (C) Copyright 2001, Texas Instruments Incorporated.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#ifdef RTP /* Allow #ifdef in siuloc.h w/o error */

/* System utility files */
#include "types.h"
#include "mgmicdsp.h"
#include "ggcfg.h"
#include "vctypes.h"

#include "globstats.h"

#include "ggCodecTestCfg.h"

/* SIU files */
#include "siuloc.h"
#include "siurtcp.h"

/* Module files */
#include <ti/mas/rcu/rcu.h>
#include "siundu.h"

/******************************************************************************
 * FUNCTION PURPOSE: Create RTP Common Unit (RCU) module
 ******************************************************************************
 * DESCRIPTION: Creates an instance of RCU. Tables of locations and sizes for
 *              RCU memory buffers with special requirements are utilzed.
 *
 *  siu_new_rcu (
 *    tint  chnum)  - channel number
 *
 *****************************************************************************/
void siu_new_rcu (tint chnum)
{
  tint         stat;
  memBuffer_t  rcuBufs[SIU_MAX_NBUFS];
  rcuSizeCfg_t rcuCfgSize;
  rcuNewCfg_t  rcuCfgNew;
  siuInst_t    *inst;

  /* initialize the siu instance */
  inst = &siuInst[chnum-SIU_FIRSTCH];

  /* configure the instance */
  rcuCfgSize.rtcpCallTable = rcuRtcpCallTable;

  rcuCfgNew.ID          = siuMakeID (SIU_MID_RCU, chnum);
  rcuCfgNew.sizeCfg     = rcuCfgSize;

  stat = siuNewEco (inst, (siuEcoGetSizes_t)gg_rcu_ctable.rcuGetSizes, &rcuCfgSize,
                    (siuEcoNew_t)gg_rcu_ctable.rcuNew, &rcuCfgNew,
                    &(inst->net.rcuInst),  rcuBufs, SIU_MID_RCU);

  siu_exc_assert (stat == SIU_NOERR, SIURTP_EXC_NEW_RCU, inst);
} /* siu_new_rcu */

/******************************************************************************
 * FUNCTION PURPOSE:  Open RCU module
 ******************************************************************************
 * DESCRIPTION: Opens one instance of a RCU module
 *
 * siu_open_rcu (
 *   tint *chnum)         - instance pointer
 *
 *****************************************************************************/
void siu_open_rcu (tint chnum)
{
  rcuCfg_t rcuCfg;
  siuInst_t *inst;

  /* initialize the siu instance */
  inst = &siuInst[chnum-SIU_FIRSTCH];
  /* default configuration of RCU is open receive out */
  rcuCfg.rcuRxOut.rcuReceiveOut.rcuReceiveOut = NULL;
  rcuCfg.rcuRxOut.rcuReceiveOut.targetInst    = NULL;
  /*
   * newApi is set to TRUE so that P2P packets such as CAS Delay can be handled
   * at Idle state
   */
  rcuCfg.rcuRxOut.newApi                      = TRUE;

  if (gg_neu_ctable.neuSendIn) {
    rcuCfg.rcuSendOut.rcuSendOut           = gg_neu_ctable.neuSendIn;
    rcuCfg.rcuSendOut.targetInst           = inst->neuInst;
  }
  else {
    rcuCfg.rcuSendOut.rcuSendOut           = siuNduSendPkt;
    rcuCfg.rcuSendOut.targetInst           = inst;
  }

  /* RTCP related definitions */
  rcuCfg.rtcpCallTable            = rcuRtcpCallTable;
//  rcuCfg.rtcpReportOut.fcn        = gg_siu_ctable.Rtcp.status_rpt;
//  rcuCfg.rtcpReportOut.targetInst = inst;

  rcuCfg.sysInst                  = inst;
  rcuCfg.rcuCasOutInst            = NULL;

  /* Pass msuInst pointer to the rcu config */
  rcuCfg.rcuMsuRtpInst  = NULL;
  rcuCfg.rcuMsuRtcpInst = NULL;


  /* Start off with RCU control of timestamp */
  rcuCfg.rcuTxParms.tsIncrPerTick = 5*8;

  rcuCfg.rcuTxParmsF.seqn    = 0;

  rcuCfg.stats = NULL;

  /* open rcu channel */
  if ((*gg_rcu_ctable.rcuOpen) (inst->net.rcuInst,&rcuCfg) != rcu_NOERR)
     siu_exc_assert (0, SIURTP_EXC_OPEN_RCU, inst);
} /* siu_open_rcu */

/******************************************************************************
 * FUNCTION PURPOSE: close an instance of RCU module
 ******************************************************************************
 * DESCRIPTION:
 *
 *****************************************************************************/
void siu_close_rcu (siuInst_t *inst)
{
  (*gg_rcu_ctable.rcuClose) (inst->net.rcuInst);
} /* siu_close_rcu */


/******************************************************************************
 * FUNCTION PURPOSE: Deletes a RCU channel
 ******************************************************************************
 * DESCRIPTION: This routine deletes all modules for RCU operation.
 *
 *
 *****************************************************************************/
void siu_delete_rcu (siuInst_t *inst)
{
  (*gg_rcu_ctable.rcuDelete) (&inst->net.rcuInst);
  inst->protocol_type = MGB_PT_NONE;
} /* siu_delete_rcu */

/******************************************************************************
 * FUNCTION PURPOSE: Inquire the RCU instance from the host
 ******************************************************************************
 * DESCRIPTION: Inquire the RCU instance from the host based on the channel ID
 *
 *  void* siuGetRcuInst (
 *    tuint chan_num)         - channel number
 *
 * Return NULL if channel is not at an valid state (such as VOICE)
 *        neuInst for the specific channel
 *
 *****************************************************************************/
void* siuGetRcuInst (tuint instance_ID)
{
  siuInst_t *inst = SIUINSTp(((instance_ID & SIU_CID_MASK) - SIU_FIRSTCH));
  return(inst->net.rcuInst);

} /* siuGetNetEncapInst */
#else
int siurtp_dummy; /* Shut off the empty file warning */
#endif /* #ifdef RTP */

/* Nothing past this point */
