/******************************************************************************
* FILE PURPOSE: Provide TFTP services
******************************************************************************
* FILE NAME:   siuVctTftp.c
*
* DESCRIPTION: Contains routines that support TFTP operation
*
* TABS: NONE
*
*
* (C) Copyright 2010, Texas Instruments Incorporated
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include <ti/mas/tftp/tftp.h>
#include "siutftp.h"
#include "common/siuloc.h"
#include "common/dsiunwal.h"
#include "ggcfg.h"
#include <stdio.h>
#include "siuVctNeu.h"
#include "siuVctTftp.h"
#include "siuVctParse.h"

#include "siuVctFileIO.h"
#include "targethw.h"

/* Config file parameters */
extern siuVctParams_t siuVctParams;

/* Input and Output buffer pointers */
siuVctIOmemDef_t *readPtr   = &(fileIO.read);
siuVctIOmemDef_t *writePtr  = &(fileIO.write);


/*  Simulate GMP for TFTP. This structures manages one chunk of memory requred
 *  for TFTP put.
 */
typedef struct {
  tint  inUse;
  tuint ID;
  tword buffer[SIU_DEFAULT_TX_MAX_PAYLOAD_SIZE];
} siuVctGmpSim_t;

#pragma DATA_SECTION(siuVctTftpPutBuffer, ".siuVctGmpSim");
siuVctGmpSim_t siuVctTftpPutBuffer = {0,0};


/******************************************************************************
 * FUNCTION PURPOSE: Return size of memory that can be allocated.
 ******************************************************************************
 * DESCRIPTION: Simulate GMP for TFTP. Return size of memory that can be
 *              allocated.
 *
 * void siuVctTftpGmpSize(
 *    tuint   ID,           - Instance ID
 *    void    *handle       - GMP handle
 * )
 *****************************************************************************/
tlong siuVctTftpGmpSize(tuint ID, void *handle)
{
  return(SIU_DEFAULT_TX_MAX_PAYLOAD_SIZE);
}

/******************************************************************************
 * FUNCTION PURPOSE: Returns a chunk of memory.
 ******************************************************************************
 * DESCRIPTION: Simulate GMP for TFTP. Returns a chunk of memory.
 *
 * void *siuVctTftpGmpAlloc(
 *    tuint   ID,           - Instance ID
 *    void    *handle       - GMP handle
 * )
 *****************************************************************************/
void *siuVctTftpGmpAlloc(tuint ID, void *handle)
{
  if(siuVctTftpPutBuffer.inUse) {
    siu_exception(ID, SIUTFTP_EXC_ALLOC_TFTP);
    return(NULL);
  }
  siuVctTftpPutBuffer.ID = ID;
  siuVctTftpPutBuffer.inUse = 1;
  return((void *)&siuVctTftpPutBuffer.buffer[0]);
}

/******************************************************************************
 * FUNCTION PURPOSE: Free a chunk of memory.
 ******************************************************************************
 * DESCRIPTION: Simulate GMP for TFTP. Free a chunk of memory.
 *
 * void siuVctTftpGmpSize(
 *    tuint   ID,           - Instance ID
 *    void    *handle,      - GMP handle
 *    void    *block        - Base address of chunk to be freed.
 * )
 *****************************************************************************/
tbool siuVctTftpGmpFree(tuint ID, void *handle, void *block)
{
  if( siuVctTftpPutBuffer.ID != ID      ||
      &siuVctTftpPutBuffer.buffer[0] != block ||
      siuVctTftpPutBuffer.inUse == 0
    )
  {
    siu_exception(ID, SIUTFTP_EXC_FREE_TFTP);
    return 0;
  }
  siuVctTftpPutBuffer.inUse  = 0;
  siuVctTftpPutBuffer.ID     = 0;

  return(1);
}


/******************************************************************************
* FUNCTION PURPOSE: TFTP Receive Out callback.
******************************************************************************
* DESCRIPTION: TFTP calls this function to store the file chunk it received.
*
* void siuVctTftpGetReceiveIn(
*         void          *inst,
*         xferpktInfo_t *pktInfo
* )
*****************************************************************************/
tint siuVctTftpGetReceiveIn(void *inst, xferpktInfo_t *pktInfo)
{
  tlong   availableSize;
  tint  i;
  tint  retval = 0;
  tint  MTUsize = siuInst[0].maxPayloadSize; /* hardcoded in siu_open_tftp() */

  /* Align valid data to base of the buffer */
  siu_invalidate_cache();
  if(readPtr->ptr_start != readPtr->ptr_working) {
    memcpy( readPtr->ptr_start, readPtr->ptr_working, readPtr->numValidBytes);
    readPtr->ptr_working = readPtr->ptr_start;
  }

  availableSize = (readPtr->memorySizeLimit - readPtr->numValidBytes);

  /* Saved file chunks received to the end of buffer */
  for(i=0; i<pktInfo->npkts; i++) {
    if( availableSize < pktInfo->pktSize[i]) {
      printf("\nSomething went wrong. No room for tftp chunk... \n");
      return 0;
    }

    memcpy(&readPtr->ptr_working[readPtr->numValidBytes], pktInfo->pktIn[i], pktInfo->pktSize[i]);
    readPtr->numValidBytes += pktInfo->pktSize[i];
    availableSize          -= pktInfo->pktSize[i];
  }

  /* Send an ACK if there is size for another file chunk */
  if( availableSize > MTUsize) {
    tftpGetData(siuInst[0].net.tftpInst);
  }

  return(retval);
}


/******************************************************************************
* FUNCTION PURPOSE: TFTP Receive Out callback.
******************************************************************************
* DESCRIPTION: TFTP calls this function to get the next file chunk to send to
*               the server.
*
* void siuVctTftpPutReceiveIn(
*         void          *inst,
*         xferpktInfo_t *pktInfo
* )
*****************************************************************************/
tint siuVctTftpPutReceiveIn(void *inst, xferpktInfo_t *pktInfo)
{

  tint    bytesConsumed;

  bytesConsumed = tftpPutData(siuInst[1].net.tftpInst, (tword *)writePtr->ptr_start, writePtr->numValidBytes);

  if(bytesConsumed > 0) {
    thwCriticalBegin();
    writePtr->numValidBytes   -= bytesConsumed;
    memcpy(&writePtr->ptr_start[0], &writePtr->ptr_start[bytesConsumed], writePtr->numValidBytes);
    writePtr->ptr_working     -= bytesConsumed;
    thwCriticalEnd();
  }

  return 0;
}

/******************************************************************************
 * FUNCTION PURPOSE: Start TFTP
 ******************************************************************************
 * DESCRIPTION: Start TFTP.
 *
 * void siuVctStartTftp(
 *         tuint req
 * )
*****************************************************************************/
void siuVctStartTftp(tftpType_e req)
{
    tftpCtrl_t ctrl;
    void  *tftpInst;

    switch(req)
    {
      case TFTP_RRQ:
        tftpInst = siuInst[0].net.tftpInst;

        /* Reset TFTP stats before starting test. */
        ctrl.code           = TFTP_CTRL_GET_STATS;
        ctrl.u.resetStats   = 1;
        tftpControl(tftpInst,&ctrl);

        ctrl.code = TFTP_CTRL_RRQ;

        ctrl.u.req.MTUsize  = siuInst[0].maxPayloadSize;
        ctrl.u.req.filename = siuVctParams.tftpGetFilename;

        ctrl.u.req.receiveOut.tftpReceiveOut  = siuVctTftpGetReceiveIn;
        ctrl.u.req.receiveOut.targetInst      = (void *)&siuInst[0];

        printf("\nStarting TFTP Get for %s... ",ctrl.u.req.filename);
        break;

      case TFTP_WRQ:
        tftpInst = siuInst[1].net.tftpInst;

        /* Reset TFTP stats before starting test. */
        ctrl.code           = TFTP_CTRL_GET_STATS;
        ctrl.u.resetStats   = 1;
        tftpControl(tftpInst,&ctrl);

        ctrl.code = TFTP_CTRL_WRQ;

        ctrl.u.req.MTUsize  = siuInst[1].maxPayloadSize;
        ctrl.u.req.filename = siuVctParams.tftpPuFilename;

        ctrl.u.req.receiveOut.tftpReceiveOut  = NULL;
        ctrl.u.req.receiveOut.targetInst      = (void *)&siuInst[1];

        printf("\nStarting TFTP Put for %s... ",ctrl.u.req.filename);
        break;
    }

    ctrl.u.req.timeout  = 10000;

    tftpControl(tftpInst, &ctrl);
    tftpStart (tftpInst);


    do {
      ctrl.code = TFTP_CTRL_GET_STATUS;
      tftpControl(tftpInst, &ctrl);

      if(ctrl.u.xferStatus == TFTP_FILE_XFER_COMPLETE) {
        printf("Error receive from server!\n");
        if(req == TFTP_WRQ)
          abort();

        return;
      }

      ctrl.code = TFTP_CTRL_GET_STATS;
      ctrl.u.resetStats = FALSE;

      tftpControl(tftpInst, &ctrl);
    } while(ctrl.u.stats.count == 0);

    printf("Server ACK received.\n");
}

/******************************************************************************
* FUNCTION PURPOSE: Configure NWAL after NEU reconfiguration for TFTP
******************************************************************************
* DESCRIPTION: siuVctConfigureNwal
*
* void siuVctConfigureNwal(
*   siuInst_t  *inst             - instance pointer
* )
*****************************************************************************/
void siuVctConfigureNwal(siuInst_t *inst)
{
    void *neuInst   = (void *)inst->neuInst;
    neuCtrl_t  neuCtrl;

    /* Get the current NEU params */
    neuCtrl.code = neu_CTRL_GET_PARAMS;
    neuControl(neuInst, &neuCtrl);

    /* NWAL configuration */
    if(!(dsiuConfigL2L3L4(neuInst,
                          &neuCtrl.u.parm.chnl,
                          &inst->udpHandle,
                          NULL,
                          NULL)))
    {
      siuErrorIndication (inst->ID, 0, MGB_ERR_NEU_BAD_CONFIG);
    }
}


/******************************************************************************
* FUNCTION PURPOSE: Begin TFTP connection to receive file from server.
******************************************************************************
* DESCRIPTION: siuVctStartTftpGet.
*
* tulong siuVctTftpGet(
*         void
* )
*****************************************************************************/
void siuVctStartTftpGet(void)
{
    void *neuInst   = (void *)siuInst[0].neuInst;

    siuVctConfigureTftpUdp(neuInst);

    siuVctConfigureNwal(&siuInst[0]);

    siuVctStartTftp(TFTP_RRQ);

    siu_invalidate_cache();

    return;
}

/******************************************************************************
* FUNCTION PURPOSE: Begin TFTP connection to send file to server.
******************************************************************************
* DESCRIPTION: siuVctStartTftpPut.
*
* void siuVctTftpPut(
*         tulong nbytes
* )
*****************************************************************************/
void siuVctStartTftpPut(void)
{
    void *neuInst   = (void *)siuInst[1].neuInst;

    siu_invalidate_cache();

    /* Codec output is ready to be shipped out. configure NEU to begin TFTP */
    siuVctConfigureTftpUdp(neuInst);

    siuVctConfigureNwal(&siuInst[1]);

    siuVctStartTftp(TFTP_WRQ);

    return;
}

/******************************************************************************
 * FUNCTION PURPOSE: Get file data via TFTP.
 ******************************************************************************
 * DESCRIPTION: This function is called to check if there is enough data in
 *              the buffer. If not, a request is sent to TFTP and then this
 *              function waits for the buffer to fill to the threshold.
 *
 * void siuVctTftpGetData(void)
 *
 *****************************************************************************/
void siuVctTftpGetData(void)
{
  void        *tftpInst = siuInst[0].net.tftpInst;
  tftpCtrl_t  tftpCtrl;

  if(readPtr->numValidBytes < readPtr->tftpThresh) {
    tftpGetData(tftpInst);

    while(readPtr->numValidBytes < readPtr->tftpThresh) {
      tftpCtrl.code = TFTP_CTRL_GET_STATUS;
      tftpControl(tftpInst,&tftpCtrl);

      if(tftpCtrl.u.xferStatus != TFTP_FILE_XFER_WORKING)
        break;
    }
  }
}

/******************************************************************************
 * FUNCTION PURPOSE: Save file data via TFTP.
 ******************************************************************************
 * DESCRIPTION: This function is called to send file data to a TFTP server.
 *              This function will wait until TFTP is sending the data.
 *
 * void siuVctTftpPutData(void)
 *
 *****************************************************************************/
void siuVctTftpPutData(tword *buf, tlong bufSize)
{
  tftpCtrl_t ctrl;
  ctrl.code = TFTP_CTRL_GET_STATUS;
  tftpControl(siuInst[1].net.tftpInst, &ctrl);

  if(ctrl.u.xferStatus == TFTP_FILE_XFER_COMPLETE)
    siuVctStartTftpPut();

  siu_put_tftp(&siuInst[1], buf, bufSize);
}


void siuVctPrintTftpStats(void *tftpInst)
{
  tftpCtrl_t  tftpCtrl;

  tftpCtrl.code           = TFTP_CTRL_GET_STATS;
  tftpCtrl.u.resetStats   = 0;
  tftpControl(tftpInst,&tftpCtrl);

  printf("\nTFTP Statistics-------------------+");
  printf("\n|  RX Bytes          : %10d |", tftpCtrl.u.stats.rxBytes);
  printf("\n|  TX Bytes          : %10d |", tftpCtrl.u.stats.txBytes);
  printf("\n|  Valid Packets     : %10d |", tftpCtrl.u.stats.count);
  printf("\n|  Duplicate Packets : %10d |", tftpCtrl.u.stats.dupPkts);
  printf("\n|  Error Packets     : %10d |", tftpCtrl.u.stats.errPkts);
  printf("\n+---------------------------------+");
  printf("\n");
}

/******************************************************************************
 * FUNCTION PURPOSE: Terminate file transfer
 ******************************************************************************
 * DESCRIPTION: This function is called to stop receiving file data from a
 *              TFTP server.
 *
 * void siuVctStopTftpGet(void)
 *
 *****************************************************************************/
void siuVctStopTftpGet(void)
{
  siu_stop_tftp(&siuInst[0]);

  printf("\nTFTP Get stopped. ");
  siuVctPrintTftpStats(siuInst[0].net.tftpInst);
}

/******************************************************************************
 * FUNCTION PURPOSE: Terminate file transfer
 ******************************************************************************
 * DESCRIPTION: This function is called to finish sending a file. This
 *              function waits until the entire buffer has been sent.
 *
 * void siuVctStopTftpGet(void)
 *
 *****************************************************************************/
void siuVctStopTftpPut(void)
{
  siu_stop_tftp(&siuInst[1]);

  printf("\nTFTP Put stopped. ");
  siuVctPrintTftpStats(siuInst[1].net.tftpInst);
  return;
}

/* nothing past this point */
