/******************************************************************************
 * FILE PURPOSE: Contains functions necessary to integrate VPPU
 ******************************************************************************
 * FILE NAME: siuVctVppu.c
 *
 * DESCRIPTION: Contains functions necessary to integrate VPPU into
 *              system, such that it can be call tabled out of the system
 *
 * (C) Copyright 2008, Texas Instruments Incorporated.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

/* System utility files */
#include "types.h"
#include "mgmicdsp.h"
#include "ggcfg.h"
/* SIU files */
#include "common/siuloc.h"
#include "siuVctVppu.h"
#include "vctypes.h"

/* Module files */
#include <ti/mas/vppu/vppu.h>
#include "util.h"
#include "ggmbuf.h"
#include "utlrepck_m.h"

#pragma DATA_SECTION (codecRfcMap, ".codecRfcMap");
codecRfcMapping_t codecRfcMap[] = {
    { EVCT_VIDEO_MPEG4SP_DEC, "RFC3016"},
    { EVCT_VIDEO_MPEG4SP_ENC, "RFC3016"},
    { EVCT_VIDEO_H263P_DEC,   "RFC2429"},
    { EVCT_VIDEO_H263P_ENC,   "RFC2429"},
    { EVCT_VIDEO_H264BP_MP_DEC,  "RFC3984"},
    { EVCT_VIDEO_H264HP_DEC,  "RFC3984"},
    { EVCT_VIDEO_H264BP_ENC,  "RFC3984"},
    { EVCT_VIDEO_MPEG2MP_DEC, "RFC2250"},
    { EVCT_VIDEO_BYPASS_DEC,  "RFC4175"},
    { EVCT_VIDEO_BYPASS_ENC,  "RFC4175"},
    { EVCT_VIDEO_OVLY_TEXT,   "RFC4396"},
    { -1                  ,   "NULL"   }
};

/******************************************************************************
 * FUNCTION PURPOSE: Return the RFC name given coding type
 ******************************************************************************
 * DESCRIPTION: Return the RFC name given coding type
 *
 *  siuVppuGetRfcName (
 *    tuint  codingType    - coding Type
 *    )
 *****************************************************************************/
char* siuVppuGetRfcName (tuint codingType) {
    codecRfcMapping_t *mapPtr = codecRfcMap;
    while ((mapPtr->codingType != codingType) && (mapPtr->codingType != -1)){
       mapPtr++;
    }
    return((char *)mapPtr->rfcName);
}

/******************************************************************************
 * FUNCTION PURPOSE: Control Video Protocol Processing Unit (VPPU) module
 ******************************************************************************
 * DESCRIPTION:
 *
 *  siu_vppu_control (
 *    tint  chnum,    - channel number
 *    mgb_t *message) - message from host
 *****************************************************************************/
void siu_vppu_control (tint chnum, mgb_t *message) {
  siuInst_t    *inst;
  vppuCtrl_t    ctrl;

  /* initialize the siu instance */
  inst = &siuInst[chnum-SIU_FIRSTCH];

  vppuControl(inst->stateInst.video.vppuInst, &ctrl);
  return;
}

/******************************************************************************
 * FUNCTION PURPOSE: Create Video Protocol Processing Unit (VPPU) module
 ******************************************************************************
 * DESCRIPTION:
 *
 *  siu_new_vppu (
 *    tint  chnum)  - channel number
 *
 *****************************************************************************/
void siu_new_vppu (tint chnum)
{
  tint         stat;
  memBuffer_t  vppuBufs[SIU_MAX_NBUFS];
  vppuNewCfg_t vppuCfgNew;
  vppuSizeCfg_t vppuCfgSize;
  siuInst_t    *inst;

  /* initialize the siu instance */
  inst = &siuInst[chnum-SIU_FIRSTCH];

  /* configure the instance */
  vppuCfgNew.ID             = siuMakeID (SIU_MID_VPPU, chnum);
  vppuCfgSize.txScratchSize = 1600; /* Max MTU Size */

  vppuCfgNew.sizeCfg        = vppuCfgSize;
  stat = siuNewEco (inst, (siuEcoGetSizes_t)vppuGetSizes, &vppuCfgSize,
                    (siuEcoNew_t)vppuNew, &vppuCfgNew,
                    &(inst->stateInst.video.vppuInst),  vppuBufs, SIU_MID_VPPU);

  siu_exc_assert (stat == SIU_NOERR, SIUVPPU_EXC_NEW_VPPU, inst);
} /* siu_new_vppu */

/******************************************************************************
 * FUNCTION PURPOSE:  Open VPPU module
 ******************************************************************************
 * DESCRIPTION: Opens one instance of a VPPU module
 *
 * siu_open_vppu (
 *   tint *chnum                 - instance pointer
 *   siuCodecCfg_t *codecCfg)    - configuration for the codec
 *****************************************************************************/
void siu_open_vppu (siuInst_t *inst, siuCodecCfg_t *codecCfg)
{
  vppuCfg_t vppuCfg;

  /* memset vppu Config to zero */
  memset(&vppuCfg,0,sizeof(vppuCfg_t));

  /* set up the VideoOut targetInst to post task from ISR */
  vppuCfg.vppuRxVideoOut.targetInst      = inst;

  /* default configuration of VPPU is open receive out */
  vppuCfg.vppuSendOut.vppuSendOut = gg_rcu_ctable.rcuSendIn;
  vppuCfg.vppuSendOut.targetInst = inst->net.rcuInst;
  vppuCfg.imgWidth   = codecCfg->maxImgWidth;
  vppuCfg.imgHeight  = codecCfg->maxImgHeight;
  vppuCfg.frameRate  = codecCfg->maxFrameRate;
  vppuCfg.maxBitRate = codecCfg->maxBitRate;
  vppuCfg.inputStreamFormat = codecCfg->inputStreamFormat;

  vppuCfg.rfcName = siuVppuGetRfcName(codecCfg->codingType);
  siu_exc_assert ((strcmp(vppuCfg.rfcName, "NULL")!=0), SIUVPPU_EXC_OPEN_VPPU, inst);

  vppuCfg.type = VPPU_TYPE_TX;

  vppuCfg.payloadType = inst->payloadType;
  vppuCfg.MTUsize     = inst->maxPayloadSize;

  /* open vppu channel */
  if (vppuOpen (inst->stateInst.video.vppuInst,&vppuCfg) != VPPU_NOERR)
     siu_exc_assert (0, SIUVPPU_EXC_OPEN_VPPU, inst);

} /* siu_open_vppu */


/******************************************************************************
 * FUNCTION PURPOSE: Send output data to VPPU
 ******************************************************************************
 * DESCRIPTION: This function is called to send output data to VPPU.
 *
 * void siuVctVppuPutData(void)
 *
 *****************************************************************************/
volatile tuint frameRate = 15; /* Adjust this according to application requirement */
void siuVctVppuPutData(tword *buf, tlong bufSize)
{
  tulong frameTimeStampIncrement;
  ifvisys_vidEncOutput_t  vidEncOutput;

  /* Initialize encoder input/output structure to zero */
  memset(&vidEncOutput, 0, sizeof(ifvisys_vidEncOutput_t));

  /* Set base address and size of the output data */
  vidEncOutput.bitStream.ptr = buf;
  vidEncOutput.bitStream.size = bufSize;

  /* Set timestamp increment for frames */
  frameTimeStampIncrement = 90000/frameRate;

  /* Call vppuSendIn */
  gg_vppu_ctable.vppuSendIn (siuInst[1].stateInst.video.vppuInst, &vidEncOutput, frameTimeStampIncrement);
}


/* Nothing past this point */
