/******************************************************************************
 * FILE PURPOSE: SIU Module Management Routines for Video.
 ******************************************************************************
 * FILE NAME:   siuvideo.c
 *
 * DESCRIPTION: Contains management routines that create and open various
 *              Video modules in a DSP system architecture to enable 
 *              dynamic switchover of codecs.
 *
 * TABS: NONE
 *
 *
 * (C) Copyright 2008, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include <stdlib.h>
#include <string.h>

#include "types.h"
#include "vctypes.h"
#include "util.h"
#include "utlrepck.h"
#include "mgmicdsp.h"  
#include "targethw.h"
#include "globcfg.h"
#include "utlxferpkt.h"
/* SIU header files */
#include "common/siu.h"
#include "common/siutmr.h"       /* system timer API */
#include "common/siuloc.h"
#include "siuport.h"
#include "common/siundu.h"

#include "ggmbuf.h"    /* Defines memory configuration data structures */
#include "ggVideoCfg.h"      /* Defines Golden Gateway configuration data */

#ifndef HDCT
#include "common/siuctbl.h"  /* SIU internal call table */
#endif
#include "siuvppu.h"
#include "siutftp.h"
#include "siummcu.h"
#include "siuvtk.h"
#include "siuatk.h"
#include "siuCustomEco.h"
#include "osal/siuIpcMsgOsal.h"
#include "osal/siuOsal.h"
#include "siutsu.h"
#include "siuovly.h"
#include <ti/mas/ovly/ovly.h>
#include "siuVigdkGmp.h"
#include "ividmc/siuVidMc.h"

/* Defines for masking protocols. Should be the same as those 
   in makegg file in the mkrel\c64x */
#define GG_H264BP_ENC_MASK      0x0001
#define GG_MPEG4_ENC_MASK       0x0002
#define GG_H263_ENC_MASK        0x0004

#define GG_MPEG2MP_DEC_MASK     0x0008
#define GG_MPEG4_DEC_MASK       0x0010
#define GG_H263_DEC_MASK        0x0020
#define GG_H264BP_MP_DEC_MASK   0x0040
#define GG_H264HP_DEC_MASK      0x0100

extern const tint gg_dummyv;

/**********************************************************************/
/** Intercore communication matrix updated from DIM API   *************/
/* DEC: lower 8 bits is 1-based channel; upper 8 bits is decoder core */
/* ENC: lower 8 bits is 1-based channel; upper 8 bits is encoder core */
#pragma DATA_SECTION (siuInterCoreConn, ".local_intercore_cfg");
siuInterCoreConn_t siuInterCoreConn[MAX_SIU_INTERCORE_CONN] = {
  { 0x0000, 0x0000 },  /* e.g. decoder 0 channel 1-> encoder 1 channel 2 */
  { 0x0000, 0x0000 },  /* e.g. decoder 2 channel 1-> encoder 3 channel 2 */
  { 0x0000, 0x0000 },  /* e.g. decoder 4 channel 1-> encoder 5 channel 2 */
  { 0x0000, 0x0000 },  /* e.g. spare */ 
  { 0x0000, 0x0000 },  /* e.g. spare */ 
  { 0x0000, 0x0000 },  /* e.g. spare */     
  { 0x0000, 0x0000 },  /* e.g. spare */ 
  { 0x0000, 0x0000 }   /* e.g. spare */     
};

/******************************************************************************
 * FUNCTION PURPOSE: Search for state machine
 ******************************************************************************
 * DESCRIPTION: Search profile for state machine via a type and a subtype.
 *              For RTP, the type is payload type, and the subtype is
 *              NSE type.  For AAL2, the type is UUI and the subtype is
 *              type 3 subtype.  Returns the state machine number.
 *
 *              To speed up the frequent case (this voice type matches
 *              last voice type), the inst->lastPacketType is used to
 *              bypass searching if this packet matches last voice packet type.
 *
 * tuint siu_search_video_stm (
 *    void *siuInst,      - siu instance pointer
 *    tuint type,         - PT
 *    tuint payloadlen)   - payload length
 *
 * Return values: 0      voice
 *                1-FFFE STM number
 *                FFFF   not found
 *****************************************************************************/
tuint siu_video_stm_search (void *siuInst, tuint type, bool subtypeValid,
                            tuint subtype, tuint payloadlen)
{
  siuInst_t *inst = (siuInst_t *)siuInst;
  tuint videoIdx, thisPacketType;
  bool video_mode = (inst->state == SIU_CHST_VIDEO);

  /* Voice profile only applies when in voice or clear voice */
  /* Must be RTP -- this function only called from ACU or RCU */
      /* Check if voice w/o unpacking profiles */
  thisPacketType = type;
  if (inst->lastPacketType == thisPacketType) return rcu_RXSTM_MEDIA; /* Quick video */

  videoIdx = siu_rtp_search_video_profile(inst, type); 

  if (rcu_RXSTM_NOMATCH != videoIdx) {
    inst->lastPacketType = thisPacketType;
  }

 /* Failure only applies to voice/clear voice */
  if (video_mode)
    return videoIdx; /* Video Stm */
    
  return rcu_RXSTM_NOMATCH;  /* Failure */
} /* siu_search_stm */


/******************************************************************************
 * FUNCTION PURPOSE: Search voice profiles for payload type
 ******************************************************************************
 * DESCRIPTION: Searches voice profile for a payload type.
 *              Returns the profile index of the match if one, else
 *              if equal to numVPRecs, not found.
 * 
 * tuint siu_rtp_search_video_profile (
 *    siuInst_t *inst,  - siu instance pointer
 *    tuint     pt)     - rtp payload type
 *
 *****************************************************************************/
tuint siu_rtp_search_video_profile (siuInst_t *inst, tuint pt)
{
  tuint i = rcu_RXSTM_NOMATCH;
  
  if(pt == inst->payloadType)
     i = rcu_RXSTM_MEDIA;
  
  return i;
} /* siu_rtp_search_profile */



/******************************************************************************
 * FUNCTION PURPOSE:  Report receive packet violation  
 ******************************************************************************
 * DESCRIPTION: Report receive packet violation.
 *
 *****************************************************************************/
void siuVigdkReportPktViolation (void *targetInst, void *vdata, tuint layer)
{
  mgb_t                send_message;
  siuInst_t            *inst        = (siuInst_t *)targetInst;
  union MGB_DMS_RECEIVE_PACKET_VIOLATION_LAYER_INFO *data = 
     (union MGB_DMS_RECEIVE_PACKET_VIOLATION_LAYER_INFO *)vdata;
  bool allowPktViol = FALSE;

  /* See if packet violation is allowed based on mask -- not pktViolReason
     "lines up" for both aal2 and rtp */
  //if ((inst->heapInst->profile->pktViolationFilter & data->rtp.pktViolReason) == 0)
  //   return;
    
  if (layer == MGB_MDS_LAYER_RTP) {
     allowPktViol = gg_rcu_ctable.pktViolThrottle (inst->net.rcuInst);
     send_message.msg_body.receive_packet_violation.layer = MGB_MDS_LAYER_RTP;
  }
  
  if (!allowPktViol)
     return;

  send_message.header.length   = MGB_HEADER_SIZE + 
                                 (sizeof(struct MGB_DMS_RECEIVE_PACKET_VIOLATION)/sizeof(dprword));
  send_message.header.chnum    = (int)(inst->ID & SIU_CID_MASK);
  send_message.header.msg_id   = MGB_DM_RECEIVE_PKT_VIOLATION;
  send_message.header.trans_id = 0; /* Not a reponse to a MIC message */
 
  send_message.msg_body.receive_packet_violation.layer_info = *data;
  
  /* send message to FIFO */
  siuNduSendMsg(&send_message); 
} /* siuVigdkReportPktViolation */ 

/******************************************************************************
 * FUNCTION PURPOSE: Inquire the connected SIU instance to do routing of YUV frame
 ******************************************************************************
 * DESCRIPTION: Inquire the connected SIU instance to do routing of YUV frame
 *
 *  void* siuGetTxInst (
 *    tuint chan_num         - channel number
 *    tuint domain)          - domain (Packet or YUV)
 *
 * Return SIU for the specific channel  
 *                         
 *****************************************************************************/
siuInst_t* siuGetTxInst (tuint instance_ID, tuint domain) {
  siuInst_t *srcInst, *dstInst;
  tuint dstChID = 0; 

  srcInst = (siuInst_t *)SIUINSTp((instance_ID & SIU_CID_MASK) - SIU_FIRSTCH);
  if (domain == MGB_CONN_YUV_DOMAIN) 
     dstChID = SIU_GET_VIDEO_ROUTE_DST_CH_ID_LD(srcInst);
  else if (domain == MGB_CONN_PACKET_DOMAIN) 
     dstChID = SIU_GET_VIDEO_ROUTE_DST_CH_ID_PD(srcInst);

  if (dstChID == 0) {
     return NULL;
  }
 
  dstInst = (siuInst_t *)SIUINSTp((dstChID & SIU_CID_MASK) - SIU_FIRSTCH);

  /* Return the siuInst only when it is configured to do TX processing */
  if(SIU_READ_TX_ENABLED(dstInst)){
    return (dstInst);
  } else {
    return (NULL);
  }

} /* siuGetTxInst */ 

/******************************************************************************
 * FUNCTION PURPOSE: Creates a full video channel
 ******************************************************************************
 * DESCRIPTION: This routine news all modules for Video operation. Creates
 *              all video modules - allocates memory resources for each module
 *              and initializes then to CLOSED state
 *                         
 *****************************************************************************/
void siuCreateVideoCh (tint chnum)
{
  siuInst_t *inst = SIUINSTp(chnum-SIU_FIRSTCH);
  /* Auxiliary channels share same address space - they are upper half of all channels */
  bool chan_full  = chnum < (SIU_FIRSTCH + ggConfig.max_channels) ? TRUE : FALSE;
  tuint heap_size = chan_full ? ggchVoiceHeapSize : ggchVoiceAuxHeapSize;
  tint slotnum; /* It is 0 based */
  /* Get a slot for this channel */
  slotnum = chnum-SIU_FIRSTCH;

  /* Initialize channel heap for voice */
  inst->chHeapID = mhmCreate ((void *)ggchVoiceHeapStartAddr[slotnum], 
                                 heap_size, 0 /* heap init pattern */);

  /* Create VTK module */
  siu_new_vtk(chnum);
  /* Create NEU module */
  siu_new_neu(chnum);
  /* Create TSU module */
  siu_new_tsu(chnum);
  /* Create custom ECO module */
  siu_new_custom_eco(chnum);
} /* siuCreateVideoCh */

/******************************************************************************
 * FUNCTION PURPOSE: 
 ******************************************************************************
 * DESCRIPTION: 
 *              
 *
 ******************************************************************************/
void siuSwitchOutVideo (siuInst_t *inst)
{
  neuCtrl_t neuConfig;
  tint      chnum = (tint)(inst->ID & SIU_CID_MASK);

  if (SIU_IS_NEU_PRESENT(inst)) {
    neuConfig.code = neu_CTRL_CHANNEL_CFG;
    (*gg_neu_ctable.neuControl)(inst->neuInst, &neuConfig);
  }
  if (SIU_IS_NEU_PRESENT(inst)) {
    (*gg_neu_ctable.neuClose) (inst->neuInst);    
  }
  siu_reset_configuration (inst);
  /*******************************************************/
  vppuClose (inst->stateInst.video.vppuInst);
  vtkClose  (inst->stateInst.video.vtkInst);
  /*******************************************************/
  if (SIU_IS_NEU_PRESENT(inst)) {
     siu_open_neu (chnum, &neuConfig.u.parm);
     neuConfig.code = neu_CTRL_CHANNEL_CFG;
     (*gg_neu_ctable.neuControl)(inst->neuInst, &neuConfig);
  }
  siu_queue_purge_channel (chnum, SIU_READ_TASK_PRIORITY_ID(inst));
}
/******************************************************************************
 * FUNCTION PURPOSE: message state machine function
 ******************************************************************************
 * DESCRIPTION: 
 *  idle a video channel
 *****************************************************************************/
void siu_msm_video_to_idle (siuInst_t *inst, mgb_t *microMessage)
{
   tuint state;

   /* Store the state */
   state = inst->state;

   /* Channel state must be closed during switch-out procedure */
   inst->state = SIU_CHST_CLOSED;

   siuSwitchOutVideo (inst);

   /* Restore the State */
   inst->state = state;

  inst->pending_state = SIU_CHST_IDLE;
} /* siu_msm_video_to_idle() */

static bool siu_is_valid_chn_video (tuint chnum)
{
  return    ( chnum < siuContext.maxNumberChannels )
         && ( SIUINSTp(chnum)->state == SIU_CHST_VIDEO);           
}

/*****************************************************************************
 * FUNCTION PURPOSE: Add or Remove connection between ENC leg & DEC leg to form a 
 *                   transcoding channel
 * return 0        : If Connect Request Message is succesful.
          errorCode: If Connect Request Message resulted in error
          -1       : If Connect Request Message is not for Video MODE
 ****************************************************************************/
tint siuVideoConnReq  (struct MGB_MDS_CONN_ELEM_T *pCmd) 
{
  tuint connType = pCmd->req_type & MGB_CONN_REQ_TYPE_MASK;
  tuint domain   = pCmd->req_type & MGB_CONN_DOMAIN_MASK;
  tuint srcChan, dstChan;
  siuInst_t *srcInst = NULL, *dstInst = NULL;

  srcChan = pCmd->source_chan; 
  dstChan = pCmd->dest_chan;

/*** INTERCORE connections (single chip interconnection assumed) ***/
    if((pCmd->source_dspId != pCmd->dest_dspId))
    {
      tuint i;
      /* Only YUV domain connections are allowed from core to core */
      if(domain != MGB_CONN_YUV_DOMAIN)
          return(-1);
      /* Intercore connection */
      if(pCmd->source_dspId == siuContext.coreId)
      { /* This core is decoder in dual-core transcoding */
        srcInst = SIUINSTp(srcChan); 
        /* If channels are not video - return FALSE indicating connect msg is not consumed */
        if (!siu_is_valid_chn_video (srcChan))
           return (-1);    

        if ((SIU_GET_VIDEO_CHANNEL_TYPE(srcInst) == SIU_CHTYPE_DECODER) || 
            (SIU_GET_VIDEO_CHANNEL_EXT_TYPE(srcInst) == SIU_CHTYPE_EXT_TSU_ONLY))
        { /* Find connection command */
          if (connType == MGB_CONN_REQ_CONN_ADD)
          { 
            tint  connID = -1;
            for (i = 0; i < MAX_SIU_INTERCORE_CONN; i ++) 
            {
              if(SIU_INTER_CORE_CONN_IS_RESET(i))
              {
                connID = i;
                break;
              }
            }
            if (connID == -1) return(MGB_ERR_VIDEO_INVD_CONNECT_REQUEST);
            SIU_SET_INTER_CORE_CONN_DEC(connID, siuContext.coreId, srcChan);
            SIU_SET_INTER_CORE_CONN_ENC(connID, pCmd->dest_dspId, dstChan);
            SIU_SET_VIDEO_ROUTE_DST_CH_ID_LD(srcInst, dstChan + 1);
            /* Open MessageQ */
            siu_ipc_osal_msgq_open_alloc();
          } else if (connType == MBG_CONN_REQ_DISC)
          {
            for (i = 0; i < MAX_SIU_INTERCORE_CONN; i ++) {
              if (SIU_INTER_CORE_CONN_ENC_DEC_MATCH(i, pCmd->dest_dspId, dstChan, siuContext.coreId, srcChan)) {
                  SIU_RESET_INTER_CORE_CONN_ENC(i);
                  SIU_RESET_INTER_CORE_CONN_DEC(i);
              }
            }
            /* Close MessageQ */
            siu_ipc_osal_msgq_close();
            SIU_SET_VIDEO_ROUTE_DST_CH_ID_LD(srcInst, 0);
          } else { /* unsupported conn_req type */	      
            return (MGB_ERR_VIDEO_INVD_CONNECT_REQUEST);
          }
        }
      } else if(pCmd->dest_dspId == siuContext.coreId)
      { /* This core is encoder in dual-core transcoding */ 
        dstInst = SIUINSTp(dstChan);
        /* If channels are not video - return FALSE indicating connect msg is not consumed */
        if (!siu_is_valid_chn_video (dstChan))
           return (-1);    
        if((SIU_GET_VIDEO_CHANNEL_TYPE(dstInst) == SIU_CHTYPE_ENCODER) || 
           (SIU_GET_VIDEO_CHANNEL_EXT_TYPE(dstInst) == SIU_CHTYPE_EXT_TSU_ONLY))
        { /* Find connection command */ 
          if (connType == MGB_CONN_REQ_CONN_ADD)
          {
            tint  connID = -1;
            for (i = 0; i < MAX_SIU_INTERCORE_CONN; i ++) 
            {
              if(SIU_INTER_CORE_CONN_IS_RESET(i))
              {
                connID = i;
                break;
              }
            }
            if (connID == -1) return(MGB_ERR_VIDEO_INVD_CONNECT_REQUEST);
            SIU_SET_INTER_CORE_CONN_DEC(connID, pCmd->source_dspId, srcChan);
            SIU_SET_INTER_CORE_CONN_ENC(connID, siuContext.coreId, dstChan);
            /* Open MessageQ */
            siu_ipc_osal_msgq_open_alloc();
          }	else if (connType == MBG_CONN_REQ_DISC)
          {
            for (i = 0; i < MAX_SIU_INTERCORE_CONN; i ++) {
              if (SIU_INTER_CORE_CONN_ENC_DEC_MATCH(i, siuContext.coreId, dstChan, pCmd->source_dspId, srcChan)) {
                  SIU_RESET_INTER_CORE_CONN_ENC(i);
                  SIU_RESET_INTER_CORE_CONN_DEC(i);
              }
            }
            /* Close MessageQ */
            siu_ipc_osal_msgq_close();
          } else { /* unsupported conn_req type */            
            return (MGB_ERR_VIDEO_INVD_CONNECT_REQUEST);
          }
        }
      }	else return (MGB_ERR_VIDEO_INVD_CONNECT_REQUEST); /* Wrong routing! */
/*** INTERCORE connections ***/
    } else {
/*** connections with in the same core ***/
      /* If channels are not video - return FALSE indicating connect msg is not consumed */
      if (!siu_is_valid_chn_video (srcChan) || !siu_is_valid_chn_video (dstChan))
         return (-1);    
      
      /* srcChan & dstChan are zero based */
      srcInst = SIUINSTp(srcChan);
      dstInst = SIUINSTp(dstChan);

      /* SRC always needs to be decoder */
      if (SIU_GET_VIDEO_CHANNEL_TYPE(srcInst) != SIU_CHTYPE_DECODER){
         return (MGB_ERR_VIDEO_INVD_CONNECT_REQUEST);
      }

      /* DST needs to be encoder for YUV domain connections
         DST can be decoder or encoder for Pkt domain connections */
      if (domain == MGB_CONN_YUV_DOMAIN) {
        if(SIU_GET_VIDEO_CHANNEL_TYPE(dstInst) != SIU_CHTYPE_ENCODER) {
           return (MGB_ERR_VIDEO_INVD_CONNECT_REQUEST);           
        }
      }

      if(dstInst->stateInst.video.ovlyInst)
      { /* Overlay channel can not be the destination        */
        return (MGB_ERR_VIDEO_INVD_CONNECT_REQUEST);
      } else if(srcInst->stateInst.video.ovlyInst)
      { /* ... but overlay channel can be the source channel */
        if(siuOvlyConnect (srcInst, dstInst, connType) == SIU_ERROR)
        {
          return (MGB_ERR_VIDEO_INVD_CONNECT_REQUEST);
        }
      } else {      
        /* only ADD and DISC are supported at this moment */
        switch (connType) {
          case MGB_CONN_REQ_CONN_ADD:
               if (domain == MGB_CONN_YUV_DOMAIN)
                  SIU_SET_VIDEO_ROUTE_DST_CH_ID_LD(srcInst,dstChan+1);
               else if(domain == MGB_CONN_PACKET_DOMAIN){
                   if(srcInst == dstInst) {
                      SIU_SET_VIDEO_ROUTE_PD_DEFAULT_CONN(srcInst,1);
                   } else {
                      if(strcmp(siuVppuGetRfcName(SIU_GET_VIDEO_CODING_TYPE(srcInst)),
                                siuVppuGetRfcName(SIU_GET_VIDEO_CODING_TYPE(dstInst))) == 0) {
                         /* Allow packet loop back only when SRC & DST are using the same RFC */
                         SIU_SET_VIDEO_ROUTE_DST_CH_ID_PD(srcInst,dstChan+1);
                      } else {
                         return (MGB_ERR_VIDEO_INVD_CONNECT_REQUEST);
                      }
                   }
               }
               break;
          case MBG_CONN_REQ_DISC:
               if (domain == MGB_CONN_YUV_DOMAIN)
                  SIU_SET_VIDEO_ROUTE_DST_CH_ID_LD(srcInst,0);
               else if(domain == MGB_CONN_PACKET_DOMAIN){
                   if(srcInst == dstInst) {
                      SIU_SET_VIDEO_ROUTE_PD_DEFAULT_CONN(srcInst,0);
                   } else {
                      SIU_SET_VIDEO_ROUTE_DST_CH_ID_PD(srcInst,0);
                   }
               }
               break;
          default:
               break;
        }
      }
    }
  return(0);
}


#pragma DATA_SECTION(siuAudioMp3DecDefaultStaticCfg, ".siuAudioCodecCfg");
#pragma DATA_SECTION(siuAudioMp3DecDefaultDynamicCfg, ".siuAudioCodecCfg");
#pragma DATA_SECTION(siuAudioAacEncDefaultStaticCfg, ".siuAudioCodecCfg");
#pragma DATA_SECTION(siuAudioAacEncDefaultDynamicCfg, ".siuAudioCodecCfg");

#if 0
tuint siuAudioMp3DecDefaultStaticCfg[139] = 
{
//  { // header
    0,                            //echo
    13,                           //length
    1,                            // chnum
    MGB_MD_AUDIO_CODEC_GEN_CFG,   // msg_id
    0,                            // trans_id
//  },
//  { // msg_body
    ifatk_AUDIO_CODEC_GEN_CFG_STATIC,   // cfgType
    6,                                  // numWords
//    { // cfgParam
      0,                                // outputPCMWidth MSW
      16,                               // outputPCMWidth LSW   = 16
      0,                                // pcmFormat MSW
      1,                                // pcmFormat LSW        = IAUDIO_INTERLEAVED
      0,                                // dataEndianness MSW
      2                                 // dataEndianness LSW   = XDM_LE_16
//    }
//  }
};

tuint siuAudioMp3DecDefaultDynamicCfg[139] = 
{
//  { // header
    0,                            //echo
    9,                            //length
    1,                            // chnum
    MGB_MD_AUDIO_CODEC_GEN_CFG,   // msg_id
    0,                            // trans_id
//  },
//  { // msg_body
    ifatk_AUDIO_CODEC_GEN_CFG_DYNAMIC,  // cfgType
    2,                                  // numWords
//    {
      0,                                // downSampleSbrFlag LSW  = 0
      0                                 // downSampleSbrFlag MSW
//    }
//  }
};

tuint siuAudioAacEncDefaultStaticCfg[139] = 
{
//  { // header
    0,                            //echo
    43,                           //length
    1,                            // chnum
    MGB_MD_AUDIO_CODEC_GEN_CFG,   // msg_id
    0,                            // trans_id
//  },
//  { // msg_body
    ifatk_AUDIO_CODEC_GEN_CFG_STATIC,   // cfgType
    36,                                 // numWords
//    { // cfgParam
      0,                                // encodingPreset MSW
      0,                                // encodingPreset LSW     = 0
      0x0001,                           // maxSampleRate MSW
      0x7700,                           // maxSampleRate LSW      = 96000
      0x0008,                           // maxBitrate MSW
      0xCA00,                           // maxBitrate LSW         = 576000
      0,                                // maxNoOfCh MSW
      1,                                // maxNoOfCh LSW          = 1
      0,                                // dataEndianness MSW
      2,                                // dataEndianness LSW     = 2
      0,                                // outObjectType MSW
      29,                               // outObjectType LSW      = AACENC_OBJ_TYP_PS
      0,                                // outFileFormat MSW
      2,                                // outFileFormat LSW      = AACENC_TT_ADTS
      0,                                // useCRC MSW
      0,                                // useCRC LSW             = AACENC_FALSE
      0,                                // useTns MSW
      1,                                // useTns LSW             = AACENC_TRUE
      0,                                // usePns MSW
      1,                                // usePns LSW             = AACENC_TRUE
      0,                                // downMixFlag MSW
      0,                                // downMixFlag LSW        = AACENC_FALSE
      0,                                // bitRateMode MSW
      0,                                // bitRateMode LSW        = AACENC_BR_MODE_CBR
      0,                                // bitRate MSW
      0xFA00,                           // bitRate LSW            = 64000
      0,                                // sampleRate MSW
      0xBB80,                           // sampleRate LSW         = 48000
      0,                                // nbInChannels MSW
      1,                                // nbInChannels LSW       = 1
      0,                                // dualMono MSW
      0,                                // dualMono LSW           = AACENC_FALSE
      0,                                // ancFlag MSW
      0,                                // ancFlag LSW            = AACENC_FALSE
      0xFFFF,                           // ancRate MSW
      0xFFFF                            // ancRate LSW            = -1
//    }
//  }
};

tuint siuAudioAacEncDefaultDynamicCfg[139] = 
{
//  { // header
    0,                            //echo
    37,                           //length
    1,                            // chnum
    MGB_MD_AUDIO_CODEC_GEN_CFG,   // msg_id
    0,                            // trans_id
//  },
//  { // msg_body
    ifatk_AUDIO_CODEC_GEN_CFG_DYNAMIC,  // cfgType
    30,                                 // numWords
//    {
      0,                                // inputFormat MSW
      1,                                // inputFormat LSW          = 1
      0,                                // bitRate MSW
      0xFA00,                           // bitRate LSW              = 64000
      0,                                // sampleRate MSW
      0xBB80,                           // sampleRate LSW           = 48000
      0,                                // numChannels MSW
      1,                                // numChannels LSW          = 1
      0,                                // numLFEChannels MSW
      0,                                // numLFEChannels LSW       = 0
      0,                                // inputBitsPerSample MSW
      16,                               // inputBitsPerSample LSW   = 16
      0,                                // dualMono MSW
      0,                                // dualMono LSW             = AACENC_FALSE
      0,                                // useTns MSW
      1,                                // useTns LSW               = AACENC_TRUE
      0,                                // usePns MSW
      1,                                // usePns LSW               = AACENC_TRUE
      0,                                // useCRC MSW
      0,                                // useCRC LSW               = AACENC_FALSE
      0,                                // downMixFlag MSW
      0,                                // downMixFlag LSW          = AACENC_FALSE
      0,                                // outObjectType MSW
      29,                               // outObjectType LSW        = AACENC_OBJ_TYP_PS
      0,                                // outFileFormat MSW
      2,                                // outFileFormat LSW        = AACENC_TT_ADTS
      0,                                // ancFlag MSW
      0,                                // ancFlag LSW              = AACENC_FALSE
      0xFFFF,                           // ancRate MSW
      0xFFFF                            // ancRate LSW              = -1
//    }
//  }
};
#else
tuint siuAudioMp3DecDefaultStaticCfg[139];
tuint siuAudioMp3DecDefaultDynamicCfg[139];
tuint siuAudioAacEncDefaultStaticCfg[139];
tuint siuAudioAacEncDefaultDynamicCfg[139];
#endif

#pragma DATA_SECTION(siuAudioCodecCfgCodecType,".siuAudioCodecCfgCodecType");
tuint siuAudioCodecCfgCodecType = EVCT_AUDIO_NONE;


/******************************************************************************
 * FUNCTION PURPOSE: Executes Codec Configuration
 ******************************************************************************
 * DESCRIPTION: 
 *
 *  void siuVideoCodecGenCfg (
 *    siuInst_t *inst         - pointer to channel information
 *    mgb_t     *message)     - pointer to micro message
 *
 *****************************************************************************/
tulong *debug_static_params_ptr = NULL;
tulong *debug_dynamic_params_ptr = NULL;
void siuAudioCodecGenCfg (siuInst_t *inst, mgb_t *message) 
{
  tuint codecCfgType;
  tuint errorCode = 0;
  tulong recvdSize = 0, sizeCodecRequired = 0;
  tulong *cfgPtr;
  atkCtrl_t  atkCtrl;

  codecCfgType = message->msg_body.audio_codec_cfg.cfgType;

  /* Once channel enters Video Mode, Static Configuration cannot be sent */
  if ((codecCfgType == ifatk_AUDIO_CODEC_GEN_CFG_STATIC) && (inst->state == SIU_CHST_VIDEO)) {
    errorCode = MGB_ERR_AUDIO_CODEC_STATIC_CFG_REJECTED;
    goto error;
  }

  if (codecCfgType == ifatk_AUDIO_CODEC_GEN_CFG_STATIC) {
     atkCtrl.code = ATK_GET_STATIC_CFG_PTR;
     errorCode = MGB_ERR_AUDIO_CODEC_STATIC_CFG_REJECTED;
  } else if (codecCfgType == ifatk_AUDIO_CODEC_GEN_CFG_DYNAMIC) {
     atkCtrl.code = ATK_GET_DYNAMIC_CFG_PTR;
     errorCode = MGB_ERR_AUDIO_CODEC_DYNAMIC_CFG_REJECTED;
  }
  /* for now, just GET_***_PTR initializes data (no need to supply from host) */
  gg_atk_ctable.atkControl(inst->stateInst.video.atkInst,&atkCtrl);
  
  cfgPtr            = atkCtrl.u.codecCfgPtr.cfgPtr;
  sizeCodecRequired = atkCtrl.u.codecCfgPtr.size;
  recvdSize = message->msg_body.audio_codec_cfg.numWords;

  /* comparing recvdSize + sizeof(tulong) because micro didn't send size as part of the structure */
  /* sizeCodecRequired is in bytes */
  if ((UTL_TINTS_TO_BYTE(recvdSize) + sizeof(tulong)) != sizeCodecRequired) {
     goto error;
  }
  
  /* size is filled into the structure by VTK itself - So cfgPtr++ */
  cfgPtr++;

  /* copy the new configuration to the Codec structure */
  utlPackTuintToTulong(message->msg_body.audio_codec_cfg.cfgParam, (tulong *)cfgPtr, recvdSize);

  if (codecCfgType == ifatk_AUDIO_CODEC_GEN_CFG_STATIC) {
    SIU_SET_AUDIO_STATIC_CFG_DONE(inst, 1);
  } else if (codecCfgType == ifatk_AUDIO_CODEC_GEN_CFG_DYNAMIC) {
    SIU_SET_AUDIO_DYNAMIC_CFG_DONE(inst, 1);
  }

  return;

error:
  siuErrorIndication (inst->ID, message->header.trans_id, errorCode);
  return;
}

/******************************************************************************
 * FUNCTION PURPOSE: Executes Codec Configuration
 ******************************************************************************
 * DESCRIPTION: 
 *
 *  void siuVideoCodecGenCfg (
 *    siuInst_t *inst         - pointer to channel information
 *    mgb_t     *message)     - pointer to micro message
 *
 *****************************************************************************/
void siuVideoCodecGenCfg (siuInst_t *inst, mgb_t *message) {
  siuVideoChannelOperation_t chType;
  tuint codecCfgType;
  tuint errorCode = 0;
  tulong recvdSize = 0, sizeCodecRequired = 0;
  tulong *cfgPtr;
  tuint codingType;
  vtkCtrl_t  vtkCtrl;
  vppuCtrl_t vppuCtrl;

  chType       = (siuVideoChannelOperation_t) SIU_GET_VIDEO_CHANNEL_TYPE(inst);
  codingType   = SIU_GET_VIDEO_CODING_TYPE(inst);
  codecCfgType = message->msg_body.video_codec_cfg.cfgType;

  if ((codingType == EVCT_VIDEO_BYPASS_ENC) || (codingType == EVCT_VIDEO_BYPASS_DEC)) {
    SIU_SET_VIDEO_STATIC_CFG_DONE(inst, 1);
    SIU_SET_VIDEO_DYNAMIC_CFG_DONE(inst, 1);
    return;
  }
 
  /* chType is set upon VIDEO_MODE. CodecGenCfg is allowed only after VIDEO_MODE */
  if((chType != SIU_CHTYPE_ENCODER) && (chType != SIU_CHTYPE_DECODER)) {
    errorCode = MGB_ERR_VIDEO_CODEC_GEN_CFG_MSG_REJECTED;
  }

  /* Once channel enters Video Mode, Static Configuration cannot be sent */
  if ((codecCfgType == ifvtk_VIDEO_CODEC_GEN_CFG_STATIC) && (inst->state == SIU_CHST_VIDEO)) {
    errorCode = MGB_ERR_VIDEO_CODEC_STATIC_CFG_REJECTED;
  }

  /* SPS/PPS configuration is allowed only when all the following conditions are met
     a) channel is in video mode (both static & dynamic configs received & committed)
     b) channel type is decoder
     c) codec is either H.264BP or H.264MP
  */
  if (codecCfgType == ifvtk_VIDEO_CODEC_GEN_CFG_SPROP_PARAM) {
    if((inst->state != SIU_CHST_VIDEO) || (chType != SIU_CHTYPE_DECODER)){
      errorCode = MGB_ERR_VIDEO_CODEC_SPROP_PARAM_REJECTED;
    } else if ((codingType != EVCT_VIDEO_H264BP_MP_DEC) &&
               (codingType != EVCT_VIDEO_H264HP_DEC)) {
      errorCode = MGB_ERR_VIDEO_CODEC_SPROP_PARAM_REJECTED;
    }
    if(inst->stateInst.video.vppuInst != NULL) {
      vppuCtrl.code = VPPU_SET_OUT_OF_BAND_INFO;
      vppuCtrl.u.outOfBandInfo.buffer = (tword *) message->msg_body.video_codec_cfg.cfgParam;
      vppuCtrl.u.outOfBandInfo.length = UTL_TINTS_TO_BYTE(message->msg_body.video_codec_cfg.numWords);
      if (vppuControl(inst->stateInst.video.vppuInst, &vppuCtrl) != VPPU_NOERR) {
         errorCode = MGB_ERR_VIDEO_CODEC_SPROP_PARAM_REJECTED;
      } else {
         return;
      }
    }
  }

  if (errorCode) {
   goto error;
  }

  /* Hack for Audio */
  if( SIU_GET_AUDIO_CODING_TYPE(inst) == EVCT_AUDIO_MP3_DEC) {
    if( codecCfgType == ifvtk_VIDEO_CODEC_GEN_CFG_STATIC) {
      siuAudioCodecGenCfg (inst, (mgb_t  *)&siuAudioMp3DecDefaultStaticCfg[0]);
    } else if ( codecCfgType == ifvtk_VIDEO_CODEC_GEN_CFG_DYNAMIC) {
      siuAudioCodecGenCfg (inst, (mgb_t *)&siuAudioMp3DecDefaultDynamicCfg[0]);
    }
  } else if(SIU_GET_AUDIO_CODING_TYPE(inst) == EVCT_AUDIO_AAC_ENC) {
    if( codecCfgType == ifvtk_VIDEO_CODEC_GEN_CFG_STATIC) {
      siuAudioCodecGenCfg (inst, (mgb_t *)&siuAudioAacEncDefaultStaticCfg[0]);
    } else if ( codecCfgType == ifvtk_VIDEO_CODEC_GEN_CFG_DYNAMIC) {
      siuAudioCodecGenCfg (inst, (mgb_t *)&siuAudioAacEncDefaultDynamicCfg[0]);
    }
  }
  /* End audio hack */


  /* Call the Codec Cfg */
  if (codecCfgType == ifvtk_VIDEO_CODEC_GEN_CFG_STATIC) {
     vtkCtrl.code = VTK_GET_STATIC_CFG_PTR;
     errorCode = MGB_ERR_VIDEO_CODEC_STATIC_CFG_REJECTED;
  } else if (codecCfgType == ifvtk_VIDEO_CODEC_GEN_CFG_DYNAMIC) {
     vtkCtrl.code = VTK_GET_DYNAMIC_CFG_PTR;
     errorCode = MGB_ERR_VIDEO_CODEC_DYNAMIC_CFG_REJECTED;
  }
  gg_vtk_ctable.vtkControl(inst->stateInst.video.vtkInst,&vtkCtrl);
  cfgPtr            = vtkCtrl.u.codecCfgPtr.cfgPtr;
  sizeCodecRequired = vtkCtrl.u.codecCfgPtr.size;
  recvdSize = message->msg_body.video_codec_cfg.numWords;

  /* comparing recvdSize + sizeof(tulong) because micro didn't send size as part of the structure */
  /* sizeCodecRequired is in bytes */
  if ((UTL_TINTS_TO_BYTE(recvdSize) + sizeof(tulong)) != sizeCodecRequired) {
     goto error;
  }

  /* check if dynamic config is violating the rule that it has to be <= static */
  if (chType == SIU_CHTYPE_ENCODER && codecCfgType == ifvtk_VIDEO_CODEC_GEN_CFG_DYNAMIC) {
     vtkCtrl.code                 = VTK_IS_DYN_UPDATE_ALLOWED;
     vtkCtrl.u.codecCfgPtr.cfgPtr = (tulong *)message->msg_body.video_codec_cfg.cfgParam;
     if (gg_vtk_ctable.vtkControl(inst->stateInst.video.vtkInst,&vtkCtrl) != VTK_NOERR) {
       goto error;
     }
  }

  /* size is filled into the structure by VTK itself - So cfgPtr++ */
  cfgPtr++;

  /* copy the new configuration to the Codec structure */
  utlPackTuintToTulong(message->msg_body.video_codec_cfg.cfgParam, (tulong *)cfgPtr, recvdSize);

  /* Configure FRC on Encoder Leg */
  if (chType == SIU_CHTYPE_ENCODER && codecCfgType == ifvtk_VIDEO_CODEC_GEN_CFG_DYNAMIC) {
      if (inst->state == SIU_CHST_VIDEO) {
         siuReconfigureEncoderLeg(inst);
      }
  }

  if (codecCfgType == ifvtk_VIDEO_CODEC_GEN_CFG_STATIC) {
    SIU_SET_VIDEO_STATIC_CFG_DONE(inst, 1);
  } else if (codecCfgType == ifvtk_VIDEO_CODEC_GEN_CFG_DYNAMIC) {
    SIU_SET_VIDEO_DYNAMIC_CFG_DONE(inst, 1);
  }
  return;

error:
  siuErrorIndication (inst->ID, message->header.trans_id, errorCode);
  return;

}

/******************************************************************************
 * FUNCTION PURPOSE: Reconfigure Encoder Leg
 ******************************************************************************
 * DESCRIPTION: Reconfigure encoder leg with dynamic configuration parameters
 *
 *  void siuReconfigureEncoderLeg (
 *    siuInst_t *inst)        - pointer to channel information
 *
 *****************************************************************************/
void siuReconfigureEncoderLeg(siuInst_t *inst)
{
  vppuCtrl_t vppuCtrl;
  tsuCtrl_t  tsuCtrl;
  vtkCtrl_t  vtkCtrl;

  /* Recover dynamically configured parameters */
  vtkCtrl.code = VTK_GET_ENC_DYN_CFG;
  gg_vtk_ctable.vtkControl(inst->stateInst.video.vtkInst, &vtkCtrl);

  /* Reset the Encoder */
  vtkCtrl.code = VTK_RESET_CODEC;
  gg_vtk_ctable.vtkControl(inst->stateInst.video.vtkInst, &vtkCtrl);
 
  /* Reconfigure FRC */
  siu_frc_configure_frameRate(&(inst->frcInst), vtkCtrl.u.vtkEncDynCfg.targetFrameRate);

  /* Reconfigure VPPU/RFC */
  if(inst->stateInst.video.vppuInst) {
    vppuCtrl.code        = VPPU_SET_RTP_INCR_TS;
    vppuCtrl.u.rtpIncrTs = 90000 / (vtkCtrl.u.vtkEncDynCfg.targetFrameRate / 1000);
    vppuControl(inst->stateInst.video.vppuInst, &vppuCtrl);
  }

  /* Reconfigure TSU */
  if(inst->stateInst.video.tsuInst != NULL) {
    tsuCtrl.u.outRes.out_width     = vtkCtrl.u.vtkEncDynCfg.img_width;
    tsuCtrl.u.outRes.out_height    = vtkCtrl.u.vtkEncDynCfg.img_height;
    tsuControl(inst->stateInst.video.tsuInst, TSU_CTL_RESET_FILTER, &tsuCtrl);
  }

  /* Reconfigure SIU width/height (used for overlay) */
  inst->yuvWidth   = vtkCtrl.u.vtkEncDynCfg.img_width;
  inst->yuvHeight  = vtkCtrl.u.vtkEncDynCfg.img_height;

  inst->frameRate  = vtkCtrl.u.vtkEncDynCfg.targetFrameRate;
  inst->bitRate    = vtkCtrl.u.vtkEncDynCfg.targetBitrate;

  return;
}

/******************************************************************************
 * FUNCTION PURPOSE: Executes change to Video Mode.
 ******************************************************************************
 * DESCRIPTION: Implements start of transition to voice mode. Assumes channel is 
 *              in an IDLE configuration upon entry.  Channel state is not changed
 *              to SIU_CHST_VIDEO, but pending_state is.
 *              All ECOs are opened at this point if there is sufficient information
 *              for its configuration, i.e. provided in open channel configuration.
 *
 *  void siuVideoMode (
 *    siuInst_t *inst         - pointer to channel information
 *    mgb_t     *message)     - pointer to micro message
 *
 *****************************************************************************/
void siuVideoMode (siuInst_t *inst, mgb_t *message)
{
  tint chnum      = (tint) (inst->ID & SIU_CID_MASK);
  siuVideoChannelOperationExt_t chTypeExt = SIU_CHTYPE_EXT_COMMON;
  siuVideoChannelOperation_t chType;
  tint codingType = message->msg_body.video_mode.codingType;
  word reorderDepth = 0; /* default value = 0 */
  word maxDisplayDelay = 0;
  tuint i;
  tuint imgYUVWidth, imgYUVHeight;

  /* Make Sure channel is in Idle Mode */
  if (inst->state != SIU_CHST_IDLE) {
    siuErrorIndication (inst->ID, message->header.trans_id, MGB_ERR_VIDEO_MODE_MSG_REJECTED);
    return;
  }

  /* Invalidate last packet type, to force siu_stm_search to re-search */
  inst->lastPacketType = 0xFFFFU;
  /* Save current payload Type */
  inst->payloadType = message->msg_body.video_mode.payloadType;
  /* No default legacy overlay operation */
  inst->stateInst.video.ovly_legacy_bitmap = 0;

  /* check if the coding type is allowed */
  siu_open_channel_block_codecs(inst,codingType);

  /* Image width and height */
  imgYUVWidth  = message->msg_body.video_mode.imgWidth;
  imgYUVHeight = message->msg_body.video_mode.imgHeight;
  
  switch(codingType) {
    case EVCT_VIDEO_H264BP_ENC:
    case EVCT_VIDEO_MPEG4SP_ENC:
    case EVCT_VIDEO_H263P_ENC:
    case EVCT_VIDEO_BYPASS_ENC:
    case EVCT_VIDEO_H264MP_ENC:
       chType = SIU_CHTYPE_ENCODER;
       break;

    case EVCT_VIDEO_H264BP_MP_DEC:
        reorderDepth    = GG_VIDEO_NUM_YUV_BUF_PER_CHAN; /* max = 12 */
        maxDisplayDelay = reorderDepth - 2;
        chType          = SIU_CHTYPE_DECODER;
        break;
    case EVCT_VIDEO_H264HP_DEC:
        reorderDepth    = 10; /* The maximal for 1080p is 10 due to memory limit */
        maxDisplayDelay = reorderDepth - 2;
        chType          = SIU_CHTYPE_DECODER;
        /* Width and height with YUV padding */
        imgYUVWidth       += GG_VIDEO_YUV_PAD_PELS  * 2;
        imgYUVHeight      += GG_VIDEO_YUV_PAD_LINES * 2;
        break;
    case EVCT_VIDEO_MPEG2MP_DEC:
        reorderDepth = GG_VIDEO_NUM_YUV_BUF_PER_CHAN; /* max = 12 */
        chType       = SIU_CHTYPE_DECODER;
        break;

    case EVCT_VIDEO_MPEG4SP_DEC:
    case EVCT_VIDEO_H263P_DEC:
    case EVCT_VIDEO_BYPASS_DEC:
        reorderDepth = 3; /* No reorder supported. Minimum depth needed = 3 */
        chType       = SIU_CHTYPE_DECODER;
        break;

    /* Identify request for overlay channel */
    case EVCT_VIDEO_OVLY_RAW_GRAPHICS:
       chType     = SIU_CHTYPE_DECODER;
       codingType = EVCT_VIDEO_BYPASS_DEC;        
       chTypeExt  = SIU_CHTYPE_EXT_OVLY_RAW;
       reorderDepth = 3; /* No reorder supported. Minimum depth needed = 3 */
       break;

    case EVCT_VIDEO_OVLY_TEXT:
       chType     = SIU_CHTYPE_DECODER;
       codingType = EVCT_VIDEO_BYPASS_DEC;       
       chTypeExt  = SIU_CHTYPE_EXT_OVLY_TTXT;
       reorderDepth = 3; /* No reorder supported. Minimum depth needed = 3 */
       break;    

    case EVCT_VIDEO_TSU_ONLY:
       reorderDepth = 7; /* No reorder supported. Minimum depth needed = 3. Set to 7 for extra buffering */
       chType = SIU_CHTYPE_ENCODER;
       codingType = EVCT_VIDEO_BYPASS_ENC;
       chTypeExt  = SIU_CHTYPE_EXT_TSU_ONLY;
       break;

    default:
       siuErrorIndication (inst->ID, message->header.trans_id, MGB_ERR_VIDEO_INVALID_CODING_TYPE);
       return;
   }

  /* If host supplied reorder depth, overwrite the DSP default value */
  if(message->msg_body.video_mode.validParams & MGB_MDS_VIDEO_MODE_VALID_YUV_REORDER_DEPTH) {
     reorderDepth = message->msg_body.video_mode.yuvReorderDepth;
     if (codingType == EVCT_VIDEO_H264BP_MP_DEC) {
        maxDisplayDelay = message->msg_body.video_mode.yuvReorderDepth - 2;
     }
  }

  /* If host supplied MTU size, overwrite the DSP default value */
  if(message->msg_body.video_mode.validParams & MGB_MDS_VIDEO_MODE_VALID_MAX_PAYLOAD_SIZE) {
     if(message->msg_body.video_mode.maxPayloadSize > SIU_DEFAULT_TX_MAX_PAYLOAD_SIZE) {
       siuErrorIndication (inst->ID, message->header.trans_id, MGB_ERR_VIDEO_INVALID_MAX_PAYLOAD_SIZE);
       return;
     }
     inst->maxPayloadSize = message->msg_body.video_mode.maxPayloadSize;
  } else {
     inst->maxPayloadSize = SIU_DEFAULT_TX_MAX_PAYLOAD_SIZE;
  }

  if(message->msg_body.video_mode.validParams & MGB_MDS_VIDEO_MODE_VALID_MC_NCORES) {
    /* If host supplied Multicore configuration, overwrite the DSP default value */
     if(message->msg_body.video_mode.nCores > SIUVID_MC_MAXCORES) {
       siuErrorIndication (inst->ID, message->header.trans_id, MGB_ERR_VIDEO_INVALID_MC_NCORES);
       return;
     }
     inst->mcInst.nCores = message->msg_body.video_mode.nCores;
     for (i=0; i<inst->mcInst.nCores; i++)
     {
       inst->mcInst.coreTeamMapping[i] = message->msg_body.video_mode.coreTeamMapping[i];
       inst->mcInst.chanIdOnCores[i] = message->msg_body.video_mode.chanIdOnCores[i];
       if(inst->mcInst.coreTeamMapping[i] >= SIUVID_MC_MAXCORES) {
         siuErrorIndication (inst->ID, message->header.trans_id, MGB_ERR_VIDEO_INVALID_MC_NCORES);
         return;
       }
     }
  } else {
    /* Default to be single core codec */
     inst->mcInst.nCores = 1;
     inst->mcInst.coreTeamMapping[0] = siuContext.coreId;
     inst->mcInst.chanIdOnCores[0] = chnum;
  }

  /* Set the non-used entries */
  for (i=inst->mcInst.nCores; i<SIUVID_MC_MAXCORES; i++)
  {
     inst->mcInst.coreTeamMapping[i] = IVIDMC_CORE_NOT_USED;
  }

  /* Save information which is needed by creating/opening software barrier */
  siuContext.ipcContext.nCoresInBarrier = inst->mcInst.nCores;
  siuContext.ipcContext.masterCoreId = inst->mcInst.coreTeamMapping[0];

  /* For multi-core codec, master core records slave core IDs and 
     posts event to open MessageQ for future master-slave messaging.
     For slave core, it attaches to the master core here. */
  if(inst->mcInst.nCores > 1) {
    if(SIU_IS_MCCODEC_MASTER(inst)) {
    tuint j = 0;
    for(i = 1; i < inst->mcInst.nCores; i++) {
      if (inst->mcInst.coreTeamMapping[i] != siuContext.coreId) {
        /* Record slave core IDs */
        siuContext.slaveCoreConn[j++] = inst->mcInst.coreTeamMapping[i];
      }
    }
    /* Open MessageQ if there are slave cores connected to a master core*/
    siu_ipc_osal_msgq_open_alloc();
    } else if(SIU_IS_MCCODEC_SLAVE(inst)) {
        /* Slave core attaches to the master core */
        siu_ipc_osal_attach_remote(inst->mcInst.coreTeamMapping[0]);
    }
  }

  SIU_SET_VIDEO_CHANNEL_TYPE(inst, chType);
  SIU_SET_VIDEO_CHANNEL_EXT_TYPE(inst, chTypeExt);
  SIU_SET_VIDEO_CODING_TYPE (inst, codingType);

  /* Initialize audio buffers */
  if(siuContext.buildID == SIU_VIGDK_BUILD_ID1_D1) {
    /* Initialize storage for audio remaining after call to decoder or encoder. */
    inst->audBuffer = gmpAllocGmc(inst->ID, siuContext.siuAudMemHandle);
    inst->audBufferSize = gmpSizeGmc(inst->ID, siuContext.siuAudMemHandle);
    inst->audBufferIndex = 0;
    /* Initilize PCM buffers used to send decoded audio to encoder core. */
  }

  if(SIU_IS_MCCODEC_MASTER(inst)) {
    /* MASTER CORE */
    /* Create barrier when there are multiple cores */
    if(inst->mcInst.nCores > 1) {
        siu_ipc_osal_barrier_create(inst->mcInst.nCores);
    }

    if((chTypeExt == SIU_CHTYPE_EXT_OVLY_RAW) || (chTypeExt == SIU_CHTYPE_EXT_OVLY_TTXT)) {
    /* Overlay channel needs only overlay module */
    /* Create overlay module */
    siu_new_ovly (chnum);    
    siu_open_ovly (chnum, codingType, 
                          message->msg_body.video_mode.imgWidth, 
                          message->msg_body.video_mode.imgHeight);
    } else inst->stateInst.video.ovlyInst = NULL;

    /* Open Custom ECO module */
    siu_open_custom_eco(chnum, codingType, 
                        message->msg_body.video_mode.imgWidth, 
                        message->msg_body.video_mode.imgHeight);

    /* configure trans-sizing unit */
    siu_open_tsu(chnum, message->msg_body.video_mode.imgWidth, 
                        message->msg_body.video_mode.imgHeight);
  
    /* Initialize YUV buffer Pool */
    siu_init_yuv(inst, imgYUVWidth, imgYUVHeight, reorderDepth);

    /* Initialize FRC */
    siu_init_frc(&(inst->frcInst));

    /* Open NDU or ACU or RCU w/ new API */
    if(SIU_IS_PROTO_RTP(inst)) 
    {
      siu_new_vppu(chnum);
      siu_open_network (chnum, gg_vppu_ctable.vppuReceiveIn, inst->stateInst.video.vppuInst, TRUE);
    } 
    else if(SIU_IS_PROTO_TFTP(inst) || inst->protocol_type == MGB_PT_NONE) 
    {
      siu_new_mmcu(chnum);
      /* If TFTP is present, MMCU will be connected with the transfer starts. 
         If TFTP is not present (MPEGTS streaming), MMCU will be connected 
         directly to NEU.
       */
      siu_open_network (chnum, mmcuReceiveIn, inst->stateInst.video.mmcuInst, TRUE);
    }
  } else {
    /* SLAVE CORE */
    /* Open barrier which is created on the master core */
    siu_ipc_osal_barrier_open(inst->mcInst.coreTeamMapping[0]);
  }
  
  /* Below code is common to MASTER core and SLAVE core */
  /* Open VTK module */
  siu_open_vtk (chnum, codingType, 
                       message->msg_body.video_mode.imgWidth, 
                       message->msg_body.video_mode.imgHeight,
                       maxDisplayDelay);

  /* Open ATK module */
  if(siuAudioCodecCfgCodecType != EVCT_AUDIO_NONE) {
    siu_exc_assert(siuContext.buildID == SIU_VIGDK_BUILD_ID1_D1, SIUATK_NOT_SUPPORTED, inst);
    /* Create ATK module*/
    siu_new_atk(chnum);
    siu_open_atk (chnum, siuAudioCodecCfgCodecType);
    SIU_SET_AUDIO_CODING_TYPE(inst,siuAudioCodecCfgCodecType);
  }

  /* VIDEO mode is not entered */
  inst->pending_state = SIU_CHST_VIDEO;

} /* siuVideoMode */

/******************************************************************************
 * FUNCTION PURPOSE: Commit State to Video Mode.
 ******************************************************************************
 * DESCRIPTION: 
 *
 *  void siuCommitStateVideo (
 *    siuInst_t *inst         - pointer to channel information
 *    mgb_t     *message)     - pointer to micro message
 *
 *****************************************************************************/
void siuCommitStateVideo (siuInst_t *inst,  mgb_t *message)
{
  siuCodecCfg_t siuCodecCfg;
  siuVideoChannelOperation_t chType;
  vppuCtrl_t vppuCtrl;
  siuInstRTP_t *p_rtp_params = &inst->rtp;
  tuint codingType;

  siu_exc_assert ( SIU_GET_VIDEO_STATIC_CFG_DONE(inst) && SIU_GET_VIDEO_DYNAMIC_CFG_DONE(inst), 
                   SIUVIDEO_EXC_COMMIT_STATE | SIU_EXCEPTION_TYPE_CHANNEL, inst);

  if(SIU_IS_MCCODEC_MASTER(inst)) {
    /* Open VCU module and return coding rate information*/
    /* Open Tx Task in Voice mode, setup mega-staggering */
    SIU_STATE_BEGIN();
    siuTxOpen (inst);   /*R9.2, do not open Tx Task until COMMIT state arrived*/
    SIU_STATE_END();
  }

  /* Record current working channel number so that siuVidMc_Map_Shmem 
     can identify the right nCores and core team mapping */
  siuContext.current_working_chnum = (inst->ID & SIU_CID_MASK) - SIU_FIRSTCH;
  /* Wait for all the participating cores to be ready before proceeding */
  siuVidMc_Swbarr(siuContext.coreId, IVIDMC_SWBARR15, inst->mcInst.nCores);
  siu_commit_vtk (inst, &siuCodecCfg);

  if(SIU_GET_AUDIO_CODING_TYPE(inst) != EVCT_AUDIO_NONE) {
    siu_exc_assert ( SIU_GET_AUDIO_STATIC_CFG_DONE(inst) && SIU_GET_AUDIO_DYNAMIC_CFG_DONE(inst), 
                   SIUVIDEO_EXC_COMMIT_STATE | SIU_EXCEPTION_TYPE_CHANNEL, inst);
    siu_commit_atk (inst);
  }

  siu_osal_wbinv_cache_all();

  /* Open VPPU module */
  if (SIU_GET_VIDEO_CHANNEL_EXT_TYPE(inst) == SIU_CHTYPE_EXT_OVLY_TTXT)
  {	/* The rest of channel is decoder bypass - only VPPU needs right
     * coding type to select proper RFC */
    siuCodecCfg.codingType = EVCT_VIDEO_OVLY_TEXT;
  }

  /* Open VPPU if present. MMCU is opened with the MMCU CFG message */
  if(inst->stateInst.video.vppuInst) {
    siu_open_vppu (inst, &siuCodecCfg);

    /* Set the initial RTP timestamp */
    vppuCtrl.code = VPPU_SET_RTP_BASE_TS;
    vppuCtrl.u.rtpBaseTs = p_rtp_params->TxInitTimeStamp;
    vppuControl(inst->stateInst.video.vppuInst, &vppuCtrl);
  }

  /* Configure Frame Rate Control Unit */
  chType = (siuVideoChannelOperation_t) SIU_GET_VIDEO_CHANNEL_TYPE(inst);
  codingType = SIU_GET_VIDEO_CODING_TYPE(inst);
  if ((chType == SIU_CHTYPE_ENCODER) && (codingType != EVCT_VIDEO_BYPASS_ENC)) {
    siuReconfigureEncoderLeg(inst);
  } else if(chType == SIU_CHTYPE_DECODER){
    /* time out and flush the YUV pool if nothing is received for 
       SIU_DEFAULT_RX_TIMEOUT_TICKS sec 
     */
    inst->timeOutTicks         = SIU_DEFAULT_RX_TIMEOUT_TICKS;
  }

  inst->state = SIU_CHST_VIDEO;    /* declare this channel as in VIDEO mode */
}

/******************************************************************************
 * FUNCTION PURPOSE: Control function for Video
 ******************************************************************************
 * DESCRIPTION: 
 *
 *  void siuCommitStateVideo (
 *    siuInst_t *inst         - pointer to channel information
 *    mgb_t     *message)     - pointer to micro message
 *
 *****************************************************************************/
void siuVideoCtrl (siuInst_t *inst, mgb_t *message)
{
  siuVideoChannelOperation_t chType;
  vppuCtrl_t vppuCtrl;
  tint retVal;

  chType = (siuVideoChannelOperation_t) SIU_GET_VIDEO_CHANNEL_TYPE(inst);

  if(message->msg_body.video_ctrl.validParams & MGB_MDS_VIDEO_CTRL_VALID_SPROP_TX_CFG) {
    /* SPS/PPS/DCI regeneration can be configured only on Encoder channel */
    if(chType != SIU_CHTYPE_ENCODER) {
       siuErrorIndication (inst->ID, message->header.trans_id, MGB_ERR_VIDEO_CTRL_REJECTED);
       return;
    }
    vppuCtrl.code         = VPPU_SET_SPROP_TX_CFG;
    vppuCtrl.u.spropTxCfg = message->msg_body.video_ctrl.spropTxCfg;
    retVal = vppuControl(inst->stateInst.video.vppuInst, &vppuCtrl);
    if (retVal != VPPU_NOERR) {
       siuErrorIndication (inst->ID, message->header.trans_id, MGB_ERR_VIDEO_CTRL_REJECTED);
       return;
    }
  }
  return;
}

/******************************************************************************
 * FUNCTION PURPOSE: Configure network for voice/fax/transparent mode
 ******************************************************************************
 * DESCRIPTION:  Opens NDU, ACU and/or SIOU
 *  
 *  void siu_open_network (
 *    tint chnum)        - Channel #
 *    void (*netReceiveIn) (void *inst, tint cnt[], void *pkts[]),
 *    void *targetInst,
 *    bool rcuApi)       - TRUE : use new RCU api; FALSE use old NDU API
 *
 *****************************************************************************/

void siu_open_network (tint chnum, tint (*netReceiveIn) (void *inst, xferPktInfo_t *pktInfo),
                       void *targetInst, bool rcuApi) 
{
  tint (*receiveOut) (void *receiveOutInst, xferPktInfo_t *pktInfo);
  void *receiveOutInst;
  siuInst_t *inst = SIUINSTp(chnum-SIU_FIRSTCH);
  siuNduConfig_t   nduConfig;   /* Set up NDU ReceiveOut */
#ifdef GG_INCLUDE_AAL2
  acuControl_t     acuConfig;   /* Set up ACU ReceiveOut */
#endif
  rcuCtrl_t        rcuConfig;   /* Set up RCU ReceiveOut */
  neuCtrl_t        neuConfig;   /* Set up NEU ReceiveOut */
  
  SIU_RXINT_BEGIN();
  /* Configure NDU/ACU/RCU receiveout */
  if (SIU_IS_PROTO_AAL2(inst)) { /* ACU present, configure both ACU and NDU */
#ifdef GG_INCLUDE_AAL2
    acuConfig.code                           = ACU_CONTROL_RECEIVEOUT;
    acuConfig.u.ReceiveOutSpec.acuReceiveOut = netReceiveIn;
    acuConfig.u.ReceiveOutSpec.targetInst    = targetInst; 
    (*gg_acu_ctable.acuControl) (inst->net.acuInst, &acuConfig);  
    receiveOut     = gg_acu_ctable.acuReceiveIn;
    receiveOutInst = inst->net.acuInst; 
#endif
  } else if ( SIU_IS_PROTO_RTP(inst) ||
              (SIU_IS_PROTO_IU(inst) && SIU_IS_IU_OVER_IP(inst)) ) {
    /* RCU present, configure both RCU and NDU */
    rcuConfig.code                          = rcu_CTRL_RECEIVEOUT;
    rcuConfig.u.rxOut.rcuReceiveOut.rcuReceiveOut = netReceiveIn;
    rcuConfig.u.rxOut.rcuReceiveOut.targetInst    = targetInst; 
    rcuConfig.u.rxOut.newApi                      = rcuApi; /* Use PVP/RCU api */
    (*gg_rcu_ctable.rcuControl) (inst->net.rcuInst, &rcuConfig); 
    receiveOut     = gg_rcu_ctable.rcuReceiveIn;
    receiveOutInst = inst->net.rcuInst;    
  } else if( SIU_IS_PROTO_TFTP(inst)) {
    /* TFTP will be hooked to MMCU when the TFTP transfer is initiated */
    receiveOut      = tftpReceiveIn;
    receiveOutInst  = inst->net.tftpInst;
  } else { /* NO ACU/RCU, just configure NDU */
    receiveOut     = netReceiveIn;
    receiveOutInst = targetInst;
  }

  if (SIU_IS_NEU_PRESENT(inst)){
    nduConfig.RxOutSpec.nduReceiveOut           = SIU_NDU_GET_NEU_RECEIVE_IN;
    nduConfig.RxOutSpec.targetInst              = inst->neuInst;
    neuConfig.u.neuReceiveOutSpec.neuReceiveOut = receiveOut;
    neuConfig.u.neuReceiveOutSpec.targetInst    = receiveOutInst;
    neuConfig.code = neu_CTRL_RX_RECEIVE_OUT_CFG;
    (*gg_neu_ctable.neuControl) (inst->neuInst, &neuConfig); 
    /* Configure ACU to point to neu for sendout, if present */
    if (SIU_IS_PROTO_AAL2(inst))  {
#ifdef GG_INCLUDE_AAL2
      acuConfig.code                     = ACU_CONTROL_SENDOUT;
      acuConfig.u.SendOutSpec.acuSendOut = gg_neu_ctable.neuSendIn;
      acuConfig.u.SendOutSpec.targetInst = inst->neuInst;
      (*gg_acu_ctable.acuControl)(inst->net.acuInst, &acuConfig);
#endif
    } else if ( SIU_IS_PROTO_RTP(inst) ||
                (SIU_IS_PROTO_IU(inst) && SIU_IS_IU_OVER_IP(inst)) ) {

      rcuConfig.code                     = rcu_CTRL_SENDOUT;
      rcuConfig.u.txOut.rcuSendOut = gg_neu_ctable.neuSendIn;
      rcuConfig.u.txOut.targetInst = inst->neuInst;
      (*gg_rcu_ctable.rcuControl)(inst->net.rcuInst, &rcuConfig);
    }
    /* TFTP Send Out configured in tftpOpen */


  }
  else {
    nduConfig.RxOutSpec.nduReceiveOut = receiveOut;
    nduConfig.RxOutSpec.targetInst    = receiveOutInst;
  }
 
  /* Open NDU */

  siuAmLinkNdu(inst, nduConfig.RxOutSpec.nduReceiveOut, nduConfig.RxOutSpec.targetInst);
  SIU_RXINT_END();
} /* siu_open_network */

void siu_open_channel_block_codecs (siuInst_t *inst, tint vctype)
{
  bool block=FALSE;
  tint dummyv_mask = 0;

  switch (vctype) {
    /* ENCODERS */
    case EVCT_VIDEO_H264BP_ENC:
       dummyv_mask = GG_H264BP_ENC_MASK;
       break;
    case EVCT_VIDEO_MPEG4SP_ENC:
       dummyv_mask  = GG_MPEG4_ENC_MASK;
       break;
    case EVCT_VIDEO_H263P_ENC:
       dummyv_mask  = GG_H263_ENC_MASK;
       break;

    /* DECODERS */
    case EVCT_VIDEO_H264BP_MP_DEC:
       dummyv_mask  = GG_H264BP_MP_DEC_MASK;
       break;
    case EVCT_VIDEO_MPEG4SP_DEC:
       dummyv_mask  = GG_MPEG4_DEC_MASK;
       break;
    case EVCT_VIDEO_H263P_DEC:
       dummyv_mask  = GG_H263_DEC_MASK;
       break;
    case EVCT_VIDEO_MPEG2MP_DEC:
       dummyv_mask  = GG_MPEG2MP_DEC_MASK;
       break;
    case EVCT_VIDEO_H264HP_DEC:
       dummyv_mask  = GG_H264HP_DEC_MASK;
       break;

    /* BYPASS */
    case EVCT_VIDEO_BYPASS_ENC:
    case EVCT_VIDEO_BYPASS_DEC:
       dummyv_mask  = 0;
       break;

    default:
       break;
  }

  if(dummyv_mask) {
    if (!(gg_dummyv & dummyv_mask)) block = TRUE;  
    if (block) siu_exc_assert (0, SIUMGR_EXC_OPEN_CODEC | SIU_EXCEPTION_TYPE_CHANNEL, inst);    
  }
} /* siu_open_channel_block_codecs */

void siuDeleteVideoCh(tint chnum)
{
  siuInst_t *inst = SIUINSTp(chnum-SIU_FIRSTCH);

  if(inst->stateInst.video.vppuInst != NULL) {
  /* Delete VPPU */
  vppuDelete(&(inst->stateInst.video.vppuInst));
  }
  
  if(inst->stateInst.video.mmcuInst != NULL) {
    /* Delete MMCU */
    mmcuDelete(&(inst->stateInst.video.mmcuInst));
  }

  /* Delete VTK */
  vtkDelete(&(inst->stateInst.video.vtkInst));

  /* Delete TSU */
  tsuDelete(&(inst->stateInst.video.tsuInst));

  /* Delete OVLY */
  if ((SIU_GET_VIDEO_CHANNEL_EXT_TYPE(inst) == SIU_CHTYPE_EXT_OVLY_RAW) || 
      (SIU_GET_VIDEO_CHANNEL_EXT_TYPE(inst) == SIU_CHTYPE_EXT_OVLY_TTXT)){
     ovlyDelete(&(inst->stateInst.video.ovlyInst));
  }

  /* Delete customEco */
  customEcoDelete(&(inst->stateInst.video.customEcoInst));    
}

void siu_msm_video_to_closed (siuInst_t *inst, mgb_t *microMessage)
{
  tint i, chnum;
  siuVideoChannelOperation_t chType;
  siuInst_t *srcInst;
  tuint dstChID; 

  SIU_STATE_BEGIN();
  chnum  = (tint) (inst->ID & SIU_CID_MASK);
  chType = (siuVideoChannelOperation_t) SIU_GET_VIDEO_CHANNEL_TYPE(inst);

  /* siu_to_closed function code */
  inst->state         = SIU_CHST_CLOSED;   /* Set channel closed */
  inst->pending_state = SIU_CHST_CLOSED;

  siuShutdownNduAcu (inst);

  /* Reset heap Instance */
  inst->heapInst = NULL;
  siuContext.numActiveChannels--;  /* Decrement number of active channels */

  if(inst->stateInst.video.vppuInst != NULL) {
  /* close VPPU */
  vppuClose(inst->stateInst.video.vppuInst);
  }
  
  if(inst->stateInst.video.mmcuInst != NULL) {
    /* close MMCU */
    mmcuClose(inst->stateInst.video.mmcuInst);
  }

  /* close VTK */
  siuVideoRxFlushFrames(inst);
  vtkClose(inst->stateInst.video.vtkInst);

  /* close OVLY */
  if ((SIU_GET_VIDEO_CHANNEL_EXT_TYPE(inst) == SIU_CHTYPE_EXT_OVLY_RAW) ||
      (SIU_GET_VIDEO_CHANNEL_EXT_TYPE(inst) == SIU_CHTYPE_EXT_OVLY_TTXT)){
     ovlyClose(inst->stateInst.video.ovlyInst);
  }

  switch(chType) {
    case SIU_CHTYPE_ENCODER:
        /* close TSU */
        tsuClose(inst->stateInst.video.tsuInst);
        break;

    case SIU_CHTYPE_DECODER:
        /* close YUV */
        siu_close_yuv(inst);
        break;
  }

  if(SIU_GET_VIDEO_CHANNEL_EXT_TYPE(inst) == SIU_CHTYPE_EXT_TSU_ONLY) {
    /* close YUV */
    siu_close_yuv(inst);
  }

  /* close customEcoInst */
  customEcoClose(inst->stateInst.video.customEcoInst);

  siuDeleteVideoCh(chnum);

  /* close NEU */
  if (SIU_IS_NEU_PRESENT(inst)){    /* if NEU exists, delete it */
    (*gg_neu_ctable.neuClose)(inst->neuInst);
    (*gg_neu_ctable.neuDelete)(&inst->neuInst);
  }
  siu_queue_purge_channel (chnum, SIU_READ_TASK_PRIORITY_ID(inst));

  /* Remove all conference connections for which current channel is destination */
  for (i=0;i<siuContext.maxNumberChannels;i++) {
     srcInst = (siuInst_t *)SIUINSTp(i);
     if(srcInst->state == SIU_CHST_VIDEO) {
       dstChID = SIU_GET_VIDEO_ROUTE_DST_CH_ID_LD(srcInst);
       if(dstChID == (inst->ID & SIU_CID_MASK)){
         SIU_SET_VIDEO_ROUTE_DST_CH_ID_LD(srcInst, 0);
       }
       dstChID = SIU_GET_VIDEO_ROUTE_DST_CH_ID_PD(srcInst);
       if(dstChID == (inst->ID & SIU_CID_MASK)){
         SIU_SET_VIDEO_ROUTE_DST_CH_ID_PD(srcInst, 0);
       }
     }
  }

  /* Remove any core-to-core connections for this channel */
  for (i = 0; i < MAX_SIU_INTERCORE_CONN; i ++) {
    if (SIU_INTER_CORE_CONN_ENC_MATCH(i, siuContext.coreId, chnum-1)) {
      SIU_RESET_INTER_CORE_CONN_ENC(i);
      SIU_RESET_INTER_CORE_CONN_DEC(i);
    }
    if (SIU_INTER_CORE_CONN_DEC_MATCH(i, siuContext.coreId, chnum-1)) {
      SIU_RESET_INTER_CORE_CONN_ENC(i);
      SIU_RESET_INTER_CORE_CONN_DEC(i);
    }
  }
  /* Post event for closing remote MessageQ */
  siu_ipc_osal_event(EVENT_ID_MESSAGEQ_CLOSE);

  /* Free TSU scratch upon close channel. If TSU is needed for different channel,
     scratch will be re-allocated in the Tx thread again */
  if (siuContext.siu_video_tsu_scratch != NULL) {
     gmpFreeGmc(inst->ID,siuContext.siuYuvMemHandle,siuContext.siu_video_tsu_scratch);
     siuContext.siu_video_tsu_scratch      = NULL;
     siuContext.max_video_tsu_scratch_size = 0;
  }
 
  SIUINSTpSETCLOSED(inst);
  SIU_STATE_END();
  return;
}

/*********************************************************************************
 * FUNCTION PURPOSE: siuVigdkPurgeAllTasks
 *********************************************************************************
  DESCRIPTION:      siuVigdkPurgeAllTasks
  Parameters :      Inputs: None
                    Output: None 
 *********************************************************************************/
void siuVigdkPurgeAllTasks(void) 
{
    vppuCtrl_t vppuCtrl;    
    siuInst_t *inst;
    int i;
    vppuCtrl.code = VPPU_RESET_FIFO;    
    
    for(i=0;i<siuContext.maxNumberChannels;i++) {
      if(siu_is_valid_chn_video(i)) {
          inst = SIUINSTp(i);
          vppuControl (inst->stateInst.video.vppuInst, &vppuCtrl);
      }
    }
}

/*********************************************************************************
 * FUNCTION PURPOSE: siuGetErrorStats
 *********************************************************************************
  DESCRIPTION:      siuGetErrorStats
  Parameters :      Inputs: None
                    Output: None 
 *********************************************************************************/
void siuGetErrorStats (void *siuInst, bool reset, tuint trans_id, tuint priority)
{
  siuInst_t *inst = (siuInst_t *)siuInst;
  siuStats_t *siuStats;
  pktStats_t *pktStats;
  vppuStats_t vppuStats;
  vppuRxStats_t *vppuRxStats;
  mgb_t send_message;
  tulong txDrops, rxDrops, routingDrops, rxLostPackets;
  tulong rxSecErr = 0;
  tulong txSecErr = 0;
  siuVideoChannelOperation_t chType;

  siuStats = (siuStats_t *)inst->heapInst;
  pktStats = (pktStats_t *)&siuStats->pktStats;
  chType       = (siuVideoChannelOperation_t) SIU_GET_VIDEO_CHANNEL_TYPE(inst);

  send_message.header.length  = MGB_HEADER_SIZE + 
    (sizeof(struct MGB_DMS_GET_ERROR_STAT)) * MGB_MSGPAR_SIZE;
  send_message.header.chnum   = (int)(inst->ID & SIU_CID_MASK);
  send_message.header.msg_id  = MGB_DM_GET_ERROR_STAT;
  send_message.header.trans_id = trans_id;

  /* Initialize all the stats/reserved values to 0 */
  memset (&send_message.msg_body.error_stat, 0, 
          sizeof(send_message.msg_body.error_stat));

  rxDrops       = 0;
  rxLostPackets = 0;

  if(inst->stateInst.video.vppuInst) {

    vppuGetStats(inst->stateInst.video.vppuInst, reset, &vppuStats); 
    vppuRxStats = &(vppuStats.stats.rx);

    if (chType == SIU_CHTYPE_ENCODER) {
      rxDrops       = 0;
      rxLostPackets = 0;
      send_message.msg_body.error_stat.rxSeqNumDisc = 0;
    } else {
      rxDrops       = vppuRxStats->numPktsDropped;
      rxLostPackets = vppuRxStats->numLostPkts;
      send_message.msg_body.error_stat.rxSeqNumDisc = vppuRxStats->numOutofOrderPkts;
    }
  }
  txDrops = 0;
  routingDrops = 0;
 
  SIU_STATE_BEGIN();

  /* If RCU is present and running, replace/update RCU stats */
  if(SIU_IS_PROTO_RTP(inst)){
	 
     txDrops     += pktStats->proto.rtp.err->txBusy;
     routingDrops = pktStats->proto.rtp.err->noRoute;

     send_message.msg_body.error_stat.p2pTxSize = 
       pktStats->proto.rtp.err->txSize + pktStats->proto.rtp.err->txRedSize;
     rxSecErr = pktStats->proto.rtp.err->rxSecErr;     
     txSecErr = pktStats->proto.rtp.err->txSecErr;
     if (chType != SIU_CHTYPE_ENCODER) {
        send_message.msg_body.error_stat.rxSeqNumDisc = vppuRxStats->numOutofOrderPkts;
        rxLostPackets = vppuRxStats->numLostPkts;
        //rxDrops += pktStats->err->proto.rtp.rxDrops;
        /* Avoid returning negative numbers when sampling with no lost packets  *
         * and lots of out of order jitter                                      */
        if (rxLostPackets > (0UL -32UL))
           rxLostPackets = 0;
     } 

	 send_message.msg_body.error_stat.RxPktDropChnState_MSW = siu_get_umsw(pktStats->proto.rtp.err->RxPktDropChnState);
     send_message.msg_body.error_stat.RxPktDropChnState_LSW = siu_get_ulsw(pktStats->proto.rtp.err->RxPktDropChnState);
     send_message.msg_body.error_stat.RxOctDrop_MSW         = siu_get_umsw(pktStats->proto.rtp.err->RxOctDrop);
     send_message.msg_body.error_stat.RxOctDrop_LSW         = siu_get_ulsw(pktStats->proto.rtp.err->RxOctDrop);
     
     if(reset) {
       memset (pktStats->proto.rtp.err,  0, sizeof(*(pktStats->proto.rtp.err)));
       memset (&pktStats->proto.rtp.dual, 0, sizeof(pktStats->proto.rtp.dual));       
     }  

	 send_message.msg_body.error_stat.rxInvPayloadDiscard_MSW = 
                  siu_get_umsw(pktStats->proto.rtp.err->rxDrops);
     send_message.msg_body.error_stat.rxInvPayloadDiscard_LSW = 
                  siu_get_ulsw(pktStats->proto.rtp.err->rxDrops);
  
  
  } else if (SIU_IS_PROTO_AAL2(inst)){
    txDrops     += pktStats->proto.aal2.err->txBusy;
    routingDrops = pktStats->proto.aal2.err->noRoute;

	send_message.msg_body.error_stat.RxPktDropChnState_MSW = siu_get_umsw(pktStats->proto.aal2.err->RxPktDropChnState);
    send_message.msg_body.error_stat.RxPktDropChnState_LSW = siu_get_ulsw(pktStats->proto.aal2.err->RxPktDropChnState);
    send_message.msg_body.error_stat.RxOctDrop_MSW         = siu_get_umsw(pktStats->proto.aal2.err->RxOctDrop);
    send_message.msg_body.error_stat.RxOctDrop_LSW         = siu_get_ulsw(pktStats->proto.aal2.err->RxOctDrop);
    
    if(reset) {
      memset (pktStats->proto.aal2.err,  0, sizeof(*(pktStats->proto.aal2.err)));     
    }

  } else {
    rxSecErr = 0;
    txSecErr = 0;
  }

  SIU_STATE_END();       

  send_message.msg_body.error_stat.RxPktDrop_MSW          = siu_get_umsw(rxDrops);
  send_message.msg_body.error_stat.RxPktDrop_LSW          = siu_get_ulsw(rxDrops);
  send_message.msg_body.error_stat.TxPktDrop_MSW          = siu_get_umsw(txDrops);
  send_message.msg_body.error_stat.TxPktDrop_LSW          = siu_get_ulsw(txDrops);
  send_message.msg_body.error_stat.RxPktDrop_SecError_MSW = siu_get_umsw(rxSecErr);
  send_message.msg_body.error_stat.RxPktDrop_SecError_LSW = siu_get_ulsw(rxSecErr);
  send_message.msg_body.error_stat.TxPktDrop_SecError_MSW = siu_get_umsw(txSecErr);
  send_message.msg_body.error_stat.TxPktDrop_SecError_LSW = siu_get_ulsw(txSecErr);
  send_message.msg_body.error_stat.PacketsLostNtwk_MSW    = siu_get_umsw(rxLostPackets);
  send_message.msg_body.error_stat.PacketsLostNtwk_LSW    = siu_get_ulsw(rxLostPackets);     

  send_message.msg_body.error_stat.rxInvSSRCDiscard_MSW    = 0;
  send_message.msg_body.error_stat.rxInvSSRCDiscard_LSW    = 0;
  
  send_message.msg_body.error_stat.rxRouting_MSW = siu_get_umsw(routingDrops);
  send_message.msg_body.error_stat.rxRouting_LSW = siu_get_ulsw(routingDrops);
     
  /* send message to FIFO */
  siuNduSendMsgPriority (&send_message,(NDU_DPRMsgType_t)priority); 

}
/* nothing past this point */
