/******************************************************************************
 * FILE PURPOSE: Implementation File for Codec Memory Management
 ******************************************************************************
 * FILE NAME:   siuVigdkCodecMgr.c
 *
 * DESCRIPTION: This file contains the Implementation for Codec Memory Management
 *
 * FUNCTION           DESCRIPTION
 * --------           -----------
 *
 * (C) Copyright 2009, Texas Instruments Inc.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include "xdc/std.h"
#include "ti/xdais/ialg.h"
#include <stdlib.h>     /* malloc/free declarations */
#include <string.h>     /* memset declaration */
#include "siuVigdkGmp.h"
#include "siuVigdkCodecMgr.h"
#include "common/siuloc.h"

//#define SIUVIGDK_CODECMGR_PRINTF
#ifdef SIUVIGDK_CODECMGR_PRINTF
#include <stdio.h>
#endif

/*********************************************************************************
 * FUNCTION PURPOSE: Compute Alignment required for buffer
 *********************************************************************************
  DESCRIPTION:      This function is called to find the alignment bytes
  Parameters :      Inputs: address         : base address
                            alignment       : alignment requirement
                    Output: return value    : number of bytes offset needed to
                                              meet the alignment requirement
 *********************************************************************************/
tulong siuVigdkMemAlign(tulong address, tulong alignment)
{
  tulong alignBytes;

  alignBytes  = ((address  + (alignment - 1)) & ~ (alignment - 1));
  alignBytes -= address;
  return(alignBytes);
}

/*********************************************************************************
 * FUNCTION PURPOSE: Get the external resolution dependent & persistent buffer size
 *********************************************************************************
  DESCRIPTION:      This function is called to get the external resolution dependent
                    & persistent buffer size
  Parameters :      Inputs: memTab          : Memory Record with all the buffer details
                            n               : Number of buffers in Memory Record
                            resDepBitMap    : Resolution depended buffer Indices
                    Output: return value    : required external resolution dependent
                                              buffer size is bytes
 *********************************************************************************/
void siuVigdkMemGetExtSize(IALG_MemRec *memTab, tuint n, tulong *resDepBitMap,
                             tulong *extResdepPersistSize, tulong *extResindepPersistSize)
{
  tulong ext_resdep_persist_size = 0;
  tulong ext_resindep_persist_size = 0;
  tulong resDep, bfield;
  tuint i, numBit = sizeof(tulong)*TYP_WORD_SIZE;

  for (i=0;i<n;i++) {
    if (i < numBit) {
      bfield = (tulong)(resDepBitMap[0] >> i);
    } else {
      bfield = (tulong)(resDepBitMap[1] >> (i-numBit));
    }
    resDep = (bfield & 0x1);
    if(memTab[i].space == IALG_EXTERNAL) {
      if(memTab[i].attrs != IALG_SCRATCH) {
        if(resDep == 1) {
          ext_resdep_persist_size += memTab[i].size;
        } else {
          ext_resindep_persist_size += memTab[i].size;
        }
      }
    }
  }
  *extResdepPersistSize = ext_resdep_persist_size;
  *extResindepPersistSize = ext_resindep_persist_size;
}

/*********************************************************************************
 * FUNCTION PURPOSE: Allocate memory for one buffer
 *********************************************************************************
  DESCRIPTION:      This function allocates memory for one buffer. This is called
                    n times, where n is the number of buffers requested by the Codec
                    Based on the property/attribute of the buffer, memory is allocated
                    appropriately
  Parameters :      Inputs: inst            : siuInstance pointer
                            memory          : memory structure that contains all the
                                              heaps (Internal, External, Res-Dep,
                                              Res-Indep, scratch)
                            memTab          : Memory Record with all the buffer details
                            i               : Buffer Index
                    Output: return value    : success/failure
 *********************************************************************************/
Int siuVigdkAllocateMemTabRequest(siuInst_t *inst, siuCodecMemAlloc_t *memory, IALG_MemRec *memTab, Int i)
{
  tulong *resDepBitMap = memory->resDepBitMap;
  tulong bfield, resDep;
  tuint j, numBit = sizeof(tulong)*TYP_WORD_SIZE;

  if (i < numBit) {
      bfield = (tulong)(resDepBitMap[0] >> i);
  } else {
      bfield = (tulong)(resDepBitMap[1] >> (i-numBit));
  }
  resDep = (bfield & 0x1);

  if( memTab->space == IALG_EXTERNAL ) {
    unsigned int  alignBytes;
    if (memTab->attrs == IALG_SCRATCH) {
       alignBytes = siuVigdkMemAlign((tulong)((memory->External).scratch), memTab->alignment);
       ((memory->External).scratch)    += alignBytes;
       ((memory->External).scratchLen) -= alignBytes;
#ifdef SIUVIGDK_CODECMGR_PRINTF
       printf("Buff %d: EXT-SCRATCH Type 3,7:alignBytes=%d,size=%d\n",i,alignBytes,memTab->size);
#endif
       siu_exc_assert (((memory->External).scratchLen) >= memTab->size, SIUMGR_EXC_EXT_SCRATCH_OVERFLOW | SIU_EXCEPTION_TYPE_CHANNEL, inst);
       /* allocate memory */
       memTab->base = ((memory->External).scratch);
       ((memory->External).scratch)    += memTab->size;
       ((memory->External).scratchLen) -= memTab->size;
    }else if (resDep == 1) {
       for (j = 0; j < SIU_VIGDK_CODEC_MGR_NUM_RESDEP_HANDLES; j++) { /* There are multiple slots available for external resDep */
         /* Exception when there is no slot available */
         siu_exc_assert (((memory->External).resDep[j]) != NULL, SIUMGR_EXC_EXT_RESDEP_OVERFLOW | SIU_EXCEPTION_TYPE_CHANNEL, inst);
         /* Alignment */
         alignBytes = siuVigdkMemAlign((tulong)((memory->External).resDep[j]), memTab->alignment);
         ((memory->External).resDep[j])    += alignBytes;
         ((memory->External).resDepLen[j]) -= alignBytes;
         /* Allocate memory when there is enough space in the slot */
         if(((memory->External).resDepLen[j]) >= memTab->size) {
#ifdef SIUVIGDK_CODECMGR_PRINTF
           printf("Buff %d: EXT-RESDEP Type 8:alignBytes=%d,size=%d\n",i,alignBytes,memTab->size);
#endif
           memTab->base = ((memory->External).resDep[j]);
           ((memory->External).resDep[j])    += memTab->size;
           ((memory->External).resDepLen[j]) -= memTab->size;
           break;
         }
       }
       /* Exception if allocation fails because of insufficient space */
       siu_exc_assert (j < SIU_VIGDK_CODEC_MGR_NUM_RESDEP_HANDLES, SIUMGR_EXC_EXT_RESDEP_OVERFLOW | SIU_EXCEPTION_TYPE_CHANNEL, inst);
    } else {
       for (j = 0; j < SIU_VIGDK_CODEC_MGR_NUM_RESINDEP_HANDLES; j++) {
         /* Exception when there is no slot available */
         siu_exc_assert (((memory->External).resIndep[j]) != NULL, SIUMGR_EXC_EXT_RESINDEP_OVERFLOW | SIU_EXCEPTION_TYPE_CHANNEL, inst);
         /* Alignment */
         alignBytes = siuVigdkMemAlign((tulong)((memory->External).resIndep[j]), memTab->alignment);
         ((memory->External).resIndep[j])    += alignBytes;
         ((memory->External).resIndepLen[j]) -= alignBytes;

         /* Allocate memory when there is enough space in the slot */
         if(((memory->External).resIndepLen[j]) >= memTab->size) {
#ifdef SIUVIGDK_CODECMGR_PRINTF
           printf("Buff %d: EXT-RESINDEP Type 4:alignBytes=%d,size=%d\n",i,alignBytes,memTab->size);
#endif
           memTab->base = ((memory->External).resIndep[j]);
           ((memory->External).resIndep[j])    += memTab->size;
           ((memory->External).resIndepLen[j]) -= memTab->size;
           break;
         }
       }
       /* Exception if allocation fails because of insufficient space */
       siu_exc_assert (j < SIU_VIGDK_CODEC_MGR_NUM_RESINDEP_HANDLES, SIUMGR_EXC_EXT_RESINDEP_OVERFLOW | SIU_EXCEPTION_TYPE_CHANNEL, inst);
    }
  } else {
	/* internal memory request */
	unsigned int  alignBytes;
	alignBytes = (((unsigned int) ((memory->Internal).scratch) + (memTab->alignment - 1)) & ~ (memTab->alignment - 1));
	if(alignBytes) {
	  alignBytes -= (unsigned int)((memory->Internal).scratch);
	  ((memory->Internal).scratch)    += alignBytes;
	  ((memory->Internal).scratchLen) -= alignBytes;
	}
#ifdef SIUVIGDK_CODECMGR_PRINTF
    printf("Buff %d: INT-SCRATCH Type 1,5:alignBytes=%d,size=%d\n",i,alignBytes,memTab->size);
#endif
    siu_exc_assert (((memory->Internal).scratchLen) >= memTab->size, SIUMGR_EXC_INT_SCRATCH_OVERFLOW | SIU_EXCEPTION_TYPE_CHANNEL, inst);
    /* allocate memory */
    memTab->base = ((memory->Internal).scratch);
    ((memory->Internal).scratch)    += memTab->size;
    ((memory->Internal).scratchLen) -= memTab->size;
  }
  return 0;
} /* siuVigdkAllocateMemTabRequest */

/*********************************************************************************
 * FUNCTION PURPOSE: Allocate memory for one buffer
 *********************************************************************************
  DESCRIPTION:      This function computes the memory requirement of the codec and
                    allocates memory from the appropriate Global Pool that fullfills
                    the codec requirement. Once memory from global pool(s) is
                    allocated, memory is assigned to the codec buffers by calling
                    other functions, which treat the global memory as heap
  Parameters :      Inputs: siuInst         : siuInstance pointer
                            memTab          : Memory Record with all the buffer details
                            n               : Number of buffers requested by the Codec
                            resDepBitMap    : Bitmap indicated whether size of buffer
                                              depends on resolution or not
                    Output: return value    : success/failure
 *********************************************************************************/
Bool siuVigdkCodecAllocMemory(void *siuInst, IALG_MemRec *memTab, Int n, tulong *resDepBitMap)
{
  Int i;
  siuInst_t *inst = (siuInst_t *)siuInst;
  siuCodecMemAlloc_t memory;
  siuCodecGmpSlots_t *codecSlots = &(inst->codecSlots);
  siuVideoChannelOperation_t chType;
  tulong resDepSize[SIU_VIGDK_CODEC_MGR_NUM_RESDEP_HANDLES], resIndepSize[SIU_VIGDK_CODEC_MGR_NUM_RESINDEP_HANDLES];
  tlong delta_persist_size;
  tulong ext_resdep_persist_size, ext_resindep_persist_size;
  siuCodecMemoryHandle_t *codecMemHandle = &(siuContext.codecMemHandle);
  extern word   *gg_vtk_scratch_area_start;
  extern tulong gg_vtk_scratch_size;
  tuint decResDepSecIndex = SIU_VIGDK_CODEC_MGR_NUM_RESDEP_HANDLES - 1;

  chType                  = (siuVideoChannelOperation_t) SIU_GET_VIDEO_CHANNEL_TYPE(inst);
  siuVigdkMemGetExtSize(memTab,n,resDepBitMap,&ext_resdep_persist_size, &ext_resindep_persist_size);

  /* Since all the buffers for audio codecs are declared as resolution independent, 
     use ext_resdep_persist_size to differentiate audio and video codecs */
  if(ext_resdep_persist_size == 0) { /* Audio codecs */
      if (chType == SIU_CHTYPE_ENCODER) {
        codecSlots->resindep[0]       = (word *)(gmpAllocGmc(inst->ID,codecMemHandle->encAudioHandle));
        codecSlots->resIndepHandle[0] = codecMemHandle->encAudioHandle;
        resIndepSize[0]               = codecMemHandle->encAudioSize;
        siu_exc_assert ((codecSlots->resindep[0] != NULL), SIUMGR_EXC_GMP_SLOT_ALLOC_FAIL, inst);
      } else {
        codecSlots->resindep[0]       = (word *)(gmpAllocGmc(inst->ID,codecMemHandle->decAudioHandle));
        codecSlots->resIndepHandle[0] = codecMemHandle->decAudioHandle;
        resIndepSize[0]               = codecMemHandle->decAudioSize;
        siu_exc_assert ((codecSlots->resindep[0] != NULL), SIUMGR_EXC_GMP_SLOT_ALLOC_FAIL, inst);
      }
     /* Audio never has additional resolution independent slots. So, buff=NULL & size=0 */
     for (i = 1; i < SIU_VIGDK_CODEC_MGR_NUM_RESINDEP_HANDLES; i++) {
       codecSlots->resindep[i]         = NULL;
       codecSlots->resIndepHandle[i]   = NULL;
       resIndepSize[i]                 = 0;
     }
     /* Audio never has resolution dependent slots. So, buff=NULL & size=0 */
     for (i = 0; i < SIU_VIGDK_CODEC_MGR_NUM_RESDEP_HANDLES; i++) {
       codecSlots->resdep[i]         = NULL;
       codecSlots->resDepHandle[i]   = NULL;
       resDepSize[i]                 = 0;
     }
  } else { /* Video codecs */
      if (chType == SIU_CHTYPE_ENCODER) {
         /* Encoder res-indep: allocate from the pool; multiple slots as defined by SIU_VIGDK_CODEC_MGR_NUM_RESINDEP_HANDLES can be used */
         delta_persist_size = ext_resindep_persist_size;
         for (i = 0; i < SIU_VIGDK_CODEC_MGR_NUM_RESINDEP_HANDLES; i++) {
           if (delta_persist_size > 0) {
             codecSlots->resindep[i]        = (word *)(gmpAllocGmc(inst->ID,codecMemHandle->encResIndepHandle[i]));
             codecSlots->resIndepHandle[i]  = codecMemHandle->encResIndepHandle[i];
             resIndepSize[i]                = codecMemHandle->encResIndepSize[i];
             siu_exc_assert ((codecSlots->resindep[i] != NULL), SIUMGR_EXC_GMP_SLOT_ALLOC_FAIL, inst);
             delta_persist_size -= codecMemHandle->encResIndepSize[i];
           } else {
             codecSlots->resindep[i]       = NULL;
             codecSlots->resIndepHandle[i] = NULL;
             resIndepSize[i]               = 0;
           }
         }

         /* Encoder res-dep: see if the buffers fit in Encoder pool */
         if (ext_resdep_persist_size < codecMemHandle->encResDepSize) {
            /* Encoder res-dep buffers fit in Encoder Pool */
            codecSlots->resdep[0]       = (word *)(gmpAllocGmc(inst->ID,codecMemHandle->encResDepHandle));
            codecSlots->resDepHandle[0] = codecMemHandle->encResDepHandle;
            resDepSize[0]               = codecMemHandle->encResDepSize;
         } else {
            /* Encoder res-dep buffers didn't fit in Encoder Pool. So, allocate from decoder res-dep pool */
            codecSlots->resdep[0]       = (word *)(gmpAllocGmc(inst->ID,codecMemHandle->decResDepHandle[0]));
            codecSlots->resDepHandle[0] = codecMemHandle->decResDepHandle[0];
            resDepSize[0]               = codecMemHandle->decResDepSize[0];
         }

         siu_exc_assert ((codecSlots->resdep[0] != NULL), SIUMGR_EXC_GMP_SLOT_ALLOC_FAIL, inst);

         /* Encoder res-dep never have additional slots. So, buff=NULL & size=0 */
         for (i = 1; i < SIU_VIGDK_CODEC_MGR_NUM_RESDEP_HANDLES; i++) {
           codecSlots->resdep[i]         = NULL;
           codecSlots->resDepHandle[i]   = NULL;
           resDepSize[i]                 = 0;
         }
      } else {
         /* Decoder res-indep: allocate from the pool; multiple slots as defined by SIU_VIGDK_CODEC_MGR_NUM_RESINDEP_HANDLES can be used */
         delta_persist_size = ext_resindep_persist_size;
         for (i = 0; i < SIU_VIGDK_CODEC_MGR_NUM_RESINDEP_HANDLES; i++) {
           if (delta_persist_size > 0) {
             codecSlots->resindep[i]        = (word *)(gmpAllocGmc(inst->ID,codecMemHandle->decResIndepHandle[i]));
             codecSlots->resIndepHandle[i]  = codecMemHandle->decResIndepHandle[i];
             resIndepSize[i]                = codecMemHandle->decResIndepSize[i];
             siu_exc_assert ((codecSlots->resindep[i] != NULL), SIUMGR_EXC_GMP_SLOT_ALLOC_FAIL, inst);
             delta_persist_size -= codecMemHandle->decResIndepSize[i];
           } else {
             codecSlots->resindep[i]       = NULL;
             codecSlots->resIndepHandle[i] = NULL;
             resIndepSize[i]               = 0;
           }
         }

         /* Decoder res-dep: allocate primary & secondary slots */
         /* multiple primary slots as defined by SIU_VIGDK_CODEC_MGR_NUM_RESDEP_HANDLES-1 can be used */
         delta_persist_size = ext_resdep_persist_size;
         for (i = 0; i < decResDepSecIndex; i++) {
           if (delta_persist_size > 0) {
             codecSlots->resdep[i]        = (word *)(gmpAllocGmc(inst->ID,codecMemHandle->decResDepHandle[i]));
             codecSlots->resDepHandle[i]  = codecMemHandle->decResDepHandle[i];
             resDepSize[i]                = codecMemHandle->decResDepSize[i];
             siu_exc_assert ((codecSlots->resdep[i] != NULL), SIUMGR_EXC_GMP_SLOT_ALLOC_FAIL, inst);
             delta_persist_size -= codecMemHandle->decResDepSize[i];
           } else {
             codecSlots->resdep[i]       = NULL;
             codecSlots->resDepHandle[i] = NULL;
             resDepSize[i]               = 0;
           }
         }

         /* If delta_persist_size > 0, secondary slot is needed (the buffers would overflow (SIU_VIGDK_CODEC_MGR_NUM_RESDEP_HANDLES - 1) slots) */
         if (delta_persist_size > 0) {
           if (delta_persist_size < codecMemHandle->decResDepSize[0]) {
              /* pick another QCIF slot */
              codecSlots->resdep[decResDepSecIndex]       = (word *)(gmpAllocGmc(inst->ID,codecMemHandle->decResDepHandle[0]));
              codecSlots->resDepHandle[decResDepSecIndex] = codecMemHandle->decResDepHandle[0];
              resDepSize[decResDepSecIndex]               = codecMemHandle->decResDepSize[0];
              siu_exc_assert ((codecSlots->resdep[decResDepSecIndex] != NULL), SIUMGR_EXC_GMP_SLOT_ALLOC_FAIL, inst);
           } else if (delta_persist_size < codecMemHandle->decResDepSize[decResDepSecIndex]) {
              /* pick CIF slot */
              codecSlots->resdep[decResDepSecIndex]       = (word *)(gmpAllocGmc(inst->ID,codecMemHandle->decResDepHandle[decResDepSecIndex]));
              codecSlots->resDepHandle[decResDepSecIndex] = codecMemHandle->decResDepHandle[decResDepSecIndex];
              resDepSize[decResDepSecIndex]               = codecMemHandle->decResDepSize[decResDepSecIndex];
              siu_exc_assert ((codecSlots->resdep[decResDepSecIndex] != NULL), SIUMGR_EXC_GMP_SLOT_ALLOC_FAIL, inst);
           } else {
              /* Exception */
              siu_exc_assert(0, SIUMGR_EXC_GMP_SLOT_ALLOC_FAIL, inst);
           }
         } else {
           codecSlots->resdep[decResDepSecIndex]       = NULL;
           codecSlots->resDepHandle[decResDepSecIndex] = NULL;
           resDepSize[decResDepSecIndex]               = 0;
         }
      }
  }

  for (i = 0; i < SIU_VIGDK_CODEC_MGR_NUM_RESDEP_HANDLES; i++) {
    /* Internal - persistent */
    memory.Internal.resDep[i]       = NULL;
    memory.Internal.resDepLen[i]    = 0;
    /* External - persistent */
    memory.External.resDep[i]       = (word *)codecSlots->resdep[i];
    memory.External.resDepLen[i]    = resDepSize[i];
  }
  for (i = 0; i < SIU_VIGDK_CODEC_MGR_NUM_RESINDEP_HANDLES; i++) {
    /* Internal - persistent */
    memory.Internal.resIndep[i]     = NULL;
    memory.Internal.resIndepLen[i]  = 0;
    /* External - persistent */
    memory.External.resIndep[i]     = (word *)codecSlots->resindep[i];
    memory.External.resIndepLen[i]  = resIndepSize[i];
  }
  /* Internal - scratch */
  memory.Internal.scratch         = gg_vtk_scratch_area_start;
  memory.Internal.scratchLen      = gg_vtk_scratch_size;

  /* External - scratch */
  memory.External.scratch         = codecMemHandle->scratch;
  memory.External.scratchLen      = codecMemHandle->scratchSize;
  memory.resDepBitMap             = resDepBitMap;

  for (i = 0; i < n; i++) {
     siuVigdkAllocateMemTabRequest(inst, &memory, &memTab[i], i);
     if (memTab[i].base == NULL) {
        return (FALSE);
     }
  }
  return (TRUE);
}

/*********************************************************************************
 * FUNCTION PURPOSE: Public API, called from VTK to allocate memory for Codec
 *********************************************************************************
  DESCRIPTION:      This function is called from VTK to allocate memory for the Codec
  Parameters :      Inputs: siuInst         : siuInstance pointer
                            fxns            : Codec initialization functions
                            p               : Codec Handle
                            params          : Static/Dynamic Params needed for Codec
                                              initialization
                            resDepBitMap    : Bitmap indicated whether size of buffer
                                              depends on resolution or not
                    Output: return value    : Codec Handle
 *********************************************************************************/
IALG_Handle siuVigdkCodecAlgCreate(void *siuInst, IALG_Fxns *fxns, IALG_Handle p, IALG_Params *params, tulong *resDepBitMap)
{
  IALG_MemRec tmp_memTab[SIU_VIGDK_CODEC_MAX_NUM_ALLOC_BUFFER];
  IALG_MemRec *memTab;
  Int n;
  IALG_Handle alg;
  IALG_Fxns *fxnsPtr;
  siuInst_t *inst = (siuInst_t *)siuInst;

  if (fxns != NULL) {
     n = fxns->algNumAlloc != NULL ? fxns->algNumAlloc() : IALG_DEFMEMRECS;
     memTab = tmp_memTab;
     n = fxns->algAlloc(params, &fxnsPtr, memTab);
     if ((n <= 0) || (n > SIU_VIGDK_CODEC_MAX_NUM_ALLOC_BUFFER)) {
        return (NULL);
     }

     if (siuVigdkCodecAllocMemory(inst, memTab, n, resDepBitMap)) {
        alg = (IALG_Handle)memTab[0].base;
        alg->fxns = fxns;
        if (fxns->algInit(alg, memTab, p, params) == IALG_EOK) {
           return (alg);
        }
          fxns->algFree(alg, memTab);
        }
     }
    return (NULL);
}

/*********************************************************************************
 * FUNCTION PURPOSE: Public API, called from VTK to free Codec memory
 *********************************************************************************
  DESCRIPTION:      This function is called from VTK to allocate memory for the Codec
  Parameters :      Inputs: siuInst         : siuInstance pointer
                    Output: return value    : None
 *********************************************************************************/
void siuVigdkCodecAlgFree(void *siuInst)
{
  siuInst_t *inst = (siuInst_t *)siuInst;
  siuCodecGmpSlots_t *codecSlots = &(inst->codecSlots);
  tuint i;

  for (i=0; i<SIU_VIGDK_CODEC_MGR_NUM_RESDEP_HANDLES; i++) {
    if (codecSlots->resdep[i] != NULL) {
      gmpFreeGmc(inst->ID, codecSlots->resDepHandle[i], codecSlots->resdep[i]);
    }
    codecSlots->resdep[i] = NULL;
  }
  for (i=0; i<SIU_VIGDK_CODEC_MGR_NUM_RESINDEP_HANDLES; i++) {
    if (codecSlots->resindep[i] != NULL) {
      gmpFreeGmc(inst->ID, codecSlots->resIndepHandle[i], codecSlots->resindep[i]);
    }
    codecSlots->resindep[i] = NULL;
  }
  return;
}

/* nothing past this point */
