#ifndef _SIUVIGDK_CODECMGR_H
#define _SIUVIGDK_CODECMGR_H
/******************************************************************************
 * FILE PURPOSE: Interface File for Codec Memory Management
 ******************************************************************************
 * FILE NAME:   siuVigdkCodecMgr.h  
 *
 * DESCRIPTION: This file contains the interface for Codec Memory Management
 *              
 * FUNCTION           DESCRIPTION
 * --------           ----------- 
 *
 * (C) Copyright 2009, Texas Instruments Inc. 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include "xdc/std.h"
#include "ti/xdais/ialg.h"
#include "types.h"

#define SIU_VIGDK_CODEC_MGR_NUM_RESDEP_HANDLES   4
#define SIU_VIGDK_CODEC_MGR_NUM_RESINDEP_HANDLES 1

#define SIU_VIGDK_CODEC_MAX_NUM_ALLOC_BUFFER 64

/*********************************************************************************
 * STRUCTURE PURPOSE: Store the GMP Pointer & Handles for the Allocated Codec slots
 *********************************************************************************
  DESCRIPTION: siuCodecGmpSlots_s is part of siuInstance
 *********************************************************************************/
typedef struct siuCodecGmpSlots_s {
  /* gmp pointer to resolution independent slot */
  word *resindep[SIU_VIGDK_CODEC_MGR_NUM_RESINDEP_HANDLES];       
  /* handle to the GMP pool from where resindep is allocated */
  word *resIndepHandle[SIU_VIGDK_CODEC_MGR_NUM_RESINDEP_HANDLES];
  /* gmp pointer to resolution independent slot(s)*/
  word *resdep[SIU_VIGDK_CODEC_MGR_NUM_RESDEP_HANDLES]; 
  /* handle(s) to the GMP pool(s) from where resdep slot(s) is(are) allocated */
  word *resDepHandle[SIU_VIGDK_CODEC_MGR_NUM_RESDEP_HANDLES];
} siuCodecGmpSlots_t;

/*********************************************************************************
 * STRUCTURE PURPOSE: Store the GMP Handles for the avaialbe Codec slots
 *********************************************************************************
  DESCRIPTION: siuCodecGmpSlots_s is part of siuContext, updated in vigdkInit()
 *********************************************************************************/
typedef struct siuCodecMemoryHandle_s {
  /* handle to the Encoder resolution Independent GMP pool */
  word *encResIndepHandle[SIU_VIGDK_CODEC_MGR_NUM_RESINDEP_HANDLES];
  /* size of memory chunk in encResIndep Pool */
  tulong encResIndepSize[SIU_VIGDK_CODEC_MGR_NUM_RESINDEP_HANDLES];
  /* handle to the Encoder resolution dependent GMP pool   */
  word *encResDepHandle;
  /* size of memory chunk in encResDep Pool */
  tulong encResDepSize;
  /* handle to the Decoder resolution Independent GMP pool */
  word *decResIndepHandle[SIU_VIGDK_CODEC_MGR_NUM_RESINDEP_HANDLES];
  /* size of memory chunk in decResIndep Pool */
  tulong decResIndepSize[SIU_VIGDK_CODEC_MGR_NUM_RESINDEP_HANDLES];
  /* handles to the Decoder resolution dependent GMP pools */
  word *decResDepHandle[SIU_VIGDK_CODEC_MGR_NUM_RESDEP_HANDLES];
  /* size of memory chunk in decResDep Pool */
  tulong decResDepSize[SIU_VIGDK_CODEC_MGR_NUM_RESDEP_HANDLES];
  /* handle to the audio decoder GMP pool   */
  word *decAudioHandle;
  /* size of memory chunk in audio decoder Pool */
  tulong decAudioSize;
  /* handle to the audio encoder GMP pool   */
  word *encAudioHandle;
  /* size of memory chunk in audio encoder Pool */
  tulong encAudioSize;
  /* handle to the codec Scratch GMP pool                  */
  word *scratch;
  /* size of scratch */
  tulong scratchSize;
} siuCodecMemoryHandle_t;


/*********************************************************************************
 * STRUCTURE PURPOSE: Temporary Stack structure
 *********************************************************************************
  DESCRIPTION: siuCodecMem_s is a temporary stack object used for book keeping
               while creating the codec. Memory is allocated from the GMP pools
               and that allocated memory is used as heap to allocate the buffers
               requested by the Codec
 *********************************************************************************/
typedef struct siuCodecMem_s {
  /* pointer to the resolution independent base address */
  word   *resIndep[SIU_VIGDK_CODEC_MGR_NUM_RESINDEP_HANDLES];
  /* length of the resolution independent buffer */
  tulong  resIndepLen[SIU_VIGDK_CODEC_MGR_NUM_RESINDEP_HANDLES];
  /* pointers to the resolution dependent base addresses */
  word   *resDep[SIU_VIGDK_CODEC_MGR_NUM_RESDEP_HANDLES];
  /* lengths of the resolution dependent buffers */
  tulong  resDepLen[SIU_VIGDK_CODEC_MGR_NUM_RESDEP_HANDLES];
  /* pointer to the system scratch */
  word   *scratch;
  /* length of system scratch */
  tulong  scratchLen;
} siuCodecMem_t;

/*********************************************************************************
 * STRUCTURE PURPOSE: Temporary Stack structure
 *********************************************************************************
  DESCRIPTION: siuCodecMemAlloc_s is a temporary stack object used for book keeping
               while creating the codec. Memory is allocated from the GMP pools
               and that allocated memory is used as heap to allocate the buffers
               requested by the Codec. There is one copy for internal and another
               copy for external buffer pool/addresses
 *********************************************************************************/
typedef struct siuCodecMemAlloc_s {
  /* Internal memory buffers */
  siuCodecMem_t Internal;
  /* External memory buffers */
  siuCodecMem_t External;
  /* Resolution Dependency bit-map. Information supplied by the Codec */
  tulong        *resDepBitMap;
} siuCodecMemAlloc_t; 

/* Private Functions */
Bool siuVigdkCodecAllocMemory(void *siuInst, IALG_MemRec *memTab, Int n, tulong *resDepBitMap);

/* Public Functions */
IALG_Handle siuVigdkCodecAlgCreate(void *siuInst, IALG_Fxns *fxns, IALG_Handle p, 
                                   IALG_Params *params, tulong *resDepBitMap);

void siuVigdkCodecAlgFree(void *siuInst);

#endif

/* nothing past this point */
