/******************************************************************************
 * FILE PURPOSE: SIU Framework Routines for VIDEO
 ******************************************************************************
 * FILE NAME:   siuVigdkFRC.c
 *
 * DESCRIPTION: Includes Frame Rate Control for Vigdk Application
 *
 * TABS: NONE
 *
 * (C) Copyright 2009, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include "common/siuloc.h"
#include "mgmicdsp.h"
#include "ggVideoCfg.h"

/* Discontinuity of 1.0 seconds */
#define SIU_FRC_SUDDEN_JUMP (90000)
/* Any frame is not allowed to be repeated more than 2 times */
#define SIU_FRC_MAX_RPT_CNT 2
/*********************************************************************************
 * FUNCTION PURPOSE: Configures the output frame rate of FRC instance
 *********************************************************************************
  DESCRIPTION:      This function is called to configure FRC instance
  Parameters :      Inputs: siuFrcInst_t    : FRC instance
                            outputFrameRate : FRC output frame Rate (encoder input frame rate)
                    Output: None
 *********************************************************************************/
void siu_frc_configure_frameRate(siuFrcInst_t *frcInst, tulong outputFrameRate)
{
  frcInst->outputFrameRate     = (tuint)(outputFrameRate/1000);
  frcInst->prevOutputFrameRate = (tuint)(frcInst->outputFrameRate);
  return; 
}
/*********************************************************************************
 * FUNCTION PURPOSE: Estimate input frame rate
 *********************************************************************************
  DESCRIPTION:      This function is called to estimate input frame rate. It is
                    calculated as median RTP TS increment from last SIU_FRC_FRAME_TO_ESTIMATE
                    frames
  Parameters :      Inputs: siuFrcInst_t    : frc Instance
 *********************************************************************************/
tuint siu_frc_estimateInputFrameRate(siuFrcInst_t *frcInst, tulong rtpTimeStampIncrement)
{
  tuint j, k, l;
  tuint tmpVal, tmpRtpTsIncr[SIU_FRC_FRAME_TO_ESTIMATE];
  tuint currTsIncr = (tuint)rtpTimeStampIncrement;

  /* Initially vector of TS increments is set to 0.          */
  /* Have last SIU_FRC_FRAME_TO_ESTIMATE timestamps recorded */
  for (j = (SIU_FRC_FRAME_TO_ESTIMATE - 1); j > 0; j --) {
     frcInst->rtpTsIncr[j] = frcInst->rtpTsIncr[j - 1];
  }
  frcInst->rtpTsIncr[0] = currTsIncr;

  if(frcInst->initialRtpTsCount <= SIU_FRC_FRAME_TO_ESTIMATE) {
    frcInst->initialRtpTsCount ++;
    /* For the initial SIU_FRC_FRAME_TO_ESTIMATE frames, report */
    /* frame rate using most recent timestamp increment         */
    return (tuint)((tulong)90000 / currTsIncr);
  }

  /* Take the median value as input frame rate estimate, after few frames */
  /* Create temporary copy of TS increments : needed for sorting */
  for(k = 0; k < SIU_FRC_FRAME_TO_ESTIMATE; k ++) tmpRtpTsIncr[k] = frcInst->rtpTsIncr[k];
  /* Sort timestamp increments (min-to-max) */ 
  for(k = 0; k < SIU_FRC_FRAME_TO_ESTIMATE; k ++) 
  {
    tmpVal = tmpRtpTsIncr[k];
    l      = k+1;
    for(j = (k + 1); j < SIU_FRC_FRAME_TO_ESTIMATE; j ++) 
    { /* Find index and minimum value of TS increments */
      if(tmpVal > tmpRtpTsIncr[j])
      {
         tmpVal = tmpRtpTsIncr[j];
         l      = j;
      }
    }
    if(l != (k + 1))
    { /* Swap minimum value with the current location */
      tmpVal = tmpRtpTsIncr[l];
      tmpRtpTsIncr[k] = tmpRtpTsIncr[l];
      tmpRtpTsIncr[l] = tmpVal;
    }
  }
  return (tuint)((tulong)90000 / tmpRtpTsIncr[SIU_FRC_FRAME_TO_ESTIMATE/2]);
}
/*********************************************************************************
 * FUNCTION PURPOSE: Run Frame Rate Control
 *********************************************************************************
  DESCRIPTION:      This function is called to run frc
  Parameters :      Inputs: siuInst_t       : channel SIU Instance
                            rtpTimeStamp    : RTP timestamp of incoming Frame
                    Output: rtpTimeStampIncrement: Instantenous RTP timestamp increment
                            retVal          : 0 for frame drop, =n(n>0) for repeats
 *********************************************************************************/
tuint siu_run_frc(void *siuInst, tulong rtpTimeStamp, tulong *rtpTimeStampIncrement)
{
  siuInst_t *inst = (siuInst_t *)siuInst;
  siuFrcInst_t *frcInst = (siuFrcInst_t *)&inst->frcInst;
  tuint retVal=0; /* 0 for frame drop, =n(n>0) for repeats */
  tuint inputFrameRate, outputFrameRate;
  tulong outputRtpTsIncr, inputRtpTsIncr;
  vtkCtrl_t vtkCtrl;

  if(frcInst->initialRtpTsCount == 0)
  { /* First frame - timestamp of transcodied frame will be one provided by */
    /*  DIM API using ENCAP_CONFIG message */
    *rtpTimeStampIncrement     = 0xFFFFFFFF;
    frcInst->initialRtpTsCount = 1;
    frcInst->prevRtpTsIn       = rtpTimeStamp;
    return 1;
  }
  inputRtpTsIncr         = rtpTimeStamp - frcInst->prevRtpTsIn;
  frcInst->prevRtpTsIn   = rtpTimeStamp;
  *rtpTimeStampIncrement = rtpTimeStamp - frcInst->prevRtpTsOut;
  if(frcInst->mode == MGB_MDS_VIDEO_FRC_CFG_MODE_DISABLE) return 1;

  /******************************************************************/
  /**** Decision logic for FRC frame drop, insertion, starts here ***/
  /******************************************************************/
  outputFrameRate = frcInst->outputFrameRate;

  /* Detect input RTP timestamp discontinuity */
  if ((inputRtpTsIncr == 0) || 
      ((inputRtpTsIncr > SIU_FRC_SUDDEN_JUMP) && (inputRtpTsIncr < (0xFFFFFFFF - SIU_FRC_SUDDEN_JUMP))) )
  {
      frcInst->numFramesUsed++;
      frcInst->initialRtpTsCount = 1;
      frcInst->nextTxFrameRtpTs  = rtpTimeStamp + (tulong)((tulong)90000 / outputFrameRate);
      *rtpTimeStampIncrement = SIU_FRC_SUDDEN_JUMP;
      return 1;
  }

  inputFrameRate  = siu_frc_estimateInputFrameRate(frcInst, inputRtpTsIncr);
  /******************************************************************/
  if(frcInst->mode == MGB_MDS_VIDEO_FRC_CFG_MODE_RTP_TS)
  { /* This indicates to the rest of framework that TX RTP TS increment is driven */
    /* by encoder configured target frame rate - e.g. for file source streaming */
    *rtpTimeStampIncrement = 0;
  } else {
    /* Calc output frame rate wrt input frame rate */
    if(inputFrameRate >= frcInst->outputFrameRate) 
    {/* use configured outputFrameRate for encoder output frame rate */ 
      outputFrameRate = frcInst->outputFrameRate;
    } else { /* use inputFrameRate (decoder frame rate) as encoder output frame rate */
      outputFrameRate = inputFrameRate;
    }
    if(frcInst->mode == MGB_MDS_VIDEO_FRC_CFG_MODE_TRACK_INPUT) {
      if(frcInst->prevOutputFrameRate != outputFrameRate)
      { /* Encoder must be notified about the change in frame rate. In this mode, FRC does the frame dropping */
        vtkCtrl.code = VTK_SET_ENC_DYN_CFG;
        vtkCtrl.u.vtkEncDynCfg.targetFrameRate = vtkCtrl.u.vtkEncDynCfg.refFrameRate = (outputFrameRate * 1000);
        gg_vtk_ctable.vtkControl(inst->stateInst.video.vtkInst, &vtkCtrl);
      }
      frcInst->prevOutputFrameRate = outputFrameRate;
      /* FRC will do the frame skipping */
    } else return 1;
  }
  /******************************************************************/
  outputRtpTsIncr = (tulong)((tulong)90000 / outputFrameRate);

  if (inputFrameRate >= outputFrameRate) {
    if (rtpTimeStamp < frcInst->nextTxFrameRtpTs) {
      frcInst->numFramesDropped++;
      retVal = 0;
    } else {
      frcInst->numFramesUsed++;
      retVal = 1;
      do {
        frcInst->nextTxFrameRtpTs += outputRtpTsIncr;
      } while (frcInst->nextTxFrameRtpTs < rtpTimeStamp);
    }
  } else { /* inputFrameRate < outputFrameRate */
    if (rtpTimeStamp > frcInst->nextTxFrameRtpTs) {
      do {
        frcInst->nextTxFrameRtpTs += outputRtpTsIncr;
        if (retVal == 0) {
          frcInst->numFramesUsed++;
        } else {
          frcInst->numFramesRepeated++;
        }
        retVal++;
      }while (rtpTimeStamp >= frcInst->nextTxFrameRtpTs);
    } else {
      retVal = 1;
      frcInst->nextTxFrameRtpTs += outputRtpTsIncr;
    }    
  }
  /* Do not allow any frame to be repeated more than 2 times as there is no practical use case
     Clamping is necessary to deal with non-compliant RTP souces */
  if (retVal >= SIU_FRC_MAX_RPT_CNT) {
     retVal = SIU_FRC_MAX_RPT_CNT;
  }
  return(retVal);
}

/******************************************************************************
 * FUNCTION PURPOSE: Returns FRC statistics
 ******************************************************************************
 * DESCRIPTION: 
 ******************************************************************************/
void siuFrcGetStats (void *siuInst, bool reset, tuint trans_id, tuint priority)
{
  siuInst_t *inst = (siuInst_t *)siuInst;
  struct iffrc_GET_STAT   *frcStats;
  siuFrcInst_t *frcInst = (siuFrcInst_t *)&inst->frcInst;
  mgb_t send_message;

  memset (&send_message, 0, sizeof(mgb_t));  
  send_message.header.length   = MGB_HEADER_SIZE + sizeof(struct iffrc_GET_STAT)
                                               / sizeof(UINT16);
  send_message.header.chnum    = (int)(inst->ID & SIU_CID_MASK);
  send_message.header.msg_id   = MGB_DM_GET_FRC_STAT;
  send_message.header.trans_id = trans_id;

  frcStats                       = &(send_message.msg_body.frc_stat);
  frcStats->outputFrameRate      = frcInst->outputFrameRate;
  frcStats->numFramesDroppedMSW  = siu_get_umsw(frcInst->numFramesDropped);
  frcStats->numFramesDroppedLSW  = siu_get_ulsw(frcInst->numFramesDropped);
  frcStats->numFramesRepeatedMSW = siu_get_umsw(frcInst->numFramesRepeated);
  frcStats->numFramesRepeatedLSW = siu_get_ulsw(frcInst->numFramesRepeated);
  frcStats->numFramesUsedMSW     = siu_get_umsw(frcInst->numFramesUsed);
  frcStats->numFramesUsedLSW     = siu_get_ulsw(frcInst->numFramesUsed);
  if(reset) { /* Reset stats */
     frcInst->numFramesDropped  = 0;
     frcInst->numFramesRepeated = 0;
     frcInst->numFramesUsed     = 0;
  }
  /* send message to FIFO */
  siuNduSendMsgPriority(&send_message, (NDU_DPRMsgType_t)priority); 
} /* siuFrcGetStats */
/******************************************************************************
 * FUNCTION PURPOSE: Message processing for frame rate control configuration
 ******************************************************************************
 * DESCRIPTION: Store FRC configuration into FRC instance
 *              
 * void siuFrcCfg (
 *   void *siuInst,     - siu instance of FRC channel
 *   mgb_t *message)    - message providing FRC configuration details
 ******************************************************************************/ 
void siuFrcCfg (void *siuInst, mgb_t *message)
{
siuInst_t *inst = (siuInst_t *)siuInst;
  if(message->msg_body.video_frc_cfg.validParams & MGB_MDS_VIDEO_FRC_CFG_VALID_MODE)
  {
    vtkCtrl_t vtkCtrl;

    siu_init_frc(&(inst->frcInst));
    inst->frcInst.mode = message->msg_body.video_frc_cfg.mode;
    vtkCtrl.code = VTK_GET_ENC_DYN_CFG;
    gg_vtk_ctable.vtkControl(inst->stateInst.video.vtkInst, &vtkCtrl);
    siu_frc_configure_frameRate(&(inst->frcInst), vtkCtrl.u.vtkEncDynCfg.targetFrameRate);
    return;      
  }
} /* siuFrcCfg */
/* nothing past this point */
