/************************************************************************
:*                                                                      *
:*     @ Copyright (C) 2009  Texas Instruments Incorporated.            *
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include "siuVigdkGmp.h"
#include <string.h>
#include <ti/mas/types/types.h>
#include "common/siuloc.h"

#define SIUGMP_INLINE inline /* Use inline to trade memory for mips */

static tint gmp_ID_generator = -1;
/* Storage area for GMP descriptors */
gmpDesc_t gmpArray[GMP_ID_MAX];

/******************************************************************************
* FUNCTION PURPOSE: Fatal GMP exception
******************************************************************************
* DESCRIPTION: Fatal exception is thrown when further operation is not allowed.
*              It is recommened to take a look at siugmpStats data structure to
*              investigate cause of fatal exception (no more free GMCs, GMC
*              could not be free-ed because of corrupted pointer values etc..).
* static void gmpException (tuint err_code) - Error codes that are defined in
*                                             siugmp.h
* Returns: None
*****************************************************************************/
static void gmpException (tuint id, siugmpStats_t *stats, tuint err_code) 
{
  if (stats != NULL) {
    stats->errcode = err_code;
  }
  siuDebugInfo (id, SIU_DEBUG_FATAL, err_code, 0, NULL);
}

/******************************************************************************
* FUNCTION PURPOSE: Creates new index to array of global memory descriptors
******************************************************************************
* DESCRIPTION: New index value is generated, and tested if maximum is exceeded.
*              Exception is generated if maximum is exceeded.
* Returns: Index to new GMP descriptor.
*****************************************************************************/
static tuint gmp_gnrt_id(void) 
{
  gmp_ID_generator ++;
  if(gmp_ID_generator < GMP_ID_MAX) {
    return (gmp_ID_generator);
  } 
  gmpException (0, NULL, GMP_EXC_ID_GNRT);
  /* Unreachable */
  return 0;
}

/******************************************************************************
* FUNCTION PURPOSE: Resets all the Global Memory Pools
******************************************************************************
* DESCRIPTION: Called during System initialization
* Returns: None
*****************************************************************************/
void gmp_reset(void) 
{
  gmp_ID_generator = -1;
  memset(gmpArray, 0, GMP_ID_MAX * sizeof(gmpDesc_t));
}

/*********************************************************************************
 * FUNCTION PURPOSE: Re-Initialize the Global Memory Pool
 *********************************************************************************
  DESCRIPTION:      This function is called once a pool needs to be destroyed and 
                    re-created with in a different address range
  Parameters :      Inputs: gmp_storage_area_start : start area for the pool
                            gmp_storage_area_end   : end area for the pool
                            sizeOfGMC              : size of a GMC
                            type                   : Parent or Child/sibling pool
                            *handle                : pointer to handle
                    Output: return handle          : Handle to the GMP
 *********************************************************************************/
void gmpResize(tword *gmp_storage_area_start, tword *gmp_storage_area_end, 
               tulong sizeOfGMC, gmpInitGmpType_t type, gmpHandle_t *handle)
{
  tlong i, index;
  gmpDesc_t *desc;
  gmpGmc_t *pGMC;
  tuint numOfGMC;

  if ((gmp_storage_area_start == NULL) || (gmp_storage_area_end == NULL) 
       ||(sizeOfGMC == 0) ||(*handle == NULL)) {
      /* return NULL handle */
      return;
  }
  desc = (gmpDesc_t *)*handle;

  /* Save size of individual GMC */
  desc->sizeOfGMC = sizeOfGMC;
  /* Calculate number of GMCs that may be stored in this memory area */
  numOfGMC  = (tuint)((gmp_storage_area_end - gmp_storage_area_start) / sizeOfGMC);
  /* Initialize head of free GMCs stored in this contigeous address space */
  desc->freeHead  = (gmpGmc_t *)&gmp_storage_area_start[0];
  /* -1 indicates special area, accessible by all pages. */

  /* Save provided information */
  desc->areaStart = gmp_storage_area_start;
  desc->areaEnd   = gmp_storage_area_end;

  /* Create list of free GMCs */
  for (i = 0, index = 0; i < numOfGMC; i ++) {
    pGMC   = (gmpGmc_t *)&gmp_storage_area_start[index];
    memset((void *)pGMC, 0, sizeOfGMC);
    index += sizeOfGMC;
    pGMC->u.next = (void *)&gmp_storage_area_start[index];
  }
  pGMC->u.next = NULL;  /* This is last GMC. End of list is terminated with
                         * NULL pointer. */

  memset(&desc->stats,0,sizeof(siugmpStats_t));
  /* Update GMP statistics */
  desc->stats.totalGMC += numOfGMC;
  /* Initialize the next ChildSiblingPool to NULL */
  desc->gmpChildSiblingPool = NULL;
  return;
}

/*********************************************************************************
 * FUNCTION PURPOSE: Initialize the Global Memory Pool
 *********************************************************************************
  DESCRIPTION:      This function is called once a memory chunk needs to be made
                    into a global memory pool
  Parameters :      Inputs: gmp_storage_area_start : start area for the pool
                            gmp_storage_area_end   : end area for the pool
                            sizeOfGMC              : size of a GMC
                            type                   : Parent or Child/sibling pool
                    Output: return handle          : Handle to the GMP
 *********************************************************************************/
gmpHandle_t gmpInit(tword *gmp_storage_area_start, tword *gmp_storage_area_end, 
                    tulong sizeOfGMC, gmpInitGmpType_t type)
{
  tint gmpID;
  gmpDesc_t *desc;

  if ((gmp_storage_area_start == NULL) || (gmp_storage_area_end == NULL) ||(sizeOfGMC == 0)) {
      /* return NULL handle */
      return (NULL);
  }

  if (type == GMP_PARENT) {
    /* Get new GMP identifier */
    gmpID = gmp_gnrt_id();
    desc  = (gmpDesc_t *)&gmpArray[gmpID];
  } else if (type == GMP_CHILD_SIBLING) {
    desc  = (gmpDesc_t *)gmp_storage_area_start;
    gmp_storage_area_start += sizeof(gmpDesc_t);
  } else {
    gmpException(0, NULL, GMP_EXC_ID_INIT_FAIL);
  }

  gmpResize(gmp_storage_area_start, gmp_storage_area_end, sizeOfGMC, type, (gmpHandle_t *)&desc);
  return(desc);
}
/*********************************************************************************
 * FUNCTION PURPOSE: Get size of GMC for a GMP.
 *********************************************************************************
  DESCRIPTION:      This function is called to find out the size of GMCs in the 
                    pool.

  Parameters :      Inputs: input_MID             : module ID
                            handle                : Handle to the GMP
                    Output: numGmcsInUse          : # of GMCs currently in use
 *********************************************************************************/
tlong gmpSizeGmc(tuint input_MID, gmpHandle_t handle)
{
  gmpDesc_t *desc = (gmpDesc_t *)handle;
  return(desc->sizeOfGMC);
}

/*********************************************************************************
 * FUNCTION PURPOSE: Get current GMP usage stats
 *********************************************************************************
  DESCRIPTION:      This function is called to find out how many GMCs are currently
                    being used in the pool
  Parameters :      Inputs: handle                : Handle to the GMP
                    Output: numGmcsInUse          : # of GMCs currently in use
 *********************************************************************************/
tuint gmpGetCurrentUsage(gmpHandle_t handle)
{
  gmpDesc_t *desc;
  siugmpStats_t *stats;

  desc  = (gmpDesc_t *)handle;
  stats = &(desc->stats);
  return(stats->currentUsage);
}

/******************************************************************************
* FUNCTION PURPOSE: Returns first free GMC, and removes that GMC from free list
******************************************************************************
* DESCRIPTION: Get first free GMC from the requested list of free GMCs.
*   static void *gmp_alloc_gmc (gmpDesc_t *gmp_desc - Pointer to GMP descriptor
* Returns: Void pointer to free GMC.
* Assumptions: Execution of this function is critically sectioned. 
*****************************************************************************/
/* The static inline grows code 64 bytes, but saves 2.6 mips / 32 channels */
static SIUGMP_INLINE void *gmp_alloc_gmc (gmpDesc_t *gmp_desc) {
  gmpGmc_t *gmpGmc;

  /* This operation needs to be atomic, since GMP allocation may be simultaneously
   * requested from background context, PCM ISR context and in future HPI ISR 
   * context. */
  gmpGmc = gmp_desc->freeHead;
  if(gmpGmc)
    gmp_desc->freeHead = (gmpGmc_t *)gmpGmc->u.next; /* Remove head from list of free */

  return (void *)gmpGmc; /* This may not be NULL - otherwise error is met */
}

/******************************************************************************
 * FUNCTION PURPOSE: Link GMC into list of free GMCs for that particular GMP.
 ******************************************************************************
 * DESCRIPTION: Most recent GMC becomes head of list of free GMCs, and the rest
 *              are linked as tail to new head of free GMCs.
 * static void gmp_free_gmc_single (gmpDesc_t *gmp_desc, - GMP descriptor 
 *                                  gmpGmc_t *gmp_gmc)   - GMC that is free-ed.
 * Returns: None
 * Assumptions: Execution of this function is critically sectioned.
 *****************************************************************************/
static SIUGMP_INLINE void gmp_free_gmc_single (gmpDesc_t *gmp_desc, gmpGmc_t *gmp_gmc) {
  gmpGmc_t  *gmp_prev_head;
/* This operation needs to be atomic, since GMP allocation may be simultaneously
 * requested from background context, PCM ISR context and in future HPI ISR 
 * context. */
  gmp_prev_head = gmp_desc->freeHead;
  gmp_desc->freeHead = gmp_gmc;
  gmp_gmc->u.next = gmp_prev_head;
}

/*********************************************************************************
 * FUNCTION PURPOSE: Public function of GMP 
 *********************************************************************************
  DESCRIPTION:      Allocate a GMC from GMP pool. GMP Pool is identified by the handle
  Parameters :      Inputs: input_MID      : Module ID of the module requesting 
                                             memory
                            handle         : handle of the Global Memory Pool from 
                                             where memory needs to be allocated
                    Output: return gmc     : Allocated memory chunk
 *********************************************************************************/
void *gmpAllocGmc (tuint input_MID, gmpHandle_t handle)
{
  gmpDesc_t     *GMPAREA = (gmpDesc_t *)handle;
  siugmpStats_t *stats   = &(GMPAREA->stats);
  void  *new_gmc;

  if (handle == NULL) {
    gmpException(input_MID, stats, GMP_EXC_NULL_HANDLE);
  }

  stats->currentUsage ++;
  if(stats->currentUsage > (stats->totalGMC +1)) {
    /* Invalid operation */
    gmpException(input_MID, stats, GMP_EXC_CANNOT_ALLOCATE); 
    return NULL;
  }

  if(stats->currentUsage > stats->maxUsage)
     stats->maxUsage = stats->currentUsage;

  new_gmc = NULL;   

  /* Try to allocate from that area */
  new_gmc = (void *)gmp_alloc_gmc(GMPAREA);

  /* If successfully allocated - return */
  if(new_gmc) {
     if(!( (new_gmc >= (void *)GMPAREA->areaStart) && 
           (new_gmc <= (void *)GMPAREA->areaEnd) )) { 
         /* Corrupted head pointer! */
        gmpException(input_MID, stats, GMP_EXC_CANNOT_ALLOCATE);
     }
     return new_gmc;
  }

  /* Decrement stats because allocation was not successful */
  stats->currentUsage--;
  stats->maxUsage--;

  /* No free GMCs - report exception, and stop.
   * Additional memory is needed (modify ggmemdef.c) */
  return NULL;
}

/*********************************************************************************
 * FUNCTION PURPOSE: Private function of GMP
 *********************************************************************************
  DESCRIPTION:      Attempt to free a GMC from a pool
  Parameters :      Inputs: input_MID      : Module ID of the module requesting 
                                             memory
                            handle         : handle of the Global Memory Pool from 
                                             where memory needs to be allocated
                            gmp_gmc        : gmc that needs to be freed
                    Output: return value   : true if freed. 
                                             false if could not be freed
 *********************************************************************************/
tbool gmpFreeGmcInternal (tuint input_MID, gmpHandle_t handle, void *gmp_gmc) 
{
  gmpDesc_t     *GMPAREA = (gmpDesc_t *)handle;
  siugmpStats_t *stats   = &(GMPAREA->stats);

  if(gmp_gmc) {
    /* De-allocate only non-NULL GMC */
    /* De-allocate only real GMC */
    /* Is base address of free-ed GMC in range of this particular memory area ? */
    if((gmp_gmc >= (void *)GMPAREA->areaStart) && (gmp_gmc < (void *)GMPAREA->areaEnd)) {
       if(stats->currentUsage == 0) {
          gmpException(input_MID, stats, GMP_EXC_CANNOT_FREE);
       }
       gmp_free_gmc_single (GMPAREA, (gmpGmc_t *)gmp_gmc);
       /* Update statistics (usage of GMCs is decreased) */
       stats->currentUsage --;                
       return TRUE;
    }
  }
  /* Invalid operation */
  return FALSE; /* this GMC or buffer was not placed in any of 
                 * GMP owned memory areas (like FIU case) */
}

/*********************************************************************************
 * FUNCTION PURPOSE: Public function of GMP
 *********************************************************************************
  DESCRIPTION:      Attempt to allocate memory smaller than GMC
  Parameters :      Inputs: input_MID      : Module ID of the module requesting 
                                             memory
                            handle         : handle of the Global Memory Pool from 
                                             where memory needs to be allocated
                            size           : size of memory that needs to be allocated
                    Output: return gmc     : Allocated memory chunk
 *********************************************************************************/
void *gmpPartialAlloc (tuint input_MID, gmpHandle_t handle, tulong size)
{
  void *ret_ptr;
  gmpDesc_t *GMPAREA = (gmpDesc_t *)handle;
  gmpDesc_t *prevHandle, *newPoolHandle, *currAreaHandle;

  if (handle == NULL) {
    gmpException(input_MID, NULL, GMP_EXC_NULL_HANDLE);
  }

  if (size == GMPAREA->sizeOfGMC){
    ret_ptr = (void *)gmpAllocGmc(input_MID,handle);
  } else {
    ret_ptr = NULL;
    newPoolHandle = (gmpDesc_t *)handle;
    do {
      if ((newPoolHandle == NULL) || (newPoolHandle->gmpChildSiblingPool == NULL)) {
        ret_ptr                     = (void *)gmpAllocGmc(input_MID,GMPAREA);
        prevHandle                  = newPoolHandle;
        newPoolHandle               = (gmpDesc_t *)ret_ptr;
        newPoolHandle->areaStart    = (word *)ret_ptr;// + sizeof(gmpDesc_t);
        newPoolHandle->areaEnd      = (word *)ret_ptr + GMPAREA->sizeOfGMC;
        handle                      = gmpInit(newPoolHandle->areaStart,newPoolHandle->areaEnd,size,GMP_CHILD_SIBLING);
        newPoolHandle->mid          = input_MID;
        prevHandle->gmpChildSiblingPool = handle;
      } else {
        handle = newPoolHandle->gmpChildSiblingPool;
      }
      currAreaHandle = (gmpDesc_t *)handle;
      if (size == currAreaHandle->sizeOfGMC) {
        ret_ptr = (void *)gmpAllocGmc(input_MID,handle);
      }
      if (ret_ptr == NULL) {
        newPoolHandle = newPoolHandle->gmpChildSiblingPool;
      }
    } while(ret_ptr == NULL);
  }
  return (ret_ptr);
}
/*********************************************************************************
 * FUNCTION PURPOSE: Public function of GMP
 *********************************************************************************
  DESCRIPTION:      Free a GMC and release it back to the GMP
  Parameters :      Inputs: input_MID      : Module ID of the module requesting 
                                             memory
                            handle         : handle of the Global Memory Pool from 
                                             where memory needs to be allocated
                            gmp_gmc        : gmc that needs to be freed
                    Output: return value   : true if freed. 
                                             exception if cannot be freed
 *********************************************************************************/
tbool gmpFreeGmc (tuint input_MID, gmpHandle_t handle, void *gmp_gmc)
{
  gmpDesc_t *GMPAREA = (gmpDesc_t *)handle;
  siugmpStats_t *stats   = &(GMPAREA->stats);
  gmpDesc_t *currHandle;
  tbool retVal = FALSE;

  if (handle == NULL) {
    gmpException(input_MID, stats, GMP_EXC_NULL_HANDLE);
  }

  currHandle = GMPAREA->gmpChildSiblingPool;
  /* look for gmp_gmc in the childSibling pool */
  while(currHandle != NULL) {
    retVal = gmpFreeGmcInternal(input_MID, currHandle, gmp_gmc);
    if (retVal == FALSE) {
      /* cannot be freed in this pool */
      currHandle = currHandle->gmpChildSiblingPool;
    } else {
      /* found the pool to free, gmc is already freed */
      /* if the currentUsage == 0, free the childSiblingPool */
      if (currHandle->stats.currentUsage == 0) {
        gmpDesc_t *prevHandle = GMPAREA;
        while(prevHandle->gmpChildSiblingPool != currHandle) {
          prevHandle = prevHandle->gmpChildSiblingPool;
        }
      prevHandle->gmpChildSiblingPool = currHandle->gmpChildSiblingPool;
      gmpFreeGmcInternal(input_MID, GMPAREA, currHandle->areaStart);
      }
    break;
    }
  }
  
  if (retVal == FALSE) {
    retVal = gmpFreeGmcInternal(input_MID, handle, gmp_gmc);
  }

  if (retVal != TRUE) {
    gmpException(input_MID, stats, GMP_EXC_CANNOT_FREE);
  }
  return(retVal);
}

static inline void siuGmpCpy (void *destv, void *srcv, int len)
{
  word *dest = (word *)destv;
  word *src  = (word *)srcv;
  int i;
  for(i=0;i<len;i++) {
    *dest++ = *src++;
  }
}

/******************************************************************************
 * FUNCTION PURPOSE: Create a FILE-like structure.
 ******************************************************************************
 * DESCRIPTION: We assume that there are only ONE writing and ONE reading process
 *              for each of FILE-like data structures.
 * void *siugmpCreate (tuint input_MID,  - channel and module identifier
 *                     tuint mode        - mode of operation. At the moment
 *                                         only write APPEND mode is supported.
 *                     tuint recordSize, - single record size (in word units !)
 *                    )
 * Return value : Pointer to head of FILE-like structure
 *****************************************************************************/
void *gmpCreate (tuint input_MID, gmpHandle_t handle)
{
    gmcHeadArea_t *headArea = (gmcHeadArea_t *)gmpAllocGmc (input_MID, handle);
    gmpDesc_t *desc = (gmpDesc_t *)handle;

    /* This is initialization */
    headArea->leftInGmcWrite    = SIUGMP_HEAD_AREA(desc->sizeOfGMC);
    headArea->leftInGmcRead     = SIUGMP_HEAD_AREA(desc->sizeOfGMC);
    headArea->lastWrite         = (void *)headArea;
    headArea->next              = NULL;
    headArea->lastRead          = (void *)headArea;
    return headArea;
}

/******************************************************************************
 * FUNCTION PURPOSE: Writing into FILE-like structure
 ******************************************************************************
 * DESCRIPTION:
 * void *siugmpWrite (tuint input_MID, - channel and module identifier
 *                    void  *head      - head of FILE-like structure
 *                    void  *srcIn     - start of linear buffer that will be
 *                                       copied into GMP based FILE-like structure
 *           int   records_to_copy     - number of RECORDS (not words !!!) to be
 *                                       copied into a FILE
 * Return value : Pointer to head area
 *****************************************************************************/
void gmpWrite (tuint input_MID, gmpHandle_t handle, void *head, void *srcIn, tint records_to_copy)
{
  gmcArea_t     *newGmc;
  gmcHeadArea_t *headArea = (gmcHeadArea_t *)head;
  word *dst;
  word *src = (word *)srcIn;
  tuint leftInGmc;
  gmpDesc_t *desc = (gmpDesc_t *)handle;
  tulong siugmpGmcAreaSize = SIUGMP_AREA(desc->sizeOfGMC);

  newGmc    = (gmcArea_t *)headArea->lastWrite;
  leftInGmc = headArea->leftInGmcWrite;
  dst   = (word *)&newGmc->block[0];
  /* Implementation tuned for one word record size (MIPS/code saving) */
  newGmc    = (gmcArea_t *)headArea->lastWrite;
  leftInGmc = headArea->leftInGmcWrite;
  dst   = (word *)&newGmc->block[0];
  dst  += (siugmpGmcAreaSize - leftInGmc);
  while (records_to_copy)
  {
    tuint this_copy;

    if (leftInGmc == 0) {
      /* Allocate new GMC */
      newGmc->next = (gmcArea_t *)gmpAllocGmc(input_MID, handle);
      newGmc = (gmcArea_t *)newGmc->next;
      newGmc->next = NULL;      
      dst = (word *)&newGmc->block[0];      
      leftInGmc = siugmpGmcAreaSize;
    }
    this_copy = records_to_copy;
    if (this_copy > leftInGmc)
      this_copy = leftInGmc;

    siuGmpCpy (dst, src, this_copy);
    leftInGmc -= this_copy;
    records_to_copy -= this_copy;
    src += this_copy;
  }
  /* This is saved for future writes */
  headArea->lastWrite      = newGmc;  
  headArea->leftInGmcWrite = leftInGmc;
}

/******************************************************************************
 * FUNCTION PURPOSE: Reading from FILE-like structure
 ******************************************************************************
 * DESCRIPTION:
 * tint *siugmpRead (tuint input_MID, - channel and module identifier
 *                   void  *dstOut,   - linear buffer where data from GMP based
 *                                      FILE-like structure are to be copied.
 *              int records_to_copy   - number of records to be copied from
 *                                      FILE-like structure into linear buffer
 * Return value : Number of records that have been successfully read
 * NOTE: This function does not treat properly case if more data are requested
 *       for read, than actually available.
 *****************************************************************************/
tint gmpRead (tuint input_MID, gmpHandle_t handle, void *head, void *dstOut, tint records_to_copy)
{
  gmcHeadArea_t *headArea = (gmcHeadArea_t *)head;
  gmcArea_t *curr;
  word *src;
  word *dst = (word *)dstOut;
  tuint leftInGmc;
  tuint request_to_read = records_to_copy;
  gmpDesc_t *desc = (gmpDesc_t *)handle;
  tulong siugmpGmcAreaSize = SIUGMP_AREA(desc->sizeOfGMC);

  curr = (gmcArea_t *)headArea->lastRead;
  leftInGmc = headArea->leftInGmcRead;
  src  = (word *)&curr->block[0];

  /* Implementation tuned for one word record size (MIPS/code saving) */
  src += siugmpGmcAreaSize - leftInGmc;
  
  while (records_to_copy)
  {
    tuint this_copy;

    this_copy = records_to_copy;
    if (this_copy > leftInGmc)
      this_copy = leftInGmc;

    siuGmpCpy (dst, src, this_copy);
    leftInGmc -= this_copy;;
    records_to_copy -= this_copy;

    if (leftInGmc || !curr->next)
      break;

    dst += this_copy;

    curr = (gmcArea_t *)curr->next;
    leftInGmc = siugmpGmcAreaSize;
    src = (word *)&curr->block[0];
  }
  headArea->leftInGmcRead = leftInGmc;
  headArea->lastRead      = curr;
  
  /* Amount of data that has been read */
  return request_to_read - records_to_copy;
}

/******************************************************************************
 * FUNCTION PURPOSE: Release resources allocated by FILE-like structure
 ******************************************************************************
 * DESCRIPTION:
 * void *siugmpDelete (tuint input_MID, - channel and module identifier
 *                     void  *headIn,   - pointer to head of FILE-like structure
 * Return value : NONE
 *****************************************************************************/
void gmpDelete (tuint input_MID, gmpHandle_t handle, void *headIn)
{
  gmcArea_t *head = (gmcArea_t *)headIn;
  gmcArea_t *tmp;
  while (head->next) {
     tmp = (gmcArea_t *)head->next;
     gmpFreeGmc (input_MID, handle, head);
     head = tmp;
  }
  gmpFreeGmc (input_MID, handle, head);
  return;
}

/* nothing after this point */
