/************************************************************************
:*                                                                      *
:*     @ Copyright (C) 2009  Texas Instruments Incorporated.            *
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#ifndef _GMP_H_
#define _GMP_H_
#include <ti/mas/types/types.h>

/* Public */

/*********************************************************************************
 * STRUCTURE PURPOSE: System has multiple Global Memory Pools. A handle is needed
 *                    to identify a pool
 *********************************************************************************
  DESCRIPTION: gmpHandle is a void * pointer
 *********************************************************************************/
typedef void* gmpHandle_t;

/*********************************************************************************
 * Exceptions
 *********************************************************************************/
 enum {
   GMP_EXC_ID_GNRT                 = 650, /* Opened an already open channel        */
   GMP_EXC_ID_INIT_FAIL            = 651, /* Cannot Initialize (parent/child pool) */
   GMP_EXC_CANNOT_ALLOCATE         = 652, /* Cannot allocate GMC                   */
   GMP_EXC_CANNOT_FREE             = 653, /* Cannot free GMC                       */
   GMP_EXC_CANNOT_ALLOCATE_PARTIAL = 654, /* Cannot allocate partial GMC           */
   GMP_EXC_NULL_HANDLE             = 655
};

/*********************************************************************************
 * GMP Initialization Type
 * GMP is a parent pool (when initialized from vigdkInit in the application)
 * GMP is a child/sibling pool (when initialized with in another GMC
 *********************************************************************************/
typedef enum {
   GMP_PARENT,
   GMP_CHILD_SIBLING
} gmpInitGmpType_t;

/* system initialization. Resets all the Global Memory Pools */
void gmp_reset(void);

/* initialze a Global Memory Pool */
gmpHandle_t gmpInit(tword *areaStart, tword *areaEnd,
                    tulong sizeofGMC, gmpInitGmpType_t type);

tlong gmpSizeGmc(tuint input_MID, gmpHandle_t handle);
void gmpResize(tword *gmp_storage_area_start, tword *gmp_storage_area_end,
               tulong sizeOfGMC, gmpInitGmpType_t type, gmpHandle_t *handle);
void *gmpCreate (tuint input_MID, gmpHandle_t handle);
void gmpWrite (tuint input_MID, gmpHandle_t handle, void *head, void *srcIn, tint records_to_copy);
tint gmpRead (tuint input_MID, gmpHandle_t handle, void *head, void *dstOut, tint records_to_copy);
tuint gmpGetCurrentUsage(gmpHandle_t handle);
void gmpDelete (tuint input_MID, gmpHandle_t handle, void *headIn);

/* alloc */
void *gmpAllocGmc (tuint input_MID, gmpHandle_t handle);
void *gmpPartialAlloc (tuint input_MID, gmpHandle_t handle, tulong size);

/* free */
tbool gmpFreeGmc (tuint input_MID, gmpHandle_t handle, void *gmp_gmc);

/* Private */


#define GMP_ID_MAX      12   /* Maximum number of global memory pools, i.e.
                             * memory areas, potentially stored in diffent data pages. */

/******************************************************************************
 * DATA DEFINITION: Global Memory Chunk (GMC) definition
 ******************************************************************************
 * DESCRIPTION: This data structure is element of linked list of GMCs used by
 *              particular process (channel).
 *              First field in this data structure is used to store pointer to
 *              next GMC in the list. If GMC is not used by any process, than
 *              it is linked into list of free GMCs.
 *              If GMC is not free, its entire content is used as payload
 *               storage area.
 *****************************************************************************/
typedef struct {
  /* Unionize pointer with payload */
  union {
    void *next;        /* If GMC is free, than this field is used as pointer to
                        * next item in list. */
    tulong payload[1];  /* Payload size is determined dynamically */
  } u;
} gmpGmc_t;

/******************************************************************************
 * DATA DEFINITION: GMP statistics information
 ******************************************************************************
 * DESCRIPTION: This data structure is used to maintain information on number
 *              of GMCs used, maximum number used since system startup, and
 *              total number of GMCs that is available.
 *****************************************************************************/
typedef struct {
    tuint  errcode;        /* Fatal error code reported. 0 means none (look at
                            * the above list of error codes. */
    tuint  dropped;        /* Total count of packets dropped by VPU due to memory
                            * shortage */
    tuint  currentUsage;   /* Current number of GMCs allocated */
    tuint  maxUsage;       /* Maximum number of GMCs that have been used simultaneously */
    tuint  totalGMC;       /* Total number of available GMCs */
} siugmpStats_t;

/******************************************************************************
 * DATA DEFINITION: Global memory pool definition
 ******************************************************************************
 * DESCRIPTION: This data structure is used to contain all state information
 *              required for proper management of single GMP allocated in
 *              contigious memory area.
 *
 *****************************************************************************/
typedef struct gmpDesc_s {
  gmpGmc_t *freeHead;   /* Pointer to head of list of free GMCs. This list is
                         * NULL terminated (NULL indicates end of list)      */
  tulong   sizeOfGMC;   /* Size of GMC in words */
  tword   *areaStart;   /* Base address of memory area used to store this GMP */
  tword   *areaEnd;     /* End  address of memory area used to store this GMP */
  tuint    mid;         /* This is userMID, indicating which module may use
                         * this memory area. */
  siugmpStats_t stats;  /* statistics */
  struct gmpDesc_s *gmpChildSiblingPool; /* pointer to child/sibling pool */
} gmpDesc_t;

/******************************************************************************
 * DATA DEFINITION: GMC used in single-linked list (array emulation)
 ******************************************************************************
 * DESCRIPTION: First element is pointer to next GMC grouped into a list.
 *              Memory storage area is aligned at even addresses. This lives
 *              room for double word pointer.
 *****************************************************************************/
typedef struct {
  void  *next;           /* Pointer to next GMC in the list */
  void  *lastWrite;      /* Pointer to last write */
  void  *lastRead;       /* Pointer to last read */
  tuint  leftInGmcWrite; /* How many words can be stored in last GMC */
  tuint  leftInGmcRead;  /* How many words can be read from last GMC */
  tulong block[1];       /* Since data structures may be stored here, we should
                          * align to even addresses */
} gmcHeadArea_t;

/******************************************************************************
 * DATA DEFINITION: GMC used in single-linked list (array emulation)
 ******************************************************************************
 * DESCRIPTION: First element is pointer to next GMC grouped into a list.
 *              Memory storage area is aligned at even addresses. This lives
 *              room for double word pointer.
 *****************************************************************************/
typedef struct {
  void   *next;     /* Pointer to next GMC in the list */
  tulong block[1];  /* Since data structures may be stored here, we should
                     * align to even addresses */
} gmcArea_t;

/* MACROs: */
/* Calculate area size that can be used for payload, from this data structure */
#define SIUGMP_AREA(a) (a - sizeof(gmcArea_t) + sizeof(tulong))
/* Calculate area size that can be used for payload, from this data structure */
#define SIUGMP_HEAD_AREA(a)  (a - sizeof(gmcHeadArea_t) + sizeof(tulong))

#endif
/* nothong past this point */
