/******************************************************************************
 * FILE PURPOSE: Implementation of Vigdk Initialization code
 ******************************************************************************
 * FILE NAME:   siuVigdkInit.c  
 *
 * DESCRIPTION: This file contains the implementation for the initialization of 
 *              SIU-VIGDK
 *              
 * FUNCTION           DESCRIPTION
 * --------           ----------- 
 *
 * (C) Copyright 2008, Texas Instruments Inc. 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include "common/siu.h"
#include "common/siuloc.h"
#include "osal/siuFcOsal.h"
#include "ggcfg.h"
#include "siuVigdkGmp.h"
#include "ividmc/siuVidMc.h"
#include "gghwcfg.h"

/*********************************************************************************
 * FUNCTION PURPOSE: System Initialization
 *********************************************************************************
  DESCRIPTION:      This function is called during initialization of the system.
                    It does the software, hardware initializations for the system
                    and also the application specific context and channel instances
  Parameters :      Inputs: None
                    Output: None
 *********************************************************************************/
void siuMain (void)
{
  tuint revId;
  
  /* This function is called during initialization of the system.           *
   * It does platform specific initialization, calls Hardware Abstraction   *
   * Layer (HAL) initialization and sets up the peripherals.                */
   revId = siuHwInit();
  
  /* This function is called during initialization of the system.           *
   * It brings up the network interface (which is essential for the         *
   * message path for future communication between DIM and DSP) and         *
   * a random seed in initialized.                                          */
   siuSwInit();

  /* Initializes application independent portions of siuContext and         *
   * all the SIU Instances.                                                 */
   siuFrameworkInit(); 

  /* Initializes video application specific portions of siuContext, which   *
   * are not initialized in common.                                         */
   siuVigdkInit();

  /* send reset indication to MIC. */
   siuRestartIndication(revId);
   
   siuContext.state = SIU_ST_READY; 
   siuContext.hw_config_received = FALSE;
   siuContext.ipcContext.ipcStarted = FALSE;
   return;

} /* siuMain */

/*********************************************************************************
 * FUNCTION PURPOSE: Video Gateway Developer Kit Initialization
 *********************************************************************************
  DESCRIPTION:      This function is called during initialization of the system.
                    It does the software initializations for the video sub system
  Parameters :      Inputs: None
                    Output: None
 *********************************************************************************/
void siuVigdkInit(void)
{
  extern const vigdkMemoryContext_t gg_supported_builds[];
  siuCodecMemoryHandle_t *codecMemHandle = &(siuContext.codecMemHandle);
  siuVidShmHandle_t      *codecShmHandle = &(siuContext.codecShmHandle);
  const vigdkMemoryContext_t *activeBuild;
  extern cregister volatile unsigned int TSCL;
  tuint i;
  tuint edmaInstances[CPU_NCORES] = GG_DMA_INSTANCE_ARRAY;
  tuint edmaQueues   [CPU_NCORES] = GG_DMA_QUEUE_ARRAY;
  
  TSCL = 0; /* Activate Timer */

  siuContext.numPendingRxVideoTasks = siuContext.maxNumPendingRxVideoTasks = 0;
  siuContext.numPendingTxVideoTasks = 0;
  siuContext.msgTaskActiveFlag = 0;
  
  /* memset the mail-box to zero */
  memset(&(siuContext.vigdkMsg),0,sizeof(vigdkMsg_t));

#ifdef PROFILE_IDLE_MIPS
  siuContext.siuIdleMIPSCycles       = siuContext.siuAccIsrCycles = 0;
  siuContext.siuUpdateIdleReportFlag = siuContext.siuMeasuringIdleTimeFlag = FALSE;
  siuContext.idleReport.reportPending = FALSE;
  siuContext.numRxTasksProcessed = 0;
  memset(&(siuContext.idleReport),0,sizeof(siuIdleMipsReport_t));
#endif

  /* Set IPC context */
  siuContext.ipcMsgContext.appMsgSize = sizeof(vigdkMsg_t) > sizeof(vigdkMcMsg_t) ? sizeof(vigdkMsg_t) : sizeof(vigdkMcMsg_t);
  siuContext.ipcMsgContext.sharedRegionType = SHARED_REGION_TYPE_SL2;
  siuContext.ipcMsgContext.msgRevCallout = siuVigdkProcessIpcMessage;

  siuContext.ipcContext.shmSize = sizeof(SIUVID_MC_SHMEM_MemTab);

  /* initialize slave core list */
  for(i=0; i<CPU_NCORES-1; i++)
    siuContext.slaveCoreConn[i] = 0xFF;

  /* Reset all the Global Memory Pools */
  gmp_reset();

  activeBuild = &gg_supported_builds[siuContext.buildID];

  /* System Memory */


  siuContext.siu_codec_frame_scratch = (word *)siu_remap_mcs_data(activeBuild->system_scratch.area_begin);
  siuContext.max_codec_frame_size    = (tulong)activeBuild->system_scratch.area_end - (tulong)activeBuild->system_scratch.area_begin;

  siuContext.siu_compound_scratch       = (word *)siu_remap_mcs_data(activeBuild->system_scratch.area_begin);
  siuContext.siu_common_scratch         = (word *)siu_remap_mcs_data(activeBuild->system_scratch.area_begin);

  siuContext.siuYuvMemHandle        = gmpInit(siu_remap_mcs_data(activeBuild->yuv.area_begin),
                                      siu_remap_mcs_data(activeBuild->yuv.area_end),
                                      activeBuild->yuv.chunk_size, GMP_PARENT);
  
  siuContext.siuAudMemHandle        = gmpInit(siu_remap_mcs_data(activeBuild->aud.area_begin),
                                              siu_remap_mcs_data(activeBuild->aud.area_end),
                                              activeBuild->aud.chunk_size, GMP_PARENT);

  siuContext.siu_video_tsu_scratch      = NULL;
  siuContext.max_video_tsu_scratch_size = 0;

  /* Modules Memory */
  /* VPPU */
  vppuContext.hdrHandle             = gmpInit(siu_remap_mcs_data(activeBuild->vppuHeadersDefault.area_begin), 
                                      siu_remap_mcs_data(activeBuild->vppuHeadersDefault.area_end), 
                                      activeBuild->vppuHeadersDefault.chunk_size, GMP_PARENT);

  vppuContext.pktHandle             = gmpInit(siu_remap_mcs_data(activeBuild->vppuPackets.area_begin), 
                                      siu_remap_mcs_data(activeBuild->vppuPackets.area_end), 
                                      activeBuild->vppuPackets.chunk_size, GMP_PARENT);

  /* RCU shares the GMP pool w/ VPPU packets */
  rcuContext.gmpHandle              = vppuContext.pktHandle;

  neuContext.gmpHandle              = vppuContext.pktHandle;

  /* MMCU shares the GMP pool w/ VPPU packets */
  mmcuContext.pktHandle              = vppuContext.pktHandle;
  mmcuContext.hdrHandle              = vppuContext.hdrHandle;
  mmcuContext.streamHandle           = vppuContext.pktHandle;
  tftpContext.memHandle              = vppuContext.pktHandle;

  tsuContext.coeffHandle            = gmpInit(siu_remap_mcs_data(activeBuild->tsu.area_begin),
                                      siu_remap_mcs_data(activeBuild->tsu.area_end),
                                      activeBuild->tsu.chunk_size, GMP_PARENT);
  tsuContext.scratchHandle          = (word*)siu_remap_mcs_data(activeBuild->system_scratch.area_begin);
  tsuContext.scratchSize            = (tulong)activeBuild->system_scratch.area_end - (tulong)activeBuild->system_scratch.area_begin;
  
  /* Codec Memory */
  memset(codecMemHandle, 0, sizeof(siuCodecMemoryHandle_t));

  codecMemHandle->encResIndepHandle[0] = gmpInit(siu_remap_mcs_data(activeBuild->enc_resindep.area_begin), 
                                      siu_remap_mcs_data(activeBuild->enc_resindep.area_end), 
                                      activeBuild->enc_resindep.chunk_size, GMP_PARENT);
  codecMemHandle->encResIndepSize[0]   = activeBuild->enc_resindep.chunk_size;

  codecMemHandle->encResDepHandle   = gmpInit(siu_remap_mcs_data(activeBuild->enc_resdep.area_begin), 
                                      siu_remap_mcs_data(activeBuild->enc_resdep.area_end), 
                                      activeBuild->enc_resdep.chunk_size, GMP_PARENT);
  codecMemHandle->encResDepSize     = activeBuild->enc_resdep.chunk_size;

  codecMemHandle->decResIndepHandle[0] = gmpInit(siu_remap_mcs_data(activeBuild->dec_resindep.area_begin), 
                                      siu_remap_mcs_data(activeBuild->dec_resindep.area_end), 
                                      activeBuild->dec_resindep.chunk_size, GMP_PARENT);
  codecMemHandle->decResIndepSize[0]   = activeBuild->dec_resindep.chunk_size;

  for (i = 1; i < SIU_VIGDK_CODEC_MGR_NUM_RESINDEP_HANDLES; i++) {
    codecMemHandle->decResIndepHandle[i] = codecMemHandle->decResIndepHandle[0];
    codecMemHandle->decResIndepSize[i]   = codecMemHandle->decResIndepSize[0];
    codecMemHandle->encResIndepHandle[i] = codecMemHandle->encResIndepHandle[0];
    codecMemHandle->encResIndepSize[i]   = codecMemHandle->encResIndepSize[0];
  }

  codecMemHandle->decResDepHandle[0]= gmpInit(siu_remap_mcs_data(activeBuild->dec_resdep.area_begin), 
                                      siu_remap_mcs_data(activeBuild->dec_resdep.area_end), 
                                      activeBuild->dec_resdep.chunk_size, GMP_PARENT);
  codecMemHandle->decResDepSize[0]  = activeBuild->dec_resdep.chunk_size;

  for (i = 1; i < SIU_VIGDK_CODEC_MGR_NUM_RESDEP_HANDLES-1; i++) {
    codecMemHandle->decResDepHandle[i] = codecMemHandle->decResDepHandle[0];
    codecMemHandle->decResDepSize[i]   = codecMemHandle->decResDepSize[0];
  }
  codecMemHandle->decResDepHandle[SIU_VIGDK_CODEC_MGR_NUM_RESDEP_HANDLES-1]= 
                              gmpInit(siu_remap_mcs_data(activeBuild->dec_resdep_sec.area_begin), 
                                      siu_remap_mcs_data(activeBuild->dec_resdep_sec.area_end), 
                                      activeBuild->dec_resdep_sec.chunk_size, GMP_PARENT);
  codecMemHandle->decResDepSize[SIU_VIGDK_CODEC_MGR_NUM_RESDEP_HANDLES-1]  = activeBuild->dec_resdep_sec.chunk_size;

  codecMemHandle->decAudioHandle   = gmpInit(siu_remap_mcs_data(activeBuild->dec_audio.area_begin), 
                                     siu_remap_mcs_data(activeBuild->dec_audio.area_end), 
                                     activeBuild->dec_audio.chunk_size, GMP_PARENT);
  codecMemHandle->decAudioSize     = activeBuild->dec_audio.chunk_size;

  codecMemHandle->encAudioHandle   = gmpInit(siu_remap_mcs_data(activeBuild->enc_audio.area_begin), 
                                     siu_remap_mcs_data(activeBuild->enc_audio.area_end), 
                                     activeBuild->enc_audio.chunk_size, GMP_PARENT);
  codecMemHandle->encAudioSize     = activeBuild->enc_audio.chunk_size;

  codecMemHandle->scratch           = siu_remap_mcs_data(activeBuild->encdec_scratch.area_begin);
  codecMemHandle->scratchSize       = activeBuild->encdec_scratch.area_end - activeBuild->encdec_scratch.area_begin;

  /* RMAN initialization */
  if(siuContext.rmanConfigFlag) {
      siu_exc_assert (siu_osal_fc_resoure_init(edmaInstances[siuContext.coreId], edmaQueues[siuContext.coreId], 
                                               siuContext.coreId, TRUE)
                      == TRUE, SIUVIGDK_EXC_RMAN_INIT_FAIL, &siuContext);
      siuContext.rmanConfigFlag = 0;
  }

  mmcuContext.scratch               = codecMemHandle->scratch;
  mmcuContext.scratch_size          = codecMemHandle->scratchSize;

  tsContext.memCxt.alloc         = mmcuContext.alloc;
  tsContext.memCxt.free          = mmcuContext.free;
  tsContext.memCxt.memHandle     = mmcuContext.pktHandle;
  tsContext.scratch              = mmcuContext.scratch;
  tsContext.criticalBegin        = mmcuContext.criticalSectionBegin;
  tsContext.criticalEnd          = mmcuContext.criticalSectionEnd;

  vtkContext.scratch             = codecMemHandle->scratch;
  vtkContext.scratchSize         = codecMemHandle->scratchSize;
  atkContext.scratch             = codecMemHandle->scratch;
  atkContext.scratchSize         = codecMemHandle->scratchSize;

  /* Codec Shared Memory */
  memset(codecShmHandle, 0, sizeof(siuVidShmHandle_t));

  codecShmHandle->intShm            = siu_remap_mcs_data(activeBuild->encdec_intshm.area_begin);
  codecShmHandle->intShmSize        = activeBuild->encdec_intshm.area_end - activeBuild->encdec_intshm.area_begin;

  codecShmHandle->extShm            = siu_remap_mcs_data(activeBuild->encdec_extshm.area_begin);
  codecShmHandle->extShmSize        = activeBuild->encdec_extshm.area_end - activeBuild->encdec_extshm.area_begin;

  /* Set the External memory cacheability based on Build ID */
  siu_invalidate_cache();
  siu_set_mar(activeBuild->marRegs, activeBuild->marVals);
  siu_invalidate_cache();
  siu_exc_assert((siuContext.maxNumberChannels == SIU_TASK_MAX_NUM_CHANNELS), SIUTASK_EXC_MAX_NUM_CH_MISMATCH, &siuInst[0]);

}

/* nothing after this point */

