/******************************************************************************
 * FILE PURPOSE: SIU RX-TX Interconnection Management for VIDEO
 ******************************************************************************
 * FILE NAME:   siuVigdkInterTxRx.c
 *
 * DESCRIPTION:  SIU RX-TX Interconnection Management Routines for Video
 *
 * TABS: NONE
 *
 * (C) Copyright 2010, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include "siuVigdkInterTxRx.h"
#include "siuVigdkGmp.h"

/******************************************************************************************
 * FUNCTION PURPOSE: To retrieve information about partner channel on transcoding leg
 ******************************************************************************************
  DESCRIPTION:      This function is called to do the look up for partner channel. If 
                    the partner channel is on the same core, that channel's siuInstance
                    is returned. If the partner channel is on a different core, the
                    connection ID for the Inter-core transcoding pair is returned
  Parameters :      Inputs : siuInst        : to extract chID information
                    Outputs: txConnId       : Connection ID for Inter-core Xcode
                             txInst         : Partner siuInstance for Intra-core Xcode
                             type           : Inter core or Intra core
 ******************************************************************************************/
siuVigdkPartnerType_t siuVigdkGetPartner(siuInst_t *inst, tint *txConnId, siuInst_t **txInst) 
{
  tint i, conn = -1;
  tint chnum = inst->ID & 0x00FF; /* 1-based */
  siuVigdkPartnerType_t type;
  
  for (i = 0; i < MAX_SIU_INTERCORE_CONN; i ++) {
    if (SIU_INTER_CORE_CONN_DEC_MATCH(i, siuContext.coreId, chnum-1)) {
      conn = i;
      break;
    }
  }

  if (conn == -1) {
    /* local core transcoding */
    *txInst = siuGetTxInst(inst->ID, MGB_CONN_YUV_DOMAIN);
    type    = SIU_VIGDK_LOCAL_CORE;
  } else {
    /* core to core transcoding */
    *txConnId = conn;
    type      = SIU_VIGDK_REMOTE_CORE;
  }

  return(type);
}

/******************************************************************************************
 * FUNCTION PURPOSE: Process message received from a remote core
 ******************************************************************************************
  DESCRIPTION:      This function is callback of IPC message receive.
  Parameters :      Inputs : pMsg        : IPC message received from a remote core
                    Outputs: None
 ******************************************************************************************/
void siuVigdkProcessIpcMessage(void *pMsg)
{
  tuint chnum0;
  siuInst_t  *sInst;
  vigdkMsg_t *msg = (vigdkMsg_t *)pMsg;

  thwCriticalBegin();

  chnum0 = msg->chnum - SIU_FIRSTCH;
  if(SIUINSTpNOTCLOSED(chnum0))
  {
    siuVideoChannelOperation_t type;

    sInst = SIUINSTp (chnum0);
    type = (siuVideoChannelOperation_t)SIU_GET_VIDEO_CHANNEL_TYPE(sInst);

    if(SIU_IS_MCCODEC_MASTER(sInst)) {
      /* MASTER CORE */
      if (type == SIU_CHTYPE_ENCODER) {
         if(msg->dstMid == SIU_MID_VENC) {
            siuTxVideoPost (sInst, (void*)msg);
         } else if (msg->dstMid == SIU_MID_VDEC) {
             /* consume the message immediately */
             if(msg->frameType == ifvisys_MEDIA_TYPE_VIDEO)
               siuYuvSetBufFree(sInst, msg->frame.yuv.yuvBuffer);
             else if (msg->frameType == ifvisys_MEDIA_TYPE_AUDIO)
               gmpFreeGmc(sInst->ID, siuContext.siuAudMemHandle, msg->frame.aud.audBuffer);

             /* Indicate that the message has been consumed */
             siu_ipc_osal_set_msg_consumed((void*)msg); 
         }
      } else if (type == SIU_CHTYPE_DECODER) {
         /* consume the message immediately */
         if(msg->frameType == ifvisys_MEDIA_TYPE_VIDEO) {
           siuYuvSetBufFree(sInst, msg->frame.yuv.yuvBuffer);
         } else if (msg->frameType == ifvisys_MEDIA_TYPE_AUDIO) {
           gmpFreeGmc(sInst->ID, siuContext.siuAudMemHandle, msg->frame.aud.audBuffer);
         }

         /* Indicate that the message has been consumed */
         siu_ipc_osal_set_msg_consumed((void*)msg); 
      }
    } else if (SIU_IS_MCCODEC_SLAVE(sInst)) {
      /* SLAVE CORE */
      if (type == SIU_CHTYPE_ENCODER)
        siuTxVideoPost(sInst, (void*)msg);
      else if (type == SIU_CHTYPE_DECODER)
        siuRxVideoPost(sInst, (void*)msg);
    }
  }
  thwCriticalEnd();
}

/*** nothing past this point ***/
