#ifndef _SIUVIGDK_INTERTXRX_H
#define _SIUVIGDK_INTERTXRX_H

/******************************************************************************
 * FILE PURPOSE: Interface File for SIU RX-TX Interconnection Management
 ******************************************************************************
 * FILE NAME:   siuVigdkInterTxRx.h
 *
 * DESCRIPTION: This file contains the interface for SIU RX-TX Interconnection
 *              Management
 *
 * TABS: NONE
 *
 * (C) Copyright 2010, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include <stdlib.h>
#include <string.h>
#include "common/siu.h"
#include "common/siuloc.h"
#include "siuport.h"
#include "osal/siuIpcMsgOsal.h"
#include "ggcfg.h"
#include "vctypes.h"
#include <ti/mas/iface/ifvisys/ifvisys.h>


siuVigdkPartnerType_t siuVigdkGetPartner(siuInst_t *inst, tint *txConnId, siuInst_t **txInst);

/*********************************************************************************
 * FUNCTION PURPOSE: Signal the partner core
 *********************************************************************************
  DESCRIPTION:      This function is called once per frame by the enc/dec core to 
                    signal the partner core
  Parameters :      Inputs: intercore_conn : information to look up the partner core
                    Output: None
 *********************************************************************************/
static inline tint siu_signal_partner_core (tint intercore_conn, vigdkMsg_t *msg)
{
  tint retVal        = SIU_NOERR;
  tuint encoder_core = SIU_GET_INTER_CORE_CONN_ENC_COREID(intercore_conn);
  tuint encoder_chan = SIU_GET_INTER_CORE_CONN_ENC_CHANID(intercore_conn);
  tuint decoder_core = SIU_GET_INTER_CORE_CONN_DEC_COREID(intercore_conn);
  tuint decoder_chan = SIU_GET_INTER_CORE_CONN_DEC_CHANID(intercore_conn);
  ipcChannel_t ipcChan;

  if((msg->srcMid == SIU_MID_VDEC) && ((msg->dstMid == SIU_MID_VENC))) {
    ipcChan.remote_core = encoder_core;
    ipcChan.remote_chan = encoder_chan;
    ipcChan.local_core = decoder_core;
    ipcChan.local_chan = decoder_chan;
    msg->chnum = encoder_chan;
  } else if ((msg->srcMid == SIU_MID_VENC) && ((msg->dstMid == SIU_MID_VDEC))) {
    ipcChan.remote_core = decoder_core;
    ipcChan.remote_chan = decoder_chan;
    ipcChan.local_core = encoder_core;
    ipcChan.local_chan = encoder_chan;
    msg->chnum = decoder_chan;
  }

  thwCriticalBegin();
  siu_ipc_osal_msg_send((void *)msg, sizeof(vigdkMsg_t), &ipcChan);
  thwCriticalEnd();
  return(retVal);
}

/*********************************************************************************
 * FUNCTION PURPOSE: Signal the partner chan
 *********************************************************************************
  DESCRIPTION:      This function is called once per frame by the enc/dec core to 
                    signal the partner chan
  Parameters :      Inputs: intercore_conn : information to look up the partner core
                    Output: None
 *********************************************************************************/
static inline tint siu_signal_partner_chan (siuInst_t *txInst, vigdkMsg_t *msg)
{
  tint retVal = SIU_ERROR;

  if((msg->srcMid == SIU_MID_VDEC) && (msg->dstMid == SIU_MID_VENC) && (txInst != NULL))
  {
       /* 
       copy the message to siuContext(siuContext.vigdkMsg used as mailbox for local core 
       transcoding
       */
       memcpy(&(siuContext.vigdkMsg), msg, sizeof(vigdkMsg_t));
       retVal = siuTxEncoderProcess (txInst, txInst->ID & SIU_CID_MASK, NULL);
       memset(&(siuContext.vigdkMsg), 0, sizeof(vigdkMsg_t));
  }

  return(retVal);
}

/*********************************************************************************
 * FUNCTION PURPOSE: Signal the slave core
 *********************************************************************************
  DESCRIPTION:      This function is called once per frame by the enc/dec master 
                    core to signal the enc/dec slave core
  Parameters :      Inputs: inst, SIU instance
                            msg, message to be sent
                    Output: None
 *********************************************************************************/
static inline void siu_signal_slave_core (siuInst_t *inst, vigdkMcMsg_t *msg)
{
  tuint master_core  = inst->mcInst.coreTeamMapping[0];
  tuint master_chan  = inst->mcInst.chanIdOnCores[0];
  tuint slave_core, slave_chan;
  ipcChannel_t ipcChan;
  tuint i;

  /* Master core sends message to all participating slave cores */
  for (i = 1; i < inst->mcInst.nCores; i ++) {
    slave_core = inst->mcInst.coreTeamMapping[i];
    slave_chan = inst->mcInst.chanIdOnCores[i];
    ipcChan.remote_core = slave_core;
    ipcChan.remote_chan = slave_chan;
    ipcChan.local_core = master_core;
    ipcChan.local_chan = master_chan;
    msg->chnum = slave_chan;

    thwCriticalBegin();
    siu_ipc_osal_msg_send((void *)msg, sizeof(vigdkMcMsg_t), &ipcChan);
    thwCriticalEnd();
  }
}

#endif
/* nothing past this point */
