/******************************************************************************
 * FILE PURPOSE: SIU RX Task Management Routines for AUDIO
 ******************************************************************************
 * FILE NAME:   siuVigdkRxAudio.c
 *
 * DESCRIPTION: SIU RX Task Management Routines for Audio
 *
 * TABS: NONE
 *
 * (C) Copyright 2011, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include "common/siuloc.h"
#include <ti/mas/iface/ifvisys/ifvisys.h>
#include "atk.h"
#include "siuVigdkInterTxRx.h"
#include "siuVigdkGmp.h"

tint siuVigdkShipoutAudio(siuInst_t *inst, word *audioBuffer, tulong audioBufsize, tulong rxFrameTimeStamp);


/*********************************************************************************
 * FUNCTION PURPOSE: Decoding call on a master core  
 *********************************************************************************
  DESCRIPTION:      Master core sends message to participating slave cores and then 
                    call vtkDecode to participate in multi-core decoding.

  Parameters :      Inputs: siuInst
                            decInput        : Input to the Decoder
                            decOutput       : Output from the Decoder
                    Output: None 
 *********************************************************************************/
tint siuRxAudioMasterDecode(void *siuInst, ifvisys_audDecInput_t *audDecInput, 
                                           ifvisys_audDecOutput_t *audDecOutput)
{
  siuInst_t *inst = (siuInst_t *)siuInst;
  tint decRetVal;
  ifvisys_audDecInput_t   locAudDecInput;
  ifvisys_audDecOutput_t  locAudDecOutput;
  tlong curr_idx = 0;

  siu_exc_assert( (inst->audBufferIndex + audDecInput->bitStream.size <= inst->audBufferSize ), SIUATK_EXC_AUDIOBUFFER_OVERFLOW, inst);

  memcpy(&inst->audBuffer[inst->audBufferIndex], audDecInput->bitStream.ptr, audDecInput->bitStream.size);
  inst->audBufferIndex += audDecInput->bitStream.size;
  
  /* Decode Process Call */
  while ((inst->audBufferIndex - curr_idx) >= 2880) {

    /* Removes dependency on YUV buffer */
    locAudDecOutput.pcm.pcmFrame.ptr = &audDecOutput->pcm.pcmFrame.ptr[audDecOutput->pcm.pcmFrame.size];
    
    locAudDecInput.bitStream.ptr  = &inst->audBuffer[curr_idx];
    locAudDecInput.bitStream.size = 2880; 
    decRetVal = atkDecode (inst->stateInst.video.atkInst, &locAudDecInput, &locAudDecOutput);
    curr_idx += decRetVal;

    /* ship-out raw audio */
    audDecOutput->pcm.pcmFrame.size += locAudDecOutput.pcm.pcmFrame.size;
    locAudDecOutput.pcm.pcmFrame.size = 0;

    siu_exc_assert( (audDecOutput->pcm.pcmFrame.size < inst->audBufferSize), SIUATK_EXC_AUDIOBUFFER_OVERFLOW, inst);
  }

  if (curr_idx > 0) {
     memmove(&inst->audBuffer[0], &inst->audBuffer[curr_idx], inst->audBufferIndex - curr_idx);
     inst->audBufferIndex -= curr_idx;
  }

  siu_invalidate_cache();

  return(curr_idx);
} /* siuRxVideoMasterDecode() */

tulong  vdebug_previous_audio_ts = 0;
tint siuVigdkRxAudioDecoderProcess(siuInst_t *inst, ifvisys_audDecInput_t *audDecInput, tulong rxFrameTimeStamp)
{
  ifvisys_audDecOutput_t audDecOutput;
  tuint decRetVal, retval;
  memset(&audDecOutput,0,sizeof(ifvisys_audDecOutput_t));

  do {
    siu_invalidate_cache();
    audDecOutput.pcm.pcmFrame.ptr = gmpAllocGmc(inst->ID, siuContext.siuAudMemHandle);
  } while(audDecOutput.pcm.pcmFrame.ptr == NULL);

  //siu_exc_assert((audDecOutput.pcm.pcmFrame.ptr != NULL), SIU_EXC_TODO, inst);

  decRetVal = siuRxAudioMasterDecode(inst, audDecInput, &audDecOutput);

  if(audDecOutput.pcm.pcmFrame.size > 0) {
    retval = siuVigdkShipoutAudio(inst, audDecOutput.pcm.pcmFrame.ptr, audDecOutput.pcm.pcmFrame.size, vdebug_previous_audio_ts);
  } else {
    gmpFreeGmc(inst->ID, siuContext.siuAudMemHandle, audDecOutput.pcm.pcmFrame.ptr);
  }

  vdebug_previous_audio_ts = rxFrameTimeStamp;
  return decRetVal;
}

/******************************************************************************************
 * FUNCTION PURPOSE: To ship out Audio buffer from the Decoder Thread
 ******************************************************************************************
  DESCRIPTION:      This function is called to send out Audio buffer once the decoder thread
                    has a Audio buffer available. The parnter will be looked up in this
                    function and the Audio buffer is appropriately disposed.
  Parameters :      Inputs : siuInst        : to extract chID information
                             yuvBuffer      : YUV buffer to be shipped out
                             yuvWidth       : Width of the Image to be shipped out
                             yuvHeight      : Height of the Image to be shipped out
                    Outputs: retVal         : success/failure
 ******************************************************************************************/
tint siuVigdkShipoutAudio(siuInst_t *inst, word *audioBuffer, tulong audioBufsize, tulong rxFrameTimeStamp)
{
  vigdkMsg_t msg;
  siuVigdkPartnerType_t partner;
  tint retVal, remoteConn;
  siuInst_t *txInst;
  
  /* Retrieve partner information, whether it is on local or remote core */
  partner = siuVigdkGetPartner(inst, &remoteConn, &txInst);

  /* Form the message to be sent to the partner */
  msg.srcMid      = SIU_MID_VDEC;
  msg.dstMid      = SIU_MID_VENC;
  msg.frameType   = ifvisys_MEDIA_TYPE_AUDIO;
  msg.frame.aud.audBuffer   = audioBuffer;
  msg.frame.aud.audSize     = audioBufsize;
  msg.frameTimeStamp = rxFrameTimeStamp;

  switch(partner)
  {
    case SIU_VIGDK_LOCAL_CORE:
      retVal = siu_signal_partner_chan(txInst, &msg);
      break;

    case SIU_VIGDK_REMOTE_CORE:
      retVal = siu_signal_partner_core(remoteConn, &msg);
      break;
  }
  
  return (retVal);
}
