/******************************************************************************
 * FILE PURPOSE: SIU RX Task Management Routines for VIDEO
 ******************************************************************************
 * FILE NAME:   siuVigdkRxThread.c
 *
 * DESCRIPTION: SIU RX Task Management Routines for Video
 *
 * TABS: NONE
 *
 * (C) Copyright 2010, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include <stdlib.h>
#include <string.h>
#include "common/siu.h"
#include "common/siuloc.h"
#include "siuport.h"
#include "siuvppu.h"
#include "ggcfg.h"
#include "emureg.h"
#include <ti/xdais/dm/xdm.h>
#include "vctypes.h"
#include <ti/mas/iface/ifvisys/ifvisys.h>
#include "siuVigdkGmp.h"
#include "siummcu.h"
#include "siutftp.h"
#include "ividmc/siuVidMc.h"
#include "siuVigdkInterTxRx.h"

/*********************************************************************************
 * FUNCTION PURPOSE: Stop TFTP Put if it is running.
 *********************************************************************************
  DESCRIPTION:

  Parameters :      Inputs: siuInst         : to get TFTP instance

                    Output: None
 *********************************************************************************/
void siuVigdkStopTftpPut(siuInst_t *inst)
{
  siuInst_t *txInst;
  tint      remoteConn;

  siuVigdkGetPartner(inst, &remoteConn, &txInst);

  if(SIU_IS_PROTO_TFTP(txInst)) {
    siu_stop_tftp(txInst);
  }
}

/*********************************************************************************
 * FUNCTION PURPOSE: Main Routine for SIU Receive Task, runs per channel.
 *********************************************************************************
  DESCRIPTION:      This function operates set of ECOs located after VPPU/MMCU. 
                    It runs in main loop (background task)
  Parameters :      Inputs: siuInst         : to extract chID information
                            chIndx          :
                            msg             :
                    Output: None
 *********************************************************************************/
tint siuRxDecoderProcess(void *siuInst, tuint chIndx, void *msg)
{
  siuInst_t *inst = (siuInst_t *)siuInst;
  word                *rxFrame  = siuContext.siu_codec_frame_scratch;
  tulong              maxRxFrameSize = siuContext.max_codec_frame_size;
  bool                skipEncode = FALSE;
  ifmmcDemuxOutput_t  demuxOutput;

  union {
    ifvisys_vidDecInput_t video;
    ifvisys_audDecInput_t audio;
  } decInput;


#ifdef EMU_PROFILE
  tuint chnum = ((inst->ID & 0xFF) - 1);
#endif
  /* initialize decoder input/ demux output structure to zero */
  memset(&decInput,  0, sizeof(decInput));
  memset(&demuxOutput, 0, sizeof(ifmmcDemuxOutput_t));

SIU_STATE_BEGIN();
  siuContext.numPendingRxVideoTasks--;
  siuContext.numRxTasksProcessed++;
SIU_STATE_END();
  /* Record time time when the rx task is processed on this channel */
  inst->prevFrameDecTimerTick = siu_timer_tick;

  /* If chennel is configured as a slave decoder, the frame has been assembled 
   * by another channel. This channel only needs to make its contribution towards 
   * decoding the frame.
   */
  if (SIU_IS_MCCODEC_SLAVE(inst)) {
#ifdef EMU_PROFILE
  EMU_PROFILE_BEGIN(EMU_TASK_ID_VTK_DECODE,chnum)
#endif
    siuRxVideoSlaveDecode(inst, msg);
#ifdef EMU_PROFILE
    EMU_PROFILE_END() /* EMU_TASK_ID_VTK_DECODE */
#endif
    return (0);
  }

    
#ifdef EMU_PROFILE
  EMU_PROFILE_BEGIN(EMU_TASK_ID_VPPU_RX,chnum)
#endif

  /* A media frame is obtained either through VPPU (RTP streaming) or MMCU (Media container). */
  if( inst->stateInst.video.vppuInst != NULL) {
    
    decInput.video.bitStream.ptr = rxFrame;

    /* Extract frame directly to YUV buffer if bypassing decoder */
    if( SIU_GET_VIDEO_CODING_TYPE(inst) == EVCT_VIDEO_BYPASS_DEC )
    {
  skipEncode = siuVideoGetYUVBuffer(inst);
      decInput.video.bitStream.ptr = inst->yuvBuffer;
    maxRxFrameSize = inst->yuv.yuvFrameSize;
  }

    gg_vppu_ctable.vppuRxGetFrame(inst->stateInst.video.vppuInst, &decInput.video, &demuxOutput.timestamp);
    memcpy(&demuxOutput.bitStream,&decInput.video.bitStream,sizeof(ifvisys_BuffDesc_t));
    /* VPPU only supports video */
    demuxOutput.mediaType = ifvisys_MEDIA_TYPE_VIDEO;

    siu_exc_assert( ((decInput.video.bitStream.size + decInput.video.auxInfo.size) <= maxRxFrameSize), SIUVPPU_EXC_RXFRAME_TOO_BIG, inst);
  } else if(inst->stateInst.video.mmcuInst != NULL) {
    demuxOutput.bitStream.ptr = rxFrame;

    if(!SIU_GET_MMCU_HDR_PARSED(inst)) {
      /* Begin parsing media container to determine content */
      siu_mmcu_parse_header(inst);
      return (0);
    } else {
      /* extract frame from container */
      mmcuRxGetFrame(inst->stateInst.video.mmcuInst, &demuxOutput);

      siu_exc_assert( (demuxOutput.bitStream.size <= maxRxFrameSize), SIUMMCU_EXC_RXFRAME_TOO_BIG, inst);
    }
  } else {
    //siu_exc_assert(0,?,inst);
  }

#ifdef EMU_PROFILE
  EMU_PROFILE_END() /* EMU_TASK_ID_VPPU_RX */
#endif

  if( demuxOutput.bitStream.size <= 0) {
    /* Stream has ended */
    SIU_RESET_MMCU_STATE(inst);
    siuVigdkStopTftpPut(inst); /* Stop posting task because file has ended. */
    return(0);
  }

  switch (demuxOutput.mediaType) {
    case ifvisys_MEDIA_TYPE_VIDEO: 
      
      /* Prepare video decoder input */
      if(inst->stateInst.video.mmcuInst != NULL)
        memcpy(&decInput.video.bitStream,&demuxOutput.bitStream,sizeof(ifvisys_BuffDesc_t));

      /* Decode frame and send YUV to encoder task */
      siuVigdkRxVideoDecoderProcess( inst, &decInput.video, demuxOutput.timestamp, skipEncode);
      
      break;

    case ifvisys_MEDIA_TYPE_AUDIO:

      /* Prepare audio decoder input */
      if(inst->stateInst.video.mmcuInst != NULL)
        memcpy(&decInput.audio.bitStream,&demuxOutput.bitStream,sizeof(ifvisys_BuffDesc_t));
            
      /* Task scheduling is based on input video frame rate. Post another task */
      siuRxVideoPost(inst, NULL);
      
      /* Decode frame and send PCM to encoder task */
      siuVigdkRxAudioDecoderProcess(inst,&decInput.audio, demuxOutput.timestamp);
      break;

    default:
      /* Received frame was neither audio nor video (data, subtitles...). Drop it and post another task. */
      siuRxVideoPost(inst, NULL);
  }

  return(0);
}


/*********************************************************************************
 * FUNCTION PURPOSE: Accept send and receive samples for Rx task.
 *********************************************************************************
  DESCRIPTION:      This function is called by VPPU when it has a frame ready for
                    processing. This function posts a task into the RX queue, which
                    gets picked up in the background loop
  Parameters :      Inputs: siuInst         : pointer to siu Instance
                    Output: None
 *********************************************************************************/
tint siuRxVideoPost (void *sInst, void *pMsg)
{
  siuInst_t *inst = (siuInst_t*)sInst;
  tuint channel = ( (tuint) (inst->ID & 0xFF) - SIU_FIRSTCH );
  tuint task_priority_ID = SIU_READ_TASK_PRIORITY_ID(inst);

  /* post the task if we can */
  if (siu_queue_write (siuContext.maxNumberChannels, channel, pMsg, task_priority_ID, SIU_TASK_TYPE_RX, SIU_GET_TS))
  {
    tuint least_busy_ch = siu_queue_purge_most_busy_channel (siuContext.maxNumberChannels, task_priority_ID);
    siuInst_t *inst_least_busy = SIUINSTp(least_busy_ch);
    vppuCtrl_t vppuCtrl;

    if(inst_least_busy->stateInst.video.vppuInst != NULL) {
      vppuCtrl.code = VPPU_RESET_FIFO;
      vppuControl (inst_least_busy->stateInst.video.vppuInst, &vppuCtrl);
    }

    siuContext.numLatePlayouts++;
    return(-1);
  }
  /* Allow multiple pending tasks per channel */
  siuContext.numPendingRxVideoTasks++;

  /* debug statistic update to keep track of the max tasks piled up for execution */
  if (siuContext.numPendingRxVideoTasks > siuContext.maxNumPendingRxVideoTasks) {
     siuContext.maxNumPendingRxVideoTasks = siuContext.numPendingRxVideoTasks;
  }
  return(0);
} /* siuRxVideoPost */

/* nothing past this point */
