/******************************************************************************
 * FILE PURPOSE: SIU RX Task Management Routines for VIDEO
 ******************************************************************************
 * FILE NAME:   siuVigdkRxVideo.c
 *
 * DESCRIPTION: SIU RX Task Management Routines for Video
 *
 * TABS: NONE
 *
 * (C) Copyright 2011, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include "common/siuloc.h"
#include <ti/mas/iface/ifvisys/ifvisys.h>
#include "siuvppu.h"
#include "siuovly.h"
#include "ggcfg.h"
#include "vctypes.h"
#include "siuVigdkInterTxRx.h"
#include <ti/mas/pktutl/pkt.h>
#include "osal/siuOsal.h"
#ifdef EMU_PROFILE
#include "emureg.h"
#endif

//#include "ividmc/siuVidMc.h"

/*********************************************************************************
 * FUNCTION PURPOSE: Routine to check & forward to Packet domain connections
 *********************************************************************************
  DESCRIPTION:      This function sends out the rx frame to non-default connections
                    and returns the default conn value
  Parameters :      Inputs: siuInst         : rx inst
                            decInput        : ifvisys_decInput_t decoder Input
                            rxFrameTs       : timestamp of rx frame
                    Output: retVal          : 1 if default connection is present
                                              0 if default connection is disconnected
 *********************************************************************************/
static inline tint siuRxVideoPacketDomainRouting(siuInst_t *inst,ifvisys_vidDecInput_t *vidDecInput, tulong rxFrameTs)
{
  ifvisys_vidEncOutput_t vidEncOutput;
  siuInst_t *siuPDloopbackInst;
  tulong frameTimeStampIncrement;
  tint retVal;

  retVal = SIU_GET_VIDEO_ROUTE_PD_DEFAULT_CONN(inst);

  siuPDloopbackInst = siuGetTxInst(inst->ID, MGB_CONN_PACKET_DOMAIN);
  if(siuPDloopbackInst == NULL)
    return(retVal);

  memset(&vidEncOutput, 0, sizeof(ifvisys_vidEncOutput_t));
  siu_run_frc(siuPDloopbackInst, rxFrameTs, &frameTimeStampIncrement);

  memcpy(&(vidEncOutput.bitStream),&(vidDecInput->bitStream),sizeof(ifvisys_BuffDesc_t));
  memcpy(&(vidEncOutput.auxInfo),&(vidDecInput->auxInfo),sizeof(ifvisys_BuffDesc_t));
  /* Frame type is not known in VPPU loop back - Mark all the frames as P frames.
     This means that there is no auto-retransmission of SPS/PPS along with every
     I frame
   */
  vidEncOutput.frameType      = IFVISYS_P_FRAME; 

  gg_vppu_ctable.vppuSendIn (siuPDloopbackInst->stateInst.video.vppuInst, &vidEncOutput, frameTimeStampIncrement);
  siu_frc_update_rtp_ts (&siuPDloopbackInst->frcInst, rxFrameTs);
  return(retVal);
} /*siuRxVideoPacketDomainRouting() */



/*********************************************************************************
 * FUNCTION PURPOSE: Decoding call on a master core  
 *********************************************************************************
  DESCRIPTION:      Master core sends message to participating slave cores and then 
                    call vtkDecode to participate in multi-core decoding.

  Parameters :      Inputs: siuInst
                            decInput        : Input to the Decoder
                            decOutput       : Output from the Decoder
                    Output: None 
 *********************************************************************************/
tint siuRxVideoMasterDecode(void *siuInst, ifvisys_vidDecInput_t *vidDecInput, 
                                                         ifvisys_vidDecOutput_t *vidDecOutput)
{
  vigdkMcMsg_t msg;
  siuInst_t *inst = (siuInst_t *)siuInst;
  tint decRetVal;

  /* Master core sends decoding request to slave cores */
  msg.type = VIGDK_MC_MSG_DEC;
  msg.inputPtr = (word *)vidDecInput->bitStream.ptr;
  msg.inputID = vidDecInput->inputID;
  msg.bitStreamSize = vidDecInput->bitStream.size;
  msg.outputPtr = vidDecOutput->yuv.yuvFrame.ptr;
  siu_signal_slave_core(inst, &msg);

  /* Wait for all the participating cores to be ready before proceeding */
  siuVidMc_Swbarr(siuContext.coreId, IVIDMC_SWBARR15, inst->mcInst.nCores);
  /* Record current working channel number so that siuVidMc_Map_Shmem 
     can identify the right nCores and core team mapping*/
  siuContext.current_working_chnum = (inst->ID & SIU_CID_MASK) - SIU_FIRSTCH;
  /* Decode Process Call */
  decRetVal = vtkDecode (inst->stateInst.video.vtkInst, vidDecInput, vidDecOutput);
  /* Wait for all the participating cores to be ready before proceeding */
  siuVidMc_Swbarr(siuContext.coreId, IVIDMC_SWBARR15, inst->mcInst.nCores);

  return(decRetVal);

} /* siuRxVideoMasterDecode() */

/*********************************************************************************
 * FUNCTION PURPOSE: Decoding call on a slave core  
 *********************************************************************************
  DESCRIPTION:      Slave core consumes message from master core and calls 
                    vtkDecode to participate in multi-core decoding
  Parameters :      Inputs: siuInst
                            msg: message from master core
                    Output: None 
 *********************************************************************************/
void siuRxVideoSlaveDecode(void *siuInst, void *msg)
{
    siuInst_t *inst = (siuInst_t *)siuInst;
    vigdkMcMsg_t *pMsg = (vigdkMcMsg_t *)msg;
    ifvisys_vidDecInput_t  vidDecInput;
    ifvisys_vidDecOutput_t vidDecOutput;
    tint decRetVal;

    thwCriticalBegin();
    if (pMsg == NULL) {
       thwCriticalEnd();
       return;
    }
    vidDecInput.bitStream.ptr = (word *)pMsg->inputPtr;
    vidDecInput.inputID = pMsg->inputID;
    vidDecInput.bitStream.size = pMsg->bitStreamSize;
    vidDecOutput.yuv.yuvFrame.ptr = (word *)pMsg->outputPtr;
    /* Indicate that the message has been consumed */
    siu_ipc_osal_set_msg_consumed(pMsg);

    thwCriticalEnd();

    /* Wait for all the participating cores to be ready before proceeding */
    siuVidMc_Swbarr(siuContext.coreId, IVIDMC_SWBARR15, inst->mcInst.nCores);
    /* Record current working channel number so that siuVidMc_Map_Shmem 
       can identify the right nCores and core team mapping*/
    siuContext.current_working_chnum = (inst->ID & SIU_CID_MASK) - SIU_FIRSTCH;
    /* Decode Process Call */
    decRetVal = vtkDecode (inst->stateInst.video.vtkInst, &vidDecInput, &vidDecOutput);
    /* Wait for all the participating cores to be ready before proceeding */
    siuVidMc_Swbarr(siuContext.coreId, IVIDMC_SWBARR15, inst->mcInst.nCores);

    if(decRetVal == XDM_EFAIL) {
      vtkCtrl_t vtkCtrl;
SIU_STATE_BEGIN();
      /* Reset the Decoder */
      vtkCtrl.code = VTK_RESET_CODEC;
      vtkControl(inst->stateInst.video.vtkInst, &vtkCtrl);
SIU_STATE_END();
    }
} /* siuRxVideoSlaveDecode() */

/*********************************************************************************
 * FUNCTION PURPOSE: Routine to flush pending YUV buffers @ close chan or timeout
 *********************************************************************************
  DESCRIPTION:      This function flushes the pending YUV buffers upon close channel 
                    or when timeout happens
  Parameters :      Inputs: siuInst         : SIU instance
                    Output:                 : None
 *********************************************************************************/
void siuVideoRxFlushFrames(void *siuInst)
{
  siuInst_t *inst = (siuInst_t *)siuInst;
  vtkCtrl_t vtkCtrl;
  ifvisys_vidDecInput_t  vidDecInput;
  ifvisys_vidDecOutput_t vidDecOutput;
  word *yuvOutputBuffer;
  tuint videoCodingType;

  if(inst->state != SIU_CHST_VIDEO) return;

  videoCodingType = SIU_GET_VIDEO_CODING_TYPE(inst);
  if ((videoCodingType == EVCT_VIDEO_BYPASS_DEC) || 
      (SIU_GET_VIDEO_CHANNEL_TYPE(inst) == SIU_CHTYPE_ENCODER) ||
      (videoCodingType == EVCT_VIDEO_BYPASS_DEC)) {
     return;
  }

  if (siuYuvAreAllBufsFlushed(inst) == TRUE)
     return;

  if (SIU_IS_MCCODEC_SLAVE(inst))
     return;

  /* initialize encoder input/output structure to zero */
  memset(&vidDecInput,  0, sizeof(ifvisys_vidDecInput_t));
  memset(&vidDecOutput, 0, sizeof(ifvisys_vidDecOutput_t));

  /* Flush the pending YUV buffers from Decoder */
  vtkCtrl.code = VTK_FLUSH_DEC;
  vtkControl(inst->stateInst.video.vtkInst, &vtkCtrl);  

  /* Record current working channel number so that siuVidMc_Map_Shmem 
     can identify the right nCores and core team mapping*/
  siuContext.current_working_chnum = (inst->ID & SIU_CID_MASK) - SIU_FIRSTCH;
  /* Decode Process Call */
  vtkDecode (inst->stateInst.video.vtkInst, &vidDecInput, &vidDecOutput);

  while(vidDecOutput.yuvValid == TRUE) {
      inst->yuv.yuvOutputID   = vidDecOutput.outputID;
      /* do writeback/invalidate */
      siu_invalidate_cache();
      /* Video Decoding is complete at this point. Shipout the complete information about  */
      /* the YUV buffer. This includes, YUV buffer pointer, Image Width and Image height   */
      yuvOutputBuffer = (inst->yuv.yuvFrame[inst->yuv.yuvOutputID - 1]).yuvFramePtr;
      
      /* Call the Encode chain */
      siuVigdkShipoutYUV(inst, yuvOutputBuffer, vidDecOutput.yuv.yuvWidth, vidDecOutput.yuv.yuvHeight, 0);
      /* Record current working channel number so that siuVidMc_Map_Shmem 
         can identify the right nCores and core team mapping*/
      siuContext.current_working_chnum = (inst->ID & SIU_CID_MASK) - SIU_FIRSTCH;
      /* Decode Process Call */
      vtkDecode (inst->stateInst.video.vtkInst, &vidDecInput, &vidDecOutput);
      
      /* Increment stats */
      inst->yuvFlushCtr++;
  }

  /* Flush the pending YUV buffers from Decoder */
  siuYuvBufPoolReset(inst);

  /* Reset the Decoder */
  vtkCtrl.code = VTK_RESET_CODEC;
  vtkControl(inst->stateInst.video.vtkInst, &vtkCtrl);
  return;
} /* siuVideoRxFlushFrames() */



/*********************************************************************************
 * FUNCTION PURPOSE: TBD
 *********************************************************************************
  DESCRIPTION:      TBD
                    
  Parameters :      Inputs: siuInst         : to extract chID information
                            chIndx          :
                    Output: None 
 *********************************************************************************/
void siuRxVideoMpeg2SwapByteOrder(ifvisys_vidDecInput_t *vidDecInput)
{
  tulong  wordCount  = 0;
  tulong  *workBuf32 = (tulong *)vidDecInput->bitStream.ptr;
  tulong  numBytes   = vidDecInput->bitStream.size;
  tuint   bytesLeft;

  /* MPEG2 decoder seems to expect 32-bit words instead bytes,
   * and the words should be big endian. This code should swap 
   * the bytes on the little endian platforms but do nothing on 
   * big endian platforms. 
   */
  for( wordCount=0 ; wordCount < numBytes/4 ; wordCount++) 
    workBuf32[wordCount] = pktRead32bits((tword *)&workBuf32[wordCount],0);

  if(bytesLeft = (numBytes % 4)) {
    workBuf32[numBytes/4] = workBuf32[numBytes/4] & (0xFFFFFFFF >> ((4-bytesLeft)*8));
    workBuf32[numBytes/4] = pktRead32bits((tword *)&workBuf32[numBytes/4],0);
  }
} /* siuRxVideoMpeg2SwapByteOrder() */



/******************************************************************************************
 * FUNCTION PURPOSE: To ship out YUV buffer from the Decoder Thread
 ******************************************************************************************
  DESCRIPTION:      This function is called to send out YUV buffer once the decoder thread
                    has a YUV buffer available. The parnter will be looked up in this
                    function and the YUV buffer is appropriately disposed.
  Parameters :      Inputs : siuInst        : to extract chID information
                             yuvBuffer      : YUV buffer to be shipped out
                             yuvWidth       : Width of the Image to be shipped out
                             yuvHeight      : Height of the Image to be shipped out
                    Outputs: retVal         : success/failure
 ******************************************************************************************/
tint siuVigdkShipoutYUV(siuInst_t *inst, word *yuvBuffer, tuint yuvWidth, tuint yuvHeight, tulong rxFrameTimeStamp)
{
  vigdkMsg_t msg;
  siuVigdkPartnerType_t partner;
  tint retVal, remoteConn;
  siuInst_t *txInst;
  tuint videoCodingType = SIU_GET_VIDEO_CODING_TYPE(inst);

  /* Retrieve partner information, whether it is on local or remote core */
  partner = siuVigdkGetPartner(inst, &remoteConn, &txInst);

  /* Form the message to be sent to the partner */
  msg.srcMid      = SIU_MID_VDEC;
  msg.dstMid      = SIU_MID_VENC;
  msg.frameType   = ifvisys_MEDIA_TYPE_VIDEO;
  
  if (videoCodingType == EVCT_VIDEO_H264HP_DEC) {
    memcpy((void *)yuvBuffer, (void *)vtkContext.scratch, (yuvWidth*yuvHeight*3)>>1);
    siu_osal_wbinv_cache(yuvBuffer, (yuvWidth*yuvHeight*3)>>1, TRUE);
  }
  msg.frame.yuv.yuvBuffer   = yuvBuffer;

  msg.frame.yuv.yuvWidth    = yuvWidth;
  msg.frame.yuv.yuvHeight   = yuvHeight;
  msg.frameTimeStamp             = rxFrameTimeStamp;

  switch(partner)
  {
    case SIU_VIGDK_LOCAL_CORE:
      retVal = siu_signal_partner_chan(txInst, &msg);
      /* In case of local core transcoding, Encoder Thread doesn't free the YUV buffer */
      siuYuvSetBufFree(inst, yuvBuffer);
      break;

    case SIU_VIGDK_REMOTE_CORE:
      /* change the ownership of the buffer so that decoder core doesn't free if being Reset */
      siuYuvBufChangeOwnership(&(inst->yuv), yuvBuffer, YUV_FRAME_NOT_OWNER);
      retVal = siu_signal_partner_core(remoteConn, &msg);
      break;
  }
  
  /* we have to mark yuvBuffer = NULL to get a new buffer next time */
  inst->yuvBuffer = NULL;
  return (retVal);
} /* siuVigdkShipoutYUV() */


/*********************************************************************************
 * FUNCTION PURPOSE: TBD
 *********************************************************************************
  DESCRIPTION:      This function operates set of ECOs located after VPPU. It runs
                    in main loop (background task)
  Parameters :      Inputs: siuInst         : to extract chID information
                            chIndx          :
                    Output: None 
 *********************************************************************************/
tint siuVigdkRxVideoDecoderProcess(siuInst_t *inst, ifvisys_vidDecInput_t *vidDecInput, tulong rxFrameTimeStamp, bool skipEncode)
{
  ifvisys_vidDecOutput_t    vidDecOutput;
  word *yuvOutputBuffer;
  tint  retval = SIU_NOERR, decRetVal;
  tuint videoCodingType;
  bool yuvValid;

#ifdef EMU_PROFILE
  tuint chnum = ((inst->ID & 0xFF) - 1);
#endif

  memset(&vidDecOutput,0,sizeof(ifvisys_vidDecOutput_t));
    
  /* If decoder is bypass-ed(RFC4175),skip decode. vppuRxGetFrame() call does the RFC *
  * processing on the packets and constructs a linear frame on the provided rxFrame  *
  * buffer for feeding the decoder. In case of bypassing decoder, a YUV buffer is    *
  * received as part of RFC processing, which can be fed to the encoder              */

  videoCodingType = SIU_GET_VIDEO_CODING_TYPE(inst);
  if (videoCodingType == EVCT_VIDEO_MPEG2MP_DEC)
  {
      siuRxVideoMpeg2SwapByteOrder(vidDecInput);
  }

  /* VPPU loop back & check if default connection is present*/
  if(siuRxVideoPacketDomainRouting(inst, vidDecInput, rxFrameTimeStamp) == 0) {
      /* default connection is disconnected - don't decode */
      goto end;
  }

  if (videoCodingType != EVCT_VIDEO_BYPASS_DEC) {
    /* This is the beginning of RX Task execution, which was posted when one frame of *
    * packets are in VPPU FIFO. We need to now acquire a YUV buffer so that we can   * 
    * call the decoder. Appropriate YUV buffer is returned after checking whether it *
    * is intra or inter core transcoding                                             */

    skipEncode = siuVideoGetYUVBuffer(inst);
#ifdef EMU_PROFILE
    EMU_PROFILE_BEGIN(EMU_TASK_ID_VTK_DECODE,chnum)
#endif
    vidDecInput->inputID             = inst->yuv.yuvInputID;
    vidDecOutput.yuv.yuvFrame.ptr   = inst->yuvBuffer;

    decRetVal = siuRxVideoMasterDecode(inst, vidDecInput, &vidDecOutput);

    inst->yuv.yuvOutputID   = vidDecOutput.outputID;
    yuvValid                = vidDecOutput.yuvValid;

    if(decRetVal == XDM_EFAIL) {
        vtkCtrl_t vtkCtrl;
        SIU_STATE_BEGIN();
        /* Reset the YUV buffer Pool */
        siuYuvBufPoolReset(inst);
        retval = SIU_ERROR;
        /* Reset the Decoder */
        vtkCtrl.code = VTK_RESET_CODEC;
        vtkControl(inst->stateInst.video.vtkInst, &vtkCtrl);
        SIU_STATE_END();
        goto end;
    }
#ifdef EMU_PROFILE
    EMU_PROFILE_END() /* EMU_TASK_ID_VTK_DECODE */
#endif
  } else {
    inst->yuv.yuvOutputID = inst->yuv.yuvInputID;
    yuvValid = TRUE;
    vidDecOutput.yuv.yuvWidth  = inst->yuvWidth; 
    vidDecOutput.yuv.yuvHeight = inst->yuvHeight;
  }

  if(inst->stateInst.video.ovlyInst)
  { /* Notify channels that are using the overlay content.         */
    /* If this is ttext overlay channel - do the rendering as well */
    siuOvlyConfUpdate (inst, vidDecInput->bitStream.ptr);
    retval = SIU_NOERR;
    siu_invalidate_cache();    
    goto end;
  }

  /* do writeback/invalidate */
  siu_invalidate_cache();

  if(yuvValid == FALSE) {
    if(vidDecOutput.allocNewBuff) {
        inst->yuvBuffer = NULL;
    }
    retval = SIU_NOERR;
    goto end;
  }
  

  /* Video Decoding is complete at this point. Shipout the complete information about  */
  /* the YUV buffer. This includes, YUV buffer pointer, Image Width and Image height   */
  yuvOutputBuffer = (inst->yuv.yuvFrame[inst->yuv.yuvOutputID - 1]).yuvFramePtr;

  if(skipEncode == TRUE) {
    inst->yuvBuffer = NULL;
    thwCriticalBegin();
    siuYuvSetBufFree(inst, yuvOutputBuffer);
    thwCriticalEnd();
    retval = SIU_NOERR;
    inst->yuvDropCtr++;
    goto end;
  }

  retval = siuVigdkShipoutYUV(inst, yuvOutputBuffer, vidDecOutput.yuv.yuvWidth, vidDecOutput.yuv.yuvHeight, rxFrameTimeStamp);

end:
  return(retval);
}

/* nothing past this point */
