/************************************************************************
:*                                                                      *
:*     @ Copyright (C) 2009  Texas Instruments Incorporated.            *
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

/******************************************************************************
 * FILE PURPOSE: Main program for SIU.
 ******************************************************************************/

#include "types.h"
#include "setjmp.h"
#include "string.h"

/* SIU header files */
#include "common/siu.h"
#include "common/siuloc.h"
#include "common/dsiunwal.h"
#include "osal/siuOsal.h"
#include "siuport.h"

/* DPR definitions */
#include "micdspdp.h"
#ifndef RESIZABLEHPI
#include "dpriram.h"
#endif

/* low-level hardware files */
#include "targethw.h"

/* Utility func. headers */
#include "utilnum.h"
#include "utlport.h"
/* GG header files */
#include "ggcfg.h"

#include "common/siuCommonMain.h"

#ifndef HDCT
#include "common/siuctbl.h"     /* SIU internal call table */
#endif
#include "ndu.h"
#include "vtk.h"

#include "rttrace.h"
#include <ti/mas/neu/neu.h>

#include "emureg.h"
#ifdef EMU_PROFILE
extern tuint emu_profile_run_mode;
extern tuint emu_profile_new_mode;
#endif

/* These are MIPS counting variables, but they have been put here 
 * because they are updated no matter siumipscnt.c is linked or not   
 * Counts cycles spent in IDLE mode (updated in utlsleep.s): */
tulong siuMIPSSleepCycles  = 0;
/* Count of passes in the main loop: */
tulong siuMIPSloop         = 0;
/* Flag indicating presence of MIPS profiling   
 * By default it is not available. If it is linked-in, siuMIPSInit()
 * will set this flag to TRUE. This is used to save some code and MIPS. 
 * Usual way is to check function pointer from call table.              */
bool siuMIPSAvailable = FALSE;
/* This flag indicates if we are allowed to go to sleep mode.
 * If MIPS counting is linked-in, we may not go to sleep mode
 * until total MIPS cycles measurement is accomplished.
 * Therefore, this variable is cleared to FALSE in siuMIPSInit(),
 * and set to TRUE, once total cycles measurement is finished. */
bool siuMIPSgo2sleep = TRUE;

/* Warning: this ordering must match the ordering of SIU_???_QUEUE_ID! */
/* This table maps SIU_???_QUEUE_ID to a processing function           */
typedef tint (*siuSchedFcn_t) (void *siuInst, tuint chnum, void *pMsg);
const siuSchedFcn_t siuSchedFcns[SIU_NUM_TASK_TYPES] = {
/* Pls note that encoder is made higher priority task than decoder!!! */
  siuRxDecoderProcess,
  siuTxEncoderProcess,
};

/******************************************************************************
 * FUNCTION PURPOSE: High Priority Transmit and Receive processing task. 
 ******************************************************************************
 * DESCRIPTION: This function runs as a BIOS SWI.  It is posted any time
 *              anything is added to the transmit or receive queue.  It runs
 *              until completion, ie when all posted receive, then transmit
 *              tasks are completed.
 *****************************************************************************/
tulong severe_MIPS = 0;
tulong alarm_MIPS = 0;

extern jmp_buf  siu_env;
void siuTxRxSwi ( void )
{
  siuInst_t *inst;
  bool continueProcess;
  tuint channel;
  tuint priority, task_type;
  tint maxIterations;
  thwCriticalState_t origState;  
#ifdef RESIZABLEHPI
  dprDs_t *dprBlock = (dprDs_t *)ggConfig.hpiBase;
#endif

#ifdef SIU_NDU_HETERO_PKT_IFACE
 tuint           nduCallTblIdx;
#endif

#ifdef EMU_PROFILE
  EMU_PROFILE_BEGIN(EMU_TASK_ID_TXRXSWI,0)
#endif  
  continueProcess = TRUE;
  
  /* 
   * store the original siu_env and call setjmp()
   * If channel-based exception occur during the task processing, the
   * program execution will resume here 
   */
  setjmp(siu_env);   

  while ( continueProcess ) {
    /* Health Check. Must be highest priority otherwise the host may time out
     * the DSP and declare it dead. */
#ifdef GG_INCLUDE_HEALTH_CHECK
      siuHealthMain(siu_timer_tick, siuContext.state);
#endif

    siuHaltCheck();

    /* Core Dump */
    if (siuContext.readMemPacing.msg_rate && siuContext.readMemPacing.isr_counter) {
      siu_memory_read_exec(&siuContext.readMemPacing);
      siuContext.readMemPacing.isr_counter --;
    }
#ifdef PROFILE_IDLE_MIPS
    if (siuContext.idleReport.reportPending) {
      siu_send_idle_report();
    }
#endif
    /* First, check whether there is a MIPS overload condition */
    if (siuContext.msgDelayCnt > SIU_MAX_MSG_DELAY_CNT) {
      if (siuContext.FreeMIPS) {
        /* Flush the queue and send the message to MICRO */
        SIU_STATE_BEGIN();
        siu_queue_purge_all ();
        siuVigdkPurgeAllTasks();
        SIU_STATE_END();
        siuErrorIndication (siuContext.ID, 0, MGB_ERR_PROCESS_OVERLOAD);
        continueProcess = FALSE;
        severe_MIPS++;
        
      }
      else {
        /* Not enough MIPS so free up some MIPS! */
        siuContext.msgDelayCnt = 0;
        /* Go to Free MIPS available mode */
        siuContext.FreeMIPS = TRUE;
        siuVigdkReduceMIPS (TRUE); /* trade QoS for some MIPS */
        alarm_MIPS++;
      }
    }
#ifdef NEU_USE_GMPISRSPLIT
#ifdef EMU_PROFILE
    emu_profile_mark (EMU_TASK_ID_PKTFIFO, 0, EMU_PROF_PUSH);
#endif
    siuProcessPktFifo();
#ifdef EMU_PROFILE
    emu_profile_mark (EMU_TASK_ID_MARK_END, 0, EMU_PROF_POP);
#endif
#endif

#ifdef PA_SS
    dsiuNwalPoll ();
#else          
    /* Run the HAL poll task */        
    if(*siuContext.halApi->halPoll !=NULL)
    {
    (*siuContext.halApi->halPoll)((tuint)siu_timer_tick, siuContext.numActiveChannels);
    }
#endif
    /* If there is a receive task event pending on the rx task queue process it.
       Else if no rx event is pending, process a transmit task event pending on 
       the tx queue.  If no events are pending on either rx or tx task queues,
       break out of loop. */
#ifdef EMU_PROFILE
    EMU_PROFILE_BEGIN(EMU_TASK_ID_PRD,0)
#endif
    siuPRDTask();
#ifdef EMU_PROFILE
    EMU_PROFILE_END()
#endif

    origState = SIU_STATE_DATA_BEGIN();
    /* Find queue and priority with tasks to process, and stay on that
     * queue/priority for ggSchedHoldTimes[priority]
     */
    for (priority = SIU_FIRST_PRIORITY_ID; priority < SIU_NUM_PRIORITY_ID; priority++) 
    {  /* Maximum time to stay doing tasks of same priority */
       tlong start_time;
       tint  exec_time;    
       bool  task_exec_done = FALSE;
       void *pMsg;
       /* Execute task of same type few times - this is beneficial from cache performance standpoint */
       /* But the execution time is limited: */       
       maxIterations = SIU_TS_SCALE(ggSchedHoldTimes[priority], siuContext.core_mhz);
       while ((maxIterations >= 0) && siu_queue_read (&channel, (void **)&pMsg, &task_type, priority)) 
       {
         if(SIUINSTpNOTCLOSED(channel))
         {
           inst = SIUINSTp(channel);
           SIU_STATE_DATA_END(origState);   /* Process runs w/ interrupts on */
#ifdef EMU_PROFILE
           EMU_PROFILE_BEGIN(EMU_TASK_ID_TXRX_TASK(priority), channel)
#endif           
/* Uncomment the 'siu_invalidate_cache()' below to invalidate cache for Profiling purpose*/ 
/* this function will invalidate cache only for C64x processor and does nothing for other processors */
/*           siu_invalidate_cache();   */
           start_time = SIU_GET_TS;
           siuSchedFcns[task_type] (inst, channel, pMsg);
           exec_time  = (tint)(SIU_GET_TS - start_time);
           task_exec_done = TRUE;
#ifdef EMU_PROFILE
           EMU_PROFILE_END()
#endif
           origState = SIU_STATE_DATA_BEGIN(); /* Protect queue operations */
           siu_queue_update ( siuContext.maxNumberChannels, siuContext.numActiveChannels, priority, channel, exec_time);             
       	   maxIterations -= exec_time;           
         }
       } /* while (max..     */
       if(task_exec_done) break; /* restart from highest priority */
    } /* for priority = ..       */
    /* Exit condition: interrupts disabled */

    if (priority == SIU_NUM_PRIORITY_ID) {
       /* no tasks processed in this iterration -- drop back to background loop */
       continueProcess = FALSE;
    }
     
    SIU_STATE_DATA_END(origState);

#ifdef SIU_NDU_HETERO_PKT_IFACE
    nduCallTblIdx = 
      SIU_PKT_GET_IDX_IFACE(siuContext.msg_pkt_iface_idx,
                            SIU_CTL_PKT_IDX_MASK,
                            SIU_CTL_PKT_IDX_SHIFT);
    /* Check signal message buffer */
    NDU_GetMsg(siuContext.ifaceNduCallTable[nduCallTblIdx],NDU_INTERRUPT_MESSAGE);
#else
    /* Check signal message buffer */
    NDU_GetMsg(NDU_pCallTable,NDU_INTERRUPT_MESSAGE);
#endif

    /* Low priority Message and Control Processing */
    origState = SIU_STATE_DATA_BEGIN();
    if ( siuContext.msgTaskCounter) {
      siuContext.msgTaskCounter = 0;
      SIU_STATE_DATA_END(origState);      
      siuGuaranteedMessageTask (); 
    }
    else {
      SIU_STATE_DATA_END(origState);
    }
    /* Check if any queued signaling msg to MIC */
#ifdef SIU_NDU_HETERO_PKT_IFACE     
    NDU_MsgBg(siuContext.ifaceNduCallTable[nduCallTblIdx]); 
#else
    NDU_MsgBg(NDU_pCallTable);
#endif

    /* Check for new packets from HPI and McBSP */
   // NDU_GetData();
  } /* While( continueProcess ) */
#ifdef EMU_PROFILE
  EMU_PROFILE_END()
#endif

} /* siuTxRxSwi() */

/******************************************************************************
 * FUNCTION PURPOSE: Low Priority message and control processing task.
 ******************************************************************************
 * DESCRIPTION:
 *
 *****************************************************************************/

void siuMessageTask( void )
{
  /* If in free MIPS mode for at least 300 ms, go back to full mode */  
  if (siuContext.FreeMIPS) {
  
    if (siuContext.msgDelayCnt > SIU_MIN_FREE_MIPS_CNT) {
      siuContext.FreeMIPS = FALSE;
      siuVigdkReduceMIPS (FALSE); /* restore provisioned QoS since we again have enough MIPS */      
      siuContext.msgDelayCnt = 0;
    }
  }
  else {
    siuContext.msgDelayCnt = 0;
  }
         
  siuGuaranteedMessageTask ();
    
  siuDebugTask (); /* Post debug messages to MICRO */
} /* siuMessageTask() */

/*****************************************************************************
 * FUNCTION PURPOSE: The SIU task main routine
 ******************************************************************************
 * DESCRIPTION: Main SIU routine that does all the processing.
 *
 *        
 * NOTE:  DPR interrupt (necessary for communication with HOST)is enabled
 *        and the message fifo is reset.  All required modules for voice/Fax 
 *        are created for the configured number of channels (i.e., all 
 *        channels are initialized to CLOSED state).  
 *        PCM and PCMSync interrupts are intialized and enabled. A Restart
 *        indication message is sent to MICRO after all these steps.
 *        Presently, overlays are not supported.  
 *
 *
 *****************************************************************************/
tuint sync_error = 0;

void siuTaskScheduler (void)
{
#ifdef GG_BIOS
  thwCriticalState_t origState;
#endif
#ifdef RESIZABLEHPI
  dprDs_t *dprBlock = (dprDs_t *)ggConfig.hpiBase;
#endif

#ifdef GG_BIOS
  if (siuContext.msgTaskActiveFlag) {
    return;
  }
#endif

#ifdef EMU_PROFILE
  if (emu_profile_new_mode != emu_profile_run_mode) {
    emu_profile_change_mode();
  }
  EMU_PROFILE_BEGIN(EMU_TASK_ID_MAIN,0)
#endif

  /* If the check value is not zero, send an error message to host */
  if (sync_error)  
  {
    siuErrorIndication (siuContext.ID, 0, MGB_ERR_FSYNCERROR);
    sync_error = 0;
  }

  switch (siuContext.state) {
    case SIU_ST_READY:
#ifdef SIMULATION_BUILD    
      siu_sim_spoof_msg(); /* Spoof messages for processing */
#endif      
#ifdef GG_BIOS
      /* Message processing, keepalive ownership pass from siuMain() to
       * siuTxRxSwi() after the first channel is opened.  This critical
       * section protects that transition.  
       */
      thwCriticalBegin();
#endif

#ifdef PA_SS
      dsiuNwalPoll ();
#endif
      /* No channels are active. DSP waits for first open channel message */
      /* Health Check */
#ifdef GG_INCLUDE_HEALTH_CHECK
        siuHealthMain(siu_timer_tick, siuContext.state);
#endif

/****************************************/
      /* Periodic task in the idle state */
      siuPRDTaskIdle();

      /* Core Dump */
      if (siuContext.readMemPacing.msg_rate && siuContext.readMemPacing.isr_counter) {
        siu_memory_read_exec(&siuContext.readMemPacing);
        siuContext.readMemPacing.isr_counter --;
      }

      siuGuaranteedMessageTask();      

      /* Power management in idle mode */
      utlSleep( siuContext.timer_period, siuContext.wake_up_interrupt_mask);

#ifdef SIMULATION_BUILD
      if (siuContext.hw_config_received) { /* Ensure hwConfig is received */
        siu_sim_init();
      }
#endif               

      
      if (siuContext.numActiveChannels > 0) {
#ifndef GG_BIOS
        thwCriticalBegin();
#endif
        siuContext.state = SIU_ST_WORKING;
        siuContext.phaseWrap = 0;           /* reset number of tasks */
        siuContext.prdLastPhase = siuContext.pcm5msCounter;
        siuContext.msgTaskCounter = 0;      /* reset number of msg tasks */
        siuContext.msgDelayCnt = 0;         /* reset message delay counter */
        siuContext.FreeMIPS = FALSE;        /* reset MIPS state */
#ifndef GG_BIOS
        thwCriticalEnd();
#endif
      }      
#ifdef GG_BIOS
      thwCriticalEnd();
#endif 
      break;

    case SIU_ST_WORKING:
#ifdef SIMULATION_BUILD    
              siu_sim_spoof_msg(); /* Spoof messages for processing */
              siu_sim_spoof_isr ();   /* Spoof ISR call */
#endif              

              siuMIPSloop++;
 
#ifdef GG_BIOS
      siu_osal_post_signal(SIU_OSAL_SIGNAL_HANDLE_TXRXSWI);
#else
      siuTxRxSwi ();     /* Directly invoke TX/RX processing if non-BIOS */
#endif
      
#ifdef GG_BIOS
      origState = thwCriticalBeginFast(); 
                          /* BIOS could interrupt message processing with
                           * a tx or rx task, causing many potential races
                           */
#endif
        siuMessageTask (); /* Low priority Message and Control Processing */

/* Verification of video processing task */
#ifdef GG_BIOS
      thwCriticalEndFast(origState);
#endif
        
      /* Power management in idle mode */
      if (!siuContext.FreeMIPS) {
        if (siuMIPSgo2sleep)
          utlSleep (siuContext.timer_period, siuContext.wake_up_interrupt_mask);          
      }
      
      if (siuContext.numActiveChannels == 0) {
        siuContext.state = SIU_ST_READY;   /* Change SIU state to READY */
      }
      
      break;

    default:
      siu_exc_assert (0, SIUMAIN_EXC_MAIN_STATE, &siuContext);    /* fatal error */
      break;
  } /* switch (siuContext.state) */

#if (defined(EMU_PROFILE) || defined(PROFILE_IDLE_MIPS))
#ifdef EMU_PROFILE
  EMU_PROFILE_END()
#endif /* EMU_PROFILE */
#ifndef GG_BIOS
      if( siuContext.state == SIU_ST_WORKING) {
        /* Enter Idle loop only when there are no pending tasks */
        if (siuContext.numPendingRxVideoTasks == 0 && siuContext.numPendingTxVideoTasks == 0) {
          tulong siu_isr_clock_old;
          volatile tulong *siu_isr_clock_p = &siu_isr_clock;
#ifdef PROFILE_IDLE_MIPS
          tulong TSCL_End, TSCL_Begin;
#endif /* PROFILE_IDLE_MIPS */
          siu_isr_clock_old = *siu_isr_clock_p;
#ifdef PROFILE_IDLE_MIPS
          SIU_STATE_BEGIN();
          siuContext.siuAccIsrCycles = 0;
          siuContext.siuMeasuringIdleTimeFlag = TRUE;
          TSCL_Begin = TSCL;
          SIU_STATE_END();
#endif /* PROFILE_IDLE_MIPS */
#ifdef EMU_PROFILE
          EMU_PROFILE_BEGIN(EMU_TASK_ID_IDLE,0)
#endif /* EMU_PROFILE */
          /* stay in idle loop only when there are no pending tasks */
          while ((siu_isr_clock_old == *siu_isr_clock_p) && (siuContext.numPendingRxVideoTasks == 0)\
                                                         && (siuContext.numPendingTxVideoTasks == 0));
#ifdef EMU_PROFILE
          EMU_PROFILE_END()
#endif /* EMU_PROFILE */
#ifdef PROFILE_IDLE_MIPS
          TSCL_End = TSCL;
          siuContext.siuIdleMIPSCycles += (TSCL_End - TSCL_Begin);
          siuContext.siuIdleMIPSCycles -= siuContext.siuAccIsrCycles;
          siuContext.siuMeasuringIdleTimeFlag = FALSE;
#endif /* PROFILE_IDLE_MIPS */
        }
      }
#ifdef PROFILE_IDLE_MIPS
      if ((siuContext.siuUpdateIdleReportFlag == TRUE) \
          && (siuContext.numPendingRxVideoTasks == 0)\
          && (siuContext.numPendingTxVideoTasks == 0)) {
        siuUpdateIdleReport();
        siuContext.siuUpdateIdleReportFlag = FALSE;
      }
#endif /* PROFILE_IDLE_MIPS */
#endif /*GG_BIOS*/  
#endif /*EMU_PROFILE*/

} /* siuTaskScheduler */

/* nothing after this point */
