/******************************************************************************
 * FILE PURPOSE: After initialization starts up the system.
 ******************************************************************************
 * FILE NAME:  siustrt.c
 *
 * DESCRIPTION:  Last stage of SIU initialization, contains creating of all the
 *               modules of all channels, HPI initialization, sending of
 *               restart indication and starting the main loop.
 *
 * NOTES:        This file can be overlaid with uninitialized data section
 *               but care should be taken. Overlaying with any channel heap
 *               is NOT recommended as functions in this file changes the
 *               heap which might cause an overwrite of this file. It can
 *               be overlaid with uninitialized codec data as codec is not
 *               initialized until the opening of a channel.
 *
 * TABS: NONE
 *
 *
 * (C) Copyright 2000, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include <stdlib.h>
#include <string.h>

#include "types.h"
#include "memmgm.h"
#include "ggmbuf.h"

/* SIU header files */
#include "common/siu.h"
#include "common/siuloc.h"
#include "siuport.h"
#ifndef HDCT
#include "common/siuctbl.h"     /* SIU internal call table */
#endif
#include "common/siualive.h"
#include "osal/siuIpcMsgOsal.h"

/* low-level hardware files */
#include "ggcfg.h"

/* dual-port RAM definitions */
#include "micdspdp.h"
#ifndef RESIZABLEHPI
#include "dpriram.h"
#endif
#include "vbmicdsp.h"
#include "dprutil.h"
#include "mgmicdsp.h"

#include "hal.h"
#include "utlrepck.h"
volatile tint HwConfig_control = 0;

/******************************************************************************
 * FUNCTION PURPOSE:  Process hardware configuration message from MIC
 ******************************************************************************
 * DESCRIPTION:   Processes a hardware configuration message from MIC.  If
 *                valid, DSP configures itself for the selected hardware
 *                configuration and reports a ready indication to MIC.  If
 *                any configuration parameters are invalid an error is reported.
 *
 *****************************************************************************/
// volatile tuint emuwait_1=1;
void siuHwConfig (mgb_t *message)
{

// while(emuwait_1);

  /* Check message parameters are valid -- if not ignore message and report
     error */
  if (((message->msg_body.hw_config.companding != MGB_MULAW_COMPANDING) &&
      (message->msg_body.hw_config.companding != MGB_ALAW_COMPANDING) &&
      (message->msg_body.hw_config.companding != MGB_LINEAR_COMPANDING)) ||
      ((message->msg_body.hw_config.bdx_delay_control != MGB_BDX_DELAY_ENABLE) &&
      (message->msg_body.hw_config.bdx_delay_control != MGB_BDX_DELAY_DISABLE)) ||
      ((message->msg_body.hw_config.clkout_config != MGB_CLKOUT_ENABLE) &&
       (message->msg_body.hw_config.clkout_config != MGB_CLKOUT_DISABLE)))
  {
     siuErrorIndication (siuContext.ID, message->header.trans_id,
                         MGB_ERR_INVLD_HW_CONFIG);
     return;
  }

  /* make sure we got a valid sampling rate for this build */
  /* Program clock PLL */
  if (message->msg_body.hw_config.clk_multplier != MGB_CLK_MULT_INVALID) {
    if (*siuContext.halApi->halPllControl != NULL) {
      (*siuContext.halApi->halPllControl) (message->msg_body.hw_config.clk_multplier,1);
    }
  }

  /* Get the values for sleep timer for power management */
  siuContext.timer_period = message->msg_body.hw_config.timer_period;
  siuContext.wake_up_interrupt_mask = message->msg_body.hw_config.wake_up_interrupt_mask;

  /* CLKOUT Pin configuration */
  if (message->msg_body.hw_config.clkout_config == MGB_CLKOUT_ENABLE) {
    thwResetCLKOFFPin();
  }
  else {
    if (message->msg_body.hw_config.clkout_config ==  MGB_CLKOUT_DISABLE) {
      thwSetCLKOFFPin();
    }
  }

  /* Keep alive - health check generation */
#ifdef GG_INCLUDE_HEALTH_CHECK
    if (siuHealthInit(siu_timer_tick,
            message->msg_body.hw_config.keep_alive,
            message->msg_body.hw_config.core_mhz) == SIU_ERROR)  {
       siuErrorIndication (siuContext.ID, message->header.trans_id,
                           MGB_ERR_INVLD_HW_CONFIG);
    }
#endif

  if (siuContext.siuInitCoreMhz)
  {
    siuContext.siuInitCoreMhz(message->header.length >= MGB_HEADER_SIZE + 27 ?
                              message->msg_body.hw_config.core_mhz : 0);
  }


#ifdef RELMSG_SUPPORT
  /* If check allows backwards compatibility with old micro */
  if (message->header.length >= 34) {
    siuContext.msg_burst_size =  message->msg_body.hw_config.msg_burst_size;
  } else {
    siuContext.msg_burst_size =  0;
  }
#endif

#ifdef SIU_TIMER_BASED_TDM
  siuTsipEmulConfig(message);
#endif

  /* Acknowledge processing of valid hardware configuration message */
  siuReadyIndication(message);

} /* siuHwConfig() */

/******************************************************************************
 * FUNCTION PURPOSE:  Process sofware configuration message from MIC
 ******************************************************************************
 * DESCRIPTION:   Processes a software configuration message from MIC.  If
 *                valid, DSP configures itself for the selected software
 *                configuration and reports a ready indication to MIC.  If
 *                any configuration parameters are invalid an error is reported.
 *
 *****************************************************************************/
void siuSwConfig (mgb_t *message) {

  if ((message->msg_body.video_sw_cfg.validParams != MGB_MDS_VIDEO_SW_CFG_VALID_RES_CLASS) ||
      (siuContext.numActiveChannels != 0)) {
     siuErrorIndication (siuContext.ID, message->header.trans_id,
                         MGB_ERR_INVALID_VIDEO_SW_CONFIG);
     return;
  }

  /*
  It is guaranteed by the host application that the same build ID is propagated
  to all the DSP cores. It'll be a problem if different cores receive different
  build IDs, because the external memory may be overlapping
  */

  /* overwrite build ID: Default is set to zero in  siuFrameworkInit() */
  switch(message->msg_body.video_sw_cfg.resClass) {
    case MGB_MDS_VIDEO_SW_CFG_RES_CLASS_LOW:
       siuContext.buildID = SIU_VIGDK_BUILD_ID0_QCIF_CIF;
       break;

    case MGB_MDS_VIDEO_SW_CFG_RES_CLASS_SD:
       siuContext.buildID = SIU_VIGDK_BUILD_ID1_D1;
       break;

    case MGB_MDS_VIDEO_SW_CFG_RES_CLASS_MC:
       siuContext.buildID = SIU_VIGDK_BUILD_ID2_MC;
       break;

    default:
      siuErrorIndication (siuContext.ID, message->header.trans_id,
                         MGB_ERR_INVALID_VIDEO_SW_CONFIG);
      return;
  }

  /* Function call to initialize memory */
  siuVigdkInit();

  if((!siuContext.ipcContext.ipcStarted) &&
	 ((siuContext.buildID == SIU_VIGDK_BUILD_ID1_D1) ||
      (siuContext.buildID == SIU_VIGDK_BUILD_ID2_MC))) {
      /* Start IPC */
      siu_ipc_osal_start(&(siuContext.ipcContext));
      /* IPC attach */
      siu_ipc_osal_attach_owner(NUM_CORE_FROM_CFG);
      /* Create messageQ */
      siu_ipc_osal_msg_start(&(siuContext.ipcMsgContext));

      if((siuContext.buildID == SIU_VIGDK_BUILD_ID2_MC) &&
         (siuContext.coreId == 0)) {
		/* Core 0 create gate and shared global for multi-core builds */
	    siu_ipc_osal_create_gate();
	    siu_ipc_osal_create_share(siuContext.ipcContext.shmSize, (void**)&siuContext.ipcContext.codecShm);
	  }
      siuContext.ipcContext.ipcStarted = TRUE;
  }

  if(siuContext.buildID == SIU_VIGDK_BUILD_ID2_MC) {
      /* Open gate and shared global */
      siu_ipc_osal_open_gate();
      siu_ipc_osal_open_share((void**)&siuContext.ipcContext.codecShm);
      /* Initialize shared global */
      siuVidMc_Init_ShmemTab(siuContext.coreId);
  }

} /* siuSwConfig() */

/* nothing after this point */
