/**
 *  @file  siuVigdkTask.c
 *  @brief Functions required for Vigdk Scheduler (queue manipulation: posting, reading)
 *
 *  Copyright (c) Texas Instruments Incorporated 2009
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include "types.h"
#include "siuVigdkTask.h"
#include <string.h>
#include "common/siuloc.h"
#ifdef GG_BIOS
#include "osal/siuOsal.h"
#endif

siuTask_t siuTask[SIU_NUM_PRIORITY_ID];  /**< Task queues */
#define SIU_TASK_PENALTY_FORGET (2)      /**< Exponenetial averaging for average task execution: */
                           /**< channel_exec_time += (channel_exec_time - exec_time) >> SIU_TASK_PENALTY_FORGET */


/* ============ siu_init_task() ========== */
/**
 *  @brief      Initialize all task queues
 *
 *              This function is used by the @ref siuFrameworkInit function.
 */
void siu_init_task (void) {
  tint priority;

  for (priority = 0; priority < SIU_NUM_PRIORITY_ID; priority++) {
    memset (&siuTask[priority], 0, sizeof(siuTask_t));
  }
} /* siu_init_task */

/* ============ siu_init_task() ========== */
/**
 *  @brief      Initialize all task queues
 *
 *              This function is used by the @ref siuFrameworkInit function.
 */
void siuConfigTask ( siuInst_t *inst, tint channelState ) {

  tuint task_priority_ID = SIU_FIRST_PRIORITY_ID;

  SIU_SET_TASK_PRIORITY_ID(inst, task_priority_ID);
} /* siuConfigTask */

/* ============ siu_queue_purge_channel() ========== */
/**
 *  @brief      Remove tasks for a single channel from task queue
 *
 *              This function is used by the @ref cid_exc_assert macro.
 *
 *  @param[in]  purgeChannel          Channel whose all tasks will be purged from the queue of the given priority
 *
 *  @param[in]  task_priority_ID      Remove tasks only from queue of this Task Priority
 *
 */

void siu_queue_purge_channel (tuint purgeChannel, tuint task_priority_ID)
{
  tint       dstIndx, srcIndx, inIndx;
  siuTask_t  *qPtr;

  qPtr    = &siuTask[task_priority_ID];
  
  inIndx  = qPtr->in_index;
  srcIndx = dstIndx = qPtr->out_index;
  while (srcIndx != inIndx)
  {
    if(qPtr->queue[srcIndx] != purgeChannel) 
    {
      qPtr->queue[dstIndx]     = qPtr->queue[srcIndx];
      qPtr->queue_ts[dstIndx]  = qPtr->queue_ts[srcIndx];
      qPtr->task_type[dstIndx] = qPtr->task_type[srcIndx];            
      dstIndx ++;
      dstIndx = (dstIndx == SIU_TASK_QUEUE_LENGTH) ? 0: dstIndx;
    }
    srcIndx ++;
    srcIndx = (srcIndx == SIU_TASK_QUEUE_LENGTH) ? 0: srcIndx;  
    qPtr->cnt --;        
  }
  qPtr->in_index = dstIndx;
  qPtr->ch_exec_time[purgeChannel] = 0;
} /* siu_queue_purge_channel */

/* ============ siu_queue_purge_most_busy_channel() ========== */
/**
 *  @brief      Find channel with the maximum execution time. Then, remove tasks for this channel from task queue
 *
 *              This function is used by the @ref cid_exc_assert macro.
 *
 *  @param[in]  max_number_channels   Maximum number of channels present in the build
 *
 *  @param[in]  task_priority_ID      Only execution time at this priority is taken into account.
 *
 */
tint siu_queue_purge_most_busy_channel (tuint max_number_channels, tuint task_priority_ID)
{
  siuTask_t *qPtr;
  tuint max_cnt;
  tint i, j=0, srcIndx, inIndx;
  tuint numTaskchannels[SIU_TASK_MAX_NUM_CHANNELS];

  qPtr = &siuTask[task_priority_ID];
  max_cnt = 0;

  inIndx  = qPtr->in_index;
  srcIndx = qPtr->out_index;

  memset(numTaskchannels, 0, SIU_TASK_MAX_NUM_CHANNELS *sizeof(tuint)); 

  while (srcIndx != inIndx)
  {
    numTaskchannels[qPtr->queue[srcIndx]]++;
    srcIndx ++;
    srcIndx = (srcIndx == SIU_TASK_QUEUE_LENGTH) ? 0: srcIndx;  
  }

  /* find max of numTaskchannels */

  for(i=0;i<SIU_TASK_MAX_NUM_CHANNELS;i++) {
    if(max_cnt < numTaskchannels[i]) {
      max_cnt = numTaskchannels[i];
      j=i;
    }
  }

  siu_queue_purge_channel (j, task_priority_ID);
  return j;
}


/* ============ siu_queue_purge_all() ========== */
/**
 *  @brief      Remove all tasks scheduled for execution from all task queues.
 *
 *              Task queues are emptied during MIPS starvation. It has to be critically sectioned.
 */
void siu_queue_purge_all(void) {
tint priority;

   /* Force queue empting */
   for (priority = 0; priority < SIU_NUM_PRIORITY_ID; priority++) {
     memset(&siuTask[priority], 0, sizeof(siuTask_t));
   }
}

/* ============ siu_queue_update() ========== */
/**
 *  @brief      Do channel execution time book-keeping in order to enforce "fairness" accross channels.
 *
 *              CPU time is equaly shared for tasks of same priority only.
 *
 *  @param[in]  max_number_channels   Maximum number of channels present in the build
 *
 *  @param[in]  numActiveChannels     Number of active channels
 *
 *  @param[in]  priority              Update execution time for this priority only.
 *
 *  @param[in]  channel               Task channel
 *
 *  @param[in]  exec_time             Execution time of most recent task
 */
void siu_queue_update ( tuint max_number_channels, tuint numActiveChannels, tuint task_priority_ID, tuint channel, tuint exec_time )
{
  tuint i;
  siuTask_t *qptr;

  qptr = &siuTask[task_priority_ID];
  numActiveChannels -= 1;
  for (i = 0; i < max_number_channels; i ++) 
  {
    qptr->ch_exec_time[i] -= (qptr->ch_exec_time[i] >> SIU_TASK_PENALTY_FORGET);
    if(i == channel) {
      qptr->ch_exec_time[i] += (exec_time * numActiveChannels) >> SIU_TASK_PENALTY_FORGET;
    }
  }
}

/* ============ siu_queue_write() ========== */
/**
 *  @brief      Stores the channel number and task type that requires task processing.
 *
 *  @param[in]  max_number_channels   Maximum number of channels present in the build
 *
 *  @param[in]  channel               Task channel
 *
 *  @param[in]  task_priority_ID      Task priority queue
 *
 *  @param[in]  task_type             Task type
 *
 * Return values: -1 = No place in queue FIFO, channel dropped 
 *                 0 = Succesfull in writing to FIFO.
 */

tint siu_queue_write ( tuint max_number_channels, tuint channel ,void *pMsg,
                       tuint task_priority_ID, tuint task_type, tulong timestamp ) 
{
  tint  inIndx;
  siuTask_t *qPtr;
  
  qPtr    = &siuTask[task_priority_ID];
  inIndx  = qPtr->in_index;

  if(qPtr->cnt >= (SIU_TASK_QUEUE_LENGTH - 1))
  { /* Let application deal with no space in queue */
    return -1;
  }

  /**** Record task ******************/
  qPtr->queue[inIndx]     = channel;
  qPtr->pMsg[inIndx]     = pMsg;
  qPtr->queue_ts[inIndx]  = timestamp;
  qPtr->task_type[inIndx] = task_type;
  /***********************************/  
  inIndx ++;
  inIndx                = (inIndx == SIU_TASK_QUEUE_LENGTH) ? 0: inIndx;
  qPtr->in_index        = inIndx;
  qPtr->cnt ++;
  
#ifdef GG_BIOS
  /* The TxRxSwi processes all queues.  Since we put something into a queue,
   * post the task.  We can post for every queue write, but stacked writes
   * will cause this to only execute once.  Since our queues keep track of
   * what was posted, we will know what to execute once this stats.
   */
  siu_osal_post_signal(SIU_OSAL_SIGNAL_HANDLE_TXRXSWI);
#endif /* GG_BIOS */

  return(0); /* Write succesfull */
} /* siu_queue_write() */

/* ============ siu_queue_read() ========== */
/**
 *  @brief      Reports back the channel number that requires task processing for current task, if any channel requires processing.
 *              Task with the least timestamp is selected. This is done with exhaustive search.
 *
 *  @param[out]  channel     Maximum number of channels present in the build
 *
 *  @param[out]  task_type   Task type
 *
 *  @param[in]  task_priority_ID      Task priority queue
 *
 * Return values: TRUE  = Channel needs processing.
 *                FALSE = FIFO is empty, no channels need processing.
 */

bool siu_queue_read( tuint *channel, void **pMsg, tuint *task_type, tuint task_priority_ID )
{
  bool ChReady = FALSE;
  siuTask_t *qPtr;
  tulong min_ts, ch_ts;
  tint i, j;
  
  qPtr = &siuTask[task_priority_ID];
  /* is the queue not empty? */
  if (qPtr->in_index != qPtr->out_index) 
  { /* Find task with the oldest timestamp: this is the next task for execution */  
    i =  qPtr->out_index;
    min_ts = qPtr->queue_ts[i] + (tulong)qPtr->ch_exec_time[qPtr->queue[i]];
    j      = i;
    for ( ; ; )
    { /* Push back channels with high recent execution time */
      i++; 
      i = (i == SIU_TASK_QUEUE_LENGTH) ? 0: i;    
      if(i == qPtr->in_index) break;
      
      ch_ts = qPtr->queue_ts[i] + (tulong)qPtr->ch_exec_time[qPtr->queue[i]];
      if((tlong)(ch_ts - min_ts) < 0)
      {
        min_ts = ch_ts;
        j = i;
      }
    }
    *channel   = qPtr->queue[j]; 
    *task_type = qPtr->task_type[j];   
    *pMsg      = qPtr->pMsg[j];
    qPtr->pMsg[j]      = qPtr->pMsg[qPtr->out_index];
    qPtr->queue[j]     = qPtr->queue[qPtr->out_index];
    qPtr->queue_ts[j]  = qPtr->queue_ts[qPtr->out_index];    
    qPtr->task_type[j] = qPtr->task_type[qPtr->out_index];        

    qPtr->out_index ++;
    qPtr->out_index = (qPtr->out_index == SIU_TASK_QUEUE_LENGTH) ? 0: qPtr->out_index;      
    qPtr->cnt --;

    ChReady = TRUE;
  }  
  return(ChReady);
} /* siu_queue_read() */

/* ============ siu_queue_dump() ========== */
/**
 *  @brief      Dump internal queue variables
 *
 *  @param[in]  task_priority_ID      Task priority queue
 */

#ifdef SIU_TASK_VERBOSE
void siu_queue_dump (tuint task_priority_ID)
{
  siuTask_t *qPtr;
  tint i, j;
  
  qPtr = &siuTask[task_priority_ID];
  printf ("\n\t\tqPtr->in_index=%d qPtr->out_index=%d qPtr->cnt=%d", qPtr->in_index, qPtr->out_index, qPtr->cnt);
  for (i = 0; i < 32; i ++) {
     if(i % 16 == 0) printf ("\n[%2d]", i);
     printf ("%3d ", qPtr->ch_exec_time[i]);
  }

}
#endif

/* Nothing past this point */
