/******************************************************************************
 * FILE PURPOSE: SIU TX Task Management Routines for VIDEO
 ******************************************************************************
 * FILE NAME:   siuVigdkTxThread.c
 *
 * DESCRIPTION: SIU TX Task Management Routines for Video
 *
 * TABS: NONE
 *
 * (C) Copyright 2010, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include <stdlib.h>
#include <string.h>
#include "common/siu.h"
#include "common/siuloc.h"
#include "siuport.h"
#include "ggcfg.h"
#include "emureg.h"
#include <ti/xdais/dm/xdm.h>
#include "vctypes.h"
#include "siutsu.h"
#include "siuovly.h"
#include "siutftp.h"
#include "siummcu.h"
#include <ti/mas/iface/ifvisys/ifvisys.h>
#include "siuVigdkGmp.h"
#include "siuVigdkInterTxRx.h"
#include "ividmc/siuVidMc.h"


/*********************************************************************************
 * FUNCTION PURPOSE: Tx Task Post Function
 *********************************************************************************
  DESCRIPTION:      This function is called by ISR initiated by partner core.
  Parameters :      Inputs: siuInst         : pointer to siuInstance
                    Output: None
 *********************************************************************************/
void siuTxVideoPost (void *siuInst, void *pMsg)
{
  siuInst_t *inst        = (siuInst_t*)siuInst;
  tuint channel = ( (tuint) (inst->ID & 0xFF) - SIU_FIRSTCH );
  tuint task_priority_ID = SIU_READ_TASK_PRIORITY_ID(inst);

  /* post the task if we can */
  if (siu_queue_write (siuContext.maxNumberChannels, channel, pMsg, task_priority_ID, SIU_TASK_TYPE_TX, SIU_GET_TS))
  {
    siuContext.numLatePlayouts++;
    return;
  }
  /* Allow multiple pending tasks per channel */
  siuContext.numPendingTxVideoTasks++;
}


/*********************************************************************************
 * FUNCTION PURPOSE: Fetch AV buffer for Encoder Task
 *********************************************************************************
  DESCRIPTION:      This function is called to get a pointer to YUV or audio buffer. Appropriate
                    buffer is returned after checking if it is intra or inter core
                    transcoding
  Parameters :      Inputs: siuInst         : to extract chID information
                    Output: yuvFrame        : pointer to YUV buffer
                            yuvWidth        : width of the Image
                            yuvHeight       : height of the Image
                            intercore_conn  : -1 if intracore else intercore conn.
 *********************************************************************************/
static inline bool siuTxGetAVBuffer(siuInst_t *inst ,void *msg, word **avFrame, tuint *yuvWidth, tuint *yuvHeight,
                                          tulong *frameTimeStamp, tint *intercore_conn, ifvisys_MediaType_t *type, tulong *frameSize) {
  tint i, conn = -1;
  tint chnum = inst->ID & 0x00FF; /* 1-based */
  vigdkMsg_t *mailbox = &(siuContext.vigdkMsg);

  *intercore_conn = conn; /* initialize to -1, which means local core transcoding */

  /* remote core transcoding */
  for (i = 0; i < MAX_SIU_INTERCORE_CONN; i ++) {
    if (SIU_INTER_CORE_CONN_ENC_MATCH(i, siuContext.coreId, chnum-1)) {
      conn = i;
      break;
    }
  }
  *intercore_conn = conn;

  if(conn >= 0) { /* Intercore transcoding */
    vigdkMsg_t *pMsg = (vigdkMsg_t *)msg;
    thwCriticalBegin();
    if (pMsg == NULL) {
       thwCriticalEnd();
       return(FALSE);
    }
    siuContext.numPendingTxVideoTasks--;
    *type           = pMsg->frameType;
    *frameTimeStamp = (tulong)pMsg->frameTimeStamp;
    switch (*type) {
      case ifvisys_MEDIA_TYPE_VIDEO:
        *avFrame       = (word *)pMsg->frame.yuv.yuvBuffer;
        *yuvWidth       = (tuint)pMsg->frame.yuv.yuvWidth;
        *yuvHeight      = (tuint)pMsg->frame.yuv.yuvHeight;
        *frameSize      = (3*(*yuvHeight)*(*yuvWidth))>>1;
        break;
      case ifvisys_MEDIA_TYPE_AUDIO:
        *avFrame        = (word *)pMsg->frame.aud.audBuffer;
        *yuvWidth       = 0;
        *yuvHeight      = 0;
        *frameSize      = (tulong)pMsg->frame.aud.audSize;
        break;
    }
    /* Indicate that the message has been consumed */
    siu_ipc_osal_set_msg_consumed(pMsg);

    thwCriticalEnd();
  } else { /* local core transcoding */
    *type           = mailbox->frameType;
    *frameTimeStamp = (tulong)mailbox->frameTimeStamp;
    switch (*type) {
      case ifvisys_MEDIA_TYPE_VIDEO:
        *avFrame       = (word *)mailbox->frame.yuv.yuvBuffer;
        *yuvWidth       = (tuint)mailbox->frame.yuv.yuvWidth;
        *yuvHeight      = (tuint)mailbox->frame.yuv.yuvHeight;
        *frameSize      = (3*(*yuvHeight)*(*yuvWidth))>>1;
        break;
      case ifvisys_MEDIA_TYPE_AUDIO:
        *avFrame       = (word *)mailbox->frame.aud.audBuffer;
        *yuvWidth       = 0;
        *yuvHeight      = 0;
        *frameSize      = (tulong)mailbox->frame.aud.audSize;
        break;
    }
  }
  return(TRUE);
} /* siuTxGetAVBuffer() */


/*********************************************************************************
 * FUNCTION PURPOSE: Main Routine for SIU Transmit Task, runs per channel.
 *********************************************************************************
  DESCRIPTION:      This function operates set of Encoder ECOs. It runs in main loop
                    (background task). Tasks are posted by IPC interrupt service
                    routine
  Parameters :      Inputs: siuInst         : to extract chID information
                            chIndx          :
                    Output: None
 *********************************************************************************/
tint siuTxEncoderProcess (void *siuInst, tuint chIndx, void *pMsg)
{
  siuInst_t *inst = (siuInst_t *)siuInst;
  word      *avInputFrame;
  tuint     yuvInputWidth, yuvInputHeight;
  tulong    frameTimeStamp, frameSize;
  tint      intercore_conn, retval = SIU_NOERR;
  ifvisys_MediaType_t frameType;
  XDAS_Int32 xdmRetVal;
  
#ifdef EMU_PROFILE
  tuint chnum = ((inst->ID & 0xFF) - 1);
#endif

  if(inst->state == SIU_CHST_EXCEPTION) {
    return(0);
  }

  if (SIU_IS_MCCODEC_SLAVE(inst)) {
#ifdef EMU_PROFILE
    EMU_PROFILE_BEGIN(EMU_TASK_ID_VTK_ENCODE,chnum)
#endif
    xdmRetVal = siuTxVideoSlaveEncode(inst, pMsg);
#ifdef EMU_PROFILE
    EMU_PROFILE_END() /* EMU_TASK_ID_VTK_ENCODE */
#endif
    if (xdmRetVal == XDM_EFAIL) {
      retval = SIU_ERROR;
    }
    return(retval);
  }

  if (siuTxGetAVBuffer(inst, pMsg, &avInputFrame, &yuvInputWidth, &yuvInputHeight, &frameTimeStamp,
                             &intercore_conn, &frameType, &frameSize) == FALSE) {
    siu_exc_assert(0, SIUVIGDK_TX_GETYUV_ERROR|SIU_EXCEPTION_TYPE_CHANNEL, inst);
  }

  if( (inst->stateInst.video.mmcuInst != NULL) && !SIU_GET_MMCU_HDR_PARSED(inst)) 
    siu_mmcu_send_header(inst);

  switch (frameType) {
    case ifvisys_MEDIA_TYPE_VIDEO:
      retval = siuTxVideoEncoderProcess(inst,avInputFrame,yuvInputWidth, yuvInputHeight, frameTimeStamp);
      break;
    case ifvisys_MEDIA_TYPE_AUDIO:
      retval = siuTxAudioEncoderProcess(inst,avInputFrame,frameSize,frameTimeStamp);
      break;
    default:
      break;
  }

  /* Encoder has finished Encoding at this point. In case of core to core transcoding,    */
  /* YUV buffer has to be released back to the free pool so that decoder could re-use it  */
  /* This is done by raising an interrupt to the partner core                             */
  if( intercore_conn >= 0 ) {
    /* core to core transcoding */
      vigdkMsg_t msg;

      msg.frameType       = frameType;
      msg.frameTimeStamp  = frameTimeStamp;
      msg.srcMid          = SIU_MID_VENC;
      msg.dstMid          = SIU_MID_VDEC;

      if(frameType == ifvisys_MEDIA_TYPE_VIDEO) {
        msg.frame.yuv.yuvBuffer      = avInputFrame;
        msg.frame.yuv.yuvWidth       = yuvInputWidth;
        msg.frame.yuv.yuvHeight      = yuvInputHeight;
      } else if (frameType == ifvisys_MEDIA_TYPE_AUDIO) {
        msg.frame.aud.audBuffer      = avInputFrame;
        msg.frame.aud.audSize        = frameSize;
      }

      /* signal the partner core that YUV buffer is consumed */
      siu_signal_partner_core(intercore_conn, &msg);
  }

  /* return value is always the cost of what it takes to run this thread.
  this is used by the scheduler */
  return(0);
}

/* nothing past this point */
