/******************************************************************************
 * FILE PURPOSE: SIU TX Task Management Routines for VIDEO
 ******************************************************************************
 * FILE NAME:   siuVigdkTxVideo.c
 *
 * DESCRIPTION: SIU TX Task Management Routines for Video
 *
 * TABS: NONE
 *
 * (C) Copyright 2011, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include "common/siuloc.h"
#include <ti/mas/iface/ifvisys/ifvisys.h>
#include "siuVigdkInterTxRx.h"
#include "emureg.h"
#include <ti/xdais/dm/xdm.h>
#include "vctypes.h"
#include "siutsu.h"
#include "siuovly.h"
#include "siutftp.h"
#include "siummcu.h"

/*********************************************************************************
 * FUNCTION PURPOSE: Decoding call on a master core
 *********************************************************************************
  DESCRIPTION:      Master core sends message to participating slave cores and then
                    call vtkEncode to participate in multi-core encoding.

  Parameters :      Inputs: siuInst
                            encInput        : Encoder input params
                            encOutput       : Encoder output parmas
                    Output: None
 *********************************************************************************/
XDAS_Int32 siuTxVideoMasterEncode(void *siuInst, ifvisys_vidEncInput_t *vidEncInput,
                                                         ifvisys_vidEncOutput_t *vidEncOutput)
{
  vigdkMcMsg_t msg;
  siuInst_t *inst = (siuInst_t *)siuInst;
  XDAS_Int32 xdmRetVal;
  tulong totalSize;
  tulong maxTxFrameSize = siuContext.max_codec_frame_size;

  /* Master core sends decoding request to slave cores */
  msg.type = VIGDK_MC_MSG_ENC;
  msg.inputPtr = (word *)vidEncInput->bitStream.ptr;
  siu_signal_slave_core(inst, &msg);

  /* Wait for all the participating cores to be ready before proceeding */
  siuVidMc_Swbarr(siuContext.coreId, IVIDMC_SWBARR15, inst->mcInst.nCores);
  /* Record current working channel number so that siuVidMc_Map_Shmem
     can identify the right nCores and core team mapping*/
  siuContext.current_working_chnum = (inst->ID & SIU_CID_MASK) - SIU_FIRSTCH;
  /* Encode Process Call */
  xdmRetVal = gg_vtk_ctable.vtkEncode (inst->stateInst.video.vtkInst, vidEncInput, vidEncOutput);
  /* Wait for all the participating cores to be ready before proceeding */
  siuVidMc_Swbarr(siuContext.coreId, IVIDMC_SWBARR15, inst->mcInst.nCores);

  totalSize = vidEncOutput->bitStream.size + vidEncOutput->mvData.size + vidEncOutput->mbArray.size + vidEncOutput->gobArray.size;
  /* Ensure that the Frame produced by Encoder fits in the memory allocated                */
  siu_exc_assert((totalSize < maxTxFrameSize), SIUVPPU_EXC_TXFRAME_TOO_BIG|SIU_EXCEPTION_TYPE_CHANNEL, inst);

  return(xdmRetVal);

} /* siuTxVideoMasterEncode() */

/*********************************************************************************
 * FUNCTION PURPOSE: Encoding call on a slave core
 *********************************************************************************
  DESCRIPTION:      Slave core consumes message from master core and calls
                    vtkEncode to participate in multi-core encoding
  Parameters :      Inputs: siuInst
                            msg: message from master core
                    Output: None
 *********************************************************************************/
XDAS_Int32 siuTxVideoSlaveEncode(void *siuInst, void *msg)
{
    siuInst_t *inst = (siuInst_t *)siuInst;
    vigdkMcMsg_t *pMsg = (vigdkMcMsg_t *)msg;
    ifvisys_vidEncInput_t  vidEncInput;
    ifvisys_vidEncOutput_t vidEncOutput;
    XDAS_Int32 xdmRetVal;
    tulong totalSize;
    tulong maxTxFrameSize = siuContext.max_codec_frame_size;

    thwCriticalBegin();
    if (pMsg == NULL) {
       thwCriticalEnd();
       return(0);
    }

    /* initialize encoder input/output structure to zero */
    memset(&vidEncInput, 0, sizeof(ifvisys_vidEncInput_t));
    memset(&vidEncOutput, 0, sizeof(ifvisys_vidEncOutput_t));

    vidEncInput.bitStream.ptr = (word *)pMsg->inputPtr;
    /* transcodedFrame is in L2 (no core competition in dual-core transcoding) */
    vidEncOutput.bitStream.ptr  = siuContext.siu_codec_frame_scratch;
    vidEncOutput.bitStream.size = 0;

    /* Indicate that the message has been consumed */
    siu_ipc_osal_set_msg_consumed(pMsg);

    thwCriticalEnd();

    /* Wait for all the participating cores to be ready before proceeding */
    siuVidMc_Swbarr(siuContext.coreId, IVIDMC_SWBARR15, inst->mcInst.nCores);
    /* Record current working channel number so that siuVidMc_Map_Shmem
        can identify the right nCores and core team mapping*/
    siuContext.current_working_chnum = (inst->ID & SIU_CID_MASK) - SIU_FIRSTCH;
    /* Encode Process Call */
    xdmRetVal = gg_vtk_ctable.vtkEncode (inst->stateInst.video.vtkInst, &vidEncInput, &vidEncOutput);
    /* Write output bit stream information in codec shared memory */
    siuVidMc_Set_outputBitStreamInfo(siuContext.coreId, (XDAS_UInt8 *)vidEncOutput.bitStream.ptr, 
                                     (XDAS_Int32)vidEncOutput.bitStream.size);
    /* Wait for all the participating cores to be ready before proceeding */
    siuVidMc_Swbarr(siuContext.coreId, IVIDMC_SWBARR15, inst->mcInst.nCores);

    totalSize = vidEncOutput.bitStream.size + vidEncOutput.mvData.size + vidEncOutput.mbArray.size + vidEncOutput.gobArray.size;
    /* Ensure that the Frame produced by Encoder fits in the memory allocated                */
    siu_exc_assert((totalSize < maxTxFrameSize), SIUVPPU_EXC_TXFRAME_TOO_BIG|SIU_EXCEPTION_TYPE_CHANNEL, inst);

    return(xdmRetVal);

} /* siuTxVideoSlaveEncode() */


/******************************************************************************************
 * FUNCTION PURPOSE: To ship out YUV buffer from the TSU-only channel
 ******************************************************************************************
  DESCRIPTION:      This function is called to send out YUV buffer once a TSU-only channel
                    has a YUV buffer available. The parnter will be looked up in this
                    function and the YUV buffer is appropriately disposed.
  Parameters :      Inputs : siuInst        : to extract chID information
                             yuvBuffer      : YUV buffer to be shipped out
                             yuvWidth       : Width of the Image to be shipped out
                             yuvHeight      : Height of the Image to be shipped out
                             rxFrameTimeStamp: Timestamp
                    Outputs: retVal         : success/failure
 ******************************************************************************************/
tint siuVigdkShipoutTsuYUV(siuInst_t *inst, word *yuvBuffer, tuint yuvWidth, tuint yuvHeight, tulong rxFrameTimeStamp)
{
  vigdkMsg_t msg;
  tint retVal, remoteConn;
  siuInst_t *txInst;
  
  /* Retrieve partner information */
  siuVigdkGetPartner(inst, &remoteConn, &txInst);

  /* Form the message to be sent to the partner */
  msg.srcMid                  = SIU_MID_VDEC;
  msg.dstMid                  = SIU_MID_VENC;
  msg.frameType               = ifvisys_MEDIA_TYPE_VIDEO;
  msg.frame.yuv.yuvBuffer     = yuvBuffer;
  msg.frame.yuv.yuvWidth      = yuvWidth;
  msg.frame.yuv.yuvHeight     = yuvHeight;
  msg.frameTimeStamp          = rxFrameTimeStamp;

  /* change the ownership of the buffer so that decoder core doesn't free if being Reset */
  siuYuvBufChangeOwnership(&(inst->yuv), yuvBuffer, YUV_FRAME_NOT_OWNER);
  retVal = siu_signal_partner_core(remoteConn, &msg);
  
  /* we have to mark yuvBuffer = NULL to get a new buffer next time */
  inst->yuvBuffer = NULL;
  return (retVal);
} /* siuVigdkShipoutTsuYUV() */



void *vdebug_video_mux_input = NULL;
/*********************************************************************************
 * FUNCTION PURPOSE: Main Routine for SIU Transmit Task, runs per channel.
 *********************************************************************************
  DESCRIPTION:      This function operates set of Encoder ECOs. It runs in main loop
                    (background task). Tasks are posted by IPC interrupt service
                    routine
  Parameters :      Inputs: siuInst         : to extract chID information
                            chIndx          :
                    Output: None
 *********************************************************************************/
tint siuTxVideoEncoderProcess (siuInst_t *inst, word *yuvInputFrame, tuint yuvInputWidth, tuint yuvInputHeight, tulong frameTimeStamp)
{
  ifvisys_vidEncInput_t   vidEncInput;
  ifvisys_vidEncOutput_t  vidEncOutput;
  tulong frameTimeStampIncrement = 0, frameSize = 0;
  tint  retval = SIU_NOERR;
  tuint frcOut, codingType, i;
  word  *toDecodeFrame, *yuvTransSizedFrame, *frameStartPtr = NULL;
  XDAS_Int32 xdmRetVal;

#ifdef EMU_PROFILE
  tuint chnum = ((inst->ID & 0xFF) - 1);
#endif

  /* initialize encoder input/output structure to zero */
  memset(&vidEncInput, 0, sizeof(ifvisys_vidEncInput_t));
  memset(&vidEncOutput, 0, sizeof(ifvisys_vidEncOutput_t));

  codingType = SIU_GET_VIDEO_CODING_TYPE(inst);

  if (codingType != EVCT_VIDEO_BYPASS_ENC) {
     frcOut = siu_run_frc(inst, frameTimeStamp, &frameTimeStampIncrement);
  } else {
     /* Encoder FPS is supplied via Codec configuration. For Encoder Bypass, there is no Encoder.
     Hence, FRC will not be run. Every frame is used once ==> Dec.FrameRate = Enc.FrameRate */
     frcOut = 1;
  }

  if (frcOut == 0) {
     /* frame dropped */
     goto end;
  }

  /* Image post processing (TSU, overlay etc) is a CPU operation. Hence, cache has to be */
  /* invalidated to be completely coherent with YUV data in external memory              */
  /* do writeback/invalidate */
  siu_invalidate_cache();

  /*********************************************************************************/
  /* trans-sizing is not in-place operation. It needs input and output YUV buffers */
  /*********************************************************************************/
#ifdef EMU_PROFILE
  EMU_PROFILE_BEGIN(EMU_TASK_ID_TSU_TX,chnum)
#endif
  /* Check if TSU needs to be invoked */
  if ((yuvInputWidth != inst->yuvWidth) || (yuvInputHeight != inst->yuvHeight)) {
    retval = siu_tsu_process(inst, yuvInputFrame, yuvInputWidth, 
                             yuvInputHeight, &yuvTransSizedFrame);
  } else {
    retval = TSU_BYPASS;
  }

#ifdef EMU_PROFILE
  EMU_PROFILE_END() /* EMU_TASK_ID_TSU_TX */
#endif

  /* If the input resolution of YUV == requested resolution to be encoded, TSU returns    */
  /* TSU_BYPASS. In that case, use the inputFrame. If TSU did trans-sizing the transsized */
  /* image is available on the yuvTransSizedFrame buffer. In this case new buffer needs   */
  /* to be fed to the decoder                                                             */
  if (retval == TSU_BYPASS){
    toDecodeFrame = yuvInputFrame;
  } else if (retval != TSU_NOERR) {
    goto end;
  } else {
    toDecodeFrame = yuvTransSizedFrame;
  }

  if(SIU_GET_VIDEO_CHANNEL_EXT_TYPE(inst) == SIU_CHTYPE_EXT_TSU_ONLY) {
      /* TSU is complete at this point. Shipout the complete information about  */
      /* the YUV buffer. This includes, YUV buffer pointer, Image Width and Image height   */
      siu_invalidate_cache();
      //yuvOutputBuffer = yuvTransSizedFrame;
      retval = siuVigdkShipoutTsuYUV(inst, toDecodeFrame, inst->yuvWidth, inst->yuvHeight, frameTimeStamp);
      goto end;
  }


#ifdef EMU_PROFILE
  EMU_PROFILE_BEGIN(EMU_TASK_ID_CUSTOM_ECO,chnum)
#endif

  /**********************************/
  /* Overlay is in-place operation. */
  /**********************************/
  siuOvlyProcess (inst, toDecodeFrame, inst->yuvWidth, inst->yuvHeight);
#ifdef EMU_PROFILE
  EMU_PROFILE_END() /* EMU_TASK_ID_CUSTOM_ECO */
#endif

  /* Video Encoding uses DMA paging to transfer YUV data in-out of DDR. Hence, cache has  */
  /* to be written back and invalidated so that Encoder picks up the post processed data  */
  /* do writeback or invalidate */
  siu_invalidate_cache();

  while (frcOut > 0) {
    if (codingType != EVCT_VIDEO_BYPASS_ENC) {
      /* Encode */
#ifdef EMU_PROFILE
  EMU_PROFILE_BEGIN(EMU_TASK_ID_VTK_ENCODE,chnum)
#endif
      vidEncInput.bitStream.ptr = toDecodeFrame;
      /* transcodedFrame is in L2 (no core competition in dual-core transcoding) */
      vidEncOutput.bitStream.ptr  = siuContext.siu_codec_frame_scratch;
      vidEncOutput.bitStream.size = 0;

      xdmRetVal = siuTxVideoMasterEncode(inst, &vidEncInput, &vidEncOutput);

#ifdef EMU_PROFILE
  EMU_PROFILE_END() /* EMU_TASK_ID_VTK_ENCODE */
#endif
      if (xdmRetVal == XDM_EFAIL) {
        retval = SIU_ERROR;
        goto end;
      }
    } else {
      /* ENCODER BYPASS */
      vidEncOutput.bitStream.ptr  = toDecodeFrame;
      vidEncOutput.bitStream.size = ((inst->yuvWidth * inst->yuvHeight *3)>>1);
      vidEncOutput.frameType      = IFVISYS_I_FRAME;
    }

#ifdef EMU_PROFILE
  EMU_PROFILE_BEGIN(EMU_TASK_ID_VPPU_TX,chnum)
#endif
  /* Ship out the Transcoded Frame on master core */
    siu_invalidate_cache();

    {
      XDAS_UInt8 *slaveOutputBitStreamPtr;
      XDAS_Int32 slaveOutputBitStreamSize;

      frameStartPtr = vidEncOutput.bitStream.ptr;
      frameSize     = vidEncOutput.bitStream.size;
      for (i=1; i<inst->mcInst.nCores; i++) {
        /* Read slave cores' output bit stream information from codec shared memory */
        siuVidMc_Get_outputBitStreamInfo(inst->mcInst.coreTeamMapping[i],
                                         &slaveOutputBitStreamPtr, &slaveOutputBitStreamSize);
        if ((tulong)(frameSize + slaveOutputBitStreamSize) > siuContext.max_codec_frame_size) {
            /* Ensure that the Frame assemblied from all the cores fits in the memory allocated */
            /* Currently disable exception since the system scratch for all the cores are contiguous
               and can be used for master core's frame assembly. Need to revisit this with better 
               memory management scheme */
            //siu_exc_assert(0, SIUVPPU_EXC_TXFRAME_TOO_BIG|SIU_EXCEPTION_TYPE_CHANNEL, inst);
        }
        if( (slaveOutputBitStreamPtr!=NULL) && (slaveOutputBitStreamSize > 0)) {
            memcpy(&frameStartPtr[frameSize], slaveOutputBitStreamPtr, slaveOutputBitStreamSize);
            frameSize += slaveOutputBitStreamSize;
        }
      }
      vidEncOutput.bitStream.ptr = frameStartPtr;
      vidEncOutput.bitStream.size = frameSize;
    }

    if(vidEncOutput.bitStream.size) {
      if(inst->stateInst.video.mmcuInst != NULL) {
        ifmmcMuxInput_t muxInput;

        muxInput.bitStream.ptr  = vidEncOutput.bitStream.ptr;
        muxInput.bitStream.size = vidEncOutput.bitStream.size;
        muxInput.frameType      = vidEncOutput.frameType;
        muxInput.mediaType      = ifvisys_MEDIA_TYPE_VIDEO;
        muxInput.ESID           = SIU_VIDEO_PID;

        vdebug_video_mux_input = (void *)&muxInput;

        if(frameTimeStampIncrement == 0xFFFFFFFF)
          muxInput.timestampIncrement = frameTimeStamp;
        else
          muxInput.timestampIncrement = frameTimeStampIncrement;

        mmcuSendIn(inst->stateInst.video.mmcuInst,&muxInput);

      } else if(SIU_IS_PROTO_TFTP(inst)) {
        siu_put_tftp(inst, frameStartPtr, frameSize);
      } else {
        gg_vppu_ctable.vppuSendIn (inst->stateInst.video.vppuInst, &vidEncOutput, frameTimeStampIncrement);
      }

      siu_frc_update_rtp_ts (&inst->frcInst, frameTimeStamp);
    }

#ifdef EMU_PROFILE
  EMU_PROFILE_END() /* EMU_TASK_ID_VPPU_TX */
#endif

    frcOut--;
  }

end:
  return (retval);
} /* siuTxVideoEncoderProcess() */

/* nothing past this point */
