/******************************************************************************
 * FILE PURPOSE: SIU Framework Routines for VIDEO
 ******************************************************************************
 * FILE NAME:   siuVigdkYUV.c
 *
 * DESCRIPTION: Includes YUV buffer management for Vigdk Application
 *
 * TABS: NONE
 *
 * (C) Copyright 2009, Texas Instruments Incorporated.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include "common/siuloc.h"
#include "mgmicdsp.h"
#include "siuVigdkGmp.h"

/*********************************************************************************
 * FUNCTION PURPOSE: Initialize YUV instance
 *********************************************************************************
  DESCRIPTION:      This function is called to initialize yuv Instance
  Parameters :      Inputs: inst            : siuInstance pointer
                            imgWidth        : Image width
                            imgHeight       : Image height
                    Output: None
 *********************************************************************************/
void siu_init_yuv (siuInst_t *inst, tuint imgWidth, tuint imgHeight, word reorderDepth) 
{
  tint i;
  siuVideoChannelOperation_t chType = (siuVideoChannelOperation_t)SIU_GET_VIDEO_CHANNEL_TYPE(inst);
  siuVideoChannelOperationExt_t chTypeExt = (siuVideoChannelOperationExt_t)SIU_GET_VIDEO_CHANNEL_EXT_TYPE(inst);

  siuInstYUV_t *yuvInst = &(inst->yuv);

  yuvInst->yuvReorderDepth = reorderDepth;

  /* allocate YUV buffers */
  if ((chType == SIU_CHTYPE_DECODER) || (chTypeExt == SIU_CHTYPE_EXT_TSU_ONLY)) {
    yuvInst->yuvFrameSize   = (imgWidth * imgHeight * 3)>>1;
    for (i=0;i<yuvInst->yuvReorderDepth;i++) {
      (yuvInst->yuvFrame[i]).yuvFramePtr = (word *)gmpPartialAlloc(inst->ID, siuContext.siuYuvMemHandle, yuvInst->yuvFrameSize);
      siu_exc_assert (((yuvInst->yuvFrame[i]).yuvFramePtr != NULL), SIUVIGDK_EXC_NEW_YUV | SIU_EXCEPTION_TYPE_CHANNEL, inst);
    }
  } else {
    /* YUV buffer is owned by Decoder */
    for (i=0;i<yuvInst->yuvReorderDepth;i++) {
      (yuvInst->yuvFrame[i]).yuvFramePtr = NULL;
    }
  }

  yuvInst->yuvInputID = yuvInst->yuvOutputID = 0;
  for (i=0;i<yuvInst->yuvReorderDepth;i++) {
    yuvInst->yuvFrame[i].yuvFrameStatus = SIU_YUV_FRAME_FREE;
  }
  yuvInst->yuvDisplayFrameCounter = 0;
  yuvInst->yuvNumFreeBufs         = yuvInst->yuvReorderDepth;

  /* store imgWidth and imgHeight in SIU */
  inst->yuvWidth  = imgWidth;
  inst->yuvHeight = imgHeight;
  inst->yuvDropCtr= 0;
  inst->yuvFlushCtr=0;
  inst->yuvBuffer = NULL;
}

/*********************************************************************************
 * FUNCTION PURPOSE: Close YUV instance
 *********************************************************************************
  DESCRIPTION:      This function is called to close yuv Instance
  Parameters :      Inputs: inst            : siuInstance pointer
                    Output: None
 *********************************************************************************/
void siu_close_yuv(siuInst_t *inst) 
{
  tuint i;
  siuInstYUV_t *yuvInst = &(inst->yuv);

  for(i=0;i<yuvInst->yuvReorderDepth;i++) {
    /* checking for yuvFramePtr != NULL because a channel close should work while recovering from 
       channel exception raised when only a few of the YUV frames were successfully allocated */
    if((yuvInst->yuvFrame[i]).yuvFramePtr != NULL){
       gmpFreeGmc(inst->ID,siuContext.siuYuvMemHandle,(yuvInst->yuvFrame[i]).yuvFramePtr);
    }
    (yuvInst->yuvFrame[i]).yuvFramePtr = NULL;
  }
  memset(yuvInst, 0, sizeof(siuInstYUV_t));
  inst->yuvBuffer = NULL;
}

/*********************************************************************************
 * FUNCTION PURPOSE: Get a free YUV buffer
 *********************************************************************************
  DESCRIPTION:      This function is called before calling the decoder to get a 
                    free YUV buffer
  Parameters :      Inputs: inst            : siuInstance pointer
                    Output: numFreeBufs     : Number of free buffers remaining after
                                              grabbing one free buffer (will be zero),
                                              when the last free buffer is picked up
 *********************************************************************************/
tint siuYuvGetFreeBuf(siuInst_t *inst)
{
  int i, pos, numFreeBufs;
  int free_buf_pos = -1;
  siuInstYUV_t *yuvInst = &(inst->yuv);
  word buf_pos;

  thwCriticalBegin();
  buf_pos = yuvInst->yuvDisplayFrameCounter;
  for (i=0; i<yuvInst->yuvReorderDepth; i++)
  {
    pos = (buf_pos + i) % yuvInst->yuvReorderDepth;
    if (yuvInst->yuvFrame[pos].yuvFrameStatus == SIU_YUV_FRAME_FREE)
    {
      free_buf_pos = pos;
      break;
    }
  }
  
  siu_exc_assert ((free_buf_pos != -1), SIUVIGDK_EXC_YUV_ALLOC_FAIL | SIU_EXCEPTION_TYPE_CHANNEL, inst);
  
  yuvInst->yuvDisplayFrameCounter = (word)free_buf_pos;
  siuYuvSetBufBusy(inst);
  numFreeBufs = yuvInst->yuvNumFreeBufs;

  thwCriticalEnd();
  return(numFreeBufs);
}

/*********************************************************************************
 * FUNCTION PURPOSE: Fetch YUV buffer for Decoder Task
 *********************************************************************************
  DESCRIPTION:      This function is called to get a pointer to YUV buffer. Appropriate 
                    YUV buffer is returned after checking if it is intra or inter core
                    transcoding
  Parameters :      Inputs: siuInst         : to extract chID information
                    Output: yuvFrame        : pointer to YUV buffer
                    Returns: TRUE - to skip encode, no free YUV; FALSE - to do encode
 *********************************************************************************/
bool siuVideoGetYUVBuffer(siuInst_t *inst) 
{
  bool retVal = FALSE;
  tint numFreeBufs;
  siuInstYUV_t *yuvInst = &(inst->yuv);

  thwCriticalBegin();

  if(inst->yuvBuffer != NULL) {
    /* We are re-using a YUV buffer which was allocated earlier */
    /* skip or don't skip encoder call must be decided looking at number of free buffers */
    if (inst->yuv.yuvNumFreeBufs == 0) {
       retVal = TRUE;
    }
    goto end;
  }

  numFreeBufs          = siuYuvGetFreeBuf(inst);
  inst->yuvBuffer      = (yuvInst->yuvFrame[inst->yuv.yuvDisplayFrameCounter]).yuvFramePtr;
  inst->yuv.yuvInputID = inst->yuv.yuvDisplayFrameCounter + 1;
  inst->yuv.yuvDisplayFrameCounter++;

  if(numFreeBufs == 0) {
    /* current YUV buffer is successfully allocated but there are no more 
       YUV buffers left for future. Hence, we need to skip Encode
     */
    retVal = TRUE;
  }
  
end:
  thwCriticalEnd();
  return(retVal);
}

/*********************************************************************************
 * FUNCTION PURPOSE: Set a free YUV buffer busy
 *********************************************************************************
  DESCRIPTION:      This function is called to mark a YUV buffer busy
  Parameters :      Inputs: inst            : siuInstance pointer
                    Output: None
 *********************************************************************************/
void siuYuvSetBufBusy(siuInst_t *inst)
{
  siuInstYUV_t *yuvInst = &(inst->yuv);

  siu_exc_assert ((yuvInst->yuvFrame[yuvInst->yuvDisplayFrameCounter].yuvFrameStatus != SIU_YUV_FRAME_BUSY), \
                   SIUVIGDK_EXC_YUV_SETBUSY_FAIL | SIU_EXCEPTION_TYPE_CHANNEL, inst);
  
  yuvInst->yuvFrame[yuvInst->yuvDisplayFrameCounter].yuvFrameStatus    = SIU_YUV_FRAME_BUSY;
  yuvInst->yuvFrame[yuvInst->yuvDisplayFrameCounter].yuvFrameOwnership = YUV_FRAME_OWNER;
  yuvInst->yuvNumFreeBufs--;
  return;
}

/*********************************************************************************
 * FUNCTION PURPOSE: Set a free YUV buffer free
 *********************************************************************************
  DESCRIPTION:      This function is called to mark a YUV buffer as free
  Parameters :      Inputs: inst            : siuInstance pointer
                            yuvBuffer       : YUV buffer pointer to be freed
                    Output: None
 *********************************************************************************/
void siuYuvSetBufFree(siuInst_t *inst, word *yuvBuffer)
{
  siuInstYUV_t *yuvInst = &(inst->yuv);
  word buf_pos  = 0;

  while(buf_pos <= yuvInst->yuvReorderDepth) {
    if(yuvInst->yuvFrame[buf_pos].yuvFramePtr == yuvBuffer) {
       break;
    }
    buf_pos++;
  }

  siu_exc_assert ((buf_pos < yuvInst->yuvReorderDepth), \
                   SIUVIGDK_EXC_YUV_FREE_FAIL1 | SIU_EXCEPTION_TYPE_CHANNEL, inst);

  siu_exc_assert ((yuvInst->yuvFrame[buf_pos].yuvFrameStatus == SIU_YUV_FRAME_BUSY), \
                   SIUVIGDK_EXC_YUV_FREE_FAIL2 | SIU_EXCEPTION_TYPE_CHANNEL, inst);

  yuvInst->yuvFrame[buf_pos].yuvFrameStatus = SIU_YUV_FRAME_FREE;
  yuvInst->yuvNumFreeBufs++;
}

/*********************************************************************************
 * FUNCTION PURPOSE: Change the Buffer ownership
 *********************************************************************************
  DESCRIPTION:      This function is called to change the ownership when the buffer
                    is handed over to a different core
  Parameters :      Inputs: yuvInst          : yuvInstance pointer
                            buffer           : Buffer for which ownership needs to changed
                            ownership        : New ownership
                    Output: None
 *********************************************************************************/
void siuYuvBufChangeOwnership(siuInstYUV_t *yuvInst, word *buffer, tint ownership) 
{
  word buf_pos  = 0;
  while(buf_pos <= yuvInst->yuvReorderDepth) {
    if(yuvInst->yuvFrame[buf_pos].yuvFramePtr == buffer) {
       break;
    }
    buf_pos++;
  }
  yuvInst->yuvFrame[buf_pos].yuvFrameOwnership = ownership;
}

/*********************************************************************************
 * FUNCTION PURPOSE: Reset YUV buffers on a Decoder Channel
 *********************************************************************************
  DESCRIPTION:      This function is called to free buffers owned by the channel
  Parameters :      Inputs: inst            : siuInstance pointer
                    Output: None
 *********************************************************************************/
void siuYuvBufPoolReset(siuInst_t *inst)
{
  siuInstYUV_t *yuvInst = &(inst->yuv);
  word buf_pos  = 0;
  while(buf_pos <= yuvInst->yuvReorderDepth) {
    /* If the channel owns the buffer then only it can be freed as part of Reset */
    if(yuvInst->yuvFrame[buf_pos].yuvFrameOwnership == YUV_FRAME_OWNER) {
      /* If the buffer is not free, then only it can be freed as part of Reset */
      if(yuvInst->yuvFrame[buf_pos].yuvFrameStatus != SIU_YUV_FRAME_FREE) {
         siuYuvSetBufFree(inst, yuvInst->yuvFrame[buf_pos].yuvFramePtr);         
      }
    }
    buf_pos++;
  }
  
  /* we have to mark yuvBuffer = NULL to get a new buffer next time */
  inst->yuvBuffer = NULL;
}

/*********************************************************************************
 * FUNCTION PURPOSE: Get Number of Free YUV buffers
 *********************************************************************************
  DESCRIPTION:      This function is called to get number of free YUV buffers
  Parameters :      Inputs: inst     : SIU Instance
                    Output: None
 *********************************************************************************/
tbool siuYuvAreAllBufsFlushed(siuInst_t *inst)
{
  siuInstYUV_t *yuvInst = &(inst->yuv);
  tuint i;
  tbool retVal = TRUE;

  for (i=0; i<yuvInst->yuvReorderDepth; i++)
  {
    if ((yuvInst->yuvFrame[i].yuvFrameOwnership == YUV_FRAME_OWNER) &&
        (yuvInst->yuvFrame[i].yuvFrameStatus == SIU_YUV_FRAME_BUSY))
    {
      retVal = FALSE;
      break;
    }
  }
  return(retVal);
}
/******************************************************************************
 * FUNCTION PURPOSE: Returns YUV statistics
 ******************************************************************************
 * DESCRIPTION: 
 ******************************************************************************/
void siuYuvGetStats (void *siuInst, bool reset, tuint trans_id, tuint priority)
{
  siuInst_t *inst = (siuInst_t *)siuInst;
  struct ifyuv_GET_STAT   *yuvStats;
  siuInstYUV_t *yuvInst = &(inst->yuv);
  mgb_t send_message;

  memset (&send_message, 0, sizeof(mgb_t));  
  send_message.header.length   = MGB_HEADER_SIZE + sizeof(struct ifyuv_GET_STAT)
                                               / sizeof(UINT16);
  send_message.header.chnum    = (int)(inst->ID & SIU_CID_MASK);
  send_message.header.msg_id   = MGB_DM_GET_YUV_STAT;
  send_message.header.trans_id = trans_id;

  yuvStats                       = &(send_message.msg_body.yuv_stat);
  yuvStats->yuvWidth             = inst->yuvWidth;
  yuvStats->yuvHeight            = inst->yuvHeight;
  yuvStats->yuvReorderDepth      = yuvInst->yuvReorderDepth;
  yuvStats->numDroppedFramesMSW  = siu_get_umsw(inst->yuvDropCtr);
  yuvStats->numDroppedFramesLSW  = siu_get_ulsw(inst->yuvDropCtr);

  if(reset) { /* Reset stats */
     inst->yuvDropCtr  = 0;
     inst->yuvFlushCtr = 0;
  }

  /* send message to FIFO */
  siuNduSendMsgPriority(&send_message, (NDU_DPRMsgType_t)priority); 
} /* siuFrcGetStats */

/*********************************************************************************
 * FUNCTION PURPOSE: Look up ID of YUV buffer
 *********************************************************************************
  DESCRIPTION:      
  Parameters :      Inputs: inst            : siuInstance pointer
                            yuvFramePtr     : pointer of YUV buffer 
                    Output: ID of YUV buffer if found; -1 otherwise
 *********************************************************************************/
tint siuLookupYUVID(siuInst_t *inst, word *yuvFramePtr) 
{
  tint i;
  siuInstYUV_t *yuvInst = &(inst->yuv);

  for(i=0; i<yuvInst->yuvReorderDepth; i++) {
      if(((tulong)yuvFramePtr > (tulong)(yuvInst->yuvFrame[i].yuvFramePtr)) && 
         ((tulong)yuvFramePtr < (tulong)(yuvInst->yuvFrame[i].yuvFramePtr) + yuvInst->yuvFrameSize))
         break;      
  }
  siu_exc_assert (i < yuvInst->yuvReorderDepth, SIUVIGDK_EXC_YUV_ALLOC_FAIL | SIU_EXCEPTION_TYPE_CHANNEL, inst);
  return (i);
}

/* nothing past this point */
