/******************************************************************************
 * FILE PURPOSE: Integrate H264 BP Encoder in GDK
 ******************************************************************************
 * FILE NAME:   vtk_h264_enc.c
 *
 * DESCRIPTION: This file contains the implementation for integrating H.264 BP 
 *              Encoder in VIDEO_GDK
 *              
 * FUNCTION           DESCRIPTION
 * --------           ----------- 
 *
 * (C) Copyright 2009, Texas Instruments Inc. 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include "vtk.h"
#include "vtk_h264_enc.h"
#include "string.h"

#pragma DATA_SECTION (H264BP_ENC_BUFFER_BIT_MAP, ".codecBitMaps");
tulong H264BP_ENC_BUFFER_BIT_MAP = 0x00001F50;

XDAS_Int32 vtkCreateH264encoder (void *sysInst, IH264VENC_Params *h264EncStaticParams, 
                              IALG_Handle *handle) {

  if ((*handle =  (IALG_Handle)vtkContext.ALG_Create (sysInst,
      (IALG_Fxns *) &H264VENC_TI_IH264VENC, 
      (IALG_Handle) NULL, 
      (IALG_Params *) h264EncStaticParams,&H264BP_ENC_BUFFER_BIT_MAP)) == NULL) {
      return XDM_EFAIL;
  }
  return XDM_EOK;
}

void vtkReduceMipsH264encoder (void *dyn_par, void *saved_pars)
{
  IH264VENC_DynamicParams *h264enc_dynpar = (IH264VENC_DynamicParams *)dyn_par;
  XDAS_Int32 *saved_parameters = (XDAS_Int32 *)saved_pars;
  /* Save original configuration */
     *saved_parameters ++ = h264enc_dynpar->lfDisableIdc;
     *saved_parameters ++ = h264enc_dynpar->intra4x4EnableIdc;
     *saved_parameters ++ = h264enc_dynpar->quartPelDisable;
     h264enc_dynpar->lfDisableIdc     = 1;  /* Deblocking susspended */
     h264enc_dynpar->intra4x4EnableIdc = 0; /* Intra 4x4 susspended  */
     h264enc_dynpar->quartPelDisable   = 1; /* Disable qpel */              
}

void vtkRestoreMipsH264encoder (void *dyn_par, void *saved_pars)
{
  IH264VENC_DynamicParams *h264enc_dynpar = (IH264VENC_DynamicParams *)dyn_par;
  XDAS_Int32 *saved_parameters = (XDAS_Int32 *)saved_pars;

    h264enc_dynpar->lfDisableIdc      = *saved_parameters++;
    h264enc_dynpar->intra4x4EnableIdc = *saved_parameters++;
    h264enc_dynpar->quartPelDisable   = *saved_parameters++;
}

/* Steps to do H.264 Codec Internal MIPS profiling */
/* 1. Enable #define VTK_PRINT_CODEC_INTERNAL_MIPS in vtkloc.h */
/* 2. Make sure that TSCL is active (by setting EMUPROF=FULL or PROF_IDLE_MIPS=YES and begin profiling */
/* 3. Critical Section protect vtkEncode() by enabling #define VTK_PROFILE_CODEC_INTERNAL_MIPS in siuVigdkTxRxThread.c */
/* 4. Compile codec library by defining #define BENCH_FILE and #define TIMER_ON in the file
      VIP_Video\h264\encoder\source\isa\c64x\inc\benchmarks.h */
/* 5. Replace the H.264 Encoder library in GDK w/ the new library & test */
/* 6. siuDebugInfo's come out of DSP per every frame, which indicate the cycles spent for different encoder blocks */

#ifdef VTK_PRINT_CODEC_INTERNAL_MIPS 
/* Timer Variables */
unsigned int wait_counter;
long long t0_wait, t1_wait;
unsigned int total_wait_time;
unsigned int curr_wait_time;
unsigned int min_wait;
unsigned int max_wait;

unsigned int subSampledDataTransferWaitTime ;
unsigned int subsampledTransferToIMX ;
unsigned int curDataDmaWait;
unsigned int refMVTransferToSeqWaitTime;
unsigned int mvDataTranfserFromSeqWaitTime ;
unsigned int last4lineTransferAfterDeblock ;
unsigned int last4lineTransferBeforeDeblock ;
unsigned int lumaRefDataTransferWaitTime ;
unsigned int CbRefDataTransferWaitTime ;
unsigned int CrRefDataTransferWaitTime ;
unsigned int verticallineTransferBeforeDeblock ;
unsigned int reconDataTransferWaitTime ;
unsigned int seq_wait_time_total ;
unsigned int intra4x4_totalTime ;
unsigned int intra4x4_Time0 ;
unsigned int intra4x4_Time1 ;
unsigned int intra4x4_Time2 ;
unsigned int H264VENC_TI_tpre_me;
unsigned int H264VENC_TI_tsrcdmawait1;


unsigned int fractional_pel_time;
unsigned int inter_proc_time;
unsigned int intra_proc_time;
unsigned int mbhdr_cavlc_time;
unsigned int dblk_time;
unsigned int only_dblk_time;
unsigned int preMeTime;
unsigned int preme_execute_time;
unsigned int preme_wait_time;
unsigned int before_loop_time;
unsigned int in_loop_time;
unsigned int after_loop_time;
unsigned int refaddr_calc_time;
unsigned int dblk_nonkrnl_time;
unsigned int dblk_krnl_time;

extern cregister volatile unsigned int TSCL;
extern cregister volatile unsigned int TSCH;

long long TSC_read() {
  return (_itoll(TSCH, TSCL));
}

void ResetTimerVariables(void)
{
  /* Timer value before start of encoding */
  wait_counter = 0;
  t0_wait= 0 ; t1_wait= 0;
  total_wait_time =0 ;
  curr_wait_time = 0;
  min_wait = 0;
  max_wait = 0;

  subSampledDataTransferWaitTime   = 0;
  subsampledTransferToIMX          = 0;
  curDataDmaWait                   = 0;
  refMVTransferToSeqWaitTime       = 0;
  mvDataTranfserFromSeqWaitTime    = 0;
  last4lineTransferAfterDeblock    = 0;
  last4lineTransferBeforeDeblock   = 0;
  lumaRefDataTransferWaitTime      = 0;
  CbRefDataTransferWaitTime        = 0;
  CrRefDataTransferWaitTime        = 0;
  verticallineTransferBeforeDeblock= 0;
  reconDataTransferWaitTime        = 0;

  preMeTime =0;
  fractional_pel_time = 0;
  inter_proc_time= 0;
  intra_proc_time = 0;
  mbhdr_cavlc_time = 0;
  dblk_time = 0;
  only_dblk_time = 0 ;
  preme_execute_time = 0 ;
  preme_wait_time = 0 ;
  before_loop_time = 0 ;
  in_loop_time = 0 ;
  after_loop_time = 0 ;
  refaddr_calc_time = 0 ;
  dblk_nonkrnl_time=0;
  dblk_krnl_time=0; seq_wait_time_total = 0;
  intra4x4_totalTime  = 0;
  intra4x4_Time0      = 0;
  intra4x4_Time1      = 0;
  intra4x4_Time2      = 0;

}

tuint vDebugFrameNum=0;
void PrintTimerVariables(void)
{
  extern unsigned int preMeTime;
  extern unsigned int fractional_pel_time;
  extern unsigned int inter_proc_time;
  extern unsigned int intra_proc_time;
  extern unsigned int mbhdr_cavlc_time;
  extern unsigned int only_dblk_time;
  extern unsigned int dblk_time;
  extern unsigned int preme_execute_time;
  extern unsigned int preme_wait_time;
  extern unsigned int before_loop_time;
  extern unsigned int in_loop_time;
  extern unsigned int after_loop_time;
  extern unsigned int refaddr_calc_time;
  siuDebugInfo(0xAAA1,0,vDebugFrameNum,2,(tuint *)&preMeTime);
  siuDebugInfo(0xAAA2,0,vDebugFrameNum,2,(tuint *)&fractional_pel_time);
  siuDebugInfo(0xAAA3,0,vDebugFrameNum,2,(tuint *)&inter_proc_time);
  siuDebugInfo(0xAAA4,0,vDebugFrameNum,2,(tuint *)&intra_proc_time);
  siuDebugInfo(0xAAA5,0,vDebugFrameNum,2,(tuint *)&mbhdr_cavlc_time);
  siuDebugInfo(0xAAA6,0,vDebugFrameNum,2,(tuint *)&only_dblk_time);
  siuDebugInfo(0xAAA7,0,vDebugFrameNum,2,(tuint *)&dblk_time);
  siuDebugInfo(0xAAA8,0,vDebugFrameNum,2,(tuint *)&preme_execute_time);
  siuDebugInfo(0xAAA9,0,vDebugFrameNum,2,(tuint *)&preme_wait_time);
  siuDebugInfo(0xAAAB,0,vDebugFrameNum,2,(tuint *)&before_loop_time);
  siuDebugInfo(0xAAAC,0,vDebugFrameNum,2,(tuint *)&in_loop_time);
  siuDebugInfo(0xAAAD,0,vDebugFrameNum,2,(tuint *)&after_loop_time);
  siuDebugInfo(0xAAAE,0,vDebugFrameNum,2,(tuint *)&refaddr_calc_time);
  vDebugFrameNum++;
}
#endif

/* nothing past this point */
