#ifndef _VTK_H
#define _VTK_H
/******************************************************************************
 * FILE PURPOSE: Defines API and public data definitions for VTK
 ******************************************************************************
 * FILE NAME: vtk.h  
 *
 * DESCRIPTION: Defines API and public data definitions for vtk module
 *
 * (C) Copyright 2008, Texas Instruments Inc. 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

/* System level header files */
#include <ti/mas/types/types.h>
#include "utlxferpkt.h"
#include "utldbg.h"
#include "memmgm.h"
#include "util.h"  /* bytes to words conversion macros */
#include "utlbitfield.h"
#include "xdc/std.h"
#include "ti/xdais/ialg.h"
#include <ti/mas/iface/ifvisys/ifvisys.h>
#include "ividmc.h"
#include "ti/xdais/dm/ividenc1.h"

/* VTK error codes */
enum {
  VTK_NOERR    = 0,
  VTK_ERROR    = 1,
  VTK_NOMEMORY = 2,
  VTK_SKIP_PROCESS = 3
};

/* Exception Messages */
enum {
   VTK_EXC_OPEN                 = 1, /* Opened an already open channel */
   VTK_EXC_DELETE               = 2, /* Deleted an open instance */
   VTK_EXC_CONTROL              = 3, /* Control request received by a closed channel */
   VTK_EXC_INVD_CODEC           = 4,
   VTK_EXC_INVD_IMG_HEIGHT      = 5,
   VTK_EXC_INVD_IMG_WIDTH       = 6,
   VTK_EXC_DECODE               = 7,
   VTK_EXC_ENCODE               = 8,
   VTK_EXC_RESOLUTION_MISMATCH  = 9,
   VTK_EXC_ALLOCHEAP            =10,
   VTK_EXC_ALLOCSCRATCH         =11,
   VTK_EXC_RX_RESOLUTION_EXCEEDED = 12
};

typedef enum {
  VTK_I_FRAME =0, 
  VTK_P_FRAME, 
  VTK_B_FRAME, 
  VTK_IDR_FRAME,
  VTK_II_FRAME,
  VTK_IP_FRAME,
  VTK_IB_FRAME,
  VTK_PI_FRAME,
  VTK_PP_FRAME,
  VTK_PB_FRAME,
  VTK_BI_FRAME,
  VTK_BP_FRAME,
  VTK_BB_FRAME,
  VTK_MBAFF_I_FRAME,
  VTK_MBAFF_P_FRAME,
  VTK_MBAFF_B_FRAME,
  VTK_MBAFF_IDR_FRAME
}vtkFrameType_t;

/******************************************************************************
 * DATA DEFINITION: VTK global parameters
 *****************************************************************************/
/******************************************************************************
 * DATA DEFINITION: Get VTK Instance
 *****************************************************************************/
/******************************************************************************
 * DATA DEFINITION:  Parameters which determine buffer sizes
 *****************************************************************************/
typedef struct {
  tint dummy;
} vtkSizeCfg_t;

/******************************************************************************
 * DATA DEFINITION:  New parameters
 *****************************************************************************/
typedef struct {
  tuint ID;                  /* Channel instance ID */
  vtkSizeCfg_t   sizeCfg;    /* Size configuration structure */ 
} vtkNewCfg_t;

/******************************************************************************
 * DATA DEFINITION: VTK Configuration
 *                : This data structure is used to retrieve codec configuration
 *                  Inorder for the content in this structure to be valid, codec
 *                  has to receive both static & dynamic configuration
 *****************************************************************************/
typedef struct {
  tuint  coreId;
  tuint  img_width;
  tuint  img_height;
  tuint  maxImgWidth;
  tuint  maxImgHeight;
  tuint  codingType;
  tuint  maxFrameRate;
  tuint  inputStreamFormat;
  word   maxDisplayDelay; /* For YUV stream order/display order frame reordering */
  tbool  decFlush;        /* XDM_FLUSH for decoder */
  tulong maxBitRate;
  tulong encOutBufSize;   /* Buffer size allocated by application. This is assigned to 
                             outputBufDesc.bufSizes. If Encoder produces a frame that 
                             doesn't fit with in this size, there will be a fatal error
                             from the encoder*/
  void*  sysInst;
} vtkCfg_t;

/******************************************************************************
 * DATA DEFINITION: VTK Encoder Dynamic Configuration
 *                : This data structure is used to retrieve dynamic codec configuration
 *****************************************************************************/
typedef struct {
  tulong targetFrameRate;
  tulong refFrameRate;
  tulong targetBitrate;
  tuint  img_width;
  tuint  img_height;
} vtkEncDynCfg_t;

/******************************************************************************
 * DATA DEFINITION: VTK statistics 
 ******************************************************************************
 * DESCRIPTION: This structure monitors vtk statistics
 *
 *****************************************************************************/
typedef struct vtkEncDecStats_s {
   tulong numIFrames;    /* received I-frames */ 
   tulong numPFrames;    /* received P-frames */
   tulong numBFrames;    /* received B-frames */
   tulong maxIFrameSize; /* maximum I-frame size */
   tulong maxPFrameSize; /* maximum P-frame size */
   tulong maxBFrameSize; /* maximum B-frame size */
   tulong totalNumBytes; /* total Bytes: Consumed by Dec or Produced by Enc */
   tulong bitrate;       /* estimated average bps (Q4) */
   tulong skippedFrames; /* Encoder skipped frames because of rate control */
   tulong errorFrames;   /* counter of undecodable frames */
   tulong lastErrorCode; /* extended errors, decoder dependent */
   tuint  lastGOP;       /* last inter I-frames gap (last GOP) */ 
// values below this line are for house keeping and computing above stats.
   tuint  lastGOPCtr;    /* num Frames elapsed after prev I-Frame */
   tuint  bitRateFrameCounter; /* count of N (=30?) frames and update bit rate */
   tulong bitRateByteCount;/* reset every N frames when bit rate is computed */
   tulong bitRateTime;   /* time when previous bit rate is computed */
} vtkEncDecStats_t;
   
typedef struct vtkStats_s {
  vtkEncDecStats_t dec;   /* stats of incoming stream */
  vtkEncDecStats_t enc;   /* stats of encoded stream  */
} vtkStats_t;
/******************************************************************************
 * DATA DEFINITION:  Realtime control data structure
 *****************************************************************************/
/* VTK Control Messages */
typedef struct {
  tulong *cfgPtr;
  tulong size;
} vtkCodecCfgPtr_t;

typedef enum {
  VTK_GET_STATIC_CFG_PTR   = 0x00,
  VTK_GET_DYNAMIC_CFG_PTR,
  VTK_COMMIT_CODEC,
  VTK_GET_CFG,
  VTK_GET_ENC_DYN_CFG,
  VTK_SET_ENC_DYN_CFG,
  VTK_GET_STATS,
  VTK_REDUCE_MIPS,
  VTK_RESTORE_MIPS,
  VTK_RESET_CODEC,
  VTK_IS_DYN_UPDATE_ALLOWED,
  VTK_FLUSH_DEC,
  VTK_SET_MC_PARAMS
} vtkCtrlCode_t;

typedef struct {
  vtkCtrlCode_t code;
  union {
    vtkCfg_t vtkCfg;
    vtkCodecCfgPtr_t codecCfgPtr;
    vtkEncDynCfg_t vtkEncDynCfg;
    IVIDMC_t vtkMcParams;
  }u;
} vtkCtrl_t;

/*****************************************************************************/
typedef struct {
  debugInfo_t debugInfo; /* Debug/exception function */
  tulong      *curTime;  /* wallclock */
  tword       *scratch;
  tulong      scratchSize;
  IALG_Handle (*ALG_Create)(void *sysInst, IALG_Fxns *fxns, IALG_Handle p, IALG_Params *params, tulong* resDepBitMap);
  void        (*ALG_Free)  (void *sysInst);
  bool        (*RES_assign)(void *codecHandle, void* resFxns);
  bool        (*RES_free)(void *codecHandle, void* resFxns);
} vtkContext_t;   

/* External references to the VTK context */
extern vtkContext_t vtkContext;

/* VTK Public functions */
void vtkClose (void *vtkInst);
tint vtkControl (void *vtkInst, vtkCtrl_t *ctrl);
void vtkDelete (void **vtkInst);
tint vtkGetSizes (tint *nbufs, const memBuffer_t **bufs, vtkSizeCfg_t *cfg);
tint vtkNew (void **vtkInst, tint nbufs, memBuffer_t *bufs, vtkNewCfg_t *cfg);
tint vtkOpen (void *vtkInst, vtkCfg_t *cfg);
tint vtkDecode   (void *vtkInst, ifvisys_vidDecInput_t *vidDecInput, ifvisys_vidDecOutput_t *vidDecOutput);
tint vtkEncode   (void *vtkInst, ifvisys_vidEncInput_t *vidEncInput, ifvisys_vidEncOutput_t *vidEncOutput);
void vtkGetStats (void *vtkInst, bool reset, vtkStats_t *stats);

/* VTK Call Table */
typedef struct {
  void (*vtkClose)     (void *vtkInst);
  tint (*vtkControl)   (void *vtkInst, vtkCtrl_t *ctrl);
  void (*vtkDelete)    (void **vtkInst);
  tint (*vtkGetSizes)  (tint *nbufs, const memBuffer_t **bufs, vtkSizeCfg_t *cfg);
  tint (*vtkNew)       (void **vtkInst, tint nbufs, memBuffer_t *bufs, vtkNewCfg_t *cfg);
  tint (*vtkOpen)      (void *vtkInst, vtkCfg_t *cfg);
  tint (*vtkDecode)    (void *vtkInst, ifvisys_vidDecInput_t *vidDecInput, ifvisys_vidDecOutput_t *vidDecOutput);
  tint (*vtkEncode)    (void *vtkInst, ifvisys_vidEncInput_t *vidEncInput, ifvisys_vidEncOutput_t *vidEncOutput);
} vtkCallTable_t;

#endif
/* Nothing past this point */
