/******************************************************************************
 * FILE PURPOSE:  Routines that create a DSP VTK channel
 ******************************************************************************
 * FILE NAME:     vtkinit.c 
 *
 * DESCRIPTION:   Contains routines that create a DSP VTK unit
 *
 * FUNCTION                     DESCRPTION
 * --------                     ----------
 *
 * (C) Copyright 2008, Texas Instruments Inc. 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
/* Standard C header files */
#include <stdio.h>
#include <stdlib.h>
#include "xdc/std.h"

/* System utility files */
#include "types.h"
#include "memmgm.h"     /* memory management */

/* VTK files */
#include "vtk.h"
#include "vtkloc.h"

/* number of buffers required by VTK */
#define VTK_NBUFS                1
#define VTK_INSTANCE_BUFN        0

/* buffers used by VTK */
memBuffer_t vtkBufs[VTK_NBUFS] = {
  MEM_CLASS_EXTERNAL, 0, 0, FALSE, NULL,  /* instance structure */
};

/******************************************************************************
 * FUNCTION PURPOSE: Sizes of buffers in use by a VTK instance.
 ******************************************************************************
 * DESCRIPTION: 
 * int vtkGetSizes (
 *                 int *nbufs,               - number of buffers
 *                 const memBuffer_t **bufs, - buffer descriptions
 *                 vtkSizeCfg_t *cfg)        - size config struct
 *
 * Return value : VTK_NOERR - successful in getting buffer sizes
 *                VTK_ERROR - 'cfg' information in conflict with pre-set
 *                                     limits or available resources
 *****************************************************************************/
tint vtkGetSizes (tint *nbufs, const memBuffer_t **bufs, vtkSizeCfg_t *cfg)
{

  /* Calculate the buffer sizes and report their descriptions */
  *nbufs = VTK_NBUFS;             /* Report the number of buffers */
  vtkBufs[VTK_INSTANCE_BUFN].size   = sizeof(vtkInst_t);
  *bufs = vtkBufs;
  return (VTK_NOERR);
} /* vtkGetSizes() */

/******************************************************************************
 * FUNCTION PURPOSE: Create an instance of VTK
 ******************************************************************************
 * DESCRIPTION: Create an an instance of VTK and initialize its memory buffers.
 *
 * tint vtkNew(
 *   void            **vtkInst,    - an address of memory location that will
 *                                  receive a pointer to instance structure
 *   tint            nbufs,        - number of buffers used
 *   memBuffer_t     *bufs,        - a vector of buffer descriptors
 *   vtkNewConfig_t  *cfg)         - a pointer to configuration structure
 *
 * Return values:  VTK_NOMEMORY    - some buffers have base address NULL
 *                 VTK_NOERR       - success
 *                 VTK_ERROR       - invalid input parameters   
 *
 *****************************************************************************/
tint vtkNew (void **vtkInst, tint nbufs, memBuffer_t *bufs, vtkNewCfg_t *cfg)
{
  int   i;
  vtkInst_t *inst;
  memBuffer_t *bufp;
  tint retval = VTK_NOERR;
  word *tmp_ptr;
  /* Test instance pointer (Must be NULL)  */
  if (*vtkInst != NULL || nbufs != VTK_NBUFS)
    return(VTK_ERROR);
  /* Allocate all dynamic buffers (base address != NULL ?)   */
  for (i = 0, bufp = &bufs[0]; i < nbufs; i++, bufp++) {
    if ( (bufp->size != 0) && (bufp->base == NULL) )
      retval = VTK_NOMEMORY;
  }

  if (retval != VTK_NOERR) 
     return retval;
  /*  Identify the instance structure  */
  bufp = &bufs[VTK_INSTANCE_BUFN];
  if (bufp->size < sizeof(vtkInst_t))
    return(VTK_ERROR);
  inst = (vtkInst_t *) bufp->base;
  /* Reset instance buffer to known state */
  memset (bufp->base, 0, sizeof(vtkInst_t));
  /* Save configuration data  */
  inst->ID         = cfg->ID;
  inst->state      = VTK_STATE_CLOSED;  /* set VTK state to CLOSED  */
  *vtkInst = (void *)inst;   /* return instance pointer  */
  return(VTK_NOERR);
} /*  vtkNew  */

/* Nothing past this point */

