/******************************************************************************
 * FILE PURPOSE: (vtk) Transmit/Receive processing
 ******************************************************************************
 * FILE NAME:   vtktxrx.c  
 *
 * DESCRIPTION: This file contains the main algorithm functions for VTK.
 *              
 * FUNCTION           DESCRIPTION
 * --------           ----------- 
 *
 * (C) Copyright 2008, Texas Instruments Inc. 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

/* Standard C header files */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "xdc/std.h"
#include "ti/xdais/ialg.h"
#include "vtk.h"
#include "vtkloc.h"
#include "emureg.h"
#include "vctypes.h"
#include <ti/mas/iface/ifvisys/ifvisys.h>



/*********************************************************************************
 * FUNCTION PURPOSE: Remove YUV padding from output datat
 *********************************************************************************
  DESCRIPTION:      
  Parameters :      Inputs: displayBufs   : display buffer pointer
                    Output: None
 *********************************************************************************/
XDAS_Void vtkRemoveYUVPadding(void *vtkInst, IVIDEO1_BufDesc *displayBufs)
{
    vtkInst_t *inst  = (vtkInst_t *)vtkInst;

    XDAS_UInt8 *s;
    XDAS_Int32 i;

    XDAS_Int32 ht;

    XDAS_Int32 height,width,pitch;
    unsigned char* scratchBufferPtr = (unsigned char*) vtkContext.scratch;
    tlong scratchBufferSize = vtkContext.scratchSize;

    height = displayBufs->frameHeight;
    width = displayBufs->frameWidth;
    pitch = displayBufs->framePitch;

    /* remove YUV padding and copy into a scratch buffer */
    if(width == pitch)
    {
        for(i = 0; i < displayBufs->numBufs; i++)
        {
            s = (XDAS_UInt8 *)displayBufs->bufDesc[i].buf;
            memcpy(scratchBufferPtr, s, sizeof(Byte)*displayBufs->bufDesc[i].bufSize);
            scratchBufferPtr += sizeof(Byte)*displayBufs->bufDesc[i].bufSize;
            scratchBufferSize -= sizeof(Byte)*displayBufs->bufDesc[i].bufSize;
            vtk_exc_assert((scratchBufferSize >= 0), VTK_EXC_ALLOCSCRATCH, inst);
        }
    }
    else
    {
        for(i = 0; i < displayBufs->numBufs; i++)
        {
            for(ht = 0 ; ht < height; ht++)
            {
                s = (XDAS_UInt8 *)displayBufs->bufDesc[i].buf + ht * pitch;
                memcpy(scratchBufferPtr, s, sizeof(Byte)*width);
                scratchBufferPtr += sizeof(Byte)*width;
                scratchBufferSize -= sizeof(Byte)*width;
                vtk_exc_assert((scratchBufferSize >= 0), VTK_EXC_ALLOCSCRATCH, inst);
            }
            if(!i)
            {
                height = height >> 1;
                width = width >> 1;
                pitch = pitch >> 1;
            }
        }
    }

   return;
}


/*********************************************************************************
 * FUNCTION PURPOSE: Decode 
 *********************************************************************************
  DESCRIPTION:      
  Parameters :      Inputs: vtkInst Pointer 
                            vidDecInput        : Input to the Decoder
                            vidDecOutput       : Output from the Decoder
                    Output: return value    : (success/failure) 
 *********************************************************************************/
tint vtkDecode   (void *vtkInst, ifvisys_vidDecInput_t *vidDecInput, ifvisys_vidDecOutput_t *vidDecOutput) {
  vtkInst_t *inst  = (vtkInst_t *)vtkInst;
  vtkDecInst_t *vtkDecInst = &(inst->codec.dec);
  XDAS_UInt32  ii;
  XDAS_Int32  retVal = XDM_EOK;
  IVIDDEC_Fxns    *ividDecFxns;
  XDM_BufDesc inputBufDesc;   /* for XDM 0.9 */
  XDM1_BufDesc inputBufDesc2; /* for XDM 1.0 */
  XDM_BufDesc outputBufDesc;  /* same buff Desc for both XDM 0.9 and 1.0 */
  XDAS_Int8*  pInputBuf   [XDM_MAX_IO_BUFFERS];
  XDAS_Int8*  pOutputBuf  [XDM_MAX_IO_BUFFERS];
  XDAS_Int32  inBufSize   [XDM_MAX_IO_BUFFERS];
  XDAS_Int32  outBufSize  [XDM_MAX_IO_BUFFERS];  

  IVIDDEC2_Fxns    *ividDecFxns2;
  tulong numBytesInNALarr;
  vtkCodecInfo_t info;

  /* make sure that the Decoder is Opened */
  vtk_exc_assert((inst->state == VTK_STATE_OPEN),VTK_EXC_DECODE, inst);

  /* set space for buffer descriptors */
  inputBufDesc.bufs       = (XDAS_Int8 **)pInputBuf;
  outputBufDesc.bufs      = (XDAS_Int8 **)pOutputBuf;
  inputBufDesc.bufSizes   = (XDAS_Int32 *)inBufSize ;
  outputBufDesc.bufSizes  = (XDAS_Int32 *)outBufSize;

  inst->statusCode        = 0;
  vidDecOutput->outputID     = (word)-1; /* initialize to invalid output ID */
  vidDecOutput->allocNewBuff = TRUE;     /* Default is true for all decoders */
  memset(&info, 0, sizeof(vtkCodecInfo_t));

    /*******************/
    /* Setup new input */
    /*******************/
    outputBufDesc.bufs[0] = (XDAS_Int8 *)vidDecOutput->yuv.yuvFrame.ptr;

    /* Activate the Decoder Algorithm */
    vtkDecInst->handle->fxns->algActivate(vtkDecInst->handle);
     
    vtkGetCodecInfo(inst, &info);

    switch (info.xdmVer) {

    case VTK_XDM_1P0:
      {
      IVIDDEC2_Status  *dec_status2       = info.decArgs.xdmver.xdm1p0.dec_status2;
      IVIDDEC2_InArgs  *dec_inargs2       = info.decArgs.xdmver.xdm1p0.dec_inargs2;
      IVIDDEC2_OutArgs *dec_outargs2      = info.decArgs.xdmver.xdm1p0.dec_outargs2;
      IVIDDEC2_DynamicParams *dec_dynpar2 = info.decArgs.xdmver.xdm1p0.dec_dynpar2;

      inputBufDesc2.descs[0].buf  = (XDAS_Int8 *)vidDecInput->bitStream.ptr;
      numBytesInNALarr            = vidDecInput->bitStream.size;
      if(inst->cfg.codingType == EVCT_VIDEO_H264HP_DEC) {
         info.decArgs.u.h264hp.inArgs.numBytesInNALarr = (XDAS_Int32 *)&numBytesInNALarr;
      }
      /* Assigning Algorithm handle fxns field to ividDecfxns */
      ividDecFxns2 = (IVIDDEC2_Fxns *)vtkDecInst->handle->fxns;
      /* Get Buffer information */
      ividDecFxns2->control((IVIDDEC2_Handle)vtkDecInst->handle, XDM_GETBUFINFO, dec_dynpar2, dec_status2);

      /* Fill up the buffers as required by algorithm */
      inputBufDesc2.numBufs          = dec_status2->bufInfo.minNumInBufs ;
      inputBufDesc2.descs[0].bufSize = dec_status2->bufInfo.minInBufSize[0];
      for(ii=0; ii<(dec_status2->bufInfo.minNumInBufs-1);ii++) {
        inputBufDesc2.descs[ii+1].buf      = inputBufDesc2.descs[ii].buf + dec_status2->bufInfo.minInBufSize[ii];
        inputBufDesc2.descs[ii +1].bufSize = dec_status2->bufInfo.minInBufSize[ii +1];
      }

      outputBufDesc.numBufs     = dec_status2->bufInfo.minNumOutBufs ;
      outputBufDesc.bufSizes[0] = dec_status2->bufInfo.minOutBufSize[0];
      for(ii=0; ii<(dec_status2->bufInfo.minNumOutBufs-1);ii++) {
        outputBufDesc.bufs[ii+1]     = outputBufDesc.bufs[ii] + dec_status2->bufInfo.minOutBufSize[ii];
        outputBufDesc.bufSizes[ii+1] = dec_status2->bufInfo.minOutBufSize[ii+1];
      }

      dec_inargs2->inputID           = vidDecInput->inputID;
      dec_inargs2->numBytes          = vidDecInput->bitStream.size;
      inputBufDesc2.descs[0].bufSize = dec_inargs2->numBytes;

      /* Set Run time parameters in the Algorithm via control()    */
      ividDecFxns2->control((IVIDDEC2_Handle)vtkDecInst->handle, XDM_SETPARAMS, dec_dynpar2, dec_status2);
      if(inst->cfg.decFlush == TRUE) {
          ividDecFxns2->control((IVIDDEC2_Handle)vtkDecInst->handle, XDM_FLUSH, dec_dynpar2, dec_status2);
      }

      inst->mcParams.swbarr(inst->mcParams.coreID, IVIDMC_SWBARR15, inst->mcParams.ncores);
      /* Basic Algorithm process() call */
      retVal = ividDecFxns2->process((IVIDDEC2_Handle)vtkDecInst->handle, 
        (XDM1_BufDesc *)&inputBufDesc2, (XDM_BufDesc *)&outputBufDesc, dec_inargs2, dec_outargs2);
      inst->mcParams.swbarr(inst->mcParams.coreID, IVIDMC_SWBARR15, inst->mcParams.ncores);

      if (retVal == XDM_EFAIL) {
        if (inst->cfg.codingType == EVCT_VIDEO_H264BP_MP_DEC) {
           if (dec_status2->extendedError == 0x20D3) {
              inst->stats.dec.errorFrames++;
              inst->stats.dec.lastErrorCode = dec_status2->extendedError;
              /* Basic Algorithm process() call */
              retVal = ividDecFxns2->process((IVIDDEC2_Handle)vtkDecInst->handle, 
                (XDM1_BufDesc *)&inputBufDesc2, (XDM_BufDesc *)&outputBufDesc, dec_inargs2, dec_outargs2);
           }
        }
      }

      /* Read status via control()      */
      ividDecFxns2->control((IVIDDEC2_Handle)vtkDecInst->handle, XDM_GETSTATUS, dec_dynpar2, dec_status2);
      inst->statusCode  = dec_status2->extendedError; /* Copy error code: it will be erased with reset */
      if(retVal == XDM_EFAIL)
      {
        inst->stats.dec.errorFrames++;
        inst->stats.dec.lastErrorCode = dec_status2->extendedError;
        /* Reset the Decoder only when no output is produced. i.e. Used concealed frames */
        if (dec_outargs2->displayBufs->bufDesc[0].buf == NULL) {
          ividDecFxns2->control((IVIDDEC2_Handle)vtkDecInst->handle, XDM_RESET, dec_dynpar2, dec_status2);
          vtkDecInst->handle->fxns->algDeactivate(vtkDecInst->handle);
          goto end;
        }
        retVal = XDM_EOK;
      }
      vtkDecInst->handle->fxns->algDeactivate(vtkDecInst->handle);

      if (inst->cfg.codingType == EVCT_VIDEO_H264BP_MP_DEC) {
        /* Frame re-ordering not supported in NAL mode */
        if(inst->cfg.inputStreamFormat == 1){ //NAL Unit Format
            vidDecOutput->allocNewBuff = FALSE;
        } else {
            vidDecOutput->allocNewBuff = TRUE;
        }
      }

      if(inst->mcParams.core_task_ID != IVIDMC_TASK_MASTER)
        goto end;

      if(dec_outargs2->displayBufs->bufDesc[0].buf == NULL)
      { 
       /* Nothing to encode (e.g. SPS/PPS) or no output due to frame re-ordering */
        vidDecOutput->yuvValid = FALSE;
        goto end;
      }

      /* update decoder statistics */
      vtkStatsUpdateFrameDetails(&inst->stats.dec, (vtkFrameType_t)dec_outargs2->displayBufs[0].frameType, dec_outargs2->bytesConsumed);

      vidDecOutput->yuvValid       = TRUE;
      vidDecOutput->outputID       = dec_outargs2->outputID[0];
      vidDecOutput->yuv.yuvWidth   = dec_outargs2->displayBufs[0].frameWidth;
      vidDecOutput->yuv.yuvHeight  = dec_outargs2->displayBufs[0].frameHeight;
      if(inst->cfg.codingType == EVCT_VIDEO_H264HP_DEC) {
        /* Remove YUV padding and pass the display YUV buffer pointer back to SIU */
        vtkRemoveYUVPadding(inst, &(dec_outargs2->displayBufs[0]));
        vidDecOutput->yuv.yuvFrame.ptr = dec_outargs2->displayBufs->bufDesc[0].buf;
      }
      vtk_exc_assert(((vidDecOutput->yuv.yuvWidth <= inst->cfg.maxImgWidth) && (vidDecOutput->yuv.yuvHeight <= inst->cfg.maxImgHeight)),
                       VTK_EXC_RX_RESOLUTION_EXCEEDED, inst);
      }
      break;

    case VTK_XDM_0P9:
      {
      IVIDDEC_Status  *dec_status       = info.decArgs.xdmver.xdm0p9.dec_status;
      IVIDDEC_InArgs  *dec_inargs       = info.decArgs.xdmver.xdm0p9.dec_inargs;
      IVIDDEC_OutArgs *dec_outargs      = info.decArgs.xdmver.xdm0p9.dec_outargs;
      IVIDDEC_DynamicParams *dec_dynpar = info.decArgs.xdmver.xdm0p9.dec_dynpar;

      inputBufDesc.bufs[0]  = (XDAS_Int8 *)vidDecInput->bitStream.ptr;
      numBytesInNALarr = vidDecInput->bitStream.size;
      
      /* Assigning Algorithm handle fxns field to ividDecfxns */
      ividDecFxns = (IVIDDEC_Fxns *)vtkDecInst->handle->fxns;

      /* Get Buffer information */
      ividDecFxns->control((IVIDDEC_Handle)vtkDecInst->handle, XDM_GETBUFINFO, dec_dynpar, dec_status);     
      /* Fill up the buffers as required by algorithm */
      inputBufDesc.numBufs     = dec_status->bufInfo.minNumInBufs ;
      inputBufDesc.bufSizes[0] = dec_status->bufInfo.minInBufSize[0];
      for(ii=0; ii<(dec_status->bufInfo.minNumInBufs-1);ii++ ) {
        inputBufDesc.bufs[ii+1]      = inputBufDesc.bufs[ii] + dec_status->bufInfo.minInBufSize[ii];
        inputBufDesc.bufSizes[ii +1] = dec_status->bufInfo.minInBufSize[ii +1];
      }

      outputBufDesc.numBufs     = dec_status->bufInfo.minNumOutBufs ;
      outputBufDesc.bufSizes[0] = dec_status->bufInfo.minOutBufSize[0];
      for(ii=0; ii<(dec_status->bufInfo.minNumOutBufs-1);ii++ ) {
        outputBufDesc.bufs[ii+1]     = outputBufDesc.bufs[ii] + dec_status->bufInfo.minOutBufSize[ii];
        outputBufDesc.bufSizes[ii+1] = dec_status->bufInfo.minOutBufSize[ii+1];
      }

      dec_inargs->inputID  = vidDecInput->inputID;
      dec_inargs->numBytes = vidDecInput->bitStream.size;

      if(inst->cfg.decFlush == TRUE) {
          ividDecFxns->control((IVIDDEC_Handle)vtkDecInst->handle, XDM_FLUSH, dec_dynpar, dec_status);
      }

      /* Basic Algorithm process() call */
      retVal = ividDecFxns->process((IVIDDEC_Handle)vtkDecInst->handle, 
               (XDM_BufDesc *)&inputBufDesc, (XDM_BufDesc *)&outputBufDesc, dec_inargs, dec_outargs);

      /* Read status via control()                                 */
      ividDecFxns->control((IVIDDEC_Handle)vtkDecInst->handle, XDM_GETSTATUS, dec_dynpar, dec_status);
      inst->statusCode  = dec_status->extendedError; /* Copy error code: it will be erased with reset */

      if(retVal == XDM_EFAIL)
      {
        inst->stats.dec.errorFrames++;
        inst->stats.dec.lastErrorCode = dec_status->extendedError;
        /* Reset the Decoder only when no output is produced. i.e. Used concealed frames */
        if(dec_outargs->displayBufs.bufs[0] == NULL) {
          ividDecFxns->control((IVIDDEC_Handle)vtkDecInst->handle, XDM_RESET, dec_dynpar, dec_status);
          vtkDecInst->handle->fxns->algDeactivate(vtkDecInst->handle);
          goto end;
        }
        retVal = XDM_EOK;
      }

      vtkDecInst->handle->fxns->algDeactivate(vtkDecInst->handle);

      if(dec_outargs->displayBufs.bufs[0] == NULL)
      {
       /* Nothing to encode (e.g. SPS/PPS) or no output due to frame re-ordering */
        vidDecOutput->yuvValid = FALSE;
        goto end;
      }

      /* update decoder statistics */
      vtkStatsUpdateFrameDetails(&inst->stats.dec, (vtkFrameType_t)dec_outargs->decodedFrameType, dec_outargs->bytesConsumed);

      vidDecOutput->yuvValid       = TRUE;
      vidDecOutput->outputID       = dec_outargs->outputID;
      vidDecOutput->yuv.yuvWidth  = dec_outargs->displayBufs.width;
      vidDecOutput->yuv.yuvHeight = (dec_outargs->displayBufs.bufSizes[0])/(dec_outargs->displayBufs.width);
      vtk_exc_assert(((vidDecOutput->yuv.yuvWidth <= inst->cfg.maxImgWidth) && (vidDecOutput->yuv.yuvHeight <= inst->cfg.maxImgHeight)),
                       VTK_EXC_RX_RESOLUTION_EXCEEDED, inst);
    }
    break;
    } /* end of switch */

end:
   return retVal;
}

/*********************************************************************************
 * FUNCTION PURPOSE: Encode 
 *********************************************************************************
  DESCRIPTION:      
  Parameters :      Inputs: vtkInst Pointer 
                            vidEncInput        : Input to the Encoder
                            vidEncOutput       : Output from the Encoder
                    Output: return value    : (success/failure) 
 *********************************************************************************/
tint vtkEncode   (void *vtkInst, ifvisys_vidEncInput_t *vidEncInput, ifvisys_vidEncOutput_t *vidEncOutput)
{
    vtkInst_t *inst  = (vtkInst_t *)vtkInst;
    vtkEncInst_t *vtkEncInst = &(inst->codec.enc);
    XDAS_UInt32  ii;
    XDAS_Int32  retVal = XDM_EOK;
  /*-----------------------------------------*/
   XDM_BufDesc           inputBufDesc;
   XDM_BufDesc           outputBufDesc;
   IVIDEO1_BufDescIn inputBufDesc1;

   XDAS_Int8*  pInputBuf   [XDM_MAX_IO_BUFFERS];
   XDAS_Int8*  pOutputBuf  [XDM_MAX_IO_BUFFERS];
   XDAS_Int32  inBufSize   [XDM_MAX_IO_BUFFERS];
   XDAS_Int32  outBufSize  [XDM_MAX_IO_BUFFERS];
  /*-----------------------------------------*/
    IVIDENC_Fxns    *ividEncFxns;
    IVIDENC1_Fxns   *ividEncFxns1;  
    vtkCodecInfo_t info;

#ifdef VTK_PRINT_CODEC_INTERNAL_MIPS
   ResetTimerVariables();
#endif

  /* make sure that the Decoder is Opened */
   vtk_exc_assert((inst->state == VTK_STATE_OPEN),VTK_EXC_ENCODE, inst);

   inst->statusCode         = 0;
   vtkEncInst->bytesGenerated = 0;
   memset(&info,0,sizeof(vtkCodecInfo_t));

   vtkGetCodecInfo(inst, &info);
    
    /* set space for buffer descriptors */
    outputBufDesc.bufs = (XDAS_Int8 **)pOutputBuf;
    outputBufDesc.bufSizes = (XDAS_Int32 *)outBufSize ;
    switch (info.xdmVer) {
      case VTK_XDM_1P0:
        {
        IVIDENC1_Status  *enc_status1       = info.encArgs.xdmver.xdm1p0.enc_status1;
        IVIDENC1_InArgs  *enc_inargs1       = info.encArgs.xdmver.xdm1p0.enc_inargs1;
        IVIDENC1_OutArgs *enc_outargs1      = info.encArgs.xdmver.xdm1p0.enc_outargs1;
        IVIDENC1_DynamicParams *enc_dynpar1 = info.encArgs.xdmver.xdm1p0.enc_dynpar1;
        XDAS_Int32 params_maxWidth          = info.encArgs.xdmver.xdm1p0.params_maxWidth;

        /* Assigning Algorithm handle fxns field to ividEncfxns */
        ividEncFxns1 = (IVIDENC1_Fxns *)vtkEncInst->handle->fxns;
        vtkEncInst->handle->fxns->algActivate(vtkEncInst->handle);
        ividEncFxns1->control((IVIDENC1_Handle)vtkEncInst->handle, XDM_SETPARAMS, enc_dynpar1, enc_status1);
        ividEncFxns1->control((IVIDENC1_Handle)vtkEncInst->handle, XDM_GETBUFINFO, enc_dynpar1, enc_status1);

        /*Fill up the buffers as required by algorithm 
        */
        inputBufDesc1.framePitch = params_maxWidth;
        inputBufDesc1.numBufs     = enc_status1->bufInfo.minNumInBufs;
        inputBufDesc1.frameWidth  = enc_dynpar1->inputWidth;
        inputBufDesc1.frameHeight = enc_dynpar1->inputHeight;

        inputBufDesc1.bufDesc[0].buf     = (XDAS_Int8 *)vidEncInput->bitStream.ptr;
        inputBufDesc1.bufDesc[0].bufSize = enc_status1->bufInfo.minInBufSize[0];

        for(ii=0; ii< (enc_status1->bufInfo.minNumInBufs - 1);ii++ )
        {
          inputBufDesc1.bufDesc[ii+1].buf     = inputBufDesc1.bufDesc[ii].buf + enc_status1->bufInfo.minInBufSize[ii];
          inputBufDesc1.bufDesc[ii+1].bufSize = enc_status1->bufInfo.minInBufSize[ii +1];
        }
        
        outputBufDesc.numBufs     = enc_status1->bufInfo.minNumOutBufs ;
        outputBufDesc.bufs[0]     = (XDAS_Int8 *)vidEncOutput->bitStream.ptr;
        outputBufDesc.bufSizes[0] = inst->cfg.encOutBufSize; //enc_status1->bufInfo.minOutBufSize[0];
        for(ii=0; ii< (enc_status1->bufInfo.minNumOutBufs-1); ii++ )
        {
          outputBufDesc.bufs[ii+1]     = outputBufDesc.bufs[ii] + enc_status1->bufInfo.minOutBufSize[ii];
          outputBufDesc.bufSizes[ii+1] = enc_status1->bufInfo.minOutBufSize[ii+1];
        }

        /* Basic Algorithm process() call */
        retVal = ividEncFxns1->process((IVIDENC1_Handle)vtkEncInst->handle, 
            (IVIDEO1_BufDescIn *)&inputBufDesc1, (XDM_BufDesc *)&outputBufDesc,
            enc_inargs1, enc_outargs1);
        /* Optional: Read status via control()                                  */
        ividEncFxns1->control((IVIDENC1_Handle)vtkEncInst->handle, XDM_GETSTATUS, enc_dynpar1, enc_status1);
        inst->statusCode = enc_status1->extendedError;
        if(retVal == XDM_EFAIL)
        {
            inst->stats.enc.lastErrorCode = enc_status1->extendedError;
            inst->stats.enc.errorFrames++;
            goto transcode_cleanup; /* Error Condition: Application may want to break off          */
        }
        /* update encoder statistics */
        vtkStatsUpdateFrameDetails(&inst->stats.enc, (vtkFrameType_t)enc_outargs1->encodedFrameType, enc_outargs1->bytesGenerated);
        /* Check for frame ready via recon buffer information                   */
        if(enc_outargs1->reconBufs.numBufs) {
          vidEncOutput->bitStream.size = enc_outargs1->bytesGenerated;
          vidEncOutput->frameType      = (ifvisys_FrameType_t)enc_outargs1->encodedFrameType;
        }
        if (inst->cfg.codingType == EVCT_VIDEO_H263P_ENC) {
           IMP4VENC_DynamicParams *extn_dynamicParams;
           IMP4VENC_Status        *mp4venc_status;
           extn_dynamicParams = &vtkEncInst->u.mpeg4.ext_dynpar;
           mp4venc_status     = &info.encArgs.u.mpeg4.status;
           /* Macro Block Indices */
           if(extn_dynamicParams->MBIndexEnable == 1) {
			 vidEncOutput->mbArray.ptr  = (word *)outputBufDesc.bufs[mp4venc_status->videncStatus.bufInfo.minNumOutBufs-1
					                                       - extn_dynamicParams->GOBIndexEnable];
             vidEncOutput->mbArray.size = ((outputBufDesc.bufSizes[mp4venc_status->videncStatus.bufInfo.minNumOutBufs-1
					                                       - extn_dynamicParams->GOBIndexEnable])>>2);
           }

           /* Group of Block(GOB) Indices */
           if(extn_dynamicParams->GOBIndexEnable == 1) {
			 vidEncOutput->gobArray.ptr  = (word *)outputBufDesc.bufs[mp4venc_status->videncStatus.bufInfo.minNumOutBufs-1];
             vidEncOutput->gobArray.size = ((outputBufDesc.bufSizes[mp4venc_status->videncStatus.bufInfo.minNumOutBufs-1])>>2);
           }
        }
        /* Switch off force Frame autonomously after every Process() call. This is to guarantee only 1 I/IDR frame to be 
           generated when the host asks to force Iframe. If the forceFrame is not switched off, every frame produced 
           will be an I-frame
         */
        enc_dynpar1->forceFrame = IVIDEO_NA_FRAME;
        }
        break;

      case VTK_XDM_0P9:
       {
        IVIDENC_Status  *enc_status       = info.encArgs.xdmver.xdm0p9.enc_status;
        IVIDENC_InArgs  *enc_inargs       = info.encArgs.xdmver.xdm0p9.enc_inargs;
        IVIDENC_OutArgs *enc_outargs      = info.encArgs.xdmver.xdm0p9.enc_outargs;
        IVIDENC_DynamicParams *enc_dynpar = info.encArgs.xdmver.xdm0p9.enc_dynpar;

        /* set space for buffer descriptors */
       inputBufDesc.bufs = (XDAS_Int8 **)pInputBuf;
       inputBufDesc.bufSizes = (XDAS_Int32 *)inBufSize ;

        /* Assigning Algorithm handle fxns field to ividEncfxns */
        ividEncFxns = (IVIDENC_Fxns *)vtkEncInst->handle->fxns;
        vtkEncInst->handle->fxns->algActivate(vtkEncInst->handle);
              
        ividEncFxns->control((IVIDENC_Handle)vtkEncInst->handle, XDM_SETPARAMS, enc_dynpar, enc_status);
        ividEncFxns->control((IVIDENC_Handle)vtkEncInst->handle, XDM_GETBUFINFO, enc_dynpar, enc_status);

        /*Fill up the buffers as required by algorithm                            */
        inputBufDesc.numBufs     = enc_status->bufInfo.minNumInBufs;
        inputBufDesc.bufs[0]     = (XDAS_Int8 *)vidEncInput->bitStream.ptr;
        inputBufDesc.bufSizes[0] = enc_status->bufInfo.minInBufSize[0];

        for(ii=0; ii< (enc_status->bufInfo.minNumInBufs - 1);ii++ )
        {
          inputBufDesc.bufs[ii+1]      = inputBufDesc.bufs[ii] + enc_status->bufInfo.minInBufSize[ii];
          inputBufDesc.bufSizes[ii +1] = enc_status->bufInfo.minInBufSize[ii +1];
        }
        
        outputBufDesc.numBufs     = enc_status->bufInfo.minNumOutBufs ;
        outputBufDesc.bufs[0]     = (XDAS_Int8 *)vidEncOutput->bitStream.ptr;
        outputBufDesc.bufSizes[0] = inst->cfg.encOutBufSize; //enc_status->bufInfo.minOutBufSize[0];
        for(ii=0; ii< (enc_status->bufInfo.minNumOutBufs-1); ii++ )
        {
          outputBufDesc.bufs[ii+1]     = outputBufDesc.bufs[ii] + enc_status->bufInfo.minOutBufSize[ii];
          outputBufDesc.bufSizes[ii+1] = enc_status->bufInfo.minOutBufSize[ii+1];
        }

        /* Basic Algorithm process() call */
        retVal = ividEncFxns->process((IVIDENC_Handle)vtkEncInst->handle, 
            (XDM_BufDesc *)&inputBufDesc, (XDM_BufDesc *)&outputBufDesc,
            enc_inargs, enc_outargs);
        /* Optional: Read status via control()                                  */
        ividEncFxns->control((IVIDENC_Handle)vtkEncInst->handle, XDM_GETSTATUS, enc_dynpar, enc_status);
        inst->statusCode = enc_status->extendedError;
        if(retVal == XDM_EFAIL)
        {
            inst->stats.enc.lastErrorCode = enc_status->extendedError;
            inst->stats.enc.errorFrames++;
            goto transcode_cleanup; /* Error Condition: Application may want to break off          */
        }
        /* update encoder statistics */
        vtkStatsUpdateFrameDetails(&inst->stats.enc, (vtkFrameType_t)enc_outargs->encodedFrameType, enc_outargs->bytesGenerated);
        /* Check for frame ready via recon buffer information                   */
        if(enc_outargs->reconBufs.numBufs) {
          vidEncOutput->bitStream.size = enc_outargs->bytesGenerated;
          vidEncOutput->frameType      = (ifvisys_FrameType_t)enc_outargs->encodedFrameType;
        }
        /* Switch off force Frame autonomously after every Process() call. This is to guarantee only 1 I/IDR frame to be 
           generated when the host asks to force Iframe. If the forceFrame is not switched off, every frame produced 
           will be an I-frame
         */
        enc_dynpar->forceIFrame = 0;
       }
        break;
    }
transcode_cleanup:
  vtkEncInst->handle->fxns->algDeactivate(vtkEncInst->handle);
#ifdef VTK_PRINT_CODEC_INTERNAL_MIPS
  PrintTimerVariables();
#endif

    return (retVal);
}

/* nothing past this point */
